import { DatabaseKind, ParserResult } from "@azimutt/models";
import * as chevrotain from "./chevrotain";
import * as generator from "./generator";
import { importDatabase } from "./sqlImport";
import { exportDatabase } from "./sqlExport";
import { parsePostgresAst } from "./postgresParser";
import { buildPostgresDatabase } from "./postgresBuilder";
import { generatePostgres, generatePostgresDiff } from "./postgresGenerator";
export function parseSql(content, dialect, opts = {}) {
    const start = Date.now();
    if (dialect === DatabaseKind.enum.postgres)
        return parsePostgresAst(content, opts).flatMap(ast => buildPostgresDatabase(ast, start, Date.now()));
    return parseSqlScript(content).map(importDatabase);
}
export function generateSql(database, dialect) {
    if (dialect === DatabaseKind.enum.postgres)
        return generatePostgres(database);
    const script = exportDatabase(database);
    return generateSqlScript(script);
}
export function generateSqlDiff(diff, dialect) {
    if (dialect === DatabaseKind.enum.postgres)
        return generatePostgresDiff(diff);
    return `Can't generate SQL diff for ${dialect} dialect`;
}
function parseSqlScript(content) {
    return parseSqlAst(content).map(chevrotain.format);
}
function generateSqlScript(script) {
    return generator.generate(script);
}
function parseSqlAst(content) {
    const { result, errors } = chevrotain.parse(content);
    if (result) {
        return ParserResult.success(result);
    }
    else {
        return ParserResult.failure(errors || []);
    }
}
function generateSqlAst(script) {
    // TODO: create a generator for the AST
    return Promise.reject(new Error('Not implemented'));
}
//# sourceMappingURL=sql.js.map