"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var globals_1 = require("@jest/globals");
var models_1 = require("@azimutt/models");
var parser_1 = require("./parser");
(0, globals_1.describe)('chevrotain parser', function () {
    (0, globals_1.test)('empty', function () {
        (0, globals_1.expect)((0, parser_1.parse)('')).toEqual({ result: [] });
    });
    (0, globals_1.describe)('select', function () {
        (0, globals_1.test)('minimal', function () {
            (0, globals_1.expect)((0, parser_1.parse)('SELECT * FROM users;')).toEqual({ result: [{
                        command: 'SELECT',
                        result: { columns: [
                                { column: { wildcard: '*', parser: (0, models_1.tokenPosition)(7, 7, 1, 8, 1, 8) } }
                            ] },
                        from: { table: { identifier: 'users', parser: (0, models_1.tokenPosition)(14, 18, 1, 15, 1, 19) } }
                    }] });
        });
        (0, globals_1.test)('basic', function () {
            (0, globals_1.expect)((0, parser_1.parse)('SELECT name FROM users WHERE id=1;')).toEqual({ result: [{
                        command: 'SELECT',
                        result: { columns: [
                                { column: { identifier: 'name', parser: (0, models_1.tokenPosition)(7, 10, 1, 8, 1, 11) } }
                            ] },
                        from: { table: { identifier: 'users', parser: (0, models_1.tokenPosition)(17, 21, 1, 18, 1, 22) } },
                        where: {
                            left: { column: { identifier: 'id', parser: (0, models_1.tokenPosition)(29, 30, 1, 30, 1, 31) } },
                            operation: { operator: '=', parser: (0, models_1.tokenPosition)(31, 31, 1, 32, 1, 32) },
                            right: { value: 1, parser: (0, models_1.tokenPosition)(32, 32, 1, 33, 1, 33) }
                        }
                    }] });
        });
        (0, globals_1.describe)('result', function () {
            (0, globals_1.test)('column', function () {
                (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.selectResultRule(); }, 'SELECT id')).toEqual({ result: { columns: [{ column: { identifier: 'id', parser: (0, models_1.tokenPosition)(7, 8, 1, 8, 1, 9) } }] } });
                (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.selectResultRule(); }, 'SELECT u.id')).toEqual({ result: { columns: [{
                                column: { identifier: 'id', parser: (0, models_1.tokenPosition)(9, 10, 1, 10, 1, 11) },
                                table: { identifier: 'u', parser: (0, models_1.tokenPosition)(7, 7, 1, 8, 1, 8) },
                            }] } });
                (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.selectResultRule(); }, 'SELECT id, name')).toEqual({ result: { columns: [
                            { column: { identifier: 'id', parser: (0, models_1.tokenPosition)(7, 8, 1, 8, 1, 9) } },
                            { column: { identifier: 'name', parser: (0, models_1.tokenPosition)(11, 14, 1, 12, 1, 15) } },
                        ] } });
            });
            (0, globals_1.test)('wildcard', function () {
                (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.selectResultRule(); }, 'SELECT *')).toEqual({ result: { columns: [{ column: { wildcard: '*', parser: (0, models_1.tokenPosition)(7, 7, 1, 8, 1, 8) } }] } });
                (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.selectResultRule(); }, 'SELECT u.*')).toEqual({ result: { columns: [{
                                column: { wildcard: '*', parser: (0, models_1.tokenPosition)(9, 9, 1, 10, 1, 10) },
                                table: { identifier: 'u', parser: (0, models_1.tokenPosition)(7, 7, 1, 8, 1, 8) },
                            }] } });
            });
            // TODO: alias, expressions
        });
        (0, globals_1.describe)('from', function () {
            (0, globals_1.test)('table', function () {
                (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.selectFromRule(); }, 'FROM users')).toEqual({ result: { table: { identifier: 'users', parser: (0, models_1.tokenPosition)(5, 9, 1, 6, 1, 10) } } });
            });
            (0, globals_1.test)('schema', function () {
                (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.selectFromRule(); }, 'FROM public.users')).toEqual({ result: {
                        schema: { identifier: 'public', parser: (0, models_1.tokenPosition)(5, 10, 1, 6, 1, 11) },
                        table: { identifier: 'users', parser: (0, models_1.tokenPosition)(12, 16, 1, 13, 1, 17) }
                    } });
            });
            // TODO: alias, joins
        });
        (0, globals_1.describe)('where', function () {
            (0, globals_1.test)('equal', function () {
                (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.selectWhereRule(); }, 'WHERE id=1')).toEqual({ result: {
                        left: { column: { identifier: 'id', parser: (0, models_1.tokenPosition)(6, 7, 1, 7, 1, 8) } },
                        operation: { operator: '=', parser: (0, models_1.tokenPosition)(8, 8, 1, 9, 1, 9) },
                        right: { value: 1, parser: (0, models_1.tokenPosition)(9, 9, 1, 10, 1, 10) }
                    } });
            });
            (0, globals_1.test)('not equal', function () {
                (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.selectWhereRule(); }, "WHERE users.id != 'abc'")).toEqual({ result: {
                        left: {
                            table: { identifier: 'users', parser: (0, models_1.tokenPosition)(6, 10, 1, 7, 1, 11) },
                            column: { identifier: 'id', parser: (0, models_1.tokenPosition)(12, 13, 1, 13, 1, 14) }
                        },
                        operation: { operator: '!=', parser: (0, models_1.tokenPosition)(15, 16, 1, 16, 1, 17) },
                        right: { value: 'abc', parser: (0, models_1.tokenPosition)(18, 22, 1, 19, 1, 23) },
                    } });
            });
            (0, globals_1.test)('less than', function () {
                (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.selectWhereRule(); }, 'WHERE "public".users.id < 12')).toEqual({ result: {
                        left: {
                            schema: { identifier: 'public', parser: (0, models_1.tokenPosition)(6, 13, 1, 7, 1, 14) },
                            table: { identifier: 'users', parser: (0, models_1.tokenPosition)(15, 19, 1, 16, 1, 20) },
                            column: { identifier: 'id', parser: (0, models_1.tokenPosition)(21, 22, 1, 22, 1, 23) }
                        },
                        operation: { operator: '<', parser: (0, models_1.tokenPosition)(24, 24, 1, 25, 1, 25) },
                        right: { value: 12, parser: (0, models_1.tokenPosition)(26, 27, 1, 27, 1, 28) },
                    } });
            });
            // TODO: IS NOT NULL, OR, AND
        });
    });
    globals_1.describe.skip('create table', function () {
        (0, globals_1.test)('basic', function () {
            var res = (0, parser_1.parse)('CREATE TABLE users (id integer, name varchar);');
            console.log('res', res);
            (0, globals_1.expect)(res).toEqual({ result: [{
                        command: 'CREATE_TABLE',
                        table: { identifier: 'users', parser: (0, models_1.tokenPosition)(13, 17, 1, 14, 1, 18) },
                        columns: [
                            { name: { identifier: 'id', parser: (0, models_1.tokenPosition)(20, 21, 1, 21, 1, 22) }, type: { identifier: 'integer', parser: (0, models_1.tokenPosition)(23, 29, 1, 24, 1, 30) } },
                            { name: { identifier: 'name', parser: (0, models_1.tokenPosition)(31, 34, 1, 32, 1, 35) }, type: { identifier: 'varchar', parser: (0, models_1.tokenPosition)(36, 42, 1, 37, 1, 43) } },
                        ]
                    }] });
        });
    });
    (0, globals_1.describe)('common', function () {
        (0, globals_1.test)('integerRule', function () {
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.integerRule(); }, '12')).toEqual({ result: { value: 12, parser: (0, models_1.tokenPosition)(0, 1, 1, 1, 1, 2) } });
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.integerRule(); }, 'bad')).toEqual({ errors: [__assign({ message: "Expecting token of type --> Integer <-- but found --> 'bad' <--", kind: 'MismatchedTokenException', level: 'error' }, (0, models_1.tokenPosition)(0, 2, 1, 1, 1, 3))] });
        });
        (0, globals_1.test)('stringRule', function () {
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.stringRule(); }, "'abc'")).toEqual({ result: { value: 'abc', parser: (0, models_1.tokenPosition)(0, 4, 1, 1, 1, 5) } });
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.stringRule(); }, "'It\\'s'")).toEqual({ result: { value: "It's", parser: (0, models_1.tokenPosition)(0, 6, 1, 1, 1, 7) } });
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.stringRule(); }, "bad")).toEqual({ errors: [__assign({ message: "Expecting token of type --> String <-- but found --> 'bad' <--", kind: 'MismatchedTokenException', level: 'error' }, (0, models_1.tokenPosition)(0, 2, 1, 1, 1, 3))] });
        });
        (0, globals_1.test)('booleanRule', function () {
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.booleanRule(); }, 'true')).toEqual({ result: { value: true, parser: (0, models_1.tokenPosition)(0, 3, 1, 1, 1, 4) } });
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.booleanRule(); }, 'false')).toEqual({ result: { value: false, parser: (0, models_1.tokenPosition)(0, 4, 1, 1, 1, 5) } });
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.booleanRule(); }, 'bad')).toEqual({ errors: [__assign({ message: "Expecting token of type --> Boolean <-- but found --> 'bad' <--", kind: 'MismatchedTokenException', level: 'error' }, (0, models_1.tokenPosition)(0, 2, 1, 1, 1, 3))] });
        });
        (0, globals_1.test)('identifierRule', function () {
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.identifierRule(); }, 'id')).toEqual({ result: { identifier: 'id', parser: (0, models_1.tokenPosition)(0, 1, 1, 1, 1, 2) } });
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.identifierRule(); }, '"my col"')).toEqual({ result: { identifier: 'my col', parser: (0, models_1.tokenPosition)(0, 7, 1, 1, 1, 8) } });
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.identifierRule(); }, '"my \\"new\\" col"')).toEqual({ result: { identifier: 'my "new" col', parser: (0, models_1.tokenPosition)(0, 15, 1, 1, 1, 16) } });
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.identifierRule(); }, 'bad col')).toEqual({ errors: [__assign({ message: "Redundant input, expecting EOF but found: col", kind: 'NotAllInputParsedException', level: 'error' }, (0, models_1.tokenPosition)(4, 6, 1, 5, 1, 7))] });
        });
        (0, globals_1.test)('tableRefRule', function () {
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.tableRefRule(); }, 'users')).toEqual({ result: { table: { identifier: 'users', parser: (0, models_1.tokenPosition)(0, 4, 1, 1, 1, 5) } } });
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.tableRefRule(); }, 'public.users')).toEqual({ result: {
                    table: { identifier: 'users', parser: (0, models_1.tokenPosition)(7, 11, 1, 8, 1, 12) },
                    schema: { identifier: 'public', parser: (0, models_1.tokenPosition)(0, 5, 1, 1, 1, 6) },
                } });
        });
        (0, globals_1.test)('columnRefRule', function () {
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.columnRefRule(); }, 'id')).toEqual({ result: { column: { identifier: 'id', parser: (0, models_1.tokenPosition)(0, 1, 1, 1, 1, 2) } } });
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.columnRefRule(); }, 'users.id')).toEqual({ result: {
                    column: { identifier: 'id', parser: (0, models_1.tokenPosition)(6, 7, 1, 7, 1, 8) },
                    table: { identifier: 'users', parser: (0, models_1.tokenPosition)(0, 4, 1, 1, 1, 5) }
                } });
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.columnRefRule(); }, 'public.users.id')).toEqual({ result: {
                    column: { identifier: 'id', parser: (0, models_1.tokenPosition)(13, 14, 1, 14, 1, 15) },
                    table: { identifier: 'users', parser: (0, models_1.tokenPosition)(7, 11, 1, 8, 1, 12) },
                    schema: { identifier: 'public', parser: (0, models_1.tokenPosition)(0, 5, 1, 1, 1, 6) },
                } });
        });
        (0, globals_1.test)('conditionOpRule', function () {
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.conditionOpRule(); }, '=')).toEqual({ result: { operator: '=', parser: (0, models_1.tokenPosition)(0, 0, 1, 1, 1, 1) } });
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.conditionOpRule(); }, '!=')).toEqual({ result: { operator: '!=', parser: (0, models_1.tokenPosition)(0, 1, 1, 1, 1, 2) } });
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.conditionOpRule(); }, '<')).toEqual({ result: { operator: '<', parser: (0, models_1.tokenPosition)(0, 0, 1, 1, 1, 1) } });
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.conditionOpRule(); }, '>')).toEqual({ result: { operator: '>', parser: (0, models_1.tokenPosition)(0, 0, 1, 1, 1, 1) } });
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.conditionOpRule(); }, 'bad')).toEqual({ errors: [__assign({ message: "Expecting: one of these possible Token sequences:\n  1. [Equal]\n  2. [NotEqual]\n  3. [LessThan]\n  4. [GreaterThan]\nbut found: 'bad'", kind: 'NoViableAltException', level: 'error' }, (0, models_1.tokenPosition)(0, 2, 1, 1, 1, 3))] });
        });
        (0, globals_1.test)('conditionElemRule', function () {
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.conditionElemRule(); }, '12')).toEqual({ result: { value: 12, parser: (0, models_1.tokenPosition)(0, 1, 1, 1, 1, 2) } });
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.conditionElemRule(); }, "'abc'")).toEqual({ result: { value: 'abc', parser: (0, models_1.tokenPosition)(0, 4, 1, 1, 1, 5) } });
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.conditionElemRule(); }, 'true')).toEqual({ result: { value: true, parser: (0, models_1.tokenPosition)(0, 3, 1, 1, 1, 4) } });
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.conditionElemRule(); }, 'id')).toEqual({ result: { column: { identifier: 'id', parser: (0, models_1.tokenPosition)(0, 1, 1, 1, 1, 2) } } });
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.conditionElemRule(); }, '=')).toEqual({ errors: [__assign({ message: "Expecting: one of these possible Token sequences:\n  1. [Integer]\n  2. [String]\n  3. [Boolean]\n  4. [Identifier]\nbut found: '='", kind: 'NoViableAltException', level: 'error' }, (0, models_1.tokenPosition)(0, 0, 1, 1, 1, 1))] });
        });
        (0, globals_1.test)('conditionRule', function () {
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.conditionRule(); }, 'id<12')).toEqual({ result: {
                    left: { column: { identifier: 'id', parser: (0, models_1.tokenPosition)(0, 1, 1, 1, 1, 2) } },
                    operation: { operator: '<', parser: (0, models_1.tokenPosition)(2, 2, 1, 3, 1, 3) },
                    right: { value: 12, parser: (0, models_1.tokenPosition)(3, 4, 1, 4, 1, 5) }
                } });
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.conditionRule(); }, 'users."first name" = \'loic\'')).toEqual({ result: {
                    left: {
                        column: { identifier: 'first name', parser: (0, models_1.tokenPosition)(6, 17, 1, 7, 1, 18) },
                        table: { identifier: 'users', parser: (0, models_1.tokenPosition)(0, 4, 1, 1, 1, 5) }
                    },
                    operation: { operator: '=', parser: (0, models_1.tokenPosition)(19, 19, 1, 20, 1, 20) },
                    right: { value: 'loic', parser: (0, models_1.tokenPosition)(21, 26, 1, 22, 1, 27) }
                } });
            (0, globals_1.expect)((0, parser_1.parseRule)(function (p) { return p.conditionRule(); }, '=')).toEqual({ errors: [__assign({ message: "Expecting: one of these possible Token sequences:\n  1. [Integer]\n  2. [String]\n  3. [Boolean]\n  4. [Identifier]\nbut found: '='", kind: 'NoViableAltException', level: 'error' }, (0, models_1.tokenPosition)(0, 0, 1, 1, 1, 1))] });
        });
    });
});
