'use strict';

var commonjsGlobal = typeof globalThis !== 'undefined' ? globalThis : typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

function getDefaultExportFromCjs (x) {
	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
}

function getAugmentedNamespace(n) {
  if (n.__esModule) return n;
  var f = n.default;
	if (typeof f == "function") {
		var a = function a () {
			if (this instanceof a) {
        return Reflect.construct(f, arguments, this.constructor);
			}
			return f.apply(this, arguments);
		};
		a.prototype = f.prototype;
  } else a = {};
  Object.defineProperty(a, '__esModule', {value: true});
	Object.keys(n).forEach(function (k) {
		var d = Object.getOwnPropertyDescriptor(n, k);
		Object.defineProperty(a, k, d.get ? d : {
			enumerable: true,
			get: function () {
				return n[k];
			}
		});
	});
	return a;
}

var out$1 = {};

var analyze = {};

var lib = {};

var external = {};

var errors = {};

var en = {};

var util = {};

var hasRequiredUtil$1;

function requireUtil$1 () {
	if (hasRequiredUtil$1) return util;
	hasRequiredUtil$1 = 1;
	(function (exports) {
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.getParsedType = exports.ZodParsedType = exports.objectUtil = exports.util = void 0;
		var util;
		(function (util) {
		    util.assertEqual = (val) => val;
		    function assertIs(_arg) { }
		    util.assertIs = assertIs;
		    function assertNever(_x) {
		        throw new Error();
		    }
		    util.assertNever = assertNever;
		    util.arrayToEnum = (items) => {
		        const obj = {};
		        for (const item of items) {
		            obj[item] = item;
		        }
		        return obj;
		    };
		    util.getValidEnumValues = (obj) => {
		        const validKeys = util.objectKeys(obj).filter((k) => typeof obj[obj[k]] !== "number");
		        const filtered = {};
		        for (const k of validKeys) {
		            filtered[k] = obj[k];
		        }
		        return util.objectValues(filtered);
		    };
		    util.objectValues = (obj) => {
		        return util.objectKeys(obj).map(function (e) {
		            return obj[e];
		        });
		    };
		    util.objectKeys = typeof Object.keys === "function" // eslint-disable-line ban/ban
		        ? (obj) => Object.keys(obj) // eslint-disable-line ban/ban
		        : (object) => {
		            const keys = [];
		            for (const key in object) {
		                if (Object.prototype.hasOwnProperty.call(object, key)) {
		                    keys.push(key);
		                }
		            }
		            return keys;
		        };
		    util.find = (arr, checker) => {
		        for (const item of arr) {
		            if (checker(item))
		                return item;
		        }
		        return undefined;
		    };
		    util.isInteger = typeof Number.isInteger === "function"
		        ? (val) => Number.isInteger(val) // eslint-disable-line ban/ban
		        : (val) => typeof val === "number" && isFinite(val) && Math.floor(val) === val;
		    function joinValues(array, separator = " | ") {
		        return array
		            .map((val) => (typeof val === "string" ? `'${val}'` : val))
		            .join(separator);
		    }
		    util.joinValues = joinValues;
		    util.jsonStringifyReplacer = (_, value) => {
		        if (typeof value === "bigint") {
		            return value.toString();
		        }
		        return value;
		    };
		})(util = exports.util || (exports.util = {}));
		(function (objectUtil) {
		    objectUtil.mergeShapes = (first, second) => {
		        return {
		            ...first,
		            ...second, // second overwrites first
		        };
		    };
		})(exports.objectUtil || (exports.objectUtil = {}));
		exports.ZodParsedType = util.arrayToEnum([
		    "string",
		    "nan",
		    "number",
		    "integer",
		    "float",
		    "boolean",
		    "date",
		    "bigint",
		    "symbol",
		    "function",
		    "undefined",
		    "null",
		    "array",
		    "object",
		    "unknown",
		    "promise",
		    "void",
		    "never",
		    "map",
		    "set",
		]);
		const getParsedType = (data) => {
		    const t = typeof data;
		    switch (t) {
		        case "undefined":
		            return exports.ZodParsedType.undefined;
		        case "string":
		            return exports.ZodParsedType.string;
		        case "number":
		            return isNaN(data) ? exports.ZodParsedType.nan : exports.ZodParsedType.number;
		        case "boolean":
		            return exports.ZodParsedType.boolean;
		        case "function":
		            return exports.ZodParsedType.function;
		        case "bigint":
		            return exports.ZodParsedType.bigint;
		        case "symbol":
		            return exports.ZodParsedType.symbol;
		        case "object":
		            if (Array.isArray(data)) {
		                return exports.ZodParsedType.array;
		            }
		            if (data === null) {
		                return exports.ZodParsedType.null;
		            }
		            if (data.then &&
		                typeof data.then === "function" &&
		                data.catch &&
		                typeof data.catch === "function") {
		                return exports.ZodParsedType.promise;
		            }
		            if (typeof Map !== "undefined" && data instanceof Map) {
		                return exports.ZodParsedType.map;
		            }
		            if (typeof Set !== "undefined" && data instanceof Set) {
		                return exports.ZodParsedType.set;
		            }
		            if (typeof Date !== "undefined" && data instanceof Date) {
		                return exports.ZodParsedType.date;
		            }
		            return exports.ZodParsedType.object;
		        default:
		            return exports.ZodParsedType.unknown;
		    }
		};
		exports.getParsedType = getParsedType; 
	} (util));
	return util;
}

var ZodError = {};

var hasRequiredZodError;

function requireZodError () {
	if (hasRequiredZodError) return ZodError;
	hasRequiredZodError = 1;
	Object.defineProperty(ZodError, "__esModule", { value: true });
	ZodError.ZodError = ZodError.quotelessJson = ZodError.ZodIssueCode = void 0;
	const util_1 = /*@__PURE__*/ requireUtil$1();
	ZodError.ZodIssueCode = util_1.util.arrayToEnum([
	    "invalid_type",
	    "invalid_literal",
	    "custom",
	    "invalid_union",
	    "invalid_union_discriminator",
	    "invalid_enum_value",
	    "unrecognized_keys",
	    "invalid_arguments",
	    "invalid_return_type",
	    "invalid_date",
	    "invalid_string",
	    "too_small",
	    "too_big",
	    "invalid_intersection_types",
	    "not_multiple_of",
	    "not_finite",
	]);
	const quotelessJson = (obj) => {
	    const json = JSON.stringify(obj, null, 2);
	    return json.replace(/"([^"]+)":/g, "$1:");
	};
	ZodError.quotelessJson = quotelessJson;
	let ZodError$1 = class ZodError extends Error {
	    constructor(issues) {
	        super();
	        this.issues = [];
	        this.addIssue = (sub) => {
	            this.issues = [...this.issues, sub];
	        };
	        this.addIssues = (subs = []) => {
	            this.issues = [...this.issues, ...subs];
	        };
	        const actualProto = new.target.prototype;
	        if (Object.setPrototypeOf) {
	            // eslint-disable-next-line ban/ban
	            Object.setPrototypeOf(this, actualProto);
	        }
	        else {
	            this.__proto__ = actualProto;
	        }
	        this.name = "ZodError";
	        this.issues = issues;
	    }
	    get errors() {
	        return this.issues;
	    }
	    format(_mapper) {
	        const mapper = _mapper ||
	            function (issue) {
	                return issue.message;
	            };
	        const fieldErrors = { _errors: [] };
	        const processError = (error) => {
	            for (const issue of error.issues) {
	                if (issue.code === "invalid_union") {
	                    issue.unionErrors.map(processError);
	                }
	                else if (issue.code === "invalid_return_type") {
	                    processError(issue.returnTypeError);
	                }
	                else if (issue.code === "invalid_arguments") {
	                    processError(issue.argumentsError);
	                }
	                else if (issue.path.length === 0) {
	                    fieldErrors._errors.push(mapper(issue));
	                }
	                else {
	                    let curr = fieldErrors;
	                    let i = 0;
	                    while (i < issue.path.length) {
	                        const el = issue.path[i];
	                        const terminal = i === issue.path.length - 1;
	                        if (!terminal) {
	                            curr[el] = curr[el] || { _errors: [] };
	                            // if (typeof el === "string") {
	                            //   curr[el] = curr[el] || { _errors: [] };
	                            // } else if (typeof el === "number") {
	                            //   const errorArray: any = [];
	                            //   errorArray._errors = [];
	                            //   curr[el] = curr[el] || errorArray;
	                            // }
	                        }
	                        else {
	                            curr[el] = curr[el] || { _errors: [] };
	                            curr[el]._errors.push(mapper(issue));
	                        }
	                        curr = curr[el];
	                        i++;
	                    }
	                }
	            }
	        };
	        processError(this);
	        return fieldErrors;
	    }
	    static assert(value) {
	        if (!(value instanceof ZodError)) {
	            throw new Error(`Not a ZodError: ${value}`);
	        }
	    }
	    toString() {
	        return this.message;
	    }
	    get message() {
	        return JSON.stringify(this.issues, util_1.util.jsonStringifyReplacer, 2);
	    }
	    get isEmpty() {
	        return this.issues.length === 0;
	    }
	    flatten(mapper = (issue) => issue.message) {
	        const fieldErrors = {};
	        const formErrors = [];
	        for (const sub of this.issues) {
	            if (sub.path.length > 0) {
	                fieldErrors[sub.path[0]] = fieldErrors[sub.path[0]] || [];
	                fieldErrors[sub.path[0]].push(mapper(sub));
	            }
	            else {
	                formErrors.push(mapper(sub));
	            }
	        }
	        return { formErrors, fieldErrors };
	    }
	    get formErrors() {
	        return this.flatten();
	    }
	};
	ZodError.ZodError = ZodError$1;
	ZodError$1.create = (issues) => {
	    const error = new ZodError$1(issues);
	    return error;
	};
	return ZodError;
}

var hasRequiredEn;

function requireEn () {
	if (hasRequiredEn) return en;
	hasRequiredEn = 1;
	Object.defineProperty(en, "__esModule", { value: true });
	const util_1 = /*@__PURE__*/ requireUtil$1();
	const ZodError_1 = /*@__PURE__*/ requireZodError();
	const errorMap = (issue, _ctx) => {
	    let message;
	    switch (issue.code) {
	        case ZodError_1.ZodIssueCode.invalid_type:
	            if (issue.received === util_1.ZodParsedType.undefined) {
	                message = "Required";
	            }
	            else {
	                message = `Expected ${issue.expected}, received ${issue.received}`;
	            }
	            break;
	        case ZodError_1.ZodIssueCode.invalid_literal:
	            message = `Invalid literal value, expected ${JSON.stringify(issue.expected, util_1.util.jsonStringifyReplacer)}`;
	            break;
	        case ZodError_1.ZodIssueCode.unrecognized_keys:
	            message = `Unrecognized key(s) in object: ${util_1.util.joinValues(issue.keys, ", ")}`;
	            break;
	        case ZodError_1.ZodIssueCode.invalid_union:
	            message = `Invalid input`;
	            break;
	        case ZodError_1.ZodIssueCode.invalid_union_discriminator:
	            message = `Invalid discriminator value. Expected ${util_1.util.joinValues(issue.options)}`;
	            break;
	        case ZodError_1.ZodIssueCode.invalid_enum_value:
	            message = `Invalid enum value. Expected ${util_1.util.joinValues(issue.options)}, received '${issue.received}'`;
	            break;
	        case ZodError_1.ZodIssueCode.invalid_arguments:
	            message = `Invalid function arguments`;
	            break;
	        case ZodError_1.ZodIssueCode.invalid_return_type:
	            message = `Invalid function return type`;
	            break;
	        case ZodError_1.ZodIssueCode.invalid_date:
	            message = `Invalid date`;
	            break;
	        case ZodError_1.ZodIssueCode.invalid_string:
	            if (typeof issue.validation === "object") {
	                if ("includes" in issue.validation) {
	                    message = `Invalid input: must include "${issue.validation.includes}"`;
	                    if (typeof issue.validation.position === "number") {
	                        message = `${message} at one or more positions greater than or equal to ${issue.validation.position}`;
	                    }
	                }
	                else if ("startsWith" in issue.validation) {
	                    message = `Invalid input: must start with "${issue.validation.startsWith}"`;
	                }
	                else if ("endsWith" in issue.validation) {
	                    message = `Invalid input: must end with "${issue.validation.endsWith}"`;
	                }
	                else {
	                    util_1.util.assertNever(issue.validation);
	                }
	            }
	            else if (issue.validation !== "regex") {
	                message = `Invalid ${issue.validation}`;
	            }
	            else {
	                message = "Invalid";
	            }
	            break;
	        case ZodError_1.ZodIssueCode.too_small:
	            if (issue.type === "array")
	                message = `Array must contain ${issue.exact ? "exactly" : issue.inclusive ? `at least` : `more than`} ${issue.minimum} element(s)`;
	            else if (issue.type === "string")
	                message = `String must contain ${issue.exact ? "exactly" : issue.inclusive ? `at least` : `over`} ${issue.minimum} character(s)`;
	            else if (issue.type === "number")
	                message = `Number must be ${issue.exact
	                    ? `exactly equal to `
	                    : issue.inclusive
	                        ? `greater than or equal to `
	                        : `greater than `}${issue.minimum}`;
	            else if (issue.type === "date")
	                message = `Date must be ${issue.exact
	                    ? `exactly equal to `
	                    : issue.inclusive
	                        ? `greater than or equal to `
	                        : `greater than `}${new Date(Number(issue.minimum))}`;
	            else
	                message = "Invalid input";
	            break;
	        case ZodError_1.ZodIssueCode.too_big:
	            if (issue.type === "array")
	                message = `Array must contain ${issue.exact ? `exactly` : issue.inclusive ? `at most` : `less than`} ${issue.maximum} element(s)`;
	            else if (issue.type === "string")
	                message = `String must contain ${issue.exact ? `exactly` : issue.inclusive ? `at most` : `under`} ${issue.maximum} character(s)`;
	            else if (issue.type === "number")
	                message = `Number must be ${issue.exact
	                    ? `exactly`
	                    : issue.inclusive
	                        ? `less than or equal to`
	                        : `less than`} ${issue.maximum}`;
	            else if (issue.type === "bigint")
	                message = `BigInt must be ${issue.exact
	                    ? `exactly`
	                    : issue.inclusive
	                        ? `less than or equal to`
	                        : `less than`} ${issue.maximum}`;
	            else if (issue.type === "date")
	                message = `Date must be ${issue.exact
	                    ? `exactly`
	                    : issue.inclusive
	                        ? `smaller than or equal to`
	                        : `smaller than`} ${new Date(Number(issue.maximum))}`;
	            else
	                message = "Invalid input";
	            break;
	        case ZodError_1.ZodIssueCode.custom:
	            message = `Invalid input`;
	            break;
	        case ZodError_1.ZodIssueCode.invalid_intersection_types:
	            message = `Intersection results could not be merged`;
	            break;
	        case ZodError_1.ZodIssueCode.not_multiple_of:
	            message = `Number must be a multiple of ${issue.multipleOf}`;
	            break;
	        case ZodError_1.ZodIssueCode.not_finite:
	            message = "Number must be finite";
	            break;
	        default:
	            message = _ctx.defaultError;
	            util_1.util.assertNever(issue);
	    }
	    return { message };
	};
	en.default = errorMap;
	return en;
}

var hasRequiredErrors;

function requireErrors () {
	if (hasRequiredErrors) return errors;
	hasRequiredErrors = 1;
	var __importDefault = (errors && errors.__importDefault) || function (mod) {
	    return (mod && mod.__esModule) ? mod : { "default": mod };
	};
	Object.defineProperty(errors, "__esModule", { value: true });
	errors.getErrorMap = errors.setErrorMap = errors.defaultErrorMap = void 0;
	const en_1 = __importDefault(/*@__PURE__*/ requireEn());
	errors.defaultErrorMap = en_1.default;
	let overrideErrorMap = en_1.default;
	function setErrorMap(map) {
	    overrideErrorMap = map;
	}
	errors.setErrorMap = setErrorMap;
	function getErrorMap() {
	    return overrideErrorMap;
	}
	errors.getErrorMap = getErrorMap;
	return errors;
}

var parseUtil = {};

var hasRequiredParseUtil;

function requireParseUtil () {
	if (hasRequiredParseUtil) return parseUtil;
	hasRequiredParseUtil = 1;
	(function (exports) {
		var __importDefault = (parseUtil && parseUtil.__importDefault) || function (mod) {
		    return (mod && mod.__esModule) ? mod : { "default": mod };
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.isAsync = exports.isValid = exports.isDirty = exports.isAborted = exports.OK = exports.DIRTY = exports.INVALID = exports.ParseStatus = exports.addIssueToContext = exports.EMPTY_PATH = exports.makeIssue = void 0;
		const errors_1 = /*@__PURE__*/ requireErrors();
		const en_1 = __importDefault(/*@__PURE__*/ requireEn());
		const makeIssue = (params) => {
		    const { data, path, errorMaps, issueData } = params;
		    const fullPath = [...path, ...(issueData.path || [])];
		    const fullIssue = {
		        ...issueData,
		        path: fullPath,
		    };
		    if (issueData.message !== undefined) {
		        return {
		            ...issueData,
		            path: fullPath,
		            message: issueData.message,
		        };
		    }
		    let errorMessage = "";
		    const maps = errorMaps
		        .filter((m) => !!m)
		        .slice()
		        .reverse();
		    for (const map of maps) {
		        errorMessage = map(fullIssue, { data, defaultError: errorMessage }).message;
		    }
		    return {
		        ...issueData,
		        path: fullPath,
		        message: errorMessage,
		    };
		};
		exports.makeIssue = makeIssue;
		exports.EMPTY_PATH = [];
		function addIssueToContext(ctx, issueData) {
		    const overrideMap = (0, errors_1.getErrorMap)();
		    const issue = (0, exports.makeIssue)({
		        issueData: issueData,
		        data: ctx.data,
		        path: ctx.path,
		        errorMaps: [
		            ctx.common.contextualErrorMap,
		            ctx.schemaErrorMap,
		            overrideMap,
		            overrideMap === en_1.default ? undefined : en_1.default, // then global default map
		        ].filter((x) => !!x),
		    });
		    ctx.common.issues.push(issue);
		}
		exports.addIssueToContext = addIssueToContext;
		class ParseStatus {
		    constructor() {
		        this.value = "valid";
		    }
		    dirty() {
		        if (this.value === "valid")
		            this.value = "dirty";
		    }
		    abort() {
		        if (this.value !== "aborted")
		            this.value = "aborted";
		    }
		    static mergeArray(status, results) {
		        const arrayValue = [];
		        for (const s of results) {
		            if (s.status === "aborted")
		                return exports.INVALID;
		            if (s.status === "dirty")
		                status.dirty();
		            arrayValue.push(s.value);
		        }
		        return { status: status.value, value: arrayValue };
		    }
		    static async mergeObjectAsync(status, pairs) {
		        const syncPairs = [];
		        for (const pair of pairs) {
		            const key = await pair.key;
		            const value = await pair.value;
		            syncPairs.push({
		                key,
		                value,
		            });
		        }
		        return ParseStatus.mergeObjectSync(status, syncPairs);
		    }
		    static mergeObjectSync(status, pairs) {
		        const finalObject = {};
		        for (const pair of pairs) {
		            const { key, value } = pair;
		            if (key.status === "aborted")
		                return exports.INVALID;
		            if (value.status === "aborted")
		                return exports.INVALID;
		            if (key.status === "dirty")
		                status.dirty();
		            if (value.status === "dirty")
		                status.dirty();
		            if (key.value !== "__proto__" &&
		                (typeof value.value !== "undefined" || pair.alwaysSet)) {
		                finalObject[key.value] = value.value;
		            }
		        }
		        return { status: status.value, value: finalObject };
		    }
		}
		exports.ParseStatus = ParseStatus;
		exports.INVALID = Object.freeze({
		    status: "aborted",
		});
		const DIRTY = (value) => ({ status: "dirty", value });
		exports.DIRTY = DIRTY;
		const OK = (value) => ({ status: "valid", value });
		exports.OK = OK;
		const isAborted = (x) => x.status === "aborted";
		exports.isAborted = isAborted;
		const isDirty = (x) => x.status === "dirty";
		exports.isDirty = isDirty;
		const isValid = (x) => x.status === "valid";
		exports.isValid = isValid;
		const isAsync = (x) => typeof Promise !== "undefined" && x instanceof Promise;
		exports.isAsync = isAsync; 
	} (parseUtil));
	return parseUtil;
}

var typeAliases = {};

var hasRequiredTypeAliases;

function requireTypeAliases () {
	if (hasRequiredTypeAliases) return typeAliases;
	hasRequiredTypeAliases = 1;
	Object.defineProperty(typeAliases, "__esModule", { value: true });
	return typeAliases;
}

var types$1 = {};

var errorUtil = {};

var hasRequiredErrorUtil;

function requireErrorUtil () {
	if (hasRequiredErrorUtil) return errorUtil;
	hasRequiredErrorUtil = 1;
	(function (exports) {
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.errorUtil = void 0;
		(function (errorUtil) {
		    errorUtil.errToObj = (message) => typeof message === "string" ? { message } : message || {};
		    errorUtil.toString = (message) => typeof message === "string" ? message : message === null || message === void 0 ? void 0 : message.message;
		})(exports.errorUtil || (exports.errorUtil = {})); 
	} (errorUtil));
	return errorUtil;
}

var hasRequiredTypes;

function requireTypes () {
	if (hasRequiredTypes) return types$1;
	hasRequiredTypes = 1;
	(function (exports) {
		var __classPrivateFieldGet = (types$1 && types$1.__classPrivateFieldGet) || function (receiver, state, kind, f) {
		    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
		    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
		    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
		};
		var __classPrivateFieldSet = (types$1 && types$1.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
		    if (kind === "m") throw new TypeError("Private method is not writable");
		    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
		    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
		    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
		};
		var _ZodEnum_cache, _ZodNativeEnum_cache;
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.boolean = exports.bigint = exports.array = exports.any = exports.coerce = exports.ZodFirstPartyTypeKind = exports.late = exports.ZodSchema = exports.Schema = exports.custom = exports.ZodReadonly = exports.ZodPipeline = exports.ZodBranded = exports.BRAND = exports.ZodNaN = exports.ZodCatch = exports.ZodDefault = exports.ZodNullable = exports.ZodOptional = exports.ZodTransformer = exports.ZodEffects = exports.ZodPromise = exports.ZodNativeEnum = exports.ZodEnum = exports.ZodLiteral = exports.ZodLazy = exports.ZodFunction = exports.ZodSet = exports.ZodMap = exports.ZodRecord = exports.ZodTuple = exports.ZodIntersection = exports.ZodDiscriminatedUnion = exports.ZodUnion = exports.ZodObject = exports.ZodArray = exports.ZodVoid = exports.ZodNever = exports.ZodUnknown = exports.ZodAny = exports.ZodNull = exports.ZodUndefined = exports.ZodSymbol = exports.ZodDate = exports.ZodBoolean = exports.ZodBigInt = exports.ZodNumber = exports.ZodString = exports.datetimeRegex = exports.ZodType = void 0;
		exports.NEVER = exports.void = exports.unknown = exports.union = exports.undefined = exports.tuple = exports.transformer = exports.symbol = exports.string = exports.strictObject = exports.set = exports.record = exports.promise = exports.preprocess = exports.pipeline = exports.ostring = exports.optional = exports.onumber = exports.oboolean = exports.object = exports.number = exports.nullable = exports.null = exports.never = exports.nativeEnum = exports.nan = exports.map = exports.literal = exports.lazy = exports.intersection = exports.instanceof = exports.function = exports.enum = exports.effect = exports.discriminatedUnion = exports.date = void 0;
		const errors_1 = /*@__PURE__*/ requireErrors();
		const errorUtil_1 = /*@__PURE__*/ requireErrorUtil();
		const parseUtil_1 = /*@__PURE__*/ requireParseUtil();
		const util_1 = /*@__PURE__*/ requireUtil$1();
		const ZodError_1 = /*@__PURE__*/ requireZodError();
		class ParseInputLazyPath {
		    constructor(parent, value, path, key) {
		        this._cachedPath = [];
		        this.parent = parent;
		        this.data = value;
		        this._path = path;
		        this._key = key;
		    }
		    get path() {
		        if (!this._cachedPath.length) {
		            if (this._key instanceof Array) {
		                this._cachedPath.push(...this._path, ...this._key);
		            }
		            else {
		                this._cachedPath.push(...this._path, this._key);
		            }
		        }
		        return this._cachedPath;
		    }
		}
		const handleResult = (ctx, result) => {
		    if ((0, parseUtil_1.isValid)(result)) {
		        return { success: true, data: result.value };
		    }
		    else {
		        if (!ctx.common.issues.length) {
		            throw new Error("Validation failed but no issues detected.");
		        }
		        return {
		            success: false,
		            get error() {
		                if (this._error)
		                    return this._error;
		                const error = new ZodError_1.ZodError(ctx.common.issues);
		                this._error = error;
		                return this._error;
		            },
		        };
		    }
		};
		function processCreateParams(params) {
		    if (!params)
		        return {};
		    const { errorMap, invalid_type_error, required_error, description } = params;
		    if (errorMap && (invalid_type_error || required_error)) {
		        throw new Error(`Can't use "invalid_type_error" or "required_error" in conjunction with custom error map.`);
		    }
		    if (errorMap)
		        return { errorMap: errorMap, description };
		    const customMap = (iss, ctx) => {
		        var _a, _b;
		        const { message } = params;
		        if (iss.code === "invalid_enum_value") {
		            return { message: message !== null && message !== void 0 ? message : ctx.defaultError };
		        }
		        if (typeof ctx.data === "undefined") {
		            return { message: (_a = message !== null && message !== void 0 ? message : required_error) !== null && _a !== void 0 ? _a : ctx.defaultError };
		        }
		        if (iss.code !== "invalid_type")
		            return { message: ctx.defaultError };
		        return { message: (_b = message !== null && message !== void 0 ? message : invalid_type_error) !== null && _b !== void 0 ? _b : ctx.defaultError };
		    };
		    return { errorMap: customMap, description };
		}
		class ZodType {
		    constructor(def) {
		        /** Alias of safeParseAsync */
		        this.spa = this.safeParseAsync;
		        this._def = def;
		        this.parse = this.parse.bind(this);
		        this.safeParse = this.safeParse.bind(this);
		        this.parseAsync = this.parseAsync.bind(this);
		        this.safeParseAsync = this.safeParseAsync.bind(this);
		        this.spa = this.spa.bind(this);
		        this.refine = this.refine.bind(this);
		        this.refinement = this.refinement.bind(this);
		        this.superRefine = this.superRefine.bind(this);
		        this.optional = this.optional.bind(this);
		        this.nullable = this.nullable.bind(this);
		        this.nullish = this.nullish.bind(this);
		        this.array = this.array.bind(this);
		        this.promise = this.promise.bind(this);
		        this.or = this.or.bind(this);
		        this.and = this.and.bind(this);
		        this.transform = this.transform.bind(this);
		        this.brand = this.brand.bind(this);
		        this.default = this.default.bind(this);
		        this.catch = this.catch.bind(this);
		        this.describe = this.describe.bind(this);
		        this.pipe = this.pipe.bind(this);
		        this.readonly = this.readonly.bind(this);
		        this.isNullable = this.isNullable.bind(this);
		        this.isOptional = this.isOptional.bind(this);
		    }
		    get description() {
		        return this._def.description;
		    }
		    _getType(input) {
		        return (0, util_1.getParsedType)(input.data);
		    }
		    _getOrReturnCtx(input, ctx) {
		        return (ctx || {
		            common: input.parent.common,
		            data: input.data,
		            parsedType: (0, util_1.getParsedType)(input.data),
		            schemaErrorMap: this._def.errorMap,
		            path: input.path,
		            parent: input.parent,
		        });
		    }
		    _processInputParams(input) {
		        return {
		            status: new parseUtil_1.ParseStatus(),
		            ctx: {
		                common: input.parent.common,
		                data: input.data,
		                parsedType: (0, util_1.getParsedType)(input.data),
		                schemaErrorMap: this._def.errorMap,
		                path: input.path,
		                parent: input.parent,
		            },
		        };
		    }
		    _parseSync(input) {
		        const result = this._parse(input);
		        if ((0, parseUtil_1.isAsync)(result)) {
		            throw new Error("Synchronous parse encountered promise.");
		        }
		        return result;
		    }
		    _parseAsync(input) {
		        const result = this._parse(input);
		        return Promise.resolve(result);
		    }
		    parse(data, params) {
		        const result = this.safeParse(data, params);
		        if (result.success)
		            return result.data;
		        throw result.error;
		    }
		    safeParse(data, params) {
		        var _a;
		        const ctx = {
		            common: {
		                issues: [],
		                async: (_a = params === null || params === void 0 ? void 0 : params.async) !== null && _a !== void 0 ? _a : false,
		                contextualErrorMap: params === null || params === void 0 ? void 0 : params.errorMap,
		            },
		            path: (params === null || params === void 0 ? void 0 : params.path) || [],
		            schemaErrorMap: this._def.errorMap,
		            parent: null,
		            data,
		            parsedType: (0, util_1.getParsedType)(data),
		        };
		        const result = this._parseSync({ data, path: ctx.path, parent: ctx });
		        return handleResult(ctx, result);
		    }
		    async parseAsync(data, params) {
		        const result = await this.safeParseAsync(data, params);
		        if (result.success)
		            return result.data;
		        throw result.error;
		    }
		    async safeParseAsync(data, params) {
		        const ctx = {
		            common: {
		                issues: [],
		                contextualErrorMap: params === null || params === void 0 ? void 0 : params.errorMap,
		                async: true,
		            },
		            path: (params === null || params === void 0 ? void 0 : params.path) || [],
		            schemaErrorMap: this._def.errorMap,
		            parent: null,
		            data,
		            parsedType: (0, util_1.getParsedType)(data),
		        };
		        const maybeAsyncResult = this._parse({ data, path: ctx.path, parent: ctx });
		        const result = await ((0, parseUtil_1.isAsync)(maybeAsyncResult)
		            ? maybeAsyncResult
		            : Promise.resolve(maybeAsyncResult));
		        return handleResult(ctx, result);
		    }
		    refine(check, message) {
		        const getIssueProperties = (val) => {
		            if (typeof message === "string" || typeof message === "undefined") {
		                return { message };
		            }
		            else if (typeof message === "function") {
		                return message(val);
		            }
		            else {
		                return message;
		            }
		        };
		        return this._refinement((val, ctx) => {
		            const result = check(val);
		            const setError = () => ctx.addIssue({
		                code: ZodError_1.ZodIssueCode.custom,
		                ...getIssueProperties(val),
		            });
		            if (typeof Promise !== "undefined" && result instanceof Promise) {
		                return result.then((data) => {
		                    if (!data) {
		                        setError();
		                        return false;
		                    }
		                    else {
		                        return true;
		                    }
		                });
		            }
		            if (!result) {
		                setError();
		                return false;
		            }
		            else {
		                return true;
		            }
		        });
		    }
		    refinement(check, refinementData) {
		        return this._refinement((val, ctx) => {
		            if (!check(val)) {
		                ctx.addIssue(typeof refinementData === "function"
		                    ? refinementData(val, ctx)
		                    : refinementData);
		                return false;
		            }
		            else {
		                return true;
		            }
		        });
		    }
		    _refinement(refinement) {
		        return new ZodEffects({
		            schema: this,
		            typeName: ZodFirstPartyTypeKind.ZodEffects,
		            effect: { type: "refinement", refinement },
		        });
		    }
		    superRefine(refinement) {
		        return this._refinement(refinement);
		    }
		    optional() {
		        return ZodOptional.create(this, this._def);
		    }
		    nullable() {
		        return ZodNullable.create(this, this._def);
		    }
		    nullish() {
		        return this.nullable().optional();
		    }
		    array() {
		        return ZodArray.create(this, this._def);
		    }
		    promise() {
		        return ZodPromise.create(this, this._def);
		    }
		    or(option) {
		        return ZodUnion.create([this, option], this._def);
		    }
		    and(incoming) {
		        return ZodIntersection.create(this, incoming, this._def);
		    }
		    transform(transform) {
		        return new ZodEffects({
		            ...processCreateParams(this._def),
		            schema: this,
		            typeName: ZodFirstPartyTypeKind.ZodEffects,
		            effect: { type: "transform", transform },
		        });
		    }
		    default(def) {
		        const defaultValueFunc = typeof def === "function" ? def : () => def;
		        return new ZodDefault({
		            ...processCreateParams(this._def),
		            innerType: this,
		            defaultValue: defaultValueFunc,
		            typeName: ZodFirstPartyTypeKind.ZodDefault,
		        });
		    }
		    brand() {
		        return new ZodBranded({
		            typeName: ZodFirstPartyTypeKind.ZodBranded,
		            type: this,
		            ...processCreateParams(this._def),
		        });
		    }
		    catch(def) {
		        const catchValueFunc = typeof def === "function" ? def : () => def;
		        return new ZodCatch({
		            ...processCreateParams(this._def),
		            innerType: this,
		            catchValue: catchValueFunc,
		            typeName: ZodFirstPartyTypeKind.ZodCatch,
		        });
		    }
		    describe(description) {
		        const This = this.constructor;
		        return new This({
		            ...this._def,
		            description,
		        });
		    }
		    pipe(target) {
		        return ZodPipeline.create(this, target);
		    }
		    readonly() {
		        return ZodReadonly.create(this);
		    }
		    isOptional() {
		        return this.safeParse(undefined).success;
		    }
		    isNullable() {
		        return this.safeParse(null).success;
		    }
		}
		exports.ZodType = ZodType;
		exports.Schema = ZodType;
		exports.ZodSchema = ZodType;
		const cuidRegex = /^c[^\s-]{8,}$/i;
		const cuid2Regex = /^[0-9a-z]+$/;
		const ulidRegex = /^[0-9A-HJKMNP-TV-Z]{26}$/;
		// const uuidRegex =
		//   /^([a-f0-9]{8}-[a-f0-9]{4}-[1-5][a-f0-9]{3}-[a-f0-9]{4}-[a-f0-9]{12}|00000000-0000-0000-0000-000000000000)$/i;
		const uuidRegex = /^[0-9a-fA-F]{8}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{12}$/i;
		const nanoidRegex = /^[a-z0-9_-]{21}$/i;
		const durationRegex = /^[-+]?P(?!$)(?:(?:[-+]?\d+Y)|(?:[-+]?\d+[.,]\d+Y$))?(?:(?:[-+]?\d+M)|(?:[-+]?\d+[.,]\d+M$))?(?:(?:[-+]?\d+W)|(?:[-+]?\d+[.,]\d+W$))?(?:(?:[-+]?\d+D)|(?:[-+]?\d+[.,]\d+D$))?(?:T(?=[\d+-])(?:(?:[-+]?\d+H)|(?:[-+]?\d+[.,]\d+H$))?(?:(?:[-+]?\d+M)|(?:[-+]?\d+[.,]\d+M$))?(?:[-+]?\d+(?:[.,]\d+)?S)?)??$/;
		// from https://stackoverflow.com/a/46181/1550155
		// old version: too slow, didn't support unicode
		// const emailRegex = /^((([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+(\.([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+)*)|((\x22)((((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(([\x01-\x08\x0b\x0c\x0e-\x1f\x7f]|\x21|[\x23-\x5b]|[\x5d-\x7e]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(\\([\x01-\x09\x0b\x0c\x0d-\x7f]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]))))*(((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(\x22)))@((([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.)+(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))$/i;
		//old email regex
		// const emailRegex = /^(([^<>()[\].,;:\s@"]+(\.[^<>()[\].,;:\s@"]+)*)|(".+"))@((?!-)([^<>()[\].,;:\s@"]+\.)+[^<>()[\].,;:\s@"]{1,})[^-<>()[\].,;:\s@"]$/i;
		// eslint-disable-next-line
		// const emailRegex =
		//   /^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[(((25[0-5])|(2[0-4][0-9])|(1[0-9]{2})|([0-9]{1,2}))\.){3}((25[0-5])|(2[0-4][0-9])|(1[0-9]{2})|([0-9]{1,2}))\])|(\[IPv6:(([a-f0-9]{1,4}:){7}|::([a-f0-9]{1,4}:){0,6}|([a-f0-9]{1,4}:){1}:([a-f0-9]{1,4}:){0,5}|([a-f0-9]{1,4}:){2}:([a-f0-9]{1,4}:){0,4}|([a-f0-9]{1,4}:){3}:([a-f0-9]{1,4}:){0,3}|([a-f0-9]{1,4}:){4}:([a-f0-9]{1,4}:){0,2}|([a-f0-9]{1,4}:){5}:([a-f0-9]{1,4}:){0,1})([a-f0-9]{1,4}|(((25[0-5])|(2[0-4][0-9])|(1[0-9]{2})|([0-9]{1,2}))\.){3}((25[0-5])|(2[0-4][0-9])|(1[0-9]{2})|([0-9]{1,2})))\])|([A-Za-z0-9]([A-Za-z0-9-]*[A-Za-z0-9])*(\.[A-Za-z]{2,})+))$/;
		// const emailRegex =
		//   /^[a-zA-Z0-9\.\!\#\$\%\&\'\*\+\/\=\?\^\_\`\{\|\}\~\-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)*$/;
		// const emailRegex =
		//   /^(?:[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*|"(?:[\x01-\x08\x0b\x0c\x0e-\x1f\x21\x23-\x5b\x5d-\x7f]|\\[\x01-\x09\x0b\x0c\x0e-\x7f])*")@(?:(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?|\[(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?|[a-z0-9-]*[a-z0-9]:(?:[\x01-\x08\x0b\x0c\x0e-\x1f\x21-\x5a\x53-\x7f]|\\[\x01-\x09\x0b\x0c\x0e-\x7f])+)\])$/i;
		const emailRegex = /^(?!\.)(?!.*\.\.)([A-Z0-9_'+\-\.]*)[A-Z0-9_+-]@([A-Z0-9][A-Z0-9\-]*\.)+[A-Z]{2,}$/i;
		// const emailRegex =
		//   /^[a-z0-9.!#$%&’*+/=?^_`{|}~-]+@[a-z0-9-]+(?:\.[a-z0-9\-]+)*$/i;
		// from https://thekevinscott.com/emojis-in-javascript/#writing-a-regular-expression
		const _emojiRegex = `^(\\p{Extended_Pictographic}|\\p{Emoji_Component})+$`;
		let emojiRegex;
		// faster, simpler, safer
		const ipv4Regex = /^(?:(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]|[0-9])\.){3}(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]|[0-9])$/;
		const ipv6Regex = /^(([a-f0-9]{1,4}:){7}|::([a-f0-9]{1,4}:){0,6}|([a-f0-9]{1,4}:){1}:([a-f0-9]{1,4}:){0,5}|([a-f0-9]{1,4}:){2}:([a-f0-9]{1,4}:){0,4}|([a-f0-9]{1,4}:){3}:([a-f0-9]{1,4}:){0,3}|([a-f0-9]{1,4}:){4}:([a-f0-9]{1,4}:){0,2}|([a-f0-9]{1,4}:){5}:([a-f0-9]{1,4}:){0,1})([a-f0-9]{1,4}|(((25[0-5])|(2[0-4][0-9])|(1[0-9]{2})|([0-9]{1,2}))\.){3}((25[0-5])|(2[0-4][0-9])|(1[0-9]{2})|([0-9]{1,2})))$/;
		// https://stackoverflow.com/questions/7860392/determine-if-string-is-in-base64-using-javascript
		const base64Regex = /^([0-9a-zA-Z+/]{4})*(([0-9a-zA-Z+/]{2}==)|([0-9a-zA-Z+/]{3}=))?$/;
		// simple
		// const dateRegexSource = `\\d{4}-\\d{2}-\\d{2}`;
		// no leap year validation
		// const dateRegexSource = `\\d{4}-((0[13578]|10|12)-31|(0[13-9]|1[0-2])-30|(0[1-9]|1[0-2])-(0[1-9]|1\\d|2\\d))`;
		// with leap year validation
		const dateRegexSource = `((\\d\\d[2468][048]|\\d\\d[13579][26]|\\d\\d0[48]|[02468][048]00|[13579][26]00)-02-29|\\d{4}-((0[13578]|1[02])-(0[1-9]|[12]\\d|3[01])|(0[469]|11)-(0[1-9]|[12]\\d|30)|(02)-(0[1-9]|1\\d|2[0-8])))`;
		const dateRegex = new RegExp(`^${dateRegexSource}$`);
		function timeRegexSource(args) {
		    // let regex = `\\d{2}:\\d{2}:\\d{2}`;
		    let regex = `([01]\\d|2[0-3]):[0-5]\\d:[0-5]\\d`;
		    if (args.precision) {
		        regex = `${regex}\\.\\d{${args.precision}}`;
		    }
		    else if (args.precision == null) {
		        regex = `${regex}(\\.\\d+)?`;
		    }
		    return regex;
		}
		function timeRegex(args) {
		    return new RegExp(`^${timeRegexSource(args)}$`);
		}
		// Adapted from https://stackoverflow.com/a/3143231
		function datetimeRegex(args) {
		    let regex = `${dateRegexSource}T${timeRegexSource(args)}`;
		    const opts = [];
		    opts.push(args.local ? `Z?` : `Z`);
		    if (args.offset)
		        opts.push(`([+-]\\d{2}:?\\d{2})`);
		    regex = `${regex}(${opts.join("|")})`;
		    return new RegExp(`^${regex}$`);
		}
		exports.datetimeRegex = datetimeRegex;
		function isValidIP(ip, version) {
		    if ((version === "v4" || !version) && ipv4Regex.test(ip)) {
		        return true;
		    }
		    if ((version === "v6" || !version) && ipv6Regex.test(ip)) {
		        return true;
		    }
		    return false;
		}
		class ZodString extends ZodType {
		    _parse(input) {
		        if (this._def.coerce) {
		            input.data = String(input.data);
		        }
		        const parsedType = this._getType(input);
		        if (parsedType !== util_1.ZodParsedType.string) {
		            const ctx = this._getOrReturnCtx(input);
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                code: ZodError_1.ZodIssueCode.invalid_type,
		                expected: util_1.ZodParsedType.string,
		                received: ctx.parsedType,
		            });
		            return parseUtil_1.INVALID;
		        }
		        const status = new parseUtil_1.ParseStatus();
		        let ctx = undefined;
		        for (const check of this._def.checks) {
		            if (check.kind === "min") {
		                if (input.data.length < check.value) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        code: ZodError_1.ZodIssueCode.too_small,
		                        minimum: check.value,
		                        type: "string",
		                        inclusive: true,
		                        exact: false,
		                        message: check.message,
		                    });
		                    status.dirty();
		                }
		            }
		            else if (check.kind === "max") {
		                if (input.data.length > check.value) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        code: ZodError_1.ZodIssueCode.too_big,
		                        maximum: check.value,
		                        type: "string",
		                        inclusive: true,
		                        exact: false,
		                        message: check.message,
		                    });
		                    status.dirty();
		                }
		            }
		            else if (check.kind === "length") {
		                const tooBig = input.data.length > check.value;
		                const tooSmall = input.data.length < check.value;
		                if (tooBig || tooSmall) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    if (tooBig) {
		                        (0, parseUtil_1.addIssueToContext)(ctx, {
		                            code: ZodError_1.ZodIssueCode.too_big,
		                            maximum: check.value,
		                            type: "string",
		                            inclusive: true,
		                            exact: true,
		                            message: check.message,
		                        });
		                    }
		                    else if (tooSmall) {
		                        (0, parseUtil_1.addIssueToContext)(ctx, {
		                            code: ZodError_1.ZodIssueCode.too_small,
		                            minimum: check.value,
		                            type: "string",
		                            inclusive: true,
		                            exact: true,
		                            message: check.message,
		                        });
		                    }
		                    status.dirty();
		                }
		            }
		            else if (check.kind === "email") {
		                if (!emailRegex.test(input.data)) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        validation: "email",
		                        code: ZodError_1.ZodIssueCode.invalid_string,
		                        message: check.message,
		                    });
		                    status.dirty();
		                }
		            }
		            else if (check.kind === "emoji") {
		                if (!emojiRegex) {
		                    emojiRegex = new RegExp(_emojiRegex, "u");
		                }
		                if (!emojiRegex.test(input.data)) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        validation: "emoji",
		                        code: ZodError_1.ZodIssueCode.invalid_string,
		                        message: check.message,
		                    });
		                    status.dirty();
		                }
		            }
		            else if (check.kind === "uuid") {
		                if (!uuidRegex.test(input.data)) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        validation: "uuid",
		                        code: ZodError_1.ZodIssueCode.invalid_string,
		                        message: check.message,
		                    });
		                    status.dirty();
		                }
		            }
		            else if (check.kind === "nanoid") {
		                if (!nanoidRegex.test(input.data)) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        validation: "nanoid",
		                        code: ZodError_1.ZodIssueCode.invalid_string,
		                        message: check.message,
		                    });
		                    status.dirty();
		                }
		            }
		            else if (check.kind === "cuid") {
		                if (!cuidRegex.test(input.data)) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        validation: "cuid",
		                        code: ZodError_1.ZodIssueCode.invalid_string,
		                        message: check.message,
		                    });
		                    status.dirty();
		                }
		            }
		            else if (check.kind === "cuid2") {
		                if (!cuid2Regex.test(input.data)) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        validation: "cuid2",
		                        code: ZodError_1.ZodIssueCode.invalid_string,
		                        message: check.message,
		                    });
		                    status.dirty();
		                }
		            }
		            else if (check.kind === "ulid") {
		                if (!ulidRegex.test(input.data)) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        validation: "ulid",
		                        code: ZodError_1.ZodIssueCode.invalid_string,
		                        message: check.message,
		                    });
		                    status.dirty();
		                }
		            }
		            else if (check.kind === "url") {
		                try {
		                    new URL(input.data);
		                }
		                catch (_a) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        validation: "url",
		                        code: ZodError_1.ZodIssueCode.invalid_string,
		                        message: check.message,
		                    });
		                    status.dirty();
		                }
		            }
		            else if (check.kind === "regex") {
		                check.regex.lastIndex = 0;
		                const testResult = check.regex.test(input.data);
		                if (!testResult) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        validation: "regex",
		                        code: ZodError_1.ZodIssueCode.invalid_string,
		                        message: check.message,
		                    });
		                    status.dirty();
		                }
		            }
		            else if (check.kind === "trim") {
		                input.data = input.data.trim();
		            }
		            else if (check.kind === "includes") {
		                if (!input.data.includes(check.value, check.position)) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        code: ZodError_1.ZodIssueCode.invalid_string,
		                        validation: { includes: check.value, position: check.position },
		                        message: check.message,
		                    });
		                    status.dirty();
		                }
		            }
		            else if (check.kind === "toLowerCase") {
		                input.data = input.data.toLowerCase();
		            }
		            else if (check.kind === "toUpperCase") {
		                input.data = input.data.toUpperCase();
		            }
		            else if (check.kind === "startsWith") {
		                if (!input.data.startsWith(check.value)) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        code: ZodError_1.ZodIssueCode.invalid_string,
		                        validation: { startsWith: check.value },
		                        message: check.message,
		                    });
		                    status.dirty();
		                }
		            }
		            else if (check.kind === "endsWith") {
		                if (!input.data.endsWith(check.value)) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        code: ZodError_1.ZodIssueCode.invalid_string,
		                        validation: { endsWith: check.value },
		                        message: check.message,
		                    });
		                    status.dirty();
		                }
		            }
		            else if (check.kind === "datetime") {
		                const regex = datetimeRegex(check);
		                if (!regex.test(input.data)) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        code: ZodError_1.ZodIssueCode.invalid_string,
		                        validation: "datetime",
		                        message: check.message,
		                    });
		                    status.dirty();
		                }
		            }
		            else if (check.kind === "date") {
		                const regex = dateRegex;
		                if (!regex.test(input.data)) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        code: ZodError_1.ZodIssueCode.invalid_string,
		                        validation: "date",
		                        message: check.message,
		                    });
		                    status.dirty();
		                }
		            }
		            else if (check.kind === "time") {
		                const regex = timeRegex(check);
		                if (!regex.test(input.data)) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        code: ZodError_1.ZodIssueCode.invalid_string,
		                        validation: "time",
		                        message: check.message,
		                    });
		                    status.dirty();
		                }
		            }
		            else if (check.kind === "duration") {
		                if (!durationRegex.test(input.data)) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        validation: "duration",
		                        code: ZodError_1.ZodIssueCode.invalid_string,
		                        message: check.message,
		                    });
		                    status.dirty();
		                }
		            }
		            else if (check.kind === "ip") {
		                if (!isValidIP(input.data, check.version)) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        validation: "ip",
		                        code: ZodError_1.ZodIssueCode.invalid_string,
		                        message: check.message,
		                    });
		                    status.dirty();
		                }
		            }
		            else if (check.kind === "base64") {
		                if (!base64Regex.test(input.data)) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        validation: "base64",
		                        code: ZodError_1.ZodIssueCode.invalid_string,
		                        message: check.message,
		                    });
		                    status.dirty();
		                }
		            }
		            else {
		                util_1.util.assertNever(check);
		            }
		        }
		        return { status: status.value, value: input.data };
		    }
		    _regex(regex, validation, message) {
		        return this.refinement((data) => regex.test(data), {
		            validation,
		            code: ZodError_1.ZodIssueCode.invalid_string,
		            ...errorUtil_1.errorUtil.errToObj(message),
		        });
		    }
		    _addCheck(check) {
		        return new ZodString({
		            ...this._def,
		            checks: [...this._def.checks, check],
		        });
		    }
		    email(message) {
		        return this._addCheck({ kind: "email", ...errorUtil_1.errorUtil.errToObj(message) });
		    }
		    url(message) {
		        return this._addCheck({ kind: "url", ...errorUtil_1.errorUtil.errToObj(message) });
		    }
		    emoji(message) {
		        return this._addCheck({ kind: "emoji", ...errorUtil_1.errorUtil.errToObj(message) });
		    }
		    uuid(message) {
		        return this._addCheck({ kind: "uuid", ...errorUtil_1.errorUtil.errToObj(message) });
		    }
		    nanoid(message) {
		        return this._addCheck({ kind: "nanoid", ...errorUtil_1.errorUtil.errToObj(message) });
		    }
		    cuid(message) {
		        return this._addCheck({ kind: "cuid", ...errorUtil_1.errorUtil.errToObj(message) });
		    }
		    cuid2(message) {
		        return this._addCheck({ kind: "cuid2", ...errorUtil_1.errorUtil.errToObj(message) });
		    }
		    ulid(message) {
		        return this._addCheck({ kind: "ulid", ...errorUtil_1.errorUtil.errToObj(message) });
		    }
		    base64(message) {
		        return this._addCheck({ kind: "base64", ...errorUtil_1.errorUtil.errToObj(message) });
		    }
		    ip(options) {
		        return this._addCheck({ kind: "ip", ...errorUtil_1.errorUtil.errToObj(options) });
		    }
		    datetime(options) {
		        var _a, _b;
		        if (typeof options === "string") {
		            return this._addCheck({
		                kind: "datetime",
		                precision: null,
		                offset: false,
		                local: false,
		                message: options,
		            });
		        }
		        return this._addCheck({
		            kind: "datetime",
		            precision: typeof (options === null || options === void 0 ? void 0 : options.precision) === "undefined" ? null : options === null || options === void 0 ? void 0 : options.precision,
		            offset: (_a = options === null || options === void 0 ? void 0 : options.offset) !== null && _a !== void 0 ? _a : false,
		            local: (_b = options === null || options === void 0 ? void 0 : options.local) !== null && _b !== void 0 ? _b : false,
		            ...errorUtil_1.errorUtil.errToObj(options === null || options === void 0 ? void 0 : options.message),
		        });
		    }
		    date(message) {
		        return this._addCheck({ kind: "date", message });
		    }
		    time(options) {
		        if (typeof options === "string") {
		            return this._addCheck({
		                kind: "time",
		                precision: null,
		                message: options,
		            });
		        }
		        return this._addCheck({
		            kind: "time",
		            precision: typeof (options === null || options === void 0 ? void 0 : options.precision) === "undefined" ? null : options === null || options === void 0 ? void 0 : options.precision,
		            ...errorUtil_1.errorUtil.errToObj(options === null || options === void 0 ? void 0 : options.message),
		        });
		    }
		    duration(message) {
		        return this._addCheck({ kind: "duration", ...errorUtil_1.errorUtil.errToObj(message) });
		    }
		    regex(regex, message) {
		        return this._addCheck({
		            kind: "regex",
		            regex: regex,
		            ...errorUtil_1.errorUtil.errToObj(message),
		        });
		    }
		    includes(value, options) {
		        return this._addCheck({
		            kind: "includes",
		            value: value,
		            position: options === null || options === void 0 ? void 0 : options.position,
		            ...errorUtil_1.errorUtil.errToObj(options === null || options === void 0 ? void 0 : options.message),
		        });
		    }
		    startsWith(value, message) {
		        return this._addCheck({
		            kind: "startsWith",
		            value: value,
		            ...errorUtil_1.errorUtil.errToObj(message),
		        });
		    }
		    endsWith(value, message) {
		        return this._addCheck({
		            kind: "endsWith",
		            value: value,
		            ...errorUtil_1.errorUtil.errToObj(message),
		        });
		    }
		    min(minLength, message) {
		        return this._addCheck({
		            kind: "min",
		            value: minLength,
		            ...errorUtil_1.errorUtil.errToObj(message),
		        });
		    }
		    max(maxLength, message) {
		        return this._addCheck({
		            kind: "max",
		            value: maxLength,
		            ...errorUtil_1.errorUtil.errToObj(message),
		        });
		    }
		    length(len, message) {
		        return this._addCheck({
		            kind: "length",
		            value: len,
		            ...errorUtil_1.errorUtil.errToObj(message),
		        });
		    }
		    /**
		     * @deprecated Use z.string().min(1) instead.
		     * @see {@link ZodString.min}
		     */
		    nonempty(message) {
		        return this.min(1, errorUtil_1.errorUtil.errToObj(message));
		    }
		    trim() {
		        return new ZodString({
		            ...this._def,
		            checks: [...this._def.checks, { kind: "trim" }],
		        });
		    }
		    toLowerCase() {
		        return new ZodString({
		            ...this._def,
		            checks: [...this._def.checks, { kind: "toLowerCase" }],
		        });
		    }
		    toUpperCase() {
		        return new ZodString({
		            ...this._def,
		            checks: [...this._def.checks, { kind: "toUpperCase" }],
		        });
		    }
		    get isDatetime() {
		        return !!this._def.checks.find((ch) => ch.kind === "datetime");
		    }
		    get isDate() {
		        return !!this._def.checks.find((ch) => ch.kind === "date");
		    }
		    get isTime() {
		        return !!this._def.checks.find((ch) => ch.kind === "time");
		    }
		    get isDuration() {
		        return !!this._def.checks.find((ch) => ch.kind === "duration");
		    }
		    get isEmail() {
		        return !!this._def.checks.find((ch) => ch.kind === "email");
		    }
		    get isURL() {
		        return !!this._def.checks.find((ch) => ch.kind === "url");
		    }
		    get isEmoji() {
		        return !!this._def.checks.find((ch) => ch.kind === "emoji");
		    }
		    get isUUID() {
		        return !!this._def.checks.find((ch) => ch.kind === "uuid");
		    }
		    get isNANOID() {
		        return !!this._def.checks.find((ch) => ch.kind === "nanoid");
		    }
		    get isCUID() {
		        return !!this._def.checks.find((ch) => ch.kind === "cuid");
		    }
		    get isCUID2() {
		        return !!this._def.checks.find((ch) => ch.kind === "cuid2");
		    }
		    get isULID() {
		        return !!this._def.checks.find((ch) => ch.kind === "ulid");
		    }
		    get isIP() {
		        return !!this._def.checks.find((ch) => ch.kind === "ip");
		    }
		    get isBase64() {
		        return !!this._def.checks.find((ch) => ch.kind === "base64");
		    }
		    get minLength() {
		        let min = null;
		        for (const ch of this._def.checks) {
		            if (ch.kind === "min") {
		                if (min === null || ch.value > min)
		                    min = ch.value;
		            }
		        }
		        return min;
		    }
		    get maxLength() {
		        let max = null;
		        for (const ch of this._def.checks) {
		            if (ch.kind === "max") {
		                if (max === null || ch.value < max)
		                    max = ch.value;
		            }
		        }
		        return max;
		    }
		}
		exports.ZodString = ZodString;
		ZodString.create = (params) => {
		    var _a;
		    return new ZodString({
		        checks: [],
		        typeName: ZodFirstPartyTypeKind.ZodString,
		        coerce: (_a = params === null || params === void 0 ? void 0 : params.coerce) !== null && _a !== void 0 ? _a : false,
		        ...processCreateParams(params),
		    });
		};
		// https://stackoverflow.com/questions/3966484/why-does-modulus-operator-return-fractional-number-in-javascript/31711034#31711034
		function floatSafeRemainder(val, step) {
		    const valDecCount = (val.toString().split(".")[1] || "").length;
		    const stepDecCount = (step.toString().split(".")[1] || "").length;
		    const decCount = valDecCount > stepDecCount ? valDecCount : stepDecCount;
		    const valInt = parseInt(val.toFixed(decCount).replace(".", ""));
		    const stepInt = parseInt(step.toFixed(decCount).replace(".", ""));
		    return (valInt % stepInt) / Math.pow(10, decCount);
		}
		class ZodNumber extends ZodType {
		    constructor() {
		        super(...arguments);
		        this.min = this.gte;
		        this.max = this.lte;
		        this.step = this.multipleOf;
		    }
		    _parse(input) {
		        if (this._def.coerce) {
		            input.data = Number(input.data);
		        }
		        const parsedType = this._getType(input);
		        if (parsedType !== util_1.ZodParsedType.number) {
		            const ctx = this._getOrReturnCtx(input);
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                code: ZodError_1.ZodIssueCode.invalid_type,
		                expected: util_1.ZodParsedType.number,
		                received: ctx.parsedType,
		            });
		            return parseUtil_1.INVALID;
		        }
		        let ctx = undefined;
		        const status = new parseUtil_1.ParseStatus();
		        for (const check of this._def.checks) {
		            if (check.kind === "int") {
		                if (!util_1.util.isInteger(input.data)) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        code: ZodError_1.ZodIssueCode.invalid_type,
		                        expected: "integer",
		                        received: "float",
		                        message: check.message,
		                    });
		                    status.dirty();
		                }
		            }
		            else if (check.kind === "min") {
		                const tooSmall = check.inclusive
		                    ? input.data < check.value
		                    : input.data <= check.value;
		                if (tooSmall) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        code: ZodError_1.ZodIssueCode.too_small,
		                        minimum: check.value,
		                        type: "number",
		                        inclusive: check.inclusive,
		                        exact: false,
		                        message: check.message,
		                    });
		                    status.dirty();
		                }
		            }
		            else if (check.kind === "max") {
		                const tooBig = check.inclusive
		                    ? input.data > check.value
		                    : input.data >= check.value;
		                if (tooBig) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        code: ZodError_1.ZodIssueCode.too_big,
		                        maximum: check.value,
		                        type: "number",
		                        inclusive: check.inclusive,
		                        exact: false,
		                        message: check.message,
		                    });
		                    status.dirty();
		                }
		            }
		            else if (check.kind === "multipleOf") {
		                if (floatSafeRemainder(input.data, check.value) !== 0) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        code: ZodError_1.ZodIssueCode.not_multiple_of,
		                        multipleOf: check.value,
		                        message: check.message,
		                    });
		                    status.dirty();
		                }
		            }
		            else if (check.kind === "finite") {
		                if (!Number.isFinite(input.data)) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        code: ZodError_1.ZodIssueCode.not_finite,
		                        message: check.message,
		                    });
		                    status.dirty();
		                }
		            }
		            else {
		                util_1.util.assertNever(check);
		            }
		        }
		        return { status: status.value, value: input.data };
		    }
		    gte(value, message) {
		        return this.setLimit("min", value, true, errorUtil_1.errorUtil.toString(message));
		    }
		    gt(value, message) {
		        return this.setLimit("min", value, false, errorUtil_1.errorUtil.toString(message));
		    }
		    lte(value, message) {
		        return this.setLimit("max", value, true, errorUtil_1.errorUtil.toString(message));
		    }
		    lt(value, message) {
		        return this.setLimit("max", value, false, errorUtil_1.errorUtil.toString(message));
		    }
		    setLimit(kind, value, inclusive, message) {
		        return new ZodNumber({
		            ...this._def,
		            checks: [
		                ...this._def.checks,
		                {
		                    kind,
		                    value,
		                    inclusive,
		                    message: errorUtil_1.errorUtil.toString(message),
		                },
		            ],
		        });
		    }
		    _addCheck(check) {
		        return new ZodNumber({
		            ...this._def,
		            checks: [...this._def.checks, check],
		        });
		    }
		    int(message) {
		        return this._addCheck({
		            kind: "int",
		            message: errorUtil_1.errorUtil.toString(message),
		        });
		    }
		    positive(message) {
		        return this._addCheck({
		            kind: "min",
		            value: 0,
		            inclusive: false,
		            message: errorUtil_1.errorUtil.toString(message),
		        });
		    }
		    negative(message) {
		        return this._addCheck({
		            kind: "max",
		            value: 0,
		            inclusive: false,
		            message: errorUtil_1.errorUtil.toString(message),
		        });
		    }
		    nonpositive(message) {
		        return this._addCheck({
		            kind: "max",
		            value: 0,
		            inclusive: true,
		            message: errorUtil_1.errorUtil.toString(message),
		        });
		    }
		    nonnegative(message) {
		        return this._addCheck({
		            kind: "min",
		            value: 0,
		            inclusive: true,
		            message: errorUtil_1.errorUtil.toString(message),
		        });
		    }
		    multipleOf(value, message) {
		        return this._addCheck({
		            kind: "multipleOf",
		            value: value,
		            message: errorUtil_1.errorUtil.toString(message),
		        });
		    }
		    finite(message) {
		        return this._addCheck({
		            kind: "finite",
		            message: errorUtil_1.errorUtil.toString(message),
		        });
		    }
		    safe(message) {
		        return this._addCheck({
		            kind: "min",
		            inclusive: true,
		            value: Number.MIN_SAFE_INTEGER,
		            message: errorUtil_1.errorUtil.toString(message),
		        })._addCheck({
		            kind: "max",
		            inclusive: true,
		            value: Number.MAX_SAFE_INTEGER,
		            message: errorUtil_1.errorUtil.toString(message),
		        });
		    }
		    get minValue() {
		        let min = null;
		        for (const ch of this._def.checks) {
		            if (ch.kind === "min") {
		                if (min === null || ch.value > min)
		                    min = ch.value;
		            }
		        }
		        return min;
		    }
		    get maxValue() {
		        let max = null;
		        for (const ch of this._def.checks) {
		            if (ch.kind === "max") {
		                if (max === null || ch.value < max)
		                    max = ch.value;
		            }
		        }
		        return max;
		    }
		    get isInt() {
		        return !!this._def.checks.find((ch) => ch.kind === "int" ||
		            (ch.kind === "multipleOf" && util_1.util.isInteger(ch.value)));
		    }
		    get isFinite() {
		        let max = null, min = null;
		        for (const ch of this._def.checks) {
		            if (ch.kind === "finite" ||
		                ch.kind === "int" ||
		                ch.kind === "multipleOf") {
		                return true;
		            }
		            else if (ch.kind === "min") {
		                if (min === null || ch.value > min)
		                    min = ch.value;
		            }
		            else if (ch.kind === "max") {
		                if (max === null || ch.value < max)
		                    max = ch.value;
		            }
		        }
		        return Number.isFinite(min) && Number.isFinite(max);
		    }
		}
		exports.ZodNumber = ZodNumber;
		ZodNumber.create = (params) => {
		    return new ZodNumber({
		        checks: [],
		        typeName: ZodFirstPartyTypeKind.ZodNumber,
		        coerce: (params === null || params === void 0 ? void 0 : params.coerce) || false,
		        ...processCreateParams(params),
		    });
		};
		class ZodBigInt extends ZodType {
		    constructor() {
		        super(...arguments);
		        this.min = this.gte;
		        this.max = this.lte;
		    }
		    _parse(input) {
		        if (this._def.coerce) {
		            input.data = BigInt(input.data);
		        }
		        const parsedType = this._getType(input);
		        if (parsedType !== util_1.ZodParsedType.bigint) {
		            const ctx = this._getOrReturnCtx(input);
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                code: ZodError_1.ZodIssueCode.invalid_type,
		                expected: util_1.ZodParsedType.bigint,
		                received: ctx.parsedType,
		            });
		            return parseUtil_1.INVALID;
		        }
		        let ctx = undefined;
		        const status = new parseUtil_1.ParseStatus();
		        for (const check of this._def.checks) {
		            if (check.kind === "min") {
		                const tooSmall = check.inclusive
		                    ? input.data < check.value
		                    : input.data <= check.value;
		                if (tooSmall) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        code: ZodError_1.ZodIssueCode.too_small,
		                        type: "bigint",
		                        minimum: check.value,
		                        inclusive: check.inclusive,
		                        message: check.message,
		                    });
		                    status.dirty();
		                }
		            }
		            else if (check.kind === "max") {
		                const tooBig = check.inclusive
		                    ? input.data > check.value
		                    : input.data >= check.value;
		                if (tooBig) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        code: ZodError_1.ZodIssueCode.too_big,
		                        type: "bigint",
		                        maximum: check.value,
		                        inclusive: check.inclusive,
		                        message: check.message,
		                    });
		                    status.dirty();
		                }
		            }
		            else if (check.kind === "multipleOf") {
		                if (input.data % check.value !== BigInt(0)) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        code: ZodError_1.ZodIssueCode.not_multiple_of,
		                        multipleOf: check.value,
		                        message: check.message,
		                    });
		                    status.dirty();
		                }
		            }
		            else {
		                util_1.util.assertNever(check);
		            }
		        }
		        return { status: status.value, value: input.data };
		    }
		    gte(value, message) {
		        return this.setLimit("min", value, true, errorUtil_1.errorUtil.toString(message));
		    }
		    gt(value, message) {
		        return this.setLimit("min", value, false, errorUtil_1.errorUtil.toString(message));
		    }
		    lte(value, message) {
		        return this.setLimit("max", value, true, errorUtil_1.errorUtil.toString(message));
		    }
		    lt(value, message) {
		        return this.setLimit("max", value, false, errorUtil_1.errorUtil.toString(message));
		    }
		    setLimit(kind, value, inclusive, message) {
		        return new ZodBigInt({
		            ...this._def,
		            checks: [
		                ...this._def.checks,
		                {
		                    kind,
		                    value,
		                    inclusive,
		                    message: errorUtil_1.errorUtil.toString(message),
		                },
		            ],
		        });
		    }
		    _addCheck(check) {
		        return new ZodBigInt({
		            ...this._def,
		            checks: [...this._def.checks, check],
		        });
		    }
		    positive(message) {
		        return this._addCheck({
		            kind: "min",
		            value: BigInt(0),
		            inclusive: false,
		            message: errorUtil_1.errorUtil.toString(message),
		        });
		    }
		    negative(message) {
		        return this._addCheck({
		            kind: "max",
		            value: BigInt(0),
		            inclusive: false,
		            message: errorUtil_1.errorUtil.toString(message),
		        });
		    }
		    nonpositive(message) {
		        return this._addCheck({
		            kind: "max",
		            value: BigInt(0),
		            inclusive: true,
		            message: errorUtil_1.errorUtil.toString(message),
		        });
		    }
		    nonnegative(message) {
		        return this._addCheck({
		            kind: "min",
		            value: BigInt(0),
		            inclusive: true,
		            message: errorUtil_1.errorUtil.toString(message),
		        });
		    }
		    multipleOf(value, message) {
		        return this._addCheck({
		            kind: "multipleOf",
		            value,
		            message: errorUtil_1.errorUtil.toString(message),
		        });
		    }
		    get minValue() {
		        let min = null;
		        for (const ch of this._def.checks) {
		            if (ch.kind === "min") {
		                if (min === null || ch.value > min)
		                    min = ch.value;
		            }
		        }
		        return min;
		    }
		    get maxValue() {
		        let max = null;
		        for (const ch of this._def.checks) {
		            if (ch.kind === "max") {
		                if (max === null || ch.value < max)
		                    max = ch.value;
		            }
		        }
		        return max;
		    }
		}
		exports.ZodBigInt = ZodBigInt;
		ZodBigInt.create = (params) => {
		    var _a;
		    return new ZodBigInt({
		        checks: [],
		        typeName: ZodFirstPartyTypeKind.ZodBigInt,
		        coerce: (_a = params === null || params === void 0 ? void 0 : params.coerce) !== null && _a !== void 0 ? _a : false,
		        ...processCreateParams(params),
		    });
		};
		class ZodBoolean extends ZodType {
		    _parse(input) {
		        if (this._def.coerce) {
		            input.data = Boolean(input.data);
		        }
		        const parsedType = this._getType(input);
		        if (parsedType !== util_1.ZodParsedType.boolean) {
		            const ctx = this._getOrReturnCtx(input);
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                code: ZodError_1.ZodIssueCode.invalid_type,
		                expected: util_1.ZodParsedType.boolean,
		                received: ctx.parsedType,
		            });
		            return parseUtil_1.INVALID;
		        }
		        return (0, parseUtil_1.OK)(input.data);
		    }
		}
		exports.ZodBoolean = ZodBoolean;
		ZodBoolean.create = (params) => {
		    return new ZodBoolean({
		        typeName: ZodFirstPartyTypeKind.ZodBoolean,
		        coerce: (params === null || params === void 0 ? void 0 : params.coerce) || false,
		        ...processCreateParams(params),
		    });
		};
		class ZodDate extends ZodType {
		    _parse(input) {
		        if (this._def.coerce) {
		            input.data = new Date(input.data);
		        }
		        const parsedType = this._getType(input);
		        if (parsedType !== util_1.ZodParsedType.date) {
		            const ctx = this._getOrReturnCtx(input);
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                code: ZodError_1.ZodIssueCode.invalid_type,
		                expected: util_1.ZodParsedType.date,
		                received: ctx.parsedType,
		            });
		            return parseUtil_1.INVALID;
		        }
		        if (isNaN(input.data.getTime())) {
		            const ctx = this._getOrReturnCtx(input);
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                code: ZodError_1.ZodIssueCode.invalid_date,
		            });
		            return parseUtil_1.INVALID;
		        }
		        const status = new parseUtil_1.ParseStatus();
		        let ctx = undefined;
		        for (const check of this._def.checks) {
		            if (check.kind === "min") {
		                if (input.data.getTime() < check.value) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        code: ZodError_1.ZodIssueCode.too_small,
		                        message: check.message,
		                        inclusive: true,
		                        exact: false,
		                        minimum: check.value,
		                        type: "date",
		                    });
		                    status.dirty();
		                }
		            }
		            else if (check.kind === "max") {
		                if (input.data.getTime() > check.value) {
		                    ctx = this._getOrReturnCtx(input, ctx);
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        code: ZodError_1.ZodIssueCode.too_big,
		                        message: check.message,
		                        inclusive: true,
		                        exact: false,
		                        maximum: check.value,
		                        type: "date",
		                    });
		                    status.dirty();
		                }
		            }
		            else {
		                util_1.util.assertNever(check);
		            }
		        }
		        return {
		            status: status.value,
		            value: new Date(input.data.getTime()),
		        };
		    }
		    _addCheck(check) {
		        return new ZodDate({
		            ...this._def,
		            checks: [...this._def.checks, check],
		        });
		    }
		    min(minDate, message) {
		        return this._addCheck({
		            kind: "min",
		            value: minDate.getTime(),
		            message: errorUtil_1.errorUtil.toString(message),
		        });
		    }
		    max(maxDate, message) {
		        return this._addCheck({
		            kind: "max",
		            value: maxDate.getTime(),
		            message: errorUtil_1.errorUtil.toString(message),
		        });
		    }
		    get minDate() {
		        let min = null;
		        for (const ch of this._def.checks) {
		            if (ch.kind === "min") {
		                if (min === null || ch.value > min)
		                    min = ch.value;
		            }
		        }
		        return min != null ? new Date(min) : null;
		    }
		    get maxDate() {
		        let max = null;
		        for (const ch of this._def.checks) {
		            if (ch.kind === "max") {
		                if (max === null || ch.value < max)
		                    max = ch.value;
		            }
		        }
		        return max != null ? new Date(max) : null;
		    }
		}
		exports.ZodDate = ZodDate;
		ZodDate.create = (params) => {
		    return new ZodDate({
		        checks: [],
		        coerce: (params === null || params === void 0 ? void 0 : params.coerce) || false,
		        typeName: ZodFirstPartyTypeKind.ZodDate,
		        ...processCreateParams(params),
		    });
		};
		class ZodSymbol extends ZodType {
		    _parse(input) {
		        const parsedType = this._getType(input);
		        if (parsedType !== util_1.ZodParsedType.symbol) {
		            const ctx = this._getOrReturnCtx(input);
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                code: ZodError_1.ZodIssueCode.invalid_type,
		                expected: util_1.ZodParsedType.symbol,
		                received: ctx.parsedType,
		            });
		            return parseUtil_1.INVALID;
		        }
		        return (0, parseUtil_1.OK)(input.data);
		    }
		}
		exports.ZodSymbol = ZodSymbol;
		ZodSymbol.create = (params) => {
		    return new ZodSymbol({
		        typeName: ZodFirstPartyTypeKind.ZodSymbol,
		        ...processCreateParams(params),
		    });
		};
		class ZodUndefined extends ZodType {
		    _parse(input) {
		        const parsedType = this._getType(input);
		        if (parsedType !== util_1.ZodParsedType.undefined) {
		            const ctx = this._getOrReturnCtx(input);
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                code: ZodError_1.ZodIssueCode.invalid_type,
		                expected: util_1.ZodParsedType.undefined,
		                received: ctx.parsedType,
		            });
		            return parseUtil_1.INVALID;
		        }
		        return (0, parseUtil_1.OK)(input.data);
		    }
		}
		exports.ZodUndefined = ZodUndefined;
		ZodUndefined.create = (params) => {
		    return new ZodUndefined({
		        typeName: ZodFirstPartyTypeKind.ZodUndefined,
		        ...processCreateParams(params),
		    });
		};
		class ZodNull extends ZodType {
		    _parse(input) {
		        const parsedType = this._getType(input);
		        if (parsedType !== util_1.ZodParsedType.null) {
		            const ctx = this._getOrReturnCtx(input);
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                code: ZodError_1.ZodIssueCode.invalid_type,
		                expected: util_1.ZodParsedType.null,
		                received: ctx.parsedType,
		            });
		            return parseUtil_1.INVALID;
		        }
		        return (0, parseUtil_1.OK)(input.data);
		    }
		}
		exports.ZodNull = ZodNull;
		ZodNull.create = (params) => {
		    return new ZodNull({
		        typeName: ZodFirstPartyTypeKind.ZodNull,
		        ...processCreateParams(params),
		    });
		};
		class ZodAny extends ZodType {
		    constructor() {
		        super(...arguments);
		        // to prevent instances of other classes from extending ZodAny. this causes issues with catchall in ZodObject.
		        this._any = true;
		    }
		    _parse(input) {
		        return (0, parseUtil_1.OK)(input.data);
		    }
		}
		exports.ZodAny = ZodAny;
		ZodAny.create = (params) => {
		    return new ZodAny({
		        typeName: ZodFirstPartyTypeKind.ZodAny,
		        ...processCreateParams(params),
		    });
		};
		class ZodUnknown extends ZodType {
		    constructor() {
		        super(...arguments);
		        // required
		        this._unknown = true;
		    }
		    _parse(input) {
		        return (0, parseUtil_1.OK)(input.data);
		    }
		}
		exports.ZodUnknown = ZodUnknown;
		ZodUnknown.create = (params) => {
		    return new ZodUnknown({
		        typeName: ZodFirstPartyTypeKind.ZodUnknown,
		        ...processCreateParams(params),
		    });
		};
		class ZodNever extends ZodType {
		    _parse(input) {
		        const ctx = this._getOrReturnCtx(input);
		        (0, parseUtil_1.addIssueToContext)(ctx, {
		            code: ZodError_1.ZodIssueCode.invalid_type,
		            expected: util_1.ZodParsedType.never,
		            received: ctx.parsedType,
		        });
		        return parseUtil_1.INVALID;
		    }
		}
		exports.ZodNever = ZodNever;
		ZodNever.create = (params) => {
		    return new ZodNever({
		        typeName: ZodFirstPartyTypeKind.ZodNever,
		        ...processCreateParams(params),
		    });
		};
		class ZodVoid extends ZodType {
		    _parse(input) {
		        const parsedType = this._getType(input);
		        if (parsedType !== util_1.ZodParsedType.undefined) {
		            const ctx = this._getOrReturnCtx(input);
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                code: ZodError_1.ZodIssueCode.invalid_type,
		                expected: util_1.ZodParsedType.void,
		                received: ctx.parsedType,
		            });
		            return parseUtil_1.INVALID;
		        }
		        return (0, parseUtil_1.OK)(input.data);
		    }
		}
		exports.ZodVoid = ZodVoid;
		ZodVoid.create = (params) => {
		    return new ZodVoid({
		        typeName: ZodFirstPartyTypeKind.ZodVoid,
		        ...processCreateParams(params),
		    });
		};
		class ZodArray extends ZodType {
		    _parse(input) {
		        const { ctx, status } = this._processInputParams(input);
		        const def = this._def;
		        if (ctx.parsedType !== util_1.ZodParsedType.array) {
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                code: ZodError_1.ZodIssueCode.invalid_type,
		                expected: util_1.ZodParsedType.array,
		                received: ctx.parsedType,
		            });
		            return parseUtil_1.INVALID;
		        }
		        if (def.exactLength !== null) {
		            const tooBig = ctx.data.length > def.exactLength.value;
		            const tooSmall = ctx.data.length < def.exactLength.value;
		            if (tooBig || tooSmall) {
		                (0, parseUtil_1.addIssueToContext)(ctx, {
		                    code: tooBig ? ZodError_1.ZodIssueCode.too_big : ZodError_1.ZodIssueCode.too_small,
		                    minimum: (tooSmall ? def.exactLength.value : undefined),
		                    maximum: (tooBig ? def.exactLength.value : undefined),
		                    type: "array",
		                    inclusive: true,
		                    exact: true,
		                    message: def.exactLength.message,
		                });
		                status.dirty();
		            }
		        }
		        if (def.minLength !== null) {
		            if (ctx.data.length < def.minLength.value) {
		                (0, parseUtil_1.addIssueToContext)(ctx, {
		                    code: ZodError_1.ZodIssueCode.too_small,
		                    minimum: def.minLength.value,
		                    type: "array",
		                    inclusive: true,
		                    exact: false,
		                    message: def.minLength.message,
		                });
		                status.dirty();
		            }
		        }
		        if (def.maxLength !== null) {
		            if (ctx.data.length > def.maxLength.value) {
		                (0, parseUtil_1.addIssueToContext)(ctx, {
		                    code: ZodError_1.ZodIssueCode.too_big,
		                    maximum: def.maxLength.value,
		                    type: "array",
		                    inclusive: true,
		                    exact: false,
		                    message: def.maxLength.message,
		                });
		                status.dirty();
		            }
		        }
		        if (ctx.common.async) {
		            return Promise.all([...ctx.data].map((item, i) => {
		                return def.type._parseAsync(new ParseInputLazyPath(ctx, item, ctx.path, i));
		            })).then((result) => {
		                return parseUtil_1.ParseStatus.mergeArray(status, result);
		            });
		        }
		        const result = [...ctx.data].map((item, i) => {
		            return def.type._parseSync(new ParseInputLazyPath(ctx, item, ctx.path, i));
		        });
		        return parseUtil_1.ParseStatus.mergeArray(status, result);
		    }
		    get element() {
		        return this._def.type;
		    }
		    min(minLength, message) {
		        return new ZodArray({
		            ...this._def,
		            minLength: { value: minLength, message: errorUtil_1.errorUtil.toString(message) },
		        });
		    }
		    max(maxLength, message) {
		        return new ZodArray({
		            ...this._def,
		            maxLength: { value: maxLength, message: errorUtil_1.errorUtil.toString(message) },
		        });
		    }
		    length(len, message) {
		        return new ZodArray({
		            ...this._def,
		            exactLength: { value: len, message: errorUtil_1.errorUtil.toString(message) },
		        });
		    }
		    nonempty(message) {
		        return this.min(1, message);
		    }
		}
		exports.ZodArray = ZodArray;
		ZodArray.create = (schema, params) => {
		    return new ZodArray({
		        type: schema,
		        minLength: null,
		        maxLength: null,
		        exactLength: null,
		        typeName: ZodFirstPartyTypeKind.ZodArray,
		        ...processCreateParams(params),
		    });
		};
		function deepPartialify(schema) {
		    if (schema instanceof ZodObject) {
		        const newShape = {};
		        for (const key in schema.shape) {
		            const fieldSchema = schema.shape[key];
		            newShape[key] = ZodOptional.create(deepPartialify(fieldSchema));
		        }
		        return new ZodObject({
		            ...schema._def,
		            shape: () => newShape,
		        });
		    }
		    else if (schema instanceof ZodArray) {
		        return new ZodArray({
		            ...schema._def,
		            type: deepPartialify(schema.element),
		        });
		    }
		    else if (schema instanceof ZodOptional) {
		        return ZodOptional.create(deepPartialify(schema.unwrap()));
		    }
		    else if (schema instanceof ZodNullable) {
		        return ZodNullable.create(deepPartialify(schema.unwrap()));
		    }
		    else if (schema instanceof ZodTuple) {
		        return ZodTuple.create(schema.items.map((item) => deepPartialify(item)));
		    }
		    else {
		        return schema;
		    }
		}
		class ZodObject extends ZodType {
		    constructor() {
		        super(...arguments);
		        this._cached = null;
		        /**
		         * @deprecated In most cases, this is no longer needed - unknown properties are now silently stripped.
		         * If you want to pass through unknown properties, use `.passthrough()` instead.
		         */
		        this.nonstrict = this.passthrough;
		        // extend<
		        //   Augmentation extends ZodRawShape,
		        //   NewOutput extends util.flatten<{
		        //     [k in keyof Augmentation | keyof Output]: k extends keyof Augmentation
		        //       ? Augmentation[k]["_output"]
		        //       : k extends keyof Output
		        //       ? Output[k]
		        //       : never;
		        //   }>,
		        //   NewInput extends util.flatten<{
		        //     [k in keyof Augmentation | keyof Input]: k extends keyof Augmentation
		        //       ? Augmentation[k]["_input"]
		        //       : k extends keyof Input
		        //       ? Input[k]
		        //       : never;
		        //   }>
		        // >(
		        //   augmentation: Augmentation
		        // ): ZodObject<
		        //   extendShape<T, Augmentation>,
		        //   UnknownKeys,
		        //   Catchall,
		        //   NewOutput,
		        //   NewInput
		        // > {
		        //   return new ZodObject({
		        //     ...this._def,
		        //     shape: () => ({
		        //       ...this._def.shape(),
		        //       ...augmentation,
		        //     }),
		        //   }) as any;
		        // }
		        /**
		         * @deprecated Use `.extend` instead
		         *  */
		        this.augment = this.extend;
		    }
		    _getCached() {
		        if (this._cached !== null)
		            return this._cached;
		        const shape = this._def.shape();
		        const keys = util_1.util.objectKeys(shape);
		        return (this._cached = { shape, keys });
		    }
		    _parse(input) {
		        const parsedType = this._getType(input);
		        if (parsedType !== util_1.ZodParsedType.object) {
		            const ctx = this._getOrReturnCtx(input);
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                code: ZodError_1.ZodIssueCode.invalid_type,
		                expected: util_1.ZodParsedType.object,
		                received: ctx.parsedType,
		            });
		            return parseUtil_1.INVALID;
		        }
		        const { status, ctx } = this._processInputParams(input);
		        const { shape, keys: shapeKeys } = this._getCached();
		        const extraKeys = [];
		        if (!(this._def.catchall instanceof ZodNever &&
		            this._def.unknownKeys === "strip")) {
		            for (const key in ctx.data) {
		                if (!shapeKeys.includes(key)) {
		                    extraKeys.push(key);
		                }
		            }
		        }
		        const pairs = [];
		        for (const key of shapeKeys) {
		            const keyValidator = shape[key];
		            const value = ctx.data[key];
		            pairs.push({
		                key: { status: "valid", value: key },
		                value: keyValidator._parse(new ParseInputLazyPath(ctx, value, ctx.path, key)),
		                alwaysSet: key in ctx.data,
		            });
		        }
		        if (this._def.catchall instanceof ZodNever) {
		            const unknownKeys = this._def.unknownKeys;
		            if (unknownKeys === "passthrough") {
		                for (const key of extraKeys) {
		                    pairs.push({
		                        key: { status: "valid", value: key },
		                        value: { status: "valid", value: ctx.data[key] },
		                    });
		                }
		            }
		            else if (unknownKeys === "strict") {
		                if (extraKeys.length > 0) {
		                    (0, parseUtil_1.addIssueToContext)(ctx, {
		                        code: ZodError_1.ZodIssueCode.unrecognized_keys,
		                        keys: extraKeys,
		                    });
		                    status.dirty();
		                }
		            }
		            else if (unknownKeys === "strip") ;
		            else {
		                throw new Error(`Internal ZodObject error: invalid unknownKeys value.`);
		            }
		        }
		        else {
		            // run catchall validation
		            const catchall = this._def.catchall;
		            for (const key of extraKeys) {
		                const value = ctx.data[key];
		                pairs.push({
		                    key: { status: "valid", value: key },
		                    value: catchall._parse(new ParseInputLazyPath(ctx, value, ctx.path, key) //, ctx.child(key), value, getParsedType(value)
		                    ),
		                    alwaysSet: key in ctx.data,
		                });
		            }
		        }
		        if (ctx.common.async) {
		            return Promise.resolve()
		                .then(async () => {
		                const syncPairs = [];
		                for (const pair of pairs) {
		                    const key = await pair.key;
		                    const value = await pair.value;
		                    syncPairs.push({
		                        key,
		                        value,
		                        alwaysSet: pair.alwaysSet,
		                    });
		                }
		                return syncPairs;
		            })
		                .then((syncPairs) => {
		                return parseUtil_1.ParseStatus.mergeObjectSync(status, syncPairs);
		            });
		        }
		        else {
		            return parseUtil_1.ParseStatus.mergeObjectSync(status, pairs);
		        }
		    }
		    get shape() {
		        return this._def.shape();
		    }
		    strict(message) {
		        errorUtil_1.errorUtil.errToObj;
		        return new ZodObject({
		            ...this._def,
		            unknownKeys: "strict",
		            ...(message !== undefined
		                ? {
		                    errorMap: (issue, ctx) => {
		                        var _a, _b, _c, _d;
		                        const defaultError = (_c = (_b = (_a = this._def).errorMap) === null || _b === void 0 ? void 0 : _b.call(_a, issue, ctx).message) !== null && _c !== void 0 ? _c : ctx.defaultError;
		                        if (issue.code === "unrecognized_keys")
		                            return {
		                                message: (_d = errorUtil_1.errorUtil.errToObj(message).message) !== null && _d !== void 0 ? _d : defaultError,
		                            };
		                        return {
		                            message: defaultError,
		                        };
		                    },
		                }
		                : {}),
		        });
		    }
		    strip() {
		        return new ZodObject({
		            ...this._def,
		            unknownKeys: "strip",
		        });
		    }
		    passthrough() {
		        return new ZodObject({
		            ...this._def,
		            unknownKeys: "passthrough",
		        });
		    }
		    // const AugmentFactory =
		    //   <Def extends ZodObjectDef>(def: Def) =>
		    //   <Augmentation extends ZodRawShape>(
		    //     augmentation: Augmentation
		    //   ): ZodObject<
		    //     extendShape<ReturnType<Def["shape"]>, Augmentation>,
		    //     Def["unknownKeys"],
		    //     Def["catchall"]
		    //   > => {
		    //     return new ZodObject({
		    //       ...def,
		    //       shape: () => ({
		    //         ...def.shape(),
		    //         ...augmentation,
		    //       }),
		    //     }) as any;
		    //   };
		    extend(augmentation) {
		        return new ZodObject({
		            ...this._def,
		            shape: () => ({
		                ...this._def.shape(),
		                ...augmentation,
		            }),
		        });
		    }
		    /**
		     * Prior to zod@1.0.12 there was a bug in the
		     * inferred type of merged objects. Please
		     * upgrade if you are experiencing issues.
		     */
		    merge(merging) {
		        const merged = new ZodObject({
		            unknownKeys: merging._def.unknownKeys,
		            catchall: merging._def.catchall,
		            shape: () => ({
		                ...this._def.shape(),
		                ...merging._def.shape(),
		            }),
		            typeName: ZodFirstPartyTypeKind.ZodObject,
		        });
		        return merged;
		    }
		    // merge<
		    //   Incoming extends AnyZodObject,
		    //   Augmentation extends Incoming["shape"],
		    //   NewOutput extends {
		    //     [k in keyof Augmentation | keyof Output]: k extends keyof Augmentation
		    //       ? Augmentation[k]["_output"]
		    //       : k extends keyof Output
		    //       ? Output[k]
		    //       : never;
		    //   },
		    //   NewInput extends {
		    //     [k in keyof Augmentation | keyof Input]: k extends keyof Augmentation
		    //       ? Augmentation[k]["_input"]
		    //       : k extends keyof Input
		    //       ? Input[k]
		    //       : never;
		    //   }
		    // >(
		    //   merging: Incoming
		    // ): ZodObject<
		    //   extendShape<T, ReturnType<Incoming["_def"]["shape"]>>,
		    //   Incoming["_def"]["unknownKeys"],
		    //   Incoming["_def"]["catchall"],
		    //   NewOutput,
		    //   NewInput
		    // > {
		    //   const merged: any = new ZodObject({
		    //     unknownKeys: merging._def.unknownKeys,
		    //     catchall: merging._def.catchall,
		    //     shape: () =>
		    //       objectUtil.mergeShapes(this._def.shape(), merging._def.shape()),
		    //     typeName: ZodFirstPartyTypeKind.ZodObject,
		    //   }) as any;
		    //   return merged;
		    // }
		    setKey(key, schema) {
		        return this.augment({ [key]: schema });
		    }
		    // merge<Incoming extends AnyZodObject>(
		    //   merging: Incoming
		    // ): //ZodObject<T & Incoming["_shape"], UnknownKeys, Catchall> = (merging) => {
		    // ZodObject<
		    //   extendShape<T, ReturnType<Incoming["_def"]["shape"]>>,
		    //   Incoming["_def"]["unknownKeys"],
		    //   Incoming["_def"]["catchall"]
		    // > {
		    //   // const mergedShape = objectUtil.mergeShapes(
		    //   //   this._def.shape(),
		    //   //   merging._def.shape()
		    //   // );
		    //   const merged: any = new ZodObject({
		    //     unknownKeys: merging._def.unknownKeys,
		    //     catchall: merging._def.catchall,
		    //     shape: () =>
		    //       objectUtil.mergeShapes(this._def.shape(), merging._def.shape()),
		    //     typeName: ZodFirstPartyTypeKind.ZodObject,
		    //   }) as any;
		    //   return merged;
		    // }
		    catchall(index) {
		        return new ZodObject({
		            ...this._def,
		            catchall: index,
		        });
		    }
		    pick(mask) {
		        const shape = {};
		        util_1.util.objectKeys(mask).forEach((key) => {
		            if (mask[key] && this.shape[key]) {
		                shape[key] = this.shape[key];
		            }
		        });
		        return new ZodObject({
		            ...this._def,
		            shape: () => shape,
		        });
		    }
		    omit(mask) {
		        const shape = {};
		        util_1.util.objectKeys(this.shape).forEach((key) => {
		            if (!mask[key]) {
		                shape[key] = this.shape[key];
		            }
		        });
		        return new ZodObject({
		            ...this._def,
		            shape: () => shape,
		        });
		    }
		    /**
		     * @deprecated
		     */
		    deepPartial() {
		        return deepPartialify(this);
		    }
		    partial(mask) {
		        const newShape = {};
		        util_1.util.objectKeys(this.shape).forEach((key) => {
		            const fieldSchema = this.shape[key];
		            if (mask && !mask[key]) {
		                newShape[key] = fieldSchema;
		            }
		            else {
		                newShape[key] = fieldSchema.optional();
		            }
		        });
		        return new ZodObject({
		            ...this._def,
		            shape: () => newShape,
		        });
		    }
		    required(mask) {
		        const newShape = {};
		        util_1.util.objectKeys(this.shape).forEach((key) => {
		            if (mask && !mask[key]) {
		                newShape[key] = this.shape[key];
		            }
		            else {
		                const fieldSchema = this.shape[key];
		                let newField = fieldSchema;
		                while (newField instanceof ZodOptional) {
		                    newField = newField._def.innerType;
		                }
		                newShape[key] = newField;
		            }
		        });
		        return new ZodObject({
		            ...this._def,
		            shape: () => newShape,
		        });
		    }
		    keyof() {
		        return createZodEnum(util_1.util.objectKeys(this.shape));
		    }
		}
		exports.ZodObject = ZodObject;
		ZodObject.create = (shape, params) => {
		    return new ZodObject({
		        shape: () => shape,
		        unknownKeys: "strip",
		        catchall: ZodNever.create(),
		        typeName: ZodFirstPartyTypeKind.ZodObject,
		        ...processCreateParams(params),
		    });
		};
		ZodObject.strictCreate = (shape, params) => {
		    return new ZodObject({
		        shape: () => shape,
		        unknownKeys: "strict",
		        catchall: ZodNever.create(),
		        typeName: ZodFirstPartyTypeKind.ZodObject,
		        ...processCreateParams(params),
		    });
		};
		ZodObject.lazycreate = (shape, params) => {
		    return new ZodObject({
		        shape,
		        unknownKeys: "strip",
		        catchall: ZodNever.create(),
		        typeName: ZodFirstPartyTypeKind.ZodObject,
		        ...processCreateParams(params),
		    });
		};
		class ZodUnion extends ZodType {
		    _parse(input) {
		        const { ctx } = this._processInputParams(input);
		        const options = this._def.options;
		        function handleResults(results) {
		            // return first issue-free validation if it exists
		            for (const result of results) {
		                if (result.result.status === "valid") {
		                    return result.result;
		                }
		            }
		            for (const result of results) {
		                if (result.result.status === "dirty") {
		                    // add issues from dirty option
		                    ctx.common.issues.push(...result.ctx.common.issues);
		                    return result.result;
		                }
		            }
		            // return invalid
		            const unionErrors = results.map((result) => new ZodError_1.ZodError(result.ctx.common.issues));
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                code: ZodError_1.ZodIssueCode.invalid_union,
		                unionErrors,
		            });
		            return parseUtil_1.INVALID;
		        }
		        if (ctx.common.async) {
		            return Promise.all(options.map(async (option) => {
		                const childCtx = {
		                    ...ctx,
		                    common: {
		                        ...ctx.common,
		                        issues: [],
		                    },
		                    parent: null,
		                };
		                return {
		                    result: await option._parseAsync({
		                        data: ctx.data,
		                        path: ctx.path,
		                        parent: childCtx,
		                    }),
		                    ctx: childCtx,
		                };
		            })).then(handleResults);
		        }
		        else {
		            let dirty = undefined;
		            const issues = [];
		            for (const option of options) {
		                const childCtx = {
		                    ...ctx,
		                    common: {
		                        ...ctx.common,
		                        issues: [],
		                    },
		                    parent: null,
		                };
		                const result = option._parseSync({
		                    data: ctx.data,
		                    path: ctx.path,
		                    parent: childCtx,
		                });
		                if (result.status === "valid") {
		                    return result;
		                }
		                else if (result.status === "dirty" && !dirty) {
		                    dirty = { result, ctx: childCtx };
		                }
		                if (childCtx.common.issues.length) {
		                    issues.push(childCtx.common.issues);
		                }
		            }
		            if (dirty) {
		                ctx.common.issues.push(...dirty.ctx.common.issues);
		                return dirty.result;
		            }
		            const unionErrors = issues.map((issues) => new ZodError_1.ZodError(issues));
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                code: ZodError_1.ZodIssueCode.invalid_union,
		                unionErrors,
		            });
		            return parseUtil_1.INVALID;
		        }
		    }
		    get options() {
		        return this._def.options;
		    }
		}
		exports.ZodUnion = ZodUnion;
		ZodUnion.create = (types, params) => {
		    return new ZodUnion({
		        options: types,
		        typeName: ZodFirstPartyTypeKind.ZodUnion,
		        ...processCreateParams(params),
		    });
		};
		/////////////////////////////////////////////////////
		/////////////////////////////////////////////////////
		//////////                                 //////////
		//////////      ZodDiscriminatedUnion      //////////
		//////////                                 //////////
		/////////////////////////////////////////////////////
		/////////////////////////////////////////////////////
		const getDiscriminator = (type) => {
		    if (type instanceof ZodLazy) {
		        return getDiscriminator(type.schema);
		    }
		    else if (type instanceof ZodEffects) {
		        return getDiscriminator(type.innerType());
		    }
		    else if (type instanceof ZodLiteral) {
		        return [type.value];
		    }
		    else if (type instanceof ZodEnum) {
		        return type.options;
		    }
		    else if (type instanceof ZodNativeEnum) {
		        // eslint-disable-next-line ban/ban
		        return util_1.util.objectValues(type.enum);
		    }
		    else if (type instanceof ZodDefault) {
		        return getDiscriminator(type._def.innerType);
		    }
		    else if (type instanceof ZodUndefined) {
		        return [undefined];
		    }
		    else if (type instanceof ZodNull) {
		        return [null];
		    }
		    else if (type instanceof ZodOptional) {
		        return [undefined, ...getDiscriminator(type.unwrap())];
		    }
		    else if (type instanceof ZodNullable) {
		        return [null, ...getDiscriminator(type.unwrap())];
		    }
		    else if (type instanceof ZodBranded) {
		        return getDiscriminator(type.unwrap());
		    }
		    else if (type instanceof ZodReadonly) {
		        return getDiscriminator(type.unwrap());
		    }
		    else if (type instanceof ZodCatch) {
		        return getDiscriminator(type._def.innerType);
		    }
		    else {
		        return [];
		    }
		};
		class ZodDiscriminatedUnion extends ZodType {
		    _parse(input) {
		        const { ctx } = this._processInputParams(input);
		        if (ctx.parsedType !== util_1.ZodParsedType.object) {
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                code: ZodError_1.ZodIssueCode.invalid_type,
		                expected: util_1.ZodParsedType.object,
		                received: ctx.parsedType,
		            });
		            return parseUtil_1.INVALID;
		        }
		        const discriminator = this.discriminator;
		        const discriminatorValue = ctx.data[discriminator];
		        const option = this.optionsMap.get(discriminatorValue);
		        if (!option) {
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                code: ZodError_1.ZodIssueCode.invalid_union_discriminator,
		                options: Array.from(this.optionsMap.keys()),
		                path: [discriminator],
		            });
		            return parseUtil_1.INVALID;
		        }
		        if (ctx.common.async) {
		            return option._parseAsync({
		                data: ctx.data,
		                path: ctx.path,
		                parent: ctx,
		            });
		        }
		        else {
		            return option._parseSync({
		                data: ctx.data,
		                path: ctx.path,
		                parent: ctx,
		            });
		        }
		    }
		    get discriminator() {
		        return this._def.discriminator;
		    }
		    get options() {
		        return this._def.options;
		    }
		    get optionsMap() {
		        return this._def.optionsMap;
		    }
		    /**
		     * The constructor of the discriminated union schema. Its behaviour is very similar to that of the normal z.union() constructor.
		     * However, it only allows a union of objects, all of which need to share a discriminator property. This property must
		     * have a different value for each object in the union.
		     * @param discriminator the name of the discriminator property
		     * @param types an array of object schemas
		     * @param params
		     */
		    static create(discriminator, options, params) {
		        // Get all the valid discriminator values
		        const optionsMap = new Map();
		        // try {
		        for (const type of options) {
		            const discriminatorValues = getDiscriminator(type.shape[discriminator]);
		            if (!discriminatorValues.length) {
		                throw new Error(`A discriminator value for key \`${discriminator}\` could not be extracted from all schema options`);
		            }
		            for (const value of discriminatorValues) {
		                if (optionsMap.has(value)) {
		                    throw new Error(`Discriminator property ${String(discriminator)} has duplicate value ${String(value)}`);
		                }
		                optionsMap.set(value, type);
		            }
		        }
		        return new ZodDiscriminatedUnion({
		            typeName: ZodFirstPartyTypeKind.ZodDiscriminatedUnion,
		            discriminator,
		            options,
		            optionsMap,
		            ...processCreateParams(params),
		        });
		    }
		}
		exports.ZodDiscriminatedUnion = ZodDiscriminatedUnion;
		function mergeValues(a, b) {
		    const aType = (0, util_1.getParsedType)(a);
		    const bType = (0, util_1.getParsedType)(b);
		    if (a === b) {
		        return { valid: true, data: a };
		    }
		    else if (aType === util_1.ZodParsedType.object && bType === util_1.ZodParsedType.object) {
		        const bKeys = util_1.util.objectKeys(b);
		        const sharedKeys = util_1.util
		            .objectKeys(a)
		            .filter((key) => bKeys.indexOf(key) !== -1);
		        const newObj = { ...a, ...b };
		        for (const key of sharedKeys) {
		            const sharedValue = mergeValues(a[key], b[key]);
		            if (!sharedValue.valid) {
		                return { valid: false };
		            }
		            newObj[key] = sharedValue.data;
		        }
		        return { valid: true, data: newObj };
		    }
		    else if (aType === util_1.ZodParsedType.array && bType === util_1.ZodParsedType.array) {
		        if (a.length !== b.length) {
		            return { valid: false };
		        }
		        const newArray = [];
		        for (let index = 0; index < a.length; index++) {
		            const itemA = a[index];
		            const itemB = b[index];
		            const sharedValue = mergeValues(itemA, itemB);
		            if (!sharedValue.valid) {
		                return { valid: false };
		            }
		            newArray.push(sharedValue.data);
		        }
		        return { valid: true, data: newArray };
		    }
		    else if (aType === util_1.ZodParsedType.date &&
		        bType === util_1.ZodParsedType.date &&
		        +a === +b) {
		        return { valid: true, data: a };
		    }
		    else {
		        return { valid: false };
		    }
		}
		class ZodIntersection extends ZodType {
		    _parse(input) {
		        const { status, ctx } = this._processInputParams(input);
		        const handleParsed = (parsedLeft, parsedRight) => {
		            if ((0, parseUtil_1.isAborted)(parsedLeft) || (0, parseUtil_1.isAborted)(parsedRight)) {
		                return parseUtil_1.INVALID;
		            }
		            const merged = mergeValues(parsedLeft.value, parsedRight.value);
		            if (!merged.valid) {
		                (0, parseUtil_1.addIssueToContext)(ctx, {
		                    code: ZodError_1.ZodIssueCode.invalid_intersection_types,
		                });
		                return parseUtil_1.INVALID;
		            }
		            if ((0, parseUtil_1.isDirty)(parsedLeft) || (0, parseUtil_1.isDirty)(parsedRight)) {
		                status.dirty();
		            }
		            return { status: status.value, value: merged.data };
		        };
		        if (ctx.common.async) {
		            return Promise.all([
		                this._def.left._parseAsync({
		                    data: ctx.data,
		                    path: ctx.path,
		                    parent: ctx,
		                }),
		                this._def.right._parseAsync({
		                    data: ctx.data,
		                    path: ctx.path,
		                    parent: ctx,
		                }),
		            ]).then(([left, right]) => handleParsed(left, right));
		        }
		        else {
		            return handleParsed(this._def.left._parseSync({
		                data: ctx.data,
		                path: ctx.path,
		                parent: ctx,
		            }), this._def.right._parseSync({
		                data: ctx.data,
		                path: ctx.path,
		                parent: ctx,
		            }));
		        }
		    }
		}
		exports.ZodIntersection = ZodIntersection;
		ZodIntersection.create = (left, right, params) => {
		    return new ZodIntersection({
		        left: left,
		        right: right,
		        typeName: ZodFirstPartyTypeKind.ZodIntersection,
		        ...processCreateParams(params),
		    });
		};
		class ZodTuple extends ZodType {
		    _parse(input) {
		        const { status, ctx } = this._processInputParams(input);
		        if (ctx.parsedType !== util_1.ZodParsedType.array) {
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                code: ZodError_1.ZodIssueCode.invalid_type,
		                expected: util_1.ZodParsedType.array,
		                received: ctx.parsedType,
		            });
		            return parseUtil_1.INVALID;
		        }
		        if (ctx.data.length < this._def.items.length) {
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                code: ZodError_1.ZodIssueCode.too_small,
		                minimum: this._def.items.length,
		                inclusive: true,
		                exact: false,
		                type: "array",
		            });
		            return parseUtil_1.INVALID;
		        }
		        const rest = this._def.rest;
		        if (!rest && ctx.data.length > this._def.items.length) {
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                code: ZodError_1.ZodIssueCode.too_big,
		                maximum: this._def.items.length,
		                inclusive: true,
		                exact: false,
		                type: "array",
		            });
		            status.dirty();
		        }
		        const items = [...ctx.data]
		            .map((item, itemIndex) => {
		            const schema = this._def.items[itemIndex] || this._def.rest;
		            if (!schema)
		                return null;
		            return schema._parse(new ParseInputLazyPath(ctx, item, ctx.path, itemIndex));
		        })
		            .filter((x) => !!x); // filter nulls
		        if (ctx.common.async) {
		            return Promise.all(items).then((results) => {
		                return parseUtil_1.ParseStatus.mergeArray(status, results);
		            });
		        }
		        else {
		            return parseUtil_1.ParseStatus.mergeArray(status, items);
		        }
		    }
		    get items() {
		        return this._def.items;
		    }
		    rest(rest) {
		        return new ZodTuple({
		            ...this._def,
		            rest,
		        });
		    }
		}
		exports.ZodTuple = ZodTuple;
		ZodTuple.create = (schemas, params) => {
		    if (!Array.isArray(schemas)) {
		        throw new Error("You must pass an array of schemas to z.tuple([ ... ])");
		    }
		    return new ZodTuple({
		        items: schemas,
		        typeName: ZodFirstPartyTypeKind.ZodTuple,
		        rest: null,
		        ...processCreateParams(params),
		    });
		};
		class ZodRecord extends ZodType {
		    get keySchema() {
		        return this._def.keyType;
		    }
		    get valueSchema() {
		        return this._def.valueType;
		    }
		    _parse(input) {
		        const { status, ctx } = this._processInputParams(input);
		        if (ctx.parsedType !== util_1.ZodParsedType.object) {
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                code: ZodError_1.ZodIssueCode.invalid_type,
		                expected: util_1.ZodParsedType.object,
		                received: ctx.parsedType,
		            });
		            return parseUtil_1.INVALID;
		        }
		        const pairs = [];
		        const keyType = this._def.keyType;
		        const valueType = this._def.valueType;
		        for (const key in ctx.data) {
		            pairs.push({
		                key: keyType._parse(new ParseInputLazyPath(ctx, key, ctx.path, key)),
		                value: valueType._parse(new ParseInputLazyPath(ctx, ctx.data[key], ctx.path, key)),
		                alwaysSet: key in ctx.data,
		            });
		        }
		        if (ctx.common.async) {
		            return parseUtil_1.ParseStatus.mergeObjectAsync(status, pairs);
		        }
		        else {
		            return parseUtil_1.ParseStatus.mergeObjectSync(status, pairs);
		        }
		    }
		    get element() {
		        return this._def.valueType;
		    }
		    static create(first, second, third) {
		        if (second instanceof ZodType) {
		            return new ZodRecord({
		                keyType: first,
		                valueType: second,
		                typeName: ZodFirstPartyTypeKind.ZodRecord,
		                ...processCreateParams(third),
		            });
		        }
		        return new ZodRecord({
		            keyType: ZodString.create(),
		            valueType: first,
		            typeName: ZodFirstPartyTypeKind.ZodRecord,
		            ...processCreateParams(second),
		        });
		    }
		}
		exports.ZodRecord = ZodRecord;
		class ZodMap extends ZodType {
		    get keySchema() {
		        return this._def.keyType;
		    }
		    get valueSchema() {
		        return this._def.valueType;
		    }
		    _parse(input) {
		        const { status, ctx } = this._processInputParams(input);
		        if (ctx.parsedType !== util_1.ZodParsedType.map) {
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                code: ZodError_1.ZodIssueCode.invalid_type,
		                expected: util_1.ZodParsedType.map,
		                received: ctx.parsedType,
		            });
		            return parseUtil_1.INVALID;
		        }
		        const keyType = this._def.keyType;
		        const valueType = this._def.valueType;
		        const pairs = [...ctx.data.entries()].map(([key, value], index) => {
		            return {
		                key: keyType._parse(new ParseInputLazyPath(ctx, key, ctx.path, [index, "key"])),
		                value: valueType._parse(new ParseInputLazyPath(ctx, value, ctx.path, [index, "value"])),
		            };
		        });
		        if (ctx.common.async) {
		            const finalMap = new Map();
		            return Promise.resolve().then(async () => {
		                for (const pair of pairs) {
		                    const key = await pair.key;
		                    const value = await pair.value;
		                    if (key.status === "aborted" || value.status === "aborted") {
		                        return parseUtil_1.INVALID;
		                    }
		                    if (key.status === "dirty" || value.status === "dirty") {
		                        status.dirty();
		                    }
		                    finalMap.set(key.value, value.value);
		                }
		                return { status: status.value, value: finalMap };
		            });
		        }
		        else {
		            const finalMap = new Map();
		            for (const pair of pairs) {
		                const key = pair.key;
		                const value = pair.value;
		                if (key.status === "aborted" || value.status === "aborted") {
		                    return parseUtil_1.INVALID;
		                }
		                if (key.status === "dirty" || value.status === "dirty") {
		                    status.dirty();
		                }
		                finalMap.set(key.value, value.value);
		            }
		            return { status: status.value, value: finalMap };
		        }
		    }
		}
		exports.ZodMap = ZodMap;
		ZodMap.create = (keyType, valueType, params) => {
		    return new ZodMap({
		        valueType,
		        keyType,
		        typeName: ZodFirstPartyTypeKind.ZodMap,
		        ...processCreateParams(params),
		    });
		};
		class ZodSet extends ZodType {
		    _parse(input) {
		        const { status, ctx } = this._processInputParams(input);
		        if (ctx.parsedType !== util_1.ZodParsedType.set) {
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                code: ZodError_1.ZodIssueCode.invalid_type,
		                expected: util_1.ZodParsedType.set,
		                received: ctx.parsedType,
		            });
		            return parseUtil_1.INVALID;
		        }
		        const def = this._def;
		        if (def.minSize !== null) {
		            if (ctx.data.size < def.minSize.value) {
		                (0, parseUtil_1.addIssueToContext)(ctx, {
		                    code: ZodError_1.ZodIssueCode.too_small,
		                    minimum: def.minSize.value,
		                    type: "set",
		                    inclusive: true,
		                    exact: false,
		                    message: def.minSize.message,
		                });
		                status.dirty();
		            }
		        }
		        if (def.maxSize !== null) {
		            if (ctx.data.size > def.maxSize.value) {
		                (0, parseUtil_1.addIssueToContext)(ctx, {
		                    code: ZodError_1.ZodIssueCode.too_big,
		                    maximum: def.maxSize.value,
		                    type: "set",
		                    inclusive: true,
		                    exact: false,
		                    message: def.maxSize.message,
		                });
		                status.dirty();
		            }
		        }
		        const valueType = this._def.valueType;
		        function finalizeSet(elements) {
		            const parsedSet = new Set();
		            for (const element of elements) {
		                if (element.status === "aborted")
		                    return parseUtil_1.INVALID;
		                if (element.status === "dirty")
		                    status.dirty();
		                parsedSet.add(element.value);
		            }
		            return { status: status.value, value: parsedSet };
		        }
		        const elements = [...ctx.data.values()].map((item, i) => valueType._parse(new ParseInputLazyPath(ctx, item, ctx.path, i)));
		        if (ctx.common.async) {
		            return Promise.all(elements).then((elements) => finalizeSet(elements));
		        }
		        else {
		            return finalizeSet(elements);
		        }
		    }
		    min(minSize, message) {
		        return new ZodSet({
		            ...this._def,
		            minSize: { value: minSize, message: errorUtil_1.errorUtil.toString(message) },
		        });
		    }
		    max(maxSize, message) {
		        return new ZodSet({
		            ...this._def,
		            maxSize: { value: maxSize, message: errorUtil_1.errorUtil.toString(message) },
		        });
		    }
		    size(size, message) {
		        return this.min(size, message).max(size, message);
		    }
		    nonempty(message) {
		        return this.min(1, message);
		    }
		}
		exports.ZodSet = ZodSet;
		ZodSet.create = (valueType, params) => {
		    return new ZodSet({
		        valueType,
		        minSize: null,
		        maxSize: null,
		        typeName: ZodFirstPartyTypeKind.ZodSet,
		        ...processCreateParams(params),
		    });
		};
		class ZodFunction extends ZodType {
		    constructor() {
		        super(...arguments);
		        this.validate = this.implement;
		    }
		    _parse(input) {
		        const { ctx } = this._processInputParams(input);
		        if (ctx.parsedType !== util_1.ZodParsedType.function) {
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                code: ZodError_1.ZodIssueCode.invalid_type,
		                expected: util_1.ZodParsedType.function,
		                received: ctx.parsedType,
		            });
		            return parseUtil_1.INVALID;
		        }
		        function makeArgsIssue(args, error) {
		            return (0, parseUtil_1.makeIssue)({
		                data: args,
		                path: ctx.path,
		                errorMaps: [
		                    ctx.common.contextualErrorMap,
		                    ctx.schemaErrorMap,
		                    (0, errors_1.getErrorMap)(),
		                    errors_1.defaultErrorMap,
		                ].filter((x) => !!x),
		                issueData: {
		                    code: ZodError_1.ZodIssueCode.invalid_arguments,
		                    argumentsError: error,
		                },
		            });
		        }
		        function makeReturnsIssue(returns, error) {
		            return (0, parseUtil_1.makeIssue)({
		                data: returns,
		                path: ctx.path,
		                errorMaps: [
		                    ctx.common.contextualErrorMap,
		                    ctx.schemaErrorMap,
		                    (0, errors_1.getErrorMap)(),
		                    errors_1.defaultErrorMap,
		                ].filter((x) => !!x),
		                issueData: {
		                    code: ZodError_1.ZodIssueCode.invalid_return_type,
		                    returnTypeError: error,
		                },
		            });
		        }
		        const params = { errorMap: ctx.common.contextualErrorMap };
		        const fn = ctx.data;
		        if (this._def.returns instanceof ZodPromise) {
		            // Would love a way to avoid disabling this rule, but we need
		            // an alias (using an arrow function was what caused 2651).
		            // eslint-disable-next-line @typescript-eslint/no-this-alias
		            const me = this;
		            return (0, parseUtil_1.OK)(async function (...args) {
		                const error = new ZodError_1.ZodError([]);
		                const parsedArgs = await me._def.args
		                    .parseAsync(args, params)
		                    .catch((e) => {
		                    error.addIssue(makeArgsIssue(args, e));
		                    throw error;
		                });
		                const result = await Reflect.apply(fn, this, parsedArgs);
		                const parsedReturns = await me._def.returns._def.type
		                    .parseAsync(result, params)
		                    .catch((e) => {
		                    error.addIssue(makeReturnsIssue(result, e));
		                    throw error;
		                });
		                return parsedReturns;
		            });
		        }
		        else {
		            // Would love a way to avoid disabling this rule, but we need
		            // an alias (using an arrow function was what caused 2651).
		            // eslint-disable-next-line @typescript-eslint/no-this-alias
		            const me = this;
		            return (0, parseUtil_1.OK)(function (...args) {
		                const parsedArgs = me._def.args.safeParse(args, params);
		                if (!parsedArgs.success) {
		                    throw new ZodError_1.ZodError([makeArgsIssue(args, parsedArgs.error)]);
		                }
		                const result = Reflect.apply(fn, this, parsedArgs.data);
		                const parsedReturns = me._def.returns.safeParse(result, params);
		                if (!parsedReturns.success) {
		                    throw new ZodError_1.ZodError([makeReturnsIssue(result, parsedReturns.error)]);
		                }
		                return parsedReturns.data;
		            });
		        }
		    }
		    parameters() {
		        return this._def.args;
		    }
		    returnType() {
		        return this._def.returns;
		    }
		    args(...items) {
		        return new ZodFunction({
		            ...this._def,
		            args: ZodTuple.create(items).rest(ZodUnknown.create()),
		        });
		    }
		    returns(returnType) {
		        return new ZodFunction({
		            ...this._def,
		            returns: returnType,
		        });
		    }
		    implement(func) {
		        const validatedFunc = this.parse(func);
		        return validatedFunc;
		    }
		    strictImplement(func) {
		        const validatedFunc = this.parse(func);
		        return validatedFunc;
		    }
		    static create(args, returns, params) {
		        return new ZodFunction({
		            args: (args
		                ? args
		                : ZodTuple.create([]).rest(ZodUnknown.create())),
		            returns: returns || ZodUnknown.create(),
		            typeName: ZodFirstPartyTypeKind.ZodFunction,
		            ...processCreateParams(params),
		        });
		    }
		}
		exports.ZodFunction = ZodFunction;
		class ZodLazy extends ZodType {
		    get schema() {
		        return this._def.getter();
		    }
		    _parse(input) {
		        const { ctx } = this._processInputParams(input);
		        const lazySchema = this._def.getter();
		        return lazySchema._parse({ data: ctx.data, path: ctx.path, parent: ctx });
		    }
		}
		exports.ZodLazy = ZodLazy;
		ZodLazy.create = (getter, params) => {
		    return new ZodLazy({
		        getter: getter,
		        typeName: ZodFirstPartyTypeKind.ZodLazy,
		        ...processCreateParams(params),
		    });
		};
		class ZodLiteral extends ZodType {
		    _parse(input) {
		        if (input.data !== this._def.value) {
		            const ctx = this._getOrReturnCtx(input);
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                received: ctx.data,
		                code: ZodError_1.ZodIssueCode.invalid_literal,
		                expected: this._def.value,
		            });
		            return parseUtil_1.INVALID;
		        }
		        return { status: "valid", value: input.data };
		    }
		    get value() {
		        return this._def.value;
		    }
		}
		exports.ZodLiteral = ZodLiteral;
		ZodLiteral.create = (value, params) => {
		    return new ZodLiteral({
		        value: value,
		        typeName: ZodFirstPartyTypeKind.ZodLiteral,
		        ...processCreateParams(params),
		    });
		};
		function createZodEnum(values, params) {
		    return new ZodEnum({
		        values,
		        typeName: ZodFirstPartyTypeKind.ZodEnum,
		        ...processCreateParams(params),
		    });
		}
		class ZodEnum extends ZodType {
		    constructor() {
		        super(...arguments);
		        _ZodEnum_cache.set(this, void 0);
		    }
		    _parse(input) {
		        if (typeof input.data !== "string") {
		            const ctx = this._getOrReturnCtx(input);
		            const expectedValues = this._def.values;
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                expected: util_1.util.joinValues(expectedValues),
		                received: ctx.parsedType,
		                code: ZodError_1.ZodIssueCode.invalid_type,
		            });
		            return parseUtil_1.INVALID;
		        }
		        if (!__classPrivateFieldGet(this, _ZodEnum_cache, "f")) {
		            __classPrivateFieldSet(this, _ZodEnum_cache, new Set(this._def.values), "f");
		        }
		        if (!__classPrivateFieldGet(this, _ZodEnum_cache, "f").has(input.data)) {
		            const ctx = this._getOrReturnCtx(input);
		            const expectedValues = this._def.values;
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                received: ctx.data,
		                code: ZodError_1.ZodIssueCode.invalid_enum_value,
		                options: expectedValues,
		            });
		            return parseUtil_1.INVALID;
		        }
		        return (0, parseUtil_1.OK)(input.data);
		    }
		    get options() {
		        return this._def.values;
		    }
		    get enum() {
		        const enumValues = {};
		        for (const val of this._def.values) {
		            enumValues[val] = val;
		        }
		        return enumValues;
		    }
		    get Values() {
		        const enumValues = {};
		        for (const val of this._def.values) {
		            enumValues[val] = val;
		        }
		        return enumValues;
		    }
		    get Enum() {
		        const enumValues = {};
		        for (const val of this._def.values) {
		            enumValues[val] = val;
		        }
		        return enumValues;
		    }
		    extract(values, newDef = this._def) {
		        return ZodEnum.create(values, {
		            ...this._def,
		            ...newDef,
		        });
		    }
		    exclude(values, newDef = this._def) {
		        return ZodEnum.create(this.options.filter((opt) => !values.includes(opt)), {
		            ...this._def,
		            ...newDef,
		        });
		    }
		}
		exports.ZodEnum = ZodEnum;
		_ZodEnum_cache = new WeakMap();
		ZodEnum.create = createZodEnum;
		class ZodNativeEnum extends ZodType {
		    constructor() {
		        super(...arguments);
		        _ZodNativeEnum_cache.set(this, void 0);
		    }
		    _parse(input) {
		        const nativeEnumValues = util_1.util.getValidEnumValues(this._def.values);
		        const ctx = this._getOrReturnCtx(input);
		        if (ctx.parsedType !== util_1.ZodParsedType.string &&
		            ctx.parsedType !== util_1.ZodParsedType.number) {
		            const expectedValues = util_1.util.objectValues(nativeEnumValues);
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                expected: util_1.util.joinValues(expectedValues),
		                received: ctx.parsedType,
		                code: ZodError_1.ZodIssueCode.invalid_type,
		            });
		            return parseUtil_1.INVALID;
		        }
		        if (!__classPrivateFieldGet(this, _ZodNativeEnum_cache, "f")) {
		            __classPrivateFieldSet(this, _ZodNativeEnum_cache, new Set(util_1.util.getValidEnumValues(this._def.values)), "f");
		        }
		        if (!__classPrivateFieldGet(this, _ZodNativeEnum_cache, "f").has(input.data)) {
		            const expectedValues = util_1.util.objectValues(nativeEnumValues);
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                received: ctx.data,
		                code: ZodError_1.ZodIssueCode.invalid_enum_value,
		                options: expectedValues,
		            });
		            return parseUtil_1.INVALID;
		        }
		        return (0, parseUtil_1.OK)(input.data);
		    }
		    get enum() {
		        return this._def.values;
		    }
		}
		exports.ZodNativeEnum = ZodNativeEnum;
		_ZodNativeEnum_cache = new WeakMap();
		ZodNativeEnum.create = (values, params) => {
		    return new ZodNativeEnum({
		        values: values,
		        typeName: ZodFirstPartyTypeKind.ZodNativeEnum,
		        ...processCreateParams(params),
		    });
		};
		class ZodPromise extends ZodType {
		    unwrap() {
		        return this._def.type;
		    }
		    _parse(input) {
		        const { ctx } = this._processInputParams(input);
		        if (ctx.parsedType !== util_1.ZodParsedType.promise &&
		            ctx.common.async === false) {
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                code: ZodError_1.ZodIssueCode.invalid_type,
		                expected: util_1.ZodParsedType.promise,
		                received: ctx.parsedType,
		            });
		            return parseUtil_1.INVALID;
		        }
		        const promisified = ctx.parsedType === util_1.ZodParsedType.promise
		            ? ctx.data
		            : Promise.resolve(ctx.data);
		        return (0, parseUtil_1.OK)(promisified.then((data) => {
		            return this._def.type.parseAsync(data, {
		                path: ctx.path,
		                errorMap: ctx.common.contextualErrorMap,
		            });
		        }));
		    }
		}
		exports.ZodPromise = ZodPromise;
		ZodPromise.create = (schema, params) => {
		    return new ZodPromise({
		        type: schema,
		        typeName: ZodFirstPartyTypeKind.ZodPromise,
		        ...processCreateParams(params),
		    });
		};
		class ZodEffects extends ZodType {
		    innerType() {
		        return this._def.schema;
		    }
		    sourceType() {
		        return this._def.schema._def.typeName === ZodFirstPartyTypeKind.ZodEffects
		            ? this._def.schema.sourceType()
		            : this._def.schema;
		    }
		    _parse(input) {
		        const { status, ctx } = this._processInputParams(input);
		        const effect = this._def.effect || null;
		        const checkCtx = {
		            addIssue: (arg) => {
		                (0, parseUtil_1.addIssueToContext)(ctx, arg);
		                if (arg.fatal) {
		                    status.abort();
		                }
		                else {
		                    status.dirty();
		                }
		            },
		            get path() {
		                return ctx.path;
		            },
		        };
		        checkCtx.addIssue = checkCtx.addIssue.bind(checkCtx);
		        if (effect.type === "preprocess") {
		            const processed = effect.transform(ctx.data, checkCtx);
		            if (ctx.common.async) {
		                return Promise.resolve(processed).then(async (processed) => {
		                    if (status.value === "aborted")
		                        return parseUtil_1.INVALID;
		                    const result = await this._def.schema._parseAsync({
		                        data: processed,
		                        path: ctx.path,
		                        parent: ctx,
		                    });
		                    if (result.status === "aborted")
		                        return parseUtil_1.INVALID;
		                    if (result.status === "dirty")
		                        return (0, parseUtil_1.DIRTY)(result.value);
		                    if (status.value === "dirty")
		                        return (0, parseUtil_1.DIRTY)(result.value);
		                    return result;
		                });
		            }
		            else {
		                if (status.value === "aborted")
		                    return parseUtil_1.INVALID;
		                const result = this._def.schema._parseSync({
		                    data: processed,
		                    path: ctx.path,
		                    parent: ctx,
		                });
		                if (result.status === "aborted")
		                    return parseUtil_1.INVALID;
		                if (result.status === "dirty")
		                    return (0, parseUtil_1.DIRTY)(result.value);
		                if (status.value === "dirty")
		                    return (0, parseUtil_1.DIRTY)(result.value);
		                return result;
		            }
		        }
		        if (effect.type === "refinement") {
		            const executeRefinement = (acc) => {
		                const result = effect.refinement(acc, checkCtx);
		                if (ctx.common.async) {
		                    return Promise.resolve(result);
		                }
		                if (result instanceof Promise) {
		                    throw new Error("Async refinement encountered during synchronous parse operation. Use .parseAsync instead.");
		                }
		                return acc;
		            };
		            if (ctx.common.async === false) {
		                const inner = this._def.schema._parseSync({
		                    data: ctx.data,
		                    path: ctx.path,
		                    parent: ctx,
		                });
		                if (inner.status === "aborted")
		                    return parseUtil_1.INVALID;
		                if (inner.status === "dirty")
		                    status.dirty();
		                // return value is ignored
		                executeRefinement(inner.value);
		                return { status: status.value, value: inner.value };
		            }
		            else {
		                return this._def.schema
		                    ._parseAsync({ data: ctx.data, path: ctx.path, parent: ctx })
		                    .then((inner) => {
		                    if (inner.status === "aborted")
		                        return parseUtil_1.INVALID;
		                    if (inner.status === "dirty")
		                        status.dirty();
		                    return executeRefinement(inner.value).then(() => {
		                        return { status: status.value, value: inner.value };
		                    });
		                });
		            }
		        }
		        if (effect.type === "transform") {
		            if (ctx.common.async === false) {
		                const base = this._def.schema._parseSync({
		                    data: ctx.data,
		                    path: ctx.path,
		                    parent: ctx,
		                });
		                if (!(0, parseUtil_1.isValid)(base))
		                    return base;
		                const result = effect.transform(base.value, checkCtx);
		                if (result instanceof Promise) {
		                    throw new Error(`Asynchronous transform encountered during synchronous parse operation. Use .parseAsync instead.`);
		                }
		                return { status: status.value, value: result };
		            }
		            else {
		                return this._def.schema
		                    ._parseAsync({ data: ctx.data, path: ctx.path, parent: ctx })
		                    .then((base) => {
		                    if (!(0, parseUtil_1.isValid)(base))
		                        return base;
		                    return Promise.resolve(effect.transform(base.value, checkCtx)).then((result) => ({ status: status.value, value: result }));
		                });
		            }
		        }
		        util_1.util.assertNever(effect);
		    }
		}
		exports.ZodEffects = ZodEffects;
		exports.ZodTransformer = ZodEffects;
		ZodEffects.create = (schema, effect, params) => {
		    return new ZodEffects({
		        schema,
		        typeName: ZodFirstPartyTypeKind.ZodEffects,
		        effect,
		        ...processCreateParams(params),
		    });
		};
		ZodEffects.createWithPreprocess = (preprocess, schema, params) => {
		    return new ZodEffects({
		        schema,
		        effect: { type: "preprocess", transform: preprocess },
		        typeName: ZodFirstPartyTypeKind.ZodEffects,
		        ...processCreateParams(params),
		    });
		};
		class ZodOptional extends ZodType {
		    _parse(input) {
		        const parsedType = this._getType(input);
		        if (parsedType === util_1.ZodParsedType.undefined) {
		            return (0, parseUtil_1.OK)(undefined);
		        }
		        return this._def.innerType._parse(input);
		    }
		    unwrap() {
		        return this._def.innerType;
		    }
		}
		exports.ZodOptional = ZodOptional;
		ZodOptional.create = (type, params) => {
		    return new ZodOptional({
		        innerType: type,
		        typeName: ZodFirstPartyTypeKind.ZodOptional,
		        ...processCreateParams(params),
		    });
		};
		class ZodNullable extends ZodType {
		    _parse(input) {
		        const parsedType = this._getType(input);
		        if (parsedType === util_1.ZodParsedType.null) {
		            return (0, parseUtil_1.OK)(null);
		        }
		        return this._def.innerType._parse(input);
		    }
		    unwrap() {
		        return this._def.innerType;
		    }
		}
		exports.ZodNullable = ZodNullable;
		ZodNullable.create = (type, params) => {
		    return new ZodNullable({
		        innerType: type,
		        typeName: ZodFirstPartyTypeKind.ZodNullable,
		        ...processCreateParams(params),
		    });
		};
		class ZodDefault extends ZodType {
		    _parse(input) {
		        const { ctx } = this._processInputParams(input);
		        let data = ctx.data;
		        if (ctx.parsedType === util_1.ZodParsedType.undefined) {
		            data = this._def.defaultValue();
		        }
		        return this._def.innerType._parse({
		            data,
		            path: ctx.path,
		            parent: ctx,
		        });
		    }
		    removeDefault() {
		        return this._def.innerType;
		    }
		}
		exports.ZodDefault = ZodDefault;
		ZodDefault.create = (type, params) => {
		    return new ZodDefault({
		        innerType: type,
		        typeName: ZodFirstPartyTypeKind.ZodDefault,
		        defaultValue: typeof params.default === "function"
		            ? params.default
		            : () => params.default,
		        ...processCreateParams(params),
		    });
		};
		class ZodCatch extends ZodType {
		    _parse(input) {
		        const { ctx } = this._processInputParams(input);
		        // newCtx is used to not collect issues from inner types in ctx
		        const newCtx = {
		            ...ctx,
		            common: {
		                ...ctx.common,
		                issues: [],
		            },
		        };
		        const result = this._def.innerType._parse({
		            data: newCtx.data,
		            path: newCtx.path,
		            parent: {
		                ...newCtx,
		            },
		        });
		        if ((0, parseUtil_1.isAsync)(result)) {
		            return result.then((result) => {
		                return {
		                    status: "valid",
		                    value: result.status === "valid"
		                        ? result.value
		                        : this._def.catchValue({
		                            get error() {
		                                return new ZodError_1.ZodError(newCtx.common.issues);
		                            },
		                            input: newCtx.data,
		                        }),
		                };
		            });
		        }
		        else {
		            return {
		                status: "valid",
		                value: result.status === "valid"
		                    ? result.value
		                    : this._def.catchValue({
		                        get error() {
		                            return new ZodError_1.ZodError(newCtx.common.issues);
		                        },
		                        input: newCtx.data,
		                    }),
		            };
		        }
		    }
		    removeCatch() {
		        return this._def.innerType;
		    }
		}
		exports.ZodCatch = ZodCatch;
		ZodCatch.create = (type, params) => {
		    return new ZodCatch({
		        innerType: type,
		        typeName: ZodFirstPartyTypeKind.ZodCatch,
		        catchValue: typeof params.catch === "function" ? params.catch : () => params.catch,
		        ...processCreateParams(params),
		    });
		};
		class ZodNaN extends ZodType {
		    _parse(input) {
		        const parsedType = this._getType(input);
		        if (parsedType !== util_1.ZodParsedType.nan) {
		            const ctx = this._getOrReturnCtx(input);
		            (0, parseUtil_1.addIssueToContext)(ctx, {
		                code: ZodError_1.ZodIssueCode.invalid_type,
		                expected: util_1.ZodParsedType.nan,
		                received: ctx.parsedType,
		            });
		            return parseUtil_1.INVALID;
		        }
		        return { status: "valid", value: input.data };
		    }
		}
		exports.ZodNaN = ZodNaN;
		ZodNaN.create = (params) => {
		    return new ZodNaN({
		        typeName: ZodFirstPartyTypeKind.ZodNaN,
		        ...processCreateParams(params),
		    });
		};
		exports.BRAND = Symbol("zod_brand");
		class ZodBranded extends ZodType {
		    _parse(input) {
		        const { ctx } = this._processInputParams(input);
		        const data = ctx.data;
		        return this._def.type._parse({
		            data,
		            path: ctx.path,
		            parent: ctx,
		        });
		    }
		    unwrap() {
		        return this._def.type;
		    }
		}
		exports.ZodBranded = ZodBranded;
		class ZodPipeline extends ZodType {
		    _parse(input) {
		        const { status, ctx } = this._processInputParams(input);
		        if (ctx.common.async) {
		            const handleAsync = async () => {
		                const inResult = await this._def.in._parseAsync({
		                    data: ctx.data,
		                    path: ctx.path,
		                    parent: ctx,
		                });
		                if (inResult.status === "aborted")
		                    return parseUtil_1.INVALID;
		                if (inResult.status === "dirty") {
		                    status.dirty();
		                    return (0, parseUtil_1.DIRTY)(inResult.value);
		                }
		                else {
		                    return this._def.out._parseAsync({
		                        data: inResult.value,
		                        path: ctx.path,
		                        parent: ctx,
		                    });
		                }
		            };
		            return handleAsync();
		        }
		        else {
		            const inResult = this._def.in._parseSync({
		                data: ctx.data,
		                path: ctx.path,
		                parent: ctx,
		            });
		            if (inResult.status === "aborted")
		                return parseUtil_1.INVALID;
		            if (inResult.status === "dirty") {
		                status.dirty();
		                return {
		                    status: "dirty",
		                    value: inResult.value,
		                };
		            }
		            else {
		                return this._def.out._parseSync({
		                    data: inResult.value,
		                    path: ctx.path,
		                    parent: ctx,
		                });
		            }
		        }
		    }
		    static create(a, b) {
		        return new ZodPipeline({
		            in: a,
		            out: b,
		            typeName: ZodFirstPartyTypeKind.ZodPipeline,
		        });
		    }
		}
		exports.ZodPipeline = ZodPipeline;
		class ZodReadonly extends ZodType {
		    _parse(input) {
		        const result = this._def.innerType._parse(input);
		        const freeze = (data) => {
		            if ((0, parseUtil_1.isValid)(data)) {
		                data.value = Object.freeze(data.value);
		            }
		            return data;
		        };
		        return (0, parseUtil_1.isAsync)(result)
		            ? result.then((data) => freeze(data))
		            : freeze(result);
		    }
		    unwrap() {
		        return this._def.innerType;
		    }
		}
		exports.ZodReadonly = ZodReadonly;
		ZodReadonly.create = (type, params) => {
		    return new ZodReadonly({
		        innerType: type,
		        typeName: ZodFirstPartyTypeKind.ZodReadonly,
		        ...processCreateParams(params),
		    });
		};
		function custom(check, params = {}, 
		/**
		 * @deprecated
		 *
		 * Pass `fatal` into the params object instead:
		 *
		 * ```ts
		 * z.string().custom((val) => val.length > 5, { fatal: false })
		 * ```
		 *
		 */
		fatal) {
		    if (check)
		        return ZodAny.create().superRefine((data, ctx) => {
		            var _a, _b;
		            if (!check(data)) {
		                const p = typeof params === "function"
		                    ? params(data)
		                    : typeof params === "string"
		                        ? { message: params }
		                        : params;
		                const _fatal = (_b = (_a = p.fatal) !== null && _a !== void 0 ? _a : fatal) !== null && _b !== void 0 ? _b : true;
		                const p2 = typeof p === "string" ? { message: p } : p;
		                ctx.addIssue({ code: "custom", ...p2, fatal: _fatal });
		            }
		        });
		    return ZodAny.create();
		}
		exports.custom = custom;
		exports.late = {
		    object: ZodObject.lazycreate,
		};
		var ZodFirstPartyTypeKind;
		(function (ZodFirstPartyTypeKind) {
		    ZodFirstPartyTypeKind["ZodString"] = "ZodString";
		    ZodFirstPartyTypeKind["ZodNumber"] = "ZodNumber";
		    ZodFirstPartyTypeKind["ZodNaN"] = "ZodNaN";
		    ZodFirstPartyTypeKind["ZodBigInt"] = "ZodBigInt";
		    ZodFirstPartyTypeKind["ZodBoolean"] = "ZodBoolean";
		    ZodFirstPartyTypeKind["ZodDate"] = "ZodDate";
		    ZodFirstPartyTypeKind["ZodSymbol"] = "ZodSymbol";
		    ZodFirstPartyTypeKind["ZodUndefined"] = "ZodUndefined";
		    ZodFirstPartyTypeKind["ZodNull"] = "ZodNull";
		    ZodFirstPartyTypeKind["ZodAny"] = "ZodAny";
		    ZodFirstPartyTypeKind["ZodUnknown"] = "ZodUnknown";
		    ZodFirstPartyTypeKind["ZodNever"] = "ZodNever";
		    ZodFirstPartyTypeKind["ZodVoid"] = "ZodVoid";
		    ZodFirstPartyTypeKind["ZodArray"] = "ZodArray";
		    ZodFirstPartyTypeKind["ZodObject"] = "ZodObject";
		    ZodFirstPartyTypeKind["ZodUnion"] = "ZodUnion";
		    ZodFirstPartyTypeKind["ZodDiscriminatedUnion"] = "ZodDiscriminatedUnion";
		    ZodFirstPartyTypeKind["ZodIntersection"] = "ZodIntersection";
		    ZodFirstPartyTypeKind["ZodTuple"] = "ZodTuple";
		    ZodFirstPartyTypeKind["ZodRecord"] = "ZodRecord";
		    ZodFirstPartyTypeKind["ZodMap"] = "ZodMap";
		    ZodFirstPartyTypeKind["ZodSet"] = "ZodSet";
		    ZodFirstPartyTypeKind["ZodFunction"] = "ZodFunction";
		    ZodFirstPartyTypeKind["ZodLazy"] = "ZodLazy";
		    ZodFirstPartyTypeKind["ZodLiteral"] = "ZodLiteral";
		    ZodFirstPartyTypeKind["ZodEnum"] = "ZodEnum";
		    ZodFirstPartyTypeKind["ZodEffects"] = "ZodEffects";
		    ZodFirstPartyTypeKind["ZodNativeEnum"] = "ZodNativeEnum";
		    ZodFirstPartyTypeKind["ZodOptional"] = "ZodOptional";
		    ZodFirstPartyTypeKind["ZodNullable"] = "ZodNullable";
		    ZodFirstPartyTypeKind["ZodDefault"] = "ZodDefault";
		    ZodFirstPartyTypeKind["ZodCatch"] = "ZodCatch";
		    ZodFirstPartyTypeKind["ZodPromise"] = "ZodPromise";
		    ZodFirstPartyTypeKind["ZodBranded"] = "ZodBranded";
		    ZodFirstPartyTypeKind["ZodPipeline"] = "ZodPipeline";
		    ZodFirstPartyTypeKind["ZodReadonly"] = "ZodReadonly";
		})(ZodFirstPartyTypeKind = exports.ZodFirstPartyTypeKind || (exports.ZodFirstPartyTypeKind = {}));
		const instanceOfType = (
		// const instanceOfType = <T extends new (...args: any[]) => any>(
		cls, params = {
		    message: `Input not instance of ${cls.name}`,
		}) => custom((data) => data instanceof cls, params);
		exports.instanceof = instanceOfType;
		const stringType = ZodString.create;
		exports.string = stringType;
		const numberType = ZodNumber.create;
		exports.number = numberType;
		const nanType = ZodNaN.create;
		exports.nan = nanType;
		const bigIntType = ZodBigInt.create;
		exports.bigint = bigIntType;
		const booleanType = ZodBoolean.create;
		exports.boolean = booleanType;
		const dateType = ZodDate.create;
		exports.date = dateType;
		const symbolType = ZodSymbol.create;
		exports.symbol = symbolType;
		const undefinedType = ZodUndefined.create;
		exports.undefined = undefinedType;
		const nullType = ZodNull.create;
		exports.null = nullType;
		const anyType = ZodAny.create;
		exports.any = anyType;
		const unknownType = ZodUnknown.create;
		exports.unknown = unknownType;
		const neverType = ZodNever.create;
		exports.never = neverType;
		const voidType = ZodVoid.create;
		exports.void = voidType;
		const arrayType = ZodArray.create;
		exports.array = arrayType;
		const objectType = ZodObject.create;
		exports.object = objectType;
		const strictObjectType = ZodObject.strictCreate;
		exports.strictObject = strictObjectType;
		const unionType = ZodUnion.create;
		exports.union = unionType;
		const discriminatedUnionType = ZodDiscriminatedUnion.create;
		exports.discriminatedUnion = discriminatedUnionType;
		const intersectionType = ZodIntersection.create;
		exports.intersection = intersectionType;
		const tupleType = ZodTuple.create;
		exports.tuple = tupleType;
		const recordType = ZodRecord.create;
		exports.record = recordType;
		const mapType = ZodMap.create;
		exports.map = mapType;
		const setType = ZodSet.create;
		exports.set = setType;
		const functionType = ZodFunction.create;
		exports.function = functionType;
		const lazyType = ZodLazy.create;
		exports.lazy = lazyType;
		const literalType = ZodLiteral.create;
		exports.literal = literalType;
		const enumType = ZodEnum.create;
		exports.enum = enumType;
		const nativeEnumType = ZodNativeEnum.create;
		exports.nativeEnum = nativeEnumType;
		const promiseType = ZodPromise.create;
		exports.promise = promiseType;
		const effectsType = ZodEffects.create;
		exports.effect = effectsType;
		exports.transformer = effectsType;
		const optionalType = ZodOptional.create;
		exports.optional = optionalType;
		const nullableType = ZodNullable.create;
		exports.nullable = nullableType;
		const preprocessType = ZodEffects.createWithPreprocess;
		exports.preprocess = preprocessType;
		const pipelineType = ZodPipeline.create;
		exports.pipeline = pipelineType;
		const ostring = () => stringType().optional();
		exports.ostring = ostring;
		const onumber = () => numberType().optional();
		exports.onumber = onumber;
		const oboolean = () => booleanType().optional();
		exports.oboolean = oboolean;
		exports.coerce = {
		    string: ((arg) => ZodString.create({ ...arg, coerce: true })),
		    number: ((arg) => ZodNumber.create({ ...arg, coerce: true })),
		    boolean: ((arg) => ZodBoolean.create({
		        ...arg,
		        coerce: true,
		    })),
		    bigint: ((arg) => ZodBigInt.create({ ...arg, coerce: true })),
		    date: ((arg) => ZodDate.create({ ...arg, coerce: true })),
		};
		exports.NEVER = parseUtil_1.INVALID; 
	} (types$1));
	return types$1;
}

var hasRequiredExternal;

function requireExternal () {
	if (hasRequiredExternal) return external;
	hasRequiredExternal = 1;
	(function (exports) {
		var __createBinding = (external && external.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __exportStar = (external && external.__exportStar) || function(m, exports) {
		    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		__exportStar(/*@__PURE__*/ requireErrors(), exports);
		__exportStar(/*@__PURE__*/ requireParseUtil(), exports);
		__exportStar(/*@__PURE__*/ requireTypeAliases(), exports);
		__exportStar(/*@__PURE__*/ requireUtil$1(), exports);
		__exportStar(/*@__PURE__*/ requireTypes(), exports);
		__exportStar(/*@__PURE__*/ requireZodError(), exports); 
	} (external));
	return external;
}

var hasRequiredLib;

function requireLib () {
	if (hasRequiredLib) return lib;
	hasRequiredLib = 1;
	(function (exports) {
		var __createBinding = (lib && lib.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __setModuleDefault = (lib && lib.__setModuleDefault) || (Object.create ? (function(o, v) {
		    Object.defineProperty(o, "default", { enumerable: true, value: v });
		}) : function(o, v) {
		    o["default"] = v;
		});
		var __importStar = (lib && lib.__importStar) || function (mod) {
		    if (mod && mod.__esModule) return mod;
		    var result = {};
		    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
		    __setModuleDefault(result, mod);
		    return result;
		};
		var __exportStar = (lib && lib.__exportStar) || function(m, exports) {
		    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.z = void 0;
		const z = __importStar(/*@__PURE__*/ requireExternal());
		exports.z = z;
		__exportStar(/*@__PURE__*/ requireExternal(), exports);
		exports.default = z; 
	} (lib));
	return lib;
}

var out = {};

var any$1 = {};

var array$1 = {};

var hasRequiredArray$1;

function requireArray$1 () {
	if (hasRequiredArray$1) return array$1;
	hasRequiredArray$1 = 1;
	(function (exports) {
		// functions sorted alphabetically
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.zip = exports.shuffle = exports.partitionT = exports.partition = exports.minBy = exports.mergeBy = exports.maxBy = exports.indexBy = exports.groupBy = exports.findLastIndex = exports.diffBy = exports.distinctBy = exports.distinct = exports.collectOne = exports.collect = exports.arraySame = void 0;
		const arraySame = (a1, a2, eq) => a1.length === a2.length && a1.every((e, i) => eq(e, a2[i]));
		exports.arraySame = arraySame;
		const collect = (arr, f) => arr.map(f).filter((u) => u !== undefined);
		exports.collect = collect;
		const collectOne = (arr, f) => {
		    for (const i of arr) {
		        const u = f(i);
		        if (u !== undefined) {
		            return u;
		        }
		    }
		    return undefined;
		};
		exports.collectOne = collectOne;
		const distinct = (arr) => {
		    const seen = new Set();
		    return arr.filter(t => {
		        if (seen.has(t))
		            return false;
		        seen.add(t);
		        return true;
		    });
		};
		exports.distinct = distinct;
		const distinctBy = (arr, by) => {
		    const seen = new Set();
		    return arr.filter(t => {
		        const key = by(t);
		        if (seen.has(key))
		            return false;
		        seen.add(key);
		        return true;
		    });
		};
		exports.distinctBy = distinctBy;
		const diffBy = (arr1, arr2, f) => {
		    const left = [];
		    const both = [];
		    const obj2 = (0, exports.indexBy)(arr2, f);
		    arr1.forEach((a1, i) => {
		        const k = f(a1, i);
		        const a2 = obj2[k];
		        if (a2 === undefined) {
		            left.push(a1);
		        }
		        else {
		            both.push({ left: a1, right: a2 });
		        }
		        delete obj2[k];
		    });
		    const right = arr2.filter((a2, i) => obj2[f(a2, i)] !== undefined);
		    return { left, right, both };
		};
		exports.diffBy = diffBy;
		const findLastIndex = (arr, p) => {
		    let i = arr.length - 1;
		    while (i >= 0) {
		        if (p(arr[i])) {
		            return i;
		        }
		        i--;
		    }
		    return -1;
		};
		exports.findLastIndex = findLastIndex;
		const groupBy = (list, getKey) => list.reduce((acc, item) => {
		    const key = getKey(item);
		    if (!acc[key])
		        acc[key] = [];
		    acc[key].push(item);
		    return acc;
		}, {});
		exports.groupBy = groupBy;
		// similar to group by but keys must be unique so values are not a list
		const indexBy = (list, getKey) => list.reduce((acc, item, i) => {
		    acc[getKey(item, i)] = item;
		    return acc;
		}, {});
		exports.indexBy = indexBy;
		const maxBy = (arr, f) => {
		    let res = undefined;
		    arr.forEach(i => {
		        if (res === undefined) {
		            res = i;
		        }
		        else if (f(res) < f(i)) {
		            res = i;
		        }
		    });
		    return res;
		};
		exports.maxBy = maxBy;
		const mergeBy = (a1, a2, getKey, merge = (i1, i2) => ({ ...i1, ...i2 })) => {
		    let others = a2.map(i2 => ({ key: getKey(i2), value: i2 }));
		    const merged = a1.map(i1 => {
		        const key = getKey(i1);
		        const [toMerge, rest] = (0, exports.partition)(others, o => o.key === key);
		        others = rest;
		        return toMerge.reduce((acc, cur) => merge(acc, cur.value), i1);
		    });
		    return merged.concat(others.map(o => o.value));
		};
		exports.mergeBy = mergeBy;
		const minBy = (arr, f) => {
		    let res = undefined;
		    arr.forEach(i => {
		        if (res === undefined) {
		            res = i;
		        }
		        else if (f(res) > f(i)) {
		            res = i;
		        }
		    });
		    return res;
		};
		exports.minBy = minBy;
		const partition = (arr, p) => {
		    const ok = [];
		    const ko = [];
		    arr.forEach(i => p(i) ? ok.push(i) : ko.push(i));
		    return [ok, ko];
		};
		exports.partition = partition;
		const partitionT = (arr, isT) => {
		    const ts = [];
		    const us = [];
		    arr.forEach(i => isT(i) ? ts.push(i) : us.push(i));
		    return [ts, us];
		};
		exports.partitionT = partitionT;
		const shuffle = (array) => {
		    // Fisher-Yates shuffle
		    const arr = [...array]; // shallow copy of array to avoid in-place updates
		    let count = arr.length;
		    let rand;
		    let temp;
		    while (count) {
		        rand = Math.random() * count-- | 0;
		        temp = arr[count];
		        arr[count] = arr[rand];
		        arr[rand] = temp;
		    }
		    return arr;
		};
		exports.shuffle = shuffle;
		const zip = (list1, list2) => list1.slice(0, list2.length).map((t, i) => [t, list2[i]]);
		exports.zip = zip;
		
	} (array$1));
	return array$1;
}

var object$1 = {};

var validation = {};

var hasRequiredValidation;

function requireValidation () {
	if (hasRequiredValidation) return validation;
	hasRequiredValidation = 1;
	// functions sorted alphabetically
	Object.defineProperty(validation, "__esModule", { value: true });
	validation.isNotUndefined = validation.isObject = void 0;
	validation.isEmpty = isEmpty;
	validation.isNever = isNever;
	function isEmpty(v) {
	    return v === undefined
	        || v === null
	        || v === ''
	        || (Array.isArray(v) && v.length === 0)
	        || (typeof v === 'object' && Object.keys(v).length === 0);
	}
	const isObject = (value) => typeof value === "object" && !Array.isArray(value) && value !== null;
	validation.isObject = isObject;
	const isNotUndefined = (t) => t !== undefined;
	validation.isNotUndefined = isNotUndefined;
	// this function should never be called, useful for making sure checks are exhaustive
	function isNever(p) {
	    throw new Error('a never param has been produced 🤯');
	}
	
	return validation;
}

var hasRequiredObject$1;

function requireObject$1 () {
	if (hasRequiredObject$1) return object$1;
	hasRequiredObject$1 = 1;
	Object.defineProperty(object$1, "__esModule", { value: true });
	object$1.equalDeep = equalDeep;
	object$1.filterKeys = filterKeys;
	object$1.filterValues = filterValues;
	object$1.mapEntries = mapEntries;
	object$1.mapValues = mapValues;
	object$1.mapEntriesAsync = mapEntriesAsync;
	object$1.mapValuesAsync = mapValuesAsync;
	object$1.minusFieldsDeep = minusFieldsDeep;
	object$1.objectSame = objectSame;
	object$1.removeEmpty = removeEmpty;
	object$1.mapEntriesDeep = mapEntriesDeep;
	object$1.removeFieldsDeep = removeFieldsDeep;
	object$1.removeUndefined = removeUndefined;
	const validation_1 = requireValidation();
	const any_1 = requireAny$1();
	// functions sorted alphabetically
	function equalDeep(a, b) {
	    if (typeof a === 'string' || typeof a === 'number' || typeof a === 'boolean' || typeof a === 'symbol' || a === null || a === undefined) {
	        return a === b;
	    }
	    else if (Array.isArray(a)) {
	        return Array.isArray(b) && a.length === b.length && a.every((v, i) => equalDeep(v, b[i]));
	    }
	    else if (typeof a === 'object') {
	        if (typeof b === 'object' && !Array.isArray(b) && b !== null) {
	            return Object.keys(a).length === Object.keys(b).length && Object.entries(a).every(([k, v]) => equalDeep(v, b[k]));
	        }
	    }
	    return false;
	}
	function filterKeys(obj, p) {
	    return Object.fromEntries(Object.entries(obj).filter(([k,]) => p(k)));
	}
	function filterValues(obj, p) {
	    return Object.fromEntries(Object.entries(obj).filter(([, v]) => p(v)));
	}
	function mapEntries(obj, f) {
	    return Object.fromEntries(Object.entries(obj).map(([k, v]) => [k, f(k, v)]));
	}
	function mapValues(obj, f) {
	    return mapEntries(obj, (_, v) => f(v));
	}
	function mapEntriesAsync(obj, f) {
	    return Promise.all(Object.entries(obj).map(([k, v]) => f(k, v).then(u => [k, u]))).then(Object.fromEntries);
	}
	function mapValuesAsync(obj, f) {
	    return mapEntriesAsync(obj, (_, v) => f(v));
	}
	// remove fields from `source` if they exist and have the same value in `ref`
	function minusFieldsDeep(source, ref) {
	    if (typeof source === 'string' || typeof source === 'number' || typeof source === 'boolean' || typeof source === 'symbol' || source === null || source === undefined) {
	        return source !== ref ? source : undefined;
	    }
	    else if (Array.isArray(source)) {
	        if (Array.isArray(ref)) {
	            const arr = source.map((v, i) => minusFieldsDeep(v, ref[i]));
	            return arr.some(v => v !== undefined) ? arr : undefined;
	        }
	        else {
	            return source;
	        }
	    }
	    else if (typeof source === 'object') {
	        if (typeof ref === 'object' && !Array.isArray(ref) && ref !== null) {
	            const obj = Object.fromEntries(Object.entries(source).map(([key, value]) => [key, minusFieldsDeep(value, ref[key])]).filter(([, v]) => v !== undefined));
	            return Object.keys(obj).length > 0 ? obj : undefined;
	        }
	        else {
	            return source;
	        }
	    }
	}
	function objectSame(a, b) {
	    if (a instanceof Date && b instanceof Date)
	        return a.getTime() === b.getTime();
	    if ((a instanceof Number || a instanceof Boolean) && (b instanceof Number || b instanceof Boolean))
	        return a.valueOf() === b.valueOf();
	    const aEntries = Object.entries(a);
	    const bEntries = Object.entries(b);
	    return aEntries.length === bEntries.length && aEntries.every(([key, value]) => (0, any_1.anySame)(value, b[key]));
	}
	function removeEmpty(obj) {
	    return filterValues(obj, v => !(0, validation_1.isEmpty)(v));
	}
	function mapEntriesDeep(obj, f, path = []) {
	    if (Array.isArray(obj)) {
	        return obj.map((item, i) => mapEntriesDeep(item, f, [...path, i]));
	    }
	    if (typeof obj === 'object' && obj !== null) {
	        return Object.fromEntries(Object.entries(obj).map(([key, value]) => {
	            const entryPath = [...path, key];
	            return [key, f(entryPath, mapEntriesDeep(value, f, entryPath))];
	        }));
	    }
	    return obj;
	}
	function removeFieldsDeep(obj, keysToRemove) {
	    if (Array.isArray(obj)) {
	        return obj.map(item => removeFieldsDeep(item, keysToRemove));
	    }
	    if (typeof obj === 'object' && obj !== null) {
	        const res = {};
	        Object.keys(obj).forEach(key => {
	            if (keysToRemove.indexOf(key) < 0) {
	                res[key] = removeFieldsDeep(obj[key], keysToRemove);
	            }
	        });
	        return res;
	    }
	    return obj;
	}
	function removeUndefined(obj) {
	    return filterValues(obj, v => v !== undefined);
	}
	
	return object$1;
}

var hasRequiredAny$1;

function requireAny$1 () {
	if (hasRequiredAny$1) return any$1;
	hasRequiredAny$1 = 1;
	Object.defineProperty(any$1, "__esModule", { value: true });
	any$1.anySame = anySame;
	any$1.getValueDeep = getValueDeep;
	any$1.limitDepth = limitDepth;
	any$1.anyAsString = anyAsString;
	const array_1 = requireArray$1();
	const object_1 = requireObject$1();
	function anySame(a, b) {
	    if (a === null && b === null)
	        return true;
	    if (a === undefined && b === undefined)
	        return true;
	    if (Array.isArray(a) && Array.isArray(b))
	        return (0, array_1.arraySame)(a, b, anySame);
	    if (typeof a === 'object' && typeof b === 'object')
	        return (0, object_1.objectSame)(a, b);
	    return a === b;
	}
	function getValueDeep(value, path) {
	    if (path.length === 0 || value === null || typeof value !== 'object') {
	        return value;
	    }
	    else {
	        const [head, ...tail] = path;
	        return getValueDeep(value[head], tail);
	    }
	}
	function limitDepth(value, depth) {
	    if (Array.isArray(value)) {
	        return depth <= 0 ? '...' : (value.length > 3 ? value.slice(0, 3).concat(['...']) : value).map(v => limitDepth(v, depth - 1));
	    }
	    else if (value === null) {
	        return value;
	    }
	    else if (typeof value === 'object') {
	        return depth <= 0 ? '...' : Object.fromEntries(Object.entries(value).map(([key, value]) => [key, limitDepth(value, depth - 1)]));
	    }
	    else if (typeof value === 'string') {
	        return value.length > 30 ? value.substring(0, 30) + '...' : value;
	    }
	    else {
	        return value;
	    }
	}
	function anyAsString(value) {
	    if (typeof value === 'string')
	        return value;
	    if (value === undefined)
	        return '';
	    if (value === null)
	        return 'null';
	    if (typeof value === 'number')
	        return isNaN(value) || !isFinite(value) ? 'null' : value.toString();
	    if (typeof value === 'boolean')
	        return value ? 'true' : 'false';
	    if (Array.isArray(value))
	        return '[' + value.map(anyAsString).join(', ') + ']';
	    if (value instanceof Date)
	        return value.toISOString();
	    if (value instanceof String || value instanceof Number || value instanceof Boolean)
	        value.toString();
	    if (typeof value === 'object')
	        return '{' + Object.entries(value).map(([k, v]) => k + ': ' + anyAsString(v)).join(', ') + '}';
	    return '';
	}
	
	return any$1;
}

var email = {};

var hasRequiredEmail;

function requireEmail () {
	if (hasRequiredEmail) return email;
	hasRequiredEmail = 1;
	Object.defineProperty(email, "__esModule", { value: true });
	email.publicEmailDomains = void 0;
	email.emailParse = emailParse;
	const object_1 = requireObject$1();
	function emailParse(email) {
	    const [, domain] = email.match(/^[^\s@]+@([^\s@.]+\.[^\s@.]+)$/) || [];
	    return (0, object_1.removeUndefined)({ full: email, domain });
	}
	// https://email-verify.my-addr.com/list-of-most-popular-email-domains.php
	email.publicEmailDomains = [
	    'aol.com',
	    'fastmail.com',
	    'gmail.com',
	    'hotmail.co.uk',
	    'hotmail.com',
	    'hotmail.fr',
	    'icloud.com',
	    'mail.ru',
	    'mail.com',
	    'msn.com',
	    'orange.fr',
	    'outlook.com',
	    'outlook.fr',
	    'protonmail.com',
	    'wanadoo.fr',
	    'yahoo.com',
	    'yahoo.fr',
	    'yandex.com',
	    'yandex.ru',
	    'zoho.com',
	];
	
	return email;
}

var error$1 = {};

var hasRequiredError$1;

function requireError$1 () {
	if (hasRequiredError$1) return error$1;
	hasRequiredError$1 = 1;
	Object.defineProperty(error$1, "__esModule", { value: true });
	error$1.errorToString = errorToString;
	const validation_1 = requireValidation();
	function errorToString(err) {
	    if (err instanceof Error) {
	        return err.message + showCause(err);
	    }
	    else if (typeof err === 'string') {
	        return err;
	    }
	    else if ((0, validation_1.isObject)(err) && typeof err.error === 'string') {
	        return typeof err.message === 'string' ? `${err.error}: ${err.message}` : err.error;
	    }
	    else if ((0, validation_1.isObject)(err) && typeof err.message === 'string') {
	        return err.message;
	    }
	    else if ((0, validation_1.isObject)(err) && (0, validation_1.isObject)(err.json) && typeof err.json.message === 'string') {
	        return err.json.message;
	    }
	    else {
	        return JSON.stringify(err);
	    }
	}
	function showCause(err) {
	    if ('cause' in err && err.cause instanceof Error) {
	        return `\n  cause: ${errorToString(err.cause)}`;
	    }
	    else {
	        return '';
	    }
	}
	
	return error$1;
}

var json = {};

var hasRequiredJson;

function requireJson () {
	if (hasRequiredJson) return json;
	hasRequiredJson = 1;
	// functions sorted alphabetically
	Object.defineProperty(json, "__esModule", { value: true });
	json.safeJsonParse = safeJsonParse;
	json.stringify = stringify;
	function safeJsonParse(value) {
	    try {
	        return JSON.parse(value);
	    }
	    catch (e) {
	        return value;
	    }
	}
	// same as JSON.stringify but allow flexible indentation
	function stringify(value, indent = undefined) {
	    return stringifyInner(value, indent, [], 0);
	}
	function stringifyInner(value, indent, path, nesting) {
	    if (value === null) {
	        return 'null';
	    }
	    else if (typeof value === 'number') {
	        return isNaN(value) || !isFinite(value) ? 'null' : value.toString();
	    }
	    else if (typeof value === 'boolean') {
	        return value ? 'true' : 'false';
	    }
	    else if (typeof value === 'string' || value instanceof String) {
	        return JSON.stringify(value); // 🤷 (escaping was not worth coding ^^)
	    }
	    else if (Array.isArray(value)) {
	        const { inner, end, depth } = stringifyIndent(indent, path, value, nesting);
	        const items = value.map((v, i) => {
	            const asNull = v === undefined || typeof v === 'function' || typeof v === 'symbol';
	            return inner + stringifyInner(asNull ? null : v, indent, [...path, i], depth);
	        });
	        return items.length > 0 ? '[' + items.join(!inner && typeof indent === 'function' ? ', ' : ',') + end + ']' : '[]';
	    }
	    else if (typeof value === 'object') {
	        if (value instanceof Date) {
	            return `"${value.toISOString()}"`;
	        }
	        else if (value instanceof Number || value instanceof Boolean) {
	            return value.toString();
	        }
	        else if ('toJSON' in value && typeof value.toJSON === 'function') {
	            return stringifyInner(value.toJSON(), indent, path, nesting);
	        }
	        else {
	            const { inner, end, depth } = stringifyIndent(indent, path, value, nesting);
	            const items = Object.entries(value).filter(([, v]) => v !== undefined && typeof v !== 'function' && typeof v !== 'symbol').map(([k, v]) => {
	                return inner + '"' + k + '":' + (inner || typeof indent === 'function' ? ' ' : '') + stringifyInner(v, indent, [...path, k], depth);
	            });
	            return items.length > 0 ? '{' + items.join(!inner && typeof indent === 'function' ? ', ' : ',') + end + '}' : '{}';
	        }
	    }
	    return '';
	}
	function stringifyIndent(indent, path, value, nesting) {
	    if (indent === undefined || indent === 0 || indent === '') {
	        return { inner: '', end: '', depth: nesting };
	    }
	    else if (typeof indent === 'number') {
	        return { inner: '\n' + ' '.repeat(indent * (nesting + 1)), end: '\n' + ' '.repeat(indent * nesting), depth: nesting + 1 };
	    }
	    else if (typeof indent === 'string') {
	        return { inner: '\n' + indent.repeat(path.length + 1), end: '\n' + indent.repeat(path.length), depth: nesting + 1 };
	    }
	    else {
	        return stringifyIndent(indent(path, value), path, value, nesting);
	    }
	}
	
	return json;
}

var logger = {};

var hasRequiredLogger;

function requireLogger () {
	if (hasRequiredLogger) return logger;
	hasRequiredLogger = 1;
	Object.defineProperty(logger, "__esModule", { value: true });
	
	return logger;
}

var number$1 = {};

var hasRequiredNumber$1;

function requireNumber$1 () {
	if (hasRequiredNumber$1) return number$1;
	hasRequiredNumber$1 = 1;
	// functions sorted alphabetically
	Object.defineProperty(number$1, "__esModule", { value: true });
	number$1.prettyNumber = prettyNumber;
	number$1.strictParseInt = strictParseInt;
	function prettyNumber(value) {
	    if (value === 0)
	        return '0';
	    if (value > 10)
	        return Math.round(value).toString();
	    if (value > 1)
	        return (Math.round(value * 10) / 10).toString();
	    return (Math.round(value * 100) / 100).toString();
	}
	function strictParseInt(value) {
	    const parsedValue = parseInt(value, 10);
	    if (isNaN(parsedValue) || parsedValue.toString() !== value) {
	        throw new Error('Not an integer.');
	    }
	    else {
	        return parsedValue;
	    }
	}
	
	return number$1;
}

var promise$1 = {};

var hasRequiredPromise$1;

function requirePromise$1 () {
	if (hasRequiredPromise$1) return promise$1;
	hasRequiredPromise$1 = 1;
	Object.defineProperty(promise$1, "__esModule", { value: true });
	promise$1.sequence = sequence;
	promise$1.sequenceSafe = sequenceSafe;
	promise$1.successes = successes;
	// functions sorted alphabetically
	// similar to `Promise.all` but sequential instead of parallel, "easy" rate limiting ^^
	function sequence(arr, perform) {
	    return arr.reduce((acc, t) => {
	        return acc.then(res => perform(t).then(u => res.concat([u])));
	    }, Promise.resolve([]));
	}
	// similar to `sequence` but do not fail and accumulate errors
	function sequenceSafe(arr, perform) {
	    return arr.reduce((acc, t) => {
	        return acc.then(([errs, res]) => perform(t)
	            .then(u => [errs, res.concat([u])])
	            .catch(err => [errs.concat([[t, err]]), res]));
	    }, Promise.resolve([[], []]));
	}
	// similar to `Promise.all` but collect only successful results and do not fail, run in parallel
	function successes(arr) {
	    return arr.reduce((acc, promise) => {
	        return acc.then(res => promise.then(p => res.concat([p])).catch(_ => res));
	    }, Promise.resolve([]));
	}
	
	return promise$1;
}

var result = {};

var hasRequiredResult;

function requireResult () {
	if (hasRequiredResult) return result;
	hasRequiredResult = 1;
	Object.defineProperty(result, "__esModule", { value: true });
	result.Result = void 0;
	class Result {
	    static success(result) {
	        return new ResultSuccess(result);
	    }
	    static failure(error) {
	        return new ResultFailure(error);
	    }
	}
	result.Result = Result;
	class ResultSuccess extends Result {
	    constructor(value) {
	        super();
	        this.value = value;
	        Object.freeze(this);
	    }
	    getOrThrow() {
	        return this.value;
	    }
	    getOrNull() {
	        return this.value;
	    }
	    getOrUndefined() {
	        return this.value;
	    }
	    errOrNull() {
	        return null;
	    }
	    swap() {
	        return new ResultFailure(this.value);
	    }
	    map(f) {
	        return new ResultSuccess(f(this.value));
	    }
	    mapError(f) {
	        return new ResultSuccess(this.value);
	    }
	    flatMap(f) {
	        return f(this.value);
	    }
	    fold(onSuccess, onFailure) {
	        return onSuccess(this.value);
	    }
	    toPromise() {
	        return Promise.resolve(this.value);
	    }
	    toJson() {
	        return { success: this.value };
	    }
	}
	class ResultFailure extends Result {
	    constructor(error) {
	        super();
	        this.error = error;
	        Object.freeze(this);
	    }
	    getOrThrow() {
	        throw this.error;
	    }
	    getOrNull() {
	        return null;
	    }
	    getOrUndefined() {
	        return undefined;
	    }
	    errOrNull() {
	        return this.error;
	    }
	    swap() {
	        return new ResultSuccess(this.error);
	    }
	    map(f) {
	        return new ResultFailure(this.error);
	    }
	    mapError(f) {
	        return new ResultFailure(f(this.error));
	    }
	    flatMap(f) {
	        return new ResultFailure(this.error);
	    }
	    fold(onSuccess, onFailure) {
	        return onFailure(this.error);
	    }
	    toPromise() {
	        return Promise.reject(this.error);
	    }
	    toJson() {
	        return { failure: this.error };
	    }
	}
	
	return result;
}

var string$1 = {};

var hasRequiredString$1;

function requireString$1 () {
	if (hasRequiredString$1) return string$1;
	hasRequiredString$1 = 1;
	(function (exports) {
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.parseUuid = exports.stringIsUuid = exports.stringIsISODate = exports.stringIsFloat = exports.stringIsInt = exports.compatibleCases = exports.isKebabLower = exports.isKebabUpper = exports.isSnakeLower = exports.isSnakeUpper = exports.isCamelLower = exports.isCamelUpper = void 0;
		exports.dateFromIsoFilename = dateFromIsoFilename;
		exports.dateToIsoFilename = dateToIsoFilename;
		exports.indent = indent;
		exports.joinLast = joinLast;
		exports.joinLimit = joinLimit;
		exports.maxLen = maxLen;
		exports.pathJoin = pathJoin;
		exports.pathParent = pathParent;
		exports.plural = plural;
		exports.pluralize = pluralize;
		exports.pluralizeL = pluralizeL;
		exports.pluralizeR = pluralizeR;
		exports.removeSurroundingParentheses = removeSurroundingParentheses;
		exports.singular = singular;
		exports.cleanText = cleanText;
		exports.slugify = slugify;
		exports.slugifyGitHub = slugifyGitHub;
		exports.splitWords = splitWords;
		exports.stripIndent = stripIndent;
		const array_1 = requireArray$1();
		const validation_1 = requireValidation();
		// functions sorted alphabetically
		function dateFromIsoFilename(d) {
		    const [, year, month, day, hour, min, sec, milli] = d.match(/(\d\d\d\d)-(\d\d)-(\d\d)T(\d\d)-(\d\d)-(\d\d)-(\d\d\d)Z/) || [];
		    if (year) {
		        return new Date(`${year}-${month}-${day}T${hour}:${min}:${sec}.${milli}Z`);
		    }
		    else {
		        throw new Error(`Invalid filename iso date: ${d}`);
		    }
		}
		function dateToIsoFilename(d) {
		    return d.toISOString().replaceAll(':', '-').replaceAll('.', '-');
		}
		function indent(value, size = 2, char = ' ') {
		    const prefix = char.repeat(size);
		    return value.split('\n').map(l => prefix + l).join('\n');
		}
		const isCamelUpper = (value) => !!value.match(/^([A-Z][a-z0-9]*)+$/);
		exports.isCamelUpper = isCamelUpper;
		const isCamelLower = (value) => !!value.match(/^([a-z][a-z0-9]*)([A-Z][a-z0-9]*)*$/);
		exports.isCamelLower = isCamelLower;
		const isSnakeUpper = (value) => !!value.match(/^([A-Z][A-Z0-9]*)(_[A-Z0-9]*)*$/);
		exports.isSnakeUpper = isSnakeUpper;
		const isSnakeLower = (value) => !!value.match(/^([a-z][a-z0-9]*)(_[a-z0-9]*)*$/);
		exports.isSnakeLower = isSnakeLower;
		const isKebabUpper = (value) => !!value.match(/^([A-Z][A-Z0-9]*)(-[A-Z0-9]*)*$/);
		exports.isKebabUpper = isKebabUpper;
		const isKebabLower = (value) => !!value.match(/^([a-z][a-z0-9]*)(-[a-z0-9]*)*$/);
		exports.isKebabLower = isKebabLower;
		const compatibleCases = (value) => [
		    (0, exports.isCamelUpper)(value) ? 'camel-upper' : undefined,
		    (0, exports.isCamelLower)(value) ? 'camel-lower' : undefined,
		    (0, exports.isSnakeUpper)(value) ? 'snake-upper' : undefined,
		    (0, exports.isSnakeLower)(value) ? 'snake-lower' : undefined,
		    (0, exports.isKebabUpper)(value) ? 'kebab-upper' : undefined,
		    (0, exports.isKebabLower)(value) ? 'kebab-lower' : undefined,
		].filter(validation_1.isNotUndefined);
		exports.compatibleCases = compatibleCases;
		function joinLast(values, sep = ', ', last = ' and ') {
		    if (values.length === 0) {
		        return '';
		    }
		    else if (values.length === 1) {
		        return values[0];
		    }
		    else {
		        return values.slice(0, -1).join(sep) + last + values[values.length - 1];
		    }
		}
		function joinLimit(values, sep = ', ', limit = 5) {
		    if (values.length > limit) {
		        return values.slice(0, limit).join(sep) + ' ...';
		    }
		    else {
		        return values.join(sep);
		    }
		}
		function maxLen(value, max) {
		    if (value.length > max) {
		        return value.slice(0, max - 3) + '...';
		    }
		    else {
		        return value;
		    }
		}
		function pathJoin(...paths) {
		    const res = paths
		        .flatMap(path => path.split('/'))
		        .filter(part => part !== '' && part !== '.') // remove useless parts
		        .filter((part, i, parts) => (part !== '..' && parts[i + 1] !== '..') || (part === '..' && (parts[i - 1] === '..' || parts[i - 1] === undefined))) // squash parents
		        .filter((part, i, parts) => (part !== '..' && parts[i + 1] !== '..') || (part === '..' && (parts[i - 1] === '..' || parts[i - 1] === undefined))) // squash parents again (if 2 levels)
		        .join('/');
		    return res.startsWith('.') || res.startsWith('~') ? res : './' + res; // add explicit relative link
		}
		function pathParent(path) {
		    return pathJoin(...path.split('/').filter(p => !!p).slice(0, -1));
		}
		function plural(word) {
		    if (word.endsWith('y') && !(word.endsWith('ay') || word.endsWith('ey') || word.endsWith('oy') || word.endsWith('uy'))) {
		        return word.slice(0, -1) + 'ies';
		    }
		    else if (word.endsWith('s') || word.endsWith('x') || word.endsWith('z') || word.endsWith('sh') || word.endsWith('ch')) {
		        return word + 'es';
		    }
		    else {
		        return word + 's';
		    }
		}
		function pluralize(count, word) {
		    if (count === 1) {
		        return `1 ${word}`;
		    }
		    else {
		        return `${count} ${plural(word)}`;
		    }
		}
		function pluralizeL(items, word) {
		    return pluralize(items.length, word);
		}
		function pluralizeR(record, word) {
		    return pluralize(Object.keys(record).length, word);
		}
		function removeSurroundingParentheses(value) {
		    if (value.startsWith('(') && value.endsWith(')')) {
		        return removeSurroundingParentheses(value.slice(1, -1));
		    }
		    else {
		        return value;
		    }
		}
		function singular(word) {
		    if (word.endsWith('ies')) {
		        return word.slice(0, -3) + 'y';
		    }
		    else if (word.endsWith('ses') || word.endsWith('xes') || word.endsWith('zes') || word.endsWith('hes')) {
		        return word.slice(0, -2);
		    }
		    else if (word.endsWith('s')) {
		        return word.slice(0, -1);
		    }
		    else {
		        return word;
		    }
		}
		function cleanText(text) {
		    return text
		        .trim()
		        .toLowerCase()
		        .normalize('NFKD') // split accented characters into their base characters and diacritical marks
		        .replace(/[\u0300-\u036f]/g, ''); // remove all the accents
		}
		function slugify(text) {
		    return cleanText(text)
		        .replace(/[^a-z0-9]/g, '-') // replace other chars with '-'
		        .replace(/-+/g, '-') // only keep a single '-'
		        .replace(/^-+/, '') // remove leading '-'
		        .replace(/-+$/, ''); // remove training '-'
		}
		function slugifyGitHub(text) {
		    return cleanText(text)
		        .replace(/[^a-z0-9 _-]/g, '') // remove non-alphanumeric characters
		        .replace(/\s/g, '-'); // replace spaces with hyphens
		}
		function splitWords(text) {
		    const { word, words } = Array.from(text).reduce(({ word, words }, cur) => {
		        const code = cur.charCodeAt(0);
		        if (isLowerChar(code) || isNumber(code)) {
		            return { word: word + cur, words: words };
		        }
		        else if (isUpperChar(code)) {
		            const len = word.length;
		            if (len === 0 || isUpperChar(word.charCodeAt(len - 1))) {
		                return { word: word + cur, words: words };
		            }
		            else {
		                return { word: cur, words: word ? words.concat([word]) : words };
		            }
		        }
		        else {
		            return { word: '', words: word ? words.concat([word]) : words };
		        }
		    }, { word: '', words: [] });
		    return words.concat([word]).filter(w => !!w).map(w => w.toLowerCase());
		}
		// avoid regex for better perf
		const isLowerChar = (code) => 97 <= code && code <= 122;
		const isUpperChar = (code) => 65 <= code && code <= 90;
		const isNumber = (code) => 48 <= code && code <= 57;
		const stringIsInt = (value) => value.match(/^\d+$/g) !== null;
		exports.stringIsInt = stringIsInt;
		const stringIsFloat = (value) => value.match(/^\d+(\.\d+)?$/g) !== null;
		exports.stringIsFloat = stringIsFloat;
		const stringIsISODate = (value) => value.match(/^\d\d\d\d-[01]\d-[0123]\dT[012]\d:[012345]\d:[012345]\d\.\d\d\dZ$/g) !== null;
		exports.stringIsISODate = stringIsISODate;
		const stringIsUuid = (value) => value.match(/^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$/gi) !== null;
		exports.stringIsUuid = stringIsUuid;
		const parseUuid = (value) => {
		    const [, version, variant] = /^[0-9a-f]{8}-[0-9a-f]{4}-([0-9a-f])[0-9a-f]{3}-([0-9a-f])[0-9a-f]{3}-[0-9a-f]{12}$/gi.exec(value) || [];
		    return { version, variant };
		};
		exports.parseUuid = parseUuid;
		function stripIndent(value) {
		    const lines = value.split('\n');
		    const lengths = lines.map(l => l.trim().length);
		    const firstIndex = lengths.findIndex(l => l > 0);
		    const lastIndex = (0, array_1.findLastIndex)(lengths, l => l > 0);
		    const content = lines.slice(firstIndex, lastIndex + 1);
		    const minIndent = content.reduce((acc, line) => {
		        const len = line.trim().length > 0 && line.match(/^(\s*).*$/)?.[1]?.length || 0;
		        return acc === undefined || (0 < len && len < acc) ? len : acc;
		    }, undefined);
		    return content.map(l => l.slice(minIndent)).join('\n');
		}
		
	} (string$1));
	return string$1;
}

var hasRequiredOut$1;

function requireOut$1 () {
	if (hasRequiredOut$1) return out;
	hasRequiredOut$1 = 1;
	(function (exports) {
		var __createBinding = (out && out.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __exportStar = (out && out.__exportStar) || function(m, exports) {
		    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		__exportStar(requireAny$1(), exports);
		__exportStar(requireArray$1(), exports);
		__exportStar(requireEmail(), exports);
		__exportStar(requireError$1(), exports);
		__exportStar(requireJson(), exports);
		__exportStar(requireLogger(), exports);
		__exportStar(requireNumber$1(), exports);
		__exportStar(requireObject$1(), exports);
		__exportStar(requirePromise$1(), exports);
		__exportStar(requireResult(), exports);
		__exportStar(requireString$1(), exports);
		__exportStar(requireValidation(), exports);
		
	} (out));
	return out;
}

var zod = {};

var hasRequiredZod;

function requireZod () {
	if (hasRequiredZod) return zod;
	hasRequiredZod = 1;
	(function (exports) {
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.zodStringify = exports.zodParseAsync = exports.zodParse = void 0;
		const zod_1 = /*@__PURE__*/ requireLib();
		const utils_1 = requireOut$1();
		const zodParse = (typ, label) => (value) => {
		    const res = typ.safeParse(value);
		    return res.success ? utils_1.Result.success(res.data) : utils_1.Result.failure(zodErrorToString(res.error, typ, label, value));
		};
		exports.zodParse = zodParse;
		const zodParseAsync = (typ, label) => (value) => (0, exports.zodParse)(typ, label)(value).mapError(e => new Error(e)).toPromise();
		exports.zodParseAsync = zodParseAsync;
		const zodStringify = (typ, label) => (value) => JSON.stringify((0, exports.zodParse)(typ, label)(value).getOrThrow());
		exports.zodStringify = zodStringify;
		const pathToString = (path) => path.length === 0 ? '_root_' : `.${path.join('.')}`;
		const normalizePath = (path) => pathToString(path.map(p => typeof p === 'number' ? '?' : p));
		function zodErrorToString(e, typ, label, value) {
		    const name = label || typ.description || 'ZodType';
		    const len = e.issues.length;
		    if (len === 0) {
		        return `Invalid ${name}, but no issue found...`;
		    }
		    else if (len === 1) {
		        return `Invalid ${name}, ${zodIssueToString(e.issues[0], value)}`;
		    }
		    else if (len <= 10) {
		        return `Invalid ${name}, ${len} issues:${e.issues.map(i => `\n- ${zodIssueToString(i, value)}`).join('')}`;
		    }
		    else {
		        const issuesGroups = (0, utils_1.groupBy)(e.issues, i => i.message + ':' + normalizePath(i.path));
		        const formattedGroups = Object.entries(issuesGroups).map(([_, [issue, ...others]]) => {
		            if (others.length === 0) {
		                return `\n- ${zodIssueToString(issue, value)}`;
		            }
		            else {
		                return `\n- at ${normalizePath(issue.path)}: ${zodIssueMessageToString(issue, value)} (${pathToString(issue.path)} and ${others.length} more)`;
		            }
		        });
		        return `Invalid ${name}, ${len} issues found in ${(0, utils_1.pluralizeL)(formattedGroups, 'group')}:${formattedGroups.join('')}`;
		    }
		}
		function zodIssueToString(issue, value) {
		    return `at ${pathToString(issue.path)}: ${zodIssueMessageToString(issue, value)}`;
		}
		function zodIssueMessageToString(issue, value) {
		    if (issue.code === zod_1.z.ZodIssueCode.unrecognized_keys) {
		        return `invalid additional key${issue.keys.length > 1 ? 's:' : ''} ${issue.keys.map(k => `'${k}' (${JSON.stringify((0, utils_1.getValueDeep)(value, issue.path.concat(k)))})`).join(', ')}`;
		    }
		    else if (issue.code === zod_1.z.ZodIssueCode.invalid_type) {
		        return `expect '${issue.expected}' but got '${issue.received}' (${JSON.stringify((0, utils_1.getValueDeep)(value, issue.path))})`;
		    }
		    else if (issue.code === zod_1.z.ZodIssueCode.invalid_literal) {
		        return `expect ${JSON.stringify(issue.expected)} but got ${JSON.stringify((0, utils_1.getValueDeep)(value, issue.path))}`;
		    }
		    else if (issue.code === zod_1.z.ZodIssueCode.invalid_enum_value) {
		        return `expect \`${issue.options.map(o => JSON.stringify(o)).join(' | ')}\` but got ${JSON.stringify((0, utils_1.getValueDeep)(value, issue.path))}`;
		    }
		    else if (issue.code === zod_1.z.ZodIssueCode.invalid_union_discriminator) {
		        return `expect \`${issue.options.map(o => JSON.stringify(o)).join(' | ')}\` but got ${JSON.stringify((0, utils_1.getValueDeep)(value, issue.path))}`;
		    }
		    else if (issue.code === zod_1.z.ZodIssueCode.invalid_union) {
		        return `invalid union for ${JSON.stringify((0, utils_1.limitDepth)((0, utils_1.getValueDeep)(value, issue.path), 3))}`;
		    }
		    else {
		        return issue.message;
		    }
		}
		
	} (zod));
	return zod;
}

var databaseUtils = {};

var database = {};

var cjs = {};

var Options = {};

var hasRequiredOptions;

function requireOptions () {
	if (hasRequiredOptions) return Options;
	hasRequiredOptions = 1;
	(function (exports) {
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.getDefaultOptions = exports.defaultOptions = exports.ignoreOverride = void 0;
		exports.ignoreOverride = Symbol("Let zodToJsonSchema decide on which parser to use");
		exports.defaultOptions = {
		    name: undefined,
		    $refStrategy: "root",
		    basePath: ["#"],
		    effectStrategy: "input",
		    pipeStrategy: "all",
		    dateStrategy: "format:date-time",
		    mapStrategy: "entries",
		    removeAdditionalStrategy: "passthrough",
		    definitionPath: "definitions",
		    target: "jsonSchema7",
		    strictUnions: false,
		    definitions: {},
		    errorMessages: false,
		    markdownDescription: false,
		    patternStrategy: "escape",
		    applyRegexFlags: false,
		    emailStrategy: "format:email",
		    base64Strategy: "contentEncoding:base64",
		    nameStrategy: "ref"
		};
		const getDefaultOptions = (options) => (typeof options === "string"
		    ? {
		        ...exports.defaultOptions,
		        name: options,
		    }
		    : {
		        ...exports.defaultOptions,
		        ...options,
		    });
		exports.getDefaultOptions = getDefaultOptions; 
	} (Options));
	return Options;
}

var Refs = {};

var hasRequiredRefs;

function requireRefs () {
	if (hasRequiredRefs) return Refs;
	hasRequiredRefs = 1;
	Object.defineProperty(Refs, "__esModule", { value: true });
	Refs.getRefs = void 0;
	const Options_js_1 = requireOptions();
	const getRefs = (options) => {
	    const _options = (0, Options_js_1.getDefaultOptions)(options);
	    const currentPath = _options.name !== undefined
	        ? [..._options.basePath, _options.definitionPath, _options.name]
	        : _options.basePath;
	    return {
	        ..._options,
	        currentPath: currentPath,
	        propertyPath: undefined,
	        seen: new Map(Object.entries(_options.definitions).map(([name, def]) => [
	            def._def,
	            {
	                def: def._def,
	                path: [..._options.basePath, _options.definitionPath, name],
	                // Resolution of references will be forced even though seen, so it's ok that the schema is undefined here for now.
	                jsonSchema: undefined,
	            },
	        ])),
	    };
	};
	Refs.getRefs = getRefs;
	return Refs;
}

var errorMessages = {};

var hasRequiredErrorMessages;

function requireErrorMessages () {
	if (hasRequiredErrorMessages) return errorMessages;
	hasRequiredErrorMessages = 1;
	Object.defineProperty(errorMessages, "__esModule", { value: true });
	errorMessages.setResponseValueAndErrors = errorMessages.addErrorMessage = void 0;
	function addErrorMessage(res, key, errorMessage, refs) {
	    if (!refs?.errorMessages)
	        return;
	    if (errorMessage) {
	        res.errorMessage = {
	            ...res.errorMessage,
	            [key]: errorMessage,
	        };
	    }
	}
	errorMessages.addErrorMessage = addErrorMessage;
	function setResponseValueAndErrors(res, key, value, errorMessage, refs) {
	    res[key] = value;
	    addErrorMessage(res, key, errorMessage, refs);
	}
	errorMessages.setResponseValueAndErrors = setResponseValueAndErrors;
	return errorMessages;
}

var parseDef = {};

var any = {};

var hasRequiredAny;

function requireAny () {
	if (hasRequiredAny) return any;
	hasRequiredAny = 1;
	Object.defineProperty(any, "__esModule", { value: true });
	any.parseAnyDef = void 0;
	function parseAnyDef() {
	    return {};
	}
	any.parseAnyDef = parseAnyDef;
	return any;
}

var array = {};

var hasRequiredArray;

function requireArray () {
	if (hasRequiredArray) return array;
	hasRequiredArray = 1;
	Object.defineProperty(array, "__esModule", { value: true });
	array.parseArrayDef = void 0;
	const zod_1 = /*@__PURE__*/ requireLib();
	const errorMessages_js_1 = requireErrorMessages();
	const parseDef_js_1 = requireParseDef();
	function parseArrayDef(def, refs) {
	    const res = {
	        type: "array",
	    };
	    if (def.type?._def?.typeName !== zod_1.ZodFirstPartyTypeKind.ZodAny) {
	        res.items = (0, parseDef_js_1.parseDef)(def.type._def, {
	            ...refs,
	            currentPath: [...refs.currentPath, "items"],
	        });
	    }
	    if (def.minLength) {
	        (0, errorMessages_js_1.setResponseValueAndErrors)(res, "minItems", def.minLength.value, def.minLength.message, refs);
	    }
	    if (def.maxLength) {
	        (0, errorMessages_js_1.setResponseValueAndErrors)(res, "maxItems", def.maxLength.value, def.maxLength.message, refs);
	    }
	    if (def.exactLength) {
	        (0, errorMessages_js_1.setResponseValueAndErrors)(res, "minItems", def.exactLength.value, def.exactLength.message, refs);
	        (0, errorMessages_js_1.setResponseValueAndErrors)(res, "maxItems", def.exactLength.value, def.exactLength.message, refs);
	    }
	    return res;
	}
	array.parseArrayDef = parseArrayDef;
	return array;
}

var bigint = {};

var hasRequiredBigint;

function requireBigint () {
	if (hasRequiredBigint) return bigint;
	hasRequiredBigint = 1;
	Object.defineProperty(bigint, "__esModule", { value: true });
	bigint.parseBigintDef = void 0;
	const errorMessages_js_1 = requireErrorMessages();
	function parseBigintDef(def, refs) {
	    const res = {
	        type: "integer",
	        format: "int64",
	    };
	    if (!def.checks)
	        return res;
	    for (const check of def.checks) {
	        switch (check.kind) {
	            case "min":
	                if (refs.target === "jsonSchema7") {
	                    if (check.inclusive) {
	                        (0, errorMessages_js_1.setResponseValueAndErrors)(res, "minimum", check.value, check.message, refs);
	                    }
	                    else {
	                        (0, errorMessages_js_1.setResponseValueAndErrors)(res, "exclusiveMinimum", check.value, check.message, refs);
	                    }
	                }
	                else {
	                    if (!check.inclusive) {
	                        res.exclusiveMinimum = true;
	                    }
	                    (0, errorMessages_js_1.setResponseValueAndErrors)(res, "minimum", check.value, check.message, refs);
	                }
	                break;
	            case "max":
	                if (refs.target === "jsonSchema7") {
	                    if (check.inclusive) {
	                        (0, errorMessages_js_1.setResponseValueAndErrors)(res, "maximum", check.value, check.message, refs);
	                    }
	                    else {
	                        (0, errorMessages_js_1.setResponseValueAndErrors)(res, "exclusiveMaximum", check.value, check.message, refs);
	                    }
	                }
	                else {
	                    if (!check.inclusive) {
	                        res.exclusiveMaximum = true;
	                    }
	                    (0, errorMessages_js_1.setResponseValueAndErrors)(res, "maximum", check.value, check.message, refs);
	                }
	                break;
	            case "multipleOf":
	                (0, errorMessages_js_1.setResponseValueAndErrors)(res, "multipleOf", check.value, check.message, refs);
	                break;
	        }
	    }
	    return res;
	}
	bigint.parseBigintDef = parseBigintDef;
	return bigint;
}

var boolean = {};

var hasRequiredBoolean;

function requireBoolean () {
	if (hasRequiredBoolean) return boolean;
	hasRequiredBoolean = 1;
	Object.defineProperty(boolean, "__esModule", { value: true });
	boolean.parseBooleanDef = void 0;
	function parseBooleanDef() {
	    return {
	        type: "boolean",
	    };
	}
	boolean.parseBooleanDef = parseBooleanDef;
	return boolean;
}

var branded = {};

var hasRequiredBranded;

function requireBranded () {
	if (hasRequiredBranded) return branded;
	hasRequiredBranded = 1;
	Object.defineProperty(branded, "__esModule", { value: true });
	branded.parseBrandedDef = void 0;
	const parseDef_js_1 = requireParseDef();
	function parseBrandedDef(_def, refs) {
	    return (0, parseDef_js_1.parseDef)(_def.type._def, refs);
	}
	branded.parseBrandedDef = parseBrandedDef;
	return branded;
}

var _catch = {};

var hasRequired_catch;

function require_catch () {
	if (hasRequired_catch) return _catch;
	hasRequired_catch = 1;
	Object.defineProperty(_catch, "__esModule", { value: true });
	_catch.parseCatchDef = void 0;
	const parseDef_js_1 = requireParseDef();
	const parseCatchDef = (def, refs) => {
	    return (0, parseDef_js_1.parseDef)(def.innerType._def, refs);
	};
	_catch.parseCatchDef = parseCatchDef;
	return _catch;
}

var date$1 = {};

var hasRequiredDate$1;

function requireDate$1 () {
	if (hasRequiredDate$1) return date$1;
	hasRequiredDate$1 = 1;
	Object.defineProperty(date$1, "__esModule", { value: true });
	date$1.parseDateDef = void 0;
	const errorMessages_js_1 = requireErrorMessages();
	function parseDateDef(def, refs, overrideDateStrategy) {
	    const strategy = overrideDateStrategy ?? refs.dateStrategy;
	    if (Array.isArray(strategy)) {
	        return {
	            anyOf: strategy.map((item, i) => parseDateDef(def, refs, item)),
	        };
	    }
	    switch (strategy) {
	        case "string":
	        case "format:date-time":
	            return {
	                type: "string",
	                format: "date-time",
	            };
	        case "format:date":
	            return {
	                type: "string",
	                format: "date",
	            };
	        case "integer":
	            return integerDateParser(def, refs);
	    }
	}
	date$1.parseDateDef = parseDateDef;
	const integerDateParser = (def, refs) => {
	    const res = {
	        type: "integer",
	        format: "unix-time",
	    };
	    if (refs.target === "openApi3") {
	        return res;
	    }
	    for (const check of def.checks) {
	        switch (check.kind) {
	            case "min":
	                (0, errorMessages_js_1.setResponseValueAndErrors)(res, "minimum", check.value, // This is in milliseconds
	                check.message, refs);
	                break;
	            case "max":
	                (0, errorMessages_js_1.setResponseValueAndErrors)(res, "maximum", check.value, // This is in milliseconds
	                check.message, refs);
	                break;
	        }
	    }
	    return res;
	};
	return date$1;
}

var _default = {};

var hasRequired_default;

function require_default () {
	if (hasRequired_default) return _default;
	hasRequired_default = 1;
	Object.defineProperty(_default, "__esModule", { value: true });
	_default.parseDefaultDef = void 0;
	const parseDef_js_1 = requireParseDef();
	function parseDefaultDef(_def, refs) {
	    return {
	        ...(0, parseDef_js_1.parseDef)(_def.innerType._def, refs),
	        default: _def.defaultValue(),
	    };
	}
	_default.parseDefaultDef = parseDefaultDef;
	return _default;
}

var effects = {};

var hasRequiredEffects;

function requireEffects () {
	if (hasRequiredEffects) return effects;
	hasRequiredEffects = 1;
	Object.defineProperty(effects, "__esModule", { value: true });
	effects.parseEffectsDef = void 0;
	const parseDef_js_1 = requireParseDef();
	function parseEffectsDef(_def, refs) {
	    return refs.effectStrategy === "input"
	        ? (0, parseDef_js_1.parseDef)(_def.schema._def, refs)
	        : {};
	}
	effects.parseEffectsDef = parseEffectsDef;
	return effects;
}

var _enum = {};

var hasRequired_enum;

function require_enum () {
	if (hasRequired_enum) return _enum;
	hasRequired_enum = 1;
	Object.defineProperty(_enum, "__esModule", { value: true });
	_enum.parseEnumDef = void 0;
	function parseEnumDef(def) {
	    return {
	        type: "string",
	        enum: def.values,
	    };
	}
	_enum.parseEnumDef = parseEnumDef;
	return _enum;
}

var intersection = {};

var hasRequiredIntersection;

function requireIntersection () {
	if (hasRequiredIntersection) return intersection;
	hasRequiredIntersection = 1;
	Object.defineProperty(intersection, "__esModule", { value: true });
	intersection.parseIntersectionDef = void 0;
	const parseDef_js_1 = requireParseDef();
	const isJsonSchema7AllOfType = (type) => {
	    if ("type" in type && type.type === "string")
	        return false;
	    return "allOf" in type;
	};
	function parseIntersectionDef(def, refs) {
	    const allOf = [
	        (0, parseDef_js_1.parseDef)(def.left._def, {
	            ...refs,
	            currentPath: [...refs.currentPath, "allOf", "0"],
	        }),
	        (0, parseDef_js_1.parseDef)(def.right._def, {
	            ...refs,
	            currentPath: [...refs.currentPath, "allOf", "1"],
	        }),
	    ].filter((x) => !!x);
	    let unevaluatedProperties = refs.target === "jsonSchema2019-09"
	        ? { unevaluatedProperties: false }
	        : undefined;
	    const mergedAllOf = [];
	    // If either of the schemas is an allOf, merge them into a single allOf
	    allOf.forEach((schema) => {
	        if (isJsonSchema7AllOfType(schema)) {
	            mergedAllOf.push(...schema.allOf);
	            if (schema.unevaluatedProperties === undefined) {
	                // If one of the schemas has no unevaluatedProperties set,
	                // the merged schema should also have no unevaluatedProperties set
	                unevaluatedProperties = undefined;
	            }
	        }
	        else {
	            let nestedSchema = schema;
	            if ("additionalProperties" in schema &&
	                schema.additionalProperties === false) {
	                const { additionalProperties, ...rest } = schema;
	                nestedSchema = rest;
	            }
	            else {
	                // As soon as one of the schemas has additionalProperties set not to false, we allow unevaluatedProperties
	                unevaluatedProperties = undefined;
	            }
	            mergedAllOf.push(nestedSchema);
	        }
	    });
	    return mergedAllOf.length
	        ? {
	            allOf: mergedAllOf,
	            ...unevaluatedProperties,
	        }
	        : undefined;
	}
	intersection.parseIntersectionDef = parseIntersectionDef;
	return intersection;
}

var literal = {};

var hasRequiredLiteral;

function requireLiteral () {
	if (hasRequiredLiteral) return literal;
	hasRequiredLiteral = 1;
	Object.defineProperty(literal, "__esModule", { value: true });
	literal.parseLiteralDef = void 0;
	function parseLiteralDef(def, refs) {
	    const parsedType = typeof def.value;
	    if (parsedType !== "bigint" &&
	        parsedType !== "number" &&
	        parsedType !== "boolean" &&
	        parsedType !== "string") {
	        return {
	            type: Array.isArray(def.value) ? "array" : "object",
	        };
	    }
	    if (refs.target === "openApi3") {
	        return {
	            type: parsedType === "bigint" ? "integer" : parsedType,
	            enum: [def.value],
	        };
	    }
	    return {
	        type: parsedType === "bigint" ? "integer" : parsedType,
	        const: def.value,
	    };
	}
	literal.parseLiteralDef = parseLiteralDef;
	return literal;
}

var map = {};

var record = {};

var string = {};

var hasRequiredString;

function requireString () {
	if (hasRequiredString) return string;
	hasRequiredString = 1;
	(function (exports) {
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.parseStringDef = exports.zodPatterns = void 0;
		const errorMessages_js_1 = requireErrorMessages();
		let emojiRegex;
		/**
		 * Generated from the regular expressions found here as of 2024-05-22:
		 * https://github.com/colinhacks/zod/blob/master/src/types.ts.
		 *
		 * Expressions with /i flag have been changed accordingly.
		 */
		exports.zodPatterns = {
		    /**
		     * `c` was changed to `[cC]` to replicate /i flag
		     */
		    cuid: /^[cC][^\s-]{8,}$/,
		    cuid2: /^[0-9a-z]+$/,
		    ulid: /^[0-9A-HJKMNP-TV-Z]{26}$/,
		    /**
		     * `a-z` was added to replicate /i flag
		     */
		    email: /^(?!\.)(?!.*\.\.)([a-zA-Z0-9_'+\-\.]*)[a-zA-Z0-9_+-]@([a-zA-Z0-9][a-zA-Z0-9\-]*\.)+[a-zA-Z]{2,}$/,
		    /**
		     * Constructed a valid Unicode RegExp
		     *
		     * Lazily instantiate since this type of regex isn't supported
		     * in all envs (e.g. React Native).
		     *
		     * See:
		     * https://github.com/colinhacks/zod/issues/2433
		     * Fix in Zod:
		     * https://github.com/colinhacks/zod/commit/9340fd51e48576a75adc919bff65dbc4a5d4c99b
		     */
		    emoji: () => {
		        if (emojiRegex === undefined) {
		            emojiRegex = RegExp("^(\\p{Extended_Pictographic}|\\p{Emoji_Component})+$", "u");
		        }
		        return emojiRegex;
		    },
		    /**
		     * Unused
		     */
		    uuid: /^[0-9a-fA-F]{8}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{12}$/,
		    /**
		     * Unused
		     */
		    ipv4: /^(?:(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]|[0-9])\.){3}(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]|[0-9])$/,
		    /**
		     * Unused
		     */
		    ipv6: /^(([a-f0-9]{1,4}:){7}|::([a-f0-9]{1,4}:){0,6}|([a-f0-9]{1,4}:){1}:([a-f0-9]{1,4}:){0,5}|([a-f0-9]{1,4}:){2}:([a-f0-9]{1,4}:){0,4}|([a-f0-9]{1,4}:){3}:([a-f0-9]{1,4}:){0,3}|([a-f0-9]{1,4}:){4}:([a-f0-9]{1,4}:){0,2}|([a-f0-9]{1,4}:){5}:([a-f0-9]{1,4}:){0,1})([a-f0-9]{1,4}|(((25[0-5])|(2[0-4][0-9])|(1[0-9]{2})|([0-9]{1,2}))\.){3}((25[0-5])|(2[0-4][0-9])|(1[0-9]{2})|([0-9]{1,2})))$/,
		    base64: /^([0-9a-zA-Z+/]{4})*(([0-9a-zA-Z+/]{2}==)|([0-9a-zA-Z+/]{3}=))?$/,
		    nanoid: /^[a-zA-Z0-9_-]{21}$/,
		};
		function parseStringDef(def, refs) {
		    const res = {
		        type: "string",
		    };
		    function processPattern(value) {
		        return refs.patternStrategy === "escape"
		            ? escapeNonAlphaNumeric(value)
		            : value;
		    }
		    if (def.checks) {
		        for (const check of def.checks) {
		            switch (check.kind) {
		                case "min":
		                    (0, errorMessages_js_1.setResponseValueAndErrors)(res, "minLength", typeof res.minLength === "number"
		                        ? Math.max(res.minLength, check.value)
		                        : check.value, check.message, refs);
		                    break;
		                case "max":
		                    (0, errorMessages_js_1.setResponseValueAndErrors)(res, "maxLength", typeof res.maxLength === "number"
		                        ? Math.min(res.maxLength, check.value)
		                        : check.value, check.message, refs);
		                    break;
		                case "email":
		                    switch (refs.emailStrategy) {
		                        case "format:email":
		                            addFormat(res, "email", check.message, refs);
		                            break;
		                        case "format:idn-email":
		                            addFormat(res, "idn-email", check.message, refs);
		                            break;
		                        case "pattern:zod":
		                            addPattern(res, exports.zodPatterns.email, check.message, refs);
		                            break;
		                    }
		                    break;
		                case "url":
		                    addFormat(res, "uri", check.message, refs);
		                    break;
		                case "uuid":
		                    addFormat(res, "uuid", check.message, refs);
		                    break;
		                case "regex":
		                    addPattern(res, check.regex, check.message, refs);
		                    break;
		                case "cuid":
		                    addPattern(res, exports.zodPatterns.cuid, check.message, refs);
		                    break;
		                case "cuid2":
		                    addPattern(res, exports.zodPatterns.cuid2, check.message, refs);
		                    break;
		                case "startsWith":
		                    addPattern(res, RegExp(`^${processPattern(check.value)}`), check.message, refs);
		                    break;
		                case "endsWith":
		                    addPattern(res, RegExp(`${processPattern(check.value)}$`), check.message, refs);
		                    break;
		                case "datetime":
		                    addFormat(res, "date-time", check.message, refs);
		                    break;
		                case "date":
		                    addFormat(res, "date", check.message, refs);
		                    break;
		                case "time":
		                    addFormat(res, "time", check.message, refs);
		                    break;
		                case "duration":
		                    addFormat(res, "duration", check.message, refs);
		                    break;
		                case "length":
		                    (0, errorMessages_js_1.setResponseValueAndErrors)(res, "minLength", typeof res.minLength === "number"
		                        ? Math.max(res.minLength, check.value)
		                        : check.value, check.message, refs);
		                    (0, errorMessages_js_1.setResponseValueAndErrors)(res, "maxLength", typeof res.maxLength === "number"
		                        ? Math.min(res.maxLength, check.value)
		                        : check.value, check.message, refs);
		                    break;
		                case "includes": {
		                    addPattern(res, RegExp(processPattern(check.value)), check.message, refs);
		                    break;
		                }
		                case "ip": {
		                    if (check.version !== "v6") {
		                        addFormat(res, "ipv4", check.message, refs);
		                    }
		                    if (check.version !== "v4") {
		                        addFormat(res, "ipv6", check.message, refs);
		                    }
		                    break;
		                }
		                case "emoji":
		                    addPattern(res, exports.zodPatterns.emoji, check.message, refs);
		                    break;
		                case "ulid": {
		                    addPattern(res, exports.zodPatterns.ulid, check.message, refs);
		                    break;
		                }
		                case "base64": {
		                    switch (refs.base64Strategy) {
		                        case "format:binary": {
		                            addFormat(res, "binary", check.message, refs);
		                            break;
		                        }
		                        case "contentEncoding:base64": {
		                            (0, errorMessages_js_1.setResponseValueAndErrors)(res, "contentEncoding", "base64", check.message, refs);
		                            break;
		                        }
		                        case "pattern:zod": {
		                            addPattern(res, exports.zodPatterns.base64, check.message, refs);
		                            break;
		                        }
		                    }
		                    break;
		                }
		                case "nanoid": {
		                    addPattern(res, exports.zodPatterns.nanoid, check.message, refs);
		                }
		            }
		        }
		    }
		    return res;
		}
		exports.parseStringDef = parseStringDef;
		const escapeNonAlphaNumeric = (value) => Array.from(value)
		    .map((c) => (/[a-zA-Z0-9]/.test(c) ? c : `\\${c}`))
		    .join("");
		const addFormat = (schema, value, message, refs) => {
		    if (schema.format || schema.anyOf?.some((x) => x.format)) {
		        if (!schema.anyOf) {
		            schema.anyOf = [];
		        }
		        if (schema.format) {
		            schema.anyOf.push({
		                format: schema.format,
		                ...(schema.errorMessage &&
		                    refs.errorMessages && {
		                    errorMessage: { format: schema.errorMessage.format },
		                }),
		            });
		            delete schema.format;
		            if (schema.errorMessage) {
		                delete schema.errorMessage.format;
		                if (Object.keys(schema.errorMessage).length === 0) {
		                    delete schema.errorMessage;
		                }
		            }
		        }
		        schema.anyOf.push({
		            format: value,
		            ...(message &&
		                refs.errorMessages && { errorMessage: { format: message } }),
		        });
		    }
		    else {
		        (0, errorMessages_js_1.setResponseValueAndErrors)(schema, "format", value, message, refs);
		    }
		};
		const addPattern = (schema, regex, message, refs) => {
		    if (schema.pattern || schema.allOf?.some((x) => x.pattern)) {
		        if (!schema.allOf) {
		            schema.allOf = [];
		        }
		        if (schema.pattern) {
		            schema.allOf.push({
		                pattern: schema.pattern,
		                ...(schema.errorMessage &&
		                    refs.errorMessages && {
		                    errorMessage: { pattern: schema.errorMessage.pattern },
		                }),
		            });
		            delete schema.pattern;
		            if (schema.errorMessage) {
		                delete schema.errorMessage.pattern;
		                if (Object.keys(schema.errorMessage).length === 0) {
		                    delete schema.errorMessage;
		                }
		            }
		        }
		        schema.allOf.push({
		            pattern: processRegExp(regex, refs),
		            ...(message &&
		                refs.errorMessages && { errorMessage: { pattern: message } }),
		        });
		    }
		    else {
		        (0, errorMessages_js_1.setResponseValueAndErrors)(schema, "pattern", processRegExp(regex, refs), message, refs);
		    }
		};
		// Mutate z.string.regex() in a best attempt to accommodate for regex flags when applyRegexFlags is true
		const processRegExp = (regexOrFunction, refs) => {
		    const regex = typeof regexOrFunction === "function" ? regexOrFunction() : regexOrFunction;
		    if (!refs.applyRegexFlags || !regex.flags)
		        return regex.source;
		    // Currently handled flags
		    const flags = {
		        i: regex.flags.includes("i"),
		        m: regex.flags.includes("m"),
		        s: regex.flags.includes("s"), // `.` matches newlines
		    };
		    // The general principle here is to step through each character, one at a time, applying mutations as flags require. We keep track when the current character is escaped, and when it's inside a group /like [this]/ or (also) a range like /[a-z]/. The following is fairly brittle imperative code; edit at your peril!
		    const source = flags.i ? regex.source.toLowerCase() : regex.source;
		    let pattern = "";
		    let isEscaped = false;
		    let inCharGroup = false;
		    let inCharRange = false;
		    for (let i = 0; i < source.length; i++) {
		        if (isEscaped) {
		            pattern += source[i];
		            isEscaped = false;
		            continue;
		        }
		        if (flags.i) {
		            if (inCharGroup) {
		                if (source[i].match(/[a-z]/)) {
		                    if (inCharRange) {
		                        pattern += source[i];
		                        pattern += `${source[i - 2]}-${source[i]}`.toUpperCase();
		                        inCharRange = false;
		                    }
		                    else if (source[i + 1] === "-" && source[i + 2]?.match(/[a-z]/)) {
		                        pattern += source[i];
		                        inCharRange = true;
		                    }
		                    else {
		                        pattern += `${source[i]}${source[i].toUpperCase()}`;
		                    }
		                    continue;
		                }
		            }
		            else if (source[i].match(/[a-z]/)) {
		                pattern += `[${source[i]}${source[i].toUpperCase()}]`;
		                continue;
		            }
		        }
		        if (flags.m) {
		            if (source[i] === "^") {
		                pattern += `(^|(?<=[\r\n]))`;
		                continue;
		            }
		            else if (source[i] === "$") {
		                pattern += `($|(?=[\r\n]))`;
		                continue;
		            }
		        }
		        if (flags.s && source[i] === ".") {
		            pattern += inCharGroup ? `${source[i]}\r\n` : `[${source[i]}\r\n]`;
		            continue;
		        }
		        pattern += source[i];
		        if (source[i] === "\\") {
		            isEscaped = true;
		        }
		        else if (inCharGroup && source[i] === "]") {
		            inCharGroup = false;
		        }
		        else if (!inCharGroup && source[i] === "[") {
		            inCharGroup = true;
		        }
		    }
		    try {
		        const regexTest = new RegExp(pattern);
		    }
		    catch {
		        console.warn(`Could not convert regex pattern at ${refs.currentPath.join("/")} to a flag-independent form! Falling back to the flag-ignorant source`);
		        return regex.source;
		    }
		    return pattern;
		}; 
	} (string));
	return string;
}

var hasRequiredRecord;

function requireRecord () {
	if (hasRequiredRecord) return record;
	hasRequiredRecord = 1;
	Object.defineProperty(record, "__esModule", { value: true });
	record.parseRecordDef = void 0;
	const zod_1 = /*@__PURE__*/ requireLib();
	const parseDef_js_1 = requireParseDef();
	const string_js_1 = requireString();
	function parseRecordDef(def, refs) {
	    if (refs.target === "openApi3" &&
	        def.keyType?._def.typeName === zod_1.ZodFirstPartyTypeKind.ZodEnum) {
	        return {
	            type: "object",
	            required: def.keyType._def.values,
	            properties: def.keyType._def.values.reduce((acc, key) => ({
	                ...acc,
	                [key]: (0, parseDef_js_1.parseDef)(def.valueType._def, {
	                    ...refs,
	                    currentPath: [...refs.currentPath, "properties", key],
	                }) ?? {},
	            }), {}),
	            additionalProperties: false,
	        };
	    }
	    const schema = {
	        type: "object",
	        additionalProperties: (0, parseDef_js_1.parseDef)(def.valueType._def, {
	            ...refs,
	            currentPath: [...refs.currentPath, "additionalProperties"],
	        }) ?? {},
	    };
	    if (refs.target === "openApi3") {
	        return schema;
	    }
	    if (def.keyType?._def.typeName === zod_1.ZodFirstPartyTypeKind.ZodString &&
	        def.keyType._def.checks?.length) {
	        const keyType = Object.entries((0, string_js_1.parseStringDef)(def.keyType._def, refs)).reduce((acc, [key, value]) => (key === "type" ? acc : { ...acc, [key]: value }), {});
	        return {
	            ...schema,
	            propertyNames: keyType,
	        };
	    }
	    else if (def.keyType?._def.typeName === zod_1.ZodFirstPartyTypeKind.ZodEnum) {
	        return {
	            ...schema,
	            propertyNames: {
	                enum: def.keyType._def.values,
	            },
	        };
	    }
	    return schema;
	}
	record.parseRecordDef = parseRecordDef;
	return record;
}

var hasRequiredMap$1;

function requireMap$1 () {
	if (hasRequiredMap$1) return map;
	hasRequiredMap$1 = 1;
	Object.defineProperty(map, "__esModule", { value: true });
	map.parseMapDef = void 0;
	const parseDef_js_1 = requireParseDef();
	const record_js_1 = requireRecord();
	function parseMapDef(def, refs) {
	    if (refs.mapStrategy === "record") {
	        return (0, record_js_1.parseRecordDef)(def, refs);
	    }
	    const keys = (0, parseDef_js_1.parseDef)(def.keyType._def, {
	        ...refs,
	        currentPath: [...refs.currentPath, "items", "items", "0"],
	    }) || {};
	    const values = (0, parseDef_js_1.parseDef)(def.valueType._def, {
	        ...refs,
	        currentPath: [...refs.currentPath, "items", "items", "1"],
	    }) || {};
	    return {
	        type: "array",
	        maxItems: 125,
	        items: {
	            type: "array",
	            items: [keys, values],
	            minItems: 2,
	            maxItems: 2,
	        },
	    };
	}
	map.parseMapDef = parseMapDef;
	return map;
}

var nativeEnum = {};

var hasRequiredNativeEnum;

function requireNativeEnum () {
	if (hasRequiredNativeEnum) return nativeEnum;
	hasRequiredNativeEnum = 1;
	Object.defineProperty(nativeEnum, "__esModule", { value: true });
	nativeEnum.parseNativeEnumDef = void 0;
	function parseNativeEnumDef(def) {
	    const object = def.values;
	    const actualKeys = Object.keys(def.values).filter((key) => {
	        return typeof object[object[key]] !== "number";
	    });
	    const actualValues = actualKeys.map((key) => object[key]);
	    const parsedTypes = Array.from(new Set(actualValues.map((values) => typeof values)));
	    return {
	        type: parsedTypes.length === 1
	            ? parsedTypes[0] === "string"
	                ? "string"
	                : "number"
	            : ["string", "number"],
	        enum: actualValues,
	    };
	}
	nativeEnum.parseNativeEnumDef = parseNativeEnumDef;
	return nativeEnum;
}

var never = {};

var hasRequiredNever;

function requireNever () {
	if (hasRequiredNever) return never;
	hasRequiredNever = 1;
	Object.defineProperty(never, "__esModule", { value: true });
	never.parseNeverDef = void 0;
	function parseNeverDef() {
	    return {
	        not: {},
	    };
	}
	never.parseNeverDef = parseNeverDef;
	return never;
}

var _null = {};

var hasRequired_null;

function require_null () {
	if (hasRequired_null) return _null;
	hasRequired_null = 1;
	Object.defineProperty(_null, "__esModule", { value: true });
	_null.parseNullDef = void 0;
	function parseNullDef(refs) {
	    return refs.target === "openApi3"
	        ? {
	            enum: ["null"],
	            nullable: true,
	        }
	        : {
	            type: "null",
	        };
	}
	_null.parseNullDef = parseNullDef;
	return _null;
}

var nullable = {};

var union = {};

var hasRequiredUnion;

function requireUnion () {
	if (hasRequiredUnion) return union;
	hasRequiredUnion = 1;
	(function (exports) {
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.parseUnionDef = exports.primitiveMappings = void 0;
		const parseDef_js_1 = requireParseDef();
		exports.primitiveMappings = {
		    ZodString: "string",
		    ZodNumber: "number",
		    ZodBigInt: "integer",
		    ZodBoolean: "boolean",
		    ZodNull: "null",
		};
		function parseUnionDef(def, refs) {
		    if (refs.target === "openApi3")
		        return asAnyOf(def, refs);
		    const options = def.options instanceof Map ? Array.from(def.options.values()) : def.options;
		    // This blocks tries to look ahead a bit to produce nicer looking schemas with type array instead of anyOf.
		    if (options.every((x) => x._def.typeName in exports.primitiveMappings &&
		        (!x._def.checks || !x._def.checks.length))) {
		        // all types in union are primitive and lack checks, so might as well squash into {type: [...]}
		        const types = options.reduce((types, x) => {
		            const type = exports.primitiveMappings[x._def.typeName]; //Can be safely casted due to row 43
		            return type && !types.includes(type) ? [...types, type] : types;
		        }, []);
		        return {
		            type: types.length > 1 ? types : types[0],
		        };
		    }
		    else if (options.every((x) => x._def.typeName === "ZodLiteral" && !x.description)) {
		        // all options literals
		        const types = options.reduce((acc, x) => {
		            const type = typeof x._def.value;
		            switch (type) {
		                case "string":
		                case "number":
		                case "boolean":
		                    return [...acc, type];
		                case "bigint":
		                    return [...acc, "integer"];
		                case "object":
		                    if (x._def.value === null)
		                        return [...acc, "null"];
		                case "symbol":
		                case "undefined":
		                case "function":
		                default:
		                    return acc;
		            }
		        }, []);
		        if (types.length === options.length) {
		            // all the literals are primitive, as far as null can be considered primitive
		            const uniqueTypes = types.filter((x, i, a) => a.indexOf(x) === i);
		            return {
		                type: uniqueTypes.length > 1 ? uniqueTypes : uniqueTypes[0],
		                enum: options.reduce((acc, x) => {
		                    return acc.includes(x._def.value) ? acc : [...acc, x._def.value];
		                }, []),
		            };
		        }
		    }
		    else if (options.every((x) => x._def.typeName === "ZodEnum")) {
		        return {
		            type: "string",
		            enum: options.reduce((acc, x) => [
		                ...acc,
		                ...x._def.values.filter((x) => !acc.includes(x)),
		            ], []),
		        };
		    }
		    return asAnyOf(def, refs);
		}
		exports.parseUnionDef = parseUnionDef;
		const asAnyOf = (def, refs) => {
		    const anyOf = (def.options instanceof Map
		        ? Array.from(def.options.values())
		        : def.options)
		        .map((x, i) => (0, parseDef_js_1.parseDef)(x._def, {
		        ...refs,
		        currentPath: [...refs.currentPath, "anyOf", `${i}`],
		    }))
		        .filter((x) => !!x &&
		        (!refs.strictUnions ||
		            (typeof x === "object" && Object.keys(x).length > 0)));
		    return anyOf.length ? { anyOf } : undefined;
		}; 
	} (union));
	return union;
}

var hasRequiredNullable;

function requireNullable () {
	if (hasRequiredNullable) return nullable;
	hasRequiredNullable = 1;
	Object.defineProperty(nullable, "__esModule", { value: true });
	nullable.parseNullableDef = void 0;
	const parseDef_js_1 = requireParseDef();
	const union_js_1 = requireUnion();
	function parseNullableDef(def, refs) {
	    if (["ZodString", "ZodNumber", "ZodBigInt", "ZodBoolean", "ZodNull"].includes(def.innerType._def.typeName) &&
	        (!def.innerType._def.checks || !def.innerType._def.checks.length)) {
	        if (refs.target === "openApi3") {
	            return {
	                type: union_js_1.primitiveMappings[def.innerType._def.typeName],
	                nullable: true,
	            };
	        }
	        return {
	            type: [
	                union_js_1.primitiveMappings[def.innerType._def.typeName],
	                "null",
	            ],
	        };
	    }
	    if (refs.target === "openApi3") {
	        const base = (0, parseDef_js_1.parseDef)(def.innerType._def, {
	            ...refs,
	            currentPath: [...refs.currentPath],
	        });
	        if (base && '$ref' in base)
	            return { allOf: [base], nullable: true };
	        return base && { ...base, nullable: true };
	    }
	    const base = (0, parseDef_js_1.parseDef)(def.innerType._def, {
	        ...refs,
	        currentPath: [...refs.currentPath, "anyOf", "0"],
	    });
	    return base && { anyOf: [base, { type: "null" }] };
	}
	nullable.parseNullableDef = parseNullableDef;
	return nullable;
}

var number = {};

var hasRequiredNumber;

function requireNumber () {
	if (hasRequiredNumber) return number;
	hasRequiredNumber = 1;
	Object.defineProperty(number, "__esModule", { value: true });
	number.parseNumberDef = void 0;
	const errorMessages_js_1 = requireErrorMessages();
	function parseNumberDef(def, refs) {
	    const res = {
	        type: "number",
	    };
	    if (!def.checks)
	        return res;
	    for (const check of def.checks) {
	        switch (check.kind) {
	            case "int":
	                res.type = "integer";
	                (0, errorMessages_js_1.addErrorMessage)(res, "type", check.message, refs);
	                break;
	            case "min":
	                if (refs.target === "jsonSchema7") {
	                    if (check.inclusive) {
	                        (0, errorMessages_js_1.setResponseValueAndErrors)(res, "minimum", check.value, check.message, refs);
	                    }
	                    else {
	                        (0, errorMessages_js_1.setResponseValueAndErrors)(res, "exclusiveMinimum", check.value, check.message, refs);
	                    }
	                }
	                else {
	                    if (!check.inclusive) {
	                        res.exclusiveMinimum = true;
	                    }
	                    (0, errorMessages_js_1.setResponseValueAndErrors)(res, "minimum", check.value, check.message, refs);
	                }
	                break;
	            case "max":
	                if (refs.target === "jsonSchema7") {
	                    if (check.inclusive) {
	                        (0, errorMessages_js_1.setResponseValueAndErrors)(res, "maximum", check.value, check.message, refs);
	                    }
	                    else {
	                        (0, errorMessages_js_1.setResponseValueAndErrors)(res, "exclusiveMaximum", check.value, check.message, refs);
	                    }
	                }
	                else {
	                    if (!check.inclusive) {
	                        res.exclusiveMaximum = true;
	                    }
	                    (0, errorMessages_js_1.setResponseValueAndErrors)(res, "maximum", check.value, check.message, refs);
	                }
	                break;
	            case "multipleOf":
	                (0, errorMessages_js_1.setResponseValueAndErrors)(res, "multipleOf", check.value, check.message, refs);
	                break;
	        }
	    }
	    return res;
	}
	number.parseNumberDef = parseNumberDef;
	return number;
}

var object = {};

var hasRequiredObject;

function requireObject () {
	if (hasRequiredObject) return object;
	hasRequiredObject = 1;
	Object.defineProperty(object, "__esModule", { value: true });
	object.parseObjectDef = void 0;
	const parseDef_js_1 = requireParseDef();
	function decideAdditionalProperties(def, refs) {
	    if (refs.removeAdditionalStrategy === "strict") {
	        return def.catchall._def.typeName === "ZodNever"
	            ? def.unknownKeys !== "strict"
	            : (0, parseDef_js_1.parseDef)(def.catchall._def, {
	                ...refs,
	                currentPath: [...refs.currentPath, "additionalProperties"],
	            }) ?? true;
	    }
	    else {
	        return def.catchall._def.typeName === "ZodNever"
	            ? def.unknownKeys === "passthrough"
	            : (0, parseDef_js_1.parseDef)(def.catchall._def, {
	                ...refs,
	                currentPath: [...refs.currentPath, "additionalProperties"],
	            }) ?? true;
	    }
	}
	function parseObjectDef(def, refs) {
	    const result = {
	        type: "object",
	        ...Object.entries(def.shape()).reduce((acc, [propName, propDef]) => {
	            if (propDef === undefined || propDef._def === undefined)
	                return acc;
	            const parsedDef = (0, parseDef_js_1.parseDef)(propDef._def, {
	                ...refs,
	                currentPath: [...refs.currentPath, "properties", propName],
	                propertyPath: [...refs.currentPath, "properties", propName],
	            });
	            if (parsedDef === undefined)
	                return acc;
	            return {
	                properties: { ...acc.properties, [propName]: parsedDef },
	                required: propDef.isOptional()
	                    ? acc.required
	                    : [...acc.required, propName],
	            };
	        }, { properties: {}, required: [] }),
	        additionalProperties: decideAdditionalProperties(def, refs),
	    };
	    if (!result.required.length)
	        delete result.required;
	    return result;
	}
	object.parseObjectDef = parseObjectDef;
	return object;
}

var optional = {};

var hasRequiredOptional;

function requireOptional () {
	if (hasRequiredOptional) return optional;
	hasRequiredOptional = 1;
	Object.defineProperty(optional, "__esModule", { value: true });
	optional.parseOptionalDef = void 0;
	const parseDef_js_1 = requireParseDef();
	const parseOptionalDef = (def, refs) => {
	    if (refs.currentPath.toString() === refs.propertyPath?.toString()) {
	        return (0, parseDef_js_1.parseDef)(def.innerType._def, refs);
	    }
	    const innerSchema = (0, parseDef_js_1.parseDef)(def.innerType._def, {
	        ...refs,
	        currentPath: [...refs.currentPath, "anyOf", "1"],
	    });
	    return innerSchema
	        ? {
	            anyOf: [
	                {
	                    not: {},
	                },
	                innerSchema,
	            ],
	        }
	        : {};
	};
	optional.parseOptionalDef = parseOptionalDef;
	return optional;
}

var pipeline = {};

var hasRequiredPipeline;

function requirePipeline () {
	if (hasRequiredPipeline) return pipeline;
	hasRequiredPipeline = 1;
	Object.defineProperty(pipeline, "__esModule", { value: true });
	pipeline.parsePipelineDef = void 0;
	const parseDef_js_1 = requireParseDef();
	const parsePipelineDef = (def, refs) => {
	    if (refs.pipeStrategy === "input") {
	        return (0, parseDef_js_1.parseDef)(def.in._def, refs);
	    }
	    else if (refs.pipeStrategy === "output") {
	        return (0, parseDef_js_1.parseDef)(def.out._def, refs);
	    }
	    const a = (0, parseDef_js_1.parseDef)(def.in._def, {
	        ...refs,
	        currentPath: [...refs.currentPath, "allOf", "0"],
	    });
	    const b = (0, parseDef_js_1.parseDef)(def.out._def, {
	        ...refs,
	        currentPath: [...refs.currentPath, "allOf", a ? "1" : "0"],
	    });
	    return {
	        allOf: [a, b].filter((x) => x !== undefined),
	    };
	};
	pipeline.parsePipelineDef = parsePipelineDef;
	return pipeline;
}

var promise = {};

var hasRequiredPromise;

function requirePromise () {
	if (hasRequiredPromise) return promise;
	hasRequiredPromise = 1;
	Object.defineProperty(promise, "__esModule", { value: true });
	promise.parsePromiseDef = void 0;
	const parseDef_js_1 = requireParseDef();
	function parsePromiseDef(def, refs) {
	    return (0, parseDef_js_1.parseDef)(def.type._def, refs);
	}
	promise.parsePromiseDef = parsePromiseDef;
	return promise;
}

var set = {};

var hasRequiredSet;

function requireSet () {
	if (hasRequiredSet) return set;
	hasRequiredSet = 1;
	Object.defineProperty(set, "__esModule", { value: true });
	set.parseSetDef = void 0;
	const errorMessages_js_1 = requireErrorMessages();
	const parseDef_js_1 = requireParseDef();
	function parseSetDef(def, refs) {
	    const items = (0, parseDef_js_1.parseDef)(def.valueType._def, {
	        ...refs,
	        currentPath: [...refs.currentPath, "items"],
	    });
	    const schema = {
	        type: "array",
	        uniqueItems: true,
	        items,
	    };
	    if (def.minSize) {
	        (0, errorMessages_js_1.setResponseValueAndErrors)(schema, "minItems", def.minSize.value, def.minSize.message, refs);
	    }
	    if (def.maxSize) {
	        (0, errorMessages_js_1.setResponseValueAndErrors)(schema, "maxItems", def.maxSize.value, def.maxSize.message, refs);
	    }
	    return schema;
	}
	set.parseSetDef = parseSetDef;
	return set;
}

var tuple = {};

var hasRequiredTuple;

function requireTuple () {
	if (hasRequiredTuple) return tuple;
	hasRequiredTuple = 1;
	Object.defineProperty(tuple, "__esModule", { value: true });
	tuple.parseTupleDef = void 0;
	const parseDef_js_1 = requireParseDef();
	function parseTupleDef(def, refs) {
	    if (def.rest) {
	        return {
	            type: "array",
	            minItems: def.items.length,
	            items: def.items
	                .map((x, i) => (0, parseDef_js_1.parseDef)(x._def, {
	                ...refs,
	                currentPath: [...refs.currentPath, "items", `${i}`],
	            }))
	                .reduce((acc, x) => (x === undefined ? acc : [...acc, x]), []),
	            additionalItems: (0, parseDef_js_1.parseDef)(def.rest._def, {
	                ...refs,
	                currentPath: [...refs.currentPath, "additionalItems"],
	            }),
	        };
	    }
	    else {
	        return {
	            type: "array",
	            minItems: def.items.length,
	            maxItems: def.items.length,
	            items: def.items
	                .map((x, i) => (0, parseDef_js_1.parseDef)(x._def, {
	                ...refs,
	                currentPath: [...refs.currentPath, "items", `${i}`],
	            }))
	                .reduce((acc, x) => (x === undefined ? acc : [...acc, x]), []),
	        };
	    }
	}
	tuple.parseTupleDef = parseTupleDef;
	return tuple;
}

var _undefined = {};

var hasRequired_undefined;

function require_undefined () {
	if (hasRequired_undefined) return _undefined;
	hasRequired_undefined = 1;
	Object.defineProperty(_undefined, "__esModule", { value: true });
	_undefined.parseUndefinedDef = void 0;
	function parseUndefinedDef() {
	    return {
	        not: {},
	    };
	}
	_undefined.parseUndefinedDef = parseUndefinedDef;
	return _undefined;
}

var unknown = {};

var hasRequiredUnknown;

function requireUnknown () {
	if (hasRequiredUnknown) return unknown;
	hasRequiredUnknown = 1;
	Object.defineProperty(unknown, "__esModule", { value: true });
	unknown.parseUnknownDef = void 0;
	function parseUnknownDef() {
	    return {};
	}
	unknown.parseUnknownDef = parseUnknownDef;
	return unknown;
}

var readonly = {};

var hasRequiredReadonly;

function requireReadonly () {
	if (hasRequiredReadonly) return readonly;
	hasRequiredReadonly = 1;
	Object.defineProperty(readonly, "__esModule", { value: true });
	readonly.parseReadonlyDef = void 0;
	const parseDef_js_1 = requireParseDef();
	const parseReadonlyDef = (def, refs) => {
	    return (0, parseDef_js_1.parseDef)(def.innerType._def, refs);
	};
	readonly.parseReadonlyDef = parseReadonlyDef;
	return readonly;
}

var hasRequiredParseDef;

function requireParseDef () {
	if (hasRequiredParseDef) return parseDef;
	hasRequiredParseDef = 1;
	Object.defineProperty(parseDef, "__esModule", { value: true });
	parseDef.parseDef = void 0;
	const zod_1 = /*@__PURE__*/ requireLib();
	const any_js_1 = requireAny();
	const array_js_1 = requireArray();
	const bigint_js_1 = requireBigint();
	const boolean_js_1 = requireBoolean();
	const branded_js_1 = requireBranded();
	const catch_js_1 = require_catch();
	const date_js_1 = requireDate$1();
	const default_js_1 = require_default();
	const effects_js_1 = requireEffects();
	const enum_js_1 = require_enum();
	const intersection_js_1 = requireIntersection();
	const literal_js_1 = requireLiteral();
	const map_js_1 = requireMap$1();
	const nativeEnum_js_1 = requireNativeEnum();
	const never_js_1 = requireNever();
	const null_js_1 = require_null();
	const nullable_js_1 = requireNullable();
	const number_js_1 = requireNumber();
	const object_js_1 = requireObject();
	const optional_js_1 = requireOptional();
	const pipeline_js_1 = requirePipeline();
	const promise_js_1 = requirePromise();
	const record_js_1 = requireRecord();
	const set_js_1 = requireSet();
	const string_js_1 = requireString();
	const tuple_js_1 = requireTuple();
	const undefined_js_1 = require_undefined();
	const union_js_1 = requireUnion();
	const unknown_js_1 = requireUnknown();
	const readonly_js_1 = requireReadonly();
	const Options_js_1 = requireOptions();
	function parseDef$1(def, refs, forceResolution = false) {
	    const seenItem = refs.seen.get(def);
	    if (refs.override) {
	        const overrideResult = refs.override?.(def, refs, seenItem, forceResolution);
	        if (overrideResult !== Options_js_1.ignoreOverride) {
	            return overrideResult;
	        }
	    }
	    if (seenItem && !forceResolution) {
	        const seenSchema = get$ref(seenItem, refs);
	        if (seenSchema !== undefined) {
	            return seenSchema;
	        }
	    }
	    const newItem = { def, path: refs.currentPath, jsonSchema: undefined };
	    refs.seen.set(def, newItem);
	    const jsonSchema = selectParser(def, def.typeName, refs);
	    if (jsonSchema) {
	        addMeta(def, refs, jsonSchema);
	    }
	    newItem.jsonSchema = jsonSchema;
	    return jsonSchema;
	}
	parseDef.parseDef = parseDef$1;
	const get$ref = (item, refs) => {
	    switch (refs.$refStrategy) {
	        case "root":
	            return { $ref: item.path.join("/") };
	        case "relative":
	            return { $ref: getRelativePath(refs.currentPath, item.path) };
	        case "none":
	        case "seen": {
	            if (item.path.length < refs.currentPath.length &&
	                item.path.every((value, index) => refs.currentPath[index] === value)) {
	                console.warn(`Recursive reference detected at ${refs.currentPath.join("/")}! Defaulting to any`);
	                return {};
	            }
	            return refs.$refStrategy === "seen" ? {} : undefined;
	        }
	    }
	};
	const getRelativePath = (pathA, pathB) => {
	    let i = 0;
	    for (; i < pathA.length && i < pathB.length; i++) {
	        if (pathA[i] !== pathB[i])
	            break;
	    }
	    return [(pathA.length - i).toString(), ...pathB.slice(i)].join("/");
	};
	const selectParser = (def, typeName, refs) => {
	    switch (typeName) {
	        case zod_1.ZodFirstPartyTypeKind.ZodString:
	            return (0, string_js_1.parseStringDef)(def, refs);
	        case zod_1.ZodFirstPartyTypeKind.ZodNumber:
	            return (0, number_js_1.parseNumberDef)(def, refs);
	        case zod_1.ZodFirstPartyTypeKind.ZodObject:
	            return (0, object_js_1.parseObjectDef)(def, refs);
	        case zod_1.ZodFirstPartyTypeKind.ZodBigInt:
	            return (0, bigint_js_1.parseBigintDef)(def, refs);
	        case zod_1.ZodFirstPartyTypeKind.ZodBoolean:
	            return (0, boolean_js_1.parseBooleanDef)();
	        case zod_1.ZodFirstPartyTypeKind.ZodDate:
	            return (0, date_js_1.parseDateDef)(def, refs);
	        case zod_1.ZodFirstPartyTypeKind.ZodUndefined:
	            return (0, undefined_js_1.parseUndefinedDef)();
	        case zod_1.ZodFirstPartyTypeKind.ZodNull:
	            return (0, null_js_1.parseNullDef)(refs);
	        case zod_1.ZodFirstPartyTypeKind.ZodArray:
	            return (0, array_js_1.parseArrayDef)(def, refs);
	        case zod_1.ZodFirstPartyTypeKind.ZodUnion:
	        case zod_1.ZodFirstPartyTypeKind.ZodDiscriminatedUnion:
	            return (0, union_js_1.parseUnionDef)(def, refs);
	        case zod_1.ZodFirstPartyTypeKind.ZodIntersection:
	            return (0, intersection_js_1.parseIntersectionDef)(def, refs);
	        case zod_1.ZodFirstPartyTypeKind.ZodTuple:
	            return (0, tuple_js_1.parseTupleDef)(def, refs);
	        case zod_1.ZodFirstPartyTypeKind.ZodRecord:
	            return (0, record_js_1.parseRecordDef)(def, refs);
	        case zod_1.ZodFirstPartyTypeKind.ZodLiteral:
	            return (0, literal_js_1.parseLiteralDef)(def, refs);
	        case zod_1.ZodFirstPartyTypeKind.ZodEnum:
	            return (0, enum_js_1.parseEnumDef)(def);
	        case zod_1.ZodFirstPartyTypeKind.ZodNativeEnum:
	            return (0, nativeEnum_js_1.parseNativeEnumDef)(def);
	        case zod_1.ZodFirstPartyTypeKind.ZodNullable:
	            return (0, nullable_js_1.parseNullableDef)(def, refs);
	        case zod_1.ZodFirstPartyTypeKind.ZodOptional:
	            return (0, optional_js_1.parseOptionalDef)(def, refs);
	        case zod_1.ZodFirstPartyTypeKind.ZodMap:
	            return (0, map_js_1.parseMapDef)(def, refs);
	        case zod_1.ZodFirstPartyTypeKind.ZodSet:
	            return (0, set_js_1.parseSetDef)(def, refs);
	        case zod_1.ZodFirstPartyTypeKind.ZodLazy:
	            return parseDef$1(def.getter()._def, refs);
	        case zod_1.ZodFirstPartyTypeKind.ZodPromise:
	            return (0, promise_js_1.parsePromiseDef)(def, refs);
	        case zod_1.ZodFirstPartyTypeKind.ZodNaN:
	        case zod_1.ZodFirstPartyTypeKind.ZodNever:
	            return (0, never_js_1.parseNeverDef)();
	        case zod_1.ZodFirstPartyTypeKind.ZodEffects:
	            return (0, effects_js_1.parseEffectsDef)(def, refs);
	        case zod_1.ZodFirstPartyTypeKind.ZodAny:
	            return (0, any_js_1.parseAnyDef)();
	        case zod_1.ZodFirstPartyTypeKind.ZodUnknown:
	            return (0, unknown_js_1.parseUnknownDef)();
	        case zod_1.ZodFirstPartyTypeKind.ZodDefault:
	            return (0, default_js_1.parseDefaultDef)(def, refs);
	        case zod_1.ZodFirstPartyTypeKind.ZodBranded:
	            return (0, branded_js_1.parseBrandedDef)(def, refs);
	        case zod_1.ZodFirstPartyTypeKind.ZodReadonly:
	            return (0, readonly_js_1.parseReadonlyDef)(def, refs);
	        case zod_1.ZodFirstPartyTypeKind.ZodCatch:
	            return (0, catch_js_1.parseCatchDef)(def, refs);
	        case zod_1.ZodFirstPartyTypeKind.ZodPipeline:
	            return (0, pipeline_js_1.parsePipelineDef)(def, refs);
	        case zod_1.ZodFirstPartyTypeKind.ZodFunction:
	        case zod_1.ZodFirstPartyTypeKind.ZodVoid:
	        case zod_1.ZodFirstPartyTypeKind.ZodSymbol:
	            return undefined;
	        default:
	            /* c8 ignore next */
	            return ((_) => undefined)();
	    }
	};
	const addMeta = (def, refs, jsonSchema) => {
	    if (def.description) {
	        jsonSchema.description = def.description;
	        if (refs.markdownDescription) {
	            jsonSchema.markdownDescription = def.description;
	        }
	    }
	    return jsonSchema;
	};
	return parseDef;
}

var zodToJsonSchema = {};

var hasRequiredZodToJsonSchema;

function requireZodToJsonSchema () {
	if (hasRequiredZodToJsonSchema) return zodToJsonSchema;
	hasRequiredZodToJsonSchema = 1;
	Object.defineProperty(zodToJsonSchema, "__esModule", { value: true });
	zodToJsonSchema.zodToJsonSchema = void 0;
	const parseDef_js_1 = requireParseDef();
	const Refs_js_1 = requireRefs();
	const zodToJsonSchema$1 = (schema, options) => {
	    const refs = (0, Refs_js_1.getRefs)(options);
	    const definitions = typeof options === "object" && options.definitions
	        ? Object.entries(options.definitions).reduce((acc, [name, schema]) => ({
	            ...acc,
	            [name]: (0, parseDef_js_1.parseDef)(schema._def, {
	                ...refs,
	                currentPath: [...refs.basePath, refs.definitionPath, name],
	            }, true) ?? {},
	        }), {})
	        : undefined;
	    const name = typeof options === "string"
	        ? options
	        : options?.nameStrategy === "title"
	            ? undefined
	            : options?.name;
	    const main = (0, parseDef_js_1.parseDef)(schema._def, name === undefined
	        ? refs
	        : {
	            ...refs,
	            currentPath: [...refs.basePath, refs.definitionPath, name],
	        }, false) ?? {};
	    const title = typeof options === "object" &&
	        options.name !== undefined &&
	        options.nameStrategy === "title"
	        ? options.name
	        : undefined;
	    if (title !== undefined) {
	        main.title = title;
	    }
	    const combined = name === undefined
	        ? definitions
	            ? {
	                ...main,
	                [refs.definitionPath]: definitions,
	            }
	            : main
	        : {
	            $ref: [
	                ...(refs.$refStrategy === "relative" ? [] : refs.basePath),
	                refs.definitionPath,
	                name,
	            ].join("/"),
	            [refs.definitionPath]: {
	                ...definitions,
	                [name]: main,
	            },
	        };
	    if (refs.target === "jsonSchema7") {
	        combined.$schema = "http://json-schema.org/draft-07/schema#";
	    }
	    else if (refs.target === "jsonSchema2019-09") {
	        combined.$schema = "https://json-schema.org/draft/2019-09/schema#";
	    }
	    return combined;
	};
	zodToJsonSchema.zodToJsonSchema = zodToJsonSchema$1;
	return zodToJsonSchema;
}

var hasRequiredCjs;

function requireCjs () {
	if (hasRequiredCjs) return cjs;
	hasRequiredCjs = 1;
	(function (exports) {
		var __createBinding = (cjs && cjs.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __exportStar = (cjs && cjs.__exportStar) || function(m, exports) {
		    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		__exportStar(requireOptions(), exports);
		__exportStar(requireRefs(), exports);
		__exportStar(requireErrorMessages(), exports);
		__exportStar(requireParseDef(), exports);
		__exportStar(requireAny(), exports);
		__exportStar(requireArray(), exports);
		__exportStar(requireBigint(), exports);
		__exportStar(requireBoolean(), exports);
		__exportStar(requireBranded(), exports);
		__exportStar(require_catch(), exports);
		__exportStar(requireDate$1(), exports);
		__exportStar(require_default(), exports);
		__exportStar(requireEffects(), exports);
		__exportStar(require_enum(), exports);
		__exportStar(requireIntersection(), exports);
		__exportStar(requireLiteral(), exports);
		__exportStar(requireMap$1(), exports);
		__exportStar(requireNativeEnum(), exports);
		__exportStar(requireNever(), exports);
		__exportStar(require_null(), exports);
		__exportStar(requireNullable(), exports);
		__exportStar(requireNumber(), exports);
		__exportStar(requireObject(), exports);
		__exportStar(requireOptional(), exports);
		__exportStar(requirePipeline(), exports);
		__exportStar(requirePromise(), exports);
		__exportStar(requireReadonly(), exports);
		__exportStar(requireRecord(), exports);
		__exportStar(requireSet(), exports);
		__exportStar(requireString(), exports);
		__exportStar(requireTuple(), exports);
		__exportStar(require_undefined(), exports);
		__exportStar(requireUnion(), exports);
		__exportStar(requireUnknown(), exports);
		__exportStar(requireZodToJsonSchema(), exports);
		const zodToJsonSchema_js_1 = requireZodToJsonSchema();
		exports.default = zodToJsonSchema_js_1.zodToJsonSchema; 
	} (cjs));
	return cjs;
}

var common = {};

var hasRequiredCommon;

function requireCommon () {
	if (hasRequiredCommon) return common;
	hasRequiredCommon = 1;
	(function (exports) {
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.JsValue = exports.JsValueLiteral = exports.SqlFragment = exports.SqlStatement = exports.SqlScript = exports.Delta = exports.Size = exports.Position = exports.Px = exports.Color = exports.DateTime = exports.Timestamp = exports.Millis = exports.Percent = exports.Markdown = exports.Slug = exports.Uuid = void 0;
		const zod_1 = /*@__PURE__*/ requireLib();
		exports.Uuid = zod_1.z.string().uuid();
		exports.Slug = zod_1.z.string();
		exports.Markdown = zod_1.z.string();
		exports.Percent = zod_1.z.number();
		exports.Millis = zod_1.z.number(); // milli-seconds
		exports.Timestamp = zod_1.z.number(); // timestamp in millis (ex: 1663007946750)
		exports.DateTime = zod_1.z.string(); /*.datetime()*/ // date in iso format ("2022-09-12T11:13:02.611616Z")
		exports.Color = zod_1.z.enum(['gray', 'red', 'orange', 'amber', 'yellow', 'lime', 'green', 'emerald', 'teal', 'cyan', 'sky', 'blue', 'indigo', 'violet', 'purple', 'fuchsia', 'pink', 'rose']);
		exports.Px = zod_1.z.number(); // number of pixels
		exports.Position = zod_1.z.object({ left: exports.Px, top: exports.Px }).strict();
		exports.Size = zod_1.z.object({ width: exports.Px, height: exports.Px }).strict();
		exports.Delta = zod_1.z.object({ dx: zod_1.z.number(), dy: zod_1.z.number() }).strict();
		exports.SqlScript = zod_1.z.string(); // a whole SQL script with several queries inside
		exports.SqlStatement = zod_1.z.string(); // a single and complete SQL statement
		exports.SqlFragment = zod_1.z.string(); // a part of SQL, can be used to build a bigger query
		exports.JsValueLiteral = zod_1.z.union([zod_1.z.string(), zod_1.z.number(), zod_1.z.boolean(), zod_1.z.date(), zod_1.z.null()]);
		exports.JsValue = zod_1.z.lazy(() => zod_1.z.union([exports.JsValueLiteral, zod_1.z.array(exports.JsValue), zod_1.z.record(exports.JsValue)]));
		
	} (common));
	return common;
}

var hasRequiredDatabase;

function requireDatabase () {
	if (hasRequiredDatabase) return database;
	hasRequiredDatabase = 1;
	(function (exports) {
		var __importDefault = (database && database.__importDefault) || function (mod) {
		    return (mod && mod.__esModule) ? mod : { "default": mod };
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.RelationExtra = exports.RelationAction = exports.RelationKind = exports.RelationLink = exports.RelationCardinality = exports.Entity = exports.entityExtraProps = exports.entityExtraKeys = exports.EntityExtra = exports.EntityStats = exports.EntityKind = exports.Attribute = exports.attributeExtraProps = exports.attributeExtraKeys = exports.AttributeExtra = exports.AttributeStats = exports.AttributeTypeParsed = exports.AttributeTypeKind = exports.PrimaryKey = exports.Index = exports.Check = exports.indexExtraKeys = exports.IndexExtra = exports.IndexStats = exports.TypeRef = exports.RelationRef = exports.ConstraintRef = exports.AttributesRef = exports.AttributeRef = exports.EntityRef = exports.Namespace = exports.Extra = exports.RelationId = exports.TypeId = exports.ConstraintId = exports.AttributesId = exports.AttributeId = exports.AttributePathId = exports.EntityId = exports.NamespaceId = exports.TypeName = exports.ConstraintName = exports.AttributeValue = exports.AttributeType = exports.AttributePath = exports.AttributeName = exports.EntityName = exports.SchemaName = exports.CatalogName = exports.DatabaseName = void 0;
		exports.DatabaseSchema = exports.Database = exports.databaseExtraKeys = exports.DatabaseExtra = exports.DatabaseStats = exports.DatabaseKind = exports.Type = exports.typeExtraProps = exports.typeExtraKeys = exports.TypeExtra = exports.Relation = exports.relationExtraProps = exports.relationExtraKeys = void 0;
		const zod_1 = /*@__PURE__*/ requireLib();
		const zod_to_json_schema_1 = __importDefault(requireCjs());
		const common_1 = requireCommon();
		// read this file from bottom to the top, to have a top-down read ^^
		// TODO:
		//   - function to check Database consistency (no relation to non-existing entity, etc)
		//   - function to diff two Database
		//   - convert Database to Project and the reverse
		exports.DatabaseName = zod_1.z.string();
		exports.CatalogName = zod_1.z.string();
		exports.SchemaName = zod_1.z.string();
		exports.EntityName = zod_1.z.string();
		exports.AttributeName = zod_1.z.string();
		exports.AttributePath = exports.AttributeName.array();
		exports.AttributeType = zod_1.z.string();
		// FIXME: remove unknown from AttributeValue
		exports.AttributeValue = zod_1.z.union([zod_1.z.string(), zod_1.z.number(), zod_1.z.boolean(), zod_1.z.date(), zod_1.z.null(), zod_1.z.unknown()]);
		exports.ConstraintName = zod_1.z.string();
		exports.TypeName = zod_1.z.string();
		exports.NamespaceId = zod_1.z.string(); // serialized Namespace, ex: 'database.catalog.schema', 'public'
		exports.EntityId = zod_1.z.string(); // serialized EntityRef (entity name with namespace), ex: 'database.catalog.schema.table', 'public.users'
		exports.AttributePathId = zod_1.z.string(); // serialized AttributePath for nested attributes: ex: 'attribute.nested_attribute', 'payload.address.street'
		exports.AttributeId = zod_1.z.string(); // serialized AttributeRef (EntityId with AttributePathId), ex: 'table(id)', 'd.c.s.events(payload.address.no)'
		exports.AttributesId = zod_1.z.string(); // serialized AttributesRef (EntityId with list of AttributePathId), ex: 'table(id, email)', 'd.c.s.events(payload.address.no, payload.address.street)'
		exports.ConstraintId = zod_1.z.string(); // serialized ConstraintRef (EntityId with ConstraintName), ex: 'table(table_pk)', 'd.c.s.events(event_created_by_fk)'
		exports.TypeId = zod_1.z.string(); // serialized TypeRef (type name with namespace), ex: 'public.post_status'
		exports.RelationId = zod_1.z.string(); // serialized RelationRef link, ex: 'posts(author)->users(id)'
		exports.Extra = zod_1.z.record(zod_1.z.any());
		exports.Namespace = zod_1.z.object({
		    database: exports.DatabaseName,
		    catalog: exports.CatalogName,
		    schema: exports.SchemaName,
		}).partial().strict();
		exports.EntityRef = exports.Namespace.extend({ entity: exports.EntityName }).strict();
		exports.AttributeRef = exports.EntityRef.extend({ attribute: exports.AttributePath }).strict();
		exports.AttributesRef = exports.EntityRef.extend({ attrs: exports.AttributePath.array() }).strict();
		exports.ConstraintRef = exports.EntityRef.extend({ constraint: exports.ConstraintName }).strict();
		exports.RelationRef = zod_1.z.object({ src: exports.AttributesRef, ref: exports.AttributesRef }).strict();
		exports.TypeRef = exports.Namespace.extend({ type: exports.TypeName }).strict();
		exports.IndexStats = zod_1.z.object({
		    size: zod_1.z.number().optional(), // used bytes
		    scans: zod_1.z.number().optional(), // number of index scans
		    scansLast: common_1.DateTime.optional(), // last index scan
		}).strict();
		exports.IndexExtra = exports.Extra.and(zod_1.z.object({
		    line: zod_1.z.number().optional(),
		    statement: zod_1.z.number().optional(),
		}));
		exports.indexExtraKeys = ['line', 'statement'];
		exports.Check = zod_1.z.object({
		    name: exports.ConstraintName.optional(),
		    attrs: exports.AttributePath.array(),
		    predicate: zod_1.z.string(),
		    doc: zod_1.z.string().optional(),
		    stats: exports.IndexStats.optional(),
		    extra: exports.IndexExtra.optional(),
		}).strict();
		exports.Index = zod_1.z.object({
		    name: exports.ConstraintName.optional(),
		    attrs: exports.AttributePath.array(),
		    unique: zod_1.z.boolean().optional(), // false when not specified
		    partial: zod_1.z.string().optional(), // false when not specified
		    definition: zod_1.z.string().optional(),
		    doc: zod_1.z.string().optional(),
		    stats: exports.IndexStats.optional(),
		    extra: exports.IndexExtra.optional(),
		}).strict();
		exports.PrimaryKey = zod_1.z.object({
		    name: exports.ConstraintName.optional(),
		    attrs: exports.AttributePath.array(),
		    doc: zod_1.z.string().optional(),
		    stats: exports.IndexStats.optional(),
		    extra: exports.IndexExtra.optional(),
		}).strict();
		exports.AttributeTypeKind = zod_1.z.enum(['string', 'int', 'float', 'bool', 'date', 'time', 'instant', 'period', 'binary', 'uuid', 'json', 'xml', 'array', 'unknown']);
		exports.AttributeTypeParsed = zod_1.z.object({
		    full: zod_1.z.string(),
		    kind: exports.AttributeTypeKind,
		    size: zod_1.z.number().optional(),
		    variable: zod_1.z.boolean().optional(),
		    encoding: zod_1.z.string().optional(),
		    array: zod_1.z.boolean().optional(),
		}).strict();
		exports.AttributeStats = zod_1.z.object({
		    nulls: zod_1.z.number().optional(), // percentage of nulls, between 0 and 1
		    bytesAvg: zod_1.z.number().optional(), // average bytes for a value
		    cardinality: zod_1.z.number().optional(), // number of different values
		    commonValues: zod_1.z.object({
		        value: exports.AttributeValue,
		        freq: zod_1.z.number()
		    }).strict().array().optional(),
		    distinctValues: exports.AttributeValue.array().optional(),
		    histogram: exports.AttributeValue.array().optional(),
		    min: exports.AttributeValue.optional(),
		    max: exports.AttributeValue.optional(),
		}).strict();
		exports.AttributeExtra = exports.Extra.and(zod_1.z.object({
		    line: zod_1.z.number().optional(),
		    statement: zod_1.z.number().optional(),
		    autoIncrement: zod_1.z.null().optional(),
		    hidden: zod_1.z.null().optional(),
		    tags: zod_1.z.string().array().optional(),
		    comment: zod_1.z.string().optional(), // if there is a comment in the attribute line
		}));
		exports.attributeExtraKeys = ['line', 'statement', 'autoIncrement', 'hidden', 'tags', 'comment'];
		exports.attributeExtraProps = ['autoIncrement', 'hidden', 'tags']; // extra keys manually set in properties
		exports.Attribute = zod_1.z.object({
		    name: exports.AttributeName,
		    type: exports.AttributeType,
		    null: zod_1.z.boolean().optional(), // false when not specified
		    gen: zod_1.z.boolean().optional(), // false when not specified
		    default: exports.AttributeValue.optional(),
		    attrs: zod_1.z.lazy(() => exports.Attribute.array().optional()),
		    doc: zod_1.z.string().optional(),
		    stats: exports.AttributeStats.optional(),
		    extra: exports.AttributeExtra.optional(),
		}).strict();
		exports.EntityKind = zod_1.z.enum(['table', 'view', 'materialized view', 'foreign table']);
		exports.EntityStats = zod_1.z.object({
		    rows: zod_1.z.number().optional(), // number of rows
		    rowsDead: zod_1.z.number().optional(), // number of dead rows
		    size: zod_1.z.number().optional(), // used bytes
		    sizeIdx: zod_1.z.number().optional(), // used bytes for indexes
		    sizeToast: zod_1.z.number().optional(), // used bytes for toasts
		    sizeToastIdx: zod_1.z.number().optional(), // used bytes for toasts indexes
		    scanSeq: zod_1.z.number().optional(), // number of seq scan
		    scanSeqLast: common_1.DateTime.optional(),
		    scanIdx: zod_1.z.number().optional(), // number of index scan
		    scanIdxLast: common_1.DateTime.optional(),
		    analyzeLast: common_1.DateTime.optional(),
		    analyzeLag: zod_1.z.number().optional(),
		    vacuumLast: common_1.DateTime.optional(),
		    vacuumLag: zod_1.z.number().optional(),
		}).strict();
		exports.EntityExtra = exports.Extra.and(zod_1.z.object({
		    line: zod_1.z.number().optional(),
		    statement: zod_1.z.number().optional(),
		    alias: zod_1.z.string().optional(),
		    color: zod_1.z.string().optional(),
		    tags: zod_1.z.string().array().optional(),
		    comment: zod_1.z.string().optional(), // if there is a comment in the entity line
		}));
		exports.entityExtraKeys = ['line', 'statement', 'alias', 'color', 'tags', 'comment'];
		exports.entityExtraProps = ['view', 'color', 'tags']; // extra keys manually set in properties (view is set in props but stored in entity def, not extra ^^)
		exports.Entity = exports.Namespace.extend({
		    name: exports.EntityName,
		    kind: exports.EntityKind.optional(), // 'table' when not specified
		    def: zod_1.z.string().optional(), // the query definition for views
		    attrs: exports.Attribute.array().optional(),
		    pk: exports.PrimaryKey.optional(),
		    indexes: exports.Index.array().optional(),
		    checks: exports.Check.array().optional(),
		    doc: zod_1.z.string().optional(),
		    stats: exports.EntityStats.optional(),
		    extra: exports.EntityExtra.optional(),
		}).strict();
		exports.RelationCardinality = zod_1.z.enum(['0', '1', 'n']);
		exports.RelationLink = exports.EntityRef.extend({ attrs: exports.AttributePath.array(), cardinality: exports.RelationCardinality.optional() });
		exports.RelationKind = zod_1.z.enum(['many-to-one', 'one-to-many', 'one-to-one', 'many-to-many']);
		exports.RelationAction = zod_1.z.enum(['no action', 'set null', 'set default', 'cascade', 'restrict']);
		exports.RelationExtra = exports.Extra.and(zod_1.z.object({
		    line: zod_1.z.number().optional(),
		    statement: zod_1.z.number().optional(),
		    inline: zod_1.z.boolean().optional(), // when defined within the parent entity
		    natural: zod_1.z.enum(['src', 'ref', 'both']).optional(), // natural join: attributes are not specified
		    onUpdate: zod_1.z.union([exports.RelationAction, zod_1.z.string()]).optional(),
		    onDelete: zod_1.z.union([exports.RelationAction, zod_1.z.string()]).optional(),
		    srcAlias: zod_1.z.string().optional(),
		    refAlias: zod_1.z.string().optional(),
		    tags: zod_1.z.string().array().optional(),
		    comment: zod_1.z.string().optional(), // if there is a comment in the relation line
		}));
		exports.relationExtraKeys = ['line', 'statement', 'inline', 'natural', 'onUpdate', 'onDelete', 'srcAlias', 'refAlias', 'tags', 'comment'];
		exports.relationExtraProps = ['onUpdate', 'onDelete', 'tags']; // extra keys manually set in properties
		exports.Relation = zod_1.z.object({
		    name: exports.ConstraintName.optional(),
		    origin: zod_1.z.enum(['fk', 'infer-name', 'infer-similar', 'infer-query', 'user']).optional(), // 'fk' when not specified
		    src: exports.RelationLink,
		    ref: exports.RelationLink,
		    polymorphic: zod_1.z.object({ attribute: exports.AttributePath, value: exports.AttributeValue }).optional(),
		    doc: zod_1.z.string().optional(),
		    extra: exports.RelationExtra.optional(),
		}).strict();
		exports.TypeExtra = exports.Extra.and(zod_1.z.object({
		    line: zod_1.z.number().optional(),
		    statement: zod_1.z.number().optional(),
		    inline: zod_1.z.boolean().optional(), // when defined within the parent entity
		    tags: zod_1.z.string().array().optional(),
		    comment: zod_1.z.string().optional(), // if there is a comment in the type line
		}));
		exports.typeExtraKeys = ['line', 'statement', 'inline', 'tags', 'comment'];
		exports.typeExtraProps = ['tags']; // extra keys manually set in properties
		exports.Type = exports.Namespace.extend({
		    name: exports.TypeName,
		    alias: zod_1.z.string().optional(),
		    values: zod_1.z.string().array().optional(),
		    attrs: exports.Attribute.array().optional(),
		    definition: zod_1.z.string().optional(),
		    doc: zod_1.z.string().optional(),
		    extra: exports.TypeExtra.optional(),
		}).strict();
		exports.DatabaseKind = zod_1.z.enum(['bigquery', 'cassandra', 'couchbase', 'db2', 'elasticsearch', 'mariadb', 'mongodb', 'mysql', 'oracle', 'postgres', 'redis', 'snowflake', 'sqlite', 'sqlserver']);
		exports.DatabaseStats = zod_1.z.object({
		    name: exports.DatabaseName,
		    kind: exports.DatabaseKind,
		    version: zod_1.z.string(),
		    size: zod_1.z.number(), // used bytes
		    extractedAt: common_1.DateTime, // legacy, see extra.createdAt instead
		    extractionDuration: common_1.Millis, // legacy, see extra.creationTimeMs instead
		}).partial().strict();
		exports.DatabaseExtra = exports.Extra.and(zod_1.z.object({
		    source: zod_1.z.string().optional(), // what/who created this database, format: `${name} <${version}>` (version is optional, inspired by package.json author)
		    createdAt: common_1.DateTime.optional(), // when it was created
		    creationTimeMs: common_1.Millis.optional(), // how long it took to create it
		    comments: zod_1.z.object({ line: zod_1.z.number(), comment: zod_1.z.string() }).array().optional(), // source line comments, used to generate them back
		    namespaces: exports.Namespace.extend({ line: zod_1.z.number(), comment: zod_1.z.string().optional() }).array().optional(), // source namespace statements, used to generate them back
		}));
		exports.databaseExtraKeys = ['source', 'createdAt', 'creationTimeMs', 'comments', 'namespaces'];
		exports.Database = zod_1.z.object({
		    entities: exports.Entity.array().optional(),
		    relations: exports.Relation.array().optional(),
		    types: exports.Type.array().optional(),
		    // functions: z.record(FunctionId, Function.array()).optional(),
		    // procedures: z.record(ProcedureId, Procedure.array()).optional(), // parse procedures and list them on tables they use
		    // triggers: z.record(TriggerId, Trigger.array()).optional(), // list triggers on tables they belong
		    doc: zod_1.z.string().optional(),
		    stats: exports.DatabaseStats.optional(),
		    extra: exports.DatabaseExtra.optional(),
		}).strict().describe('Database');
		// keep it sync with backend/priv/static/aml_schema.json (see test)
		exports.DatabaseSchema = (0, zod_to_json_schema_1.default)(exports.Database, {
		    name: 'Database',
		    definitions: {
		        DatabaseName: exports.DatabaseName, CatalogName: exports.CatalogName, SchemaName: exports.SchemaName, Namespace: exports.Namespace,
		        Entity: exports.Entity, EntityRef: exports.EntityRef, EntityName: exports.EntityName, EntityKind: exports.EntityKind,
		        Attribute: exports.Attribute, AttributeRef: exports.AttributeRef, AttributeName: exports.AttributeName, AttributePath: exports.AttributePath, AttributeType: exports.AttributeType, AttributeValue: exports.AttributeValue,
		        PrimaryKey: exports.PrimaryKey, Index: exports.Index, Check: exports.Check, ConstraintName: exports.ConstraintName,
		        Relation: exports.Relation, RelationLink: exports.RelationLink, RelationCardinality: exports.RelationCardinality,
		        Type: exports.Type, TypeName: exports.TypeName,
		        Extra: exports.Extra, DatabaseExtra: exports.DatabaseExtra, EntityExtra: exports.EntityExtra, AttributeExtra: exports.AttributeExtra, IndexExtra: exports.IndexExtra, RelationExtra: exports.RelationExtra, TypeExtra: exports.TypeExtra
		    }
		});
		
	} (database));
	return database;
}

var parserResult = {};

var hasRequiredParserResult;

function requireParserResult () {
	if (hasRequiredParserResult) return parserResult;
	hasRequiredParserResult = 1;
	(function (exports) {
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.positionEndAdd = exports.positionStartAdd = exports.mergePositions = exports.tokenPosition = exports.isEditorPosition = exports.isTokenEditor = exports.isTokenOffset = exports.isTokenPosition = exports.isParserErrorLevel = exports.ParserError = exports.ParserErrorLevel = exports.TokenPosition = exports.TokenOffset = exports.TokenEditor = exports.EditorPosition = exports.ParserResult = void 0;
		const zod_1 = /*@__PURE__*/ requireLib();
		const utils_1 = requireOut$1();
		class ParserResult {
		    constructor(result, errors) {
		        this.result = result;
		        this.errors = errors;
		        if (this.errors?.length === 0)
		            this.errors = undefined;
		        Object.freeze(this);
		    }
		    static success(result) {
		        return new ParserResult(result);
		    }
		    static failure(errors) {
		        return new ParserResult(undefined, errors);
		    }
		    map(f) {
		        return new ParserResult(this.result !== undefined ? f(this.result) : undefined, this.errors);
		    }
		    mapError(f) {
		        return new ParserResult(this.result, this.errors !== undefined ? f(this.errors) : undefined);
		    }
		    flatMap(f) {
		        if (this.result !== undefined) {
		            const res = f(this.result);
		            const errors = (this.errors || []).concat(res.errors || []);
		            return new ParserResult(res.result, errors);
		        }
		        else {
		            return new ParserResult(undefined, this.errors);
		        }
		    }
		    fold(onSuccess, onFailure) {
		        return this.result !== undefined ? onSuccess(this.result) : onFailure(this.errors || []);
		    }
		}
		exports.ParserResult = ParserResult;
		exports.EditorPosition = zod_1.z.object({ line: zod_1.z.number(), column: zod_1.z.number() }).strict();
		exports.TokenEditor = zod_1.z.object({ start: exports.EditorPosition, end: exports.EditorPosition }).strict();
		exports.TokenOffset = zod_1.z.object({ start: zod_1.z.number(), end: zod_1.z.number() }).strict();
		exports.TokenPosition = zod_1.z.object({ offset: exports.TokenOffset, position: exports.TokenEditor }).strict();
		exports.ParserErrorLevel = zod_1.z.enum(['error', 'warning', 'info', 'hint']);
		exports.ParserError = exports.TokenPosition.extend({ message: zod_1.z.string(), kind: zod_1.z.string(), level: exports.ParserErrorLevel }).strict();
		const isParserErrorLevel = (value) => typeof value === 'string' && ['error', 'warning', 'info', 'hint'].includes(value);
		exports.isParserErrorLevel = isParserErrorLevel;
		const isTokenPosition = (value) => (0, utils_1.isObject)(value) && ('offset' in value && (0, exports.isTokenOffset)(value.offset)) && ('position' in value && (0, exports.isTokenEditor)(value.position));
		exports.isTokenPosition = isTokenPosition;
		const isTokenOffset = (value) => (0, utils_1.isObject)(value) && ('start' in value && typeof value.start === 'number') && ('end' in value && typeof value.end === 'number');
		exports.isTokenOffset = isTokenOffset;
		const isTokenEditor = (value) => (0, utils_1.isObject)(value) && ('start' in value && (0, exports.isEditorPosition)(value.start)) && ('end' in value && (0, exports.isEditorPosition)(value.end));
		exports.isTokenEditor = isTokenEditor;
		const isEditorPosition = (value) => (0, utils_1.isObject)(value) && ('line' in value && typeof value.line === 'number') && ('column' in value && typeof value.column === 'number');
		exports.isEditorPosition = isEditorPosition;
		const tokenPosition = (offsetStart, offsetEnd, positionStartLine, positionStartColumn, positionEndLine, positionEndColumn) => ({ offset: { start: offsetStart, end: offsetEnd }, position: { start: { line: positionStartLine, column: positionStartColumn }, end: { line: positionEndLine, column: positionEndColumn } } });
		exports.tokenPosition = tokenPosition;
		const mergePositions = (positions) => {
		    const pos = positions.filter(utils_1.isNotUndefined).filter(p => !!p.offset);
		    return ({
		        offset: { start: posStart(pos.map(p => p.offset.start)), end: posEnd(pos.map(p => p.offset.end)) },
		        position: {
		            start: { line: posStart(pos.map(p => p.position.start.line)), column: posStart(pos.map(p => p.position.start.column)) },
		            end: { line: posEnd(pos.map(p => p.position.end.line)), column: posEnd(pos.map(p => p.position.end.column)) }
		        }
		    });
		};
		exports.mergePositions = mergePositions;
		const positionStartAdd = (pos, value) => ({
		    ...pos,
		    offset: { ...pos.offset, start: pos.offset.start + value },
		    position: { ...pos.position, start: { ...pos.position.start, column: pos.position.start.column + value } }
		});
		exports.positionStartAdd = positionStartAdd;
		const positionEndAdd = (pos, value) => ({
		    ...pos,
		    offset: { ...pos.offset, end: pos.offset.end + value },
		    position: { ...pos.position, end: { ...pos.position.end, column: pos.position.end.column + value } }
		});
		exports.positionEndAdd = positionEndAdd;
		const posStart = (values) => {
		    const valid = values.filter(n => n >= 0 && !Number.isNaN(n) && Number.isFinite(n));
		    return valid.length > 0 ? Math.min(...valid) : 0;
		};
		const posEnd = (values) => {
		    const valid = values.filter(n => n >= 0 && !Number.isNaN(n) && Number.isFinite(n));
		    return valid.length > 0 ? Math.max(...valid) : 0;
		};
		
	} (parserResult));
	return parserResult;
}

var legacyDatabase = {};

var hasRequiredLegacyDatabase;

function requireLegacyDatabase () {
	if (hasRequiredLegacyDatabase) return legacyDatabase;
	hasRequiredLegacyDatabase = 1;
	(function (exports) {
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.LegacyDatabase = exports.LegacyType = exports.LegacyTypeName = exports.LegacyRelation = exports.LegacyColumnRef = exports.LegacyTableRef = exports.LegacyRelationName = exports.LegacyTable = exports.LegacyTableDbStats = exports.LegacyColumn = exports.LegacyColumnDbStats = exports.LegacyCheck = exports.LegacyIndex = exports.LegacyUnique = exports.LegacyPrimaryKey = exports.LegacyColumnId = exports.LegacyColumnValue = exports.LegacyColumnType = exports.LegacyColumnName = exports.LegacyTableId = exports.LegacyTableName = exports.LegacySchemaName = exports.LegacyJsValue = exports.legacyColumnTypeUnknown = exports.legacyColumnPathSeparator = void 0;
		exports.schemaToColumns = schemaToColumns;
		exports.databaseFromLegacy = databaseFromLegacy;
		exports.databaseToLegacy = databaseToLegacy;
		exports.tableDbStatsFromLegacy = tableDbStatsFromLegacy;
		exports.tableDbStatsToLegacy = tableDbStatsToLegacy;
		exports.columnValueFromLegacy = columnValueFromLegacy;
		exports.columnValueToLegacy = columnValueToLegacy;
		exports.primaryKeyFromLegacy = primaryKeyFromLegacy;
		exports.columnNameFromLegacy = columnNameFromLegacy;
		exports.uniqueFromLegacy = uniqueFromLegacy;
		exports.indexFromLegacy = indexFromLegacy;
		exports.checkFromLegacy = checkFromLegacy;
		exports.relationFromLegacy = relationFromLegacy;
		exports.tableRefFromId = tableRefFromId;
		exports.tableRefToId = tableRefToId;
		exports.columnRefFromLegacy = columnRefFromLegacy;
		exports.columnRefToLegacy = columnRefToLegacy;
		exports.typeFromLegacy = typeFromLegacy;
		exports.legacyDatabaseJsonFormat = legacyDatabaseJsonFormat;
		const zod_1 = /*@__PURE__*/ requireLib();
		const utils_1 = requireOut$1();
		const common_1 = requireCommon();
		exports.legacyColumnPathSeparator = ":";
		exports.legacyColumnTypeUnknown = 'unknown';
		const LegacyJsValueLiteral = zod_1.z.union([zod_1.z.string(), zod_1.z.number(), zod_1.z.boolean(), zod_1.z.date(), zod_1.z.null()]);
		exports.LegacyJsValue = zod_1.z.lazy(() => zod_1.z.union([LegacyJsValueLiteral, zod_1.z.array(exports.LegacyJsValue), zod_1.z.record(exports.LegacyJsValue)]));
		exports.LegacySchemaName = zod_1.z.string();
		exports.LegacyTableName = zod_1.z.string();
		exports.LegacyTableId = zod_1.z.string();
		exports.LegacyColumnName = zod_1.z.string();
		exports.LegacyColumnType = zod_1.z.string();
		exports.LegacyColumnValue = zod_1.z.string();
		exports.LegacyColumnId = zod_1.z.string();
		exports.LegacyPrimaryKey = zod_1.z.object({ name: zod_1.z.string().nullish(), columns: exports.LegacyColumnName.array() }).strict();
		exports.LegacyUnique = zod_1.z.object({
		    name: zod_1.z.string().nullish(),
		    columns: exports.LegacyColumnName.array(),
		    definition: zod_1.z.string().nullish()
		}).strict();
		exports.LegacyIndex = zod_1.z.object({
		    name: zod_1.z.string().nullish(),
		    columns: exports.LegacyColumnName.array(),
		    definition: zod_1.z.string().nullish()
		}).strict();
		exports.LegacyCheck = zod_1.z.object({
		    name: zod_1.z.string().nullish(),
		    columns: exports.LegacyColumnName.array(),
		    predicate: zod_1.z.string().nullish()
		}).strict();
		exports.LegacyColumnDbStats = zod_1.z.object({
		    nulls: zod_1.z.number().optional(), // percentage of nulls
		    bytesAvg: zod_1.z.number().optional(), // average bytes for a value
		    cardinality: zod_1.z.number().optional(), // number of different values
		    commonValues: zod_1.z.object({
		        value: exports.LegacyColumnValue,
		        freq: zod_1.z.number()
		    }).strict().array().optional(),
		    histogram: exports.LegacyColumnValue.array().optional(),
		}).strict();
		exports.LegacyColumn = zod_1.z.object({
		    name: exports.LegacyColumnName,
		    type: exports.LegacyColumnType,
		    nullable: zod_1.z.boolean().nullish(),
		    default: exports.LegacyColumnValue.nullish(),
		    comment: zod_1.z.string().nullish(),
		    values: zod_1.z.string().array().nullish(),
		    columns: zod_1.z.lazy(() => exports.LegacyColumn.array().nullish()),
		    stats: exports.LegacyColumnDbStats.nullish()
		}).strict();
		exports.LegacyTableDbStats = zod_1.z.object({
		    rows: zod_1.z.number().optional(), // number of rows
		    size: zod_1.z.number().optional(), // used bytes
		    sizeIdx: zod_1.z.number().optional(), // used bytes for indexes
		    scanSeq: zod_1.z.number().optional(), // number of seq scan
		    scanSeqLast: common_1.DateTime.optional(),
		    scanIdx: zod_1.z.number().optional(), // number of index scan
		    scanIdxLast: common_1.DateTime.optional(),
		    analyzeLast: common_1.DateTime.optional(),
		    vacuumLast: common_1.DateTime.optional(),
		}).strict();
		exports.LegacyTable = zod_1.z.object({
		    schema: exports.LegacySchemaName,
		    table: exports.LegacyTableName,
		    columns: exports.LegacyColumn.array(),
		    view: zod_1.z.boolean().nullish(),
		    definition: zod_1.z.string().nullish(), // query definition for views
		    primaryKey: exports.LegacyPrimaryKey.nullish(),
		    uniques: exports.LegacyUnique.array().nullish(),
		    indexes: exports.LegacyIndex.array().nullish(),
		    checks: exports.LegacyCheck.array().nullish(),
		    comment: zod_1.z.string().nullish(),
		    stats: exports.LegacyTableDbStats.nullish(),
		}).strict();
		exports.LegacyRelationName = zod_1.z.string();
		exports.LegacyTableRef = zod_1.z.object({
		    schema: exports.LegacySchemaName,
		    table: exports.LegacyTableName,
		}).strict();
		exports.LegacyColumnRef = zod_1.z.object({
		    table: exports.LegacyTableId,
		    column: exports.LegacyColumnName
		}).strict();
		exports.LegacyRelation = zod_1.z.object({ name: exports.LegacyRelationName, src: exports.LegacyColumnRef, ref: exports.LegacyColumnRef }).strict();
		const LegacyTypeContent = zod_1.z.union([
		    zod_1.z.object({ values: zod_1.z.string().array().nullable() }),
		    zod_1.z.object({ definition: zod_1.z.string() })
		]);
		exports.LegacyTypeName = zod_1.z.string();
		exports.LegacyType = zod_1.z.object({
		    schema: exports.LegacySchemaName,
		    name: exports.LegacyTypeName
		}).and(LegacyTypeContent);
		exports.LegacyDatabase = zod_1.z.object({
		    tables: exports.LegacyTable.array(),
		    relations: exports.LegacyRelation.array(),
		    types: exports.LegacyType.array().nullish()
		}).strict().describe('LegacyDatabase');
		function schemaToColumns(schema, flatten, path = []) {
		    // TODO: if string with few values (< 10% of docs), handle it like an enum and add values in comment
		    if (schema.nested && flatten > 0) {
		        return Object.entries(schema.nested).flatMap(([key, value]) => {
		            return [{
		                    name: path.map(p => p + '.').join('') + key,
		                    type: value.type,
		                    nullable: value.nullable
		                }, ...schemaToColumns(value, flatten - 1, [...path, key])];
		        });
		    }
		    else if (schema.nested) {
		        return Object.entries(schema.nested).map(([key, value]) => {
		            return (0, utils_1.removeUndefined)({
		                name: path.map(p => p + '.').join('') + key,
		                type: value.type,
		                nullable: value.nullable,
		                columns: value.nested ? schemaToColumns(value, 0, []) : undefined
		            });
		        });
		    }
		    else {
		        return [];
		    }
		}
		function databaseFromLegacy(db) {
		    return (0, utils_1.removeEmpty)({
		        entities: db.tables.map(tableFromLegacy),
		        relations: db.relations.map(relationFromLegacy),
		        types: db.types?.map(typeFromLegacy)
		    });
		}
		function databaseToLegacy(db) {
		    return (0, utils_1.removeUndefined)({
		        tables: db.entities?.map(tableToLegacy) || [],
		        relations: db.relations?.flatMap(relationToLegacy) || [],
		        types: db.types?.map(typeToLegacy)
		    });
		}
		function tableFromLegacy(t) {
		    return (0, utils_1.removeEmpty)({
		        database: undefined,
		        catalog: undefined,
		        schema: t.schema || undefined,
		        name: t.table,
		        kind: t.view ? 'view' : undefined,
		        def: t.definition || undefined,
		        attrs: t.columns.map(columnFromLegacy),
		        pk: t.primaryKey ? primaryKeyFromLegacy(t.primaryKey) : undefined,
		        indexes: (t.uniques || []).map(uniqueFromLegacy).concat((t.indexes || []).map(indexFromLegacy)),
		        checks: t.checks?.map(checkFromLegacy),
		        doc: t.comment || undefined,
		        stats: t.stats ? tableDbStatsFromLegacy(t.stats) : undefined,
		    });
		}
		function tableToLegacy(e) {
		    const uniques = e.indexes?.filter(i => i.unique) || [];
		    const indexes = e.indexes?.filter(i => !i.unique) || [];
		    return (0, utils_1.removeUndefined)({
		        schema: e.schema || '',
		        table: e.name,
		        columns: (e.attrs || []).map(columnToLegacy),
		        view: e.kind === 'view' || e.kind === 'materialized view' || undefined,
		        definition: e.def,
		        primaryKey: e.pk ? primaryKeyToLegacy(e.pk) : undefined,
		        uniques: uniques.length > 0 ? uniques.map(uniqueToLegacy) : undefined,
		        indexes: indexes.length > 0 ? indexes.map(indexToLegacy) : undefined,
		        checks: e.checks ? e.checks.map(checkToLegacy) : undefined,
		        comment: e.doc,
		        stats: e.stats ? tableDbStatsToLegacy(e.stats) : undefined,
		    });
		}
		function tableDbStatsFromLegacy(s) {
		    return (0, utils_1.removeUndefined)({
		        rows: s.rows,
		        size: s.size,
		        sizeIdx: s.sizeIdx,
		        scanSeq: s.scanSeq,
		        scanSeqLast: s.scanSeqLast,
		        scanIdx: s.scanIdx,
		        scanIdxLast: s.scanIdxLast,
		        analyzeLast: s.analyzeLast,
		        vacuumLast: s.vacuumLast,
		    });
		}
		function tableDbStatsToLegacy(s) {
		    return (0, utils_1.removeUndefined)({
		        rows: s.rows,
		        size: s.size,
		        sizeIdx: s.sizeIdx,
		        scanSeq: s.scanSeq,
		        scanSeqLast: s.scanSeqLast,
		        scanIdx: s.scanIdx,
		        scanIdxLast: s.scanIdxLast,
		        analyzeLast: s.analyzeLast,
		        vacuumLast: s.vacuumLast,
		    });
		}
		function columnFromLegacy(c) {
		    return (0, utils_1.removeUndefined)({
		        name: c.name,
		        type: c.type,
		        null: c.nullable || undefined,
		        default: c.default ? columnValueFromLegacy(c.default) : undefined,
		        attrs: c.columns ? c.columns.map(columnFromLegacy) : undefined,
		        doc: c.comment || undefined,
		        stats: c.stats || c.values ? (0, utils_1.removeUndefined)({
		            nulls: c.stats?.nulls,
		            bytesAvg: c.stats?.bytesAvg,
		            cardinality: c.stats?.cardinality,
		            commonValues: c.stats?.commonValues?.map(v => ({ value: columnValueFromLegacy(v.value), freq: v.freq })),
		            distinctValues: c.values?.map(columnValueFromLegacy),
		            histogram: c.stats?.histogram?.map(columnValueFromLegacy),
		            min: undefined,
		            max: undefined,
		        }) : undefined,
		    });
		}
		function columnToLegacy(a) {
		    return (0, utils_1.removeEmpty)({
		        name: a.name,
		        type: a.type,
		        nullable: a.null,
		        default: a.default ? columnValueToLegacy(a.default) : undefined,
		        comment: a.doc,
		        values: a.stats?.distinctValues?.map(columnValueToLegacy),
		        columns: a.attrs?.map(columnToLegacy),
		        stats: a.stats ? (0, utils_1.removeUndefined)({
		            nulls: a.stats.nulls,
		            bytesAvg: a.stats.bytesAvg,
		            cardinality: a.stats.cardinality,
		            commonValues: a.stats.commonValues?.map(v => ({ value: columnValueToLegacy(v.value), freq: v.freq })),
		            histogram: a.stats.histogram?.map(columnValueToLegacy),
		        }) : undefined,
		    });
		}
		function columnValueFromLegacy(v) {
		    return v;
		}
		function columnValueToLegacy(v) {
		    if (v === undefined)
		        return 'null';
		    if (v === null)
		        return 'null';
		    if (typeof v === 'object')
		        return JSON.stringify(v);
		    return v.toString(); // TODO: improve?
		}
		function primaryKeyFromLegacy(pk) {
		    return (0, utils_1.removeUndefined)({
		        name: pk.name || undefined,
		        attrs: pk.columns.map(columnNameFromLegacy)
		    });
		}
		function primaryKeyToLegacy(pk) {
		    return (0, utils_1.removeUndefined)({
		        name: pk.name,
		        columns: pk.attrs.map(columnNameToLegacy)
		    });
		}
		function columnNameFromLegacy(n) {
		    return n.split(exports.legacyColumnPathSeparator);
		}
		function columnNameToLegacy(p) {
		    return p.join(exports.legacyColumnPathSeparator);
		}
		function uniqueFromLegacy(u) {
		    return (0, utils_1.removeUndefined)({
		        name: u.name || undefined,
		        attrs: u.columns.map(columnNameFromLegacy),
		        unique: true,
		        definition: u.definition || undefined,
		    });
		}
		function uniqueToLegacy(i) {
		    return (0, utils_1.removeUndefined)({
		        name: i.name,
		        columns: i.attrs.map(columnNameToLegacy),
		        definition: i.definition
		    });
		}
		function indexFromLegacy(i) {
		    return (0, utils_1.removeUndefined)({
		        name: i.name || undefined,
		        attrs: i.columns.map(columnNameFromLegacy),
		        definition: i.definition || undefined,
		    });
		}
		function indexToLegacy(i) {
		    return (0, utils_1.removeUndefined)({
		        name: i.name,
		        columns: i.attrs.map(columnNameToLegacy),
		        definition: i.definition
		    });
		}
		function checkFromLegacy(c) {
		    return (0, utils_1.removeUndefined)({
		        name: c.name || undefined,
		        attrs: c.columns.map(columnNameFromLegacy),
		        predicate: c.predicate || '',
		    });
		}
		function checkToLegacy(c) {
		    return (0, utils_1.removeUndefined)({
		        name: c.name,
		        columns: c.attrs.map(columnNameToLegacy),
		        predicate: c.predicate || undefined
		    });
		}
		function relationFromLegacy(r) {
		    return (0, utils_1.removeUndefined)({
		        name: r.name || undefined,
		        src: { ...columnRefFromLegacy2(r.src), attrs: [r.src.column.split(exports.legacyColumnPathSeparator)] },
		        ref: { ...columnRefFromLegacy2(r.ref), attrs: [r.ref.column.split(exports.legacyColumnPathSeparator)] },
		    });
		}
		function relationToLegacy(r) {
		    return (0, utils_1.zip)(r.src.attrs, r.ref.attrs).map(([srcAttr, refAttr]) => ({ name: r.name || '', src: columnRefToLegacy2(r.src, srcAttr), ref: columnRefToLegacy2(r.ref, refAttr) }));
		}
		function tableRefFromId(id) {
		    const [schema, table] = id.split('.');
		    return table ? { schema, table } : { schema: '', table: schema };
		}
		function tableRefToId(ref) {
		    return `${ref.schema}.${ref.table}`;
		}
		function columnRefFromLegacy(c) {
		    // FIXME: use function from ColumnName to AttributePath
		    const { schema, table } = tableRefFromId(c.table);
		    return (0, utils_1.removeUndefined)({ schema: schema || undefined, entity: table, attribute: [c.column] });
		}
		function columnRefToLegacy(a) {
		    // FIXME: use function from AttributePath to ColumnName
		    return { table: tableRefToId({ schema: a.schema || '', table: a.entity }), column: a.attribute.join(exports.legacyColumnPathSeparator) };
		}
		function columnRefFromLegacy2(e) {
		    const { schema, table } = tableRefFromId(e.table);
		    return (0, utils_1.removeUndefined)({ schema: schema || undefined, entity: table });
		}
		function columnRefToLegacy2(e, c) {
		    return { table: tableRefToId({ schema: e.schema || '', table: e.entity }), column: c.join(exports.legacyColumnPathSeparator) };
		}
		function typeFromLegacy(t) {
		    if ('values' in t) {
		        return (0, utils_1.removeUndefined)({ schema: t.schema || undefined, name: t.name, values: t.values || undefined });
		    }
		    else {
		        return (0, utils_1.removeUndefined)({ schema: t.schema || undefined, name: t.name, definition: t.definition });
		    }
		}
		function typeToLegacy(t) {
		    if (t.values) {
		        return { schema: t.schema || '', name: t.name, values: t.values };
		    }
		    else {
		        return { schema: t.schema || '', name: t.name, definition: t.definition || '' };
		    }
		}
		function legacyDatabaseJsonFormat(database) {
		    return (0, utils_1.stringify)(database, (path, value) => {
		        const last = path[path.length - 1];
		        // if (last === 'tables' || last === 'relations' || last === 'types') return 0
		        if (path.includes('columns') && last !== 'columns')
		            return 0;
		        if (path.includes('primaryKey'))
		            return 0;
		        if (path.includes('uniques') && path.length > 3)
		            return 0;
		        if (path.includes('indexes') && path.length > 3)
		            return 0;
		        if (path.includes('checks') && path.length > 3)
		            return 0;
		        if (path.includes('relations') && path.length > 2)
		            return 0;
		        if (path.includes('types') && path.length > 1)
		            return 0;
		        return 2;
		    });
		}
		
	} (legacyDatabase));
	return legacyDatabase;
}

var hasRequiredDatabaseUtils;

function requireDatabaseUtils () {
	if (hasRequiredDatabaseUtils) return databaseUtils;
	hasRequiredDatabaseUtils = 1;
	(function (exports) {
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.indexTypes = exports.indexRelations = exports.indexEntities = exports.relationRefSame = exports.relationRefFromId = exports.relationRefToId = exports.relationToId = exports.relationToRef = exports.relationLinkToAttributeRef = exports.relationLinkToEntityRef = exports.typeToId = exports.typeToRef = exports.typeToNamespace = exports.typeRefSame = exports.typeRefFromId = exports.typeRefToId = exports.attributeTypeParse = exports.constraintRefSame = exports.constraintRefFromId = exports.constraintRefToId = exports.attributesRefSame = exports.attributesRefFromId = exports.attributesRefToId = exports.attributeRefSame = exports.attributeRefToEntity = exports.attributeRefFromId = exports.attributeRefToId = exports.attributePathSame = exports.attributePathFromId = exports.attributePathToId = exports.entityRefFromAttribute = exports.entityToId = exports.entityToRef = exports.entityToNamespace = exports.entityRefSame = exports.entityRefFromId = exports.entityRefToId = exports.namespaceSame = exports.namespaceFromId = exports.namespaceToId = exports.namespace = void 0;
		exports.removeQuotes = removeQuotes;
		exports.getAttribute = getAttribute;
		exports.getPeerAttributes = getPeerAttributes;
		exports.flattenAttributes = flattenAttributes;
		exports.attributeValueToString = attributeValueToString;
		exports.parseJsonDatabase = parseJsonDatabase;
		exports.generateJsonDatabase = generateJsonDatabase;
		exports.mergeDatabase = mergeDatabase;
		exports.mergeDatabaseExtra = mergeDatabaseExtra;
		exports.mergeEntity = mergeEntity;
		exports.mergeEntityExtra = mergeEntityExtra;
		exports.mergeAttribute = mergeAttribute;
		exports.mergeAttributeExtra = mergeAttributeExtra;
		exports.mergePrimaryKey = mergePrimaryKey;
		exports.mergeIndex = mergeIndex;
		exports.mergeCheck = mergeCheck;
		exports.mergeIndexExtra = mergeIndexExtra;
		exports.mergeRelation = mergeRelation;
		exports.mergeRelationExtra = mergeRelationExtra;
		exports.mergeType = mergeType;
		exports.mergeTypeExtra = mergeTypeExtra;
		exports.mergeStats = mergeStats;
		const utils_1 = requireOut$1();
		const zod_1 = requireZod();
		const database_1 = requireDatabase();
		const parserResult_1 = requireParserResult();
		const legacyDatabase_1 = requireLegacyDatabase();
		const namespace = (n) => (0, utils_1.removeUndefined)({ database: n.database, catalog: n.catalog, schema: n.schema });
		exports.namespace = namespace;
		const namespaceToId = (n) => [
		    n.database || '',
		    n.catalog || '',
		    n.schema || ''
		].map(addQuotes).join('.').replace(/^\.+/, '');
		exports.namespaceToId = namespaceToId;
		const namespaceFromId = (id) => {
		    const [schema, catalog, ...database] = id.split('.').reverse().map(removeQuotes);
		    return (0, utils_1.filterValues)({ database: database.reverse().join('.'), catalog, schema }, v => !!v);
		};
		exports.namespaceFromId = namespaceFromId;
		const namespaceSame = (a, b) => (a.schema === b.schema || a.schema === '*' || b.schema === '*') &&
		    (a.catalog === b.catalog || a.catalog === '*' || b.catalog === '*') &&
		    (a.database === b.database || a.database === '*' || b.database === '*');
		exports.namespaceSame = namespaceSame;
		const entityRefToId = (ref) => {
		    const ns = (0, exports.namespaceToId)(ref);
		    return ns ? `${ns}.${ref.entity}` : addQuotes(ref.entity);
		};
		exports.entityRefToId = entityRefToId;
		const entityRefFromId = (id) => {
		    const [entity, schema, catalog, ...database] = id.split('.').reverse().map(removeQuotes);
		    const namespace = (0, utils_1.filterValues)({ database: database.reverse().join('.'), catalog, schema }, v => !!v);
		    return { ...namespace, entity };
		};
		exports.entityRefFromId = entityRefFromId;
		const entityRefSame = (a, b) => (a.entity === b.entity || a.entity === '*' || b.entity === '*') && (0, exports.namespaceSame)(a, b);
		exports.entityRefSame = entityRefSame;
		const entityToNamespace = (e) => (0, exports.namespace)(e);
		exports.entityToNamespace = entityToNamespace;
		const entityToRef = (e) => (0, utils_1.removeUndefined)({ ...(0, exports.namespace)(e), entity: e.name });
		exports.entityToRef = entityToRef;
		const entityToId = (e) => (0, exports.entityRefToId)((0, exports.entityToRef)(e));
		exports.entityToId = entityToId;
		const entityRefFromAttribute = (a) => {
		    const { attribute, ...ref } = a;
		    return ref;
		};
		exports.entityRefFromAttribute = entityRefFromAttribute;
		const attributePathToId = (path) => path.join('.');
		exports.attributePathToId = attributePathToId;
		const attributePathFromId = (path) => path.split('.');
		exports.attributePathFromId = attributePathFromId;
		const attributePathSame = (p1, p2) => (0, utils_1.arraySame)(p1, p2, (i1, i2) => i1 === i2 || i1 === '*' || i2 === '*');
		exports.attributePathSame = attributePathSame;
		const attributeRefToId = (ref) => `${(0, exports.entityRefToId)(ref)}(${(0, exports.attributePathToId)(ref.attribute)})`;
		exports.attributeRefToId = attributeRefToId;
		const attributeRefFromId = (id) => {
		    const [, entityId, attributeId] = id.match(/^(.*)\((.*)\)$/) || [];
		    const entity = (0, exports.entityRefFromId)(entityId || id);
		    const attribute = (0, exports.attributePathFromId)(attributeId || '');
		    return { ...entity, attribute };
		};
		exports.attributeRefFromId = attributeRefFromId;
		const attributeRefToEntity = ({ attribute, ...ref }) => ref;
		exports.attributeRefToEntity = attributeRefToEntity;
		const attributeRefSame = (a, b) => (0, exports.entityRefSame)(a, b) && (0, exports.attributePathSame)(a.attribute, b.attribute);
		exports.attributeRefSame = attributeRefSame;
		const attributesRefToId = (ref) => `${(0, exports.entityRefToId)(ref)}(${ref.attrs.map(exports.attributePathToId).join(', ')})`;
		exports.attributesRefToId = attributesRefToId;
		const attributesRefFromId = (id) => {
		    const [, entityId, attributeId] = id.match(/^([^(]*)\(([^)]*)\)$/) || [];
		    const entity = (0, exports.entityRefFromId)(entityId || id);
		    const attrs = (attributeId || '').split(',').map(a => (0, exports.attributePathFromId)(a.trim()));
		    return { ...entity, attrs };
		};
		exports.attributesRefFromId = attributesRefFromId;
		const attributesRefSame = (a, b) => (0, exports.entityRefSame)(a, b) && (0, utils_1.arraySame)(a.attrs, b.attrs, exports.attributePathSame);
		exports.attributesRefSame = attributesRefSame;
		const constraintRefToId = (ref) => `${(0, exports.entityRefToId)(ref)}(${ref.constraint})`;
		exports.constraintRefToId = constraintRefToId;
		const constraintRefFromId = (id) => {
		    const [, entityId, constraintName] = id.match(/^(.*)\((.*)\)$/) || [];
		    const entity = (0, exports.entityRefFromId)(entityId || id);
		    return { ...entity, constraint: constraintName };
		};
		exports.constraintRefFromId = constraintRefFromId;
		const constraintRefSame = (a, b) => (0, exports.entityRefSame)(a, b) && (a.constraint === b.constraint || a.constraint === '*' || b.constraint === '*');
		exports.constraintRefSame = constraintRefSame;
		// similar to frontend/src/Models/Project/ColumnType.elm
		const attributeTypeParse = (type) => {
		    let res;
		    if (type.endsWith('[]'))
		        return { ...(0, exports.attributeTypeParse)(type.slice(0, -2)), array: true };
		    if (res = /^array<(.*)>$/gi.exec(type)) {
		        const [, subType] = res;
		        return { ...(0, exports.attributeTypeParse)(subType), array: true };
		    }
		    if (res = /^(?:n)?(var)?(?:bit|string|(?:bp)?char(?:acter)?)(\s+varying)?(?:\((\d+)\))?(?:\s+character set ([^ ]+))?$/gi.exec(type)) {
		        const [, var1, var2, size, encoding] = res;
		        return (0, utils_1.removeUndefined)({ full: type, kind: 'string', size: size ? parseInt(size) : undefined, variable: var1 || var2 ? true : undefined, encoding });
		    }
		    if (/^(tiny|medium|long|ci)?text$/gi.exec(type))
		        return { full: type, kind: 'string', variable: true };
		    if (res = /^(tiny|small|big)?(?:int(?:eger)?|serial)(2|4|8|64)?(?:\((\d+)\))?(?:\s+unsigned)?$/gi.exec(type)) {
		        const [, textSize, size] = res;
		        return (0, utils_1.removeUndefined)({ full: type, kind: 'int', size: size === '64' ? 8 : size ? parseInt(size) : textSize ? parseSize(textSize) : 4 });
		    }
		    if (res = /^(?:real|float)([248])?$/gi.exec(type)) {
		        const [, size] = res;
		        return { full: type, kind: 'float', size: size ? parseInt(size) : 4 };
		    }
		    if (/^double precision$/gi.exec(type))
		        return { full: type, kind: 'float', size: 8 };
		    if (/^(?:decimal|numeric|number)(\s*\(\s*\d+\s*,\s*\d+\s*\))?$/gi.exec(type))
		        return { full: type, kind: 'float' };
		    if (/^bool(?:ean)?$/gi.exec(type))
		        return { full: type, kind: 'bool' };
		    if (/^date$/gi.exec(type))
		        return { full: type, kind: 'date', size: 4 };
		    if (res = /^time(tz)?(\s+with time zone)?(\s+without time zone)?$/gi.exec(type)) {
		        const [, tz, hasTz] = res;
		        return { full: type, kind: 'time', size: tz || hasTz ? 12 : 8 };
		    }
		    if (/^timestamp(tz)?(?:\((\d+)\))?(\s+with time zone)?(\s+without time zone)?$/gi.exec(type))
		        return { full: type, kind: 'instant', size: 8 };
		    if (/^datetime$/gi.exec(type))
		        return { full: type, kind: 'instant' };
		    if (/^interval(?:\((\d+)\))?(\s+'[^']+')?$/gi.exec(type))
		        return { full: type, kind: 'period', size: 16 };
		    if (/^bytea$/gi.exec(type))
		        return { full: type, kind: 'binary' };
		    if (/^(tiny|medium|long)?blob$/gi.exec(type))
		        return { full: type, kind: 'binary', variable: true };
		    if (/^uuid$/gi.exec(type))
		        return { full: type, kind: 'uuid' };
		    if (/^jsonb?$/gi.exec(type))
		        return { full: type, kind: 'json' };
		    if (/^xml$/gi.exec(type))
		        return { full: type, kind: 'xml' };
		    return { full: type, kind: 'unknown' };
		};
		exports.attributeTypeParse = attributeTypeParse;
		const parseSize = (textSize) => textSize === 'tiny' ? 1 : textSize === 'small' ? 2 : textSize === 'big' ? 8 : undefined;
		const typeRefToId = (ref) => {
		    const ns = (0, exports.namespaceToId)(ref);
		    return ns ? `${ns}.${ref.type}` : addQuotes(ref.type);
		};
		exports.typeRefToId = typeRefToId;
		const typeRefFromId = (id) => {
		    const [type, schema, catalog, ...database] = id.split('.').reverse().map(removeQuotes);
		    const namespace = (0, utils_1.filterValues)({ database: database.reverse().join('.'), catalog, schema }, v => !!v);
		    return { ...namespace, type };
		};
		exports.typeRefFromId = typeRefFromId;
		const typeRefSame = (a, b) => (a.type === b.type || a.type === '*' || b.type === '*') &&
		    (a.schema === b.schema || a.schema === '*' || b.schema === '*') &&
		    (a.catalog === b.catalog || a.catalog === '*' || b.catalog === '*') &&
		    (a.database === b.database || a.database === '*' || b.database === '*');
		exports.typeRefSame = typeRefSame;
		const typeToNamespace = (t) => (0, exports.namespace)(t);
		exports.typeToNamespace = typeToNamespace;
		const typeToRef = (t) => (0, utils_1.removeUndefined)({ ...(0, exports.namespace)(t), type: t.name });
		exports.typeToRef = typeToRef;
		const typeToId = (t) => (0, exports.typeRefToId)((0, exports.typeToRef)(t));
		exports.typeToId = typeToId;
		const relationLinkToEntityRef = (l) => (0, utils_1.removeUndefined)({ ...(0, exports.namespace)(l), entity: l.entity });
		exports.relationLinkToEntityRef = relationLinkToEntityRef;
		const relationLinkToAttributeRef = (l) => ({ ...(0, exports.relationLinkToEntityRef)(l), attrs: l.attrs });
		exports.relationLinkToAttributeRef = relationLinkToAttributeRef;
		const relationToRef = (r) => ({ src: (0, exports.relationLinkToAttributeRef)(r.src), ref: (0, exports.relationLinkToAttributeRef)(r.ref) });
		exports.relationToRef = relationToRef;
		const relationToId = (r) => (0, exports.relationRefToId)((0, exports.relationToRef)(r));
		exports.relationToId = relationToId;
		const relationRefToId = (ref) => `${(0, exports.attributesRefToId)(ref.src)}->${(0, exports.attributesRefToId)(ref.ref)}`;
		exports.relationRefToId = relationRefToId;
		const relationRefFromId = (id) => {
		    const [, src, ref] = id.match(/^([^(]*\([^)]*\))->([^(]*\([^)]*\))$/) || [];
		    return { src: (0, exports.attributesRefFromId)(src), ref: (0, exports.attributesRefFromId)(ref) };
		};
		exports.relationRefFromId = relationRefFromId;
		const relationRefSame = (a, b) => (0, exports.attributesRefSame)(a.src, b.src) && (0, exports.attributesRefSame)(a.ref, b.ref);
		exports.relationRefSame = relationRefSame;
		function addQuotes(value) {
		    if (value.match(/^\w*$/)) {
		        return value;
		    }
		    else {
		        return `"${value}"`;
		    }
		}
		function removeQuotes(value) {
		    if (value.startsWith('"') && value.endsWith('"')) {
		        return value.slice(1, -1);
		    }
		    else if (value.startsWith("'") && value.endsWith("'")) {
		        return value.slice(1, -1);
		    }
		    else if (value.startsWith("`") && value.endsWith("`")) {
		        return value.slice(1, -1);
		    }
		    else if (value.startsWith("[") && value.endsWith("]")) { // SQL Server way ^^
		        return value.slice(1, -1);
		    }
		    else {
		        return value;
		    }
		}
		function getAttribute(attrs, path) {
		    const [head, ...tail] = path;
		    const attr = (attrs || []).find(a => a.name == head);
		    if (attr && tail.length === 0) {
		        return attr;
		    }
		    else if (attr && tail.length > 0) {
		        return getAttribute(attr.attrs || [], tail);
		    }
		    else {
		        return undefined;
		    }
		}
		function getPeerAttributes(attrs, path) {
		    const [head, ...tail] = path;
		    if (attrs && tail.length > 0) {
		        const attr = (attrs || []).find(a => a.name == head);
		        return getPeerAttributes(attr?.attrs, tail);
		    }
		    else {
		        return attrs || [];
		    }
		}
		function flattenAttributes(attrs) {
		    return (attrs || []).flatMap(a => flattenAttribute(a));
		}
		function flattenAttribute(attr, p = []) {
		    const path = [...p, attr.name];
		    return [{ ...attr, path }, ...(attr.attrs || []).flatMap(a => flattenAttribute(a, path))];
		}
		function attributeValueToString(value) {
		    if (typeof value === 'string')
		        return value;
		    if (typeof value === 'number')
		        return value.toString();
		    if (typeof value === 'boolean')
		        return value.toString();
		    if (value instanceof Date)
		        return value.toISOString();
		    if (value === null)
		        return 'null';
		    return JSON.stringify(value);
		}
		const indexEntities = (entities) => (0, utils_1.indexBy)(entities, exports.entityToId);
		exports.indexEntities = indexEntities;
		const indexRelations = (relations) => (0, utils_1.mapValues)((0, utils_1.groupBy)(relations, r => (0, exports.entityRefToId)(r.src)), rels => (0, utils_1.groupBy)(rels, r => (0, exports.entityRefToId)(r.ref)));
		exports.indexRelations = indexRelations;
		const indexTypes = (types) => (0, utils_1.indexBy)(types, exports.typeToId);
		exports.indexTypes = indexTypes;
		function parseJsonDatabase(content) {
		    let json = undefined;
		    try {
		        json = JSON.parse(content);
		    }
		    catch (e) {
		        return parserResult_1.ParserResult.failure([{ message: (0, utils_1.errorToString)(e), kind: 'MalformedJson', level: 'error', offset: { start: 0, end: 0 }, position: { start: { line: 0, column: 0 }, end: { line: 0, column: 0 } } }]);
		    }
		    return (0, zod_1.zodParse)(database_1.Database)(json).fold(db => parserResult_1.ParserResult.success(db), err => parserResult_1.ParserResult.failure([{ message: err, kind: 'InvalidJson', level: 'error', offset: { start: 0, end: 0 }, position: { start: { line: 0, column: 0 }, end: { line: 0, column: 0 } } }]));
		}
		function generateJsonDatabase(database) {
		    return (0, utils_1.stringify)(database, (path, value) => {
		        const last = path[path.length - 1];
		        // if (last === 'entities' || last === 'relations' || last === 'types') return 0
		        if (path.includes('attrs') && last !== 'attrs')
		            return 0;
		        if (path.includes('pk'))
		            return 0;
		        if (path.includes('indexes') && path.length > 3)
		            return 0;
		        if (path.includes('checks') && path.length > 3)
		            return 0;
		        if (path.includes('relations') && path.length > 2)
		            return 0;
		        if (path.includes('types') && path.length > 1)
		            return 0;
		        if (path.includes('stats'))
		            return 0;
		        if (path.includes('extra') && (path[0] !== 'extra' || path.length > 3))
		            return 0;
		        return 2;
		    }) + '\n';
		}
		// the first database has priority on the second one when it makes sense
		function mergeDatabase(a, b) {
		    return (0, utils_1.removeEmpty)({
		        entities: (0, utils_1.mergeBy)(a.entities || [], b.entities || [], exports.entityToId, mergeEntity),
		        relations: (0, utils_1.mergeBy)(a.relations || [], b.relations || [], exports.relationToId, mergeRelation),
		        types: (0, utils_1.mergeBy)(a.types || [], b.types || [], exports.typeToId, mergeType),
		        doc: a.doc || b.doc,
		        stats: mergeStats(a.stats, b.stats),
		        extra: mergeDatabaseExtra(a.extra, b.extra),
		    });
		}
		function mergeDatabaseExtra(a, b) {
		    if (a === undefined)
		        return b;
		    if (b === undefined)
		        return a;
		    const extra = (0, utils_1.filterKeys)(Object.assign({}, b || {}, a || {}), k => !database_1.databaseExtraKeys.includes(k.toString())); // a should overrides b
		    return (0, utils_1.removeUndefined)({
		        source: mergeOneOrSame(a.source, b.source),
		        createdAt: mergeOneOrSame(a.createdAt, b.createdAt),
		        creationTimeMs: mergeOneOrSame(a.creationTimeMs, b.creationTimeMs),
		        comments: a.comments && b.comments ? a.comments.concat(b.comments) : a.comments || b.comments, // keep all comments
		        namespaces: a.namespaces && b.namespaces ? undefined : a.namespaces || b.namespaces, // remove if defined in both
		        ...extra
		    });
		}
		function mergeEntity(a, b) {
		    return (0, utils_1.removeUndefined)({
		        database: a.database, // keep the 'a' reference (database, catalog, schema, name), should be the same as b
		        catalog: a.catalog,
		        schema: a.schema,
		        name: a.name,
		        kind: a.kind, // don't change kind, undefined means table
		        def: a.def || b.def,
		        attrs: a.attrs && b.attrs ? (0, utils_1.mergeBy)(a.attrs, b.attrs, attr => attr.name, mergeAttribute) : a.attrs || b.attrs,
		        pk: a.pk && b.pk ? mergePrimaryKey(a.pk, b.pk) : a.pk || b.pk,
		        indexes: a.indexes && b.indexes ? (0, utils_1.mergeBy)(a.indexes, b.indexes, i => i.attrs.map(exports.attributePathToId).join(','), mergeIndex) : a.indexes || b.indexes,
		        checks: a.checks && b.checks ? (0, utils_1.mergeBy)(a.checks, b.checks, c => c.predicate + ':' + c.attrs.map(exports.attributePathToId).join(','), mergeCheck) : a.checks || b.checks,
		        doc: a.doc || b.doc,
		        stats: mergeStats(a.stats, b.stats),
		        extra: mergeEntityExtra(a.extra, b.extra),
		    });
		}
		function mergeEntityExtra(a, b) {
		    if (a === undefined)
		        return b;
		    if (b === undefined)
		        return a;
		    const extra = (0, utils_1.filterKeys)(Object.assign({}, b || {}, a || {}), k => !database_1.entityExtraKeys.includes(k.toString())); // a should overrides b
		    return (0, utils_1.removeUndefined)({
		        line: a.line || b.line,
		        statement: a.statement || b.statement,
		        alias: a.alias || b.alias,
		        color: a.color || b.color,
		        tags: a.tags && b.tags ? (0, utils_1.distinct)(a.tags.concat(b.tags)) : a.tags || b.tags,
		        comment: a.comment || b.comment,
		        ...extra
		    });
		}
		function mergeAttribute(a, b) {
		    return (0, utils_1.removeUndefined)({
		        name: a.name, // keep the 'a' reference (name), should be the same as b
		        type: a.type === '' || a.type === legacyDatabase_1.legacyColumnTypeUnknown ? b.type : a.type,
		        null: a.null || b.null,
		        gen: a.gen, // don't change unique, undefined means 'false'
		        default: a.default || b.default,
		        attrs: a.attrs && b.attrs ? (0, utils_1.mergeBy)(a.attrs, b.attrs, attr => attr.name, mergeAttribute) : a.attrs || b.attrs,
		        doc: a.doc || b.doc,
		        stats: mergeStats(a.stats, b.stats),
		        extra: mergeAttributeExtra(a.extra, b.extra),
		    });
		}
		function mergeAttributeExtra(a, b) {
		    if (a === undefined)
		        return b;
		    if (b === undefined)
		        return a;
		    const extra = (0, utils_1.filterKeys)(Object.assign({}, b || {}, a || {}), k => !database_1.attributeExtraKeys.includes(k.toString())); // a should overrides b
		    return (0, utils_1.removeUndefined)({
		        line: a.line || b.line,
		        statement: a.statement || b.statement,
		        autoIncrement: a.autoIncrement === undefined ? b.autoIncrement : a.autoIncrement,
		        hidden: a.hidden === undefined ? b.hidden : a.hidden,
		        tags: a.tags && b.tags ? (0, utils_1.distinct)(a.tags.concat(b.tags)) : a.tags || b.tags,
		        comment: a.comment || b.comment,
		        ...extra
		    });
		}
		function mergePrimaryKey(a, b) {
		    return (0, utils_1.removeUndefined)({
		        name: a.name || b.name,
		        attrs: a.attrs, // don't change primary key attrs
		        doc: a.doc || b.doc,
		        stats: mergeStats(a.stats, b.stats),
		        extra: mergeIndexExtra(a.extra, b.extra),
		    });
		}
		function mergeIndex(a, b) {
		    return (0, utils_1.removeUndefined)({
		        name: a.name || b.name,
		        attrs: a.attrs, // keep the 'a' reference (attrs & predicate), should be the same as b
		        unique: a.unique, // don't change unique, undefined means 'false'
		        partial: a.partial, // don't change partial, undefined means 'false'
		        definition: a.definition || b.definition,
		        doc: a.doc || b.doc,
		        stats: mergeStats(a.stats, b.stats),
		        extra: mergeIndexExtra(a.extra, b.extra),
		    });
		}
		function mergeCheck(a, b) {
		    return (0, utils_1.removeUndefined)({
		        name: a.name || b.name,
		        attrs: a.attrs, // keep the 'a' reference (attrs & predicate), should be the same as b
		        predicate: a.predicate,
		        doc: a.doc || b.doc,
		        stats: mergeStats(a.stats, b.stats),
		        extra: mergeIndexExtra(a.extra, b.extra),
		    });
		}
		function mergeIndexExtra(a, b) {
		    if (a === undefined)
		        return b;
		    if (b === undefined)
		        return a;
		    const extra = (0, utils_1.filterKeys)(Object.assign({}, b || {}, a || {}), k => !database_1.indexExtraKeys.includes(k.toString())); // a should overrides b
		    return (0, utils_1.removeUndefined)({
		        line: a.line || b.line,
		        statement: a.statement || b.statement,
		        ...extra
		    });
		}
		function mergeRelation(a, b) {
		    return (0, utils_1.removeUndefined)({
		        name: a.name || b.name,
		        origin: a.origin, // don't change origin, undefined means 'fk'
		        src: a.src, // keep the 'a' reference (src & ref), should be the same as b
		        ref: a.ref,
		        polymorphic: a.polymorphic || b.polymorphic,
		        doc: a.doc || b.doc,
		        extra: mergeRelationExtra(a.extra, b.extra),
		    });
		}
		function mergeRelationExtra(a, b) {
		    if (a === undefined)
		        return b;
		    if (b === undefined)
		        return a;
		    const extra = (0, utils_1.filterKeys)(Object.assign({}, b || {}, a || {}), k => !database_1.relationExtraKeys.includes(k.toString())); // a should overrides b
		    return (0, utils_1.removeUndefined)({
		        line: a.line || b.line,
		        statement: a.statement || b.statement,
		        inline: mergeOneOrSame(a.inline, b.inline),
		        natural: a.natural || b.natural,
		        onUpdate: a.onUpdate || b.onUpdate,
		        onDelete: a.onDelete || b.onDelete,
		        srcAlias: a.srcAlias || b.srcAlias,
		        refAlias: a.refAlias || b.refAlias,
		        tags: a.tags && b.tags ? (0, utils_1.distinct)(a.tags.concat(b.tags)) : a.tags || b.tags,
		        comment: a.comment || b.comment,
		        ...extra
		    });
		}
		function mergeType(a, b) {
		    return (0, utils_1.removeUndefined)({
		        database: a.database, // keep the 'a' reference (database, catalog, schema, name), should be the same as b
		        catalog: a.catalog,
		        schema: a.schema,
		        name: a.name,
		        alias: a.alias || b.alias,
		        values: a.values && b.values ? (0, utils_1.distinct)(a.values.concat(b.values)) : a.values || b.values,
		        attrs: a.attrs && b.attrs ? (0, utils_1.mergeBy)(a.attrs, b.attrs, attr => attr.name, mergeAttribute) : a.attrs || b.attrs,
		        definition: a.definition || b.definition,
		        doc: a.doc || b.doc,
		        extra: mergeTypeExtra(a.extra, b.extra),
		    });
		}
		function mergeTypeExtra(a, b) {
		    if (a === undefined)
		        return b;
		    if (b === undefined)
		        return a;
		    const extra = (0, utils_1.filterKeys)(Object.assign({}, b || {}, a || {}), k => !database_1.typeExtraKeys.includes(k.toString())); // a should overrides b
		    return (0, utils_1.removeUndefined)({
		        line: a.line || b.line,
		        statement: a.statement || b.statement,
		        inline: mergeOneOrSame(a.inline, b.inline),
		        tags: a.tags && b.tags ? (0, utils_1.distinct)(a.tags.concat(b.tags)) : a.tags || b.tags,
		        comment: a.comment || b.comment,
		        ...extra
		    });
		}
		function mergeStats(a, b) {
		    // keep stats only when just one db has them, otherwise discard them to avoid false stats on db but allow extending a db with stats
		    if (a === undefined)
		        return b;
		    if (b === undefined)
		        return a;
		    return undefined;
		}
		function mergeOneOrSame(a, b) {
		    if (a === undefined)
		        return b;
		    if (b === undefined)
		        return a;
		    if (a === b)
		        return a;
		    return undefined; // remove if defined differently in both
		}
		
	} (databaseUtils));
	return databaseUtils;
}

var connector = {};

var sql = {};

var hasRequiredSql;

function requireSql () {
	if (hasRequiredSql) return sql;
	hasRequiredSql = 1;
	Object.defineProperty(sql, "__esModule", { value: true });
	sql.getMainEntity = getMainEntity;
	sql.getEntities = getEntities;
	sql.formatSql = formatSql;
	sql.parseSqlScript = parseSqlScript;
	sql.parseSqlStatement = parseSqlStatement;
	sql.parseSelectStatement = parseSelectStatement;
	sql.parseSelectColumn = parseSelectColumn;
	sql.parseSelectTable = parseSelectTable;
	sql.parseSelectJoinKind = parseSelectJoinKind;
	sql.parseCondition = parseCondition;
	sql.parseConditionBool = parseConditionBool;
	sql.parseSqlConditionBoolOp = parseSqlConditionBoolOp;
	sql.parseConditionExp = parseConditionExp;
	sql.parseSqlConditionExpOp = parseSqlConditionExpOp;
	sql.parseConditionNull = parseConditionNull;
	sql.parseConditionIn = parseConditionIn;
	sql.parseValue = parseValue;
	const utils_1 = requireOut$1();
	const databaseUtils_1 = requireDatabaseUtils();
	function getMainEntity(sql) {
	    const res = sql.match(new RegExp(insertRegex, 'i'))
	        || sql.match(new RegExp(updateRegex, 'i'))
	        || sql.match(new RegExp(deleteRegex, 'i'))
	        || sql.match(new RegExp(fromRegex, 'i'));
	    return res ? extractEntityRegex(res) : undefined;
	}
	function getEntities(sql) {
	    const lower = sql.trim().toLowerCase();
	    const matches = lower.startsWith('insert') ? [...sql.matchAll(new RegExp(insertRegex, 'gi'))] :
	        lower.startsWith('update') ? [...sql.matchAll(new RegExp(updateRegex, 'gi'))] :
	            lower.startsWith('delete') ? [...sql.matchAll(new RegExp(deleteRegex, 'gi'))] :
	                lower.startsWith('select') ? [...sql.matchAll(new RegExp(fromRegex, 'gi')), ...sql.matchAll(new RegExp(joinRegex, 'gi'))] : [];
	    return matches.map(extractEntityRegex).filter(utils_1.isNotUndefined);
	}
	const identifierRegex = `["'\`\\[]?\\b\\w+\\b["'\`\\]]?`;
	const entityRegex = `(?:${identifierRegex}\\.)?${identifierRegex}`;
	const insertRegex = `^INSERT\\s+INTO\\s+(${entityRegex})(?:\\s|\\()`;
	const updateRegex = `^UPDATE\\s+(${entityRegex})\\s`;
	const deleteRegex = `^DELETE\\s+FROM\\s+(${entityRegex})\\s`;
	const fromRegex = `\\s+FROM\\s+(${entityRegex})(?:\\s|;)`;
	const joinRegex = `\\s+JOIN\\s+(${entityRegex})\\s`;
	function extractEntityRegex(res) {
	    const [, value] = res;
	    const [entity, schema] = value?.replaceAll(/["'`\[\]]/g, '')?.split('.')?.reverse() || [];
	    return entity ? (0, utils_1.removeUndefined)({ entity, schema }) : undefined;
	}
	function formatSql(sql) {
	    const maxLength = 200;
	    const singleLine = sql.trim().split('\n')
	        .map(line => line.replaceAll(/\s+/g, ' ').replaceAll(/--.*/g, '').trim())
	        .map((line, i) => i === 0 || line.startsWith(',') ? line : ' ' + line)
	        .join('');
	    if (singleLine.length > maxLength) {
	        const [, select, other] = singleLine.match(/^SELECT\s+(.+)\s+FROM(.*)/) || [];
	        return (0, utils_1.maxLen)(select ? `SELECT ${(0, utils_1.maxLen)(select, 40)} FROM${other}` : singleLine, maxLength);
	    }
	    else {
	        return singleLine;
	    }
	}
	function parseSqlScript(script) {
	    return script
	        .split('\n') // get lines
	        .filter(line => !!line.trim() && !line.trim().startsWith('--')) // remove empty & comment lines
	        .join('\n')
	        .split(';') // get statements (naive)
	        .map(s => parseSqlStatement(s.trim() + ';'))
	        .filter(utils_1.isNotUndefined);
	}
	function parseSqlStatement(statement) {
	    try {
	        if (statement.trim().match(/^SELECT\s+/i)) {
	            return parseSelectStatement(statement);
	        }
	        else {
	            return undefined;
	        }
	    }
	    catch {
	        return undefined;
	    }
	}
	function parseSelectStatement(statement) {
	    const selectRegex = 'SELECT(?:\\s+DISTINCT)?(?:\\s+TOP\\s+(\\d+))?\\s+(.+?)';
	    const fromRegex = 'FROM\\s+(.+?)';
	    const whereRegex = 'WHERE\\s+(.+?)';
	    const groupByRegex = 'GROUP\\s+BY\\s+(.+?)';
	    const havingRegex = 'HAVING\\s+(.+?)';
	    const orderByRegex = 'ORDER\\s+BY\\s+(.+?)';
	    const limitRegex = 'LIMIT\\s+(\\d+)';
	    const offsetRegex = 'OFFSET\\s+(\\d+)(?:\\s+ROWS?)?';
	    const fetchRegex = 'FETCH\\s+(?:FIRST|NEXT)\\s+(\\d+)\\s+ROWS?\\s+ONLY';
	    const optRegex = [whereRegex, groupByRegex, havingRegex, orderByRegex, limitRegex, offsetRegex, fetchRegex].map(r => `(?:\\s+${r})?`).join('');
	    const regex = new RegExp(`^${selectRegex}\\s+${fromRegex}${optRegex}\\s*;$`, 'i');
	    const [, top, select, from, where, groupBy, having, orderBy, limit, offset, fetch] = statement.trim().match(regex) || [];
	    const parsedColumns = (select || '').split(',').map((c, i) => parseSelectColumn(c.trim(), i + 1)).filter(c => !!c);
	    const parsedTable = parseSelectTable(from || '');
	    if (parsedTable && parsedColumns.length > 0) {
	        return (0, utils_1.removeEmpty)({
	            command: 'SELECT',
	            table: parsedTable.table,
	            columns: parsedColumns,
	            joins: parsedTable.joins,
	            where: where && parseCondition(where),
	            groupBy,
	            having: having && parseCondition(having),
	            orderBy,
	            offset: offset ? parseInt(offset) : undefined,
	            limit: limit ? parseInt(limit) : fetch ? parseInt(fetch) : top ? parseInt(top) : undefined
	        });
	    }
	    else {
	        return undefined;
	    }
	}
	function parseSelectColumn(column, index) {
	    const [, scope, def, alias] = column.match(/^(?:(.+?)\.)?(.+?)(?:\s+AS\s+(.+?))?$/i) || [];
	    if (def?.match(/^([a-zA-Z0-9_$#]+?|["`].+?["`]?)$/i)) { // simple column
	        return (0, utils_1.removeUndefined)({ name: (0, databaseUtils_1.removeQuotes)(alias || def), scope: scope ? (0, databaseUtils_1.removeQuotes)(scope) : undefined, col: [(0, databaseUtils_1.removeQuotes)(def)] });
	    }
	    else if (def === '*') {
	        return (0, utils_1.removeUndefined)({ name: '*', scope: scope ? (0, databaseUtils_1.removeQuotes)(scope) : undefined });
	    }
	    else {
	        return (0, utils_1.removeUndefined)({ name: (0, databaseUtils_1.removeQuotes)(alias || `col_${index}`), def: def ? (0, databaseUtils_1.removeQuotes)(def) : undefined });
	    }
	}
	function parseSelectTable(tables) {
	    const [table, ...joins] = tables.replaceAll(/((?:INNER|LEFT|RIGHT|FULL)(?:\s+OUTER)?\s+)?JOIN\s+/gi, 'JOIN $1').split(/JOIN\s+/i) || [];
	    const [, tableSchema, tableName, tableAlias] = ((table || '').trim().match(/^(?:(.+?)\.)?(.+?)(?:\s+(.+?))?$/i) || []).map(r => r ? (0, databaseUtils_1.removeQuotes)(r) : undefined);
	    if (tableName) {
	        const tableFormatted = (0, utils_1.removeUndefined)({ name: tableName, schema: tableSchema, alias: tableAlias });
	        const joinsFormatted = joins.map(j => {
	            const [, kind, schema, table, alias, on] = (j.trim().match(/^((?:INNER|LEFT|RIGHT|FULL)(?:\s+OUTER)?\s+)?(?:(.+?)\.)?(.+?)(?:\s+(.+?))?\s+ON\s+(.+?)$/i) || []).map(r => r ? (0, databaseUtils_1.removeQuotes)(r) : undefined);
	            return table ? (0, utils_1.removeUndefined)({ table, schema, alias, kind: parseSelectJoinKind(kind || ''), on: parseCondition(on || '') }) : undefined;
	        }).filter(j => !!j);
	        return (0, utils_1.removeEmpty)({ table: tableFormatted, joins: joinsFormatted });
	    }
	    else {
	        return undefined;
	    }
	}
	function parseSelectJoinKind(kind) {
	    if (kind.match(/^INNER\s+/i))
	        return 'INNER';
	    if (kind.match(/^LEFT\s+/i))
	        return 'LEFT';
	    if (kind.match(/^RIGHT\s+/i))
	        return 'RIGHT';
	    if (kind.match(/^FULL\s+/i))
	        return 'FULL';
	    return undefined;
	}
	function parseCondition(cond) {
	    if (cond.search(/AND|OR/i) !== -1) {
	        return parseConditionBool(cond);
	    }
	    else if (cond.search(/[=<>]|\s+LIKE\s+/) !== -1) {
	        return parseConditionExp(cond);
	    }
	    else if (cond.search(/\s+NULL/i) !== -1) {
	        return parseConditionNull(cond);
	    }
	    else if (cond.search(/\s+IN\s+\(/i) !== -1) {
	        return parseConditionIn(cond);
	    }
	    else {
	        return cond; // can't parse condition yet :/
	    }
	}
	function parseConditionBool(cond) {
	    return cond.replaceAll(/\s+(AND|OR)\s+/gi, '|$1 ').split('|').reduce((acc, item) => {
	        if (acc === undefined) {
	            return parseCondition(item);
	        }
	        else {
	            const [, opS, rightS] = item.match(/^(AND|OR)\s+(.+?)$/i) || [];
	            const op = parseSqlConditionBoolOp(opS);
	            const right = parseCondition(rightS);
	            return op && right ? { op, left: acc, right } : acc;
	        }
	    }, undefined) || cond;
	}
	function parseSqlConditionBoolOp(kind) {
	    if (kind.match(/^AND$/i))
	        return 'AND';
	    if (kind.match(/^OR$/i))
	        return 'OR';
	    return undefined;
	}
	function parseConditionExp(cond) {
	    const [, leftS, opS, rightS] = cond.match(/^(.+?)\s*(=|!=|>|>=|<|<=|<>|LIKE)\s*(.+?)$/) || [];
	    const op = parseSqlConditionExpOp(opS || '');
	    const left = parseValue(leftS || '');
	    const right = parseValue(rightS || '');
	    return op && left !== undefined && right !== undefined ? { op, left, right } : cond;
	}
	function parseSqlConditionExpOp(kind) {
	    if (kind === '=')
	        return kind;
	    if (kind === '!=')
	        return kind;
	    if (kind === '>')
	        return kind;
	    if (kind === '>=')
	        return kind;
	    if (kind === '<')
	        return kind;
	    if (kind === '<=')
	        return kind;
	    if (kind === '<>')
	        return kind;
	    if (kind === 'LIKE')
	        return kind;
	    return undefined;
	}
	function parseConditionNull(cond) {
	    const [, valueS, not] = cond.match(/^(.+?)\s+IS\s+(NOT\s+)?NULL$/i) || [];
	    const op = (not || '').trim().match(/^NOT$/i) ? 'NOT NULL' : 'NULL';
	    const value = parseValue(valueS || '');
	    return value ? { op, value } : cond;
	}
	function parseConditionIn(cond) {
	    const [, valueS, not, valuesS] = cond.match(/^(.+?)\s+(NOT\s+)?IN\s+\((.+?)\)$/i) || [];
	    const op = (not || '').trim().match(/^NOT$/i) ? 'NOT IN' : 'IN';
	    const value = parseValue(valueS || '');
	    const values = valuesS.split(',').map(v => parseValue(v.trim())).filter(utils_1.isNotUndefined);
	    return value ? { op, value, values } : cond;
	}
	function parseValue(value) {
	    let res = value.match(/^(\d+)$/);
	    if (res)
	        return parseInt(res[1]);
	    res = value.match(/^'([^']+)'$/);
	    if (res)
	        return res[1];
	    res = value.match(/^(?:(.+?)\.)?([a-zA-Z0-9_$#]+?|["`].+?["`])$/i);
	    if (res)
	        return (0, utils_1.removeUndefined)({ column: (0, databaseUtils_1.removeQuotes)(res[2]), scope: res[1] ? (0, databaseUtils_1.removeQuotes)(res[1]) : undefined });
	    return undefined;
	}
	
	return sql;
}

var hasRequiredConnector;

function requireConnector () {
	if (hasRequiredConnector) return connector;
	hasRequiredConnector = 1;
	(function (exports) {
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.logQueryIfNeeded = exports.connectorScopeLevels = exports.ConnectorAttributeStats = exports.ConnectorAttributeStatsValue = exports.ConnectorEntityStats = exports.QueryAnalyze = exports.QueryResults = exports.QueryResultsAttribute = exports.DatabaseQuery = exports.QueryId = exports.connectorSchemaOptsDefaults = exports.ConnectorSchemaConfOpts = exports.ConnectorSchemaDataOpts = exports.ConnectorScopeOpts = exports.ConnectorConfOpts = void 0;
		exports.formatConnectorScope = formatConnectorScope;
		exports.isPolymorphic = isPolymorphic;
		exports.queryError = queryError;
		exports.handleError = handleError;
		exports.buildQueryAttributes = buildQueryAttributes;
		const zod_1 = /*@__PURE__*/ requireLib();
		const utils_1 = requireOut$1();
		const common_1 = requireCommon();
		const database_1 = requireDatabase();
		const sql_1 = requireSql();
		exports.ConnectorConfOpts = zod_1.z.object({
		    logQueries: zod_1.z.boolean().optional() // default: false, log executed queries using the provided logger
		}).strict();
		exports.ConnectorScopeOpts = zod_1.z.object({
		    // filters: limit the scope of the extraction
		    database: database_1.DatabaseName.optional(), // export only this database or database pattern (use LIKE if contains %, equality otherwise)
		    catalog: database_1.CatalogName.optional(), // export only this catalog or catalog pattern (use LIKE if contains %, equality otherwise)
		    schema: database_1.SchemaName.optional(), // export only this schema or schema pattern (use LIKE if contains %, equality otherwise)
		    entity: database_1.EntityName.optional(), // export only this entity or entity pattern (use LIKE if contains %, equality otherwise)
		}).strict();
		exports.ConnectorSchemaDataOpts = zod_1.z.object({
		    // data access: get more interesting result, beware of performance
		    sampleSize: zod_1.z.number().optional(), // default: 100, number of documents used to infer a schema (document databases, json attributes in relational db...)
		    inferMixedJson: zod_1.z.string().optional(), // when inferring JSON, will differentiate documents on this attribute, useful when storing several documents in the same collection in Mongo or Couchbase
		    inferJsonAttributes: zod_1.z.boolean().optional(), // will get sample values from JSON attributes to infer a schema (nested attributes)
		    inferPolymorphicRelations: zod_1.z.boolean().optional(), // will get distinct values from the kind attribute of polymorphic relations to create relations
		    inferRelationsFromJoins: zod_1.z.boolean().optional(), // will fetch historical queries and suggest relations based on joins
		    inferPii: zod_1.z.boolean().optional(), // will fetch sample rows from tables to detect columns with Personal Identifiable Information
		});
		exports.ConnectorSchemaConfOpts = zod_1.z.object({
		    // post analysis:
		    inferRelations: zod_1.z.boolean().optional(), // default: false, infer relations based on attribute names
		    // behavior
		    ignoreErrors: zod_1.z.boolean().optional(), // default: false, ignore errors when fetching the schema, just log them
		});
		exports.connectorSchemaOptsDefaults = {
		    sampleSize: 100
		};
		exports.QueryId = zod_1.z.string();
		// TODO define more generic and meaningful structure
		exports.DatabaseQuery = zod_1.z.object({
		    id: exports.QueryId, // query id to group duplicates
		    user: zod_1.z.string().optional(), // the user executing the query
		    database: database_1.DatabaseName, // the database in which the query was executed
		    query: zod_1.z.string(),
		    rows: zod_1.z.number(), // accumulated rows retrieved or affected by the query
		    plan: zod_1.z.object({ count: zod_1.z.number(), minTime: common_1.Millis, maxTime: common_1.Millis, sumTime: common_1.Millis, meanTime: common_1.Millis, sdTime: common_1.Millis }).strict().optional(),
		    exec: zod_1.z.object({ count: zod_1.z.number(), minTime: common_1.Millis, maxTime: common_1.Millis, sumTime: common_1.Millis, meanTime: common_1.Millis, sdTime: common_1.Millis }).strict().optional(),
		    blocks: zod_1.z.object({ sumRead: zod_1.z.number(), sumWrite: zod_1.z.number(), sumHit: zod_1.z.number(), sumDirty: zod_1.z.number() }).optional(), // data from tables & indexes
		    blocksTmp: zod_1.z.object({ sumRead: zod_1.z.number(), sumWrite: zod_1.z.number(), sumHit: zod_1.z.number(), sumDirty: zod_1.z.number() }).optional(), // data from temporary tables & indexes
		    blocksQuery: zod_1.z.object({ sumRead: zod_1.z.number(), sumWrite: zod_1.z.number() }).optional(), // data used for the query (sorts, hashes...)
		}).strict().describe('DatabaseQuery');
		exports.QueryResultsAttribute = zod_1.z.object({
		    name: zod_1.z.string(),
		    ref: database_1.AttributeRef.optional()
		}).strict();
		exports.QueryResults = zod_1.z.object({
		    query: zod_1.z.string(),
		    attributes: exports.QueryResultsAttribute.array(),
		    rows: common_1.JsValue.array()
		}).strict().describe('QueryResults');
		// TODO
		exports.QueryAnalyze = zod_1.z.object({}).strict().describe('QueryAnalyze');
		exports.ConnectorEntityStats = database_1.EntityRef.extend({
		    rows: zod_1.z.number(),
		    sampleValues: zod_1.z.record(database_1.AttributeValue)
		}).strict().describe('ConnectorEntityStats');
		exports.ConnectorAttributeStatsValue = zod_1.z.object({
		    value: database_1.AttributeValue,
		    count: zod_1.z.number()
		}).strict();
		exports.ConnectorAttributeStats = database_1.AttributeRef.extend({
		    type: database_1.AttributeType,
		    rows: zod_1.z.number(),
		    nulls: zod_1.z.number(),
		    cardinality: zod_1.z.number(),
		    commonValues: exports.ConnectorAttributeStatsValue.array()
		}).strict().describe('ConnectorAttributeStats');
		exports.connectorScopeLevels = ['database', 'catalog', 'schema', 'entity'];
		function formatConnectorScope(names, opts) {
		    const filters = exports.connectorScopeLevels
		        .map(key => [names[key], opts[key]])
		        .filter((v) => !!v[0] && !!v[1])
		        .map(([name, opt]) => `${opt.includes('%') ? (0, utils_1.plural)(name) : name} '${opt}'`);
		    if (filters.length > 0) {
		        return (0, utils_1.joinLast)(filters, ', ', ' and ');
		    }
		    else {
		        return '';
		    }
		}
		const logQueryIfNeeded = (id, name, sql, parameters, exec, count, { logger, logQueries }) => {
		    if (logQueries) {
		        const start = Date.now();
		        const queryId = `#${id}${name ? ' ' + name : ''}`;
		        logger.log(`${queryId} exec:\n${(0, utils_1.indent)((0, utils_1.stripIndent)(sql))}`);
		        const res = exec(sql, parameters);
		        res.then(r => logger.log(`${queryId} success: ${count(r)} rows in ${Date.now() - start} ms`), e => logger.log(`${queryId} failure: ${e} in ${Date.now() - start} ms`));
		        return res;
		    }
		    else {
		        return exec(sql, parameters);
		    }
		};
		exports.logQueryIfNeeded = logQueryIfNeeded;
		function isPolymorphic(attr, entityAttrs) {
		    return ['type', 'class', 'kind'].some(suffix => {
		        if (attr.endsWith(suffix)) {
		            const related = attr.slice(0, -suffix.length) + 'id';
		            return entityAttrs.some(c => c === related);
		        }
		        else if (attr.endsWith(suffix.toUpperCase())) {
		            const related = attr.slice(0, -suffix.length) + 'ID';
		            return entityAttrs.some(c => c === related);
		        }
		        else if (attr.endsWith(suffix.charAt(0).toUpperCase() + suffix.slice(1))) {
		            const related = attr.slice(0, -suffix.length) + 'Id';
		            return entityAttrs.some(c => c === related);
		        }
		        else {
		            return false;
		        }
		    });
		}
		function queryError(name, sql, err) {
		    const formattedSql = (0, utils_1.stripIndent)(sql);
		    if (typeof err === 'object' && err !== null) {
		        return new Error(err.constructor.name +
		            ('code' in err && err.code ? ` ${err.code}` : '') +
		            (name ? ` on query ${name}` : '') +
		            ('message' in err && err.message ? `: ${err.message}` : '') +
		            `\nQuery: ${formattedSql}`);
		    }
		    else if (err) {
		        return new Error(`Error ${JSON.stringify(err)}\n on '${formattedSql}'`);
		    }
		    else {
		        return new Error(`Error on '${formattedSql}'`);
		    }
		}
		function handleError(msg, onError, { logger, ignoreErrors }) {
		    return (err) => {
		        if (ignoreErrors) {
		            logger.warn(`${msg}. Ignoring...`);
		            return Promise.resolve(onError);
		        }
		        else {
		            return Promise.reject(err);
		        }
		    };
		}
		// assign column ref to query columns by parsing the sql query
		function buildQueryAttributes(fields, query) {
		    const keys = {};
		    const statement = (0, sql_1.parseSqlStatement)(query);
		    const wildcardTable = getWildcardSelectTable(statement);
		    const allDefinedColumns = getAllDefinedColumns(statement, fields.length);
		    return fields.map((field, i) => {
		        const name = uniqueName(field.name, keys);
		        keys[name] = true;
		        const ref = field.table && field.column ? { schema: field.schema, entity: field.table, attribute: [field.column] } :
		            wildcardTable ? { ...wildcardTable, attribute: [field.name] } :
		                allDefinedColumns ? allDefinedColumns[i] : undefined;
		        return (0, utils_1.removeUndefined)({ name, ref });
		    });
		}
		function getWildcardSelectTable(statement) {
		    if (statement && statement.command === 'SELECT') {
		        if (statement.joins === undefined && statement.columns.length === 1 && statement.columns[0].name === '*') {
		            return (0, utils_1.removeUndefined)({ schema: statement.table.schema, entity: statement.table.name });
		        }
		    }
		    return undefined;
		}
		function getAllDefinedColumns(statement, colCount) {
		    if (statement && statement.command === 'SELECT') {
		        if (statement.columns.length === colCount && statement.columns.every(c => c.name !== '*')) {
		            const mainEntity = (0, utils_1.removeUndefined)({ schema: statement.table.schema, entity: statement.table.name });
		            const aliases = (statement.joins || []).reduce((acc, j) => {
		                return j.alias ? { ...acc, [j.alias]: (0, utils_1.removeUndefined)({ schema: j.schema, entity: j.table }) } : acc;
		            }, statement.table.alias ? { [statement.table.alias]: mainEntity } : {});
		            return statement.columns.map(c => {
		                return c.col ? { ...c.scope ? aliases[c.scope] : mainEntity, attribute: c.col } : undefined;
		            });
		        }
		    }
		    return [];
		}
		function uniqueName(name, currentNames, cpt = 1) {
		    const newName = cpt === 1 ? name : `${name}_${cpt}`;
		    if (currentNames[newName]) {
		        return uniqueName(name, currentNames, cpt + 1);
		    }
		    else {
		        return newName;
		    }
		}
		
	} (connector));
	return connector;
}

var rule = {};

var hasRequiredRule;

function requireRule () {
	if (hasRequiredRule) return rule;
	hasRequiredRule = 1;
	(function (exports) {
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.AnalyzeReportHtmlResult = exports.AnalyzeReportHtmlStats = exports.AnalyzeStats = exports.AnalyzeReportLevel = exports.AnalyzeHistory = exports.AnalyzeReport = exports.AnalyzeReportResult = exports.AnalyzeReportRuleSummary = exports.AnalyzeReportRule = exports.AnalyzeReportViolation = exports.RuleViolation = exports.RuleConf = exports.ruleLevelsShown = exports.RuleLevel = exports.RuleName = exports.RuleId = void 0;
		const zod_1 = /*@__PURE__*/ requireLib();
		const common_1 = requireCommon();
		const database_1 = requireDatabase();
		const connector_1 = requireConnector();
		exports.RuleId = zod_1.z.string();
		exports.RuleName = zod_1.z.string();
		exports.RuleLevel = zod_1.z.enum(["high", "medium", "low", "hint", "off"]); // from highest to lowest
		exports.ruleLevelsShown = exports.RuleLevel.options.filter(l => l !== exports.RuleLevel.enum.off);
		exports.RuleConf = zod_1.z.object({
		    level: exports.RuleLevel,
		}).strict().describe("RuleConf");
		exports.RuleViolation = zod_1.z.object({
		    ruleId: exports.RuleId,
		    ruleName: exports.RuleName,
		    ruleLevel: exports.RuleLevel,
		    message: zod_1.z.string(),
		    // entity & attribute allow to locate the violation
		    entity: database_1.EntityRef.optional(),
		    attribute: database_1.AttributePath.optional(),
		    // extra allow to keep structured information about the violation
		    extra: zod_1.z.record(zod_1.z.any()).optional(),
		}).strict();
		exports.AnalyzeReportViolation = zod_1.z.object({
		    message: zod_1.z.string(),
		    entity: database_1.EntityRef.optional(),
		    attribute: database_1.AttributePath.optional(),
		    extra: zod_1.z.record(zod_1.z.any()).optional(),
		}).strict().describe("AnalyzeReportViolation");
		exports.AnalyzeReportRule = zod_1.z.object({
		    name: zod_1.z.string(),
		    description: zod_1.z.string().optional(), // optional for retro-compatibility
		    level: exports.RuleLevel,
		    conf: zod_1.z.record(zod_1.z.string(), zod_1.z.any()),
		    violations: exports.AnalyzeReportViolation.array(),
		    totalViolations: zod_1.z.number(),
		}).strict().describe("AnalyzeReportRule");
		exports.AnalyzeReportRuleSummary = zod_1.z.object({
		    name: zod_1.z.string(),
		    totalViolations: zod_1.z.number(),
		}).strict().describe("AnalyzeReportRuleSummary");
		exports.AnalyzeReportResult = zod_1.z.record(exports.RuleId, exports.AnalyzeReportRule);
		exports.AnalyzeReport = zod_1.z.object({
		    analysis: exports.AnalyzeReportResult,
		    // TODO: insights: z.object({mostUsedEntities: z.object({entity: EntityRef, queries: QueryId.array()}).array().optional()}).optional()
		    database: database_1.Database,
		    queries: connector_1.DatabaseQuery.array(),
		}).strict().describe("AnalyzeReport");
		exports.AnalyzeHistory = zod_1.z.object({
		    report: zod_1.z.string(),
		    date: common_1.Timestamp,
		    database: database_1.Database,
		    queries: connector_1.DatabaseQuery.array(),
		}).strict().describe("AnalyzeHistory");
		exports.AnalyzeReportLevel = zod_1.z.object({
		    level: exports.RuleLevel,
		    levelViolationsCount: zod_1.z.number(),
		    rules: zod_1.z.array(exports.AnalyzeReportRule),
		});
		exports.AnalyzeStats = zod_1.z.object({
		    nb_entities: zod_1.z.number(),
		    nb_relations: zod_1.z.number(),
		    nb_queries: zod_1.z.number(),
		    nb_types: zod_1.z.number(),
		    nb_rules: zod_1.z.number(),
		    nb_violations: zod_1.z.number(),
		    violations: zod_1.z.record(exports.RuleLevel, zod_1.z.number()),
		});
		exports.AnalyzeReportHtmlStats = zod_1.z.object({
		    nb_entities: zod_1.z.number(),
		    nb_relations: zod_1.z.number(),
		    nb_queries: zod_1.z.number(),
		    nb_types: zod_1.z.number(),
		    nb_rules: zod_1.z.number(),
		});
		exports.AnalyzeReportHtmlResult = zod_1.z.object({
		    rules: zod_1.z.array(exports.AnalyzeReportRule),
		    stats: exports.AnalyzeReportHtmlStats,
		});
		
	} (rule));
	return rule;
}

var attributeEmpty = {};

var hasRequiredAttributeEmpty;

function requireAttributeEmpty () {
	if (hasRequiredAttributeEmpty) return attributeEmpty;
	hasRequiredAttributeEmpty = 1;
	Object.defineProperty(attributeEmpty, "__esModule", { value: true });
	attributeEmpty.attributeEmptyRule = void 0;
	attributeEmpty.getEmptyAttributes = getEmptyAttributes;
	attributeEmpty.isAttributeEmpty = isAttributeEmpty;
	const utils_1 = requireOut$1();
	const database_1 = requireDatabase();
	const databaseUtils_1 = requireDatabaseUtils();
	const rule_1 = requireRule();
	const ruleId = 'attribute-empty';
	const ruleName = 'empty attribute';
	const ruleDescription = 'nullable attributes with cardinality=0 or 100% nulls when entity has some rows';
	const CustomRuleConf = rule_1.RuleConf.extend({
	    ignores: database_1.AttributeId.array().optional()
	}).strict().describe('AttributeEmptyConf');
	attributeEmpty.attributeEmptyRule = {
	    id: ruleId,
	    name: ruleName,
	    description: ruleDescription,
	    conf: { level: rule_1.RuleLevel.enum.low },
	    zConf: CustomRuleConf,
	    analyze(conf, now, db, queries, history, reference) {
	        const refIgnores = reference.map(r => r.entity && r.attribute ? { ...r.entity, attribute: r.attribute } : undefined).filter(utils_1.isNotUndefined);
	        const ignores = refIgnores.concat(conf.ignores?.map(databaseUtils_1.attributeRefFromId) || []);
	        return (db.entities || [])
	            .flatMap(getEmptyAttributes)
	            .filter(r => !ignores.some(i => (0, databaseUtils_1.attributeRefSame)(i, r)))
	            .map(r => ({
	            ruleId,
	            ruleName,
	            ruleLevel: conf.level,
	            message: `Attribute ${(0, databaseUtils_1.attributeRefToId)(r)} is empty.`,
	            entity: (0, databaseUtils_1.attributeRefToEntity)(r),
	            attribute: r.attribute,
	        }));
	    }
	};
	function getEmptyAttributes(entity) {
	    if (entity.stats?.rows || 0 > 0) {
	        return (entity.attrs || []).filter(isAttributeEmpty).map(attribute => ({ ...(0, databaseUtils_1.entityToRef)(entity), attribute: [attribute.name] }));
	    }
	    else {
	        return [];
	    }
	}
	function isAttributeEmpty(a) {
	    return a.null === true && ((a.stats?.cardinality !== undefined ? a.stats.cardinality === 0 : false) || (a.stats?.nulls !== undefined ? a.stats.nulls === 1 : false));
	}
	
	return attributeEmpty;
}

var attributeNameInconsistent = {};

var entityNameInconsistent = {};

var hasRequiredEntityNameInconsistent;

function requireEntityNameInconsistent () {
	if (hasRequiredEntityNameInconsistent) return entityNameInconsistent;
	hasRequiredEntityNameInconsistent = 1;
	Object.defineProperty(entityNameInconsistent, "__esModule", { value: true });
	entityNameInconsistent.entityNameInconsistentRule = void 0;
	entityNameInconsistent.checkNamingConsistency = checkNamingConsistency;
	entityNameInconsistent.bestCase = bestCase;
	entityNameInconsistent.addCases = addCases;
	const utils_1 = requireOut$1();
	const database_1 = requireDatabase();
	const databaseUtils_1 = requireDatabaseUtils();
	const rule_1 = requireRule();
	/**
	 * Keeping the same naming convention for all your tables will help avoid typos and understand things.
	 */
	const ruleId = 'entity-name-inconsistent';
	const ruleName = 'inconsistent entity name';
	const ruleDescription = 'entities with names not following the most common convention among entity names';
	const CustomRuleConf = rule_1.RuleConf.extend({
	    ignores: database_1.EntityId.array().optional(),
	}).strict().describe('EntityNameInconsistentConf');
	entityNameInconsistent.entityNameInconsistentRule = {
	    id: ruleId,
	    name: ruleName,
	    description: ruleDescription,
	    conf: { level: rule_1.RuleLevel.enum.low },
	    zConf: CustomRuleConf,
	    analyze(conf, now, db, queries, history, reference) {
	        const refIgnores = reference.map(r => r.entity).filter(utils_1.isNotUndefined);
	        const ignores = refIgnores.concat(conf.ignores?.map(databaseUtils_1.entityRefFromId) || []);
	        const inconsistencies = checkNamingConsistency(db.entities || []);
	        return inconsistencies.invalid
	            .filter(e => !ignores.some(i => (0, databaseUtils_1.entityRefSame)(i, e)))
	            .map(entity => ({
	            ruleId,
	            ruleName,
	            ruleLevel: conf.level,
	            message: `Entity ${(0, databaseUtils_1.entityRefToId)(entity)} doesn't follow naming convention ${inconsistencies.convention}.`,
	            entity: entity
	        }));
	    }
	};
	// same as frontend/src/PagesComponents/Organization_/Project_/Views/Modals/SchemaAnalysis/NamingConsistency.elm
	function checkNamingConsistency(entities) {
	    const counts = entities.reduce((acc, entity) => addCases(acc, entity.name), {});
	    const entityCase = bestCase(counts);
	    return {
	        convention: entityCase.name,
	        invalid: entities.filter(e => !entityCase.isValid(e.name)).map(databaseUtils_1.entityToRef)
	    };
	}
	function bestCase(counts) {
	    const bestCase = Object.entries(counts).reduce(([prevCase, prevCount], [curCase, curCount]) => {
	        return curCount > prevCount ? [curCase, curCount] : [prevCase, prevCount];
	    }, ['snake-lower', 0])[0];
	    return [
	        { name: 'camel-upper', isValid: utils_1.isCamelUpper },
	        { name: 'camel-lower', isValid: utils_1.isCamelLower },
	        { name: 'snake-upper', isValid: utils_1.isSnakeUpper },
	        { name: 'snake-lower', isValid: utils_1.isSnakeLower },
	        { name: 'kebab-upper', isValid: utils_1.isKebabUpper },
	        { name: 'kebab-lower', isValid: utils_1.isKebabLower },
	    ].find(c => c.name === bestCase) || { name: 'snake-lower', isValid: utils_1.isSnakeLower };
	}
	function addCases(acc, value) {
	    return (0, utils_1.compatibleCases)(value).reduce((acc, c) => ({ ...acc, [c]: (acc[c] || 0) + 1 }), acc);
	}
	
	return entityNameInconsistent;
}

var hasRequiredAttributeNameInconsistent;

function requireAttributeNameInconsistent () {
	if (hasRequiredAttributeNameInconsistent) return attributeNameInconsistent;
	hasRequiredAttributeNameInconsistent = 1;
	Object.defineProperty(attributeNameInconsistent, "__esModule", { value: true });
	attributeNameInconsistent.attributeNameInconsistentRule = void 0;
	attributeNameInconsistent.checkNamingConsistency = checkNamingConsistency;
	const utils_1 = requireOut$1();
	const database_1 = requireDatabase();
	const databaseUtils_1 = requireDatabaseUtils();
	const rule_1 = requireRule();
	const entityNameInconsistent_1 = requireEntityNameInconsistent();
	/**
	 * Keeping the same naming convention for all your columns will help avoid typos and understand things.
	 */
	const ruleId = 'attribute-name-inconsistent';
	const ruleName = 'inconsistent attribute name';
	const ruleDescription = 'attributes with names not following the most common convention among attribute names';
	const CustomRuleConf = rule_1.RuleConf.extend({
	    ignores: database_1.AttributeId.array().optional(),
	}).strict().describe('AttributeNameInconsistentConf');
	attributeNameInconsistent.attributeNameInconsistentRule = {
	    id: ruleId,
	    name: ruleName,
	    description: ruleDescription,
	    conf: { level: rule_1.RuleLevel.enum.low },
	    zConf: CustomRuleConf,
	    analyze(conf, now, db, queries, history, reference) {
	        const refIgnores = reference.map(r => r.entity && r.attribute ? { ...r.entity, attribute: r.attribute } : undefined).filter(utils_1.isNotUndefined);
	        const ignores = refIgnores.concat(conf.ignores?.map(databaseUtils_1.attributeRefFromId) || []);
	        const inconsistencies = checkNamingConsistency(db.entities || []);
	        return inconsistencies.invalid
	            .filter(a => !ignores.some(i => (0, databaseUtils_1.attributeRefSame)(i, a)))
	            .map(attribute => ({
	            ruleId,
	            ruleName,
	            ruleLevel: conf.level,
	            message: `Attribute ${(0, databaseUtils_1.attributeRefToId)(attribute)} doesn't follow naming convention ${inconsistencies.convention}.`,
	            entity: (0, databaseUtils_1.entityRefFromAttribute)(attribute),
	            attribute: attribute.attribute
	        }));
	    }
	};
	// same as frontend/src/PagesComponents/Organization_/Project_/Views/Modals/SchemaAnalysis/NamingConsistency.elm
	function checkNamingConsistency(entities) {
	    const counts = entities.reduce((acc, entity) => (0, databaseUtils_1.flattenAttributes)(entity.attrs).reduce((acc, a) => (0, entityNameInconsistent_1.addCases)(acc, a.name), acc), {});
	    const attributeCase = (0, entityNameInconsistent_1.bestCase)(counts);
	    return {
	        convention: attributeCase.name,
	        invalid: entities.flatMap(e => (0, databaseUtils_1.flattenAttributes)(e.attrs).filter(a => !attributeCase.isValid(a.name)).map(a => ({ ...(0, databaseUtils_1.entityToRef)(e), attribute: a.path })))
	    };
	}
	
	return attributeNameInconsistent;
}

var attributeTypeBad = {};

var hasRequiredAttributeTypeBad;

function requireAttributeTypeBad () {
	if (hasRequiredAttributeTypeBad) return attributeTypeBad;
	hasRequiredAttributeTypeBad = 1;
	Object.defineProperty(attributeTypeBad, "__esModule", { value: true });
	attributeTypeBad.attributeTypeBadRule = void 0;
	attributeTypeBad.getAttributesWithBadType = getAttributesWithBadType;
	attributeTypeBad.suggestedAttributeType = suggestedAttributeType;
	const utils_1 = requireOut$1();
	const database_1 = requireDatabase();
	const databaseUtils_1 = requireDatabaseUtils();
	const rule_1 = requireRule();
	const ruleId = 'attribute-type-bad';
	const ruleName = 'bad attribute type';
	const ruleDescription = 'text attributes with all statistic values that could have been an other type (int, decimal, uuid, timestamp)';
	const CustomRuleConf = rule_1.RuleConf.extend({
	    ignores: database_1.AttributeId.array().optional()
	}).strict().describe('AttributeTypeBadConf');
	attributeTypeBad.attributeTypeBadRule = {
	    id: ruleId,
	    name: ruleName,
	    description: ruleDescription,
	    conf: { level: rule_1.RuleLevel.enum.medium },
	    zConf: CustomRuleConf,
	    analyze(conf, now, db, queries, history, reference) {
	        const refIgnores = reference.map(r => r.entity && r.attribute ? { ...r.entity, attribute: r.attribute } : undefined).filter(utils_1.isNotUndefined);
	        const ignores = refIgnores.concat(conf.ignores?.map(databaseUtils_1.attributeRefFromId) || []);
	        return (db.entities || [])
	            .flatMap(getAttributesWithBadType)
	            .filter(r => !ignores.some(i => (0, databaseUtils_1.attributeRefSame)(i, r.ref)))
	            .map(r => ({
	            ruleId,
	            ruleName,
	            ruleLevel: conf.level,
	            message: `Attribute ${(0, databaseUtils_1.attributeRefToId)(r.ref)} with type ${r.type} could have type ${r.suggestion}.`,
	            entity: (0, databaseUtils_1.attributeRefToEntity)(r.ref),
	            attribute: r.ref.attribute,
	            extra: { attribute: r.ref, currentType: r.type, suggestedType: r.suggestion, sampleValues: r.values.slice(0, 10) }
	        }));
	    }
	};
	function getAttributesWithBadType(entity) {
	    return (entity.attrs || []).map(a => {
	        const suggestion = suggestedAttributeType(a);
	        return suggestion ? { ref: { ...(0, databaseUtils_1.entityToRef)(entity), attribute: [a.name] }, type: a.type, ...suggestion } : undefined;
	    }).filter(utils_1.isNotUndefined);
	}
	function suggestedAttributeType(a) {
	    const type = (0, databaseUtils_1.attributeTypeParse)(a.type);
	    const values = [
	        ...a.stats?.commonValues?.map(v => v.value) || [],
	        ...a.stats?.histogram || [],
	        ...a.stats?.distinctValues || [],
	    ].filter((v) => typeof v === 'string');
	    if (type.kind === database_1.AttributeTypeKind.enum.string && values.length > 0) {
	        if (values.every(utils_1.stringIsISODate))
	            return { suggestion: 'timestamp', values: values };
	        if (values.every(utils_1.stringIsUuid))
	            return { suggestion: 'uuid', values: values };
	        if (values.every(utils_1.stringIsInt))
	            return { suggestion: 'int', values: values };
	        if (values.every(utils_1.stringIsFloat))
	            return { suggestion: 'decimal', values: values };
	    }
	    return undefined;
	}
	
	return attributeTypeBad;
}

var attributeTypeInconsistent = {};

var hasRequiredAttributeTypeInconsistent;

function requireAttributeTypeInconsistent () {
	if (hasRequiredAttributeTypeInconsistent) return attributeTypeInconsistent;
	hasRequiredAttributeTypeInconsistent = 1;
	Object.defineProperty(attributeTypeInconsistent, "__esModule", { value: true });
	attributeTypeInconsistent.attributeTypeInconsistentRule = void 0;
	attributeTypeInconsistent.getInconsistentAttributeTypes = getInconsistentAttributeTypes;
	const utils_1 = requireOut$1();
	const database_1 = requireDatabase();
	const databaseUtils_1 = requireDatabaseUtils();
	const rule_1 = requireRule();
	/**
	 * There is nothing wrong with inconsistent types on columns with identical name.
	 * But sometimes, it's a hint of a same value or concept stored with a different format and may cause issues or misunderstandings.
	 * Of course, not every column with the same name stores the exact same thing, and it may be totally fine.
	 * Just have a look to be aware, and fix what need to be fixed.
	 */
	const ruleId = 'attribute-type-inconsistent';
	const ruleName = 'inconsistent attribute type';
	const ruleDescription = 'attribute names which have different types in other entities (help for consistency)';
	const CustomRuleConf = rule_1.RuleConf.extend({
	    ignores: database_1.AttributeName.array().optional()
	}).strict().describe('AttributeTypeInconsistentConf');
	attributeTypeInconsistent.attributeTypeInconsistentRule = {
	    id: ruleId,
	    name: ruleName,
	    description: ruleDescription,
	    conf: { level: rule_1.RuleLevel.enum.hint },
	    zConf: CustomRuleConf,
	    analyze(conf, now, db, queries, history, reference) {
	        const refIgnores = reference.map(r => r.attribute?.[0]).filter(utils_1.isNotUndefined);
	        const ignores = refIgnores.concat(conf.ignores || []);
	        return Object.entries(getInconsistentAttributeTypes(db.entities || []))
	            .filter(([name,]) => !ignores.some(i => i === name))
	            .map(([name, refs]) => {
	            const refsByType = Object.entries((0, utils_1.groupBy)(refs, r => r.value.type)).sort(([, a], [, b]) => a.length - b.length);
	            const message = `Attribute ${name} has several types: ${refsByType.map(([t, [r, ...others]]) => `${t} in ${(0, databaseUtils_1.attributeRefToId)(r.ref)}${others.length > 0 ? ` and ${(0, utils_1.pluralize)(others.length, 'other')}` : ''}`).join(', ')}.`;
	            const { attribute, ...entity } = refsByType[0][1][0].ref;
	            const attributes = refs.map(r => ({ ...r.ref, type: r.value.type }));
	            return { ruleId, ruleName, ruleLevel: conf.level, message, entity, attribute: [name], extra: { attributes } };
	        });
	    }
	};
	// same as frontend/src/PagesComponents/Organization_/Project_/Views/Modals/SchemaAnalysis/InconsistentTypeOnColumns.elm
	function getInconsistentAttributeTypes(entities) {
	    const attributes = entities.flatMap(e => (0, databaseUtils_1.flattenAttributes)(e.attrs).map(({ path, ...a }) => ({
	        ref: { ...(0, databaseUtils_1.entityToRef)(e), attribute: path },
	        value: a
	    })));
	    const attributesByName = (0, utils_1.groupBy)(attributes, a => a.value.name);
	    return (0, utils_1.filterValues)(attributesByName, (attrs) => !attrs.every(a => a.value.type === attrs[0].value.type));
	}
	
	return attributeTypeInconsistent;
}

var entityEmpty = {};

var hasRequiredEntityEmpty;

function requireEntityEmpty () {
	if (hasRequiredEntityEmpty) return entityEmpty;
	hasRequiredEntityEmpty = 1;
	Object.defineProperty(entityEmpty, "__esModule", { value: true });
	entityEmpty.entityEmptyRule = void 0;
	entityEmpty.isEntityEmpty = isEntityEmpty;
	const utils_1 = requireOut$1();
	const database_1 = requireDatabase();
	const databaseUtils_1 = requireDatabaseUtils();
	const rule_1 = requireRule();
	const ruleId = 'entity-empty';
	const ruleName = 'empty entity';
	const ruleDescription = 'entities with 0 rows or 0 bytes';
	const CustomRuleConf = rule_1.RuleConf.extend({
	    ignores: database_1.EntityId.array().optional(),
	}).strict().describe('EntityEmptyConf');
	entityEmpty.entityEmptyRule = {
	    id: ruleId,
	    name: ruleName,
	    description: ruleDescription,
	    conf: { level: rule_1.RuleLevel.enum.low },
	    zConf: CustomRuleConf,
	    analyze(conf, now, db, queries, history, reference) {
	        const refIgnores = reference.map(r => r.entity).filter(utils_1.isNotUndefined);
	        const ignores = refIgnores.concat(conf.ignores?.map(databaseUtils_1.entityRefFromId) || []);
	        return (db.entities || [])
	            .filter(e => isEntityEmpty(e))
	            .filter(e => !ignores.some(i => (0, databaseUtils_1.entityRefSame)(i, (0, databaseUtils_1.entityToRef)(e))))
	            .map(e => ({
	            ruleId,
	            ruleName,
	            ruleLevel: conf.level,
	            message: `Entity ${(0, databaseUtils_1.entityToId)(e)} is empty.`,
	            entity: (0, databaseUtils_1.entityToRef)(e),
	        }));
	    }
	};
	function isEntityEmpty(entity) {
	    return (entity.stats?.rows !== undefined ? entity.stats.rows === 0 : false) || (entity.stats?.size !== undefined ? entity.stats.size === 0 : false);
	}
	
	return entityEmpty;
}

var entityGrowFast = {};

var bytes = {};

var hasRequiredBytes;

function requireBytes () {
	if (hasRequiredBytes) return bytes;
	hasRequiredBytes = 1;
	(function (exports) {
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.Po = exports.To = exports.Go = exports.Mo = exports.Ko = exports.Bytes = void 0;
		exports.showBytes = showBytes;
		const zod_1 = /*@__PURE__*/ requireLib();
		const utils_1 = requireOut$1();
		exports.Bytes = zod_1.z.number(); // number of bytes
		exports.Ko = 1000;
		exports.Mo = 1000 * exports.Ko;
		exports.Go = 1000 * exports.Mo;
		exports.To = 1000 * exports.Go;
		exports.Po = 1000 * exports.To;
		function showBytes(bytes) {
		    if (bytes > exports.Po)
		        return `${(0, utils_1.prettyNumber)(bytes / exports.Po)} Po`;
		    if (bytes > exports.To)
		        return `${(0, utils_1.prettyNumber)(bytes / exports.To)} To`;
		    if (bytes > exports.Go)
		        return `${(0, utils_1.prettyNumber)(bytes / exports.Go)} Go`;
		    if (bytes > exports.Mo)
		        return `${(0, utils_1.prettyNumber)(bytes / exports.Mo)} Mo`;
		    if (bytes > exports.Ko)
		        return `${(0, utils_1.prettyNumber)(bytes / exports.Ko)} ko`;
		    return `${Math.round(bytes)} bytes`;
		}
		
	} (bytes));
	return bytes;
}

var date = {};

var hasRequiredDate;

function requireDate () {
	if (hasRequiredDate) return date;
	hasRequiredDate = 1;
	Object.defineProperty(date, "__esModule", { value: true });
	date.showDate = showDate;
	function showDate(date) {
	    return `${new Date(date).toISOString().split('T')[0]}`;
	}
	
	return date;
}

var duration = {};

var hasRequiredDuration;

function requireDuration () {
	if (hasRequiredDuration) return duration;
	hasRequiredDuration = 1;
	(function (exports) {
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.oneYear = exports.oneMonth = exports.oneWeek = exports.oneDay = exports.oneHour = exports.oneMinute = exports.oneSecond = exports.Duration = void 0;
		exports.showDuration = showDuration;
		const zod_1 = /*@__PURE__*/ requireLib();
		exports.Duration = zod_1.z.number(); // in millis
		exports.oneSecond = 1000;
		exports.oneMinute = 60 * exports.oneSecond;
		exports.oneHour = 60 * exports.oneMinute;
		exports.oneDay = 24 * exports.oneHour;
		exports.oneWeek = 7 * exports.oneDay;
		exports.oneMonth = 30 * exports.oneDay;
		exports.oneYear = 365 * exports.oneDay;
		function showDuration(millis) {
		    if (millis > 2 * 365 * 24 * 60 * 60 * 1000) {
		        return `${Math.round(millis / 1000 / 60 / 60 / 24 / 365)} years`;
		    }
		    else if (millis > 2 * 30 * 24 * 60 * 60 * 1000) {
		        return `${Math.round(millis / 1000 / 60 / 60 / 24 / 30)} months`;
		    }
		    else if (millis > 2 * 7 * 24 * 60 * 60 * 1000) {
		        return `${Math.round(millis / 1000 / 60 / 60 / 24 / 7)} weeks`;
		    }
		    else if (millis > 2 * 24 * 60 * 60 * 1000) {
		        return `${Math.round(millis / 1000 / 60 / 60 / 24)} days`;
		    }
		    else if (millis > 2 * 60 * 60 * 1000) {
		        return `${Math.round(millis / 1000 / 60 / 60)} hours`;
		    }
		    else if (millis > 2 * 60 * 1000) {
		        return `${Math.round(millis / 1000 / 60)} mins`;
		    }
		    else if (millis > 10 * 1000) {
		        return `${Math.round(millis / 1000)} secs`;
		    }
		    else {
		        return `${Math.round(millis)} ms`;
		    }
		}
		
	} (duration));
	return duration;
}

var percent = {};

var hasRequiredPercent;

function requirePercent () {
	if (hasRequiredPercent) return percent;
	hasRequiredPercent = 1;
	Object.defineProperty(percent, "__esModule", { value: true });
	percent.computePercent = computePercent;
	percent.showPercent = showPercent;
	function computePercent(start, end) {
	    return (end - start) / start;
	}
	function showPercent(pc) {
	    return `${Math.round(pc * 100)}%`;
	}
	
	return percent;
}

var hasRequiredEntityGrowFast;

function requireEntityGrowFast () {
	if (hasRequiredEntityGrowFast) return entityGrowFast;
	hasRequiredEntityGrowFast = 1;
	Object.defineProperty(entityGrowFast, "__esModule", { value: true });
	entityGrowFast.entityGrowFastRule = void 0;
	entityGrowFast.isEntityGrowingFast = isEntityGrowingFast;
	const zod_1 = /*@__PURE__*/ requireLib();
	const utils_1 = requireOut$1();
	const common_1 = requireCommon();
	const database_1 = requireDatabase();
	const databaseUtils_1 = requireDatabaseUtils();
	const bytes_1 = requireBytes();
	const date_1 = requireDate();
	const duration_1 = requireDuration();
	const percent_1 = requirePercent();
	const rule_1 = requireRule();
	const ruleId = 'entity-grow-fast';
	const ruleName = 'fast growing entity';
	const ruleDescription = 'entities with rows or bytes growing over the daily/monthly/yearly rate thresholds and a size over the thresholds also';
	const CustomRuleConf = rule_1.RuleConf.extend({
	    ignores: database_1.EntityId.array().optional(),
	    minRows: zod_1.z.number(),
	    minSize: bytes_1.Bytes,
	    maxGrowthYearly: common_1.Percent,
	    maxGrowthMonthly: common_1.Percent,
	    maxGrowthDaily: common_1.Percent,
	}).strict().describe('EntityGrowFastConf');
	entityGrowFast.entityGrowFastRule = {
	    id: ruleId,
	    name: ruleName,
	    description: ruleDescription,
	    conf: { level: rule_1.RuleLevel.enum.medium, minRows: 10000, minSize: 10 * bytes_1.Mo, maxGrowthYearly: 2, maxGrowthMonthly: 0.1, maxGrowthDaily: 0.01 },
	    zConf: CustomRuleConf,
	    analyze(conf, now, db, queries, history, reference) {
	        const refIgnores = reference.map(r => r.entity).filter(utils_1.isNotUndefined);
	        const ignores = refIgnores.concat(conf.ignores?.map(databaseUtils_1.entityRefFromId) || []);
	        const historyEntitiesById = history.map(h => ({ report: h.report, date: h.date, entities: (0, utils_1.indexBy)(h.database.entities || [], databaseUtils_1.entityToId) }));
	        return (db.entities || [])
	            .filter(e => !ignores.some(i => (0, databaseUtils_1.entityRefSame)(i, (0, databaseUtils_1.entityToRef)(e))))
	            .map(e => {
	            const previous = historyEntitiesById.map(h => {
	                const entity = h.entities[(0, databaseUtils_1.entityToId)(e)];
	                return entity ? { report: h.report, date: h.date, entity } : undefined;
	            }).filter(utils_1.isNotUndefined);
	            return isEntityGrowingFast(now, e, previous, conf.minRows, conf.minSize, conf.maxGrowthYearly, conf.maxGrowthMonthly, conf.maxGrowthDaily);
	        })
	            .filter(utils_1.isNotUndefined)
	            .map(r => {
	            const diff = r.kind === 'size' ? (0, bytes_1.showBytes)(r.diff) : (0, utils_1.pluralize)(r.diff, 'row');
	            const rate = r.period > duration_1.oneYear ? ` (${(0, percent_1.showPercent)(r.yearly)} yearly)` : r.period > duration_1.oneMonth ? ` (${(0, percent_1.showPercent)(r.monthly)} monthly)` : r.period > duration_1.oneDay ? ` (${(0, percent_1.showPercent)(r.daily)} daily)` : '';
	            return {
	                ruleId,
	                ruleName,
	                ruleLevel: conf.level,
	                message: `Entity ${(0, databaseUtils_1.entityToId)(r.current)} has grown by ${(0, percent_1.showPercent)(r.growth)} (${diff}) since ${(0, date_1.showDate)(r.date)}${rate}.`,
	                entity: (0, databaseUtils_1.entityToRef)(r.current),
	                extra: (0, utils_1.removeUndefined)({
	                    previousReport: r.report,
	                    previousDate: new Date(r.date).toISOString(),
	                    periodMillis: r.period,
	                    growth: r.growth,
	                    kind: r.kind,
	                    previousIndexes: r.previous?.indexes?.length,
	                    currentIndexes: r.current.indexes?.length,
	                    previous: r.previous?.stats,
	                    current: r.current.stats,
	                })
	            };
	        });
	    }
	};
	function isEntityGrowingFast(now, current, history, minRows, minSize, maxGrowthYearly, maxGrowthMonthly, maxGrowthDaily) {
	    const { rows, size } = current.stats || {};
	    if ((rows && rows > minRows) || (size && size > minSize)) {
	        const fastGrowing = history.map(h => {
	            const hRows = h.entity.stats?.rows;
	            const hSize = h.entity.stats?.size;
	            const gRows = rows && hRows && rows > hRows ? { kind: 'rows', value: rows - hRows, percent: (0, percent_1.computePercent)(hRows, rows) } : undefined;
	            const gSize = size && hSize && size > hSize ? { kind: 'size', value: size - hSize, percent: (0, percent_1.computePercent)(hSize, size) } : undefined;
	            const growth = gRows && gSize ? (gRows.percent > gSize.percent ? gRows : gSize) : gRows || gSize;
	            if (growth) {
	                const period = now - h.date;
	                const daily = growth.percent / (period / duration_1.oneDay);
	                const monthly = growth.percent / (period / duration_1.oneMonth);
	                const yearly = growth.percent / (period / duration_1.oneYear);
	                if ((period > duration_1.oneDay && daily > maxGrowthDaily) || (period > duration_1.oneMonth && monthly > maxGrowthMonthly) || (period > duration_1.oneYear && yearly > maxGrowthYearly)) {
	                    return { date: h.date, report: h.report, current, previous: h.entity, kind: growth.kind, diff: growth.value, growth: growth.percent, period, daily, monthly, yearly };
	                }
	            }
	            return undefined;
	        }).filter(utils_1.isNotUndefined);
	        return (0, utils_1.maxBy)(fastGrowing, g => g.monthly);
	    }
	    return undefined;
	}
	
	return entityGrowFast;
}

var entityIndexNone = {};

var hasRequiredEntityIndexNone;

function requireEntityIndexNone () {
	if (hasRequiredEntityIndexNone) return entityIndexNone;
	hasRequiredEntityIndexNone = 1;
	Object.defineProperty(entityIndexNone, "__esModule", { value: true });
	entityIndexNone.entityIndexNoneRule = void 0;
	entityIndexNone.hasNoIndex = hasNoIndex;
	const utils_1 = requireOut$1();
	const database_1 = requireDatabase();
	const databaseUtils_1 = requireDatabaseUtils();
	const rule_1 = requireRule();
	const ruleId = 'entity-index-none';
	const ruleName = 'entity without index';
	const ruleDescription = 'entities with no primary key and no index (only tables)';
	const CustomRuleConf = rule_1.RuleConf.extend({
	    ignores: database_1.EntityId.array().optional(),
	}).strict().describe('EntityIndexNoneConf');
	entityIndexNone.entityIndexNoneRule = {
	    id: ruleId,
	    name: ruleName,
	    description: ruleDescription,
	    conf: { level: rule_1.RuleLevel.enum.high },
	    zConf: CustomRuleConf,
	    analyze(conf, now, db, queries, history, reference) {
	        const refIgnores = reference.map(r => r.entity).filter(utils_1.isNotUndefined);
	        const ignores = refIgnores.concat(conf.ignores?.map(databaseUtils_1.entityRefFromId) || []);
	        return (db.entities || []).filter(hasNoIndex)
	            .filter(e => !ignores.some(i => (0, databaseUtils_1.entityRefSame)(i, (0, databaseUtils_1.entityToRef)(e))))
	            .map(e => ({
	            ruleId,
	            ruleName,
	            ruleLevel: conf.level,
	            message: `Entity ${(0, databaseUtils_1.entityToId)(e)} has no index.`,
	            entity: (0, databaseUtils_1.entityToRef)(e),
	        }));
	    }
	};
	// same as frontend/src/PagesComponents/Organization_/Project_/Views/Modals/SchemaAnalysis/TableWithoutIndex.elm
	function hasNoIndex(entity) {
	    return (entity.kind === undefined || entity.kind === database_1.EntityKind.enum.table) && entity.pk === undefined && (entity.indexes || []).length === 0;
	}
	
	return entityIndexNone;
}

var entityIndexTooHeavy = {};

var hasRequiredEntityIndexTooHeavy;

function requireEntityIndexTooHeavy () {
	if (hasRequiredEntityIndexTooHeavy) return entityIndexTooHeavy;
	hasRequiredEntityIndexTooHeavy = 1;
	Object.defineProperty(entityIndexTooHeavy, "__esModule", { value: true });
	entityIndexTooHeavy.entityIndexTooHeavyRule = void 0;
	entityIndexTooHeavy.hasTooHeavyIndexes = hasTooHeavyIndexes;
	const zod_1 = /*@__PURE__*/ requireLib();
	const utils_1 = requireOut$1();
	const database_1 = requireDatabase();
	const databaseUtils_1 = requireDatabaseUtils();
	const rule_1 = requireRule();
	const ruleId = 'entity-index-too-heavy';
	const ruleName = 'entity with too heavy indexes';
	const ruleDescription = 'entities with index weight over data exceeding threshold';
	const CustomRuleConf = rule_1.RuleConf.extend({
	    ignores: database_1.EntityId.array().optional(),
	    ratio: zod_1.z.number(),
	}).strict().describe('EntityIndexTooHeavyConf');
	entityIndexTooHeavy.entityIndexTooHeavyRule = {
	    id: ruleId,
	    name: ruleName,
	    description: ruleDescription,
	    conf: { level: rule_1.RuleLevel.enum.medium, ratio: 1 },
	    zConf: CustomRuleConf,
	    analyze(conf, now, db, queries, history, reference) {
	        const refIgnores = reference.map(r => r.entity).filter(utils_1.isNotUndefined);
	        const ignores = refIgnores.concat(conf.ignores?.map(databaseUtils_1.entityRefFromId) || []);
	        return (db.entities || [])
	            .filter(e => hasTooHeavyIndexes(e, conf.ratio))
	            .filter(e => !ignores.some(i => (0, databaseUtils_1.entityRefSame)(i, (0, databaseUtils_1.entityToRef)(e))))
	            .map(e => {
	            const ratio = e.stats?.sizeIdx && e.stats?.size ? e.stats.sizeIdx / e.stats.size : 0;
	            return {
	                ruleId,
	                ruleName,
	                ruleLevel: conf.level,
	                message: `Entity ${(0, databaseUtils_1.entityToId)(e)} has too heavy indexes (${(0, utils_1.prettyNumber)(ratio)}x data size, ${(e.pk ? 1 : 0) + (e.indexes?.length || 0)} indexes).`,
	                entity: (0, databaseUtils_1.entityToRef)(e),
	                extra: { ratio }
	            };
	        })
	            .sort((a, b) => -(a.extra.ratio - b.extra.ratio));
	    }
	};
	function hasTooHeavyIndexes(entity, ratio) {
	    return entity.stats?.sizeIdx && entity.stats?.size ? entity.stats.sizeIdx / entity.stats.size > ratio : false;
	}
	
	return entityIndexTooHeavy;
}

var entityIndexTooMany = {};

var hasRequiredEntityIndexTooMany;

function requireEntityIndexTooMany () {
	if (hasRequiredEntityIndexTooMany) return entityIndexTooMany;
	hasRequiredEntityIndexTooMany = 1;
	Object.defineProperty(entityIndexTooMany, "__esModule", { value: true });
	entityIndexTooMany.entityIndexTooManyRule = void 0;
	entityIndexTooMany.hasTooManyIndexes = hasTooManyIndexes;
	const zod_1 = /*@__PURE__*/ requireLib();
	const utils_1 = requireOut$1();
	const database_1 = requireDatabase();
	const databaseUtils_1 = requireDatabaseUtils();
	const rule_1 = requireRule();
	const ruleId = 'entity-index-too-many';
	const ruleName = 'entity with too many indexes';
	const ruleDescription = 'entities with number of indexes exceeding the threshold';
	const CustomRuleConf = rule_1.RuleConf.extend({
	    ignores: database_1.EntityId.array().optional(),
	    max: zod_1.z.number(),
	}).strict().describe('EntityIndexTooManyConf');
	entityIndexTooMany.entityIndexTooManyRule = {
	    id: ruleId,
	    name: ruleName,
	    description: ruleDescription,
	    conf: { level: rule_1.RuleLevel.enum.medium, max: 20 },
	    zConf: CustomRuleConf,
	    analyze(conf, now, db, queries, history, reference) {
	        const refIgnores = reference.map(r => r.entity).filter(utils_1.isNotUndefined);
	        const ignores = refIgnores.concat(conf.ignores?.map(databaseUtils_1.entityRefFromId) || []);
	        return (db.entities || [])
	            .filter(e => hasTooManyIndexes(e, conf.max))
	            .filter(e => !ignores.some(i => (0, databaseUtils_1.entityRefSame)(i, (0, databaseUtils_1.entityToRef)(e))))
	            .map(e => ({
	            ruleId,
	            ruleName,
	            ruleLevel: conf.level,
	            message: `Entity ${(0, databaseUtils_1.entityToId)(e)} has too many indexes (${e.indexes?.length || 0}).`,
	            entity: (0, databaseUtils_1.entityToRef)(e),
	            extra: {
	                count: e.indexes?.length || 0,
	                indexes: (e.indexes || []).map(({ stats, extra, ...i }) => i)
	            }
	        }));
	    }
	};
	function hasTooManyIndexes(entity, max) {
	    return (entity.indexes?.length || 0) > max;
	}
	
	return entityIndexTooMany;
}

var entityNotClean = {};

var hasRequiredEntityNotClean;

function requireEntityNotClean () {
	if (hasRequiredEntityNotClean) return entityNotClean;
	hasRequiredEntityNotClean = 1;
	Object.defineProperty(entityNotClean, "__esModule", { value: true });
	entityNotClean.entityNotCleanRule = void 0;
	entityNotClean.isEntityNotClean = isEntityNotClean;
	const zod_1 = /*@__PURE__*/ requireLib();
	const utils_1 = requireOut$1();
	const database_1 = requireDatabase();
	const databaseUtils_1 = requireDatabaseUtils();
	const duration_1 = requireDuration();
	const rule_1 = requireRule();
	const ruleId = 'entity-not-clean';
	const ruleName = 'entity not clean';
	const ruleDescription = 'entities having dead rows, analyze lag/last run or vacuum lag/last run exceeding the thresholds';
	const CustomRuleConf = rule_1.RuleConf.extend({
	    ignores: database_1.EntityId.array().optional(),
	    maxDeadRows: zod_1.z.number(),
	    maxVacuumLag: zod_1.z.number(),
	    maxAnalyzeLag: zod_1.z.number(),
	    maxVacuumDelayMs: zod_1.z.number(),
	    maxAnalyzeDelayMs: zod_1.z.number(),
	}).strict().describe('EntityNotCleanConf');
	entityNotClean.entityNotCleanRule = {
	    id: ruleId,
	    name: ruleName,
	    description: ruleDescription,
	    conf: { level: rule_1.RuleLevel.enum.high, maxDeadRows: 30000, maxVacuumLag: 30000, maxAnalyzeLag: 30000, maxVacuumDelayMs: duration_1.oneDay, maxAnalyzeDelayMs: duration_1.oneDay },
	    zConf: CustomRuleConf,
	    analyze(conf, now, db, queries, history, reference) {
	        const refIgnores = reference.map(r => r.entity).filter(utils_1.isNotUndefined);
	        const ignores = refIgnores.concat(conf.ignores?.map(databaseUtils_1.entityRefFromId) || []);
	        const dbDate = db.stats?.extractedAt ? new Date(db.stats.extractedAt).getTime() : now;
	        return (db.entities || [])
	            .filter(e => isEntityNotClean(e, dbDate, conf.maxDeadRows, conf.maxVacuumLag, conf.maxAnalyzeLag, conf.maxVacuumDelayMs, conf.maxAnalyzeDelayMs))
	            .filter(e => !ignores.some(i => (0, databaseUtils_1.entityRefSame)(i, (0, databaseUtils_1.entityToRef)(e))))
	            .map(e => {
	            const reason = isEntityNotClean(e, dbDate, conf.maxDeadRows, conf.maxVacuumLag, conf.maxAnalyzeLag, conf.maxVacuumDelayMs, conf.maxAnalyzeDelayMs);
	            const value = isEntityNotCleanValue(e, dbDate, conf.maxDeadRows, conf.maxVacuumLag, conf.maxAnalyzeLag, conf.maxVacuumDelayMs, conf.maxAnalyzeDelayMs);
	            return {
	                ruleId,
	                ruleName,
	                ruleLevel: conf.level,
	                message: `Entity ${(0, databaseUtils_1.entityToId)(e)} has ${reason} (${value}).`,
	                entity: (0, databaseUtils_1.entityToRef)(e),
	                extra: { reason, value }
	            };
	        });
	    }
	};
	function isEntityNotClean(entity, now, maxDeadRows, maxVacuumLag, maxAnalyzeLag, maxVacuumDelayMs, maxAnalyzeDelayMs) {
	    if ((entity.kind === undefined || entity.kind === database_1.EntityKind.enum.table) && entity.stats) {
	        const s = entity.stats;
	        if (s.rowsDead && s.rowsDead > maxDeadRows)
	            return 'many dead rows';
	        if (s.vacuumLag && s.vacuumLag > maxVacuumLag)
	            return 'high vacuum lag';
	        if (s.analyzeLag && s.analyzeLag > maxAnalyzeLag)
	            return 'high analyze lag';
	        if (s.vacuumLast && now - new Date(s.vacuumLast).getTime() > maxVacuumDelayMs)
	            return 'old vacuum';
	        if (s.analyzeLast && now - new Date(s.analyzeLast).getTime() > maxAnalyzeDelayMs)
	            return 'old analyze';
	    }
	    return '';
	}
	function isEntityNotCleanValue(entity, now, maxDeadRows, maxVacuumLag, maxAnalyzeLag, maxVacuumDelayMs, maxAnalyzeDelayMs) {
	    if ((entity.kind === undefined || entity.kind === database_1.EntityKind.enum.table) && entity.stats) {
	        const s = entity.stats;
	        if (s.rowsDead && s.rowsDead > maxDeadRows)
	            return s.rowsDead;
	        if (s.vacuumLag && s.vacuumLag > maxVacuumLag)
	            return s.vacuumLag;
	        if (s.analyzeLag && s.analyzeLag > maxAnalyzeLag)
	            return s.analyzeLag;
	        if (s.vacuumLast && now - new Date(s.vacuumLast).getTime() > maxVacuumDelayMs)
	            return s.vacuumLast;
	        if (s.analyzeLast && now - new Date(s.analyzeLast).getTime() > maxAnalyzeDelayMs)
	            return s.analyzeLast;
	    }
	    return '';
	}
	
	return entityNotClean;
}

var entityTooLarge = {};

var hasRequiredEntityTooLarge;

function requireEntityTooLarge () {
	if (hasRequiredEntityTooLarge) return entityTooLarge;
	hasRequiredEntityTooLarge = 1;
	Object.defineProperty(entityTooLarge, "__esModule", { value: true });
	entityTooLarge.entityTooLargeRule = void 0;
	entityTooLarge.isEntityTooLarge = isEntityTooLarge;
	const zod_1 = /*@__PURE__*/ requireLib();
	const utils_1 = requireOut$1();
	const database_1 = requireDatabase();
	const databaseUtils_1 = requireDatabaseUtils();
	const rule_1 = requireRule();
	const ruleId = 'entity-too-large';
	const ruleName = 'entity too large';
	const ruleDescription = 'entities with number of attributes exceeding the threshold';
	const CustomRuleConf = rule_1.RuleConf.extend({
	    ignores: database_1.EntityId.array().optional(),
	    max: zod_1.z.number()
	}).strict().describe('EntityTooLargeConf');
	entityTooLarge.entityTooLargeRule = {
	    id: ruleId,
	    name: ruleName,
	    description: ruleDescription,
	    conf: { level: rule_1.RuleLevel.enum.medium, max: 30 },
	    zConf: CustomRuleConf,
	    analyze(conf, now, db, queries, history, reference) {
	        const refIgnores = reference.map(r => r.entity).filter(utils_1.isNotUndefined);
	        const ignores = refIgnores.concat(conf.ignores?.map(databaseUtils_1.entityRefFromId) || []);
	        return (db.entities || [])
	            .filter(e => isEntityTooLarge(e, conf.max))
	            .filter(e => !ignores.some(i => (0, databaseUtils_1.entityRefSame)(i, (0, databaseUtils_1.entityToRef)(e))))
	            .map(e => ({
	            ruleId,
	            ruleName,
	            ruleLevel: conf.level,
	            message: `Entity ${(0, databaseUtils_1.entityToId)(e)} has too many attributes (${e.attrs?.length || 0}).`,
	            entity: (0, databaseUtils_1.entityToRef)(e),
	            extra: { attributes: e.attrs?.length || 0 }
	        }));
	    }
	};
	// same as frontend/src/PagesComponents/Organization_/Project_/Views/Modals/SchemaAnalysis/TableTooBig.elm
	function isEntityTooLarge(entity, max) {
	    return (entity.attrs?.length || 0) > max;
	}
	
	return entityTooLarge;
}

var entityUnused = {};

var hasRequiredEntityUnused;

function requireEntityUnused () {
	if (hasRequiredEntityUnused) return entityUnused;
	hasRequiredEntityUnused = 1;
	Object.defineProperty(entityUnused, "__esModule", { value: true });
	entityUnused.entityUnusedRule = void 0;
	entityUnused.isEntityUnused = isEntityUnused;
	const zod_1 = /*@__PURE__*/ requireLib();
	const utils_1 = requireOut$1();
	const database_1 = requireDatabase();
	const databaseUtils_1 = requireDatabaseUtils();
	const date_1 = requireDate();
	const duration_1 = requireDuration();
	const rule_1 = requireRule();
	const ruleId = 'entity-unused';
	const ruleName = 'unused entity';
	const ruleDescription = 'entities with the same number of scans (sequential & indexes) during the threshold period or the last scan older than the threshold';
	const CustomRuleConf = rule_1.RuleConf.extend({
	    ignores: database_1.EntityId.array().optional(),
	    minDays: zod_1.z.number()
	}).strict().describe('EntityUnusedConf');
	entityUnused.entityUnusedRule = {
	    id: ruleId,
	    name: ruleName,
	    description: ruleDescription,
	    conf: { level: rule_1.RuleLevel.enum.medium, minDays: 10 },
	    zConf: CustomRuleConf,
	    analyze(conf, now, db, queries, history, reference) {
	        const refIgnores = reference.map(r => r.entity).filter(utils_1.isNotUndefined);
	        const ignores = refIgnores.concat(conf.ignores?.map(databaseUtils_1.entityRefFromId) || []);
	        const historyEntitiesById = history.map(h => ({ report: h.report, date: h.date, entities: (0, utils_1.indexBy)(h.database.entities || [], databaseUtils_1.entityToId) }));
	        return (db.entities || [])
	            .filter(e => !ignores.some(i => (0, databaseUtils_1.entityRefSame)(i, (0, databaseUtils_1.entityToRef)(e))))
	            .map(e => {
	            const previous = historyEntitiesById.map(h => {
	                const entity = h.entities[(0, databaseUtils_1.entityToId)(e)];
	                return entity ? { report: h.report, date: h.date, entity } : undefined;
	            }).filter(utils_1.isNotUndefined);
	            return isEntityUnused(now, e, previous, conf.minDays);
	        })
	            .filter(utils_1.isNotUndefined)
	            .map(r => ({
	            ruleId,
	            ruleName,
	            ruleLevel: conf.level,
	            message: `Entity ${(0, databaseUtils_1.entityToId)(r.current)} is unused since ${(0, date_1.showDate)(r.date)} (check all instances to be sure!).`,
	            entity: (0, databaseUtils_1.entityToRef)(r.current),
	            extra: (0, utils_1.removeUndefined)({
	                previousReport: r.report,
	                previousDate: new Date(r.date).toISOString(),
	                previous: r.previous?.stats,
	                current: r.current.stats,
	            })
	        }));
	    }
	};
	function isEntityUnused(now, current, history, minDays) {
	    const { scanSeq, scanIdx, scanSeqLast, scanIdxLast } = current.stats || {};
	    const scans = (scanSeq || 0) + (scanIdx || 0);
	    if (scans > 0) {
	        const sameAsCurrent = history.filter(h => now - h.date > minDays * duration_1.oneDay).map(h => {
	            const hScans = (h.entity.stats?.scanSeq || 0) + (h.entity.stats?.scanIdx || 0);
	            return hScans === scans ? { date: h.date, current, previous: h.entity, report: h.report } : undefined;
	        }).filter(utils_1.isNotUndefined);
	        const res = (0, utils_1.minBy)(sameAsCurrent, d => d.date);
	        if (res)
	            return res;
	    }
	    if (scanSeqLast || scanIdxLast) {
	        const lastSeqScan = scanSeqLast ? new Date(scanSeqLast).getTime() : 0;
	        const lastIdxScan = scanIdxLast ? new Date(scanIdxLast).getTime() : 0;
	        const lastScan = Math.max(lastSeqScan, lastIdxScan);
	        if (now - lastScan > minDays * duration_1.oneDay) {
	            return { date: lastScan, current };
	        }
	    }
	    return undefined;
	}
	
	return entityUnused;
}

var indexDuplicated = {};

var hasRequiredIndexDuplicated;

function requireIndexDuplicated () {
	if (hasRequiredIndexDuplicated) return indexDuplicated;
	hasRequiredIndexDuplicated = 1;
	Object.defineProperty(indexDuplicated, "__esModule", { value: true });
	indexDuplicated.indexDuplicatedRule = void 0;
	indexDuplicated.getDuplicatedIndexes = getDuplicatedIndexes;
	const utils_1 = requireOut$1();
	const database_1 = requireDatabase();
	const databaseUtils_1 = requireDatabaseUtils();
	const rule_1 = requireRule();
	/**
	 * Indexes are great to speed read performances, but they come at the cost of reducing write performances.
	 * This is call write amplification, where writing a single row MUST update one or several indexes.
	 * Most of the time this is not an issue because we read much more than we write, but still, only useful indexes should be kept.
	 * This rule identify duplicated indexes, meaning they are redundant and the smallest one could be removed to speed writes without impacting reads.
	 */
	const ruleId = 'index-duplicated';
	const ruleName = 'duplicated index';
	const ruleDescription = 'indexes with all columns covered by other indexes';
	const CustomRuleConf = rule_1.RuleConf.extend({
	    ignores: database_1.AttributesId.array().optional()
	}).strict().describe('IndexDuplicatedConf');
	indexDuplicated.indexDuplicatedRule = {
	    id: ruleId,
	    name: ruleName,
	    description: ruleDescription,
	    conf: { level: rule_1.RuleLevel.enum.high },
	    zConf: CustomRuleConf,
	    analyze(conf, now, db, queries, history, reference) {
	        const refIgnores = reference.map(r => r.entity && Array.isArray(r.extra?.index?.attrs) ? { ...r.entity, attrs: r.extra.index.attrs } : undefined).filter(utils_1.isNotUndefined);
	        const ignores = refIgnores.concat(conf.ignores?.map(databaseUtils_1.attributesRefFromId) || []);
	        return (db.entities || []).flatMap(getDuplicatedIndexes)
	            .filter(idx => !ignores.some(i => (0, databaseUtils_1.attributesRefSame)(i, { ...(0, databaseUtils_1.entityToRef)(idx.entity), attrs: idx.index.attrs })))
	            .map(i => {
	            const entity = (0, databaseUtils_1.entityToRef)(i.entity);
	            const indexName = `${i.index.name ? i.index.name + ' ' : ''}on ${(0, databaseUtils_1.attributesRefToId)({ ...entity, attrs: i.index.attrs })}`;
	            const message = `Index ${indexName} can be deleted, it's covered by: ${i.coveredBy.map(by => `${by.name || ''}(${by.attrs.map(databaseUtils_1.attributePathToId).join(', ')})`).join(', ')}.`;
	            const { stats, extra, ...index } = i.index;
	            const coveredBy = i.coveredBy.map(({ stats, extra, ...index }) => index);
	            return { ruleId, ruleName, ruleLevel: conf.level, message, entity, attribute: i.index.attrs[0], extra: { index, coveredBy } };
	        });
	    }
	};
	// same as frontend/src/PagesComponents/Organization_/Project_/Views/Modals/SchemaAnalysis/IndexDuplicated.elm
	function getDuplicatedIndexes(entity) {
	    const indexes = (entity.indexes || [])
	        .filter(i => !i.attrs.some(a => a[0] === 'unknown' || a[0] === '*expression*')) // ignore indexes with expressions
	        .sort((a, b) => (a.attrs.length - b.attrs.length) || (a.attrs.join(',').localeCompare(b.attrs.join(','))));
	    return findDuplicated(entity, indexes);
	}
	function findDuplicated(entity, indexes, duplicates = []) {
	    const [index, ...tail] = indexes;
	    if (tail.length > 0) {
	        const indexAttrs = index.attrs.map(databaseUtils_1.attributePathToId);
	        const coveredBy = tail.filter(index => indexAttrs.every((a, i) => a === (0, databaseUtils_1.attributePathToId)(index.attrs[i])));
	        if (coveredBy.length > 0) {
	            return findDuplicated(entity, tail, duplicates.concat([{ entity, index, coveredBy }]));
	        }
	        else {
	            return findDuplicated(entity, tail, duplicates);
	        }
	    }
	    else {
	        return duplicates;
	    }
	}
	
	return indexDuplicated;
}

var indexGrowFast = {};

var hasRequiredIndexGrowFast;

function requireIndexGrowFast () {
	if (hasRequiredIndexGrowFast) return indexGrowFast;
	hasRequiredIndexGrowFast = 1;
	Object.defineProperty(indexGrowFast, "__esModule", { value: true });
	indexGrowFast.indexGrowFastRule = void 0;
	indexGrowFast.isIndexGrowingFast = isIndexGrowingFast;
	const utils_1 = requireOut$1();
	const common_1 = requireCommon();
	const database_1 = requireDatabase();
	const databaseUtils_1 = requireDatabaseUtils();
	const bytes_1 = requireBytes();
	const date_1 = requireDate();
	const duration_1 = requireDuration();
	const percent_1 = requirePercent();
	const rule_1 = requireRule();
	const ruleId = 'index-grow-fast';
	const ruleName = 'fast growing index';
	const ruleDescription = 'indexes with rows or bytes growing over the daily/monthly/yearly rate thresholds and a size over the thresholds also';
	const CustomRuleConf = rule_1.RuleConf.extend({
	    ignores: database_1.ConstraintId.array().optional(),
	    minSize: bytes_1.Bytes,
	    maxGrowthYearly: common_1.Percent,
	    maxGrowthMonthly: common_1.Percent,
	    maxGrowthDaily: common_1.Percent,
	}).strict().describe('IndexGrowFastConf');
	indexGrowFast.indexGrowFastRule = {
	    id: ruleId,
	    name: ruleName,
	    description: ruleDescription,
	    conf: { level: rule_1.RuleLevel.enum.medium, minSize: 10 * bytes_1.Mo, maxGrowthYearly: 2, maxGrowthMonthly: 0.1, maxGrowthDaily: 0.01 },
	    zConf: CustomRuleConf,
	    analyze(conf, now, db, queries, history, reference) {
	        const refIgnores = reference.map(r => r.entity && r.extra?.index?.name ? { ...r.entity, constraint: r.extra.index.name } : undefined).filter(utils_1.isNotUndefined);
	        const ignores = refIgnores.concat(conf.ignores?.map(databaseUtils_1.constraintRefFromId) || []);
	        const historyEntitiesById = history.map(h => ({ report: h.report, date: h.date, entities: (0, utils_1.indexBy)(h.database.entities || [], databaseUtils_1.entityToId) }));
	        return (db.entities || [])
	            .flatMap(e => (e.indexes || []).map(i => ({ entity: e, index: i })))
	            .filter(({ entity, index: { name } }) => !(name && ignores.some(i => (0, databaseUtils_1.constraintRefSame)(i, { ...(0, databaseUtils_1.entityToRef)(entity), constraint: name }))))
	            .map(e => {
	            const previous = historyEntitiesById.map(h => {
	                const entity = h.entities[(0, databaseUtils_1.entityToId)(e.entity)];
	                const index = entity ? entity.indexes?.find(i => i.name === e.index.name) : undefined;
	                return entity && index ? { report: h.report, date: h.date, value: { entity, index } } : undefined;
	            }).filter(utils_1.isNotUndefined);
	            return isIndexGrowingFast(now, e, previous, conf.minSize, conf.maxGrowthYearly, conf.maxGrowthMonthly, conf.maxGrowthDaily);
	        })
	            .filter(utils_1.isNotUndefined)
	            .map(r => {
	            const name = `${r.current.index.name}(${r.current.index.attrs.map(databaseUtils_1.attributePathToId).join(', ')}) on ${(0, databaseUtils_1.entityToId)(r.current.entity)}`;
	            const rate = r.period > duration_1.oneYear ? ` (${(0, percent_1.showPercent)(r.yearly)} yearly)` : r.period > duration_1.oneMonth ? ` (${(0, percent_1.showPercent)(r.monthly)} monthly)` : r.period > duration_1.oneDay ? ` (${(0, percent_1.showPercent)(r.daily)} daily)` : '';
	            return {
	                ruleId,
	                ruleName,
	                ruleLevel: conf.level,
	                message: `Index ${name} has grown by ${(0, percent_1.showPercent)(r.growth)} (${(0, bytes_1.showBytes)(r.diff)}) since ${(0, date_1.showDate)(r.date)}${rate}.`,
	                entity: (0, databaseUtils_1.entityToRef)(r.current.entity),
	                extra: (0, utils_1.removeUndefined)({
	                    previousReport: r.report,
	                    previousDate: new Date(r.date).toISOString(),
	                    periodMillis: r.period,
	                    growth: r.growth,
	                    previous: r.previous.index.stats,
	                    current: r.current.index.stats,
	                })
	            };
	        });
	    }
	};
	function isIndexGrowingFast(now, current, history, minSize, maxGrowthYearly, maxGrowthMonthly, maxGrowthDaily) {
	    const { size } = current.index.stats || {};
	    if (size && size > minSize) {
	        const fastGrowing = history.map(h => {
	            const hSize = h.value.index.stats?.size;
	            const growth = hSize && size > hSize ? { value: size - hSize, percent: (0, percent_1.computePercent)(hSize, size) } : undefined;
	            if (growth) {
	                const period = now - h.date;
	                const daily = growth.percent / (period / duration_1.oneDay);
	                const monthly = growth.percent / (period / duration_1.oneMonth);
	                const yearly = growth.percent / (period / duration_1.oneYear);
	                if ((period > duration_1.oneDay && daily > maxGrowthDaily) || (period > duration_1.oneMonth && monthly > maxGrowthMonthly) || (period > duration_1.oneYear && yearly > maxGrowthYearly)) {
	                    return { date: h.date, report: h.report, current, previous: h.value, diff: growth.value, growth: growth.percent, period, daily, monthly, yearly };
	                }
	            }
	        }).filter(utils_1.isNotUndefined);
	        return (0, utils_1.maxBy)(fastGrowing, g => g.monthly);
	    }
	    return undefined;
	}
	
	return indexGrowFast;
}

var indexOnRelation = {};

var hasRequiredIndexOnRelation;

function requireIndexOnRelation () {
	if (hasRequiredIndexOnRelation) return indexOnRelation;
	hasRequiredIndexOnRelation = 1;
	Object.defineProperty(indexOnRelation, "__esModule", { value: true });
	indexOnRelation.indexOnRelationRule = void 0;
	indexOnRelation.getMissingIndexOnRelation = getMissingIndexOnRelation;
	const utils_1 = requireOut$1();
	const database_1 = requireDatabase();
	const databaseUtils_1 = requireDatabaseUtils();
	const rule_1 = requireRule();
	/**
	 * Relations are often used on JOIN or WHERE clauses to fetch more data or limit rows.
	 * It can be a good practice to add indexes on most relation columns.
	 */
	const ruleId = 'index-on-relation';
	const ruleName = 'index on relation';
	const ruleDescription = 'relation attributes with no index';
	const CustomRuleConf = rule_1.RuleConf.extend({
	    ignores: database_1.AttributesId.array().optional()
	}).strict().describe('IndexOnRelationConf');
	indexOnRelation.indexOnRelationRule = {
	    id: ruleId,
	    name: ruleName,
	    description: ruleDescription,
	    conf: { level: rule_1.RuleLevel.enum.medium },
	    zConf: CustomRuleConf,
	    analyze(conf, now, db, queries, history, reference) {
	        const refIgnores = reference.map(r => r.entity && Array.isArray(r.extra?.indexAttrs) ? { ...r.entity, attrs: r.extra.indexAttrs } : undefined).filter(utils_1.isNotUndefined);
	        const ignores = refIgnores.concat(conf.ignores?.map(databaseUtils_1.attributesRefFromId) || []);
	        const entities = (0, utils_1.indexBy)(db.entities || [], databaseUtils_1.entityToId);
	        return (db.relations || [])
	            .flatMap(r => getMissingIndexOnRelation(r, entities))
	            .filter(idx => !ignores.some(i => (0, databaseUtils_1.attributesRefSame)(i, { ...idx.ref, attrs: idx.ref.attrs })))
	            .map(i => ({
	            ruleId,
	            ruleName,
	            ruleLevel: conf.level,
	            message: `Create an index on ${(0, databaseUtils_1.attributesRefToId)({ ...i.ref, attrs: i.ref.attrs })} to improve ${(0, databaseUtils_1.relationToId)(i.relation)} relation.`,
	            entity: (0, databaseUtils_1.relationLinkToEntityRef)(i.ref),
	            attribute: i.ref.attrs[0],
	            extra: { indexAttrs: i.ref.attrs, relation: i.relation }
	        }));
	    }
	};
	// same as frontend/src/PagesComponents/Organization_/Project_/Views/Modals/SchemaAnalysis/IndexOnForeignKeys.elm
	function getMissingIndexOnRelation(relation, entities) {
	    return [
	        hasIndex(relation, relation.ref, entities),
	        hasIndex(relation, relation.src, entities),
	    ].filter(utils_1.isNotUndefined);
	}
	function hasIndex(relation, ref, entities) {
	    const entity = entities[(0, databaseUtils_1.entityRefToId)(ref)];
	    if (!entity)
	        return undefined; // don't suggest new index if entity is not found
	    if (matchIndex(entity.pk?.attrs || [], ref.attrs) || entity.indexes?.find(i => matchIndex(i.attrs, ref.attrs)) !== undefined) {
	        return undefined;
	    }
	    else {
	        return { relation, ref };
	    }
	}
	function matchIndex(indexAttrs, attributes) {
	    return attributes.every((a, i) => (0, databaseUtils_1.attributePathToId)(a) === (0, databaseUtils_1.attributePathToId)(indexAttrs[i] || []));
	}
	
	return indexOnRelation;
}

var indexUnused = {};

var hasRequiredIndexUnused;

function requireIndexUnused () {
	if (hasRequiredIndexUnused) return indexUnused;
	hasRequiredIndexUnused = 1;
	Object.defineProperty(indexUnused, "__esModule", { value: true });
	indexUnused.indexUnusedRule = void 0;
	indexUnused.isIndexUnused = isIndexUnused;
	const zod_1 = /*@__PURE__*/ requireLib();
	const utils_1 = requireOut$1();
	const database_1 = requireDatabase();
	const databaseUtils_1 = requireDatabaseUtils();
	const date_1 = requireDate();
	const duration_1 = requireDuration();
	const rule_1 = requireRule();
	const ruleId = 'index-unused';
	const ruleName = 'unused index';
	const ruleDescription = 'indexes with the same number of scans during the threshold period or the last scan older than the threshold';
	const CustomRuleConf = rule_1.RuleConf.extend({
	    ignores: database_1.ConstraintId.array().optional(),
	    minDays: zod_1.z.number()
	}).strict().describe('IndexUnusedConf');
	indexUnused.indexUnusedRule = {
	    id: ruleId,
	    name: ruleName,
	    description: ruleDescription,
	    conf: { level: rule_1.RuleLevel.enum.medium, minDays: 10 },
	    zConf: CustomRuleConf,
	    analyze(conf, now, db, queries, history, reference) {
	        const refIgnores = reference.map(r => r.entity && r.extra?.index?.name ? { ...r.entity, constraint: r.extra.index.name } : undefined).filter(utils_1.isNotUndefined);
	        const ignores = refIgnores.concat(conf.ignores?.map(databaseUtils_1.constraintRefFromId) || []);
	        const historyEntitiesById = history.map(h => ({ report: h.report, date: h.date, entities: (0, utils_1.indexBy)(h.database.entities || [], databaseUtils_1.entityToId) }));
	        return (db.entities || [])
	            .flatMap(e => (e.indexes || []).map(i => ({ entity: e, index: i })))
	            .filter(({ entity, index: { name } }) => !(name && ignores.some(i => (0, databaseUtils_1.constraintRefSame)(i, { ...(0, databaseUtils_1.entityToRef)(entity), constraint: name }))))
	            .map(e => {
	            const previous = historyEntitiesById.map(h => {
	                const entity = h.entities[(0, databaseUtils_1.entityToId)(e.entity)];
	                const index = entity ? entity.indexes?.find(i => i.name === e.index.name) : undefined;
	                return entity && index ? { report: h.report, date: h.date, value: { entity, index } } : undefined;
	            }).filter(utils_1.isNotUndefined);
	            return isIndexUnused(now, e, previous, conf.minDays);
	        })
	            .filter(utils_1.isNotUndefined)
	            .map(r => ({
	            ruleId,
	            ruleName,
	            ruleLevel: conf.level,
	            message: `Index ${r.current.index.name}(${r.current.index.attrs.map(databaseUtils_1.attributePathToId).join(', ')}) on ${(0, databaseUtils_1.entityToId)(r.current.entity)} is unused since ${(0, date_1.showDate)(r.date)} (check all instances to be sure!).`,
	            entity: (0, databaseUtils_1.entityToRef)(r.current.entity),
	            extra: (0, utils_1.removeUndefined)({
	                index: r.current.index,
	                previousReport: r.report,
	                previousDate: new Date(r.date).toISOString(),
	                previous: r.previous?.index?.stats,
	                current: r.current.index.stats,
	            })
	        }));
	    }
	};
	function isIndexUnused(now, current, history, minDays) {
	    const { scans, scansLast } = current.index.stats || {};
	    if (scans && scans > 0) {
	        const sameAsCurrent = history.filter(h => now - h.date > minDays * duration_1.oneDay).map(h => h.value.index.stats?.scans === scans ? { date: h.date, current, previous: h.value, report: h.report } : undefined).filter(utils_1.isNotUndefined);
	        const res = (0, utils_1.minBy)(sameAsCurrent, d => d.date);
	        if (res)
	            return res;
	    }
	    if (scansLast) {
	        const lastScan = new Date(scansLast).getTime();
	        if (now - lastScan > minDays * duration_1.oneDay) {
	            return { date: lastScan, current };
	        }
	    }
	    return undefined;
	}
	
	return indexUnused;
}

var primaryKeyMissing = {};

var hasRequiredPrimaryKeyMissing;

function requirePrimaryKeyMissing () {
	if (hasRequiredPrimaryKeyMissing) return primaryKeyMissing;
	hasRequiredPrimaryKeyMissing = 1;
	Object.defineProperty(primaryKeyMissing, "__esModule", { value: true });
	primaryKeyMissing.primaryKeyMissingRule = void 0;
	primaryKeyMissing.isPrimaryKeysMissing = isPrimaryKeysMissing;
	const utils_1 = requireOut$1();
	const database_1 = requireDatabase();
	const databaseUtils_1 = requireDatabaseUtils();
	const rule_1 = requireRule();
	/**
	 * Primary Keys are the default unique way to get a single row in a table.
	 * They are also often used on foreign keys to reference a single record.
	 * Having a primary key on every table is a common best practice.
	 */
	const ruleId = 'primary-key-missing';
	const ruleName = 'missing primary key';
	const ruleDescription = 'entities with no primary key (only tables)';
	const CustomRuleConf = rule_1.RuleConf.extend({
	    ignores: database_1.EntityId.array().optional(),
	}).strict().describe('PrimaryKeyMissingConf');
	primaryKeyMissing.primaryKeyMissingRule = {
	    id: ruleId,
	    name: ruleName,
	    description: ruleDescription,
	    conf: { level: rule_1.RuleLevel.enum.high },
	    zConf: CustomRuleConf,
	    analyze(conf, now, db, queries, history, reference) {
	        const refIgnores = reference.map(r => r.entity).filter(utils_1.isNotUndefined);
	        const ignores = refIgnores.concat(conf.ignores?.map(databaseUtils_1.entityRefFromId) || []);
	        return (db.entities || [])
	            .filter(isPrimaryKeysMissing)
	            .filter(e => !ignores.some(i => (0, databaseUtils_1.entityRefSame)(i, (0, databaseUtils_1.entityToRef)(e))))
	            .map(e => ({
	            ruleId,
	            ruleName,
	            ruleLevel: conf.level,
	            message: `Entity ${(0, databaseUtils_1.entityToId)(e)} has no primary key.`,
	            entity: (0, databaseUtils_1.entityToRef)(e)
	        }));
	    }
	};
	// same as frontend/src/PagesComponents/Organization_/Project_/Views/Modals/SchemaAnalysis/PrimaryKeyMissing.elm
	function isPrimaryKeysMissing(entity) {
	    return (entity.kind === undefined || entity.kind === database_1.EntityKind.enum.table) && entity.pk === undefined;
	}
	
	return primaryKeyMissing;
}

var primaryKeyNotBusiness = {};

var hasRequiredPrimaryKeyNotBusiness;

function requirePrimaryKeyNotBusiness () {
	if (hasRequiredPrimaryKeyNotBusiness) return primaryKeyNotBusiness;
	hasRequiredPrimaryKeyNotBusiness = 1;
	Object.defineProperty(primaryKeyNotBusiness, "__esModule", { value: true });
	primaryKeyNotBusiness.primaryKeyNotBusinessRule = void 0;
	primaryKeyNotBusiness.isPrimaryKeyTechnical = isPrimaryKeyTechnical;
	const utils_1 = requireOut$1();
	const database_1 = requireDatabase();
	const databaseUtils_1 = requireDatabaseUtils();
	const rule_1 = requireRule();
	/**
	 * Primary Keys are the default unique way to get a single row in a table.
	 * They are also often used on foreign keys to reference a single record.
	 * Having a primary key on every table is a common best practice.
	 */
	const ruleId = 'primary-key-not-business';
	const ruleName = 'business primary key forbidden';
	const ruleDescription = 'entities with primary key which don\'t end with id or has a relation';
	const CustomRuleConf = rule_1.RuleConf.extend({
	    ignores: database_1.EntityId.array().optional(),
	}).strict().describe('PrimaryKeyNotBusinessConf');
	primaryKeyNotBusiness.primaryKeyNotBusinessRule = {
	    id: ruleId,
	    name: ruleName,
	    description: ruleDescription,
	    conf: { level: rule_1.RuleLevel.enum.medium },
	    zConf: CustomRuleConf,
	    analyze(conf, now, db, queries, history, reference) {
	        const refIgnores = reference.map(r => r.entity).filter(utils_1.isNotUndefined);
	        const ignores = refIgnores.concat(conf.ignores?.map(databaseUtils_1.entityRefFromId) || []);
	        const relations = (0, utils_1.groupBy)(db.relations || [], r => (0, databaseUtils_1.entityRefToId)(r.src));
	        return (db.entities || [])
	            .filter(e => !isPrimaryKeyTechnical(e, relations[(0, databaseUtils_1.entityToId)(e)] || []))
	            .filter(e => !ignores.some(i => (0, databaseUtils_1.entityRefSame)(i, (0, databaseUtils_1.entityToRef)(e))))
	            .map(e => {
	            const { stats, extra, ...primaryKey } = e.pk || {};
	            return {
	                ruleId,
	                ruleName,
	                ruleLevel: conf.level,
	                message: `Entity ${(0, databaseUtils_1.entityToId)(e)} should have a technical primary key, current one is: (${e.pk?.attrs.map(databaseUtils_1.attributePathToId).join(', ')}).`,
	                entity: (0, databaseUtils_1.entityToRef)(e),
	                attribute: e.pk?.attrs?.[0],
	                extra: { primaryKey }
	            };
	        });
	    }
	};
	function isPrimaryKeyTechnical(entity, relations) {
	    if ((entity.kind === undefined || entity.kind === 'table') && !!entity.pk) {
	        // primary key attributes should either end with 'id' or have a relation, otherwise it's likely business
	        return entity.pk.attrs.every(pkAttr => {
	            const isAutoIncrement = false; // FIXME: compute this!
	            const endsWithId = pkAttr.slice(-1)[0].toLowerCase().endsWith('id');
	            const hasRelation = relations.find(r => r.src.attrs.find(a => (0, databaseUtils_1.attributePathToId)(a) === (0, databaseUtils_1.attributePathToId)(pkAttr)));
	            return !isAutoIncrement && (endsWithId || hasRelation);
	        });
	    }
	    else {
	        return true; // not table or no primary key
	    }
	}
	
	return primaryKeyNotBusiness;
}

var queryDegrading = {};

var hasRequiredQueryDegrading;

function requireQueryDegrading () {
	if (hasRequiredQueryDegrading) return queryDegrading;
	hasRequiredQueryDegrading = 1;
	Object.defineProperty(queryDegrading, "__esModule", { value: true });
	queryDegrading.queryDegradingRule = void 0;
	queryDegrading.getDegradingQuery = getDegradingQuery;
	const zod_1 = /*@__PURE__*/ requireLib();
	const utils_1 = requireOut$1();
	const common_1 = requireCommon();
	const databaseUtils_1 = requireDatabaseUtils();
	const date_1 = requireDate();
	const duration_1 = requireDuration();
	const percent_1 = requirePercent();
	const sql_1 = requireSql();
	const connector_1 = requireConnector();
	const rule_1 = requireRule();
	const ruleId = 'query-degrading';
	const ruleName = 'degrading query';
	const ruleDescription = 'queries with mean execution time growing over daily/monthly rate thresholds and executions and duration over the thresholds also';
	const CustomRuleConf = rule_1.RuleConf.extend({
	    ignores: connector_1.QueryId.array().optional(),
	    minExec: zod_1.z.number(),
	    minDuration: duration_1.Duration,
	    maxDegradation: common_1.Percent,
	    maxDegradationMonthly: common_1.Percent,
	    maxDegradationDaily: common_1.Percent,
	}).strict().describe('QueryDegradingConf');
	queryDegrading.queryDegradingRule = {
	    id: ruleId,
	    name: ruleName,
	    description: ruleDescription,
	    conf: { level: rule_1.RuleLevel.enum.high, minExec: 10, minDuration: 100, maxDegradation: 1, maxDegradationMonthly: 0.2, maxDegradationDaily: 0.1 },
	    zConf: CustomRuleConf,
	    analyze(conf, now, db, queries, history, reference) {
	        const refIgnores = reference.map(r => r.extra?.queryId).filter(utils_1.isNotUndefined);
	        const ignores = refIgnores.concat(conf.ignores || []);
	        const historyQueriesById = history.map(h => ({ report: h.report, date: h.date, queries: (0, utils_1.indexBy)(h.queries, q => q.id) }));
	        return queries
	            .filter(q => !ignores.some(i => i === q.id))
	            .map(query => {
	            const previous = historyQueriesById.map(h => {
	                const queryHist = h.queries[query.id];
	                return queryHist ? { report: h.report, date: h.date, query: queryHist } : undefined;
	            }).filter(utils_1.isNotUndefined);
	            return getDegradingQuery(now, query, previous, conf.minExec, conf.minDuration, conf.maxDegradation, conf.maxDegradationMonthly, conf.maxDegradationDaily);
	        })
	            .filter(utils_1.isNotUndefined)
	            .sort((a, b) => -(a.daily - b.daily))
	            .map(r => {
	            const entity = (0, sql_1.getMainEntity)(r.current.query);
	            const degradation = `${(0, percent_1.showPercent)(r.degradation)} (${(0, duration_1.showDuration)(r.previous.exec?.meanTime || 0)} to ${(0, duration_1.showDuration)(r.current.exec?.meanTime || 0)})`;
	            const rate = r.period > duration_1.oneMonth ? `${(0, percent_1.showPercent)(r.monthly)} monthly, ` : r.period > duration_1.oneDay ? `${(0, percent_1.showPercent)(r.daily)} daily, ` : '';
	            return (0, utils_1.removeUndefined)({
	                ruleId,
	                ruleName,
	                ruleLevel: conf.level,
	                message: `Query ${r.current.id}${entity ? ` on ${(0, databaseUtils_1.entityRefToId)(entity)}` : ''} degraded mean exec time by ${degradation} since ${(0, date_1.showDate)(r.date)} (${rate}${(0, sql_1.formatSql)(r.current.query)}).`,
	                entity,
	                extra: {
	                    queryId: r.current.id,
	                    query: r.current.query,
	                    degradationPercent: r.degradation,
	                    periodMillis: r.period,
	                    previousDate: new Date(r.date).toISOString(),
	                    previousReport: r.report,
	                    previousStats: queryStats(r.previous),
	                    currentStats: queryStats(r.current),
	                    entities: (0, sql_1.getEntities)(r.current.query)
	                }
	            });
	        });
	    }
	};
	function queryStats({ id, database, query, ...stats }) {
	    return stats;
	}
	function getDegradingQuery(now, query, history, minExec, minDuration, maxDegradation, maxDegradationMonthly, maxDegradationDaily) {
	    const meanTime = query.exec?.meanTime;
	    const count = query.exec?.count;
	    if (meanTime && meanTime > minDuration && count && count >= minExec) {
	        const meanTimeHistory = history.map(h => h.query.exec?.count && h.query.exec?.meanTime ? { report: h.report, date: h.date, query: h.query, count: h.query.exec.count, meanTime: h.query.exec.meanTime } : undefined).filter(utils_1.isNotUndefined);
	        const degradations = meanTimeHistory.filter(p => p.count >= minExec).map(previous => {
	            const period = now - previous.date;
	            const degradation = (0, percent_1.computePercent)(previous.meanTime, meanTime);
	            const daily = degradation / (period / duration_1.oneDay);
	            const monthly = degradation / (period / duration_1.oneMonth);
	            if (degradation >= maxDegradation || (period > duration_1.oneDay && daily >= maxDegradationDaily) || (period > duration_1.oneMonth && monthly >= maxDegradationMonthly)) {
	                return { report: previous.report, date: previous.date, previous: previous.query, current: query, degradation, period, daily, monthly };
	            }
	            else {
	                return undefined;
	            }
	        }).filter(utils_1.isNotUndefined);
	        // return maxBy(degradations, d => d.since) // earliest degradation
	        return (0, utils_1.maxBy)(degradations, d => d.daily); // worst degradation
	    }
	    return undefined;
	}
	
	return queryDegrading;
}

var queryExpensive = {};

var hasRequiredQueryExpensive;

function requireQueryExpensive () {
	if (hasRequiredQueryExpensive) return queryExpensive;
	hasRequiredQueryExpensive = 1;
	Object.defineProperty(queryExpensive, "__esModule", { value: true });
	queryExpensive.queryExpensiveRule = void 0;
	const utils_1 = requireOut$1();
	const databaseUtils_1 = requireDatabaseUtils();
	const duration_1 = requireDuration();
	const sql_1 = requireSql();
	const connector_1 = requireConnector();
	const rule_1 = requireRule();
	const ruleId = 'query-expensive';
	const ruleName = 'expensive query';
	const ruleDescription = 'top 20 queries by total execution time';
	const CustomRuleConf = rule_1.RuleConf.extend({
	    ignores: connector_1.QueryId.array().optional(),
	}).strict().describe('QueryExpensiveConf');
	queryExpensive.queryExpensiveRule = {
	    id: ruleId,
	    name: ruleName,
	    description: ruleDescription,
	    conf: { level: rule_1.RuleLevel.enum.hint },
	    zConf: CustomRuleConf,
	    analyze(conf, now, db, queries, history, reference) {
	        const refIgnores = reference.map(r => r.extra?.queryId).filter(utils_1.isNotUndefined);
	        const ignores = refIgnores.concat(conf.ignores || []);
	        return queries
	            .filter(q => !ignores.some(i => i === q.id))
	            .sort((a, b) => -((a.exec?.sumTime || 0) - (b.exec?.sumTime || 0)))
	            .slice(0, 20)
	            .map(q => {
	            const entity = (0, sql_1.getMainEntity)(q.query);
	            const { id, database, query, ...stats } = q;
	            return (0, utils_1.removeUndefined)({
	                ruleId,
	                ruleName,
	                ruleLevel: conf.level,
	                message: `Query ${q.id}${entity ? ` on ${(0, databaseUtils_1.entityRefToId)(entity)}` : ''} is one of the most expensive, cumulated ${(0, duration_1.showDuration)(q.exec?.sumTime || 0)} exec time in ${(0, utils_1.pluralize)(q.exec?.count || 0, 'execution')} (${(0, sql_1.formatSql)(q.query)})`,
	                entity,
	                extra: { queryId: id, query, stats, entities: (0, sql_1.getEntities)(q.query) }
	            });
	        });
	    }
	};
	
	return queryExpensive;
}

var queryHighVariation = {};

var hasRequiredQueryHighVariation;

function requireQueryHighVariation () {
	if (hasRequiredQueryHighVariation) return queryHighVariation;
	hasRequiredQueryHighVariation = 1;
	Object.defineProperty(queryHighVariation, "__esModule", { value: true });
	queryHighVariation.queryHighVariationRule = void 0;
	const zod_1 = /*@__PURE__*/ requireLib();
	const utils_1 = requireOut$1();
	const databaseUtils_1 = requireDatabaseUtils();
	const duration_1 = requireDuration();
	const sql_1 = requireSql();
	const connector_1 = requireConnector();
	const rule_1 = requireRule();
	const ruleId = 'query-high-variation';
	const ruleName = 'query with high variation';
	const ruleDescription = 'top 20 queries by execution time standard deviation when execution count is over the threshold';
	const CustomRuleConf = rule_1.RuleConf.extend({
	    ignores: connector_1.QueryId.array().optional(),
	    minExec: zod_1.z.number().optional(), // optional for retro-compatibility
	}).strict().describe('QueryHighVariationConf');
	queryHighVariation.queryHighVariationRule = {
	    id: ruleId,
	    name: ruleName,
	    description: ruleDescription,
	    conf: { level: rule_1.RuleLevel.enum.hint, minExec: 10 },
	    zConf: CustomRuleConf,
	    analyze(conf, now, db, queries, history, reference) {
	        const refIgnores = reference.map(r => r.extra?.queryId).filter(utils_1.isNotUndefined);
	        const ignores = refIgnores.concat(conf.ignores || []);
	        return queries
	            .filter(q => !ignores.some(i => i === q.id))
	            .filter(q => (q.exec?.count || 0) >= (conf.minExec || 0))
	            .sort((a, b) => -((a.exec?.sdTime || 0) - (b.exec?.sdTime || 0)))
	            .slice(0, 20)
	            .map(q => {
	            const entity = (0, sql_1.getMainEntity)(q.query);
	            const sd = (0, duration_1.showDuration)(q.exec?.sdTime || 0);
	            const min = (0, duration_1.showDuration)(q.exec?.minTime || 0);
	            const max = (0, duration_1.showDuration)(q.exec?.maxTime || 0);
	            const { id, database, query, ...stats } = q;
	            return (0, utils_1.removeUndefined)({
	                ruleId,
	                ruleName,
	                ruleLevel: conf.level,
	                message: `Query ${q.id}${entity ? ` on ${(0, databaseUtils_1.entityRefToId)(entity)}` : ''} has high variation, with ${sd} standard deviation and exec time ranging from ${min} to ${max} (${(0, sql_1.formatSql)(q.query)})`,
	                entity,
	                extra: { queryId: id, query, stats, entities: (0, sql_1.getEntities)(q.query) }
	            });
	        });
	    }
	};
	
	return queryHighVariation;
}

var queryTooSlow = {};

var hasRequiredQueryTooSlow;

function requireQueryTooSlow () {
	if (hasRequiredQueryTooSlow) return queryTooSlow;
	hasRequiredQueryTooSlow = 1;
	Object.defineProperty(queryTooSlow, "__esModule", { value: true });
	queryTooSlow.queryTooSlowRule = void 0;
	queryTooSlow.isQueryTooSlow = isQueryTooSlow;
	const zod_1 = /*@__PURE__*/ requireLib();
	const utils_1 = requireOut$1();
	const databaseUtils_1 = requireDatabaseUtils();
	const duration_1 = requireDuration();
	const sql_1 = requireSql();
	const connector_1 = requireConnector();
	const rule_1 = requireRule();
	const ruleId = 'query-too-slow';
	const ruleName = 'too slow query';
	const ruleDescription = 'queries with mean execution time over the threshold';
	const CustomRuleConf = rule_1.RuleConf.extend({
	    ignores: connector_1.QueryId.array().optional(),
	    maxMs: zod_1.z.number(),
	}).strict().describe('QueryTooSlowConf');
	queryTooSlow.queryTooSlowRule = {
	    id: ruleId,
	    name: ruleName,
	    description: ruleDescription,
	    conf: { level: rule_1.RuleLevel.enum.high, maxMs: 1000 },
	    zConf: CustomRuleConf,
	    analyze(conf, now, db, queries, history, reference) {
	        const refIgnores = reference.map(r => r.extra?.queryId).filter(utils_1.isNotUndefined);
	        const ignores = refIgnores.concat(conf.ignores || []);
	        return queries
	            .filter(q => isQueryTooSlow(q, conf.maxMs))
	            .filter(q => !ignores.some(i => i === q.id))
	            .sort((a, b) => -((a.exec?.meanTime || 0) - (b.exec?.meanTime || 0)))
	            .map(q => {
	            const entity = (0, sql_1.getMainEntity)(q.query);
	            const { id, database, query, ...stats } = q;
	            return (0, utils_1.removeUndefined)({
	                ruleId,
	                ruleName,
	                ruleLevel: conf.level,
	                message: `Query ${q.id}${entity ? ` on ${(0, databaseUtils_1.entityRefToId)(entity)}` : ''} is too slow (${(0, duration_1.showDuration)(q.exec?.meanTime || 0)}, ${(0, sql_1.formatSql)(q.query)}).`,
	                entity,
	                extra: { queryId: id, query, stats, entities: (0, sql_1.getEntities)(q.query) }
	            });
	        });
	    }
	};
	function isQueryTooSlow(query, maxMs) {
	    return (query.exec?.meanTime || 0) > maxMs;
	}
	
	return queryTooSlow;
}

var relationMisaligned = {};

var hasRequiredRelationMisaligned;

function requireRelationMisaligned () {
	if (hasRequiredRelationMisaligned) return relationMisaligned;
	hasRequiredRelationMisaligned = 1;
	Object.defineProperty(relationMisaligned, "__esModule", { value: true });
	relationMisaligned.relationMisalignedRule = void 0;
	relationMisaligned.getMisalignedRelation = getMisalignedRelation;
	const utils_1 = requireOut$1();
	const database_1 = requireDatabase();
	const databaseUtils_1 = requireDatabaseUtils();
	const rule_1 = requireRule();
	/**
	 * Relations are made to link entities by referencing a target record from a source record, mostly using the primary key.
	 * As columns on both side of the relation store the same thing, they should also have the same type.
	 * Sometimes, subtle differences (like varchar length) may not be an issue, it's far from ideal and may become one at some point.
	 */
	const ruleId = 'relation-misaligned-type';
	const ruleName = 'misaligned relation';
	const ruleDescription = 'relations with different attribute type on each side';
	const CustomRuleConf = rule_1.RuleConf.extend({
	    ignores: database_1.RelationId.array().optional()
	}).strict().describe('RelationMisalignedConf');
	relationMisaligned.relationMisalignedRule = {
	    id: ruleId,
	    name: ruleName,
	    description: ruleDescription,
	    conf: { level: rule_1.RuleLevel.enum.high },
	    zConf: CustomRuleConf,
	    analyze(conf, now, db, queries, history, reference) {
	        const refIgnores = reference.map(r => r.extra?.relation ? (0, databaseUtils_1.relationToRef)(r.extra?.relation) : undefined).filter(utils_1.isNotUndefined);
	        const ignores = refIgnores.concat(conf.ignores?.map(databaseUtils_1.relationRefFromId) || []);
	        const entities = (0, utils_1.indexBy)(db.entities || [], databaseUtils_1.entityToId);
	        return (db.relations || [])
	            .map(r => getMisalignedRelation(r, entities))
	            .filter(utils_1.isNotUndefined)
	            .filter(v => !ignores.some(i => (0, databaseUtils_1.relationRefSame)(i, (0, databaseUtils_1.relationToRef)(v.relation))))
	            .map(violation => {
	            const { extra, ...relation } = violation.relation;
	            return {
	                ruleId,
	                ruleName,
	                ruleLevel: conf.level,
	                message: `Relation ${relationName(violation.relation)} link attributes different types: ${violation.misalignedTypes.map(formatMisalignedType).join(', ')}`,
	                entity: (0, databaseUtils_1.relationLinkToEntityRef)(violation.relation.src),
	                attribute: violation.relation.src.attrs[0],
	                extra: { relation, misalignedTypes: violation.misalignedTypes }
	            };
	        });
	    }
	};
	const relationName = (r) => r.name || (0, databaseUtils_1.relationToId)(r);
	const formatMisalignedType = (t) => `${(0, databaseUtils_1.attributeRefToId)(t.src)}: ${t.srcType} != ${(0, databaseUtils_1.attributeRefToId)(t.ref)}: ${t.refType}`;
	// same as frontend/src/PagesComponents/Organization_/Project_/Views/Modals/SchemaAnalysis/InconsistentTypeOnRelations.elm
	function getMisalignedRelation(relation, entities) {
	    const src = entities[(0, databaseUtils_1.entityRefToId)(relation.src)];
	    const ref = entities[(0, databaseUtils_1.entityRefToId)(relation.ref)];
	    if (!src && !ref) {
	        return undefined;
	    }
	    const attrs = (0, utils_1.zip)(relation.src.attrs, relation.ref.attrs).map(([srcAttr, refAttr]) => ({
	        src: srcAttr,
	        srcAttr: (0, databaseUtils_1.getAttribute)(src.attrs, srcAttr),
	        ref: refAttr,
	        refAttr: (0, databaseUtils_1.getAttribute)(ref.attrs, refAttr),
	    }));
	    if (attrs.some(a => !a.srcAttr || !a.refAttr)) {
	        return undefined;
	    }
	    const misalignedTypes = attrs.flatMap(attr => attr.srcAttr && attr.refAttr && attr.srcAttr.type !== attr.refAttr.type ? [{
	            src: { ...(0, databaseUtils_1.relationLinkToEntityRef)(relation.src), attribute: attr.src },
	            srcType: attr.srcAttr?.type,
	            ref: { ...(0, databaseUtils_1.relationLinkToEntityRef)(relation.ref), attribute: attr.ref },
	            refType: attr.refAttr?.type,
	        }] : []);
	    return misalignedTypes.length > 0 ? { relation, misalignedTypes } : undefined;
	}
	
	return relationMisaligned;
}

var relationMissAttribute = {};

var hasRequiredRelationMissAttribute;

function requireRelationMissAttribute () {
	if (hasRequiredRelationMissAttribute) return relationMissAttribute;
	hasRequiredRelationMissAttribute = 1;
	Object.defineProperty(relationMissAttribute, "__esModule", { value: true });
	relationMissAttribute.relationMissAttributeRule = void 0;
	relationMissAttribute.getMissingAttributeRelations = getMissingAttributeRelations;
	const utils_1 = requireOut$1();
	const database_1 = requireDatabase();
	const databaseUtils_1 = requireDatabaseUtils();
	const rule_1 = requireRule();
	const ruleId = 'relation-miss-attribute';
	const ruleName = 'attribute not found in relation';
	const ruleDescription = 'relations with an attribute not found in the entities';
	const CustomRuleConf = rule_1.RuleConf.extend({
	    ignores: database_1.AttributeId.array().optional(),
	}).strict().describe('RelationMissAttributeConf');
	relationMissAttribute.relationMissAttributeRule = {
	    id: ruleId,
	    name: ruleName,
	    description: ruleDescription,
	    conf: { level: rule_1.RuleLevel.enum.high },
	    zConf: CustomRuleConf,
	    analyze(conf, now, db, queries, history, reference) {
	        const relIgnores = reference.map(r => r.extra?.relation ? (0, databaseUtils_1.relationToRef)(r.extra?.relation) : undefined).filter(utils_1.isNotUndefined);
	        const attrIgnores = conf.ignores?.map(databaseUtils_1.attributeRefFromId) || [];
	        const entities = (0, utils_1.indexBy)(db.entities || [], databaseUtils_1.entityToId);
	        return (db.relations || [])
	            .map(r => getMissingAttributeRelations(r, entities))
	            .filter(utils_1.isNotUndefined)
	            .filter(v => !relIgnores.some(i => (0, databaseUtils_1.relationRefSame)(i, (0, databaseUtils_1.relationToRef)(v.relation))))
	            .map(v => ({ ...v, missingAttrs: v?.missing?.filter(a => !attrIgnores.some(i => (0, databaseUtils_1.attributeRefSame)(i, a))) }))
	            .filter(v => v.missingAttrs.length > 0)
	            .map(violation => {
	            const { attribute, ...entity } = violation.missingAttrs[0];
	            return {
	                ruleId,
	                ruleName,
	                ruleLevel: conf.level,
	                message: `Relation ${relationName(violation.relation)}, not found attributes: ${violation.missingAttrs.map(databaseUtils_1.attributeRefToId).join(', ')}`,
	                entity,
	                attribute,
	                extra: violation
	            };
	        });
	    }
	};
	const relationName = (r) => r.name || (0, databaseUtils_1.relationToId)(r);
	function getMissingAttributeRelations(relation, entities) {
	    const src = entities[(0, databaseUtils_1.entityRefToId)(relation.src)];
	    const ref = entities[(0, databaseUtils_1.entityRefToId)(relation.ref)];
	    if (!src || !ref) {
	        return undefined;
	    }
	    const attrs = (0, utils_1.zip)(relation.src.attrs, relation.ref.attrs).map(([srcAttr, refAttr]) => ({
	        src: srcAttr,
	        srcAttr: (0, databaseUtils_1.getAttribute)(src.attrs, srcAttr),
	        ref: refAttr,
	        refAttr: (0, databaseUtils_1.getAttribute)(ref.attrs, refAttr),
	    }));
	    const missing = attrs.flatMap(attr => [
	        attr.srcAttr ? undefined : { ...(0, databaseUtils_1.relationLinkToEntityRef)(relation.src), attribute: attr.src },
	        attr.refAttr ? undefined : { ...(0, databaseUtils_1.relationLinkToEntityRef)(relation.ref), attribute: attr.ref },
	    ].filter(utils_1.isNotUndefined));
	    return missing.length > 0 ? { relation, missing } : undefined;
	}
	
	return relationMissAttribute;
}

var relationMissEntity = {};

var hasRequiredRelationMissEntity;

function requireRelationMissEntity () {
	if (hasRequiredRelationMissEntity) return relationMissEntity;
	hasRequiredRelationMissEntity = 1;
	Object.defineProperty(relationMissEntity, "__esModule", { value: true });
	relationMissEntity.relationMissEntityRule = void 0;
	relationMissEntity.getMissingEntityRelations = getMissingEntityRelations;
	const utils_1 = requireOut$1();
	const database_1 = requireDatabase();
	const databaseUtils_1 = requireDatabaseUtils();
	const rule_1 = requireRule();
	const ruleId = 'relation-miss-entity';
	const ruleName = 'entity not found in relation';
	const ruleDescription = 'relations with an entity not found in the database';
	const CustomRuleConf = rule_1.RuleConf.extend({
	    ignores: database_1.EntityId.array().optional(),
	}).strict().describe('RelationMissEntityConf');
	relationMissEntity.relationMissEntityRule = {
	    id: ruleId,
	    name: ruleName,
	    description: ruleDescription,
	    conf: { level: rule_1.RuleLevel.enum.high },
	    zConf: CustomRuleConf,
	    analyze(conf, now, db, queries, history, reference) {
	        const relIgnores = reference.map(r => r.extra?.relation ? (0, databaseUtils_1.relationToRef)(r.extra?.relation) : undefined).filter(utils_1.isNotUndefined);
	        const entIgnores = conf.ignores?.map(databaseUtils_1.entityRefFromId) || [];
	        const entities = (0, utils_1.indexBy)(db.entities || [], databaseUtils_1.entityToId);
	        return (db.relations || [])
	            .map(r => getMissingEntityRelations(r, entities))
	            .filter(utils_1.isNotUndefined)
	            .filter(v => !relIgnores.some(i => (0, databaseUtils_1.relationRefSame)(i, (0, databaseUtils_1.relationToRef)(v.relation))))
	            .map(v => ({ ...v, missingEntities: v?.missing?.filter(e => !entIgnores.some(i => (0, databaseUtils_1.entityRefSame)(i, e))) }))
	            .filter(v => v.missingEntities.length > 0)
	            .map(violation => ({
	            ruleId,
	            ruleName,
	            ruleLevel: conf.level,
	            message: `Relation ${relationName(violation.relation)}, not found entities: ${violation.missingEntities.map(databaseUtils_1.entityRefToId).join(', ')}`,
	            entity: violation.missingEntities[0],
	            extra: violation
	        }));
	    }
	};
	const relationName = (r) => r.name || (0, databaseUtils_1.relationToId)(r);
	function getMissingEntityRelations(relation, entities) {
	    const src = entities[(0, databaseUtils_1.entityRefToId)(relation.src)];
	    const ref = entities[(0, databaseUtils_1.entityRefToId)(relation.ref)];
	    const missing = [src ? undefined : (0, databaseUtils_1.relationLinkToEntityRef)(relation.src), ref ? undefined : (0, databaseUtils_1.relationLinkToEntityRef)(relation.ref)].filter(utils_1.isNotUndefined);
	    return missing.length > 0 ? { relation, missing } : undefined;
	}
	
	return relationMissEntity;
}

var relationMissing = {};

var hasRequiredRelationMissing;

function requireRelationMissing () {
	if (hasRequiredRelationMissing) return relationMissing;
	hasRequiredRelationMissing = 1;
	Object.defineProperty(relationMissing, "__esModule", { value: true });
	relationMissing.relationMissingRule = void 0;
	relationMissing.getMissingRelations = getMissingRelations;
	const utils_1 = requireOut$1();
	const database_1 = requireDatabase();
	const databaseUtils_1 = requireDatabaseUtils();
	const rule_1 = requireRule();
	/**
	 * If relations are not defined as foreign key, it could be great to identify them
	 */
	const ruleId = 'relation-missing';
	const ruleName = 'missing relation';
	const ruleDescription = 'relations not present in the database but could exist';
	const CustomRuleConf = rule_1.RuleConf.extend({
	    ignores: database_1.RelationId.array().optional(),
	}).strict().describe('RelationMissingConf');
	relationMissing.relationMissingRule = {
	    id: ruleId,
	    name: ruleName,
	    description: ruleDescription,
	    conf: { level: rule_1.RuleLevel.enum.medium },
	    zConf: CustomRuleConf,
	    analyze(conf, now, db, queries, history, reference) {
	        const refIgnores = reference.map(r => r.extra?.relation ? (0, databaseUtils_1.relationToRef)(r.extra?.relation) : undefined).filter(utils_1.isNotUndefined);
	        const ignores = refIgnores.concat(conf.ignores?.map(databaseUtils_1.relationRefFromId) || []);
	        return getMissingRelations(db.entities || [], db.relations || [])
	            .filter(r => !ignores.some(i => (0, databaseUtils_1.relationRefSame)(i, (0, databaseUtils_1.relationToRef)(r))))
	            .map(r => ({
	            ruleId,
	            ruleName,
	            ruleLevel: conf.level,
	            message: `Create a relation from ${(0, databaseUtils_1.attributesRefToId)({ ...r.src, attrs: r.src.attrs })} to ${(0, databaseUtils_1.attributesRefToId)({ ...r.ref, attrs: r.ref.attrs })}.`,
	            entity: (0, databaseUtils_1.relationLinkToEntityRef)(r.src),
	            attribute: r.src.attrs[0],
	            extra: { relation: r }
	        }));
	    }
	};
	// same as frontend/src/PagesComponents/Organization_/Project_/Views/Modals/SchemaAnalysis/RelationMissing.elm
	function getMissingRelations(entities, relations) {
	    const tableEntities = entities.filter(e => e.kind === undefined || e.kind === 'table'); // don't suggest relation to a view
	    const entitiesByName = (0, utils_1.groupBy)(tableEntities, e => (0, utils_1.splitWords)(e.name).map(utils_1.singular).join('_'));
	    const entitiesByPrefixName = (0, utils_1.groupBy)(tableEntities.filter(e => (0, utils_1.splitWords)(e.name).length > 1), e => (0, utils_1.splitWords)(e.name).slice(1).map(utils_1.singular).join('_'));
	    const entitiesByPrefixName2 = (0, utils_1.groupBy)(tableEntities.filter(e => (0, utils_1.splitWords)(e.name).length > 2), e => (0, utils_1.splitWords)(e.name).slice(2).map(utils_1.singular).join('_'));
	    const relationsBySrc = (0, utils_1.groupBy)(relations, r => (0, databaseUtils_1.entityRefToId)(r.src));
	    return tableEntities.flatMap(entity => {
	        return (0, databaseUtils_1.flattenAttributes)(entity.attrs).flatMap(({ path, ...attr }) => {
	            const attrWords = (0, utils_1.splitWords)(attr.name).map(utils_1.singular);
	            const lastWord = attrWords[attrWords.length - 1];
	            let results = [];
	            if (lastWord === 'id' && attrWords.length > 1) {
	                return guessAllRelationTargetsWithPoly(entitiesByName, entitiesByPrefixName, entitiesByPrefixName2, entity, path, attrWords);
	            }
	            else if (lastWord.endsWith('id') && lastWord.length > 2) {
	                const words = [...attrWords.slice(0, -1), lastWord.slice(0, -2), lastWord.slice(-2)];
	                return guessAllRelationTargetsWithPoly(entitiesByName, entitiesByPrefixName, entitiesByPrefixName2, entity, path, words);
	            }
	            else if (lastWord === 'by') {
	                const entityRef = (0, databaseUtils_1.entityToRef)(entity);
	                let targets = guessRelationTargets(entitiesByName, entityRef, ['user', 'id']);
	                targets = targets.length === 0 ? guessRelationTargets(entitiesByName, entityRef, ['account', 'id']) : targets;
	                results = targets.map(ref => ({ src: { ...entityRef, attrs: [path] }, ref: { ...ref.entity, attrs: [ref.attr] }, origin: 'infer-name' }));
	            }
	            return results;
	        });
	    }).filter(r => !loopingRelation(r)).filter(r => !relationAlreadyExist(relationsBySrc, r));
	}
	function guessAllRelationTargetsWithPoly(entitiesByName, entitiesByPrefixName, entitiesByPrefixName2, entity, attrPath, attrWords) {
	    const poly = getPolymorphicColumn(entity, attrPath);
	    const entityRef = (0, databaseUtils_1.entityToRef)(entity);
	    if (poly) {
	        return poly.values.flatMap(value => {
	            const words = (0, utils_1.splitWords)((0, databaseUtils_1.attributeValueToString)(value)).map(utils_1.singular).concat(['id']);
	            return guessRelationTargetsWithPrefixes(entitiesByName, entitiesByPrefixName, entitiesByPrefixName2, entityRef, attrPath, words)
	                .map(r => ({ ...r, polymorphic: { attribute: poly.path, value } }));
	        });
	    }
	    else {
	        return guessRelationTargetsWithPrefixes(entitiesByName, entitiesByPrefixName, entitiesByPrefixName2, entityRef, attrPath, attrWords);
	    }
	}
	function guessRelationTargetsWithPrefixes(entitiesByName, entitiesByPrefixName, entitiesByPrefixName2, entityRef, attrPath, attrWords) {
	    let targets = guessRelationTargets(entitiesByName, entityRef, attrWords);
	    targets = targets.length === 0 ? guessRelationTargets(entitiesByPrefixName, entityRef, attrWords) : targets;
	    targets = targets.length === 0 ? guessRelationTargets(entitiesByPrefixName2, entityRef, attrWords) : targets;
	    return targets.map(ref => ({ src: { ...entityRef, attrs: [attrPath] }, ref: { ...ref.entity, attrs: [ref.attr] }, origin: 'infer-name' }));
	}
	function guessRelationTargets(entitiesByName, ref, attrWords) {
	    return (entitiesByName[attrWords.slice(0, -1).join('_')] || []).concat(entitiesByName[attrWords.slice(1, -1).join('_')] || [], // remove hypothetical prefixes (up to 3)
	    entitiesByName[attrWords.slice(2, -1).join('_')] || [], entitiesByName[attrWords.slice(3, -1).join('_')] || [], entitiesByName[attrWords.slice(0, -2).join('_')] || [], // remove hypothetical suffixes (up to 3)
	    entitiesByName[attrWords.slice(0, -3).join('_')] || [], entitiesByName[attrWords.slice(0, -4).join('_')] || [], entitiesByName[attrWords.slice(1, -2).join('_')] || [], // remove hypothetical prefixes & suffixes (up to 2)
	    entitiesByName[attrWords.slice(2, -2).join('_')] || [], entitiesByName[attrWords.slice(1, -3).join('_')] || [], entitiesByName[attrWords.slice(2, -3).join('_')] || []).sort((a, b) => namespaceScore(a, ref) - namespaceScore(b, ref)).flatMap((entity) => {
	        const attrName = attrWords.slice(-1)[0];
	        const attr = entity.attrs?.find(a => a.name === attrName) || entity.attrs?.find(a => {
	            const words = a.name.endsWith('id') ? (0, utils_1.splitWords)(a.name.slice(0, -2)).concat(['id']) : (0, utils_1.splitWords)(a.name);
	            return words.join('_') === attrWords.slice(words.length * -1).join('_');
	        });
	        return attr ? [{ entity: (0, databaseUtils_1.entityToRef)(entity), attr: [attr.name] }] : [];
	    });
	}
	// lower is nearer
	const namespaceScore = (e, ref) => (e.schema !== ref.schema ? 1 : 0) + (e.catalog !== ref.catalog ? 10 : 0) + (e.database !== ref.database ? 100 : 0);
	function getPolymorphicColumn(entity, attribute) {
	    const suffixes = ['type', 'kind', 'class'];
	    const parent = attribute.slice(0, -1);
	    const last = attribute[attribute.length - 1];
	    const prefix = last.endsWith('ids') ? last.slice(0, -3) : (last.endsWith('id') ? last.slice(0, -2) : last);
	    const attr = (0, databaseUtils_1.getPeerAttributes)(entity.attrs, parent).find(a => suffixes.some(s => a.name.toLowerCase() === `${prefix}${s}`.toLowerCase()));
	    return attr?.stats?.distinctValues ? { path: parent.concat([attr.name]), values: attr.stats.distinctValues } : undefined;
	}
	function loopingRelation(r) {
	    return (0, databaseUtils_1.entityRefToId)(r.src) === (0, databaseUtils_1.entityRefToId)(r.ref) && r.src.attrs.map(databaseUtils_1.attributePathToId).join(',') === r.ref.attrs.map(databaseUtils_1.attributePathToId).join(',');
	}
	function relationAlreadyExist(relationsBySrc, relation) {
	    const relationRefEntity = (0, databaseUtils_1.entityRefToId)(relation.ref);
	    const relationSrcAttrs = relation.src.attrs.map(databaseUtils_1.attributePathToId).join(',');
	    const relationRefAttrs = relation.ref.attrs.map(databaseUtils_1.attributePathToId).join(',');
	    const sameRelations = (relationsBySrc[(0, databaseUtils_1.entityRefToId)(relation.src)] || []) // from same entity
	        .filter(r => (0, databaseUtils_1.entityRefToId)(r.ref) === relationRefEntity) // to same entity
	        .filter(r => r.src.attrs.map(databaseUtils_1.attributePathToId).join(',') === relationSrcAttrs) // from same attributes
	        .filter(r => r.ref.attrs.map(databaseUtils_1.attributePathToId).join(',') === relationRefAttrs); // to same attributes
	    return sameRelations.length > 0;
	}
	
	return relationMissing;
}

var hasRequiredAnalyze;

function requireAnalyze () {
	if (hasRequiredAnalyze) return analyze;
	hasRequiredAnalyze = 1;
	(function (exports) {
		var __createBinding = (analyze && analyze.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __exportStar = (analyze && analyze.__exportStar) || function(m, exports) {
		    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.RulesConf = exports.analyzeRules = void 0;
		exports.analyzeDatabase = analyzeDatabase;
		exports.scoreDatabase = scoreDatabase;
		const zod_1 = /*@__PURE__*/ requireLib();
		const utils_1 = requireOut$1();
		const zod_2 = requireZod();
		const databaseUtils_1 = requireDatabaseUtils();
		const connector_1 = requireConnector();
		const rule_1 = requireRule();
		const attributeEmpty_1 = requireAttributeEmpty();
		const attributeNameInconsistent_1 = requireAttributeNameInconsistent();
		const attributeTypeBad_1 = requireAttributeTypeBad();
		const attributeTypeInconsistent_1 = requireAttributeTypeInconsistent();
		const entityEmpty_1 = requireEntityEmpty();
		const entityGrowFast_1 = requireEntityGrowFast();
		const entityIndexNone_1 = requireEntityIndexNone();
		const entityIndexTooHeavy_1 = requireEntityIndexTooHeavy();
		const entityIndexTooMany_1 = requireEntityIndexTooMany();
		const entityNameInconsistent_1 = requireEntityNameInconsistent();
		const entityNotClean_1 = requireEntityNotClean();
		const entityTooLarge_1 = requireEntityTooLarge();
		const entityUnused_1 = requireEntityUnused();
		const indexDuplicated_1 = requireIndexDuplicated();
		const indexGrowFast_1 = requireIndexGrowFast();
		const indexOnRelation_1 = requireIndexOnRelation();
		const indexUnused_1 = requireIndexUnused();
		const primaryKeyMissing_1 = requirePrimaryKeyMissing();
		const primaryKeyNotBusiness_1 = requirePrimaryKeyNotBusiness();
		const queryDegrading_1 = requireQueryDegrading();
		const queryExpensive_1 = requireQueryExpensive();
		const queryHighVariation_1 = requireQueryHighVariation();
		const queryTooSlow_1 = requireQueryTooSlow();
		const relationMisaligned_1 = requireRelationMisaligned();
		const relationMissAttribute_1 = requireRelationMissAttribute();
		const relationMissEntity_1 = requireRelationMissEntity();
		const relationMissing_1 = requireRelationMissing();
		__exportStar(requireRule(), exports);
		exports.analyzeRules = [
		    // hint rules
		    attributeTypeInconsistent_1.attributeTypeInconsistentRule,
		    queryExpensive_1.queryExpensiveRule,
		    queryHighVariation_1.queryHighVariationRule,
		    // low rules
		    entityEmpty_1.entityEmptyRule,
		    attributeEmpty_1.attributeEmptyRule,
		    entityNameInconsistent_1.entityNameInconsistentRule,
		    attributeNameInconsistent_1.attributeNameInconsistentRule,
		    // medium rules
		    entityUnused_1.entityUnusedRule,
		    indexUnused_1.indexUnusedRule,
		    attributeTypeBad_1.attributeTypeBadRule,
		    entityGrowFast_1.entityGrowFastRule,
		    indexGrowFast_1.indexGrowFastRule,
		    entityTooLarge_1.entityTooLargeRule,
		    entityIndexTooMany_1.entityIndexTooManyRule,
		    entityIndexTooHeavy_1.entityIndexTooHeavyRule,
		    primaryKeyNotBusiness_1.primaryKeyNotBusinessRule,
		    indexOnRelation_1.indexOnRelationRule,
		    relationMissing_1.relationMissingRule,
		    // high rules
		    indexDuplicated_1.indexDuplicatedRule,
		    queryTooSlow_1.queryTooSlowRule,
		    queryDegrading_1.queryDegradingRule,
		    entityNotClean_1.entityNotCleanRule,
		    primaryKeyMissing_1.primaryKeyMissingRule,
		    entityIndexNone_1.entityIndexNoneRule,
		    relationMisaligned_1.relationMisalignedRule,
		    relationMissAttribute_1.relationMissAttributeRule,
		    relationMissEntity_1.relationMissEntityRule,
		];
		exports.RulesConf = zod_1.z.object({
		    database: connector_1.ConnectorConfOpts.merge(connector_1.ConnectorScopeOpts).merge(connector_1.ConnectorSchemaDataOpts).strict().optional(),
		    rules: zod_1.z.object(Object.fromEntries(exports.analyzeRules.map(r => [r.id, r.zConf.optional()]))).strict().optional(),
		}).strict().describe('RulesConf');
		// TODO: split rules by kind? (schema, query, data...)
		function analyzeDatabase(conf, now, db, queries, history, reference, ruleNames) {
		    // TODO: use uuid or bigint for single primary key, not int or varchar ones
		    // TODO: use uuidv7 for sorted uuids, not v4
		    // TODO: uuids not stored as CHAR(36) => field ending with `id` and with type CHAR(36) => suggest type `uuid`/`BINARY(16)` instead (depend on db)
		    // TODO: warn on queries with ORDER BY RAND()
		    // TODO: constraints should be deferrable (pk, fk, unique)
		    // TODO: tables/columns with incrementing names
		    // TODO: columns with no (or very few) null but no NOT NULL constraint => suggest add NOT NULL constraint
		    // TODO: columns with all (or mostly) different values but no UNIQUE constraint => suggest to add UNIQUE constraint
		    // TODO: columns with all (or mostly) null values => suggest to remove (hint)
		    // TODO: columns with mostly the same data as an other one (common values & histogram) => is duplicate? suggest to remove? (hint)
		    // TODO: queries not using indexes
		    // TODO: JSON columns with different schemas (% of similarity between schemas)
		    // TODO: sequence/auto_increment exhaustion
		    // TODO: use varchar over char (https://youtu.be/ifEpT5STEU0?si=fcLBuwrgluG9crwt&t=90)
		    // TODO: auto_explain: index creation (https://pganalyze.com/docs/explain/setup/self_managed/01_auto_explain_check)
		    // TODO: missing relations from JOIN clauses
		    // TODO: columns frequently used on JOIN, WHERE and ORDER BY clauses should have an index
		    // TODO: monitor index fragmentation
		    // TODO: monitor replication lag
		    // TODO: alert on deadlocks
		    // TODO: suggest partial indexes: when there is often IS NULL or IS NOT NULL on query where clauses
		    // TODO: detect n+1 queries?
		    // TODO: blocking queries (> 1 min)
		    // TODO: special case: soft delete => if deleted_at column, suggest index on `deleted_at IS NULL`
		    // TODO: if index on column with many nulls: suggest two indexes: one with values where not null and one is IS NULL (ex: `btree (org_id)` => `btree (org_id) WHERE org_id IS NOT NULL` & `btree (org_id IS NULL) WHERE org_id IS NULL`)
		    // TODO: single column indexes on low cardinality column => suggest bitmap index instead
		    // TODO: warn queries sans where clause ^^
		    // TODO: locks using https://www.postgresql.org/docs/current/view-pg-locks.html
		    // TODO: no "SELECT *" query
		    const rules = ruleNames.length > 0 ? exports.analyzeRules.filter(r => ruleNames.indexOf(r.id) !== -1 || ruleNames.indexOf(r.name) !== -1) : exports.analyzeRules;
		    return Object.fromEntries(rules.map(r => {
		        const ruleConf = Object.assign({}, r.conf, conf.rules?.[r.id]);
		        return [r.id, (0, zod_2.zodParse)(r.zConf, r.id)(ruleConf).fold(ruleConf => ({
		                rule: r,
		                conf: ruleConf,
		                violations: ruleConf.level === rule_1.RuleLevel.enum.off ? [] : r.analyze(ruleConf, now, db, queries, history, reference?.[r.id]?.violations || [])
		            }), err => ({ rule: r, conf: ruleConf, violations: [{ ruleId: r.id, ruleName: r.name, ruleLevel: r.conf.level, message: `Invalid conf: ${err}` }] }))];
		    }));
		}
		// interesting:
		// - most used tables (in queries)
		// - table growth rate (Go/Month)
		// - query slow down rate (ms/month, mean & max)
		// - unused tables / indexes
		function scoreDatabase(db, violations) {
		    // TODO: compute table importance: page rank & data cardinality (other interesting inputs: storage size, nb columns, nb relations, nb queries, nb indexes), use it to weight the overall score
		    (0, utils_1.groupBy)(violations, v => v.entity ? (0, databaseUtils_1.entityRefToId)(v.entity) : 'unknown');
		    return { score: 0, entities: {} };
		}
		// safe migration rules:
		// - always concurrently in create/drop indexes
		// - disable validations at the beginning, enable them at the end (foreign keys, unique, pk, checks...)
		// - no default value on add column (will update every row otherwise)
		// - drop foreign keys before dropping a table
		
	} (analyze));
	return analyze;
}

var constants$1 = {};

var hasRequiredConstants$1;

function requireConstants$1 () {
	if (hasRequiredConstants$1) return constants$1;
	hasRequiredConstants$1 = 1;
	Object.defineProperty(constants$1, "__esModule", { value: true });
	constants$1.azimuttEmail = void 0;
	constants$1.azimuttEmail = 'contact@azimutt.app';
	
	return constants$1;
}

var databaseDiff = {};

var hasRequiredDatabaseDiff;

function requireDatabaseDiff () {
	if (hasRequiredDatabaseDiff) return databaseDiff;
	hasRequiredDatabaseDiff = 1;
	(function (exports) {
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.DatabaseDiff = exports.TypeDiff = exports.RelationDiff = exports.EntityDiff = exports.CheckDiff = exports.IndexDiff = exports.AttributeDiff = exports.ExtraDiff = exports.OptValueDiff = exports.ValueDiff = exports.ArrayDiff = void 0;
		exports.databaseDiff = databaseDiff;
		exports.attributesDiff = attributesDiff;
		exports.indexesDiff = indexesDiff;
		exports.checksDiff = checksDiff;
		const zod_1 = /*@__PURE__*/ requireLib();
		const utils_1 = requireOut$1();
		const database_1 = requireDatabase();
		const databaseUtils_1 = requireDatabaseUtils();
		// FIXME: Work In Progress
		// cf https://github.com/andreyvit/json-diff
		const ArrayDiff = (schema, schemaDiff) => zod_1.z.object({
		    unchanged: schema.and(zod_1.z.object({ i: zod_1.z.number() })).array(), // both present but same
		    updated: schemaDiff.array(), // both present but different
		    created: schema.and(zod_1.z.object({ i: zod_1.z.number() })).array(), // after only
		    deleted: schema.and(zod_1.z.object({ i: zod_1.z.number() })).array(), // before only
		}).strict().describe('ArrayDiff');
		exports.ArrayDiff = ArrayDiff;
		const ValueDiff = (schema) => zod_1.z.object({ before: schema, after: schema }).strict();
		exports.ValueDiff = ValueDiff;
		const OptValueDiff = (schema) => zod_1.z.object({ before: schema.optional(), after: schema.optional() }).strict();
		exports.OptValueDiff = OptValueDiff;
		exports.ExtraDiff = zod_1.z.record(zod_1.z.object({ before: zod_1.z.any(), after: zod_1.z.any() })).describe('ExtraDiff');
		exports.AttributeDiff = zod_1.z.object({
		    name: database_1.AttributeName,
		    i: (0, exports.ValueDiff)(zod_1.z.number()).optional(),
		    type: (0, exports.ValueDiff)(database_1.AttributeType).optional(),
		    null: (0, exports.OptValueDiff)(zod_1.z.boolean()).optional(),
		    // gen: z.boolean().optional(), // false when not specified
		    default: (0, exports.OptValueDiff)(database_1.AttributeValue).optional(),
		    // attrs: z.lazy(() => Attribute.array().optional()),
		    doc: (0, exports.OptValueDiff)(zod_1.z.string()).optional(),
		    // stats: AttributeStats.optional(),
		    extra: exports.ExtraDiff.optional(),
		}).strict().describe('AttributeDiff');
		exports.IndexDiff = zod_1.z.object({
		    attrs: database_1.AttributePath.array(),
		    name: database_1.ConstraintName.optional(), // keep the name even if it didn't change
		    i: (0, exports.ValueDiff)(zod_1.z.number()).optional(),
		    rename: (0, exports.OptValueDiff)(database_1.ConstraintName).optional(),
		    unique: (0, exports.OptValueDiff)(zod_1.z.boolean()).optional(),
		    partial: (0, exports.OptValueDiff)(zod_1.z.string()).optional(),
		    definition: (0, exports.OptValueDiff)(zod_1.z.string()).optional(),
		    doc: (0, exports.OptValueDiff)(zod_1.z.string()).optional(),
		    // stats: IndexStats.optional(),
		    extra: exports.ExtraDiff.optional(),
		}).strict().describe('IndexDiff');
		exports.CheckDiff = zod_1.z.object({
		    attrs: database_1.AttributePath.array(),
		    name: database_1.ConstraintName.optional(), // keep the name even if it didn't change
		    i: (0, exports.ValueDiff)(zod_1.z.number()).optional(),
		    rename: (0, exports.OptValueDiff)(database_1.ConstraintName).optional(),
		    predicate: (0, exports.OptValueDiff)(zod_1.z.string()).optional(),
		    doc: (0, exports.OptValueDiff)(zod_1.z.string()).optional(),
		    // stats: IndexStats.optional(),
		    extra: exports.ExtraDiff.optional(),
		}).strict().describe('CheckDiff');
		exports.EntityDiff = database_1.Namespace.extend({
		    name: database_1.EntityName,
		    rename: (0, exports.OptValueDiff)(database_1.EntityName).optional(),
		    kind: (0, exports.OptValueDiff)(database_1.EntityKind).optional(),
		    def: (0, exports.OptValueDiff)(zod_1.z.string()).optional(),
		    attrs: (0, exports.ArrayDiff)(database_1.Attribute, exports.AttributeDiff).optional(),
		    pk: (0, exports.OptValueDiff)(database_1.PrimaryKey).optional(),
		    indexes: (0, exports.ArrayDiff)(database_1.Index, exports.IndexDiff).optional(),
		    checks: (0, exports.ArrayDiff)(database_1.Check, exports.CheckDiff).optional(),
		    doc: (0, exports.OptValueDiff)(zod_1.z.string()).optional(),
		    // stats: EntityStats.optional(),
		    extra: exports.ExtraDiff.optional(),
		}).strict().describe('EntityDiff');
		exports.RelationDiff = zod_1.z.object({
		    src: database_1.EntityRef.extend({ attrs: database_1.AttributePath.array() }),
		    ref: database_1.EntityRef.extend({ attrs: database_1.AttributePath.array() }),
		    name: database_1.ConstraintName.optional(),
		    rename: (0, exports.OptValueDiff)(zod_1.z.string()).optional(),
		}).strict().describe('RelationDiff');
		exports.TypeDiff = database_1.Namespace.extend({
		    name: database_1.TypeName,
		    rename: (0, exports.OptValueDiff)(database_1.TypeName).optional(),
		    alias: (0, exports.OptValueDiff)(zod_1.z.string()).optional(),
		    values: (0, exports.OptValueDiff)(zod_1.z.string().array()).optional(),
		    attrs: (0, exports.ArrayDiff)(database_1.Attribute, exports.AttributeDiff).optional(),
		    definition: (0, exports.OptValueDiff)(zod_1.z.string()).optional(),
		    doc: (0, exports.OptValueDiff)(zod_1.z.string()).optional(),
		    extra: exports.ExtraDiff.optional(),
		}).strict().describe('TypeDiff');
		exports.DatabaseDiff = zod_1.z.object({
		    entities: (0, exports.ArrayDiff)(database_1.Entity, exports.EntityDiff).optional(),
		    relations: (0, exports.ArrayDiff)(database_1.Relation, exports.RelationDiff).optional(),
		    types: (0, exports.ArrayDiff)(database_1.Type, exports.TypeDiff).optional(),
		}).strict().describe('DatabaseDiff');
		function databaseDiff(before, after) {
		    const entities = guessRenames(arrayDiffBy(before.entities || [], after.entities || [], databaseUtils_1.entityToId, entityDiff), entityDiff);
		    const relations = arrayDiffBy(before.relations || [], after.relations || [], databaseUtils_1.relationToId, relationDiff);
		    const types = guessRenames(arrayDiffBy(before.types || [], after.types || [], databaseUtils_1.typeToId, typeDiff), typeDiff);
		    return (0, utils_1.removeEmpty)({ entities, relations, types });
		}
		function entityDiff(before, after) {
		    const rename = before.name === after.name ? undefined : valueDiff(before.name, after.name);
		    const kind = before.kind === undefined && after.kind === undefined ? undefined : valueDiff(before.kind, after.kind);
		    const def = before.def === after.def ? undefined : valueDiff(before.def, after.def);
		    const attrs = attributesDiff(before.attrs || [], after.attrs || []);
		    const pk = (0, utils_1.equalDeep)(before.pk, after.pk) ? undefined : valueDiff(before.pk, after.pk);
		    const indexes = indexesDiff(before.indexes || [], after.indexes || []);
		    const checks = checksDiff(before.checks || [], after.checks || []);
		    const doc = before.doc === after.doc ? undefined : valueDiff(before.doc, after.doc);
		    const extra = extraDiff(before.extra || {}, after.extra || {});
		    if ([rename, kind, def, attrs, pk, indexes, checks, doc, extra].every(v => v === undefined))
		        return undefined;
		    return (0, utils_1.removeUndefined)({ ...(0, databaseUtils_1.typeToNamespace)(after), name: after.name, rename, kind, def, attrs, pk, indexes, checks, doc, extra });
		}
		function relationDiff(before, after) {
		    const rename = before.name === after.name ? undefined : valueDiff(before.name, after.name);
		    const doc = before.doc === after.doc ? undefined : valueDiff(before.doc, after.doc);
		    const extra = extraDiff(before.extra || {}, after.extra || {});
		    if ([rename, doc, extra].every(v => v === undefined))
		        return undefined;
		    return (0, utils_1.removeUndefined)({ src: (0, databaseUtils_1.relationLinkToAttributeRef)(after.src), ref: (0, databaseUtils_1.relationLinkToAttributeRef)(after.ref), name: after.name, rename, doc, extra });
		}
		function typeDiff(before, after) {
		    const rename = before.name === after.name ? undefined : valueDiff(before.name, after.name);
		    const alias = before.alias === after.alias ? undefined : valueDiff(before.alias, after.alias);
		    const values = (0, utils_1.arraySame)(before.values || [], after.values || [], (a, b) => a === b) ? undefined : valueDiff(before.values, after.values);
		    const attrs = attributesDiff(before.attrs || [], after.attrs || []);
		    const definition = before.definition === after.definition ? undefined : valueDiff(before.definition, after.definition);
		    const doc = before.doc === after.doc ? undefined : valueDiff(before.doc, after.doc);
		    const extra = extraDiff(before.extra || {}, after.extra || {});
		    if ([rename, alias, values, attrs, definition, doc, extra].every(v => v === undefined))
		        return undefined;
		    return (0, utils_1.removeUndefined)({ ...(0, databaseUtils_1.typeToNamespace)(after), name: after.name, rename, alias, values, attrs, definition, doc, extra });
		}
		function attributesDiff(before, after) {
		    return arrayDiffBy(before, after, a => a.name, attributeDiff);
		}
		function attributeDiff(before, after) {
		    const i = before.i === after.i ? undefined : valueDiff(before.i, after.i);
		    const type = before.type === after.type ? undefined : valueDiff(before.type, after.type);
		    const nullable = before.null === after.null ? undefined : valueDiff(before.null, after.null);
		    const defaultValue = before.default === after.default ? undefined : valueDiff(before.default, after.default);
		    const doc = before.doc === after.doc ? undefined : valueDiff(before.doc, after.doc);
		    const extra = extraDiff(before.extra || {}, after.extra || {});
		    if ([i, type, nullable, defaultValue, doc, extra].every(v => v === undefined))
		        return undefined;
		    return (0, utils_1.removeUndefined)({ ...(0, databaseUtils_1.typeToNamespace)(before), name: before.name, i, type, null: nullable, default: defaultValue, doc, extra });
		}
		function indexesDiff(before, after) {
		    return guessRenames(arrayDiffBy(before, after, i => i.name || i.attrs.map(databaseUtils_1.attributePathToId).join(','), indexDiff), indexDiff);
		}
		function indexDiff(before, after) {
		    const i = before.i === after.i ? undefined : valueDiff(before.i, after.i);
		    const rename = before.name === after.name ? undefined : valueDiff(before.name, after.name);
		    const unique = before.unique === after.unique ? undefined : valueDiff(before.unique, after.unique);
		    const partial = before.partial === after.partial ? undefined : valueDiff(before.partial, after.partial);
		    const definition = before.definition === after.definition ? undefined : valueDiff(before.definition, after.definition);
		    const doc = before.doc === after.doc ? undefined : valueDiff(before.doc, after.doc);
		    const extra = extraDiff(before.extra || {}, after.extra || {});
		    if ([i, rename, unique, partial, definition, doc, extra].every(v => v === undefined))
		        return undefined;
		    return (0, utils_1.removeUndefined)({ attrs: after.attrs, name: after.name, i, rename, unique, partial, definition, doc, extra });
		}
		function checksDiff(before, after) {
		    return guessRenames(arrayDiffBy(before, after, c => c.name || c.attrs.map(databaseUtils_1.attributePathToId).join(','), checkDiff), checkDiff);
		}
		function checkDiff(before, after) {
		    const i = before.i === after.i ? undefined : valueDiff(before.i, after.i);
		    const rename = before.name === after.name ? undefined : valueDiff(before.name, after.name);
		    const predicate = before.predicate === after.predicate ? undefined : valueDiff(before.predicate, after.predicate);
		    const doc = before.doc === after.doc ? undefined : valueDiff(before.doc, after.doc);
		    const extra = extraDiff(before.extra || {}, after.extra || {});
		    if ([i, rename, predicate, doc, extra].every(v => v === undefined))
		        return undefined;
		    return (0, utils_1.removeUndefined)({ attrs: after.attrs, name: after.name, i, rename, predicate, doc, extra });
		}
		function extraDiff(before, after) {
		    const res = {};
		    Object.entries(before).forEach(([key, value]) => {
		        if (!(0, utils_1.anySame)(value, after[key])) {
		            res[key] = { before: value, after: after[key] };
		        }
		    });
		    Object.entries(after).forEach(([key, value]) => {
		        if (!(key in before)) {
		            res[key] = { before: undefined, after: value };
		        }
		    });
		    return Object.keys(res).length === 0 ? undefined : res;
		}
		// generic functions
		function arrayDiffBy(leftArr, rightArr, getKey, makeDiff) {
		    const { left: deleted, right: created, both: common } = (0, utils_1.diffBy)(leftArr.map((v, i) => Object.assign({ i }, v)), rightArr.map((v, i) => Object.assign({ i }, v)), getKey);
		    const updated = [];
		    const unchanged = [];
		    common.forEach(({ left, right }) => {
		        const res = makeDiff(left, right);
		        res === undefined ? unchanged.push(left) : updated.push(res);
		    });
		    return unchanged.length > 0 || updated.length > 0 || created.length > 0 || deleted.length > 0 ? (0, utils_1.removeEmpty)({ unchanged, updated, created, deleted }) : undefined;
		}
		function valueDiff(before, after) {
		    return { before, after };
		}
		function guessRenames(arr, makeDiff) {
		    const { unchanged, updated, created, deleted } = arr || {};
		    const newUnchanged = unchanged || [], newUpdated = updated || [], newDeleted = deleted || [];
		    const newCreated = (created || []).filter(afterValue => {
		        const { name, ...after } = afterValue;
		        const sameDeleted = newDeleted.findIndex(({ name, ...d }) => (0, utils_1.objectSame)(after, d));
		        if (sameDeleted !== -1) {
		            const diff = makeDiff(newDeleted[sameDeleted], afterValue);
		            diff && newUpdated.push(diff);
		            newDeleted.splice(sameDeleted, 1);
		            return false;
		        }
		        else {
		            return true;
		        }
		    });
		    return newUnchanged.length > 0 || newUpdated.length > 0 || newCreated.length > 0 || newDeleted.length > 0 ? (0, utils_1.removeEmpty)({ unchanged: newUnchanged, updated: newUpdated, created: newCreated, deleted: newDeleted }) : undefined;
		}
		
	} (databaseDiff));
	return databaseDiff;
}

var databaseUrl = {};

var hasRequiredDatabaseUrl;

function requireDatabaseUrl () {
	if (hasRequiredDatabaseUrl) return databaseUrl;
	hasRequiredDatabaseUrl = 1;
	Object.defineProperty(databaseUrl, "__esModule", { value: true });
	databaseUrl.DatabaseUrlParsed = databaseUrl.DatabaseUrl = void 0;
	databaseUrl.parseDatabaseUrl = parseDatabaseUrl;
	databaseUrl.parseDatabaseOptions = parseDatabaseOptions;
	databaseUrl.formatDatabaseOptions = formatDatabaseOptions;
	const zod_1 = /*@__PURE__*/ requireLib();
	const utils_1 = requireOut$1();
	const database_1 = requireDatabase();
	databaseUrl.DatabaseUrl = zod_1.z.string();
	databaseUrl.DatabaseUrlParsed = zod_1.z.object({
	    full: zod_1.z.string(),
	    kind: database_1.DatabaseKind.optional(),
	    user: zod_1.z.string().optional(),
	    pass: zod_1.z.string().optional(),
	    host: zod_1.z.string().optional(),
	    port: zod_1.z.number().optional(),
	    db: zod_1.z.string().optional(),
	    options: zod_1.z.record(zod_1.z.string()).optional(),
	}).strict();
	// vertically align regexes with variable names ^^
	const bq = /^(?:jdbc:)?bigquery:\/\/(?:([^:]+):([^@]*)@)?(?:https:\/\/)?([^:/?&]+)?(?::(\d+))?(?:\/([^?]+))?(?:\?(.+))?$/;
	const couchbaseRegexxxxxxxxxxx = /^couchbases?:\/\/(?:([^:]+):([^@]*)@)?([^:/?&]+)(?::(\d+))?(?:\/([^?]+))?(?:\?(.+))?$/;
	const mariadbRegexxxxxxx = /^(?:jdbc:)?mariadb:\/\/(?:([^:]+):([^@]*)@)?([^:/?&]+)(?::(\d+))?(?:\/([^?]+))?(?:\?(.+))?$/;
	const mongoRegexxxxxxxxxx = /^mongodb(?:\+srv)?:\/\/(?:([^:]+):([^@]*)@)?([^:/?&]+)(?::(\d+))?(?:\/([^?]+))?(?:\?(.+))?$/;
	const mysqlRegexxxxxxxxxxx = /^(?:jdbc:)?mysql:\/\/(?:([^:]+):([^@]*)@)?([^:/?&]+)(?::(\d+))?(?:\/([^?]+))?(?:\?(.+))?$/;
	const oracleRegexxxxxx = /^(?:jdbc:)?oracle:thin:(?:([^\/]+)\/([^@]+)@)?(?:\/\/)?([^:\/]+)(?::(\d+))?(?:[\/:]([^?]+))?(?:\?(.+))?$/;
	const postgresRe = /^(?:jdbc:)?postgres(?:ql)?:\/\/(?:([^:]+):([^@]*)@)?([^:/?&]+)(?::(\d+))?(?:\/([^?]+))?(?:\?(.+))?$/;
	const sqlserver = /^(?:jdbc:)?sqlserver(?:ql)?:\/\/(?:([^:]+):([^@]*)@)?([^:/?&]+)(?::(\d+))?(?:\/([^?]+))?(?:\?(.+))?$/;
	const snowflakeRegexxx = /^(?:jdbc:)?snowflake:\/\/(?:([^:]+):([^@]*)@)?([^:/?&]+)(?::(\d+))?(?:\/([^?]+))?(?:\?(.+))?$/;
	const snowflakeRegexxxxxxxxxxxxxx = /^https:\/\/(?:([^:]+):([^@]*)@)?(.+?(?:\.privatelink)?\.snowflakecomputing\.com)(?::(\d+))?(?:\/([^?]+))?$/;
	function parseDatabaseUrl(rawUrl) {
	    const url = rawUrl.trim();
	    const bigqueryMatches = url.match(bq);
	    if (bigqueryMatches) {
	        const kind = 'bigquery';
	        const [, user, pass, host, port, db, optionsStr] = bigqueryMatches;
	        const { email: user2, key: pass2, project: db2, ...options } = parseDatabaseOptions(optionsStr);
	        return (0, utils_1.removeUndefined)({ full: url, kind, user: user || user2, pass: pass || pass2, host, port: port ? parseInt(port) : undefined, db: db || db2, options: Object.keys(options).length > 0 ? options : undefined });
	    }
	    const couchbaseMatches = url.match(couchbaseRegexxxxxxxxxxx);
	    if (couchbaseMatches) {
	        const kind = 'couchbase';
	        const [, user, pass, host, port, db, optionsStr] = couchbaseMatches;
	        const options = optionsStr ? parseDatabaseOptions(optionsStr) : undefined;
	        return (0, utils_1.removeUndefined)({ full: url, kind, user, pass, host, port: port ? parseInt(port) : undefined, db, options });
	    }
	    const mariadbMatches = url.match(mariadbRegexxxxxxx);
	    if (mariadbMatches) {
	        const kind = 'mariadb';
	        const [, user, pass, host, port, db, optionsStr] = mariadbMatches;
	        const options = optionsStr ? parseDatabaseOptions(optionsStr) : undefined;
	        return (0, utils_1.removeUndefined)({ full: url, kind, user, pass, host, port: port ? parseInt(port) : undefined, db, options });
	    }
	    const mongodbMatches = url.match(mongoRegexxxxxxxxxx);
	    if (mongodbMatches) {
	        const kind = 'mongodb';
	        const [, user, pass, host, port, db, optionsStr] = mongodbMatches;
	        const options = optionsStr ? parseDatabaseOptions(optionsStr) : undefined;
	        return (0, utils_1.removeUndefined)({ full: url, kind, user, pass, host, port: port ? parseInt(port) : undefined, db, options });
	    }
	    const mysqlMatches = url.match(mysqlRegexxxxxxxxxxx);
	    if (mysqlMatches) {
	        const kind = 'mysql';
	        const [, user, pass, host, port, db, optionsStr] = mysqlMatches;
	        const options = optionsStr ? parseDatabaseOptions(optionsStr) : undefined;
	        return (0, utils_1.removeUndefined)({ full: url, kind, user, pass, host, port: port ? parseInt(port) : undefined, db, options });
	    }
	    const oracleMatches = url.match(oracleRegexxxxxx);
	    if (oracleMatches) {
	        const kind = 'oracle';
	        const [, user, pass, host, port, db, optionsStr] = oracleMatches;
	        const options = optionsStr ? parseDatabaseOptions(optionsStr) : undefined;
	        return (0, utils_1.removeUndefined)({ full: url, kind, user, pass, host, port: port ? parseInt(port) : undefined, db, options });
	    }
	    const postgresMatches = url.match(postgresRe);
	    if (postgresMatches) {
	        const kind = 'postgres';
	        const [, user, pass, host, port, db, optionsStr] = postgresMatches;
	        const options = optionsStr ? parseDatabaseOptions(optionsStr) : undefined;
	        return (0, utils_1.removeUndefined)({ full: url, kind, user, pass, host, port: port ? parseInt(port) : undefined, db, options });
	    }
	    const snowflakeMatches = url.match(snowflakeRegexxx) || url.match(snowflakeRegexxxxxxxxxxxxxx);
	    if (snowflakeMatches) {
	        const kind = 'snowflake';
	        const [, user, pass, host, port, db, optionsStr] = snowflakeMatches;
	        const { db: db2, user: user2, ...options } = parseDatabaseOptions(optionsStr);
	        return (0, utils_1.removeUndefined)({ full: url, kind, user: user || user2, pass, host, port: port ? parseInt(port) : undefined, db: db || db2, options: Object.keys(options).length > 0 ? options : undefined });
	    }
	    const sqlserverMatches = url.match(sqlserver) || parseSqlServerUrl(url);
	    if (sqlserverMatches) {
	        const kind = 'sqlserver';
	        const [, user, pass, host, port, db, optionsStr] = sqlserverMatches;
	        const options = optionsStr ? parseDatabaseOptions(optionsStr) : undefined;
	        return (0, utils_1.removeUndefined)({ full: url, kind, user, pass, host, port: port ? parseInt(port) : undefined, db, options });
	    }
	    return { full: url };
	}
	function parseDatabaseOptions(options) {
	    return Object.fromEntries((options || '')
	        .split('&')
	        .filter(o => !!o)
	        .map(o => o.split('=')));
	}
	function formatDatabaseOptions(opts) {
	    return Object.entries(opts).filter(([k, _]) => k).map(([k, v]) => `${k}=${v}`).join('&') || undefined;
	}
	// https://www.connectionstrings.com/sql-server 🤯
	function parseSqlServerUrl(url) {
	    const props = Object.fromEntries(url.split(';').map(part => part.split('=')).map(([key, value]) => [key.toLowerCase(), value]));
	    const { server, ['data source']: dataSource, database, ['initial catalog']: initialCatalog, ['user id']: user, password, ...other } = props;
	    const hostPart = server || dataSource;
	    const databasePart = database || initialCatalog;
	    if (hostPart && databasePart) {
	        const [host, port] = hostPart.split(',');
	        const options = Object.entries(other).map(([k, v]) => `${k}=${v}`).join('&');
	        return options ? [url, user, password, host, port, databasePart, options] : [url, user, password, host, port, databasePart];
	    }
	    return null;
	}
	
	return databaseUrl;
}

var inferSchema = {};

var hasRequiredInferSchema;

function requireInferSchema () {
	if (hasRequiredInferSchema) return inferSchema;
	hasRequiredInferSchema = 1;
	Object.defineProperty(inferSchema, "__esModule", { value: true });
	inferSchema.valuesToSchema = valuesToSchema;
	inferSchema.schemaToAttributes = schemaToAttributes;
	inferSchema.valueToSchema = valueToSchema;
	inferSchema.sumType = sumType;
	const utils_1 = requireOut$1();
	function valuesToSchema(values) {
	    if (values.length > 0) {
	        return sumSchema(values.map(valueToSchema));
	    }
	    else {
	        return valueToSchema(undefined);
	    }
	}
	// flatten: how many top level should be flattened
	function schemaToAttributes(schema, flatten = 0, path = []) {
	    // TODO: if string with few values (< 10% of docs), handle it like an enum and add values in comment
	    if (schema.nested && flatten > 0) {
	        return Object.entries(schema.nested).flatMap(([key, value]) => {
	            return [(0, utils_1.removeUndefined)({
	                    name: path.map(p => p + '.').join('') + key,
	                    type: value.type,
	                    null: value.nullable
	                }), ...schemaToAttributes(value, flatten - 1, [...path, key])];
	        });
	    }
	    else if (schema.nested) {
	        return Object.entries(schema.nested).flatMap(([key, value]) => {
	            const name = path.map(p => p + '.').join('') + key;
	            return [(0, utils_1.removeUndefined)({
	                    name,
	                    type: value.type,
	                    null: value.nullable,
	                    attrs: value.nested ? schemaToAttributes(value) : undefined
	                })];
	        });
	    }
	    else {
	        return [];
	    }
	}
	// 👇️ Private functions, some are exported only for tests
	// If you use them, beware of breaking changes!
	function valueToSchema(value) {
	    if (Array.isArray(value) && value.length > 0) {
	        const schemas = value.map(valueToSchema);
	        return cleanSchema({
	            type: sumType(schemas.map(s => s.type)) + '[]',
	            values: schemas.flatMap(s => s.values),
	            nullable: false,
	            nested: mergeNested(schemas.map(s => s.nested || {}))
	        });
	    }
	    else if (typeof value === 'object' && value !== null && value.constructor.name === 'Object') {
	        return cleanSchema({
	            type: getType(value),
	            values: [value],
	            nullable: isNullable(value),
	            nested: (0, utils_1.mapValues)(value, valueToSchema)
	        });
	    }
	    else if (isNullable(value)) {
	        return { type: getType(value), values: [], nullable: true };
	    }
	    else {
	        return { type: getType(value), values: [value] };
	    }
	}
	function mergeNested(items) {
	    const nested = {};
	    items.forEach(i => Object.entries(i).forEach(([key, schema]) => {
	        nested[key] = nested[key] ? [...nested[key], schema] : [schema];
	    }));
	    // TODO: if more than 10 keys, they are all integers or uuids or have the same size or are hex values & have the same type inside => treat this as record instead of object
	    return (0, utils_1.mapValues)(nested, sumSchema);
	}
	function sumSchema(schemas) {
	    return cleanSchema({
	        type: sumType(schemas.map(s => s.type).filter(t => t !== 'null')) || 'null',
	        values: schemas.flatMap(s => s.values),
	        nullable: schemas.reduce((acc, schema) => schema.nullable || false || acc, false),
	        nested: mergeNested(schemas.map(s => s.nested || {}))
	    });
	}
	function cleanSchema(schema) {
	    const res = { type: schema.type, values: schema.values };
	    if (schema.nested && Object.keys(schema.nested).length > 0) {
	        res.nested = schema.nested;
	    }
	    if (schema.nullable) {
	        res.nullable = schema.nullable;
	    }
	    return res;
	}
	function getType(value) {
	    if (value === undefined || value === null) {
	        return 'null';
	    }
	    else if (Array.isArray(value)) {
	        return value.length > 0 ? sumType(value.map(getType)) + '[]' : '[]';
	    }
	    else if (typeof value === 'object') {
	        return value.constructor.name;
	    }
	    else {
	        return typeof value;
	    }
	}
	function sumType(types) {
	    types = (0, utils_1.distinct)(types);
	    types = types.some(t => t !== '[]' && t.endsWith('[]')) ? types.filter(t => t !== '[]') : types;
	    return types.map(t => t.indexOf('|') === -1 ? t : `(${t})`).join('|');
	}
	function isNullable(value) {
	    return value === null || value === undefined;
	}
	
	return inferSchema;
}

var desktopBridge = {};

var hasRequiredDesktopBridge;

function requireDesktopBridge () {
	if (hasRequiredDesktopBridge) return desktopBridge;
	hasRequiredDesktopBridge = 1;
	Object.defineProperty(desktopBridge, "__esModule", { value: true });
	
	return desktopBridge;
}

var legacyProject = {};

var llm = {};

var llmUtils = {};

var hasRequiredLlmUtils;

function requireLlmUtils () {
	if (hasRequiredLlmUtils) return llmUtils;
	hasRequiredLlmUtils = 1;
	Object.defineProperty(llmUtils, "__esModule", { value: true });
	llmUtils.dbToPrompt = dbToPrompt;
	llmUtils.entityToPrompt = entityToPrompt;
	llmUtils.cleanSqlAnswer = cleanSqlAnswer;
	const utils_1 = requireOut$1();
	const databaseUtils_1 = requireDatabaseUtils();
	function dbToPrompt(db) {
	    // TODO: depending on selected model, decide if db can be fully integrated or should be compressed (no types, filter tables...) or have additional data (sample values, stats...)
	    const relationsBySrc = (0, utils_1.groupBy)(db.relations || [], r => (0, databaseUtils_1.entityRefToId)(r.src));
	    const entities = (db.entities || []).map(e => entityToPrompt(e, relationsBySrc[(0, databaseUtils_1.entityToId)(e)] || []));
	    return `\`\`\`
${entities.join('\n')}
\`\`\``;
	}
	function entityToPrompt(e, rels) {
	    const [compositeRelations, simpleRelations] = (0, utils_1.partition)(rels, r => r.src.attrs.length > 1);
	    const simpleRelationsByAttr = (0, utils_1.groupBy)(simpleRelations, r => (0, databaseUtils_1.attributePathToId)(r.src.attrs[0]));
	    const polymorphicRelations = Object.values(simpleRelationsByAttr).filter(rs => rs.length > 1).flatMap(rs => rs);
	    const attrs = (e.attrs || []).map(a => attributeToPrompt(a, simpleRelationsByAttr[a.name] || []));
	    const otherRels = compositeRelations.concat(polymorphicRelations).map(entityForeignKey);
	    return `CREATE TABLE ${(0, databaseUtils_1.entityToId)(e)} (${attrs.concat(otherRels).join(', ')});`;
	}
	function attributeToPrompt(a, rels) {
	    return `${a.name} ${a.type}${rels.length === 1 ? ` REFERENCES ${(0, databaseUtils_1.entityRefToId)(rels[0].ref)}(${(0, databaseUtils_1.attributePathToId)(rels[0].ref.attrs[0])})` : ''}`;
	}
	function entityForeignKey(r) {
	    return `FOREIGN KEY (${r.src.attrs.map(databaseUtils_1.attributePathToId).join(', ')}) REFERENCES ${(0, databaseUtils_1.entityRefToId)(r.ref)}(${r.ref.attrs.map(databaseUtils_1.attributePathToId).join(', ')})`;
	}
	function cleanSqlAnswer(answer) {
	    const mdCode = answer.trim().match(/^(?:```)?(?:sql)?\n?(.+?)\n?(?:```)?$/s);
	    if (mdCode)
	        return mdCode[1];
	    return answer.trim();
	}
	
	return llmUtils;
}

var openai$2 = {};

var web = {};

var webTypes = {};

/**
 * Disclaimer: modules in _shims aren't intended to be imported by SDK users.
 */

var hasRequiredWebTypes;

function requireWebTypes () {
	if (hasRequiredWebTypes) return webTypes;
	hasRequiredWebTypes = 1;

	return webTypes;
}

var registry$1 = {};

var hasRequiredRegistry;

function requireRegistry () {
	if (hasRequiredRegistry) return registry$1;
	hasRequiredRegistry = 1;
	(function (exports) {
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.setShims = exports.isFsReadStream = exports.fileFromPath = exports.getDefaultAgent = exports.getMultipartRequestOptions = exports.ReadableStream = exports.File = exports.Blob = exports.FormData = exports.Headers = exports.Response = exports.Request = exports.fetch = exports.kind = exports.auto = void 0;
		exports.auto = false;
		exports.kind = undefined;
		exports.fetch = undefined;
		exports.Request = undefined;
		exports.Response = undefined;
		exports.Headers = undefined;
		exports.FormData = undefined;
		exports.Blob = undefined;
		exports.File = undefined;
		exports.ReadableStream = undefined;
		exports.getMultipartRequestOptions = undefined;
		exports.getDefaultAgent = undefined;
		exports.fileFromPath = undefined;
		exports.isFsReadStream = undefined;
		function setShims(shims, options = { auto: false }) {
		    if (exports.auto) {
		        throw new Error(`you must \`import 'openai/shims/${shims.kind}'\` before importing anything else from openai`);
		    }
		    if (exports.kind) {
		        throw new Error(`can't \`import 'openai/shims/${shims.kind}'\` after \`import 'openai/shims/${exports.kind}'\``);
		    }
		    exports.auto = options.auto;
		    exports.kind = shims.kind;
		    exports.fetch = shims.fetch;
		    exports.Request = shims.Request;
		    exports.Response = shims.Response;
		    exports.Headers = shims.Headers;
		    exports.FormData = shims.FormData;
		    exports.Blob = shims.Blob;
		    exports.File = shims.File;
		    exports.ReadableStream = shims.ReadableStream;
		    exports.getMultipartRequestOptions = shims.getMultipartRequestOptions;
		    exports.getDefaultAgent = shims.getDefaultAgent;
		    exports.fileFromPath = shims.fileFromPath;
		    exports.isFsReadStream = shims.isFsReadStream;
		}
		exports.setShims = setShims;
		
	} (registry$1));
	return registry$1;
}

var webRuntime = {};

var MultipartBody = {};

var hasRequiredMultipartBody;

function requireMultipartBody () {
	if (hasRequiredMultipartBody) return MultipartBody;
	hasRequiredMultipartBody = 1;
	Object.defineProperty(MultipartBody, "__esModule", { value: true });
	MultipartBody.MultipartBody = void 0;
	/**
	 * Disclaimer: modules in _shims aren't intended to be imported by SDK users.
	 */
	let MultipartBody$1 = class MultipartBody {
	    constructor(body) {
	        this.body = body;
	    }
	    get [Symbol.toStringTag]() {
	        return 'MultipartBody';
	    }
	};
	MultipartBody.MultipartBody = MultipartBody$1;
	
	return MultipartBody;
}

var hasRequiredWebRuntime;

function requireWebRuntime () {
	if (hasRequiredWebRuntime) return webRuntime;
	hasRequiredWebRuntime = 1;
	Object.defineProperty(webRuntime, "__esModule", { value: true });
	webRuntime.getRuntime = void 0;
	/**
	 * Disclaimer: modules in _shims aren't intended to be imported by SDK users.
	 */
	const MultipartBody_1 = /*@__PURE__*/ requireMultipartBody();
	function getRuntime({ manuallyImported } = {}) {
	    const recommendation = manuallyImported ?
	        `You may need to use polyfills`
	        : `Add one of these imports before your first \`import … from 'openai'\`:
- \`import 'openai/shims/node'\` (if you're running on Node)
- \`import 'openai/shims/web'\` (otherwise)
`;
	    let _fetch, _Request, _Response, _Headers;
	    try {
	        // @ts-ignore
	        _fetch = fetch;
	        // @ts-ignore
	        _Request = Request;
	        // @ts-ignore
	        _Response = Response;
	        // @ts-ignore
	        _Headers = Headers;
	    }
	    catch (error) {
	        throw new Error(`this environment is missing the following Web Fetch API type: ${error.message}. ${recommendation}`);
	    }
	    return {
	        kind: 'web',
	        fetch: _fetch,
	        Request: _Request,
	        Response: _Response,
	        Headers: _Headers,
	        FormData: 
	        // @ts-ignore
	        typeof FormData !== 'undefined' ? FormData : (class FormData {
	            // @ts-ignore
	            constructor() {
	                throw new Error(`file uploads aren't supported in this environment yet as 'FormData' is undefined. ${recommendation}`);
	            }
	        }),
	        Blob: typeof Blob !== 'undefined' ? Blob : (class Blob {
	            constructor() {
	                throw new Error(`file uploads aren't supported in this environment yet as 'Blob' is undefined. ${recommendation}`);
	            }
	        }),
	        File: 
	        // @ts-ignore
	        typeof File !== 'undefined' ? File : (class File {
	            // @ts-ignore
	            constructor() {
	                throw new Error(`file uploads aren't supported in this environment yet as 'File' is undefined. ${recommendation}`);
	            }
	        }),
	        ReadableStream: 
	        // @ts-ignore
	        typeof ReadableStream !== 'undefined' ? ReadableStream : (class ReadableStream {
	            // @ts-ignore
	            constructor() {
	                throw new Error(`streaming isn't supported in this environment yet as 'ReadableStream' is undefined. ${recommendation}`);
	            }
	        }),
	        getMultipartRequestOptions: async (
	        // @ts-ignore
	        form, opts) => ({
	            ...opts,
	            body: new MultipartBody_1.MultipartBody(form),
	        }),
	        getDefaultAgent: (url) => undefined,
	        fileFromPath: () => {
	            throw new Error('The `fileFromPath` function is only supported in Node. See the README for more details: https://www.github.com/openai/openai-node#file-uploads');
	        },
	        isFsReadStream: (value) => false,
	    };
	}
	webRuntime.getRuntime = getRuntime;
	
	return webRuntime;
}

var hasRequiredWeb;

function requireWeb () {
	if (hasRequiredWeb) return web;
	hasRequiredWeb = 1;
	var __createBinding = (web && web.__createBinding) || (Object.create ? (function(o, m, k, k2) {
	    if (k2 === undefined) k2 = k;
	    var desc = Object.getOwnPropertyDescriptor(m, k);
	    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
	      desc = { enumerable: true, get: function() { return m[k]; } };
	    }
	    Object.defineProperty(o, k2, desc);
	}) : (function(o, m, k, k2) {
	    if (k2 === undefined) k2 = k;
	    o[k2] = m[k];
	}));
	var __setModuleDefault = (web && web.__setModuleDefault) || (Object.create ? (function(o, v) {
	    Object.defineProperty(o, "default", { enumerable: true, value: v });
	}) : function(o, v) {
	    o["default"] = v;
	});
	var __importStar = (web && web.__importStar) || function (mod) {
	    if (mod && mod.__esModule) return mod;
	    var result = {};
	    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
	    __setModuleDefault(result, mod);
	    return result;
	};
	Object.defineProperty(web, "__esModule", { value: true });
	// @ts-ignore
	__importStar(/*@__PURE__*/ requireWebTypes());
	const registry_1 = /*@__PURE__*/ requireRegistry();
	const web_runtime_1 = /*@__PURE__*/ requireWebRuntime();
	(0, registry_1.setShims)((0, web_runtime_1.getRuntime)({ manuallyImported: true }));
	
	return web;
}

var openai$1 = {exports: {}};

var error = {};

var core = {};

var version$3 = {};

var hasRequiredVersion$1;

function requireVersion$1 () {
	if (hasRequiredVersion$1) return version$3;
	hasRequiredVersion$1 = 1;
	Object.defineProperty(version$3, "__esModule", { value: true });
	version$3.VERSION = void 0;
	version$3.VERSION = '4.65.0'; // x-release-please-version
	
	return version$3;
}

var streaming = {};

var _shims = {};

let auto = false;
let kind = undefined;
let fetch$1 = undefined;
let Request$1 = undefined;
let Response$1 = undefined;
let Headers$1 = undefined;
let FormData$1 = undefined;
let Blob$1 = undefined;
let File$1 = undefined;
let ReadableStream$1 = undefined;
let getMultipartRequestOptions = undefined;
let getDefaultAgent = undefined;
let fileFromPath = undefined;
let isFsReadStream = undefined;
function setShims(shims, options = { auto: false }) {
    if (auto) {
        throw new Error(`you must \`import 'openai/shims/${shims.kind}'\` before importing anything else from openai`);
    }
    if (kind) {
        throw new Error(`can't \`import 'openai/shims/${shims.kind}'\` after \`import 'openai/shims/${kind}'\``);
    }
    auto = options.auto;
    kind = shims.kind;
    fetch$1 = shims.fetch;
    Request$1 = shims.Request;
    Response$1 = shims.Response;
    Headers$1 = shims.Headers;
    FormData$1 = shims.FormData;
    Blob$1 = shims.Blob;
    File$1 = shims.File;
    ReadableStream$1 = shims.ReadableStream;
    getMultipartRequestOptions = shims.getMultipartRequestOptions;
    getDefaultAgent = shims.getDefaultAgent;
    fileFromPath = shims.fileFromPath;
    isFsReadStream = shims.isFsReadStream;
}

var registry = /*#__PURE__*/Object.freeze({
	__proto__: null,
	get Blob () { return Blob$1; },
	get File () { return File$1; },
	get FormData () { return FormData$1; },
	get Headers () { return Headers$1; },
	get ReadableStream () { return ReadableStream$1; },
	get Request () { return Request$1; },
	get Response () { return Response$1; },
	get auto () { return auto; },
	get fetch () { return fetch$1; },
	get fileFromPath () { return fileFromPath; },
	get getDefaultAgent () { return getDefaultAgent; },
	get getMultipartRequestOptions () { return getMultipartRequestOptions; },
	get isFsReadStream () { return isFsReadStream; },
	get kind () { return kind; },
	setShims: setShims
});

var require$$0 = /*@__PURE__*/getAugmentedNamespace(registry);

var runtime = {};

var hasRequiredRuntime;

function requireRuntime () {
	if (hasRequiredRuntime) return runtime;
	hasRequiredRuntime = 1;
	(function (exports) {
		var __createBinding = (runtime && runtime.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __exportStar = (runtime && runtime.__exportStar) || function(m, exports) {
		    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		/**
		 * Disclaimer: modules in _shims aren't intended to be imported by SDK users.
		 */
		__exportStar(/*@__PURE__*/ requireWebRuntime(), exports);
		
	} (runtime));
	return runtime;
}

/**
 * Disclaimer: modules in _shims aren't intended to be imported by SDK users.
 */

var hasRequired_shims;

function require_shims () {
	if (hasRequired_shims) return _shims;
	hasRequired_shims = 1;
	(function (exports) {
		const shims = require$$0;
		const auto = requireRuntime();
		if (!shims.kind) shims.setShims(auto.getRuntime(), { auto: true });
		for (const property of Object.keys(shims)) {
		  Object.defineProperty(exports, property, {
		    get() {
		      return shims[property];
		    },
		  });
		} 
	} (_shims));
	return _shims;
}

var hasRequiredStreaming;

function requireStreaming () {
	if (hasRequiredStreaming) return streaming;
	hasRequiredStreaming = 1;
	Object.defineProperty(streaming, "__esModule", { value: true });
	streaming.readableStreamAsyncIterable = streaming._decodeChunks = streaming._iterSSEMessages = streaming.Stream = void 0;
	const index_1 = require_shims();
	const error_1 = /*@__PURE__*/ requireError();
	const error_2 = /*@__PURE__*/ requireError();
	class Stream {
	    constructor(iterator, controller) {
	        this.iterator = iterator;
	        this.controller = controller;
	    }
	    static fromSSEResponse(response, controller) {
	        let consumed = false;
	        async function* iterator() {
	            if (consumed) {
	                throw new Error('Cannot iterate over a consumed stream, use `.tee()` to split the stream.');
	            }
	            consumed = true;
	            let done = false;
	            try {
	                for await (const sse of _iterSSEMessages(response, controller)) {
	                    if (done)
	                        continue;
	                    if (sse.data.startsWith('[DONE]')) {
	                        done = true;
	                        continue;
	                    }
	                    if (sse.event === null) {
	                        let data;
	                        try {
	                            data = JSON.parse(sse.data);
	                        }
	                        catch (e) {
	                            console.error(`Could not parse message into JSON:`, sse.data);
	                            console.error(`From chunk:`, sse.raw);
	                            throw e;
	                        }
	                        if (data && data.error) {
	                            throw new error_2.APIError(undefined, data.error, undefined, undefined);
	                        }
	                        yield data;
	                    }
	                    else {
	                        let data;
	                        try {
	                            data = JSON.parse(sse.data);
	                        }
	                        catch (e) {
	                            console.error(`Could not parse message into JSON:`, sse.data);
	                            console.error(`From chunk:`, sse.raw);
	                            throw e;
	                        }
	                        // TODO: Is this where the error should be thrown?
	                        if (sse.event == 'error') {
	                            throw new error_2.APIError(undefined, data.error, data.message, undefined);
	                        }
	                        yield { event: sse.event, data: data };
	                    }
	                }
	                done = true;
	            }
	            catch (e) {
	                // If the user calls `stream.controller.abort()`, we should exit without throwing.
	                if (e instanceof Error && e.name === 'AbortError')
	                    return;
	                throw e;
	            }
	            finally {
	                // If the user `break`s, abort the ongoing request.
	                if (!done)
	                    controller.abort();
	            }
	        }
	        return new Stream(iterator, controller);
	    }
	    /**
	     * Generates a Stream from a newline-separated ReadableStream
	     * where each item is a JSON value.
	     */
	    static fromReadableStream(readableStream, controller) {
	        let consumed = false;
	        async function* iterLines() {
	            const lineDecoder = new LineDecoder();
	            const iter = readableStreamAsyncIterable(readableStream);
	            for await (const chunk of iter) {
	                for (const line of lineDecoder.decode(chunk)) {
	                    yield line;
	                }
	            }
	            for (const line of lineDecoder.flush()) {
	                yield line;
	            }
	        }
	        async function* iterator() {
	            if (consumed) {
	                throw new Error('Cannot iterate over a consumed stream, use `.tee()` to split the stream.');
	            }
	            consumed = true;
	            let done = false;
	            try {
	                for await (const line of iterLines()) {
	                    if (done)
	                        continue;
	                    if (line)
	                        yield JSON.parse(line);
	                }
	                done = true;
	            }
	            catch (e) {
	                // If the user calls `stream.controller.abort()`, we should exit without throwing.
	                if (e instanceof Error && e.name === 'AbortError')
	                    return;
	                throw e;
	            }
	            finally {
	                // If the user `break`s, abort the ongoing request.
	                if (!done)
	                    controller.abort();
	            }
	        }
	        return new Stream(iterator, controller);
	    }
	    [Symbol.asyncIterator]() {
	        return this.iterator();
	    }
	    /**
	     * Splits the stream into two streams which can be
	     * independently read from at different speeds.
	     */
	    tee() {
	        const left = [];
	        const right = [];
	        const iterator = this.iterator();
	        const teeIterator = (queue) => {
	            return {
	                next: () => {
	                    if (queue.length === 0) {
	                        const result = iterator.next();
	                        left.push(result);
	                        right.push(result);
	                    }
	                    return queue.shift();
	                },
	            };
	        };
	        return [
	            new Stream(() => teeIterator(left), this.controller),
	            new Stream(() => teeIterator(right), this.controller),
	        ];
	    }
	    /**
	     * Converts this stream to a newline-separated ReadableStream of
	     * JSON stringified values in the stream
	     * which can be turned back into a Stream with `Stream.fromReadableStream()`.
	     */
	    toReadableStream() {
	        const self = this;
	        let iter;
	        const encoder = new TextEncoder();
	        return new index_1.ReadableStream({
	            async start() {
	                iter = self[Symbol.asyncIterator]();
	            },
	            async pull(ctrl) {
	                try {
	                    const { value, done } = await iter.next();
	                    if (done)
	                        return ctrl.close();
	                    const bytes = encoder.encode(JSON.stringify(value) + '\n');
	                    ctrl.enqueue(bytes);
	                }
	                catch (err) {
	                    ctrl.error(err);
	                }
	            },
	            async cancel() {
	                await iter.return?.();
	            },
	        });
	    }
	}
	streaming.Stream = Stream;
	async function* _iterSSEMessages(response, controller) {
	    if (!response.body) {
	        controller.abort();
	        throw new error_1.OpenAIError(`Attempted to iterate over a response with no body`);
	    }
	    const sseDecoder = new SSEDecoder();
	    const lineDecoder = new LineDecoder();
	    const iter = readableStreamAsyncIterable(response.body);
	    for await (const sseChunk of iterSSEChunks(iter)) {
	        for (const line of lineDecoder.decode(sseChunk)) {
	            const sse = sseDecoder.decode(line);
	            if (sse)
	                yield sse;
	        }
	    }
	    for (const line of lineDecoder.flush()) {
	        const sse = sseDecoder.decode(line);
	        if (sse)
	            yield sse;
	    }
	}
	streaming._iterSSEMessages = _iterSSEMessages;
	/**
	 * Given an async iterable iterator, iterates over it and yields full
	 * SSE chunks, i.e. yields when a double new-line is encountered.
	 */
	async function* iterSSEChunks(iterator) {
	    let data = new Uint8Array();
	    for await (const chunk of iterator) {
	        if (chunk == null) {
	            continue;
	        }
	        const binaryChunk = chunk instanceof ArrayBuffer ? new Uint8Array(chunk)
	            : typeof chunk === 'string' ? new TextEncoder().encode(chunk)
	                : chunk;
	        let newData = new Uint8Array(data.length + binaryChunk.length);
	        newData.set(data);
	        newData.set(binaryChunk, data.length);
	        data = newData;
	        let patternIndex;
	        while ((patternIndex = findDoubleNewlineIndex(data)) !== -1) {
	            yield data.slice(0, patternIndex);
	            data = data.slice(patternIndex);
	        }
	    }
	    if (data.length > 0) {
	        yield data;
	    }
	}
	function findDoubleNewlineIndex(buffer) {
	    // This function searches the buffer for the end patterns (\r\r, \n\n, \r\n\r\n)
	    // and returns the index right after the first occurrence of any pattern,
	    // or -1 if none of the patterns are found.
	    const newline = 0x0a; // \n
	    const carriage = 0x0d; // \r
	    for (let i = 0; i < buffer.length - 2; i++) {
	        if (buffer[i] === newline && buffer[i + 1] === newline) {
	            // \n\n
	            return i + 2;
	        }
	        if (buffer[i] === carriage && buffer[i + 1] === carriage) {
	            // \r\r
	            return i + 2;
	        }
	        if (buffer[i] === carriage &&
	            buffer[i + 1] === newline &&
	            i + 3 < buffer.length &&
	            buffer[i + 2] === carriage &&
	            buffer[i + 3] === newline) {
	            // \r\n\r\n
	            return i + 4;
	        }
	    }
	    return -1;
	}
	class SSEDecoder {
	    constructor() {
	        this.event = null;
	        this.data = [];
	        this.chunks = [];
	    }
	    decode(line) {
	        if (line.endsWith('\r')) {
	            line = line.substring(0, line.length - 1);
	        }
	        if (!line) {
	            // empty line and we didn't previously encounter any messages
	            if (!this.event && !this.data.length)
	                return null;
	            const sse = {
	                event: this.event,
	                data: this.data.join('\n'),
	                raw: this.chunks,
	            };
	            this.event = null;
	            this.data = [];
	            this.chunks = [];
	            return sse;
	        }
	        this.chunks.push(line);
	        if (line.startsWith(':')) {
	            return null;
	        }
	        let [fieldname, _, value] = partition(line, ':');
	        if (value.startsWith(' ')) {
	            value = value.substring(1);
	        }
	        if (fieldname === 'event') {
	            this.event = value;
	        }
	        else if (fieldname === 'data') {
	            this.data.push(value);
	        }
	        return null;
	    }
	}
	/**
	 * A re-implementation of httpx's `LineDecoder` in Python that handles incrementally
	 * reading lines from text.
	 *
	 * https://github.com/encode/httpx/blob/920333ea98118e9cf617f246905d7b202510941c/httpx/_decoders.py#L258
	 */
	class LineDecoder {
	    constructor() {
	        this.buffer = [];
	        this.trailingCR = false;
	    }
	    decode(chunk) {
	        let text = this.decodeText(chunk);
	        if (this.trailingCR) {
	            text = '\r' + text;
	            this.trailingCR = false;
	        }
	        if (text.endsWith('\r')) {
	            this.trailingCR = true;
	            text = text.slice(0, -1);
	        }
	        if (!text) {
	            return [];
	        }
	        const trailingNewline = LineDecoder.NEWLINE_CHARS.has(text[text.length - 1] || '');
	        let lines = text.split(LineDecoder.NEWLINE_REGEXP);
	        // if there is a trailing new line then the last entry will be an empty
	        // string which we don't care about
	        if (trailingNewline) {
	            lines.pop();
	        }
	        if (lines.length === 1 && !trailingNewline) {
	            this.buffer.push(lines[0]);
	            return [];
	        }
	        if (this.buffer.length > 0) {
	            lines = [this.buffer.join('') + lines[0], ...lines.slice(1)];
	            this.buffer = [];
	        }
	        if (!trailingNewline) {
	            this.buffer = [lines.pop() || ''];
	        }
	        return lines;
	    }
	    decodeText(bytes) {
	        if (bytes == null)
	            return '';
	        if (typeof bytes === 'string')
	            return bytes;
	        // Node:
	        if (typeof Buffer !== 'undefined') {
	            if (bytes instanceof Buffer) {
	                return bytes.toString();
	            }
	            if (bytes instanceof Uint8Array) {
	                return Buffer.from(bytes).toString();
	            }
	            throw new error_1.OpenAIError(`Unexpected: received non-Uint8Array (${bytes.constructor.name}) stream chunk in an environment with a global "Buffer" defined, which this library assumes to be Node. Please report this error.`);
	        }
	        // Browser
	        if (typeof TextDecoder !== 'undefined') {
	            if (bytes instanceof Uint8Array || bytes instanceof ArrayBuffer) {
	                this.textDecoder ?? (this.textDecoder = new TextDecoder('utf8'));
	                return this.textDecoder.decode(bytes);
	            }
	            throw new error_1.OpenAIError(`Unexpected: received non-Uint8Array/ArrayBuffer (${bytes.constructor.name}) in a web platform. Please report this error.`);
	        }
	        throw new error_1.OpenAIError(`Unexpected: neither Buffer nor TextDecoder are available as globals. Please report this error.`);
	    }
	    flush() {
	        if (!this.buffer.length && !this.trailingCR) {
	            return [];
	        }
	        const lines = [this.buffer.join('')];
	        this.buffer = [];
	        this.trailingCR = false;
	        return lines;
	    }
	}
	// prettier-ignore
	LineDecoder.NEWLINE_CHARS = new Set(['\n', '\r']);
	LineDecoder.NEWLINE_REGEXP = /\r\n|[\n\r]/g;
	/** This is an internal helper function that's just used for testing */
	function _decodeChunks(chunks) {
	    const decoder = new LineDecoder();
	    const lines = [];
	    for (const chunk of chunks) {
	        lines.push(...decoder.decode(chunk));
	    }
	    return lines;
	}
	streaming._decodeChunks = _decodeChunks;
	function partition(str, delimiter) {
	    const index = str.indexOf(delimiter);
	    if (index !== -1) {
	        return [str.substring(0, index), delimiter, str.substring(index + delimiter.length)];
	    }
	    return [str, '', ''];
	}
	/**
	 * Most browsers don't yet have async iterable support for ReadableStream,
	 * and Node has a very different way of reading bytes from its "ReadableStream".
	 *
	 * This polyfill was pulled from https://github.com/MattiasBuelens/web-streams-polyfill/pull/122#issuecomment-1627354490
	 */
	function readableStreamAsyncIterable(stream) {
	    if (stream[Symbol.asyncIterator])
	        return stream;
	    const reader = stream.getReader();
	    return {
	        async next() {
	            try {
	                const result = await reader.read();
	                if (result?.done)
	                    reader.releaseLock(); // release lock when stream becomes closed
	                return result;
	            }
	            catch (e) {
	                reader.releaseLock(); // release lock when stream becomes errored
	                throw e;
	            }
	        },
	        async return() {
	            const cancelPromise = reader.cancel();
	            reader.releaseLock();
	            await cancelPromise;
	            return { done: true, value: undefined };
	        },
	        [Symbol.asyncIterator]() {
	            return this;
	        },
	    };
	}
	streaming.readableStreamAsyncIterable = readableStreamAsyncIterable;
	
	return streaming;
}

var uploads$1 = {};

var hasRequiredUploads$1;

function requireUploads$1 () {
	if (hasRequiredUploads$1) return uploads$1;
	hasRequiredUploads$1 = 1;
	(function (exports) {
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.createForm = exports.multipartFormRequestOptions = exports.maybeMultipartFormRequestOptions = exports.isMultipartBody = exports.toFile = exports.isUploadable = exports.isBlobLike = exports.isFileLike = exports.isResponseLike = exports.fileFromPath = void 0;
		const index_1 = require_shims();
		var index_2 = require_shims();
		Object.defineProperty(exports, "fileFromPath", { enumerable: true, get: function () { return index_2.fileFromPath; } });
		const isResponseLike = (value) => value != null &&
		    typeof value === 'object' &&
		    typeof value.url === 'string' &&
		    typeof value.blob === 'function';
		exports.isResponseLike = isResponseLike;
		const isFileLike = (value) => value != null &&
		    typeof value === 'object' &&
		    typeof value.name === 'string' &&
		    typeof value.lastModified === 'number' &&
		    (0, exports.isBlobLike)(value);
		exports.isFileLike = isFileLike;
		/**
		 * The BlobLike type omits arrayBuffer() because @types/node-fetch@^2.6.4 lacks it; but this check
		 * adds the arrayBuffer() method type because it is available and used at runtime
		 */
		const isBlobLike = (value) => value != null &&
		    typeof value === 'object' &&
		    typeof value.size === 'number' &&
		    typeof value.type === 'string' &&
		    typeof value.text === 'function' &&
		    typeof value.slice === 'function' &&
		    typeof value.arrayBuffer === 'function';
		exports.isBlobLike = isBlobLike;
		const isUploadable = (value) => {
		    return (0, exports.isFileLike)(value) || (0, exports.isResponseLike)(value) || (0, index_1.isFsReadStream)(value);
		};
		exports.isUploadable = isUploadable;
		/**
		 * Helper for creating a {@link File} to pass to an SDK upload method from a variety of different data formats
		 * @param value the raw content of the file.  Can be an {@link Uploadable}, {@link BlobLikePart}, or {@link AsyncIterable} of {@link BlobLikePart}s
		 * @param {string=} name the name of the file. If omitted, toFile will try to determine a file name from bits if possible
		 * @param {Object=} options additional properties
		 * @param {string=} options.type the MIME type of the content
		 * @param {number=} options.lastModified the last modified timestamp
		 * @returns a {@link File} with the given properties
		 */
		async function toFile(value, name, options) {
		    // If it's a promise, resolve it.
		    value = await value;
		    // If we've been given a `File` we don't need to do anything
		    if ((0, exports.isFileLike)(value)) {
		        return value;
		    }
		    if ((0, exports.isResponseLike)(value)) {
		        const blob = await value.blob();
		        name || (name = new URL(value.url).pathname.split(/[\\/]/).pop() ?? 'unknown_file');
		        // we need to convert the `Blob` into an array buffer because the `Blob` class
		        // that `node-fetch` defines is incompatible with the web standard which results
		        // in `new File` interpreting it as a string instead of binary data.
		        const data = (0, exports.isBlobLike)(blob) ? [(await blob.arrayBuffer())] : [blob];
		        return new index_1.File(data, name, options);
		    }
		    const bits = await getBytes(value);
		    name || (name = getName(value) ?? 'unknown_file');
		    if (!options?.type) {
		        const type = bits[0]?.type;
		        if (typeof type === 'string') {
		            options = { ...options, type };
		        }
		    }
		    return new index_1.File(bits, name, options);
		}
		exports.toFile = toFile;
		async function getBytes(value) {
		    let parts = [];
		    if (typeof value === 'string' ||
		        ArrayBuffer.isView(value) || // includes Uint8Array, Buffer, etc.
		        value instanceof ArrayBuffer) {
		        parts.push(value);
		    }
		    else if ((0, exports.isBlobLike)(value)) {
		        parts.push(await value.arrayBuffer());
		    }
		    else if (isAsyncIterableIterator(value) // includes Readable, ReadableStream, etc.
		    ) {
		        for await (const chunk of value) {
		            parts.push(chunk); // TODO, consider validating?
		        }
		    }
		    else {
		        throw new Error(`Unexpected data type: ${typeof value}; constructor: ${value?.constructor
		            ?.name}; props: ${propsForError(value)}`);
		    }
		    return parts;
		}
		function propsForError(value) {
		    const props = Object.getOwnPropertyNames(value);
		    return `[${props.map((p) => `"${p}"`).join(', ')}]`;
		}
		function getName(value) {
		    return (getStringFromMaybeBuffer(value.name) ||
		        getStringFromMaybeBuffer(value.filename) ||
		        // For fs.ReadStream
		        getStringFromMaybeBuffer(value.path)?.split(/[\\/]/).pop());
		}
		const getStringFromMaybeBuffer = (x) => {
		    if (typeof x === 'string')
		        return x;
		    if (typeof Buffer !== 'undefined' && x instanceof Buffer)
		        return String(x);
		    return undefined;
		};
		const isAsyncIterableIterator = (value) => value != null && typeof value === 'object' && typeof value[Symbol.asyncIterator] === 'function';
		const isMultipartBody = (body) => body && typeof body === 'object' && body.body && body[Symbol.toStringTag] === 'MultipartBody';
		exports.isMultipartBody = isMultipartBody;
		/**
		 * Returns a multipart/form-data request if any part of the given request body contains a File / Blob value.
		 * Otherwise returns the request as is.
		 */
		const maybeMultipartFormRequestOptions = async (opts) => {
		    if (!hasUploadableValue(opts.body))
		        return opts;
		    const form = await (0, exports.createForm)(opts.body);
		    return (0, index_1.getMultipartRequestOptions)(form, opts);
		};
		exports.maybeMultipartFormRequestOptions = maybeMultipartFormRequestOptions;
		const multipartFormRequestOptions = async (opts) => {
		    const form = await (0, exports.createForm)(opts.body);
		    return (0, index_1.getMultipartRequestOptions)(form, opts);
		};
		exports.multipartFormRequestOptions = multipartFormRequestOptions;
		const createForm = async (body) => {
		    const form = new index_1.FormData();
		    await Promise.all(Object.entries(body || {}).map(([key, value]) => addFormValue(form, key, value)));
		    return form;
		};
		exports.createForm = createForm;
		const hasUploadableValue = (value) => {
		    if ((0, exports.isUploadable)(value))
		        return true;
		    if (Array.isArray(value))
		        return value.some(hasUploadableValue);
		    if (value && typeof value === 'object') {
		        for (const k in value) {
		            if (hasUploadableValue(value[k]))
		                return true;
		        }
		    }
		    return false;
		};
		const addFormValue = async (form, key, value) => {
		    if (value === undefined)
		        return;
		    if (value == null) {
		        throw new TypeError(`Received null for "${key}"; to pass null in FormData, you must use the string 'null'`);
		    }
		    // TODO: make nested formats configurable
		    if (typeof value === 'string' || typeof value === 'number' || typeof value === 'boolean') {
		        form.append(key, String(value));
		    }
		    else if ((0, exports.isUploadable)(value)) {
		        const file = await toFile(value);
		        form.append(key, file);
		    }
		    else if (Array.isArray(value)) {
		        await Promise.all(value.map((entry) => addFormValue(form, key + '[]', entry)));
		    }
		    else if (typeof value === 'object') {
		        await Promise.all(Object.entries(value).map(([name, prop]) => addFormValue(form, `${key}[${name}]`, prop)));
		    }
		    else {
		        throw new TypeError(`Invalid value given to form, expected a string, number, boolean, object, Array, File or Blob but got ${value} instead`);
		    }
		};
		
	} (uploads$1));
	return uploads$1;
}

var hasRequiredCore;

function requireCore () {
	if (hasRequiredCore) return core;
	hasRequiredCore = 1;
	(function (exports) {
		var __classPrivateFieldSet = (core && core.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
		    if (kind === "m") throw new TypeError("Private method is not writable");
		    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
		    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
		    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
		};
		var __classPrivateFieldGet = (core && core.__classPrivateFieldGet) || function (receiver, state, kind, f) {
		    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
		    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
		    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
		};
		var _AbstractPage_client;
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.isObj = exports.toBase64 = exports.getHeader = exports.getRequiredHeader = exports.isHeadersProtocol = exports.isRunningInBrowser = exports.debug = exports.hasOwn = exports.isEmptyObj = exports.maybeCoerceBoolean = exports.maybeCoerceFloat = exports.maybeCoerceInteger = exports.coerceBoolean = exports.coerceFloat = exports.coerceInteger = exports.readEnv = exports.ensurePresent = exports.castToError = exports.sleep = exports.safeJSON = exports.isRequestOptions = exports.createResponseHeaders = exports.PagePromise = exports.AbstractPage = exports.APIClient = exports.APIPromise = exports.createForm = exports.multipartFormRequestOptions = exports.maybeMultipartFormRequestOptions = void 0;
		const version_1 = /*@__PURE__*/ requireVersion$1();
		const streaming_1 = /*@__PURE__*/ requireStreaming();
		const error_1 = /*@__PURE__*/ requireError();
		const index_1 = require_shims();
		const uploads_1 = /*@__PURE__*/ requireUploads$1();
		var uploads_2 = /*@__PURE__*/ requireUploads$1();
		Object.defineProperty(exports, "maybeMultipartFormRequestOptions", { enumerable: true, get: function () { return uploads_2.maybeMultipartFormRequestOptions; } });
		Object.defineProperty(exports, "multipartFormRequestOptions", { enumerable: true, get: function () { return uploads_2.multipartFormRequestOptions; } });
		Object.defineProperty(exports, "createForm", { enumerable: true, get: function () { return uploads_2.createForm; } });
		async function defaultParseResponse(props) {
		    const { response } = props;
		    if (props.options.stream) {
		        debug('response', response.status, response.url, response.headers, response.body);
		        // Note: there is an invariant here that isn't represented in the type system
		        // that if you set `stream: true` the response type must also be `Stream<T>`
		        if (props.options.__streamClass) {
		            return props.options.__streamClass.fromSSEResponse(response, props.controller);
		        }
		        return streaming_1.Stream.fromSSEResponse(response, props.controller);
		    }
		    // fetch refuses to read the body when the status code is 204.
		    if (response.status === 204) {
		        return null;
		    }
		    if (props.options.__binaryResponse) {
		        return response;
		    }
		    const contentType = response.headers.get('content-type');
		    const isJSON = contentType?.includes('application/json') || contentType?.includes('application/vnd.api+json');
		    if (isJSON) {
		        const json = await response.json();
		        debug('response', response.status, response.url, response.headers, json);
		        return _addRequestID(json, response);
		    }
		    const text = await response.text();
		    debug('response', response.status, response.url, response.headers, text);
		    // TODO handle blob, arraybuffer, other content types, etc.
		    return text;
		}
		function _addRequestID(value, response) {
		    if (!value || typeof value !== 'object' || Array.isArray(value)) {
		        return value;
		    }
		    return Object.defineProperty(value, '_request_id', {
		        value: response.headers.get('x-request-id'),
		        enumerable: false,
		    });
		}
		/**
		 * A subclass of `Promise` providing additional helper methods
		 * for interacting with the SDK.
		 */
		class APIPromise extends Promise {
		    constructor(responsePromise, parseResponse = defaultParseResponse) {
		        super((resolve) => {
		            // this is maybe a bit weird but this has to be a no-op to not implicitly
		            // parse the response body; instead .then, .catch, .finally are overridden
		            // to parse the response
		            resolve(null);
		        });
		        this.responsePromise = responsePromise;
		        this.parseResponse = parseResponse;
		    }
		    _thenUnwrap(transform) {
		        return new APIPromise(this.responsePromise, async (props) => _addRequestID(transform(await this.parseResponse(props)), props.response));
		    }
		    /**
		     * Gets the raw `Response` instance instead of parsing the response
		     * data.
		     *
		     * If you want to parse the response body but still get the `Response`
		     * instance, you can use {@link withResponse()}.
		     *
		     * 👋 Getting the wrong TypeScript type for `Response`?
		     * Try setting `"moduleResolution": "NodeNext"` if you can,
		     * or add one of these imports before your first `import … from 'openai'`:
		     * - `import 'openai/shims/node'` (if you're running on Node)
		     * - `import 'openai/shims/web'` (otherwise)
		     */
		    asResponse() {
		        return this.responsePromise.then((p) => p.response);
		    }
		    /**
		     * Gets the parsed response data and the raw `Response` instance.
		     *
		     * If you just want to get the raw `Response` instance without parsing it,
		     * you can use {@link asResponse()}.
		     *
		     *
		     * 👋 Getting the wrong TypeScript type for `Response`?
		     * Try setting `"moduleResolution": "NodeNext"` if you can,
		     * or add one of these imports before your first `import … from 'openai'`:
		     * - `import 'openai/shims/node'` (if you're running on Node)
		     * - `import 'openai/shims/web'` (otherwise)
		     */
		    async withResponse() {
		        const [data, response] = await Promise.all([this.parse(), this.asResponse()]);
		        return { data, response };
		    }
		    parse() {
		        if (!this.parsedPromise) {
		            this.parsedPromise = this.responsePromise.then(this.parseResponse);
		        }
		        return this.parsedPromise;
		    }
		    then(onfulfilled, onrejected) {
		        return this.parse().then(onfulfilled, onrejected);
		    }
		    catch(onrejected) {
		        return this.parse().catch(onrejected);
		    }
		    finally(onfinally) {
		        return this.parse().finally(onfinally);
		    }
		}
		exports.APIPromise = APIPromise;
		class APIClient {
		    constructor({ baseURL, maxRetries = 2, timeout = 600000, // 10 minutes
		    httpAgent, fetch: overridenFetch, }) {
		        this.baseURL = baseURL;
		        this.maxRetries = validatePositiveInteger('maxRetries', maxRetries);
		        this.timeout = validatePositiveInteger('timeout', timeout);
		        this.httpAgent = httpAgent;
		        this.fetch = overridenFetch ?? index_1.fetch;
		    }
		    authHeaders(opts) {
		        return {};
		    }
		    /**
		     * Override this to add your own default headers, for example:
		     *
		     *  {
		     *    ...super.defaultHeaders(),
		     *    Authorization: 'Bearer 123',
		     *  }
		     */
		    defaultHeaders(opts) {
		        return {
		            Accept: 'application/json',
		            'Content-Type': 'application/json',
		            'User-Agent': this.getUserAgent(),
		            ...getPlatformHeaders(),
		            ...this.authHeaders(opts),
		        };
		    }
		    /**
		     * Override this to add your own headers validation:
		     */
		    validateHeaders(headers, customHeaders) { }
		    defaultIdempotencyKey() {
		        return `stainless-node-retry-${uuid4()}`;
		    }
		    get(path, opts) {
		        return this.methodRequest('get', path, opts);
		    }
		    post(path, opts) {
		        return this.methodRequest('post', path, opts);
		    }
		    patch(path, opts) {
		        return this.methodRequest('patch', path, opts);
		    }
		    put(path, opts) {
		        return this.methodRequest('put', path, opts);
		    }
		    delete(path, opts) {
		        return this.methodRequest('delete', path, opts);
		    }
		    methodRequest(method, path, opts) {
		        return this.request(Promise.resolve(opts).then(async (opts) => {
		            const body = opts && (0, uploads_1.isBlobLike)(opts?.body) ? new DataView(await opts.body.arrayBuffer())
		                : opts?.body instanceof DataView ? opts.body
		                    : opts?.body instanceof ArrayBuffer ? new DataView(opts.body)
		                        : opts && ArrayBuffer.isView(opts?.body) ? new DataView(opts.body.buffer)
		                            : opts?.body;
		            return { method, path, ...opts, body };
		        }));
		    }
		    getAPIList(path, Page, opts) {
		        return this.requestAPIList(Page, { method: 'get', path, ...opts });
		    }
		    calculateContentLength(body) {
		        if (typeof body === 'string') {
		            if (typeof Buffer !== 'undefined') {
		                return Buffer.byteLength(body, 'utf8').toString();
		            }
		            if (typeof TextEncoder !== 'undefined') {
		                const encoder = new TextEncoder();
		                const encoded = encoder.encode(body);
		                return encoded.length.toString();
		            }
		        }
		        else if (ArrayBuffer.isView(body)) {
		            return body.byteLength.toString();
		        }
		        return null;
		    }
		    buildRequest(options, { retryCount = 0 } = {}) {
		        const { method, path, query, headers: headers = {} } = options;
		        const body = ArrayBuffer.isView(options.body) || (options.__binaryRequest && typeof options.body === 'string') ?
		            options.body
		            : (0, uploads_1.isMultipartBody)(options.body) ? options.body.body
		                : options.body ? JSON.stringify(options.body, null, 2)
		                    : null;
		        const contentLength = this.calculateContentLength(body);
		        const url = this.buildURL(path, query);
		        if ('timeout' in options)
		            validatePositiveInteger('timeout', options.timeout);
		        const timeout = options.timeout ?? this.timeout;
		        const httpAgent = options.httpAgent ?? this.httpAgent ?? (0, index_1.getDefaultAgent)(url);
		        const minAgentTimeout = timeout + 1000;
		        if (typeof httpAgent?.options?.timeout === 'number' &&
		            minAgentTimeout > (httpAgent.options.timeout ?? 0)) {
		            // Allow any given request to bump our agent active socket timeout.
		            // This may seem strange, but leaking active sockets should be rare and not particularly problematic,
		            // and without mutating agent we would need to create more of them.
		            // This tradeoff optimizes for performance.
		            httpAgent.options.timeout = minAgentTimeout;
		        }
		        if (this.idempotencyHeader && method !== 'get') {
		            if (!options.idempotencyKey)
		                options.idempotencyKey = this.defaultIdempotencyKey();
		            headers[this.idempotencyHeader] = options.idempotencyKey;
		        }
		        const reqHeaders = this.buildHeaders({ options, headers, contentLength, retryCount });
		        const req = {
		            method,
		            ...(body && { body: body }),
		            headers: reqHeaders,
		            ...(httpAgent && { agent: httpAgent }),
		            // @ts-ignore node-fetch uses a custom AbortSignal type that is
		            // not compatible with standard web types
		            signal: options.signal ?? null,
		        };
		        return { req, url, timeout };
		    }
		    buildHeaders({ options, headers, contentLength, retryCount, }) {
		        const reqHeaders = {};
		        if (contentLength) {
		            reqHeaders['content-length'] = contentLength;
		        }
		        const defaultHeaders = this.defaultHeaders(options);
		        applyHeadersMut(reqHeaders, defaultHeaders);
		        applyHeadersMut(reqHeaders, headers);
		        // let builtin fetch set the Content-Type for multipart bodies
		        if ((0, uploads_1.isMultipartBody)(options.body) && index_1.kind !== 'node') {
		            delete reqHeaders['content-type'];
		        }
		        // Don't set the retry count header if it was already set or removed by the caller. We check `headers`,
		        // which can contain nulls, instead of `reqHeaders` to account for the removal case.
		        if ((0, exports.getHeader)(headers, 'x-stainless-retry-count') === undefined) {
		            reqHeaders['x-stainless-retry-count'] = String(retryCount);
		        }
		        this.validateHeaders(reqHeaders, headers);
		        return reqHeaders;
		    }
		    /**
		     * Used as a callback for mutating the given `FinalRequestOptions` object.
		     */
		    async prepareOptions(options) { }
		    /**
		     * Used as a callback for mutating the given `RequestInit` object.
		     *
		     * This is useful for cases where you want to add certain headers based off of
		     * the request properties, e.g. `method` or `url`.
		     */
		    async prepareRequest(request, { url, options }) { }
		    parseHeaders(headers) {
		        return (!headers ? {}
		            : Symbol.iterator in headers ?
		                Object.fromEntries(Array.from(headers).map((header) => [...header]))
		                : { ...headers });
		    }
		    makeStatusError(status, error, message, headers) {
		        return error_1.APIError.generate(status, error, message, headers);
		    }
		    request(options, remainingRetries = null) {
		        return new APIPromise(this.makeRequest(options, remainingRetries));
		    }
		    async makeRequest(optionsInput, retriesRemaining) {
		        const options = await optionsInput;
		        const maxRetries = options.maxRetries ?? this.maxRetries;
		        if (retriesRemaining == null) {
		            retriesRemaining = maxRetries;
		        }
		        await this.prepareOptions(options);
		        const { req, url, timeout } = this.buildRequest(options, { retryCount: maxRetries - retriesRemaining });
		        await this.prepareRequest(req, { url, options });
		        debug('request', url, options, req.headers);
		        if (options.signal?.aborted) {
		            throw new error_1.APIUserAbortError();
		        }
		        const controller = new AbortController();
		        const response = await this.fetchWithTimeout(url, req, timeout, controller).catch(exports.castToError);
		        if (response instanceof Error) {
		            if (options.signal?.aborted) {
		                throw new error_1.APIUserAbortError();
		            }
		            if (retriesRemaining) {
		                return this.retryRequest(options, retriesRemaining);
		            }
		            if (response.name === 'AbortError') {
		                throw new error_1.APIConnectionTimeoutError();
		            }
		            throw new error_1.APIConnectionError({ cause: response });
		        }
		        const responseHeaders = (0, exports.createResponseHeaders)(response.headers);
		        if (!response.ok) {
		            if (retriesRemaining && this.shouldRetry(response)) {
		                const retryMessage = `retrying, ${retriesRemaining} attempts remaining`;
		                debug(`response (error; ${retryMessage})`, response.status, url, responseHeaders);
		                return this.retryRequest(options, retriesRemaining, responseHeaders);
		            }
		            const errText = await response.text().catch((e) => (0, exports.castToError)(e).message);
		            const errJSON = (0, exports.safeJSON)(errText);
		            const errMessage = errJSON ? undefined : errText;
		            const retryMessage = retriesRemaining ? `(error; no more retries left)` : `(error; not retryable)`;
		            debug(`response (error; ${retryMessage})`, response.status, url, responseHeaders, errMessage);
		            const err = this.makeStatusError(response.status, errJSON, errMessage, responseHeaders);
		            throw err;
		        }
		        return { response, options, controller };
		    }
		    requestAPIList(Page, options) {
		        const request = this.makeRequest(options, null);
		        return new PagePromise(this, request, Page);
		    }
		    buildURL(path, query) {
		        const url = isAbsoluteURL(path) ?
		            new URL(path)
		            : new URL(this.baseURL + (this.baseURL.endsWith('/') && path.startsWith('/') ? path.slice(1) : path));
		        const defaultQuery = this.defaultQuery();
		        if (!isEmptyObj(defaultQuery)) {
		            query = { ...defaultQuery, ...query };
		        }
		        if (typeof query === 'object' && query && !Array.isArray(query)) {
		            url.search = this.stringifyQuery(query);
		        }
		        return url.toString();
		    }
		    stringifyQuery(query) {
		        return Object.entries(query)
		            .filter(([_, value]) => typeof value !== 'undefined')
		            .map(([key, value]) => {
		            if (typeof value === 'string' || typeof value === 'number' || typeof value === 'boolean') {
		                return `${encodeURIComponent(key)}=${encodeURIComponent(value)}`;
		            }
		            if (value === null) {
		                return `${encodeURIComponent(key)}=`;
		            }
		            throw new error_1.OpenAIError(`Cannot stringify type ${typeof value}; Expected string, number, boolean, or null. If you need to pass nested query parameters, you can manually encode them, e.g. { query: { 'foo[key1]': value1, 'foo[key2]': value2 } }, and please open a GitHub issue requesting better support for your use case.`);
		        })
		            .join('&');
		    }
		    async fetchWithTimeout(url, init, ms, controller) {
		        const { signal, ...options } = init || {};
		        if (signal)
		            signal.addEventListener('abort', () => controller.abort());
		        const timeout = setTimeout(() => controller.abort(), ms);
		        return (this.getRequestClient()
		            // use undefined this binding; fetch errors if bound to something else in browser/cloudflare
		            .fetch.call(undefined, url, { signal: controller.signal, ...options })
		            .finally(() => {
		            clearTimeout(timeout);
		        }));
		    }
		    getRequestClient() {
		        return { fetch: this.fetch };
		    }
		    shouldRetry(response) {
		        // Note this is not a standard header.
		        const shouldRetryHeader = response.headers.get('x-should-retry');
		        // If the server explicitly says whether or not to retry, obey.
		        if (shouldRetryHeader === 'true')
		            return true;
		        if (shouldRetryHeader === 'false')
		            return false;
		        // Retry on request timeouts.
		        if (response.status === 408)
		            return true;
		        // Retry on lock timeouts.
		        if (response.status === 409)
		            return true;
		        // Retry on rate limits.
		        if (response.status === 429)
		            return true;
		        // Retry internal errors.
		        if (response.status >= 500)
		            return true;
		        return false;
		    }
		    async retryRequest(options, retriesRemaining, responseHeaders) {
		        let timeoutMillis;
		        // Note the `retry-after-ms` header may not be standard, but is a good idea and we'd like proactive support for it.
		        const retryAfterMillisHeader = responseHeaders?.['retry-after-ms'];
		        if (retryAfterMillisHeader) {
		            const timeoutMs = parseFloat(retryAfterMillisHeader);
		            if (!Number.isNaN(timeoutMs)) {
		                timeoutMillis = timeoutMs;
		            }
		        }
		        // About the Retry-After header: https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Retry-After
		        const retryAfterHeader = responseHeaders?.['retry-after'];
		        if (retryAfterHeader && !timeoutMillis) {
		            const timeoutSeconds = parseFloat(retryAfterHeader);
		            if (!Number.isNaN(timeoutSeconds)) {
		                timeoutMillis = timeoutSeconds * 1000;
		            }
		            else {
		                timeoutMillis = Date.parse(retryAfterHeader) - Date.now();
		            }
		        }
		        // If the API asks us to wait a certain amount of time (and it's a reasonable amount),
		        // just do what it says, but otherwise calculate a default
		        if (!(timeoutMillis && 0 <= timeoutMillis && timeoutMillis < 60 * 1000)) {
		            const maxRetries = options.maxRetries ?? this.maxRetries;
		            timeoutMillis = this.calculateDefaultRetryTimeoutMillis(retriesRemaining, maxRetries);
		        }
		        await (0, exports.sleep)(timeoutMillis);
		        return this.makeRequest(options, retriesRemaining - 1);
		    }
		    calculateDefaultRetryTimeoutMillis(retriesRemaining, maxRetries) {
		        const initialRetryDelay = 0.5;
		        const maxRetryDelay = 8.0;
		        const numRetries = maxRetries - retriesRemaining;
		        // Apply exponential backoff, but not more than the max.
		        const sleepSeconds = Math.min(initialRetryDelay * Math.pow(2, numRetries), maxRetryDelay);
		        // Apply some jitter, take up to at most 25 percent of the retry time.
		        const jitter = 1 - Math.random() * 0.25;
		        return sleepSeconds * jitter * 1000;
		    }
		    getUserAgent() {
		        return `${this.constructor.name}/JS ${version_1.VERSION}`;
		    }
		}
		exports.APIClient = APIClient;
		class AbstractPage {
		    constructor(client, response, body, options) {
		        _AbstractPage_client.set(this, void 0);
		        __classPrivateFieldSet(this, _AbstractPage_client, client, "f");
		        this.options = options;
		        this.response = response;
		        this.body = body;
		    }
		    hasNextPage() {
		        const items = this.getPaginatedItems();
		        if (!items.length)
		            return false;
		        return this.nextPageInfo() != null;
		    }
		    async getNextPage() {
		        const nextInfo = this.nextPageInfo();
		        if (!nextInfo) {
		            throw new error_1.OpenAIError('No next page expected; please check `.hasNextPage()` before calling `.getNextPage()`.');
		        }
		        const nextOptions = { ...this.options };
		        if ('params' in nextInfo && typeof nextOptions.query === 'object') {
		            nextOptions.query = { ...nextOptions.query, ...nextInfo.params };
		        }
		        else if ('url' in nextInfo) {
		            const params = [...Object.entries(nextOptions.query || {}), ...nextInfo.url.searchParams.entries()];
		            for (const [key, value] of params) {
		                nextInfo.url.searchParams.set(key, value);
		            }
		            nextOptions.query = undefined;
		            nextOptions.path = nextInfo.url.toString();
		        }
		        return await __classPrivateFieldGet(this, _AbstractPage_client, "f").requestAPIList(this.constructor, nextOptions);
		    }
		    async *iterPages() {
		        // eslint-disable-next-line @typescript-eslint/no-this-alias
		        let page = this;
		        yield page;
		        while (page.hasNextPage()) {
		            page = await page.getNextPage();
		            yield page;
		        }
		    }
		    async *[(_AbstractPage_client = new WeakMap(), Symbol.asyncIterator)]() {
		        for await (const page of this.iterPages()) {
		            for (const item of page.getPaginatedItems()) {
		                yield item;
		            }
		        }
		    }
		}
		exports.AbstractPage = AbstractPage;
		/**
		 * This subclass of Promise will resolve to an instantiated Page once the request completes.
		 *
		 * It also implements AsyncIterable to allow auto-paginating iteration on an unawaited list call, eg:
		 *
		 *    for await (const item of client.items.list()) {
		 *      console.log(item)
		 *    }
		 */
		class PagePromise extends APIPromise {
		    constructor(client, request, Page) {
		        super(request, async (props) => new Page(client, props.response, await defaultParseResponse(props), props.options));
		    }
		    /**
		     * Allow auto-paginating iteration on an unawaited list call, eg:
		     *
		     *    for await (const item of client.items.list()) {
		     *      console.log(item)
		     *    }
		     */
		    async *[Symbol.asyncIterator]() {
		        const page = await this;
		        for await (const item of page) {
		            yield item;
		        }
		    }
		}
		exports.PagePromise = PagePromise;
		const createResponseHeaders = (headers) => {
		    return new Proxy(Object.fromEntries(
		    // @ts-ignore
		    headers.entries()), {
		        get(target, name) {
		            const key = name.toString();
		            return target[key.toLowerCase()] || target[key];
		        },
		    });
		};
		exports.createResponseHeaders = createResponseHeaders;
		// This is required so that we can determine if a given object matches the RequestOptions
		// type at runtime. While this requires duplication, it is enforced by the TypeScript
		// compiler such that any missing / extraneous keys will cause an error.
		const requestOptionsKeys = {
		    method: true,
		    path: true,
		    query: true,
		    body: true,
		    headers: true,
		    maxRetries: true,
		    stream: true,
		    timeout: true,
		    httpAgent: true,
		    signal: true,
		    idempotencyKey: true,
		    __binaryRequest: true,
		    __binaryResponse: true,
		    __streamClass: true,
		};
		const isRequestOptions = (obj) => {
		    return (typeof obj === 'object' &&
		        obj !== null &&
		        !isEmptyObj(obj) &&
		        Object.keys(obj).every((k) => hasOwn(requestOptionsKeys, k)));
		};
		exports.isRequestOptions = isRequestOptions;
		const getPlatformProperties = () => {
		    if (typeof Deno !== 'undefined' && Deno.build != null) {
		        return {
		            'X-Stainless-Lang': 'js',
		            'X-Stainless-Package-Version': version_1.VERSION,
		            'X-Stainless-OS': normalizePlatform(Deno.build.os),
		            'X-Stainless-Arch': normalizeArch(Deno.build.arch),
		            'X-Stainless-Runtime': 'deno',
		            'X-Stainless-Runtime-Version': typeof Deno.version === 'string' ? Deno.version : Deno.version?.deno ?? 'unknown',
		        };
		    }
		    if (typeof EdgeRuntime !== 'undefined') {
		        return {
		            'X-Stainless-Lang': 'js',
		            'X-Stainless-Package-Version': version_1.VERSION,
		            'X-Stainless-OS': 'Unknown',
		            'X-Stainless-Arch': `other:${EdgeRuntime}`,
		            'X-Stainless-Runtime': 'edge',
		            'X-Stainless-Runtime-Version': process.version,
		        };
		    }
		    // Check if Node.js
		    if (Object.prototype.toString.call(typeof process !== 'undefined' ? process : 0) === '[object process]') {
		        return {
		            'X-Stainless-Lang': 'js',
		            'X-Stainless-Package-Version': version_1.VERSION,
		            'X-Stainless-OS': normalizePlatform(process.platform),
		            'X-Stainless-Arch': normalizeArch(process.arch),
		            'X-Stainless-Runtime': 'node',
		            'X-Stainless-Runtime-Version': process.version,
		        };
		    }
		    const browserInfo = getBrowserInfo();
		    if (browserInfo) {
		        return {
		            'X-Stainless-Lang': 'js',
		            'X-Stainless-Package-Version': version_1.VERSION,
		            'X-Stainless-OS': 'Unknown',
		            'X-Stainless-Arch': 'unknown',
		            'X-Stainless-Runtime': `browser:${browserInfo.browser}`,
		            'X-Stainless-Runtime-Version': browserInfo.version,
		        };
		    }
		    // TODO add support for Cloudflare workers, etc.
		    return {
		        'X-Stainless-Lang': 'js',
		        'X-Stainless-Package-Version': version_1.VERSION,
		        'X-Stainless-OS': 'Unknown',
		        'X-Stainless-Arch': 'unknown',
		        'X-Stainless-Runtime': 'unknown',
		        'X-Stainless-Runtime-Version': 'unknown',
		    };
		};
		// Note: modified from https://github.com/JS-DevTools/host-environment/blob/b1ab79ecde37db5d6e163c050e54fe7d287d7c92/src/isomorphic.browser.ts
		function getBrowserInfo() {
		    if (typeof navigator === 'undefined' || !navigator) {
		        return null;
		    }
		    // NOTE: The order matters here!
		    const browserPatterns = [
		        { key: 'edge', pattern: /Edge(?:\W+(\d+)\.(\d+)(?:\.(\d+))?)?/ },
		        { key: 'ie', pattern: /MSIE(?:\W+(\d+)\.(\d+)(?:\.(\d+))?)?/ },
		        { key: 'ie', pattern: /Trident(?:.*rv\:(\d+)\.(\d+)(?:\.(\d+))?)?/ },
		        { key: 'chrome', pattern: /Chrome(?:\W+(\d+)\.(\d+)(?:\.(\d+))?)?/ },
		        { key: 'firefox', pattern: /Firefox(?:\W+(\d+)\.(\d+)(?:\.(\d+))?)?/ },
		        { key: 'safari', pattern: /(?:Version\W+(\d+)\.(\d+)(?:\.(\d+))?)?(?:\W+Mobile\S*)?\W+Safari/ },
		    ];
		    // Find the FIRST matching browser
		    for (const { key, pattern } of browserPatterns) {
		        const match = pattern.exec(navigator.userAgent);
		        if (match) {
		            const major = match[1] || 0;
		            const minor = match[2] || 0;
		            const patch = match[3] || 0;
		            return { browser: key, version: `${major}.${minor}.${patch}` };
		        }
		    }
		    return null;
		}
		const normalizeArch = (arch) => {
		    // Node docs:
		    // - https://nodejs.org/api/process.html#processarch
		    // Deno docs:
		    // - https://doc.deno.land/deno/stable/~/Deno.build
		    if (arch === 'x32')
		        return 'x32';
		    if (arch === 'x86_64' || arch === 'x64')
		        return 'x64';
		    if (arch === 'arm')
		        return 'arm';
		    if (arch === 'aarch64' || arch === 'arm64')
		        return 'arm64';
		    if (arch)
		        return `other:${arch}`;
		    return 'unknown';
		};
		const normalizePlatform = (platform) => {
		    // Node platforms:
		    // - https://nodejs.org/api/process.html#processplatform
		    // Deno platforms:
		    // - https://doc.deno.land/deno/stable/~/Deno.build
		    // - https://github.com/denoland/deno/issues/14799
		    platform = platform.toLowerCase();
		    // NOTE: this iOS check is untested and may not work
		    // Node does not work natively on IOS, there is a fork at
		    // https://github.com/nodejs-mobile/nodejs-mobile
		    // however it is unknown at the time of writing how to detect if it is running
		    if (platform.includes('ios'))
		        return 'iOS';
		    if (platform === 'android')
		        return 'Android';
		    if (platform === 'darwin')
		        return 'MacOS';
		    if (platform === 'win32')
		        return 'Windows';
		    if (platform === 'freebsd')
		        return 'FreeBSD';
		    if (platform === 'openbsd')
		        return 'OpenBSD';
		    if (platform === 'linux')
		        return 'Linux';
		    if (platform)
		        return `Other:${platform}`;
		    return 'Unknown';
		};
		let _platformHeaders;
		const getPlatformHeaders = () => {
		    return (_platformHeaders ?? (_platformHeaders = getPlatformProperties()));
		};
		const safeJSON = (text) => {
		    try {
		        return JSON.parse(text);
		    }
		    catch (err) {
		        return undefined;
		    }
		};
		exports.safeJSON = safeJSON;
		// https://stackoverflow.com/a/19709846
		const startsWithSchemeRegexp = new RegExp('^(?:[a-z]+:)?//', 'i');
		const isAbsoluteURL = (url) => {
		    return startsWithSchemeRegexp.test(url);
		};
		const sleep = (ms) => new Promise((resolve) => setTimeout(resolve, ms));
		exports.sleep = sleep;
		const validatePositiveInteger = (name, n) => {
		    if (typeof n !== 'number' || !Number.isInteger(n)) {
		        throw new error_1.OpenAIError(`${name} must be an integer`);
		    }
		    if (n < 0) {
		        throw new error_1.OpenAIError(`${name} must be a positive integer`);
		    }
		    return n;
		};
		const castToError = (err) => {
		    if (err instanceof Error)
		        return err;
		    if (typeof err === 'object' && err !== null) {
		        try {
		            return new Error(JSON.stringify(err));
		        }
		        catch { }
		    }
		    return new Error(err);
		};
		exports.castToError = castToError;
		const ensurePresent = (value) => {
		    if (value == null)
		        throw new error_1.OpenAIError(`Expected a value to be given but received ${value} instead.`);
		    return value;
		};
		exports.ensurePresent = ensurePresent;
		/**
		 * Read an environment variable.
		 *
		 * Trims beginning and trailing whitespace.
		 *
		 * Will return undefined if the environment variable doesn't exist or cannot be accessed.
		 */
		const readEnv = (env) => {
		    if (typeof process !== 'undefined') {
		        return process.env?.[env]?.trim() ?? undefined;
		    }
		    if (typeof Deno !== 'undefined') {
		        return Deno.env?.get?.(env)?.trim();
		    }
		    return undefined;
		};
		exports.readEnv = readEnv;
		const coerceInteger = (value) => {
		    if (typeof value === 'number')
		        return Math.round(value);
		    if (typeof value === 'string')
		        return parseInt(value, 10);
		    throw new error_1.OpenAIError(`Could not coerce ${value} (type: ${typeof value}) into a number`);
		};
		exports.coerceInteger = coerceInteger;
		const coerceFloat = (value) => {
		    if (typeof value === 'number')
		        return value;
		    if (typeof value === 'string')
		        return parseFloat(value);
		    throw new error_1.OpenAIError(`Could not coerce ${value} (type: ${typeof value}) into a number`);
		};
		exports.coerceFloat = coerceFloat;
		const coerceBoolean = (value) => {
		    if (typeof value === 'boolean')
		        return value;
		    if (typeof value === 'string')
		        return value === 'true';
		    return Boolean(value);
		};
		exports.coerceBoolean = coerceBoolean;
		const maybeCoerceInteger = (value) => {
		    if (value === undefined) {
		        return undefined;
		    }
		    return (0, exports.coerceInteger)(value);
		};
		exports.maybeCoerceInteger = maybeCoerceInteger;
		const maybeCoerceFloat = (value) => {
		    if (value === undefined) {
		        return undefined;
		    }
		    return (0, exports.coerceFloat)(value);
		};
		exports.maybeCoerceFloat = maybeCoerceFloat;
		const maybeCoerceBoolean = (value) => {
		    if (value === undefined) {
		        return undefined;
		    }
		    return (0, exports.coerceBoolean)(value);
		};
		exports.maybeCoerceBoolean = maybeCoerceBoolean;
		// https://stackoverflow.com/a/34491287
		function isEmptyObj(obj) {
		    if (!obj)
		        return true;
		    for (const _k in obj)
		        return false;
		    return true;
		}
		exports.isEmptyObj = isEmptyObj;
		// https://eslint.org/docs/latest/rules/no-prototype-builtins
		function hasOwn(obj, key) {
		    return Object.prototype.hasOwnProperty.call(obj, key);
		}
		exports.hasOwn = hasOwn;
		/**
		 * Copies headers from "newHeaders" onto "targetHeaders",
		 * using lower-case for all properties,
		 * ignoring any keys with undefined values,
		 * and deleting any keys with null values.
		 */
		function applyHeadersMut(targetHeaders, newHeaders) {
		    for (const k in newHeaders) {
		        if (!hasOwn(newHeaders, k))
		            continue;
		        const lowerKey = k.toLowerCase();
		        if (!lowerKey)
		            continue;
		        const val = newHeaders[k];
		        if (val === null) {
		            delete targetHeaders[lowerKey];
		        }
		        else if (val !== undefined) {
		            targetHeaders[lowerKey] = val;
		        }
		    }
		}
		function debug(action, ...args) {
		    if (typeof process !== 'undefined' && process?.env?.['DEBUG'] === 'true') {
		        console.log(`OpenAI:DEBUG:${action}`, ...args);
		    }
		}
		exports.debug = debug;
		/**
		 * https://stackoverflow.com/a/2117523
		 */
		const uuid4 = () => {
		    return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, (c) => {
		        const r = (Math.random() * 16) | 0;
		        const v = c === 'x' ? r : (r & 0x3) | 0x8;
		        return v.toString(16);
		    });
		};
		const isRunningInBrowser = () => {
		    return (
		    // @ts-ignore
		    typeof window !== 'undefined' &&
		        // @ts-ignore
		        typeof window.document !== 'undefined' &&
		        // @ts-ignore
		        typeof navigator !== 'undefined');
		};
		exports.isRunningInBrowser = isRunningInBrowser;
		const isHeadersProtocol = (headers) => {
		    return typeof headers?.get === 'function';
		};
		exports.isHeadersProtocol = isHeadersProtocol;
		const getRequiredHeader = (headers, header) => {
		    const foundHeader = (0, exports.getHeader)(headers, header);
		    if (foundHeader === undefined) {
		        throw new Error(`Could not find ${header} header`);
		    }
		    return foundHeader;
		};
		exports.getRequiredHeader = getRequiredHeader;
		const getHeader = (headers, header) => {
		    const lowerCasedHeader = header.toLowerCase();
		    if ((0, exports.isHeadersProtocol)(headers)) {
		        // to deal with the case where the header looks like Stainless-Event-Id
		        const intercapsHeader = header[0]?.toUpperCase() +
		            header.substring(1).replace(/([^\w])(\w)/g, (_m, g1, g2) => g1 + g2.toUpperCase());
		        for (const key of [header, lowerCasedHeader, header.toUpperCase(), intercapsHeader]) {
		            const value = headers.get(key);
		            if (value) {
		                return value;
		            }
		        }
		    }
		    for (const [key, value] of Object.entries(headers)) {
		        if (key.toLowerCase() === lowerCasedHeader) {
		            if (Array.isArray(value)) {
		                if (value.length <= 1)
		                    return value[0];
		                console.warn(`Received ${value.length} entries for the ${header} header, using the first entry.`);
		                return value[0];
		            }
		            return value;
		        }
		    }
		    return undefined;
		};
		exports.getHeader = getHeader;
		/**
		 * Encodes a string to Base64 format.
		 */
		const toBase64 = (str) => {
		    if (!str)
		        return '';
		    if (typeof Buffer !== 'undefined') {
		        return Buffer.from(str).toString('base64');
		    }
		    if (typeof btoa !== 'undefined') {
		        return btoa(str);
		    }
		    throw new error_1.OpenAIError('Cannot generate b64 string; Expected `Buffer` or `btoa` to be defined');
		};
		exports.toBase64 = toBase64;
		function isObj(obj) {
		    return obj != null && typeof obj === 'object' && !Array.isArray(obj);
		}
		exports.isObj = isObj;
		
	} (core));
	return core;
}

var hasRequiredError;

function requireError () {
	if (hasRequiredError) return error;
	hasRequiredError = 1;
	// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
	Object.defineProperty(error, "__esModule", { value: true });
	error.ContentFilterFinishReasonError = error.LengthFinishReasonError = error.InternalServerError = error.RateLimitError = error.UnprocessableEntityError = error.ConflictError = error.NotFoundError = error.PermissionDeniedError = error.AuthenticationError = error.BadRequestError = error.APIConnectionTimeoutError = error.APIConnectionError = error.APIUserAbortError = error.APIError = error.OpenAIError = void 0;
	const core_1 = /*@__PURE__*/ requireCore();
	class OpenAIError extends Error {
	}
	error.OpenAIError = OpenAIError;
	class APIError extends OpenAIError {
	    constructor(status, error, message, headers) {
	        super(`${APIError.makeMessage(status, error, message)}`);
	        this.status = status;
	        this.headers = headers;
	        this.request_id = headers?.['x-request-id'];
	        const data = error;
	        this.error = data;
	        this.code = data?.['code'];
	        this.param = data?.['param'];
	        this.type = data?.['type'];
	    }
	    static makeMessage(status, error, message) {
	        const msg = error?.message ?
	            typeof error.message === 'string' ?
	                error.message
	                : JSON.stringify(error.message)
	            : error ? JSON.stringify(error)
	                : message;
	        if (status && msg) {
	            return `${status} ${msg}`;
	        }
	        if (status) {
	            return `${status} status code (no body)`;
	        }
	        if (msg) {
	            return msg;
	        }
	        return '(no status code or body)';
	    }
	    static generate(status, errorResponse, message, headers) {
	        if (!status) {
	            return new APIConnectionError({ message, cause: (0, core_1.castToError)(errorResponse) });
	        }
	        const error = errorResponse?.['error'];
	        if (status === 400) {
	            return new BadRequestError(status, error, message, headers);
	        }
	        if (status === 401) {
	            return new AuthenticationError(status, error, message, headers);
	        }
	        if (status === 403) {
	            return new PermissionDeniedError(status, error, message, headers);
	        }
	        if (status === 404) {
	            return new NotFoundError(status, error, message, headers);
	        }
	        if (status === 409) {
	            return new ConflictError(status, error, message, headers);
	        }
	        if (status === 422) {
	            return new UnprocessableEntityError(status, error, message, headers);
	        }
	        if (status === 429) {
	            return new RateLimitError(status, error, message, headers);
	        }
	        if (status >= 500) {
	            return new InternalServerError(status, error, message, headers);
	        }
	        return new APIError(status, error, message, headers);
	    }
	}
	error.APIError = APIError;
	class APIUserAbortError extends APIError {
	    constructor({ message } = {}) {
	        super(undefined, undefined, message || 'Request was aborted.', undefined);
	        this.status = undefined;
	    }
	}
	error.APIUserAbortError = APIUserAbortError;
	class APIConnectionError extends APIError {
	    constructor({ message, cause }) {
	        super(undefined, undefined, message || 'Connection error.', undefined);
	        this.status = undefined;
	        // in some environments the 'cause' property is already declared
	        // @ts-ignore
	        if (cause)
	            this.cause = cause;
	    }
	}
	error.APIConnectionError = APIConnectionError;
	class APIConnectionTimeoutError extends APIConnectionError {
	    constructor({ message } = {}) {
	        super({ message: message ?? 'Request timed out.' });
	    }
	}
	error.APIConnectionTimeoutError = APIConnectionTimeoutError;
	class BadRequestError extends APIError {
	    constructor() {
	        super(...arguments);
	        this.status = 400;
	    }
	}
	error.BadRequestError = BadRequestError;
	class AuthenticationError extends APIError {
	    constructor() {
	        super(...arguments);
	        this.status = 401;
	    }
	}
	error.AuthenticationError = AuthenticationError;
	class PermissionDeniedError extends APIError {
	    constructor() {
	        super(...arguments);
	        this.status = 403;
	    }
	}
	error.PermissionDeniedError = PermissionDeniedError;
	class NotFoundError extends APIError {
	    constructor() {
	        super(...arguments);
	        this.status = 404;
	    }
	}
	error.NotFoundError = NotFoundError;
	class ConflictError extends APIError {
	    constructor() {
	        super(...arguments);
	        this.status = 409;
	    }
	}
	error.ConflictError = ConflictError;
	class UnprocessableEntityError extends APIError {
	    constructor() {
	        super(...arguments);
	        this.status = 422;
	    }
	}
	error.UnprocessableEntityError = UnprocessableEntityError;
	class RateLimitError extends APIError {
	    constructor() {
	        super(...arguments);
	        this.status = 429;
	    }
	}
	error.RateLimitError = RateLimitError;
	class InternalServerError extends APIError {
	}
	error.InternalServerError = InternalServerError;
	class LengthFinishReasonError extends OpenAIError {
	    constructor() {
	        super(`Could not parse response content as the length limit was reached`);
	    }
	}
	error.LengthFinishReasonError = LengthFinishReasonError;
	class ContentFilterFinishReasonError extends OpenAIError {
	    constructor() {
	        super(`Could not parse response content as the request was rejected by the content filter`);
	    }
	}
	error.ContentFilterFinishReasonError = ContentFilterFinishReasonError;
	
	return error;
}

var qs = {};

var formats = {};

var hasRequiredFormats;

function requireFormats () {
	if (hasRequiredFormats) return formats;
	hasRequiredFormats = 1;
	Object.defineProperty(formats, "__esModule", { value: true });
	formats.RFC3986 = formats.RFC1738 = formats.formatters = formats.default_format = void 0;
	formats.default_format = 'RFC3986';
	formats.formatters = {
	    RFC1738: (v) => String(v).replace(/%20/g, '+'),
	    RFC3986: (v) => String(v),
	};
	formats.RFC1738 = 'RFC1738';
	formats.RFC3986 = 'RFC3986';
	
	return formats;
}

var stringify = {};

var utils = {};

var hasRequiredUtils;

function requireUtils () {
	if (hasRequiredUtils) return utils;
	hasRequiredUtils = 1;
	Object.defineProperty(utils, "__esModule", { value: true });
	utils.maybe_map = utils.combine = utils.is_buffer = utils.is_regexp = utils.compact = utils.encode = utils.decode = utils.assign_single_source = utils.merge = void 0;
	const formats_1 = /*@__PURE__*/ requireFormats();
	const has = Object.prototype.hasOwnProperty;
	const is_array = Array.isArray;
	const hex_table = (() => {
	    const array = [];
	    for (let i = 0; i < 256; ++i) {
	        array.push('%' + ((i < 16 ? '0' : '') + i.toString(16)).toUpperCase());
	    }
	    return array;
	})();
	function compact_queue(queue) {
	    while (queue.length > 1) {
	        const item = queue.pop();
	        if (!item)
	            continue;
	        const obj = item.obj[item.prop];
	        if (is_array(obj)) {
	            const compacted = [];
	            for (let j = 0; j < obj.length; ++j) {
	                if (typeof obj[j] !== 'undefined') {
	                    compacted.push(obj[j]);
	                }
	            }
	            // @ts-ignore
	            item.obj[item.prop] = compacted;
	        }
	    }
	}
	function array_to_object(source, options) {
	    const obj = options && options.plainObjects ? Object.create(null) : {};
	    for (let i = 0; i < source.length; ++i) {
	        if (typeof source[i] !== 'undefined') {
	            obj[i] = source[i];
	        }
	    }
	    return obj;
	}
	function merge(target, source, options = {}) {
	    if (!source) {
	        return target;
	    }
	    if (typeof source !== 'object') {
	        if (is_array(target)) {
	            target.push(source);
	        }
	        else if (target && typeof target === 'object') {
	            if ((options && (options.plainObjects || options.allowPrototypes)) ||
	                !has.call(Object.prototype, source)) {
	                target[source] = true;
	            }
	        }
	        else {
	            return [target, source];
	        }
	        return target;
	    }
	    if (!target || typeof target !== 'object') {
	        return [target].concat(source);
	    }
	    let mergeTarget = target;
	    if (is_array(target) && !is_array(source)) {
	        // @ts-ignore
	        mergeTarget = array_to_object(target, options);
	    }
	    if (is_array(target) && is_array(source)) {
	        source.forEach(function (item, i) {
	            if (has.call(target, i)) {
	                const targetItem = target[i];
	                if (targetItem && typeof targetItem === 'object' && item && typeof item === 'object') {
	                    target[i] = merge(targetItem, item, options);
	                }
	                else {
	                    target.push(item);
	                }
	            }
	            else {
	                target[i] = item;
	            }
	        });
	        return target;
	    }
	    return Object.keys(source).reduce(function (acc, key) {
	        const value = source[key];
	        if (has.call(acc, key)) {
	            acc[key] = merge(acc[key], value, options);
	        }
	        else {
	            acc[key] = value;
	        }
	        return acc;
	    }, mergeTarget);
	}
	utils.merge = merge;
	function assign_single_source(target, source) {
	    return Object.keys(source).reduce(function (acc, key) {
	        acc[key] = source[key];
	        return acc;
	    }, target);
	}
	utils.assign_single_source = assign_single_source;
	function decode(str, _, charset) {
	    const strWithoutPlus = str.replace(/\+/g, ' ');
	    if (charset === 'iso-8859-1') {
	        // unescape never throws, no try...catch needed:
	        return strWithoutPlus.replace(/%[0-9a-f]{2}/gi, unescape);
	    }
	    // utf-8
	    try {
	        return decodeURIComponent(strWithoutPlus);
	    }
	    catch (e) {
	        return strWithoutPlus;
	    }
	}
	utils.decode = decode;
	const limit = 1024;
	const encode = (str, _defaultEncoder, charset, _kind, format) => {
	    // This code was originally written by Brian White for the io.js core querystring library.
	    // It has been adapted here for stricter adherence to RFC 3986
	    if (str.length === 0) {
	        return str;
	    }
	    let string = str;
	    if (typeof str === 'symbol') {
	        string = Symbol.prototype.toString.call(str);
	    }
	    else if (typeof str !== 'string') {
	        string = String(str);
	    }
	    if (charset === 'iso-8859-1') {
	        return escape(string).replace(/%u[0-9a-f]{4}/gi, function ($0) {
	            return '%26%23' + parseInt($0.slice(2), 16) + '%3B';
	        });
	    }
	    let out = '';
	    for (let j = 0; j < string.length; j += limit) {
	        const segment = string.length >= limit ? string.slice(j, j + limit) : string;
	        const arr = [];
	        for (let i = 0; i < segment.length; ++i) {
	            let c = segment.charCodeAt(i);
	            if (c === 0x2d || // -
	                c === 0x2e || // .
	                c === 0x5f || // _
	                c === 0x7e || // ~
	                (c >= 0x30 && c <= 0x39) || // 0-9
	                (c >= 0x41 && c <= 0x5a) || // a-z
	                (c >= 0x61 && c <= 0x7a) || // A-Z
	                (format === formats_1.RFC1738 && (c === 0x28 || c === 0x29)) // ( )
	            ) {
	                arr[arr.length] = segment.charAt(i);
	                continue;
	            }
	            if (c < 0x80) {
	                arr[arr.length] = hex_table[c];
	                continue;
	            }
	            if (c < 0x800) {
	                arr[arr.length] = hex_table[0xc0 | (c >> 6)] + hex_table[0x80 | (c & 0x3f)];
	                continue;
	            }
	            if (c < 0xd800 || c >= 0xe000) {
	                arr[arr.length] =
	                    hex_table[0xe0 | (c >> 12)] + hex_table[0x80 | ((c >> 6) & 0x3f)] + hex_table[0x80 | (c & 0x3f)];
	                continue;
	            }
	            i += 1;
	            c = 0x10000 + (((c & 0x3ff) << 10) | (segment.charCodeAt(i) & 0x3ff));
	            arr[arr.length] =
	                hex_table[0xf0 | (c >> 18)] +
	                    hex_table[0x80 | ((c >> 12) & 0x3f)] +
	                    hex_table[0x80 | ((c >> 6) & 0x3f)] +
	                    hex_table[0x80 | (c & 0x3f)];
	        }
	        out += arr.join('');
	    }
	    return out;
	};
	utils.encode = encode;
	function compact(value) {
	    const queue = [{ obj: { o: value }, prop: 'o' }];
	    const refs = [];
	    for (let i = 0; i < queue.length; ++i) {
	        const item = queue[i];
	        // @ts-ignore
	        const obj = item.obj[item.prop];
	        const keys = Object.keys(obj);
	        for (let j = 0; j < keys.length; ++j) {
	            const key = keys[j];
	            const val = obj[key];
	            if (typeof val === 'object' && val !== null && refs.indexOf(val) === -1) {
	                queue.push({ obj: obj, prop: key });
	                refs.push(val);
	            }
	        }
	    }
	    compact_queue(queue);
	    return value;
	}
	utils.compact = compact;
	function is_regexp(obj) {
	    return Object.prototype.toString.call(obj) === '[object RegExp]';
	}
	utils.is_regexp = is_regexp;
	function is_buffer(obj) {
	    if (!obj || typeof obj !== 'object') {
	        return false;
	    }
	    return !!(obj.constructor && obj.constructor.isBuffer && obj.constructor.isBuffer(obj));
	}
	utils.is_buffer = is_buffer;
	function combine(a, b) {
	    return [].concat(a, b);
	}
	utils.combine = combine;
	function maybe_map(val, fn) {
	    if (is_array(val)) {
	        const mapped = [];
	        for (let i = 0; i < val.length; i += 1) {
	            mapped.push(fn(val[i]));
	        }
	        return mapped;
	    }
	    return fn(val);
	}
	utils.maybe_map = maybe_map;
	
	return utils;
}

var hasRequiredStringify;

function requireStringify () {
	if (hasRequiredStringify) return stringify;
	hasRequiredStringify = 1;
	Object.defineProperty(stringify, "__esModule", { value: true });
	stringify.stringify = void 0;
	const utils_1 = /*@__PURE__*/ requireUtils();
	const formats_1 = /*@__PURE__*/ requireFormats();
	const has = Object.prototype.hasOwnProperty;
	const array_prefix_generators = {
	    brackets(prefix) {
	        return String(prefix) + '[]';
	    },
	    comma: 'comma',
	    indices(prefix, key) {
	        return String(prefix) + '[' + key + ']';
	    },
	    repeat(prefix) {
	        return String(prefix);
	    },
	};
	const is_array = Array.isArray;
	const push = Array.prototype.push;
	const push_to_array = function (arr, value_or_array) {
	    push.apply(arr, is_array(value_or_array) ? value_or_array : [value_or_array]);
	};
	const to_ISO = Date.prototype.toISOString;
	const defaults = {
	    addQueryPrefix: false,
	    allowDots: false,
	    allowEmptyArrays: false,
	    arrayFormat: 'indices',
	    charset: 'utf-8',
	    charsetSentinel: false,
	    delimiter: '&',
	    encode: true,
	    encodeDotInKeys: false,
	    encoder: utils_1.encode,
	    encodeValuesOnly: false,
	    format: formats_1.default_format,
	    formatter: formats_1.formatters[formats_1.default_format],
	    /** @deprecated */
	    indices: false,
	    serializeDate(date) {
	        return to_ISO.call(date);
	    },
	    skipNulls: false,
	    strictNullHandling: false,
	};
	function is_non_nullish_primitive(v) {
	    return (typeof v === 'string' ||
	        typeof v === 'number' ||
	        typeof v === 'boolean' ||
	        typeof v === 'symbol' ||
	        typeof v === 'bigint');
	}
	const sentinel = {};
	function inner_stringify(object, prefix, generateArrayPrefix, commaRoundTrip, allowEmptyArrays, strictNullHandling, skipNulls, encodeDotInKeys, encoder, filter, sort, allowDots, serializeDate, format, formatter, encodeValuesOnly, charset, sideChannel) {
	    let obj = object;
	    let tmp_sc = sideChannel;
	    let step = 0;
	    let find_flag = false;
	    while ((tmp_sc = tmp_sc.get(sentinel)) !== void undefined && !find_flag) {
	        // Where object last appeared in the ref tree
	        const pos = tmp_sc.get(object);
	        step += 1;
	        if (typeof pos !== 'undefined') {
	            if (pos === step) {
	                throw new RangeError('Cyclic object value');
	            }
	            else {
	                find_flag = true; // Break while
	            }
	        }
	        if (typeof tmp_sc.get(sentinel) === 'undefined') {
	            step = 0;
	        }
	    }
	    if (typeof filter === 'function') {
	        obj = filter(prefix, obj);
	    }
	    else if (obj instanceof Date) {
	        obj = serializeDate?.(obj);
	    }
	    else if (generateArrayPrefix === 'comma' && is_array(obj)) {
	        obj = (0, utils_1.maybe_map)(obj, function (value) {
	            if (value instanceof Date) {
	                return serializeDate?.(value);
	            }
	            return value;
	        });
	    }
	    if (obj === null) {
	        if (strictNullHandling) {
	            return encoder && !encodeValuesOnly ?
	                // @ts-expect-error
	                encoder(prefix, defaults.encoder, charset, 'key', format)
	                : prefix;
	        }
	        obj = '';
	    }
	    if (is_non_nullish_primitive(obj) || (0, utils_1.is_buffer)(obj)) {
	        if (encoder) {
	            const key_value = encodeValuesOnly ? prefix
	                // @ts-expect-error
	                : encoder(prefix, defaults.encoder, charset, 'key', format);
	            return [
	                formatter?.(key_value) +
	                    '=' +
	                    // @ts-expect-error
	                    formatter?.(encoder(obj, defaults.encoder, charset, 'value', format)),
	            ];
	        }
	        return [formatter?.(prefix) + '=' + formatter?.(String(obj))];
	    }
	    const values = [];
	    if (typeof obj === 'undefined') {
	        return values;
	    }
	    let obj_keys;
	    if (generateArrayPrefix === 'comma' && is_array(obj)) {
	        // we need to join elements in
	        if (encodeValuesOnly && encoder) {
	            // @ts-expect-error values only
	            obj = (0, utils_1.maybe_map)(obj, encoder);
	        }
	        obj_keys = [{ value: obj.length > 0 ? obj.join(',') || null : void undefined }];
	    }
	    else if (is_array(filter)) {
	        obj_keys = filter;
	    }
	    else {
	        const keys = Object.keys(obj);
	        obj_keys = sort ? keys.sort(sort) : keys;
	    }
	    const encoded_prefix = encodeDotInKeys ? String(prefix).replace(/\./g, '%2E') : String(prefix);
	    const adjusted_prefix = commaRoundTrip && is_array(obj) && obj.length === 1 ? encoded_prefix + '[]' : encoded_prefix;
	    if (allowEmptyArrays && is_array(obj) && obj.length === 0) {
	        return adjusted_prefix + '[]';
	    }
	    for (let j = 0; j < obj_keys.length; ++j) {
	        const key = obj_keys[j];
	        const value = 
	        // @ts-ignore
	        typeof key === 'object' && typeof key.value !== 'undefined' ? key.value : obj[key];
	        if (skipNulls && value === null) {
	            continue;
	        }
	        // @ts-ignore
	        const encoded_key = allowDots && encodeDotInKeys ? key.replace(/\./g, '%2E') : key;
	        const key_prefix = is_array(obj) ?
	            typeof generateArrayPrefix === 'function' ?
	                generateArrayPrefix(adjusted_prefix, encoded_key)
	                : adjusted_prefix
	            : adjusted_prefix + (allowDots ? '.' + encoded_key : '[' + encoded_key + ']');
	        sideChannel.set(object, step);
	        const valueSideChannel = new WeakMap();
	        valueSideChannel.set(sentinel, sideChannel);
	        push_to_array(values, inner_stringify(value, key_prefix, generateArrayPrefix, commaRoundTrip, allowEmptyArrays, strictNullHandling, skipNulls, encodeDotInKeys, 
	        // @ts-ignore
	        generateArrayPrefix === 'comma' && encodeValuesOnly && is_array(obj) ? null : encoder, filter, sort, allowDots, serializeDate, format, formatter, encodeValuesOnly, charset, valueSideChannel));
	    }
	    return values;
	}
	function normalize_stringify_options(opts = defaults) {
	    if (typeof opts.allowEmptyArrays !== 'undefined' && typeof opts.allowEmptyArrays !== 'boolean') {
	        throw new TypeError('`allowEmptyArrays` option can only be `true` or `false`, when provided');
	    }
	    if (typeof opts.encodeDotInKeys !== 'undefined' && typeof opts.encodeDotInKeys !== 'boolean') {
	        throw new TypeError('`encodeDotInKeys` option can only be `true` or `false`, when provided');
	    }
	    if (opts.encoder !== null && typeof opts.encoder !== 'undefined' && typeof opts.encoder !== 'function') {
	        throw new TypeError('Encoder has to be a function.');
	    }
	    const charset = opts.charset || defaults.charset;
	    if (typeof opts.charset !== 'undefined' && opts.charset !== 'utf-8' && opts.charset !== 'iso-8859-1') {
	        throw new TypeError('The charset option must be either utf-8, iso-8859-1, or undefined');
	    }
	    let format = formats_1.default_format;
	    if (typeof opts.format !== 'undefined') {
	        if (!has.call(formats_1.formatters, opts.format)) {
	            throw new TypeError('Unknown format option provided.');
	        }
	        format = opts.format;
	    }
	    const formatter = formats_1.formatters[format];
	    let filter = defaults.filter;
	    if (typeof opts.filter === 'function' || is_array(opts.filter)) {
	        filter = opts.filter;
	    }
	    let arrayFormat;
	    if (opts.arrayFormat && opts.arrayFormat in array_prefix_generators) {
	        arrayFormat = opts.arrayFormat;
	    }
	    else if ('indices' in opts) {
	        arrayFormat = opts.indices ? 'indices' : 'repeat';
	    }
	    else {
	        arrayFormat = defaults.arrayFormat;
	    }
	    if ('commaRoundTrip' in opts && typeof opts.commaRoundTrip !== 'boolean') {
	        throw new TypeError('`commaRoundTrip` must be a boolean, or absent');
	    }
	    const allowDots = typeof opts.allowDots === 'undefined' ?
	        !!opts.encodeDotInKeys === true ?
	            true
	            : defaults.allowDots
	        : !!opts.allowDots;
	    return {
	        addQueryPrefix: typeof opts.addQueryPrefix === 'boolean' ? opts.addQueryPrefix : defaults.addQueryPrefix,
	        // @ts-ignore
	        allowDots: allowDots,
	        allowEmptyArrays: typeof opts.allowEmptyArrays === 'boolean' ? !!opts.allowEmptyArrays : defaults.allowEmptyArrays,
	        arrayFormat: arrayFormat,
	        charset: charset,
	        charsetSentinel: typeof opts.charsetSentinel === 'boolean' ? opts.charsetSentinel : defaults.charsetSentinel,
	        commaRoundTrip: !!opts.commaRoundTrip,
	        delimiter: typeof opts.delimiter === 'undefined' ? defaults.delimiter : opts.delimiter,
	        encode: typeof opts.encode === 'boolean' ? opts.encode : defaults.encode,
	        encodeDotInKeys: typeof opts.encodeDotInKeys === 'boolean' ? opts.encodeDotInKeys : defaults.encodeDotInKeys,
	        encoder: typeof opts.encoder === 'function' ? opts.encoder : defaults.encoder,
	        encodeValuesOnly: typeof opts.encodeValuesOnly === 'boolean' ? opts.encodeValuesOnly : defaults.encodeValuesOnly,
	        filter: filter,
	        format: format,
	        formatter: formatter,
	        serializeDate: typeof opts.serializeDate === 'function' ? opts.serializeDate : defaults.serializeDate,
	        skipNulls: typeof opts.skipNulls === 'boolean' ? opts.skipNulls : defaults.skipNulls,
	        // @ts-ignore
	        sort: typeof opts.sort === 'function' ? opts.sort : null,
	        strictNullHandling: typeof opts.strictNullHandling === 'boolean' ? opts.strictNullHandling : defaults.strictNullHandling,
	    };
	}
	function stringify$1(object, opts = {}) {
	    let obj = object;
	    const options = normalize_stringify_options(opts);
	    let obj_keys;
	    let filter;
	    if (typeof options.filter === 'function') {
	        filter = options.filter;
	        obj = filter('', obj);
	    }
	    else if (is_array(options.filter)) {
	        filter = options.filter;
	        obj_keys = filter;
	    }
	    const keys = [];
	    if (typeof obj !== 'object' || obj === null) {
	        return '';
	    }
	    const generateArrayPrefix = array_prefix_generators[options.arrayFormat];
	    const commaRoundTrip = generateArrayPrefix === 'comma' && options.commaRoundTrip;
	    if (!obj_keys) {
	        obj_keys = Object.keys(obj);
	    }
	    if (options.sort) {
	        obj_keys.sort(options.sort);
	    }
	    const sideChannel = new WeakMap();
	    for (let i = 0; i < obj_keys.length; ++i) {
	        const key = obj_keys[i];
	        if (options.skipNulls && obj[key] === null) {
	            continue;
	        }
	        push_to_array(keys, inner_stringify(obj[key], key, 
	        // @ts-expect-error
	        generateArrayPrefix, commaRoundTrip, options.allowEmptyArrays, options.strictNullHandling, options.skipNulls, options.encodeDotInKeys, options.encode ? options.encoder : null, options.filter, options.sort, options.allowDots, options.serializeDate, options.format, options.formatter, options.encodeValuesOnly, options.charset, sideChannel));
	    }
	    const joined = keys.join(options.delimiter);
	    let prefix = options.addQueryPrefix === true ? '?' : '';
	    if (options.charsetSentinel) {
	        if (options.charset === 'iso-8859-1') {
	            // encodeURIComponent('&#10003;'), the "numeric entity" representation of a checkmark
	            prefix += 'utf8=%26%2310003%3B&';
	        }
	        else {
	            // encodeURIComponent('✓')
	            prefix += 'utf8=%E2%9C%93&';
	        }
	    }
	    return joined.length > 0 ? prefix + joined : '';
	}
	stringify.stringify = stringify$1;
	
	return stringify;
}

var hasRequiredQs;

function requireQs () {
	if (hasRequiredQs) return qs;
	hasRequiredQs = 1;
	(function (exports) {
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.formats = exports.stringify = void 0;
		const formats_1 = /*@__PURE__*/ requireFormats();
		const formats = {
		    formatters: formats_1.formatters,
		    RFC1738: formats_1.RFC1738,
		    RFC3986: formats_1.RFC3986,
		    default: formats_1.default_format,
		};
		exports.formats = formats;
		var stringify_1 = /*@__PURE__*/ requireStringify();
		Object.defineProperty(exports, "stringify", { enumerable: true, get: function () { return stringify_1.stringify; } });
		
	} (qs));
	return qs;
}

var pagination = {};

var hasRequiredPagination;

function requirePagination () {
	if (hasRequiredPagination) return pagination;
	hasRequiredPagination = 1;
	// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
	Object.defineProperty(pagination, "__esModule", { value: true });
	pagination.CursorPage = pagination.Page = void 0;
	const core_1 = /*@__PURE__*/ requireCore();
	/**
	 * Note: no pagination actually occurs yet, this is for forwards-compatibility.
	 */
	class Page extends core_1.AbstractPage {
	    constructor(client, response, body, options) {
	        super(client, response, body, options);
	        this.data = body.data || [];
	        this.object = body.object;
	    }
	    getPaginatedItems() {
	        return this.data ?? [];
	    }
	    // @deprecated Please use `nextPageInfo()` instead
	    /**
	     * This page represents a response that isn't actually paginated at the API level
	     * so there will never be any next page params.
	     */
	    nextPageParams() {
	        return null;
	    }
	    nextPageInfo() {
	        return null;
	    }
	}
	pagination.Page = Page;
	class CursorPage extends core_1.AbstractPage {
	    constructor(client, response, body, options) {
	        super(client, response, body, options);
	        this.data = body.data || [];
	    }
	    getPaginatedItems() {
	        return this.data ?? [];
	    }
	    // @deprecated Please use `nextPageInfo()` instead
	    nextPageParams() {
	        const info = this.nextPageInfo();
	        if (!info)
	            return null;
	        if ('params' in info)
	            return info.params;
	        const params = Object.fromEntries(info.url.searchParams);
	        if (!Object.keys(params).length)
	            return null;
	        return params;
	    }
	    nextPageInfo() {
	        const data = this.getPaginatedItems();
	        if (!data.length) {
	            return null;
	        }
	        const id = data[data.length - 1]?.id;
	        if (!id) {
	            return null;
	        }
	        return { params: { after: id } };
	    }
	}
	pagination.CursorPage = CursorPage;
	
	return pagination;
}

var resources = {};

var chat$2 = {};

var completions$2 = {};

var resource = {};

var hasRequiredResource;

function requireResource () {
	if (hasRequiredResource) return resource;
	hasRequiredResource = 1;
	// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
	Object.defineProperty(resource, "__esModule", { value: true });
	resource.APIResource = void 0;
	class APIResource {
	    constructor(client) {
	        this._client = client;
	    }
	}
	resource.APIResource = APIResource;
	
	return resource;
}

var hasRequiredCompletions$2;

function requireCompletions$2 () {
	if (hasRequiredCompletions$2) return completions$2;
	hasRequiredCompletions$2 = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.Completions = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		class Completions extends resource_1.APIResource {
		    create(body, options) {
		        return this._client.post('/chat/completions', { body, ...options, stream: body.stream ?? false });
		    }
		}
		exports.Completions = Completions;
		(function (Completions) {
		})(Completions = exports.Completions || (exports.Completions = {}));
		
	} (completions$2));
	return completions$2;
}

var chat$1 = {};

var hasRequiredChat$2;

function requireChat$2 () {
	if (hasRequiredChat$2) return chat$1;
	hasRequiredChat$2 = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		var __createBinding = (chat$1 && chat$1.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __setModuleDefault = (chat$1 && chat$1.__setModuleDefault) || (Object.create ? (function(o, v) {
		    Object.defineProperty(o, "default", { enumerable: true, value: v });
		}) : function(o, v) {
		    o["default"] = v;
		});
		var __importStar = (chat$1 && chat$1.__importStar) || function (mod) {
		    if (mod && mod.__esModule) return mod;
		    var result = {};
		    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
		    __setModuleDefault(result, mod);
		    return result;
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.Chat = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		const CompletionsAPI = __importStar(/*@__PURE__*/ requireCompletions$2());
		class Chat extends resource_1.APIResource {
		    constructor() {
		        super(...arguments);
		        this.completions = new CompletionsAPI.Completions(this._client);
		    }
		}
		exports.Chat = Chat;
		(function (Chat) {
		    Chat.Completions = CompletionsAPI.Completions;
		})(Chat = exports.Chat || (exports.Chat = {}));
		
	} (chat$1));
	return chat$1;
}

var hasRequiredChat$1;

function requireChat$1 () {
	if (hasRequiredChat$1) return chat$2;
	hasRequiredChat$1 = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.Chat = exports.Completions = void 0;
		var completions_1 = /*@__PURE__*/ requireCompletions$2();
		Object.defineProperty(exports, "Completions", { enumerable: true, get: function () { return completions_1.Completions; } });
		var chat_1 = /*@__PURE__*/ requireChat$2();
		Object.defineProperty(exports, "Chat", { enumerable: true, get: function () { return chat_1.Chat; } });
		
	} (chat$2));
	return chat$2;
}

var shared = {};

var hasRequiredShared;

function requireShared () {
	if (hasRequiredShared) return shared;
	hasRequiredShared = 1;
	// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
	Object.defineProperty(shared, "__esModule", { value: true });
	
	return shared;
}

var audio = {};

var speech = {};

var hasRequiredSpeech;

function requireSpeech () {
	if (hasRequiredSpeech) return speech;
	hasRequiredSpeech = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.Speech = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		class Speech extends resource_1.APIResource {
		    /**
		     * Generates audio from the input text.
		     */
		    create(body, options) {
		        return this._client.post('/audio/speech', { body, ...options, __binaryResponse: true });
		    }
		}
		exports.Speech = Speech;
		(function (Speech) {
		})(Speech = exports.Speech || (exports.Speech = {}));
		
	} (speech));
	return speech;
}

var transcriptions = {};

var hasRequiredTranscriptions;

function requireTranscriptions () {
	if (hasRequiredTranscriptions) return transcriptions;
	hasRequiredTranscriptions = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		var __createBinding = (transcriptions && transcriptions.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __setModuleDefault = (transcriptions && transcriptions.__setModuleDefault) || (Object.create ? (function(o, v) {
		    Object.defineProperty(o, "default", { enumerable: true, value: v });
		}) : function(o, v) {
		    o["default"] = v;
		});
		var __importStar = (transcriptions && transcriptions.__importStar) || function (mod) {
		    if (mod && mod.__esModule) return mod;
		    var result = {};
		    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
		    __setModuleDefault(result, mod);
		    return result;
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.Transcriptions = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		const Core = __importStar(/*@__PURE__*/ requireCore());
		class Transcriptions extends resource_1.APIResource {
		    /**
		     * Transcribes audio into the input language.
		     */
		    create(body, options) {
		        return this._client.post('/audio/transcriptions', Core.multipartFormRequestOptions({ body, ...options }));
		    }
		}
		exports.Transcriptions = Transcriptions;
		(function (Transcriptions) {
		})(Transcriptions = exports.Transcriptions || (exports.Transcriptions = {}));
		
	} (transcriptions));
	return transcriptions;
}

var translations = {};

var hasRequiredTranslations;

function requireTranslations () {
	if (hasRequiredTranslations) return translations;
	hasRequiredTranslations = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		var __createBinding = (translations && translations.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __setModuleDefault = (translations && translations.__setModuleDefault) || (Object.create ? (function(o, v) {
		    Object.defineProperty(o, "default", { enumerable: true, value: v });
		}) : function(o, v) {
		    o["default"] = v;
		});
		var __importStar = (translations && translations.__importStar) || function (mod) {
		    if (mod && mod.__esModule) return mod;
		    var result = {};
		    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
		    __setModuleDefault(result, mod);
		    return result;
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.Translations = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		const Core = __importStar(/*@__PURE__*/ requireCore());
		class Translations extends resource_1.APIResource {
		    /**
		     * Translates audio into English.
		     */
		    create(body, options) {
		        return this._client.post('/audio/translations', Core.multipartFormRequestOptions({ body, ...options }));
		    }
		}
		exports.Translations = Translations;
		(function (Translations) {
		})(Translations = exports.Translations || (exports.Translations = {}));
		
	} (translations));
	return translations;
}

var hasRequiredAudio;

function requireAudio () {
	if (hasRequiredAudio) return audio;
	hasRequiredAudio = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		var __createBinding = (audio && audio.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __setModuleDefault = (audio && audio.__setModuleDefault) || (Object.create ? (function(o, v) {
		    Object.defineProperty(o, "default", { enumerable: true, value: v });
		}) : function(o, v) {
		    o["default"] = v;
		});
		var __importStar = (audio && audio.__importStar) || function (mod) {
		    if (mod && mod.__esModule) return mod;
		    var result = {};
		    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
		    __setModuleDefault(result, mod);
		    return result;
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.Audio = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		const SpeechAPI = __importStar(/*@__PURE__*/ requireSpeech());
		const TranscriptionsAPI = __importStar(/*@__PURE__*/ requireTranscriptions());
		const TranslationsAPI = __importStar(/*@__PURE__*/ requireTranslations());
		class Audio extends resource_1.APIResource {
		    constructor() {
		        super(...arguments);
		        this.transcriptions = new TranscriptionsAPI.Transcriptions(this._client);
		        this.translations = new TranslationsAPI.Translations(this._client);
		        this.speech = new SpeechAPI.Speech(this._client);
		    }
		}
		exports.Audio = Audio;
		(function (Audio) {
		    Audio.Transcriptions = TranscriptionsAPI.Transcriptions;
		    Audio.Translations = TranslationsAPI.Translations;
		    Audio.Speech = SpeechAPI.Speech;
		})(Audio = exports.Audio || (exports.Audio = {}));
		
	} (audio));
	return audio;
}

var batches = {};

var hasRequiredBatches;

function requireBatches () {
	if (hasRequiredBatches) return batches;
	hasRequiredBatches = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		var __createBinding = (batches && batches.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __setModuleDefault = (batches && batches.__setModuleDefault) || (Object.create ? (function(o, v) {
		    Object.defineProperty(o, "default", { enumerable: true, value: v });
		}) : function(o, v) {
		    o["default"] = v;
		});
		var __importStar = (batches && batches.__importStar) || function (mod) {
		    if (mod && mod.__esModule) return mod;
		    var result = {};
		    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
		    __setModuleDefault(result, mod);
		    return result;
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.BatchesPage = exports.Batches = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		const core_1 = /*@__PURE__*/ requireCore();
		const BatchesAPI = __importStar(/*@__PURE__*/ requireBatches());
		const pagination_1 = /*@__PURE__*/ requirePagination();
		class Batches extends resource_1.APIResource {
		    /**
		     * Creates and executes a batch from an uploaded file of requests
		     */
		    create(body, options) {
		        return this._client.post('/batches', { body, ...options });
		    }
		    /**
		     * Retrieves a batch.
		     */
		    retrieve(batchId, options) {
		        return this._client.get(`/batches/${batchId}`, options);
		    }
		    list(query = {}, options) {
		        if ((0, core_1.isRequestOptions)(query)) {
		            return this.list({}, query);
		        }
		        return this._client.getAPIList('/batches', BatchesPage, { query, ...options });
		    }
		    /**
		     * Cancels an in-progress batch. The batch will be in status `cancelling` for up to
		     * 10 minutes, before changing to `cancelled`, where it will have partial results
		     * (if any) available in the output file.
		     */
		    cancel(batchId, options) {
		        return this._client.post(`/batches/${batchId}/cancel`, options);
		    }
		}
		exports.Batches = Batches;
		class BatchesPage extends pagination_1.CursorPage {
		}
		exports.BatchesPage = BatchesPage;
		(function (Batches) {
		    Batches.BatchesPage = BatchesAPI.BatchesPage;
		})(Batches = exports.Batches || (exports.Batches = {}));
		
	} (batches));
	return batches;
}

var beta = {};

var assistants = {};

var hasRequiredAssistants;

function requireAssistants () {
	if (hasRequiredAssistants) return assistants;
	hasRequiredAssistants = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		var __createBinding = (assistants && assistants.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __setModuleDefault = (assistants && assistants.__setModuleDefault) || (Object.create ? (function(o, v) {
		    Object.defineProperty(o, "default", { enumerable: true, value: v });
		}) : function(o, v) {
		    o["default"] = v;
		});
		var __importStar = (assistants && assistants.__importStar) || function (mod) {
		    if (mod && mod.__esModule) return mod;
		    var result = {};
		    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
		    __setModuleDefault(result, mod);
		    return result;
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.AssistantsPage = exports.Assistants = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		const core_1 = /*@__PURE__*/ requireCore();
		const AssistantsAPI = __importStar(/*@__PURE__*/ requireAssistants());
		const pagination_1 = /*@__PURE__*/ requirePagination();
		class Assistants extends resource_1.APIResource {
		    /**
		     * Create an assistant with a model and instructions.
		     */
		    create(body, options) {
		        return this._client.post('/assistants', {
		            body,
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		    /**
		     * Retrieves an assistant.
		     */
		    retrieve(assistantId, options) {
		        return this._client.get(`/assistants/${assistantId}`, {
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		    /**
		     * Modifies an assistant.
		     */
		    update(assistantId, body, options) {
		        return this._client.post(`/assistants/${assistantId}`, {
		            body,
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		    list(query = {}, options) {
		        if ((0, core_1.isRequestOptions)(query)) {
		            return this.list({}, query);
		        }
		        return this._client.getAPIList('/assistants', AssistantsPage, {
		            query,
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		    /**
		     * Delete an assistant.
		     */
		    del(assistantId, options) {
		        return this._client.delete(`/assistants/${assistantId}`, {
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		}
		exports.Assistants = Assistants;
		class AssistantsPage extends pagination_1.CursorPage {
		}
		exports.AssistantsPage = AssistantsPage;
		(function (Assistants) {
		    Assistants.AssistantsPage = AssistantsAPI.AssistantsPage;
		})(Assistants = exports.Assistants || (exports.Assistants = {}));
		
	} (assistants));
	return assistants;
}

var chat = {};

var completions$1 = {};

var ChatCompletionRunner = {};

var AbstractChatCompletionRunner = {};

var RunnableFunction = {};

var hasRequiredRunnableFunction;

function requireRunnableFunction () {
	if (hasRequiredRunnableFunction) return RunnableFunction;
	hasRequiredRunnableFunction = 1;
	Object.defineProperty(RunnableFunction, "__esModule", { value: true });
	RunnableFunction.ParsingToolFunction = RunnableFunction.ParsingFunction = RunnableFunction.isRunnableFunctionWithParse = void 0;
	function isRunnableFunctionWithParse(fn) {
	    return typeof fn.parse === 'function';
	}
	RunnableFunction.isRunnableFunctionWithParse = isRunnableFunctionWithParse;
	/**
	 * This is helper class for passing a `function` and `parse` where the `function`
	 * argument type matches the `parse` return type.
	 *
	 * @deprecated - please use ParsingToolFunction instead.
	 */
	class ParsingFunction {
	    constructor(input) {
	        this.function = input.function;
	        this.parse = input.parse;
	        this.parameters = input.parameters;
	        this.description = input.description;
	        this.name = input.name;
	    }
	}
	RunnableFunction.ParsingFunction = ParsingFunction;
	/**
	 * This is helper class for passing a `function` and `parse` where the `function`
	 * argument type matches the `parse` return type.
	 */
	class ParsingToolFunction {
	    constructor(input) {
	        this.type = 'function';
	        this.function = input;
	    }
	}
	RunnableFunction.ParsingToolFunction = ParsingToolFunction;
	
	return RunnableFunction;
}

var chatCompletionUtils = {};

var hasRequiredChatCompletionUtils;

function requireChatCompletionUtils () {
	if (hasRequiredChatCompletionUtils) return chatCompletionUtils;
	hasRequiredChatCompletionUtils = 1;
	Object.defineProperty(chatCompletionUtils, "__esModule", { value: true });
	chatCompletionUtils.isPresent = chatCompletionUtils.isToolMessage = chatCompletionUtils.isFunctionMessage = chatCompletionUtils.isAssistantMessage = void 0;
	const isAssistantMessage = (message) => {
	    return message?.role === 'assistant';
	};
	chatCompletionUtils.isAssistantMessage = isAssistantMessage;
	const isFunctionMessage = (message) => {
	    return message?.role === 'function';
	};
	chatCompletionUtils.isFunctionMessage = isFunctionMessage;
	const isToolMessage = (message) => {
	    return message?.role === 'tool';
	};
	chatCompletionUtils.isToolMessage = isToolMessage;
	function isPresent(obj) {
	    return obj != null;
	}
	chatCompletionUtils.isPresent = isPresent;
	
	return chatCompletionUtils;
}

var EventStream = {};

var hasRequiredEventStream;

function requireEventStream () {
	if (hasRequiredEventStream) return EventStream;
	hasRequiredEventStream = 1;
	var __classPrivateFieldSet = (EventStream && EventStream.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
	    if (kind === "m") throw new TypeError("Private method is not writable");
	    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
	    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
	    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
	};
	var __classPrivateFieldGet = (EventStream && EventStream.__classPrivateFieldGet) || function (receiver, state, kind, f) {
	    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
	    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
	    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
	};
	var _EventStream_instances, _EventStream_connectedPromise, _EventStream_resolveConnectedPromise, _EventStream_rejectConnectedPromise, _EventStream_endPromise, _EventStream_resolveEndPromise, _EventStream_rejectEndPromise, _EventStream_listeners, _EventStream_ended, _EventStream_errored, _EventStream_aborted, _EventStream_catchingPromiseCreated, _EventStream_handleError;
	Object.defineProperty(EventStream, "__esModule", { value: true });
	EventStream.EventStream = void 0;
	const error_1 = /*@__PURE__*/ requireError();
	let EventStream$1 = class EventStream {
	    constructor() {
	        _EventStream_instances.add(this);
	        this.controller = new AbortController();
	        _EventStream_connectedPromise.set(this, void 0);
	        _EventStream_resolveConnectedPromise.set(this, () => { });
	        _EventStream_rejectConnectedPromise.set(this, () => { });
	        _EventStream_endPromise.set(this, void 0);
	        _EventStream_resolveEndPromise.set(this, () => { });
	        _EventStream_rejectEndPromise.set(this, () => { });
	        _EventStream_listeners.set(this, {});
	        _EventStream_ended.set(this, false);
	        _EventStream_errored.set(this, false);
	        _EventStream_aborted.set(this, false);
	        _EventStream_catchingPromiseCreated.set(this, false);
	        __classPrivateFieldSet(this, _EventStream_connectedPromise, new Promise((resolve, reject) => {
	            __classPrivateFieldSet(this, _EventStream_resolveConnectedPromise, resolve, "f");
	            __classPrivateFieldSet(this, _EventStream_rejectConnectedPromise, reject, "f");
	        }), "f");
	        __classPrivateFieldSet(this, _EventStream_endPromise, new Promise((resolve, reject) => {
	            __classPrivateFieldSet(this, _EventStream_resolveEndPromise, resolve, "f");
	            __classPrivateFieldSet(this, _EventStream_rejectEndPromise, reject, "f");
	        }), "f");
	        // Don't let these promises cause unhandled rejection errors.
	        // we will manually cause an unhandled rejection error later
	        // if the user hasn't registered any error listener or called
	        // any promise-returning method.
	        __classPrivateFieldGet(this, _EventStream_connectedPromise, "f").catch(() => { });
	        __classPrivateFieldGet(this, _EventStream_endPromise, "f").catch(() => { });
	    }
	    _run(executor) {
	        // Unfortunately if we call `executor()` immediately we get runtime errors about
	        // references to `this` before the `super()` constructor call returns.
	        setTimeout(() => {
	            executor().then(() => {
	                this._emitFinal();
	                this._emit('end');
	            }, __classPrivateFieldGet(this, _EventStream_instances, "m", _EventStream_handleError).bind(this));
	        }, 0);
	    }
	    _connected() {
	        if (this.ended)
	            return;
	        __classPrivateFieldGet(this, _EventStream_resolveConnectedPromise, "f").call(this);
	        this._emit('connect');
	    }
	    get ended() {
	        return __classPrivateFieldGet(this, _EventStream_ended, "f");
	    }
	    get errored() {
	        return __classPrivateFieldGet(this, _EventStream_errored, "f");
	    }
	    get aborted() {
	        return __classPrivateFieldGet(this, _EventStream_aborted, "f");
	    }
	    abort() {
	        this.controller.abort();
	    }
	    /**
	     * Adds the listener function to the end of the listeners array for the event.
	     * No checks are made to see if the listener has already been added. Multiple calls passing
	     * the same combination of event and listener will result in the listener being added, and
	     * called, multiple times.
	     * @returns this ChatCompletionStream, so that calls can be chained
	     */
	    on(event, listener) {
	        const listeners = __classPrivateFieldGet(this, _EventStream_listeners, "f")[event] || (__classPrivateFieldGet(this, _EventStream_listeners, "f")[event] = []);
	        listeners.push({ listener });
	        return this;
	    }
	    /**
	     * Removes the specified listener from the listener array for the event.
	     * off() will remove, at most, one instance of a listener from the listener array. If any single
	     * listener has been added multiple times to the listener array for the specified event, then
	     * off() must be called multiple times to remove each instance.
	     * @returns this ChatCompletionStream, so that calls can be chained
	     */
	    off(event, listener) {
	        const listeners = __classPrivateFieldGet(this, _EventStream_listeners, "f")[event];
	        if (!listeners)
	            return this;
	        const index = listeners.findIndex((l) => l.listener === listener);
	        if (index >= 0)
	            listeners.splice(index, 1);
	        return this;
	    }
	    /**
	     * Adds a one-time listener function for the event. The next time the event is triggered,
	     * this listener is removed and then invoked.
	     * @returns this ChatCompletionStream, so that calls can be chained
	     */
	    once(event, listener) {
	        const listeners = __classPrivateFieldGet(this, _EventStream_listeners, "f")[event] || (__classPrivateFieldGet(this, _EventStream_listeners, "f")[event] = []);
	        listeners.push({ listener, once: true });
	        return this;
	    }
	    /**
	     * This is similar to `.once()`, but returns a Promise that resolves the next time
	     * the event is triggered, instead of calling a listener callback.
	     * @returns a Promise that resolves the next time given event is triggered,
	     * or rejects if an error is emitted.  (If you request the 'error' event,
	     * returns a promise that resolves with the error).
	     *
	     * Example:
	     *
	     *   const message = await stream.emitted('message') // rejects if the stream errors
	     */
	    emitted(event) {
	        return new Promise((resolve, reject) => {
	            __classPrivateFieldSet(this, _EventStream_catchingPromiseCreated, true, "f");
	            if (event !== 'error')
	                this.once('error', reject);
	            this.once(event, resolve);
	        });
	    }
	    async done() {
	        __classPrivateFieldSet(this, _EventStream_catchingPromiseCreated, true, "f");
	        await __classPrivateFieldGet(this, _EventStream_endPromise, "f");
	    }
	    _emit(event, ...args) {
	        // make sure we don't emit any events after end
	        if (__classPrivateFieldGet(this, _EventStream_ended, "f")) {
	            return;
	        }
	        if (event === 'end') {
	            __classPrivateFieldSet(this, _EventStream_ended, true, "f");
	            __classPrivateFieldGet(this, _EventStream_resolveEndPromise, "f").call(this);
	        }
	        const listeners = __classPrivateFieldGet(this, _EventStream_listeners, "f")[event];
	        if (listeners) {
	            __classPrivateFieldGet(this, _EventStream_listeners, "f")[event] = listeners.filter((l) => !l.once);
	            listeners.forEach(({ listener }) => listener(...args));
	        }
	        if (event === 'abort') {
	            const error = args[0];
	            if (!__classPrivateFieldGet(this, _EventStream_catchingPromiseCreated, "f") && !listeners?.length) {
	                Promise.reject(error);
	            }
	            __classPrivateFieldGet(this, _EventStream_rejectConnectedPromise, "f").call(this, error);
	            __classPrivateFieldGet(this, _EventStream_rejectEndPromise, "f").call(this, error);
	            this._emit('end');
	            return;
	        }
	        if (event === 'error') {
	            // NOTE: _emit('error', error) should only be called from #handleError().
	            const error = args[0];
	            if (!__classPrivateFieldGet(this, _EventStream_catchingPromiseCreated, "f") && !listeners?.length) {
	                // Trigger an unhandled rejection if the user hasn't registered any error handlers.
	                // If you are seeing stack traces here, make sure to handle errors via either:
	                // - runner.on('error', () => ...)
	                // - await runner.done()
	                // - await runner.finalChatCompletion()
	                // - etc.
	                Promise.reject(error);
	            }
	            __classPrivateFieldGet(this, _EventStream_rejectConnectedPromise, "f").call(this, error);
	            __classPrivateFieldGet(this, _EventStream_rejectEndPromise, "f").call(this, error);
	            this._emit('end');
	        }
	    }
	    _emitFinal() { }
	};
	EventStream.EventStream = EventStream$1;
	_EventStream_connectedPromise = new WeakMap(), _EventStream_resolveConnectedPromise = new WeakMap(), _EventStream_rejectConnectedPromise = new WeakMap(), _EventStream_endPromise = new WeakMap(), _EventStream_resolveEndPromise = new WeakMap(), _EventStream_rejectEndPromise = new WeakMap(), _EventStream_listeners = new WeakMap(), _EventStream_ended = new WeakMap(), _EventStream_errored = new WeakMap(), _EventStream_aborted = new WeakMap(), _EventStream_catchingPromiseCreated = new WeakMap(), _EventStream_instances = new WeakSet(), _EventStream_handleError = function _EventStream_handleError(error) {
	    __classPrivateFieldSet(this, _EventStream_errored, true, "f");
	    if (error instanceof Error && error.name === 'AbortError') {
	        error = new error_1.APIUserAbortError();
	    }
	    if (error instanceof error_1.APIUserAbortError) {
	        __classPrivateFieldSet(this, _EventStream_aborted, true, "f");
	        return this._emit('abort', error);
	    }
	    if (error instanceof error_1.OpenAIError) {
	        return this._emit('error', error);
	    }
	    if (error instanceof Error) {
	        const openAIError = new error_1.OpenAIError(error.message);
	        // @ts-ignore
	        openAIError.cause = error;
	        return this._emit('error', openAIError);
	    }
	    return this._emit('error', new error_1.OpenAIError(String(error)));
	};
	
	return EventStream;
}

var parser$3 = {};

var hasRequiredParser$2;

function requireParser$2 () {
	if (hasRequiredParser$2) return parser$3;
	hasRequiredParser$2 = 1;
	Object.defineProperty(parser$3, "__esModule", { value: true });
	parser$3.validateInputTools = parser$3.hasAutoParseableInput = parser$3.shouldParseToolCall = parser$3.parseChatCompletion = parser$3.maybeParseChatCompletion = parser$3.isAutoParsableTool = parser$3.makeParseableTool = parser$3.isAutoParsableResponseFormat = parser$3.makeParseableResponseFormat = void 0;
	const error_1 = /*@__PURE__*/ requireError();
	function makeParseableResponseFormat(response_format, parser) {
	    const obj = { ...response_format };
	    Object.defineProperties(obj, {
	        $brand: {
	            value: 'auto-parseable-response-format',
	            enumerable: false,
	        },
	        $parseRaw: {
	            value: parser,
	            enumerable: false,
	        },
	    });
	    return obj;
	}
	parser$3.makeParseableResponseFormat = makeParseableResponseFormat;
	function isAutoParsableResponseFormat(response_format) {
	    return response_format?.['$brand'] === 'auto-parseable-response-format';
	}
	parser$3.isAutoParsableResponseFormat = isAutoParsableResponseFormat;
	function makeParseableTool(tool, { parser, callback, }) {
	    const obj = { ...tool };
	    Object.defineProperties(obj, {
	        $brand: {
	            value: 'auto-parseable-tool',
	            enumerable: false,
	        },
	        $parseRaw: {
	            value: parser,
	            enumerable: false,
	        },
	        $callback: {
	            value: callback,
	            enumerable: false,
	        },
	    });
	    return obj;
	}
	parser$3.makeParseableTool = makeParseableTool;
	function isAutoParsableTool(tool) {
	    return tool?.['$brand'] === 'auto-parseable-tool';
	}
	parser$3.isAutoParsableTool = isAutoParsableTool;
	function maybeParseChatCompletion(completion, params) {
	    if (!params || !hasAutoParseableInput(params)) {
	        return {
	            ...completion,
	            choices: completion.choices.map((choice) => ({
	                ...choice,
	                message: { ...choice.message, parsed: null, tool_calls: choice.message.tool_calls ?? [] },
	            })),
	        };
	    }
	    return parseChatCompletion(completion, params);
	}
	parser$3.maybeParseChatCompletion = maybeParseChatCompletion;
	function parseChatCompletion(completion, params) {
	    const choices = completion.choices.map((choice) => {
	        if (choice.finish_reason === 'length') {
	            throw new error_1.LengthFinishReasonError();
	        }
	        if (choice.finish_reason === 'content_filter') {
	            throw new error_1.ContentFilterFinishReasonError();
	        }
	        return {
	            ...choice,
	            message: {
	                ...choice.message,
	                tool_calls: choice.message.tool_calls?.map((toolCall) => parseToolCall(params, toolCall)) ?? [],
	                parsed: choice.message.content && !choice.message.refusal ?
	                    parseResponseFormat(params, choice.message.content)
	                    : null,
	            },
	        };
	    });
	    return { ...completion, choices };
	}
	parser$3.parseChatCompletion = parseChatCompletion;
	function parseResponseFormat(params, content) {
	    if (params.response_format?.type !== 'json_schema') {
	        return null;
	    }
	    if (params.response_format?.type === 'json_schema') {
	        if ('$parseRaw' in params.response_format) {
	            const response_format = params.response_format;
	            return response_format.$parseRaw(content);
	        }
	        return JSON.parse(content);
	    }
	    return null;
	}
	function parseToolCall(params, toolCall) {
	    const inputTool = params.tools?.find((inputTool) => inputTool.function?.name === toolCall.function.name);
	    return {
	        ...toolCall,
	        function: {
	            ...toolCall.function,
	            parsed_arguments: isAutoParsableTool(inputTool) ? inputTool.$parseRaw(toolCall.function.arguments)
	                : inputTool?.function.strict ? JSON.parse(toolCall.function.arguments)
	                    : null,
	        },
	    };
	}
	function shouldParseToolCall(params, toolCall) {
	    if (!params) {
	        return false;
	    }
	    const inputTool = params.tools?.find((inputTool) => inputTool.function?.name === toolCall.function.name);
	    return isAutoParsableTool(inputTool) || inputTool?.function.strict || false;
	}
	parser$3.shouldParseToolCall = shouldParseToolCall;
	function hasAutoParseableInput(params) {
	    if (isAutoParsableResponseFormat(params.response_format)) {
	        return true;
	    }
	    return (params.tools?.some((t) => isAutoParsableTool(t) || (t.type === 'function' && t.function.strict === true)) ?? false);
	}
	parser$3.hasAutoParseableInput = hasAutoParseableInput;
	function validateInputTools(tools) {
	    for (const tool of tools ?? []) {
	        if (tool.type !== 'function') {
	            throw new error_1.OpenAIError(`Currently only \`function\` tool types support auto-parsing; Received \`${tool.type}\``);
	        }
	        if (tool.function.strict !== true) {
	            throw new error_1.OpenAIError(`The \`${tool.function.name}\` tool is not marked with \`strict: true\`. Only strict function tools can be auto-parsed`);
	        }
	    }
	}
	parser$3.validateInputTools = validateInputTools;
	
	return parser$3;
}

var hasRequiredAbstractChatCompletionRunner;

function requireAbstractChatCompletionRunner () {
	if (hasRequiredAbstractChatCompletionRunner) return AbstractChatCompletionRunner;
	hasRequiredAbstractChatCompletionRunner = 1;
	var __classPrivateFieldGet = (AbstractChatCompletionRunner && AbstractChatCompletionRunner.__classPrivateFieldGet) || function (receiver, state, kind, f) {
	    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
	    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
	    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
	};
	var _AbstractChatCompletionRunner_instances, _AbstractChatCompletionRunner_getFinalContent, _AbstractChatCompletionRunner_getFinalMessage, _AbstractChatCompletionRunner_getFinalFunctionCall, _AbstractChatCompletionRunner_getFinalFunctionCallResult, _AbstractChatCompletionRunner_calculateTotalUsage, _AbstractChatCompletionRunner_validateParams, _AbstractChatCompletionRunner_stringifyFunctionCallResult;
	Object.defineProperty(AbstractChatCompletionRunner, "__esModule", { value: true });
	AbstractChatCompletionRunner.AbstractChatCompletionRunner = void 0;
	const error_1 = /*@__PURE__*/ requireError();
	const RunnableFunction_1 = /*@__PURE__*/ requireRunnableFunction();
	const chatCompletionUtils_1 = /*@__PURE__*/ requireChatCompletionUtils();
	const EventStream_1 = /*@__PURE__*/ requireEventStream();
	const parser_1 = /*@__PURE__*/ requireParser$2();
	const DEFAULT_MAX_CHAT_COMPLETIONS = 10;
	let AbstractChatCompletionRunner$1 = class AbstractChatCompletionRunner extends EventStream_1.EventStream {
	    constructor() {
	        super(...arguments);
	        _AbstractChatCompletionRunner_instances.add(this);
	        this._chatCompletions = [];
	        this.messages = [];
	    }
	    _addChatCompletion(chatCompletion) {
	        this._chatCompletions.push(chatCompletion);
	        this._emit('chatCompletion', chatCompletion);
	        const message = chatCompletion.choices[0]?.message;
	        if (message)
	            this._addMessage(message);
	        return chatCompletion;
	    }
	    _addMessage(message, emit = true) {
	        if (!('content' in message))
	            message.content = null;
	        this.messages.push(message);
	        if (emit) {
	            this._emit('message', message);
	            if (((0, chatCompletionUtils_1.isFunctionMessage)(message) || (0, chatCompletionUtils_1.isToolMessage)(message)) && message.content) {
	                // Note, this assumes that {role: 'tool', content: …} is always the result of a call of tool of type=function.
	                this._emit('functionCallResult', message.content);
	            }
	            else if ((0, chatCompletionUtils_1.isAssistantMessage)(message) && message.function_call) {
	                this._emit('functionCall', message.function_call);
	            }
	            else if ((0, chatCompletionUtils_1.isAssistantMessage)(message) && message.tool_calls) {
	                for (const tool_call of message.tool_calls) {
	                    if (tool_call.type === 'function') {
	                        this._emit('functionCall', tool_call.function);
	                    }
	                }
	            }
	        }
	    }
	    /**
	     * @returns a promise that resolves with the final ChatCompletion, or rejects
	     * if an error occurred or the stream ended prematurely without producing a ChatCompletion.
	     */
	    async finalChatCompletion() {
	        await this.done();
	        const completion = this._chatCompletions[this._chatCompletions.length - 1];
	        if (!completion)
	            throw new error_1.OpenAIError('stream ended without producing a ChatCompletion');
	        return completion;
	    }
	    /**
	     * @returns a promise that resolves with the content of the final ChatCompletionMessage, or rejects
	     * if an error occurred or the stream ended prematurely without producing a ChatCompletionMessage.
	     */
	    async finalContent() {
	        await this.done();
	        return __classPrivateFieldGet(this, _AbstractChatCompletionRunner_instances, "m", _AbstractChatCompletionRunner_getFinalContent).call(this);
	    }
	    /**
	     * @returns a promise that resolves with the the final assistant ChatCompletionMessage response,
	     * or rejects if an error occurred or the stream ended prematurely without producing a ChatCompletionMessage.
	     */
	    async finalMessage() {
	        await this.done();
	        return __classPrivateFieldGet(this, _AbstractChatCompletionRunner_instances, "m", _AbstractChatCompletionRunner_getFinalMessage).call(this);
	    }
	    /**
	     * @returns a promise that resolves with the content of the final FunctionCall, or rejects
	     * if an error occurred or the stream ended prematurely without producing a ChatCompletionMessage.
	     */
	    async finalFunctionCall() {
	        await this.done();
	        return __classPrivateFieldGet(this, _AbstractChatCompletionRunner_instances, "m", _AbstractChatCompletionRunner_getFinalFunctionCall).call(this);
	    }
	    async finalFunctionCallResult() {
	        await this.done();
	        return __classPrivateFieldGet(this, _AbstractChatCompletionRunner_instances, "m", _AbstractChatCompletionRunner_getFinalFunctionCallResult).call(this);
	    }
	    async totalUsage() {
	        await this.done();
	        return __classPrivateFieldGet(this, _AbstractChatCompletionRunner_instances, "m", _AbstractChatCompletionRunner_calculateTotalUsage).call(this);
	    }
	    allChatCompletions() {
	        return [...this._chatCompletions];
	    }
	    _emitFinal() {
	        const completion = this._chatCompletions[this._chatCompletions.length - 1];
	        if (completion)
	            this._emit('finalChatCompletion', completion);
	        const finalMessage = __classPrivateFieldGet(this, _AbstractChatCompletionRunner_instances, "m", _AbstractChatCompletionRunner_getFinalMessage).call(this);
	        if (finalMessage)
	            this._emit('finalMessage', finalMessage);
	        const finalContent = __classPrivateFieldGet(this, _AbstractChatCompletionRunner_instances, "m", _AbstractChatCompletionRunner_getFinalContent).call(this);
	        if (finalContent)
	            this._emit('finalContent', finalContent);
	        const finalFunctionCall = __classPrivateFieldGet(this, _AbstractChatCompletionRunner_instances, "m", _AbstractChatCompletionRunner_getFinalFunctionCall).call(this);
	        if (finalFunctionCall)
	            this._emit('finalFunctionCall', finalFunctionCall);
	        const finalFunctionCallResult = __classPrivateFieldGet(this, _AbstractChatCompletionRunner_instances, "m", _AbstractChatCompletionRunner_getFinalFunctionCallResult).call(this);
	        if (finalFunctionCallResult != null)
	            this._emit('finalFunctionCallResult', finalFunctionCallResult);
	        if (this._chatCompletions.some((c) => c.usage)) {
	            this._emit('totalUsage', __classPrivateFieldGet(this, _AbstractChatCompletionRunner_instances, "m", _AbstractChatCompletionRunner_calculateTotalUsage).call(this));
	        }
	    }
	    async _createChatCompletion(client, params, options) {
	        const signal = options?.signal;
	        if (signal) {
	            if (signal.aborted)
	                this.controller.abort();
	            signal.addEventListener('abort', () => this.controller.abort());
	        }
	        __classPrivateFieldGet(this, _AbstractChatCompletionRunner_instances, "m", _AbstractChatCompletionRunner_validateParams).call(this, params);
	        const chatCompletion = await client.chat.completions.create({ ...params, stream: false }, { ...options, signal: this.controller.signal });
	        this._connected();
	        return this._addChatCompletion((0, parser_1.parseChatCompletion)(chatCompletion, params));
	    }
	    async _runChatCompletion(client, params, options) {
	        for (const message of params.messages) {
	            this._addMessage(message, false);
	        }
	        return await this._createChatCompletion(client, params, options);
	    }
	    async _runFunctions(client, params, options) {
	        const role = 'function';
	        const { function_call = 'auto', stream, ...restParams } = params;
	        const singleFunctionToCall = typeof function_call !== 'string' && function_call?.name;
	        const { maxChatCompletions = DEFAULT_MAX_CHAT_COMPLETIONS } = options || {};
	        const functionsByName = {};
	        for (const f of params.functions) {
	            functionsByName[f.name || f.function.name] = f;
	        }
	        const functions = params.functions.map((f) => ({
	            name: f.name || f.function.name,
	            parameters: f.parameters,
	            description: f.description,
	        }));
	        for (const message of params.messages) {
	            this._addMessage(message, false);
	        }
	        for (let i = 0; i < maxChatCompletions; ++i) {
	            const chatCompletion = await this._createChatCompletion(client, {
	                ...restParams,
	                function_call,
	                functions,
	                messages: [...this.messages],
	            }, options);
	            const message = chatCompletion.choices[0]?.message;
	            if (!message) {
	                throw new error_1.OpenAIError(`missing message in ChatCompletion response`);
	            }
	            if (!message.function_call)
	                return;
	            const { name, arguments: args } = message.function_call;
	            const fn = functionsByName[name];
	            if (!fn) {
	                const content = `Invalid function_call: ${JSON.stringify(name)}. Available options are: ${functions
	                    .map((f) => JSON.stringify(f.name))
	                    .join(', ')}. Please try again`;
	                this._addMessage({ role, name, content });
	                continue;
	            }
	            else if (singleFunctionToCall && singleFunctionToCall !== name) {
	                const content = `Invalid function_call: ${JSON.stringify(name)}. ${JSON.stringify(singleFunctionToCall)} requested. Please try again`;
	                this._addMessage({ role, name, content });
	                continue;
	            }
	            let parsed;
	            try {
	                parsed = (0, RunnableFunction_1.isRunnableFunctionWithParse)(fn) ? await fn.parse(args) : args;
	            }
	            catch (error) {
	                this._addMessage({
	                    role,
	                    name,
	                    content: error instanceof Error ? error.message : String(error),
	                });
	                continue;
	            }
	            // @ts-expect-error it can't rule out `never` type.
	            const rawContent = await fn.function(parsed, this);
	            const content = __classPrivateFieldGet(this, _AbstractChatCompletionRunner_instances, "m", _AbstractChatCompletionRunner_stringifyFunctionCallResult).call(this, rawContent);
	            this._addMessage({ role, name, content });
	            if (singleFunctionToCall)
	                return;
	        }
	    }
	    async _runTools(client, params, options) {
	        const role = 'tool';
	        const { tool_choice = 'auto', stream, ...restParams } = params;
	        const singleFunctionToCall = typeof tool_choice !== 'string' && tool_choice?.function?.name;
	        const { maxChatCompletions = DEFAULT_MAX_CHAT_COMPLETIONS } = options || {};
	        // TODO(someday): clean this logic up
	        const inputTools = params.tools.map((tool) => {
	            if ((0, parser_1.isAutoParsableTool)(tool)) {
	                if (!tool.$callback) {
	                    throw new error_1.OpenAIError('Tool given to `.runTools()` that does not have an associated function');
	                }
	                return {
	                    type: 'function',
	                    function: {
	                        function: tool.$callback,
	                        name: tool.function.name,
	                        description: tool.function.description || '',
	                        parameters: tool.function.parameters,
	                        parse: tool.$parseRaw,
	                        strict: true,
	                    },
	                };
	            }
	            return tool;
	        });
	        const functionsByName = {};
	        for (const f of inputTools) {
	            if (f.type === 'function') {
	                functionsByName[f.function.name || f.function.function.name] = f.function;
	            }
	        }
	        const tools = 'tools' in params ?
	            inputTools.map((t) => t.type === 'function' ?
	                {
	                    type: 'function',
	                    function: {
	                        name: t.function.name || t.function.function.name,
	                        parameters: t.function.parameters,
	                        description: t.function.description,
	                        strict: t.function.strict,
	                    },
	                }
	                : t)
	            : undefined;
	        for (const message of params.messages) {
	            this._addMessage(message, false);
	        }
	        for (let i = 0; i < maxChatCompletions; ++i) {
	            const chatCompletion = await this._createChatCompletion(client, {
	                ...restParams,
	                tool_choice,
	                tools,
	                messages: [...this.messages],
	            }, options);
	            const message = chatCompletion.choices[0]?.message;
	            if (!message) {
	                throw new error_1.OpenAIError(`missing message in ChatCompletion response`);
	            }
	            if (!message.tool_calls?.length) {
	                return;
	            }
	            for (const tool_call of message.tool_calls) {
	                if (tool_call.type !== 'function')
	                    continue;
	                const tool_call_id = tool_call.id;
	                const { name, arguments: args } = tool_call.function;
	                const fn = functionsByName[name];
	                if (!fn) {
	                    const content = `Invalid tool_call: ${JSON.stringify(name)}. Available options are: ${Object.keys(functionsByName)
	                        .map((name) => JSON.stringify(name))
	                        .join(', ')}. Please try again`;
	                    this._addMessage({ role, tool_call_id, content });
	                    continue;
	                }
	                else if (singleFunctionToCall && singleFunctionToCall !== name) {
	                    const content = `Invalid tool_call: ${JSON.stringify(name)}. ${JSON.stringify(singleFunctionToCall)} requested. Please try again`;
	                    this._addMessage({ role, tool_call_id, content });
	                    continue;
	                }
	                let parsed;
	                try {
	                    parsed = (0, RunnableFunction_1.isRunnableFunctionWithParse)(fn) ? await fn.parse(args) : args;
	                }
	                catch (error) {
	                    const content = error instanceof Error ? error.message : String(error);
	                    this._addMessage({ role, tool_call_id, content });
	                    continue;
	                }
	                // @ts-expect-error it can't rule out `never` type.
	                const rawContent = await fn.function(parsed, this);
	                const content = __classPrivateFieldGet(this, _AbstractChatCompletionRunner_instances, "m", _AbstractChatCompletionRunner_stringifyFunctionCallResult).call(this, rawContent);
	                this._addMessage({ role, tool_call_id, content });
	                if (singleFunctionToCall) {
	                    return;
	                }
	            }
	        }
	        return;
	    }
	};
	AbstractChatCompletionRunner.AbstractChatCompletionRunner = AbstractChatCompletionRunner$1;
	_AbstractChatCompletionRunner_instances = new WeakSet(), _AbstractChatCompletionRunner_getFinalContent = function _AbstractChatCompletionRunner_getFinalContent() {
	    return __classPrivateFieldGet(this, _AbstractChatCompletionRunner_instances, "m", _AbstractChatCompletionRunner_getFinalMessage).call(this).content ?? null;
	}, _AbstractChatCompletionRunner_getFinalMessage = function _AbstractChatCompletionRunner_getFinalMessage() {
	    let i = this.messages.length;
	    while (i-- > 0) {
	        const message = this.messages[i];
	        if ((0, chatCompletionUtils_1.isAssistantMessage)(message)) {
	            const { function_call, ...rest } = message;
	            const ret = {
	                ...rest,
	                content: message.content ?? null,
	                refusal: message.refusal ?? null,
	            };
	            if (function_call) {
	                ret.function_call = function_call;
	            }
	            return ret;
	        }
	    }
	    throw new error_1.OpenAIError('stream ended without producing a ChatCompletionMessage with role=assistant');
	}, _AbstractChatCompletionRunner_getFinalFunctionCall = function _AbstractChatCompletionRunner_getFinalFunctionCall() {
	    for (let i = this.messages.length - 1; i >= 0; i--) {
	        const message = this.messages[i];
	        if ((0, chatCompletionUtils_1.isAssistantMessage)(message) && message?.function_call) {
	            return message.function_call;
	        }
	        if ((0, chatCompletionUtils_1.isAssistantMessage)(message) && message?.tool_calls?.length) {
	            return message.tool_calls.at(-1)?.function;
	        }
	    }
	    return;
	}, _AbstractChatCompletionRunner_getFinalFunctionCallResult = function _AbstractChatCompletionRunner_getFinalFunctionCallResult() {
	    for (let i = this.messages.length - 1; i >= 0; i--) {
	        const message = this.messages[i];
	        if ((0, chatCompletionUtils_1.isFunctionMessage)(message) && message.content != null) {
	            return message.content;
	        }
	        if ((0, chatCompletionUtils_1.isToolMessage)(message) &&
	            message.content != null &&
	            typeof message.content === 'string' &&
	            this.messages.some((x) => x.role === 'assistant' &&
	                x.tool_calls?.some((y) => y.type === 'function' && y.id === message.tool_call_id))) {
	            return message.content;
	        }
	    }
	    return;
	}, _AbstractChatCompletionRunner_calculateTotalUsage = function _AbstractChatCompletionRunner_calculateTotalUsage() {
	    const total = {
	        completion_tokens: 0,
	        prompt_tokens: 0,
	        total_tokens: 0,
	    };
	    for (const { usage } of this._chatCompletions) {
	        if (usage) {
	            total.completion_tokens += usage.completion_tokens;
	            total.prompt_tokens += usage.prompt_tokens;
	            total.total_tokens += usage.total_tokens;
	        }
	    }
	    return total;
	}, _AbstractChatCompletionRunner_validateParams = function _AbstractChatCompletionRunner_validateParams(params) {
	    if (params.n != null && params.n > 1) {
	        throw new error_1.OpenAIError('ChatCompletion convenience helpers only support n=1 at this time. To use n>1, please use chat.completions.create() directly.');
	    }
	}, _AbstractChatCompletionRunner_stringifyFunctionCallResult = function _AbstractChatCompletionRunner_stringifyFunctionCallResult(rawContent) {
	    return (typeof rawContent === 'string' ? rawContent
	        : rawContent === undefined ? 'undefined'
	            : JSON.stringify(rawContent));
	};
	
	return AbstractChatCompletionRunner;
}

var hasRequiredChatCompletionRunner;

function requireChatCompletionRunner () {
	if (hasRequiredChatCompletionRunner) return ChatCompletionRunner;
	hasRequiredChatCompletionRunner = 1;
	Object.defineProperty(ChatCompletionRunner, "__esModule", { value: true });
	ChatCompletionRunner.ChatCompletionRunner = void 0;
	const AbstractChatCompletionRunner_1 = /*@__PURE__*/ requireAbstractChatCompletionRunner();
	const chatCompletionUtils_1 = /*@__PURE__*/ requireChatCompletionUtils();
	let ChatCompletionRunner$1 = class ChatCompletionRunner extends AbstractChatCompletionRunner_1.AbstractChatCompletionRunner {
	    /** @deprecated - please use `runTools` instead. */
	    static runFunctions(client, params, options) {
	        const runner = new ChatCompletionRunner();
	        const opts = {
	            ...options,
	            headers: { ...options?.headers, 'X-Stainless-Helper-Method': 'runFunctions' },
	        };
	        runner._run(() => runner._runFunctions(client, params, opts));
	        return runner;
	    }
	    static runTools(client, params, options) {
	        const runner = new ChatCompletionRunner();
	        const opts = {
	            ...options,
	            headers: { ...options?.headers, 'X-Stainless-Helper-Method': 'runTools' },
	        };
	        runner._run(() => runner._runTools(client, params, opts));
	        return runner;
	    }
	    _addMessage(message, emit = true) {
	        super._addMessage(message, emit);
	        if ((0, chatCompletionUtils_1.isAssistantMessage)(message) && message.content) {
	            this._emit('content', message.content);
	        }
	    }
	};
	ChatCompletionRunner.ChatCompletionRunner = ChatCompletionRunner$1;
	
	return ChatCompletionRunner;
}

var ChatCompletionStreamingRunner = {};

var ChatCompletionStream = {};

var parser$2 = {};

var hasRequiredParser$1;

function requireParser$1 () {
	if (hasRequiredParser$1) return parser$2;
	hasRequiredParser$1 = 1;
	Object.defineProperty(parser$2, "__esModule", { value: true });
	parser$2.MalformedJSON = parser$2.PartialJSON = parser$2.partialParse = void 0;
	const STR = 0b000000001;
	const NUM = 0b000000010;
	const ARR = 0b000000100;
	const OBJ = 0b000001000;
	const NULL = 0b000010000;
	const BOOL = 0b000100000;
	const NAN = 0b001000000;
	const INFINITY = 0b010000000;
	const MINUS_INFINITY = 0b100000000;
	const INF = INFINITY | MINUS_INFINITY;
	const SPECIAL = NULL | BOOL | INF | NAN;
	const ATOM = STR | NUM | SPECIAL;
	const COLLECTION = ARR | OBJ;
	const ALL = ATOM | COLLECTION;
	const Allow = {
	    STR,
	    NUM,
	    ARR,
	    OBJ,
	    NULL,
	    BOOL,
	    NAN,
	    INFINITY,
	    MINUS_INFINITY,
	    INF,
	    SPECIAL,
	    ATOM,
	    COLLECTION,
	    ALL,
	};
	// The JSON string segment was unable to be parsed completely
	class PartialJSON extends Error {
	}
	parser$2.PartialJSON = PartialJSON;
	class MalformedJSON extends Error {
	}
	parser$2.MalformedJSON = MalformedJSON;
	/**
	 * Parse incomplete JSON
	 * @param {string} jsonString Partial JSON to be parsed
	 * @param {number} allowPartial Specify what types are allowed to be partial, see {@link Allow} for details
	 * @returns The parsed JSON
	 * @throws {PartialJSON} If the JSON is incomplete (related to the `allow` parameter)
	 * @throws {MalformedJSON} If the JSON is malformed
	 */
	function parseJSON(jsonString, allowPartial = Allow.ALL) {
	    if (typeof jsonString !== 'string') {
	        throw new TypeError(`expecting str, got ${typeof jsonString}`);
	    }
	    if (!jsonString.trim()) {
	        throw new Error(`${jsonString} is empty`);
	    }
	    return _parseJSON(jsonString.trim(), allowPartial);
	}
	const _parseJSON = (jsonString, allow) => {
	    const length = jsonString.length;
	    let index = 0;
	    const markPartialJSON = (msg) => {
	        throw new PartialJSON(`${msg} at position ${index}`);
	    };
	    const throwMalformedError = (msg) => {
	        throw new MalformedJSON(`${msg} at position ${index}`);
	    };
	    const parseAny = () => {
	        skipBlank();
	        if (index >= length)
	            markPartialJSON('Unexpected end of input');
	        if (jsonString[index] === '"')
	            return parseStr();
	        if (jsonString[index] === '{')
	            return parseObj();
	        if (jsonString[index] === '[')
	            return parseArr();
	        if (jsonString.substring(index, index + 4) === 'null' ||
	            (Allow.NULL & allow && length - index < 4 && 'null'.startsWith(jsonString.substring(index)))) {
	            index += 4;
	            return null;
	        }
	        if (jsonString.substring(index, index + 4) === 'true' ||
	            (Allow.BOOL & allow && length - index < 4 && 'true'.startsWith(jsonString.substring(index)))) {
	            index += 4;
	            return true;
	        }
	        if (jsonString.substring(index, index + 5) === 'false' ||
	            (Allow.BOOL & allow && length - index < 5 && 'false'.startsWith(jsonString.substring(index)))) {
	            index += 5;
	            return false;
	        }
	        if (jsonString.substring(index, index + 8) === 'Infinity' ||
	            (Allow.INFINITY & allow && length - index < 8 && 'Infinity'.startsWith(jsonString.substring(index)))) {
	            index += 8;
	            return Infinity;
	        }
	        if (jsonString.substring(index, index + 9) === '-Infinity' ||
	            (Allow.MINUS_INFINITY & allow &&
	                1 < length - index &&
	                length - index < 9 &&
	                '-Infinity'.startsWith(jsonString.substring(index)))) {
	            index += 9;
	            return -Infinity;
	        }
	        if (jsonString.substring(index, index + 3) === 'NaN' ||
	            (Allow.NAN & allow && length - index < 3 && 'NaN'.startsWith(jsonString.substring(index)))) {
	            index += 3;
	            return NaN;
	        }
	        return parseNum();
	    };
	    const parseStr = () => {
	        const start = index;
	        let escape = false;
	        index++; // skip initial quote
	        while (index < length && (jsonString[index] !== '"' || (escape && jsonString[index - 1] === '\\'))) {
	            escape = jsonString[index] === '\\' ? !escape : false;
	            index++;
	        }
	        if (jsonString.charAt(index) == '"') {
	            try {
	                return JSON.parse(jsonString.substring(start, ++index - Number(escape)));
	            }
	            catch (e) {
	                throwMalformedError(String(e));
	            }
	        }
	        else if (Allow.STR & allow) {
	            try {
	                return JSON.parse(jsonString.substring(start, index - Number(escape)) + '"');
	            }
	            catch (e) {
	                // SyntaxError: Invalid escape sequence
	                return JSON.parse(jsonString.substring(start, jsonString.lastIndexOf('\\')) + '"');
	            }
	        }
	        markPartialJSON('Unterminated string literal');
	    };
	    const parseObj = () => {
	        index++; // skip initial brace
	        skipBlank();
	        const obj = {};
	        try {
	            while (jsonString[index] !== '}') {
	                skipBlank();
	                if (index >= length && Allow.OBJ & allow)
	                    return obj;
	                const key = parseStr();
	                skipBlank();
	                index++; // skip colon
	                try {
	                    const value = parseAny();
	                    Object.defineProperty(obj, key, { value, writable: true, enumerable: true, configurable: true });
	                }
	                catch (e) {
	                    if (Allow.OBJ & allow)
	                        return obj;
	                    else
	                        throw e;
	                }
	                skipBlank();
	                if (jsonString[index] === ',')
	                    index++; // skip comma
	            }
	        }
	        catch (e) {
	            if (Allow.OBJ & allow)
	                return obj;
	            else
	                markPartialJSON("Expected '}' at end of object");
	        }
	        index++; // skip final brace
	        return obj;
	    };
	    const parseArr = () => {
	        index++; // skip initial bracket
	        const arr = [];
	        try {
	            while (jsonString[index] !== ']') {
	                arr.push(parseAny());
	                skipBlank();
	                if (jsonString[index] === ',') {
	                    index++; // skip comma
	                }
	            }
	        }
	        catch (e) {
	            if (Allow.ARR & allow) {
	                return arr;
	            }
	            markPartialJSON("Expected ']' at end of array");
	        }
	        index++; // skip final bracket
	        return arr;
	    };
	    const parseNum = () => {
	        if (index === 0) {
	            if (jsonString === '-' && Allow.NUM & allow)
	                markPartialJSON("Not sure what '-' is");
	            try {
	                return JSON.parse(jsonString);
	            }
	            catch (e) {
	                if (Allow.NUM & allow) {
	                    try {
	                        if ('.' === jsonString[jsonString.length - 1])
	                            return JSON.parse(jsonString.substring(0, jsonString.lastIndexOf('.')));
	                        return JSON.parse(jsonString.substring(0, jsonString.lastIndexOf('e')));
	                    }
	                    catch (e) { }
	                }
	                throwMalformedError(String(e));
	            }
	        }
	        const start = index;
	        if (jsonString[index] === '-')
	            index++;
	        while (jsonString[index] && !',]}'.includes(jsonString[index]))
	            index++;
	        if (index == length && !(Allow.NUM & allow))
	            markPartialJSON('Unterminated number literal');
	        try {
	            return JSON.parse(jsonString.substring(start, index));
	        }
	        catch (e) {
	            if (jsonString.substring(start, index) === '-' && Allow.NUM & allow)
	                markPartialJSON("Not sure what '-' is");
	            try {
	                return JSON.parse(jsonString.substring(start, jsonString.lastIndexOf('e')));
	            }
	            catch (e) {
	                throwMalformedError(String(e));
	            }
	        }
	    };
	    const skipBlank = () => {
	        while (index < length && ' \n\r\t'.includes(jsonString[index])) {
	            index++;
	        }
	    };
	    return parseAny();
	};
	// using this function with malformed JSON is undefined behavior
	const partialParse = (input) => parseJSON(input, Allow.ALL ^ Allow.NUM);
	parser$2.partialParse = partialParse;
	
	return parser$2;
}

var hasRequiredChatCompletionStream;

function requireChatCompletionStream () {
	if (hasRequiredChatCompletionStream) return ChatCompletionStream;
	hasRequiredChatCompletionStream = 1;
	var __classPrivateFieldSet = (ChatCompletionStream && ChatCompletionStream.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
	    if (kind === "m") throw new TypeError("Private method is not writable");
	    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
	    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
	    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
	};
	var __classPrivateFieldGet = (ChatCompletionStream && ChatCompletionStream.__classPrivateFieldGet) || function (receiver, state, kind, f) {
	    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
	    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
	    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
	};
	var _ChatCompletionStream_instances, _ChatCompletionStream_params, _ChatCompletionStream_choiceEventStates, _ChatCompletionStream_currentChatCompletionSnapshot, _ChatCompletionStream_beginRequest, _ChatCompletionStream_getChoiceEventState, _ChatCompletionStream_addChunk, _ChatCompletionStream_emitToolCallDoneEvent, _ChatCompletionStream_emitContentDoneEvents, _ChatCompletionStream_endRequest, _ChatCompletionStream_getAutoParseableResponseFormat, _ChatCompletionStream_accumulateChatCompletion;
	Object.defineProperty(ChatCompletionStream, "__esModule", { value: true });
	ChatCompletionStream.ChatCompletionStream = void 0;
	const error_1 = /*@__PURE__*/ requireError();
	const AbstractChatCompletionRunner_1 = /*@__PURE__*/ requireAbstractChatCompletionRunner();
	const streaming_1 = /*@__PURE__*/ requireStreaming();
	const parser_1 = /*@__PURE__*/ requireParser$2();
	const parser_2 = /*@__PURE__*/ requireParser$1();
	let ChatCompletionStream$1 = class ChatCompletionStream extends AbstractChatCompletionRunner_1.AbstractChatCompletionRunner {
	    constructor(params) {
	        super();
	        _ChatCompletionStream_instances.add(this);
	        _ChatCompletionStream_params.set(this, void 0);
	        _ChatCompletionStream_choiceEventStates.set(this, void 0);
	        _ChatCompletionStream_currentChatCompletionSnapshot.set(this, void 0);
	        __classPrivateFieldSet(this, _ChatCompletionStream_params, params, "f");
	        __classPrivateFieldSet(this, _ChatCompletionStream_choiceEventStates, [], "f");
	    }
	    get currentChatCompletionSnapshot() {
	        return __classPrivateFieldGet(this, _ChatCompletionStream_currentChatCompletionSnapshot, "f");
	    }
	    /**
	     * Intended for use on the frontend, consuming a stream produced with
	     * `.toReadableStream()` on the backend.
	     *
	     * Note that messages sent to the model do not appear in `.on('message')`
	     * in this context.
	     */
	    static fromReadableStream(stream) {
	        const runner = new ChatCompletionStream(null);
	        runner._run(() => runner._fromReadableStream(stream));
	        return runner;
	    }
	    static createChatCompletion(client, params, options) {
	        const runner = new ChatCompletionStream(params);
	        runner._run(() => runner._runChatCompletion(client, { ...params, stream: true }, { ...options, headers: { ...options?.headers, 'X-Stainless-Helper-Method': 'stream' } }));
	        return runner;
	    }
	    async _createChatCompletion(client, params, options) {
	        super._createChatCompletion;
	        const signal = options?.signal;
	        if (signal) {
	            if (signal.aborted)
	                this.controller.abort();
	            signal.addEventListener('abort', () => this.controller.abort());
	        }
	        __classPrivateFieldGet(this, _ChatCompletionStream_instances, "m", _ChatCompletionStream_beginRequest).call(this);
	        const stream = await client.chat.completions.create({ ...params, stream: true }, { ...options, signal: this.controller.signal });
	        this._connected();
	        for await (const chunk of stream) {
	            __classPrivateFieldGet(this, _ChatCompletionStream_instances, "m", _ChatCompletionStream_addChunk).call(this, chunk);
	        }
	        if (stream.controller.signal?.aborted) {
	            throw new error_1.APIUserAbortError();
	        }
	        return this._addChatCompletion(__classPrivateFieldGet(this, _ChatCompletionStream_instances, "m", _ChatCompletionStream_endRequest).call(this));
	    }
	    async _fromReadableStream(readableStream, options) {
	        const signal = options?.signal;
	        if (signal) {
	            if (signal.aborted)
	                this.controller.abort();
	            signal.addEventListener('abort', () => this.controller.abort());
	        }
	        __classPrivateFieldGet(this, _ChatCompletionStream_instances, "m", _ChatCompletionStream_beginRequest).call(this);
	        this._connected();
	        const stream = streaming_1.Stream.fromReadableStream(readableStream, this.controller);
	        let chatId;
	        for await (const chunk of stream) {
	            if (chatId && chatId !== chunk.id) {
	                // A new request has been made.
	                this._addChatCompletion(__classPrivateFieldGet(this, _ChatCompletionStream_instances, "m", _ChatCompletionStream_endRequest).call(this));
	            }
	            __classPrivateFieldGet(this, _ChatCompletionStream_instances, "m", _ChatCompletionStream_addChunk).call(this, chunk);
	            chatId = chunk.id;
	        }
	        if (stream.controller.signal?.aborted) {
	            throw new error_1.APIUserAbortError();
	        }
	        return this._addChatCompletion(__classPrivateFieldGet(this, _ChatCompletionStream_instances, "m", _ChatCompletionStream_endRequest).call(this));
	    }
	    [(_ChatCompletionStream_params = new WeakMap(), _ChatCompletionStream_choiceEventStates = new WeakMap(), _ChatCompletionStream_currentChatCompletionSnapshot = new WeakMap(), _ChatCompletionStream_instances = new WeakSet(), _ChatCompletionStream_beginRequest = function _ChatCompletionStream_beginRequest() {
	        if (this.ended)
	            return;
	        __classPrivateFieldSet(this, _ChatCompletionStream_currentChatCompletionSnapshot, undefined, "f");
	    }, _ChatCompletionStream_getChoiceEventState = function _ChatCompletionStream_getChoiceEventState(choice) {
	        let state = __classPrivateFieldGet(this, _ChatCompletionStream_choiceEventStates, "f")[choice.index];
	        if (state) {
	            return state;
	        }
	        state = {
	            content_done: false,
	            refusal_done: false,
	            logprobs_content_done: false,
	            logprobs_refusal_done: false,
	            done_tool_calls: new Set(),
	            current_tool_call_index: null,
	        };
	        __classPrivateFieldGet(this, _ChatCompletionStream_choiceEventStates, "f")[choice.index] = state;
	        return state;
	    }, _ChatCompletionStream_addChunk = function _ChatCompletionStream_addChunk(chunk) {
	        if (this.ended)
	            return;
	        const completion = __classPrivateFieldGet(this, _ChatCompletionStream_instances, "m", _ChatCompletionStream_accumulateChatCompletion).call(this, chunk);
	        this._emit('chunk', chunk, completion);
	        for (const choice of chunk.choices) {
	            const choiceSnapshot = completion.choices[choice.index];
	            if (choice.delta.content != null &&
	                choiceSnapshot.message?.role === 'assistant' &&
	                choiceSnapshot.message?.content) {
	                this._emit('content', choice.delta.content, choiceSnapshot.message.content);
	                this._emit('content.delta', {
	                    delta: choice.delta.content,
	                    snapshot: choiceSnapshot.message.content,
	                    parsed: choiceSnapshot.message.parsed,
	                });
	            }
	            if (choice.delta.refusal != null &&
	                choiceSnapshot.message?.role === 'assistant' &&
	                choiceSnapshot.message?.refusal) {
	                this._emit('refusal.delta', {
	                    delta: choice.delta.refusal,
	                    snapshot: choiceSnapshot.message.refusal,
	                });
	            }
	            if (choice.logprobs?.content != null && choiceSnapshot.message?.role === 'assistant') {
	                this._emit('logprobs.content.delta', {
	                    content: choice.logprobs?.content,
	                    snapshot: choiceSnapshot.logprobs?.content ?? [],
	                });
	            }
	            if (choice.logprobs?.refusal != null && choiceSnapshot.message?.role === 'assistant') {
	                this._emit('logprobs.refusal.delta', {
	                    refusal: choice.logprobs?.refusal,
	                    snapshot: choiceSnapshot.logprobs?.refusal ?? [],
	                });
	            }
	            const state = __classPrivateFieldGet(this, _ChatCompletionStream_instances, "m", _ChatCompletionStream_getChoiceEventState).call(this, choiceSnapshot);
	            if (choiceSnapshot.finish_reason) {
	                __classPrivateFieldGet(this, _ChatCompletionStream_instances, "m", _ChatCompletionStream_emitContentDoneEvents).call(this, choiceSnapshot);
	                if (state.current_tool_call_index != null) {
	                    __classPrivateFieldGet(this, _ChatCompletionStream_instances, "m", _ChatCompletionStream_emitToolCallDoneEvent).call(this, choiceSnapshot, state.current_tool_call_index);
	                }
	            }
	            for (const toolCall of choice.delta.tool_calls ?? []) {
	                if (state.current_tool_call_index !== toolCall.index) {
	                    __classPrivateFieldGet(this, _ChatCompletionStream_instances, "m", _ChatCompletionStream_emitContentDoneEvents).call(this, choiceSnapshot);
	                    // new tool call started, the previous one is done
	                    if (state.current_tool_call_index != null) {
	                        __classPrivateFieldGet(this, _ChatCompletionStream_instances, "m", _ChatCompletionStream_emitToolCallDoneEvent).call(this, choiceSnapshot, state.current_tool_call_index);
	                    }
	                }
	                state.current_tool_call_index = toolCall.index;
	            }
	            for (const toolCallDelta of choice.delta.tool_calls ?? []) {
	                const toolCallSnapshot = choiceSnapshot.message.tool_calls?.[toolCallDelta.index];
	                if (!toolCallSnapshot?.type) {
	                    continue;
	                }
	                if (toolCallSnapshot?.type === 'function') {
	                    this._emit('tool_calls.function.arguments.delta', {
	                        name: toolCallSnapshot.function?.name,
	                        index: toolCallDelta.index,
	                        arguments: toolCallSnapshot.function.arguments,
	                        parsed_arguments: toolCallSnapshot.function.parsed_arguments,
	                        arguments_delta: toolCallDelta.function?.arguments ?? '',
	                    });
	                }
	                else {
	                    assertNever(toolCallSnapshot?.type);
	                }
	            }
	        }
	    }, _ChatCompletionStream_emitToolCallDoneEvent = function _ChatCompletionStream_emitToolCallDoneEvent(choiceSnapshot, toolCallIndex) {
	        const state = __classPrivateFieldGet(this, _ChatCompletionStream_instances, "m", _ChatCompletionStream_getChoiceEventState).call(this, choiceSnapshot);
	        if (state.done_tool_calls.has(toolCallIndex)) {
	            // we've already fired the done event
	            return;
	        }
	        const toolCallSnapshot = choiceSnapshot.message.tool_calls?.[toolCallIndex];
	        if (!toolCallSnapshot) {
	            throw new Error('no tool call snapshot');
	        }
	        if (!toolCallSnapshot.type) {
	            throw new Error('tool call snapshot missing `type`');
	        }
	        if (toolCallSnapshot.type === 'function') {
	            const inputTool = __classPrivateFieldGet(this, _ChatCompletionStream_params, "f")?.tools?.find((tool) => tool.type === 'function' && tool.function.name === toolCallSnapshot.function.name);
	            this._emit('tool_calls.function.arguments.done', {
	                name: toolCallSnapshot.function.name,
	                index: toolCallIndex,
	                arguments: toolCallSnapshot.function.arguments,
	                parsed_arguments: (0, parser_1.isAutoParsableTool)(inputTool) ? inputTool.$parseRaw(toolCallSnapshot.function.arguments)
	                    : inputTool?.function.strict ? JSON.parse(toolCallSnapshot.function.arguments)
	                        : null,
	            });
	        }
	        else {
	            assertNever(toolCallSnapshot.type);
	        }
	    }, _ChatCompletionStream_emitContentDoneEvents = function _ChatCompletionStream_emitContentDoneEvents(choiceSnapshot) {
	        const state = __classPrivateFieldGet(this, _ChatCompletionStream_instances, "m", _ChatCompletionStream_getChoiceEventState).call(this, choiceSnapshot);
	        if (choiceSnapshot.message.content && !state.content_done) {
	            state.content_done = true;
	            const responseFormat = __classPrivateFieldGet(this, _ChatCompletionStream_instances, "m", _ChatCompletionStream_getAutoParseableResponseFormat).call(this);
	            this._emit('content.done', {
	                content: choiceSnapshot.message.content,
	                parsed: responseFormat ? responseFormat.$parseRaw(choiceSnapshot.message.content) : null,
	            });
	        }
	        if (choiceSnapshot.message.refusal && !state.refusal_done) {
	            state.refusal_done = true;
	            this._emit('refusal.done', { refusal: choiceSnapshot.message.refusal });
	        }
	        if (choiceSnapshot.logprobs?.content && !state.logprobs_content_done) {
	            state.logprobs_content_done = true;
	            this._emit('logprobs.content.done', { content: choiceSnapshot.logprobs.content });
	        }
	        if (choiceSnapshot.logprobs?.refusal && !state.logprobs_refusal_done) {
	            state.logprobs_refusal_done = true;
	            this._emit('logprobs.refusal.done', { refusal: choiceSnapshot.logprobs.refusal });
	        }
	    }, _ChatCompletionStream_endRequest = function _ChatCompletionStream_endRequest() {
	        if (this.ended) {
	            throw new error_1.OpenAIError(`stream has ended, this shouldn't happen`);
	        }
	        const snapshot = __classPrivateFieldGet(this, _ChatCompletionStream_currentChatCompletionSnapshot, "f");
	        if (!snapshot) {
	            throw new error_1.OpenAIError(`request ended without sending any chunks`);
	        }
	        __classPrivateFieldSet(this, _ChatCompletionStream_currentChatCompletionSnapshot, undefined, "f");
	        __classPrivateFieldSet(this, _ChatCompletionStream_choiceEventStates, [], "f");
	        return finalizeChatCompletion(snapshot, __classPrivateFieldGet(this, _ChatCompletionStream_params, "f"));
	    }, _ChatCompletionStream_getAutoParseableResponseFormat = function _ChatCompletionStream_getAutoParseableResponseFormat() {
	        const responseFormat = __classPrivateFieldGet(this, _ChatCompletionStream_params, "f")?.response_format;
	        if ((0, parser_1.isAutoParsableResponseFormat)(responseFormat)) {
	            return responseFormat;
	        }
	        return null;
	    }, _ChatCompletionStream_accumulateChatCompletion = function _ChatCompletionStream_accumulateChatCompletion(chunk) {
	        var _a, _b, _c, _d;
	        let snapshot = __classPrivateFieldGet(this, _ChatCompletionStream_currentChatCompletionSnapshot, "f");
	        const { choices, ...rest } = chunk;
	        if (!snapshot) {
	            snapshot = __classPrivateFieldSet(this, _ChatCompletionStream_currentChatCompletionSnapshot, {
	                ...rest,
	                choices: [],
	            }, "f");
	        }
	        else {
	            Object.assign(snapshot, rest);
	        }
	        for (const { delta, finish_reason, index, logprobs = null, ...other } of chunk.choices) {
	            let choice = snapshot.choices[index];
	            if (!choice) {
	                choice = snapshot.choices[index] = { finish_reason, index, message: {}, logprobs, ...other };
	            }
	            if (logprobs) {
	                if (!choice.logprobs) {
	                    choice.logprobs = Object.assign({}, logprobs);
	                }
	                else {
	                    const { content, refusal, ...rest } = logprobs;
	                    Object.assign(choice.logprobs, rest);
	                    if (content) {
	                        (_a = choice.logprobs).content ?? (_a.content = []);
	                        choice.logprobs.content.push(...content);
	                    }
	                    if (refusal) {
	                        (_b = choice.logprobs).refusal ?? (_b.refusal = []);
	                        choice.logprobs.refusal.push(...refusal);
	                    }
	                }
	            }
	            if (finish_reason) {
	                choice.finish_reason = finish_reason;
	                if (__classPrivateFieldGet(this, _ChatCompletionStream_params, "f") && (0, parser_1.hasAutoParseableInput)(__classPrivateFieldGet(this, _ChatCompletionStream_params, "f"))) {
	                    if (finish_reason === 'length') {
	                        throw new error_1.LengthFinishReasonError();
	                    }
	                    if (finish_reason === 'content_filter') {
	                        throw new error_1.ContentFilterFinishReasonError();
	                    }
	                }
	            }
	            Object.assign(choice, other);
	            if (!delta)
	                continue; // Shouldn't happen; just in case.
	            const { content, refusal, function_call, role, tool_calls, ...rest } = delta;
	            Object.assign(choice.message, rest);
	            if (refusal) {
	                choice.message.refusal = (choice.message.refusal || '') + refusal;
	            }
	            if (role)
	                choice.message.role = role;
	            if (function_call) {
	                if (!choice.message.function_call) {
	                    choice.message.function_call = function_call;
	                }
	                else {
	                    if (function_call.name)
	                        choice.message.function_call.name = function_call.name;
	                    if (function_call.arguments) {
	                        (_c = choice.message.function_call).arguments ?? (_c.arguments = '');
	                        choice.message.function_call.arguments += function_call.arguments;
	                    }
	                }
	            }
	            if (content) {
	                choice.message.content = (choice.message.content || '') + content;
	                if (!choice.message.refusal && __classPrivateFieldGet(this, _ChatCompletionStream_instances, "m", _ChatCompletionStream_getAutoParseableResponseFormat).call(this)) {
	                    choice.message.parsed = (0, parser_2.partialParse)(choice.message.content);
	                }
	            }
	            if (tool_calls) {
	                if (!choice.message.tool_calls)
	                    choice.message.tool_calls = [];
	                for (const { index, id, type, function: fn, ...rest } of tool_calls) {
	                    const tool_call = ((_d = choice.message.tool_calls)[index] ?? (_d[index] = {}));
	                    Object.assign(tool_call, rest);
	                    if (id)
	                        tool_call.id = id;
	                    if (type)
	                        tool_call.type = type;
	                    if (fn)
	                        tool_call.function ?? (tool_call.function = { name: fn.name ?? '', arguments: '' });
	                    if (fn?.name)
	                        tool_call.function.name = fn.name;
	                    if (fn?.arguments) {
	                        tool_call.function.arguments += fn.arguments;
	                        if ((0, parser_1.shouldParseToolCall)(__classPrivateFieldGet(this, _ChatCompletionStream_params, "f"), tool_call)) {
	                            tool_call.function.parsed_arguments = (0, parser_2.partialParse)(tool_call.function.arguments);
	                        }
	                    }
	                }
	            }
	        }
	        return snapshot;
	    }, Symbol.asyncIterator)]() {
	        const pushQueue = [];
	        const readQueue = [];
	        let done = false;
	        this.on('chunk', (chunk) => {
	            const reader = readQueue.shift();
	            if (reader) {
	                reader.resolve(chunk);
	            }
	            else {
	                pushQueue.push(chunk);
	            }
	        });
	        this.on('end', () => {
	            done = true;
	            for (const reader of readQueue) {
	                reader.resolve(undefined);
	            }
	            readQueue.length = 0;
	        });
	        this.on('abort', (err) => {
	            done = true;
	            for (const reader of readQueue) {
	                reader.reject(err);
	            }
	            readQueue.length = 0;
	        });
	        this.on('error', (err) => {
	            done = true;
	            for (const reader of readQueue) {
	                reader.reject(err);
	            }
	            readQueue.length = 0;
	        });
	        return {
	            next: async () => {
	                if (!pushQueue.length) {
	                    if (done) {
	                        return { value: undefined, done: true };
	                    }
	                    return new Promise((resolve, reject) => readQueue.push({ resolve, reject })).then((chunk) => (chunk ? { value: chunk, done: false } : { value: undefined, done: true }));
	                }
	                const chunk = pushQueue.shift();
	                return { value: chunk, done: false };
	            },
	            return: async () => {
	                this.abort();
	                return { value: undefined, done: true };
	            },
	        };
	    }
	    toReadableStream() {
	        const stream = new streaming_1.Stream(this[Symbol.asyncIterator].bind(this), this.controller);
	        return stream.toReadableStream();
	    }
	};
	ChatCompletionStream.ChatCompletionStream = ChatCompletionStream$1;
	function finalizeChatCompletion(snapshot, params) {
	    const { id, choices, created, model, system_fingerprint, ...rest } = snapshot;
	    const completion = {
	        ...rest,
	        id,
	        choices: choices.map(({ message, finish_reason, index, logprobs, ...choiceRest }) => {
	            if (!finish_reason) {
	                throw new error_1.OpenAIError(`missing finish_reason for choice ${index}`);
	            }
	            const { content = null, function_call, tool_calls, ...messageRest } = message;
	            const role = message.role; // this is what we expect; in theory it could be different which would make our types a slight lie but would be fine.
	            if (!role) {
	                throw new error_1.OpenAIError(`missing role for choice ${index}`);
	            }
	            if (function_call) {
	                const { arguments: args, name } = function_call;
	                if (args == null) {
	                    throw new error_1.OpenAIError(`missing function_call.arguments for choice ${index}`);
	                }
	                if (!name) {
	                    throw new error_1.OpenAIError(`missing function_call.name for choice ${index}`);
	                }
	                return {
	                    ...choiceRest,
	                    message: {
	                        content,
	                        function_call: { arguments: args, name },
	                        role,
	                        refusal: message.refusal ?? null,
	                    },
	                    finish_reason,
	                    index,
	                    logprobs,
	                };
	            }
	            if (tool_calls) {
	                return {
	                    ...choiceRest,
	                    index,
	                    finish_reason,
	                    logprobs,
	                    message: {
	                        ...messageRest,
	                        role,
	                        content,
	                        refusal: message.refusal ?? null,
	                        tool_calls: tool_calls.map((tool_call, i) => {
	                            const { function: fn, type, id, ...toolRest } = tool_call;
	                            const { arguments: args, name, ...fnRest } = fn || {};
	                            if (id == null) {
	                                throw new error_1.OpenAIError(`missing choices[${index}].tool_calls[${i}].id\n${str(snapshot)}`);
	                            }
	                            if (type == null) {
	                                throw new error_1.OpenAIError(`missing choices[${index}].tool_calls[${i}].type\n${str(snapshot)}`);
	                            }
	                            if (name == null) {
	                                throw new error_1.OpenAIError(`missing choices[${index}].tool_calls[${i}].function.name\n${str(snapshot)}`);
	                            }
	                            if (args == null) {
	                                throw new error_1.OpenAIError(`missing choices[${index}].tool_calls[${i}].function.arguments\n${str(snapshot)}`);
	                            }
	                            return { ...toolRest, id, type, function: { ...fnRest, name, arguments: args } };
	                        }),
	                    },
	                };
	            }
	            return {
	                ...choiceRest,
	                message: { ...messageRest, content, role, refusal: message.refusal ?? null },
	                finish_reason,
	                index,
	                logprobs,
	            };
	        }),
	        created,
	        model,
	        object: 'chat.completion',
	        ...(system_fingerprint ? { system_fingerprint } : {}),
	    };
	    return (0, parser_1.maybeParseChatCompletion)(completion, params);
	}
	function str(x) {
	    return JSON.stringify(x);
	}
	function assertNever(_x) { }
	
	return ChatCompletionStream;
}

var hasRequiredChatCompletionStreamingRunner;

function requireChatCompletionStreamingRunner () {
	if (hasRequiredChatCompletionStreamingRunner) return ChatCompletionStreamingRunner;
	hasRequiredChatCompletionStreamingRunner = 1;
	Object.defineProperty(ChatCompletionStreamingRunner, "__esModule", { value: true });
	ChatCompletionStreamingRunner.ChatCompletionStreamingRunner = void 0;
	const ChatCompletionStream_1 = /*@__PURE__*/ requireChatCompletionStream();
	let ChatCompletionStreamingRunner$1 = class ChatCompletionStreamingRunner extends ChatCompletionStream_1.ChatCompletionStream {
	    static fromReadableStream(stream) {
	        const runner = new ChatCompletionStreamingRunner(null);
	        runner._run(() => runner._fromReadableStream(stream));
	        return runner;
	    }
	    /** @deprecated - please use `runTools` instead. */
	    static runFunctions(client, params, options) {
	        const runner = new ChatCompletionStreamingRunner(null);
	        const opts = {
	            ...options,
	            headers: { ...options?.headers, 'X-Stainless-Helper-Method': 'runFunctions' },
	        };
	        runner._run(() => runner._runFunctions(client, params, opts));
	        return runner;
	    }
	    static runTools(client, params, options) {
	        const runner = new ChatCompletionStreamingRunner(
	        // @ts-expect-error TODO these types are incompatible
	        params);
	        const opts = {
	            ...options,
	            headers: { ...options?.headers, 'X-Stainless-Helper-Method': 'runTools' },
	        };
	        runner._run(() => runner._runTools(client, params, opts));
	        return runner;
	    }
	};
	ChatCompletionStreamingRunner.ChatCompletionStreamingRunner = ChatCompletionStreamingRunner$1;
	
	return ChatCompletionStreamingRunner;
}

var hasRequiredCompletions$1;

function requireCompletions$1 () {
	if (hasRequiredCompletions$1) return completions$1;
	hasRequiredCompletions$1 = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.Completions = exports.ChatCompletionStream = exports.ParsingToolFunction = exports.ParsingFunction = exports.ChatCompletionStreamingRunner = exports.ChatCompletionRunner = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		const ChatCompletionRunner_1 = /*@__PURE__*/ requireChatCompletionRunner();
		var ChatCompletionRunner_2 = /*@__PURE__*/ requireChatCompletionRunner();
		Object.defineProperty(exports, "ChatCompletionRunner", { enumerable: true, get: function () { return ChatCompletionRunner_2.ChatCompletionRunner; } });
		const ChatCompletionStreamingRunner_1 = /*@__PURE__*/ requireChatCompletionStreamingRunner();
		var ChatCompletionStreamingRunner_2 = /*@__PURE__*/ requireChatCompletionStreamingRunner();
		Object.defineProperty(exports, "ChatCompletionStreamingRunner", { enumerable: true, get: function () { return ChatCompletionStreamingRunner_2.ChatCompletionStreamingRunner; } });
		var RunnableFunction_1 = /*@__PURE__*/ requireRunnableFunction();
		Object.defineProperty(exports, "ParsingFunction", { enumerable: true, get: function () { return RunnableFunction_1.ParsingFunction; } });
		Object.defineProperty(exports, "ParsingToolFunction", { enumerable: true, get: function () { return RunnableFunction_1.ParsingToolFunction; } });
		const ChatCompletionStream_1 = /*@__PURE__*/ requireChatCompletionStream();
		const parser_1 = /*@__PURE__*/ requireParser$2();
		var ChatCompletionStream_2 = /*@__PURE__*/ requireChatCompletionStream();
		Object.defineProperty(exports, "ChatCompletionStream", { enumerable: true, get: function () { return ChatCompletionStream_2.ChatCompletionStream; } });
		class Completions extends resource_1.APIResource {
		    parse(body, options) {
		        (0, parser_1.validateInputTools)(body.tools);
		        return this._client.chat.completions
		            .create(body, {
		            ...options,
		            headers: {
		                ...options?.headers,
		                'X-Stainless-Helper-Method': 'beta.chat.completions.parse',
		            },
		        })
		            ._thenUnwrap((completion) => (0, parser_1.parseChatCompletion)(completion, body));
		    }
		    runFunctions(body, options) {
		        if (body.stream) {
		            return ChatCompletionStreamingRunner_1.ChatCompletionStreamingRunner.runFunctions(this._client, body, options);
		        }
		        return ChatCompletionRunner_1.ChatCompletionRunner.runFunctions(this._client, body, options);
		    }
		    runTools(body, options) {
		        if (body.stream) {
		            return ChatCompletionStreamingRunner_1.ChatCompletionStreamingRunner.runTools(this._client, body, options);
		        }
		        return ChatCompletionRunner_1.ChatCompletionRunner.runTools(this._client, body, options);
		    }
		    /**
		     * Creates a chat completion stream
		     */
		    stream(body, options) {
		        return ChatCompletionStream_1.ChatCompletionStream.createChatCompletion(this._client, body, options);
		    }
		}
		exports.Completions = Completions;
		
	} (completions$1));
	return completions$1;
}

var hasRequiredChat;

function requireChat () {
	if (hasRequiredChat) return chat;
	hasRequiredChat = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		var __createBinding = (chat && chat.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __setModuleDefault = (chat && chat.__setModuleDefault) || (Object.create ? (function(o, v) {
		    Object.defineProperty(o, "default", { enumerable: true, value: v });
		}) : function(o, v) {
		    o["default"] = v;
		});
		var __importStar = (chat && chat.__importStar) || function (mod) {
		    if (mod && mod.__esModule) return mod;
		    var result = {};
		    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
		    __setModuleDefault(result, mod);
		    return result;
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.Chat = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		const CompletionsAPI = __importStar(/*@__PURE__*/ requireCompletions$1());
		class Chat extends resource_1.APIResource {
		    constructor() {
		        super(...arguments);
		        this.completions = new CompletionsAPI.Completions(this._client);
		    }
		}
		exports.Chat = Chat;
		(function (Chat) {
		    Chat.Completions = CompletionsAPI.Completions;
		})(Chat = exports.Chat || (exports.Chat = {}));
		
	} (chat));
	return chat;
}

var threads = {};

var AssistantStream = {};

var hasRequiredAssistantStream;

function requireAssistantStream () {
	if (hasRequiredAssistantStream) return AssistantStream;
	hasRequiredAssistantStream = 1;
	var __createBinding = (AssistantStream && AssistantStream.__createBinding) || (Object.create ? (function(o, m, k, k2) {
	    if (k2 === undefined) k2 = k;
	    var desc = Object.getOwnPropertyDescriptor(m, k);
	    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
	      desc = { enumerable: true, get: function() { return m[k]; } };
	    }
	    Object.defineProperty(o, k2, desc);
	}) : (function(o, m, k, k2) {
	    if (k2 === undefined) k2 = k;
	    o[k2] = m[k];
	}));
	var __setModuleDefault = (AssistantStream && AssistantStream.__setModuleDefault) || (Object.create ? (function(o, v) {
	    Object.defineProperty(o, "default", { enumerable: true, value: v });
	}) : function(o, v) {
	    o["default"] = v;
	});
	var __importStar = (AssistantStream && AssistantStream.__importStar) || function (mod) {
	    if (mod && mod.__esModule) return mod;
	    var result = {};
	    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
	    __setModuleDefault(result, mod);
	    return result;
	};
	var __classPrivateFieldGet = (AssistantStream && AssistantStream.__classPrivateFieldGet) || function (receiver, state, kind, f) {
	    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
	    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
	    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
	};
	var __classPrivateFieldSet = (AssistantStream && AssistantStream.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
	    if (kind === "m") throw new TypeError("Private method is not writable");
	    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
	    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
	    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
	};
	var _AssistantStream_instances, _AssistantStream_events, _AssistantStream_runStepSnapshots, _AssistantStream_messageSnapshots, _AssistantStream_messageSnapshot, _AssistantStream_finalRun, _AssistantStream_currentContentIndex, _AssistantStream_currentContent, _AssistantStream_currentToolCallIndex, _AssistantStream_currentToolCall, _AssistantStream_currentEvent, _AssistantStream_currentRunSnapshot, _AssistantStream_currentRunStepSnapshot, _AssistantStream_addEvent, _AssistantStream_endRequest, _AssistantStream_handleMessage, _AssistantStream_handleRunStep, _AssistantStream_handleEvent, _AssistantStream_accumulateRunStep, _AssistantStream_accumulateMessage, _AssistantStream_accumulateContent, _AssistantStream_handleRun;
	Object.defineProperty(AssistantStream, "__esModule", { value: true });
	AssistantStream.AssistantStream = void 0;
	const Core = __importStar(/*@__PURE__*/ requireCore());
	const streaming_1 = /*@__PURE__*/ requireStreaming();
	const error_1 = /*@__PURE__*/ requireError();
	const EventStream_1 = /*@__PURE__*/ requireEventStream();
	let AssistantStream$1 = class AssistantStream extends EventStream_1.EventStream {
	    constructor() {
	        super(...arguments);
	        _AssistantStream_instances.add(this);
	        //Track all events in a single list for reference
	        _AssistantStream_events.set(this, []);
	        //Used to accumulate deltas
	        //We are accumulating many types so the value here is not strict
	        _AssistantStream_runStepSnapshots.set(this, {});
	        _AssistantStream_messageSnapshots.set(this, {});
	        _AssistantStream_messageSnapshot.set(this, void 0);
	        _AssistantStream_finalRun.set(this, void 0);
	        _AssistantStream_currentContentIndex.set(this, void 0);
	        _AssistantStream_currentContent.set(this, void 0);
	        _AssistantStream_currentToolCallIndex.set(this, void 0);
	        _AssistantStream_currentToolCall.set(this, void 0);
	        //For current snapshot methods
	        _AssistantStream_currentEvent.set(this, void 0);
	        _AssistantStream_currentRunSnapshot.set(this, void 0);
	        _AssistantStream_currentRunStepSnapshot.set(this, void 0);
	    }
	    [(_AssistantStream_events = new WeakMap(), _AssistantStream_runStepSnapshots = new WeakMap(), _AssistantStream_messageSnapshots = new WeakMap(), _AssistantStream_messageSnapshot = new WeakMap(), _AssistantStream_finalRun = new WeakMap(), _AssistantStream_currentContentIndex = new WeakMap(), _AssistantStream_currentContent = new WeakMap(), _AssistantStream_currentToolCallIndex = new WeakMap(), _AssistantStream_currentToolCall = new WeakMap(), _AssistantStream_currentEvent = new WeakMap(), _AssistantStream_currentRunSnapshot = new WeakMap(), _AssistantStream_currentRunStepSnapshot = new WeakMap(), _AssistantStream_instances = new WeakSet(), Symbol.asyncIterator)]() {
	        const pushQueue = [];
	        const readQueue = [];
	        let done = false;
	        //Catch all for passing along all events
	        this.on('event', (event) => {
	            const reader = readQueue.shift();
	            if (reader) {
	                reader.resolve(event);
	            }
	            else {
	                pushQueue.push(event);
	            }
	        });
	        this.on('end', () => {
	            done = true;
	            for (const reader of readQueue) {
	                reader.resolve(undefined);
	            }
	            readQueue.length = 0;
	        });
	        this.on('abort', (err) => {
	            done = true;
	            for (const reader of readQueue) {
	                reader.reject(err);
	            }
	            readQueue.length = 0;
	        });
	        this.on('error', (err) => {
	            done = true;
	            for (const reader of readQueue) {
	                reader.reject(err);
	            }
	            readQueue.length = 0;
	        });
	        return {
	            next: async () => {
	                if (!pushQueue.length) {
	                    if (done) {
	                        return { value: undefined, done: true };
	                    }
	                    return new Promise((resolve, reject) => readQueue.push({ resolve, reject })).then((chunk) => (chunk ? { value: chunk, done: false } : { value: undefined, done: true }));
	                }
	                const chunk = pushQueue.shift();
	                return { value: chunk, done: false };
	            },
	            return: async () => {
	                this.abort();
	                return { value: undefined, done: true };
	            },
	        };
	    }
	    static fromReadableStream(stream) {
	        const runner = new AssistantStream();
	        runner._run(() => runner._fromReadableStream(stream));
	        return runner;
	    }
	    async _fromReadableStream(readableStream, options) {
	        const signal = options?.signal;
	        if (signal) {
	            if (signal.aborted)
	                this.controller.abort();
	            signal.addEventListener('abort', () => this.controller.abort());
	        }
	        this._connected();
	        const stream = streaming_1.Stream.fromReadableStream(readableStream, this.controller);
	        for await (const event of stream) {
	            __classPrivateFieldGet(this, _AssistantStream_instances, "m", _AssistantStream_addEvent).call(this, event);
	        }
	        if (stream.controller.signal?.aborted) {
	            throw new error_1.APIUserAbortError();
	        }
	        return this._addRun(__classPrivateFieldGet(this, _AssistantStream_instances, "m", _AssistantStream_endRequest).call(this));
	    }
	    toReadableStream() {
	        const stream = new streaming_1.Stream(this[Symbol.asyncIterator].bind(this), this.controller);
	        return stream.toReadableStream();
	    }
	    static createToolAssistantStream(threadId, runId, runs, params, options) {
	        const runner = new AssistantStream();
	        runner._run(() => runner._runToolAssistantStream(threadId, runId, runs, params, {
	            ...options,
	            headers: { ...options?.headers, 'X-Stainless-Helper-Method': 'stream' },
	        }));
	        return runner;
	    }
	    async _createToolAssistantStream(run, threadId, runId, params, options) {
	        const signal = options?.signal;
	        if (signal) {
	            if (signal.aborted)
	                this.controller.abort();
	            signal.addEventListener('abort', () => this.controller.abort());
	        }
	        const body = { ...params, stream: true };
	        const stream = await run.submitToolOutputs(threadId, runId, body, {
	            ...options,
	            signal: this.controller.signal,
	        });
	        this._connected();
	        for await (const event of stream) {
	            __classPrivateFieldGet(this, _AssistantStream_instances, "m", _AssistantStream_addEvent).call(this, event);
	        }
	        if (stream.controller.signal?.aborted) {
	            throw new error_1.APIUserAbortError();
	        }
	        return this._addRun(__classPrivateFieldGet(this, _AssistantStream_instances, "m", _AssistantStream_endRequest).call(this));
	    }
	    static createThreadAssistantStream(params, thread, options) {
	        const runner = new AssistantStream();
	        runner._run(() => runner._threadAssistantStream(params, thread, {
	            ...options,
	            headers: { ...options?.headers, 'X-Stainless-Helper-Method': 'stream' },
	        }));
	        return runner;
	    }
	    static createAssistantStream(threadId, runs, params, options) {
	        const runner = new AssistantStream();
	        runner._run(() => runner._runAssistantStream(threadId, runs, params, {
	            ...options,
	            headers: { ...options?.headers, 'X-Stainless-Helper-Method': 'stream' },
	        }));
	        return runner;
	    }
	    currentEvent() {
	        return __classPrivateFieldGet(this, _AssistantStream_currentEvent, "f");
	    }
	    currentRun() {
	        return __classPrivateFieldGet(this, _AssistantStream_currentRunSnapshot, "f");
	    }
	    currentMessageSnapshot() {
	        return __classPrivateFieldGet(this, _AssistantStream_messageSnapshot, "f");
	    }
	    currentRunStepSnapshot() {
	        return __classPrivateFieldGet(this, _AssistantStream_currentRunStepSnapshot, "f");
	    }
	    async finalRunSteps() {
	        await this.done();
	        return Object.values(__classPrivateFieldGet(this, _AssistantStream_runStepSnapshots, "f"));
	    }
	    async finalMessages() {
	        await this.done();
	        return Object.values(__classPrivateFieldGet(this, _AssistantStream_messageSnapshots, "f"));
	    }
	    async finalRun() {
	        await this.done();
	        if (!__classPrivateFieldGet(this, _AssistantStream_finalRun, "f"))
	            throw Error('Final run was not received.');
	        return __classPrivateFieldGet(this, _AssistantStream_finalRun, "f");
	    }
	    async _createThreadAssistantStream(thread, params, options) {
	        const signal = options?.signal;
	        if (signal) {
	            if (signal.aborted)
	                this.controller.abort();
	            signal.addEventListener('abort', () => this.controller.abort());
	        }
	        const body = { ...params, stream: true };
	        const stream = await thread.createAndRun(body, { ...options, signal: this.controller.signal });
	        this._connected();
	        for await (const event of stream) {
	            __classPrivateFieldGet(this, _AssistantStream_instances, "m", _AssistantStream_addEvent).call(this, event);
	        }
	        if (stream.controller.signal?.aborted) {
	            throw new error_1.APIUserAbortError();
	        }
	        return this._addRun(__classPrivateFieldGet(this, _AssistantStream_instances, "m", _AssistantStream_endRequest).call(this));
	    }
	    async _createAssistantStream(run, threadId, params, options) {
	        const signal = options?.signal;
	        if (signal) {
	            if (signal.aborted)
	                this.controller.abort();
	            signal.addEventListener('abort', () => this.controller.abort());
	        }
	        const body = { ...params, stream: true };
	        const stream = await run.create(threadId, body, { ...options, signal: this.controller.signal });
	        this._connected();
	        for await (const event of stream) {
	            __classPrivateFieldGet(this, _AssistantStream_instances, "m", _AssistantStream_addEvent).call(this, event);
	        }
	        if (stream.controller.signal?.aborted) {
	            throw new error_1.APIUserAbortError();
	        }
	        return this._addRun(__classPrivateFieldGet(this, _AssistantStream_instances, "m", _AssistantStream_endRequest).call(this));
	    }
	    static accumulateDelta(acc, delta) {
	        for (const [key, deltaValue] of Object.entries(delta)) {
	            if (!acc.hasOwnProperty(key)) {
	                acc[key] = deltaValue;
	                continue;
	            }
	            let accValue = acc[key];
	            if (accValue === null || accValue === undefined) {
	                acc[key] = deltaValue;
	                continue;
	            }
	            // We don't accumulate these special properties
	            if (key === 'index' || key === 'type') {
	                acc[key] = deltaValue;
	                continue;
	            }
	            // Type-specific accumulation logic
	            if (typeof accValue === 'string' && typeof deltaValue === 'string') {
	                accValue += deltaValue;
	            }
	            else if (typeof accValue === 'number' && typeof deltaValue === 'number') {
	                accValue += deltaValue;
	            }
	            else if (Core.isObj(accValue) && Core.isObj(deltaValue)) {
	                accValue = this.accumulateDelta(accValue, deltaValue);
	            }
	            else if (Array.isArray(accValue) && Array.isArray(deltaValue)) {
	                if (accValue.every((x) => typeof x === 'string' || typeof x === 'number')) {
	                    accValue.push(...deltaValue); // Use spread syntax for efficient addition
	                    continue;
	                }
	                for (const deltaEntry of deltaValue) {
	                    if (!Core.isObj(deltaEntry)) {
	                        throw new Error(`Expected array delta entry to be an object but got: ${deltaEntry}`);
	                    }
	                    const index = deltaEntry['index'];
	                    if (index == null) {
	                        console.error(deltaEntry);
	                        throw new Error('Expected array delta entry to have an `index` property');
	                    }
	                    if (typeof index !== 'number') {
	                        throw new Error(`Expected array delta entry \`index\` property to be a number but got ${index}`);
	                    }
	                    const accEntry = accValue[index];
	                    if (accEntry == null) {
	                        accValue.push(deltaEntry);
	                    }
	                    else {
	                        accValue[index] = this.accumulateDelta(accEntry, deltaEntry);
	                    }
	                }
	                continue;
	            }
	            else {
	                throw Error(`Unhandled record type: ${key}, deltaValue: ${deltaValue}, accValue: ${accValue}`);
	            }
	            acc[key] = accValue;
	        }
	        return acc;
	    }
	    _addRun(run) {
	        return run;
	    }
	    async _threadAssistantStream(params, thread, options) {
	        return await this._createThreadAssistantStream(thread, params, options);
	    }
	    async _runAssistantStream(threadId, runs, params, options) {
	        return await this._createAssistantStream(runs, threadId, params, options);
	    }
	    async _runToolAssistantStream(threadId, runId, runs, params, options) {
	        return await this._createToolAssistantStream(runs, threadId, runId, params, options);
	    }
	};
	AssistantStream.AssistantStream = AssistantStream$1;
	_AssistantStream_addEvent = function _AssistantStream_addEvent(event) {
	    if (this.ended)
	        return;
	    __classPrivateFieldSet(this, _AssistantStream_currentEvent, event, "f");
	    __classPrivateFieldGet(this, _AssistantStream_instances, "m", _AssistantStream_handleEvent).call(this, event);
	    switch (event.event) {
	        case 'thread.created':
	            //No action on this event.
	            break;
	        case 'thread.run.created':
	        case 'thread.run.queued':
	        case 'thread.run.in_progress':
	        case 'thread.run.requires_action':
	        case 'thread.run.completed':
	        case 'thread.run.failed':
	        case 'thread.run.cancelling':
	        case 'thread.run.cancelled':
	        case 'thread.run.expired':
	            __classPrivateFieldGet(this, _AssistantStream_instances, "m", _AssistantStream_handleRun).call(this, event);
	            break;
	        case 'thread.run.step.created':
	        case 'thread.run.step.in_progress':
	        case 'thread.run.step.delta':
	        case 'thread.run.step.completed':
	        case 'thread.run.step.failed':
	        case 'thread.run.step.cancelled':
	        case 'thread.run.step.expired':
	            __classPrivateFieldGet(this, _AssistantStream_instances, "m", _AssistantStream_handleRunStep).call(this, event);
	            break;
	        case 'thread.message.created':
	        case 'thread.message.in_progress':
	        case 'thread.message.delta':
	        case 'thread.message.completed':
	        case 'thread.message.incomplete':
	            __classPrivateFieldGet(this, _AssistantStream_instances, "m", _AssistantStream_handleMessage).call(this, event);
	            break;
	        case 'error':
	            //This is included for completeness, but errors are processed in the SSE event processing so this should not occur
	            throw new Error('Encountered an error event in event processing - errors should be processed earlier');
	    }
	}, _AssistantStream_endRequest = function _AssistantStream_endRequest() {
	    if (this.ended) {
	        throw new error_1.OpenAIError(`stream has ended, this shouldn't happen`);
	    }
	    if (!__classPrivateFieldGet(this, _AssistantStream_finalRun, "f"))
	        throw Error('Final run has not been received');
	    return __classPrivateFieldGet(this, _AssistantStream_finalRun, "f");
	}, _AssistantStream_handleMessage = function _AssistantStream_handleMessage(event) {
	    const [accumulatedMessage, newContent] = __classPrivateFieldGet(this, _AssistantStream_instances, "m", _AssistantStream_accumulateMessage).call(this, event, __classPrivateFieldGet(this, _AssistantStream_messageSnapshot, "f"));
	    __classPrivateFieldSet(this, _AssistantStream_messageSnapshot, accumulatedMessage, "f");
	    __classPrivateFieldGet(this, _AssistantStream_messageSnapshots, "f")[accumulatedMessage.id] = accumulatedMessage;
	    for (const content of newContent) {
	        const snapshotContent = accumulatedMessage.content[content.index];
	        if (snapshotContent?.type == 'text') {
	            this._emit('textCreated', snapshotContent.text);
	        }
	    }
	    switch (event.event) {
	        case 'thread.message.created':
	            this._emit('messageCreated', event.data);
	            break;
	        case 'thread.message.in_progress':
	            break;
	        case 'thread.message.delta':
	            this._emit('messageDelta', event.data.delta, accumulatedMessage);
	            if (event.data.delta.content) {
	                for (const content of event.data.delta.content) {
	                    //If it is text delta, emit a text delta event
	                    if (content.type == 'text' && content.text) {
	                        let textDelta = content.text;
	                        let snapshot = accumulatedMessage.content[content.index];
	                        if (snapshot && snapshot.type == 'text') {
	                            this._emit('textDelta', textDelta, snapshot.text);
	                        }
	                        else {
	                            throw Error('The snapshot associated with this text delta is not text or missing');
	                        }
	                    }
	                    if (content.index != __classPrivateFieldGet(this, _AssistantStream_currentContentIndex, "f")) {
	                        //See if we have in progress content
	                        if (__classPrivateFieldGet(this, _AssistantStream_currentContent, "f")) {
	                            switch (__classPrivateFieldGet(this, _AssistantStream_currentContent, "f").type) {
	                                case 'text':
	                                    this._emit('textDone', __classPrivateFieldGet(this, _AssistantStream_currentContent, "f").text, __classPrivateFieldGet(this, _AssistantStream_messageSnapshot, "f"));
	                                    break;
	                                case 'image_file':
	                                    this._emit('imageFileDone', __classPrivateFieldGet(this, _AssistantStream_currentContent, "f").image_file, __classPrivateFieldGet(this, _AssistantStream_messageSnapshot, "f"));
	                                    break;
	                            }
	                        }
	                        __classPrivateFieldSet(this, _AssistantStream_currentContentIndex, content.index, "f");
	                    }
	                    __classPrivateFieldSet(this, _AssistantStream_currentContent, accumulatedMessage.content[content.index], "f");
	                }
	            }
	            break;
	        case 'thread.message.completed':
	        case 'thread.message.incomplete':
	            //We emit the latest content we were working on on completion (including incomplete)
	            if (__classPrivateFieldGet(this, _AssistantStream_currentContentIndex, "f") !== undefined) {
	                const currentContent = event.data.content[__classPrivateFieldGet(this, _AssistantStream_currentContentIndex, "f")];
	                if (currentContent) {
	                    switch (currentContent.type) {
	                        case 'image_file':
	                            this._emit('imageFileDone', currentContent.image_file, __classPrivateFieldGet(this, _AssistantStream_messageSnapshot, "f"));
	                            break;
	                        case 'text':
	                            this._emit('textDone', currentContent.text, __classPrivateFieldGet(this, _AssistantStream_messageSnapshot, "f"));
	                            break;
	                    }
	                }
	            }
	            if (__classPrivateFieldGet(this, _AssistantStream_messageSnapshot, "f")) {
	                this._emit('messageDone', event.data);
	            }
	            __classPrivateFieldSet(this, _AssistantStream_messageSnapshot, undefined, "f");
	    }
	}, _AssistantStream_handleRunStep = function _AssistantStream_handleRunStep(event) {
	    const accumulatedRunStep = __classPrivateFieldGet(this, _AssistantStream_instances, "m", _AssistantStream_accumulateRunStep).call(this, event);
	    __classPrivateFieldSet(this, _AssistantStream_currentRunStepSnapshot, accumulatedRunStep, "f");
	    switch (event.event) {
	        case 'thread.run.step.created':
	            this._emit('runStepCreated', event.data);
	            break;
	        case 'thread.run.step.delta':
	            const delta = event.data.delta;
	            if (delta.step_details &&
	                delta.step_details.type == 'tool_calls' &&
	                delta.step_details.tool_calls &&
	                accumulatedRunStep.step_details.type == 'tool_calls') {
	                for (const toolCall of delta.step_details.tool_calls) {
	                    if (toolCall.index == __classPrivateFieldGet(this, _AssistantStream_currentToolCallIndex, "f")) {
	                        this._emit('toolCallDelta', toolCall, accumulatedRunStep.step_details.tool_calls[toolCall.index]);
	                    }
	                    else {
	                        if (__classPrivateFieldGet(this, _AssistantStream_currentToolCall, "f")) {
	                            this._emit('toolCallDone', __classPrivateFieldGet(this, _AssistantStream_currentToolCall, "f"));
	                        }
	                        __classPrivateFieldSet(this, _AssistantStream_currentToolCallIndex, toolCall.index, "f");
	                        __classPrivateFieldSet(this, _AssistantStream_currentToolCall, accumulatedRunStep.step_details.tool_calls[toolCall.index], "f");
	                        if (__classPrivateFieldGet(this, _AssistantStream_currentToolCall, "f"))
	                            this._emit('toolCallCreated', __classPrivateFieldGet(this, _AssistantStream_currentToolCall, "f"));
	                    }
	                }
	            }
	            this._emit('runStepDelta', event.data.delta, accumulatedRunStep);
	            break;
	        case 'thread.run.step.completed':
	        case 'thread.run.step.failed':
	        case 'thread.run.step.cancelled':
	        case 'thread.run.step.expired':
	            __classPrivateFieldSet(this, _AssistantStream_currentRunStepSnapshot, undefined, "f");
	            const details = event.data.step_details;
	            if (details.type == 'tool_calls') {
	                if (__classPrivateFieldGet(this, _AssistantStream_currentToolCall, "f")) {
	                    this._emit('toolCallDone', __classPrivateFieldGet(this, _AssistantStream_currentToolCall, "f"));
	                    __classPrivateFieldSet(this, _AssistantStream_currentToolCall, undefined, "f");
	                }
	            }
	            this._emit('runStepDone', event.data, accumulatedRunStep);
	            break;
	    }
	}, _AssistantStream_handleEvent = function _AssistantStream_handleEvent(event) {
	    __classPrivateFieldGet(this, _AssistantStream_events, "f").push(event);
	    this._emit('event', event);
	}, _AssistantStream_accumulateRunStep = function _AssistantStream_accumulateRunStep(event) {
	    switch (event.event) {
	        case 'thread.run.step.created':
	            __classPrivateFieldGet(this, _AssistantStream_runStepSnapshots, "f")[event.data.id] = event.data;
	            return event.data;
	        case 'thread.run.step.delta':
	            let snapshot = __classPrivateFieldGet(this, _AssistantStream_runStepSnapshots, "f")[event.data.id];
	            if (!snapshot) {
	                throw Error('Received a RunStepDelta before creation of a snapshot');
	            }
	            let data = event.data;
	            if (data.delta) {
	                const accumulated = AssistantStream$1.accumulateDelta(snapshot, data.delta);
	                __classPrivateFieldGet(this, _AssistantStream_runStepSnapshots, "f")[event.data.id] = accumulated;
	            }
	            return __classPrivateFieldGet(this, _AssistantStream_runStepSnapshots, "f")[event.data.id];
	        case 'thread.run.step.completed':
	        case 'thread.run.step.failed':
	        case 'thread.run.step.cancelled':
	        case 'thread.run.step.expired':
	        case 'thread.run.step.in_progress':
	            __classPrivateFieldGet(this, _AssistantStream_runStepSnapshots, "f")[event.data.id] = event.data;
	            break;
	    }
	    if (__classPrivateFieldGet(this, _AssistantStream_runStepSnapshots, "f")[event.data.id])
	        return __classPrivateFieldGet(this, _AssistantStream_runStepSnapshots, "f")[event.data.id];
	    throw new Error('No snapshot available');
	}, _AssistantStream_accumulateMessage = function _AssistantStream_accumulateMessage(event, snapshot) {
	    let newContent = [];
	    switch (event.event) {
	        case 'thread.message.created':
	            //On creation the snapshot is just the initial message
	            return [event.data, newContent];
	        case 'thread.message.delta':
	            if (!snapshot) {
	                throw Error('Received a delta with no existing snapshot (there should be one from message creation)');
	            }
	            let data = event.data;
	            //If this delta does not have content, nothing to process
	            if (data.delta.content) {
	                for (const contentElement of data.delta.content) {
	                    if (contentElement.index in snapshot.content) {
	                        let currentContent = snapshot.content[contentElement.index];
	                        snapshot.content[contentElement.index] = __classPrivateFieldGet(this, _AssistantStream_instances, "m", _AssistantStream_accumulateContent).call(this, contentElement, currentContent);
	                    }
	                    else {
	                        snapshot.content[contentElement.index] = contentElement;
	                        // This is a new element
	                        newContent.push(contentElement);
	                    }
	                }
	            }
	            return [snapshot, newContent];
	        case 'thread.message.in_progress':
	        case 'thread.message.completed':
	        case 'thread.message.incomplete':
	            //No changes on other thread events
	            if (snapshot) {
	                return [snapshot, newContent];
	            }
	            else {
	                throw Error('Received thread message event with no existing snapshot');
	            }
	    }
	    throw Error('Tried to accumulate a non-message event');
	}, _AssistantStream_accumulateContent = function _AssistantStream_accumulateContent(contentElement, currentContent) {
	    return AssistantStream$1.accumulateDelta(currentContent, contentElement);
	}, _AssistantStream_handleRun = function _AssistantStream_handleRun(event) {
	    __classPrivateFieldSet(this, _AssistantStream_currentRunSnapshot, event.data, "f");
	    switch (event.event) {
	        case 'thread.run.created':
	            break;
	        case 'thread.run.queued':
	            break;
	        case 'thread.run.in_progress':
	            break;
	        case 'thread.run.requires_action':
	        case 'thread.run.cancelled':
	        case 'thread.run.failed':
	        case 'thread.run.completed':
	        case 'thread.run.expired':
	            __classPrivateFieldSet(this, _AssistantStream_finalRun, event.data, "f");
	            if (__classPrivateFieldGet(this, _AssistantStream_currentToolCall, "f")) {
	                this._emit('toolCallDone', __classPrivateFieldGet(this, _AssistantStream_currentToolCall, "f"));
	                __classPrivateFieldSet(this, _AssistantStream_currentToolCall, undefined, "f");
	            }
	            break;
	    }
	};
	
	return AssistantStream;
}

var messages = {};

var hasRequiredMessages;

function requireMessages () {
	if (hasRequiredMessages) return messages;
	hasRequiredMessages = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		var __createBinding = (messages && messages.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __setModuleDefault = (messages && messages.__setModuleDefault) || (Object.create ? (function(o, v) {
		    Object.defineProperty(o, "default", { enumerable: true, value: v });
		}) : function(o, v) {
		    o["default"] = v;
		});
		var __importStar = (messages && messages.__importStar) || function (mod) {
		    if (mod && mod.__esModule) return mod;
		    var result = {};
		    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
		    __setModuleDefault(result, mod);
		    return result;
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.MessagesPage = exports.Messages = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		const core_1 = /*@__PURE__*/ requireCore();
		const MessagesAPI = __importStar(/*@__PURE__*/ requireMessages());
		const pagination_1 = /*@__PURE__*/ requirePagination();
		class Messages extends resource_1.APIResource {
		    /**
		     * Create a message.
		     */
		    create(threadId, body, options) {
		        return this._client.post(`/threads/${threadId}/messages`, {
		            body,
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		    /**
		     * Retrieve a message.
		     */
		    retrieve(threadId, messageId, options) {
		        return this._client.get(`/threads/${threadId}/messages/${messageId}`, {
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		    /**
		     * Modifies a message.
		     */
		    update(threadId, messageId, body, options) {
		        return this._client.post(`/threads/${threadId}/messages/${messageId}`, {
		            body,
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		    list(threadId, query = {}, options) {
		        if ((0, core_1.isRequestOptions)(query)) {
		            return this.list(threadId, {}, query);
		        }
		        return this._client.getAPIList(`/threads/${threadId}/messages`, MessagesPage, {
		            query,
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		    /**
		     * Deletes a message.
		     */
		    del(threadId, messageId, options) {
		        return this._client.delete(`/threads/${threadId}/messages/${messageId}`, {
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		}
		exports.Messages = Messages;
		class MessagesPage extends pagination_1.CursorPage {
		}
		exports.MessagesPage = MessagesPage;
		(function (Messages) {
		    Messages.MessagesPage = MessagesAPI.MessagesPage;
		})(Messages = exports.Messages || (exports.Messages = {}));
		
	} (messages));
	return messages;
}

var runs = {};

var steps = {};

var hasRequiredSteps;

function requireSteps () {
	if (hasRequiredSteps) return steps;
	hasRequiredSteps = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		var __createBinding = (steps && steps.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __setModuleDefault = (steps && steps.__setModuleDefault) || (Object.create ? (function(o, v) {
		    Object.defineProperty(o, "default", { enumerable: true, value: v });
		}) : function(o, v) {
		    o["default"] = v;
		});
		var __importStar = (steps && steps.__importStar) || function (mod) {
		    if (mod && mod.__esModule) return mod;
		    var result = {};
		    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
		    __setModuleDefault(result, mod);
		    return result;
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.RunStepsPage = exports.Steps = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		const core_1 = /*@__PURE__*/ requireCore();
		const StepsAPI = __importStar(/*@__PURE__*/ requireSteps());
		const pagination_1 = /*@__PURE__*/ requirePagination();
		class Steps extends resource_1.APIResource {
		    retrieve(threadId, runId, stepId, query = {}, options) {
		        if ((0, core_1.isRequestOptions)(query)) {
		            return this.retrieve(threadId, runId, stepId, {}, query);
		        }
		        return this._client.get(`/threads/${threadId}/runs/${runId}/steps/${stepId}`, {
		            query,
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		    list(threadId, runId, query = {}, options) {
		        if ((0, core_1.isRequestOptions)(query)) {
		            return this.list(threadId, runId, {}, query);
		        }
		        return this._client.getAPIList(`/threads/${threadId}/runs/${runId}/steps`, RunStepsPage, {
		            query,
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		}
		exports.Steps = Steps;
		class RunStepsPage extends pagination_1.CursorPage {
		}
		exports.RunStepsPage = RunStepsPage;
		(function (Steps) {
		    Steps.RunStepsPage = StepsAPI.RunStepsPage;
		})(Steps = exports.Steps || (exports.Steps = {}));
		
	} (steps));
	return steps;
}

var hasRequiredRuns;

function requireRuns () {
	if (hasRequiredRuns) return runs;
	hasRequiredRuns = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		var __createBinding = (runs && runs.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __setModuleDefault = (runs && runs.__setModuleDefault) || (Object.create ? (function(o, v) {
		    Object.defineProperty(o, "default", { enumerable: true, value: v });
		}) : function(o, v) {
		    o["default"] = v;
		});
		var __importStar = (runs && runs.__importStar) || function (mod) {
		    if (mod && mod.__esModule) return mod;
		    var result = {};
		    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
		    __setModuleDefault(result, mod);
		    return result;
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.RunsPage = exports.Runs = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		const core_1 = /*@__PURE__*/ requireCore();
		const AssistantStream_1 = /*@__PURE__*/ requireAssistantStream();
		const core_2 = /*@__PURE__*/ requireCore();
		const RunsAPI = __importStar(/*@__PURE__*/ requireRuns());
		const StepsAPI = __importStar(/*@__PURE__*/ requireSteps());
		const pagination_1 = /*@__PURE__*/ requirePagination();
		class Runs extends resource_1.APIResource {
		    constructor() {
		        super(...arguments);
		        this.steps = new StepsAPI.Steps(this._client);
		    }
		    create(threadId, params, options) {
		        const { include, ...body } = params;
		        return this._client.post(`/threads/${threadId}/runs`, {
		            query: { include },
		            body,
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		            stream: params.stream ?? false,
		        });
		    }
		    /**
		     * Retrieves a run.
		     */
		    retrieve(threadId, runId, options) {
		        return this._client.get(`/threads/${threadId}/runs/${runId}`, {
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		    /**
		     * Modifies a run.
		     */
		    update(threadId, runId, body, options) {
		        return this._client.post(`/threads/${threadId}/runs/${runId}`, {
		            body,
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		    list(threadId, query = {}, options) {
		        if ((0, core_1.isRequestOptions)(query)) {
		            return this.list(threadId, {}, query);
		        }
		        return this._client.getAPIList(`/threads/${threadId}/runs`, RunsPage, {
		            query,
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		    /**
		     * Cancels a run that is `in_progress`.
		     */
		    cancel(threadId, runId, options) {
		        return this._client.post(`/threads/${threadId}/runs/${runId}/cancel`, {
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		    /**
		     * A helper to create a run an poll for a terminal state. More information on Run
		     * lifecycles can be found here:
		     * https://platform.openai.com/docs/assistants/how-it-works/runs-and-run-steps
		     */
		    async createAndPoll(threadId, body, options) {
		        const run = await this.create(threadId, body, options);
		        return await this.poll(threadId, run.id, options);
		    }
		    /**
		     * Create a Run stream
		     *
		     * @deprecated use `stream` instead
		     */
		    createAndStream(threadId, body, options) {
		        return AssistantStream_1.AssistantStream.createAssistantStream(threadId, this._client.beta.threads.runs, body, options);
		    }
		    /**
		     * A helper to poll a run status until it reaches a terminal state. More
		     * information on Run lifecycles can be found here:
		     * https://platform.openai.com/docs/assistants/how-it-works/runs-and-run-steps
		     */
		    async poll(threadId, runId, options) {
		        const headers = { ...options?.headers, 'X-Stainless-Poll-Helper': 'true' };
		        if (options?.pollIntervalMs) {
		            headers['X-Stainless-Custom-Poll-Interval'] = options.pollIntervalMs.toString();
		        }
		        while (true) {
		            const { data: run, response } = await this.retrieve(threadId, runId, {
		                ...options,
		                headers: { ...options?.headers, ...headers },
		            }).withResponse();
		            switch (run.status) {
		                //If we are in any sort of intermediate state we poll
		                case 'queued':
		                case 'in_progress':
		                case 'cancelling':
		                    let sleepInterval = 5000;
		                    if (options?.pollIntervalMs) {
		                        sleepInterval = options.pollIntervalMs;
		                    }
		                    else {
		                        const headerInterval = response.headers.get('openai-poll-after-ms');
		                        if (headerInterval) {
		                            const headerIntervalMs = parseInt(headerInterval);
		                            if (!isNaN(headerIntervalMs)) {
		                                sleepInterval = headerIntervalMs;
		                            }
		                        }
		                    }
		                    await (0, core_2.sleep)(sleepInterval);
		                    break;
		                //We return the run in any terminal state.
		                case 'requires_action':
		                case 'incomplete':
		                case 'cancelled':
		                case 'completed':
		                case 'failed':
		                case 'expired':
		                    return run;
		            }
		        }
		    }
		    /**
		     * Create a Run stream
		     */
		    stream(threadId, body, options) {
		        return AssistantStream_1.AssistantStream.createAssistantStream(threadId, this._client.beta.threads.runs, body, options);
		    }
		    submitToolOutputs(threadId, runId, body, options) {
		        return this._client.post(`/threads/${threadId}/runs/${runId}/submit_tool_outputs`, {
		            body,
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		            stream: body.stream ?? false,
		        });
		    }
		    /**
		     * A helper to submit a tool output to a run and poll for a terminal run state.
		     * More information on Run lifecycles can be found here:
		     * https://platform.openai.com/docs/assistants/how-it-works/runs-and-run-steps
		     */
		    async submitToolOutputsAndPoll(threadId, runId, body, options) {
		        const run = await this.submitToolOutputs(threadId, runId, body, options);
		        return await this.poll(threadId, run.id, options);
		    }
		    /**
		     * Submit the tool outputs from a previous run and stream the run to a terminal
		     * state. More information on Run lifecycles can be found here:
		     * https://platform.openai.com/docs/assistants/how-it-works/runs-and-run-steps
		     */
		    submitToolOutputsStream(threadId, runId, body, options) {
		        return AssistantStream_1.AssistantStream.createToolAssistantStream(threadId, runId, this._client.beta.threads.runs, body, options);
		    }
		}
		exports.Runs = Runs;
		class RunsPage extends pagination_1.CursorPage {
		}
		exports.RunsPage = RunsPage;
		(function (Runs) {
		    Runs.RunsPage = RunsAPI.RunsPage;
		    Runs.Steps = StepsAPI.Steps;
		    Runs.RunStepsPage = StepsAPI.RunStepsPage;
		})(Runs = exports.Runs || (exports.Runs = {}));
		
	} (runs));
	return runs;
}

var hasRequiredThreads;

function requireThreads () {
	if (hasRequiredThreads) return threads;
	hasRequiredThreads = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		var __createBinding = (threads && threads.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __setModuleDefault = (threads && threads.__setModuleDefault) || (Object.create ? (function(o, v) {
		    Object.defineProperty(o, "default", { enumerable: true, value: v });
		}) : function(o, v) {
		    o["default"] = v;
		});
		var __importStar = (threads && threads.__importStar) || function (mod) {
		    if (mod && mod.__esModule) return mod;
		    var result = {};
		    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
		    __setModuleDefault(result, mod);
		    return result;
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.Threads = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		const core_1 = /*@__PURE__*/ requireCore();
		const AssistantStream_1 = /*@__PURE__*/ requireAssistantStream();
		const MessagesAPI = __importStar(/*@__PURE__*/ requireMessages());
		const RunsAPI = __importStar(/*@__PURE__*/ requireRuns());
		class Threads extends resource_1.APIResource {
		    constructor() {
		        super(...arguments);
		        this.runs = new RunsAPI.Runs(this._client);
		        this.messages = new MessagesAPI.Messages(this._client);
		    }
		    create(body = {}, options) {
		        if ((0, core_1.isRequestOptions)(body)) {
		            return this.create({}, body);
		        }
		        return this._client.post('/threads', {
		            body,
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		    /**
		     * Retrieves a thread.
		     */
		    retrieve(threadId, options) {
		        return this._client.get(`/threads/${threadId}`, {
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		    /**
		     * Modifies a thread.
		     */
		    update(threadId, body, options) {
		        return this._client.post(`/threads/${threadId}`, {
		            body,
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		    /**
		     * Delete a thread.
		     */
		    del(threadId, options) {
		        return this._client.delete(`/threads/${threadId}`, {
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		    createAndRun(body, options) {
		        return this._client.post('/threads/runs', {
		            body,
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		            stream: body.stream ?? false,
		        });
		    }
		    /**
		     * A helper to create a thread, start a run and then poll for a terminal state.
		     * More information on Run lifecycles can be found here:
		     * https://platform.openai.com/docs/assistants/how-it-works/runs-and-run-steps
		     */
		    async createAndRunPoll(body, options) {
		        const run = await this.createAndRun(body, options);
		        return await this.runs.poll(run.thread_id, run.id, options);
		    }
		    /**
		     * Create a thread and stream the run back
		     */
		    createAndRunStream(body, options) {
		        return AssistantStream_1.AssistantStream.createThreadAssistantStream(body, this._client.beta.threads, options);
		    }
		}
		exports.Threads = Threads;
		(function (Threads) {
		    Threads.Runs = RunsAPI.Runs;
		    Threads.RunsPage = RunsAPI.RunsPage;
		    Threads.Messages = MessagesAPI.Messages;
		    Threads.MessagesPage = MessagesAPI.MessagesPage;
		})(Threads = exports.Threads || (exports.Threads = {}));
		
	} (threads));
	return threads;
}

var vectorStores = {};

var fileBatches = {};

var Util = {};

var hasRequiredUtil;

function requireUtil () {
	if (hasRequiredUtil) return Util;
	hasRequiredUtil = 1;
	Object.defineProperty(Util, "__esModule", { value: true });
	Util.allSettledWithThrow = void 0;
	/**
	 * Like `Promise.allSettled()` but throws an error if any promises are rejected.
	 */
	const allSettledWithThrow = async (promises) => {
	    const results = await Promise.allSettled(promises);
	    const rejected = results.filter((result) => result.status === 'rejected');
	    if (rejected.length) {
	        for (const result of rejected) {
	            console.error(result.reason);
	        }
	        throw new Error(`${rejected.length} promise(s) failed - see the above errors`);
	    }
	    // Note: TS was complaining about using `.filter().map()` here for some reason
	    const values = [];
	    for (const result of results) {
	        if (result.status === 'fulfilled') {
	            values.push(result.value);
	        }
	    }
	    return values;
	};
	Util.allSettledWithThrow = allSettledWithThrow;
	
	return Util;
}

var files$1 = {};

var hasRequiredFiles$1;

function requireFiles$1 () {
	if (hasRequiredFiles$1) return files$1;
	hasRequiredFiles$1 = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		var __createBinding = (files$1 && files$1.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __setModuleDefault = (files$1 && files$1.__setModuleDefault) || (Object.create ? (function(o, v) {
		    Object.defineProperty(o, "default", { enumerable: true, value: v });
		}) : function(o, v) {
		    o["default"] = v;
		});
		var __importStar = (files$1 && files$1.__importStar) || function (mod) {
		    if (mod && mod.__esModule) return mod;
		    var result = {};
		    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
		    __setModuleDefault(result, mod);
		    return result;
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.VectorStoreFilesPage = exports.Files = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		const core_1 = /*@__PURE__*/ requireCore();
		const FilesAPI = __importStar(/*@__PURE__*/ requireFiles$1());
		const pagination_1 = /*@__PURE__*/ requirePagination();
		class Files extends resource_1.APIResource {
		    /**
		     * Create a vector store file by attaching a
		     * [File](https://platform.openai.com/docs/api-reference/files) to a
		     * [vector store](https://platform.openai.com/docs/api-reference/vector-stores/object).
		     */
		    create(vectorStoreId, body, options) {
		        return this._client.post(`/vector_stores/${vectorStoreId}/files`, {
		            body,
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		    /**
		     * Retrieves a vector store file.
		     */
		    retrieve(vectorStoreId, fileId, options) {
		        return this._client.get(`/vector_stores/${vectorStoreId}/files/${fileId}`, {
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		    list(vectorStoreId, query = {}, options) {
		        if ((0, core_1.isRequestOptions)(query)) {
		            return this.list(vectorStoreId, {}, query);
		        }
		        return this._client.getAPIList(`/vector_stores/${vectorStoreId}/files`, VectorStoreFilesPage, {
		            query,
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		    /**
		     * Delete a vector store file. This will remove the file from the vector store but
		     * the file itself will not be deleted. To delete the file, use the
		     * [delete file](https://platform.openai.com/docs/api-reference/files/delete)
		     * endpoint.
		     */
		    del(vectorStoreId, fileId, options) {
		        return this._client.delete(`/vector_stores/${vectorStoreId}/files/${fileId}`, {
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		    /**
		     * Attach a file to the given vector store and wait for it to be processed.
		     */
		    async createAndPoll(vectorStoreId, body, options) {
		        const file = await this.create(vectorStoreId, body, options);
		        return await this.poll(vectorStoreId, file.id, options);
		    }
		    /**
		     * Wait for the vector store file to finish processing.
		     *
		     * Note: this will return even if the file failed to process, you need to check
		     * file.last_error and file.status to handle these cases
		     */
		    async poll(vectorStoreId, fileId, options) {
		        const headers = { ...options?.headers, 'X-Stainless-Poll-Helper': 'true' };
		        if (options?.pollIntervalMs) {
		            headers['X-Stainless-Custom-Poll-Interval'] = options.pollIntervalMs.toString();
		        }
		        while (true) {
		            const fileResponse = await this.retrieve(vectorStoreId, fileId, {
		                ...options,
		                headers,
		            }).withResponse();
		            const file = fileResponse.data;
		            switch (file.status) {
		                case 'in_progress':
		                    let sleepInterval = 5000;
		                    if (options?.pollIntervalMs) {
		                        sleepInterval = options.pollIntervalMs;
		                    }
		                    else {
		                        const headerInterval = fileResponse.response.headers.get('openai-poll-after-ms');
		                        if (headerInterval) {
		                            const headerIntervalMs = parseInt(headerInterval);
		                            if (!isNaN(headerIntervalMs)) {
		                                sleepInterval = headerIntervalMs;
		                            }
		                        }
		                    }
		                    await (0, core_1.sleep)(sleepInterval);
		                    break;
		                case 'failed':
		                case 'completed':
		                    return file;
		            }
		        }
		    }
		    /**
		     * Upload a file to the `files` API and then attach it to the given vector store.
		     *
		     * Note the file will be asynchronously processed (you can use the alternative
		     * polling helper method to wait for processing to complete).
		     */
		    async upload(vectorStoreId, file, options) {
		        const fileInfo = await this._client.files.create({ file: file, purpose: 'assistants' }, options);
		        return this.create(vectorStoreId, { file_id: fileInfo.id }, options);
		    }
		    /**
		     * Add a file to a vector store and poll until processing is complete.
		     */
		    async uploadAndPoll(vectorStoreId, file, options) {
		        const fileInfo = await this.upload(vectorStoreId, file, options);
		        return await this.poll(vectorStoreId, fileInfo.id, options);
		    }
		}
		exports.Files = Files;
		class VectorStoreFilesPage extends pagination_1.CursorPage {
		}
		exports.VectorStoreFilesPage = VectorStoreFilesPage;
		(function (Files) {
		    Files.VectorStoreFilesPage = FilesAPI.VectorStoreFilesPage;
		})(Files = exports.Files || (exports.Files = {}));
		
	} (files$1));
	return files$1;
}

var hasRequiredFileBatches;

function requireFileBatches () {
	if (hasRequiredFileBatches) return fileBatches;
	hasRequiredFileBatches = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.VectorStoreFilesPage = exports.FileBatches = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		const core_1 = /*@__PURE__*/ requireCore();
		const core_2 = /*@__PURE__*/ requireCore();
		const Util_1 = /*@__PURE__*/ requireUtil();
		const files_1 = /*@__PURE__*/ requireFiles$1();
		Object.defineProperty(exports, "VectorStoreFilesPage", { enumerable: true, get: function () { return files_1.VectorStoreFilesPage; } });
		class FileBatches extends resource_1.APIResource {
		    /**
		     * Create a vector store file batch.
		     */
		    create(vectorStoreId, body, options) {
		        return this._client.post(`/vector_stores/${vectorStoreId}/file_batches`, {
		            body,
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		    /**
		     * Retrieves a vector store file batch.
		     */
		    retrieve(vectorStoreId, batchId, options) {
		        return this._client.get(`/vector_stores/${vectorStoreId}/file_batches/${batchId}`, {
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		    /**
		     * Cancel a vector store file batch. This attempts to cancel the processing of
		     * files in this batch as soon as possible.
		     */
		    cancel(vectorStoreId, batchId, options) {
		        return this._client.post(`/vector_stores/${vectorStoreId}/file_batches/${batchId}/cancel`, {
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		    /**
		     * Create a vector store batch and poll until all files have been processed.
		     */
		    async createAndPoll(vectorStoreId, body, options) {
		        const batch = await this.create(vectorStoreId, body);
		        return await this.poll(vectorStoreId, batch.id, options);
		    }
		    listFiles(vectorStoreId, batchId, query = {}, options) {
		        if ((0, core_1.isRequestOptions)(query)) {
		            return this.listFiles(vectorStoreId, batchId, {}, query);
		        }
		        return this._client.getAPIList(`/vector_stores/${vectorStoreId}/file_batches/${batchId}/files`, files_1.VectorStoreFilesPage, { query, ...options, headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers } });
		    }
		    /**
		     * Wait for the given file batch to be processed.
		     *
		     * Note: this will return even if one of the files failed to process, you need to
		     * check batch.file_counts.failed_count to handle this case.
		     */
		    async poll(vectorStoreId, batchId, options) {
		        const headers = { ...options?.headers, 'X-Stainless-Poll-Helper': 'true' };
		        if (options?.pollIntervalMs) {
		            headers['X-Stainless-Custom-Poll-Interval'] = options.pollIntervalMs.toString();
		        }
		        while (true) {
		            const { data: batch, response } = await this.retrieve(vectorStoreId, batchId, {
		                ...options,
		                headers,
		            }).withResponse();
		            switch (batch.status) {
		                case 'in_progress':
		                    let sleepInterval = 5000;
		                    if (options?.pollIntervalMs) {
		                        sleepInterval = options.pollIntervalMs;
		                    }
		                    else {
		                        const headerInterval = response.headers.get('openai-poll-after-ms');
		                        if (headerInterval) {
		                            const headerIntervalMs = parseInt(headerInterval);
		                            if (!isNaN(headerIntervalMs)) {
		                                sleepInterval = headerIntervalMs;
		                            }
		                        }
		                    }
		                    await (0, core_2.sleep)(sleepInterval);
		                    break;
		                case 'failed':
		                case 'cancelled':
		                case 'completed':
		                    return batch;
		            }
		        }
		    }
		    /**
		     * Uploads the given files concurrently and then creates a vector store file batch.
		     *
		     * The concurrency limit is configurable using the `maxConcurrency` parameter.
		     */
		    async uploadAndPoll(vectorStoreId, { files, fileIds = [] }, options) {
		        if (files == null || files.length == 0) {
		            throw new Error(`No \`files\` provided to process. If you've already uploaded files you should use \`.createAndPoll()\` instead`);
		        }
		        const configuredConcurrency = options?.maxConcurrency ?? 5;
		        // We cap the number of workers at the number of files (so we don't start any unnecessary workers)
		        const concurrencyLimit = Math.min(configuredConcurrency, files.length);
		        const client = this._client;
		        const fileIterator = files.values();
		        const allFileIds = [...fileIds];
		        // This code is based on this design. The libraries don't accommodate our environment limits.
		        // https://stackoverflow.com/questions/40639432/what-is-the-best-way-to-limit-concurrency-when-using-es6s-promise-all
		        async function processFiles(iterator) {
		            for (let item of iterator) {
		                const fileObj = await client.files.create({ file: item, purpose: 'assistants' }, options);
		                allFileIds.push(fileObj.id);
		            }
		        }
		        // Start workers to process results
		        const workers = Array(concurrencyLimit).fill(fileIterator).map(processFiles);
		        // Wait for all processing to complete.
		        await (0, Util_1.allSettledWithThrow)(workers);
		        return await this.createAndPoll(vectorStoreId, {
		            file_ids: allFileIds,
		        });
		    }
		}
		exports.FileBatches = FileBatches;
		(function (FileBatches) {
		})(FileBatches = exports.FileBatches || (exports.FileBatches = {}));
		
	} (fileBatches));
	return fileBatches;
}

var hasRequiredVectorStores;

function requireVectorStores () {
	if (hasRequiredVectorStores) return vectorStores;
	hasRequiredVectorStores = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		var __createBinding = (vectorStores && vectorStores.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __setModuleDefault = (vectorStores && vectorStores.__setModuleDefault) || (Object.create ? (function(o, v) {
		    Object.defineProperty(o, "default", { enumerable: true, value: v });
		}) : function(o, v) {
		    o["default"] = v;
		});
		var __importStar = (vectorStores && vectorStores.__importStar) || function (mod) {
		    if (mod && mod.__esModule) return mod;
		    var result = {};
		    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
		    __setModuleDefault(result, mod);
		    return result;
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.VectorStoresPage = exports.VectorStores = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		const core_1 = /*@__PURE__*/ requireCore();
		const VectorStoresAPI = __importStar(/*@__PURE__*/ requireVectorStores());
		const FileBatchesAPI = __importStar(/*@__PURE__*/ requireFileBatches());
		const FilesAPI = __importStar(/*@__PURE__*/ requireFiles$1());
		const pagination_1 = /*@__PURE__*/ requirePagination();
		class VectorStores extends resource_1.APIResource {
		    constructor() {
		        super(...arguments);
		        this.files = new FilesAPI.Files(this._client);
		        this.fileBatches = new FileBatchesAPI.FileBatches(this._client);
		    }
		    /**
		     * Create a vector store.
		     */
		    create(body, options) {
		        return this._client.post('/vector_stores', {
		            body,
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		    /**
		     * Retrieves a vector store.
		     */
		    retrieve(vectorStoreId, options) {
		        return this._client.get(`/vector_stores/${vectorStoreId}`, {
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		    /**
		     * Modifies a vector store.
		     */
		    update(vectorStoreId, body, options) {
		        return this._client.post(`/vector_stores/${vectorStoreId}`, {
		            body,
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		    list(query = {}, options) {
		        if ((0, core_1.isRequestOptions)(query)) {
		            return this.list({}, query);
		        }
		        return this._client.getAPIList('/vector_stores', VectorStoresPage, {
		            query,
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		    /**
		     * Delete a vector store.
		     */
		    del(vectorStoreId, options) {
		        return this._client.delete(`/vector_stores/${vectorStoreId}`, {
		            ...options,
		            headers: { 'OpenAI-Beta': 'assistants=v2', ...options?.headers },
		        });
		    }
		}
		exports.VectorStores = VectorStores;
		class VectorStoresPage extends pagination_1.CursorPage {
		}
		exports.VectorStoresPage = VectorStoresPage;
		(function (VectorStores) {
		    VectorStores.VectorStoresPage = VectorStoresAPI.VectorStoresPage;
		    VectorStores.Files = FilesAPI.Files;
		    VectorStores.VectorStoreFilesPage = FilesAPI.VectorStoreFilesPage;
		    VectorStores.FileBatches = FileBatchesAPI.FileBatches;
		})(VectorStores = exports.VectorStores || (exports.VectorStores = {}));
		
	} (vectorStores));
	return vectorStores;
}

var hasRequiredBeta;

function requireBeta () {
	if (hasRequiredBeta) return beta;
	hasRequiredBeta = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		var __createBinding = (beta && beta.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __setModuleDefault = (beta && beta.__setModuleDefault) || (Object.create ? (function(o, v) {
		    Object.defineProperty(o, "default", { enumerable: true, value: v });
		}) : function(o, v) {
		    o["default"] = v;
		});
		var __importStar = (beta && beta.__importStar) || function (mod) {
		    if (mod && mod.__esModule) return mod;
		    var result = {};
		    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
		    __setModuleDefault(result, mod);
		    return result;
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.Beta = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		const AssistantsAPI = __importStar(/*@__PURE__*/ requireAssistants());
		const ChatAPI = __importStar(/*@__PURE__*/ requireChat());
		const ThreadsAPI = __importStar(/*@__PURE__*/ requireThreads());
		const VectorStoresAPI = __importStar(/*@__PURE__*/ requireVectorStores());
		class Beta extends resource_1.APIResource {
		    constructor() {
		        super(...arguments);
		        this.vectorStores = new VectorStoresAPI.VectorStores(this._client);
		        this.chat = new ChatAPI.Chat(this._client);
		        this.assistants = new AssistantsAPI.Assistants(this._client);
		        this.threads = new ThreadsAPI.Threads(this._client);
		    }
		}
		exports.Beta = Beta;
		(function (Beta) {
		    Beta.VectorStores = VectorStoresAPI.VectorStores;
		    Beta.VectorStoresPage = VectorStoresAPI.VectorStoresPage;
		    Beta.Chat = ChatAPI.Chat;
		    Beta.Assistants = AssistantsAPI.Assistants;
		    Beta.AssistantsPage = AssistantsAPI.AssistantsPage;
		    Beta.Threads = ThreadsAPI.Threads;
		})(Beta = exports.Beta || (exports.Beta = {}));
		
	} (beta));
	return beta;
}

var completions = {};

var hasRequiredCompletions;

function requireCompletions () {
	if (hasRequiredCompletions) return completions;
	hasRequiredCompletions = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.Completions = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		class Completions extends resource_1.APIResource {
		    create(body, options) {
		        return this._client.post('/completions', { body, ...options, stream: body.stream ?? false });
		    }
		}
		exports.Completions = Completions;
		(function (Completions) {
		})(Completions = exports.Completions || (exports.Completions = {}));
		
	} (completions));
	return completions;
}

var embeddings = {};

var hasRequiredEmbeddings;

function requireEmbeddings () {
	if (hasRequiredEmbeddings) return embeddings;
	hasRequiredEmbeddings = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.Embeddings = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		class Embeddings extends resource_1.APIResource {
		    /**
		     * Creates an embedding vector representing the input text.
		     */
		    create(body, options) {
		        return this._client.post('/embeddings', { body, ...options });
		    }
		}
		exports.Embeddings = Embeddings;
		(function (Embeddings) {
		})(Embeddings = exports.Embeddings || (exports.Embeddings = {}));
		
	} (embeddings));
	return embeddings;
}

var files = {};

var hasRequiredFiles;

function requireFiles () {
	if (hasRequiredFiles) return files;
	hasRequiredFiles = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		var __createBinding = (files && files.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __setModuleDefault = (files && files.__setModuleDefault) || (Object.create ? (function(o, v) {
		    Object.defineProperty(o, "default", { enumerable: true, value: v });
		}) : function(o, v) {
		    o["default"] = v;
		});
		var __importStar = (files && files.__importStar) || function (mod) {
		    if (mod && mod.__esModule) return mod;
		    var result = {};
		    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
		    __setModuleDefault(result, mod);
		    return result;
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.FileObjectsPage = exports.Files = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		const core_1 = /*@__PURE__*/ requireCore();
		const core_2 = /*@__PURE__*/ requireCore();
		const error_1 = /*@__PURE__*/ requireError();
		const Core = __importStar(/*@__PURE__*/ requireCore());
		const FilesAPI = __importStar(/*@__PURE__*/ requireFiles());
		const pagination_1 = /*@__PURE__*/ requirePagination();
		class Files extends resource_1.APIResource {
		    /**
		     * Upload a file that can be used across various endpoints. Individual files can be
		     * up to 512 MB, and the size of all files uploaded by one organization can be up
		     * to 100 GB.
		     *
		     * The Assistants API supports files up to 2 million tokens and of specific file
		     * types. See the
		     * [Assistants Tools guide](https://platform.openai.com/docs/assistants/tools) for
		     * details.
		     *
		     * The Fine-tuning API only supports `.jsonl` files. The input also has certain
		     * required formats for fine-tuning
		     * [chat](https://platform.openai.com/docs/api-reference/fine-tuning/chat-input) or
		     * [completions](https://platform.openai.com/docs/api-reference/fine-tuning/completions-input)
		     * models.
		     *
		     * The Batch API only supports `.jsonl` files up to 100 MB in size. The input also
		     * has a specific required
		     * [format](https://platform.openai.com/docs/api-reference/batch/request-input).
		     *
		     * Please [contact us](https://help.openai.com/) if you need to increase these
		     * storage limits.
		     */
		    create(body, options) {
		        return this._client.post('/files', Core.multipartFormRequestOptions({ body, ...options }));
		    }
		    /**
		     * Returns information about a specific file.
		     */
		    retrieve(fileId, options) {
		        return this._client.get(`/files/${fileId}`, options);
		    }
		    list(query = {}, options) {
		        if ((0, core_1.isRequestOptions)(query)) {
		            return this.list({}, query);
		        }
		        return this._client.getAPIList('/files', FileObjectsPage, { query, ...options });
		    }
		    /**
		     * Delete a file.
		     */
		    del(fileId, options) {
		        return this._client.delete(`/files/${fileId}`, options);
		    }
		    /**
		     * Returns the contents of the specified file.
		     */
		    content(fileId, options) {
		        return this._client.get(`/files/${fileId}/content`, { ...options, __binaryResponse: true });
		    }
		    /**
		     * Returns the contents of the specified file.
		     *
		     * @deprecated The `.content()` method should be used instead
		     */
		    retrieveContent(fileId, options) {
		        return this._client.get(`/files/${fileId}/content`, {
		            ...options,
		            headers: { Accept: 'application/json', ...options?.headers },
		        });
		    }
		    /**
		     * Waits for the given file to be processed, default timeout is 30 mins.
		     */
		    async waitForProcessing(id, { pollInterval = 5000, maxWait = 30 * 60 * 1000 } = {}) {
		        const TERMINAL_STATES = new Set(['processed', 'error', 'deleted']);
		        const start = Date.now();
		        let file = await this.retrieve(id);
		        while (!file.status || !TERMINAL_STATES.has(file.status)) {
		            await (0, core_2.sleep)(pollInterval);
		            file = await this.retrieve(id);
		            if (Date.now() - start > maxWait) {
		                throw new error_1.APIConnectionTimeoutError({
		                    message: `Giving up on waiting for file ${id} to finish processing after ${maxWait} milliseconds.`,
		                });
		            }
		        }
		        return file;
		    }
		}
		exports.Files = Files;
		/**
		 * Note: no pagination actually occurs yet, this is for forwards-compatibility.
		 */
		class FileObjectsPage extends pagination_1.Page {
		}
		exports.FileObjectsPage = FileObjectsPage;
		(function (Files) {
		    Files.FileObjectsPage = FilesAPI.FileObjectsPage;
		})(Files = exports.Files || (exports.Files = {}));
		
	} (files));
	return files;
}

var fineTuning = {};

var jobs = {};

var checkpoints = {};

var hasRequiredCheckpoints;

function requireCheckpoints () {
	if (hasRequiredCheckpoints) return checkpoints;
	hasRequiredCheckpoints = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		var __createBinding = (checkpoints && checkpoints.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __setModuleDefault = (checkpoints && checkpoints.__setModuleDefault) || (Object.create ? (function(o, v) {
		    Object.defineProperty(o, "default", { enumerable: true, value: v });
		}) : function(o, v) {
		    o["default"] = v;
		});
		var __importStar = (checkpoints && checkpoints.__importStar) || function (mod) {
		    if (mod && mod.__esModule) return mod;
		    var result = {};
		    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
		    __setModuleDefault(result, mod);
		    return result;
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.FineTuningJobCheckpointsPage = exports.Checkpoints = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		const core_1 = /*@__PURE__*/ requireCore();
		const CheckpointsAPI = __importStar(/*@__PURE__*/ requireCheckpoints());
		const pagination_1 = /*@__PURE__*/ requirePagination();
		class Checkpoints extends resource_1.APIResource {
		    list(fineTuningJobId, query = {}, options) {
		        if ((0, core_1.isRequestOptions)(query)) {
		            return this.list(fineTuningJobId, {}, query);
		        }
		        return this._client.getAPIList(`/fine_tuning/jobs/${fineTuningJobId}/checkpoints`, FineTuningJobCheckpointsPage, { query, ...options });
		    }
		}
		exports.Checkpoints = Checkpoints;
		class FineTuningJobCheckpointsPage extends pagination_1.CursorPage {
		}
		exports.FineTuningJobCheckpointsPage = FineTuningJobCheckpointsPage;
		(function (Checkpoints) {
		    Checkpoints.FineTuningJobCheckpointsPage = CheckpointsAPI.FineTuningJobCheckpointsPage;
		})(Checkpoints = exports.Checkpoints || (exports.Checkpoints = {}));
		
	} (checkpoints));
	return checkpoints;
}

var hasRequiredJobs;

function requireJobs () {
	if (hasRequiredJobs) return jobs;
	hasRequiredJobs = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		var __createBinding = (jobs && jobs.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __setModuleDefault = (jobs && jobs.__setModuleDefault) || (Object.create ? (function(o, v) {
		    Object.defineProperty(o, "default", { enumerable: true, value: v });
		}) : function(o, v) {
		    o["default"] = v;
		});
		var __importStar = (jobs && jobs.__importStar) || function (mod) {
		    if (mod && mod.__esModule) return mod;
		    var result = {};
		    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
		    __setModuleDefault(result, mod);
		    return result;
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.FineTuningJobEventsPage = exports.FineTuningJobsPage = exports.Jobs = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		const core_1 = /*@__PURE__*/ requireCore();
		const JobsAPI = __importStar(/*@__PURE__*/ requireJobs());
		const CheckpointsAPI = __importStar(/*@__PURE__*/ requireCheckpoints());
		const pagination_1 = /*@__PURE__*/ requirePagination();
		class Jobs extends resource_1.APIResource {
		    constructor() {
		        super(...arguments);
		        this.checkpoints = new CheckpointsAPI.Checkpoints(this._client);
		    }
		    /**
		     * Creates a fine-tuning job which begins the process of creating a new model from
		     * a given dataset.
		     *
		     * Response includes details of the enqueued job including job status and the name
		     * of the fine-tuned models once complete.
		     *
		     * [Learn more about fine-tuning](https://platform.openai.com/docs/guides/fine-tuning)
		     */
		    create(body, options) {
		        return this._client.post('/fine_tuning/jobs', { body, ...options });
		    }
		    /**
		     * Get info about a fine-tuning job.
		     *
		     * [Learn more about fine-tuning](https://platform.openai.com/docs/guides/fine-tuning)
		     */
		    retrieve(fineTuningJobId, options) {
		        return this._client.get(`/fine_tuning/jobs/${fineTuningJobId}`, options);
		    }
		    list(query = {}, options) {
		        if ((0, core_1.isRequestOptions)(query)) {
		            return this.list({}, query);
		        }
		        return this._client.getAPIList('/fine_tuning/jobs', FineTuningJobsPage, { query, ...options });
		    }
		    /**
		     * Immediately cancel a fine-tune job.
		     */
		    cancel(fineTuningJobId, options) {
		        return this._client.post(`/fine_tuning/jobs/${fineTuningJobId}/cancel`, options);
		    }
		    listEvents(fineTuningJobId, query = {}, options) {
		        if ((0, core_1.isRequestOptions)(query)) {
		            return this.listEvents(fineTuningJobId, {}, query);
		        }
		        return this._client.getAPIList(`/fine_tuning/jobs/${fineTuningJobId}/events`, FineTuningJobEventsPage, {
		            query,
		            ...options,
		        });
		    }
		}
		exports.Jobs = Jobs;
		class FineTuningJobsPage extends pagination_1.CursorPage {
		}
		exports.FineTuningJobsPage = FineTuningJobsPage;
		class FineTuningJobEventsPage extends pagination_1.CursorPage {
		}
		exports.FineTuningJobEventsPage = FineTuningJobEventsPage;
		(function (Jobs) {
		    Jobs.FineTuningJobsPage = JobsAPI.FineTuningJobsPage;
		    Jobs.FineTuningJobEventsPage = JobsAPI.FineTuningJobEventsPage;
		    Jobs.Checkpoints = CheckpointsAPI.Checkpoints;
		    Jobs.FineTuningJobCheckpointsPage = CheckpointsAPI.FineTuningJobCheckpointsPage;
		})(Jobs = exports.Jobs || (exports.Jobs = {}));
		
	} (jobs));
	return jobs;
}

var hasRequiredFineTuning;

function requireFineTuning () {
	if (hasRequiredFineTuning) return fineTuning;
	hasRequiredFineTuning = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		var __createBinding = (fineTuning && fineTuning.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __setModuleDefault = (fineTuning && fineTuning.__setModuleDefault) || (Object.create ? (function(o, v) {
		    Object.defineProperty(o, "default", { enumerable: true, value: v });
		}) : function(o, v) {
		    o["default"] = v;
		});
		var __importStar = (fineTuning && fineTuning.__importStar) || function (mod) {
		    if (mod && mod.__esModule) return mod;
		    var result = {};
		    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
		    __setModuleDefault(result, mod);
		    return result;
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.FineTuning = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		const JobsAPI = __importStar(/*@__PURE__*/ requireJobs());
		class FineTuning extends resource_1.APIResource {
		    constructor() {
		        super(...arguments);
		        this.jobs = new JobsAPI.Jobs(this._client);
		    }
		}
		exports.FineTuning = FineTuning;
		(function (FineTuning) {
		    FineTuning.Jobs = JobsAPI.Jobs;
		    FineTuning.FineTuningJobsPage = JobsAPI.FineTuningJobsPage;
		    FineTuning.FineTuningJobEventsPage = JobsAPI.FineTuningJobEventsPage;
		})(FineTuning = exports.FineTuning || (exports.FineTuning = {}));
		
	} (fineTuning));
	return fineTuning;
}

var images = {};

var hasRequiredImages;

function requireImages () {
	if (hasRequiredImages) return images;
	hasRequiredImages = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		var __createBinding = (images && images.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __setModuleDefault = (images && images.__setModuleDefault) || (Object.create ? (function(o, v) {
		    Object.defineProperty(o, "default", { enumerable: true, value: v });
		}) : function(o, v) {
		    o["default"] = v;
		});
		var __importStar = (images && images.__importStar) || function (mod) {
		    if (mod && mod.__esModule) return mod;
		    var result = {};
		    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
		    __setModuleDefault(result, mod);
		    return result;
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.Images = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		const Core = __importStar(/*@__PURE__*/ requireCore());
		class Images extends resource_1.APIResource {
		    /**
		     * Creates a variation of a given image.
		     */
		    createVariation(body, options) {
		        return this._client.post('/images/variations', Core.multipartFormRequestOptions({ body, ...options }));
		    }
		    /**
		     * Creates an edited or extended image given an original image and a prompt.
		     */
		    edit(body, options) {
		        return this._client.post('/images/edits', Core.multipartFormRequestOptions({ body, ...options }));
		    }
		    /**
		     * Creates an image given a prompt.
		     */
		    generate(body, options) {
		        return this._client.post('/images/generations', { body, ...options });
		    }
		}
		exports.Images = Images;
		(function (Images) {
		})(Images = exports.Images || (exports.Images = {}));
		
	} (images));
	return images;
}

var models = {};

var hasRequiredModels;

function requireModels () {
	if (hasRequiredModels) return models;
	hasRequiredModels = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		var __createBinding = (models && models.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __setModuleDefault = (models && models.__setModuleDefault) || (Object.create ? (function(o, v) {
		    Object.defineProperty(o, "default", { enumerable: true, value: v });
		}) : function(o, v) {
		    o["default"] = v;
		});
		var __importStar = (models && models.__importStar) || function (mod) {
		    if (mod && mod.__esModule) return mod;
		    var result = {};
		    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
		    __setModuleDefault(result, mod);
		    return result;
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.ModelsPage = exports.Models = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		const ModelsAPI = __importStar(/*@__PURE__*/ requireModels());
		const pagination_1 = /*@__PURE__*/ requirePagination();
		class Models extends resource_1.APIResource {
		    /**
		     * Retrieves a model instance, providing basic information about the model such as
		     * the owner and permissioning.
		     */
		    retrieve(model, options) {
		        return this._client.get(`/models/${model}`, options);
		    }
		    /**
		     * Lists the currently available models, and provides basic information about each
		     * one such as the owner and availability.
		     */
		    list(options) {
		        return this._client.getAPIList('/models', ModelsPage, options);
		    }
		    /**
		     * Delete a fine-tuned model. You must have the Owner role in your organization to
		     * delete a model.
		     */
		    del(model, options) {
		        return this._client.delete(`/models/${model}`, options);
		    }
		}
		exports.Models = Models;
		/**
		 * Note: no pagination actually occurs yet, this is for forwards-compatibility.
		 */
		class ModelsPage extends pagination_1.Page {
		}
		exports.ModelsPage = ModelsPage;
		(function (Models) {
		    Models.ModelsPage = ModelsAPI.ModelsPage;
		})(Models = exports.Models || (exports.Models = {}));
		
	} (models));
	return models;
}

var moderations = {};

var hasRequiredModerations;

function requireModerations () {
	if (hasRequiredModerations) return moderations;
	hasRequiredModerations = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.Moderations = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		class Moderations extends resource_1.APIResource {
		    /**
		     * Classifies if text and/or image inputs are potentially harmful. Learn more in
		     * the [moderation guide](https://platform.openai.com/docs/guides/moderation).
		     */
		    create(body, options) {
		        return this._client.post('/moderations', { body, ...options });
		    }
		}
		exports.Moderations = Moderations;
		(function (Moderations) {
		})(Moderations = exports.Moderations || (exports.Moderations = {}));
		
	} (moderations));
	return moderations;
}

var uploads = {};

var parts = {};

var hasRequiredParts;

function requireParts () {
	if (hasRequiredParts) return parts;
	hasRequiredParts = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		var __createBinding = (parts && parts.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __setModuleDefault = (parts && parts.__setModuleDefault) || (Object.create ? (function(o, v) {
		    Object.defineProperty(o, "default", { enumerable: true, value: v });
		}) : function(o, v) {
		    o["default"] = v;
		});
		var __importStar = (parts && parts.__importStar) || function (mod) {
		    if (mod && mod.__esModule) return mod;
		    var result = {};
		    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
		    __setModuleDefault(result, mod);
		    return result;
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.Parts = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		const Core = __importStar(/*@__PURE__*/ requireCore());
		class Parts extends resource_1.APIResource {
		    /**
		     * Adds a
		     * [Part](https://platform.openai.com/docs/api-reference/uploads/part-object) to an
		     * [Upload](https://platform.openai.com/docs/api-reference/uploads/object) object.
		     * A Part represents a chunk of bytes from the file you are trying to upload.
		     *
		     * Each Part can be at most 64 MB, and you can add Parts until you hit the Upload
		     * maximum of 8 GB.
		     *
		     * It is possible to add multiple Parts in parallel. You can decide the intended
		     * order of the Parts when you
		     * [complete the Upload](https://platform.openai.com/docs/api-reference/uploads/complete).
		     */
		    create(uploadId, body, options) {
		        return this._client.post(`/uploads/${uploadId}/parts`, Core.multipartFormRequestOptions({ body, ...options }));
		    }
		}
		exports.Parts = Parts;
		(function (Parts) {
		})(Parts = exports.Parts || (exports.Parts = {}));
		
	} (parts));
	return parts;
}

var hasRequiredUploads;

function requireUploads () {
	if (hasRequiredUploads) return uploads;
	hasRequiredUploads = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		var __createBinding = (uploads && uploads.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __setModuleDefault = (uploads && uploads.__setModuleDefault) || (Object.create ? (function(o, v) {
		    Object.defineProperty(o, "default", { enumerable: true, value: v });
		}) : function(o, v) {
		    o["default"] = v;
		});
		var __importStar = (uploads && uploads.__importStar) || function (mod) {
		    if (mod && mod.__esModule) return mod;
		    var result = {};
		    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
		    __setModuleDefault(result, mod);
		    return result;
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.Uploads = void 0;
		const resource_1 = /*@__PURE__*/ requireResource();
		const PartsAPI = __importStar(/*@__PURE__*/ requireParts());
		class Uploads extends resource_1.APIResource {
		    constructor() {
		        super(...arguments);
		        this.parts = new PartsAPI.Parts(this._client);
		    }
		    /**
		     * Creates an intermediate
		     * [Upload](https://platform.openai.com/docs/api-reference/uploads/object) object
		     * that you can add
		     * [Parts](https://platform.openai.com/docs/api-reference/uploads/part-object) to.
		     * Currently, an Upload can accept at most 8 GB in total and expires after an hour
		     * after you create it.
		     *
		     * Once you complete the Upload, we will create a
		     * [File](https://platform.openai.com/docs/api-reference/files/object) object that
		     * contains all the parts you uploaded. This File is usable in the rest of our
		     * platform as a regular File object.
		     *
		     * For certain `purpose`s, the correct `mime_type` must be specified. Please refer
		     * to documentation for the supported MIME types for your use case:
		     *
		     * - [Assistants](https://platform.openai.com/docs/assistants/tools/file-search/supported-files)
		     *
		     * For guidance on the proper filename extensions for each purpose, please follow
		     * the documentation on
		     * [creating a File](https://platform.openai.com/docs/api-reference/files/create).
		     */
		    create(body, options) {
		        return this._client.post('/uploads', { body, ...options });
		    }
		    /**
		     * Cancels the Upload. No Parts may be added after an Upload is cancelled.
		     */
		    cancel(uploadId, options) {
		        return this._client.post(`/uploads/${uploadId}/cancel`, options);
		    }
		    /**
		     * Completes the
		     * [Upload](https://platform.openai.com/docs/api-reference/uploads/object).
		     *
		     * Within the returned Upload object, there is a nested
		     * [File](https://platform.openai.com/docs/api-reference/files/object) object that
		     * is ready to use in the rest of the platform.
		     *
		     * You can specify the order of the Parts by passing in an ordered list of the Part
		     * IDs.
		     *
		     * The number of bytes uploaded upon completion must match the number of bytes
		     * initially specified when creating the Upload object. No Parts may be added after
		     * an Upload is completed.
		     */
		    complete(uploadId, body, options) {
		        return this._client.post(`/uploads/${uploadId}/complete`, { body, ...options });
		    }
		}
		exports.Uploads = Uploads;
		(function (Uploads) {
		    Uploads.Parts = PartsAPI.Parts;
		})(Uploads = exports.Uploads || (exports.Uploads = {}));
		
	} (uploads));
	return uploads;
}

var hasRequiredResources;

function requireResources () {
	if (hasRequiredResources) return resources;
	hasRequiredResources = 1;
	(function (exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		var __createBinding = (resources && resources.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __exportStar = (resources && resources.__exportStar) || function(m, exports) {
		    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.Uploads = exports.Moderations = exports.Models = exports.ModelsPage = exports.Images = exports.FineTuning = exports.Files = exports.FileObjectsPage = exports.Embeddings = exports.Completions = exports.Beta = exports.Batches = exports.BatchesPage = exports.Audio = void 0;
		__exportStar(/*@__PURE__*/ requireChat$1(), exports);
		__exportStar(/*@__PURE__*/ requireShared(), exports);
		var audio_1 = /*@__PURE__*/ requireAudio();
		Object.defineProperty(exports, "Audio", { enumerable: true, get: function () { return audio_1.Audio; } });
		var batches_1 = /*@__PURE__*/ requireBatches();
		Object.defineProperty(exports, "BatchesPage", { enumerable: true, get: function () { return batches_1.BatchesPage; } });
		Object.defineProperty(exports, "Batches", { enumerable: true, get: function () { return batches_1.Batches; } });
		var beta_1 = /*@__PURE__*/ requireBeta();
		Object.defineProperty(exports, "Beta", { enumerable: true, get: function () { return beta_1.Beta; } });
		var completions_1 = /*@__PURE__*/ requireCompletions();
		Object.defineProperty(exports, "Completions", { enumerable: true, get: function () { return completions_1.Completions; } });
		var embeddings_1 = /*@__PURE__*/ requireEmbeddings();
		Object.defineProperty(exports, "Embeddings", { enumerable: true, get: function () { return embeddings_1.Embeddings; } });
		var files_1 = /*@__PURE__*/ requireFiles();
		Object.defineProperty(exports, "FileObjectsPage", { enumerable: true, get: function () { return files_1.FileObjectsPage; } });
		Object.defineProperty(exports, "Files", { enumerable: true, get: function () { return files_1.Files; } });
		var fine_tuning_1 = /*@__PURE__*/ requireFineTuning();
		Object.defineProperty(exports, "FineTuning", { enumerable: true, get: function () { return fine_tuning_1.FineTuning; } });
		var images_1 = /*@__PURE__*/ requireImages();
		Object.defineProperty(exports, "Images", { enumerable: true, get: function () { return images_1.Images; } });
		var models_1 = /*@__PURE__*/ requireModels();
		Object.defineProperty(exports, "ModelsPage", { enumerable: true, get: function () { return models_1.ModelsPage; } });
		Object.defineProperty(exports, "Models", { enumerable: true, get: function () { return models_1.Models; } });
		var moderations_1 = /*@__PURE__*/ requireModerations();
		Object.defineProperty(exports, "Moderations", { enumerable: true, get: function () { return moderations_1.Moderations; } });
		var uploads_1 = /*@__PURE__*/ requireUploads();
		Object.defineProperty(exports, "Uploads", { enumerable: true, get: function () { return uploads_1.Uploads; } });
		
	} (resources));
	return resources;
}

var openai = openai$1.exports;

var hasRequiredOpenai$1;

function requireOpenai$1 () {
	if (hasRequiredOpenai$1) return openai$1.exports;
	hasRequiredOpenai$1 = 1;
	(function (module, exports) {
		// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
		var __createBinding = (openai && openai.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __setModuleDefault = (openai && openai.__setModuleDefault) || (Object.create ? (function(o, v) {
		    Object.defineProperty(o, "default", { enumerable: true, value: v });
		}) : function(o, v) {
		    o["default"] = v;
		});
		var __importStar = (openai && openai.__importStar) || function (mod) {
		    if (mod && mod.__esModule) return mod;
		    var result = {};
		    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
		    __setModuleDefault(result, mod);
		    return result;
		};
		var _a;
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.AzureOpenAI = exports.fileFromPath = exports.toFile = exports.UnprocessableEntityError = exports.PermissionDeniedError = exports.InternalServerError = exports.AuthenticationError = exports.BadRequestError = exports.RateLimitError = exports.ConflictError = exports.NotFoundError = exports.APIUserAbortError = exports.APIConnectionTimeoutError = exports.APIConnectionError = exports.APIError = exports.OpenAIError = exports.OpenAI = void 0;
		const Errors = __importStar(/*@__PURE__*/ requireError());
		const Uploads = __importStar(/*@__PURE__*/ requireUploads$1());
		const qs = __importStar(/*@__PURE__*/ requireQs());
		const Core = __importStar(/*@__PURE__*/ requireCore());
		const Pagination = __importStar(/*@__PURE__*/ requirePagination());
		const API = __importStar(/*@__PURE__*/ requireResources());
		/**
		 * API Client for interfacing with the OpenAI API.
		 */
		class OpenAI extends Core.APIClient {
		    /**
		     * API Client for interfacing with the OpenAI API.
		     *
		     * @param {string | undefined} [opts.apiKey=process.env['OPENAI_API_KEY'] ?? undefined]
		     * @param {string | null | undefined} [opts.organization=process.env['OPENAI_ORG_ID'] ?? null]
		     * @param {string | null | undefined} [opts.project=process.env['OPENAI_PROJECT_ID'] ?? null]
		     * @param {string} [opts.baseURL=process.env['OPENAI_BASE_URL'] ?? https://api.openai.com/v1] - Override the default base URL for the API.
		     * @param {number} [opts.timeout=10 minutes] - The maximum amount of time (in milliseconds) the client will wait for a response before timing out.
		     * @param {number} [opts.httpAgent] - An HTTP agent used to manage HTTP(s) connections.
		     * @param {Core.Fetch} [opts.fetch] - Specify a custom `fetch` function implementation.
		     * @param {number} [opts.maxRetries=2] - The maximum number of times the client will retry a request.
		     * @param {Core.Headers} opts.defaultHeaders - Default headers to include with every request to the API.
		     * @param {Core.DefaultQuery} opts.defaultQuery - Default query parameters to include with every request to the API.
		     * @param {boolean} [opts.dangerouslyAllowBrowser=false] - By default, client-side use of this library is not allowed, as it risks exposing your secret API credentials to attackers.
		     */
		    constructor({ baseURL = Core.readEnv('OPENAI_BASE_URL'), apiKey = Core.readEnv('OPENAI_API_KEY'), organization = Core.readEnv('OPENAI_ORG_ID') ?? null, project = Core.readEnv('OPENAI_PROJECT_ID') ?? null, ...opts } = {}) {
		        if (apiKey === undefined) {
		            throw new Errors.OpenAIError("The OPENAI_API_KEY environment variable is missing or empty; either provide it, or instantiate the OpenAI client with an apiKey option, like new OpenAI({ apiKey: 'My API Key' }).");
		        }
		        const options = {
		            apiKey,
		            organization,
		            project,
		            ...opts,
		            baseURL: baseURL || `https://api.openai.com/v1`,
		        };
		        if (!options.dangerouslyAllowBrowser && Core.isRunningInBrowser()) {
		            throw new Errors.OpenAIError("It looks like you're running in a browser-like environment.\n\nThis is disabled by default, as it risks exposing your secret API credentials to attackers.\nIf you understand the risks and have appropriate mitigations in place,\nyou can set the `dangerouslyAllowBrowser` option to `true`, e.g.,\n\nnew OpenAI({ apiKey, dangerouslyAllowBrowser: true });\n\nhttps://help.openai.com/en/articles/5112595-best-practices-for-api-key-safety\n");
		        }
		        super({
		            baseURL: options.baseURL,
		            timeout: options.timeout ?? 600000 /* 10 minutes */,
		            httpAgent: options.httpAgent,
		            maxRetries: options.maxRetries,
		            fetch: options.fetch,
		        });
		        this.completions = new API.Completions(this);
		        this.chat = new API.Chat(this);
		        this.embeddings = new API.Embeddings(this);
		        this.files = new API.Files(this);
		        this.images = new API.Images(this);
		        this.audio = new API.Audio(this);
		        this.moderations = new API.Moderations(this);
		        this.models = new API.Models(this);
		        this.fineTuning = new API.FineTuning(this);
		        this.beta = new API.Beta(this);
		        this.batches = new API.Batches(this);
		        this.uploads = new API.Uploads(this);
		        this._options = options;
		        this.apiKey = apiKey;
		        this.organization = organization;
		        this.project = project;
		    }
		    defaultQuery() {
		        return this._options.defaultQuery;
		    }
		    defaultHeaders(opts) {
		        return {
		            ...super.defaultHeaders(opts),
		            'OpenAI-Organization': this.organization,
		            'OpenAI-Project': this.project,
		            ...this._options.defaultHeaders,
		        };
		    }
		    authHeaders(opts) {
		        return { Authorization: `Bearer ${this.apiKey}` };
		    }
		    stringifyQuery(query) {
		        return qs.stringify(query, { arrayFormat: 'brackets' });
		    }
		}
		exports.OpenAI = OpenAI;
		_a = OpenAI;
		OpenAI.OpenAI = _a;
		OpenAI.DEFAULT_TIMEOUT = 600000; // 10 minutes
		OpenAI.OpenAIError = Errors.OpenAIError;
		OpenAI.APIError = Errors.APIError;
		OpenAI.APIConnectionError = Errors.APIConnectionError;
		OpenAI.APIConnectionTimeoutError = Errors.APIConnectionTimeoutError;
		OpenAI.APIUserAbortError = Errors.APIUserAbortError;
		OpenAI.NotFoundError = Errors.NotFoundError;
		OpenAI.ConflictError = Errors.ConflictError;
		OpenAI.RateLimitError = Errors.RateLimitError;
		OpenAI.BadRequestError = Errors.BadRequestError;
		OpenAI.AuthenticationError = Errors.AuthenticationError;
		OpenAI.InternalServerError = Errors.InternalServerError;
		OpenAI.PermissionDeniedError = Errors.PermissionDeniedError;
		OpenAI.UnprocessableEntityError = Errors.UnprocessableEntityError;
		OpenAI.toFile = Uploads.toFile;
		OpenAI.fileFromPath = Uploads.fileFromPath;
		exports.OpenAIError = Errors.OpenAIError, exports.APIError = Errors.APIError, exports.APIConnectionError = Errors.APIConnectionError, exports.APIConnectionTimeoutError = Errors.APIConnectionTimeoutError, exports.APIUserAbortError = Errors.APIUserAbortError, exports.NotFoundError = Errors.NotFoundError, exports.ConflictError = Errors.ConflictError, exports.RateLimitError = Errors.RateLimitError, exports.BadRequestError = Errors.BadRequestError, exports.AuthenticationError = Errors.AuthenticationError, exports.InternalServerError = Errors.InternalServerError, exports.PermissionDeniedError = Errors.PermissionDeniedError, exports.UnprocessableEntityError = Errors.UnprocessableEntityError;
		exports.toFile = Uploads.toFile;
		exports.fileFromPath = Uploads.fileFromPath;
		(function (OpenAI) {
		    OpenAI.Page = Pagination.Page;
		    OpenAI.CursorPage = Pagination.CursorPage;
		    OpenAI.Completions = API.Completions;
		    OpenAI.Chat = API.Chat;
		    OpenAI.Embeddings = API.Embeddings;
		    OpenAI.Files = API.Files;
		    OpenAI.FileObjectsPage = API.FileObjectsPage;
		    OpenAI.Images = API.Images;
		    OpenAI.Audio = API.Audio;
		    OpenAI.Moderations = API.Moderations;
		    OpenAI.Models = API.Models;
		    OpenAI.ModelsPage = API.ModelsPage;
		    OpenAI.FineTuning = API.FineTuning;
		    OpenAI.Beta = API.Beta;
		    OpenAI.Batches = API.Batches;
		    OpenAI.BatchesPage = API.BatchesPage;
		    OpenAI.Uploads = API.Uploads;
		})(OpenAI = exports.OpenAI || (exports.OpenAI = {}));
		/** API Client for interfacing with the Azure OpenAI API. */
		class AzureOpenAI extends OpenAI {
		    /**
		     * API Client for interfacing with the Azure OpenAI API.
		     *
		     * @param {string | undefined} [opts.apiVersion=process.env['OPENAI_API_VERSION'] ?? undefined]
		     * @param {string | undefined} [opts.endpoint=process.env['AZURE_OPENAI_ENDPOINT'] ?? undefined] - Your Azure endpoint, including the resource, e.g. `https://example-resource.azure.openai.com/`
		     * @param {string | undefined} [opts.apiKey=process.env['AZURE_OPENAI_API_KEY'] ?? undefined]
		     * @param {string | undefined} opts.deployment - A model deployment, if given, sets the base client URL to include `/deployments/{deployment}`.
		     * @param {string | null | undefined} [opts.organization=process.env['OPENAI_ORG_ID'] ?? null]
		     * @param {string} [opts.baseURL=process.env['OPENAI_BASE_URL']] - Sets the base URL for the API, e.g. `https://example-resource.azure.openai.com/openai/`.
		     * @param {number} [opts.timeout=10 minutes] - The maximum amount of time (in milliseconds) the client will wait for a response before timing out.
		     * @param {number} [opts.httpAgent] - An HTTP agent used to manage HTTP(s) connections.
		     * @param {Core.Fetch} [opts.fetch] - Specify a custom `fetch` function implementation.
		     * @param {number} [opts.maxRetries=2] - The maximum number of times the client will retry a request.
		     * @param {Core.Headers} opts.defaultHeaders - Default headers to include with every request to the API.
		     * @param {Core.DefaultQuery} opts.defaultQuery - Default query parameters to include with every request to the API.
		     * @param {boolean} [opts.dangerouslyAllowBrowser=false] - By default, client-side use of this library is not allowed, as it risks exposing your secret API credentials to attackers.
		     */
		    constructor({ baseURL = Core.readEnv('OPENAI_BASE_URL'), apiKey = Core.readEnv('AZURE_OPENAI_API_KEY'), apiVersion = Core.readEnv('OPENAI_API_VERSION'), endpoint, deployment, azureADTokenProvider, dangerouslyAllowBrowser, ...opts } = {}) {
		        if (!apiVersion) {
		            throw new Errors.OpenAIError("The OPENAI_API_VERSION environment variable is missing or empty; either provide it, or instantiate the AzureOpenAI client with an apiVersion option, like new AzureOpenAI({ apiVersion: 'My API Version' }).");
		        }
		        if (typeof azureADTokenProvider === 'function') {
		            dangerouslyAllowBrowser = true;
		        }
		        if (!azureADTokenProvider && !apiKey) {
		            throw new Errors.OpenAIError('Missing credentials. Please pass one of `apiKey` and `azureADTokenProvider`, or set the `AZURE_OPENAI_API_KEY` environment variable.');
		        }
		        if (azureADTokenProvider && apiKey) {
		            throw new Errors.OpenAIError('The `apiKey` and `azureADTokenProvider` arguments are mutually exclusive; only one can be passed at a time.');
		        }
		        // define a sentinel value to avoid any typing issues
		        apiKey ?? (apiKey = API_KEY_SENTINEL);
		        opts.defaultQuery = { ...opts.defaultQuery, 'api-version': apiVersion };
		        if (!baseURL) {
		            if (!endpoint) {
		                endpoint = process.env['AZURE_OPENAI_ENDPOINT'];
		            }
		            if (!endpoint) {
		                throw new Errors.OpenAIError('Must provide one of the `baseURL` or `endpoint` arguments, or the `AZURE_OPENAI_ENDPOINT` environment variable');
		            }
		            baseURL = `${endpoint}/openai`;
		        }
		        else {
		            if (endpoint) {
		                throw new Errors.OpenAIError('baseURL and endpoint are mutually exclusive');
		            }
		        }
		        super({
		            apiKey,
		            baseURL,
		            ...opts,
		            ...(dangerouslyAllowBrowser !== undefined ? { dangerouslyAllowBrowser } : {}),
		        });
		        this.apiVersion = '';
		        this._azureADTokenProvider = azureADTokenProvider;
		        this.apiVersion = apiVersion;
		        this._deployment = deployment;
		    }
		    buildRequest(options) {
		        if (_deployments_endpoints.has(options.path) && options.method === 'post' && options.body !== undefined) {
		            if (!Core.isObj(options.body)) {
		                throw new Error('Expected request body to be an object');
		            }
		            const model = this._deployment || options.body['model'];
		            if (model !== undefined && !this.baseURL.includes('/deployments')) {
		                options.path = `/deployments/${model}${options.path}`;
		            }
		        }
		        return super.buildRequest(options);
		    }
		    async _getAzureADToken() {
		        if (typeof this._azureADTokenProvider === 'function') {
		            const token = await this._azureADTokenProvider();
		            if (!token || typeof token !== 'string') {
		                throw new Errors.OpenAIError(`Expected 'azureADTokenProvider' argument to return a string but it returned ${token}`);
		            }
		            return token;
		        }
		        return undefined;
		    }
		    authHeaders(opts) {
		        return {};
		    }
		    async prepareOptions(opts) {
		        /**
		         * The user should provide a bearer token provider if they want
		         * to use Azure AD authentication. The user shouldn't set the
		         * Authorization header manually because the header is overwritten
		         * with the Azure AD token if a bearer token provider is provided.
		         */
		        if (opts.headers?.['api-key']) {
		            return super.prepareOptions(opts);
		        }
		        const token = await this._getAzureADToken();
		        opts.headers ?? (opts.headers = {});
		        if (token) {
		            opts.headers['Authorization'] = `Bearer ${token}`;
		        }
		        else if (this.apiKey !== API_KEY_SENTINEL) {
		            opts.headers['api-key'] = this.apiKey;
		        }
		        else {
		            throw new Errors.OpenAIError('Unable to handle auth');
		        }
		        return super.prepareOptions(opts);
		    }
		}
		exports.AzureOpenAI = AzureOpenAI;
		const _deployments_endpoints = new Set([
		    '/completions',
		    '/chat/completions',
		    '/embeddings',
		    '/audio/transcriptions',
		    '/audio/translations',
		    '/audio/speech',
		    '/images/generations',
		]);
		const API_KEY_SENTINEL = '<Missing Key>';
		// ---------------------- End Azure ----------------------
		exports = module.exports = OpenAI;
		module.exports.AzureOpenAI = AzureOpenAI;
		exports.default = OpenAI;
		
	} (openai$1, openai$1.exports));
	return openai$1.exports;
}

var hasRequiredOpenai;

function requireOpenai () {
	if (hasRequiredOpenai) return openai$2;
	hasRequiredOpenai = 1;
	var __importDefault = (openai$2 && openai$2.__importDefault) || function (mod) {
	    return (mod && mod.__esModule) ? mod : { "default": mod };
	};
	Object.defineProperty(openai$2, "__esModule", { value: true });
	openai$2.OpenAIConnector = openai$2.OpenAIModel = openai$2.OpenAIKey = void 0;
	const zod_1 = /*@__PURE__*/ requireLib();
	requireWeb(); // FIXME 'web' fails on frontend/ts-src/types/ports.test.disabled.ts tests :/
	const openai_1 = __importDefault(/*@__PURE__*/ requireOpenai$1());
	// create a key: https://platform.openai.com/api-keys
	openai$2.OpenAIKey = zod_1.z.string();
	openai$2.OpenAIModel = zod_1.z.enum(['gpt-4o', 'gpt-4-turbo', 'gpt-3.5-turbo']);
	class OpenAIConnector {
	    constructor(opts) {
	        this.opts = opts;
	        this.query = async (system, user) => {
	            // console.log(`system: ${system}`)
	            // console.log(`user: ${user}`)
	            // https://platform.openai.com/docs/api-reference/chat/create
	            const completion = await this.openai.chat.completions.create({
	                model: this.opts.model,
	                messages: [
	                    { role: 'system', content: system },
	                    { role: 'user', content: user },
	                ].filter(m => m.content),
	            });
	            const answer = completion.choices?.[0].message.content;
	            // console.log(`answer ${this.opts.model}: ${answer}`)
	            return answer || 'Invalid LLM answer :/';
	        };
	        this.openai = new openai_1.default({ apiKey: opts.apiKey, dangerouslyAllowBrowser: true });
	    }
	}
	openai$2.OpenAIConnector = OpenAIConnector;
	
	return openai$2;
}

var hasRequiredLlm;

function requireLlm () {
	if (hasRequiredLlm) return llm;
	hasRequiredLlm = 1;
	(function (exports) {
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.OpenAIModel = exports.OpenAIKey = exports.OpenAIConnector = void 0;
		exports.textToSql = textToSql;
		exports.sqlToText = sqlToText;
		exports.updateSql = updateSql;
		exports.fixSql = fixSql;
		exports.optimizeSql = optimizeSql;
		exports.explainQueryPlan = explainQueryPlan;
		const llmUtils_1 = requireLlmUtils();
		// gpt-3.5-turbo: context window: 16k tokens
		// gpt-4-turbo: context window: 128k tokens
		// 1 token ~ 4 chars: https://platform.openai.com/tokenizer
		var openai_1 = requireOpenai();
		Object.defineProperty(exports, "OpenAIConnector", { enumerable: true, get: function () { return openai_1.OpenAIConnector; } });
		Object.defineProperty(exports, "OpenAIKey", { enumerable: true, get: function () { return openai_1.OpenAIKey; } });
		Object.defineProperty(exports, "OpenAIModel", { enumerable: true, get: function () { return openai_1.OpenAIModel; } });
		// TODO: suggest schema improvements: design, perf
		async function textToSql(llm, dialect, userPrompt, db) {
		    // https://docs.anthropic.com/en/prompt-library/sql-sorcerer
		    // https://dev.to/datalynx/llms-for-text-to-sql-problems-the-benchmark-vs-real-world-performance-2064
		    // if db too big (>100 tables), select relevant tables with a prompt
		    // hint with tables on the current layout?
		    const systemPrompt = 'You are a pragmatic data analyst focusing on query performance and correctness.\n' +
		        `Here is the database you have at your disposal:\n${(0, llmUtils_1.dbToPrompt)(db)}\n` +
		        `Answer the user request generating only one valid SQL query using ${dialect} dialect and formatted nicely, no other text at all, this is very important.`;
		    const answer = await llm.query(systemPrompt, userPrompt);
		    return (0, llmUtils_1.cleanSqlAnswer)(answer);
		}
		async function sqlToText(llm, sqlQuery) {
		    // TODO: ask for name, description and explanation of the query
		    const systemPrompt = 'You are an experimented data analyst.\n' +
		        'The user will with you an SQL query and you will explain it how it works.\n' +
		        'Answer in a clear and concise way, not extra explanation not linked to the query.';
		    return await llm.query(systemPrompt, sqlQuery);
		}
		async function updateSql(llm, dialect, sqlQuery, userPrompt, db) {
		    // TODO: update SQL with english prompt
		    return Promise.reject('Not implemented');
		}
		async function fixSql(llm, dialect, sqlQuery, error, db) {
		    // TODO: fix SQL query when it doesn't work
		    return Promise.reject('Not implemented');
		}
		async function optimizeSql(llm, dialect, sqlQuery, db) {
		    // TODO: rewrite query for better performance, provide indexes & query plan => challenge, make it clear when not possible!
		    return Promise.reject('Not implemented');
		}
		async function explainQueryPlan(llm, dialect, sqlQuery, queryPlan) {
		    // TODO: explain query plan
		    return Promise.reject('Not implemented');
		}
		// TODO: suggest tables for a specific topic (build layout)
		// TODO: chat with your db
		// TODO: suggest schema changes for a new feature
		// TODO: function calling to interact with Azimutt (ex: "Show all the tables related to projects")
		// TODO: detect PII, https://docs.anthropic.com/en/prompt-library/pii-purifier
		
	} (llm));
	return llm;
}

var hasRequiredLegacyProject;

function requireLegacyProject () {
	if (hasRequiredLegacyProject) return legacyProject;
	hasRequiredLegacyProject = 1;
	(function (exports) {
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.LegacyTableRowStateLoading = exports.LegacyTableRowStateFailure = exports.LegacyTableRowStateSuccess = exports.LegacyTableRowColumn = exports.LegacyRowPrimaryKey = exports.LegacyRowValue = exports.LegacySqlQueryOrigin = exports.LegacyMemo = exports.LegacyTableMeta = exports.LegacyColumnMeta = exports.LegacyTag = exports.LegacyNotes = exports.LegacyTableProps = exports.LegacyCanvasProps = exports.LegacySource = exports.LegacyProjectType = exports.LegacyProjectRelation = exports.LegacyProjectColumnRef = exports.LegacyProjectTable = exports.LegacyProjectCheck = exports.LegacyProjectIndex = exports.LegacyProjectUnique = exports.LegacyProjectPrimaryKey = exports.LegacyProjectColumn = exports.LegacyComment = exports.LegacyOrigin = exports.LegacySourceOrigin = exports.LegacySourceKind = exports.LegacyAmlEditor = exports.LegacyJsonRemoteFile = exports.LegacyJsonLocalFile = exports.LegacyPrismaRemoteFile = exports.LegacyPrismaLocalFile = exports.LegacySqlRemoteFile = exports.LegacySqlLocalFile = exports.LegacyDatabaseConnection = exports.DatabaseUrlStorage = exports.LegacyProjectTokenId = exports.LegacyZoomLevel = exports.LegacyLayoutName = exports.LegacyMemoId = exports.LegacyTableRowId = exports.LegacyLineIndex = exports.LegacyLine = exports.LegacyColumnPathStr = exports.LegacySourceName = exports.LegacySourceId = exports.LegacyProjectName = exports.LegacyProjectSlug = exports.LegacyProjectId = void 0;
		exports.LegacyProjectInfo = exports.LegacyProjectInfoRemote = exports.LegacyProjectInfoLocal = exports.LegacyProjectStats = exports.LegacyProjectJson = exports.LegacyProject = exports.LegacyProjectVersion = exports.LegacyProjectVisibility = exports.LegacyProjectStorage = exports.LegacySettings = exports.LegacyOrganization = exports.LegacyHerokuResource = exports.LegacyCleverCloudResource = exports.LegacyHerokuId = exports.LegacyCleverCloudId = exports.LegacyPlan = exports.LegacyPlanId = exports.LegacyOrganizationName = exports.LegacyOrganizationSlug = exports.LegacyOrganizationId = exports.LegacyLayout = exports.LegacyGroup = exports.LegacyTableRow = exports.LegacyTableRowState = void 0;
		exports.legacyIsLocal = legacyIsLocal;
		exports.legacyIsRemote = legacyIsRemote;
		exports.legacyParseTableId = legacyParseTableId;
		exports.legacyBuildProjectDraft = legacyBuildProjectDraft;
		exports.legacyBuildProjectLocal = legacyBuildProjectLocal;
		exports.legacyBuildProjectRemote = legacyBuildProjectRemote;
		exports.legacyBuildProjectJson = legacyBuildProjectJson;
		exports.legacyComputeStats = legacyComputeStats;
		exports.sourceToDatabase = sourceToDatabase;
		exports.projectTableToEntity = projectTableToEntity;
		exports.projectColumnToAttribute = projectColumnToAttribute;
		exports.projectTypeFromLegacy = projectTypeFromLegacy;
		const zod_1 = /*@__PURE__*/ requireLib();
		const utils_1 = requireOut$1();
		const common_1 = requireCommon();
		const legacyDatabase_1 = requireLegacyDatabase();
		const zod_2 = requireZod();
		const database_1 = requireDatabase();
		const databaseUrl_1 = requireDatabaseUrl();
		const llm_1 = requireLlm();
		exports.LegacyProjectId = common_1.Uuid;
		exports.LegacyProjectSlug = common_1.Slug;
		exports.LegacyProjectName = zod_1.z.string();
		exports.LegacySourceId = common_1.Uuid;
		exports.LegacySourceName = zod_1.z.string();
		exports.LegacyColumnPathStr = zod_1.z.string();
		exports.LegacyLine = zod_1.z.string();
		exports.LegacyLineIndex = zod_1.z.number();
		exports.LegacyTableRowId = zod_1.z.number();
		exports.LegacyMemoId = zod_1.z.number();
		exports.LegacyLayoutName = zod_1.z.string();
		exports.LegacyZoomLevel = zod_1.z.number();
		exports.LegacyProjectTokenId = common_1.Uuid;
		exports.DatabaseUrlStorage = zod_1.z.enum(['memory', 'browser', 'project']);
		exports.LegacyDatabaseConnection = zod_1.z.object({
		    kind: zod_1.z.literal('DatabaseConnection'),
		    engine: database_1.DatabaseKind.optional(),
		    url: zod_1.z.string().optional(),
		    storage: exports.DatabaseUrlStorage.optional(),
		}).strict();
		exports.LegacySqlLocalFile = zod_1.z.object({
		    kind: zod_1.z.literal('SqlLocalFile'),
		    name: zod_1.z.string(),
		    size: zod_1.z.number(),
		    modified: common_1.Timestamp
		}).strict();
		exports.LegacySqlRemoteFile = zod_1.z.object({
		    kind: zod_1.z.literal('SqlRemoteFile'),
		    url: zod_1.z.string(),
		    size: zod_1.z.number()
		}).strict();
		exports.LegacyPrismaLocalFile = zod_1.z.object({
		    kind: zod_1.z.literal('PrismaLocalFile'),
		    name: zod_1.z.string(),
		    size: zod_1.z.number(),
		    modified: common_1.Timestamp
		}).strict();
		exports.LegacyPrismaRemoteFile = zod_1.z.object({
		    kind: zod_1.z.literal('PrismaRemoteFile'),
		    url: zod_1.z.string(),
		    size: zod_1.z.number()
		}).strict();
		exports.LegacyJsonLocalFile = zod_1.z.object({
		    kind: zod_1.z.literal('JsonLocalFile'),
		    name: zod_1.z.string(),
		    size: zod_1.z.number(),
		    modified: common_1.Timestamp
		}).strict();
		exports.LegacyJsonRemoteFile = zod_1.z.object({
		    kind: zod_1.z.literal('JsonRemoteFile'),
		    url: zod_1.z.string(),
		    size: zod_1.z.number()
		}).strict();
		exports.LegacyAmlEditor = zod_1.z.object({
		    kind: zod_1.z.literal('AmlEditor')
		}).strict();
		exports.LegacySourceKind = zod_1.z.discriminatedUnion('kind', [exports.LegacyDatabaseConnection, exports.LegacySqlLocalFile, exports.LegacySqlRemoteFile, exports.LegacyPrismaLocalFile, exports.LegacyPrismaRemoteFile, exports.LegacyJsonLocalFile, exports.LegacyJsonRemoteFile, exports.LegacyAmlEditor]);
		exports.LegacySourceOrigin = zod_1.z.enum(['import-project', 'sql-source', 'prisma-source', 'json-source']);
		exports.LegacyOrigin = zod_1.z.object({
		    id: exports.LegacySourceId,
		    lines: exports.LegacyLineIndex.array().optional()
		}).strict();
		exports.LegacyComment = zod_1.z.object({
		    text: zod_1.z.string(),
		    origins: exports.LegacyOrigin.array().optional()
		}).strict();
		exports.LegacyProjectColumn = zod_1.z.object({
		    name: legacyDatabase_1.LegacyColumnName,
		    type: legacyDatabase_1.LegacyColumnType,
		    nullable: zod_1.z.boolean().optional(),
		    default: zod_1.z.string().optional(),
		    comment: exports.LegacyComment.optional(),
		    values: zod_1.z.string().array().optional(),
		    columns: zod_1.z.lazy(() => exports.LegacyProjectColumn.array().optional()),
		    stats: legacyDatabase_1.LegacyColumnDbStats.optional(),
		    origins: exports.LegacyOrigin.array().optional()
		}).strict();
		exports.LegacyProjectPrimaryKey = zod_1.z.object({
		    name: zod_1.z.string().optional(),
		    columns: legacyDatabase_1.LegacyColumnName.array(),
		    origins: exports.LegacyOrigin.array().optional()
		}).strict();
		exports.LegacyProjectUnique = zod_1.z.object({
		    name: zod_1.z.string(),
		    columns: legacyDatabase_1.LegacyColumnName.array(),
		    definition: zod_1.z.string().optional(),
		    origins: exports.LegacyOrigin.array().optional()
		}).strict();
		exports.LegacyProjectIndex = zod_1.z.object({
		    name: zod_1.z.string(),
		    columns: legacyDatabase_1.LegacyColumnName.array(),
		    definition: zod_1.z.string().optional(),
		    origins: exports.LegacyOrigin.array().optional()
		}).strict();
		exports.LegacyProjectCheck = zod_1.z.object({
		    name: zod_1.z.string(),
		    columns: legacyDatabase_1.LegacyColumnName.array(),
		    predicate: zod_1.z.string().optional(),
		    origins: exports.LegacyOrigin.array().optional()
		}).strict();
		exports.LegacyProjectTable = zod_1.z.object({
		    schema: legacyDatabase_1.LegacySchemaName,
		    table: legacyDatabase_1.LegacyTableName,
		    view: zod_1.z.boolean().optional(),
		    definition: zod_1.z.string().optional(),
		    columns: exports.LegacyProjectColumn.array(),
		    primaryKey: exports.LegacyProjectPrimaryKey.optional(),
		    uniques: exports.LegacyProjectUnique.array().optional(),
		    indexes: exports.LegacyProjectIndex.array().optional(),
		    checks: exports.LegacyProjectCheck.array().optional(),
		    comment: exports.LegacyComment.optional(),
		    stats: legacyDatabase_1.LegacyTableDbStats.optional(),
		    origins: exports.LegacyOrigin.array().optional()
		}).strict();
		exports.LegacyProjectColumnRef = zod_1.z.object({
		    table: legacyDatabase_1.LegacyTableId,
		    column: legacyDatabase_1.LegacyColumnName
		}).strict();
		exports.LegacyProjectRelation = zod_1.z.object({
		    name: legacyDatabase_1.LegacyRelationName,
		    src: exports.LegacyProjectColumnRef,
		    ref: exports.LegacyProjectColumnRef,
		    origins: exports.LegacyOrigin.array().optional()
		}).strict();
		exports.LegacyProjectType = zod_1.z.object({
		    schema: legacyDatabase_1.LegacySchemaName,
		    name: legacyDatabase_1.LegacyTypeName,
		    value: zod_1.z.union([zod_1.z.object({ enum: zod_1.z.string().array() }).strict(), zod_1.z.object({ definition: zod_1.z.string() }).strict()]),
		    origins: exports.LegacyOrigin.array().optional()
		}).strict();
		exports.LegacySource = zod_1.z.object({
		    id: exports.LegacySourceId,
		    name: exports.LegacySourceName,
		    kind: exports.LegacySourceKind,
		    content: exports.LegacyLine.array(),
		    tables: exports.LegacyProjectTable.array(),
		    relations: exports.LegacyProjectRelation.array(),
		    types: exports.LegacyProjectType.array().optional(),
		    enabled: zod_1.z.boolean().optional(),
		    fromSample: zod_1.z.string().optional(),
		    createdAt: common_1.Timestamp,
		    updatedAt: common_1.Timestamp
		}).strict();
		exports.LegacyCanvasProps = zod_1.z.object({
		    position: common_1.Position,
		    zoom: exports.LegacyZoomLevel
		}).strict();
		exports.LegacyTableProps = zod_1.z.object({
		    id: legacyDatabase_1.LegacyTableId,
		    position: common_1.Position,
		    size: common_1.Size,
		    color: common_1.Color,
		    columns: legacyDatabase_1.LegacyColumnName.array(),
		    selected: zod_1.z.boolean().optional(),
		    collapsed: zod_1.z.boolean().optional(),
		    hiddenColumns: zod_1.z.boolean().optional()
		}).strict();
		exports.LegacyNotes = zod_1.z.string();
		exports.LegacyTag = zod_1.z.string();
		exports.LegacyColumnMeta = zod_1.z.object({
		    notes: exports.LegacyNotes.optional(),
		    tags: exports.LegacyTag.array().optional()
		}).strict();
		exports.LegacyTableMeta = zod_1.z.object({
		    notes: exports.LegacyNotes.optional(),
		    tags: exports.LegacyTag.array().optional(),
		    columns: zod_1.z.record(exports.LegacyColumnPathStr, exports.LegacyColumnMeta)
		}).strict();
		exports.LegacyMemo = zod_1.z.object({
		    id: exports.LegacyMemoId,
		    content: zod_1.z.string(),
		    position: common_1.Position,
		    size: common_1.Size,
		    color: common_1.Color.optional(),
		    selected: zod_1.z.boolean().optional()
		}).strict();
		exports.LegacySqlQueryOrigin = zod_1.z.object({ sql: zod_1.z.string(), origin: zod_1.z.string(), db: zod_1.z.string() }).strict();
		exports.LegacyRowValue = zod_1.z.object({
		    column: exports.LegacyColumnPathStr,
		    value: legacyDatabase_1.LegacyJsValue
		}).strict();
		exports.LegacyRowPrimaryKey = exports.LegacyRowValue.array();
		exports.LegacyTableRowColumn = zod_1.z.object({
		    path: exports.LegacyColumnPathStr,
		    value: legacyDatabase_1.LegacyJsValue,
		    linkedBy: zod_1.z.record(legacyDatabase_1.LegacyTableId, exports.LegacyRowPrimaryKey.array()).optional()
		}).strict();
		exports.LegacyTableRowStateSuccess = zod_1.z.object({
		    columns: exports.LegacyTableRowColumn.array(),
		    startedAt: common_1.Timestamp,
		    loadedAt: common_1.Timestamp
		}).strict();
		exports.LegacyTableRowStateFailure = zod_1.z.object({
		    query: exports.LegacySqlQueryOrigin,
		    error: zod_1.z.string(),
		    startedAt: common_1.Timestamp,
		    failedAt: common_1.Timestamp
		}).strict();
		exports.LegacyTableRowStateLoading = zod_1.z.object({
		    query: exports.LegacySqlQueryOrigin,
		    startedAt: common_1.Timestamp
		}).strict();
		exports.LegacyTableRowState = zod_1.z.union([exports.LegacyTableRowStateSuccess, exports.LegacyTableRowStateFailure, exports.LegacyTableRowStateLoading]);
		exports.LegacyTableRow = zod_1.z.object({
		    id: exports.LegacyTableRowId,
		    position: common_1.Position,
		    size: common_1.Size,
		    source: exports.LegacySourceId,
		    table: legacyDatabase_1.LegacyTableId,
		    primaryKey: exports.LegacyRowPrimaryKey,
		    state: exports.LegacyTableRowState,
		    hidden: exports.LegacyColumnPathStr.array().optional(),
		    showHiddenColumns: zod_1.z.boolean().optional(),
		    selected: zod_1.z.boolean().optional(),
		    collapsed: zod_1.z.boolean().optional()
		}).strict();
		exports.LegacyGroup = zod_1.z.object({
		    name: zod_1.z.string(),
		    tables: legacyDatabase_1.LegacyTableId.array(),
		    color: common_1.Color,
		    collapsed: zod_1.z.boolean().optional()
		}).strict();
		exports.LegacyLayout = zod_1.z.object({
		    canvas: exports.LegacyCanvasProps.optional(),
		    tables: exports.LegacyTableProps.array(),
		    tableRows: exports.LegacyTableRow.array().optional(),
		    groups: exports.LegacyGroup.array().optional(),
		    memos: exports.LegacyMemo.array().optional(),
		    createdAt: common_1.Timestamp,
		    updatedAt: common_1.Timestamp
		}).strict();
		exports.LegacyOrganizationId = common_1.Uuid;
		exports.LegacyOrganizationSlug = common_1.Slug;
		exports.LegacyOrganizationName = zod_1.z.string();
		exports.LegacyPlanId = zod_1.z.enum(['free', 'solo', 'team', 'enterprise', 'pro']);
		exports.LegacyPlan = zod_1.z.object({
		    id: exports.LegacyPlanId,
		    name: zod_1.z.string(),
		    data_exploration: zod_1.z.boolean(),
		    colors: zod_1.z.boolean(),
		    aml: zod_1.z.number().nullable(),
		    schema_export: zod_1.z.boolean(),
		    ai: zod_1.z.boolean(),
		    analysis: zod_1.z.string(),
		    project_export: zod_1.z.boolean(),
		    projects: zod_1.z.number().nullable(),
		    project_dbs: zod_1.z.number().nullable(),
		    project_layouts: zod_1.z.number().nullable(),
		    layout_tables: zod_1.z.number().nullable(),
		    project_doc: zod_1.z.number().nullable(),
		    project_share: zod_1.z.boolean(),
		    streak: zod_1.z.number(),
		}).strict();
		exports.LegacyCleverCloudId = common_1.Uuid;
		exports.LegacyHerokuId = common_1.Uuid;
		exports.LegacyCleverCloudResource = zod_1.z.object({ id: exports.LegacyCleverCloudId }).strict();
		exports.LegacyHerokuResource = zod_1.z.object({ id: exports.LegacyHerokuId }).strict();
		exports.LegacyOrganization = zod_1.z.object({
		    id: exports.LegacyOrganizationId,
		    slug: exports.LegacyOrganizationSlug,
		    name: exports.LegacyOrganizationName,
		    plan: exports.LegacyPlan,
		    logo: zod_1.z.string().url(),
		    description: zod_1.z.string().optional(),
		    clever_cloud: exports.LegacyCleverCloudResource.optional(),
		    heroku: exports.LegacyHerokuResource.optional(),
		}).strict();
		exports.LegacySettings = zod_1.z.object({
		    findPath: zod_1.z.object({
		        maxPathLength: zod_1.z.number().optional(),
		        ignoredTables: zod_1.z.string().optional(),
		        ignoredColumns: zod_1.z.string().optional()
		    }).strict().optional(),
		    defaultSchema: legacyDatabase_1.LegacySchemaName.optional(),
		    removedSchemas: legacyDatabase_1.LegacySchemaName.array().optional(),
		    removeViews: zod_1.z.boolean().optional(),
		    removedTables: zod_1.z.string().optional(),
		    hiddenColumns: zod_1.z.object({
		        list: zod_1.z.string().optional(),
		        max: zod_1.z.number().optional(),
		        props: zod_1.z.boolean().optional(),
		        relations: zod_1.z.boolean().optional()
		    }).strict().optional(),
		    columnOrder: zod_1.z.enum(['sql', 'property', 'name', 'type']).optional(),
		    relationStyle: zod_1.z.enum(['Bezier', 'Straight', 'Steps']).optional(),
		    columnBasicTypes: zod_1.z.boolean().optional(),
		    collapseTableColumns: zod_1.z.boolean().optional(),
		    llm: zod_1.z.object({
		        key: llm_1.OpenAIKey,
		        model: llm_1.OpenAIModel,
		    }).strict().optional(),
		}).strict();
		exports.LegacyProjectStorage = zod_1.z.enum(['local', 'remote']);
		exports.LegacyProjectVisibility = zod_1.z.enum(['none', 'read', 'write']);
		exports.LegacyProjectVersion = zod_1.z.union([zod_1.z.literal(1), zod_1.z.literal(2)]);
		exports.LegacyProject = zod_1.z.object({
		    organization: exports.LegacyOrganization.optional(),
		    id: exports.LegacyProjectId,
		    slug: exports.LegacyProjectSlug,
		    name: exports.LegacyProjectName,
		    description: zod_1.z.string().optional(),
		    sources: exports.LegacySource.array(),
		    ignoredRelations: zod_1.z.record(exports.LegacyColumnPathStr.array()).optional(),
		    notes: zod_1.z.record(zod_1.z.string()).optional(),
		    metadata: zod_1.z.record(legacyDatabase_1.LegacyTableId, exports.LegacyTableMeta).optional(),
		    usedLayout: exports.LegacyLayoutName.optional(),
		    layouts: zod_1.z.record(exports.LegacyLayoutName, exports.LegacyLayout),
		    tableRowsSeq: zod_1.z.number().optional(),
		    settings: exports.LegacySettings.optional(),
		    storage: exports.LegacyProjectStorage,
		    visibility: exports.LegacyProjectVisibility,
		    createdAt: common_1.Timestamp,
		    updatedAt: common_1.Timestamp,
		    version: exports.LegacyProjectVersion
		}).strict().describe('LegacyProject');
		exports.LegacyProjectJson = exports.LegacyProject.omit({ organization: true, id: true, storage: true, visibility: true, createdAt: true, updatedAt: true }).extend({ _type: zod_1.z.literal('json') }).strict().describe('LegacyProjectJson');
		exports.LegacyProjectStats = zod_1.z.object({
		    nbSources: zod_1.z.number(),
		    nbTables: zod_1.z.number(),
		    nbColumns: zod_1.z.number(),
		    nbRelations: zod_1.z.number(),
		    nbTypes: zod_1.z.number(),
		    nbComments: zod_1.z.number(),
		    nbLayouts: zod_1.z.number(),
		    nbNotes: zod_1.z.number(),
		    nbMemos: zod_1.z.number()
		}).strict().describe('LegacyProjectStats');
		exports.LegacyProjectInfoLocal = exports.LegacyProjectStats.extend({
		    organization: exports.LegacyOrganization.optional(),
		    id: exports.LegacyProjectId,
		    slug: exports.LegacyProjectSlug,
		    name: exports.LegacyProjectName,
		    description: zod_1.z.string().optional(),
		    storage: zod_1.z.literal(exports.LegacyProjectStorage.enum.local),
		    visibility: exports.LegacyProjectVisibility,
		    encodingVersion: exports.LegacyProjectVersion,
		    createdAt: common_1.Timestamp,
		    updatedAt: common_1.Timestamp
		}).strict();
		exports.LegacyProjectInfoRemote = exports.LegacyProjectInfoLocal.omit({ storage: true }).extend({ storage: zod_1.z.literal(exports.LegacyProjectStorage.enum.remote) }).strict();
		exports.LegacyProjectInfo = zod_1.z.discriminatedUnion('storage', [exports.LegacyProjectInfoLocal, exports.LegacyProjectInfoRemote]);
		function legacyIsLocal(p) {
		    return p.storage === exports.LegacyProjectStorage.enum.local;
		}
		function legacyIsRemote(p) {
		    return p.storage === exports.LegacyProjectStorage.enum.remote;
		}
		function legacyParseTableId(id) {
		    const [schema, table] = id.split(".");
		    return table === undefined ? { schema: "", table: schema } : { schema, table };
		}
		function legacyBuildProjectDraft(id, { _type, ...p }) {
		    return (0, zod_2.zodParse)(exports.LegacyProject)({
		        ...p,
		        id,
		        slug: id,
		        storage: exports.LegacyProjectStorage.enum.local,
		        visibility: exports.LegacyProjectVisibility.enum.none,
		        createdAt: Date.now(),
		        updatedAt: Date.now()
		    }).getOrThrow();
		}
		function legacyBuildProjectLocal(info, { _type, ...p }) {
		    return (0, zod_2.zodParse)(exports.LegacyProject)({
		        ...p,
		        organization: info.organization,
		        id: info.id,
		        storage: exports.LegacyProjectStorage.enum.local,
		        visibility: exports.LegacyProjectVisibility.enum.none,
		        createdAt: info.createdAt,
		        updatedAt: info.updatedAt
		    }).getOrThrow();
		}
		function legacyBuildProjectRemote(info, { _type, ...p }) {
		    return (0, zod_2.zodParse)(exports.LegacyProject)({
		        ...p,
		        organization: info.organization,
		        id: info.id,
		        slug: info.slug,
		        storage: exports.LegacyProjectStorage.enum.remote,
		        visibility: info.visibility,
		        createdAt: info.createdAt,
		        updatedAt: info.updatedAt
		    }).getOrThrow();
		}
		function legacyBuildProjectJson({ organization, id, storage, visibility, createdAt, updatedAt, ...p }) {
		    return (0, zod_2.zodParse)(exports.LegacyProjectJson)({ ...p, _type: 'json' }).getOrThrow();
		}
		function legacyComputeStats(p) {
		    // should be the same as `tables`, `relations` and `types` in src/Models/Project.elm
		    const tables = (0, utils_1.groupBy)(p.sources.flatMap(s => s.tables), t => `${t.schema}.${t.table}`);
		    const relations = (0, utils_1.groupBy)(p.sources.flatMap(s => s.relations), r => `${r.src.table}.${r.src.column}->${r.ref.table}.${r.ref.column}`);
		    const types = (0, utils_1.groupBy)(p.sources.flatMap(s => s.types || []), t => `${t.schema}.${t.name}`);
		    return (0, zod_2.zodParse)(exports.LegacyProjectStats)({
		        nbSources: p.sources.length,
		        nbTables: Object.keys(tables).length,
		        nbColumns: Object.values(tables).map(same => Math.max(...same.map(t => t.columns.length))).reduce((acc, cols) => acc + cols, 0),
		        nbRelations: Object.keys(relations).length,
		        nbTypes: Object.keys(types).length,
		        nbComments: p.sources.flatMap(s => s.tables.flatMap(t => [t.comment].concat(t.columns.map(c => c.comment)).filter(c => !!c))).length,
		        nbLayouts: Object.keys(p.layouts).length,
		        nbNotes: Object.keys(p.notes || {}).length,
		        nbMemos: Object.values(p.layouts).flatMap(l => l.memos || []).length,
		    }).getOrThrow();
		}
		function sourceToDatabase(s) {
		    return (0, utils_1.removeEmpty)({
		        entities: s.tables.map(projectTableToEntity),
		        relations: s.relations.map(legacyDatabase_1.relationFromLegacy),
		        types: s.types?.map(projectTypeFromLegacy),
		        stats: (0, utils_1.removeUndefined)({
		            name: s.name,
		            kind: s.kind.kind === 'DatabaseConnection' ? (s.kind.engine ? s.kind.engine : s.kind.url ? (0, databaseUrl_1.parseDatabaseUrl)(s.kind.url).kind : undefined) : undefined
		        })
		    });
		}
		function projectTableToEntity(t) {
		    return (0, utils_1.removeUndefined)({
		        database: undefined,
		        catalog: undefined,
		        schema: t.schema || undefined,
		        name: t.table,
		        kind: t.view ? 'view' : undefined,
		        def: t.definition,
		        attrs: t.columns.map(projectColumnToAttribute),
		        pk: t.primaryKey ? (0, legacyDatabase_1.primaryKeyFromLegacy)(t.primaryKey) : undefined,
		        indexes: (t.uniques || []).map(legacyDatabase_1.uniqueFromLegacy).concat((t.indexes || []).map(legacyDatabase_1.indexFromLegacy)),
		        checks: t.checks?.map(legacyDatabase_1.checkFromLegacy),
		        doc: t.comment?.text,
		        stats: t.stats ? (0, legacyDatabase_1.tableDbStatsFromLegacy)(t.stats) : undefined,
		        extra: undefined,
		    });
		}
		function projectColumnToAttribute(c) {
		    return (0, utils_1.removeEmpty)({
		        name: c.name,
		        type: c.type,
		        null: c.nullable,
		        gen: undefined,
		        default: c.default,
		        attrs: c.columns?.map(projectColumnToAttribute),
		        doc: c.comment?.text,
		        stats: (0, utils_1.removeUndefined)({
		            nulls: c.stats?.nulls,
		            bytesAvg: c.stats?.bytesAvg,
		            cardinality: c.stats?.cardinality,
		            commonValues: c.stats?.commonValues?.map(v => ({ value: (0, legacyDatabase_1.columnValueFromLegacy)(v.value), freq: v.freq })),
		            distinctValues: c.values?.map(legacyDatabase_1.columnValueFromLegacy),
		            histogram: c.stats?.histogram?.map(legacyDatabase_1.columnValueFromLegacy),
		            min: undefined,
		            max: undefined,
		        })
		    });
		}
		function projectTypeFromLegacy(t) {
		    if ('enum' in t.value) {
		        return (0, utils_1.removeUndefined)({ schema: t.schema || undefined, name: t.name, values: t.value.enum || undefined });
		    }
		    else {
		        return (0, utils_1.removeUndefined)({ schema: t.schema || undefined, name: t.name, definition: t.value.definition });
		    }
		}
		
	} (legacyProject));
	return legacyProject;
}

var legacyQuery = {};

var hasRequiredLegacyQuery;

function requireLegacyQuery () {
	if (hasRequiredLegacyQuery) return legacyQuery;
	hasRequiredLegacyQuery = 1;
	(function (exports) {
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.LegacyDatabaseQueryResults = exports.LegacyDatabaseQueryResultsColumn = void 0;
		exports.queryResultsFromLegacy = queryResultsFromLegacy;
		exports.queryResultsToLegacy = queryResultsToLegacy;
		const zod_1 = /*@__PURE__*/ requireLib();
		const utils_1 = requireOut$1();
		const legacyDatabase_1 = requireLegacyDatabase();
		exports.LegacyDatabaseQueryResultsColumn = zod_1.z.object({
		    name: zod_1.z.string(),
		    ref: legacyDatabase_1.LegacyColumnRef.optional(),
		}).strict();
		exports.LegacyDatabaseQueryResults = zod_1.z.object({
		    query: zod_1.z.string(),
		    columns: exports.LegacyDatabaseQueryResultsColumn.array(),
		    rows: legacyDatabase_1.LegacyJsValue.array(),
		}).strict().describe('LegacyDatabaseQueryResults');
		function queryResultsFromLegacy(r) {
		    return {
		        query: r.query,
		        attributes: r.columns.map(({ name, ref }) => (0, utils_1.removeUndefined)({
		            name,
		            ref: ref ? (0, legacyDatabase_1.columnRefFromLegacy)(ref) : undefined
		        })),
		        rows: r.rows
		    };
		}
		function queryResultsToLegacy(r) {
		    return {
		        query: r.query,
		        columns: r.attributes.map(({ name, ref }) => (0, utils_1.removeUndefined)({
		            name,
		            ref: ref ? (0, legacyDatabase_1.columnRefToLegacy)(ref) : undefined
		        })),
		        rows: r.rows.map(jsValueToLegacy)
		    };
		}
		function jsValueToLegacy(v) {
		    return v;
		}
		
	} (legacyQuery));
	return legacyQuery;
}

var legacyStats = {};

var hasRequiredLegacyStats;

function requireLegacyStats () {
	if (hasRequiredLegacyStats) return legacyStats;
	hasRequiredLegacyStats = 1;
	(function (exports) {
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.LegacyColumnStats = exports.LegacyColumnCommonValue = exports.LegacyTableStats = exports.LegacyTableSampleValues = void 0;
		exports.tableStatsFromLegacy = tableStatsFromLegacy;
		exports.tableStatsToLegacy = tableStatsToLegacy;
		exports.columnStatsFromLegacy = columnStatsFromLegacy;
		exports.columnStatsToLegacy = columnStatsToLegacy;
		const zod_1 = /*@__PURE__*/ requireLib();
		const utils_1 = requireOut$1();
		const legacyDatabase_1 = requireLegacyDatabase();
		exports.LegacyTableSampleValues = zod_1.z.record(legacyDatabase_1.LegacyColumnValue);
		// keep sync with frontend/src/Models/Project/TableStats.elm
		exports.LegacyTableStats = zod_1.z.object({
		    schema: legacyDatabase_1.LegacySchemaName.nullable(),
		    table: legacyDatabase_1.LegacyTableName,
		    rows: zod_1.z.number(),
		    sample_values: exports.LegacyTableSampleValues
		}).strict().describe('LegacyTableStats');
		function tableStatsFromLegacy(s) {
		    return (0, utils_1.removeUndefined)({
		        schema: s.schema || undefined,
		        entity: s.table,
		        rows: s.rows,
		        sampleValues: (0, utils_1.mapValues)(s.sample_values, legacyDatabase_1.columnValueFromLegacy)
		    });
		}
		function tableStatsToLegacy(s) {
		    return {
		        schema: s.schema || null,
		        table: s.entity,
		        rows: s.rows,
		        sample_values: (0, utils_1.mapValues)(s.sampleValues, legacyDatabase_1.columnValueToLegacy)
		    };
		}
		exports.LegacyColumnCommonValue = zod_1.z.object({ value: legacyDatabase_1.LegacyColumnValue, count: zod_1.z.number() });
		// keep sync with frontend/src/Models/Project/ColumnStats.elm
		exports.LegacyColumnStats = zod_1.z.object({
		    schema: legacyDatabase_1.LegacySchemaName.nullable(),
		    table: legacyDatabase_1.LegacyTableName,
		    column: legacyDatabase_1.LegacyColumnName,
		    type: legacyDatabase_1.LegacyColumnType,
		    rows: zod_1.z.number(),
		    nulls: zod_1.z.number(),
		    cardinality: zod_1.z.number(),
		    common_values: exports.LegacyColumnCommonValue.array()
		}).strict().describe('LegacyColumnStats');
		function columnStatsFromLegacy(s) {
		    return {
		        schema: s.schema || undefined,
		        entity: s.table,
		        attribute: [s.column],
		        type: s.type,
		        rows: s.rows,
		        nulls: s.nulls,
		        cardinality: s.cardinality,
		        commonValues: s.common_values.map(({ value, count }) => ({ value: (0, legacyDatabase_1.columnValueFromLegacy)(value), count }))
		    };
		}
		function columnStatsToLegacy(s) {
		    return {
		        schema: s.schema || null,
		        table: s.entity,
		        column: s.attribute.join(legacyDatabase_1.legacyColumnPathSeparator),
		        type: s.type,
		        rows: s.rows,
		        nulls: s.nulls,
		        cardinality: s.cardinality,
		        common_values: s.commonValues.map(({ value, count }) => ({ value: (0, legacyDatabase_1.columnValueToLegacy)(value), count }))
		    };
		}
		
	} (legacyStats));
	return legacyStats;
}

var project = {};

var hasRequiredProject;

function requireProject () {
	if (hasRequiredProject) return project;
	hasRequiredProject = 1;
	(function (exports) {
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.ProjectUserStats = exports.Project = exports.ProjectStats = exports.EntityDoc = exports.AttributeDoc = exports.UserPropertyValue = exports.UserProperty = exports.UserPropertyKind = exports.UserPropertyName = exports.Tag = exports.TagName = exports.Layout = exports.LayoutItem = exports.Source = exports.LayoutName = exports.SourceKind = exports.SourceName = exports.SourceId = exports.ProjectName = exports.ProjectSlug = exports.ProjectId = void 0;
		const zod_1 = /*@__PURE__*/ requireLib();
		const common_1 = requireCommon();
		const database_1 = requireDatabase();
		// FIXME: Work In Progress
		// read this file from bottom to the top, to have a top-down read ^^
		exports.ProjectId = common_1.Uuid;
		exports.ProjectSlug = zod_1.z.string();
		exports.ProjectName = zod_1.z.string();
		exports.SourceId = common_1.Uuid;
		exports.SourceName = zod_1.z.string();
		exports.SourceKind = zod_1.z.enum(['Connection', 'SQL', 'AML', 'DBML', 'Prisma', 'JSON']);
		exports.LayoutName = zod_1.z.string();
		// SourceAttribute: firstSeen
		// SourceEntity: firstSeen
		exports.Source = zod_1.z.object({
		    id: exports.SourceId,
		    name: exports.SourceName,
		    kind: exports.SourceKind,
		    engine: database_1.DatabaseKind.optional(),
		    url: zod_1.z.string().optional(), // db url, file url or file path
		    database: database_1.Database,
		    content: zod_1.z.string().optional(),
		    inferredRelations: database_1.Relation.array().optional(),
		    ignoredRelations: database_1.Relation.array().optional(),
		    addedRelations: database_1.Relation.array().optional(),
		    namespaceMappings: zod_1.z.record(database_1.NamespaceId, database_1.NamespaceId).optional(), // change namespaces when merging sources together
		    enabled: zod_1.z.boolean().optional(), // TODO: move to layout
		    createdAt: zod_1.z.date(),
		    updatedAt: zod_1.z.date(),
		}).strict();
		/*
		 * Layout part
		 */
		// entity, row, note...
		exports.LayoutItem = zod_1.z.object({});
		exports.Layout = zod_1.z.object({
		    name: exports.LayoutName,
		    items: exports.LayoutItem.array(),
		    itemSeq: zod_1.z.number().optional()
		    // groups
		}).strict();
		/*
		 * Documentation part
		 */
		exports.TagName = zod_1.z.string();
		exports.Tag = zod_1.z.object({
		    name: zod_1.z.string(),
		    description: zod_1.z.string().optional(),
		    color: common_1.Color
		}).strict();
		// user properties allows to fill structured documentation, for example: ownership, lineage, data quality, expiration, link to resources...
		exports.UserPropertyName = zod_1.z.string();
		exports.UserPropertyKind = zod_1.z.enum([
		    'string', // single line text
		    'text', // multiline text, formatted with markdown
		    'enum', // pre-defined set of values
		    'url',
		    'email',
		    'phone',
		    'date',
		    'bool', // checkbox
		    'number',
		    'rating', // 5 stars rating
		    'progress', // 0-100% progress bar
		    'user', // user uuid
		    'entity', // entity ref
		    'attribute', // attribute ref
		    'row' // row ref (table with primary key)
		]);
		exports.UserProperty = zod_1.z.object({
		    name: exports.UserPropertyName,
		    description: zod_1.z.string().optional(),
		    // icon: z.enum(['???']).optional(),
		    kind: exports.UserPropertyKind,
		    on: zod_1.z.enum(['entity', 'attribute']).array(),
		    multi: zod_1.z.boolean().optional(),
		    enum_values: zod_1.z.object({
		        name: zod_1.z.string(),
		        description: zod_1.z.string().optional(),
		        color: common_1.Color
		    }).strict().array().optional(),
		    string_values: zod_1.z.string().array().optional(),
		    validation_regex: zod_1.z.string().optional()
		}).strict();
		exports.UserPropertyValue = zod_1.z.union([zod_1.z.string(), zod_1.z.string().array()]);
		exports.AttributeDoc = zod_1.z.object({
		    defaultShow: zod_1.z.boolean().optional(), // true when not specified
		    defaultPos: zod_1.z.number().optional(),
		    alias: zod_1.z.string().optional(),
		    doc: common_1.Markdown.optional(),
		    tags: exports.TagName.array().optional(),
		    props: zod_1.z.record(exports.UserPropertyName, exports.UserPropertyValue).optional()
		}).strict();
		exports.EntityDoc = zod_1.z.object({
		    alias: zod_1.z.string().optional(),
		    doc: common_1.Markdown.optional(),
		    tags: exports.TagName.array().optional(),
		    props: zod_1.z.record(exports.UserPropertyName, exports.UserPropertyValue).optional(),
		    attrs: zod_1.z.record(database_1.AttributePathId, exports.AttributeDoc).optional(),
		}).strict();
		exports.ProjectStats = zod_1.z.object({
		    shownEntities: zod_1.z.record(database_1.EntityId, zod_1.z.number()).optional(), // get rough idea of entity popularity
		    shownLayouts: zod_1.z.record(exports.LayoutName, zod_1.z.number()).optional(), // get rough idea of layout popularity
		}).strict();
		exports.Project = zod_1.z.object({
		    id: exports.ProjectId,
		    slug: exports.ProjectSlug,
		    name: exports.ProjectName,
		    // storage
		    // version
		    sources: exports.Source.array(),
		    layouts: zod_1.z.record(exports.LayoutName, exports.Layout.array()),
		    metadata: zod_1.z.record(database_1.EntityId, exports.EntityDoc).optional(),
		    // queries
		    stats: exports.ProjectStats.optional(),
		    // createdAt
		    // updatedAt
		}).strict();
		// stored only in the user browser
		exports.ProjectUserStats = zod_1.z.object({
		    shownEntities: zod_1.z.record(database_1.EntityId, zod_1.z.number()).optional(),
		    shownLayouts: zod_1.z.record(exports.LayoutName, zod_1.z.number()).optional(),
		    queryHistory: zod_1.z.object({
		        source: exports.SourceId,
		        query: zod_1.z.string(),
		        ranAt: zod_1.z.date(),
		    }).array().optional(),
		}).strict();
		
	} (project));
	return project;
}

var hasRequiredOut;

function requireOut () {
	if (hasRequiredOut) return out$1;
	hasRequiredOut = 1;
	(function (exports) {
		var __createBinding = (out$1 && out$1.__createBinding) || (Object.create ? (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    var desc = Object.getOwnPropertyDescriptor(m, k);
		    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
		      desc = { enumerable: true, get: function() { return m[k]; } };
		    }
		    Object.defineProperty(o, k2, desc);
		}) : (function(o, m, k, k2) {
		    if (k2 === undefined) k2 = k;
		    o[k2] = m[k];
		}));
		var __exportStar = (out$1 && out$1.__exportStar) || function(m, exports) {
		    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		__exportStar(requireAnalyze(), exports);
		__exportStar(requireCommon(), exports);
		__exportStar(requireConstants$1(), exports);
		__exportStar(requireDatabase(), exports);
		__exportStar(requireDatabaseDiff(), exports);
		__exportStar(requireDatabaseUrl(), exports);
		__exportStar(requireDatabaseUtils(), exports);
		__exportStar(requireBytes(), exports);
		__exportStar(requireDate(), exports);
		__exportStar(requireDuration(), exports);
		__exportStar(requirePercent(), exports);
		__exportStar(requireSql(), exports);
		__exportStar(requireInferSchema(), exports);
		__exportStar(requireConnector(), exports);
		__exportStar(requireDesktopBridge(), exports);
		__exportStar(requireLegacyDatabase(), exports);
		__exportStar(requireLegacyProject(), exports);
		__exportStar(requireLegacyQuery(), exports);
		__exportStar(requireLegacyStats(), exports);
		__exportStar(requireLlm(), exports);
		__exportStar(requireParserResult(), exports);
		__exportStar(requireProject(), exports);
		__exportStar(requireZod(), exports);
		
	} (out$1));
	return out$1;
}

var outExports$1 = requireOut();

var name = "@azimutt/parser-sql";
var version$2 = "0.1.3";
var description = "Parse and Generate SQL.";
var keywords = [
];
var homepage = "https://azimutt.app";
var author = {
	name: "Loïc Knuchel",
	email: "loicknuchel@gmail.com",
	url: "https://loicknuchel.fr"
};
var license = "MIT";
var repository = {
	type: "git",
	url: "git+https://github.com/azimuttapp/azimutt.git",
	directory: "libs/parser-sql"
};
var type = "module";
var main = "./out/index.js";
var types = "./out/index.d.ts";
var scripts = {
	test: "jest",
	build: "rm -rf ./out && tsc && npm run build:browser",
	"build:docker": "npx tsc",
	"build:browser": "rollup --config rollup.config.ts --configPlugin typescript",
	"dry-publish": "pnpm run build && pnpm test && pnpm pack"
};
var dependencies = {
	"@azimutt/models": "workspace:^",
	"@azimutt/utils": "workspace:^",
	chevrotain: "10.5.0",
	zod: "3.23.8"
};
var devDependencies = {
	"@chevrotain/types": "^10.5.0",
	"@jest/globals": "^29.7.0",
	"@rollup/plugin-commonjs": "^28.0.1",
	"@rollup/plugin-json": "^6.1.0",
	"@rollup/plugin-node-resolve": "^15.3.0",
	"@rollup/plugin-terser": "^0.4.4",
	"@rollup/plugin-typescript": "^12.1.1",
	"@types/jest": "^29.5.14",
	"@types/node": "^22.9.0",
	jest: "^29.7.0",
	rollup: "^4.25.0",
	"ts-jest": "^29.2.5",
	typescript: "^5.6.3"
};
var packageJson = {
	name: name,
	version: version$2,
	description: description,
	keywords: keywords,
	homepage: homepage,
	author: author,
	license: license,
	repository: repository,
	type: type,
	main: main,
	types: types,
	scripts: scripts,
	dependencies: dependencies,
	devDependencies: devDependencies
};

var api$3 = {};

var version$1 = {};

var hasRequiredVersion;

function requireVersion () {
	if (hasRequiredVersion) return version$1;
	hasRequiredVersion = 1;
	Object.defineProperty(version$1, "__esModule", { value: true });
	version$1.VERSION = void 0;
	// needs a separate module as this is required inside chevrotain productive code
	// and also in the entry point for webpack(api.ts).
	// A separate file avoids cyclic dependencies and webpack errors.
	version$1.VERSION = "10.5.0";
	
	return version$1;
}

var parser$1 = {};

/** Used for built-in method references. */

var _isPrototype;
var hasRequired_isPrototype;

function require_isPrototype () {
	if (hasRequired_isPrototype) return _isPrototype;
	hasRequired_isPrototype = 1;
	var objectProto = Object.prototype;

	/**
	 * Checks if `value` is likely a prototype object.
	 *
	 * @private
	 * @param {*} value The value to check.
	 * @returns {boolean} Returns `true` if `value` is a prototype, else `false`.
	 */
	function isPrototype(value) {
	  var Ctor = value && value.constructor,
	      proto = (typeof Ctor == 'function' && Ctor.prototype) || objectProto;

	  return value === proto;
	}

	_isPrototype = isPrototype;
	return _isPrototype;
}

/**
 * Creates a unary function that invokes `func` with its argument transformed.
 *
 * @private
 * @param {Function} func The function to wrap.
 * @param {Function} transform The argument transform.
 * @returns {Function} Returns the new function.
 */

var _overArg;
var hasRequired_overArg;

function require_overArg () {
	if (hasRequired_overArg) return _overArg;
	hasRequired_overArg = 1;
	function overArg(func, transform) {
	  return function(arg) {
	    return func(transform(arg));
	  };
	}

	_overArg = overArg;
	return _overArg;
}

var _nativeKeys;
var hasRequired_nativeKeys;

function require_nativeKeys () {
	if (hasRequired_nativeKeys) return _nativeKeys;
	hasRequired_nativeKeys = 1;
	var overArg = require_overArg();

	/* Built-in method references for those with the same name as other `lodash` methods. */
	var nativeKeys = overArg(Object.keys, Object);

	_nativeKeys = nativeKeys;
	return _nativeKeys;
}

var _baseKeys;
var hasRequired_baseKeys;

function require_baseKeys () {
	if (hasRequired_baseKeys) return _baseKeys;
	hasRequired_baseKeys = 1;
	var isPrototype = require_isPrototype(),
	    nativeKeys = require_nativeKeys();

	/** Used for built-in method references. */
	var objectProto = Object.prototype;

	/** Used to check objects for own properties. */
	var hasOwnProperty = objectProto.hasOwnProperty;

	/**
	 * The base implementation of `_.keys` which doesn't treat sparse arrays as dense.
	 *
	 * @private
	 * @param {Object} object The object to query.
	 * @returns {Array} Returns the array of property names.
	 */
	function baseKeys(object) {
	  if (!isPrototype(object)) {
	    return nativeKeys(object);
	  }
	  var result = [];
	  for (var key in Object(object)) {
	    if (hasOwnProperty.call(object, key) && key != 'constructor') {
	      result.push(key);
	    }
	  }
	  return result;
	}

	_baseKeys = baseKeys;
	return _baseKeys;
}

/** Detect free variable `global` from Node.js. */

var _freeGlobal;
var hasRequired_freeGlobal;

function require_freeGlobal () {
	if (hasRequired_freeGlobal) return _freeGlobal;
	hasRequired_freeGlobal = 1;
	var freeGlobal = typeof commonjsGlobal == 'object' && commonjsGlobal && commonjsGlobal.Object === Object && commonjsGlobal;

	_freeGlobal = freeGlobal;
	return _freeGlobal;
}

var _root;
var hasRequired_root;

function require_root () {
	if (hasRequired_root) return _root;
	hasRequired_root = 1;
	var freeGlobal = require_freeGlobal();

	/** Detect free variable `self`. */
	var freeSelf = typeof self == 'object' && self && self.Object === Object && self;

	/** Used as a reference to the global object. */
	var root = freeGlobal || freeSelf || Function('return this')();

	_root = root;
	return _root;
}

var _Symbol;
var hasRequired_Symbol;

function require_Symbol () {
	if (hasRequired_Symbol) return _Symbol;
	hasRequired_Symbol = 1;
	var root = require_root();

	/** Built-in value references. */
	var Symbol = root.Symbol;

	_Symbol = Symbol;
	return _Symbol;
}

var _getRawTag;
var hasRequired_getRawTag;

function require_getRawTag () {
	if (hasRequired_getRawTag) return _getRawTag;
	hasRequired_getRawTag = 1;
	var Symbol = require_Symbol();

	/** Used for built-in method references. */
	var objectProto = Object.prototype;

	/** Used to check objects for own properties. */
	var hasOwnProperty = objectProto.hasOwnProperty;

	/**
	 * Used to resolve the
	 * [`toStringTag`](http://ecma-international.org/ecma-262/7.0/#sec-object.prototype.tostring)
	 * of values.
	 */
	var nativeObjectToString = objectProto.toString;

	/** Built-in value references. */
	var symToStringTag = Symbol ? Symbol.toStringTag : undefined;

	/**
	 * A specialized version of `baseGetTag` which ignores `Symbol.toStringTag` values.
	 *
	 * @private
	 * @param {*} value The value to query.
	 * @returns {string} Returns the raw `toStringTag`.
	 */
	function getRawTag(value) {
	  var isOwn = hasOwnProperty.call(value, symToStringTag),
	      tag = value[symToStringTag];

	  try {
	    value[symToStringTag] = undefined;
	    var unmasked = true;
	  } catch (e) {}

	  var result = nativeObjectToString.call(value);
	  if (unmasked) {
	    if (isOwn) {
	      value[symToStringTag] = tag;
	    } else {
	      delete value[symToStringTag];
	    }
	  }
	  return result;
	}

	_getRawTag = getRawTag;
	return _getRawTag;
}

/** Used for built-in method references. */

var _objectToString;
var hasRequired_objectToString;

function require_objectToString () {
	if (hasRequired_objectToString) return _objectToString;
	hasRequired_objectToString = 1;
	var objectProto = Object.prototype;

	/**
	 * Used to resolve the
	 * [`toStringTag`](http://ecma-international.org/ecma-262/7.0/#sec-object.prototype.tostring)
	 * of values.
	 */
	var nativeObjectToString = objectProto.toString;

	/**
	 * Converts `value` to a string using `Object.prototype.toString`.
	 *
	 * @private
	 * @param {*} value The value to convert.
	 * @returns {string} Returns the converted string.
	 */
	function objectToString(value) {
	  return nativeObjectToString.call(value);
	}

	_objectToString = objectToString;
	return _objectToString;
}

var _baseGetTag;
var hasRequired_baseGetTag;

function require_baseGetTag () {
	if (hasRequired_baseGetTag) return _baseGetTag;
	hasRequired_baseGetTag = 1;
	var Symbol = require_Symbol(),
	    getRawTag = require_getRawTag(),
	    objectToString = require_objectToString();

	/** `Object#toString` result references. */
	var nullTag = '[object Null]',
	    undefinedTag = '[object Undefined]';

	/** Built-in value references. */
	var symToStringTag = Symbol ? Symbol.toStringTag : undefined;

	/**
	 * The base implementation of `getTag` without fallbacks for buggy environments.
	 *
	 * @private
	 * @param {*} value The value to query.
	 * @returns {string} Returns the `toStringTag`.
	 */
	function baseGetTag(value) {
	  if (value == null) {
	    return value === undefined ? undefinedTag : nullTag;
	  }
	  return (symToStringTag && symToStringTag in Object(value))
	    ? getRawTag(value)
	    : objectToString(value);
	}

	_baseGetTag = baseGetTag;
	return _baseGetTag;
}

/**
 * Checks if `value` is the
 * [language type](http://www.ecma-international.org/ecma-262/7.0/#sec-ecmascript-language-types)
 * of `Object`. (e.g. arrays, functions, objects, regexes, `new Number(0)`, and `new String('')`)
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is an object, else `false`.
 * @example
 *
 * _.isObject({});
 * // => true
 *
 * _.isObject([1, 2, 3]);
 * // => true
 *
 * _.isObject(_.noop);
 * // => true
 *
 * _.isObject(null);
 * // => false
 */

var isObject_1;
var hasRequiredIsObject;

function requireIsObject () {
	if (hasRequiredIsObject) return isObject_1;
	hasRequiredIsObject = 1;
	function isObject(value) {
	  var type = typeof value;
	  return value != null && (type == 'object' || type == 'function');
	}

	isObject_1 = isObject;
	return isObject_1;
}

var isFunction_1;
var hasRequiredIsFunction;

function requireIsFunction () {
	if (hasRequiredIsFunction) return isFunction_1;
	hasRequiredIsFunction = 1;
	var baseGetTag = require_baseGetTag(),
	    isObject = requireIsObject();

	/** `Object#toString` result references. */
	var asyncTag = '[object AsyncFunction]',
	    funcTag = '[object Function]',
	    genTag = '[object GeneratorFunction]',
	    proxyTag = '[object Proxy]';

	/**
	 * Checks if `value` is classified as a `Function` object.
	 *
	 * @static
	 * @memberOf _
	 * @since 0.1.0
	 * @category Lang
	 * @param {*} value The value to check.
	 * @returns {boolean} Returns `true` if `value` is a function, else `false`.
	 * @example
	 *
	 * _.isFunction(_);
	 * // => true
	 *
	 * _.isFunction(/abc/);
	 * // => false
	 */
	function isFunction(value) {
	  if (!isObject(value)) {
	    return false;
	  }
	  // The use of `Object#toString` avoids issues with the `typeof` operator
	  // in Safari 9 which returns 'object' for typed arrays and other constructors.
	  var tag = baseGetTag(value);
	  return tag == funcTag || tag == genTag || tag == asyncTag || tag == proxyTag;
	}

	isFunction_1 = isFunction;
	return isFunction_1;
}

var _coreJsData;
var hasRequired_coreJsData;

function require_coreJsData () {
	if (hasRequired_coreJsData) return _coreJsData;
	hasRequired_coreJsData = 1;
	var root = require_root();

	/** Used to detect overreaching core-js shims. */
	var coreJsData = root['__core-js_shared__'];

	_coreJsData = coreJsData;
	return _coreJsData;
}

var _isMasked;
var hasRequired_isMasked;

function require_isMasked () {
	if (hasRequired_isMasked) return _isMasked;
	hasRequired_isMasked = 1;
	var coreJsData = require_coreJsData();

	/** Used to detect methods masquerading as native. */
	var maskSrcKey = (function() {
	  var uid = /[^.]+$/.exec(coreJsData && coreJsData.keys && coreJsData.keys.IE_PROTO || '');
	  return uid ? ('Symbol(src)_1.' + uid) : '';
	}());

	/**
	 * Checks if `func` has its source masked.
	 *
	 * @private
	 * @param {Function} func The function to check.
	 * @returns {boolean} Returns `true` if `func` is masked, else `false`.
	 */
	function isMasked(func) {
	  return !!maskSrcKey && (maskSrcKey in func);
	}

	_isMasked = isMasked;
	return _isMasked;
}

/** Used for built-in method references. */

var _toSource;
var hasRequired_toSource;

function require_toSource () {
	if (hasRequired_toSource) return _toSource;
	hasRequired_toSource = 1;
	var funcProto = Function.prototype;

	/** Used to resolve the decompiled source of functions. */
	var funcToString = funcProto.toString;

	/**
	 * Converts `func` to its source code.
	 *
	 * @private
	 * @param {Function} func The function to convert.
	 * @returns {string} Returns the source code.
	 */
	function toSource(func) {
	  if (func != null) {
	    try {
	      return funcToString.call(func);
	    } catch (e) {}
	    try {
	      return (func + '');
	    } catch (e) {}
	  }
	  return '';
	}

	_toSource = toSource;
	return _toSource;
}

var _baseIsNative;
var hasRequired_baseIsNative;

function require_baseIsNative () {
	if (hasRequired_baseIsNative) return _baseIsNative;
	hasRequired_baseIsNative = 1;
	var isFunction = requireIsFunction(),
	    isMasked = require_isMasked(),
	    isObject = requireIsObject(),
	    toSource = require_toSource();

	/**
	 * Used to match `RegExp`
	 * [syntax characters](http://ecma-international.org/ecma-262/7.0/#sec-patterns).
	 */
	var reRegExpChar = /[\\^$.*+?()[\]{}|]/g;

	/** Used to detect host constructors (Safari). */
	var reIsHostCtor = /^\[object .+?Constructor\]$/;

	/** Used for built-in method references. */
	var funcProto = Function.prototype,
	    objectProto = Object.prototype;

	/** Used to resolve the decompiled source of functions. */
	var funcToString = funcProto.toString;

	/** Used to check objects for own properties. */
	var hasOwnProperty = objectProto.hasOwnProperty;

	/** Used to detect if a method is native. */
	var reIsNative = RegExp('^' +
	  funcToString.call(hasOwnProperty).replace(reRegExpChar, '\\$&')
	  .replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g, '$1.*?') + '$'
	);

	/**
	 * The base implementation of `_.isNative` without bad shim checks.
	 *
	 * @private
	 * @param {*} value The value to check.
	 * @returns {boolean} Returns `true` if `value` is a native function,
	 *  else `false`.
	 */
	function baseIsNative(value) {
	  if (!isObject(value) || isMasked(value)) {
	    return false;
	  }
	  var pattern = isFunction(value) ? reIsNative : reIsHostCtor;
	  return pattern.test(toSource(value));
	}

	_baseIsNative = baseIsNative;
	return _baseIsNative;
}

/**
 * Gets the value at `key` of `object`.
 *
 * @private
 * @param {Object} [object] The object to query.
 * @param {string} key The key of the property to get.
 * @returns {*} Returns the property value.
 */

var _getValue;
var hasRequired_getValue;

function require_getValue () {
	if (hasRequired_getValue) return _getValue;
	hasRequired_getValue = 1;
	function getValue(object, key) {
	  return object == null ? undefined : object[key];
	}

	_getValue = getValue;
	return _getValue;
}

var _getNative;
var hasRequired_getNative;

function require_getNative () {
	if (hasRequired_getNative) return _getNative;
	hasRequired_getNative = 1;
	var baseIsNative = require_baseIsNative(),
	    getValue = require_getValue();

	/**
	 * Gets the native function at `key` of `object`.
	 *
	 * @private
	 * @param {Object} object The object to query.
	 * @param {string} key The key of the method to get.
	 * @returns {*} Returns the function if it's native, else `undefined`.
	 */
	function getNative(object, key) {
	  var value = getValue(object, key);
	  return baseIsNative(value) ? value : undefined;
	}

	_getNative = getNative;
	return _getNative;
}

var _DataView;
var hasRequired_DataView;

function require_DataView () {
	if (hasRequired_DataView) return _DataView;
	hasRequired_DataView = 1;
	var getNative = require_getNative(),
	    root = require_root();

	/* Built-in method references that are verified to be native. */
	var DataView = getNative(root, 'DataView');

	_DataView = DataView;
	return _DataView;
}

var _Map;
var hasRequired_Map;

function require_Map () {
	if (hasRequired_Map) return _Map;
	hasRequired_Map = 1;
	var getNative = require_getNative(),
	    root = require_root();

	/* Built-in method references that are verified to be native. */
	var Map = getNative(root, 'Map');

	_Map = Map;
	return _Map;
}

var _Promise;
var hasRequired_Promise;

function require_Promise () {
	if (hasRequired_Promise) return _Promise;
	hasRequired_Promise = 1;
	var getNative = require_getNative(),
	    root = require_root();

	/* Built-in method references that are verified to be native. */
	var Promise = getNative(root, 'Promise');

	_Promise = Promise;
	return _Promise;
}

var _Set;
var hasRequired_Set;

function require_Set () {
	if (hasRequired_Set) return _Set;
	hasRequired_Set = 1;
	var getNative = require_getNative(),
	    root = require_root();

	/* Built-in method references that are verified to be native. */
	var Set = getNative(root, 'Set');

	_Set = Set;
	return _Set;
}

var _WeakMap;
var hasRequired_WeakMap;

function require_WeakMap () {
	if (hasRequired_WeakMap) return _WeakMap;
	hasRequired_WeakMap = 1;
	var getNative = require_getNative(),
	    root = require_root();

	/* Built-in method references that are verified to be native. */
	var WeakMap = getNative(root, 'WeakMap');

	_WeakMap = WeakMap;
	return _WeakMap;
}

var _getTag;
var hasRequired_getTag;

function require_getTag () {
	if (hasRequired_getTag) return _getTag;
	hasRequired_getTag = 1;
	var DataView = require_DataView(),
	    Map = require_Map(),
	    Promise = require_Promise(),
	    Set = require_Set(),
	    WeakMap = require_WeakMap(),
	    baseGetTag = require_baseGetTag(),
	    toSource = require_toSource();

	/** `Object#toString` result references. */
	var mapTag = '[object Map]',
	    objectTag = '[object Object]',
	    promiseTag = '[object Promise]',
	    setTag = '[object Set]',
	    weakMapTag = '[object WeakMap]';

	var dataViewTag = '[object DataView]';

	/** Used to detect maps, sets, and weakmaps. */
	var dataViewCtorString = toSource(DataView),
	    mapCtorString = toSource(Map),
	    promiseCtorString = toSource(Promise),
	    setCtorString = toSource(Set),
	    weakMapCtorString = toSource(WeakMap);

	/**
	 * Gets the `toStringTag` of `value`.
	 *
	 * @private
	 * @param {*} value The value to query.
	 * @returns {string} Returns the `toStringTag`.
	 */
	var getTag = baseGetTag;

	// Fallback for data views, maps, sets, and weak maps in IE 11 and promises in Node.js < 6.
	if ((DataView && getTag(new DataView(new ArrayBuffer(1))) != dataViewTag) ||
	    (Map && getTag(new Map) != mapTag) ||
	    (Promise && getTag(Promise.resolve()) != promiseTag) ||
	    (Set && getTag(new Set) != setTag) ||
	    (WeakMap && getTag(new WeakMap) != weakMapTag)) {
	  getTag = function(value) {
	    var result = baseGetTag(value),
	        Ctor = result == objectTag ? value.constructor : undefined,
	        ctorString = Ctor ? toSource(Ctor) : '';

	    if (ctorString) {
	      switch (ctorString) {
	        case dataViewCtorString: return dataViewTag;
	        case mapCtorString: return mapTag;
	        case promiseCtorString: return promiseTag;
	        case setCtorString: return setTag;
	        case weakMapCtorString: return weakMapTag;
	      }
	    }
	    return result;
	  };
	}

	_getTag = getTag;
	return _getTag;
}

/**
 * Checks if `value` is object-like. A value is object-like if it's not `null`
 * and has a `typeof` result of "object".
 *
 * @static
 * @memberOf _
 * @since 4.0.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is object-like, else `false`.
 * @example
 *
 * _.isObjectLike({});
 * // => true
 *
 * _.isObjectLike([1, 2, 3]);
 * // => true
 *
 * _.isObjectLike(_.noop);
 * // => false
 *
 * _.isObjectLike(null);
 * // => false
 */

var isObjectLike_1;
var hasRequiredIsObjectLike;

function requireIsObjectLike () {
	if (hasRequiredIsObjectLike) return isObjectLike_1;
	hasRequiredIsObjectLike = 1;
	function isObjectLike(value) {
	  return value != null && typeof value == 'object';
	}

	isObjectLike_1 = isObjectLike;
	return isObjectLike_1;
}

var _baseIsArguments;
var hasRequired_baseIsArguments;

function require_baseIsArguments () {
	if (hasRequired_baseIsArguments) return _baseIsArguments;
	hasRequired_baseIsArguments = 1;
	var baseGetTag = require_baseGetTag(),
	    isObjectLike = requireIsObjectLike();

	/** `Object#toString` result references. */
	var argsTag = '[object Arguments]';

	/**
	 * The base implementation of `_.isArguments`.
	 *
	 * @private
	 * @param {*} value The value to check.
	 * @returns {boolean} Returns `true` if `value` is an `arguments` object,
	 */
	function baseIsArguments(value) {
	  return isObjectLike(value) && baseGetTag(value) == argsTag;
	}

	_baseIsArguments = baseIsArguments;
	return _baseIsArguments;
}

var isArguments_1;
var hasRequiredIsArguments;

function requireIsArguments () {
	if (hasRequiredIsArguments) return isArguments_1;
	hasRequiredIsArguments = 1;
	var baseIsArguments = require_baseIsArguments(),
	    isObjectLike = requireIsObjectLike();

	/** Used for built-in method references. */
	var objectProto = Object.prototype;

	/** Used to check objects for own properties. */
	var hasOwnProperty = objectProto.hasOwnProperty;

	/** Built-in value references. */
	var propertyIsEnumerable = objectProto.propertyIsEnumerable;

	/**
	 * Checks if `value` is likely an `arguments` object.
	 *
	 * @static
	 * @memberOf _
	 * @since 0.1.0
	 * @category Lang
	 * @param {*} value The value to check.
	 * @returns {boolean} Returns `true` if `value` is an `arguments` object,
	 *  else `false`.
	 * @example
	 *
	 * _.isArguments(function() { return arguments; }());
	 * // => true
	 *
	 * _.isArguments([1, 2, 3]);
	 * // => false
	 */
	var isArguments = baseIsArguments(function() { return arguments; }()) ? baseIsArguments : function(value) {
	  return isObjectLike(value) && hasOwnProperty.call(value, 'callee') &&
	    !propertyIsEnumerable.call(value, 'callee');
	};

	isArguments_1 = isArguments;
	return isArguments_1;
}

/**
 * Checks if `value` is classified as an `Array` object.
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is an array, else `false`.
 * @example
 *
 * _.isArray([1, 2, 3]);
 * // => true
 *
 * _.isArray(document.body.children);
 * // => false
 *
 * _.isArray('abc');
 * // => false
 *
 * _.isArray(_.noop);
 * // => false
 */

var isArray_1;
var hasRequiredIsArray;

function requireIsArray () {
	if (hasRequiredIsArray) return isArray_1;
	hasRequiredIsArray = 1;
	var isArray = Array.isArray;

	isArray_1 = isArray;
	return isArray_1;
}

/** Used as references for various `Number` constants. */

var isLength_1;
var hasRequiredIsLength;

function requireIsLength () {
	if (hasRequiredIsLength) return isLength_1;
	hasRequiredIsLength = 1;
	var MAX_SAFE_INTEGER = 9007199254740991;

	/**
	 * Checks if `value` is a valid array-like length.
	 *
	 * **Note:** This method is loosely based on
	 * [`ToLength`](http://ecma-international.org/ecma-262/7.0/#sec-tolength).
	 *
	 * @static
	 * @memberOf _
	 * @since 4.0.0
	 * @category Lang
	 * @param {*} value The value to check.
	 * @returns {boolean} Returns `true` if `value` is a valid length, else `false`.
	 * @example
	 *
	 * _.isLength(3);
	 * // => true
	 *
	 * _.isLength(Number.MIN_VALUE);
	 * // => false
	 *
	 * _.isLength(Infinity);
	 * // => false
	 *
	 * _.isLength('3');
	 * // => false
	 */
	function isLength(value) {
	  return typeof value == 'number' &&
	    value > -1 && value % 1 == 0 && value <= MAX_SAFE_INTEGER;
	}

	isLength_1 = isLength;
	return isLength_1;
}

var isArrayLike_1;
var hasRequiredIsArrayLike;

function requireIsArrayLike () {
	if (hasRequiredIsArrayLike) return isArrayLike_1;
	hasRequiredIsArrayLike = 1;
	var isFunction = requireIsFunction(),
	    isLength = requireIsLength();

	/**
	 * Checks if `value` is array-like. A value is considered array-like if it's
	 * not a function and has a `value.length` that's an integer greater than or
	 * equal to `0` and less than or equal to `Number.MAX_SAFE_INTEGER`.
	 *
	 * @static
	 * @memberOf _
	 * @since 4.0.0
	 * @category Lang
	 * @param {*} value The value to check.
	 * @returns {boolean} Returns `true` if `value` is array-like, else `false`.
	 * @example
	 *
	 * _.isArrayLike([1, 2, 3]);
	 * // => true
	 *
	 * _.isArrayLike(document.body.children);
	 * // => true
	 *
	 * _.isArrayLike('abc');
	 * // => true
	 *
	 * _.isArrayLike(_.noop);
	 * // => false
	 */
	function isArrayLike(value) {
	  return value != null && isLength(value.length) && !isFunction(value);
	}

	isArrayLike_1 = isArrayLike;
	return isArrayLike_1;
}

var isBuffer = {exports: {}};

/**
 * This method returns `false`.
 *
 * @static
 * @memberOf _
 * @since 4.13.0
 * @category Util
 * @returns {boolean} Returns `false`.
 * @example
 *
 * _.times(2, _.stubFalse);
 * // => [false, false]
 */

var stubFalse_1;
var hasRequiredStubFalse;

function requireStubFalse () {
	if (hasRequiredStubFalse) return stubFalse_1;
	hasRequiredStubFalse = 1;
	function stubFalse() {
	  return false;
	}

	stubFalse_1 = stubFalse;
	return stubFalse_1;
}

isBuffer.exports;

var hasRequiredIsBuffer;

function requireIsBuffer () {
	if (hasRequiredIsBuffer) return isBuffer.exports;
	hasRequiredIsBuffer = 1;
	(function (module, exports) {
		var root = require_root(),
		    stubFalse = requireStubFalse();

		/** Detect free variable `exports`. */
		var freeExports = exports && !exports.nodeType && exports;

		/** Detect free variable `module`. */
		var freeModule = freeExports && 'object' == 'object' && module && !module.nodeType && module;

		/** Detect the popular CommonJS extension `module.exports`. */
		var moduleExports = freeModule && freeModule.exports === freeExports;

		/** Built-in value references. */
		var Buffer = moduleExports ? root.Buffer : undefined;

		/* Built-in method references for those with the same name as other `lodash` methods. */
		var nativeIsBuffer = Buffer ? Buffer.isBuffer : undefined;

		/**
		 * Checks if `value` is a buffer.
		 *
		 * @static
		 * @memberOf _
		 * @since 4.3.0
		 * @category Lang
		 * @param {*} value The value to check.
		 * @returns {boolean} Returns `true` if `value` is a buffer, else `false`.
		 * @example
		 *
		 * _.isBuffer(new Buffer(2));
		 * // => true
		 *
		 * _.isBuffer(new Uint8Array(2));
		 * // => false
		 */
		var isBuffer = nativeIsBuffer || stubFalse;

		module.exports = isBuffer; 
	} (isBuffer, isBuffer.exports));
	return isBuffer.exports;
}

var _baseIsTypedArray;
var hasRequired_baseIsTypedArray;

function require_baseIsTypedArray () {
	if (hasRequired_baseIsTypedArray) return _baseIsTypedArray;
	hasRequired_baseIsTypedArray = 1;
	var baseGetTag = require_baseGetTag(),
	    isLength = requireIsLength(),
	    isObjectLike = requireIsObjectLike();

	/** `Object#toString` result references. */
	var argsTag = '[object Arguments]',
	    arrayTag = '[object Array]',
	    boolTag = '[object Boolean]',
	    dateTag = '[object Date]',
	    errorTag = '[object Error]',
	    funcTag = '[object Function]',
	    mapTag = '[object Map]',
	    numberTag = '[object Number]',
	    objectTag = '[object Object]',
	    regexpTag = '[object RegExp]',
	    setTag = '[object Set]',
	    stringTag = '[object String]',
	    weakMapTag = '[object WeakMap]';

	var arrayBufferTag = '[object ArrayBuffer]',
	    dataViewTag = '[object DataView]',
	    float32Tag = '[object Float32Array]',
	    float64Tag = '[object Float64Array]',
	    int8Tag = '[object Int8Array]',
	    int16Tag = '[object Int16Array]',
	    int32Tag = '[object Int32Array]',
	    uint8Tag = '[object Uint8Array]',
	    uint8ClampedTag = '[object Uint8ClampedArray]',
	    uint16Tag = '[object Uint16Array]',
	    uint32Tag = '[object Uint32Array]';

	/** Used to identify `toStringTag` values of typed arrays. */
	var typedArrayTags = {};
	typedArrayTags[float32Tag] = typedArrayTags[float64Tag] =
	typedArrayTags[int8Tag] = typedArrayTags[int16Tag] =
	typedArrayTags[int32Tag] = typedArrayTags[uint8Tag] =
	typedArrayTags[uint8ClampedTag] = typedArrayTags[uint16Tag] =
	typedArrayTags[uint32Tag] = true;
	typedArrayTags[argsTag] = typedArrayTags[arrayTag] =
	typedArrayTags[arrayBufferTag] = typedArrayTags[boolTag] =
	typedArrayTags[dataViewTag] = typedArrayTags[dateTag] =
	typedArrayTags[errorTag] = typedArrayTags[funcTag] =
	typedArrayTags[mapTag] = typedArrayTags[numberTag] =
	typedArrayTags[objectTag] = typedArrayTags[regexpTag] =
	typedArrayTags[setTag] = typedArrayTags[stringTag] =
	typedArrayTags[weakMapTag] = false;

	/**
	 * The base implementation of `_.isTypedArray` without Node.js optimizations.
	 *
	 * @private
	 * @param {*} value The value to check.
	 * @returns {boolean} Returns `true` if `value` is a typed array, else `false`.
	 */
	function baseIsTypedArray(value) {
	  return isObjectLike(value) &&
	    isLength(value.length) && !!typedArrayTags[baseGetTag(value)];
	}

	_baseIsTypedArray = baseIsTypedArray;
	return _baseIsTypedArray;
}

/**
 * The base implementation of `_.unary` without support for storing metadata.
 *
 * @private
 * @param {Function} func The function to cap arguments for.
 * @returns {Function} Returns the new capped function.
 */

var _baseUnary;
var hasRequired_baseUnary;

function require_baseUnary () {
	if (hasRequired_baseUnary) return _baseUnary;
	hasRequired_baseUnary = 1;
	function baseUnary(func) {
	  return function(value) {
	    return func(value);
	  };
	}

	_baseUnary = baseUnary;
	return _baseUnary;
}

var _nodeUtil = {exports: {}};

_nodeUtil.exports;

var hasRequired_nodeUtil;

function require_nodeUtil () {
	if (hasRequired_nodeUtil) return _nodeUtil.exports;
	hasRequired_nodeUtil = 1;
	(function (module, exports) {
		var freeGlobal = require_freeGlobal();

		/** Detect free variable `exports`. */
		var freeExports = exports && !exports.nodeType && exports;

		/** Detect free variable `module`. */
		var freeModule = freeExports && 'object' == 'object' && module && !module.nodeType && module;

		/** Detect the popular CommonJS extension `module.exports`. */
		var moduleExports = freeModule && freeModule.exports === freeExports;

		/** Detect free variable `process` from Node.js. */
		var freeProcess = moduleExports && freeGlobal.process;

		/** Used to access faster Node.js helpers. */
		var nodeUtil = (function() {
		  try {
		    // Use `util.types` for Node.js 10+.
		    var types = freeModule && freeModule.require && freeModule.require('util').types;

		    if (types) {
		      return types;
		    }

		    // Legacy `process.binding('util')` for Node.js < 10.
		    return freeProcess && freeProcess.binding && freeProcess.binding('util');
		  } catch (e) {}
		}());

		module.exports = nodeUtil; 
	} (_nodeUtil, _nodeUtil.exports));
	return _nodeUtil.exports;
}

var isTypedArray_1;
var hasRequiredIsTypedArray;

function requireIsTypedArray () {
	if (hasRequiredIsTypedArray) return isTypedArray_1;
	hasRequiredIsTypedArray = 1;
	var baseIsTypedArray = require_baseIsTypedArray(),
	    baseUnary = require_baseUnary(),
	    nodeUtil = require_nodeUtil();

	/* Node.js helper references. */
	var nodeIsTypedArray = nodeUtil && nodeUtil.isTypedArray;

	/**
	 * Checks if `value` is classified as a typed array.
	 *
	 * @static
	 * @memberOf _
	 * @since 3.0.0
	 * @category Lang
	 * @param {*} value The value to check.
	 * @returns {boolean} Returns `true` if `value` is a typed array, else `false`.
	 * @example
	 *
	 * _.isTypedArray(new Uint8Array);
	 * // => true
	 *
	 * _.isTypedArray([]);
	 * // => false
	 */
	var isTypedArray = nodeIsTypedArray ? baseUnary(nodeIsTypedArray) : baseIsTypedArray;

	isTypedArray_1 = isTypedArray;
	return isTypedArray_1;
}

var isEmpty_1;
var hasRequiredIsEmpty;

function requireIsEmpty () {
	if (hasRequiredIsEmpty) return isEmpty_1;
	hasRequiredIsEmpty = 1;
	var baseKeys = require_baseKeys(),
	    getTag = require_getTag(),
	    isArguments = requireIsArguments(),
	    isArray = requireIsArray(),
	    isArrayLike = requireIsArrayLike(),
	    isBuffer = requireIsBuffer(),
	    isPrototype = require_isPrototype(),
	    isTypedArray = requireIsTypedArray();

	/** `Object#toString` result references. */
	var mapTag = '[object Map]',
	    setTag = '[object Set]';

	/** Used for built-in method references. */
	var objectProto = Object.prototype;

	/** Used to check objects for own properties. */
	var hasOwnProperty = objectProto.hasOwnProperty;

	/**
	 * Checks if `value` is an empty object, collection, map, or set.
	 *
	 * Objects are considered empty if they have no own enumerable string keyed
	 * properties.
	 *
	 * Array-like values such as `arguments` objects, arrays, buffers, strings, or
	 * jQuery-like collections are considered empty if they have a `length` of `0`.
	 * Similarly, maps and sets are considered empty if they have a `size` of `0`.
	 *
	 * @static
	 * @memberOf _
	 * @since 0.1.0
	 * @category Lang
	 * @param {*} value The value to check.
	 * @returns {boolean} Returns `true` if `value` is empty, else `false`.
	 * @example
	 *
	 * _.isEmpty(null);
	 * // => true
	 *
	 * _.isEmpty(true);
	 * // => true
	 *
	 * _.isEmpty(1);
	 * // => true
	 *
	 * _.isEmpty([1, 2, 3]);
	 * // => false
	 *
	 * _.isEmpty({ 'a': 1 });
	 * // => false
	 */
	function isEmpty(value) {
	  if (value == null) {
	    return true;
	  }
	  if (isArrayLike(value) &&
	      (isArray(value) || typeof value == 'string' || typeof value.splice == 'function' ||
	        isBuffer(value) || isTypedArray(value) || isArguments(value))) {
	    return !value.length;
	  }
	  var tag = getTag(value);
	  if (tag == mapTag || tag == setTag) {
	    return !value.size;
	  }
	  if (isPrototype(value)) {
	    return !baseKeys(value).length;
	  }
	  for (var key in value) {
	    if (hasOwnProperty.call(value, key)) {
	      return false;
	    }
	  }
	  return true;
	}

	isEmpty_1 = isEmpty;
	return isEmpty_1;
}

/**
 * A specialized version of `_.map` for arrays without support for iteratee
 * shorthands.
 *
 * @private
 * @param {Array} [array] The array to iterate over.
 * @param {Function} iteratee The function invoked per iteration.
 * @returns {Array} Returns the new mapped array.
 */

var _arrayMap;
var hasRequired_arrayMap;

function require_arrayMap () {
	if (hasRequired_arrayMap) return _arrayMap;
	hasRequired_arrayMap = 1;
	function arrayMap(array, iteratee) {
	  var index = -1,
	      length = array == null ? 0 : array.length,
	      result = Array(length);

	  while (++index < length) {
	    result[index] = iteratee(array[index], index, array);
	  }
	  return result;
	}

	_arrayMap = arrayMap;
	return _arrayMap;
}

/**
 * Removes all key-value entries from the list cache.
 *
 * @private
 * @name clear
 * @memberOf ListCache
 */

var _listCacheClear;
var hasRequired_listCacheClear;

function require_listCacheClear () {
	if (hasRequired_listCacheClear) return _listCacheClear;
	hasRequired_listCacheClear = 1;
	function listCacheClear() {
	  this.__data__ = [];
	  this.size = 0;
	}

	_listCacheClear = listCacheClear;
	return _listCacheClear;
}

/**
 * Performs a
 * [`SameValueZero`](http://ecma-international.org/ecma-262/7.0/#sec-samevaluezero)
 * comparison between two values to determine if they are equivalent.
 *
 * @static
 * @memberOf _
 * @since 4.0.0
 * @category Lang
 * @param {*} value The value to compare.
 * @param {*} other The other value to compare.
 * @returns {boolean} Returns `true` if the values are equivalent, else `false`.
 * @example
 *
 * var object = { 'a': 1 };
 * var other = { 'a': 1 };
 *
 * _.eq(object, object);
 * // => true
 *
 * _.eq(object, other);
 * // => false
 *
 * _.eq('a', 'a');
 * // => true
 *
 * _.eq('a', Object('a'));
 * // => false
 *
 * _.eq(NaN, NaN);
 * // => true
 */

var eq_1;
var hasRequiredEq;

function requireEq () {
	if (hasRequiredEq) return eq_1;
	hasRequiredEq = 1;
	function eq(value, other) {
	  return value === other || (value !== value && other !== other);
	}

	eq_1 = eq;
	return eq_1;
}

var _assocIndexOf;
var hasRequired_assocIndexOf;

function require_assocIndexOf () {
	if (hasRequired_assocIndexOf) return _assocIndexOf;
	hasRequired_assocIndexOf = 1;
	var eq = requireEq();

	/**
	 * Gets the index at which the `key` is found in `array` of key-value pairs.
	 *
	 * @private
	 * @param {Array} array The array to inspect.
	 * @param {*} key The key to search for.
	 * @returns {number} Returns the index of the matched value, else `-1`.
	 */
	function assocIndexOf(array, key) {
	  var length = array.length;
	  while (length--) {
	    if (eq(array[length][0], key)) {
	      return length;
	    }
	  }
	  return -1;
	}

	_assocIndexOf = assocIndexOf;
	return _assocIndexOf;
}

var _listCacheDelete;
var hasRequired_listCacheDelete;

function require_listCacheDelete () {
	if (hasRequired_listCacheDelete) return _listCacheDelete;
	hasRequired_listCacheDelete = 1;
	var assocIndexOf = require_assocIndexOf();

	/** Used for built-in method references. */
	var arrayProto = Array.prototype;

	/** Built-in value references. */
	var splice = arrayProto.splice;

	/**
	 * Removes `key` and its value from the list cache.
	 *
	 * @private
	 * @name delete
	 * @memberOf ListCache
	 * @param {string} key The key of the value to remove.
	 * @returns {boolean} Returns `true` if the entry was removed, else `false`.
	 */
	function listCacheDelete(key) {
	  var data = this.__data__,
	      index = assocIndexOf(data, key);

	  if (index < 0) {
	    return false;
	  }
	  var lastIndex = data.length - 1;
	  if (index == lastIndex) {
	    data.pop();
	  } else {
	    splice.call(data, index, 1);
	  }
	  --this.size;
	  return true;
	}

	_listCacheDelete = listCacheDelete;
	return _listCacheDelete;
}

var _listCacheGet;
var hasRequired_listCacheGet;

function require_listCacheGet () {
	if (hasRequired_listCacheGet) return _listCacheGet;
	hasRequired_listCacheGet = 1;
	var assocIndexOf = require_assocIndexOf();

	/**
	 * Gets the list cache value for `key`.
	 *
	 * @private
	 * @name get
	 * @memberOf ListCache
	 * @param {string} key The key of the value to get.
	 * @returns {*} Returns the entry value.
	 */
	function listCacheGet(key) {
	  var data = this.__data__,
	      index = assocIndexOf(data, key);

	  return index < 0 ? undefined : data[index][1];
	}

	_listCacheGet = listCacheGet;
	return _listCacheGet;
}

var _listCacheHas;
var hasRequired_listCacheHas;

function require_listCacheHas () {
	if (hasRequired_listCacheHas) return _listCacheHas;
	hasRequired_listCacheHas = 1;
	var assocIndexOf = require_assocIndexOf();

	/**
	 * Checks if a list cache value for `key` exists.
	 *
	 * @private
	 * @name has
	 * @memberOf ListCache
	 * @param {string} key The key of the entry to check.
	 * @returns {boolean} Returns `true` if an entry for `key` exists, else `false`.
	 */
	function listCacheHas(key) {
	  return assocIndexOf(this.__data__, key) > -1;
	}

	_listCacheHas = listCacheHas;
	return _listCacheHas;
}

var _listCacheSet;
var hasRequired_listCacheSet;

function require_listCacheSet () {
	if (hasRequired_listCacheSet) return _listCacheSet;
	hasRequired_listCacheSet = 1;
	var assocIndexOf = require_assocIndexOf();

	/**
	 * Sets the list cache `key` to `value`.
	 *
	 * @private
	 * @name set
	 * @memberOf ListCache
	 * @param {string} key The key of the value to set.
	 * @param {*} value The value to set.
	 * @returns {Object} Returns the list cache instance.
	 */
	function listCacheSet(key, value) {
	  var data = this.__data__,
	      index = assocIndexOf(data, key);

	  if (index < 0) {
	    ++this.size;
	    data.push([key, value]);
	  } else {
	    data[index][1] = value;
	  }
	  return this;
	}

	_listCacheSet = listCacheSet;
	return _listCacheSet;
}

var _ListCache;
var hasRequired_ListCache;

function require_ListCache () {
	if (hasRequired_ListCache) return _ListCache;
	hasRequired_ListCache = 1;
	var listCacheClear = require_listCacheClear(),
	    listCacheDelete = require_listCacheDelete(),
	    listCacheGet = require_listCacheGet(),
	    listCacheHas = require_listCacheHas(),
	    listCacheSet = require_listCacheSet();

	/**
	 * Creates an list cache object.
	 *
	 * @private
	 * @constructor
	 * @param {Array} [entries] The key-value pairs to cache.
	 */
	function ListCache(entries) {
	  var index = -1,
	      length = entries == null ? 0 : entries.length;

	  this.clear();
	  while (++index < length) {
	    var entry = entries[index];
	    this.set(entry[0], entry[1]);
	  }
	}

	// Add methods to `ListCache`.
	ListCache.prototype.clear = listCacheClear;
	ListCache.prototype['delete'] = listCacheDelete;
	ListCache.prototype.get = listCacheGet;
	ListCache.prototype.has = listCacheHas;
	ListCache.prototype.set = listCacheSet;

	_ListCache = ListCache;
	return _ListCache;
}

var _stackClear;
var hasRequired_stackClear;

function require_stackClear () {
	if (hasRequired_stackClear) return _stackClear;
	hasRequired_stackClear = 1;
	var ListCache = require_ListCache();

	/**
	 * Removes all key-value entries from the stack.
	 *
	 * @private
	 * @name clear
	 * @memberOf Stack
	 */
	function stackClear() {
	  this.__data__ = new ListCache;
	  this.size = 0;
	}

	_stackClear = stackClear;
	return _stackClear;
}

/**
 * Removes `key` and its value from the stack.
 *
 * @private
 * @name delete
 * @memberOf Stack
 * @param {string} key The key of the value to remove.
 * @returns {boolean} Returns `true` if the entry was removed, else `false`.
 */

var _stackDelete;
var hasRequired_stackDelete;

function require_stackDelete () {
	if (hasRequired_stackDelete) return _stackDelete;
	hasRequired_stackDelete = 1;
	function stackDelete(key) {
	  var data = this.__data__,
	      result = data['delete'](key);

	  this.size = data.size;
	  return result;
	}

	_stackDelete = stackDelete;
	return _stackDelete;
}

/**
 * Gets the stack value for `key`.
 *
 * @private
 * @name get
 * @memberOf Stack
 * @param {string} key The key of the value to get.
 * @returns {*} Returns the entry value.
 */

var _stackGet;
var hasRequired_stackGet;

function require_stackGet () {
	if (hasRequired_stackGet) return _stackGet;
	hasRequired_stackGet = 1;
	function stackGet(key) {
	  return this.__data__.get(key);
	}

	_stackGet = stackGet;
	return _stackGet;
}

/**
 * Checks if a stack value for `key` exists.
 *
 * @private
 * @name has
 * @memberOf Stack
 * @param {string} key The key of the entry to check.
 * @returns {boolean} Returns `true` if an entry for `key` exists, else `false`.
 */

var _stackHas;
var hasRequired_stackHas;

function require_stackHas () {
	if (hasRequired_stackHas) return _stackHas;
	hasRequired_stackHas = 1;
	function stackHas(key) {
	  return this.__data__.has(key);
	}

	_stackHas = stackHas;
	return _stackHas;
}

var _nativeCreate;
var hasRequired_nativeCreate;

function require_nativeCreate () {
	if (hasRequired_nativeCreate) return _nativeCreate;
	hasRequired_nativeCreate = 1;
	var getNative = require_getNative();

	/* Built-in method references that are verified to be native. */
	var nativeCreate = getNative(Object, 'create');

	_nativeCreate = nativeCreate;
	return _nativeCreate;
}

var _hashClear;
var hasRequired_hashClear;

function require_hashClear () {
	if (hasRequired_hashClear) return _hashClear;
	hasRequired_hashClear = 1;
	var nativeCreate = require_nativeCreate();

	/**
	 * Removes all key-value entries from the hash.
	 *
	 * @private
	 * @name clear
	 * @memberOf Hash
	 */
	function hashClear() {
	  this.__data__ = nativeCreate ? nativeCreate(null) : {};
	  this.size = 0;
	}

	_hashClear = hashClear;
	return _hashClear;
}

/**
 * Removes `key` and its value from the hash.
 *
 * @private
 * @name delete
 * @memberOf Hash
 * @param {Object} hash The hash to modify.
 * @param {string} key The key of the value to remove.
 * @returns {boolean} Returns `true` if the entry was removed, else `false`.
 */

var _hashDelete;
var hasRequired_hashDelete;

function require_hashDelete () {
	if (hasRequired_hashDelete) return _hashDelete;
	hasRequired_hashDelete = 1;
	function hashDelete(key) {
	  var result = this.has(key) && delete this.__data__[key];
	  this.size -= result ? 1 : 0;
	  return result;
	}

	_hashDelete = hashDelete;
	return _hashDelete;
}

var _hashGet;
var hasRequired_hashGet;

function require_hashGet () {
	if (hasRequired_hashGet) return _hashGet;
	hasRequired_hashGet = 1;
	var nativeCreate = require_nativeCreate();

	/** Used to stand-in for `undefined` hash values. */
	var HASH_UNDEFINED = '__lodash_hash_undefined__';

	/** Used for built-in method references. */
	var objectProto = Object.prototype;

	/** Used to check objects for own properties. */
	var hasOwnProperty = objectProto.hasOwnProperty;

	/**
	 * Gets the hash value for `key`.
	 *
	 * @private
	 * @name get
	 * @memberOf Hash
	 * @param {string} key The key of the value to get.
	 * @returns {*} Returns the entry value.
	 */
	function hashGet(key) {
	  var data = this.__data__;
	  if (nativeCreate) {
	    var result = data[key];
	    return result === HASH_UNDEFINED ? undefined : result;
	  }
	  return hasOwnProperty.call(data, key) ? data[key] : undefined;
	}

	_hashGet = hashGet;
	return _hashGet;
}

var _hashHas;
var hasRequired_hashHas;

function require_hashHas () {
	if (hasRequired_hashHas) return _hashHas;
	hasRequired_hashHas = 1;
	var nativeCreate = require_nativeCreate();

	/** Used for built-in method references. */
	var objectProto = Object.prototype;

	/** Used to check objects for own properties. */
	var hasOwnProperty = objectProto.hasOwnProperty;

	/**
	 * Checks if a hash value for `key` exists.
	 *
	 * @private
	 * @name has
	 * @memberOf Hash
	 * @param {string} key The key of the entry to check.
	 * @returns {boolean} Returns `true` if an entry for `key` exists, else `false`.
	 */
	function hashHas(key) {
	  var data = this.__data__;
	  return nativeCreate ? (data[key] !== undefined) : hasOwnProperty.call(data, key);
	}

	_hashHas = hashHas;
	return _hashHas;
}

var _hashSet;
var hasRequired_hashSet;

function require_hashSet () {
	if (hasRequired_hashSet) return _hashSet;
	hasRequired_hashSet = 1;
	var nativeCreate = require_nativeCreate();

	/** Used to stand-in for `undefined` hash values. */
	var HASH_UNDEFINED = '__lodash_hash_undefined__';

	/**
	 * Sets the hash `key` to `value`.
	 *
	 * @private
	 * @name set
	 * @memberOf Hash
	 * @param {string} key The key of the value to set.
	 * @param {*} value The value to set.
	 * @returns {Object} Returns the hash instance.
	 */
	function hashSet(key, value) {
	  var data = this.__data__;
	  this.size += this.has(key) ? 0 : 1;
	  data[key] = (nativeCreate && value === undefined) ? HASH_UNDEFINED : value;
	  return this;
	}

	_hashSet = hashSet;
	return _hashSet;
}

var _Hash;
var hasRequired_Hash;

function require_Hash () {
	if (hasRequired_Hash) return _Hash;
	hasRequired_Hash = 1;
	var hashClear = require_hashClear(),
	    hashDelete = require_hashDelete(),
	    hashGet = require_hashGet(),
	    hashHas = require_hashHas(),
	    hashSet = require_hashSet();

	/**
	 * Creates a hash object.
	 *
	 * @private
	 * @constructor
	 * @param {Array} [entries] The key-value pairs to cache.
	 */
	function Hash(entries) {
	  var index = -1,
	      length = entries == null ? 0 : entries.length;

	  this.clear();
	  while (++index < length) {
	    var entry = entries[index];
	    this.set(entry[0], entry[1]);
	  }
	}

	// Add methods to `Hash`.
	Hash.prototype.clear = hashClear;
	Hash.prototype['delete'] = hashDelete;
	Hash.prototype.get = hashGet;
	Hash.prototype.has = hashHas;
	Hash.prototype.set = hashSet;

	_Hash = Hash;
	return _Hash;
}

var _mapCacheClear;
var hasRequired_mapCacheClear;

function require_mapCacheClear () {
	if (hasRequired_mapCacheClear) return _mapCacheClear;
	hasRequired_mapCacheClear = 1;
	var Hash = require_Hash(),
	    ListCache = require_ListCache(),
	    Map = require_Map();

	/**
	 * Removes all key-value entries from the map.
	 *
	 * @private
	 * @name clear
	 * @memberOf MapCache
	 */
	function mapCacheClear() {
	  this.size = 0;
	  this.__data__ = {
	    'hash': new Hash,
	    'map': new (Map || ListCache),
	    'string': new Hash
	  };
	}

	_mapCacheClear = mapCacheClear;
	return _mapCacheClear;
}

/**
 * Checks if `value` is suitable for use as unique object key.
 *
 * @private
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is suitable, else `false`.
 */

var _isKeyable;
var hasRequired_isKeyable;

function require_isKeyable () {
	if (hasRequired_isKeyable) return _isKeyable;
	hasRequired_isKeyable = 1;
	function isKeyable(value) {
	  var type = typeof value;
	  return (type == 'string' || type == 'number' || type == 'symbol' || type == 'boolean')
	    ? (value !== '__proto__')
	    : (value === null);
	}

	_isKeyable = isKeyable;
	return _isKeyable;
}

var _getMapData;
var hasRequired_getMapData;

function require_getMapData () {
	if (hasRequired_getMapData) return _getMapData;
	hasRequired_getMapData = 1;
	var isKeyable = require_isKeyable();

	/**
	 * Gets the data for `map`.
	 *
	 * @private
	 * @param {Object} map The map to query.
	 * @param {string} key The reference key.
	 * @returns {*} Returns the map data.
	 */
	function getMapData(map, key) {
	  var data = map.__data__;
	  return isKeyable(key)
	    ? data[typeof key == 'string' ? 'string' : 'hash']
	    : data.map;
	}

	_getMapData = getMapData;
	return _getMapData;
}

var _mapCacheDelete;
var hasRequired_mapCacheDelete;

function require_mapCacheDelete () {
	if (hasRequired_mapCacheDelete) return _mapCacheDelete;
	hasRequired_mapCacheDelete = 1;
	var getMapData = require_getMapData();

	/**
	 * Removes `key` and its value from the map.
	 *
	 * @private
	 * @name delete
	 * @memberOf MapCache
	 * @param {string} key The key of the value to remove.
	 * @returns {boolean} Returns `true` if the entry was removed, else `false`.
	 */
	function mapCacheDelete(key) {
	  var result = getMapData(this, key)['delete'](key);
	  this.size -= result ? 1 : 0;
	  return result;
	}

	_mapCacheDelete = mapCacheDelete;
	return _mapCacheDelete;
}

var _mapCacheGet;
var hasRequired_mapCacheGet;

function require_mapCacheGet () {
	if (hasRequired_mapCacheGet) return _mapCacheGet;
	hasRequired_mapCacheGet = 1;
	var getMapData = require_getMapData();

	/**
	 * Gets the map value for `key`.
	 *
	 * @private
	 * @name get
	 * @memberOf MapCache
	 * @param {string} key The key of the value to get.
	 * @returns {*} Returns the entry value.
	 */
	function mapCacheGet(key) {
	  return getMapData(this, key).get(key);
	}

	_mapCacheGet = mapCacheGet;
	return _mapCacheGet;
}

var _mapCacheHas;
var hasRequired_mapCacheHas;

function require_mapCacheHas () {
	if (hasRequired_mapCacheHas) return _mapCacheHas;
	hasRequired_mapCacheHas = 1;
	var getMapData = require_getMapData();

	/**
	 * Checks if a map value for `key` exists.
	 *
	 * @private
	 * @name has
	 * @memberOf MapCache
	 * @param {string} key The key of the entry to check.
	 * @returns {boolean} Returns `true` if an entry for `key` exists, else `false`.
	 */
	function mapCacheHas(key) {
	  return getMapData(this, key).has(key);
	}

	_mapCacheHas = mapCacheHas;
	return _mapCacheHas;
}

var _mapCacheSet;
var hasRequired_mapCacheSet;

function require_mapCacheSet () {
	if (hasRequired_mapCacheSet) return _mapCacheSet;
	hasRequired_mapCacheSet = 1;
	var getMapData = require_getMapData();

	/**
	 * Sets the map `key` to `value`.
	 *
	 * @private
	 * @name set
	 * @memberOf MapCache
	 * @param {string} key The key of the value to set.
	 * @param {*} value The value to set.
	 * @returns {Object} Returns the map cache instance.
	 */
	function mapCacheSet(key, value) {
	  var data = getMapData(this, key),
	      size = data.size;

	  data.set(key, value);
	  this.size += data.size == size ? 0 : 1;
	  return this;
	}

	_mapCacheSet = mapCacheSet;
	return _mapCacheSet;
}

var _MapCache;
var hasRequired_MapCache;

function require_MapCache () {
	if (hasRequired_MapCache) return _MapCache;
	hasRequired_MapCache = 1;
	var mapCacheClear = require_mapCacheClear(),
	    mapCacheDelete = require_mapCacheDelete(),
	    mapCacheGet = require_mapCacheGet(),
	    mapCacheHas = require_mapCacheHas(),
	    mapCacheSet = require_mapCacheSet();

	/**
	 * Creates a map cache object to store key-value pairs.
	 *
	 * @private
	 * @constructor
	 * @param {Array} [entries] The key-value pairs to cache.
	 */
	function MapCache(entries) {
	  var index = -1,
	      length = entries == null ? 0 : entries.length;

	  this.clear();
	  while (++index < length) {
	    var entry = entries[index];
	    this.set(entry[0], entry[1]);
	  }
	}

	// Add methods to `MapCache`.
	MapCache.prototype.clear = mapCacheClear;
	MapCache.prototype['delete'] = mapCacheDelete;
	MapCache.prototype.get = mapCacheGet;
	MapCache.prototype.has = mapCacheHas;
	MapCache.prototype.set = mapCacheSet;

	_MapCache = MapCache;
	return _MapCache;
}

var _stackSet;
var hasRequired_stackSet;

function require_stackSet () {
	if (hasRequired_stackSet) return _stackSet;
	hasRequired_stackSet = 1;
	var ListCache = require_ListCache(),
	    Map = require_Map(),
	    MapCache = require_MapCache();

	/** Used as the size to enable large array optimizations. */
	var LARGE_ARRAY_SIZE = 200;

	/**
	 * Sets the stack `key` to `value`.
	 *
	 * @private
	 * @name set
	 * @memberOf Stack
	 * @param {string} key The key of the value to set.
	 * @param {*} value The value to set.
	 * @returns {Object} Returns the stack cache instance.
	 */
	function stackSet(key, value) {
	  var data = this.__data__;
	  if (data instanceof ListCache) {
	    var pairs = data.__data__;
	    if (!Map || (pairs.length < LARGE_ARRAY_SIZE - 1)) {
	      pairs.push([key, value]);
	      this.size = ++data.size;
	      return this;
	    }
	    data = this.__data__ = new MapCache(pairs);
	  }
	  data.set(key, value);
	  this.size = data.size;
	  return this;
	}

	_stackSet = stackSet;
	return _stackSet;
}

var _Stack;
var hasRequired_Stack;

function require_Stack () {
	if (hasRequired_Stack) return _Stack;
	hasRequired_Stack = 1;
	var ListCache = require_ListCache(),
	    stackClear = require_stackClear(),
	    stackDelete = require_stackDelete(),
	    stackGet = require_stackGet(),
	    stackHas = require_stackHas(),
	    stackSet = require_stackSet();

	/**
	 * Creates a stack cache object to store key-value pairs.
	 *
	 * @private
	 * @constructor
	 * @param {Array} [entries] The key-value pairs to cache.
	 */
	function Stack(entries) {
	  var data = this.__data__ = new ListCache(entries);
	  this.size = data.size;
	}

	// Add methods to `Stack`.
	Stack.prototype.clear = stackClear;
	Stack.prototype['delete'] = stackDelete;
	Stack.prototype.get = stackGet;
	Stack.prototype.has = stackHas;
	Stack.prototype.set = stackSet;

	_Stack = Stack;
	return _Stack;
}

/** Used to stand-in for `undefined` hash values. */

var _setCacheAdd;
var hasRequired_setCacheAdd;

function require_setCacheAdd () {
	if (hasRequired_setCacheAdd) return _setCacheAdd;
	hasRequired_setCacheAdd = 1;
	var HASH_UNDEFINED = '__lodash_hash_undefined__';

	/**
	 * Adds `value` to the array cache.
	 *
	 * @private
	 * @name add
	 * @memberOf SetCache
	 * @alias push
	 * @param {*} value The value to cache.
	 * @returns {Object} Returns the cache instance.
	 */
	function setCacheAdd(value) {
	  this.__data__.set(value, HASH_UNDEFINED);
	  return this;
	}

	_setCacheAdd = setCacheAdd;
	return _setCacheAdd;
}

/**
 * Checks if `value` is in the array cache.
 *
 * @private
 * @name has
 * @memberOf SetCache
 * @param {*} value The value to search for.
 * @returns {number} Returns `true` if `value` is found, else `false`.
 */

var _setCacheHas;
var hasRequired_setCacheHas;

function require_setCacheHas () {
	if (hasRequired_setCacheHas) return _setCacheHas;
	hasRequired_setCacheHas = 1;
	function setCacheHas(value) {
	  return this.__data__.has(value);
	}

	_setCacheHas = setCacheHas;
	return _setCacheHas;
}

var _SetCache;
var hasRequired_SetCache;

function require_SetCache () {
	if (hasRequired_SetCache) return _SetCache;
	hasRequired_SetCache = 1;
	var MapCache = require_MapCache(),
	    setCacheAdd = require_setCacheAdd(),
	    setCacheHas = require_setCacheHas();

	/**
	 *
	 * Creates an array cache object to store unique values.
	 *
	 * @private
	 * @constructor
	 * @param {Array} [values] The values to cache.
	 */
	function SetCache(values) {
	  var index = -1,
	      length = values == null ? 0 : values.length;

	  this.__data__ = new MapCache;
	  while (++index < length) {
	    this.add(values[index]);
	  }
	}

	// Add methods to `SetCache`.
	SetCache.prototype.add = SetCache.prototype.push = setCacheAdd;
	SetCache.prototype.has = setCacheHas;

	_SetCache = SetCache;
	return _SetCache;
}

/**
 * A specialized version of `_.some` for arrays without support for iteratee
 * shorthands.
 *
 * @private
 * @param {Array} [array] The array to iterate over.
 * @param {Function} predicate The function invoked per iteration.
 * @returns {boolean} Returns `true` if any element passes the predicate check,
 *  else `false`.
 */

var _arraySome;
var hasRequired_arraySome;

function require_arraySome () {
	if (hasRequired_arraySome) return _arraySome;
	hasRequired_arraySome = 1;
	function arraySome(array, predicate) {
	  var index = -1,
	      length = array == null ? 0 : array.length;

	  while (++index < length) {
	    if (predicate(array[index], index, array)) {
	      return true;
	    }
	  }
	  return false;
	}

	_arraySome = arraySome;
	return _arraySome;
}

/**
 * Checks if a `cache` value for `key` exists.
 *
 * @private
 * @param {Object} cache The cache to query.
 * @param {string} key The key of the entry to check.
 * @returns {boolean} Returns `true` if an entry for `key` exists, else `false`.
 */

var _cacheHas;
var hasRequired_cacheHas;

function require_cacheHas () {
	if (hasRequired_cacheHas) return _cacheHas;
	hasRequired_cacheHas = 1;
	function cacheHas(cache, key) {
	  return cache.has(key);
	}

	_cacheHas = cacheHas;
	return _cacheHas;
}

var _equalArrays;
var hasRequired_equalArrays;

function require_equalArrays () {
	if (hasRequired_equalArrays) return _equalArrays;
	hasRequired_equalArrays = 1;
	var SetCache = require_SetCache(),
	    arraySome = require_arraySome(),
	    cacheHas = require_cacheHas();

	/** Used to compose bitmasks for value comparisons. */
	var COMPARE_PARTIAL_FLAG = 1,
	    COMPARE_UNORDERED_FLAG = 2;

	/**
	 * A specialized version of `baseIsEqualDeep` for arrays with support for
	 * partial deep comparisons.
	 *
	 * @private
	 * @param {Array} array The array to compare.
	 * @param {Array} other The other array to compare.
	 * @param {number} bitmask The bitmask flags. See `baseIsEqual` for more details.
	 * @param {Function} customizer The function to customize comparisons.
	 * @param {Function} equalFunc The function to determine equivalents of values.
	 * @param {Object} stack Tracks traversed `array` and `other` objects.
	 * @returns {boolean} Returns `true` if the arrays are equivalent, else `false`.
	 */
	function equalArrays(array, other, bitmask, customizer, equalFunc, stack) {
	  var isPartial = bitmask & COMPARE_PARTIAL_FLAG,
	      arrLength = array.length,
	      othLength = other.length;

	  if (arrLength != othLength && !(isPartial && othLength > arrLength)) {
	    return false;
	  }
	  // Check that cyclic values are equal.
	  var arrStacked = stack.get(array);
	  var othStacked = stack.get(other);
	  if (arrStacked && othStacked) {
	    return arrStacked == other && othStacked == array;
	  }
	  var index = -1,
	      result = true,
	      seen = (bitmask & COMPARE_UNORDERED_FLAG) ? new SetCache : undefined;

	  stack.set(array, other);
	  stack.set(other, array);

	  // Ignore non-index properties.
	  while (++index < arrLength) {
	    var arrValue = array[index],
	        othValue = other[index];

	    if (customizer) {
	      var compared = isPartial
	        ? customizer(othValue, arrValue, index, other, array, stack)
	        : customizer(arrValue, othValue, index, array, other, stack);
	    }
	    if (compared !== undefined) {
	      if (compared) {
	        continue;
	      }
	      result = false;
	      break;
	    }
	    // Recursively compare arrays (susceptible to call stack limits).
	    if (seen) {
	      if (!arraySome(other, function(othValue, othIndex) {
	            if (!cacheHas(seen, othIndex) &&
	                (arrValue === othValue || equalFunc(arrValue, othValue, bitmask, customizer, stack))) {
	              return seen.push(othIndex);
	            }
	          })) {
	        result = false;
	        break;
	      }
	    } else if (!(
	          arrValue === othValue ||
	            equalFunc(arrValue, othValue, bitmask, customizer, stack)
	        )) {
	      result = false;
	      break;
	    }
	  }
	  stack['delete'](array);
	  stack['delete'](other);
	  return result;
	}

	_equalArrays = equalArrays;
	return _equalArrays;
}

var _Uint8Array;
var hasRequired_Uint8Array;

function require_Uint8Array () {
	if (hasRequired_Uint8Array) return _Uint8Array;
	hasRequired_Uint8Array = 1;
	var root = require_root();

	/** Built-in value references. */
	var Uint8Array = root.Uint8Array;

	_Uint8Array = Uint8Array;
	return _Uint8Array;
}

/**
 * Converts `map` to its key-value pairs.
 *
 * @private
 * @param {Object} map The map to convert.
 * @returns {Array} Returns the key-value pairs.
 */

var _mapToArray;
var hasRequired_mapToArray;

function require_mapToArray () {
	if (hasRequired_mapToArray) return _mapToArray;
	hasRequired_mapToArray = 1;
	function mapToArray(map) {
	  var index = -1,
	      result = Array(map.size);

	  map.forEach(function(value, key) {
	    result[++index] = [key, value];
	  });
	  return result;
	}

	_mapToArray = mapToArray;
	return _mapToArray;
}

/**
 * Converts `set` to an array of its values.
 *
 * @private
 * @param {Object} set The set to convert.
 * @returns {Array} Returns the values.
 */

var _setToArray;
var hasRequired_setToArray;

function require_setToArray () {
	if (hasRequired_setToArray) return _setToArray;
	hasRequired_setToArray = 1;
	function setToArray(set) {
	  var index = -1,
	      result = Array(set.size);

	  set.forEach(function(value) {
	    result[++index] = value;
	  });
	  return result;
	}

	_setToArray = setToArray;
	return _setToArray;
}

var _equalByTag;
var hasRequired_equalByTag;

function require_equalByTag () {
	if (hasRequired_equalByTag) return _equalByTag;
	hasRequired_equalByTag = 1;
	var Symbol = require_Symbol(),
	    Uint8Array = require_Uint8Array(),
	    eq = requireEq(),
	    equalArrays = require_equalArrays(),
	    mapToArray = require_mapToArray(),
	    setToArray = require_setToArray();

	/** Used to compose bitmasks for value comparisons. */
	var COMPARE_PARTIAL_FLAG = 1,
	    COMPARE_UNORDERED_FLAG = 2;

	/** `Object#toString` result references. */
	var boolTag = '[object Boolean]',
	    dateTag = '[object Date]',
	    errorTag = '[object Error]',
	    mapTag = '[object Map]',
	    numberTag = '[object Number]',
	    regexpTag = '[object RegExp]',
	    setTag = '[object Set]',
	    stringTag = '[object String]',
	    symbolTag = '[object Symbol]';

	var arrayBufferTag = '[object ArrayBuffer]',
	    dataViewTag = '[object DataView]';

	/** Used to convert symbols to primitives and strings. */
	var symbolProto = Symbol ? Symbol.prototype : undefined,
	    symbolValueOf = symbolProto ? symbolProto.valueOf : undefined;

	/**
	 * A specialized version of `baseIsEqualDeep` for comparing objects of
	 * the same `toStringTag`.
	 *
	 * **Note:** This function only supports comparing values with tags of
	 * `Boolean`, `Date`, `Error`, `Number`, `RegExp`, or `String`.
	 *
	 * @private
	 * @param {Object} object The object to compare.
	 * @param {Object} other The other object to compare.
	 * @param {string} tag The `toStringTag` of the objects to compare.
	 * @param {number} bitmask The bitmask flags. See `baseIsEqual` for more details.
	 * @param {Function} customizer The function to customize comparisons.
	 * @param {Function} equalFunc The function to determine equivalents of values.
	 * @param {Object} stack Tracks traversed `object` and `other` objects.
	 * @returns {boolean} Returns `true` if the objects are equivalent, else `false`.
	 */
	function equalByTag(object, other, tag, bitmask, customizer, equalFunc, stack) {
	  switch (tag) {
	    case dataViewTag:
	      if ((object.byteLength != other.byteLength) ||
	          (object.byteOffset != other.byteOffset)) {
	        return false;
	      }
	      object = object.buffer;
	      other = other.buffer;

	    case arrayBufferTag:
	      if ((object.byteLength != other.byteLength) ||
	          !equalFunc(new Uint8Array(object), new Uint8Array(other))) {
	        return false;
	      }
	      return true;

	    case boolTag:
	    case dateTag:
	    case numberTag:
	      // Coerce booleans to `1` or `0` and dates to milliseconds.
	      // Invalid dates are coerced to `NaN`.
	      return eq(+object, +other);

	    case errorTag:
	      return object.name == other.name && object.message == other.message;

	    case regexpTag:
	    case stringTag:
	      // Coerce regexes to strings and treat strings, primitives and objects,
	      // as equal. See http://www.ecma-international.org/ecma-262/7.0/#sec-regexp.prototype.tostring
	      // for more details.
	      return object == (other + '');

	    case mapTag:
	      var convert = mapToArray;

	    case setTag:
	      var isPartial = bitmask & COMPARE_PARTIAL_FLAG;
	      convert || (convert = setToArray);

	      if (object.size != other.size && !isPartial) {
	        return false;
	      }
	      // Assume cyclic values are equal.
	      var stacked = stack.get(object);
	      if (stacked) {
	        return stacked == other;
	      }
	      bitmask |= COMPARE_UNORDERED_FLAG;

	      // Recursively compare objects (susceptible to call stack limits).
	      stack.set(object, other);
	      var result = equalArrays(convert(object), convert(other), bitmask, customizer, equalFunc, stack);
	      stack['delete'](object);
	      return result;

	    case symbolTag:
	      if (symbolValueOf) {
	        return symbolValueOf.call(object) == symbolValueOf.call(other);
	      }
	  }
	  return false;
	}

	_equalByTag = equalByTag;
	return _equalByTag;
}

/**
 * Appends the elements of `values` to `array`.
 *
 * @private
 * @param {Array} array The array to modify.
 * @param {Array} values The values to append.
 * @returns {Array} Returns `array`.
 */

var _arrayPush;
var hasRequired_arrayPush;

function require_arrayPush () {
	if (hasRequired_arrayPush) return _arrayPush;
	hasRequired_arrayPush = 1;
	function arrayPush(array, values) {
	  var index = -1,
	      length = values.length,
	      offset = array.length;

	  while (++index < length) {
	    array[offset + index] = values[index];
	  }
	  return array;
	}

	_arrayPush = arrayPush;
	return _arrayPush;
}

var _baseGetAllKeys;
var hasRequired_baseGetAllKeys;

function require_baseGetAllKeys () {
	if (hasRequired_baseGetAllKeys) return _baseGetAllKeys;
	hasRequired_baseGetAllKeys = 1;
	var arrayPush = require_arrayPush(),
	    isArray = requireIsArray();

	/**
	 * The base implementation of `getAllKeys` and `getAllKeysIn` which uses
	 * `keysFunc` and `symbolsFunc` to get the enumerable property names and
	 * symbols of `object`.
	 *
	 * @private
	 * @param {Object} object The object to query.
	 * @param {Function} keysFunc The function to get the keys of `object`.
	 * @param {Function} symbolsFunc The function to get the symbols of `object`.
	 * @returns {Array} Returns the array of property names and symbols.
	 */
	function baseGetAllKeys(object, keysFunc, symbolsFunc) {
	  var result = keysFunc(object);
	  return isArray(object) ? result : arrayPush(result, symbolsFunc(object));
	}

	_baseGetAllKeys = baseGetAllKeys;
	return _baseGetAllKeys;
}

/**
 * A specialized version of `_.filter` for arrays without support for
 * iteratee shorthands.
 *
 * @private
 * @param {Array} [array] The array to iterate over.
 * @param {Function} predicate The function invoked per iteration.
 * @returns {Array} Returns the new filtered array.
 */

var _arrayFilter;
var hasRequired_arrayFilter;

function require_arrayFilter () {
	if (hasRequired_arrayFilter) return _arrayFilter;
	hasRequired_arrayFilter = 1;
	function arrayFilter(array, predicate) {
	  var index = -1,
	      length = array == null ? 0 : array.length,
	      resIndex = 0,
	      result = [];

	  while (++index < length) {
	    var value = array[index];
	    if (predicate(value, index, array)) {
	      result[resIndex++] = value;
	    }
	  }
	  return result;
	}

	_arrayFilter = arrayFilter;
	return _arrayFilter;
}

/**
 * This method returns a new empty array.
 *
 * @static
 * @memberOf _
 * @since 4.13.0
 * @category Util
 * @returns {Array} Returns the new empty array.
 * @example
 *
 * var arrays = _.times(2, _.stubArray);
 *
 * console.log(arrays);
 * // => [[], []]
 *
 * console.log(arrays[0] === arrays[1]);
 * // => false
 */

var stubArray_1;
var hasRequiredStubArray;

function requireStubArray () {
	if (hasRequiredStubArray) return stubArray_1;
	hasRequiredStubArray = 1;
	function stubArray() {
	  return [];
	}

	stubArray_1 = stubArray;
	return stubArray_1;
}

var _getSymbols;
var hasRequired_getSymbols;

function require_getSymbols () {
	if (hasRequired_getSymbols) return _getSymbols;
	hasRequired_getSymbols = 1;
	var arrayFilter = require_arrayFilter(),
	    stubArray = requireStubArray();

	/** Used for built-in method references. */
	var objectProto = Object.prototype;

	/** Built-in value references. */
	var propertyIsEnumerable = objectProto.propertyIsEnumerable;

	/* Built-in method references for those with the same name as other `lodash` methods. */
	var nativeGetSymbols = Object.getOwnPropertySymbols;

	/**
	 * Creates an array of the own enumerable symbols of `object`.
	 *
	 * @private
	 * @param {Object} object The object to query.
	 * @returns {Array} Returns the array of symbols.
	 */
	var getSymbols = !nativeGetSymbols ? stubArray : function(object) {
	  if (object == null) {
	    return [];
	  }
	  object = Object(object);
	  return arrayFilter(nativeGetSymbols(object), function(symbol) {
	    return propertyIsEnumerable.call(object, symbol);
	  });
	};

	_getSymbols = getSymbols;
	return _getSymbols;
}

/**
 * The base implementation of `_.times` without support for iteratee shorthands
 * or max array length checks.
 *
 * @private
 * @param {number} n The number of times to invoke `iteratee`.
 * @param {Function} iteratee The function invoked per iteration.
 * @returns {Array} Returns the array of results.
 */

var _baseTimes;
var hasRequired_baseTimes;

function require_baseTimes () {
	if (hasRequired_baseTimes) return _baseTimes;
	hasRequired_baseTimes = 1;
	function baseTimes(n, iteratee) {
	  var index = -1,
	      result = Array(n);

	  while (++index < n) {
	    result[index] = iteratee(index);
	  }
	  return result;
	}

	_baseTimes = baseTimes;
	return _baseTimes;
}

/** Used as references for various `Number` constants. */

var _isIndex;
var hasRequired_isIndex;

function require_isIndex () {
	if (hasRequired_isIndex) return _isIndex;
	hasRequired_isIndex = 1;
	var MAX_SAFE_INTEGER = 9007199254740991;

	/** Used to detect unsigned integer values. */
	var reIsUint = /^(?:0|[1-9]\d*)$/;

	/**
	 * Checks if `value` is a valid array-like index.
	 *
	 * @private
	 * @param {*} value The value to check.
	 * @param {number} [length=MAX_SAFE_INTEGER] The upper bounds of a valid index.
	 * @returns {boolean} Returns `true` if `value` is a valid index, else `false`.
	 */
	function isIndex(value, length) {
	  var type = typeof value;
	  length = length == null ? MAX_SAFE_INTEGER : length;

	  return !!length &&
	    (type == 'number' ||
	      (type != 'symbol' && reIsUint.test(value))) &&
	        (value > -1 && value % 1 == 0 && value < length);
	}

	_isIndex = isIndex;
	return _isIndex;
}

var _arrayLikeKeys;
var hasRequired_arrayLikeKeys;

function require_arrayLikeKeys () {
	if (hasRequired_arrayLikeKeys) return _arrayLikeKeys;
	hasRequired_arrayLikeKeys = 1;
	var baseTimes = require_baseTimes(),
	    isArguments = requireIsArguments(),
	    isArray = requireIsArray(),
	    isBuffer = requireIsBuffer(),
	    isIndex = require_isIndex(),
	    isTypedArray = requireIsTypedArray();

	/** Used for built-in method references. */
	var objectProto = Object.prototype;

	/** Used to check objects for own properties. */
	var hasOwnProperty = objectProto.hasOwnProperty;

	/**
	 * Creates an array of the enumerable property names of the array-like `value`.
	 *
	 * @private
	 * @param {*} value The value to query.
	 * @param {boolean} inherited Specify returning inherited property names.
	 * @returns {Array} Returns the array of property names.
	 */
	function arrayLikeKeys(value, inherited) {
	  var isArr = isArray(value),
	      isArg = !isArr && isArguments(value),
	      isBuff = !isArr && !isArg && isBuffer(value),
	      isType = !isArr && !isArg && !isBuff && isTypedArray(value),
	      skipIndexes = isArr || isArg || isBuff || isType,
	      result = skipIndexes ? baseTimes(value.length, String) : [],
	      length = result.length;

	  for (var key in value) {
	    if ((inherited || hasOwnProperty.call(value, key)) &&
	        !(skipIndexes && (
	           // Safari 9 has enumerable `arguments.length` in strict mode.
	           key == 'length' ||
	           // Node.js 0.10 has enumerable non-index properties on buffers.
	           (isBuff && (key == 'offset' || key == 'parent')) ||
	           // PhantomJS 2 has enumerable non-index properties on typed arrays.
	           (isType && (key == 'buffer' || key == 'byteLength' || key == 'byteOffset')) ||
	           // Skip index properties.
	           isIndex(key, length)
	        ))) {
	      result.push(key);
	    }
	  }
	  return result;
	}

	_arrayLikeKeys = arrayLikeKeys;
	return _arrayLikeKeys;
}

var keys_1;
var hasRequiredKeys$1;

function requireKeys$1 () {
	if (hasRequiredKeys$1) return keys_1;
	hasRequiredKeys$1 = 1;
	var arrayLikeKeys = require_arrayLikeKeys(),
	    baseKeys = require_baseKeys(),
	    isArrayLike = requireIsArrayLike();

	/**
	 * Creates an array of the own enumerable property names of `object`.
	 *
	 * **Note:** Non-object values are coerced to objects. See the
	 * [ES spec](http://ecma-international.org/ecma-262/7.0/#sec-object.keys)
	 * for more details.
	 *
	 * @static
	 * @since 0.1.0
	 * @memberOf _
	 * @category Object
	 * @param {Object} object The object to query.
	 * @returns {Array} Returns the array of property names.
	 * @example
	 *
	 * function Foo() {
	 *   this.a = 1;
	 *   this.b = 2;
	 * }
	 *
	 * Foo.prototype.c = 3;
	 *
	 * _.keys(new Foo);
	 * // => ['a', 'b'] (iteration order is not guaranteed)
	 *
	 * _.keys('hi');
	 * // => ['0', '1']
	 */
	function keys(object) {
	  return isArrayLike(object) ? arrayLikeKeys(object) : baseKeys(object);
	}

	keys_1 = keys;
	return keys_1;
}

var _getAllKeys;
var hasRequired_getAllKeys;

function require_getAllKeys () {
	if (hasRequired_getAllKeys) return _getAllKeys;
	hasRequired_getAllKeys = 1;
	var baseGetAllKeys = require_baseGetAllKeys(),
	    getSymbols = require_getSymbols(),
	    keys = requireKeys$1();

	/**
	 * Creates an array of own enumerable property names and symbols of `object`.
	 *
	 * @private
	 * @param {Object} object The object to query.
	 * @returns {Array} Returns the array of property names and symbols.
	 */
	function getAllKeys(object) {
	  return baseGetAllKeys(object, keys, getSymbols);
	}

	_getAllKeys = getAllKeys;
	return _getAllKeys;
}

var _equalObjects;
var hasRequired_equalObjects;

function require_equalObjects () {
	if (hasRequired_equalObjects) return _equalObjects;
	hasRequired_equalObjects = 1;
	var getAllKeys = require_getAllKeys();

	/** Used to compose bitmasks for value comparisons. */
	var COMPARE_PARTIAL_FLAG = 1;

	/** Used for built-in method references. */
	var objectProto = Object.prototype;

	/** Used to check objects for own properties. */
	var hasOwnProperty = objectProto.hasOwnProperty;

	/**
	 * A specialized version of `baseIsEqualDeep` for objects with support for
	 * partial deep comparisons.
	 *
	 * @private
	 * @param {Object} object The object to compare.
	 * @param {Object} other The other object to compare.
	 * @param {number} bitmask The bitmask flags. See `baseIsEqual` for more details.
	 * @param {Function} customizer The function to customize comparisons.
	 * @param {Function} equalFunc The function to determine equivalents of values.
	 * @param {Object} stack Tracks traversed `object` and `other` objects.
	 * @returns {boolean} Returns `true` if the objects are equivalent, else `false`.
	 */
	function equalObjects(object, other, bitmask, customizer, equalFunc, stack) {
	  var isPartial = bitmask & COMPARE_PARTIAL_FLAG,
	      objProps = getAllKeys(object),
	      objLength = objProps.length,
	      othProps = getAllKeys(other),
	      othLength = othProps.length;

	  if (objLength != othLength && !isPartial) {
	    return false;
	  }
	  var index = objLength;
	  while (index--) {
	    var key = objProps[index];
	    if (!(isPartial ? key in other : hasOwnProperty.call(other, key))) {
	      return false;
	    }
	  }
	  // Check that cyclic values are equal.
	  var objStacked = stack.get(object);
	  var othStacked = stack.get(other);
	  if (objStacked && othStacked) {
	    return objStacked == other && othStacked == object;
	  }
	  var result = true;
	  stack.set(object, other);
	  stack.set(other, object);

	  var skipCtor = isPartial;
	  while (++index < objLength) {
	    key = objProps[index];
	    var objValue = object[key],
	        othValue = other[key];

	    if (customizer) {
	      var compared = isPartial
	        ? customizer(othValue, objValue, key, other, object, stack)
	        : customizer(objValue, othValue, key, object, other, stack);
	    }
	    // Recursively compare objects (susceptible to call stack limits).
	    if (!(compared === undefined
	          ? (objValue === othValue || equalFunc(objValue, othValue, bitmask, customizer, stack))
	          : compared
	        )) {
	      result = false;
	      break;
	    }
	    skipCtor || (skipCtor = key == 'constructor');
	  }
	  if (result && !skipCtor) {
	    var objCtor = object.constructor,
	        othCtor = other.constructor;

	    // Non `Object` object instances with different constructors are not equal.
	    if (objCtor != othCtor &&
	        ('constructor' in object && 'constructor' in other) &&
	        !(typeof objCtor == 'function' && objCtor instanceof objCtor &&
	          typeof othCtor == 'function' && othCtor instanceof othCtor)) {
	      result = false;
	    }
	  }
	  stack['delete'](object);
	  stack['delete'](other);
	  return result;
	}

	_equalObjects = equalObjects;
	return _equalObjects;
}

var _baseIsEqualDeep;
var hasRequired_baseIsEqualDeep;

function require_baseIsEqualDeep () {
	if (hasRequired_baseIsEqualDeep) return _baseIsEqualDeep;
	hasRequired_baseIsEqualDeep = 1;
	var Stack = require_Stack(),
	    equalArrays = require_equalArrays(),
	    equalByTag = require_equalByTag(),
	    equalObjects = require_equalObjects(),
	    getTag = require_getTag(),
	    isArray = requireIsArray(),
	    isBuffer = requireIsBuffer(),
	    isTypedArray = requireIsTypedArray();

	/** Used to compose bitmasks for value comparisons. */
	var COMPARE_PARTIAL_FLAG = 1;

	/** `Object#toString` result references. */
	var argsTag = '[object Arguments]',
	    arrayTag = '[object Array]',
	    objectTag = '[object Object]';

	/** Used for built-in method references. */
	var objectProto = Object.prototype;

	/** Used to check objects for own properties. */
	var hasOwnProperty = objectProto.hasOwnProperty;

	/**
	 * A specialized version of `baseIsEqual` for arrays and objects which performs
	 * deep comparisons and tracks traversed objects enabling objects with circular
	 * references to be compared.
	 *
	 * @private
	 * @param {Object} object The object to compare.
	 * @param {Object} other The other object to compare.
	 * @param {number} bitmask The bitmask flags. See `baseIsEqual` for more details.
	 * @param {Function} customizer The function to customize comparisons.
	 * @param {Function} equalFunc The function to determine equivalents of values.
	 * @param {Object} [stack] Tracks traversed `object` and `other` objects.
	 * @returns {boolean} Returns `true` if the objects are equivalent, else `false`.
	 */
	function baseIsEqualDeep(object, other, bitmask, customizer, equalFunc, stack) {
	  var objIsArr = isArray(object),
	      othIsArr = isArray(other),
	      objTag = objIsArr ? arrayTag : getTag(object),
	      othTag = othIsArr ? arrayTag : getTag(other);

	  objTag = objTag == argsTag ? objectTag : objTag;
	  othTag = othTag == argsTag ? objectTag : othTag;

	  var objIsObj = objTag == objectTag,
	      othIsObj = othTag == objectTag,
	      isSameTag = objTag == othTag;

	  if (isSameTag && isBuffer(object)) {
	    if (!isBuffer(other)) {
	      return false;
	    }
	    objIsArr = true;
	    objIsObj = false;
	  }
	  if (isSameTag && !objIsObj) {
	    stack || (stack = new Stack);
	    return (objIsArr || isTypedArray(object))
	      ? equalArrays(object, other, bitmask, customizer, equalFunc, stack)
	      : equalByTag(object, other, objTag, bitmask, customizer, equalFunc, stack);
	  }
	  if (!(bitmask & COMPARE_PARTIAL_FLAG)) {
	    var objIsWrapped = objIsObj && hasOwnProperty.call(object, '__wrapped__'),
	        othIsWrapped = othIsObj && hasOwnProperty.call(other, '__wrapped__');

	    if (objIsWrapped || othIsWrapped) {
	      var objUnwrapped = objIsWrapped ? object.value() : object,
	          othUnwrapped = othIsWrapped ? other.value() : other;

	      stack || (stack = new Stack);
	      return equalFunc(objUnwrapped, othUnwrapped, bitmask, customizer, stack);
	    }
	  }
	  if (!isSameTag) {
	    return false;
	  }
	  stack || (stack = new Stack);
	  return equalObjects(object, other, bitmask, customizer, equalFunc, stack);
	}

	_baseIsEqualDeep = baseIsEqualDeep;
	return _baseIsEqualDeep;
}

var _baseIsEqual;
var hasRequired_baseIsEqual;

function require_baseIsEqual () {
	if (hasRequired_baseIsEqual) return _baseIsEqual;
	hasRequired_baseIsEqual = 1;
	var baseIsEqualDeep = require_baseIsEqualDeep(),
	    isObjectLike = requireIsObjectLike();

	/**
	 * The base implementation of `_.isEqual` which supports partial comparisons
	 * and tracks traversed objects.
	 *
	 * @private
	 * @param {*} value The value to compare.
	 * @param {*} other The other value to compare.
	 * @param {boolean} bitmask The bitmask flags.
	 *  1 - Unordered comparison
	 *  2 - Partial comparison
	 * @param {Function} [customizer] The function to customize comparisons.
	 * @param {Object} [stack] Tracks traversed `value` and `other` objects.
	 * @returns {boolean} Returns `true` if the values are equivalent, else `false`.
	 */
	function baseIsEqual(value, other, bitmask, customizer, stack) {
	  if (value === other) {
	    return true;
	  }
	  if (value == null || other == null || (!isObjectLike(value) && !isObjectLike(other))) {
	    return value !== value && other !== other;
	  }
	  return baseIsEqualDeep(value, other, bitmask, customizer, baseIsEqual, stack);
	}

	_baseIsEqual = baseIsEqual;
	return _baseIsEqual;
}

var _baseIsMatch;
var hasRequired_baseIsMatch;

function require_baseIsMatch () {
	if (hasRequired_baseIsMatch) return _baseIsMatch;
	hasRequired_baseIsMatch = 1;
	var Stack = require_Stack(),
	    baseIsEqual = require_baseIsEqual();

	/** Used to compose bitmasks for value comparisons. */
	var COMPARE_PARTIAL_FLAG = 1,
	    COMPARE_UNORDERED_FLAG = 2;

	/**
	 * The base implementation of `_.isMatch` without support for iteratee shorthands.
	 *
	 * @private
	 * @param {Object} object The object to inspect.
	 * @param {Object} source The object of property values to match.
	 * @param {Array} matchData The property names, values, and compare flags to match.
	 * @param {Function} [customizer] The function to customize comparisons.
	 * @returns {boolean} Returns `true` if `object` is a match, else `false`.
	 */
	function baseIsMatch(object, source, matchData, customizer) {
	  var index = matchData.length,
	      length = index,
	      noCustomizer = !customizer;

	  if (object == null) {
	    return !length;
	  }
	  object = Object(object);
	  while (index--) {
	    var data = matchData[index];
	    if ((noCustomizer && data[2])
	          ? data[1] !== object[data[0]]
	          : !(data[0] in object)
	        ) {
	      return false;
	    }
	  }
	  while (++index < length) {
	    data = matchData[index];
	    var key = data[0],
	        objValue = object[key],
	        srcValue = data[1];

	    if (noCustomizer && data[2]) {
	      if (objValue === undefined && !(key in object)) {
	        return false;
	      }
	    } else {
	      var stack = new Stack;
	      if (customizer) {
	        var result = customizer(objValue, srcValue, key, object, source, stack);
	      }
	      if (!(result === undefined
	            ? baseIsEqual(srcValue, objValue, COMPARE_PARTIAL_FLAG | COMPARE_UNORDERED_FLAG, customizer, stack)
	            : result
	          )) {
	        return false;
	      }
	    }
	  }
	  return true;
	}

	_baseIsMatch = baseIsMatch;
	return _baseIsMatch;
}

var _isStrictComparable;
var hasRequired_isStrictComparable;

function require_isStrictComparable () {
	if (hasRequired_isStrictComparable) return _isStrictComparable;
	hasRequired_isStrictComparable = 1;
	var isObject = requireIsObject();

	/**
	 * Checks if `value` is suitable for strict equality comparisons, i.e. `===`.
	 *
	 * @private
	 * @param {*} value The value to check.
	 * @returns {boolean} Returns `true` if `value` if suitable for strict
	 *  equality comparisons, else `false`.
	 */
	function isStrictComparable(value) {
	  return value === value && !isObject(value);
	}

	_isStrictComparable = isStrictComparable;
	return _isStrictComparable;
}

var _getMatchData;
var hasRequired_getMatchData;

function require_getMatchData () {
	if (hasRequired_getMatchData) return _getMatchData;
	hasRequired_getMatchData = 1;
	var isStrictComparable = require_isStrictComparable(),
	    keys = requireKeys$1();

	/**
	 * Gets the property names, values, and compare flags of `object`.
	 *
	 * @private
	 * @param {Object} object The object to query.
	 * @returns {Array} Returns the match data of `object`.
	 */
	function getMatchData(object) {
	  var result = keys(object),
	      length = result.length;

	  while (length--) {
	    var key = result[length],
	        value = object[key];

	    result[length] = [key, value, isStrictComparable(value)];
	  }
	  return result;
	}

	_getMatchData = getMatchData;
	return _getMatchData;
}

/**
 * A specialized version of `matchesProperty` for source values suitable
 * for strict equality comparisons, i.e. `===`.
 *
 * @private
 * @param {string} key The key of the property to get.
 * @param {*} srcValue The value to match.
 * @returns {Function} Returns the new spec function.
 */

var _matchesStrictComparable;
var hasRequired_matchesStrictComparable;

function require_matchesStrictComparable () {
	if (hasRequired_matchesStrictComparable) return _matchesStrictComparable;
	hasRequired_matchesStrictComparable = 1;
	function matchesStrictComparable(key, srcValue) {
	  return function(object) {
	    if (object == null) {
	      return false;
	    }
	    return object[key] === srcValue &&
	      (srcValue !== undefined || (key in Object(object)));
	  };
	}

	_matchesStrictComparable = matchesStrictComparable;
	return _matchesStrictComparable;
}

var _baseMatches;
var hasRequired_baseMatches;

function require_baseMatches () {
	if (hasRequired_baseMatches) return _baseMatches;
	hasRequired_baseMatches = 1;
	var baseIsMatch = require_baseIsMatch(),
	    getMatchData = require_getMatchData(),
	    matchesStrictComparable = require_matchesStrictComparable();

	/**
	 * The base implementation of `_.matches` which doesn't clone `source`.
	 *
	 * @private
	 * @param {Object} source The object of property values to match.
	 * @returns {Function} Returns the new spec function.
	 */
	function baseMatches(source) {
	  var matchData = getMatchData(source);
	  if (matchData.length == 1 && matchData[0][2]) {
	    return matchesStrictComparable(matchData[0][0], matchData[0][1]);
	  }
	  return function(object) {
	    return object === source || baseIsMatch(object, source, matchData);
	  };
	}

	_baseMatches = baseMatches;
	return _baseMatches;
}

var isSymbol_1;
var hasRequiredIsSymbol;

function requireIsSymbol () {
	if (hasRequiredIsSymbol) return isSymbol_1;
	hasRequiredIsSymbol = 1;
	var baseGetTag = require_baseGetTag(),
	    isObjectLike = requireIsObjectLike();

	/** `Object#toString` result references. */
	var symbolTag = '[object Symbol]';

	/**
	 * Checks if `value` is classified as a `Symbol` primitive or object.
	 *
	 * @static
	 * @memberOf _
	 * @since 4.0.0
	 * @category Lang
	 * @param {*} value The value to check.
	 * @returns {boolean} Returns `true` if `value` is a symbol, else `false`.
	 * @example
	 *
	 * _.isSymbol(Symbol.iterator);
	 * // => true
	 *
	 * _.isSymbol('abc');
	 * // => false
	 */
	function isSymbol(value) {
	  return typeof value == 'symbol' ||
	    (isObjectLike(value) && baseGetTag(value) == symbolTag);
	}

	isSymbol_1 = isSymbol;
	return isSymbol_1;
}

var _isKey;
var hasRequired_isKey;

function require_isKey () {
	if (hasRequired_isKey) return _isKey;
	hasRequired_isKey = 1;
	var isArray = requireIsArray(),
	    isSymbol = requireIsSymbol();

	/** Used to match property names within property paths. */
	var reIsDeepProp = /\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,
	    reIsPlainProp = /^\w*$/;

	/**
	 * Checks if `value` is a property name and not a property path.
	 *
	 * @private
	 * @param {*} value The value to check.
	 * @param {Object} [object] The object to query keys on.
	 * @returns {boolean} Returns `true` if `value` is a property name, else `false`.
	 */
	function isKey(value, object) {
	  if (isArray(value)) {
	    return false;
	  }
	  var type = typeof value;
	  if (type == 'number' || type == 'symbol' || type == 'boolean' ||
	      value == null || isSymbol(value)) {
	    return true;
	  }
	  return reIsPlainProp.test(value) || !reIsDeepProp.test(value) ||
	    (object != null && value in Object(object));
	}

	_isKey = isKey;
	return _isKey;
}

var memoize_1;
var hasRequiredMemoize;

function requireMemoize () {
	if (hasRequiredMemoize) return memoize_1;
	hasRequiredMemoize = 1;
	var MapCache = require_MapCache();

	/** Error message constants. */
	var FUNC_ERROR_TEXT = 'Expected a function';

	/**
	 * Creates a function that memoizes the result of `func`. If `resolver` is
	 * provided, it determines the cache key for storing the result based on the
	 * arguments provided to the memoized function. By default, the first argument
	 * provided to the memoized function is used as the map cache key. The `func`
	 * is invoked with the `this` binding of the memoized function.
	 *
	 * **Note:** The cache is exposed as the `cache` property on the memoized
	 * function. Its creation may be customized by replacing the `_.memoize.Cache`
	 * constructor with one whose instances implement the
	 * [`Map`](http://ecma-international.org/ecma-262/7.0/#sec-properties-of-the-map-prototype-object)
	 * method interface of `clear`, `delete`, `get`, `has`, and `set`.
	 *
	 * @static
	 * @memberOf _
	 * @since 0.1.0
	 * @category Function
	 * @param {Function} func The function to have its output memoized.
	 * @param {Function} [resolver] The function to resolve the cache key.
	 * @returns {Function} Returns the new memoized function.
	 * @example
	 *
	 * var object = { 'a': 1, 'b': 2 };
	 * var other = { 'c': 3, 'd': 4 };
	 *
	 * var values = _.memoize(_.values);
	 * values(object);
	 * // => [1, 2]
	 *
	 * values(other);
	 * // => [3, 4]
	 *
	 * object.a = 2;
	 * values(object);
	 * // => [1, 2]
	 *
	 * // Modify the result cache.
	 * values.cache.set(object, ['a', 'b']);
	 * values(object);
	 * // => ['a', 'b']
	 *
	 * // Replace `_.memoize.Cache`.
	 * _.memoize.Cache = WeakMap;
	 */
	function memoize(func, resolver) {
	  if (typeof func != 'function' || (resolver != null && typeof resolver != 'function')) {
	    throw new TypeError(FUNC_ERROR_TEXT);
	  }
	  var memoized = function() {
	    var args = arguments,
	        key = resolver ? resolver.apply(this, args) : args[0],
	        cache = memoized.cache;

	    if (cache.has(key)) {
	      return cache.get(key);
	    }
	    var result = func.apply(this, args);
	    memoized.cache = cache.set(key, result) || cache;
	    return result;
	  };
	  memoized.cache = new (memoize.Cache || MapCache);
	  return memoized;
	}

	// Expose `MapCache`.
	memoize.Cache = MapCache;

	memoize_1 = memoize;
	return memoize_1;
}

var _memoizeCapped;
var hasRequired_memoizeCapped;

function require_memoizeCapped () {
	if (hasRequired_memoizeCapped) return _memoizeCapped;
	hasRequired_memoizeCapped = 1;
	var memoize = requireMemoize();

	/** Used as the maximum memoize cache size. */
	var MAX_MEMOIZE_SIZE = 500;

	/**
	 * A specialized version of `_.memoize` which clears the memoized function's
	 * cache when it exceeds `MAX_MEMOIZE_SIZE`.
	 *
	 * @private
	 * @param {Function} func The function to have its output memoized.
	 * @returns {Function} Returns the new memoized function.
	 */
	function memoizeCapped(func) {
	  var result = memoize(func, function(key) {
	    if (cache.size === MAX_MEMOIZE_SIZE) {
	      cache.clear();
	    }
	    return key;
	  });

	  var cache = result.cache;
	  return result;
	}

	_memoizeCapped = memoizeCapped;
	return _memoizeCapped;
}

var _stringToPath;
var hasRequired_stringToPath;

function require_stringToPath () {
	if (hasRequired_stringToPath) return _stringToPath;
	hasRequired_stringToPath = 1;
	var memoizeCapped = require_memoizeCapped();

	/** Used to match property names within property paths. */
	var rePropName = /[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g;

	/** Used to match backslashes in property paths. */
	var reEscapeChar = /\\(\\)?/g;

	/**
	 * Converts `string` to a property path array.
	 *
	 * @private
	 * @param {string} string The string to convert.
	 * @returns {Array} Returns the property path array.
	 */
	var stringToPath = memoizeCapped(function(string) {
	  var result = [];
	  if (string.charCodeAt(0) === 46 /* . */) {
	    result.push('');
	  }
	  string.replace(rePropName, function(match, number, quote, subString) {
	    result.push(quote ? subString.replace(reEscapeChar, '$1') : (number || match));
	  });
	  return result;
	});

	_stringToPath = stringToPath;
	return _stringToPath;
}

var _baseToString;
var hasRequired_baseToString;

function require_baseToString () {
	if (hasRequired_baseToString) return _baseToString;
	hasRequired_baseToString = 1;
	var Symbol = require_Symbol(),
	    arrayMap = require_arrayMap(),
	    isArray = requireIsArray(),
	    isSymbol = requireIsSymbol();

	/** Used as references for various `Number` constants. */
	var INFINITY = 1 / 0;

	/** Used to convert symbols to primitives and strings. */
	var symbolProto = Symbol ? Symbol.prototype : undefined,
	    symbolToString = symbolProto ? symbolProto.toString : undefined;

	/**
	 * The base implementation of `_.toString` which doesn't convert nullish
	 * values to empty strings.
	 *
	 * @private
	 * @param {*} value The value to process.
	 * @returns {string} Returns the string.
	 */
	function baseToString(value) {
	  // Exit early for strings to avoid a performance hit in some environments.
	  if (typeof value == 'string') {
	    return value;
	  }
	  if (isArray(value)) {
	    // Recursively convert values (susceptible to call stack limits).
	    return arrayMap(value, baseToString) + '';
	  }
	  if (isSymbol(value)) {
	    return symbolToString ? symbolToString.call(value) : '';
	  }
	  var result = (value + '');
	  return (result == '0' && (1 / value) == -INFINITY) ? '-0' : result;
	}

	_baseToString = baseToString;
	return _baseToString;
}

var toString_1;
var hasRequiredToString;

function requireToString () {
	if (hasRequiredToString) return toString_1;
	hasRequiredToString = 1;
	var baseToString = require_baseToString();

	/**
	 * Converts `value` to a string. An empty string is returned for `null`
	 * and `undefined` values. The sign of `-0` is preserved.
	 *
	 * @static
	 * @memberOf _
	 * @since 4.0.0
	 * @category Lang
	 * @param {*} value The value to convert.
	 * @returns {string} Returns the converted string.
	 * @example
	 *
	 * _.toString(null);
	 * // => ''
	 *
	 * _.toString(-0);
	 * // => '-0'
	 *
	 * _.toString([1, 2, 3]);
	 * // => '1,2,3'
	 */
	function toString(value) {
	  return value == null ? '' : baseToString(value);
	}

	toString_1 = toString;
	return toString_1;
}

var _castPath;
var hasRequired_castPath;

function require_castPath () {
	if (hasRequired_castPath) return _castPath;
	hasRequired_castPath = 1;
	var isArray = requireIsArray(),
	    isKey = require_isKey(),
	    stringToPath = require_stringToPath(),
	    toString = requireToString();

	/**
	 * Casts `value` to a path array if it's not one.
	 *
	 * @private
	 * @param {*} value The value to inspect.
	 * @param {Object} [object] The object to query keys on.
	 * @returns {Array} Returns the cast property path array.
	 */
	function castPath(value, object) {
	  if (isArray(value)) {
	    return value;
	  }
	  return isKey(value, object) ? [value] : stringToPath(toString(value));
	}

	_castPath = castPath;
	return _castPath;
}

var _toKey;
var hasRequired_toKey;

function require_toKey () {
	if (hasRequired_toKey) return _toKey;
	hasRequired_toKey = 1;
	var isSymbol = requireIsSymbol();

	/** Used as references for various `Number` constants. */
	var INFINITY = 1 / 0;

	/**
	 * Converts `value` to a string key if it's not a string or symbol.
	 *
	 * @private
	 * @param {*} value The value to inspect.
	 * @returns {string|symbol} Returns the key.
	 */
	function toKey(value) {
	  if (typeof value == 'string' || isSymbol(value)) {
	    return value;
	  }
	  var result = (value + '');
	  return (result == '0' && (1 / value) == -INFINITY) ? '-0' : result;
	}

	_toKey = toKey;
	return _toKey;
}

var _baseGet;
var hasRequired_baseGet;

function require_baseGet () {
	if (hasRequired_baseGet) return _baseGet;
	hasRequired_baseGet = 1;
	var castPath = require_castPath(),
	    toKey = require_toKey();

	/**
	 * The base implementation of `_.get` without support for default values.
	 *
	 * @private
	 * @param {Object} object The object to query.
	 * @param {Array|string} path The path of the property to get.
	 * @returns {*} Returns the resolved value.
	 */
	function baseGet(object, path) {
	  path = castPath(path, object);

	  var index = 0,
	      length = path.length;

	  while (object != null && index < length) {
	    object = object[toKey(path[index++])];
	  }
	  return (index && index == length) ? object : undefined;
	}

	_baseGet = baseGet;
	return _baseGet;
}

var get_1;
var hasRequiredGet;

function requireGet () {
	if (hasRequiredGet) return get_1;
	hasRequiredGet = 1;
	var baseGet = require_baseGet();

	/**
	 * Gets the value at `path` of `object`. If the resolved value is
	 * `undefined`, the `defaultValue` is returned in its place.
	 *
	 * @static
	 * @memberOf _
	 * @since 3.7.0
	 * @category Object
	 * @param {Object} object The object to query.
	 * @param {Array|string} path The path of the property to get.
	 * @param {*} [defaultValue] The value returned for `undefined` resolved values.
	 * @returns {*} Returns the resolved value.
	 * @example
	 *
	 * var object = { 'a': [{ 'b': { 'c': 3 } }] };
	 *
	 * _.get(object, 'a[0].b.c');
	 * // => 3
	 *
	 * _.get(object, ['a', '0', 'b', 'c']);
	 * // => 3
	 *
	 * _.get(object, 'a.b.c', 'default');
	 * // => 'default'
	 */
	function get(object, path, defaultValue) {
	  var result = object == null ? undefined : baseGet(object, path);
	  return result === undefined ? defaultValue : result;
	}

	get_1 = get;
	return get_1;
}

/**
 * The base implementation of `_.hasIn` without support for deep paths.
 *
 * @private
 * @param {Object} [object] The object to query.
 * @param {Array|string} key The key to check.
 * @returns {boolean} Returns `true` if `key` exists, else `false`.
 */

var _baseHasIn;
var hasRequired_baseHasIn;

function require_baseHasIn () {
	if (hasRequired_baseHasIn) return _baseHasIn;
	hasRequired_baseHasIn = 1;
	function baseHasIn(object, key) {
	  return object != null && key in Object(object);
	}

	_baseHasIn = baseHasIn;
	return _baseHasIn;
}

var _hasPath;
var hasRequired_hasPath;

function require_hasPath () {
	if (hasRequired_hasPath) return _hasPath;
	hasRequired_hasPath = 1;
	var castPath = require_castPath(),
	    isArguments = requireIsArguments(),
	    isArray = requireIsArray(),
	    isIndex = require_isIndex(),
	    isLength = requireIsLength(),
	    toKey = require_toKey();

	/**
	 * Checks if `path` exists on `object`.
	 *
	 * @private
	 * @param {Object} object The object to query.
	 * @param {Array|string} path The path to check.
	 * @param {Function} hasFunc The function to check properties.
	 * @returns {boolean} Returns `true` if `path` exists, else `false`.
	 */
	function hasPath(object, path, hasFunc) {
	  path = castPath(path, object);

	  var index = -1,
	      length = path.length,
	      result = false;

	  while (++index < length) {
	    var key = toKey(path[index]);
	    if (!(result = object != null && hasFunc(object, key))) {
	      break;
	    }
	    object = object[key];
	  }
	  if (result || ++index != length) {
	    return result;
	  }
	  length = object == null ? 0 : object.length;
	  return !!length && isLength(length) && isIndex(key, length) &&
	    (isArray(object) || isArguments(object));
	}

	_hasPath = hasPath;
	return _hasPath;
}

var hasIn_1;
var hasRequiredHasIn;

function requireHasIn () {
	if (hasRequiredHasIn) return hasIn_1;
	hasRequiredHasIn = 1;
	var baseHasIn = require_baseHasIn(),
	    hasPath = require_hasPath();

	/**
	 * Checks if `path` is a direct or inherited property of `object`.
	 *
	 * @static
	 * @memberOf _
	 * @since 4.0.0
	 * @category Object
	 * @param {Object} object The object to query.
	 * @param {Array|string} path The path to check.
	 * @returns {boolean} Returns `true` if `path` exists, else `false`.
	 * @example
	 *
	 * var object = _.create({ 'a': _.create({ 'b': 2 }) });
	 *
	 * _.hasIn(object, 'a');
	 * // => true
	 *
	 * _.hasIn(object, 'a.b');
	 * // => true
	 *
	 * _.hasIn(object, ['a', 'b']);
	 * // => true
	 *
	 * _.hasIn(object, 'b');
	 * // => false
	 */
	function hasIn(object, path) {
	  return object != null && hasPath(object, path, baseHasIn);
	}

	hasIn_1 = hasIn;
	return hasIn_1;
}

var _baseMatchesProperty;
var hasRequired_baseMatchesProperty;

function require_baseMatchesProperty () {
	if (hasRequired_baseMatchesProperty) return _baseMatchesProperty;
	hasRequired_baseMatchesProperty = 1;
	var baseIsEqual = require_baseIsEqual(),
	    get = requireGet(),
	    hasIn = requireHasIn(),
	    isKey = require_isKey(),
	    isStrictComparable = require_isStrictComparable(),
	    matchesStrictComparable = require_matchesStrictComparable(),
	    toKey = require_toKey();

	/** Used to compose bitmasks for value comparisons. */
	var COMPARE_PARTIAL_FLAG = 1,
	    COMPARE_UNORDERED_FLAG = 2;

	/**
	 * The base implementation of `_.matchesProperty` which doesn't clone `srcValue`.
	 *
	 * @private
	 * @param {string} path The path of the property to get.
	 * @param {*} srcValue The value to match.
	 * @returns {Function} Returns the new spec function.
	 */
	function baseMatchesProperty(path, srcValue) {
	  if (isKey(path) && isStrictComparable(srcValue)) {
	    return matchesStrictComparable(toKey(path), srcValue);
	  }
	  return function(object) {
	    var objValue = get(object, path);
	    return (objValue === undefined && objValue === srcValue)
	      ? hasIn(object, path)
	      : baseIsEqual(srcValue, objValue, COMPARE_PARTIAL_FLAG | COMPARE_UNORDERED_FLAG);
	  };
	}

	_baseMatchesProperty = baseMatchesProperty;
	return _baseMatchesProperty;
}

/**
 * This method returns the first argument it receives.
 *
 * @static
 * @since 0.1.0
 * @memberOf _
 * @category Util
 * @param {*} value Any value.
 * @returns {*} Returns `value`.
 * @example
 *
 * var object = { 'a': 1 };
 *
 * console.log(_.identity(object) === object);
 * // => true
 */

var identity_1;
var hasRequiredIdentity;

function requireIdentity () {
	if (hasRequiredIdentity) return identity_1;
	hasRequiredIdentity = 1;
	function identity(value) {
	  return value;
	}

	identity_1 = identity;
	return identity_1;
}

/**
 * The base implementation of `_.property` without support for deep paths.
 *
 * @private
 * @param {string} key The key of the property to get.
 * @returns {Function} Returns the new accessor function.
 */

var _baseProperty;
var hasRequired_baseProperty;

function require_baseProperty () {
	if (hasRequired_baseProperty) return _baseProperty;
	hasRequired_baseProperty = 1;
	function baseProperty(key) {
	  return function(object) {
	    return object == null ? undefined : object[key];
	  };
	}

	_baseProperty = baseProperty;
	return _baseProperty;
}

var _basePropertyDeep;
var hasRequired_basePropertyDeep;

function require_basePropertyDeep () {
	if (hasRequired_basePropertyDeep) return _basePropertyDeep;
	hasRequired_basePropertyDeep = 1;
	var baseGet = require_baseGet();

	/**
	 * A specialized version of `baseProperty` which supports deep paths.
	 *
	 * @private
	 * @param {Array|string} path The path of the property to get.
	 * @returns {Function} Returns the new accessor function.
	 */
	function basePropertyDeep(path) {
	  return function(object) {
	    return baseGet(object, path);
	  };
	}

	_basePropertyDeep = basePropertyDeep;
	return _basePropertyDeep;
}

var property_1;
var hasRequiredProperty;

function requireProperty () {
	if (hasRequiredProperty) return property_1;
	hasRequiredProperty = 1;
	var baseProperty = require_baseProperty(),
	    basePropertyDeep = require_basePropertyDeep(),
	    isKey = require_isKey(),
	    toKey = require_toKey();

	/**
	 * Creates a function that returns the value at `path` of a given object.
	 *
	 * @static
	 * @memberOf _
	 * @since 2.4.0
	 * @category Util
	 * @param {Array|string} path The path of the property to get.
	 * @returns {Function} Returns the new accessor function.
	 * @example
	 *
	 * var objects = [
	 *   { 'a': { 'b': 2 } },
	 *   { 'a': { 'b': 1 } }
	 * ];
	 *
	 * _.map(objects, _.property('a.b'));
	 * // => [2, 1]
	 *
	 * _.map(_.sortBy(objects, _.property(['a', 'b'])), 'a.b');
	 * // => [1, 2]
	 */
	function property(path) {
	  return isKey(path) ? baseProperty(toKey(path)) : basePropertyDeep(path);
	}

	property_1 = property;
	return property_1;
}

var _baseIteratee;
var hasRequired_baseIteratee;

function require_baseIteratee () {
	if (hasRequired_baseIteratee) return _baseIteratee;
	hasRequired_baseIteratee = 1;
	var baseMatches = require_baseMatches(),
	    baseMatchesProperty = require_baseMatchesProperty(),
	    identity = requireIdentity(),
	    isArray = requireIsArray(),
	    property = requireProperty();

	/**
	 * The base implementation of `_.iteratee`.
	 *
	 * @private
	 * @param {*} [value=_.identity] The value to convert to an iteratee.
	 * @returns {Function} Returns the iteratee.
	 */
	function baseIteratee(value) {
	  // Don't store the `typeof` result in a variable to avoid a JIT bug in Safari 9.
	  // See https://bugs.webkit.org/show_bug.cgi?id=156034 for more details.
	  if (typeof value == 'function') {
	    return value;
	  }
	  if (value == null) {
	    return identity;
	  }
	  if (typeof value == 'object') {
	    return isArray(value)
	      ? baseMatchesProperty(value[0], value[1])
	      : baseMatches(value);
	  }
	  return property(value);
	}

	_baseIteratee = baseIteratee;
	return _baseIteratee;
}

/**
 * Creates a base function for methods like `_.forIn` and `_.forOwn`.
 *
 * @private
 * @param {boolean} [fromRight] Specify iterating from right to left.
 * @returns {Function} Returns the new base function.
 */

var _createBaseFor;
var hasRequired_createBaseFor;

function require_createBaseFor () {
	if (hasRequired_createBaseFor) return _createBaseFor;
	hasRequired_createBaseFor = 1;
	function createBaseFor(fromRight) {
	  return function(object, iteratee, keysFunc) {
	    var index = -1,
	        iterable = Object(object),
	        props = keysFunc(object),
	        length = props.length;

	    while (length--) {
	      var key = props[fromRight ? length : ++index];
	      if (iteratee(iterable[key], key, iterable) === false) {
	        break;
	      }
	    }
	    return object;
	  };
	}

	_createBaseFor = createBaseFor;
	return _createBaseFor;
}

var _baseFor;
var hasRequired_baseFor;

function require_baseFor () {
	if (hasRequired_baseFor) return _baseFor;
	hasRequired_baseFor = 1;
	var createBaseFor = require_createBaseFor();

	/**
	 * The base implementation of `baseForOwn` which iterates over `object`
	 * properties returned by `keysFunc` and invokes `iteratee` for each property.
	 * Iteratee functions may exit iteration early by explicitly returning `false`.
	 *
	 * @private
	 * @param {Object} object The object to iterate over.
	 * @param {Function} iteratee The function invoked per iteration.
	 * @param {Function} keysFunc The function to get the keys of `object`.
	 * @returns {Object} Returns `object`.
	 */
	var baseFor = createBaseFor();

	_baseFor = baseFor;
	return _baseFor;
}

var _baseForOwn;
var hasRequired_baseForOwn;

function require_baseForOwn () {
	if (hasRequired_baseForOwn) return _baseForOwn;
	hasRequired_baseForOwn = 1;
	var baseFor = require_baseFor(),
	    keys = requireKeys$1();

	/**
	 * The base implementation of `_.forOwn` without support for iteratee shorthands.
	 *
	 * @private
	 * @param {Object} object The object to iterate over.
	 * @param {Function} iteratee The function invoked per iteration.
	 * @returns {Object} Returns `object`.
	 */
	function baseForOwn(object, iteratee) {
	  return object && baseFor(object, iteratee, keys);
	}

	_baseForOwn = baseForOwn;
	return _baseForOwn;
}

var _createBaseEach;
var hasRequired_createBaseEach;

function require_createBaseEach () {
	if (hasRequired_createBaseEach) return _createBaseEach;
	hasRequired_createBaseEach = 1;
	var isArrayLike = requireIsArrayLike();

	/**
	 * Creates a `baseEach` or `baseEachRight` function.
	 *
	 * @private
	 * @param {Function} eachFunc The function to iterate over a collection.
	 * @param {boolean} [fromRight] Specify iterating from right to left.
	 * @returns {Function} Returns the new base function.
	 */
	function createBaseEach(eachFunc, fromRight) {
	  return function(collection, iteratee) {
	    if (collection == null) {
	      return collection;
	    }
	    if (!isArrayLike(collection)) {
	      return eachFunc(collection, iteratee);
	    }
	    var length = collection.length,
	        index = fromRight ? length : -1,
	        iterable = Object(collection);

	    while ((fromRight ? index-- : ++index < length)) {
	      if (iteratee(iterable[index], index, iterable) === false) {
	        break;
	      }
	    }
	    return collection;
	  };
	}

	_createBaseEach = createBaseEach;
	return _createBaseEach;
}

var _baseEach;
var hasRequired_baseEach;

function require_baseEach () {
	if (hasRequired_baseEach) return _baseEach;
	hasRequired_baseEach = 1;
	var baseForOwn = require_baseForOwn(),
	    createBaseEach = require_createBaseEach();

	/**
	 * The base implementation of `_.forEach` without support for iteratee shorthands.
	 *
	 * @private
	 * @param {Array|Object} collection The collection to iterate over.
	 * @param {Function} iteratee The function invoked per iteration.
	 * @returns {Array|Object} Returns `collection`.
	 */
	var baseEach = createBaseEach(baseForOwn);

	_baseEach = baseEach;
	return _baseEach;
}

var _baseMap;
var hasRequired_baseMap;

function require_baseMap () {
	if (hasRequired_baseMap) return _baseMap;
	hasRequired_baseMap = 1;
	var baseEach = require_baseEach(),
	    isArrayLike = requireIsArrayLike();

	/**
	 * The base implementation of `_.map` without support for iteratee shorthands.
	 *
	 * @private
	 * @param {Array|Object} collection The collection to iterate over.
	 * @param {Function} iteratee The function invoked per iteration.
	 * @returns {Array} Returns the new mapped array.
	 */
	function baseMap(collection, iteratee) {
	  var index = -1,
	      result = isArrayLike(collection) ? Array(collection.length) : [];

	  baseEach(collection, function(value, key, collection) {
	    result[++index] = iteratee(value, key, collection);
	  });
	  return result;
	}

	_baseMap = baseMap;
	return _baseMap;
}

var map_1;
var hasRequiredMap;

function requireMap () {
	if (hasRequiredMap) return map_1;
	hasRequiredMap = 1;
	var arrayMap = require_arrayMap(),
	    baseIteratee = require_baseIteratee(),
	    baseMap = require_baseMap(),
	    isArray = requireIsArray();

	/**
	 * Creates an array of values by running each element in `collection` thru
	 * `iteratee`. The iteratee is invoked with three arguments:
	 * (value, index|key, collection).
	 *
	 * Many lodash methods are guarded to work as iteratees for methods like
	 * `_.every`, `_.filter`, `_.map`, `_.mapValues`, `_.reject`, and `_.some`.
	 *
	 * The guarded methods are:
	 * `ary`, `chunk`, `curry`, `curryRight`, `drop`, `dropRight`, `every`,
	 * `fill`, `invert`, `parseInt`, `random`, `range`, `rangeRight`, `repeat`,
	 * `sampleSize`, `slice`, `some`, `sortBy`, `split`, `take`, `takeRight`,
	 * `template`, `trim`, `trimEnd`, `trimStart`, and `words`
	 *
	 * @static
	 * @memberOf _
	 * @since 0.1.0
	 * @category Collection
	 * @param {Array|Object} collection The collection to iterate over.
	 * @param {Function} [iteratee=_.identity] The function invoked per iteration.
	 * @returns {Array} Returns the new mapped array.
	 * @example
	 *
	 * function square(n) {
	 *   return n * n;
	 * }
	 *
	 * _.map([4, 8], square);
	 * // => [16, 64]
	 *
	 * _.map({ 'a': 4, 'b': 8 }, square);
	 * // => [16, 64] (iteration order is not guaranteed)
	 *
	 * var users = [
	 *   { 'user': 'barney' },
	 *   { 'user': 'fred' }
	 * ];
	 *
	 * // The `_.property` iteratee shorthand.
	 * _.map(users, 'user');
	 * // => ['barney', 'fred']
	 */
	function map(collection, iteratee) {
	  var func = isArray(collection) ? arrayMap : baseMap;
	  return func(collection, baseIteratee(iteratee, 3));
	}

	map_1 = map;
	return map_1;
}

/**
 * A specialized version of `_.forEach` for arrays without support for
 * iteratee shorthands.
 *
 * @private
 * @param {Array} [array] The array to iterate over.
 * @param {Function} iteratee The function invoked per iteration.
 * @returns {Array} Returns `array`.
 */

var _arrayEach;
var hasRequired_arrayEach;

function require_arrayEach () {
	if (hasRequired_arrayEach) return _arrayEach;
	hasRequired_arrayEach = 1;
	function arrayEach(array, iteratee) {
	  var index = -1,
	      length = array == null ? 0 : array.length;

	  while (++index < length) {
	    if (iteratee(array[index], index, array) === false) {
	      break;
	    }
	  }
	  return array;
	}

	_arrayEach = arrayEach;
	return _arrayEach;
}

var _castFunction;
var hasRequired_castFunction;

function require_castFunction () {
	if (hasRequired_castFunction) return _castFunction;
	hasRequired_castFunction = 1;
	var identity = requireIdentity();

	/**
	 * Casts `value` to `identity` if it's not a function.
	 *
	 * @private
	 * @param {*} value The value to inspect.
	 * @returns {Function} Returns cast function.
	 */
	function castFunction(value) {
	  return typeof value == 'function' ? value : identity;
	}

	_castFunction = castFunction;
	return _castFunction;
}

var forEach_1;
var hasRequiredForEach;

function requireForEach () {
	if (hasRequiredForEach) return forEach_1;
	hasRequiredForEach = 1;
	var arrayEach = require_arrayEach(),
	    baseEach = require_baseEach(),
	    castFunction = require_castFunction(),
	    isArray = requireIsArray();

	/**
	 * Iterates over elements of `collection` and invokes `iteratee` for each element.
	 * The iteratee is invoked with three arguments: (value, index|key, collection).
	 * Iteratee functions may exit iteration early by explicitly returning `false`.
	 *
	 * **Note:** As with other "Collections" methods, objects with a "length"
	 * property are iterated like arrays. To avoid this behavior use `_.forIn`
	 * or `_.forOwn` for object iteration.
	 *
	 * @static
	 * @memberOf _
	 * @since 0.1.0
	 * @alias each
	 * @category Collection
	 * @param {Array|Object} collection The collection to iterate over.
	 * @param {Function} [iteratee=_.identity] The function invoked per iteration.
	 * @returns {Array|Object} Returns `collection`.
	 * @see _.forEachRight
	 * @example
	 *
	 * _.forEach([1, 2], function(value) {
	 *   console.log(value);
	 * });
	 * // => Logs `1` then `2`.
	 *
	 * _.forEach({ 'a': 1, 'b': 2 }, function(value, key) {
	 *   console.log(key);
	 * });
	 * // => Logs 'a' then 'b' (iteration order is not guaranteed).
	 */
	function forEach(collection, iteratee) {
	  var func = isArray(collection) ? arrayEach : baseEach;
	  return func(collection, castFunction(iteratee));
	}

	forEach_1 = forEach;
	return forEach_1;
}

var _baseValues;
var hasRequired_baseValues;

function require_baseValues () {
	if (hasRequired_baseValues) return _baseValues;
	hasRequired_baseValues = 1;
	var arrayMap = require_arrayMap();

	/**
	 * The base implementation of `_.values` and `_.valuesIn` which creates an
	 * array of `object` property values corresponding to the property names
	 * of `props`.
	 *
	 * @private
	 * @param {Object} object The object to query.
	 * @param {Array} props The property names to get values for.
	 * @returns {Object} Returns the array of property values.
	 */
	function baseValues(object, props) {
	  return arrayMap(props, function(key) {
	    return object[key];
	  });
	}

	_baseValues = baseValues;
	return _baseValues;
}

var values_1;
var hasRequiredValues;

function requireValues () {
	if (hasRequiredValues) return values_1;
	hasRequiredValues = 1;
	var baseValues = require_baseValues(),
	    keys = requireKeys$1();

	/**
	 * Creates an array of the own enumerable string keyed property values of `object`.
	 *
	 * **Note:** Non-object values are coerced to objects.
	 *
	 * @static
	 * @since 0.1.0
	 * @memberOf _
	 * @category Object
	 * @param {Object} object The object to query.
	 * @returns {Array} Returns the array of property values.
	 * @example
	 *
	 * function Foo() {
	 *   this.a = 1;
	 *   this.b = 2;
	 * }
	 *
	 * Foo.prototype.c = 3;
	 *
	 * _.values(new Foo);
	 * // => [1, 2] (iteration order is not guaranteed)
	 *
	 * _.values('hi');
	 * // => ['h', 'i']
	 */
	function values(object) {
	  return object == null ? [] : baseValues(object, keys(object));
	}

	values_1 = values;
	return values_1;
}

/** Used for built-in method references. */

var _baseHas;
var hasRequired_baseHas;

function require_baseHas () {
	if (hasRequired_baseHas) return _baseHas;
	hasRequired_baseHas = 1;
	var objectProto = Object.prototype;

	/** Used to check objects for own properties. */
	var hasOwnProperty = objectProto.hasOwnProperty;

	/**
	 * The base implementation of `_.has` without support for deep paths.
	 *
	 * @private
	 * @param {Object} [object] The object to query.
	 * @param {Array|string} key The key to check.
	 * @returns {boolean} Returns `true` if `key` exists, else `false`.
	 */
	function baseHas(object, key) {
	  return object != null && hasOwnProperty.call(object, key);
	}

	_baseHas = baseHas;
	return _baseHas;
}

var has_1;
var hasRequiredHas;

function requireHas () {
	if (hasRequiredHas) return has_1;
	hasRequiredHas = 1;
	var baseHas = require_baseHas(),
	    hasPath = require_hasPath();

	/**
	 * Checks if `path` is a direct property of `object`.
	 *
	 * @static
	 * @since 0.1.0
	 * @memberOf _
	 * @category Object
	 * @param {Object} object The object to query.
	 * @param {Array|string} path The path to check.
	 * @returns {boolean} Returns `true` if `path` exists, else `false`.
	 * @example
	 *
	 * var object = { 'a': { 'b': 2 } };
	 * var other = _.create({ 'a': _.create({ 'b': 2 }) });
	 *
	 * _.has(object, 'a');
	 * // => true
	 *
	 * _.has(object, 'a.b');
	 * // => true
	 *
	 * _.has(object, ['a', 'b']);
	 * // => true
	 *
	 * _.has(other, 'a');
	 * // => false
	 */
	function has(object, path) {
	  return object != null && hasPath(object, path, baseHas);
	}

	has_1 = has;
	return has_1;
}

var _defineProperty;
var hasRequired_defineProperty;

function require_defineProperty () {
	if (hasRequired_defineProperty) return _defineProperty;
	hasRequired_defineProperty = 1;
	var getNative = require_getNative();

	var defineProperty = (function() {
	  try {
	    var func = getNative(Object, 'defineProperty');
	    func({}, '', {});
	    return func;
	  } catch (e) {}
	}());

	_defineProperty = defineProperty;
	return _defineProperty;
}

var _baseAssignValue;
var hasRequired_baseAssignValue;

function require_baseAssignValue () {
	if (hasRequired_baseAssignValue) return _baseAssignValue;
	hasRequired_baseAssignValue = 1;
	var defineProperty = require_defineProperty();

	/**
	 * The base implementation of `assignValue` and `assignMergeValue` without
	 * value checks.
	 *
	 * @private
	 * @param {Object} object The object to modify.
	 * @param {string} key The key of the property to assign.
	 * @param {*} value The value to assign.
	 */
	function baseAssignValue(object, key, value) {
	  if (key == '__proto__' && defineProperty) {
	    defineProperty(object, key, {
	      'configurable': true,
	      'enumerable': true,
	      'value': value,
	      'writable': true
	    });
	  } else {
	    object[key] = value;
	  }
	}

	_baseAssignValue = baseAssignValue;
	return _baseAssignValue;
}

var _assignValue;
var hasRequired_assignValue;

function require_assignValue () {
	if (hasRequired_assignValue) return _assignValue;
	hasRequired_assignValue = 1;
	var baseAssignValue = require_baseAssignValue(),
	    eq = requireEq();

	/** Used for built-in method references. */
	var objectProto = Object.prototype;

	/** Used to check objects for own properties. */
	var hasOwnProperty = objectProto.hasOwnProperty;

	/**
	 * Assigns `value` to `key` of `object` if the existing value is not equivalent
	 * using [`SameValueZero`](http://ecma-international.org/ecma-262/7.0/#sec-samevaluezero)
	 * for equality comparisons.
	 *
	 * @private
	 * @param {Object} object The object to modify.
	 * @param {string} key The key of the property to assign.
	 * @param {*} value The value to assign.
	 */
	function assignValue(object, key, value) {
	  var objValue = object[key];
	  if (!(hasOwnProperty.call(object, key) && eq(objValue, value)) ||
	      (value === undefined && !(key in object))) {
	    baseAssignValue(object, key, value);
	  }
	}

	_assignValue = assignValue;
	return _assignValue;
}

var _copyObject;
var hasRequired_copyObject;

function require_copyObject () {
	if (hasRequired_copyObject) return _copyObject;
	hasRequired_copyObject = 1;
	var assignValue = require_assignValue(),
	    baseAssignValue = require_baseAssignValue();

	/**
	 * Copies properties of `source` to `object`.
	 *
	 * @private
	 * @param {Object} source The object to copy properties from.
	 * @param {Array} props The property identifiers to copy.
	 * @param {Object} [object={}] The object to copy properties to.
	 * @param {Function} [customizer] The function to customize copied values.
	 * @returns {Object} Returns `object`.
	 */
	function copyObject(source, props, object, customizer) {
	  var isNew = !object;
	  object || (object = {});

	  var index = -1,
	      length = props.length;

	  while (++index < length) {
	    var key = props[index];

	    var newValue = customizer
	      ? customizer(object[key], source[key], key, object, source)
	      : undefined;

	    if (newValue === undefined) {
	      newValue = source[key];
	    }
	    if (isNew) {
	      baseAssignValue(object, key, newValue);
	    } else {
	      assignValue(object, key, newValue);
	    }
	  }
	  return object;
	}

	_copyObject = copyObject;
	return _copyObject;
}

var _baseAssign;
var hasRequired_baseAssign;

function require_baseAssign () {
	if (hasRequired_baseAssign) return _baseAssign;
	hasRequired_baseAssign = 1;
	var copyObject = require_copyObject(),
	    keys = requireKeys$1();

	/**
	 * The base implementation of `_.assign` without support for multiple sources
	 * or `customizer` functions.
	 *
	 * @private
	 * @param {Object} object The destination object.
	 * @param {Object} source The source object.
	 * @returns {Object} Returns `object`.
	 */
	function baseAssign(object, source) {
	  return object && copyObject(source, keys(source), object);
	}

	_baseAssign = baseAssign;
	return _baseAssign;
}

/**
 * This function is like
 * [`Object.keys`](http://ecma-international.org/ecma-262/7.0/#sec-object.keys)
 * except that it includes inherited enumerable properties.
 *
 * @private
 * @param {Object} object The object to query.
 * @returns {Array} Returns the array of property names.
 */

var _nativeKeysIn;
var hasRequired_nativeKeysIn;

function require_nativeKeysIn () {
	if (hasRequired_nativeKeysIn) return _nativeKeysIn;
	hasRequired_nativeKeysIn = 1;
	function nativeKeysIn(object) {
	  var result = [];
	  if (object != null) {
	    for (var key in Object(object)) {
	      result.push(key);
	    }
	  }
	  return result;
	}

	_nativeKeysIn = nativeKeysIn;
	return _nativeKeysIn;
}

var _baseKeysIn;
var hasRequired_baseKeysIn;

function require_baseKeysIn () {
	if (hasRequired_baseKeysIn) return _baseKeysIn;
	hasRequired_baseKeysIn = 1;
	var isObject = requireIsObject(),
	    isPrototype = require_isPrototype(),
	    nativeKeysIn = require_nativeKeysIn();

	/** Used for built-in method references. */
	var objectProto = Object.prototype;

	/** Used to check objects for own properties. */
	var hasOwnProperty = objectProto.hasOwnProperty;

	/**
	 * The base implementation of `_.keysIn` which doesn't treat sparse arrays as dense.
	 *
	 * @private
	 * @param {Object} object The object to query.
	 * @returns {Array} Returns the array of property names.
	 */
	function baseKeysIn(object) {
	  if (!isObject(object)) {
	    return nativeKeysIn(object);
	  }
	  var isProto = isPrototype(object),
	      result = [];

	  for (var key in object) {
	    if (!(key == 'constructor' && (isProto || !hasOwnProperty.call(object, key)))) {
	      result.push(key);
	    }
	  }
	  return result;
	}

	_baseKeysIn = baseKeysIn;
	return _baseKeysIn;
}

var keysIn_1;
var hasRequiredKeysIn;

function requireKeysIn () {
	if (hasRequiredKeysIn) return keysIn_1;
	hasRequiredKeysIn = 1;
	var arrayLikeKeys = require_arrayLikeKeys(),
	    baseKeysIn = require_baseKeysIn(),
	    isArrayLike = requireIsArrayLike();

	/**
	 * Creates an array of the own and inherited enumerable property names of `object`.
	 *
	 * **Note:** Non-object values are coerced to objects.
	 *
	 * @static
	 * @memberOf _
	 * @since 3.0.0
	 * @category Object
	 * @param {Object} object The object to query.
	 * @returns {Array} Returns the array of property names.
	 * @example
	 *
	 * function Foo() {
	 *   this.a = 1;
	 *   this.b = 2;
	 * }
	 *
	 * Foo.prototype.c = 3;
	 *
	 * _.keysIn(new Foo);
	 * // => ['a', 'b', 'c'] (iteration order is not guaranteed)
	 */
	function keysIn(object) {
	  return isArrayLike(object) ? arrayLikeKeys(object, true) : baseKeysIn(object);
	}

	keysIn_1 = keysIn;
	return keysIn_1;
}

var _baseAssignIn;
var hasRequired_baseAssignIn;

function require_baseAssignIn () {
	if (hasRequired_baseAssignIn) return _baseAssignIn;
	hasRequired_baseAssignIn = 1;
	var copyObject = require_copyObject(),
	    keysIn = requireKeysIn();

	/**
	 * The base implementation of `_.assignIn` without support for multiple sources
	 * or `customizer` functions.
	 *
	 * @private
	 * @param {Object} object The destination object.
	 * @param {Object} source The source object.
	 * @returns {Object} Returns `object`.
	 */
	function baseAssignIn(object, source) {
	  return object && copyObject(source, keysIn(source), object);
	}

	_baseAssignIn = baseAssignIn;
	return _baseAssignIn;
}

var _cloneBuffer = {exports: {}};

_cloneBuffer.exports;

var hasRequired_cloneBuffer;

function require_cloneBuffer () {
	if (hasRequired_cloneBuffer) return _cloneBuffer.exports;
	hasRequired_cloneBuffer = 1;
	(function (module, exports) {
		var root = require_root();

		/** Detect free variable `exports`. */
		var freeExports = exports && !exports.nodeType && exports;

		/** Detect free variable `module`. */
		var freeModule = freeExports && 'object' == 'object' && module && !module.nodeType && module;

		/** Detect the popular CommonJS extension `module.exports`. */
		var moduleExports = freeModule && freeModule.exports === freeExports;

		/** Built-in value references. */
		var Buffer = moduleExports ? root.Buffer : undefined,
		    allocUnsafe = Buffer ? Buffer.allocUnsafe : undefined;

		/**
		 * Creates a clone of  `buffer`.
		 *
		 * @private
		 * @param {Buffer} buffer The buffer to clone.
		 * @param {boolean} [isDeep] Specify a deep clone.
		 * @returns {Buffer} Returns the cloned buffer.
		 */
		function cloneBuffer(buffer, isDeep) {
		  if (isDeep) {
		    return buffer.slice();
		  }
		  var length = buffer.length,
		      result = allocUnsafe ? allocUnsafe(length) : new buffer.constructor(length);

		  buffer.copy(result);
		  return result;
		}

		module.exports = cloneBuffer; 
	} (_cloneBuffer, _cloneBuffer.exports));
	return _cloneBuffer.exports;
}

/**
 * Copies the values of `source` to `array`.
 *
 * @private
 * @param {Array} source The array to copy values from.
 * @param {Array} [array=[]] The array to copy values to.
 * @returns {Array} Returns `array`.
 */

var _copyArray;
var hasRequired_copyArray;

function require_copyArray () {
	if (hasRequired_copyArray) return _copyArray;
	hasRequired_copyArray = 1;
	function copyArray(source, array) {
	  var index = -1,
	      length = source.length;

	  array || (array = Array(length));
	  while (++index < length) {
	    array[index] = source[index];
	  }
	  return array;
	}

	_copyArray = copyArray;
	return _copyArray;
}

var _copySymbols;
var hasRequired_copySymbols;

function require_copySymbols () {
	if (hasRequired_copySymbols) return _copySymbols;
	hasRequired_copySymbols = 1;
	var copyObject = require_copyObject(),
	    getSymbols = require_getSymbols();

	/**
	 * Copies own symbols of `source` to `object`.
	 *
	 * @private
	 * @param {Object} source The object to copy symbols from.
	 * @param {Object} [object={}] The object to copy symbols to.
	 * @returns {Object} Returns `object`.
	 */
	function copySymbols(source, object) {
	  return copyObject(source, getSymbols(source), object);
	}

	_copySymbols = copySymbols;
	return _copySymbols;
}

var _getPrototype;
var hasRequired_getPrototype;

function require_getPrototype () {
	if (hasRequired_getPrototype) return _getPrototype;
	hasRequired_getPrototype = 1;
	var overArg = require_overArg();

	/** Built-in value references. */
	var getPrototype = overArg(Object.getPrototypeOf, Object);

	_getPrototype = getPrototype;
	return _getPrototype;
}

var _getSymbolsIn;
var hasRequired_getSymbolsIn;

function require_getSymbolsIn () {
	if (hasRequired_getSymbolsIn) return _getSymbolsIn;
	hasRequired_getSymbolsIn = 1;
	var arrayPush = require_arrayPush(),
	    getPrototype = require_getPrototype(),
	    getSymbols = require_getSymbols(),
	    stubArray = requireStubArray();

	/* Built-in method references for those with the same name as other `lodash` methods. */
	var nativeGetSymbols = Object.getOwnPropertySymbols;

	/**
	 * Creates an array of the own and inherited enumerable symbols of `object`.
	 *
	 * @private
	 * @param {Object} object The object to query.
	 * @returns {Array} Returns the array of symbols.
	 */
	var getSymbolsIn = !nativeGetSymbols ? stubArray : function(object) {
	  var result = [];
	  while (object) {
	    arrayPush(result, getSymbols(object));
	    object = getPrototype(object);
	  }
	  return result;
	};

	_getSymbolsIn = getSymbolsIn;
	return _getSymbolsIn;
}

var _copySymbolsIn;
var hasRequired_copySymbolsIn;

function require_copySymbolsIn () {
	if (hasRequired_copySymbolsIn) return _copySymbolsIn;
	hasRequired_copySymbolsIn = 1;
	var copyObject = require_copyObject(),
	    getSymbolsIn = require_getSymbolsIn();

	/**
	 * Copies own and inherited symbols of `source` to `object`.
	 *
	 * @private
	 * @param {Object} source The object to copy symbols from.
	 * @param {Object} [object={}] The object to copy symbols to.
	 * @returns {Object} Returns `object`.
	 */
	function copySymbolsIn(source, object) {
	  return copyObject(source, getSymbolsIn(source), object);
	}

	_copySymbolsIn = copySymbolsIn;
	return _copySymbolsIn;
}

var _getAllKeysIn;
var hasRequired_getAllKeysIn;

function require_getAllKeysIn () {
	if (hasRequired_getAllKeysIn) return _getAllKeysIn;
	hasRequired_getAllKeysIn = 1;
	var baseGetAllKeys = require_baseGetAllKeys(),
	    getSymbolsIn = require_getSymbolsIn(),
	    keysIn = requireKeysIn();

	/**
	 * Creates an array of own and inherited enumerable property names and
	 * symbols of `object`.
	 *
	 * @private
	 * @param {Object} object The object to query.
	 * @returns {Array} Returns the array of property names and symbols.
	 */
	function getAllKeysIn(object) {
	  return baseGetAllKeys(object, keysIn, getSymbolsIn);
	}

	_getAllKeysIn = getAllKeysIn;
	return _getAllKeysIn;
}

/** Used for built-in method references. */

var _initCloneArray;
var hasRequired_initCloneArray;

function require_initCloneArray () {
	if (hasRequired_initCloneArray) return _initCloneArray;
	hasRequired_initCloneArray = 1;
	var objectProto = Object.prototype;

	/** Used to check objects for own properties. */
	var hasOwnProperty = objectProto.hasOwnProperty;

	/**
	 * Initializes an array clone.
	 *
	 * @private
	 * @param {Array} array The array to clone.
	 * @returns {Array} Returns the initialized clone.
	 */
	function initCloneArray(array) {
	  var length = array.length,
	      result = new array.constructor(length);

	  // Add properties assigned by `RegExp#exec`.
	  if (length && typeof array[0] == 'string' && hasOwnProperty.call(array, 'index')) {
	    result.index = array.index;
	    result.input = array.input;
	  }
	  return result;
	}

	_initCloneArray = initCloneArray;
	return _initCloneArray;
}

var _cloneArrayBuffer;
var hasRequired_cloneArrayBuffer;

function require_cloneArrayBuffer () {
	if (hasRequired_cloneArrayBuffer) return _cloneArrayBuffer;
	hasRequired_cloneArrayBuffer = 1;
	var Uint8Array = require_Uint8Array();

	/**
	 * Creates a clone of `arrayBuffer`.
	 *
	 * @private
	 * @param {ArrayBuffer} arrayBuffer The array buffer to clone.
	 * @returns {ArrayBuffer} Returns the cloned array buffer.
	 */
	function cloneArrayBuffer(arrayBuffer) {
	  var result = new arrayBuffer.constructor(arrayBuffer.byteLength);
	  new Uint8Array(result).set(new Uint8Array(arrayBuffer));
	  return result;
	}

	_cloneArrayBuffer = cloneArrayBuffer;
	return _cloneArrayBuffer;
}

var _cloneDataView;
var hasRequired_cloneDataView;

function require_cloneDataView () {
	if (hasRequired_cloneDataView) return _cloneDataView;
	hasRequired_cloneDataView = 1;
	var cloneArrayBuffer = require_cloneArrayBuffer();

	/**
	 * Creates a clone of `dataView`.
	 *
	 * @private
	 * @param {Object} dataView The data view to clone.
	 * @param {boolean} [isDeep] Specify a deep clone.
	 * @returns {Object} Returns the cloned data view.
	 */
	function cloneDataView(dataView, isDeep) {
	  var buffer = isDeep ? cloneArrayBuffer(dataView.buffer) : dataView.buffer;
	  return new dataView.constructor(buffer, dataView.byteOffset, dataView.byteLength);
	}

	_cloneDataView = cloneDataView;
	return _cloneDataView;
}

/** Used to match `RegExp` flags from their coerced string values. */

var _cloneRegExp;
var hasRequired_cloneRegExp;

function require_cloneRegExp () {
	if (hasRequired_cloneRegExp) return _cloneRegExp;
	hasRequired_cloneRegExp = 1;
	var reFlags = /\w*$/;

	/**
	 * Creates a clone of `regexp`.
	 *
	 * @private
	 * @param {Object} regexp The regexp to clone.
	 * @returns {Object} Returns the cloned regexp.
	 */
	function cloneRegExp(regexp) {
	  var result = new regexp.constructor(regexp.source, reFlags.exec(regexp));
	  result.lastIndex = regexp.lastIndex;
	  return result;
	}

	_cloneRegExp = cloneRegExp;
	return _cloneRegExp;
}

var _cloneSymbol;
var hasRequired_cloneSymbol;

function require_cloneSymbol () {
	if (hasRequired_cloneSymbol) return _cloneSymbol;
	hasRequired_cloneSymbol = 1;
	var Symbol = require_Symbol();

	/** Used to convert symbols to primitives and strings. */
	var symbolProto = Symbol ? Symbol.prototype : undefined,
	    symbolValueOf = symbolProto ? symbolProto.valueOf : undefined;

	/**
	 * Creates a clone of the `symbol` object.
	 *
	 * @private
	 * @param {Object} symbol The symbol object to clone.
	 * @returns {Object} Returns the cloned symbol object.
	 */
	function cloneSymbol(symbol) {
	  return symbolValueOf ? Object(symbolValueOf.call(symbol)) : {};
	}

	_cloneSymbol = cloneSymbol;
	return _cloneSymbol;
}

var _cloneTypedArray;
var hasRequired_cloneTypedArray;

function require_cloneTypedArray () {
	if (hasRequired_cloneTypedArray) return _cloneTypedArray;
	hasRequired_cloneTypedArray = 1;
	var cloneArrayBuffer = require_cloneArrayBuffer();

	/**
	 * Creates a clone of `typedArray`.
	 *
	 * @private
	 * @param {Object} typedArray The typed array to clone.
	 * @param {boolean} [isDeep] Specify a deep clone.
	 * @returns {Object} Returns the cloned typed array.
	 */
	function cloneTypedArray(typedArray, isDeep) {
	  var buffer = isDeep ? cloneArrayBuffer(typedArray.buffer) : typedArray.buffer;
	  return new typedArray.constructor(buffer, typedArray.byteOffset, typedArray.length);
	}

	_cloneTypedArray = cloneTypedArray;
	return _cloneTypedArray;
}

var _initCloneByTag;
var hasRequired_initCloneByTag;

function require_initCloneByTag () {
	if (hasRequired_initCloneByTag) return _initCloneByTag;
	hasRequired_initCloneByTag = 1;
	var cloneArrayBuffer = require_cloneArrayBuffer(),
	    cloneDataView = require_cloneDataView(),
	    cloneRegExp = require_cloneRegExp(),
	    cloneSymbol = require_cloneSymbol(),
	    cloneTypedArray = require_cloneTypedArray();

	/** `Object#toString` result references. */
	var boolTag = '[object Boolean]',
	    dateTag = '[object Date]',
	    mapTag = '[object Map]',
	    numberTag = '[object Number]',
	    regexpTag = '[object RegExp]',
	    setTag = '[object Set]',
	    stringTag = '[object String]',
	    symbolTag = '[object Symbol]';

	var arrayBufferTag = '[object ArrayBuffer]',
	    dataViewTag = '[object DataView]',
	    float32Tag = '[object Float32Array]',
	    float64Tag = '[object Float64Array]',
	    int8Tag = '[object Int8Array]',
	    int16Tag = '[object Int16Array]',
	    int32Tag = '[object Int32Array]',
	    uint8Tag = '[object Uint8Array]',
	    uint8ClampedTag = '[object Uint8ClampedArray]',
	    uint16Tag = '[object Uint16Array]',
	    uint32Tag = '[object Uint32Array]';

	/**
	 * Initializes an object clone based on its `toStringTag`.
	 *
	 * **Note:** This function only supports cloning values with tags of
	 * `Boolean`, `Date`, `Error`, `Map`, `Number`, `RegExp`, `Set`, or `String`.
	 *
	 * @private
	 * @param {Object} object The object to clone.
	 * @param {string} tag The `toStringTag` of the object to clone.
	 * @param {boolean} [isDeep] Specify a deep clone.
	 * @returns {Object} Returns the initialized clone.
	 */
	function initCloneByTag(object, tag, isDeep) {
	  var Ctor = object.constructor;
	  switch (tag) {
	    case arrayBufferTag:
	      return cloneArrayBuffer(object);

	    case boolTag:
	    case dateTag:
	      return new Ctor(+object);

	    case dataViewTag:
	      return cloneDataView(object, isDeep);

	    case float32Tag: case float64Tag:
	    case int8Tag: case int16Tag: case int32Tag:
	    case uint8Tag: case uint8ClampedTag: case uint16Tag: case uint32Tag:
	      return cloneTypedArray(object, isDeep);

	    case mapTag:
	      return new Ctor;

	    case numberTag:
	    case stringTag:
	      return new Ctor(object);

	    case regexpTag:
	      return cloneRegExp(object);

	    case setTag:
	      return new Ctor;

	    case symbolTag:
	      return cloneSymbol(object);
	  }
	}

	_initCloneByTag = initCloneByTag;
	return _initCloneByTag;
}

var _baseCreate;
var hasRequired_baseCreate;

function require_baseCreate () {
	if (hasRequired_baseCreate) return _baseCreate;
	hasRequired_baseCreate = 1;
	var isObject = requireIsObject();

	/** Built-in value references. */
	var objectCreate = Object.create;

	/**
	 * The base implementation of `_.create` without support for assigning
	 * properties to the created object.
	 *
	 * @private
	 * @param {Object} proto The object to inherit from.
	 * @returns {Object} Returns the new object.
	 */
	var baseCreate = (function() {
	  function object() {}
	  return function(proto) {
	    if (!isObject(proto)) {
	      return {};
	    }
	    if (objectCreate) {
	      return objectCreate(proto);
	    }
	    object.prototype = proto;
	    var result = new object;
	    object.prototype = undefined;
	    return result;
	  };
	}());

	_baseCreate = baseCreate;
	return _baseCreate;
}

var _initCloneObject;
var hasRequired_initCloneObject;

function require_initCloneObject () {
	if (hasRequired_initCloneObject) return _initCloneObject;
	hasRequired_initCloneObject = 1;
	var baseCreate = require_baseCreate(),
	    getPrototype = require_getPrototype(),
	    isPrototype = require_isPrototype();

	/**
	 * Initializes an object clone.
	 *
	 * @private
	 * @param {Object} object The object to clone.
	 * @returns {Object} Returns the initialized clone.
	 */
	function initCloneObject(object) {
	  return (typeof object.constructor == 'function' && !isPrototype(object))
	    ? baseCreate(getPrototype(object))
	    : {};
	}

	_initCloneObject = initCloneObject;
	return _initCloneObject;
}

var _baseIsMap;
var hasRequired_baseIsMap;

function require_baseIsMap () {
	if (hasRequired_baseIsMap) return _baseIsMap;
	hasRequired_baseIsMap = 1;
	var getTag = require_getTag(),
	    isObjectLike = requireIsObjectLike();

	/** `Object#toString` result references. */
	var mapTag = '[object Map]';

	/**
	 * The base implementation of `_.isMap` without Node.js optimizations.
	 *
	 * @private
	 * @param {*} value The value to check.
	 * @returns {boolean} Returns `true` if `value` is a map, else `false`.
	 */
	function baseIsMap(value) {
	  return isObjectLike(value) && getTag(value) == mapTag;
	}

	_baseIsMap = baseIsMap;
	return _baseIsMap;
}

var isMap_1;
var hasRequiredIsMap;

function requireIsMap () {
	if (hasRequiredIsMap) return isMap_1;
	hasRequiredIsMap = 1;
	var baseIsMap = require_baseIsMap(),
	    baseUnary = require_baseUnary(),
	    nodeUtil = require_nodeUtil();

	/* Node.js helper references. */
	var nodeIsMap = nodeUtil && nodeUtil.isMap;

	/**
	 * Checks if `value` is classified as a `Map` object.
	 *
	 * @static
	 * @memberOf _
	 * @since 4.3.0
	 * @category Lang
	 * @param {*} value The value to check.
	 * @returns {boolean} Returns `true` if `value` is a map, else `false`.
	 * @example
	 *
	 * _.isMap(new Map);
	 * // => true
	 *
	 * _.isMap(new WeakMap);
	 * // => false
	 */
	var isMap = nodeIsMap ? baseUnary(nodeIsMap) : baseIsMap;

	isMap_1 = isMap;
	return isMap_1;
}

var _baseIsSet;
var hasRequired_baseIsSet;

function require_baseIsSet () {
	if (hasRequired_baseIsSet) return _baseIsSet;
	hasRequired_baseIsSet = 1;
	var getTag = require_getTag(),
	    isObjectLike = requireIsObjectLike();

	/** `Object#toString` result references. */
	var setTag = '[object Set]';

	/**
	 * The base implementation of `_.isSet` without Node.js optimizations.
	 *
	 * @private
	 * @param {*} value The value to check.
	 * @returns {boolean} Returns `true` if `value` is a set, else `false`.
	 */
	function baseIsSet(value) {
	  return isObjectLike(value) && getTag(value) == setTag;
	}

	_baseIsSet = baseIsSet;
	return _baseIsSet;
}

var isSet_1;
var hasRequiredIsSet;

function requireIsSet () {
	if (hasRequiredIsSet) return isSet_1;
	hasRequiredIsSet = 1;
	var baseIsSet = require_baseIsSet(),
	    baseUnary = require_baseUnary(),
	    nodeUtil = require_nodeUtil();

	/* Node.js helper references. */
	var nodeIsSet = nodeUtil && nodeUtil.isSet;

	/**
	 * Checks if `value` is classified as a `Set` object.
	 *
	 * @static
	 * @memberOf _
	 * @since 4.3.0
	 * @category Lang
	 * @param {*} value The value to check.
	 * @returns {boolean} Returns `true` if `value` is a set, else `false`.
	 * @example
	 *
	 * _.isSet(new Set);
	 * // => true
	 *
	 * _.isSet(new WeakSet);
	 * // => false
	 */
	var isSet = nodeIsSet ? baseUnary(nodeIsSet) : baseIsSet;

	isSet_1 = isSet;
	return isSet_1;
}

var _baseClone;
var hasRequired_baseClone;

function require_baseClone () {
	if (hasRequired_baseClone) return _baseClone;
	hasRequired_baseClone = 1;
	var Stack = require_Stack(),
	    arrayEach = require_arrayEach(),
	    assignValue = require_assignValue(),
	    baseAssign = require_baseAssign(),
	    baseAssignIn = require_baseAssignIn(),
	    cloneBuffer = require_cloneBuffer(),
	    copyArray = require_copyArray(),
	    copySymbols = require_copySymbols(),
	    copySymbolsIn = require_copySymbolsIn(),
	    getAllKeys = require_getAllKeys(),
	    getAllKeysIn = require_getAllKeysIn(),
	    getTag = require_getTag(),
	    initCloneArray = require_initCloneArray(),
	    initCloneByTag = require_initCloneByTag(),
	    initCloneObject = require_initCloneObject(),
	    isArray = requireIsArray(),
	    isBuffer = requireIsBuffer(),
	    isMap = requireIsMap(),
	    isObject = requireIsObject(),
	    isSet = requireIsSet(),
	    keys = requireKeys$1(),
	    keysIn = requireKeysIn();

	/** Used to compose bitmasks for cloning. */
	var CLONE_DEEP_FLAG = 1,
	    CLONE_FLAT_FLAG = 2,
	    CLONE_SYMBOLS_FLAG = 4;

	/** `Object#toString` result references. */
	var argsTag = '[object Arguments]',
	    arrayTag = '[object Array]',
	    boolTag = '[object Boolean]',
	    dateTag = '[object Date]',
	    errorTag = '[object Error]',
	    funcTag = '[object Function]',
	    genTag = '[object GeneratorFunction]',
	    mapTag = '[object Map]',
	    numberTag = '[object Number]',
	    objectTag = '[object Object]',
	    regexpTag = '[object RegExp]',
	    setTag = '[object Set]',
	    stringTag = '[object String]',
	    symbolTag = '[object Symbol]',
	    weakMapTag = '[object WeakMap]';

	var arrayBufferTag = '[object ArrayBuffer]',
	    dataViewTag = '[object DataView]',
	    float32Tag = '[object Float32Array]',
	    float64Tag = '[object Float64Array]',
	    int8Tag = '[object Int8Array]',
	    int16Tag = '[object Int16Array]',
	    int32Tag = '[object Int32Array]',
	    uint8Tag = '[object Uint8Array]',
	    uint8ClampedTag = '[object Uint8ClampedArray]',
	    uint16Tag = '[object Uint16Array]',
	    uint32Tag = '[object Uint32Array]';

	/** Used to identify `toStringTag` values supported by `_.clone`. */
	var cloneableTags = {};
	cloneableTags[argsTag] = cloneableTags[arrayTag] =
	cloneableTags[arrayBufferTag] = cloneableTags[dataViewTag] =
	cloneableTags[boolTag] = cloneableTags[dateTag] =
	cloneableTags[float32Tag] = cloneableTags[float64Tag] =
	cloneableTags[int8Tag] = cloneableTags[int16Tag] =
	cloneableTags[int32Tag] = cloneableTags[mapTag] =
	cloneableTags[numberTag] = cloneableTags[objectTag] =
	cloneableTags[regexpTag] = cloneableTags[setTag] =
	cloneableTags[stringTag] = cloneableTags[symbolTag] =
	cloneableTags[uint8Tag] = cloneableTags[uint8ClampedTag] =
	cloneableTags[uint16Tag] = cloneableTags[uint32Tag] = true;
	cloneableTags[errorTag] = cloneableTags[funcTag] =
	cloneableTags[weakMapTag] = false;

	/**
	 * The base implementation of `_.clone` and `_.cloneDeep` which tracks
	 * traversed objects.
	 *
	 * @private
	 * @param {*} value The value to clone.
	 * @param {boolean} bitmask The bitmask flags.
	 *  1 - Deep clone
	 *  2 - Flatten inherited properties
	 *  4 - Clone symbols
	 * @param {Function} [customizer] The function to customize cloning.
	 * @param {string} [key] The key of `value`.
	 * @param {Object} [object] The parent object of `value`.
	 * @param {Object} [stack] Tracks traversed objects and their clone counterparts.
	 * @returns {*} Returns the cloned value.
	 */
	function baseClone(value, bitmask, customizer, key, object, stack) {
	  var result,
	      isDeep = bitmask & CLONE_DEEP_FLAG,
	      isFlat = bitmask & CLONE_FLAT_FLAG,
	      isFull = bitmask & CLONE_SYMBOLS_FLAG;

	  if (customizer) {
	    result = object ? customizer(value, key, object, stack) : customizer(value);
	  }
	  if (result !== undefined) {
	    return result;
	  }
	  if (!isObject(value)) {
	    return value;
	  }
	  var isArr = isArray(value);
	  if (isArr) {
	    result = initCloneArray(value);
	    if (!isDeep) {
	      return copyArray(value, result);
	    }
	  } else {
	    var tag = getTag(value),
	        isFunc = tag == funcTag || tag == genTag;

	    if (isBuffer(value)) {
	      return cloneBuffer(value, isDeep);
	    }
	    if (tag == objectTag || tag == argsTag || (isFunc && !object)) {
	      result = (isFlat || isFunc) ? {} : initCloneObject(value);
	      if (!isDeep) {
	        return isFlat
	          ? copySymbolsIn(value, baseAssignIn(result, value))
	          : copySymbols(value, baseAssign(result, value));
	      }
	    } else {
	      if (!cloneableTags[tag]) {
	        return object ? value : {};
	      }
	      result = initCloneByTag(value, tag, isDeep);
	    }
	  }
	  // Check for circular references and return its corresponding clone.
	  stack || (stack = new Stack);
	  var stacked = stack.get(value);
	  if (stacked) {
	    return stacked;
	  }
	  stack.set(value, result);

	  if (isSet(value)) {
	    value.forEach(function(subValue) {
	      result.add(baseClone(subValue, bitmask, customizer, subValue, value, stack));
	    });
	  } else if (isMap(value)) {
	    value.forEach(function(subValue, key) {
	      result.set(key, baseClone(subValue, bitmask, customizer, key, value, stack));
	    });
	  }

	  var keysFunc = isFull
	    ? (isFlat ? getAllKeysIn : getAllKeys)
	    : (isFlat ? keysIn : keys);

	  var props = isArr ? undefined : keysFunc(value);
	  arrayEach(props || value, function(subValue, key) {
	    if (props) {
	      key = subValue;
	      subValue = value[key];
	    }
	    // Recursively populate clone (susceptible to call stack limits).
	    assignValue(result, key, baseClone(subValue, bitmask, customizer, key, value, stack));
	  });
	  return result;
	}

	_baseClone = baseClone;
	return _baseClone;
}

var clone_1;
var hasRequiredClone;

function requireClone () {
	if (hasRequiredClone) return clone_1;
	hasRequiredClone = 1;
	var baseClone = require_baseClone();

	/** Used to compose bitmasks for cloning. */
	var CLONE_SYMBOLS_FLAG = 4;

	/**
	 * Creates a shallow clone of `value`.
	 *
	 * **Note:** This method is loosely based on the
	 * [structured clone algorithm](https://mdn.io/Structured_clone_algorithm)
	 * and supports cloning arrays, array buffers, booleans, date objects, maps,
	 * numbers, `Object` objects, regexes, sets, strings, symbols, and typed
	 * arrays. The own enumerable properties of `arguments` objects are cloned
	 * as plain objects. An empty object is returned for uncloneable values such
	 * as error objects, functions, DOM nodes, and WeakMaps.
	 *
	 * @static
	 * @memberOf _
	 * @since 0.1.0
	 * @category Lang
	 * @param {*} value The value to clone.
	 * @returns {*} Returns the cloned value.
	 * @see _.cloneDeep
	 * @example
	 *
	 * var objects = [{ 'a': 1 }, { 'b': 2 }];
	 *
	 * var shallow = _.clone(objects);
	 * console.log(shallow[0] === objects[0]);
	 * // => true
	 */
	function clone(value) {
	  return baseClone(value, CLONE_SYMBOLS_FLAG);
	}

	clone_1 = clone;
	return clone_1;
}

var api$2 = {};

var print = {};

var hasRequiredPrint;

function requirePrint () {
	if (hasRequiredPrint) return print;
	hasRequiredPrint = 1;
	Object.defineProperty(print, "__esModule", { value: true });
	print.PRINT_WARNING = print.PRINT_ERROR = void 0;
	function PRINT_ERROR(msg) {
	    /* istanbul ignore else - can't override global.console in node.js */
	    if (console && console.error) {
	        console.error("Error: ".concat(msg));
	    }
	}
	print.PRINT_ERROR = PRINT_ERROR;
	function PRINT_WARNING(msg) {
	    /* istanbul ignore else - can't override global.console in node.js*/
	    if (console && console.warn) {
	        // TODO: modify docs accordingly
	        console.warn("Warning: ".concat(msg));
	    }
	}
	print.PRINT_WARNING = PRINT_WARNING;
	
	return print;
}

var timer = {};

var hasRequiredTimer;

function requireTimer () {
	if (hasRequiredTimer) return timer;
	hasRequiredTimer = 1;
	Object.defineProperty(timer, "__esModule", { value: true });
	timer.timer = void 0;
	function timer$1(func) {
	    var start = new Date().getTime();
	    var val = func();
	    var end = new Date().getTime();
	    var total = end - start;
	    return { time: total, value: val };
	}
	timer.timer = timer$1;
	
	return timer;
}

var toFastProperties = {};

var hasRequiredToFastProperties;

function requireToFastProperties () {
	if (hasRequiredToFastProperties) return toFastProperties;
	hasRequiredToFastProperties = 1;
	Object.defineProperty(toFastProperties, "__esModule", { value: true });
	toFastProperties.toFastProperties = void 0;
	// based on: https://github.com/petkaantonov/bluebird/blob/b97c0d2d487e8c5076e8bd897e0dcd4622d31846/src/util.js#L201-L216
	function toFastProperties$1(toBecomeFast) {
	    function FakeConstructor() { }
	    // If our object is used as a constructor it would receive
	    FakeConstructor.prototype = toBecomeFast;
	    var fakeInstance = new FakeConstructor();
	    function fakeAccess() {
	        return typeof fakeInstance.bar;
	    }
	    // help V8 understand this is a "real" prototype by actually using
	    // the fake instance.
	    fakeAccess();
	    fakeAccess();
	    // Always true condition to suppress the Firefox warning of unreachable
	    // code after a return statement.
	    return toBecomeFast;
	}
	toFastProperties.toFastProperties = toFastProperties$1;
	
	return toFastProperties;
}

var hasRequiredApi$3;

function requireApi$3 () {
	if (hasRequiredApi$3) return api$2;
	hasRequiredApi$3 = 1;
	(function (exports) {
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.toFastProperties = exports.timer = exports.PRINT_ERROR = exports.PRINT_WARNING = void 0;
		var print_1 = requirePrint();
		Object.defineProperty(exports, "PRINT_WARNING", { enumerable: true, get: function () { return print_1.PRINT_WARNING; } });
		Object.defineProperty(exports, "PRINT_ERROR", { enumerable: true, get: function () { return print_1.PRINT_ERROR; } });
		var timer_1 = requireTimer();
		Object.defineProperty(exports, "timer", { enumerable: true, get: function () { return timer_1.timer; } });
		var to_fast_properties_1 = requireToFastProperties();
		Object.defineProperty(exports, "toFastProperties", { enumerable: true, get: function () { return to_fast_properties_1.toFastProperties; } });
		
	} (api$2));
	return api$2;
}

var follow = {};

var rest = {};

/**
 * The base implementation of `_.slice` without an iteratee call guard.
 *
 * @private
 * @param {Array} array The array to slice.
 * @param {number} [start=0] The start position.
 * @param {number} [end=array.length] The end position.
 * @returns {Array} Returns the slice of `array`.
 */

var _baseSlice;
var hasRequired_baseSlice;

function require_baseSlice () {
	if (hasRequired_baseSlice) return _baseSlice;
	hasRequired_baseSlice = 1;
	function baseSlice(array, start, end) {
	  var index = -1,
	      length = array.length;

	  if (start < 0) {
	    start = -start > length ? 0 : (length + start);
	  }
	  end = end > length ? length : end;
	  if (end < 0) {
	    end += length;
	  }
	  length = start > end ? 0 : ((end - start) >>> 0);
	  start >>>= 0;

	  var result = Array(length);
	  while (++index < length) {
	    result[index] = array[index + start];
	  }
	  return result;
	}

	_baseSlice = baseSlice;
	return _baseSlice;
}

/** Used to match a single whitespace character. */

var _trimmedEndIndex;
var hasRequired_trimmedEndIndex;

function require_trimmedEndIndex () {
	if (hasRequired_trimmedEndIndex) return _trimmedEndIndex;
	hasRequired_trimmedEndIndex = 1;
	var reWhitespace = /\s/;

	/**
	 * Used by `_.trim` and `_.trimEnd` to get the index of the last non-whitespace
	 * character of `string`.
	 *
	 * @private
	 * @param {string} string The string to inspect.
	 * @returns {number} Returns the index of the last non-whitespace character.
	 */
	function trimmedEndIndex(string) {
	  var index = string.length;

	  while (index-- && reWhitespace.test(string.charAt(index))) {}
	  return index;
	}

	_trimmedEndIndex = trimmedEndIndex;
	return _trimmedEndIndex;
}

var _baseTrim;
var hasRequired_baseTrim;

function require_baseTrim () {
	if (hasRequired_baseTrim) return _baseTrim;
	hasRequired_baseTrim = 1;
	var trimmedEndIndex = require_trimmedEndIndex();

	/** Used to match leading whitespace. */
	var reTrimStart = /^\s+/;

	/**
	 * The base implementation of `_.trim`.
	 *
	 * @private
	 * @param {string} string The string to trim.
	 * @returns {string} Returns the trimmed string.
	 */
	function baseTrim(string) {
	  return string
	    ? string.slice(0, trimmedEndIndex(string) + 1).replace(reTrimStart, '')
	    : string;
	}

	_baseTrim = baseTrim;
	return _baseTrim;
}

var toNumber_1;
var hasRequiredToNumber;

function requireToNumber () {
	if (hasRequiredToNumber) return toNumber_1;
	hasRequiredToNumber = 1;
	var baseTrim = require_baseTrim(),
	    isObject = requireIsObject(),
	    isSymbol = requireIsSymbol();

	/** Used as references for various `Number` constants. */
	var NAN = 0 / 0;

	/** Used to detect bad signed hexadecimal string values. */
	var reIsBadHex = /^[-+]0x[0-9a-f]+$/i;

	/** Used to detect binary string values. */
	var reIsBinary = /^0b[01]+$/i;

	/** Used to detect octal string values. */
	var reIsOctal = /^0o[0-7]+$/i;

	/** Built-in method references without a dependency on `root`. */
	var freeParseInt = parseInt;

	/**
	 * Converts `value` to a number.
	 *
	 * @static
	 * @memberOf _
	 * @since 4.0.0
	 * @category Lang
	 * @param {*} value The value to process.
	 * @returns {number} Returns the number.
	 * @example
	 *
	 * _.toNumber(3.2);
	 * // => 3.2
	 *
	 * _.toNumber(Number.MIN_VALUE);
	 * // => 5e-324
	 *
	 * _.toNumber(Infinity);
	 * // => Infinity
	 *
	 * _.toNumber('3.2');
	 * // => 3.2
	 */
	function toNumber(value) {
	  if (typeof value == 'number') {
	    return value;
	  }
	  if (isSymbol(value)) {
	    return NAN;
	  }
	  if (isObject(value)) {
	    var other = typeof value.valueOf == 'function' ? value.valueOf() : value;
	    value = isObject(other) ? (other + '') : other;
	  }
	  if (typeof value != 'string') {
	    return value === 0 ? value : +value;
	  }
	  value = baseTrim(value);
	  var isBinary = reIsBinary.test(value);
	  return (isBinary || reIsOctal.test(value))
	    ? freeParseInt(value.slice(2), isBinary ? 2 : 8)
	    : (reIsBadHex.test(value) ? NAN : +value);
	}

	toNumber_1 = toNumber;
	return toNumber_1;
}

var toFinite_1;
var hasRequiredToFinite;

function requireToFinite () {
	if (hasRequiredToFinite) return toFinite_1;
	hasRequiredToFinite = 1;
	var toNumber = requireToNumber();

	/** Used as references for various `Number` constants. */
	var INFINITY = 1 / 0,
	    MAX_INTEGER = 1.7976931348623157e+308;

	/**
	 * Converts `value` to a finite number.
	 *
	 * @static
	 * @memberOf _
	 * @since 4.12.0
	 * @category Lang
	 * @param {*} value The value to convert.
	 * @returns {number} Returns the converted number.
	 * @example
	 *
	 * _.toFinite(3.2);
	 * // => 3.2
	 *
	 * _.toFinite(Number.MIN_VALUE);
	 * // => 5e-324
	 *
	 * _.toFinite(Infinity);
	 * // => 1.7976931348623157e+308
	 *
	 * _.toFinite('3.2');
	 * // => 3.2
	 */
	function toFinite(value) {
	  if (!value) {
	    return value === 0 ? value : 0;
	  }
	  value = toNumber(value);
	  if (value === INFINITY || value === -INFINITY) {
	    var sign = (value < 0 ? -1 : 1);
	    return sign * MAX_INTEGER;
	  }
	  return value === value ? value : 0;
	}

	toFinite_1 = toFinite;
	return toFinite_1;
}

var toInteger_1;
var hasRequiredToInteger;

function requireToInteger () {
	if (hasRequiredToInteger) return toInteger_1;
	hasRequiredToInteger = 1;
	var toFinite = requireToFinite();

	/**
	 * Converts `value` to an integer.
	 *
	 * **Note:** This method is loosely based on
	 * [`ToInteger`](http://www.ecma-international.org/ecma-262/7.0/#sec-tointeger).
	 *
	 * @static
	 * @memberOf _
	 * @since 4.0.0
	 * @category Lang
	 * @param {*} value The value to convert.
	 * @returns {number} Returns the converted integer.
	 * @example
	 *
	 * _.toInteger(3.2);
	 * // => 3
	 *
	 * _.toInteger(Number.MIN_VALUE);
	 * // => 0
	 *
	 * _.toInteger(Infinity);
	 * // => 1.7976931348623157e+308
	 *
	 * _.toInteger('3.2');
	 * // => 3
	 */
	function toInteger(value) {
	  var result = toFinite(value),
	      remainder = result % 1;

	  return result === result ? (remainder ? result - remainder : result) : 0;
	}

	toInteger_1 = toInteger;
	return toInteger_1;
}

var drop_1;
var hasRequiredDrop;

function requireDrop () {
	if (hasRequiredDrop) return drop_1;
	hasRequiredDrop = 1;
	var baseSlice = require_baseSlice(),
	    toInteger = requireToInteger();

	/**
	 * Creates a slice of `array` with `n` elements dropped from the beginning.
	 *
	 * @static
	 * @memberOf _
	 * @since 0.5.0
	 * @category Array
	 * @param {Array} array The array to query.
	 * @param {number} [n=1] The number of elements to drop.
	 * @param- {Object} [guard] Enables use as an iteratee for methods like `_.map`.
	 * @returns {Array} Returns the slice of `array`.
	 * @example
	 *
	 * _.drop([1, 2, 3]);
	 * // => [2, 3]
	 *
	 * _.drop([1, 2, 3], 2);
	 * // => [3]
	 *
	 * _.drop([1, 2, 3], 5);
	 * // => []
	 *
	 * _.drop([1, 2, 3], 0);
	 * // => [1, 2, 3]
	 */
	function drop(array, n, guard) {
	  var length = array == null ? 0 : array.length;
	  if (!length) {
	    return [];
	  }
	  n = (guard || n === undefined) ? 1 : toInteger(n);
	  return baseSlice(array, n < 0 ? 0 : n, length);
	}

	drop_1 = drop;
	return drop_1;
}

var api$1 = {};

var model$1 = {};

var isString_1;
var hasRequiredIsString;

function requireIsString () {
	if (hasRequiredIsString) return isString_1;
	hasRequiredIsString = 1;
	var baseGetTag = require_baseGetTag(),
	    isArray = requireIsArray(),
	    isObjectLike = requireIsObjectLike();

	/** `Object#toString` result references. */
	var stringTag = '[object String]';

	/**
	 * Checks if `value` is classified as a `String` primitive or object.
	 *
	 * @static
	 * @since 0.1.0
	 * @memberOf _
	 * @category Lang
	 * @param {*} value The value to check.
	 * @returns {boolean} Returns `true` if `value` is a string, else `false`.
	 * @example
	 *
	 * _.isString('abc');
	 * // => true
	 *
	 * _.isString(1);
	 * // => false
	 */
	function isString(value) {
	  return typeof value == 'string' ||
	    (!isArray(value) && isObjectLike(value) && baseGetTag(value) == stringTag);
	}

	isString_1 = isString;
	return isString_1;
}

var _baseIsRegExp;
var hasRequired_baseIsRegExp;

function require_baseIsRegExp () {
	if (hasRequired_baseIsRegExp) return _baseIsRegExp;
	hasRequired_baseIsRegExp = 1;
	var baseGetTag = require_baseGetTag(),
	    isObjectLike = requireIsObjectLike();

	/** `Object#toString` result references. */
	var regexpTag = '[object RegExp]';

	/**
	 * The base implementation of `_.isRegExp` without Node.js optimizations.
	 *
	 * @private
	 * @param {*} value The value to check.
	 * @returns {boolean} Returns `true` if `value` is a regexp, else `false`.
	 */
	function baseIsRegExp(value) {
	  return isObjectLike(value) && baseGetTag(value) == regexpTag;
	}

	_baseIsRegExp = baseIsRegExp;
	return _baseIsRegExp;
}

var isRegExp_1;
var hasRequiredIsRegExp;

function requireIsRegExp () {
	if (hasRequiredIsRegExp) return isRegExp_1;
	hasRequiredIsRegExp = 1;
	var baseIsRegExp = require_baseIsRegExp(),
	    baseUnary = require_baseUnary(),
	    nodeUtil = require_nodeUtil();

	/* Node.js helper references. */
	var nodeIsRegExp = nodeUtil && nodeUtil.isRegExp;

	/**
	 * Checks if `value` is classified as a `RegExp` object.
	 *
	 * @static
	 * @memberOf _
	 * @since 0.1.0
	 * @category Lang
	 * @param {*} value The value to check.
	 * @returns {boolean} Returns `true` if `value` is a regexp, else `false`.
	 * @example
	 *
	 * _.isRegExp(/abc/);
	 * // => true
	 *
	 * _.isRegExp('/abc/');
	 * // => false
	 */
	var isRegExp = nodeIsRegExp ? baseUnary(nodeIsRegExp) : baseIsRegExp;

	isRegExp_1 = isRegExp;
	return isRegExp_1;
}

var _baseSet;
var hasRequired_baseSet;

function require_baseSet () {
	if (hasRequired_baseSet) return _baseSet;
	hasRequired_baseSet = 1;
	var assignValue = require_assignValue(),
	    castPath = require_castPath(),
	    isIndex = require_isIndex(),
	    isObject = requireIsObject(),
	    toKey = require_toKey();

	/**
	 * The base implementation of `_.set`.
	 *
	 * @private
	 * @param {Object} object The object to modify.
	 * @param {Array|string} path The path of the property to set.
	 * @param {*} value The value to set.
	 * @param {Function} [customizer] The function to customize path creation.
	 * @returns {Object} Returns `object`.
	 */
	function baseSet(object, path, value, customizer) {
	  if (!isObject(object)) {
	    return object;
	  }
	  path = castPath(path, object);

	  var index = -1,
	      length = path.length,
	      lastIndex = length - 1,
	      nested = object;

	  while (nested != null && ++index < length) {
	    var key = toKey(path[index]),
	        newValue = value;

	    if (key === '__proto__' || key === 'constructor' || key === 'prototype') {
	      return object;
	    }

	    if (index != lastIndex) {
	      var objValue = nested[key];
	      newValue = customizer ? customizer(objValue, key, nested) : undefined;
	      if (newValue === undefined) {
	        newValue = isObject(objValue)
	          ? objValue
	          : (isIndex(path[index + 1]) ? [] : {});
	      }
	    }
	    assignValue(nested, key, newValue);
	    nested = nested[key];
	  }
	  return object;
	}

	_baseSet = baseSet;
	return _baseSet;
}

var _basePickBy;
var hasRequired_basePickBy;

function require_basePickBy () {
	if (hasRequired_basePickBy) return _basePickBy;
	hasRequired_basePickBy = 1;
	var baseGet = require_baseGet(),
	    baseSet = require_baseSet(),
	    castPath = require_castPath();

	/**
	 * The base implementation of  `_.pickBy` without support for iteratee shorthands.
	 *
	 * @private
	 * @param {Object} object The source object.
	 * @param {string[]} paths The property paths to pick.
	 * @param {Function} predicate The function invoked per property.
	 * @returns {Object} Returns the new object.
	 */
	function basePickBy(object, paths, predicate) {
	  var index = -1,
	      length = paths.length,
	      result = {};

	  while (++index < length) {
	    var path = paths[index],
	        value = baseGet(object, path);

	    if (predicate(value, path)) {
	      baseSet(result, castPath(path, object), value);
	    }
	  }
	  return result;
	}

	_basePickBy = basePickBy;
	return _basePickBy;
}

var pickBy_1;
var hasRequiredPickBy;

function requirePickBy () {
	if (hasRequiredPickBy) return pickBy_1;
	hasRequiredPickBy = 1;
	var arrayMap = require_arrayMap(),
	    baseIteratee = require_baseIteratee(),
	    basePickBy = require_basePickBy(),
	    getAllKeysIn = require_getAllKeysIn();

	/**
	 * Creates an object composed of the `object` properties `predicate` returns
	 * truthy for. The predicate is invoked with two arguments: (value, key).
	 *
	 * @static
	 * @memberOf _
	 * @since 4.0.0
	 * @category Object
	 * @param {Object} object The source object.
	 * @param {Function} [predicate=_.identity] The function invoked per property.
	 * @returns {Object} Returns the new object.
	 * @example
	 *
	 * var object = { 'a': 1, 'b': '2', 'c': 3 };
	 *
	 * _.pickBy(object, _.isNumber);
	 * // => { 'a': 1, 'c': 3 }
	 */
	function pickBy(object, predicate) {
	  if (object == null) {
	    return {};
	  }
	  var props = arrayMap(getAllKeysIn(object), function(prop) {
	    return [prop];
	  });
	  predicate = baseIteratee(predicate);
	  return basePickBy(object, props, function(value, path) {
	    return predicate(value, path[0]);
	  });
	}

	pickBy_1 = pickBy;
	return pickBy_1;
}

/**
 * A faster alternative to `Function#apply`, this function invokes `func`
 * with the `this` binding of `thisArg` and the arguments of `args`.
 *
 * @private
 * @param {Function} func The function to invoke.
 * @param {*} thisArg The `this` binding of `func`.
 * @param {Array} args The arguments to invoke `func` with.
 * @returns {*} Returns the result of `func`.
 */

var _apply;
var hasRequired_apply;

function require_apply () {
	if (hasRequired_apply) return _apply;
	hasRequired_apply = 1;
	function apply(func, thisArg, args) {
	  switch (args.length) {
	    case 0: return func.call(thisArg);
	    case 1: return func.call(thisArg, args[0]);
	    case 2: return func.call(thisArg, args[0], args[1]);
	    case 3: return func.call(thisArg, args[0], args[1], args[2]);
	  }
	  return func.apply(thisArg, args);
	}

	_apply = apply;
	return _apply;
}

var _overRest;
var hasRequired_overRest;

function require_overRest () {
	if (hasRequired_overRest) return _overRest;
	hasRequired_overRest = 1;
	var apply = require_apply();

	/* Built-in method references for those with the same name as other `lodash` methods. */
	var nativeMax = Math.max;

	/**
	 * A specialized version of `baseRest` which transforms the rest array.
	 *
	 * @private
	 * @param {Function} func The function to apply a rest parameter to.
	 * @param {number} [start=func.length-1] The start position of the rest parameter.
	 * @param {Function} transform The rest array transform.
	 * @returns {Function} Returns the new function.
	 */
	function overRest(func, start, transform) {
	  start = nativeMax(start === undefined ? (func.length - 1) : start, 0);
	  return function() {
	    var args = arguments,
	        index = -1,
	        length = nativeMax(args.length - start, 0),
	        array = Array(length);

	    while (++index < length) {
	      array[index] = args[start + index];
	    }
	    index = -1;
	    var otherArgs = Array(start + 1);
	    while (++index < start) {
	      otherArgs[index] = args[index];
	    }
	    otherArgs[start] = transform(array);
	    return apply(func, this, otherArgs);
	  };
	}

	_overRest = overRest;
	return _overRest;
}

/**
 * Creates a function that returns `value`.
 *
 * @static
 * @memberOf _
 * @since 2.4.0
 * @category Util
 * @param {*} value The value to return from the new function.
 * @returns {Function} Returns the new constant function.
 * @example
 *
 * var objects = _.times(2, _.constant({ 'a': 1 }));
 *
 * console.log(objects);
 * // => [{ 'a': 1 }, { 'a': 1 }]
 *
 * console.log(objects[0] === objects[1]);
 * // => true
 */

var constant_1;
var hasRequiredConstant;

function requireConstant () {
	if (hasRequiredConstant) return constant_1;
	hasRequiredConstant = 1;
	function constant(value) {
	  return function() {
	    return value;
	  };
	}

	constant_1 = constant;
	return constant_1;
}

var _baseSetToString;
var hasRequired_baseSetToString;

function require_baseSetToString () {
	if (hasRequired_baseSetToString) return _baseSetToString;
	hasRequired_baseSetToString = 1;
	var constant = requireConstant(),
	    defineProperty = require_defineProperty(),
	    identity = requireIdentity();

	/**
	 * The base implementation of `setToString` without support for hot loop shorting.
	 *
	 * @private
	 * @param {Function} func The function to modify.
	 * @param {Function} string The `toString` result.
	 * @returns {Function} Returns `func`.
	 */
	var baseSetToString = !defineProperty ? identity : function(func, string) {
	  return defineProperty(func, 'toString', {
	    'configurable': true,
	    'enumerable': false,
	    'value': constant(string),
	    'writable': true
	  });
	};

	_baseSetToString = baseSetToString;
	return _baseSetToString;
}

/** Used to detect hot functions by number of calls within a span of milliseconds. */

var _shortOut;
var hasRequired_shortOut;

function require_shortOut () {
	if (hasRequired_shortOut) return _shortOut;
	hasRequired_shortOut = 1;
	var HOT_COUNT = 800,
	    HOT_SPAN = 16;

	/* Built-in method references for those with the same name as other `lodash` methods. */
	var nativeNow = Date.now;

	/**
	 * Creates a function that'll short out and invoke `identity` instead
	 * of `func` when it's called `HOT_COUNT` or more times in `HOT_SPAN`
	 * milliseconds.
	 *
	 * @private
	 * @param {Function} func The function to restrict.
	 * @returns {Function} Returns the new shortable function.
	 */
	function shortOut(func) {
	  var count = 0,
	      lastCalled = 0;

	  return function() {
	    var stamp = nativeNow(),
	        remaining = HOT_SPAN - (stamp - lastCalled);

	    lastCalled = stamp;
	    if (remaining > 0) {
	      if (++count >= HOT_COUNT) {
	        return arguments[0];
	      }
	    } else {
	      count = 0;
	    }
	    return func.apply(undefined, arguments);
	  };
	}

	_shortOut = shortOut;
	return _shortOut;
}

var _setToString;
var hasRequired_setToString;

function require_setToString () {
	if (hasRequired_setToString) return _setToString;
	hasRequired_setToString = 1;
	var baseSetToString = require_baseSetToString(),
	    shortOut = require_shortOut();

	/**
	 * Sets the `toString` method of `func` to return `string`.
	 *
	 * @private
	 * @param {Function} func The function to modify.
	 * @param {Function} string The `toString` result.
	 * @returns {Function} Returns `func`.
	 */
	var setToString = shortOut(baseSetToString);

	_setToString = setToString;
	return _setToString;
}

var _baseRest;
var hasRequired_baseRest;

function require_baseRest () {
	if (hasRequired_baseRest) return _baseRest;
	hasRequired_baseRest = 1;
	var identity = requireIdentity(),
	    overRest = require_overRest(),
	    setToString = require_setToString();

	/**
	 * The base implementation of `_.rest` which doesn't validate or coerce arguments.
	 *
	 * @private
	 * @param {Function} func The function to apply a rest parameter to.
	 * @param {number} [start=func.length-1] The start position of the rest parameter.
	 * @returns {Function} Returns the new function.
	 */
	function baseRest(func, start) {
	  return setToString(overRest(func, start, identity), func + '');
	}

	_baseRest = baseRest;
	return _baseRest;
}

var _isIterateeCall;
var hasRequired_isIterateeCall;

function require_isIterateeCall () {
	if (hasRequired_isIterateeCall) return _isIterateeCall;
	hasRequired_isIterateeCall = 1;
	var eq = requireEq(),
	    isArrayLike = requireIsArrayLike(),
	    isIndex = require_isIndex(),
	    isObject = requireIsObject();

	/**
	 * Checks if the given arguments are from an iteratee call.
	 *
	 * @private
	 * @param {*} value The potential iteratee value argument.
	 * @param {*} index The potential iteratee index or key argument.
	 * @param {*} object The potential iteratee object argument.
	 * @returns {boolean} Returns `true` if the arguments are from an iteratee call,
	 *  else `false`.
	 */
	function isIterateeCall(value, index, object) {
	  if (!isObject(object)) {
	    return false;
	  }
	  var type = typeof index;
	  if (type == 'number'
	        ? (isArrayLike(object) && isIndex(index, object.length))
	        : (type == 'string' && index in object)
	      ) {
	    return eq(object[index], value);
	  }
	  return false;
	}

	_isIterateeCall = isIterateeCall;
	return _isIterateeCall;
}

var _createAssigner;
var hasRequired_createAssigner;

function require_createAssigner () {
	if (hasRequired_createAssigner) return _createAssigner;
	hasRequired_createAssigner = 1;
	var baseRest = require_baseRest(),
	    isIterateeCall = require_isIterateeCall();

	/**
	 * Creates a function like `_.assign`.
	 *
	 * @private
	 * @param {Function} assigner The function to assign values.
	 * @returns {Function} Returns the new assigner function.
	 */
	function createAssigner(assigner) {
	  return baseRest(function(object, sources) {
	    var index = -1,
	        length = sources.length,
	        customizer = length > 1 ? sources[length - 1] : undefined,
	        guard = length > 2 ? sources[2] : undefined;

	    customizer = (assigner.length > 3 && typeof customizer == 'function')
	      ? (length--, customizer)
	      : undefined;

	    if (guard && isIterateeCall(sources[0], sources[1], guard)) {
	      customizer = length < 3 ? undefined : customizer;
	      length = 1;
	    }
	    object = Object(object);
	    while (++index < length) {
	      var source = sources[index];
	      if (source) {
	        assigner(object, source, index, customizer);
	      }
	    }
	    return object;
	  });
	}

	_createAssigner = createAssigner;
	return _createAssigner;
}

var assign_1;
var hasRequiredAssign;

function requireAssign () {
	if (hasRequiredAssign) return assign_1;
	hasRequiredAssign = 1;
	var assignValue = require_assignValue(),
	    copyObject = require_copyObject(),
	    createAssigner = require_createAssigner(),
	    isArrayLike = requireIsArrayLike(),
	    isPrototype = require_isPrototype(),
	    keys = requireKeys$1();

	/** Used for built-in method references. */
	var objectProto = Object.prototype;

	/** Used to check objects for own properties. */
	var hasOwnProperty = objectProto.hasOwnProperty;

	/**
	 * Assigns own enumerable string keyed properties of source objects to the
	 * destination object. Source objects are applied from left to right.
	 * Subsequent sources overwrite property assignments of previous sources.
	 *
	 * **Note:** This method mutates `object` and is loosely based on
	 * [`Object.assign`](https://mdn.io/Object/assign).
	 *
	 * @static
	 * @memberOf _
	 * @since 0.10.0
	 * @category Object
	 * @param {Object} object The destination object.
	 * @param {...Object} [sources] The source objects.
	 * @returns {Object} Returns `object`.
	 * @see _.assignIn
	 * @example
	 *
	 * function Foo() {
	 *   this.a = 1;
	 * }
	 *
	 * function Bar() {
	 *   this.c = 3;
	 * }
	 *
	 * Foo.prototype.b = 2;
	 * Bar.prototype.d = 4;
	 *
	 * _.assign({ 'a': 0 }, new Foo, new Bar);
	 * // => { 'a': 1, 'c': 3 }
	 */
	var assign = createAssigner(function(object, source) {
	  if (isPrototype(source) || isArrayLike(source)) {
	    copyObject(source, keys(source), object);
	    return;
	  }
	  for (var key in source) {
	    if (hasOwnProperty.call(source, key)) {
	      assignValue(object, key, source[key]);
	    }
	  }
	});

	assign_1 = assign;
	return assign_1;
}

var hasRequiredModel$1;

function requireModel$1 () {
	if (hasRequiredModel$1) return model$1;
	hasRequiredModel$1 = 1;
	var __extends = (model$1 && model$1.__extends) || (function () {
	    var extendStatics = function (d, b) {
	        extendStatics = Object.setPrototypeOf ||
	            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
	            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
	        return extendStatics(d, b);
	    };
	    return function (d, b) {
	        if (typeof b !== "function" && b !== null)
	            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
	        extendStatics(d, b);
	        function __() { this.constructor = d; }
	        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
	    };
	})();
	var __importDefault = (model$1 && model$1.__importDefault) || function (mod) {
	    return (mod && mod.__esModule) ? mod : { "default": mod };
	};
	Object.defineProperty(model$1, "__esModule", { value: true });
	model$1.serializeProduction = model$1.serializeGrammar = model$1.Terminal = model$1.Alternation = model$1.RepetitionWithSeparator = model$1.Repetition = model$1.RepetitionMandatoryWithSeparator = model$1.RepetitionMandatory = model$1.Option = model$1.Alternative = model$1.Rule = model$1.NonTerminal = model$1.AbstractProduction = void 0;
	var map_1 = __importDefault(requireMap());
	var forEach_1 = __importDefault(requireForEach());
	var isString_1 = __importDefault(requireIsString());
	var isRegExp_1 = __importDefault(requireIsRegExp());
	var pickBy_1 = __importDefault(requirePickBy());
	var assign_1 = __importDefault(requireAssign());
	// TODO: duplicated code to avoid extracting another sub-package -- how to avoid?
	function tokenLabel(tokType) {
	    if (hasTokenLabel(tokType)) {
	        return tokType.LABEL;
	    }
	    else {
	        return tokType.name;
	    }
	}
	// TODO: duplicated code to avoid extracting another sub-package -- how to avoid?
	function hasTokenLabel(obj) {
	    return (0, isString_1.default)(obj.LABEL) && obj.LABEL !== "";
	}
	var AbstractProduction = /** @class */ (function () {
	    function AbstractProduction(_definition) {
	        this._definition = _definition;
	    }
	    Object.defineProperty(AbstractProduction.prototype, "definition", {
	        get: function () {
	            return this._definition;
	        },
	        set: function (value) {
	            this._definition = value;
	        },
	        enumerable: false,
	        configurable: true
	    });
	    AbstractProduction.prototype.accept = function (visitor) {
	        visitor.visit(this);
	        (0, forEach_1.default)(this.definition, function (prod) {
	            prod.accept(visitor);
	        });
	    };
	    return AbstractProduction;
	}());
	model$1.AbstractProduction = AbstractProduction;
	var NonTerminal = /** @class */ (function (_super) {
	    __extends(NonTerminal, _super);
	    function NonTerminal(options) {
	        var _this = _super.call(this, []) || this;
	        _this.idx = 1;
	        (0, assign_1.default)(_this, (0, pickBy_1.default)(options, function (v) { return v !== undefined; }));
	        return _this;
	    }
	    Object.defineProperty(NonTerminal.prototype, "definition", {
	        get: function () {
	            if (this.referencedRule !== undefined) {
	                return this.referencedRule.definition;
	            }
	            return [];
	        },
	        set: function (definition) {
	            // immutable
	        },
	        enumerable: false,
	        configurable: true
	    });
	    NonTerminal.prototype.accept = function (visitor) {
	        visitor.visit(this);
	        // don't visit children of a reference, we will get cyclic infinite loops if we do so
	    };
	    return NonTerminal;
	}(AbstractProduction));
	model$1.NonTerminal = NonTerminal;
	var Rule = /** @class */ (function (_super) {
	    __extends(Rule, _super);
	    function Rule(options) {
	        var _this = _super.call(this, options.definition) || this;
	        _this.orgText = "";
	        (0, assign_1.default)(_this, (0, pickBy_1.default)(options, function (v) { return v !== undefined; }));
	        return _this;
	    }
	    return Rule;
	}(AbstractProduction));
	model$1.Rule = Rule;
	var Alternative = /** @class */ (function (_super) {
	    __extends(Alternative, _super);
	    function Alternative(options) {
	        var _this = _super.call(this, options.definition) || this;
	        _this.ignoreAmbiguities = false;
	        (0, assign_1.default)(_this, (0, pickBy_1.default)(options, function (v) { return v !== undefined; }));
	        return _this;
	    }
	    return Alternative;
	}(AbstractProduction));
	model$1.Alternative = Alternative;
	var Option = /** @class */ (function (_super) {
	    __extends(Option, _super);
	    function Option(options) {
	        var _this = _super.call(this, options.definition) || this;
	        _this.idx = 1;
	        (0, assign_1.default)(_this, (0, pickBy_1.default)(options, function (v) { return v !== undefined; }));
	        return _this;
	    }
	    return Option;
	}(AbstractProduction));
	model$1.Option = Option;
	var RepetitionMandatory = /** @class */ (function (_super) {
	    __extends(RepetitionMandatory, _super);
	    function RepetitionMandatory(options) {
	        var _this = _super.call(this, options.definition) || this;
	        _this.idx = 1;
	        (0, assign_1.default)(_this, (0, pickBy_1.default)(options, function (v) { return v !== undefined; }));
	        return _this;
	    }
	    return RepetitionMandatory;
	}(AbstractProduction));
	model$1.RepetitionMandatory = RepetitionMandatory;
	var RepetitionMandatoryWithSeparator = /** @class */ (function (_super) {
	    __extends(RepetitionMandatoryWithSeparator, _super);
	    function RepetitionMandatoryWithSeparator(options) {
	        var _this = _super.call(this, options.definition) || this;
	        _this.idx = 1;
	        (0, assign_1.default)(_this, (0, pickBy_1.default)(options, function (v) { return v !== undefined; }));
	        return _this;
	    }
	    return RepetitionMandatoryWithSeparator;
	}(AbstractProduction));
	model$1.RepetitionMandatoryWithSeparator = RepetitionMandatoryWithSeparator;
	var Repetition = /** @class */ (function (_super) {
	    __extends(Repetition, _super);
	    function Repetition(options) {
	        var _this = _super.call(this, options.definition) || this;
	        _this.idx = 1;
	        (0, assign_1.default)(_this, (0, pickBy_1.default)(options, function (v) { return v !== undefined; }));
	        return _this;
	    }
	    return Repetition;
	}(AbstractProduction));
	model$1.Repetition = Repetition;
	var RepetitionWithSeparator = /** @class */ (function (_super) {
	    __extends(RepetitionWithSeparator, _super);
	    function RepetitionWithSeparator(options) {
	        var _this = _super.call(this, options.definition) || this;
	        _this.idx = 1;
	        (0, assign_1.default)(_this, (0, pickBy_1.default)(options, function (v) { return v !== undefined; }));
	        return _this;
	    }
	    return RepetitionWithSeparator;
	}(AbstractProduction));
	model$1.RepetitionWithSeparator = RepetitionWithSeparator;
	var Alternation = /** @class */ (function (_super) {
	    __extends(Alternation, _super);
	    function Alternation(options) {
	        var _this = _super.call(this, options.definition) || this;
	        _this.idx = 1;
	        _this.ignoreAmbiguities = false;
	        _this.hasPredicates = false;
	        (0, assign_1.default)(_this, (0, pickBy_1.default)(options, function (v) { return v !== undefined; }));
	        return _this;
	    }
	    Object.defineProperty(Alternation.prototype, "definition", {
	        get: function () {
	            return this._definition;
	        },
	        set: function (value) {
	            this._definition = value;
	        },
	        enumerable: false,
	        configurable: true
	    });
	    return Alternation;
	}(AbstractProduction));
	model$1.Alternation = Alternation;
	var Terminal = /** @class */ (function () {
	    function Terminal(options) {
	        this.idx = 1;
	        (0, assign_1.default)(this, (0, pickBy_1.default)(options, function (v) { return v !== undefined; }));
	    }
	    Terminal.prototype.accept = function (visitor) {
	        visitor.visit(this);
	    };
	    return Terminal;
	}());
	model$1.Terminal = Terminal;
	function serializeGrammar(topRules) {
	    return (0, map_1.default)(topRules, serializeProduction);
	}
	model$1.serializeGrammar = serializeGrammar;
	function serializeProduction(node) {
	    function convertDefinition(definition) {
	        return (0, map_1.default)(definition, serializeProduction);
	    }
	    /* istanbul ignore else */
	    if (node instanceof NonTerminal) {
	        var serializedNonTerminal = {
	            type: "NonTerminal",
	            name: node.nonTerminalName,
	            idx: node.idx
	        };
	        if ((0, isString_1.default)(node.label)) {
	            serializedNonTerminal.label = node.label;
	        }
	        return serializedNonTerminal;
	    }
	    else if (node instanceof Alternative) {
	        return {
	            type: "Alternative",
	            definition: convertDefinition(node.definition)
	        };
	    }
	    else if (node instanceof Option) {
	        return {
	            type: "Option",
	            idx: node.idx,
	            definition: convertDefinition(node.definition)
	        };
	    }
	    else if (node instanceof RepetitionMandatory) {
	        return {
	            type: "RepetitionMandatory",
	            idx: node.idx,
	            definition: convertDefinition(node.definition)
	        };
	    }
	    else if (node instanceof RepetitionMandatoryWithSeparator) {
	        return {
	            type: "RepetitionMandatoryWithSeparator",
	            idx: node.idx,
	            separator: (serializeProduction(new Terminal({ terminalType: node.separator }))),
	            definition: convertDefinition(node.definition)
	        };
	    }
	    else if (node instanceof RepetitionWithSeparator) {
	        return {
	            type: "RepetitionWithSeparator",
	            idx: node.idx,
	            separator: (serializeProduction(new Terminal({ terminalType: node.separator }))),
	            definition: convertDefinition(node.definition)
	        };
	    }
	    else if (node instanceof Repetition) {
	        return {
	            type: "Repetition",
	            idx: node.idx,
	            definition: convertDefinition(node.definition)
	        };
	    }
	    else if (node instanceof Alternation) {
	        return {
	            type: "Alternation",
	            idx: node.idx,
	            definition: convertDefinition(node.definition)
	        };
	    }
	    else if (node instanceof Terminal) {
	        var serializedTerminal = {
	            type: "Terminal",
	            name: node.terminalType.name,
	            label: tokenLabel(node.terminalType),
	            idx: node.idx
	        };
	        if ((0, isString_1.default)(node.label)) {
	            serializedTerminal.terminalLabel = node.label;
	        }
	        var pattern = node.terminalType.PATTERN;
	        if (node.terminalType.PATTERN) {
	            serializedTerminal.pattern = (0, isRegExp_1.default)(pattern)
	                ? pattern.source
	                : pattern;
	        }
	        return serializedTerminal;
	    }
	    else if (node instanceof Rule) {
	        return {
	            type: "Rule",
	            name: node.name,
	            orgText: node.orgText,
	            definition: convertDefinition(node.definition)
	        };
	    }
	    else {
	        throw Error("non exhaustive match");
	    }
	}
	model$1.serializeProduction = serializeProduction;
	
	return model$1;
}

var visitor = {};

var hasRequiredVisitor;

function requireVisitor () {
	if (hasRequiredVisitor) return visitor;
	hasRequiredVisitor = 1;
	Object.defineProperty(visitor, "__esModule", { value: true });
	visitor.GAstVisitor = void 0;
	var model_1 = requireModel$1();
	var GAstVisitor = /** @class */ (function () {
	    function GAstVisitor() {
	    }
	    GAstVisitor.prototype.visit = function (node) {
	        var nodeAny = node;
	        switch (nodeAny.constructor) {
	            case model_1.NonTerminal:
	                return this.visitNonTerminal(nodeAny);
	            case model_1.Alternative:
	                return this.visitAlternative(nodeAny);
	            case model_1.Option:
	                return this.visitOption(nodeAny);
	            case model_1.RepetitionMandatory:
	                return this.visitRepetitionMandatory(nodeAny);
	            case model_1.RepetitionMandatoryWithSeparator:
	                return this.visitRepetitionMandatoryWithSeparator(nodeAny);
	            case model_1.RepetitionWithSeparator:
	                return this.visitRepetitionWithSeparator(nodeAny);
	            case model_1.Repetition:
	                return this.visitRepetition(nodeAny);
	            case model_1.Alternation:
	                return this.visitAlternation(nodeAny);
	            case model_1.Terminal:
	                return this.visitTerminal(nodeAny);
	            case model_1.Rule:
	                return this.visitRule(nodeAny);
	            /* istanbul ignore next */
	            default:
	                throw Error("non exhaustive match");
	        }
	    };
	    /* istanbul ignore next - testing the fact a NOOP function exists is non-trivial  */
	    GAstVisitor.prototype.visitNonTerminal = function (node) { };
	    /* istanbul ignore next - testing the fact a NOOP function exists is non-trivial  */
	    GAstVisitor.prototype.visitAlternative = function (node) { };
	    /* istanbul ignore next - testing the fact a NOOP function exists is non-trivial  */
	    GAstVisitor.prototype.visitOption = function (node) { };
	    /* istanbul ignore next - testing the fact a NOOP function exists is non-trivial  */
	    GAstVisitor.prototype.visitRepetition = function (node) { };
	    /* istanbul ignore next - testing the fact a NOOP function exists is non-trivial  */
	    GAstVisitor.prototype.visitRepetitionMandatory = function (node) { };
	    /* istanbul ignore next - testing the fact a NOOP function exists is non-trivial  */
	    GAstVisitor.prototype.visitRepetitionMandatoryWithSeparator = function (node) { };
	    /* istanbul ignore next - testing the fact a NOOP function exists is non-trivial  */
	    GAstVisitor.prototype.visitRepetitionWithSeparator = function (node) { };
	    /* istanbul ignore next - testing the fact a NOOP function exists is non-trivial  */
	    GAstVisitor.prototype.visitAlternation = function (node) { };
	    /* istanbul ignore next - testing the fact a NOOP function exists is non-trivial  */
	    GAstVisitor.prototype.visitTerminal = function (node) { };
	    /* istanbul ignore next - testing the fact a NOOP function exists is non-trivial  */
	    GAstVisitor.prototype.visitRule = function (node) { };
	    return GAstVisitor;
	}());
	visitor.GAstVisitor = GAstVisitor;
	
	return visitor;
}

var helpers = {};

var _baseSome;
var hasRequired_baseSome;

function require_baseSome () {
	if (hasRequired_baseSome) return _baseSome;
	hasRequired_baseSome = 1;
	var baseEach = require_baseEach();

	/**
	 * The base implementation of `_.some` without support for iteratee shorthands.
	 *
	 * @private
	 * @param {Array|Object} collection The collection to iterate over.
	 * @param {Function} predicate The function invoked per iteration.
	 * @returns {boolean} Returns `true` if any element passes the predicate check,
	 *  else `false`.
	 */
	function baseSome(collection, predicate) {
	  var result;

	  baseEach(collection, function(value, index, collection) {
	    result = predicate(value, index, collection);
	    return !result;
	  });
	  return !!result;
	}

	_baseSome = baseSome;
	return _baseSome;
}

var some_1;
var hasRequiredSome;

function requireSome () {
	if (hasRequiredSome) return some_1;
	hasRequiredSome = 1;
	var arraySome = require_arraySome(),
	    baseIteratee = require_baseIteratee(),
	    baseSome = require_baseSome(),
	    isArray = requireIsArray(),
	    isIterateeCall = require_isIterateeCall();

	/**
	 * Checks if `predicate` returns truthy for **any** element of `collection`.
	 * Iteration is stopped once `predicate` returns truthy. The predicate is
	 * invoked with three arguments: (value, index|key, collection).
	 *
	 * @static
	 * @memberOf _
	 * @since 0.1.0
	 * @category Collection
	 * @param {Array|Object} collection The collection to iterate over.
	 * @param {Function} [predicate=_.identity] The function invoked per iteration.
	 * @param- {Object} [guard] Enables use as an iteratee for methods like `_.map`.
	 * @returns {boolean} Returns `true` if any element passes the predicate check,
	 *  else `false`.
	 * @example
	 *
	 * _.some([null, 0, 'yes', false], Boolean);
	 * // => true
	 *
	 * var users = [
	 *   { 'user': 'barney', 'active': true },
	 *   { 'user': 'fred',   'active': false }
	 * ];
	 *
	 * // The `_.matches` iteratee shorthand.
	 * _.some(users, { 'user': 'barney', 'active': false });
	 * // => false
	 *
	 * // The `_.matchesProperty` iteratee shorthand.
	 * _.some(users, ['active', false]);
	 * // => true
	 *
	 * // The `_.property` iteratee shorthand.
	 * _.some(users, 'active');
	 * // => true
	 */
	function some(collection, predicate, guard) {
	  var func = isArray(collection) ? arraySome : baseSome;
	  if (guard && isIterateeCall(collection, predicate, guard)) {
	    predicate = undefined;
	  }
	  return func(collection, baseIteratee(predicate, 3));
	}

	some_1 = some;
	return some_1;
}

/**
 * A specialized version of `_.every` for arrays without support for
 * iteratee shorthands.
 *
 * @private
 * @param {Array} [array] The array to iterate over.
 * @param {Function} predicate The function invoked per iteration.
 * @returns {boolean} Returns `true` if all elements pass the predicate check,
 *  else `false`.
 */

var _arrayEvery;
var hasRequired_arrayEvery;

function require_arrayEvery () {
	if (hasRequired_arrayEvery) return _arrayEvery;
	hasRequired_arrayEvery = 1;
	function arrayEvery(array, predicate) {
	  var index = -1,
	      length = array == null ? 0 : array.length;

	  while (++index < length) {
	    if (!predicate(array[index], index, array)) {
	      return false;
	    }
	  }
	  return true;
	}

	_arrayEvery = arrayEvery;
	return _arrayEvery;
}

var _baseEvery;
var hasRequired_baseEvery;

function require_baseEvery () {
	if (hasRequired_baseEvery) return _baseEvery;
	hasRequired_baseEvery = 1;
	var baseEach = require_baseEach();

	/**
	 * The base implementation of `_.every` without support for iteratee shorthands.
	 *
	 * @private
	 * @param {Array|Object} collection The collection to iterate over.
	 * @param {Function} predicate The function invoked per iteration.
	 * @returns {boolean} Returns `true` if all elements pass the predicate check,
	 *  else `false`
	 */
	function baseEvery(collection, predicate) {
	  var result = true;
	  baseEach(collection, function(value, index, collection) {
	    result = !!predicate(value, index, collection);
	    return result;
	  });
	  return result;
	}

	_baseEvery = baseEvery;
	return _baseEvery;
}

var every_1;
var hasRequiredEvery;

function requireEvery () {
	if (hasRequiredEvery) return every_1;
	hasRequiredEvery = 1;
	var arrayEvery = require_arrayEvery(),
	    baseEvery = require_baseEvery(),
	    baseIteratee = require_baseIteratee(),
	    isArray = requireIsArray(),
	    isIterateeCall = require_isIterateeCall();

	/**
	 * Checks if `predicate` returns truthy for **all** elements of `collection`.
	 * Iteration is stopped once `predicate` returns falsey. The predicate is
	 * invoked with three arguments: (value, index|key, collection).
	 *
	 * **Note:** This method returns `true` for
	 * [empty collections](https://en.wikipedia.org/wiki/Empty_set) because
	 * [everything is true](https://en.wikipedia.org/wiki/Vacuous_truth) of
	 * elements of empty collections.
	 *
	 * @static
	 * @memberOf _
	 * @since 0.1.0
	 * @category Collection
	 * @param {Array|Object} collection The collection to iterate over.
	 * @param {Function} [predicate=_.identity] The function invoked per iteration.
	 * @param- {Object} [guard] Enables use as an iteratee for methods like `_.map`.
	 * @returns {boolean} Returns `true` if all elements pass the predicate check,
	 *  else `false`.
	 * @example
	 *
	 * _.every([true, 1, null, 'yes'], Boolean);
	 * // => false
	 *
	 * var users = [
	 *   { 'user': 'barney', 'age': 36, 'active': false },
	 *   { 'user': 'fred',   'age': 40, 'active': false }
	 * ];
	 *
	 * // The `_.matches` iteratee shorthand.
	 * _.every(users, { 'user': 'barney', 'active': false });
	 * // => false
	 *
	 * // The `_.matchesProperty` iteratee shorthand.
	 * _.every(users, ['active', false]);
	 * // => true
	 *
	 * // The `_.property` iteratee shorthand.
	 * _.every(users, 'active');
	 * // => false
	 */
	function every(collection, predicate, guard) {
	  var func = isArray(collection) ? arrayEvery : baseEvery;
	  if (guard && isIterateeCall(collection, predicate, guard)) {
	    predicate = undefined;
	  }
	  return func(collection, baseIteratee(predicate, 3));
	}

	every_1 = every;
	return every_1;
}

/**
 * The base implementation of `_.findIndex` and `_.findLastIndex` without
 * support for iteratee shorthands.
 *
 * @private
 * @param {Array} array The array to inspect.
 * @param {Function} predicate The function invoked per iteration.
 * @param {number} fromIndex The index to search from.
 * @param {boolean} [fromRight] Specify iterating from right to left.
 * @returns {number} Returns the index of the matched value, else `-1`.
 */

var _baseFindIndex;
var hasRequired_baseFindIndex;

function require_baseFindIndex () {
	if (hasRequired_baseFindIndex) return _baseFindIndex;
	hasRequired_baseFindIndex = 1;
	function baseFindIndex(array, predicate, fromIndex, fromRight) {
	  var length = array.length,
	      index = fromIndex + (fromRight ? 1 : -1);

	  while ((fromRight ? index-- : ++index < length)) {
	    if (predicate(array[index], index, array)) {
	      return index;
	    }
	  }
	  return -1;
	}

	_baseFindIndex = baseFindIndex;
	return _baseFindIndex;
}

/**
 * The base implementation of `_.isNaN` without support for number objects.
 *
 * @private
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is `NaN`, else `false`.
 */

var _baseIsNaN;
var hasRequired_baseIsNaN;

function require_baseIsNaN () {
	if (hasRequired_baseIsNaN) return _baseIsNaN;
	hasRequired_baseIsNaN = 1;
	function baseIsNaN(value) {
	  return value !== value;
	}

	_baseIsNaN = baseIsNaN;
	return _baseIsNaN;
}

/**
 * A specialized version of `_.indexOf` which performs strict equality
 * comparisons of values, i.e. `===`.
 *
 * @private
 * @param {Array} array The array to inspect.
 * @param {*} value The value to search for.
 * @param {number} fromIndex The index to search from.
 * @returns {number} Returns the index of the matched value, else `-1`.
 */

var _strictIndexOf;
var hasRequired_strictIndexOf;

function require_strictIndexOf () {
	if (hasRequired_strictIndexOf) return _strictIndexOf;
	hasRequired_strictIndexOf = 1;
	function strictIndexOf(array, value, fromIndex) {
	  var index = fromIndex - 1,
	      length = array.length;

	  while (++index < length) {
	    if (array[index] === value) {
	      return index;
	    }
	  }
	  return -1;
	}

	_strictIndexOf = strictIndexOf;
	return _strictIndexOf;
}

var _baseIndexOf;
var hasRequired_baseIndexOf;

function require_baseIndexOf () {
	if (hasRequired_baseIndexOf) return _baseIndexOf;
	hasRequired_baseIndexOf = 1;
	var baseFindIndex = require_baseFindIndex(),
	    baseIsNaN = require_baseIsNaN(),
	    strictIndexOf = require_strictIndexOf();

	/**
	 * The base implementation of `_.indexOf` without `fromIndex` bounds checks.
	 *
	 * @private
	 * @param {Array} array The array to inspect.
	 * @param {*} value The value to search for.
	 * @param {number} fromIndex The index to search from.
	 * @returns {number} Returns the index of the matched value, else `-1`.
	 */
	function baseIndexOf(array, value, fromIndex) {
	  return value === value
	    ? strictIndexOf(array, value, fromIndex)
	    : baseFindIndex(array, baseIsNaN, fromIndex);
	}

	_baseIndexOf = baseIndexOf;
	return _baseIndexOf;
}

var includes_1;
var hasRequiredIncludes;

function requireIncludes () {
	if (hasRequiredIncludes) return includes_1;
	hasRequiredIncludes = 1;
	var baseIndexOf = require_baseIndexOf(),
	    isArrayLike = requireIsArrayLike(),
	    isString = requireIsString(),
	    toInteger = requireToInteger(),
	    values = requireValues();

	/* Built-in method references for those with the same name as other `lodash` methods. */
	var nativeMax = Math.max;

	/**
	 * Checks if `value` is in `collection`. If `collection` is a string, it's
	 * checked for a substring of `value`, otherwise
	 * [`SameValueZero`](http://ecma-international.org/ecma-262/7.0/#sec-samevaluezero)
	 * is used for equality comparisons. If `fromIndex` is negative, it's used as
	 * the offset from the end of `collection`.
	 *
	 * @static
	 * @memberOf _
	 * @since 0.1.0
	 * @category Collection
	 * @param {Array|Object|string} collection The collection to inspect.
	 * @param {*} value The value to search for.
	 * @param {number} [fromIndex=0] The index to search from.
	 * @param- {Object} [guard] Enables use as an iteratee for methods like `_.reduce`.
	 * @returns {boolean} Returns `true` if `value` is found, else `false`.
	 * @example
	 *
	 * _.includes([1, 2, 3], 1);
	 * // => true
	 *
	 * _.includes([1, 2, 3], 1, 2);
	 * // => false
	 *
	 * _.includes({ 'a': 1, 'b': 2 }, 1);
	 * // => true
	 *
	 * _.includes('abcd', 'bc');
	 * // => true
	 */
	function includes(collection, value, fromIndex, guard) {
	  collection = isArrayLike(collection) ? collection : values(collection);
	  fromIndex = (fromIndex && !guard) ? toInteger(fromIndex) : 0;

	  var length = collection.length;
	  if (fromIndex < 0) {
	    fromIndex = nativeMax(length + fromIndex, 0);
	  }
	  return isString(collection)
	    ? (fromIndex <= length && collection.indexOf(value, fromIndex) > -1)
	    : (!!length && baseIndexOf(collection, value, fromIndex) > -1);
	}

	includes_1 = includes;
	return includes_1;
}

var hasRequiredHelpers;

function requireHelpers () {
	if (hasRequiredHelpers) return helpers;
	hasRequiredHelpers = 1;
	var __importDefault = (helpers && helpers.__importDefault) || function (mod) {
	    return (mod && mod.__esModule) ? mod : { "default": mod };
	};
	Object.defineProperty(helpers, "__esModule", { value: true });
	helpers.getProductionDslName = helpers.isBranchingProd = helpers.isOptionalProd = helpers.isSequenceProd = void 0;
	var some_1 = __importDefault(requireSome());
	var every_1 = __importDefault(requireEvery());
	var includes_1 = __importDefault(requireIncludes());
	var model_1 = requireModel$1();
	function isSequenceProd(prod) {
	    return (prod instanceof model_1.Alternative ||
	        prod instanceof model_1.Option ||
	        prod instanceof model_1.Repetition ||
	        prod instanceof model_1.RepetitionMandatory ||
	        prod instanceof model_1.RepetitionMandatoryWithSeparator ||
	        prod instanceof model_1.RepetitionWithSeparator ||
	        prod instanceof model_1.Terminal ||
	        prod instanceof model_1.Rule);
	}
	helpers.isSequenceProd = isSequenceProd;
	function isOptionalProd(prod, alreadyVisited) {
	    if (alreadyVisited === void 0) { alreadyVisited = []; }
	    var isDirectlyOptional = prod instanceof model_1.Option ||
	        prod instanceof model_1.Repetition ||
	        prod instanceof model_1.RepetitionWithSeparator;
	    if (isDirectlyOptional) {
	        return true;
	    }
	    // note that this can cause infinite loop if one optional empty TOP production has a cyclic dependency with another
	    // empty optional top rule
	    // may be indirectly optional ((A?B?C?) | (D?E?F?))
	    if (prod instanceof model_1.Alternation) {
	        // for OR its enough for just one of the alternatives to be optional
	        return (0, some_1.default)(prod.definition, function (subProd) {
	            return isOptionalProd(subProd, alreadyVisited);
	        });
	    }
	    else if (prod instanceof model_1.NonTerminal && (0, includes_1.default)(alreadyVisited, prod)) {
	        // avoiding stack overflow due to infinite recursion
	        return false;
	    }
	    else if (prod instanceof model_1.AbstractProduction) {
	        if (prod instanceof model_1.NonTerminal) {
	            alreadyVisited.push(prod);
	        }
	        return (0, every_1.default)(prod.definition, function (subProd) {
	            return isOptionalProd(subProd, alreadyVisited);
	        });
	    }
	    else {
	        return false;
	    }
	}
	helpers.isOptionalProd = isOptionalProd;
	function isBranchingProd(prod) {
	    return prod instanceof model_1.Alternation;
	}
	helpers.isBranchingProd = isBranchingProd;
	function getProductionDslName(prod) {
	    /* istanbul ignore else */
	    if (prod instanceof model_1.NonTerminal) {
	        return "SUBRULE";
	    }
	    else if (prod instanceof model_1.Option) {
	        return "OPTION";
	    }
	    else if (prod instanceof model_1.Alternation) {
	        return "OR";
	    }
	    else if (prod instanceof model_1.RepetitionMandatory) {
	        return "AT_LEAST_ONE";
	    }
	    else if (prod instanceof model_1.RepetitionMandatoryWithSeparator) {
	        return "AT_LEAST_ONE_SEP";
	    }
	    else if (prod instanceof model_1.RepetitionWithSeparator) {
	        return "MANY_SEP";
	    }
	    else if (prod instanceof model_1.Repetition) {
	        return "MANY";
	    }
	    else if (prod instanceof model_1.Terminal) {
	        return "CONSUME";
	    }
	    else {
	        throw Error("non exhaustive match");
	    }
	}
	helpers.getProductionDslName = getProductionDslName;
	
	return helpers;
}

var hasRequiredApi$2;

function requireApi$2 () {
	if (hasRequiredApi$2) return api$1;
	hasRequiredApi$2 = 1;
	(function (exports) {
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.isSequenceProd = exports.isBranchingProd = exports.isOptionalProd = exports.getProductionDslName = exports.GAstVisitor = exports.serializeProduction = exports.serializeGrammar = exports.Alternative = exports.Alternation = exports.RepetitionWithSeparator = exports.RepetitionMandatoryWithSeparator = exports.RepetitionMandatory = exports.Repetition = exports.Option = exports.NonTerminal = exports.Terminal = exports.Rule = void 0;
		var model_1 = requireModel$1();
		Object.defineProperty(exports, "Rule", { enumerable: true, get: function () { return model_1.Rule; } });
		Object.defineProperty(exports, "Terminal", { enumerable: true, get: function () { return model_1.Terminal; } });
		Object.defineProperty(exports, "NonTerminal", { enumerable: true, get: function () { return model_1.NonTerminal; } });
		Object.defineProperty(exports, "Option", { enumerable: true, get: function () { return model_1.Option; } });
		Object.defineProperty(exports, "Repetition", { enumerable: true, get: function () { return model_1.Repetition; } });
		Object.defineProperty(exports, "RepetitionMandatory", { enumerable: true, get: function () { return model_1.RepetitionMandatory; } });
		Object.defineProperty(exports, "RepetitionMandatoryWithSeparator", { enumerable: true, get: function () { return model_1.RepetitionMandatoryWithSeparator; } });
		Object.defineProperty(exports, "RepetitionWithSeparator", { enumerable: true, get: function () { return model_1.RepetitionWithSeparator; } });
		Object.defineProperty(exports, "Alternation", { enumerable: true, get: function () { return model_1.Alternation; } });
		Object.defineProperty(exports, "Alternative", { enumerable: true, get: function () { return model_1.Alternative; } });
		Object.defineProperty(exports, "serializeGrammar", { enumerable: true, get: function () { return model_1.serializeGrammar; } });
		Object.defineProperty(exports, "serializeProduction", { enumerable: true, get: function () { return model_1.serializeProduction; } });
		var visitor_1 = requireVisitor();
		Object.defineProperty(exports, "GAstVisitor", { enumerable: true, get: function () { return visitor_1.GAstVisitor; } });
		var helpers_1 = requireHelpers();
		Object.defineProperty(exports, "getProductionDslName", { enumerable: true, get: function () { return helpers_1.getProductionDslName; } });
		Object.defineProperty(exports, "isOptionalProd", { enumerable: true, get: function () { return helpers_1.isOptionalProd; } });
		Object.defineProperty(exports, "isBranchingProd", { enumerable: true, get: function () { return helpers_1.isBranchingProd; } });
		Object.defineProperty(exports, "isSequenceProd", { enumerable: true, get: function () { return helpers_1.isSequenceProd; } });
		
	} (api$1));
	return api$1;
}

var hasRequiredRest;

function requireRest () {
	if (hasRequiredRest) return rest;
	hasRequiredRest = 1;
	var __importDefault = (rest && rest.__importDefault) || function (mod) {
	    return (mod && mod.__esModule) ? mod : { "default": mod };
	};
	Object.defineProperty(rest, "__esModule", { value: true });
	rest.RestWalker = void 0;
	var drop_1 = __importDefault(requireDrop());
	var forEach_1 = __importDefault(requireForEach());
	var gast_1 = requireApi$2();
	/**
	 *  A Grammar Walker that computes the "remaining" grammar "after" a productions in the grammar.
	 */
	var RestWalker = /** @class */ (function () {
	    function RestWalker() {
	    }
	    RestWalker.prototype.walk = function (prod, prevRest) {
	        var _this = this;
	        if (prevRest === void 0) { prevRest = []; }
	        (0, forEach_1.default)(prod.definition, function (subProd, index) {
	            var currRest = (0, drop_1.default)(prod.definition, index + 1);
	            /* istanbul ignore else */
	            if (subProd instanceof gast_1.NonTerminal) {
	                _this.walkProdRef(subProd, currRest, prevRest);
	            }
	            else if (subProd instanceof gast_1.Terminal) {
	                _this.walkTerminal(subProd, currRest, prevRest);
	            }
	            else if (subProd instanceof gast_1.Alternative) {
	                _this.walkFlat(subProd, currRest, prevRest);
	            }
	            else if (subProd instanceof gast_1.Option) {
	                _this.walkOption(subProd, currRest, prevRest);
	            }
	            else if (subProd instanceof gast_1.RepetitionMandatory) {
	                _this.walkAtLeastOne(subProd, currRest, prevRest);
	            }
	            else if (subProd instanceof gast_1.RepetitionMandatoryWithSeparator) {
	                _this.walkAtLeastOneSep(subProd, currRest, prevRest);
	            }
	            else if (subProd instanceof gast_1.RepetitionWithSeparator) {
	                _this.walkManySep(subProd, currRest, prevRest);
	            }
	            else if (subProd instanceof gast_1.Repetition) {
	                _this.walkMany(subProd, currRest, prevRest);
	            }
	            else if (subProd instanceof gast_1.Alternation) {
	                _this.walkOr(subProd, currRest, prevRest);
	            }
	            else {
	                throw Error("non exhaustive match");
	            }
	        });
	    };
	    RestWalker.prototype.walkTerminal = function (terminal, currRest, prevRest) { };
	    RestWalker.prototype.walkProdRef = function (refProd, currRest, prevRest) { };
	    RestWalker.prototype.walkFlat = function (flatProd, currRest, prevRest) {
	        // ABCDEF => after the D the rest is EF
	        var fullOrRest = currRest.concat(prevRest);
	        this.walk(flatProd, fullOrRest);
	    };
	    RestWalker.prototype.walkOption = function (optionProd, currRest, prevRest) {
	        // ABC(DE)?F => after the (DE)? the rest is F
	        var fullOrRest = currRest.concat(prevRest);
	        this.walk(optionProd, fullOrRest);
	    };
	    RestWalker.prototype.walkAtLeastOne = function (atLeastOneProd, currRest, prevRest) {
	        // ABC(DE)+F => after the (DE)+ the rest is (DE)?F
	        var fullAtLeastOneRest = [
	            new gast_1.Option({ definition: atLeastOneProd.definition })
	        ].concat(currRest, prevRest);
	        this.walk(atLeastOneProd, fullAtLeastOneRest);
	    };
	    RestWalker.prototype.walkAtLeastOneSep = function (atLeastOneSepProd, currRest, prevRest) {
	        // ABC DE(,DE)* F => after the (,DE)+ the rest is (,DE)?F
	        var fullAtLeastOneSepRest = restForRepetitionWithSeparator(atLeastOneSepProd, currRest, prevRest);
	        this.walk(atLeastOneSepProd, fullAtLeastOneSepRest);
	    };
	    RestWalker.prototype.walkMany = function (manyProd, currRest, prevRest) {
	        // ABC(DE)*F => after the (DE)* the rest is (DE)?F
	        var fullManyRest = [
	            new gast_1.Option({ definition: manyProd.definition })
	        ].concat(currRest, prevRest);
	        this.walk(manyProd, fullManyRest);
	    };
	    RestWalker.prototype.walkManySep = function (manySepProd, currRest, prevRest) {
	        // ABC (DE(,DE)*)? F => after the (,DE)* the rest is (,DE)?F
	        var fullManySepRest = restForRepetitionWithSeparator(manySepProd, currRest, prevRest);
	        this.walk(manySepProd, fullManySepRest);
	    };
	    RestWalker.prototype.walkOr = function (orProd, currRest, prevRest) {
	        var _this = this;
	        // ABC(D|E|F)G => when finding the (D|E|F) the rest is G
	        var fullOrRest = currRest.concat(prevRest);
	        // walk all different alternatives
	        (0, forEach_1.default)(orProd.definition, function (alt) {
	            // wrapping each alternative in a single definition wrapper
	            // to avoid errors in computing the rest of that alternative in the invocation to computeInProdFollows
	            // (otherwise for OR([alt1,alt2]) alt2 will be considered in 'rest' of alt1
	            var prodWrapper = new gast_1.Alternative({ definition: [alt] });
	            _this.walk(prodWrapper, fullOrRest);
	        });
	    };
	    return RestWalker;
	}());
	rest.RestWalker = RestWalker;
	function restForRepetitionWithSeparator(repSepProd, currRest, prevRest) {
	    var repSepRest = [
	        new gast_1.Option({
	            definition: [
	                new gast_1.Terminal({ terminalType: repSepProd.separator })
	            ].concat(repSepProd.definition)
	        })
	    ];
	    var fullRepSepRest = repSepRest.concat(currRest, prevRest);
	    return fullRepSepRest;
	}
	
	return rest;
}

var first$1 = {};

var _isFlattenable;
var hasRequired_isFlattenable;

function require_isFlattenable () {
	if (hasRequired_isFlattenable) return _isFlattenable;
	hasRequired_isFlattenable = 1;
	var Symbol = require_Symbol(),
	    isArguments = requireIsArguments(),
	    isArray = requireIsArray();

	/** Built-in value references. */
	var spreadableSymbol = Symbol ? Symbol.isConcatSpreadable : undefined;

	/**
	 * Checks if `value` is a flattenable `arguments` object or array.
	 *
	 * @private
	 * @param {*} value The value to check.
	 * @returns {boolean} Returns `true` if `value` is flattenable, else `false`.
	 */
	function isFlattenable(value) {
	  return isArray(value) || isArguments(value) ||
	    !!(spreadableSymbol && value && value[spreadableSymbol]);
	}

	_isFlattenable = isFlattenable;
	return _isFlattenable;
}

var _baseFlatten;
var hasRequired_baseFlatten;

function require_baseFlatten () {
	if (hasRequired_baseFlatten) return _baseFlatten;
	hasRequired_baseFlatten = 1;
	var arrayPush = require_arrayPush(),
	    isFlattenable = require_isFlattenable();

	/**
	 * The base implementation of `_.flatten` with support for restricting flattening.
	 *
	 * @private
	 * @param {Array} array The array to flatten.
	 * @param {number} depth The maximum recursion depth.
	 * @param {boolean} [predicate=isFlattenable] The function invoked per iteration.
	 * @param {boolean} [isStrict] Restrict to values that pass `predicate` checks.
	 * @param {Array} [result=[]] The initial result value.
	 * @returns {Array} Returns the new flattened array.
	 */
	function baseFlatten(array, depth, predicate, isStrict, result) {
	  var index = -1,
	      length = array.length;

	  predicate || (predicate = isFlattenable);
	  result || (result = []);

	  while (++index < length) {
	    var value = array[index];
	    if (depth > 0 && predicate(value)) {
	      if (depth > 1) {
	        // Recursively flatten arrays (susceptible to call stack limits).
	        baseFlatten(value, depth - 1, predicate, isStrict, result);
	      } else {
	        arrayPush(result, value);
	      }
	    } else if (!isStrict) {
	      result[result.length] = value;
	    }
	  }
	  return result;
	}

	_baseFlatten = baseFlatten;
	return _baseFlatten;
}

var flatten_1;
var hasRequiredFlatten;

function requireFlatten () {
	if (hasRequiredFlatten) return flatten_1;
	hasRequiredFlatten = 1;
	var baseFlatten = require_baseFlatten();

	/**
	 * Flattens `array` a single level deep.
	 *
	 * @static
	 * @memberOf _
	 * @since 0.1.0
	 * @category Array
	 * @param {Array} array The array to flatten.
	 * @returns {Array} Returns the new flattened array.
	 * @example
	 *
	 * _.flatten([1, [2, [3, [4]], 5]]);
	 * // => [1, 2, [3, [4]], 5]
	 */
	function flatten(array) {
	  var length = array == null ? 0 : array.length;
	  return length ? baseFlatten(array, 1) : [];
	}

	flatten_1 = flatten;
	return flatten_1;
}

var _arrayIncludes;
var hasRequired_arrayIncludes;

function require_arrayIncludes () {
	if (hasRequired_arrayIncludes) return _arrayIncludes;
	hasRequired_arrayIncludes = 1;
	var baseIndexOf = require_baseIndexOf();

	/**
	 * A specialized version of `_.includes` for arrays without support for
	 * specifying an index to search from.
	 *
	 * @private
	 * @param {Array} [array] The array to inspect.
	 * @param {*} target The value to search for.
	 * @returns {boolean} Returns `true` if `target` is found, else `false`.
	 */
	function arrayIncludes(array, value) {
	  var length = array == null ? 0 : array.length;
	  return !!length && baseIndexOf(array, value, 0) > -1;
	}

	_arrayIncludes = arrayIncludes;
	return _arrayIncludes;
}

/**
 * This function is like `arrayIncludes` except that it accepts a comparator.
 *
 * @private
 * @param {Array} [array] The array to inspect.
 * @param {*} target The value to search for.
 * @param {Function} comparator The comparator invoked per element.
 * @returns {boolean} Returns `true` if `target` is found, else `false`.
 */

var _arrayIncludesWith;
var hasRequired_arrayIncludesWith;

function require_arrayIncludesWith () {
	if (hasRequired_arrayIncludesWith) return _arrayIncludesWith;
	hasRequired_arrayIncludesWith = 1;
	function arrayIncludesWith(array, value, comparator) {
	  var index = -1,
	      length = array == null ? 0 : array.length;

	  while (++index < length) {
	    if (comparator(value, array[index])) {
	      return true;
	    }
	  }
	  return false;
	}

	_arrayIncludesWith = arrayIncludesWith;
	return _arrayIncludesWith;
}

/**
 * This method returns `undefined`.
 *
 * @static
 * @memberOf _
 * @since 2.3.0
 * @category Util
 * @example
 *
 * _.times(2, _.noop);
 * // => [undefined, undefined]
 */

var noop_1;
var hasRequiredNoop;

function requireNoop () {
	if (hasRequiredNoop) return noop_1;
	hasRequiredNoop = 1;
	function noop() {
	  // No operation performed.
	}

	noop_1 = noop;
	return noop_1;
}

var _createSet;
var hasRequired_createSet;

function require_createSet () {
	if (hasRequired_createSet) return _createSet;
	hasRequired_createSet = 1;
	var Set = require_Set(),
	    noop = requireNoop(),
	    setToArray = require_setToArray();

	/** Used as references for various `Number` constants. */
	var INFINITY = 1 / 0;

	/**
	 * Creates a set object of `values`.
	 *
	 * @private
	 * @param {Array} values The values to add to the set.
	 * @returns {Object} Returns the new set.
	 */
	var createSet = !(Set && (1 / setToArray(new Set([,-0]))[1]) == INFINITY) ? noop : function(values) {
	  return new Set(values);
	};

	_createSet = createSet;
	return _createSet;
}

var _baseUniq;
var hasRequired_baseUniq;

function require_baseUniq () {
	if (hasRequired_baseUniq) return _baseUniq;
	hasRequired_baseUniq = 1;
	var SetCache = require_SetCache(),
	    arrayIncludes = require_arrayIncludes(),
	    arrayIncludesWith = require_arrayIncludesWith(),
	    cacheHas = require_cacheHas(),
	    createSet = require_createSet(),
	    setToArray = require_setToArray();

	/** Used as the size to enable large array optimizations. */
	var LARGE_ARRAY_SIZE = 200;

	/**
	 * The base implementation of `_.uniqBy` without support for iteratee shorthands.
	 *
	 * @private
	 * @param {Array} array The array to inspect.
	 * @param {Function} [iteratee] The iteratee invoked per element.
	 * @param {Function} [comparator] The comparator invoked per element.
	 * @returns {Array} Returns the new duplicate free array.
	 */
	function baseUniq(array, iteratee, comparator) {
	  var index = -1,
	      includes = arrayIncludes,
	      length = array.length,
	      isCommon = true,
	      result = [],
	      seen = result;

	  if (comparator) {
	    isCommon = false;
	    includes = arrayIncludesWith;
	  }
	  else if (length >= LARGE_ARRAY_SIZE) {
	    var set = iteratee ? null : createSet(array);
	    if (set) {
	      return setToArray(set);
	    }
	    isCommon = false;
	    includes = cacheHas;
	    seen = new SetCache;
	  }
	  else {
	    seen = iteratee ? [] : result;
	  }
	  outer:
	  while (++index < length) {
	    var value = array[index],
	        computed = iteratee ? iteratee(value) : value;

	    value = (comparator || value !== 0) ? value : 0;
	    if (isCommon && computed === computed) {
	      var seenIndex = seen.length;
	      while (seenIndex--) {
	        if (seen[seenIndex] === computed) {
	          continue outer;
	        }
	      }
	      if (iteratee) {
	        seen.push(computed);
	      }
	      result.push(value);
	    }
	    else if (!includes(seen, computed, comparator)) {
	      if (seen !== result) {
	        seen.push(computed);
	      }
	      result.push(value);
	    }
	  }
	  return result;
	}

	_baseUniq = baseUniq;
	return _baseUniq;
}

var uniq_1;
var hasRequiredUniq;

function requireUniq () {
	if (hasRequiredUniq) return uniq_1;
	hasRequiredUniq = 1;
	var baseUniq = require_baseUniq();

	/**
	 * Creates a duplicate-free version of an array, using
	 * [`SameValueZero`](http://ecma-international.org/ecma-262/7.0/#sec-samevaluezero)
	 * for equality comparisons, in which only the first occurrence of each element
	 * is kept. The order of result values is determined by the order they occur
	 * in the array.
	 *
	 * @static
	 * @memberOf _
	 * @since 0.1.0
	 * @category Array
	 * @param {Array} array The array to inspect.
	 * @returns {Array} Returns the new duplicate free array.
	 * @example
	 *
	 * _.uniq([2, 1, 2]);
	 * // => [2, 1]
	 */
	function uniq(array) {
	  return (array && array.length) ? baseUniq(array) : [];
	}

	uniq_1 = uniq;
	return uniq_1;
}

var hasRequiredFirst$1;

function requireFirst$1 () {
	if (hasRequiredFirst$1) return first$1;
	hasRequiredFirst$1 = 1;
	var __importDefault = (first$1 && first$1.__importDefault) || function (mod) {
	    return (mod && mod.__esModule) ? mod : { "default": mod };
	};
	Object.defineProperty(first$1, "__esModule", { value: true });
	first$1.firstForTerminal = first$1.firstForBranching = first$1.firstForSequence = first$1.first = void 0;
	var flatten_1 = __importDefault(requireFlatten());
	var uniq_1 = __importDefault(requireUniq());
	var map_1 = __importDefault(requireMap());
	var gast_1 = requireApi$2();
	var gast_2 = requireApi$2();
	function first(prod) {
	    /* istanbul ignore else */
	    if (prod instanceof gast_1.NonTerminal) {
	        // this could in theory cause infinite loops if
	        // (1) prod A refs prod B.
	        // (2) prod B refs prod A
	        // (3) AB can match the empty set
	        // in other words a cycle where everything is optional so the first will keep
	        // looking ahead for the next optional part and will never exit
	        // currently there is no safeguard for this unique edge case because
	        // (1) not sure a grammar in which this can happen is useful for anything (productive)
	        return first(prod.referencedRule);
	    }
	    else if (prod instanceof gast_1.Terminal) {
	        return firstForTerminal(prod);
	    }
	    else if ((0, gast_2.isSequenceProd)(prod)) {
	        return firstForSequence(prod);
	    }
	    else if ((0, gast_2.isBranchingProd)(prod)) {
	        return firstForBranching(prod);
	    }
	    else {
	        throw Error("non exhaustive match");
	    }
	}
	first$1.first = first;
	function firstForSequence(prod) {
	    var firstSet = [];
	    var seq = prod.definition;
	    var nextSubProdIdx = 0;
	    var hasInnerProdsRemaining = seq.length > nextSubProdIdx;
	    var currSubProd;
	    // so we enter the loop at least once (if the definition is not empty
	    var isLastInnerProdOptional = true;
	    // scan a sequence until it's end or until we have found a NONE optional production in it
	    while (hasInnerProdsRemaining && isLastInnerProdOptional) {
	        currSubProd = seq[nextSubProdIdx];
	        isLastInnerProdOptional = (0, gast_2.isOptionalProd)(currSubProd);
	        firstSet = firstSet.concat(first(currSubProd));
	        nextSubProdIdx = nextSubProdIdx + 1;
	        hasInnerProdsRemaining = seq.length > nextSubProdIdx;
	    }
	    return (0, uniq_1.default)(firstSet);
	}
	first$1.firstForSequence = firstForSequence;
	function firstForBranching(prod) {
	    var allAlternativesFirsts = (0, map_1.default)(prod.definition, function (innerProd) {
	        return first(innerProd);
	    });
	    return (0, uniq_1.default)((0, flatten_1.default)(allAlternativesFirsts));
	}
	first$1.firstForBranching = firstForBranching;
	function firstForTerminal(terminal) {
	    return [terminal.terminalType];
	}
	first$1.firstForTerminal = firstForTerminal;
	
	return first$1;
}

var constants = {};

var hasRequiredConstants;

function requireConstants () {
	if (hasRequiredConstants) return constants;
	hasRequiredConstants = 1;
	Object.defineProperty(constants, "__esModule", { value: true });
	constants.IN = void 0;
	// TODO: can this be removed? where is it used?
	constants.IN = "_~IN~_";
	
	return constants;
}

var hasRequiredFollow;

function requireFollow () {
	if (hasRequiredFollow) return follow;
	hasRequiredFollow = 1;
	var __extends = (follow && follow.__extends) || (function () {
	    var extendStatics = function (d, b) {
	        extendStatics = Object.setPrototypeOf ||
	            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
	            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
	        return extendStatics(d, b);
	    };
	    return function (d, b) {
	        if (typeof b !== "function" && b !== null)
	            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
	        extendStatics(d, b);
	        function __() { this.constructor = d; }
	        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
	    };
	})();
	var __importDefault = (follow && follow.__importDefault) || function (mod) {
	    return (mod && mod.__esModule) ? mod : { "default": mod };
	};
	Object.defineProperty(follow, "__esModule", { value: true });
	follow.buildInProdFollowPrefix = follow.buildBetweenProdsFollowPrefix = follow.computeAllProdsFollows = follow.ResyncFollowsWalker = void 0;
	var rest_1 = requireRest();
	var first_1 = requireFirst$1();
	var forEach_1 = __importDefault(requireForEach());
	var assign_1 = __importDefault(requireAssign());
	var constants_1 = requireConstants();
	var gast_1 = requireApi$2();
	// This ResyncFollowsWalker computes all of the follows required for RESYNC
	// (skipping reference production).
	var ResyncFollowsWalker = /** @class */ (function (_super) {
	    __extends(ResyncFollowsWalker, _super);
	    function ResyncFollowsWalker(topProd) {
	        var _this = _super.call(this) || this;
	        _this.topProd = topProd;
	        _this.follows = {};
	        return _this;
	    }
	    ResyncFollowsWalker.prototype.startWalking = function () {
	        this.walk(this.topProd);
	        return this.follows;
	    };
	    ResyncFollowsWalker.prototype.walkTerminal = function (terminal, currRest, prevRest) {
	        // do nothing! just like in the public sector after 13:00
	    };
	    ResyncFollowsWalker.prototype.walkProdRef = function (refProd, currRest, prevRest) {
	        var followName = buildBetweenProdsFollowPrefix(refProd.referencedRule, refProd.idx) +
	            this.topProd.name;
	        var fullRest = currRest.concat(prevRest);
	        var restProd = new gast_1.Alternative({ definition: fullRest });
	        var t_in_topProd_follows = (0, first_1.first)(restProd);
	        this.follows[followName] = t_in_topProd_follows;
	    };
	    return ResyncFollowsWalker;
	}(rest_1.RestWalker));
	follow.ResyncFollowsWalker = ResyncFollowsWalker;
	function computeAllProdsFollows(topProductions) {
	    var reSyncFollows = {};
	    (0, forEach_1.default)(topProductions, function (topProd) {
	        var currRefsFollow = new ResyncFollowsWalker(topProd).startWalking();
	        (0, assign_1.default)(reSyncFollows, currRefsFollow);
	    });
	    return reSyncFollows;
	}
	follow.computeAllProdsFollows = computeAllProdsFollows;
	function buildBetweenProdsFollowPrefix(inner, occurenceInParent) {
	    return inner.name + occurenceInParent + constants_1.IN;
	}
	follow.buildBetweenProdsFollowPrefix = buildBetweenProdsFollowPrefix;
	function buildInProdFollowPrefix(terminal) {
	    var terminalName = terminal.terminalType.name;
	    return terminalName + terminal.idx + constants_1.IN;
	}
	follow.buildInProdFollowPrefix = buildInProdFollowPrefix;
	
	return follow;
}

var tokens_public = {};

/**
 * Checks if `value` is `undefined`.
 *
 * @static
 * @since 0.1.0
 * @memberOf _
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is `undefined`, else `false`.
 * @example
 *
 * _.isUndefined(void 0);
 * // => true
 *
 * _.isUndefined(null);
 * // => false
 */

var isUndefined_1;
var hasRequiredIsUndefined;

function requireIsUndefined () {
	if (hasRequiredIsUndefined) return isUndefined_1;
	hasRequiredIsUndefined = 1;
	function isUndefined(value) {
	  return value === undefined;
	}

	isUndefined_1 = isUndefined;
	return isUndefined_1;
}

var lexer_public = {};

var lexer$2 = {};

var regexpToAst$1 = {exports: {}};

var regexpToAst = regexpToAst$1.exports;

var hasRequiredRegexpToAst;

function requireRegexpToAst () {
	if (hasRequiredRegexpToAst) return regexpToAst$1.exports;
	hasRequiredRegexpToAst = 1;
	(function (module) {
(function(root, factory) {
		    // istanbul ignore next
		    if (module.exports) {
		        module.exports = factory();
		    } else {
		        // istanbul ignore next
		        root.regexpToAst = factory();
		    }
		})(
		    typeof self !== "undefined"
		        ? // istanbul ignore next
		          self
		        : regexpToAst,
		    function() {
		        // references
		        // https://hackernoon.com/the-madness-of-parsing-real-world-javascript-regexps-d9ee336df983
		        // https://www.ecma-international.org/ecma-262/8.0/index.html#prod-Pattern
		        function RegExpParser() {}

		        RegExpParser.prototype.saveState = function() {
		            return {
		                idx: this.idx,
		                input: this.input,
		                groupIdx: this.groupIdx
		            }
		        };

		        RegExpParser.prototype.restoreState = function(newState) {
		            this.idx = newState.idx;
		            this.input = newState.input;
		            this.groupIdx = newState.groupIdx;
		        };

		        RegExpParser.prototype.pattern = function(input) {
		            // parser state
		            this.idx = 0;
		            this.input = input;
		            this.groupIdx = 0;

		            this.consumeChar("/");
		            var value = this.disjunction();
		            this.consumeChar("/");

		            var flags = {
		                type: "Flags",
		                loc: { begin: this.idx, end: input.length },
		                global: false,
		                ignoreCase: false,
		                multiLine: false,
		                unicode: false,
		                sticky: false
		            };

		            while (this.isRegExpFlag()) {
		                switch (this.popChar()) {
		                    case "g":
		                        addFlag(flags, "global");
		                        break
		                    case "i":
		                        addFlag(flags, "ignoreCase");
		                        break
		                    case "m":
		                        addFlag(flags, "multiLine");
		                        break
		                    case "u":
		                        addFlag(flags, "unicode");
		                        break
		                    case "y":
		                        addFlag(flags, "sticky");
		                        break
		                }
		            }

		            if (this.idx !== this.input.length) {
		                throw Error(
		                    "Redundant input: " + this.input.substring(this.idx)
		                )
		            }
		            return {
		                type: "Pattern",
		                flags: flags,
		                value: value,
		                loc: this.loc(0)
		            }
		        };

		        RegExpParser.prototype.disjunction = function() {
		            var alts = [];
		            var begin = this.idx;

		            alts.push(this.alternative());

		            while (this.peekChar() === "|") {
		                this.consumeChar("|");
		                alts.push(this.alternative());
		            }

		            return { type: "Disjunction", value: alts, loc: this.loc(begin) }
		        };

		        RegExpParser.prototype.alternative = function() {
		            var terms = [];
		            var begin = this.idx;

		            while (this.isTerm()) {
		                terms.push(this.term());
		            }

		            return { type: "Alternative", value: terms, loc: this.loc(begin) }
		        };

		        RegExpParser.prototype.term = function() {
		            if (this.isAssertion()) {
		                return this.assertion()
		            } else {
		                return this.atom()
		            }
		        };

		        RegExpParser.prototype.assertion = function() {
		            var begin = this.idx;
		            switch (this.popChar()) {
		                case "^":
		                    return {
		                        type: "StartAnchor",
		                        loc: this.loc(begin)
		                    }
		                case "$":
		                    return { type: "EndAnchor", loc: this.loc(begin) }
		                // '\b' or '\B'
		                case "\\":
		                    switch (this.popChar()) {
		                        case "b":
		                            return {
		                                type: "WordBoundary",
		                                loc: this.loc(begin)
		                            }
		                        case "B":
		                            return {
		                                type: "NonWordBoundary",
		                                loc: this.loc(begin)
		                            }
		                    }
		                    // istanbul ignore next
		                    throw Error("Invalid Assertion Escape")
		                // '(?=' or '(?!'
		                case "(":
		                    this.consumeChar("?");

		                    var type;
		                    switch (this.popChar()) {
		                        case "=":
		                            type = "Lookahead";
		                            break
		                        case "!":
		                            type = "NegativeLookahead";
		                            break
		                    }
		                    ASSERT_EXISTS(type);

		                    var disjunction = this.disjunction();

		                    this.consumeChar(")");

		                    return {
		                        type: type,
		                        value: disjunction,
		                        loc: this.loc(begin)
		                    }
		            }
		            // istanbul ignore next
		            ASSERT_NEVER_REACH_HERE();
		        };

		        RegExpParser.prototype.quantifier = function(isBacktracking) {
		            var range;
		            var begin = this.idx;
		            switch (this.popChar()) {
		                case "*":
		                    range = {
		                        atLeast: 0,
		                        atMost: Infinity
		                    };
		                    break
		                case "+":
		                    range = {
		                        atLeast: 1,
		                        atMost: Infinity
		                    };
		                    break
		                case "?":
		                    range = {
		                        atLeast: 0,
		                        atMost: 1
		                    };
		                    break
		                case "{":
		                    var atLeast = this.integerIncludingZero();
		                    switch (this.popChar()) {
		                        case "}":
		                            range = {
		                                atLeast: atLeast,
		                                atMost: atLeast
		                            };
		                            break
		                        case ",":
		                            var atMost;
		                            if (this.isDigit()) {
		                                atMost = this.integerIncludingZero();
		                                range = {
		                                    atLeast: atLeast,
		                                    atMost: atMost
		                                };
		                            } else {
		                                range = {
		                                    atLeast: atLeast,
		                                    atMost: Infinity
		                                };
		                            }
		                            this.consumeChar("}");
		                            break
		                    }
		                    // throwing exceptions from "ASSERT_EXISTS" during backtracking
		                    // causes severe performance degradations
		                    if (isBacktracking === true && range === undefined) {
		                        return undefined
		                    }
		                    ASSERT_EXISTS(range);
		                    break
		            }

		            // throwing exceptions from "ASSERT_EXISTS" during backtracking
		            // causes severe performance degradations
		            if (isBacktracking === true && range === undefined) {
		                return undefined
		            }

		            ASSERT_EXISTS(range);

		            if (this.peekChar(0) === "?") {
		                this.consumeChar("?");
		                range.greedy = false;
		            } else {
		                range.greedy = true;
		            }

		            range.type = "Quantifier";
		            range.loc = this.loc(begin);
		            return range
		        };

		        RegExpParser.prototype.atom = function() {
		            var atom;
		            var begin = this.idx;
		            switch (this.peekChar()) {
		                case ".":
		                    atom = this.dotAll();
		                    break
		                case "\\":
		                    atom = this.atomEscape();
		                    break
		                case "[":
		                    atom = this.characterClass();
		                    break
		                case "(":
		                    atom = this.group();
		                    break
		            }

		            if (atom === undefined && this.isPatternCharacter()) {
		                atom = this.patternCharacter();
		            }

		            ASSERT_EXISTS(atom);

		            atom.loc = this.loc(begin);

		            if (this.isQuantifier()) {
		                atom.quantifier = this.quantifier();
		            }

		            return atom
		        };

		        RegExpParser.prototype.dotAll = function() {
		            this.consumeChar(".");
		            return {
		                type: "Set",
		                complement: true,
		                value: [cc("\n"), cc("\r"), cc("\u2028"), cc("\u2029")]
		            }
		        };

		        RegExpParser.prototype.atomEscape = function() {
		            this.consumeChar("\\");

		            switch (this.peekChar()) {
		                case "1":
		                case "2":
		                case "3":
		                case "4":
		                case "5":
		                case "6":
		                case "7":
		                case "8":
		                case "9":
		                    return this.decimalEscapeAtom()
		                case "d":
		                case "D":
		                case "s":
		                case "S":
		                case "w":
		                case "W":
		                    return this.characterClassEscape()
		                case "f":
		                case "n":
		                case "r":
		                case "t":
		                case "v":
		                    return this.controlEscapeAtom()
		                case "c":
		                    return this.controlLetterEscapeAtom()
		                case "0":
		                    return this.nulCharacterAtom()
		                case "x":
		                    return this.hexEscapeSequenceAtom()
		                case "u":
		                    return this.regExpUnicodeEscapeSequenceAtom()
		                default:
		                    return this.identityEscapeAtom()
		            }
		        };

		        RegExpParser.prototype.decimalEscapeAtom = function() {
		            var value = this.positiveInteger();

		            return { type: "GroupBackReference", value: value }
		        };

		        RegExpParser.prototype.characterClassEscape = function() {
		            var set;
		            var complement = false;
		            switch (this.popChar()) {
		                case "d":
		                    set = digitsCharCodes;
		                    break
		                case "D":
		                    set = digitsCharCodes;
		                    complement = true;
		                    break
		                case "s":
		                    set = whitespaceCodes;
		                    break
		                case "S":
		                    set = whitespaceCodes;
		                    complement = true;
		                    break
		                case "w":
		                    set = wordCharCodes;
		                    break
		                case "W":
		                    set = wordCharCodes;
		                    complement = true;
		                    break
		            }

		            ASSERT_EXISTS(set);

		            return { type: "Set", value: set, complement: complement }
		        };

		        RegExpParser.prototype.controlEscapeAtom = function() {
		            var escapeCode;
		            switch (this.popChar()) {
		                case "f":
		                    escapeCode = cc("\f");
		                    break
		                case "n":
		                    escapeCode = cc("\n");
		                    break
		                case "r":
		                    escapeCode = cc("\r");
		                    break
		                case "t":
		                    escapeCode = cc("\t");
		                    break
		                case "v":
		                    escapeCode = cc("\v");
		                    break
		            }
		            ASSERT_EXISTS(escapeCode);

		            return { type: "Character", value: escapeCode }
		        };

		        RegExpParser.prototype.controlLetterEscapeAtom = function() {
		            this.consumeChar("c");
		            var letter = this.popChar();
		            if (/[a-zA-Z]/.test(letter) === false) {
		                throw Error("Invalid ")
		            }

		            var letterCode = letter.toUpperCase().charCodeAt(0) - 64;
		            return { type: "Character", value: letterCode }
		        };

		        RegExpParser.prototype.nulCharacterAtom = function() {
		            // TODO implement '[lookahead ∉ DecimalDigit]'
		            // TODO: for the deprecated octal escape sequence
		            this.consumeChar("0");
		            return { type: "Character", value: cc("\0") }
		        };

		        RegExpParser.prototype.hexEscapeSequenceAtom = function() {
		            this.consumeChar("x");
		            return this.parseHexDigits(2)
		        };

		        RegExpParser.prototype.regExpUnicodeEscapeSequenceAtom = function() {
		            this.consumeChar("u");
		            return this.parseHexDigits(4)
		        };

		        RegExpParser.prototype.identityEscapeAtom = function() {
		            // TODO: implement "SourceCharacter but not UnicodeIDContinue"
		            // // http://unicode.org/reports/tr31/#Specific_Character_Adjustments
		            var escapedChar = this.popChar();
		            return { type: "Character", value: cc(escapedChar) }
		        };

		        RegExpParser.prototype.classPatternCharacterAtom = function() {
		            switch (this.peekChar()) {
		                // istanbul ignore next
		                case "\n":
		                // istanbul ignore next
		                case "\r":
		                // istanbul ignore next
		                case "\u2028":
		                // istanbul ignore next
		                case "\u2029":
		                // istanbul ignore next
		                case "\\":
		                // istanbul ignore next
		                case "]":
		                    throw Error("TBD")
		                default:
		                    var nextChar = this.popChar();
		                    return { type: "Character", value: cc(nextChar) }
		            }
		        };

		        RegExpParser.prototype.characterClass = function() {
		            var set = [];
		            var complement = false;
		            this.consumeChar("[");
		            if (this.peekChar(0) === "^") {
		                this.consumeChar("^");
		                complement = true;
		            }

		            while (this.isClassAtom()) {
		                var from = this.classAtom();
		                var isFromSingleChar = from.type === "Character";
		                if (isFromSingleChar && this.isRangeDash()) {
		                    this.consumeChar("-");
		                    var to = this.classAtom();
		                    var isToSingleChar = to.type === "Character";

		                    // a range can only be used when both sides are single characters
		                    if (isToSingleChar) {
		                        if (to.value < from.value) {
		                            throw Error("Range out of order in character class")
		                        }
		                        set.push({ from: from.value, to: to.value });
		                    } else {
		                        // literal dash
		                        insertToSet(from.value, set);
		                        set.push(cc("-"));
		                        insertToSet(to.value, set);
		                    }
		                } else {
		                    insertToSet(from.value, set);
		                }
		            }

		            this.consumeChar("]");

		            return { type: "Set", complement: complement, value: set }
		        };

		        RegExpParser.prototype.classAtom = function() {
		            switch (this.peekChar()) {
		                // istanbul ignore next
		                case "]":
		                // istanbul ignore next
		                case "\n":
		                // istanbul ignore next
		                case "\r":
		                // istanbul ignore next
		                case "\u2028":
		                // istanbul ignore next
		                case "\u2029":
		                    throw Error("TBD")
		                case "\\":
		                    return this.classEscape()
		                default:
		                    return this.classPatternCharacterAtom()
		            }
		        };

		        RegExpParser.prototype.classEscape = function() {
		            this.consumeChar("\\");
		            switch (this.peekChar()) {
		                // Matches a backspace.
		                // (Not to be confused with \b word boundary outside characterClass)
		                case "b":
		                    this.consumeChar("b");
		                    return { type: "Character", value: cc("\u0008") }
		                case "d":
		                case "D":
		                case "s":
		                case "S":
		                case "w":
		                case "W":
		                    return this.characterClassEscape()
		                case "f":
		                case "n":
		                case "r":
		                case "t":
		                case "v":
		                    return this.controlEscapeAtom()
		                case "c":
		                    return this.controlLetterEscapeAtom()
		                case "0":
		                    return this.nulCharacterAtom()
		                case "x":
		                    return this.hexEscapeSequenceAtom()
		                case "u":
		                    return this.regExpUnicodeEscapeSequenceAtom()
		                default:
		                    return this.identityEscapeAtom()
		            }
		        };

		        RegExpParser.prototype.group = function() {
		            var capturing = true;
		            this.consumeChar("(");
		            switch (this.peekChar(0)) {
		                case "?":
		                    this.consumeChar("?");
		                    this.consumeChar(":");
		                    capturing = false;
		                    break
		                default:
		                    this.groupIdx++;
		                    break
		            }
		            var value = this.disjunction();
		            this.consumeChar(")");

		            var groupAst = {
		                type: "Group",
		                capturing: capturing,
		                value: value
		            };

		            if (capturing) {
		                groupAst.idx = this.groupIdx;
		            }

		            return groupAst
		        };

		        RegExpParser.prototype.positiveInteger = function() {
		            var number = this.popChar();

		            // istanbul ignore next - can't ever get here due to previous lookahead checks
		            // still implementing this error checking in case this ever changes.
		            if (decimalPatternNoZero.test(number) === false) {
		                throw Error("Expecting a positive integer")
		            }

		            while (decimalPattern.test(this.peekChar(0))) {
		                number += this.popChar();
		            }

		            return parseInt(number, 10)
		        };

		        RegExpParser.prototype.integerIncludingZero = function() {
		            var number = this.popChar();
		            if (decimalPattern.test(number) === false) {
		                throw Error("Expecting an integer")
		            }

		            while (decimalPattern.test(this.peekChar(0))) {
		                number += this.popChar();
		            }

		            return parseInt(number, 10)
		        };

		        RegExpParser.prototype.patternCharacter = function() {
		            var nextChar = this.popChar();
		            switch (nextChar) {
		                // istanbul ignore next
		                case "\n":
		                // istanbul ignore next
		                case "\r":
		                // istanbul ignore next
		                case "\u2028":
		                // istanbul ignore next
		                case "\u2029":
		                // istanbul ignore next
		                case "^":
		                // istanbul ignore next
		                case "$":
		                // istanbul ignore next
		                case "\\":
		                // istanbul ignore next
		                case ".":
		                // istanbul ignore next
		                case "*":
		                // istanbul ignore next
		                case "+":
		                // istanbul ignore next
		                case "?":
		                // istanbul ignore next
		                case "(":
		                // istanbul ignore next
		                case ")":
		                // istanbul ignore next
		                case "[":
		                // istanbul ignore next
		                case "|":
		                    // istanbul ignore next
		                    throw Error("TBD")
		                default:
		                    return { type: "Character", value: cc(nextChar) }
		            }
		        };
		        RegExpParser.prototype.isRegExpFlag = function() {
		            switch (this.peekChar(0)) {
		                case "g":
		                case "i":
		                case "m":
		                case "u":
		                case "y":
		                    return true
		                default:
		                    return false
		            }
		        };

		        RegExpParser.prototype.isRangeDash = function() {
		            return this.peekChar() === "-" && this.isClassAtom(1)
		        };

		        RegExpParser.prototype.isDigit = function() {
		            return decimalPattern.test(this.peekChar(0))
		        };

		        RegExpParser.prototype.isClassAtom = function(howMuch) {
		            if (howMuch === undefined) {
		                howMuch = 0;
		            }

		            switch (this.peekChar(howMuch)) {
		                case "]":
		                case "\n":
		                case "\r":
		                case "\u2028":
		                case "\u2029":
		                    return false
		                default:
		                    return true
		            }
		        };

		        RegExpParser.prototype.isTerm = function() {
		            return this.isAtom() || this.isAssertion()
		        };

		        RegExpParser.prototype.isAtom = function() {
		            if (this.isPatternCharacter()) {
		                return true
		            }

		            switch (this.peekChar(0)) {
		                case ".":
		                case "\\": // atomEscape
		                case "[": // characterClass
		                // TODO: isAtom must be called before isAssertion - disambiguate
		                case "(": // group
		                    return true
		                default:
		                    return false
		            }
		        };

		        RegExpParser.prototype.isAssertion = function() {
		            switch (this.peekChar(0)) {
		                case "^":
		                case "$":
		                    return true
		                // '\b' or '\B'
		                case "\\":
		                    switch (this.peekChar(1)) {
		                        case "b":
		                        case "B":
		                            return true
		                        default:
		                            return false
		                    }
		                // '(?=' or '(?!'
		                case "(":
		                    return (
		                        this.peekChar(1) === "?" &&
		                        (this.peekChar(2) === "=" || this.peekChar(2) === "!")
		                    )
		                default:
		                    return false
		            }
		        };

		        RegExpParser.prototype.isQuantifier = function() {
		            var prevState = this.saveState();
		            try {
		                return this.quantifier(true) !== undefined
		            } catch (e) {
		                return false
		            } finally {
		                this.restoreState(prevState);
		            }
		        };

		        RegExpParser.prototype.isPatternCharacter = function() {
		            switch (this.peekChar()) {
		                case "^":
		                case "$":
		                case "\\":
		                case ".":
		                case "*":
		                case "+":
		                case "?":
		                case "(":
		                case ")":
		                case "[":
		                case "|":
		                case "/":
		                case "\n":
		                case "\r":
		                case "\u2028":
		                case "\u2029":
		                    return false
		                default:
		                    return true
		            }
		        };

		        RegExpParser.prototype.parseHexDigits = function(howMany) {
		            var hexString = "";
		            for (var i = 0; i < howMany; i++) {
		                var hexChar = this.popChar();
		                if (hexDigitPattern.test(hexChar) === false) {
		                    throw Error("Expecting a HexDecimal digits")
		                }
		                hexString += hexChar;
		            }
		            var charCode = parseInt(hexString, 16);
		            return { type: "Character", value: charCode }
		        };

		        RegExpParser.prototype.peekChar = function(howMuch) {
		            if (howMuch === undefined) {
		                howMuch = 0;
		            }
		            return this.input[this.idx + howMuch]
		        };

		        RegExpParser.prototype.popChar = function() {
		            var nextChar = this.peekChar(0);
		            this.consumeChar();
		            return nextChar
		        };

		        RegExpParser.prototype.consumeChar = function(char) {
		            if (char !== undefined && this.input[this.idx] !== char) {
		                throw Error(
		                    "Expected: '" +
		                        char +
		                        "' but found: '" +
		                        this.input[this.idx] +
		                        "' at offset: " +
		                        this.idx
		                )
		            }

		            if (this.idx >= this.input.length) {
		                throw Error("Unexpected end of input")
		            }
		            this.idx++;
		        };

		        RegExpParser.prototype.loc = function(begin) {
		            return { begin: begin, end: this.idx }
		        };

		        // consts and utilities
		        var hexDigitPattern = /[0-9a-fA-F]/;
		        var decimalPattern = /[0-9]/;
		        var decimalPatternNoZero = /[1-9]/;

		        function cc(char) {
		            return char.charCodeAt(0)
		        }

		        function insertToSet(item, set) {
		            if (item.length !== undefined) {
		                item.forEach(function(subItem) {
		                    set.push(subItem);
		                });
		            } else {
		                set.push(item);
		            }
		        }

		        function addFlag(flagObj, flagKey) {
		            if (flagObj[flagKey] === true) {
		                throw "duplicate flag " + flagKey
		            }

		            flagObj[flagKey] = true;
		        }

		        function ASSERT_EXISTS(obj) {
		            // istanbul ignore next
		            if (obj === undefined) {
		                throw Error("Internal Error - Should never get here!")
		            }
		        }

		        // istanbul ignore next
		        function ASSERT_NEVER_REACH_HERE() {
		            throw Error("Internal Error - Should never get here!")
		        }

		        var i;
		        var digitsCharCodes = [];
		        for (i = cc("0"); i <= cc("9"); i++) {
		            digitsCharCodes.push(i);
		        }

		        var wordCharCodes = [cc("_")].concat(digitsCharCodes);
		        for (i = cc("a"); i <= cc("z"); i++) {
		            wordCharCodes.push(i);
		        }

		        for (i = cc("A"); i <= cc("Z"); i++) {
		            wordCharCodes.push(i);
		        }

		        // https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/RegExp#character-classes
		        var whitespaceCodes = [
		            cc(" "),
		            cc("\f"),
		            cc("\n"),
		            cc("\r"),
		            cc("\t"),
		            cc("\v"),
		            cc("\t"),
		            cc("\u00a0"),
		            cc("\u1680"),
		            cc("\u2000"),
		            cc("\u2001"),
		            cc("\u2002"),
		            cc("\u2003"),
		            cc("\u2004"),
		            cc("\u2005"),
		            cc("\u2006"),
		            cc("\u2007"),
		            cc("\u2008"),
		            cc("\u2009"),
		            cc("\u200a"),
		            cc("\u2028"),
		            cc("\u2029"),
		            cc("\u202f"),
		            cc("\u205f"),
		            cc("\u3000"),
		            cc("\ufeff")
		        ];

		        function BaseRegExpVisitor() {}

		        BaseRegExpVisitor.prototype.visitChildren = function(node) {
		            for (var key in node) {
		                var child = node[key];
		                /* istanbul ignore else */
		                if (node.hasOwnProperty(key)) {
		                    if (child.type !== undefined) {
		                        this.visit(child);
		                    } else if (Array.isArray(child)) {
		                        child.forEach(function(subChild) {
		                            this.visit(subChild);
		                        }, this);
		                    }
		                }
		            }
		        };

		        BaseRegExpVisitor.prototype.visit = function(node) {
		            switch (node.type) {
		                case "Pattern":
		                    this.visitPattern(node);
		                    break
		                case "Flags":
		                    this.visitFlags(node);
		                    break
		                case "Disjunction":
		                    this.visitDisjunction(node);
		                    break
		                case "Alternative":
		                    this.visitAlternative(node);
		                    break
		                case "StartAnchor":
		                    this.visitStartAnchor(node);
		                    break
		                case "EndAnchor":
		                    this.visitEndAnchor(node);
		                    break
		                case "WordBoundary":
		                    this.visitWordBoundary(node);
		                    break
		                case "NonWordBoundary":
		                    this.visitNonWordBoundary(node);
		                    break
		                case "Lookahead":
		                    this.visitLookahead(node);
		                    break
		                case "NegativeLookahead":
		                    this.visitNegativeLookahead(node);
		                    break
		                case "Character":
		                    this.visitCharacter(node);
		                    break
		                case "Set":
		                    this.visitSet(node);
		                    break
		                case "Group":
		                    this.visitGroup(node);
		                    break
		                case "GroupBackReference":
		                    this.visitGroupBackReference(node);
		                    break
		                case "Quantifier":
		                    this.visitQuantifier(node);
		                    break
		            }

		            this.visitChildren(node);
		        };

		        BaseRegExpVisitor.prototype.visitPattern = function(node) {};

		        BaseRegExpVisitor.prototype.visitFlags = function(node) {};

		        BaseRegExpVisitor.prototype.visitDisjunction = function(node) {};

		        BaseRegExpVisitor.prototype.visitAlternative = function(node) {};

		        // Assertion
		        BaseRegExpVisitor.prototype.visitStartAnchor = function(node) {};

		        BaseRegExpVisitor.prototype.visitEndAnchor = function(node) {};

		        BaseRegExpVisitor.prototype.visitWordBoundary = function(node) {};

		        BaseRegExpVisitor.prototype.visitNonWordBoundary = function(node) {};

		        BaseRegExpVisitor.prototype.visitLookahead = function(node) {};

		        BaseRegExpVisitor.prototype.visitNegativeLookahead = function(node) {};

		        // atoms
		        BaseRegExpVisitor.prototype.visitCharacter = function(node) {};

		        BaseRegExpVisitor.prototype.visitSet = function(node) {};

		        BaseRegExpVisitor.prototype.visitGroup = function(node) {};

		        BaseRegExpVisitor.prototype.visitGroupBackReference = function(node) {};

		        BaseRegExpVisitor.prototype.visitQuantifier = function(node) {};

		        return {
		            RegExpParser: RegExpParser,
		            BaseRegExpVisitor: BaseRegExpVisitor,
		            VERSION: "0.5.0"
		        }
		    }
		); 
	} (regexpToAst$1));
	return regexpToAst$1.exports;
}

/**
 * Gets the first element of `array`.
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @alias first
 * @category Array
 * @param {Array} array The array to query.
 * @returns {*} Returns the first element of `array`.
 * @example
 *
 * _.head([1, 2, 3]);
 * // => 1
 *
 * _.head([]);
 * // => undefined
 */

var head_1;
var hasRequiredHead;

function requireHead () {
	if (hasRequiredHead) return head_1;
	hasRequiredHead = 1;
	function head(array) {
	  return (array && array.length) ? array[0] : undefined;
	}

	head_1 = head;
	return head_1;
}

var first;
var hasRequiredFirst;

function requireFirst () {
	if (hasRequiredFirst) return first;
	hasRequiredFirst = 1;
	first = requireHead();
	return first;
}

/**
 * Creates an array with all falsey values removed. The values `false`, `null`,
 * `0`, `""`, `undefined`, and `NaN` are falsey.
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Array
 * @param {Array} array The array to compact.
 * @returns {Array} Returns the new array of filtered values.
 * @example
 *
 * _.compact([0, 1, false, 2, '', 3]);
 * // => [1, 2, 3]
 */

var compact_1;
var hasRequiredCompact;

function requireCompact () {
	if (hasRequiredCompact) return compact_1;
	hasRequiredCompact = 1;
	function compact(array) {
	  var index = -1,
	      length = array == null ? 0 : array.length,
	      resIndex = 0,
	      result = [];

	  while (++index < length) {
	    var value = array[index];
	    if (value) {
	      result[resIndex++] = value;
	    }
	  }
	  return result;
	}

	compact_1 = compact;
	return compact_1;
}

var _baseFilter;
var hasRequired_baseFilter;

function require_baseFilter () {
	if (hasRequired_baseFilter) return _baseFilter;
	hasRequired_baseFilter = 1;
	var baseEach = require_baseEach();

	/**
	 * The base implementation of `_.filter` without support for iteratee shorthands.
	 *
	 * @private
	 * @param {Array|Object} collection The collection to iterate over.
	 * @param {Function} predicate The function invoked per iteration.
	 * @returns {Array} Returns the new filtered array.
	 */
	function baseFilter(collection, predicate) {
	  var result = [];
	  baseEach(collection, function(value, index, collection) {
	    if (predicate(value, index, collection)) {
	      result.push(value);
	    }
	  });
	  return result;
	}

	_baseFilter = baseFilter;
	return _baseFilter;
}

/** Error message constants. */

var negate_1;
var hasRequiredNegate;

function requireNegate () {
	if (hasRequiredNegate) return negate_1;
	hasRequiredNegate = 1;
	var FUNC_ERROR_TEXT = 'Expected a function';

	/**
	 * Creates a function that negates the result of the predicate `func`. The
	 * `func` predicate is invoked with the `this` binding and arguments of the
	 * created function.
	 *
	 * @static
	 * @memberOf _
	 * @since 3.0.0
	 * @category Function
	 * @param {Function} predicate The predicate to negate.
	 * @returns {Function} Returns the new negated function.
	 * @example
	 *
	 * function isEven(n) {
	 *   return n % 2 == 0;
	 * }
	 *
	 * _.filter([1, 2, 3, 4, 5, 6], _.negate(isEven));
	 * // => [1, 3, 5]
	 */
	function negate(predicate) {
	  if (typeof predicate != 'function') {
	    throw new TypeError(FUNC_ERROR_TEXT);
	  }
	  return function() {
	    var args = arguments;
	    switch (args.length) {
	      case 0: return !predicate.call(this);
	      case 1: return !predicate.call(this, args[0]);
	      case 2: return !predicate.call(this, args[0], args[1]);
	      case 3: return !predicate.call(this, args[0], args[1], args[2]);
	    }
	    return !predicate.apply(this, args);
	  };
	}

	negate_1 = negate;
	return negate_1;
}

var reject_1;
var hasRequiredReject;

function requireReject () {
	if (hasRequiredReject) return reject_1;
	hasRequiredReject = 1;
	var arrayFilter = require_arrayFilter(),
	    baseFilter = require_baseFilter(),
	    baseIteratee = require_baseIteratee(),
	    isArray = requireIsArray(),
	    negate = requireNegate();

	/**
	 * The opposite of `_.filter`; this method returns the elements of `collection`
	 * that `predicate` does **not** return truthy for.
	 *
	 * @static
	 * @memberOf _
	 * @since 0.1.0
	 * @category Collection
	 * @param {Array|Object} collection The collection to iterate over.
	 * @param {Function} [predicate=_.identity] The function invoked per iteration.
	 * @returns {Array} Returns the new filtered array.
	 * @see _.filter
	 * @example
	 *
	 * var users = [
	 *   { 'user': 'barney', 'age': 36, 'active': false },
	 *   { 'user': 'fred',   'age': 40, 'active': true }
	 * ];
	 *
	 * _.reject(users, function(o) { return !o.active; });
	 * // => objects for ['fred']
	 *
	 * // The `_.matches` iteratee shorthand.
	 * _.reject(users, { 'age': 40, 'active': true });
	 * // => objects for ['barney']
	 *
	 * // The `_.matchesProperty` iteratee shorthand.
	 * _.reject(users, ['active', false]);
	 * // => objects for ['fred']
	 *
	 * // The `_.property` iteratee shorthand.
	 * _.reject(users, 'active');
	 * // => objects for ['barney']
	 */
	function reject(collection, predicate) {
	  var func = isArray(collection) ? arrayFilter : baseFilter;
	  return func(collection, negate(baseIteratee(predicate, 3)));
	}

	reject_1 = reject;
	return reject_1;
}

var _baseDifference;
var hasRequired_baseDifference;

function require_baseDifference () {
	if (hasRequired_baseDifference) return _baseDifference;
	hasRequired_baseDifference = 1;
	var SetCache = require_SetCache(),
	    arrayIncludes = require_arrayIncludes(),
	    arrayIncludesWith = require_arrayIncludesWith(),
	    arrayMap = require_arrayMap(),
	    baseUnary = require_baseUnary(),
	    cacheHas = require_cacheHas();

	/** Used as the size to enable large array optimizations. */
	var LARGE_ARRAY_SIZE = 200;

	/**
	 * The base implementation of methods like `_.difference` without support
	 * for excluding multiple arrays or iteratee shorthands.
	 *
	 * @private
	 * @param {Array} array The array to inspect.
	 * @param {Array} values The values to exclude.
	 * @param {Function} [iteratee] The iteratee invoked per element.
	 * @param {Function} [comparator] The comparator invoked per element.
	 * @returns {Array} Returns the new array of filtered values.
	 */
	function baseDifference(array, values, iteratee, comparator) {
	  var index = -1,
	      includes = arrayIncludes,
	      isCommon = true,
	      length = array.length,
	      result = [],
	      valuesLength = values.length;

	  if (!length) {
	    return result;
	  }
	  if (iteratee) {
	    values = arrayMap(values, baseUnary(iteratee));
	  }
	  if (comparator) {
	    includes = arrayIncludesWith;
	    isCommon = false;
	  }
	  else if (values.length >= LARGE_ARRAY_SIZE) {
	    includes = cacheHas;
	    isCommon = false;
	    values = new SetCache(values);
	  }
	  outer:
	  while (++index < length) {
	    var value = array[index],
	        computed = iteratee == null ? value : iteratee(value);

	    value = (comparator || value !== 0) ? value : 0;
	    if (isCommon && computed === computed) {
	      var valuesIndex = valuesLength;
	      while (valuesIndex--) {
	        if (values[valuesIndex] === computed) {
	          continue outer;
	        }
	      }
	      result.push(value);
	    }
	    else if (!includes(values, computed, comparator)) {
	      result.push(value);
	    }
	  }
	  return result;
	}

	_baseDifference = baseDifference;
	return _baseDifference;
}

var isArrayLikeObject_1;
var hasRequiredIsArrayLikeObject;

function requireIsArrayLikeObject () {
	if (hasRequiredIsArrayLikeObject) return isArrayLikeObject_1;
	hasRequiredIsArrayLikeObject = 1;
	var isArrayLike = requireIsArrayLike(),
	    isObjectLike = requireIsObjectLike();

	/**
	 * This method is like `_.isArrayLike` except that it also checks if `value`
	 * is an object.
	 *
	 * @static
	 * @memberOf _
	 * @since 4.0.0
	 * @category Lang
	 * @param {*} value The value to check.
	 * @returns {boolean} Returns `true` if `value` is an array-like object,
	 *  else `false`.
	 * @example
	 *
	 * _.isArrayLikeObject([1, 2, 3]);
	 * // => true
	 *
	 * _.isArrayLikeObject(document.body.children);
	 * // => true
	 *
	 * _.isArrayLikeObject('abc');
	 * // => false
	 *
	 * _.isArrayLikeObject(_.noop);
	 * // => false
	 */
	function isArrayLikeObject(value) {
	  return isObjectLike(value) && isArrayLike(value);
	}

	isArrayLikeObject_1 = isArrayLikeObject;
	return isArrayLikeObject_1;
}

var difference_1;
var hasRequiredDifference;

function requireDifference () {
	if (hasRequiredDifference) return difference_1;
	hasRequiredDifference = 1;
	var baseDifference = require_baseDifference(),
	    baseFlatten = require_baseFlatten(),
	    baseRest = require_baseRest(),
	    isArrayLikeObject = requireIsArrayLikeObject();

	/**
	 * Creates an array of `array` values not included in the other given arrays
	 * using [`SameValueZero`](http://ecma-international.org/ecma-262/7.0/#sec-samevaluezero)
	 * for equality comparisons. The order and references of result values are
	 * determined by the first array.
	 *
	 * **Note:** Unlike `_.pullAll`, this method returns a new array.
	 *
	 * @static
	 * @memberOf _
	 * @since 0.1.0
	 * @category Array
	 * @param {Array} array The array to inspect.
	 * @param {...Array} [values] The values to exclude.
	 * @returns {Array} Returns the new array of filtered values.
	 * @see _.without, _.xor
	 * @example
	 *
	 * _.difference([2, 1], [2, 3]);
	 * // => [1]
	 */
	var difference = baseRest(function(array, values) {
	  return isArrayLikeObject(array)
	    ? baseDifference(array, baseFlatten(values, 1, isArrayLikeObject, true))
	    : [];
	});

	difference_1 = difference;
	return difference_1;
}

var indexOf_1;
var hasRequiredIndexOf;

function requireIndexOf () {
	if (hasRequiredIndexOf) return indexOf_1;
	hasRequiredIndexOf = 1;
	var baseIndexOf = require_baseIndexOf(),
	    toInteger = requireToInteger();

	/* Built-in method references for those with the same name as other `lodash` methods. */
	var nativeMax = Math.max;

	/**
	 * Gets the index at which the first occurrence of `value` is found in `array`
	 * using [`SameValueZero`](http://ecma-international.org/ecma-262/7.0/#sec-samevaluezero)
	 * for equality comparisons. If `fromIndex` is negative, it's used as the
	 * offset from the end of `array`.
	 *
	 * @static
	 * @memberOf _
	 * @since 0.1.0
	 * @category Array
	 * @param {Array} array The array to inspect.
	 * @param {*} value The value to search for.
	 * @param {number} [fromIndex=0] The index to search from.
	 * @returns {number} Returns the index of the matched value, else `-1`.
	 * @example
	 *
	 * _.indexOf([1, 2, 1, 2], 2);
	 * // => 1
	 *
	 * // Search from the `fromIndex`.
	 * _.indexOf([1, 2, 1, 2], 2, 2);
	 * // => 3
	 */
	function indexOf(array, value, fromIndex) {
	  var length = array == null ? 0 : array.length;
	  if (!length) {
	    return -1;
	  }
	  var index = fromIndex == null ? 0 : toInteger(fromIndex);
	  if (index < 0) {
	    index = nativeMax(length + index, 0);
	  }
	  return baseIndexOf(array, value, index);
	}

	indexOf_1 = indexOf;
	return indexOf_1;
}

var _createFind;
var hasRequired_createFind;

function require_createFind () {
	if (hasRequired_createFind) return _createFind;
	hasRequired_createFind = 1;
	var baseIteratee = require_baseIteratee(),
	    isArrayLike = requireIsArrayLike(),
	    keys = requireKeys$1();

	/**
	 * Creates a `_.find` or `_.findLast` function.
	 *
	 * @private
	 * @param {Function} findIndexFunc The function to find the collection index.
	 * @returns {Function} Returns the new find function.
	 */
	function createFind(findIndexFunc) {
	  return function(collection, predicate, fromIndex) {
	    var iterable = Object(collection);
	    if (!isArrayLike(collection)) {
	      var iteratee = baseIteratee(predicate, 3);
	      collection = keys(collection);
	      predicate = function(key) { return iteratee(iterable[key], key, iterable); };
	    }
	    var index = findIndexFunc(collection, predicate, fromIndex);
	    return index > -1 ? iterable[iteratee ? collection[index] : index] : undefined;
	  };
	}

	_createFind = createFind;
	return _createFind;
}

var findIndex_1;
var hasRequiredFindIndex;

function requireFindIndex () {
	if (hasRequiredFindIndex) return findIndex_1;
	hasRequiredFindIndex = 1;
	var baseFindIndex = require_baseFindIndex(),
	    baseIteratee = require_baseIteratee(),
	    toInteger = requireToInteger();

	/* Built-in method references for those with the same name as other `lodash` methods. */
	var nativeMax = Math.max;

	/**
	 * This method is like `_.find` except that it returns the index of the first
	 * element `predicate` returns truthy for instead of the element itself.
	 *
	 * @static
	 * @memberOf _
	 * @since 1.1.0
	 * @category Array
	 * @param {Array} array The array to inspect.
	 * @param {Function} [predicate=_.identity] The function invoked per iteration.
	 * @param {number} [fromIndex=0] The index to search from.
	 * @returns {number} Returns the index of the found element, else `-1`.
	 * @example
	 *
	 * var users = [
	 *   { 'user': 'barney',  'active': false },
	 *   { 'user': 'fred',    'active': false },
	 *   { 'user': 'pebbles', 'active': true }
	 * ];
	 *
	 * _.findIndex(users, function(o) { return o.user == 'barney'; });
	 * // => 0
	 *
	 * // The `_.matches` iteratee shorthand.
	 * _.findIndex(users, { 'user': 'fred', 'active': false });
	 * // => 1
	 *
	 * // The `_.matchesProperty` iteratee shorthand.
	 * _.findIndex(users, ['active', false]);
	 * // => 0
	 *
	 * // The `_.property` iteratee shorthand.
	 * _.findIndex(users, 'active');
	 * // => 2
	 */
	function findIndex(array, predicate, fromIndex) {
	  var length = array == null ? 0 : array.length;
	  if (!length) {
	    return -1;
	  }
	  var index = fromIndex == null ? 0 : toInteger(fromIndex);
	  if (index < 0) {
	    index = nativeMax(length + index, 0);
	  }
	  return baseFindIndex(array, baseIteratee(predicate, 3), index);
	}

	findIndex_1 = findIndex;
	return findIndex_1;
}

var find_1;
var hasRequiredFind;

function requireFind () {
	if (hasRequiredFind) return find_1;
	hasRequiredFind = 1;
	var createFind = require_createFind(),
	    findIndex = requireFindIndex();

	/**
	 * Iterates over elements of `collection`, returning the first element
	 * `predicate` returns truthy for. The predicate is invoked with three
	 * arguments: (value, index|key, collection).
	 *
	 * @static
	 * @memberOf _
	 * @since 0.1.0
	 * @category Collection
	 * @param {Array|Object} collection The collection to inspect.
	 * @param {Function} [predicate=_.identity] The function invoked per iteration.
	 * @param {number} [fromIndex=0] The index to search from.
	 * @returns {*} Returns the matched element, else `undefined`.
	 * @example
	 *
	 * var users = [
	 *   { 'user': 'barney',  'age': 36, 'active': true },
	 *   { 'user': 'fred',    'age': 40, 'active': false },
	 *   { 'user': 'pebbles', 'age': 1,  'active': true }
	 * ];
	 *
	 * _.find(users, function(o) { return o.age < 40; });
	 * // => object for 'barney'
	 *
	 * // The `_.matches` iteratee shorthand.
	 * _.find(users, { 'age': 1, 'active': true });
	 * // => object for 'pebbles'
	 *
	 * // The `_.matchesProperty` iteratee shorthand.
	 * _.find(users, ['active', false]);
	 * // => object for 'fred'
	 *
	 * // The `_.property` iteratee shorthand.
	 * _.find(users, 'active');
	 * // => object for 'barney'
	 */
	var find = createFind(findIndex);

	find_1 = find;
	return find_1;
}

var filter_1;
var hasRequiredFilter;

function requireFilter () {
	if (hasRequiredFilter) return filter_1;
	hasRequiredFilter = 1;
	var arrayFilter = require_arrayFilter(),
	    baseFilter = require_baseFilter(),
	    baseIteratee = require_baseIteratee(),
	    isArray = requireIsArray();

	/**
	 * Iterates over elements of `collection`, returning an array of all elements
	 * `predicate` returns truthy for. The predicate is invoked with three
	 * arguments: (value, index|key, collection).
	 *
	 * **Note:** Unlike `_.remove`, this method returns a new array.
	 *
	 * @static
	 * @memberOf _
	 * @since 0.1.0
	 * @category Collection
	 * @param {Array|Object} collection The collection to iterate over.
	 * @param {Function} [predicate=_.identity] The function invoked per iteration.
	 * @returns {Array} Returns the new filtered array.
	 * @see _.reject
	 * @example
	 *
	 * var users = [
	 *   { 'user': 'barney', 'age': 36, 'active': true },
	 *   { 'user': 'fred',   'age': 40, 'active': false }
	 * ];
	 *
	 * _.filter(users, function(o) { return !o.active; });
	 * // => objects for ['fred']
	 *
	 * // The `_.matches` iteratee shorthand.
	 * _.filter(users, { 'age': 36, 'active': true });
	 * // => objects for ['barney']
	 *
	 * // The `_.matchesProperty` iteratee shorthand.
	 * _.filter(users, ['active', false]);
	 * // => objects for ['fred']
	 *
	 * // The `_.property` iteratee shorthand.
	 * _.filter(users, 'active');
	 * // => objects for ['barney']
	 *
	 * // Combining several predicates using `_.overEvery` or `_.overSome`.
	 * _.filter(users, _.overSome([{ 'age': 36 }, ['age', 40]]));
	 * // => objects for ['fred', 'barney']
	 */
	function filter(collection, predicate) {
	  var func = isArray(collection) ? arrayFilter : baseFilter;
	  return func(collection, baseIteratee(predicate, 3));
	}

	filter_1 = filter;
	return filter_1;
}

var defaults_1;
var hasRequiredDefaults;

function requireDefaults () {
	if (hasRequiredDefaults) return defaults_1;
	hasRequiredDefaults = 1;
	var baseRest = require_baseRest(),
	    eq = requireEq(),
	    isIterateeCall = require_isIterateeCall(),
	    keysIn = requireKeysIn();

	/** Used for built-in method references. */
	var objectProto = Object.prototype;

	/** Used to check objects for own properties. */
	var hasOwnProperty = objectProto.hasOwnProperty;

	/**
	 * Assigns own and inherited enumerable string keyed properties of source
	 * objects to the destination object for all destination properties that
	 * resolve to `undefined`. Source objects are applied from left to right.
	 * Once a property is set, additional values of the same property are ignored.
	 *
	 * **Note:** This method mutates `object`.
	 *
	 * @static
	 * @since 0.1.0
	 * @memberOf _
	 * @category Object
	 * @param {Object} object The destination object.
	 * @param {...Object} [sources] The source objects.
	 * @returns {Object} Returns `object`.
	 * @see _.defaultsDeep
	 * @example
	 *
	 * _.defaults({ 'a': 1 }, { 'b': 2 }, { 'a': 3 });
	 * // => { 'a': 1, 'b': 2 }
	 */
	var defaults = baseRest(function(object, sources) {
	  object = Object(object);

	  var index = -1;
	  var length = sources.length;
	  var guard = length > 2 ? sources[2] : undefined;

	  if (guard && isIterateeCall(sources[0], sources[1], guard)) {
	    length = 1;
	  }

	  while (++index < length) {
	    var source = sources[index];
	    var props = keysIn(source);
	    var propsIndex = -1;
	    var propsLength = props.length;

	    while (++propsIndex < propsLength) {
	      var key = props[propsIndex];
	      var value = object[key];

	      if (value === undefined ||
	          (eq(value, objectProto[key]) && !hasOwnProperty.call(object, key))) {
	        object[key] = source[key];
	      }
	    }
	  }

	  return object;
	});

	defaults_1 = defaults;
	return defaults_1;
}

/**
 * A specialized version of `_.reduce` for arrays without support for
 * iteratee shorthands.
 *
 * @private
 * @param {Array} [array] The array to iterate over.
 * @param {Function} iteratee The function invoked per iteration.
 * @param {*} [accumulator] The initial value.
 * @param {boolean} [initAccum] Specify using the first element of `array` as
 *  the initial value.
 * @returns {*} Returns the accumulated value.
 */

var _arrayReduce;
var hasRequired_arrayReduce;

function require_arrayReduce () {
	if (hasRequired_arrayReduce) return _arrayReduce;
	hasRequired_arrayReduce = 1;
	function arrayReduce(array, iteratee, accumulator, initAccum) {
	  var index = -1,
	      length = array == null ? 0 : array.length;

	  if (initAccum && length) {
	    accumulator = array[++index];
	  }
	  while (++index < length) {
	    accumulator = iteratee(accumulator, array[index], index, array);
	  }
	  return accumulator;
	}

	_arrayReduce = arrayReduce;
	return _arrayReduce;
}

/**
 * The base implementation of `_.reduce` and `_.reduceRight`, without support
 * for iteratee shorthands, which iterates over `collection` using `eachFunc`.
 *
 * @private
 * @param {Array|Object} collection The collection to iterate over.
 * @param {Function} iteratee The function invoked per iteration.
 * @param {*} accumulator The initial value.
 * @param {boolean} initAccum Specify using the first or last element of
 *  `collection` as the initial value.
 * @param {Function} eachFunc The function to iterate over `collection`.
 * @returns {*} Returns the accumulated value.
 */

var _baseReduce;
var hasRequired_baseReduce;

function require_baseReduce () {
	if (hasRequired_baseReduce) return _baseReduce;
	hasRequired_baseReduce = 1;
	function baseReduce(collection, iteratee, accumulator, initAccum, eachFunc) {
	  eachFunc(collection, function(value, index, collection) {
	    accumulator = initAccum
	      ? (initAccum = false, value)
	      : iteratee(accumulator, value, index, collection);
	  });
	  return accumulator;
	}

	_baseReduce = baseReduce;
	return _baseReduce;
}

var reduce_1;
var hasRequiredReduce;

function requireReduce () {
	if (hasRequiredReduce) return reduce_1;
	hasRequiredReduce = 1;
	var arrayReduce = require_arrayReduce(),
	    baseEach = require_baseEach(),
	    baseIteratee = require_baseIteratee(),
	    baseReduce = require_baseReduce(),
	    isArray = requireIsArray();

	/**
	 * Reduces `collection` to a value which is the accumulated result of running
	 * each element in `collection` thru `iteratee`, where each successive
	 * invocation is supplied the return value of the previous. If `accumulator`
	 * is not given, the first element of `collection` is used as the initial
	 * value. The iteratee is invoked with four arguments:
	 * (accumulator, value, index|key, collection).
	 *
	 * Many lodash methods are guarded to work as iteratees for methods like
	 * `_.reduce`, `_.reduceRight`, and `_.transform`.
	 *
	 * The guarded methods are:
	 * `assign`, `defaults`, `defaultsDeep`, `includes`, `merge`, `orderBy`,
	 * and `sortBy`
	 *
	 * @static
	 * @memberOf _
	 * @since 0.1.0
	 * @category Collection
	 * @param {Array|Object} collection The collection to iterate over.
	 * @param {Function} [iteratee=_.identity] The function invoked per iteration.
	 * @param {*} [accumulator] The initial value.
	 * @returns {*} Returns the accumulated value.
	 * @see _.reduceRight
	 * @example
	 *
	 * _.reduce([1, 2], function(sum, n) {
	 *   return sum + n;
	 * }, 0);
	 * // => 3
	 *
	 * _.reduce({ 'a': 1, 'b': 2, 'c': 1 }, function(result, value, key) {
	 *   (result[value] || (result[value] = [])).push(key);
	 *   return result;
	 * }, {});
	 * // => { '1': ['a', 'c'], '2': ['b'] } (iteration order is not guaranteed)
	 */
	function reduce(collection, iteratee, accumulator) {
	  var func = isArray(collection) ? arrayReduce : baseReduce,
	      initAccum = arguments.length < 3;

	  return func(collection, baseIteratee(iteratee, 4), accumulator, initAccum, baseEach);
	}

	reduce_1 = reduce;
	return reduce_1;
}

var reg_exp = {};

var reg_exp_parser = {};

var hasRequiredReg_exp_parser;

function requireReg_exp_parser () {
	if (hasRequiredReg_exp_parser) return reg_exp_parser;
	hasRequiredReg_exp_parser = 1;
	Object.defineProperty(reg_exp_parser, "__esModule", { value: true });
	reg_exp_parser.clearRegExpParserCache = reg_exp_parser.getRegExpAst = void 0;
	var regexp_to_ast_1 = requireRegexpToAst();
	var regExpAstCache = {};
	var regExpParser = new regexp_to_ast_1.RegExpParser();
	function getRegExpAst(regExp) {
	    var regExpStr = regExp.toString();
	    if (regExpAstCache.hasOwnProperty(regExpStr)) {
	        return regExpAstCache[regExpStr];
	    }
	    else {
	        var regExpAst = regExpParser.pattern(regExpStr);
	        regExpAstCache[regExpStr] = regExpAst;
	        return regExpAst;
	    }
	}
	reg_exp_parser.getRegExpAst = getRegExpAst;
	function clearRegExpParserCache() {
	    regExpAstCache = {};
	}
	reg_exp_parser.clearRegExpParserCache = clearRegExpParserCache;
	
	return reg_exp_parser;
}

var hasRequiredReg_exp;

function requireReg_exp () {
	if (hasRequiredReg_exp) return reg_exp;
	hasRequiredReg_exp = 1;
	(function (exports) {
		var __extends = (reg_exp && reg_exp.__extends) || (function () {
		    var extendStatics = function (d, b) {
		        extendStatics = Object.setPrototypeOf ||
		            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
		            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
		        return extendStatics(d, b);
		    };
		    return function (d, b) {
		        if (typeof b !== "function" && b !== null)
		            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
		        extendStatics(d, b);
		        function __() { this.constructor = d; }
		        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
		    };
		})();
		var __importDefault = (reg_exp && reg_exp.__importDefault) || function (mod) {
		    return (mod && mod.__esModule) ? mod : { "default": mod };
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.canMatchCharCode = exports.firstCharOptimizedIndices = exports.getOptimizedStartCodesIndices = exports.failedOptimizationPrefixMsg = void 0;
		var regexp_to_ast_1 = requireRegexpToAst();
		var isArray_1 = __importDefault(requireIsArray());
		var every_1 = __importDefault(requireEvery());
		var forEach_1 = __importDefault(requireForEach());
		var find_1 = __importDefault(requireFind());
		var values_1 = __importDefault(requireValues());
		var includes_1 = __importDefault(requireIncludes());
		var utils_1 = requireApi$3();
		var reg_exp_parser_1 = requireReg_exp_parser();
		var lexer_1 = requireLexer();
		var complementErrorMessage = "Complement Sets are not supported for first char optimization";
		exports.failedOptimizationPrefixMsg = 'Unable to use "first char" lexer optimizations:\n';
		function getOptimizedStartCodesIndices(regExp, ensureOptimizations) {
		    if (ensureOptimizations === void 0) { ensureOptimizations = false; }
		    try {
		        var ast = (0, reg_exp_parser_1.getRegExpAst)(regExp);
		        var firstChars = firstCharOptimizedIndices(ast.value, {}, ast.flags.ignoreCase);
		        return firstChars;
		    }
		    catch (e) {
		        /* istanbul ignore next */
		        // Testing this relies on the regexp-to-ast library having a bug... */
		        // TODO: only the else branch needs to be ignored, try to fix with newer prettier / tsc
		        if (e.message === complementErrorMessage) {
		            if (ensureOptimizations) {
		                (0, utils_1.PRINT_WARNING)("".concat(exports.failedOptimizationPrefixMsg) +
		                    "\tUnable to optimize: < ".concat(regExp.toString(), " >\n") +
		                    "\tComplement Sets cannot be automatically optimized.\n" +
		                    "\tThis will disable the lexer's first char optimizations.\n" +
		                    "\tSee: https://chevrotain.io/docs/guide/resolving_lexer_errors.html#COMPLEMENT for details.");
		            }
		        }
		        else {
		            var msgSuffix = "";
		            if (ensureOptimizations) {
		                msgSuffix =
		                    "\n\tThis will disable the lexer's first char optimizations.\n" +
		                        "\tSee: https://chevrotain.io/docs/guide/resolving_lexer_errors.html#REGEXP_PARSING for details.";
		            }
		            (0, utils_1.PRINT_ERROR)("".concat(exports.failedOptimizationPrefixMsg, "\n") +
		                "\tFailed parsing: < ".concat(regExp.toString(), " >\n") +
		                "\tUsing the regexp-to-ast library version: ".concat(regexp_to_ast_1.VERSION, "\n") +
		                "\tPlease open an issue at: https://github.com/bd82/regexp-to-ast/issues" +
		                msgSuffix);
		        }
		    }
		    return [];
		}
		exports.getOptimizedStartCodesIndices = getOptimizedStartCodesIndices;
		function firstCharOptimizedIndices(ast, result, ignoreCase) {
		    switch (ast.type) {
		        case "Disjunction":
		            for (var i = 0; i < ast.value.length; i++) {
		                firstCharOptimizedIndices(ast.value[i], result, ignoreCase);
		            }
		            break;
		        case "Alternative":
		            var terms = ast.value;
		            for (var i = 0; i < terms.length; i++) {
		                var term = terms[i];
		                // skip terms that cannot effect the first char results
		                switch (term.type) {
		                    case "EndAnchor":
		                    // A group back reference cannot affect potential starting char.
		                    // because if a back reference is the first production than automatically
		                    // the group being referenced has had to come BEFORE so its codes have already been added
		                    case "GroupBackReference":
		                    // assertions do not affect potential starting codes
		                    case "Lookahead":
		                    case "NegativeLookahead":
		                    case "StartAnchor":
		                    case "WordBoundary":
		                    case "NonWordBoundary":
		                        continue;
		                }
		                var atom = term;
		                switch (atom.type) {
		                    case "Character":
		                        addOptimizedIdxToResult(atom.value, result, ignoreCase);
		                        break;
		                    case "Set":
		                        if (atom.complement === true) {
		                            throw Error(complementErrorMessage);
		                        }
		                        (0, forEach_1.default)(atom.value, function (code) {
		                            if (typeof code === "number") {
		                                addOptimizedIdxToResult(code, result, ignoreCase);
		                            }
		                            else {
		                                // range
		                                var range = code;
		                                // cannot optimize when ignoreCase is
		                                if (ignoreCase === true) {
		                                    for (var rangeCode = range.from; rangeCode <= range.to; rangeCode++) {
		                                        addOptimizedIdxToResult(rangeCode, result, ignoreCase);
		                                    }
		                                }
		                                // Optimization (2 orders of magnitude less work for very large ranges)
		                                else {
		                                    // handle unoptimized values
		                                    for (var rangeCode = range.from; rangeCode <= range.to && rangeCode < lexer_1.minOptimizationVal; rangeCode++) {
		                                        addOptimizedIdxToResult(rangeCode, result, ignoreCase);
		                                    }
		                                    // Less common charCode where we optimize for faster init time, by using larger "buckets"
		                                    if (range.to >= lexer_1.minOptimizationVal) {
		                                        var minUnOptVal = range.from >= lexer_1.minOptimizationVal
		                                            ? range.from
		                                            : lexer_1.minOptimizationVal;
		                                        var maxUnOptVal = range.to;
		                                        var minOptIdx = (0, lexer_1.charCodeToOptimizedIndex)(minUnOptVal);
		                                        var maxOptIdx = (0, lexer_1.charCodeToOptimizedIndex)(maxUnOptVal);
		                                        for (var currOptIdx = minOptIdx; currOptIdx <= maxOptIdx; currOptIdx++) {
		                                            result[currOptIdx] = currOptIdx;
		                                        }
		                                    }
		                                }
		                            }
		                        });
		                        break;
		                    case "Group":
		                        firstCharOptimizedIndices(atom.value, result, ignoreCase);
		                        break;
		                    /* istanbul ignore next */
		                    default:
		                        throw Error("Non Exhaustive Match");
		                }
		                // reached a mandatory production, no more **start** codes can be found on this alternative
		                var isOptionalQuantifier = atom.quantifier !== undefined && atom.quantifier.atLeast === 0;
		                if (
		                // A group may be optional due to empty contents /(?:)/
		                // or if everything inside it is optional /((a)?)/
		                (atom.type === "Group" && isWholeOptional(atom) === false) ||
		                    // If this term is not a group it may only be optional if it has an optional quantifier
		                    (atom.type !== "Group" && isOptionalQuantifier === false)) {
		                    break;
		                }
		            }
		            break;
		        /* istanbul ignore next */
		        default:
		            throw Error("non exhaustive match!");
		    }
		    // console.log(Object.keys(result).length)
		    return (0, values_1.default)(result);
		}
		exports.firstCharOptimizedIndices = firstCharOptimizedIndices;
		function addOptimizedIdxToResult(code, result, ignoreCase) {
		    var optimizedCharIdx = (0, lexer_1.charCodeToOptimizedIndex)(code);
		    result[optimizedCharIdx] = optimizedCharIdx;
		    if (ignoreCase === true) {
		        handleIgnoreCase(code, result);
		    }
		}
		function handleIgnoreCase(code, result) {
		    var char = String.fromCharCode(code);
		    var upperChar = char.toUpperCase();
		    /* istanbul ignore else */
		    if (upperChar !== char) {
		        var optimizedCharIdx = (0, lexer_1.charCodeToOptimizedIndex)(upperChar.charCodeAt(0));
		        result[optimizedCharIdx] = optimizedCharIdx;
		    }
		    else {
		        var lowerChar = char.toLowerCase();
		        if (lowerChar !== char) {
		            var optimizedCharIdx = (0, lexer_1.charCodeToOptimizedIndex)(lowerChar.charCodeAt(0));
		            result[optimizedCharIdx] = optimizedCharIdx;
		        }
		    }
		}
		function findCode(setNode, targetCharCodes) {
		    return (0, find_1.default)(setNode.value, function (codeOrRange) {
		        if (typeof codeOrRange === "number") {
		            return (0, includes_1.default)(targetCharCodes, codeOrRange);
		        }
		        else {
		            // range
		            var range_1 = codeOrRange;
		            return ((0, find_1.default)(targetCharCodes, function (targetCode) { return range_1.from <= targetCode && targetCode <= range_1.to; }) !== undefined);
		        }
		    });
		}
		function isWholeOptional(ast) {
		    var quantifier = ast.quantifier;
		    if (quantifier && quantifier.atLeast === 0) {
		        return true;
		    }
		    if (!ast.value) {
		        return false;
		    }
		    return (0, isArray_1.default)(ast.value)
		        ? (0, every_1.default)(ast.value, isWholeOptional)
		        : isWholeOptional(ast.value);
		}
		var CharCodeFinder = /** @class */ (function (_super) {
		    __extends(CharCodeFinder, _super);
		    function CharCodeFinder(targetCharCodes) {
		        var _this = _super.call(this) || this;
		        _this.targetCharCodes = targetCharCodes;
		        _this.found = false;
		        return _this;
		    }
		    CharCodeFinder.prototype.visitChildren = function (node) {
		        // No need to keep looking...
		        if (this.found === true) {
		            return;
		        }
		        // switch lookaheads as they do not actually consume any characters thus
		        // finding a charCode at lookahead context does not mean that regexp can actually contain it in a match.
		        switch (node.type) {
		            case "Lookahead":
		                this.visitLookahead(node);
		                return;
		            case "NegativeLookahead":
		                this.visitNegativeLookahead(node);
		                return;
		        }
		        _super.prototype.visitChildren.call(this, node);
		    };
		    CharCodeFinder.prototype.visitCharacter = function (node) {
		        if ((0, includes_1.default)(this.targetCharCodes, node.value)) {
		            this.found = true;
		        }
		    };
		    CharCodeFinder.prototype.visitSet = function (node) {
		        if (node.complement) {
		            if (findCode(node, this.targetCharCodes) === undefined) {
		                this.found = true;
		            }
		        }
		        else {
		            if (findCode(node, this.targetCharCodes) !== undefined) {
		                this.found = true;
		            }
		        }
		    };
		    return CharCodeFinder;
		}(regexp_to_ast_1.BaseRegExpVisitor));
		function canMatchCharCode(charCodes, pattern) {
		    if (pattern instanceof RegExp) {
		        var ast = (0, reg_exp_parser_1.getRegExpAst)(pattern);
		        var charCodeFinder = new CharCodeFinder(charCodes);
		        charCodeFinder.visit(ast);
		        return charCodeFinder.found;
		    }
		    else {
		        return ((0, find_1.default)(pattern, function (char) {
		            return (0, includes_1.default)(charCodes, char.charCodeAt(0));
		        }) !== undefined);
		    }
		}
		exports.canMatchCharCode = canMatchCharCode;
		
	} (reg_exp));
	return reg_exp;
}

var hasRequiredLexer;

function requireLexer () {
	if (hasRequiredLexer) return lexer$2;
	hasRequiredLexer = 1;
	(function (exports) {
		var __extends = (lexer$2 && lexer$2.__extends) || (function () {
		    var extendStatics = function (d, b) {
		        extendStatics = Object.setPrototypeOf ||
		            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
		            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
		        return extendStatics(d, b);
		    };
		    return function (d, b) {
		        if (typeof b !== "function" && b !== null)
		            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
		        extendStatics(d, b);
		        function __() { this.constructor = d; }
		        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
		    };
		})();
		var __importDefault = (lexer$2 && lexer$2.__importDefault) || function (mod) {
		    return (mod && mod.__esModule) ? mod : { "default": mod };
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.charCodeToOptimizedIndex = exports.minOptimizationVal = exports.buildLineBreakIssueMessage = exports.LineTerminatorOptimizedTester = exports.isShortPattern = exports.isCustomPattern = exports.cloneEmptyGroups = exports.performWarningRuntimeChecks = exports.performRuntimeChecks = exports.addStickyFlag = exports.addStartOfInput = exports.findUnreachablePatterns = exports.findModesThatDoNotExist = exports.findInvalidGroupType = exports.findDuplicatePatterns = exports.findUnsupportedFlags = exports.findStartOfInputAnchor = exports.findEmptyMatchRegExps = exports.findEndOfInputAnchor = exports.findInvalidPatterns = exports.findMissingPatterns = exports.validatePatterns = exports.analyzeTokenTypes = exports.enableSticky = exports.disableSticky = exports.SUPPORT_STICKY = exports.MODES = exports.DEFAULT_MODE = void 0;
		var regexp_to_ast_1 = requireRegexpToAst();
		var lexer_public_1 = requireLexer_public();
		var first_1 = __importDefault(requireFirst());
		var isEmpty_1 = __importDefault(requireIsEmpty());
		var compact_1 = __importDefault(requireCompact());
		var isArray_1 = __importDefault(requireIsArray());
		var values_1 = __importDefault(requireValues());
		var flatten_1 = __importDefault(requireFlatten());
		var reject_1 = __importDefault(requireReject());
		var difference_1 = __importDefault(requireDifference());
		var indexOf_1 = __importDefault(requireIndexOf());
		var map_1 = __importDefault(requireMap());
		var forEach_1 = __importDefault(requireForEach());
		var isString_1 = __importDefault(requireIsString());
		var isFunction_1 = __importDefault(requireIsFunction());
		var isUndefined_1 = __importDefault(requireIsUndefined());
		var find_1 = __importDefault(requireFind());
		var has_1 = __importDefault(requireHas());
		var keys_1 = __importDefault(requireKeys$1());
		var isRegExp_1 = __importDefault(requireIsRegExp());
		var filter_1 = __importDefault(requireFilter());
		var defaults_1 = __importDefault(requireDefaults());
		var reduce_1 = __importDefault(requireReduce());
		var includes_1 = __importDefault(requireIncludes());
		var utils_1 = requireApi$3();
		var reg_exp_1 = requireReg_exp();
		var reg_exp_parser_1 = requireReg_exp_parser();
		var PATTERN = "PATTERN";
		exports.DEFAULT_MODE = "defaultMode";
		exports.MODES = "modes";
		exports.SUPPORT_STICKY = typeof new RegExp("(?:)").sticky === "boolean";
		function disableSticky() {
		    exports.SUPPORT_STICKY = false;
		}
		exports.disableSticky = disableSticky;
		function enableSticky() {
		    exports.SUPPORT_STICKY = true;
		}
		exports.enableSticky = enableSticky;
		function analyzeTokenTypes(tokenTypes, options) {
		    options = (0, defaults_1.default)(options, {
		        useSticky: exports.SUPPORT_STICKY,
		        debug: false,
		        safeMode: false,
		        positionTracking: "full",
		        lineTerminatorCharacters: ["\r", "\n"],
		        tracer: function (msg, action) { return action(); }
		    });
		    var tracer = options.tracer;
		    tracer("initCharCodeToOptimizedIndexMap", function () {
		        initCharCodeToOptimizedIndexMap();
		    });
		    var onlyRelevantTypes;
		    tracer("Reject Lexer.NA", function () {
		        onlyRelevantTypes = (0, reject_1.default)(tokenTypes, function (currType) {
		            return currType[PATTERN] === lexer_public_1.Lexer.NA;
		        });
		    });
		    var hasCustom = false;
		    var allTransformedPatterns;
		    tracer("Transform Patterns", function () {
		        hasCustom = false;
		        allTransformedPatterns = (0, map_1.default)(onlyRelevantTypes, function (currType) {
		            var currPattern = currType[PATTERN];
		            /* istanbul ignore else */
		            if ((0, isRegExp_1.default)(currPattern)) {
		                var regExpSource = currPattern.source;
		                if (regExpSource.length === 1 &&
		                    // only these regExp meta characters which can appear in a length one regExp
		                    regExpSource !== "^" &&
		                    regExpSource !== "$" &&
		                    regExpSource !== "." &&
		                    !currPattern.ignoreCase) {
		                    return regExpSource;
		                }
		                else if (regExpSource.length === 2 &&
		                    regExpSource[0] === "\\" &&
		                    // not a meta character
		                    !(0, includes_1.default)([
		                        "d",
		                        "D",
		                        "s",
		                        "S",
		                        "t",
		                        "r",
		                        "n",
		                        "t",
		                        "0",
		                        "c",
		                        "b",
		                        "B",
		                        "f",
		                        "v",
		                        "w",
		                        "W"
		                    ], regExpSource[1])) {
		                    // escaped meta Characters: /\+/ /\[/
		                    // or redundant escaping: /\a/
		                    // without the escaping "\"
		                    return regExpSource[1];
		                }
		                else {
		                    return options.useSticky
		                        ? addStickyFlag(currPattern)
		                        : addStartOfInput(currPattern);
		                }
		            }
		            else if ((0, isFunction_1.default)(currPattern)) {
		                hasCustom = true;
		                // CustomPatternMatcherFunc - custom patterns do not require any transformations, only wrapping in a RegExp Like object
		                return { exec: currPattern };
		            }
		            else if (typeof currPattern === "object") {
		                hasCustom = true;
		                // ICustomPattern
		                return currPattern;
		            }
		            else if (typeof currPattern === "string") {
		                if (currPattern.length === 1) {
		                    return currPattern;
		                }
		                else {
		                    var escapedRegExpString = currPattern.replace(/[\\^$.*+?()[\]{}|]/g, "\\$&");
		                    var wrappedRegExp = new RegExp(escapedRegExpString);
		                    return options.useSticky
		                        ? addStickyFlag(wrappedRegExp)
		                        : addStartOfInput(wrappedRegExp);
		                }
		            }
		            else {
		                throw Error("non exhaustive match");
		            }
		        });
		    });
		    var patternIdxToType;
		    var patternIdxToGroup;
		    var patternIdxToLongerAltIdxArr;
		    var patternIdxToPushMode;
		    var patternIdxToPopMode;
		    tracer("misc mapping", function () {
		        patternIdxToType = (0, map_1.default)(onlyRelevantTypes, function (currType) { return currType.tokenTypeIdx; });
		        patternIdxToGroup = (0, map_1.default)(onlyRelevantTypes, function (clazz) {
		            var groupName = clazz.GROUP;
		            /* istanbul ignore next */
		            if (groupName === lexer_public_1.Lexer.SKIPPED) {
		                return undefined;
		            }
		            else if ((0, isString_1.default)(groupName)) {
		                return groupName;
		            }
		            else if ((0, isUndefined_1.default)(groupName)) {
		                return false;
		            }
		            else {
		                throw Error("non exhaustive match");
		            }
		        });
		        patternIdxToLongerAltIdxArr = (0, map_1.default)(onlyRelevantTypes, function (clazz) {
		            var longerAltType = clazz.LONGER_ALT;
		            if (longerAltType) {
		                var longerAltIdxArr = (0, isArray_1.default)(longerAltType)
		                    ? (0, map_1.default)(longerAltType, function (type) { return (0, indexOf_1.default)(onlyRelevantTypes, type); })
		                    : [(0, indexOf_1.default)(onlyRelevantTypes, longerAltType)];
		                return longerAltIdxArr;
		            }
		        });
		        patternIdxToPushMode = (0, map_1.default)(onlyRelevantTypes, function (clazz) { return clazz.PUSH_MODE; });
		        patternIdxToPopMode = (0, map_1.default)(onlyRelevantTypes, function (clazz) {
		            return (0, has_1.default)(clazz, "POP_MODE");
		        });
		    });
		    var patternIdxToCanLineTerminator;
		    tracer("Line Terminator Handling", function () {
		        var lineTerminatorCharCodes = getCharCodes(options.lineTerminatorCharacters);
		        patternIdxToCanLineTerminator = (0, map_1.default)(onlyRelevantTypes, function (tokType) { return false; });
		        if (options.positionTracking !== "onlyOffset") {
		            patternIdxToCanLineTerminator = (0, map_1.default)(onlyRelevantTypes, function (tokType) {
		                if ((0, has_1.default)(tokType, "LINE_BREAKS")) {
		                    return !!tokType.LINE_BREAKS;
		                }
		                else {
		                    return (checkLineBreaksIssues(tokType, lineTerminatorCharCodes) === false &&
		                        (0, reg_exp_1.canMatchCharCode)(lineTerminatorCharCodes, tokType.PATTERN));
		                }
		            });
		        }
		    });
		    var patternIdxToIsCustom;
		    var patternIdxToShort;
		    var emptyGroups;
		    var patternIdxToConfig;
		    tracer("Misc Mapping #2", function () {
		        patternIdxToIsCustom = (0, map_1.default)(onlyRelevantTypes, isCustomPattern);
		        patternIdxToShort = (0, map_1.default)(allTransformedPatterns, isShortPattern);
		        emptyGroups = (0, reduce_1.default)(onlyRelevantTypes, function (acc, clazz) {
		            var groupName = clazz.GROUP;
		            if ((0, isString_1.default)(groupName) && !(groupName === lexer_public_1.Lexer.SKIPPED)) {
		                acc[groupName] = [];
		            }
		            return acc;
		        }, {});
		        patternIdxToConfig = (0, map_1.default)(allTransformedPatterns, function (x, idx) {
		            return {
		                pattern: allTransformedPatterns[idx],
		                longerAlt: patternIdxToLongerAltIdxArr[idx],
		                canLineTerminator: patternIdxToCanLineTerminator[idx],
		                isCustom: patternIdxToIsCustom[idx],
		                short: patternIdxToShort[idx],
		                group: patternIdxToGroup[idx],
		                push: patternIdxToPushMode[idx],
		                pop: patternIdxToPopMode[idx],
		                tokenTypeIdx: patternIdxToType[idx],
		                tokenType: onlyRelevantTypes[idx]
		            };
		        });
		    });
		    var canBeOptimized = true;
		    var charCodeToPatternIdxToConfig = [];
		    if (!options.safeMode) {
		        tracer("First Char Optimization", function () {
		            charCodeToPatternIdxToConfig = (0, reduce_1.default)(onlyRelevantTypes, function (result, currTokType, idx) {
		                if (typeof currTokType.PATTERN === "string") {
		                    var charCode = currTokType.PATTERN.charCodeAt(0);
		                    var optimizedIdx = charCodeToOptimizedIndex(charCode);
		                    addToMapOfArrays(result, optimizedIdx, patternIdxToConfig[idx]);
		                }
		                else if ((0, isArray_1.default)(currTokType.START_CHARS_HINT)) {
		                    var lastOptimizedIdx_1;
		                    (0, forEach_1.default)(currTokType.START_CHARS_HINT, function (charOrInt) {
		                        var charCode = typeof charOrInt === "string"
		                            ? charOrInt.charCodeAt(0)
		                            : charOrInt;
		                        var currOptimizedIdx = charCodeToOptimizedIndex(charCode);
		                        // Avoid adding the config multiple times
		                        /* istanbul ignore else */
		                        // - Difficult to check this scenario effects as it is only a performance
		                        //   optimization that does not change correctness
		                        if (lastOptimizedIdx_1 !== currOptimizedIdx) {
		                            lastOptimizedIdx_1 = currOptimizedIdx;
		                            addToMapOfArrays(result, currOptimizedIdx, patternIdxToConfig[idx]);
		                        }
		                    });
		                }
		                else if ((0, isRegExp_1.default)(currTokType.PATTERN)) {
		                    if (currTokType.PATTERN.unicode) {
		                        canBeOptimized = false;
		                        if (options.ensureOptimizations) {
		                            (0, utils_1.PRINT_ERROR)("".concat(reg_exp_1.failedOptimizationPrefixMsg) +
		                                "\tUnable to analyze < ".concat(currTokType.PATTERN.toString(), " > pattern.\n") +
		                                "\tThe regexp unicode flag is not currently supported by the regexp-to-ast library.\n" +
		                                "\tThis will disable the lexer's first char optimizations.\n" +
		                                "\tFor details See: https://chevrotain.io/docs/guide/resolving_lexer_errors.html#UNICODE_OPTIMIZE");
		                        }
		                    }
		                    else {
		                        var optimizedCodes = (0, reg_exp_1.getOptimizedStartCodesIndices)(currTokType.PATTERN, options.ensureOptimizations);
		                        /* istanbul ignore if */
		                        // start code will only be empty given an empty regExp or failure of regexp-to-ast library
		                        // the first should be a different validation and the second cannot be tested.
		                        if ((0, isEmpty_1.default)(optimizedCodes)) {
		                            // we cannot understand what codes may start possible matches
		                            // The optimization correctness requires knowing start codes for ALL patterns.
		                            // Not actually sure this is an error, no debug message
		                            canBeOptimized = false;
		                        }
		                        (0, forEach_1.default)(optimizedCodes, function (code) {
		                            addToMapOfArrays(result, code, patternIdxToConfig[idx]);
		                        });
		                    }
		                }
		                else {
		                    if (options.ensureOptimizations) {
		                        (0, utils_1.PRINT_ERROR)("".concat(reg_exp_1.failedOptimizationPrefixMsg) +
		                            "\tTokenType: <".concat(currTokType.name, "> is using a custom token pattern without providing <start_chars_hint> parameter.\n") +
		                            "\tThis will disable the lexer's first char optimizations.\n" +
		                            "\tFor details See: https://chevrotain.io/docs/guide/resolving_lexer_errors.html#CUSTOM_OPTIMIZE");
		                    }
		                    canBeOptimized = false;
		                }
		                return result;
		            }, []);
		        });
		    }
		    return {
		        emptyGroups: emptyGroups,
		        patternIdxToConfig: patternIdxToConfig,
		        charCodeToPatternIdxToConfig: charCodeToPatternIdxToConfig,
		        hasCustom: hasCustom,
		        canBeOptimized: canBeOptimized
		    };
		}
		exports.analyzeTokenTypes = analyzeTokenTypes;
		function validatePatterns(tokenTypes, validModesNames) {
		    var errors = [];
		    var missingResult = findMissingPatterns(tokenTypes);
		    errors = errors.concat(missingResult.errors);
		    var invalidResult = findInvalidPatterns(missingResult.valid);
		    var validTokenTypes = invalidResult.valid;
		    errors = errors.concat(invalidResult.errors);
		    errors = errors.concat(validateRegExpPattern(validTokenTypes));
		    errors = errors.concat(findInvalidGroupType(validTokenTypes));
		    errors = errors.concat(findModesThatDoNotExist(validTokenTypes, validModesNames));
		    errors = errors.concat(findUnreachablePatterns(validTokenTypes));
		    return errors;
		}
		exports.validatePatterns = validatePatterns;
		function validateRegExpPattern(tokenTypes) {
		    var errors = [];
		    var withRegExpPatterns = (0, filter_1.default)(tokenTypes, function (currTokType) {
		        return (0, isRegExp_1.default)(currTokType[PATTERN]);
		    });
		    errors = errors.concat(findEndOfInputAnchor(withRegExpPatterns));
		    errors = errors.concat(findStartOfInputAnchor(withRegExpPatterns));
		    errors = errors.concat(findUnsupportedFlags(withRegExpPatterns));
		    errors = errors.concat(findDuplicatePatterns(withRegExpPatterns));
		    errors = errors.concat(findEmptyMatchRegExps(withRegExpPatterns));
		    return errors;
		}
		function findMissingPatterns(tokenTypes) {
		    var tokenTypesWithMissingPattern = (0, filter_1.default)(tokenTypes, function (currType) {
		        return !(0, has_1.default)(currType, PATTERN);
		    });
		    var errors = (0, map_1.default)(tokenTypesWithMissingPattern, function (currType) {
		        return {
		            message: "Token Type: ->" +
		                currType.name +
		                "<- missing static 'PATTERN' property",
		            type: lexer_public_1.LexerDefinitionErrorType.MISSING_PATTERN,
		            tokenTypes: [currType]
		        };
		    });
		    var valid = (0, difference_1.default)(tokenTypes, tokenTypesWithMissingPattern);
		    return { errors: errors, valid: valid };
		}
		exports.findMissingPatterns = findMissingPatterns;
		function findInvalidPatterns(tokenTypes) {
		    var tokenTypesWithInvalidPattern = (0, filter_1.default)(tokenTypes, function (currType) {
		        var pattern = currType[PATTERN];
		        return (!(0, isRegExp_1.default)(pattern) &&
		            !(0, isFunction_1.default)(pattern) &&
		            !(0, has_1.default)(pattern, "exec") &&
		            !(0, isString_1.default)(pattern));
		    });
		    var errors = (0, map_1.default)(tokenTypesWithInvalidPattern, function (currType) {
		        return {
		            message: "Token Type: ->" +
		                currType.name +
		                "<- static 'PATTERN' can only be a RegExp, a" +
		                " Function matching the {CustomPatternMatcherFunc} type or an Object matching the {ICustomPattern} interface.",
		            type: lexer_public_1.LexerDefinitionErrorType.INVALID_PATTERN,
		            tokenTypes: [currType]
		        };
		    });
		    var valid = (0, difference_1.default)(tokenTypes, tokenTypesWithInvalidPattern);
		    return { errors: errors, valid: valid };
		}
		exports.findInvalidPatterns = findInvalidPatterns;
		var end_of_input = /[^\\][$]/;
		function findEndOfInputAnchor(tokenTypes) {
		    var EndAnchorFinder = /** @class */ (function (_super) {
		        __extends(EndAnchorFinder, _super);
		        function EndAnchorFinder() {
		            var _this = _super !== null && _super.apply(this, arguments) || this;
		            _this.found = false;
		            return _this;
		        }
		        EndAnchorFinder.prototype.visitEndAnchor = function (node) {
		            this.found = true;
		        };
		        return EndAnchorFinder;
		    }(regexp_to_ast_1.BaseRegExpVisitor));
		    var invalidRegex = (0, filter_1.default)(tokenTypes, function (currType) {
		        var pattern = currType.PATTERN;
		        try {
		            var regexpAst = (0, reg_exp_parser_1.getRegExpAst)(pattern);
		            var endAnchorVisitor = new EndAnchorFinder();
		            endAnchorVisitor.visit(regexpAst);
		            return endAnchorVisitor.found;
		        }
		        catch (e) {
		            // old behavior in case of runtime exceptions with regexp-to-ast.
		            /* istanbul ignore next - cannot ensure an error in regexp-to-ast*/
		            return end_of_input.test(pattern.source);
		        }
		    });
		    var errors = (0, map_1.default)(invalidRegex, function (currType) {
		        return {
		            message: "Unexpected RegExp Anchor Error:\n" +
		                "\tToken Type: ->" +
		                currType.name +
		                "<- static 'PATTERN' cannot contain end of input anchor '$'\n" +
		                "\tSee chevrotain.io/docs/guide/resolving_lexer_errors.html#ANCHORS" +
		                "\tfor details.",
		            type: lexer_public_1.LexerDefinitionErrorType.EOI_ANCHOR_FOUND,
		            tokenTypes: [currType]
		        };
		    });
		    return errors;
		}
		exports.findEndOfInputAnchor = findEndOfInputAnchor;
		function findEmptyMatchRegExps(tokenTypes) {
		    var matchesEmptyString = (0, filter_1.default)(tokenTypes, function (currType) {
		        var pattern = currType.PATTERN;
		        return pattern.test("");
		    });
		    var errors = (0, map_1.default)(matchesEmptyString, function (currType) {
		        return {
		            message: "Token Type: ->" +
		                currType.name +
		                "<- static 'PATTERN' must not match an empty string",
		            type: lexer_public_1.LexerDefinitionErrorType.EMPTY_MATCH_PATTERN,
		            tokenTypes: [currType]
		        };
		    });
		    return errors;
		}
		exports.findEmptyMatchRegExps = findEmptyMatchRegExps;
		var start_of_input = /[^\\[][\^]|^\^/;
		function findStartOfInputAnchor(tokenTypes) {
		    var StartAnchorFinder = /** @class */ (function (_super) {
		        __extends(StartAnchorFinder, _super);
		        function StartAnchorFinder() {
		            var _this = _super !== null && _super.apply(this, arguments) || this;
		            _this.found = false;
		            return _this;
		        }
		        StartAnchorFinder.prototype.visitStartAnchor = function (node) {
		            this.found = true;
		        };
		        return StartAnchorFinder;
		    }(regexp_to_ast_1.BaseRegExpVisitor));
		    var invalidRegex = (0, filter_1.default)(tokenTypes, function (currType) {
		        var pattern = currType.PATTERN;
		        try {
		            var regexpAst = (0, reg_exp_parser_1.getRegExpAst)(pattern);
		            var startAnchorVisitor = new StartAnchorFinder();
		            startAnchorVisitor.visit(regexpAst);
		            return startAnchorVisitor.found;
		        }
		        catch (e) {
		            // old behavior in case of runtime exceptions with regexp-to-ast.
		            /* istanbul ignore next - cannot ensure an error in regexp-to-ast*/
		            return start_of_input.test(pattern.source);
		        }
		    });
		    var errors = (0, map_1.default)(invalidRegex, function (currType) {
		        return {
		            message: "Unexpected RegExp Anchor Error:\n" +
		                "\tToken Type: ->" +
		                currType.name +
		                "<- static 'PATTERN' cannot contain start of input anchor '^'\n" +
		                "\tSee https://chevrotain.io/docs/guide/resolving_lexer_errors.html#ANCHORS" +
		                "\tfor details.",
		            type: lexer_public_1.LexerDefinitionErrorType.SOI_ANCHOR_FOUND,
		            tokenTypes: [currType]
		        };
		    });
		    return errors;
		}
		exports.findStartOfInputAnchor = findStartOfInputAnchor;
		function findUnsupportedFlags(tokenTypes) {
		    var invalidFlags = (0, filter_1.default)(tokenTypes, function (currType) {
		        var pattern = currType[PATTERN];
		        return pattern instanceof RegExp && (pattern.multiline || pattern.global);
		    });
		    var errors = (0, map_1.default)(invalidFlags, function (currType) {
		        return {
		            message: "Token Type: ->" +
		                currType.name +
		                "<- static 'PATTERN' may NOT contain global('g') or multiline('m')",
		            type: lexer_public_1.LexerDefinitionErrorType.UNSUPPORTED_FLAGS_FOUND,
		            tokenTypes: [currType]
		        };
		    });
		    return errors;
		}
		exports.findUnsupportedFlags = findUnsupportedFlags;
		// This can only test for identical duplicate RegExps, not semantically equivalent ones.
		function findDuplicatePatterns(tokenTypes) {
		    var found = [];
		    var identicalPatterns = (0, map_1.default)(tokenTypes, function (outerType) {
		        return (0, reduce_1.default)(tokenTypes, function (result, innerType) {
		            if (outerType.PATTERN.source === innerType.PATTERN.source &&
		                !(0, includes_1.default)(found, innerType) &&
		                innerType.PATTERN !== lexer_public_1.Lexer.NA) {
		                // this avoids duplicates in the result, each Token Type may only appear in one "set"
		                // in essence we are creating Equivalence classes on equality relation.
		                found.push(innerType);
		                result.push(innerType);
		                return result;
		            }
		            return result;
		        }, []);
		    });
		    identicalPatterns = (0, compact_1.default)(identicalPatterns);
		    var duplicatePatterns = (0, filter_1.default)(identicalPatterns, function (currIdenticalSet) {
		        return currIdenticalSet.length > 1;
		    });
		    var errors = (0, map_1.default)(duplicatePatterns, function (setOfIdentical) {
		        var tokenTypeNames = (0, map_1.default)(setOfIdentical, function (currType) {
		            return currType.name;
		        });
		        var dupPatternSrc = (0, first_1.default)(setOfIdentical).PATTERN;
		        return {
		            message: "The same RegExp pattern ->".concat(dupPatternSrc, "<-") +
		                "has been used in all of the following Token Types: ".concat(tokenTypeNames.join(", "), " <-"),
		            type: lexer_public_1.LexerDefinitionErrorType.DUPLICATE_PATTERNS_FOUND,
		            tokenTypes: setOfIdentical
		        };
		    });
		    return errors;
		}
		exports.findDuplicatePatterns = findDuplicatePatterns;
		function findInvalidGroupType(tokenTypes) {
		    var invalidTypes = (0, filter_1.default)(tokenTypes, function (clazz) {
		        if (!(0, has_1.default)(clazz, "GROUP")) {
		            return false;
		        }
		        var group = clazz.GROUP;
		        return group !== lexer_public_1.Lexer.SKIPPED && group !== lexer_public_1.Lexer.NA && !(0, isString_1.default)(group);
		    });
		    var errors = (0, map_1.default)(invalidTypes, function (currType) {
		        return {
		            message: "Token Type: ->" +
		                currType.name +
		                "<- static 'GROUP' can only be Lexer.SKIPPED/Lexer.NA/A String",
		            type: lexer_public_1.LexerDefinitionErrorType.INVALID_GROUP_TYPE_FOUND,
		            tokenTypes: [currType]
		        };
		    });
		    return errors;
		}
		exports.findInvalidGroupType = findInvalidGroupType;
		function findModesThatDoNotExist(tokenTypes, validModes) {
		    var invalidModes = (0, filter_1.default)(tokenTypes, function (clazz) {
		        return (clazz.PUSH_MODE !== undefined && !(0, includes_1.default)(validModes, clazz.PUSH_MODE));
		    });
		    var errors = (0, map_1.default)(invalidModes, function (tokType) {
		        var msg = "Token Type: ->".concat(tokType.name, "<- static 'PUSH_MODE' value cannot refer to a Lexer Mode ->").concat(tokType.PUSH_MODE, "<-") +
		            "which does not exist";
		        return {
		            message: msg,
		            type: lexer_public_1.LexerDefinitionErrorType.PUSH_MODE_DOES_NOT_EXIST,
		            tokenTypes: [tokType]
		        };
		    });
		    return errors;
		}
		exports.findModesThatDoNotExist = findModesThatDoNotExist;
		function findUnreachablePatterns(tokenTypes) {
		    var errors = [];
		    var canBeTested = (0, reduce_1.default)(tokenTypes, function (result, tokType, idx) {
		        var pattern = tokType.PATTERN;
		        if (pattern === lexer_public_1.Lexer.NA) {
		            return result;
		        }
		        // a more comprehensive validation for all forms of regExps would require
		        // deeper regExp analysis capabilities
		        if ((0, isString_1.default)(pattern)) {
		            result.push({ str: pattern, idx: idx, tokenType: tokType });
		        }
		        else if ((0, isRegExp_1.default)(pattern) && noMetaChar(pattern)) {
		            result.push({ str: pattern.source, idx: idx, tokenType: tokType });
		        }
		        return result;
		    }, []);
		    (0, forEach_1.default)(tokenTypes, function (tokType, testIdx) {
		        (0, forEach_1.default)(canBeTested, function (_a) {
		            var str = _a.str, idx = _a.idx, tokenType = _a.tokenType;
		            if (testIdx < idx && testTokenType(str, tokType.PATTERN)) {
		                var msg = "Token: ->".concat(tokenType.name, "<- can never be matched.\n") +
		                    "Because it appears AFTER the Token Type ->".concat(tokType.name, "<-") +
		                    "in the lexer's definition.\n" +
		                    "See https://chevrotain.io/docs/guide/resolving_lexer_errors.html#UNREACHABLE";
		                errors.push({
		                    message: msg,
		                    type: lexer_public_1.LexerDefinitionErrorType.UNREACHABLE_PATTERN,
		                    tokenTypes: [tokType, tokenType]
		                });
		            }
		        });
		    });
		    return errors;
		}
		exports.findUnreachablePatterns = findUnreachablePatterns;
		function testTokenType(str, pattern) {
		    /* istanbul ignore else */
		    if ((0, isRegExp_1.default)(pattern)) {
		        var regExpArray = pattern.exec(str);
		        return regExpArray !== null && regExpArray.index === 0;
		    }
		    else if ((0, isFunction_1.default)(pattern)) {
		        // maintain the API of custom patterns
		        return pattern(str, 0, [], {});
		    }
		    else if ((0, has_1.default)(pattern, "exec")) {
		        // maintain the API of custom patterns
		        return pattern.exec(str, 0, [], {});
		    }
		    else if (typeof pattern === "string") {
		        return pattern === str;
		    }
		    else {
		        throw Error("non exhaustive match");
		    }
		}
		function noMetaChar(regExp) {
		    //https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/RegExp
		    var metaChars = [
		        ".",
		        "\\",
		        "[",
		        "]",
		        "|",
		        "^",
		        "$",
		        "(",
		        ")",
		        "?",
		        "*",
		        "+",
		        "{"
		    ];
		    return ((0, find_1.default)(metaChars, function (char) { return regExp.source.indexOf(char) !== -1; }) === undefined);
		}
		function addStartOfInput(pattern) {
		    var flags = pattern.ignoreCase ? "i" : "";
		    // always wrapping in a none capturing group preceded by '^' to make sure matching can only work on start of input.
		    // duplicate/redundant start of input markers have no meaning (/^^^^A/ === /^A/)
		    return new RegExp("^(?:".concat(pattern.source, ")"), flags);
		}
		exports.addStartOfInput = addStartOfInput;
		function addStickyFlag(pattern) {
		    var flags = pattern.ignoreCase ? "iy" : "y";
		    // always wrapping in a none capturing group preceded by '^' to make sure matching can only work on start of input.
		    // duplicate/redundant start of input markers have no meaning (/^^^^A/ === /^A/)
		    return new RegExp("".concat(pattern.source), flags);
		}
		exports.addStickyFlag = addStickyFlag;
		function performRuntimeChecks(lexerDefinition, trackLines, lineTerminatorCharacters) {
		    var errors = [];
		    // some run time checks to help the end users.
		    if (!(0, has_1.default)(lexerDefinition, exports.DEFAULT_MODE)) {
		        errors.push({
		            message: "A MultiMode Lexer cannot be initialized without a <" +
		                exports.DEFAULT_MODE +
		                "> property in its definition\n",
		            type: lexer_public_1.LexerDefinitionErrorType.MULTI_MODE_LEXER_WITHOUT_DEFAULT_MODE
		        });
		    }
		    if (!(0, has_1.default)(lexerDefinition, exports.MODES)) {
		        errors.push({
		            message: "A MultiMode Lexer cannot be initialized without a <" +
		                exports.MODES +
		                "> property in its definition\n",
		            type: lexer_public_1.LexerDefinitionErrorType.MULTI_MODE_LEXER_WITHOUT_MODES_PROPERTY
		        });
		    }
		    if ((0, has_1.default)(lexerDefinition, exports.MODES) &&
		        (0, has_1.default)(lexerDefinition, exports.DEFAULT_MODE) &&
		        !(0, has_1.default)(lexerDefinition.modes, lexerDefinition.defaultMode)) {
		        errors.push({
		            message: "A MultiMode Lexer cannot be initialized with a ".concat(exports.DEFAULT_MODE, ": <").concat(lexerDefinition.defaultMode, ">") +
		                "which does not exist\n",
		            type: lexer_public_1.LexerDefinitionErrorType.MULTI_MODE_LEXER_DEFAULT_MODE_VALUE_DOES_NOT_EXIST
		        });
		    }
		    if ((0, has_1.default)(lexerDefinition, exports.MODES)) {
		        (0, forEach_1.default)(lexerDefinition.modes, function (currModeValue, currModeName) {
		            (0, forEach_1.default)(currModeValue, function (currTokType, currIdx) {
		                if ((0, isUndefined_1.default)(currTokType)) {
		                    errors.push({
		                        message: "A Lexer cannot be initialized using an undefined Token Type. Mode:" +
		                            "<".concat(currModeName, "> at index: <").concat(currIdx, ">\n"),
		                        type: lexer_public_1.LexerDefinitionErrorType.LEXER_DEFINITION_CANNOT_CONTAIN_UNDEFINED
		                    });
		                }
		                else if ((0, has_1.default)(currTokType, "LONGER_ALT")) {
		                    var longerAlt = (0, isArray_1.default)(currTokType.LONGER_ALT)
		                        ? currTokType.LONGER_ALT
		                        : [currTokType.LONGER_ALT];
		                    (0, forEach_1.default)(longerAlt, function (currLongerAlt) {
		                        if (!(0, isUndefined_1.default)(currLongerAlt) &&
		                            !(0, includes_1.default)(currModeValue, currLongerAlt)) {
		                            errors.push({
		                                message: "A MultiMode Lexer cannot be initialized with a longer_alt <".concat(currLongerAlt.name, "> on token <").concat(currTokType.name, "> outside of mode <").concat(currModeName, ">\n"),
		                                type: lexer_public_1.LexerDefinitionErrorType.MULTI_MODE_LEXER_LONGER_ALT_NOT_IN_CURRENT_MODE
		                            });
		                        }
		                    });
		                }
		            });
		        });
		    }
		    return errors;
		}
		exports.performRuntimeChecks = performRuntimeChecks;
		function performWarningRuntimeChecks(lexerDefinition, trackLines, lineTerminatorCharacters) {
		    var warnings = [];
		    var hasAnyLineBreak = false;
		    var allTokenTypes = (0, compact_1.default)((0, flatten_1.default)((0, values_1.default)(lexerDefinition.modes)));
		    var concreteTokenTypes = (0, reject_1.default)(allTokenTypes, function (currType) { return currType[PATTERN] === lexer_public_1.Lexer.NA; });
		    var terminatorCharCodes = getCharCodes(lineTerminatorCharacters);
		    if (trackLines) {
		        (0, forEach_1.default)(concreteTokenTypes, function (tokType) {
		            var currIssue = checkLineBreaksIssues(tokType, terminatorCharCodes);
		            if (currIssue !== false) {
		                var message = buildLineBreakIssueMessage(tokType, currIssue);
		                var warningDescriptor = {
		                    message: message,
		                    type: currIssue.issue,
		                    tokenType: tokType
		                };
		                warnings.push(warningDescriptor);
		            }
		            else {
		                // we don't want to attempt to scan if the user explicitly specified the line_breaks option.
		                if ((0, has_1.default)(tokType, "LINE_BREAKS")) {
		                    if (tokType.LINE_BREAKS === true) {
		                        hasAnyLineBreak = true;
		                    }
		                }
		                else {
		                    if ((0, reg_exp_1.canMatchCharCode)(terminatorCharCodes, tokType.PATTERN)) {
		                        hasAnyLineBreak = true;
		                    }
		                }
		            }
		        });
		    }
		    if (trackLines && !hasAnyLineBreak) {
		        warnings.push({
		            message: "Warning: No LINE_BREAKS Found.\n" +
		                "\tThis Lexer has been defined to track line and column information,\n" +
		                "\tBut none of the Token Types can be identified as matching a line terminator.\n" +
		                "\tSee https://chevrotain.io/docs/guide/resolving_lexer_errors.html#LINE_BREAKS \n" +
		                "\tfor details.",
		            type: lexer_public_1.LexerDefinitionErrorType.NO_LINE_BREAKS_FLAGS
		        });
		    }
		    return warnings;
		}
		exports.performWarningRuntimeChecks = performWarningRuntimeChecks;
		function cloneEmptyGroups(emptyGroups) {
		    var clonedResult = {};
		    var groupKeys = (0, keys_1.default)(emptyGroups);
		    (0, forEach_1.default)(groupKeys, function (currKey) {
		        var currGroupValue = emptyGroups[currKey];
		        /* istanbul ignore else */
		        if ((0, isArray_1.default)(currGroupValue)) {
		            clonedResult[currKey] = [];
		        }
		        else {
		            throw Error("non exhaustive match");
		        }
		    });
		    return clonedResult;
		}
		exports.cloneEmptyGroups = cloneEmptyGroups;
		// TODO: refactor to avoid duplication
		function isCustomPattern(tokenType) {
		    var pattern = tokenType.PATTERN;
		    /* istanbul ignore else */
		    if ((0, isRegExp_1.default)(pattern)) {
		        return false;
		    }
		    else if ((0, isFunction_1.default)(pattern)) {
		        // CustomPatternMatcherFunc - custom patterns do not require any transformations, only wrapping in a RegExp Like object
		        return true;
		    }
		    else if ((0, has_1.default)(pattern, "exec")) {
		        // ICustomPattern
		        return true;
		    }
		    else if ((0, isString_1.default)(pattern)) {
		        return false;
		    }
		    else {
		        throw Error("non exhaustive match");
		    }
		}
		exports.isCustomPattern = isCustomPattern;
		function isShortPattern(pattern) {
		    if ((0, isString_1.default)(pattern) && pattern.length === 1) {
		        return pattern.charCodeAt(0);
		    }
		    else {
		        return false;
		    }
		}
		exports.isShortPattern = isShortPattern;
		/**
		 * Faster than using a RegExp for default newline detection during lexing.
		 */
		exports.LineTerminatorOptimizedTester = {
		    // implements /\n|\r\n?/g.test
		    test: function (text) {
		        var len = text.length;
		        for (var i = this.lastIndex; i < len; i++) {
		            var c = text.charCodeAt(i);
		            if (c === 10) {
		                this.lastIndex = i + 1;
		                return true;
		            }
		            else if (c === 13) {
		                if (text.charCodeAt(i + 1) === 10) {
		                    this.lastIndex = i + 2;
		                }
		                else {
		                    this.lastIndex = i + 1;
		                }
		                return true;
		            }
		        }
		        return false;
		    },
		    lastIndex: 0
		};
		function checkLineBreaksIssues(tokType, lineTerminatorCharCodes) {
		    if ((0, has_1.default)(tokType, "LINE_BREAKS")) {
		        // if the user explicitly declared the line_breaks option we will respect their choice
		        // and assume it is correct.
		        return false;
		    }
		    else {
		        /* istanbul ignore else */
		        if ((0, isRegExp_1.default)(tokType.PATTERN)) {
		            try {
		                // TODO: why is the casting suddenly needed?
		                (0, reg_exp_1.canMatchCharCode)(lineTerminatorCharCodes, tokType.PATTERN);
		            }
		            catch (e) {
		                /* istanbul ignore next - to test this we would have to mock <canMatchCharCode> to throw an error */
		                return {
		                    issue: lexer_public_1.LexerDefinitionErrorType.IDENTIFY_TERMINATOR,
		                    errMsg: e.message
		                };
		            }
		            return false;
		        }
		        else if ((0, isString_1.default)(tokType.PATTERN)) {
		            // string literal patterns can always be analyzed to detect line terminator usage
		            return false;
		        }
		        else if (isCustomPattern(tokType)) {
		            // custom token types
		            return { issue: lexer_public_1.LexerDefinitionErrorType.CUSTOM_LINE_BREAK };
		        }
		        else {
		            throw Error("non exhaustive match");
		        }
		    }
		}
		function buildLineBreakIssueMessage(tokType, details) {
		    /* istanbul ignore else */
		    if (details.issue === lexer_public_1.LexerDefinitionErrorType.IDENTIFY_TERMINATOR) {
		        return ("Warning: unable to identify line terminator usage in pattern.\n" +
		            "\tThe problem is in the <".concat(tokType.name, "> Token Type\n") +
		            "\t Root cause: ".concat(details.errMsg, ".\n") +
		            "\tFor details See: https://chevrotain.io/docs/guide/resolving_lexer_errors.html#IDENTIFY_TERMINATOR");
		    }
		    else if (details.issue === lexer_public_1.LexerDefinitionErrorType.CUSTOM_LINE_BREAK) {
		        return ("Warning: A Custom Token Pattern should specify the <line_breaks> option.\n" +
		            "\tThe problem is in the <".concat(tokType.name, "> Token Type\n") +
		            "\tFor details See: https://chevrotain.io/docs/guide/resolving_lexer_errors.html#CUSTOM_LINE_BREAK");
		    }
		    else {
		        throw Error("non exhaustive match");
		    }
		}
		exports.buildLineBreakIssueMessage = buildLineBreakIssueMessage;
		function getCharCodes(charsOrCodes) {
		    var charCodes = (0, map_1.default)(charsOrCodes, function (numOrString) {
		        if ((0, isString_1.default)(numOrString)) {
		            return numOrString.charCodeAt(0);
		        }
		        else {
		            return numOrString;
		        }
		    });
		    return charCodes;
		}
		function addToMapOfArrays(map, key, value) {
		    if (map[key] === undefined) {
		        map[key] = [value];
		    }
		    else {
		        map[key].push(value);
		    }
		}
		exports.minOptimizationVal = 256;
		/**
		 * We are mapping charCode above ASCI (256) into buckets each in the size of 256.
		 * This is because ASCI are the most common start chars so each one of those will get its own
		 * possible token configs vector.
		 *
		 * Tokens starting with charCodes "above" ASCI are uncommon, so we can "afford"
		 * to place these into buckets of possible token configs, What we gain from
		 * this is avoiding the case of creating an optimization 'charCodeToPatternIdxToConfig'
		 * which would contain 10,000+ arrays of small size (e.g unicode Identifiers scenario).
		 * Our 'charCodeToPatternIdxToConfig' max size will now be:
		 * 256 + (2^16 / 2^8) - 1 === 511
		 *
		 * note the hack for fast division integer part extraction
		 * See: https://stackoverflow.com/a/4228528
		 */
		var charCodeToOptimizedIdxMap = [];
		function charCodeToOptimizedIndex(charCode) {
		    return charCode < exports.minOptimizationVal
		        ? charCode
		        : charCodeToOptimizedIdxMap[charCode];
		}
		exports.charCodeToOptimizedIndex = charCodeToOptimizedIndex;
		/**
		 * This is a compromise between cold start / hot running performance
		 * Creating this array takes ~3ms on a modern machine,
		 * But if we perform the computation at runtime as needed the CSS Lexer benchmark
		 * performance degrades by ~10%
		 *
		 * TODO: Perhaps it should be lazy initialized only if a charCode > 255 is used.
		 */
		function initCharCodeToOptimizedIndexMap() {
		    if ((0, isEmpty_1.default)(charCodeToOptimizedIdxMap)) {
		        charCodeToOptimizedIdxMap = new Array(65536);
		        for (var i = 0; i < 65536; i++) {
		            charCodeToOptimizedIdxMap[i] = i > 255 ? 255 + ~~(i / 255) : i;
		        }
		    }
		}
		
	} (lexer$2));
	return lexer$2;
}

/**
 * Gets the last element of `array`.
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Array
 * @param {Array} array The array to query.
 * @returns {*} Returns the last element of `array`.
 * @example
 *
 * _.last([1, 2, 3]);
 * // => 3
 */

var last_1;
var hasRequiredLast;

function requireLast () {
	if (hasRequiredLast) return last_1;
	hasRequiredLast = 1;
	function last(array) {
	  var length = array == null ? 0 : array.length;
	  return length ? array[length - 1] : undefined;
	}

	last_1 = last;
	return last_1;
}

var tokens = {};

var hasRequiredTokens;

function requireTokens () {
	if (hasRequiredTokens) return tokens;
	hasRequiredTokens = 1;
	(function (exports) {
		var __importDefault = (tokens && tokens.__importDefault) || function (mod) {
		    return (mod && mod.__esModule) ? mod : { "default": mod };
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.isTokenType = exports.hasExtendingTokensTypesMapProperty = exports.hasExtendingTokensTypesProperty = exports.hasCategoriesProperty = exports.hasShortKeyProperty = exports.singleAssignCategoriesToksMap = exports.assignCategoriesMapProp = exports.assignCategoriesTokensProp = exports.assignTokenDefaultProps = exports.expandCategories = exports.augmentTokenTypes = exports.tokenIdxToClass = exports.tokenShortNameIdx = exports.tokenStructuredMatcherNoCategories = exports.tokenStructuredMatcher = void 0;
		var isEmpty_1 = __importDefault(requireIsEmpty());
		var compact_1 = __importDefault(requireCompact());
		var isArray_1 = __importDefault(requireIsArray());
		var flatten_1 = __importDefault(requireFlatten());
		var difference_1 = __importDefault(requireDifference());
		var map_1 = __importDefault(requireMap());
		var forEach_1 = __importDefault(requireForEach());
		var has_1 = __importDefault(requireHas());
		var includes_1 = __importDefault(requireIncludes());
		var clone_1 = __importDefault(requireClone());
		function tokenStructuredMatcher(tokInstance, tokConstructor) {
		    var instanceType = tokInstance.tokenTypeIdx;
		    if (instanceType === tokConstructor.tokenTypeIdx) {
		        return true;
		    }
		    else {
		        return (tokConstructor.isParent === true &&
		            tokConstructor.categoryMatchesMap[instanceType] === true);
		    }
		}
		exports.tokenStructuredMatcher = tokenStructuredMatcher;
		// Optimized tokenMatcher in case our grammar does not use token categories
		// Being so tiny it is much more likely to be in-lined and this avoid the function call overhead
		function tokenStructuredMatcherNoCategories(token, tokType) {
		    return token.tokenTypeIdx === tokType.tokenTypeIdx;
		}
		exports.tokenStructuredMatcherNoCategories = tokenStructuredMatcherNoCategories;
		exports.tokenShortNameIdx = 1;
		exports.tokenIdxToClass = {};
		function augmentTokenTypes(tokenTypes) {
		    // collect the parent Token Types as well.
		    var tokenTypesAndParents = expandCategories(tokenTypes);
		    // add required tokenType and categoryMatches properties
		    assignTokenDefaultProps(tokenTypesAndParents);
		    // fill up the categoryMatches
		    assignCategoriesMapProp(tokenTypesAndParents);
		    assignCategoriesTokensProp(tokenTypesAndParents);
		    (0, forEach_1.default)(tokenTypesAndParents, function (tokType) {
		        tokType.isParent = tokType.categoryMatches.length > 0;
		    });
		}
		exports.augmentTokenTypes = augmentTokenTypes;
		function expandCategories(tokenTypes) {
		    var result = (0, clone_1.default)(tokenTypes);
		    var categories = tokenTypes;
		    var searching = true;
		    while (searching) {
		        categories = (0, compact_1.default)((0, flatten_1.default)((0, map_1.default)(categories, function (currTokType) { return currTokType.CATEGORIES; })));
		        var newCategories = (0, difference_1.default)(categories, result);
		        result = result.concat(newCategories);
		        if ((0, isEmpty_1.default)(newCategories)) {
		            searching = false;
		        }
		        else {
		            categories = newCategories;
		        }
		    }
		    return result;
		}
		exports.expandCategories = expandCategories;
		function assignTokenDefaultProps(tokenTypes) {
		    (0, forEach_1.default)(tokenTypes, function (currTokType) {
		        if (!hasShortKeyProperty(currTokType)) {
		            exports.tokenIdxToClass[exports.tokenShortNameIdx] = currTokType;
		            currTokType.tokenTypeIdx = exports.tokenShortNameIdx++;
		        }
		        // CATEGORIES? : TokenType | TokenType[]
		        if (hasCategoriesProperty(currTokType) &&
		            !(0, isArray_1.default)(currTokType.CATEGORIES)
		        // &&
		        // !isUndefined(currTokType.CATEGORIES.PATTERN)
		        ) {
		            currTokType.CATEGORIES = [currTokType.CATEGORIES];
		        }
		        if (!hasCategoriesProperty(currTokType)) {
		            currTokType.CATEGORIES = [];
		        }
		        if (!hasExtendingTokensTypesProperty(currTokType)) {
		            currTokType.categoryMatches = [];
		        }
		        if (!hasExtendingTokensTypesMapProperty(currTokType)) {
		            currTokType.categoryMatchesMap = {};
		        }
		    });
		}
		exports.assignTokenDefaultProps = assignTokenDefaultProps;
		function assignCategoriesTokensProp(tokenTypes) {
		    (0, forEach_1.default)(tokenTypes, function (currTokType) {
		        // avoid duplications
		        currTokType.categoryMatches = [];
		        (0, forEach_1.default)(currTokType.categoryMatchesMap, function (val, key) {
		            currTokType.categoryMatches.push(exports.tokenIdxToClass[key].tokenTypeIdx);
		        });
		    });
		}
		exports.assignCategoriesTokensProp = assignCategoriesTokensProp;
		function assignCategoriesMapProp(tokenTypes) {
		    (0, forEach_1.default)(tokenTypes, function (currTokType) {
		        singleAssignCategoriesToksMap([], currTokType);
		    });
		}
		exports.assignCategoriesMapProp = assignCategoriesMapProp;
		function singleAssignCategoriesToksMap(path, nextNode) {
		    (0, forEach_1.default)(path, function (pathNode) {
		        nextNode.categoryMatchesMap[pathNode.tokenTypeIdx] = true;
		    });
		    (0, forEach_1.default)(nextNode.CATEGORIES, function (nextCategory) {
		        var newPath = path.concat(nextNode);
		        // avoids infinite loops due to cyclic categories.
		        if (!(0, includes_1.default)(newPath, nextCategory)) {
		            singleAssignCategoriesToksMap(newPath, nextCategory);
		        }
		    });
		}
		exports.singleAssignCategoriesToksMap = singleAssignCategoriesToksMap;
		function hasShortKeyProperty(tokType) {
		    return (0, has_1.default)(tokType, "tokenTypeIdx");
		}
		exports.hasShortKeyProperty = hasShortKeyProperty;
		function hasCategoriesProperty(tokType) {
		    return (0, has_1.default)(tokType, "CATEGORIES");
		}
		exports.hasCategoriesProperty = hasCategoriesProperty;
		function hasExtendingTokensTypesProperty(tokType) {
		    return (0, has_1.default)(tokType, "categoryMatches");
		}
		exports.hasExtendingTokensTypesProperty = hasExtendingTokensTypesProperty;
		function hasExtendingTokensTypesMapProperty(tokType) {
		    return (0, has_1.default)(tokType, "categoryMatchesMap");
		}
		exports.hasExtendingTokensTypesMapProperty = hasExtendingTokensTypesMapProperty;
		function isTokenType(tokType) {
		    return (0, has_1.default)(tokType, "tokenTypeIdx");
		}
		exports.isTokenType = isTokenType;
		
	} (tokens));
	return tokens;
}

var lexer_errors_public = {};

var hasRequiredLexer_errors_public;

function requireLexer_errors_public () {
	if (hasRequiredLexer_errors_public) return lexer_errors_public;
	hasRequiredLexer_errors_public = 1;
	Object.defineProperty(lexer_errors_public, "__esModule", { value: true });
	lexer_errors_public.defaultLexerErrorProvider = void 0;
	lexer_errors_public.defaultLexerErrorProvider = {
	    buildUnableToPopLexerModeMessage: function (token) {
	        return "Unable to pop Lexer Mode after encountering Token ->".concat(token.image, "<- The Mode Stack is empty");
	    },
	    buildUnexpectedCharactersMessage: function (fullText, startOffset, length, line, column) {
	        return ("unexpected character: ->".concat(fullText.charAt(startOffset), "<- at offset: ").concat(startOffset, ",") + " skipped ".concat(length, " characters."));
	    }
	};
	
	return lexer_errors_public;
}

var hasRequiredLexer_public;

function requireLexer_public () {
	if (hasRequiredLexer_public) return lexer_public;
	hasRequiredLexer_public = 1;
	(function (exports) {
		var __importDefault = (lexer_public && lexer_public.__importDefault) || function (mod) {
		    return (mod && mod.__esModule) ? mod : { "default": mod };
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.Lexer = exports.LexerDefinitionErrorType = void 0;
		var lexer_1 = requireLexer();
		var noop_1 = __importDefault(requireNoop());
		var isEmpty_1 = __importDefault(requireIsEmpty());
		var isArray_1 = __importDefault(requireIsArray());
		var last_1 = __importDefault(requireLast());
		var reject_1 = __importDefault(requireReject());
		var map_1 = __importDefault(requireMap());
		var forEach_1 = __importDefault(requireForEach());
		var keys_1 = __importDefault(requireKeys$1());
		var isUndefined_1 = __importDefault(requireIsUndefined());
		var identity_1 = __importDefault(requireIdentity());
		var assign_1 = __importDefault(requireAssign());
		var reduce_1 = __importDefault(requireReduce());
		var clone_1 = __importDefault(requireClone());
		var utils_1 = requireApi$3();
		var tokens_1 = requireTokens();
		var lexer_errors_public_1 = requireLexer_errors_public();
		var reg_exp_parser_1 = requireReg_exp_parser();
		(function (LexerDefinitionErrorType) {
		    LexerDefinitionErrorType[LexerDefinitionErrorType["MISSING_PATTERN"] = 0] = "MISSING_PATTERN";
		    LexerDefinitionErrorType[LexerDefinitionErrorType["INVALID_PATTERN"] = 1] = "INVALID_PATTERN";
		    LexerDefinitionErrorType[LexerDefinitionErrorType["EOI_ANCHOR_FOUND"] = 2] = "EOI_ANCHOR_FOUND";
		    LexerDefinitionErrorType[LexerDefinitionErrorType["UNSUPPORTED_FLAGS_FOUND"] = 3] = "UNSUPPORTED_FLAGS_FOUND";
		    LexerDefinitionErrorType[LexerDefinitionErrorType["DUPLICATE_PATTERNS_FOUND"] = 4] = "DUPLICATE_PATTERNS_FOUND";
		    LexerDefinitionErrorType[LexerDefinitionErrorType["INVALID_GROUP_TYPE_FOUND"] = 5] = "INVALID_GROUP_TYPE_FOUND";
		    LexerDefinitionErrorType[LexerDefinitionErrorType["PUSH_MODE_DOES_NOT_EXIST"] = 6] = "PUSH_MODE_DOES_NOT_EXIST";
		    LexerDefinitionErrorType[LexerDefinitionErrorType["MULTI_MODE_LEXER_WITHOUT_DEFAULT_MODE"] = 7] = "MULTI_MODE_LEXER_WITHOUT_DEFAULT_MODE";
		    LexerDefinitionErrorType[LexerDefinitionErrorType["MULTI_MODE_LEXER_WITHOUT_MODES_PROPERTY"] = 8] = "MULTI_MODE_LEXER_WITHOUT_MODES_PROPERTY";
		    LexerDefinitionErrorType[LexerDefinitionErrorType["MULTI_MODE_LEXER_DEFAULT_MODE_VALUE_DOES_NOT_EXIST"] = 9] = "MULTI_MODE_LEXER_DEFAULT_MODE_VALUE_DOES_NOT_EXIST";
		    LexerDefinitionErrorType[LexerDefinitionErrorType["LEXER_DEFINITION_CANNOT_CONTAIN_UNDEFINED"] = 10] = "LEXER_DEFINITION_CANNOT_CONTAIN_UNDEFINED";
		    LexerDefinitionErrorType[LexerDefinitionErrorType["SOI_ANCHOR_FOUND"] = 11] = "SOI_ANCHOR_FOUND";
		    LexerDefinitionErrorType[LexerDefinitionErrorType["EMPTY_MATCH_PATTERN"] = 12] = "EMPTY_MATCH_PATTERN";
		    LexerDefinitionErrorType[LexerDefinitionErrorType["NO_LINE_BREAKS_FLAGS"] = 13] = "NO_LINE_BREAKS_FLAGS";
		    LexerDefinitionErrorType[LexerDefinitionErrorType["UNREACHABLE_PATTERN"] = 14] = "UNREACHABLE_PATTERN";
		    LexerDefinitionErrorType[LexerDefinitionErrorType["IDENTIFY_TERMINATOR"] = 15] = "IDENTIFY_TERMINATOR";
		    LexerDefinitionErrorType[LexerDefinitionErrorType["CUSTOM_LINE_BREAK"] = 16] = "CUSTOM_LINE_BREAK";
		    LexerDefinitionErrorType[LexerDefinitionErrorType["MULTI_MODE_LEXER_LONGER_ALT_NOT_IN_CURRENT_MODE"] = 17] = "MULTI_MODE_LEXER_LONGER_ALT_NOT_IN_CURRENT_MODE";
		})(exports.LexerDefinitionErrorType || (exports.LexerDefinitionErrorType = {}));
		var DEFAULT_LEXER_CONFIG = {
		    deferDefinitionErrorsHandling: false,
		    positionTracking: "full",
		    lineTerminatorsPattern: /\n|\r\n?/g,
		    lineTerminatorCharacters: ["\n", "\r"],
		    ensureOptimizations: false,
		    safeMode: false,
		    errorMessageProvider: lexer_errors_public_1.defaultLexerErrorProvider,
		    traceInitPerf: false,
		    skipValidations: false,
		    recoveryEnabled: true
		};
		Object.freeze(DEFAULT_LEXER_CONFIG);
		var Lexer = /** @class */ (function () {
		    function Lexer(lexerDefinition, config) {
		        if (config === void 0) { config = DEFAULT_LEXER_CONFIG; }
		        var _this = this;
		        this.lexerDefinition = lexerDefinition;
		        this.lexerDefinitionErrors = [];
		        this.lexerDefinitionWarning = [];
		        this.patternIdxToConfig = {};
		        this.charCodeToPatternIdxToConfig = {};
		        this.modes = [];
		        this.emptyGroups = {};
		        this.trackStartLines = true;
		        this.trackEndLines = true;
		        this.hasCustom = false;
		        this.canModeBeOptimized = {};
		        // Duplicated from the parser's perf trace trait to allow future extraction
		        // of the lexer to a separate package.
		        this.TRACE_INIT = function (phaseDesc, phaseImpl) {
		            // No need to optimize this using NOOP pattern because
		            // It is not called in a hot spot...
		            if (_this.traceInitPerf === true) {
		                _this.traceInitIndent++;
		                var indent = new Array(_this.traceInitIndent + 1).join("\t");
		                if (_this.traceInitIndent < _this.traceInitMaxIdent) {
		                    console.log("".concat(indent, "--> <").concat(phaseDesc, ">"));
		                }
		                var _a = (0, utils_1.timer)(phaseImpl), time = _a.time, value = _a.value;
		                /* istanbul ignore next - Difficult to reproduce specific performance behavior (>10ms) in tests */
		                var traceMethod = time > 10 ? console.warn : console.log;
		                if (_this.traceInitIndent < _this.traceInitMaxIdent) {
		                    traceMethod("".concat(indent, "<-- <").concat(phaseDesc, "> time: ").concat(time, "ms"));
		                }
		                _this.traceInitIndent--;
		                return value;
		            }
		            else {
		                return phaseImpl();
		            }
		        };
		        if (typeof config === "boolean") {
		            throw Error("The second argument to the Lexer constructor is now an ILexerConfig Object.\n" +
		                "a boolean 2nd argument is no longer supported");
		        }
		        // todo: defaults func?
		        this.config = (0, assign_1.default)({}, DEFAULT_LEXER_CONFIG, config);
		        var traceInitVal = this.config.traceInitPerf;
		        if (traceInitVal === true) {
		            this.traceInitMaxIdent = Infinity;
		            this.traceInitPerf = true;
		        }
		        else if (typeof traceInitVal === "number") {
		            this.traceInitMaxIdent = traceInitVal;
		            this.traceInitPerf = true;
		        }
		        this.traceInitIndent = -1;
		        this.TRACE_INIT("Lexer Constructor", function () {
		            var actualDefinition;
		            var hasOnlySingleMode = true;
		            _this.TRACE_INIT("Lexer Config handling", function () {
		                if (_this.config.lineTerminatorsPattern ===
		                    DEFAULT_LEXER_CONFIG.lineTerminatorsPattern) {
		                    // optimized built-in implementation for the defaults definition of lineTerminators
		                    _this.config.lineTerminatorsPattern = lexer_1.LineTerminatorOptimizedTester;
		                }
		                else {
		                    if (_this.config.lineTerminatorCharacters ===
		                        DEFAULT_LEXER_CONFIG.lineTerminatorCharacters) {
		                        throw Error("Error: Missing <lineTerminatorCharacters> property on the Lexer config.\n" +
		                            "\tFor details See: https://chevrotain.io/docs/guide/resolving_lexer_errors.html#MISSING_LINE_TERM_CHARS");
		                    }
		                }
		                if (config.safeMode && config.ensureOptimizations) {
		                    throw Error('"safeMode" and "ensureOptimizations" flags are mutually exclusive.');
		                }
		                _this.trackStartLines = /full|onlyStart/i.test(_this.config.positionTracking);
		                _this.trackEndLines = /full/i.test(_this.config.positionTracking);
		                // Convert SingleModeLexerDefinition into a IMultiModeLexerDefinition.
		                if ((0, isArray_1.default)(lexerDefinition)) {
		                    actualDefinition = {
		                        modes: { defaultMode: (0, clone_1.default)(lexerDefinition) },
		                        defaultMode: lexer_1.DEFAULT_MODE
		                    };
		                }
		                else {
		                    // no conversion needed, input should already be a IMultiModeLexerDefinition
		                    hasOnlySingleMode = false;
		                    actualDefinition = (0, clone_1.default)(lexerDefinition);
		                }
		            });
		            if (_this.config.skipValidations === false) {
		                _this.TRACE_INIT("performRuntimeChecks", function () {
		                    _this.lexerDefinitionErrors = _this.lexerDefinitionErrors.concat((0, lexer_1.performRuntimeChecks)(actualDefinition, _this.trackStartLines, _this.config.lineTerminatorCharacters));
		                });
		                _this.TRACE_INIT("performWarningRuntimeChecks", function () {
		                    _this.lexerDefinitionWarning = _this.lexerDefinitionWarning.concat((0, lexer_1.performWarningRuntimeChecks)(actualDefinition, _this.trackStartLines, _this.config.lineTerminatorCharacters));
		                });
		            }
		            // for extra robustness to avoid throwing an none informative error message
		            actualDefinition.modes = actualDefinition.modes
		                ? actualDefinition.modes
		                : {};
		            // an error of undefined TokenTypes will be detected in "performRuntimeChecks" above.
		            // this transformation is to increase robustness in the case of partially invalid lexer definition.
		            (0, forEach_1.default)(actualDefinition.modes, function (currModeValue, currModeName) {
		                actualDefinition.modes[currModeName] = (0, reject_1.default)(currModeValue, function (currTokType) { return (0, isUndefined_1.default)(currTokType); });
		            });
		            var allModeNames = (0, keys_1.default)(actualDefinition.modes);
		            (0, forEach_1.default)(actualDefinition.modes, function (currModDef, currModName) {
		                _this.TRACE_INIT("Mode: <".concat(currModName, "> processing"), function () {
		                    _this.modes.push(currModName);
		                    if (_this.config.skipValidations === false) {
		                        _this.TRACE_INIT("validatePatterns", function () {
		                            _this.lexerDefinitionErrors = _this.lexerDefinitionErrors.concat((0, lexer_1.validatePatterns)(currModDef, allModeNames));
		                        });
		                    }
		                    // If definition errors were encountered, the analysis phase may fail unexpectedly/
		                    // Considering a lexer with definition errors may never be used, there is no point
		                    // to performing the analysis anyhow...
		                    if ((0, isEmpty_1.default)(_this.lexerDefinitionErrors)) {
		                        (0, tokens_1.augmentTokenTypes)(currModDef);
		                        var currAnalyzeResult_1;
		                        _this.TRACE_INIT("analyzeTokenTypes", function () {
		                            currAnalyzeResult_1 = (0, lexer_1.analyzeTokenTypes)(currModDef, {
		                                lineTerminatorCharacters: _this.config.lineTerminatorCharacters,
		                                positionTracking: config.positionTracking,
		                                ensureOptimizations: config.ensureOptimizations,
		                                safeMode: config.safeMode,
		                                tracer: _this.TRACE_INIT
		                            });
		                        });
		                        _this.patternIdxToConfig[currModName] =
		                            currAnalyzeResult_1.patternIdxToConfig;
		                        _this.charCodeToPatternIdxToConfig[currModName] =
		                            currAnalyzeResult_1.charCodeToPatternIdxToConfig;
		                        _this.emptyGroups = (0, assign_1.default)({}, _this.emptyGroups, currAnalyzeResult_1.emptyGroups);
		                        _this.hasCustom = currAnalyzeResult_1.hasCustom || _this.hasCustom;
		                        _this.canModeBeOptimized[currModName] =
		                            currAnalyzeResult_1.canBeOptimized;
		                    }
		                });
		            });
		            _this.defaultMode = actualDefinition.defaultMode;
		            if (!(0, isEmpty_1.default)(_this.lexerDefinitionErrors) &&
		                !_this.config.deferDefinitionErrorsHandling) {
		                var allErrMessages = (0, map_1.default)(_this.lexerDefinitionErrors, function (error) {
		                    return error.message;
		                });
		                var allErrMessagesString = allErrMessages.join("-----------------------\n");
		                throw new Error("Errors detected in definition of Lexer:\n" + allErrMessagesString);
		            }
		            // Only print warning if there are no errors, This will avoid pl
		            (0, forEach_1.default)(_this.lexerDefinitionWarning, function (warningDescriptor) {
		                (0, utils_1.PRINT_WARNING)(warningDescriptor.message);
		            });
		            _this.TRACE_INIT("Choosing sub-methods implementations", function () {
		                // Choose the relevant internal implementations for this specific parser.
		                // These implementations should be in-lined by the JavaScript engine
		                // to provide optimal performance in each scenario.
		                if (lexer_1.SUPPORT_STICKY) {
		                    _this.chopInput = identity_1.default;
		                    _this.match = _this.matchWithTest;
		                }
		                else {
		                    _this.updateLastIndex = noop_1.default;
		                    _this.match = _this.matchWithExec;
		                }
		                if (hasOnlySingleMode) {
		                    _this.handleModes = noop_1.default;
		                }
		                if (_this.trackStartLines === false) {
		                    _this.computeNewColumn = identity_1.default;
		                }
		                if (_this.trackEndLines === false) {
		                    _this.updateTokenEndLineColumnLocation = noop_1.default;
		                }
		                if (/full/i.test(_this.config.positionTracking)) {
		                    _this.createTokenInstance = _this.createFullToken;
		                }
		                else if (/onlyStart/i.test(_this.config.positionTracking)) {
		                    _this.createTokenInstance = _this.createStartOnlyToken;
		                }
		                else if (/onlyOffset/i.test(_this.config.positionTracking)) {
		                    _this.createTokenInstance = _this.createOffsetOnlyToken;
		                }
		                else {
		                    throw Error("Invalid <positionTracking> config option: \"".concat(_this.config.positionTracking, "\""));
		                }
		                if (_this.hasCustom) {
		                    _this.addToken = _this.addTokenUsingPush;
		                    _this.handlePayload = _this.handlePayloadWithCustom;
		                }
		                else {
		                    _this.addToken = _this.addTokenUsingMemberAccess;
		                    _this.handlePayload = _this.handlePayloadNoCustom;
		                }
		            });
		            _this.TRACE_INIT("Failed Optimization Warnings", function () {
		                var unOptimizedModes = (0, reduce_1.default)(_this.canModeBeOptimized, function (cannotBeOptimized, canBeOptimized, modeName) {
		                    if (canBeOptimized === false) {
		                        cannotBeOptimized.push(modeName);
		                    }
		                    return cannotBeOptimized;
		                }, []);
		                if (config.ensureOptimizations && !(0, isEmpty_1.default)(unOptimizedModes)) {
		                    throw Error("Lexer Modes: < ".concat(unOptimizedModes.join(", "), " > cannot be optimized.\n") +
		                        '\t Disable the "ensureOptimizations" lexer config flag to silently ignore this and run the lexer in an un-optimized mode.\n' +
		                        "\t Or inspect the console log for details on how to resolve these issues.");
		                }
		            });
		            _this.TRACE_INIT("clearRegExpParserCache", function () {
		                (0, reg_exp_parser_1.clearRegExpParserCache)();
		            });
		            _this.TRACE_INIT("toFastProperties", function () {
		                (0, utils_1.toFastProperties)(_this);
		            });
		        });
		    }
		    Lexer.prototype.tokenize = function (text, initialMode) {
		        if (initialMode === void 0) { initialMode = this.defaultMode; }
		        if (!(0, isEmpty_1.default)(this.lexerDefinitionErrors)) {
		            var allErrMessages = (0, map_1.default)(this.lexerDefinitionErrors, function (error) {
		                return error.message;
		            });
		            var allErrMessagesString = allErrMessages.join("-----------------------\n");
		            throw new Error("Unable to Tokenize because Errors detected in definition of Lexer:\n" +
		                allErrMessagesString);
		        }
		        return this.tokenizeInternal(text, initialMode);
		    };
		    // There is quite a bit of duplication between this and "tokenizeInternalLazy"
		    // This is intentional due to performance considerations.
		    // this method also used quite a bit of `!` none null assertions because it is too optimized
		    // for `tsc` to always understand it is "safe"
		    Lexer.prototype.tokenizeInternal = function (text, initialMode) {
		        var _this = this;
		        var i, j, k, matchAltImage, longerAlt, matchedImage, payload, altPayload, imageLength, group, tokType, newToken, errLength, msg, match;
		        var orgText = text;
		        var orgLength = orgText.length;
		        var offset = 0;
		        var matchedTokensIndex = 0;
		        // initializing the tokensArray to the "guessed" size.
		        // guessing too little will still reduce the number of array re-sizes on pushes.
		        // guessing too large (Tested by guessing x4 too large) may cost a bit more of memory
		        // but would still have a faster runtime by avoiding (All but one) array resizing.
		        var guessedNumberOfTokens = this.hasCustom
		            ? 0 // will break custom token pattern APIs the matchedTokens array will contain undefined elements.
		            : Math.floor(text.length / 10);
		        var matchedTokens = new Array(guessedNumberOfTokens);
		        var errors = [];
		        var line = this.trackStartLines ? 1 : undefined;
		        var column = this.trackStartLines ? 1 : undefined;
		        var groups = (0, lexer_1.cloneEmptyGroups)(this.emptyGroups);
		        var trackLines = this.trackStartLines;
		        var lineTerminatorPattern = this.config.lineTerminatorsPattern;
		        var currModePatternsLength = 0;
		        var patternIdxToConfig = [];
		        var currCharCodeToPatternIdxToConfig = [];
		        var modeStack = [];
		        var emptyArray = [];
		        Object.freeze(emptyArray);
		        var getPossiblePatterns;
		        function getPossiblePatternsSlow() {
		            return patternIdxToConfig;
		        }
		        function getPossiblePatternsOptimized(charCode) {
		            var optimizedCharIdx = (0, lexer_1.charCodeToOptimizedIndex)(charCode);
		            var possiblePatterns = currCharCodeToPatternIdxToConfig[optimizedCharIdx];
		            if (possiblePatterns === undefined) {
		                return emptyArray;
		            }
		            else {
		                return possiblePatterns;
		            }
		        }
		        var pop_mode = function (popToken) {
		            // TODO: perhaps avoid this error in the edge case there is no more input?
		            if (modeStack.length === 1 &&
		                // if we have both a POP_MODE and a PUSH_MODE this is in-fact a "transition"
		                // So no error should occur.
		                popToken.tokenType.PUSH_MODE === undefined) {
		                // if we try to pop the last mode there lexer will no longer have ANY mode.
		                // thus the pop is ignored, an error will be created and the lexer will continue parsing in the previous mode.
		                var msg_1 = _this.config.errorMessageProvider.buildUnableToPopLexerModeMessage(popToken);
		                errors.push({
		                    offset: popToken.startOffset,
		                    line: popToken.startLine,
		                    column: popToken.startColumn,
		                    length: popToken.image.length,
		                    message: msg_1
		                });
		            }
		            else {
		                modeStack.pop();
		                var newMode = (0, last_1.default)(modeStack);
		                patternIdxToConfig = _this.patternIdxToConfig[newMode];
		                currCharCodeToPatternIdxToConfig =
		                    _this.charCodeToPatternIdxToConfig[newMode];
		                currModePatternsLength = patternIdxToConfig.length;
		                var modeCanBeOptimized = _this.canModeBeOptimized[newMode] && _this.config.safeMode === false;
		                if (currCharCodeToPatternIdxToConfig && modeCanBeOptimized) {
		                    getPossiblePatterns = getPossiblePatternsOptimized;
		                }
		                else {
		                    getPossiblePatterns = getPossiblePatternsSlow;
		                }
		            }
		        };
		        function push_mode(newMode) {
		            modeStack.push(newMode);
		            currCharCodeToPatternIdxToConfig =
		                this.charCodeToPatternIdxToConfig[newMode];
		            patternIdxToConfig = this.patternIdxToConfig[newMode];
		            currModePatternsLength = patternIdxToConfig.length;
		            currModePatternsLength = patternIdxToConfig.length;
		            var modeCanBeOptimized = this.canModeBeOptimized[newMode] && this.config.safeMode === false;
		            if (currCharCodeToPatternIdxToConfig && modeCanBeOptimized) {
		                getPossiblePatterns = getPossiblePatternsOptimized;
		            }
		            else {
		                getPossiblePatterns = getPossiblePatternsSlow;
		            }
		        }
		        // this pattern seems to avoid a V8 de-optimization, although that de-optimization does not
		        // seem to matter performance wise.
		        push_mode.call(this, initialMode);
		        var currConfig;
		        var recoveryEnabled = this.config.recoveryEnabled;
		        while (offset < orgLength) {
		            matchedImage = null;
		            var nextCharCode = orgText.charCodeAt(offset);
		            var chosenPatternIdxToConfig = getPossiblePatterns(nextCharCode);
		            var chosenPatternsLength = chosenPatternIdxToConfig.length;
		            for (i = 0; i < chosenPatternsLength; i++) {
		                currConfig = chosenPatternIdxToConfig[i];
		                var currPattern = currConfig.pattern;
		                payload = null;
		                // manually in-lined because > 600 chars won't be in-lined in V8
		                var singleCharCode = currConfig.short;
		                if (singleCharCode !== false) {
		                    if (nextCharCode === singleCharCode) {
		                        // single character string
		                        matchedImage = currPattern;
		                    }
		                }
		                else if (currConfig.isCustom === true) {
		                    match = currPattern.exec(orgText, offset, matchedTokens, groups);
		                    if (match !== null) {
		                        matchedImage = match[0];
		                        if (match.payload !== undefined) {
		                            payload = match.payload;
		                        }
		                    }
		                    else {
		                        matchedImage = null;
		                    }
		                }
		                else {
		                    this.updateLastIndex(currPattern, offset);
		                    matchedImage = this.match(currPattern, text, offset);
		                }
		                if (matchedImage !== null) {
		                    // even though this pattern matched we must try a another longer alternative.
		                    // this can be used to prioritize keywords over identifiers
		                    longerAlt = currConfig.longerAlt;
		                    if (longerAlt !== undefined) {
		                        // TODO: micro optimize, avoid extra prop access
		                        // by saving/linking longerAlt on the original config?
		                        var longerAltLength = longerAlt.length;
		                        for (k = 0; k < longerAltLength; k++) {
		                            var longerAltConfig = patternIdxToConfig[longerAlt[k]];
		                            var longerAltPattern = longerAltConfig.pattern;
		                            altPayload = null;
		                            // single Char can never be a longer alt so no need to test it.
		                            // manually in-lined because > 600 chars won't be in-lined in V8
		                            if (longerAltConfig.isCustom === true) {
		                                match = longerAltPattern.exec(orgText, offset, matchedTokens, groups);
		                                if (match !== null) {
		                                    matchAltImage = match[0];
		                                    if (match.payload !== undefined) {
		                                        altPayload = match.payload;
		                                    }
		                                }
		                                else {
		                                    matchAltImage = null;
		                                }
		                            }
		                            else {
		                                this.updateLastIndex(longerAltPattern, offset);
		                                matchAltImage = this.match(longerAltPattern, text, offset);
		                            }
		                            if (matchAltImage && matchAltImage.length > matchedImage.length) {
		                                matchedImage = matchAltImage;
		                                payload = altPayload;
		                                currConfig = longerAltConfig;
		                                // Exit the loop early after matching one of the longer alternatives
		                                // The first matched alternative takes precedence
		                                break;
		                            }
		                        }
		                    }
		                    break;
		                }
		            }
		            // successful match
		            if (matchedImage !== null) {
		                imageLength = matchedImage.length;
		                group = currConfig.group;
		                if (group !== undefined) {
		                    tokType = currConfig.tokenTypeIdx;
		                    // TODO: "offset + imageLength" and the new column may be computed twice in case of "full" location information inside
		                    // createFullToken method
		                    newToken = this.createTokenInstance(matchedImage, offset, tokType, currConfig.tokenType, line, column, imageLength);
		                    this.handlePayload(newToken, payload);
		                    // TODO: optimize NOOP in case there are no special groups?
		                    if (group === false) {
		                        matchedTokensIndex = this.addToken(matchedTokens, matchedTokensIndex, newToken);
		                    }
		                    else {
		                        groups[group].push(newToken);
		                    }
		                }
		                text = this.chopInput(text, imageLength);
		                offset = offset + imageLength;
		                // TODO: with newlines the column may be assigned twice
		                column = this.computeNewColumn(column, imageLength);
		                if (trackLines === true && currConfig.canLineTerminator === true) {
		                    var numOfLTsInMatch = 0;
		                    var foundTerminator = void 0;
		                    var lastLTEndOffset = void 0;
		                    lineTerminatorPattern.lastIndex = 0;
		                    do {
		                        foundTerminator = lineTerminatorPattern.test(matchedImage);
		                        if (foundTerminator === true) {
		                            lastLTEndOffset = lineTerminatorPattern.lastIndex - 1;
		                            numOfLTsInMatch++;
		                        }
		                    } while (foundTerminator === true);
		                    if (numOfLTsInMatch !== 0) {
		                        line = line + numOfLTsInMatch;
		                        column = imageLength - lastLTEndOffset;
		                        this.updateTokenEndLineColumnLocation(newToken, group, lastLTEndOffset, numOfLTsInMatch, line, column, imageLength);
		                    }
		                }
		                // will be NOOP if no modes present
		                this.handleModes(currConfig, pop_mode, push_mode, newToken);
		            }
		            else {
		                // error recovery, drop characters until we identify a valid token's start point
		                var errorStartOffset = offset;
		                var errorLine = line;
		                var errorColumn = column;
		                var foundResyncPoint = recoveryEnabled === false;
		                while (foundResyncPoint === false && offset < orgLength) {
		                    // Identity Func (when sticky flag is enabled)
		                    text = this.chopInput(text, 1);
		                    offset++;
		                    for (j = 0; j < currModePatternsLength; j++) {
		                        var currConfig_1 = patternIdxToConfig[j];
		                        var currPattern = currConfig_1.pattern;
		                        // manually in-lined because > 600 chars won't be in-lined in V8
		                        var singleCharCode = currConfig_1.short;
		                        if (singleCharCode !== false) {
		                            if (orgText.charCodeAt(offset) === singleCharCode) {
		                                // single character string
		                                foundResyncPoint = true;
		                            }
		                        }
		                        else if (currConfig_1.isCustom === true) {
		                            foundResyncPoint =
		                                currPattern.exec(orgText, offset, matchedTokens, groups) !== null;
		                        }
		                        else {
		                            this.updateLastIndex(currPattern, offset);
		                            foundResyncPoint = currPattern.exec(text) !== null;
		                        }
		                        if (foundResyncPoint === true) {
		                            break;
		                        }
		                    }
		                }
		                errLength = offset - errorStartOffset;
		                // at this point we either re-synced or reached the end of the input text
		                msg = this.config.errorMessageProvider.buildUnexpectedCharactersMessage(orgText, errorStartOffset, errLength, errorLine, errorColumn);
		                errors.push({
		                    offset: errorStartOffset,
		                    line: errorLine,
		                    column: errorColumn,
		                    length: errLength,
		                    message: msg
		                });
		                if (recoveryEnabled === false) {
		                    break;
		                }
		            }
		        }
		        // if we do have custom patterns which push directly into the
		        // TODO: custom tokens should not push directly??
		        if (!this.hasCustom) {
		            // if we guessed a too large size for the tokens array this will shrink it to the right size.
		            matchedTokens.length = matchedTokensIndex;
		        }
		        return {
		            tokens: matchedTokens,
		            groups: groups,
		            errors: errors
		        };
		    };
		    Lexer.prototype.handleModes = function (config, pop_mode, push_mode, newToken) {
		        if (config.pop === true) {
		            // need to save the PUSH_MODE property as if the mode is popped
		            // patternIdxToPopMode is updated to reflect the new mode after popping the stack
		            var pushMode = config.push;
		            pop_mode(newToken);
		            if (pushMode !== undefined) {
		                push_mode.call(this, pushMode);
		            }
		        }
		        else if (config.push !== undefined) {
		            push_mode.call(this, config.push);
		        }
		    };
		    Lexer.prototype.chopInput = function (text, length) {
		        return text.substring(length);
		    };
		    Lexer.prototype.updateLastIndex = function (regExp, newLastIndex) {
		        regExp.lastIndex = newLastIndex;
		    };
		    // TODO: decrease this under 600 characters? inspect stripping comments option in TSC compiler
		    Lexer.prototype.updateTokenEndLineColumnLocation = function (newToken, group, lastLTIdx, numOfLTsInMatch, line, column, imageLength) {
		        var lastCharIsLT, fixForEndingInLT;
		        if (group !== undefined) {
		            // a none skipped multi line Token, need to update endLine/endColumn
		            lastCharIsLT = lastLTIdx === imageLength - 1;
		            fixForEndingInLT = lastCharIsLT ? -1 : 0;
		            if (!(numOfLTsInMatch === 1 && lastCharIsLT === true)) {
		                // if a token ends in a LT that last LT only affects the line numbering of following Tokens
		                newToken.endLine = line + fixForEndingInLT;
		                // the last LT in a token does not affect the endColumn either as the [columnStart ... columnEnd)
		                // inclusive to exclusive range.
		                newToken.endColumn = column - 1 + -fixForEndingInLT;
		            }
		            // else single LT in the last character of a token, no need to modify the endLine/EndColumn
		        }
		    };
		    Lexer.prototype.computeNewColumn = function (oldColumn, imageLength) {
		        return oldColumn + imageLength;
		    };
		    Lexer.prototype.createOffsetOnlyToken = function (image, startOffset, tokenTypeIdx, tokenType) {
		        return {
		            image: image,
		            startOffset: startOffset,
		            tokenTypeIdx: tokenTypeIdx,
		            tokenType: tokenType
		        };
		    };
		    Lexer.prototype.createStartOnlyToken = function (image, startOffset, tokenTypeIdx, tokenType, startLine, startColumn) {
		        return {
		            image: image,
		            startOffset: startOffset,
		            startLine: startLine,
		            startColumn: startColumn,
		            tokenTypeIdx: tokenTypeIdx,
		            tokenType: tokenType
		        };
		    };
		    Lexer.prototype.createFullToken = function (image, startOffset, tokenTypeIdx, tokenType, startLine, startColumn, imageLength) {
		        return {
		            image: image,
		            startOffset: startOffset,
		            endOffset: startOffset + imageLength - 1,
		            startLine: startLine,
		            endLine: startLine,
		            startColumn: startColumn,
		            endColumn: startColumn + imageLength - 1,
		            tokenTypeIdx: tokenTypeIdx,
		            tokenType: tokenType
		        };
		    };
		    Lexer.prototype.addTokenUsingPush = function (tokenVector, index, tokenToAdd) {
		        tokenVector.push(tokenToAdd);
		        return index;
		    };
		    Lexer.prototype.addTokenUsingMemberAccess = function (tokenVector, index, tokenToAdd) {
		        tokenVector[index] = tokenToAdd;
		        index++;
		        return index;
		    };
		    Lexer.prototype.handlePayloadNoCustom = function (token, payload) { };
		    Lexer.prototype.handlePayloadWithCustom = function (token, payload) {
		        if (payload !== null) {
		            token.payload = payload;
		        }
		    };
		    Lexer.prototype.matchWithTest = function (pattern, text, offset) {
		        var found = pattern.test(text);
		        if (found === true) {
		            return text.substring(offset, pattern.lastIndex);
		        }
		        return null;
		    };
		    Lexer.prototype.matchWithExec = function (pattern, text) {
		        var regExpArray = pattern.exec(text);
		        return regExpArray !== null ? regExpArray[0] : null;
		    };
		    Lexer.SKIPPED = "This marks a skipped Token pattern, this means each token identified by it will" +
		        "be consumed and then thrown into oblivion, this can be used to for example to completely ignore whitespace.";
		    Lexer.NA = /NOT_APPLICABLE/;
		    return Lexer;
		}());
		exports.Lexer = Lexer;
		
	} (lexer_public));
	return lexer_public;
}

var hasRequiredTokens_public;

function requireTokens_public () {
	if (hasRequiredTokens_public) return tokens_public;
	hasRequiredTokens_public = 1;
	(function (exports) {
		var __importDefault = (tokens_public && tokens_public.__importDefault) || function (mod) {
		    return (mod && mod.__esModule) ? mod : { "default": mod };
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.tokenMatcher = exports.createTokenInstance = exports.EOF = exports.createToken = exports.hasTokenLabel = exports.tokenName = exports.tokenLabel = void 0;
		var isString_1 = __importDefault(requireIsString());
		var has_1 = __importDefault(requireHas());
		var isUndefined_1 = __importDefault(requireIsUndefined());
		var lexer_public_1 = requireLexer_public();
		var tokens_1 = requireTokens();
		function tokenLabel(tokType) {
		    if (hasTokenLabel(tokType)) {
		        return tokType.LABEL;
		    }
		    else {
		        return tokType.name;
		    }
		}
		exports.tokenLabel = tokenLabel;
		function tokenName(tokType) {
		    return tokType.name;
		}
		exports.tokenName = tokenName;
		function hasTokenLabel(obj) {
		    return (0, isString_1.default)(obj.LABEL) && obj.LABEL !== "";
		}
		exports.hasTokenLabel = hasTokenLabel;
		var PARENT = "parent";
		var CATEGORIES = "categories";
		var LABEL = "label";
		var GROUP = "group";
		var PUSH_MODE = "push_mode";
		var POP_MODE = "pop_mode";
		var LONGER_ALT = "longer_alt";
		var LINE_BREAKS = "line_breaks";
		var START_CHARS_HINT = "start_chars_hint";
		function createToken(config) {
		    return createTokenInternal(config);
		}
		exports.createToken = createToken;
		function createTokenInternal(config) {
		    var pattern = config.pattern;
		    var tokenType = {};
		    tokenType.name = config.name;
		    if (!(0, isUndefined_1.default)(pattern)) {
		        tokenType.PATTERN = pattern;
		    }
		    if ((0, has_1.default)(config, PARENT)) {
		        throw ("The parent property is no longer supported.\n" +
		            "See: https://github.com/chevrotain/chevrotain/issues/564#issuecomment-349062346 for details.");
		    }
		    if ((0, has_1.default)(config, CATEGORIES)) {
		        // casting to ANY as this will be fixed inside `augmentTokenTypes``
		        tokenType.CATEGORIES = config[CATEGORIES];
		    }
		    (0, tokens_1.augmentTokenTypes)([tokenType]);
		    if ((0, has_1.default)(config, LABEL)) {
		        tokenType.LABEL = config[LABEL];
		    }
		    if ((0, has_1.default)(config, GROUP)) {
		        tokenType.GROUP = config[GROUP];
		    }
		    if ((0, has_1.default)(config, POP_MODE)) {
		        tokenType.POP_MODE = config[POP_MODE];
		    }
		    if ((0, has_1.default)(config, PUSH_MODE)) {
		        tokenType.PUSH_MODE = config[PUSH_MODE];
		    }
		    if ((0, has_1.default)(config, LONGER_ALT)) {
		        tokenType.LONGER_ALT = config[LONGER_ALT];
		    }
		    if ((0, has_1.default)(config, LINE_BREAKS)) {
		        tokenType.LINE_BREAKS = config[LINE_BREAKS];
		    }
		    if ((0, has_1.default)(config, START_CHARS_HINT)) {
		        tokenType.START_CHARS_HINT = config[START_CHARS_HINT];
		    }
		    return tokenType;
		}
		exports.EOF = createToken({ name: "EOF", pattern: lexer_public_1.Lexer.NA });
		(0, tokens_1.augmentTokenTypes)([exports.EOF]);
		function createTokenInstance(tokType, image, startOffset, endOffset, startLine, endLine, startColumn, endColumn) {
		    return {
		        image: image,
		        startOffset: startOffset,
		        endOffset: endOffset,
		        startLine: startLine,
		        endLine: endLine,
		        startColumn: startColumn,
		        endColumn: endColumn,
		        tokenTypeIdx: tokType.tokenTypeIdx,
		        tokenType: tokType
		    };
		}
		exports.createTokenInstance = createTokenInstance;
		function tokenMatcher(token, tokType) {
		    return (0, tokens_1.tokenStructuredMatcher)(token, tokType);
		}
		exports.tokenMatcher = tokenMatcher;
		
	} (tokens_public));
	return tokens_public;
}

var errors_public = {};

var hasRequiredErrors_public;

function requireErrors_public () {
	if (hasRequiredErrors_public) return errors_public;
	hasRequiredErrors_public = 1;
	(function (exports) {
		var __importDefault = (errors_public && errors_public.__importDefault) || function (mod) {
		    return (mod && mod.__esModule) ? mod : { "default": mod };
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.defaultGrammarValidatorErrorProvider = exports.defaultGrammarResolverErrorProvider = exports.defaultParserErrorProvider = void 0;
		var tokens_public_1 = requireTokens_public();
		var first_1 = __importDefault(requireFirst());
		var map_1 = __importDefault(requireMap());
		var reduce_1 = __importDefault(requireReduce());
		var gast_1 = requireApi$2();
		var gast_2 = requireApi$2();
		exports.defaultParserErrorProvider = {
		    buildMismatchTokenMessage: function (_a) {
		        var expected = _a.expected, actual = _a.actual; _a.previous; _a.ruleName;
		        var hasLabel = (0, tokens_public_1.hasTokenLabel)(expected);
		        var expectedMsg = hasLabel
		            ? "--> ".concat((0, tokens_public_1.tokenLabel)(expected), " <--")
		            : "token of type --> ".concat(expected.name, " <--");
		        var msg = "Expecting ".concat(expectedMsg, " but found --> '").concat(actual.image, "' <--");
		        return msg;
		    },
		    buildNotAllInputParsedMessage: function (_a) {
		        var firstRedundant = _a.firstRedundant; _a.ruleName;
		        return "Redundant input, expecting EOF but found: " + firstRedundant.image;
		    },
		    buildNoViableAltMessage: function (_a) {
		        var expectedPathsPerAlt = _a.expectedPathsPerAlt, actual = _a.actual; _a.previous; var customUserDescription = _a.customUserDescription; _a.ruleName;
		        var errPrefix = "Expecting: ";
		        // TODO: issue: No Viable Alternative Error may have incomplete details. #502
		        var actualText = (0, first_1.default)(actual).image;
		        var errSuffix = "\nbut found: '" + actualText + "'";
		        if (customUserDescription) {
		            return errPrefix + customUserDescription + errSuffix;
		        }
		        else {
		            var allLookAheadPaths = (0, reduce_1.default)(expectedPathsPerAlt, function (result, currAltPaths) { return result.concat(currAltPaths); }, []);
		            var nextValidTokenSequences = (0, map_1.default)(allLookAheadPaths, function (currPath) {
		                return "[".concat((0, map_1.default)(currPath, function (currTokenType) { return (0, tokens_public_1.tokenLabel)(currTokenType); }).join(", "), "]");
		            });
		            var nextValidSequenceItems = (0, map_1.default)(nextValidTokenSequences, function (itemMsg, idx) { return "  ".concat(idx + 1, ". ").concat(itemMsg); });
		            var calculatedDescription = "one of these possible Token sequences:\n".concat(nextValidSequenceItems.join("\n"));
		            return errPrefix + calculatedDescription + errSuffix;
		        }
		    },
		    buildEarlyExitMessage: function (_a) {
		        var expectedIterationPaths = _a.expectedIterationPaths, actual = _a.actual, customUserDescription = _a.customUserDescription; _a.ruleName;
		        var errPrefix = "Expecting: ";
		        // TODO: issue: No Viable Alternative Error may have incomplete details. #502
		        var actualText = (0, first_1.default)(actual).image;
		        var errSuffix = "\nbut found: '" + actualText + "'";
		        if (customUserDescription) {
		            return errPrefix + customUserDescription + errSuffix;
		        }
		        else {
		            var nextValidTokenSequences = (0, map_1.default)(expectedIterationPaths, function (currPath) {
		                return "[".concat((0, map_1.default)(currPath, function (currTokenType) { return (0, tokens_public_1.tokenLabel)(currTokenType); }).join(","), "]");
		            });
		            var calculatedDescription = "expecting at least one iteration which starts with one of these possible Token sequences::\n  " +
		                "<".concat(nextValidTokenSequences.join(" ,"), ">");
		            return errPrefix + calculatedDescription + errSuffix;
		        }
		    }
		};
		Object.freeze(exports.defaultParserErrorProvider);
		exports.defaultGrammarResolverErrorProvider = {
		    buildRuleNotFoundError: function (topLevelRule, undefinedRule) {
		        var msg = "Invalid grammar, reference to a rule which is not defined: ->" +
		            undefinedRule.nonTerminalName +
		            "<-\n" +
		            "inside top level rule: ->" +
		            topLevelRule.name +
		            "<-";
		        return msg;
		    }
		};
		exports.defaultGrammarValidatorErrorProvider = {
		    buildDuplicateFoundError: function (topLevelRule, duplicateProds) {
		        function getExtraProductionArgument(prod) {
		            if (prod instanceof gast_1.Terminal) {
		                return prod.terminalType.name;
		            }
		            else if (prod instanceof gast_1.NonTerminal) {
		                return prod.nonTerminalName;
		            }
		            else {
		                return "";
		            }
		        }
		        var topLevelName = topLevelRule.name;
		        var duplicateProd = (0, first_1.default)(duplicateProds);
		        var index = duplicateProd.idx;
		        var dslName = (0, gast_2.getProductionDslName)(duplicateProd);
		        var extraArgument = getExtraProductionArgument(duplicateProd);
		        var hasExplicitIndex = index > 0;
		        var msg = "->".concat(dslName).concat(hasExplicitIndex ? index : "", "<- ").concat(extraArgument ? "with argument: ->".concat(extraArgument, "<-") : "", "\n                  appears more than once (").concat(duplicateProds.length, " times) in the top level rule: ->").concat(topLevelName, "<-.                  \n                  For further details see: https://chevrotain.io/docs/FAQ.html#NUMERICAL_SUFFIXES \n                  ");
		        // white space trimming time! better to trim afterwards as it allows to use WELL formatted multi line template strings...
		        msg = msg.replace(/[ \t]+/g, " ");
		        msg = msg.replace(/\s\s+/g, "\n");
		        return msg;
		    },
		    buildNamespaceConflictError: function (rule) {
		        var errMsg = "Namespace conflict found in grammar.\n" +
		            "The grammar has both a Terminal(Token) and a Non-Terminal(Rule) named: <".concat(rule.name, ">.\n") +
		            "To resolve this make sure each Terminal and Non-Terminal names are unique\n" +
		            "This is easy to accomplish by using the convention that Terminal names start with an uppercase letter\n" +
		            "and Non-Terminal names start with a lower case letter.";
		        return errMsg;
		    },
		    buildAlternationPrefixAmbiguityError: function (options) {
		        var pathMsg = (0, map_1.default)(options.prefixPath, function (currTok) {
		            return (0, tokens_public_1.tokenLabel)(currTok);
		        }).join(", ");
		        var occurrence = options.alternation.idx === 0 ? "" : options.alternation.idx;
		        var errMsg = "Ambiguous alternatives: <".concat(options.ambiguityIndices.join(" ,"), "> due to common lookahead prefix\n") +
		            "in <OR".concat(occurrence, "> inside <").concat(options.topLevelRule.name, "> Rule,\n") +
		            "<".concat(pathMsg, "> may appears as a prefix path in all these alternatives.\n") +
		            "See: https://chevrotain.io/docs/guide/resolving_grammar_errors.html#COMMON_PREFIX\n" +
		            "For Further details.";
		        return errMsg;
		    },
		    buildAlternationAmbiguityError: function (options) {
		        var pathMsg = (0, map_1.default)(options.prefixPath, function (currtok) {
		            return (0, tokens_public_1.tokenLabel)(currtok);
		        }).join(", ");
		        var occurrence = options.alternation.idx === 0 ? "" : options.alternation.idx;
		        var currMessage = "Ambiguous Alternatives Detected: <".concat(options.ambiguityIndices.join(" ,"), "> in <OR").concat(occurrence, ">") +
		            " inside <".concat(options.topLevelRule.name, "> Rule,\n") +
		            "<".concat(pathMsg, "> may appears as a prefix path in all these alternatives.\n");
		        currMessage =
		            currMessage +
		                "See: https://chevrotain.io/docs/guide/resolving_grammar_errors.html#AMBIGUOUS_ALTERNATIVES\n" +
		                "For Further details.";
		        return currMessage;
		    },
		    buildEmptyRepetitionError: function (options) {
		        var dslName = (0, gast_2.getProductionDslName)(options.repetition);
		        if (options.repetition.idx !== 0) {
		            dslName += options.repetition.idx;
		        }
		        var errMsg = "The repetition <".concat(dslName, "> within Rule <").concat(options.topLevelRule.name, "> can never consume any tokens.\n") +
		            "This could lead to an infinite loop.";
		        return errMsg;
		    },
		    // TODO: remove - `errors_public` from nyc.config.js exclude
		    //       once this method is fully removed from this file
		    buildTokenNameError: function (options) {
		        /* istanbul ignore next */
		        return "deprecated";
		    },
		    buildEmptyAlternationError: function (options) {
		        var errMsg = "Ambiguous empty alternative: <".concat(options.emptyChoiceIdx + 1, ">") +
		            " in <OR".concat(options.alternation.idx, "> inside <").concat(options.topLevelRule.name, "> Rule.\n") +
		            "Only the last alternative may be an empty alternative.";
		        return errMsg;
		    },
		    buildTooManyAlternativesError: function (options) {
		        var errMsg = "An Alternation cannot have more than 256 alternatives:\n" +
		            "<OR".concat(options.alternation.idx, "> inside <").concat(options.topLevelRule.name, "> Rule.\n has ").concat(options.alternation.definition.length + 1, " alternatives.");
		        return errMsg;
		    },
		    buildLeftRecursionError: function (options) {
		        var ruleName = options.topLevelRule.name;
		        var pathNames = (0, map_1.default)(options.leftRecursionPath, function (currRule) { return currRule.name; });
		        var leftRecursivePath = "".concat(ruleName, " --> ").concat(pathNames
		            .concat([ruleName])
		            .join(" --> "));
		        var errMsg = "Left Recursion found in grammar.\n" +
		            "rule: <".concat(ruleName, "> can be invoked from itself (directly or indirectly)\n") +
		            "without consuming any Tokens. The grammar path that causes this is: \n ".concat(leftRecursivePath, "\n") +
		            " To fix this refactor your grammar to remove the left recursion.\n" +
		            "see: https://en.wikipedia.org/wiki/LL_parser#Left_factoring.";
		        return errMsg;
		    },
		    // TODO: remove - `errors_public` from nyc.config.js exclude
		    //       once this method is fully removed from this file
		    buildInvalidRuleNameError: function (options) {
		        /* istanbul ignore next */
		        return "deprecated";
		    },
		    buildDuplicateRuleNameError: function (options) {
		        var ruleName;
		        if (options.topLevelRule instanceof gast_1.Rule) {
		            ruleName = options.topLevelRule.name;
		        }
		        else {
		            ruleName = options.topLevelRule;
		        }
		        var errMsg = "Duplicate definition, rule: ->".concat(ruleName, "<- is already defined in the grammar: ->").concat(options.grammarName, "<-");
		        return errMsg;
		    }
		};
		
	} (errors_public));
	return errors_public;
}

var gast_resolver_public = {};

var resolver = {};

var hasRequiredResolver;

function requireResolver () {
	if (hasRequiredResolver) return resolver;
	hasRequiredResolver = 1;
	var __extends = (resolver && resolver.__extends) || (function () {
	    var extendStatics = function (d, b) {
	        extendStatics = Object.setPrototypeOf ||
	            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
	            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
	        return extendStatics(d, b);
	    };
	    return function (d, b) {
	        if (typeof b !== "function" && b !== null)
	            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
	        extendStatics(d, b);
	        function __() { this.constructor = d; }
	        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
	    };
	})();
	var __importDefault = (resolver && resolver.__importDefault) || function (mod) {
	    return (mod && mod.__esModule) ? mod : { "default": mod };
	};
	Object.defineProperty(resolver, "__esModule", { value: true });
	resolver.GastRefResolverVisitor = resolver.resolveGrammar = void 0;
	var parser_1 = requireParser();
	var forEach_1 = __importDefault(requireForEach());
	var values_1 = __importDefault(requireValues());
	var gast_1 = requireApi$2();
	function resolveGrammar(topLevels, errMsgProvider) {
	    var refResolver = new GastRefResolverVisitor(topLevels, errMsgProvider);
	    refResolver.resolveRefs();
	    return refResolver.errors;
	}
	resolver.resolveGrammar = resolveGrammar;
	var GastRefResolverVisitor = /** @class */ (function (_super) {
	    __extends(GastRefResolverVisitor, _super);
	    function GastRefResolverVisitor(nameToTopRule, errMsgProvider) {
	        var _this = _super.call(this) || this;
	        _this.nameToTopRule = nameToTopRule;
	        _this.errMsgProvider = errMsgProvider;
	        _this.errors = [];
	        return _this;
	    }
	    GastRefResolverVisitor.prototype.resolveRefs = function () {
	        var _this = this;
	        (0, forEach_1.default)((0, values_1.default)(this.nameToTopRule), function (prod) {
	            _this.currTopLevel = prod;
	            prod.accept(_this);
	        });
	    };
	    GastRefResolverVisitor.prototype.visitNonTerminal = function (node) {
	        var ref = this.nameToTopRule[node.nonTerminalName];
	        if (!ref) {
	            var msg = this.errMsgProvider.buildRuleNotFoundError(this.currTopLevel, node);
	            this.errors.push({
	                message: msg,
	                type: parser_1.ParserDefinitionErrorType.UNRESOLVED_SUBRULE_REF,
	                ruleName: this.currTopLevel.name,
	                unresolvedRefName: node.nonTerminalName
	            });
	        }
	        else {
	            node.referencedRule = ref;
	        }
	    };
	    return GastRefResolverVisitor;
	}(gast_1.GAstVisitor));
	resolver.GastRefResolverVisitor = GastRefResolverVisitor;
	
	return resolver;
}

var checks = {};

/**
 * A specialized version of `baseAggregator` for arrays.
 *
 * @private
 * @param {Array} [array] The array to iterate over.
 * @param {Function} setter The function to set `accumulator` values.
 * @param {Function} iteratee The iteratee to transform keys.
 * @param {Object} accumulator The initial aggregated object.
 * @returns {Function} Returns `accumulator`.
 */

var _arrayAggregator;
var hasRequired_arrayAggregator;

function require_arrayAggregator () {
	if (hasRequired_arrayAggregator) return _arrayAggregator;
	hasRequired_arrayAggregator = 1;
	function arrayAggregator(array, setter, iteratee, accumulator) {
	  var index = -1,
	      length = array == null ? 0 : array.length;

	  while (++index < length) {
	    var value = array[index];
	    setter(accumulator, value, iteratee(value), array);
	  }
	  return accumulator;
	}

	_arrayAggregator = arrayAggregator;
	return _arrayAggregator;
}

var _baseAggregator;
var hasRequired_baseAggregator;

function require_baseAggregator () {
	if (hasRequired_baseAggregator) return _baseAggregator;
	hasRequired_baseAggregator = 1;
	var baseEach = require_baseEach();

	/**
	 * Aggregates elements of `collection` on `accumulator` with keys transformed
	 * by `iteratee` and values set by `setter`.
	 *
	 * @private
	 * @param {Array|Object} collection The collection to iterate over.
	 * @param {Function} setter The function to set `accumulator` values.
	 * @param {Function} iteratee The iteratee to transform keys.
	 * @param {Object} accumulator The initial aggregated object.
	 * @returns {Function} Returns `accumulator`.
	 */
	function baseAggregator(collection, setter, iteratee, accumulator) {
	  baseEach(collection, function(value, key, collection) {
	    setter(accumulator, value, iteratee(value), collection);
	  });
	  return accumulator;
	}

	_baseAggregator = baseAggregator;
	return _baseAggregator;
}

var _createAggregator;
var hasRequired_createAggregator;

function require_createAggregator () {
	if (hasRequired_createAggregator) return _createAggregator;
	hasRequired_createAggregator = 1;
	var arrayAggregator = require_arrayAggregator(),
	    baseAggregator = require_baseAggregator(),
	    baseIteratee = require_baseIteratee(),
	    isArray = requireIsArray();

	/**
	 * Creates a function like `_.groupBy`.
	 *
	 * @private
	 * @param {Function} setter The function to set accumulator values.
	 * @param {Function} [initializer] The accumulator object initializer.
	 * @returns {Function} Returns the new aggregator function.
	 */
	function createAggregator(setter, initializer) {
	  return function(collection, iteratee) {
	    var func = isArray(collection) ? arrayAggregator : baseAggregator,
	        accumulator = initializer ? initializer() : {};

	    return func(collection, setter, baseIteratee(iteratee, 2), accumulator);
	  };
	}

	_createAggregator = createAggregator;
	return _createAggregator;
}

var groupBy_1;
var hasRequiredGroupBy;

function requireGroupBy () {
	if (hasRequiredGroupBy) return groupBy_1;
	hasRequiredGroupBy = 1;
	var baseAssignValue = require_baseAssignValue(),
	    createAggregator = require_createAggregator();

	/** Used for built-in method references. */
	var objectProto = Object.prototype;

	/** Used to check objects for own properties. */
	var hasOwnProperty = objectProto.hasOwnProperty;

	/**
	 * Creates an object composed of keys generated from the results of running
	 * each element of `collection` thru `iteratee`. The order of grouped values
	 * is determined by the order they occur in `collection`. The corresponding
	 * value of each key is an array of elements responsible for generating the
	 * key. The iteratee is invoked with one argument: (value).
	 *
	 * @static
	 * @memberOf _
	 * @since 0.1.0
	 * @category Collection
	 * @param {Array|Object} collection The collection to iterate over.
	 * @param {Function} [iteratee=_.identity] The iteratee to transform keys.
	 * @returns {Object} Returns the composed aggregate object.
	 * @example
	 *
	 * _.groupBy([6.1, 4.2, 6.3], Math.floor);
	 * // => { '4': [4.2], '6': [6.1, 6.3] }
	 *
	 * // The `_.property` iteratee shorthand.
	 * _.groupBy(['one', 'two', 'three'], 'length');
	 * // => { '3': ['one', 'two'], '5': ['three'] }
	 */
	var groupBy = createAggregator(function(result, value, key) {
	  if (hasOwnProperty.call(result, key)) {
	    result[key].push(value);
	  } else {
	    baseAssignValue(result, key, [value]);
	  }
	});

	groupBy_1 = groupBy;
	return groupBy_1;
}

var flatMap_1;
var hasRequiredFlatMap;

function requireFlatMap () {
	if (hasRequiredFlatMap) return flatMap_1;
	hasRequiredFlatMap = 1;
	var baseFlatten = require_baseFlatten(),
	    map = requireMap();

	/**
	 * Creates a flattened array of values by running each element in `collection`
	 * thru `iteratee` and flattening the mapped results. The iteratee is invoked
	 * with three arguments: (value, index|key, collection).
	 *
	 * @static
	 * @memberOf _
	 * @since 4.0.0
	 * @category Collection
	 * @param {Array|Object} collection The collection to iterate over.
	 * @param {Function} [iteratee=_.identity] The function invoked per iteration.
	 * @returns {Array} Returns the new flattened array.
	 * @example
	 *
	 * function duplicate(n) {
	 *   return [n, n];
	 * }
	 *
	 * _.flatMap([1, 2], duplicate);
	 * // => [1, 1, 2, 2]
	 */
	function flatMap(collection, iteratee) {
	  return baseFlatten(map(collection, iteratee), 1);
	}

	flatMap_1 = flatMap;
	return flatMap_1;
}

var lookahead = {};

var interpreter = {};

var dropRight_1;
var hasRequiredDropRight;

function requireDropRight () {
	if (hasRequiredDropRight) return dropRight_1;
	hasRequiredDropRight = 1;
	var baseSlice = require_baseSlice(),
	    toInteger = requireToInteger();

	/**
	 * Creates a slice of `array` with `n` elements dropped from the end.
	 *
	 * @static
	 * @memberOf _
	 * @since 3.0.0
	 * @category Array
	 * @param {Array} array The array to query.
	 * @param {number} [n=1] The number of elements to drop.
	 * @param- {Object} [guard] Enables use as an iteratee for methods like `_.map`.
	 * @returns {Array} Returns the slice of `array`.
	 * @example
	 *
	 * _.dropRight([1, 2, 3]);
	 * // => [1, 2]
	 *
	 * _.dropRight([1, 2, 3], 2);
	 * // => [1]
	 *
	 * _.dropRight([1, 2, 3], 5);
	 * // => []
	 *
	 * _.dropRight([1, 2, 3], 0);
	 * // => [1, 2, 3]
	 */
	function dropRight(array, n, guard) {
	  var length = array == null ? 0 : array.length;
	  if (!length) {
	    return [];
	  }
	  n = (guard || n === undefined) ? 1 : toInteger(n);
	  n = length - n;
	  return baseSlice(array, 0, n < 0 ? 0 : n);
	}

	dropRight_1 = dropRight;
	return dropRight_1;
}

var hasRequiredInterpreter;

function requireInterpreter () {
	if (hasRequiredInterpreter) return interpreter;
	hasRequiredInterpreter = 1;
	var __extends = (interpreter && interpreter.__extends) || (function () {
	    var extendStatics = function (d, b) {
	        extendStatics = Object.setPrototypeOf ||
	            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
	            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
	        return extendStatics(d, b);
	    };
	    return function (d, b) {
	        if (typeof b !== "function" && b !== null)
	            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
	        extendStatics(d, b);
	        function __() { this.constructor = d; }
	        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
	    };
	})();
	var __importDefault = (interpreter && interpreter.__importDefault) || function (mod) {
	    return (mod && mod.__esModule) ? mod : { "default": mod };
	};
	Object.defineProperty(interpreter, "__esModule", { value: true });
	interpreter.nextPossibleTokensAfter = interpreter.possiblePathsFrom = interpreter.NextTerminalAfterAtLeastOneSepWalker = interpreter.NextTerminalAfterAtLeastOneWalker = interpreter.NextTerminalAfterManySepWalker = interpreter.NextTerminalAfterManyWalker = interpreter.AbstractNextTerminalAfterProductionWalker = interpreter.NextAfterTokenWalker = interpreter.AbstractNextPossibleTokensWalker = void 0;
	var rest_1 = requireRest();
	var first_1 = __importDefault(requireFirst());
	var isEmpty_1 = __importDefault(requireIsEmpty());
	var dropRight_1 = __importDefault(requireDropRight());
	var drop_1 = __importDefault(requireDrop());
	var last_1 = __importDefault(requireLast());
	var forEach_1 = __importDefault(requireForEach());
	var clone_1 = __importDefault(requireClone());
	var first_2 = requireFirst$1();
	var gast_1 = requireApi$2();
	var AbstractNextPossibleTokensWalker = /** @class */ (function (_super) {
	    __extends(AbstractNextPossibleTokensWalker, _super);
	    function AbstractNextPossibleTokensWalker(topProd, path) {
	        var _this = _super.call(this) || this;
	        _this.topProd = topProd;
	        _this.path = path;
	        _this.possibleTokTypes = [];
	        _this.nextProductionName = "";
	        _this.nextProductionOccurrence = 0;
	        _this.found = false;
	        _this.isAtEndOfPath = false;
	        return _this;
	    }
	    AbstractNextPossibleTokensWalker.prototype.startWalking = function () {
	        this.found = false;
	        if (this.path.ruleStack[0] !== this.topProd.name) {
	            throw Error("The path does not start with the walker's top Rule!");
	        }
	        // immutable for the win
	        this.ruleStack = (0, clone_1.default)(this.path.ruleStack).reverse(); // intelij bug requires assertion
	        this.occurrenceStack = (0, clone_1.default)(this.path.occurrenceStack).reverse(); // intelij bug requires assertion
	        // already verified that the first production is valid, we now seek the 2nd production
	        this.ruleStack.pop();
	        this.occurrenceStack.pop();
	        this.updateExpectedNext();
	        this.walk(this.topProd);
	        return this.possibleTokTypes;
	    };
	    AbstractNextPossibleTokensWalker.prototype.walk = function (prod, prevRest) {
	        if (prevRest === void 0) { prevRest = []; }
	        // stop scanning once we found the path
	        if (!this.found) {
	            _super.prototype.walk.call(this, prod, prevRest);
	        }
	    };
	    AbstractNextPossibleTokensWalker.prototype.walkProdRef = function (refProd, currRest, prevRest) {
	        // found the next production, need to keep walking in it
	        if (refProd.referencedRule.name === this.nextProductionName &&
	            refProd.idx === this.nextProductionOccurrence) {
	            var fullRest = currRest.concat(prevRest);
	            this.updateExpectedNext();
	            this.walk(refProd.referencedRule, fullRest);
	        }
	    };
	    AbstractNextPossibleTokensWalker.prototype.updateExpectedNext = function () {
	        // need to consume the Terminal
	        if ((0, isEmpty_1.default)(this.ruleStack)) {
	            // must reset nextProductionXXX to avoid walking down another Top Level production while what we are
	            // really seeking is the last Terminal...
	            this.nextProductionName = "";
	            this.nextProductionOccurrence = 0;
	            this.isAtEndOfPath = true;
	        }
	        else {
	            this.nextProductionName = this.ruleStack.pop();
	            this.nextProductionOccurrence = this.occurrenceStack.pop();
	        }
	    };
	    return AbstractNextPossibleTokensWalker;
	}(rest_1.RestWalker));
	interpreter.AbstractNextPossibleTokensWalker = AbstractNextPossibleTokensWalker;
	var NextAfterTokenWalker = /** @class */ (function (_super) {
	    __extends(NextAfterTokenWalker, _super);
	    function NextAfterTokenWalker(topProd, path) {
	        var _this = _super.call(this, topProd, path) || this;
	        _this.path = path;
	        _this.nextTerminalName = "";
	        _this.nextTerminalOccurrence = 0;
	        _this.nextTerminalName = _this.path.lastTok.name;
	        _this.nextTerminalOccurrence = _this.path.lastTokOccurrence;
	        return _this;
	    }
	    NextAfterTokenWalker.prototype.walkTerminal = function (terminal, currRest, prevRest) {
	        if (this.isAtEndOfPath &&
	            terminal.terminalType.name === this.nextTerminalName &&
	            terminal.idx === this.nextTerminalOccurrence &&
	            !this.found) {
	            var fullRest = currRest.concat(prevRest);
	            var restProd = new gast_1.Alternative({ definition: fullRest });
	            this.possibleTokTypes = (0, first_2.first)(restProd);
	            this.found = true;
	        }
	    };
	    return NextAfterTokenWalker;
	}(AbstractNextPossibleTokensWalker));
	interpreter.NextAfterTokenWalker = NextAfterTokenWalker;
	/**
	 * This walker only "walks" a single "TOP" level in the Grammar Ast, this means
	 * it never "follows" production refs
	 */
	var AbstractNextTerminalAfterProductionWalker = /** @class */ (function (_super) {
	    __extends(AbstractNextTerminalAfterProductionWalker, _super);
	    function AbstractNextTerminalAfterProductionWalker(topRule, occurrence) {
	        var _this = _super.call(this) || this;
	        _this.topRule = topRule;
	        _this.occurrence = occurrence;
	        _this.result = {
	            token: undefined,
	            occurrence: undefined,
	            isEndOfRule: undefined
	        };
	        return _this;
	    }
	    AbstractNextTerminalAfterProductionWalker.prototype.startWalking = function () {
	        this.walk(this.topRule);
	        return this.result;
	    };
	    return AbstractNextTerminalAfterProductionWalker;
	}(rest_1.RestWalker));
	interpreter.AbstractNextTerminalAfterProductionWalker = AbstractNextTerminalAfterProductionWalker;
	var NextTerminalAfterManyWalker = /** @class */ (function (_super) {
	    __extends(NextTerminalAfterManyWalker, _super);
	    function NextTerminalAfterManyWalker() {
	        return _super !== null && _super.apply(this, arguments) || this;
	    }
	    NextTerminalAfterManyWalker.prototype.walkMany = function (manyProd, currRest, prevRest) {
	        if (manyProd.idx === this.occurrence) {
	            var firstAfterMany = (0, first_1.default)(currRest.concat(prevRest));
	            this.result.isEndOfRule = firstAfterMany === undefined;
	            if (firstAfterMany instanceof gast_1.Terminal) {
	                this.result.token = firstAfterMany.terminalType;
	                this.result.occurrence = firstAfterMany.idx;
	            }
	        }
	        else {
	            _super.prototype.walkMany.call(this, manyProd, currRest, prevRest);
	        }
	    };
	    return NextTerminalAfterManyWalker;
	}(AbstractNextTerminalAfterProductionWalker));
	interpreter.NextTerminalAfterManyWalker = NextTerminalAfterManyWalker;
	var NextTerminalAfterManySepWalker = /** @class */ (function (_super) {
	    __extends(NextTerminalAfterManySepWalker, _super);
	    function NextTerminalAfterManySepWalker() {
	        return _super !== null && _super.apply(this, arguments) || this;
	    }
	    NextTerminalAfterManySepWalker.prototype.walkManySep = function (manySepProd, currRest, prevRest) {
	        if (manySepProd.idx === this.occurrence) {
	            var firstAfterManySep = (0, first_1.default)(currRest.concat(prevRest));
	            this.result.isEndOfRule = firstAfterManySep === undefined;
	            if (firstAfterManySep instanceof gast_1.Terminal) {
	                this.result.token = firstAfterManySep.terminalType;
	                this.result.occurrence = firstAfterManySep.idx;
	            }
	        }
	        else {
	            _super.prototype.walkManySep.call(this, manySepProd, currRest, prevRest);
	        }
	    };
	    return NextTerminalAfterManySepWalker;
	}(AbstractNextTerminalAfterProductionWalker));
	interpreter.NextTerminalAfterManySepWalker = NextTerminalAfterManySepWalker;
	var NextTerminalAfterAtLeastOneWalker = /** @class */ (function (_super) {
	    __extends(NextTerminalAfterAtLeastOneWalker, _super);
	    function NextTerminalAfterAtLeastOneWalker() {
	        return _super !== null && _super.apply(this, arguments) || this;
	    }
	    NextTerminalAfterAtLeastOneWalker.prototype.walkAtLeastOne = function (atLeastOneProd, currRest, prevRest) {
	        if (atLeastOneProd.idx === this.occurrence) {
	            var firstAfterAtLeastOne = (0, first_1.default)(currRest.concat(prevRest));
	            this.result.isEndOfRule = firstAfterAtLeastOne === undefined;
	            if (firstAfterAtLeastOne instanceof gast_1.Terminal) {
	                this.result.token = firstAfterAtLeastOne.terminalType;
	                this.result.occurrence = firstAfterAtLeastOne.idx;
	            }
	        }
	        else {
	            _super.prototype.walkAtLeastOne.call(this, atLeastOneProd, currRest, prevRest);
	        }
	    };
	    return NextTerminalAfterAtLeastOneWalker;
	}(AbstractNextTerminalAfterProductionWalker));
	interpreter.NextTerminalAfterAtLeastOneWalker = NextTerminalAfterAtLeastOneWalker;
	// TODO: reduce code duplication in the AfterWalkers
	var NextTerminalAfterAtLeastOneSepWalker = /** @class */ (function (_super) {
	    __extends(NextTerminalAfterAtLeastOneSepWalker, _super);
	    function NextTerminalAfterAtLeastOneSepWalker() {
	        return _super !== null && _super.apply(this, arguments) || this;
	    }
	    NextTerminalAfterAtLeastOneSepWalker.prototype.walkAtLeastOneSep = function (atleastOneSepProd, currRest, prevRest) {
	        if (atleastOneSepProd.idx === this.occurrence) {
	            var firstAfterfirstAfterAtLeastOneSep = (0, first_1.default)(currRest.concat(prevRest));
	            this.result.isEndOfRule = firstAfterfirstAfterAtLeastOneSep === undefined;
	            if (firstAfterfirstAfterAtLeastOneSep instanceof gast_1.Terminal) {
	                this.result.token = firstAfterfirstAfterAtLeastOneSep.terminalType;
	                this.result.occurrence = firstAfterfirstAfterAtLeastOneSep.idx;
	            }
	        }
	        else {
	            _super.prototype.walkAtLeastOneSep.call(this, atleastOneSepProd, currRest, prevRest);
	        }
	    };
	    return NextTerminalAfterAtLeastOneSepWalker;
	}(AbstractNextTerminalAfterProductionWalker));
	interpreter.NextTerminalAfterAtLeastOneSepWalker = NextTerminalAfterAtLeastOneSepWalker;
	function possiblePathsFrom(targetDef, maxLength, currPath) {
	    if (currPath === void 0) { currPath = []; }
	    // avoid side effects
	    currPath = (0, clone_1.default)(currPath);
	    var result = [];
	    var i = 0;
	    // TODO: avoid inner funcs
	    function remainingPathWith(nextDef) {
	        return nextDef.concat((0, drop_1.default)(targetDef, i + 1));
	    }
	    // TODO: avoid inner funcs
	    function getAlternativesForProd(definition) {
	        var alternatives = possiblePathsFrom(remainingPathWith(definition), maxLength, currPath);
	        return result.concat(alternatives);
	    }
	    /**
	     * Mandatory productions will halt the loop as the paths computed from their recursive calls will already contain the
	     * following (rest) of the targetDef.
	     *
	     * For optional productions (Option/Repetition/...) the loop will continue to represent the paths that do not include the
	     * the optional production.
	     */
	    while (currPath.length < maxLength && i < targetDef.length) {
	        var prod = targetDef[i];
	        /* istanbul ignore else */
	        if (prod instanceof gast_1.Alternative) {
	            return getAlternativesForProd(prod.definition);
	        }
	        else if (prod instanceof gast_1.NonTerminal) {
	            return getAlternativesForProd(prod.definition);
	        }
	        else if (prod instanceof gast_1.Option) {
	            result = getAlternativesForProd(prod.definition);
	        }
	        else if (prod instanceof gast_1.RepetitionMandatory) {
	            var newDef = prod.definition.concat([
	                new gast_1.Repetition({
	                    definition: prod.definition
	                })
	            ]);
	            return getAlternativesForProd(newDef);
	        }
	        else if (prod instanceof gast_1.RepetitionMandatoryWithSeparator) {
	            var newDef = [
	                new gast_1.Alternative({ definition: prod.definition }),
	                new gast_1.Repetition({
	                    definition: [new gast_1.Terminal({ terminalType: prod.separator })].concat(prod.definition)
	                })
	            ];
	            return getAlternativesForProd(newDef);
	        }
	        else if (prod instanceof gast_1.RepetitionWithSeparator) {
	            var newDef = prod.definition.concat([
	                new gast_1.Repetition({
	                    definition: [new gast_1.Terminal({ terminalType: prod.separator })].concat(prod.definition)
	                })
	            ]);
	            result = getAlternativesForProd(newDef);
	        }
	        else if (prod instanceof gast_1.Repetition) {
	            var newDef = prod.definition.concat([
	                new gast_1.Repetition({
	                    definition: prod.definition
	                })
	            ]);
	            result = getAlternativesForProd(newDef);
	        }
	        else if (prod instanceof gast_1.Alternation) {
	            (0, forEach_1.default)(prod.definition, function (currAlt) {
	                // TODO: this is a limited check for empty alternatives
	                //   It would prevent a common case of infinite loops during parser initialization.
	                //   However **in-directly** empty alternatives may still cause issues.
	                if ((0, isEmpty_1.default)(currAlt.definition) === false) {
	                    result = getAlternativesForProd(currAlt.definition);
	                }
	            });
	            return result;
	        }
	        else if (prod instanceof gast_1.Terminal) {
	            currPath.push(prod.terminalType);
	        }
	        else {
	            throw Error("non exhaustive match");
	        }
	        i++;
	    }
	    result.push({
	        partialPath: currPath,
	        suffixDef: (0, drop_1.default)(targetDef, i)
	    });
	    return result;
	}
	interpreter.possiblePathsFrom = possiblePathsFrom;
	function nextPossibleTokensAfter(initialDef, tokenVector, tokMatcher, maxLookAhead) {
	    var EXIT_NON_TERMINAL = "EXIT_NONE_TERMINAL";
	    // to avoid creating a new Array each time.
	    var EXIT_NON_TERMINAL_ARR = [EXIT_NON_TERMINAL];
	    var EXIT_ALTERNATIVE = "EXIT_ALTERNATIVE";
	    var foundCompletePath = false;
	    var tokenVectorLength = tokenVector.length;
	    var minimalAlternativesIndex = tokenVectorLength - maxLookAhead - 1;
	    var result = [];
	    var possiblePaths = [];
	    possiblePaths.push({
	        idx: -1,
	        def: initialDef,
	        ruleStack: [],
	        occurrenceStack: []
	    });
	    while (!(0, isEmpty_1.default)(possiblePaths)) {
	        var currPath = possiblePaths.pop();
	        // skip alternatives if no more results can be found (assuming deterministic grammar with fixed lookahead)
	        if (currPath === EXIT_ALTERNATIVE) {
	            if (foundCompletePath &&
	                (0, last_1.default)(possiblePaths).idx <= minimalAlternativesIndex) {
	                // remove irrelevant alternative
	                possiblePaths.pop();
	            }
	            continue;
	        }
	        var currDef = currPath.def;
	        var currIdx = currPath.idx;
	        var currRuleStack = currPath.ruleStack;
	        var currOccurrenceStack = currPath.occurrenceStack;
	        // For Example: an empty path could exist in a valid grammar in the case of an EMPTY_ALT
	        if ((0, isEmpty_1.default)(currDef)) {
	            continue;
	        }
	        var prod = currDef[0];
	        /* istanbul ignore else */
	        if (prod === EXIT_NON_TERMINAL) {
	            var nextPath = {
	                idx: currIdx,
	                def: (0, drop_1.default)(currDef),
	                ruleStack: (0, dropRight_1.default)(currRuleStack),
	                occurrenceStack: (0, dropRight_1.default)(currOccurrenceStack)
	            };
	            possiblePaths.push(nextPath);
	        }
	        else if (prod instanceof gast_1.Terminal) {
	            /* istanbul ignore else */
	            if (currIdx < tokenVectorLength - 1) {
	                var nextIdx = currIdx + 1;
	                var actualToken = tokenVector[nextIdx];
	                if (tokMatcher(actualToken, prod.terminalType)) {
	                    var nextPath = {
	                        idx: nextIdx,
	                        def: (0, drop_1.default)(currDef),
	                        ruleStack: currRuleStack,
	                        occurrenceStack: currOccurrenceStack
	                    };
	                    possiblePaths.push(nextPath);
	                }
	                // end of the line
	            }
	            else if (currIdx === tokenVectorLength - 1) {
	                // IGNORE ABOVE ELSE
	                result.push({
	                    nextTokenType: prod.terminalType,
	                    nextTokenOccurrence: prod.idx,
	                    ruleStack: currRuleStack,
	                    occurrenceStack: currOccurrenceStack
	                });
	                foundCompletePath = true;
	            }
	            else {
	                throw Error("non exhaustive match");
	            }
	        }
	        else if (prod instanceof gast_1.NonTerminal) {
	            var newRuleStack = (0, clone_1.default)(currRuleStack);
	            newRuleStack.push(prod.nonTerminalName);
	            var newOccurrenceStack = (0, clone_1.default)(currOccurrenceStack);
	            newOccurrenceStack.push(prod.idx);
	            var nextPath = {
	                idx: currIdx,
	                def: prod.definition.concat(EXIT_NON_TERMINAL_ARR, (0, drop_1.default)(currDef)),
	                ruleStack: newRuleStack,
	                occurrenceStack: newOccurrenceStack
	            };
	            possiblePaths.push(nextPath);
	        }
	        else if (prod instanceof gast_1.Option) {
	            // the order of alternatives is meaningful, FILO (Last path will be traversed first).
	            var nextPathWithout = {
	                idx: currIdx,
	                def: (0, drop_1.default)(currDef),
	                ruleStack: currRuleStack,
	                occurrenceStack: currOccurrenceStack
	            };
	            possiblePaths.push(nextPathWithout);
	            // required marker to avoid backtracking paths whose higher priority alternatives already matched
	            possiblePaths.push(EXIT_ALTERNATIVE);
	            var nextPathWith = {
	                idx: currIdx,
	                def: prod.definition.concat((0, drop_1.default)(currDef)),
	                ruleStack: currRuleStack,
	                occurrenceStack: currOccurrenceStack
	            };
	            possiblePaths.push(nextPathWith);
	        }
	        else if (prod instanceof gast_1.RepetitionMandatory) {
	            // TODO:(THE NEW operators here take a while...) (convert once?)
	            var secondIteration = new gast_1.Repetition({
	                definition: prod.definition,
	                idx: prod.idx
	            });
	            var nextDef = prod.definition.concat([secondIteration], (0, drop_1.default)(currDef));
	            var nextPath = {
	                idx: currIdx,
	                def: nextDef,
	                ruleStack: currRuleStack,
	                occurrenceStack: currOccurrenceStack
	            };
	            possiblePaths.push(nextPath);
	        }
	        else if (prod instanceof gast_1.RepetitionMandatoryWithSeparator) {
	            // TODO:(THE NEW operators here take a while...) (convert once?)
	            var separatorGast = new gast_1.Terminal({
	                terminalType: prod.separator
	            });
	            var secondIteration = new gast_1.Repetition({
	                definition: [separatorGast].concat(prod.definition),
	                idx: prod.idx
	            });
	            var nextDef = prod.definition.concat([secondIteration], (0, drop_1.default)(currDef));
	            var nextPath = {
	                idx: currIdx,
	                def: nextDef,
	                ruleStack: currRuleStack,
	                occurrenceStack: currOccurrenceStack
	            };
	            possiblePaths.push(nextPath);
	        }
	        else if (prod instanceof gast_1.RepetitionWithSeparator) {
	            // the order of alternatives is meaningful, FILO (Last path will be traversed first).
	            var nextPathWithout = {
	                idx: currIdx,
	                def: (0, drop_1.default)(currDef),
	                ruleStack: currRuleStack,
	                occurrenceStack: currOccurrenceStack
	            };
	            possiblePaths.push(nextPathWithout);
	            // required marker to avoid backtracking paths whose higher priority alternatives already matched
	            possiblePaths.push(EXIT_ALTERNATIVE);
	            var separatorGast = new gast_1.Terminal({
	                terminalType: prod.separator
	            });
	            var nthRepetition = new gast_1.Repetition({
	                definition: [separatorGast].concat(prod.definition),
	                idx: prod.idx
	            });
	            var nextDef = prod.definition.concat([nthRepetition], (0, drop_1.default)(currDef));
	            var nextPathWith = {
	                idx: currIdx,
	                def: nextDef,
	                ruleStack: currRuleStack,
	                occurrenceStack: currOccurrenceStack
	            };
	            possiblePaths.push(nextPathWith);
	        }
	        else if (prod instanceof gast_1.Repetition) {
	            // the order of alternatives is meaningful, FILO (Last path will be traversed first).
	            var nextPathWithout = {
	                idx: currIdx,
	                def: (0, drop_1.default)(currDef),
	                ruleStack: currRuleStack,
	                occurrenceStack: currOccurrenceStack
	            };
	            possiblePaths.push(nextPathWithout);
	            // required marker to avoid backtracking paths whose higher priority alternatives already matched
	            possiblePaths.push(EXIT_ALTERNATIVE);
	            // TODO: an empty repetition will cause infinite loops here, will the parser detect this in selfAnalysis?
	            var nthRepetition = new gast_1.Repetition({
	                definition: prod.definition,
	                idx: prod.idx
	            });
	            var nextDef = prod.definition.concat([nthRepetition], (0, drop_1.default)(currDef));
	            var nextPathWith = {
	                idx: currIdx,
	                def: nextDef,
	                ruleStack: currRuleStack,
	                occurrenceStack: currOccurrenceStack
	            };
	            possiblePaths.push(nextPathWith);
	        }
	        else if (prod instanceof gast_1.Alternation) {
	            // the order of alternatives is meaningful, FILO (Last path will be traversed first).
	            for (var i = prod.definition.length - 1; i >= 0; i--) {
	                var currAlt = prod.definition[i];
	                var currAltPath = {
	                    idx: currIdx,
	                    def: currAlt.definition.concat((0, drop_1.default)(currDef)),
	                    ruleStack: currRuleStack,
	                    occurrenceStack: currOccurrenceStack
	                };
	                possiblePaths.push(currAltPath);
	                possiblePaths.push(EXIT_ALTERNATIVE);
	            }
	        }
	        else if (prod instanceof gast_1.Alternative) {
	            possiblePaths.push({
	                idx: currIdx,
	                def: prod.definition.concat((0, drop_1.default)(currDef)),
	                ruleStack: currRuleStack,
	                occurrenceStack: currOccurrenceStack
	            });
	        }
	        else if (prod instanceof gast_1.Rule) {
	            // last because we should only encounter at most a single one of these per invocation.
	            possiblePaths.push(expandTopLevelRule(prod, currIdx, currRuleStack, currOccurrenceStack));
	        }
	        else {
	            throw Error("non exhaustive match");
	        }
	    }
	    return result;
	}
	interpreter.nextPossibleTokensAfter = nextPossibleTokensAfter;
	function expandTopLevelRule(topRule, currIdx, currRuleStack, currOccurrenceStack) {
	    var newRuleStack = (0, clone_1.default)(currRuleStack);
	    newRuleStack.push(topRule.name);
	    var newCurrOccurrenceStack = (0, clone_1.default)(currOccurrenceStack);
	    // top rule is always assumed to have been called with occurrence index 1
	    newCurrOccurrenceStack.push(1);
	    return {
	        idx: currIdx,
	        def: topRule.definition,
	        ruleStack: newRuleStack,
	        occurrenceStack: newCurrOccurrenceStack
	    };
	}
	
	return interpreter;
}

var hasRequiredLookahead;

function requireLookahead () {
	if (hasRequiredLookahead) return lookahead;
	hasRequiredLookahead = 1;
	(function (exports) {
		var __extends = (lookahead && lookahead.__extends) || (function () {
		    var extendStatics = function (d, b) {
		        extendStatics = Object.setPrototypeOf ||
		            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
		            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
		        return extendStatics(d, b);
		    };
		    return function (d, b) {
		        if (typeof b !== "function" && b !== null)
		            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
		        extendStatics(d, b);
		        function __() { this.constructor = d; }
		        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
		    };
		})();
		var __importDefault = (lookahead && lookahead.__importDefault) || function (mod) {
		    return (mod && mod.__esModule) ? mod : { "default": mod };
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.areTokenCategoriesNotUsed = exports.isStrictPrefixOfPath = exports.containsPath = exports.getLookaheadPathsForOptionalProd = exports.getLookaheadPathsForOr = exports.lookAheadSequenceFromAlternatives = exports.buildSingleAlternativeLookaheadFunction = exports.buildAlternativesLookAheadFunc = exports.buildLookaheadFuncForOptionalProd = exports.buildLookaheadFuncForOr = exports.getLookaheadPaths = exports.getProdType = exports.PROD_TYPE = void 0;
		var isEmpty_1 = __importDefault(requireIsEmpty());
		var flatten_1 = __importDefault(requireFlatten());
		var every_1 = __importDefault(requireEvery());
		var map_1 = __importDefault(requireMap());
		var forEach_1 = __importDefault(requireForEach());
		var has_1 = __importDefault(requireHas());
		var reduce_1 = __importDefault(requireReduce());
		var interpreter_1 = requireInterpreter();
		var rest_1 = requireRest();
		var tokens_1 = requireTokens();
		var gast_1 = requireApi$2();
		var gast_2 = requireApi$2();
		var PROD_TYPE;
		(function (PROD_TYPE) {
		    PROD_TYPE[PROD_TYPE["OPTION"] = 0] = "OPTION";
		    PROD_TYPE[PROD_TYPE["REPETITION"] = 1] = "REPETITION";
		    PROD_TYPE[PROD_TYPE["REPETITION_MANDATORY"] = 2] = "REPETITION_MANDATORY";
		    PROD_TYPE[PROD_TYPE["REPETITION_MANDATORY_WITH_SEPARATOR"] = 3] = "REPETITION_MANDATORY_WITH_SEPARATOR";
		    PROD_TYPE[PROD_TYPE["REPETITION_WITH_SEPARATOR"] = 4] = "REPETITION_WITH_SEPARATOR";
		    PROD_TYPE[PROD_TYPE["ALTERNATION"] = 5] = "ALTERNATION";
		})(PROD_TYPE = exports.PROD_TYPE || (exports.PROD_TYPE = {}));
		function getProdType(prod) {
		    /* istanbul ignore else */
		    if (prod instanceof gast_1.Option || prod === "Option") {
		        return PROD_TYPE.OPTION;
		    }
		    else if (prod instanceof gast_1.Repetition || prod === "Repetition") {
		        return PROD_TYPE.REPETITION;
		    }
		    else if (prod instanceof gast_1.RepetitionMandatory ||
		        prod === "RepetitionMandatory") {
		        return PROD_TYPE.REPETITION_MANDATORY;
		    }
		    else if (prod instanceof gast_1.RepetitionMandatoryWithSeparator ||
		        prod === "RepetitionMandatoryWithSeparator") {
		        return PROD_TYPE.REPETITION_MANDATORY_WITH_SEPARATOR;
		    }
		    else if (prod instanceof gast_1.RepetitionWithSeparator ||
		        prod === "RepetitionWithSeparator") {
		        return PROD_TYPE.REPETITION_WITH_SEPARATOR;
		    }
		    else if (prod instanceof gast_1.Alternation || prod === "Alternation") {
		        return PROD_TYPE.ALTERNATION;
		    }
		    else {
		        throw Error("non exhaustive match");
		    }
		}
		exports.getProdType = getProdType;
		function getLookaheadPaths(options) {
		    var occurrence = options.occurrence, rule = options.rule, prodType = options.prodType, maxLookahead = options.maxLookahead;
		    var type = getProdType(prodType);
		    if (type === PROD_TYPE.ALTERNATION) {
		        return getLookaheadPathsForOr(occurrence, rule, maxLookahead);
		    }
		    else {
		        return getLookaheadPathsForOptionalProd(occurrence, rule, type, maxLookahead);
		    }
		}
		exports.getLookaheadPaths = getLookaheadPaths;
		function buildLookaheadFuncForOr(occurrence, ruleGrammar, maxLookahead, hasPredicates, dynamicTokensEnabled, laFuncBuilder) {
		    var lookAheadPaths = getLookaheadPathsForOr(occurrence, ruleGrammar, maxLookahead);
		    var tokenMatcher = areTokenCategoriesNotUsed(lookAheadPaths)
		        ? tokens_1.tokenStructuredMatcherNoCategories
		        : tokens_1.tokenStructuredMatcher;
		    return laFuncBuilder(lookAheadPaths, hasPredicates, tokenMatcher, dynamicTokensEnabled);
		}
		exports.buildLookaheadFuncForOr = buildLookaheadFuncForOr;
		/**
		 *  When dealing with an Optional production (OPTION/MANY/2nd iteration of AT_LEAST_ONE/...) we need to compare
		 *  the lookahead "inside" the production and the lookahead immediately "after" it in the same top level rule (context free).
		 *
		 *  Example: given a production:
		 *  ABC(DE)?DF
		 *
		 *  The optional '(DE)?' should only be entered if we see 'DE'. a single Token 'D' is not sufficient to distinguish between the two
		 *  alternatives.
		 *
		 *  @returns A Lookahead function which will return true IFF the parser should parse the Optional production.
		 */
		function buildLookaheadFuncForOptionalProd(occurrence, ruleGrammar, k, dynamicTokensEnabled, prodType, lookaheadBuilder) {
		    var lookAheadPaths = getLookaheadPathsForOptionalProd(occurrence, ruleGrammar, prodType, k);
		    var tokenMatcher = areTokenCategoriesNotUsed(lookAheadPaths)
		        ? tokens_1.tokenStructuredMatcherNoCategories
		        : tokens_1.tokenStructuredMatcher;
		    return lookaheadBuilder(lookAheadPaths[0], tokenMatcher, dynamicTokensEnabled);
		}
		exports.buildLookaheadFuncForOptionalProd = buildLookaheadFuncForOptionalProd;
		function buildAlternativesLookAheadFunc(alts, hasPredicates, tokenMatcher, dynamicTokensEnabled) {
		    var numOfAlts = alts.length;
		    var areAllOneTokenLookahead = (0, every_1.default)(alts, function (currAlt) {
		        return (0, every_1.default)(currAlt, function (currPath) {
		            return currPath.length === 1;
		        });
		    });
		    // This version takes into account the predicates as well.
		    if (hasPredicates) {
		        /**
		         * @returns {number} - The chosen alternative index
		         */
		        return function (orAlts) {
		            // unfortunately the predicates must be extracted every single time
		            // as they cannot be cached due to references to parameters(vars) which are no longer valid.
		            // note that in the common case of no predicates, no cpu time will be wasted on this (see else block)
		            var predicates = (0, map_1.default)(orAlts, function (currAlt) { return currAlt.GATE; });
		            for (var t = 0; t < numOfAlts; t++) {
		                var currAlt = alts[t];
		                var currNumOfPaths = currAlt.length;
		                var currPredicate = predicates[t];
		                if (currPredicate !== undefined && currPredicate.call(this) === false) {
		                    // if the predicate does not match there is no point in checking the paths
		                    continue;
		                }
		                nextPath: for (var j = 0; j < currNumOfPaths; j++) {
		                    var currPath = currAlt[j];
		                    var currPathLength = currPath.length;
		                    for (var i = 0; i < currPathLength; i++) {
		                        var nextToken = this.LA(i + 1);
		                        if (tokenMatcher(nextToken, currPath[i]) === false) {
		                            // mismatch in current path
		                            // try the next pth
		                            continue nextPath;
		                        }
		                    }
		                    // found a full path that matches.
		                    // this will also work for an empty ALT as the loop will be skipped
		                    return t;
		                }
		                // none of the paths for the current alternative matched
		                // try the next alternative
		            }
		            // none of the alternatives could be matched
		            return undefined;
		        };
		    }
		    else if (areAllOneTokenLookahead && !dynamicTokensEnabled) {
		        // optimized (common) case of all the lookaheads paths requiring only
		        // a single token lookahead. These Optimizations cannot work if dynamically defined Tokens are used.
		        var singleTokenAlts = (0, map_1.default)(alts, function (currAlt) {
		            return (0, flatten_1.default)(currAlt);
		        });
		        var choiceToAlt_1 = (0, reduce_1.default)(singleTokenAlts, function (result, currAlt, idx) {
		            (0, forEach_1.default)(currAlt, function (currTokType) {
		                if (!(0, has_1.default)(result, currTokType.tokenTypeIdx)) {
		                    result[currTokType.tokenTypeIdx] = idx;
		                }
		                (0, forEach_1.default)(currTokType.categoryMatches, function (currExtendingType) {
		                    if (!(0, has_1.default)(result, currExtendingType)) {
		                        result[currExtendingType] = idx;
		                    }
		                });
		            });
		            return result;
		        }, {});
		        /**
		         * @returns {number} - The chosen alternative index
		         */
		        return function () {
		            var nextToken = this.LA(1);
		            return choiceToAlt_1[nextToken.tokenTypeIdx];
		        };
		    }
		    else {
		        // optimized lookahead without needing to check the predicates at all.
		        // this causes code duplication which is intentional to improve performance.
		        /**
		         * @returns {number} - The chosen alternative index
		         */
		        return function () {
		            for (var t = 0; t < numOfAlts; t++) {
		                var currAlt = alts[t];
		                var currNumOfPaths = currAlt.length;
		                nextPath: for (var j = 0; j < currNumOfPaths; j++) {
		                    var currPath = currAlt[j];
		                    var currPathLength = currPath.length;
		                    for (var i = 0; i < currPathLength; i++) {
		                        var nextToken = this.LA(i + 1);
		                        if (tokenMatcher(nextToken, currPath[i]) === false) {
		                            // mismatch in current path
		                            // try the next pth
		                            continue nextPath;
		                        }
		                    }
		                    // found a full path that matches.
		                    // this will also work for an empty ALT as the loop will be skipped
		                    return t;
		                }
		                // none of the paths for the current alternative matched
		                // try the next alternative
		            }
		            // none of the alternatives could be matched
		            return undefined;
		        };
		    }
		}
		exports.buildAlternativesLookAheadFunc = buildAlternativesLookAheadFunc;
		function buildSingleAlternativeLookaheadFunction(alt, tokenMatcher, dynamicTokensEnabled) {
		    var areAllOneTokenLookahead = (0, every_1.default)(alt, function (currPath) {
		        return currPath.length === 1;
		    });
		    var numOfPaths = alt.length;
		    // optimized (common) case of all the lookaheads paths requiring only
		    // a single token lookahead.
		    if (areAllOneTokenLookahead && !dynamicTokensEnabled) {
		        var singleTokensTypes = (0, flatten_1.default)(alt);
		        if (singleTokensTypes.length === 1 &&
		            (0, isEmpty_1.default)(singleTokensTypes[0].categoryMatches)) {
		            var expectedTokenType = singleTokensTypes[0];
		            var expectedTokenUniqueKey_1 = expectedTokenType.tokenTypeIdx;
		            return function () {
		                return this.LA(1).tokenTypeIdx === expectedTokenUniqueKey_1;
		            };
		        }
		        else {
		            var choiceToAlt_2 = (0, reduce_1.default)(singleTokensTypes, function (result, currTokType, idx) {
		                result[currTokType.tokenTypeIdx] = true;
		                (0, forEach_1.default)(currTokType.categoryMatches, function (currExtendingType) {
		                    result[currExtendingType] = true;
		                });
		                return result;
		            }, []);
		            return function () {
		                var nextToken = this.LA(1);
		                return choiceToAlt_2[nextToken.tokenTypeIdx] === true;
		            };
		        }
		    }
		    else {
		        return function () {
		            nextPath: for (var j = 0; j < numOfPaths; j++) {
		                var currPath = alt[j];
		                var currPathLength = currPath.length;
		                for (var i = 0; i < currPathLength; i++) {
		                    var nextToken = this.LA(i + 1);
		                    if (tokenMatcher(nextToken, currPath[i]) === false) {
		                        // mismatch in current path
		                        // try the next pth
		                        continue nextPath;
		                    }
		                }
		                // found a full path that matches.
		                return true;
		            }
		            // none of the paths matched
		            return false;
		        };
		    }
		}
		exports.buildSingleAlternativeLookaheadFunction = buildSingleAlternativeLookaheadFunction;
		var RestDefinitionFinderWalker = /** @class */ (function (_super) {
		    __extends(RestDefinitionFinderWalker, _super);
		    function RestDefinitionFinderWalker(topProd, targetOccurrence, targetProdType) {
		        var _this = _super.call(this) || this;
		        _this.topProd = topProd;
		        _this.targetOccurrence = targetOccurrence;
		        _this.targetProdType = targetProdType;
		        return _this;
		    }
		    RestDefinitionFinderWalker.prototype.startWalking = function () {
		        this.walk(this.topProd);
		        return this.restDef;
		    };
		    RestDefinitionFinderWalker.prototype.checkIsTarget = function (node, expectedProdType, currRest, prevRest) {
		        if (node.idx === this.targetOccurrence &&
		            this.targetProdType === expectedProdType) {
		            this.restDef = currRest.concat(prevRest);
		            return true;
		        }
		        // performance optimization, do not iterate over the entire Grammar ast after we have found the target
		        return false;
		    };
		    RestDefinitionFinderWalker.prototype.walkOption = function (optionProd, currRest, prevRest) {
		        if (!this.checkIsTarget(optionProd, PROD_TYPE.OPTION, currRest, prevRest)) {
		            _super.prototype.walkOption.call(this, optionProd, currRest, prevRest);
		        }
		    };
		    RestDefinitionFinderWalker.prototype.walkAtLeastOne = function (atLeastOneProd, currRest, prevRest) {
		        if (!this.checkIsTarget(atLeastOneProd, PROD_TYPE.REPETITION_MANDATORY, currRest, prevRest)) {
		            _super.prototype.walkOption.call(this, atLeastOneProd, currRest, prevRest);
		        }
		    };
		    RestDefinitionFinderWalker.prototype.walkAtLeastOneSep = function (atLeastOneSepProd, currRest, prevRest) {
		        if (!this.checkIsTarget(atLeastOneSepProd, PROD_TYPE.REPETITION_MANDATORY_WITH_SEPARATOR, currRest, prevRest)) {
		            _super.prototype.walkOption.call(this, atLeastOneSepProd, currRest, prevRest);
		        }
		    };
		    RestDefinitionFinderWalker.prototype.walkMany = function (manyProd, currRest, prevRest) {
		        if (!this.checkIsTarget(manyProd, PROD_TYPE.REPETITION, currRest, prevRest)) {
		            _super.prototype.walkOption.call(this, manyProd, currRest, prevRest);
		        }
		    };
		    RestDefinitionFinderWalker.prototype.walkManySep = function (manySepProd, currRest, prevRest) {
		        if (!this.checkIsTarget(manySepProd, PROD_TYPE.REPETITION_WITH_SEPARATOR, currRest, prevRest)) {
		            _super.prototype.walkOption.call(this, manySepProd, currRest, prevRest);
		        }
		    };
		    return RestDefinitionFinderWalker;
		}(rest_1.RestWalker));
		/**
		 * Returns the definition of a target production in a top level level rule.
		 */
		var InsideDefinitionFinderVisitor = /** @class */ (function (_super) {
		    __extends(InsideDefinitionFinderVisitor, _super);
		    function InsideDefinitionFinderVisitor(targetOccurrence, targetProdType, targetRef) {
		        var _this = _super.call(this) || this;
		        _this.targetOccurrence = targetOccurrence;
		        _this.targetProdType = targetProdType;
		        _this.targetRef = targetRef;
		        _this.result = [];
		        return _this;
		    }
		    InsideDefinitionFinderVisitor.prototype.checkIsTarget = function (node, expectedProdName) {
		        if (node.idx === this.targetOccurrence &&
		            this.targetProdType === expectedProdName &&
		            (this.targetRef === undefined || node === this.targetRef)) {
		            this.result = node.definition;
		        }
		    };
		    InsideDefinitionFinderVisitor.prototype.visitOption = function (node) {
		        this.checkIsTarget(node, PROD_TYPE.OPTION);
		    };
		    InsideDefinitionFinderVisitor.prototype.visitRepetition = function (node) {
		        this.checkIsTarget(node, PROD_TYPE.REPETITION);
		    };
		    InsideDefinitionFinderVisitor.prototype.visitRepetitionMandatory = function (node) {
		        this.checkIsTarget(node, PROD_TYPE.REPETITION_MANDATORY);
		    };
		    InsideDefinitionFinderVisitor.prototype.visitRepetitionMandatoryWithSeparator = function (node) {
		        this.checkIsTarget(node, PROD_TYPE.REPETITION_MANDATORY_WITH_SEPARATOR);
		    };
		    InsideDefinitionFinderVisitor.prototype.visitRepetitionWithSeparator = function (node) {
		        this.checkIsTarget(node, PROD_TYPE.REPETITION_WITH_SEPARATOR);
		    };
		    InsideDefinitionFinderVisitor.prototype.visitAlternation = function (node) {
		        this.checkIsTarget(node, PROD_TYPE.ALTERNATION);
		    };
		    return InsideDefinitionFinderVisitor;
		}(gast_2.GAstVisitor));
		function initializeArrayOfArrays(size) {
		    var result = new Array(size);
		    for (var i = 0; i < size; i++) {
		        result[i] = [];
		    }
		    return result;
		}
		/**
		 * A sort of hash function between a Path in the grammar and a string.
		 * Note that this returns multiple "hashes" to support the scenario of token categories.
		 * -  A single path with categories may match multiple **actual** paths.
		 */
		function pathToHashKeys(path) {
		    var keys = [""];
		    for (var i = 0; i < path.length; i++) {
		        var tokType = path[i];
		        var longerKeys = [];
		        for (var j = 0; j < keys.length; j++) {
		            var currShorterKey = keys[j];
		            longerKeys.push(currShorterKey + "_" + tokType.tokenTypeIdx);
		            for (var t = 0; t < tokType.categoryMatches.length; t++) {
		                var categoriesKeySuffix = "_" + tokType.categoryMatches[t];
		                longerKeys.push(currShorterKey + categoriesKeySuffix);
		            }
		        }
		        keys = longerKeys;
		    }
		    return keys;
		}
		/**
		 * Imperative style due to being called from a hot spot
		 */
		function isUniquePrefixHash(altKnownPathsKeys, searchPathKeys, idx) {
		    for (var currAltIdx = 0; currAltIdx < altKnownPathsKeys.length; currAltIdx++) {
		        // We only want to test vs the other alternatives
		        if (currAltIdx === idx) {
		            continue;
		        }
		        var otherAltKnownPathsKeys = altKnownPathsKeys[currAltIdx];
		        for (var searchIdx = 0; searchIdx < searchPathKeys.length; searchIdx++) {
		            var searchKey = searchPathKeys[searchIdx];
		            if (otherAltKnownPathsKeys[searchKey] === true) {
		                return false;
		            }
		        }
		    }
		    // None of the SearchPathKeys were found in any of the other alternatives
		    return true;
		}
		function lookAheadSequenceFromAlternatives(altsDefs, k) {
		    var partialAlts = (0, map_1.default)(altsDefs, function (currAlt) {
		        return (0, interpreter_1.possiblePathsFrom)([currAlt], 1);
		    });
		    var finalResult = initializeArrayOfArrays(partialAlts.length);
		    var altsHashes = (0, map_1.default)(partialAlts, function (currAltPaths) {
		        var dict = {};
		        (0, forEach_1.default)(currAltPaths, function (item) {
		            var keys = pathToHashKeys(item.partialPath);
		            (0, forEach_1.default)(keys, function (currKey) {
		                dict[currKey] = true;
		            });
		        });
		        return dict;
		    });
		    var newData = partialAlts;
		    // maxLookahead loop
		    for (var pathLength = 1; pathLength <= k; pathLength++) {
		        var currDataset = newData;
		        newData = initializeArrayOfArrays(currDataset.length);
		        var _loop_1 = function (altIdx) {
		            var currAltPathsAndSuffixes = currDataset[altIdx];
		            // paths in current alternative loop
		            for (var currPathIdx = 0; currPathIdx < currAltPathsAndSuffixes.length; currPathIdx++) {
		                var currPathPrefix = currAltPathsAndSuffixes[currPathIdx].partialPath;
		                var suffixDef = currAltPathsAndSuffixes[currPathIdx].suffixDef;
		                var prefixKeys = pathToHashKeys(currPathPrefix);
		                var isUnique = isUniquePrefixHash(altsHashes, prefixKeys, altIdx);
		                // End of the line for this path.
		                if (isUnique || (0, isEmpty_1.default)(suffixDef) || currPathPrefix.length === k) {
		                    var currAltResult = finalResult[altIdx];
		                    // TODO: Can we implement a containsPath using Maps/Dictionaries?
		                    if (containsPath(currAltResult, currPathPrefix) === false) {
		                        currAltResult.push(currPathPrefix);
		                        // Update all new  keys for the current path.
		                        for (var j = 0; j < prefixKeys.length; j++) {
		                            var currKey = prefixKeys[j];
		                            altsHashes[altIdx][currKey] = true;
		                        }
		                    }
		                }
		                // Expand longer paths
		                else {
		                    var newPartialPathsAndSuffixes = (0, interpreter_1.possiblePathsFrom)(suffixDef, pathLength + 1, currPathPrefix);
		                    newData[altIdx] = newData[altIdx].concat(newPartialPathsAndSuffixes);
		                    // Update keys for new known paths
		                    (0, forEach_1.default)(newPartialPathsAndSuffixes, function (item) {
		                        var prefixKeys = pathToHashKeys(item.partialPath);
		                        (0, forEach_1.default)(prefixKeys, function (key) {
		                            altsHashes[altIdx][key] = true;
		                        });
		                    });
		                }
		            }
		        };
		        // alternatives loop
		        for (var altIdx = 0; altIdx < currDataset.length; altIdx++) {
		            _loop_1(altIdx);
		        }
		    }
		    return finalResult;
		}
		exports.lookAheadSequenceFromAlternatives = lookAheadSequenceFromAlternatives;
		function getLookaheadPathsForOr(occurrence, ruleGrammar, k, orProd) {
		    var visitor = new InsideDefinitionFinderVisitor(occurrence, PROD_TYPE.ALTERNATION, orProd);
		    ruleGrammar.accept(visitor);
		    return lookAheadSequenceFromAlternatives(visitor.result, k);
		}
		exports.getLookaheadPathsForOr = getLookaheadPathsForOr;
		function getLookaheadPathsForOptionalProd(occurrence, ruleGrammar, prodType, k) {
		    var insideDefVisitor = new InsideDefinitionFinderVisitor(occurrence, prodType);
		    ruleGrammar.accept(insideDefVisitor);
		    var insideDef = insideDefVisitor.result;
		    var afterDefWalker = new RestDefinitionFinderWalker(ruleGrammar, occurrence, prodType);
		    var afterDef = afterDefWalker.startWalking();
		    var insideFlat = new gast_1.Alternative({ definition: insideDef });
		    var afterFlat = new gast_1.Alternative({ definition: afterDef });
		    return lookAheadSequenceFromAlternatives([insideFlat, afterFlat], k);
		}
		exports.getLookaheadPathsForOptionalProd = getLookaheadPathsForOptionalProd;
		function containsPath(alternative, searchPath) {
		    compareOtherPath: for (var i = 0; i < alternative.length; i++) {
		        var otherPath = alternative[i];
		        if (otherPath.length !== searchPath.length) {
		            continue;
		        }
		        for (var j = 0; j < otherPath.length; j++) {
		            var searchTok = searchPath[j];
		            var otherTok = otherPath[j];
		            var matchingTokens = searchTok === otherTok ||
		                otherTok.categoryMatchesMap[searchTok.tokenTypeIdx] !== undefined;
		            if (matchingTokens === false) {
		                continue compareOtherPath;
		            }
		        }
		        return true;
		    }
		    return false;
		}
		exports.containsPath = containsPath;
		function isStrictPrefixOfPath(prefix, other) {
		    return (prefix.length < other.length &&
		        (0, every_1.default)(prefix, function (tokType, idx) {
		            var otherTokType = other[idx];
		            return (tokType === otherTokType ||
		                otherTokType.categoryMatchesMap[tokType.tokenTypeIdx]);
		        }));
		}
		exports.isStrictPrefixOfPath = isStrictPrefixOfPath;
		function areTokenCategoriesNotUsed(lookAheadPaths) {
		    return (0, every_1.default)(lookAheadPaths, function (singleAltPaths) {
		        return (0, every_1.default)(singleAltPaths, function (singlePath) {
		            return (0, every_1.default)(singlePath, function (token) { return (0, isEmpty_1.default)(token.categoryMatches); });
		        });
		    });
		}
		exports.areTokenCategoriesNotUsed = areTokenCategoriesNotUsed;
		
	} (lookahead));
	return lookahead;
}

var hasRequiredChecks;

function requireChecks () {
	if (hasRequiredChecks) return checks;
	hasRequiredChecks = 1;
	var __extends = (checks && checks.__extends) || (function () {
	    var extendStatics = function (d, b) {
	        extendStatics = Object.setPrototypeOf ||
	            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
	            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
	        return extendStatics(d, b);
	    };
	    return function (d, b) {
	        if (typeof b !== "function" && b !== null)
	            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
	        extendStatics(d, b);
	        function __() { this.constructor = d; }
	        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
	    };
	})();
	var __assign = (checks && checks.__assign) || function () {
	    __assign = Object.assign || function(t) {
	        for (var s, i = 1, n = arguments.length; i < n; i++) {
	            s = arguments[i];
	            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
	                t[p] = s[p];
	        }
	        return t;
	    };
	    return __assign.apply(this, arguments);
	};
	var __importDefault = (checks && checks.__importDefault) || function (mod) {
	    return (mod && mod.__esModule) ? mod : { "default": mod };
	};
	Object.defineProperty(checks, "__esModule", { value: true });
	checks.checkPrefixAlternativesAmbiguities = checks.validateSomeNonEmptyLookaheadPath = checks.validateTooManyAlts = checks.RepetitionCollector = checks.validateAmbiguousAlternationAlternatives = checks.validateEmptyOrAlternative = checks.getFirstNoneTerminal = checks.validateNoLeftRecursion = checks.validateRuleIsOverridden = checks.validateRuleDoesNotAlreadyExist = checks.OccurrenceValidationCollector = checks.identifyProductionForDuplicates = checks.validateGrammar = checks.validateLookahead = void 0;
	var first_1 = __importDefault(requireFirst());
	var isEmpty_1 = __importDefault(requireIsEmpty());
	var drop_1 = __importDefault(requireDrop());
	var flatten_1 = __importDefault(requireFlatten());
	var filter_1 = __importDefault(requireFilter());
	var reject_1 = __importDefault(requireReject());
	var difference_1 = __importDefault(requireDifference());
	var map_1 = __importDefault(requireMap());
	var forEach_1 = __importDefault(requireForEach());
	var groupBy_1 = __importDefault(requireGroupBy());
	var reduce_1 = __importDefault(requireReduce());
	var pickBy_1 = __importDefault(requirePickBy());
	var values_1 = __importDefault(requireValues());
	var includes_1 = __importDefault(requireIncludes());
	var flatMap_1 = __importDefault(requireFlatMap());
	var clone_1 = __importDefault(requireClone());
	var parser_1 = requireParser();
	var gast_1 = requireApi$2();
	var lookahead_1 = requireLookahead();
	var interpreter_1 = requireInterpreter();
	var gast_2 = requireApi$2();
	var gast_3 = requireApi$2();
	var dropRight_1 = __importDefault(requireDropRight());
	var compact_1 = __importDefault(requireCompact());
	var tokens_1 = requireTokens();
	function validateLookahead(options) {
	    var lookaheadValidationErrorMessages = options.lookaheadStrategy.validate({
	        rules: options.rules,
	        tokenTypes: options.tokenTypes,
	        grammarName: options.grammarName
	    });
	    return (0, map_1.default)(lookaheadValidationErrorMessages, function (errorMessage) { return (__assign({ type: parser_1.ParserDefinitionErrorType.CUSTOM_LOOKAHEAD_VALIDATION }, errorMessage)); });
	}
	checks.validateLookahead = validateLookahead;
	function validateGrammar(topLevels, tokenTypes, errMsgProvider, grammarName) {
	    var duplicateErrors = (0, flatMap_1.default)(topLevels, function (currTopLevel) { return validateDuplicateProductions(currTopLevel, errMsgProvider); });
	    var termsNamespaceConflictErrors = checkTerminalAndNoneTerminalsNameSpace(topLevels, tokenTypes, errMsgProvider);
	    var tooManyAltsErrors = (0, flatMap_1.default)(topLevels, function (curRule) {
	        return validateTooManyAlts(curRule, errMsgProvider);
	    });
	    var duplicateRulesError = (0, flatMap_1.default)(topLevels, function (curRule) {
	        return validateRuleDoesNotAlreadyExist(curRule, topLevels, grammarName, errMsgProvider);
	    });
	    return duplicateErrors.concat(termsNamespaceConflictErrors, tooManyAltsErrors, duplicateRulesError);
	}
	checks.validateGrammar = validateGrammar;
	function validateDuplicateProductions(topLevelRule, errMsgProvider) {
	    var collectorVisitor = new OccurrenceValidationCollector();
	    topLevelRule.accept(collectorVisitor);
	    var allRuleProductions = collectorVisitor.allProductions;
	    var productionGroups = (0, groupBy_1.default)(allRuleProductions, identifyProductionForDuplicates);
	    var duplicates = (0, pickBy_1.default)(productionGroups, function (currGroup) {
	        return currGroup.length > 1;
	    });
	    var errors = (0, map_1.default)((0, values_1.default)(duplicates), function (currDuplicates) {
	        var firstProd = (0, first_1.default)(currDuplicates);
	        var msg = errMsgProvider.buildDuplicateFoundError(topLevelRule, currDuplicates);
	        var dslName = (0, gast_1.getProductionDslName)(firstProd);
	        var defError = {
	            message: msg,
	            type: parser_1.ParserDefinitionErrorType.DUPLICATE_PRODUCTIONS,
	            ruleName: topLevelRule.name,
	            dslName: dslName,
	            occurrence: firstProd.idx
	        };
	        var param = getExtraProductionArgument(firstProd);
	        if (param) {
	            defError.parameter = param;
	        }
	        return defError;
	    });
	    return errors;
	}
	function identifyProductionForDuplicates(prod) {
	    return "".concat((0, gast_1.getProductionDslName)(prod), "_#_").concat(prod.idx, "_#_").concat(getExtraProductionArgument(prod));
	}
	checks.identifyProductionForDuplicates = identifyProductionForDuplicates;
	function getExtraProductionArgument(prod) {
	    if (prod instanceof gast_2.Terminal) {
	        return prod.terminalType.name;
	    }
	    else if (prod instanceof gast_2.NonTerminal) {
	        return prod.nonTerminalName;
	    }
	    else {
	        return "";
	    }
	}
	var OccurrenceValidationCollector = /** @class */ (function (_super) {
	    __extends(OccurrenceValidationCollector, _super);
	    function OccurrenceValidationCollector() {
	        var _this = _super !== null && _super.apply(this, arguments) || this;
	        _this.allProductions = [];
	        return _this;
	    }
	    OccurrenceValidationCollector.prototype.visitNonTerminal = function (subrule) {
	        this.allProductions.push(subrule);
	    };
	    OccurrenceValidationCollector.prototype.visitOption = function (option) {
	        this.allProductions.push(option);
	    };
	    OccurrenceValidationCollector.prototype.visitRepetitionWithSeparator = function (manySep) {
	        this.allProductions.push(manySep);
	    };
	    OccurrenceValidationCollector.prototype.visitRepetitionMandatory = function (atLeastOne) {
	        this.allProductions.push(atLeastOne);
	    };
	    OccurrenceValidationCollector.prototype.visitRepetitionMandatoryWithSeparator = function (atLeastOneSep) {
	        this.allProductions.push(atLeastOneSep);
	    };
	    OccurrenceValidationCollector.prototype.visitRepetition = function (many) {
	        this.allProductions.push(many);
	    };
	    OccurrenceValidationCollector.prototype.visitAlternation = function (or) {
	        this.allProductions.push(or);
	    };
	    OccurrenceValidationCollector.prototype.visitTerminal = function (terminal) {
	        this.allProductions.push(terminal);
	    };
	    return OccurrenceValidationCollector;
	}(gast_3.GAstVisitor));
	checks.OccurrenceValidationCollector = OccurrenceValidationCollector;
	function validateRuleDoesNotAlreadyExist(rule, allRules, className, errMsgProvider) {
	    var errors = [];
	    var occurrences = (0, reduce_1.default)(allRules, function (result, curRule) {
	        if (curRule.name === rule.name) {
	            return result + 1;
	        }
	        return result;
	    }, 0);
	    if (occurrences > 1) {
	        var errMsg = errMsgProvider.buildDuplicateRuleNameError({
	            topLevelRule: rule,
	            grammarName: className
	        });
	        errors.push({
	            message: errMsg,
	            type: parser_1.ParserDefinitionErrorType.DUPLICATE_RULE_NAME,
	            ruleName: rule.name
	        });
	    }
	    return errors;
	}
	checks.validateRuleDoesNotAlreadyExist = validateRuleDoesNotAlreadyExist;
	// TODO: is there anyway to get only the rule names of rules inherited from the super grammars?
	// This is not part of the IGrammarErrorProvider because the validation cannot be performed on
	// The grammar structure, only at runtime.
	function validateRuleIsOverridden(ruleName, definedRulesNames, className) {
	    var errors = [];
	    var errMsg;
	    if (!(0, includes_1.default)(definedRulesNames, ruleName)) {
	        errMsg =
	            "Invalid rule override, rule: ->".concat(ruleName, "<- cannot be overridden in the grammar: ->").concat(className, "<-") +
	                "as it is not defined in any of the super grammars ";
	        errors.push({
	            message: errMsg,
	            type: parser_1.ParserDefinitionErrorType.INVALID_RULE_OVERRIDE,
	            ruleName: ruleName
	        });
	    }
	    return errors;
	}
	checks.validateRuleIsOverridden = validateRuleIsOverridden;
	function validateNoLeftRecursion(topRule, currRule, errMsgProvider, path) {
	    if (path === void 0) { path = []; }
	    var errors = [];
	    var nextNonTerminals = getFirstNoneTerminal(currRule.definition);
	    if ((0, isEmpty_1.default)(nextNonTerminals)) {
	        return [];
	    }
	    else {
	        var ruleName = topRule.name;
	        var foundLeftRecursion = (0, includes_1.default)(nextNonTerminals, topRule);
	        if (foundLeftRecursion) {
	            errors.push({
	                message: errMsgProvider.buildLeftRecursionError({
	                    topLevelRule: topRule,
	                    leftRecursionPath: path
	                }),
	                type: parser_1.ParserDefinitionErrorType.LEFT_RECURSION,
	                ruleName: ruleName
	            });
	        }
	        // we are only looking for cyclic paths leading back to the specific topRule
	        // other cyclic paths are ignored, we still need this difference to avoid infinite loops...
	        var validNextSteps = (0, difference_1.default)(nextNonTerminals, path.concat([topRule]));
	        var errorsFromNextSteps = (0, flatMap_1.default)(validNextSteps, function (currRefRule) {
	            var newPath = (0, clone_1.default)(path);
	            newPath.push(currRefRule);
	            return validateNoLeftRecursion(topRule, currRefRule, errMsgProvider, newPath);
	        });
	        return errors.concat(errorsFromNextSteps);
	    }
	}
	checks.validateNoLeftRecursion = validateNoLeftRecursion;
	function getFirstNoneTerminal(definition) {
	    var result = [];
	    if ((0, isEmpty_1.default)(definition)) {
	        return result;
	    }
	    var firstProd = (0, first_1.default)(definition);
	    /* istanbul ignore else */
	    if (firstProd instanceof gast_2.NonTerminal) {
	        result.push(firstProd.referencedRule);
	    }
	    else if (firstProd instanceof gast_2.Alternative ||
	        firstProd instanceof gast_2.Option ||
	        firstProd instanceof gast_2.RepetitionMandatory ||
	        firstProd instanceof gast_2.RepetitionMandatoryWithSeparator ||
	        firstProd instanceof gast_2.RepetitionWithSeparator ||
	        firstProd instanceof gast_2.Repetition) {
	        result = result.concat(getFirstNoneTerminal(firstProd.definition));
	    }
	    else if (firstProd instanceof gast_2.Alternation) {
	        // each sub definition in alternation is a FLAT
	        result = (0, flatten_1.default)((0, map_1.default)(firstProd.definition, function (currSubDef) {
	            return getFirstNoneTerminal(currSubDef.definition);
	        }));
	    }
	    else if (firstProd instanceof gast_2.Terminal) ;
	    else {
	        throw Error("non exhaustive match");
	    }
	    var isFirstOptional = (0, gast_1.isOptionalProd)(firstProd);
	    var hasMore = definition.length > 1;
	    if (isFirstOptional && hasMore) {
	        var rest = (0, drop_1.default)(definition);
	        return result.concat(getFirstNoneTerminal(rest));
	    }
	    else {
	        return result;
	    }
	}
	checks.getFirstNoneTerminal = getFirstNoneTerminal;
	var OrCollector = /** @class */ (function (_super) {
	    __extends(OrCollector, _super);
	    function OrCollector() {
	        var _this = _super !== null && _super.apply(this, arguments) || this;
	        _this.alternations = [];
	        return _this;
	    }
	    OrCollector.prototype.visitAlternation = function (node) {
	        this.alternations.push(node);
	    };
	    return OrCollector;
	}(gast_3.GAstVisitor));
	function validateEmptyOrAlternative(topLevelRule, errMsgProvider) {
	    var orCollector = new OrCollector();
	    topLevelRule.accept(orCollector);
	    var ors = orCollector.alternations;
	    var errors = (0, flatMap_1.default)(ors, function (currOr) {
	        var exceptLast = (0, dropRight_1.default)(currOr.definition);
	        return (0, flatMap_1.default)(exceptLast, function (currAlternative, currAltIdx) {
	            var possibleFirstInAlt = (0, interpreter_1.nextPossibleTokensAfter)([currAlternative], [], tokens_1.tokenStructuredMatcher, 1);
	            if ((0, isEmpty_1.default)(possibleFirstInAlt)) {
	                return [
	                    {
	                        message: errMsgProvider.buildEmptyAlternationError({
	                            topLevelRule: topLevelRule,
	                            alternation: currOr,
	                            emptyChoiceIdx: currAltIdx
	                        }),
	                        type: parser_1.ParserDefinitionErrorType.NONE_LAST_EMPTY_ALT,
	                        ruleName: topLevelRule.name,
	                        occurrence: currOr.idx,
	                        alternative: currAltIdx + 1
	                    }
	                ];
	            }
	            else {
	                return [];
	            }
	        });
	    });
	    return errors;
	}
	checks.validateEmptyOrAlternative = validateEmptyOrAlternative;
	function validateAmbiguousAlternationAlternatives(topLevelRule, globalMaxLookahead, errMsgProvider) {
	    var orCollector = new OrCollector();
	    topLevelRule.accept(orCollector);
	    var ors = orCollector.alternations;
	    // New Handling of ignoring ambiguities
	    // - https://github.com/chevrotain/chevrotain/issues/869
	    ors = (0, reject_1.default)(ors, function (currOr) { return currOr.ignoreAmbiguities === true; });
	    var errors = (0, flatMap_1.default)(ors, function (currOr) {
	        var currOccurrence = currOr.idx;
	        var actualMaxLookahead = currOr.maxLookahead || globalMaxLookahead;
	        var alternatives = (0, lookahead_1.getLookaheadPathsForOr)(currOccurrence, topLevelRule, actualMaxLookahead, currOr);
	        var altsAmbiguityErrors = checkAlternativesAmbiguities(alternatives, currOr, topLevelRule, errMsgProvider);
	        var altsPrefixAmbiguityErrors = checkPrefixAlternativesAmbiguities(alternatives, currOr, topLevelRule, errMsgProvider);
	        return altsAmbiguityErrors.concat(altsPrefixAmbiguityErrors);
	    });
	    return errors;
	}
	checks.validateAmbiguousAlternationAlternatives = validateAmbiguousAlternationAlternatives;
	var RepetitionCollector = /** @class */ (function (_super) {
	    __extends(RepetitionCollector, _super);
	    function RepetitionCollector() {
	        var _this = _super !== null && _super.apply(this, arguments) || this;
	        _this.allProductions = [];
	        return _this;
	    }
	    RepetitionCollector.prototype.visitRepetitionWithSeparator = function (manySep) {
	        this.allProductions.push(manySep);
	    };
	    RepetitionCollector.prototype.visitRepetitionMandatory = function (atLeastOne) {
	        this.allProductions.push(atLeastOne);
	    };
	    RepetitionCollector.prototype.visitRepetitionMandatoryWithSeparator = function (atLeastOneSep) {
	        this.allProductions.push(atLeastOneSep);
	    };
	    RepetitionCollector.prototype.visitRepetition = function (many) {
	        this.allProductions.push(many);
	    };
	    return RepetitionCollector;
	}(gast_3.GAstVisitor));
	checks.RepetitionCollector = RepetitionCollector;
	function validateTooManyAlts(topLevelRule, errMsgProvider) {
	    var orCollector = new OrCollector();
	    topLevelRule.accept(orCollector);
	    var ors = orCollector.alternations;
	    var errors = (0, flatMap_1.default)(ors, function (currOr) {
	        if (currOr.definition.length > 255) {
	            return [
	                {
	                    message: errMsgProvider.buildTooManyAlternativesError({
	                        topLevelRule: topLevelRule,
	                        alternation: currOr
	                    }),
	                    type: parser_1.ParserDefinitionErrorType.TOO_MANY_ALTS,
	                    ruleName: topLevelRule.name,
	                    occurrence: currOr.idx
	                }
	            ];
	        }
	        else {
	            return [];
	        }
	    });
	    return errors;
	}
	checks.validateTooManyAlts = validateTooManyAlts;
	function validateSomeNonEmptyLookaheadPath(topLevelRules, maxLookahead, errMsgProvider) {
	    var errors = [];
	    (0, forEach_1.default)(topLevelRules, function (currTopRule) {
	        var collectorVisitor = new RepetitionCollector();
	        currTopRule.accept(collectorVisitor);
	        var allRuleProductions = collectorVisitor.allProductions;
	        (0, forEach_1.default)(allRuleProductions, function (currProd) {
	            var prodType = (0, lookahead_1.getProdType)(currProd);
	            var actualMaxLookahead = currProd.maxLookahead || maxLookahead;
	            var currOccurrence = currProd.idx;
	            var paths = (0, lookahead_1.getLookaheadPathsForOptionalProd)(currOccurrence, currTopRule, prodType, actualMaxLookahead);
	            var pathsInsideProduction = paths[0];
	            if ((0, isEmpty_1.default)((0, flatten_1.default)(pathsInsideProduction))) {
	                var errMsg = errMsgProvider.buildEmptyRepetitionError({
	                    topLevelRule: currTopRule,
	                    repetition: currProd
	                });
	                errors.push({
	                    message: errMsg,
	                    type: parser_1.ParserDefinitionErrorType.NO_NON_EMPTY_LOOKAHEAD,
	                    ruleName: currTopRule.name
	                });
	            }
	        });
	    });
	    return errors;
	}
	checks.validateSomeNonEmptyLookaheadPath = validateSomeNonEmptyLookaheadPath;
	function checkAlternativesAmbiguities(alternatives, alternation, rule, errMsgProvider) {
	    var foundAmbiguousPaths = [];
	    var identicalAmbiguities = (0, reduce_1.default)(alternatives, function (result, currAlt, currAltIdx) {
	        // ignore (skip) ambiguities with this alternative
	        if (alternation.definition[currAltIdx].ignoreAmbiguities === true) {
	            return result;
	        }
	        (0, forEach_1.default)(currAlt, function (currPath) {
	            var altsCurrPathAppearsIn = [currAltIdx];
	            (0, forEach_1.default)(alternatives, function (currOtherAlt, currOtherAltIdx) {
	                if (currAltIdx !== currOtherAltIdx &&
	                    (0, lookahead_1.containsPath)(currOtherAlt, currPath) &&
	                    // ignore (skip) ambiguities with this "other" alternative
	                    alternation.definition[currOtherAltIdx].ignoreAmbiguities !== true) {
	                    altsCurrPathAppearsIn.push(currOtherAltIdx);
	                }
	            });
	            if (altsCurrPathAppearsIn.length > 1 &&
	                !(0, lookahead_1.containsPath)(foundAmbiguousPaths, currPath)) {
	                foundAmbiguousPaths.push(currPath);
	                result.push({
	                    alts: altsCurrPathAppearsIn,
	                    path: currPath
	                });
	            }
	        });
	        return result;
	    }, []);
	    var currErrors = (0, map_1.default)(identicalAmbiguities, function (currAmbDescriptor) {
	        var ambgIndices = (0, map_1.default)(currAmbDescriptor.alts, function (currAltIdx) { return currAltIdx + 1; });
	        var currMessage = errMsgProvider.buildAlternationAmbiguityError({
	            topLevelRule: rule,
	            alternation: alternation,
	            ambiguityIndices: ambgIndices,
	            prefixPath: currAmbDescriptor.path
	        });
	        return {
	            message: currMessage,
	            type: parser_1.ParserDefinitionErrorType.AMBIGUOUS_ALTS,
	            ruleName: rule.name,
	            occurrence: alternation.idx,
	            alternatives: currAmbDescriptor.alts
	        };
	    });
	    return currErrors;
	}
	function checkPrefixAlternativesAmbiguities(alternatives, alternation, rule, errMsgProvider) {
	    // flatten
	    var pathsAndIndices = (0, reduce_1.default)(alternatives, function (result, currAlt, idx) {
	        var currPathsAndIdx = (0, map_1.default)(currAlt, function (currPath) {
	            return { idx: idx, path: currPath };
	        });
	        return result.concat(currPathsAndIdx);
	    }, []);
	    var errors = (0, compact_1.default)((0, flatMap_1.default)(pathsAndIndices, function (currPathAndIdx) {
	        var alternativeGast = alternation.definition[currPathAndIdx.idx];
	        // ignore (skip) ambiguities with this alternative
	        if (alternativeGast.ignoreAmbiguities === true) {
	            return [];
	        }
	        var targetIdx = currPathAndIdx.idx;
	        var targetPath = currPathAndIdx.path;
	        var prefixAmbiguitiesPathsAndIndices = (0, filter_1.default)(pathsAndIndices, function (searchPathAndIdx) {
	            // prefix ambiguity can only be created from lower idx (higher priority) path
	            return (
	            // ignore (skip) ambiguities with this "other" alternative
	            alternation.definition[searchPathAndIdx.idx].ignoreAmbiguities !==
	                true &&
	                searchPathAndIdx.idx < targetIdx &&
	                // checking for strict prefix because identical lookaheads
	                // will be be detected using a different validation.
	                (0, lookahead_1.isStrictPrefixOfPath)(searchPathAndIdx.path, targetPath));
	        });
	        var currPathPrefixErrors = (0, map_1.default)(prefixAmbiguitiesPathsAndIndices, function (currAmbPathAndIdx) {
	            var ambgIndices = [currAmbPathAndIdx.idx + 1, targetIdx + 1];
	            var occurrence = alternation.idx === 0 ? "" : alternation.idx;
	            var message = errMsgProvider.buildAlternationPrefixAmbiguityError({
	                topLevelRule: rule,
	                alternation: alternation,
	                ambiguityIndices: ambgIndices,
	                prefixPath: currAmbPathAndIdx.path
	            });
	            return {
	                message: message,
	                type: parser_1.ParserDefinitionErrorType.AMBIGUOUS_PREFIX_ALTS,
	                ruleName: rule.name,
	                occurrence: occurrence,
	                alternatives: ambgIndices
	            };
	        });
	        return currPathPrefixErrors;
	    }));
	    return errors;
	}
	checks.checkPrefixAlternativesAmbiguities = checkPrefixAlternativesAmbiguities;
	function checkTerminalAndNoneTerminalsNameSpace(topLevels, tokenTypes, errMsgProvider) {
	    var errors = [];
	    var tokenNames = (0, map_1.default)(tokenTypes, function (currToken) { return currToken.name; });
	    (0, forEach_1.default)(topLevels, function (currRule) {
	        var currRuleName = currRule.name;
	        if ((0, includes_1.default)(tokenNames, currRuleName)) {
	            var errMsg = errMsgProvider.buildNamespaceConflictError(currRule);
	            errors.push({
	                message: errMsg,
	                type: parser_1.ParserDefinitionErrorType.CONFLICT_TOKENS_RULES_NAMESPACE,
	                ruleName: currRuleName
	            });
	        }
	    });
	    return errors;
	}
	
	return checks;
}

var hasRequiredGast_resolver_public;

function requireGast_resolver_public () {
	if (hasRequiredGast_resolver_public) return gast_resolver_public;
	hasRequiredGast_resolver_public = 1;
	var __importDefault = (gast_resolver_public && gast_resolver_public.__importDefault) || function (mod) {
	    return (mod && mod.__esModule) ? mod : { "default": mod };
	};
	Object.defineProperty(gast_resolver_public, "__esModule", { value: true });
	gast_resolver_public.validateGrammar = gast_resolver_public.resolveGrammar = void 0;
	var forEach_1 = __importDefault(requireForEach());
	var defaults_1 = __importDefault(requireDefaults());
	var resolver_1 = requireResolver();
	var checks_1 = requireChecks();
	var errors_public_1 = requireErrors_public();
	function resolveGrammar(options) {
	    var actualOptions = (0, defaults_1.default)(options, {
	        errMsgProvider: errors_public_1.defaultGrammarResolverErrorProvider
	    });
	    var topRulesTable = {};
	    (0, forEach_1.default)(options.rules, function (rule) {
	        topRulesTable[rule.name] = rule;
	    });
	    return (0, resolver_1.resolveGrammar)(topRulesTable, actualOptions.errMsgProvider);
	}
	gast_resolver_public.resolveGrammar = resolveGrammar;
	function validateGrammar(options) {
	    options = (0, defaults_1.default)(options, {
	        errMsgProvider: errors_public_1.defaultGrammarValidatorErrorProvider
	    });
	    return (0, checks_1.validateGrammar)(options.rules, options.tokenTypes, options.errMsgProvider, options.grammarName);
	}
	gast_resolver_public.validateGrammar = validateGrammar;
	
	return gast_resolver_public;
}

var recoverable = {};

var exceptions_public = {};

var hasRequiredExceptions_public;

function requireExceptions_public () {
	if (hasRequiredExceptions_public) return exceptions_public;
	hasRequiredExceptions_public = 1;
	var __extends = (exceptions_public && exceptions_public.__extends) || (function () {
	    var extendStatics = function (d, b) {
	        extendStatics = Object.setPrototypeOf ||
	            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
	            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
	        return extendStatics(d, b);
	    };
	    return function (d, b) {
	        if (typeof b !== "function" && b !== null)
	            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
	        extendStatics(d, b);
	        function __() { this.constructor = d; }
	        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
	    };
	})();
	var __importDefault = (exceptions_public && exceptions_public.__importDefault) || function (mod) {
	    return (mod && mod.__esModule) ? mod : { "default": mod };
	};
	Object.defineProperty(exceptions_public, "__esModule", { value: true });
	exceptions_public.EarlyExitException = exceptions_public.NotAllInputParsedException = exceptions_public.NoViableAltException = exceptions_public.MismatchedTokenException = exceptions_public.isRecognitionException = void 0;
	var includes_1 = __importDefault(requireIncludes());
	var MISMATCHED_TOKEN_EXCEPTION = "MismatchedTokenException";
	var NO_VIABLE_ALT_EXCEPTION = "NoViableAltException";
	var EARLY_EXIT_EXCEPTION = "EarlyExitException";
	var NOT_ALL_INPUT_PARSED_EXCEPTION = "NotAllInputParsedException";
	var RECOGNITION_EXCEPTION_NAMES = [
	    MISMATCHED_TOKEN_EXCEPTION,
	    NO_VIABLE_ALT_EXCEPTION,
	    EARLY_EXIT_EXCEPTION,
	    NOT_ALL_INPUT_PARSED_EXCEPTION
	];
	Object.freeze(RECOGNITION_EXCEPTION_NAMES);
	// hacks to bypass no support for custom Errors in javascript/typescript
	function isRecognitionException(error) {
	    // can't do instanceof on hacked custom js exceptions
	    return (0, includes_1.default)(RECOGNITION_EXCEPTION_NAMES, error.name);
	}
	exceptions_public.isRecognitionException = isRecognitionException;
	var RecognitionException = /** @class */ (function (_super) {
	    __extends(RecognitionException, _super);
	    function RecognitionException(message, token) {
	        var _newTarget = this.constructor;
	        var _this = _super.call(this, message) || this;
	        _this.token = token;
	        _this.resyncedTokens = [];
	        // fix prototype chain when typescript target is ES5
	        Object.setPrototypeOf(_this, _newTarget.prototype);
	        /* istanbul ignore next - V8 workaround to remove constructor from stacktrace when typescript target is ES5 */
	        if (Error.captureStackTrace) {
	            Error.captureStackTrace(_this, _this.constructor);
	        }
	        return _this;
	    }
	    return RecognitionException;
	}(Error));
	var MismatchedTokenException = /** @class */ (function (_super) {
	    __extends(MismatchedTokenException, _super);
	    function MismatchedTokenException(message, token, previousToken) {
	        var _this = _super.call(this, message, token) || this;
	        _this.previousToken = previousToken;
	        _this.name = MISMATCHED_TOKEN_EXCEPTION;
	        return _this;
	    }
	    return MismatchedTokenException;
	}(RecognitionException));
	exceptions_public.MismatchedTokenException = MismatchedTokenException;
	var NoViableAltException = /** @class */ (function (_super) {
	    __extends(NoViableAltException, _super);
	    function NoViableAltException(message, token, previousToken) {
	        var _this = _super.call(this, message, token) || this;
	        _this.previousToken = previousToken;
	        _this.name = NO_VIABLE_ALT_EXCEPTION;
	        return _this;
	    }
	    return NoViableAltException;
	}(RecognitionException));
	exceptions_public.NoViableAltException = NoViableAltException;
	var NotAllInputParsedException = /** @class */ (function (_super) {
	    __extends(NotAllInputParsedException, _super);
	    function NotAllInputParsedException(message, token) {
	        var _this = _super.call(this, message, token) || this;
	        _this.name = NOT_ALL_INPUT_PARSED_EXCEPTION;
	        return _this;
	    }
	    return NotAllInputParsedException;
	}(RecognitionException));
	exceptions_public.NotAllInputParsedException = NotAllInputParsedException;
	var EarlyExitException = /** @class */ (function (_super) {
	    __extends(EarlyExitException, _super);
	    function EarlyExitException(message, token, previousToken) {
	        var _this = _super.call(this, message, token) || this;
	        _this.previousToken = previousToken;
	        _this.name = EARLY_EXIT_EXCEPTION;
	        return _this;
	    }
	    return EarlyExitException;
	}(RecognitionException));
	exceptions_public.EarlyExitException = EarlyExitException;
	
	return exceptions_public;
}

var hasRequiredRecoverable;

function requireRecoverable () {
	if (hasRequiredRecoverable) return recoverable;
	hasRequiredRecoverable = 1;
	(function (exports) {
		var __extends = (recoverable && recoverable.__extends) || (function () {
		    var extendStatics = function (d, b) {
		        extendStatics = Object.setPrototypeOf ||
		            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
		            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
		        return extendStatics(d, b);
		    };
		    return function (d, b) {
		        if (typeof b !== "function" && b !== null)
		            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
		        extendStatics(d, b);
		        function __() { this.constructor = d; }
		        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
		    };
		})();
		var __importDefault = (recoverable && recoverable.__importDefault) || function (mod) {
		    return (mod && mod.__esModule) ? mod : { "default": mod };
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.attemptInRepetitionRecovery = exports.Recoverable = exports.InRuleRecoveryException = exports.IN_RULE_RECOVERY_EXCEPTION = exports.EOF_FOLLOW_KEY = void 0;
		var tokens_public_1 = requireTokens_public();
		var isEmpty_1 = __importDefault(requireIsEmpty());
		var dropRight_1 = __importDefault(requireDropRight());
		var flatten_1 = __importDefault(requireFlatten());
		var map_1 = __importDefault(requireMap());
		var find_1 = __importDefault(requireFind());
		var has_1 = __importDefault(requireHas());
		var includes_1 = __importDefault(requireIncludes());
		var clone_1 = __importDefault(requireClone());
		var exceptions_public_1 = requireExceptions_public();
		var constants_1 = requireConstants();
		var parser_1 = requireParser();
		exports.EOF_FOLLOW_KEY = {};
		exports.IN_RULE_RECOVERY_EXCEPTION = "InRuleRecoveryException";
		var InRuleRecoveryException = /** @class */ (function (_super) {
		    __extends(InRuleRecoveryException, _super);
		    function InRuleRecoveryException(message) {
		        var _this = _super.call(this, message) || this;
		        _this.name = exports.IN_RULE_RECOVERY_EXCEPTION;
		        return _this;
		    }
		    return InRuleRecoveryException;
		}(Error));
		exports.InRuleRecoveryException = InRuleRecoveryException;
		/**
		 * This trait is responsible for the error recovery and fault tolerant logic
		 */
		var Recoverable = /** @class */ (function () {
		    function Recoverable() {
		    }
		    Recoverable.prototype.initRecoverable = function (config) {
		        this.firstAfterRepMap = {};
		        this.resyncFollows = {};
		        this.recoveryEnabled = (0, has_1.default)(config, "recoveryEnabled")
		            ? config.recoveryEnabled // assumes end user provides the correct config value/type
		            : parser_1.DEFAULT_PARSER_CONFIG.recoveryEnabled;
		        // performance optimization, NOOP will be inlined which
		        // effectively means that this optional feature does not exist
		        // when not used.
		        if (this.recoveryEnabled) {
		            this.attemptInRepetitionRecovery = attemptInRepetitionRecovery;
		        }
		    };
		    Recoverable.prototype.getTokenToInsert = function (tokType) {
		        var tokToInsert = (0, tokens_public_1.createTokenInstance)(tokType, "", NaN, NaN, NaN, NaN, NaN, NaN);
		        tokToInsert.isInsertedInRecovery = true;
		        return tokToInsert;
		    };
		    Recoverable.prototype.canTokenTypeBeInsertedInRecovery = function (tokType) {
		        return true;
		    };
		    Recoverable.prototype.canTokenTypeBeDeletedInRecovery = function (tokType) {
		        return true;
		    };
		    Recoverable.prototype.tryInRepetitionRecovery = function (grammarRule, grammarRuleArgs, lookAheadFunc, expectedTokType) {
		        var _this = this;
		        // TODO: can the resyncTokenType be cached?
		        var reSyncTokType = this.findReSyncTokenType();
		        var savedLexerState = this.exportLexerState();
		        var resyncedTokens = [];
		        var passedResyncPoint = false;
		        var nextTokenWithoutResync = this.LA(1);
		        var currToken = this.LA(1);
		        var generateErrorMessage = function () {
		            var previousToken = _this.LA(0);
		            // we are preemptively re-syncing before an error has been detected, therefor we must reproduce
		            // the error that would have been thrown
		            var msg = _this.errorMessageProvider.buildMismatchTokenMessage({
		                expected: expectedTokType,
		                actual: nextTokenWithoutResync,
		                previous: previousToken,
		                ruleName: _this.getCurrRuleFullName()
		            });
		            var error = new exceptions_public_1.MismatchedTokenException(msg, nextTokenWithoutResync, _this.LA(0));
		            // the first token here will be the original cause of the error, this is not part of the resyncedTokens property.
		            error.resyncedTokens = (0, dropRight_1.default)(resyncedTokens);
		            _this.SAVE_ERROR(error);
		        };
		        while (!passedResyncPoint) {
		            // re-synced to a point where we can safely exit the repetition/
		            if (this.tokenMatcher(currToken, expectedTokType)) {
		                generateErrorMessage();
		                return; // must return here to avoid reverting the inputIdx
		            }
		            else if (lookAheadFunc.call(this)) {
		                // we skipped enough tokens so we can resync right back into another iteration of the repetition grammar rule
		                generateErrorMessage();
		                // recursive invocation in other to support multiple re-syncs in the same top level repetition grammar rule
		                grammarRule.apply(this, grammarRuleArgs);
		                return; // must return here to avoid reverting the inputIdx
		            }
		            else if (this.tokenMatcher(currToken, reSyncTokType)) {
		                passedResyncPoint = true;
		            }
		            else {
		                currToken = this.SKIP_TOKEN();
		                this.addToResyncTokens(currToken, resyncedTokens);
		            }
		        }
		        // we were unable to find a CLOSER point to resync inside the Repetition, reset the state.
		        // The parsing exception we were trying to prevent will happen in the NEXT parsing step. it may be handled by
		        // "between rules" resync recovery later in the flow.
		        this.importLexerState(savedLexerState);
		    };
		    Recoverable.prototype.shouldInRepetitionRecoveryBeTried = function (expectTokAfterLastMatch, nextTokIdx, notStuck) {
		        // Edge case of arriving from a MANY repetition which is stuck
		        // Attempting recovery in this case could cause an infinite loop
		        if (notStuck === false) {
		            return false;
		        }
		        // no need to recover, next token is what we expect...
		        if (this.tokenMatcher(this.LA(1), expectTokAfterLastMatch)) {
		            return false;
		        }
		        // error recovery is disabled during backtracking as it can make the parser ignore a valid grammar path
		        // and prefer some backtracking path that includes recovered errors.
		        if (this.isBackTracking()) {
		            return false;
		        }
		        // if we can perform inRule recovery (single token insertion or deletion) we always prefer that recovery algorithm
		        // because if it works, it makes the least amount of changes to the input stream (greedy algorithm)
		        //noinspection RedundantIfStatementJS
		        if (this.canPerformInRuleRecovery(expectTokAfterLastMatch, this.getFollowsForInRuleRecovery(expectTokAfterLastMatch, nextTokIdx))) {
		            return false;
		        }
		        return true;
		    };
		    // Error Recovery functionality
		    Recoverable.prototype.getFollowsForInRuleRecovery = function (tokType, tokIdxInRule) {
		        var grammarPath = this.getCurrentGrammarPath(tokType, tokIdxInRule);
		        var follows = this.getNextPossibleTokenTypes(grammarPath);
		        return follows;
		    };
		    Recoverable.prototype.tryInRuleRecovery = function (expectedTokType, follows) {
		        if (this.canRecoverWithSingleTokenInsertion(expectedTokType, follows)) {
		            var tokToInsert = this.getTokenToInsert(expectedTokType);
		            return tokToInsert;
		        }
		        if (this.canRecoverWithSingleTokenDeletion(expectedTokType)) {
		            var nextTok = this.SKIP_TOKEN();
		            this.consumeToken();
		            return nextTok;
		        }
		        throw new InRuleRecoveryException("sad sad panda");
		    };
		    Recoverable.prototype.canPerformInRuleRecovery = function (expectedToken, follows) {
		        return (this.canRecoverWithSingleTokenInsertion(expectedToken, follows) ||
		            this.canRecoverWithSingleTokenDeletion(expectedToken));
		    };
		    Recoverable.prototype.canRecoverWithSingleTokenInsertion = function (expectedTokType, follows) {
		        var _this = this;
		        if (!this.canTokenTypeBeInsertedInRecovery(expectedTokType)) {
		            return false;
		        }
		        // must know the possible following tokens to perform single token insertion
		        if ((0, isEmpty_1.default)(follows)) {
		            return false;
		        }
		        var mismatchedTok = this.LA(1);
		        var isMisMatchedTokInFollows = (0, find_1.default)(follows, function (possibleFollowsTokType) {
		            return _this.tokenMatcher(mismatchedTok, possibleFollowsTokType);
		        }) !== undefined;
		        return isMisMatchedTokInFollows;
		    };
		    Recoverable.prototype.canRecoverWithSingleTokenDeletion = function (expectedTokType) {
		        if (!this.canTokenTypeBeDeletedInRecovery(expectedTokType)) {
		            return false;
		        }
		        var isNextTokenWhatIsExpected = this.tokenMatcher(this.LA(2), expectedTokType);
		        return isNextTokenWhatIsExpected;
		    };
		    Recoverable.prototype.isInCurrentRuleReSyncSet = function (tokenTypeIdx) {
		        var followKey = this.getCurrFollowKey();
		        var currentRuleReSyncSet = this.getFollowSetFromFollowKey(followKey);
		        return (0, includes_1.default)(currentRuleReSyncSet, tokenTypeIdx);
		    };
		    Recoverable.prototype.findReSyncTokenType = function () {
		        var allPossibleReSyncTokTypes = this.flattenFollowSet();
		        // this loop will always terminate as EOF is always in the follow stack and also always (virtually) in the input
		        var nextToken = this.LA(1);
		        var k = 2;
		        while (true) {
		            var foundMatch = (0, find_1.default)(allPossibleReSyncTokTypes, function (resyncTokType) {
		                var canMatch = (0, tokens_public_1.tokenMatcher)(nextToken, resyncTokType);
		                return canMatch;
		            });
		            if (foundMatch !== undefined) {
		                return foundMatch;
		            }
		            nextToken = this.LA(k);
		            k++;
		        }
		    };
		    Recoverable.prototype.getCurrFollowKey = function () {
		        // the length is at least one as we always add the ruleName to the stack before invoking the rule.
		        if (this.RULE_STACK.length === 1) {
		            return exports.EOF_FOLLOW_KEY;
		        }
		        var currRuleShortName = this.getLastExplicitRuleShortName();
		        var currRuleIdx = this.getLastExplicitRuleOccurrenceIndex();
		        var prevRuleShortName = this.getPreviousExplicitRuleShortName();
		        return {
		            ruleName: this.shortRuleNameToFullName(currRuleShortName),
		            idxInCallingRule: currRuleIdx,
		            inRule: this.shortRuleNameToFullName(prevRuleShortName)
		        };
		    };
		    Recoverable.prototype.buildFullFollowKeyStack = function () {
		        var _this = this;
		        var explicitRuleStack = this.RULE_STACK;
		        var explicitOccurrenceStack = this.RULE_OCCURRENCE_STACK;
		        return (0, map_1.default)(explicitRuleStack, function (ruleName, idx) {
		            if (idx === 0) {
		                return exports.EOF_FOLLOW_KEY;
		            }
		            return {
		                ruleName: _this.shortRuleNameToFullName(ruleName),
		                idxInCallingRule: explicitOccurrenceStack[idx],
		                inRule: _this.shortRuleNameToFullName(explicitRuleStack[idx - 1])
		            };
		        });
		    };
		    Recoverable.prototype.flattenFollowSet = function () {
		        var _this = this;
		        var followStack = (0, map_1.default)(this.buildFullFollowKeyStack(), function (currKey) {
		            return _this.getFollowSetFromFollowKey(currKey);
		        });
		        return (0, flatten_1.default)(followStack);
		    };
		    Recoverable.prototype.getFollowSetFromFollowKey = function (followKey) {
		        if (followKey === exports.EOF_FOLLOW_KEY) {
		            return [tokens_public_1.EOF];
		        }
		        var followName = followKey.ruleName + followKey.idxInCallingRule + constants_1.IN + followKey.inRule;
		        return this.resyncFollows[followName];
		    };
		    // It does not make any sense to include a virtual EOF token in the list of resynced tokens
		    // as EOF does not really exist and thus does not contain any useful information (line/column numbers)
		    Recoverable.prototype.addToResyncTokens = function (token, resyncTokens) {
		        if (!this.tokenMatcher(token, tokens_public_1.EOF)) {
		            resyncTokens.push(token);
		        }
		        return resyncTokens;
		    };
		    Recoverable.prototype.reSyncTo = function (tokType) {
		        var resyncedTokens = [];
		        var nextTok = this.LA(1);
		        while (this.tokenMatcher(nextTok, tokType) === false) {
		            nextTok = this.SKIP_TOKEN();
		            this.addToResyncTokens(nextTok, resyncedTokens);
		        }
		        // the last token is not part of the error.
		        return (0, dropRight_1.default)(resyncedTokens);
		    };
		    Recoverable.prototype.attemptInRepetitionRecovery = function (prodFunc, args, lookaheadFunc, dslMethodIdx, prodOccurrence, nextToksWalker, notStuck) {
		        // by default this is a NO-OP
		        // The actual implementation is with the function(not method) below
		    };
		    Recoverable.prototype.getCurrentGrammarPath = function (tokType, tokIdxInRule) {
		        var pathRuleStack = this.getHumanReadableRuleStack();
		        var pathOccurrenceStack = (0, clone_1.default)(this.RULE_OCCURRENCE_STACK);
		        var grammarPath = {
		            ruleStack: pathRuleStack,
		            occurrenceStack: pathOccurrenceStack,
		            lastTok: tokType,
		            lastTokOccurrence: tokIdxInRule
		        };
		        return grammarPath;
		    };
		    Recoverable.prototype.getHumanReadableRuleStack = function () {
		        var _this = this;
		        return (0, map_1.default)(this.RULE_STACK, function (currShortName) {
		            return _this.shortRuleNameToFullName(currShortName);
		        });
		    };
		    return Recoverable;
		}());
		exports.Recoverable = Recoverable;
		function attemptInRepetitionRecovery(prodFunc, args, lookaheadFunc, dslMethodIdx, prodOccurrence, nextToksWalker, notStuck) {
		    var key = this.getKeyForAutomaticLookahead(dslMethodIdx, prodOccurrence);
		    var firstAfterRepInfo = this.firstAfterRepMap[key];
		    if (firstAfterRepInfo === undefined) {
		        var currRuleName = this.getCurrRuleFullName();
		        var ruleGrammar = this.getGAstProductions()[currRuleName];
		        var walker = new nextToksWalker(ruleGrammar, prodOccurrence);
		        firstAfterRepInfo = walker.startWalking();
		        this.firstAfterRepMap[key] = firstAfterRepInfo;
		    }
		    var expectTokAfterLastMatch = firstAfterRepInfo.token;
		    var nextTokIdx = firstAfterRepInfo.occurrence;
		    var isEndOfRule = firstAfterRepInfo.isEndOfRule;
		    // special edge case of a TOP most repetition after which the input should END.
		    // this will force an attempt for inRule recovery in that scenario.
		    if (this.RULE_STACK.length === 1 &&
		        isEndOfRule &&
		        expectTokAfterLastMatch === undefined) {
		        expectTokAfterLastMatch = tokens_public_1.EOF;
		        nextTokIdx = 1;
		    }
		    // We don't have anything to re-sync to...
		    // this condition was extracted from `shouldInRepetitionRecoveryBeTried` to act as a type-guard
		    if (expectTokAfterLastMatch === undefined || nextTokIdx === undefined) {
		        return;
		    }
		    if (this.shouldInRepetitionRecoveryBeTried(expectTokAfterLastMatch, nextTokIdx, notStuck)) {
		        // TODO: performance optimization: instead of passing the original args here, we modify
		        // the args param (or create a new one) and make sure the lookahead func is explicitly provided
		        // to avoid searching the cache for it once more.
		        this.tryInRepetitionRecovery(prodFunc, args, lookaheadFunc, expectTokAfterLastMatch);
		    }
		}
		exports.attemptInRepetitionRecovery = attemptInRepetitionRecovery;
		
	} (recoverable));
	return recoverable;
}

var looksahead = {};

var keys = {};

var hasRequiredKeys;

function requireKeys () {
	if (hasRequiredKeys) return keys;
	hasRequiredKeys = 1;
	(function (exports) {
		// Lookahead keys are 32Bit integers in the form
		// TTTTTTTT-ZZZZZZZZZZZZ-YYYY-XXXXXXXX
		// XXXX -> Occurrence Index bitmap.
		// YYYY -> DSL Method Type bitmap.
		// ZZZZZZZZZZZZZZZ -> Rule short Index bitmap.
		// TTTTTTTTT -> alternation alternative index bitmap
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.getKeyForAutomaticLookahead = exports.AT_LEAST_ONE_SEP_IDX = exports.MANY_SEP_IDX = exports.AT_LEAST_ONE_IDX = exports.MANY_IDX = exports.OPTION_IDX = exports.OR_IDX = exports.BITS_FOR_ALT_IDX = exports.BITS_FOR_RULE_IDX = exports.BITS_FOR_OCCURRENCE_IDX = exports.BITS_FOR_METHOD_TYPE = void 0;
		exports.BITS_FOR_METHOD_TYPE = 4;
		exports.BITS_FOR_OCCURRENCE_IDX = 8;
		exports.BITS_FOR_RULE_IDX = 12;
		// TODO: validation, this means that there may at most 2^8 --> 256 alternatives for an alternation.
		exports.BITS_FOR_ALT_IDX = 8;
		// short string used as part of mapping keys.
		// being short improves the performance when composing KEYS for maps out of these
		// The 5 - 8 bits (16 possible values, are reserved for the DSL method indices)
		exports.OR_IDX = 1 << exports.BITS_FOR_OCCURRENCE_IDX;
		exports.OPTION_IDX = 2 << exports.BITS_FOR_OCCURRENCE_IDX;
		exports.MANY_IDX = 3 << exports.BITS_FOR_OCCURRENCE_IDX;
		exports.AT_LEAST_ONE_IDX = 4 << exports.BITS_FOR_OCCURRENCE_IDX;
		exports.MANY_SEP_IDX = 5 << exports.BITS_FOR_OCCURRENCE_IDX;
		exports.AT_LEAST_ONE_SEP_IDX = 6 << exports.BITS_FOR_OCCURRENCE_IDX;
		// this actually returns a number, but it is always used as a string (object prop key)
		function getKeyForAutomaticLookahead(ruleIdx, dslMethodIdx, occurrence) {
		    return occurrence | dslMethodIdx | ruleIdx;
		}
		exports.getKeyForAutomaticLookahead = getKeyForAutomaticLookahead;
		32 - exports.BITS_FOR_ALT_IDX;
		
	} (keys));
	return keys;
}

var llk_lookahead = {};

var hasRequiredLlk_lookahead;

function requireLlk_lookahead () {
	if (hasRequiredLlk_lookahead) return llk_lookahead;
	hasRequiredLlk_lookahead = 1;
	var __spreadArray = (llk_lookahead && llk_lookahead.__spreadArray) || function (to, from, pack) {
	    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
	        if (ar || !(i in from)) {
	            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
	            ar[i] = from[i];
	        }
	    }
	    return to.concat(ar || Array.prototype.slice.call(from));
	};
	var __importDefault = (llk_lookahead && llk_lookahead.__importDefault) || function (mod) {
	    return (mod && mod.__esModule) ? mod : { "default": mod };
	};
	Object.defineProperty(llk_lookahead, "__esModule", { value: true });
	llk_lookahead.LLkLookaheadStrategy = void 0;
	var flatMap_1 = __importDefault(requireFlatMap());
	var isEmpty_1 = __importDefault(requireIsEmpty());
	var errors_public_1 = requireErrors_public();
	var parser_1 = requireParser();
	var checks_1 = requireChecks();
	var lookahead_1 = requireLookahead();
	var LLkLookaheadStrategy = /** @class */ (function () {
	    function LLkLookaheadStrategy(options) {
	        var _a;
	        this.maxLookahead =
	            (_a = options === null || options === void 0 ? void 0 : options.maxLookahead) !== null && _a !== void 0 ? _a : parser_1.DEFAULT_PARSER_CONFIG.maxLookahead;
	    }
	    LLkLookaheadStrategy.prototype.validate = function (options) {
	        var leftRecursionErrors = this.validateNoLeftRecursion(options.rules);
	        if ((0, isEmpty_1.default)(leftRecursionErrors)) {
	            var emptyAltErrors = this.validateEmptyOrAlternatives(options.rules);
	            var ambiguousAltsErrors = this.validateAmbiguousAlternationAlternatives(options.rules, this.maxLookahead);
	            var emptyRepetitionErrors = this.validateSomeNonEmptyLookaheadPath(options.rules, this.maxLookahead);
	            var allErrors = __spreadArray(__spreadArray(__spreadArray(__spreadArray([], leftRecursionErrors, true), emptyAltErrors, true), ambiguousAltsErrors, true), emptyRepetitionErrors, true);
	            return allErrors;
	        }
	        return leftRecursionErrors;
	    };
	    LLkLookaheadStrategy.prototype.validateNoLeftRecursion = function (rules) {
	        return (0, flatMap_1.default)(rules, function (currTopRule) {
	            return (0, checks_1.validateNoLeftRecursion)(currTopRule, currTopRule, errors_public_1.defaultGrammarValidatorErrorProvider);
	        });
	    };
	    LLkLookaheadStrategy.prototype.validateEmptyOrAlternatives = function (rules) {
	        return (0, flatMap_1.default)(rules, function (currTopRule) {
	            return (0, checks_1.validateEmptyOrAlternative)(currTopRule, errors_public_1.defaultGrammarValidatorErrorProvider);
	        });
	    };
	    LLkLookaheadStrategy.prototype.validateAmbiguousAlternationAlternatives = function (rules, maxLookahead) {
	        return (0, flatMap_1.default)(rules, function (currTopRule) {
	            return (0, checks_1.validateAmbiguousAlternationAlternatives)(currTopRule, maxLookahead, errors_public_1.defaultGrammarValidatorErrorProvider);
	        });
	    };
	    LLkLookaheadStrategy.prototype.validateSomeNonEmptyLookaheadPath = function (rules, maxLookahead) {
	        return (0, checks_1.validateSomeNonEmptyLookaheadPath)(rules, maxLookahead, errors_public_1.defaultGrammarValidatorErrorProvider);
	    };
	    LLkLookaheadStrategy.prototype.buildLookaheadForAlternation = function (options) {
	        return (0, lookahead_1.buildLookaheadFuncForOr)(options.prodOccurrence, options.rule, options.maxLookahead, options.hasPredicates, options.dynamicTokensEnabled, lookahead_1.buildAlternativesLookAheadFunc);
	    };
	    LLkLookaheadStrategy.prototype.buildLookaheadForOptional = function (options) {
	        return (0, lookahead_1.buildLookaheadFuncForOptionalProd)(options.prodOccurrence, options.rule, options.maxLookahead, options.dynamicTokensEnabled, (0, lookahead_1.getProdType)(options.prodType), lookahead_1.buildSingleAlternativeLookaheadFunction);
	    };
	    return LLkLookaheadStrategy;
	}());
	llk_lookahead.LLkLookaheadStrategy = LLkLookaheadStrategy;
	
	return llk_lookahead;
}

var hasRequiredLooksahead;

function requireLooksahead () {
	if (hasRequiredLooksahead) return looksahead;
	hasRequiredLooksahead = 1;
	var __extends = (looksahead && looksahead.__extends) || (function () {
	    var extendStatics = function (d, b) {
	        extendStatics = Object.setPrototypeOf ||
	            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
	            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
	        return extendStatics(d, b);
	    };
	    return function (d, b) {
	        if (typeof b !== "function" && b !== null)
	            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
	        extendStatics(d, b);
	        function __() { this.constructor = d; }
	        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
	    };
	})();
	var __importDefault = (looksahead && looksahead.__importDefault) || function (mod) {
	    return (mod && mod.__esModule) ? mod : { "default": mod };
	};
	Object.defineProperty(looksahead, "__esModule", { value: true });
	looksahead.collectMethods = looksahead.LooksAhead = void 0;
	var forEach_1 = __importDefault(requireForEach());
	var has_1 = __importDefault(requireHas());
	var parser_1 = requireParser();
	var keys_1 = requireKeys();
	var gast_1 = requireApi$2();
	var gast_2 = requireApi$2();
	var llk_lookahead_1 = requireLlk_lookahead();
	/**
	 * Trait responsible for the lookahead related utilities and optimizations.
	 */
	var LooksAhead = /** @class */ (function () {
	    function LooksAhead() {
	    }
	    LooksAhead.prototype.initLooksAhead = function (config) {
	        this.dynamicTokensEnabled = (0, has_1.default)(config, "dynamicTokensEnabled")
	            ? config.dynamicTokensEnabled // assumes end user provides the correct config value/type
	            : parser_1.DEFAULT_PARSER_CONFIG.dynamicTokensEnabled;
	        this.maxLookahead = (0, has_1.default)(config, "maxLookahead")
	            ? config.maxLookahead // assumes end user provides the correct config value/type
	            : parser_1.DEFAULT_PARSER_CONFIG.maxLookahead;
	        this.lookaheadStrategy = (0, has_1.default)(config, "lookaheadStrategy")
	            ? config.lookaheadStrategy // assumes end user provides the correct config value/type
	            : new llk_lookahead_1.LLkLookaheadStrategy({ maxLookahead: this.maxLookahead });
	        this.lookAheadFuncsCache = new Map();
	    };
	    LooksAhead.prototype.preComputeLookaheadFunctions = function (rules) {
	        var _this = this;
	        (0, forEach_1.default)(rules, function (currRule) {
	            _this.TRACE_INIT("".concat(currRule.name, " Rule Lookahead"), function () {
	                var _a = collectMethods(currRule), alternation = _a.alternation, repetition = _a.repetition, option = _a.option, repetitionMandatory = _a.repetitionMandatory, repetitionMandatoryWithSeparator = _a.repetitionMandatoryWithSeparator, repetitionWithSeparator = _a.repetitionWithSeparator;
	                (0, forEach_1.default)(alternation, function (currProd) {
	                    var prodIdx = currProd.idx === 0 ? "" : currProd.idx;
	                    _this.TRACE_INIT("".concat((0, gast_2.getProductionDslName)(currProd)).concat(prodIdx), function () {
	                        var laFunc = _this.lookaheadStrategy.buildLookaheadForAlternation({
	                            prodOccurrence: currProd.idx,
	                            rule: currRule,
	                            maxLookahead: currProd.maxLookahead || _this.maxLookahead,
	                            hasPredicates: currProd.hasPredicates,
	                            dynamicTokensEnabled: _this.dynamicTokensEnabled
	                        });
	                        var key = (0, keys_1.getKeyForAutomaticLookahead)(_this.fullRuleNameToShort[currRule.name], keys_1.OR_IDX, currProd.idx);
	                        _this.setLaFuncCache(key, laFunc);
	                    });
	                });
	                (0, forEach_1.default)(repetition, function (currProd) {
	                    _this.computeLookaheadFunc(currRule, currProd.idx, keys_1.MANY_IDX, "Repetition", currProd.maxLookahead, (0, gast_2.getProductionDslName)(currProd));
	                });
	                (0, forEach_1.default)(option, function (currProd) {
	                    _this.computeLookaheadFunc(currRule, currProd.idx, keys_1.OPTION_IDX, "Option", currProd.maxLookahead, (0, gast_2.getProductionDslName)(currProd));
	                });
	                (0, forEach_1.default)(repetitionMandatory, function (currProd) {
	                    _this.computeLookaheadFunc(currRule, currProd.idx, keys_1.AT_LEAST_ONE_IDX, "RepetitionMandatory", currProd.maxLookahead, (0, gast_2.getProductionDslName)(currProd));
	                });
	                (0, forEach_1.default)(repetitionMandatoryWithSeparator, function (currProd) {
	                    _this.computeLookaheadFunc(currRule, currProd.idx, keys_1.AT_LEAST_ONE_SEP_IDX, "RepetitionMandatoryWithSeparator", currProd.maxLookahead, (0, gast_2.getProductionDslName)(currProd));
	                });
	                (0, forEach_1.default)(repetitionWithSeparator, function (currProd) {
	                    _this.computeLookaheadFunc(currRule, currProd.idx, keys_1.MANY_SEP_IDX, "RepetitionWithSeparator", currProd.maxLookahead, (0, gast_2.getProductionDslName)(currProd));
	                });
	            });
	        });
	    };
	    LooksAhead.prototype.computeLookaheadFunc = function (rule, prodOccurrence, prodKey, prodType, prodMaxLookahead, dslMethodName) {
	        var _this = this;
	        this.TRACE_INIT("".concat(dslMethodName).concat(prodOccurrence === 0 ? "" : prodOccurrence), function () {
	            var laFunc = _this.lookaheadStrategy.buildLookaheadForOptional({
	                prodOccurrence: prodOccurrence,
	                rule: rule,
	                maxLookahead: prodMaxLookahead || _this.maxLookahead,
	                dynamicTokensEnabled: _this.dynamicTokensEnabled,
	                prodType: prodType
	            });
	            var key = (0, keys_1.getKeyForAutomaticLookahead)(_this.fullRuleNameToShort[rule.name], prodKey, prodOccurrence);
	            _this.setLaFuncCache(key, laFunc);
	        });
	    };
	    // this actually returns a number, but it is always used as a string (object prop key)
	    LooksAhead.prototype.getKeyForAutomaticLookahead = function (dslMethodIdx, occurrence) {
	        var currRuleShortName = this.getLastExplicitRuleShortName();
	        return (0, keys_1.getKeyForAutomaticLookahead)(currRuleShortName, dslMethodIdx, occurrence);
	    };
	    LooksAhead.prototype.getLaFuncFromCache = function (key) {
	        return this.lookAheadFuncsCache.get(key);
	    };
	    /* istanbul ignore next */
	    LooksAhead.prototype.setLaFuncCache = function (key, value) {
	        this.lookAheadFuncsCache.set(key, value);
	    };
	    return LooksAhead;
	}());
	looksahead.LooksAhead = LooksAhead;
	var DslMethodsCollectorVisitor = /** @class */ (function (_super) {
	    __extends(DslMethodsCollectorVisitor, _super);
	    function DslMethodsCollectorVisitor() {
	        var _this = _super !== null && _super.apply(this, arguments) || this;
	        _this.dslMethods = {
	            option: [],
	            alternation: [],
	            repetition: [],
	            repetitionWithSeparator: [],
	            repetitionMandatory: [],
	            repetitionMandatoryWithSeparator: []
	        };
	        return _this;
	    }
	    DslMethodsCollectorVisitor.prototype.reset = function () {
	        this.dslMethods = {
	            option: [],
	            alternation: [],
	            repetition: [],
	            repetitionWithSeparator: [],
	            repetitionMandatory: [],
	            repetitionMandatoryWithSeparator: []
	        };
	    };
	    DslMethodsCollectorVisitor.prototype.visitOption = function (option) {
	        this.dslMethods.option.push(option);
	    };
	    DslMethodsCollectorVisitor.prototype.visitRepetitionWithSeparator = function (manySep) {
	        this.dslMethods.repetitionWithSeparator.push(manySep);
	    };
	    DslMethodsCollectorVisitor.prototype.visitRepetitionMandatory = function (atLeastOne) {
	        this.dslMethods.repetitionMandatory.push(atLeastOne);
	    };
	    DslMethodsCollectorVisitor.prototype.visitRepetitionMandatoryWithSeparator = function (atLeastOneSep) {
	        this.dslMethods.repetitionMandatoryWithSeparator.push(atLeastOneSep);
	    };
	    DslMethodsCollectorVisitor.prototype.visitRepetition = function (many) {
	        this.dslMethods.repetition.push(many);
	    };
	    DslMethodsCollectorVisitor.prototype.visitAlternation = function (or) {
	        this.dslMethods.alternation.push(or);
	    };
	    return DslMethodsCollectorVisitor;
	}(gast_1.GAstVisitor));
	var collectorVisitor = new DslMethodsCollectorVisitor();
	function collectMethods(rule) {
	    collectorVisitor.reset();
	    rule.accept(collectorVisitor);
	    var dslMethods = collectorVisitor.dslMethods;
	    // avoid uncleaned references
	    collectorVisitor.reset();
	    return dslMethods;
	}
	looksahead.collectMethods = collectMethods;
	
	return looksahead;
}

var tree_builder = {};

var cst = {};

var hasRequiredCst;

function requireCst () {
	if (hasRequiredCst) return cst;
	hasRequiredCst = 1;
	Object.defineProperty(cst, "__esModule", { value: true });
	cst.addNoneTerminalToCst = cst.addTerminalToCst = cst.setNodeLocationFull = cst.setNodeLocationOnlyOffset = void 0;
	/**
	 * This nodeLocation tracking is not efficient and should only be used
	 * when error recovery is enabled or the Token Vector contains virtual Tokens
	 * (e.g, Python Indent/Outdent)
	 * As it executes the calculation for every single terminal/nonTerminal
	 * and does not rely on the fact the token vector is **sorted**
	 */
	function setNodeLocationOnlyOffset(currNodeLocation, newLocationInfo) {
	    // First (valid) update for this cst node
	    if (isNaN(currNodeLocation.startOffset) === true) {
	        // assumption1: Token location information is either NaN or a valid number
	        // assumption2: Token location information is fully valid if it exist
	        // (both start/end offsets exist and are numbers).
	        currNodeLocation.startOffset = newLocationInfo.startOffset;
	        currNodeLocation.endOffset = newLocationInfo.endOffset;
	    }
	    // Once the startOffset has been updated with a valid number it should never receive
	    // any farther updates as the Token vector is sorted.
	    // We still have to check this this condition for every new possible location info
	    // because with error recovery enabled we may encounter invalid tokens (NaN location props)
	    else if (currNodeLocation.endOffset < newLocationInfo.endOffset === true) {
	        currNodeLocation.endOffset = newLocationInfo.endOffset;
	    }
	}
	cst.setNodeLocationOnlyOffset = setNodeLocationOnlyOffset;
	/**
	 * This nodeLocation tracking is not efficient and should only be used
	 * when error recovery is enabled or the Token Vector contains virtual Tokens
	 * (e.g, Python Indent/Outdent)
	 * As it executes the calculation for every single terminal/nonTerminal
	 * and does not rely on the fact the token vector is **sorted**
	 */
	function setNodeLocationFull(currNodeLocation, newLocationInfo) {
	    // First (valid) update for this cst node
	    if (isNaN(currNodeLocation.startOffset) === true) {
	        // assumption1: Token location information is either NaN or a valid number
	        // assumption2: Token location information is fully valid if it exist
	        // (all start/end props exist and are numbers).
	        currNodeLocation.startOffset = newLocationInfo.startOffset;
	        currNodeLocation.startColumn = newLocationInfo.startColumn;
	        currNodeLocation.startLine = newLocationInfo.startLine;
	        currNodeLocation.endOffset = newLocationInfo.endOffset;
	        currNodeLocation.endColumn = newLocationInfo.endColumn;
	        currNodeLocation.endLine = newLocationInfo.endLine;
	    }
	    // Once the start props has been updated with a valid number it should never receive
	    // any farther updates as the Token vector is sorted.
	    // We still have to check this this condition for every new possible location info
	    // because with error recovery enabled we may encounter invalid tokens (NaN location props)
	    else if (currNodeLocation.endOffset < newLocationInfo.endOffset === true) {
	        currNodeLocation.endOffset = newLocationInfo.endOffset;
	        currNodeLocation.endColumn = newLocationInfo.endColumn;
	        currNodeLocation.endLine = newLocationInfo.endLine;
	    }
	}
	cst.setNodeLocationFull = setNodeLocationFull;
	function addTerminalToCst(node, token, tokenTypeName) {
	    if (node.children[tokenTypeName] === undefined) {
	        node.children[tokenTypeName] = [token];
	    }
	    else {
	        node.children[tokenTypeName].push(token);
	    }
	}
	cst.addTerminalToCst = addTerminalToCst;
	function addNoneTerminalToCst(node, ruleName, ruleResult) {
	    if (node.children[ruleName] === undefined) {
	        node.children[ruleName] = [ruleResult];
	    }
	    else {
	        node.children[ruleName].push(ruleResult);
	    }
	}
	cst.addNoneTerminalToCst = addNoneTerminalToCst;
	
	return cst;
}

var cst_visitor = {};

var lang_extensions = {};

var hasRequiredLang_extensions;

function requireLang_extensions () {
	if (hasRequiredLang_extensions) return lang_extensions;
	hasRequiredLang_extensions = 1;
	Object.defineProperty(lang_extensions, "__esModule", { value: true });
	lang_extensions.defineNameProp = void 0;
	var NAME = "name";
	function defineNameProp(obj, nameValue) {
	    Object.defineProperty(obj, NAME, {
	        enumerable: false,
	        configurable: true,
	        writable: false,
	        value: nameValue
	    });
	}
	lang_extensions.defineNameProp = defineNameProp;
	
	return lang_extensions;
}

var hasRequiredCst_visitor;

function requireCst_visitor () {
	if (hasRequiredCst_visitor) return cst_visitor;
	hasRequiredCst_visitor = 1;
	(function (exports) {
		var __importDefault = (cst_visitor && cst_visitor.__importDefault) || function (mod) {
		    return (mod && mod.__esModule) ? mod : { "default": mod };
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.validateMissingCstMethods = exports.validateVisitor = exports.CstVisitorDefinitionError = exports.createBaseVisitorConstructorWithDefaults = exports.createBaseSemanticVisitorConstructor = exports.defaultVisit = void 0;
		var isEmpty_1 = __importDefault(requireIsEmpty());
		var compact_1 = __importDefault(requireCompact());
		var isArray_1 = __importDefault(requireIsArray());
		var map_1 = __importDefault(requireMap());
		var forEach_1 = __importDefault(requireForEach());
		var filter_1 = __importDefault(requireFilter());
		var keys_1 = __importDefault(requireKeys$1());
		var isFunction_1 = __importDefault(requireIsFunction());
		var isUndefined_1 = __importDefault(requireIsUndefined());
		var lang_extensions_1 = requireLang_extensions();
		function defaultVisit(ctx, param) {
		    var childrenNames = (0, keys_1.default)(ctx);
		    var childrenNamesLength = childrenNames.length;
		    for (var i = 0; i < childrenNamesLength; i++) {
		        var currChildName = childrenNames[i];
		        var currChildArray = ctx[currChildName];
		        var currChildArrayLength = currChildArray.length;
		        for (var j = 0; j < currChildArrayLength; j++) {
		            var currChild = currChildArray[j];
		            // distinction between Tokens Children and CstNode children
		            if (currChild.tokenTypeIdx === undefined) {
		                this[currChild.name](currChild.children, param);
		            }
		        }
		    }
		    // defaultVisit does not support generic out param
		}
		exports.defaultVisit = defaultVisit;
		function createBaseSemanticVisitorConstructor(grammarName, ruleNames) {
		    var derivedConstructor = function () { };
		    // can be overwritten according to:
		    // https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Function/
		    // name?redirectlocale=en-US&redirectslug=JavaScript%2FReference%2FGlobal_Objects%2FFunction%2Fname
		    (0, lang_extensions_1.defineNameProp)(derivedConstructor, grammarName + "BaseSemantics");
		    var semanticProto = {
		        visit: function (cstNode, param) {
		            // enables writing more concise visitor methods when CstNode has only a single child
		            if ((0, isArray_1.default)(cstNode)) {
		                // A CST Node's children dictionary can never have empty arrays as values
		                // If a key is defined there will be at least one element in the corresponding value array.
		                cstNode = cstNode[0];
		            }
		            // enables passing optional CstNodes concisely.
		            if ((0, isUndefined_1.default)(cstNode)) {
		                return undefined;
		            }
		            return this[cstNode.name](cstNode.children, param);
		        },
		        validateVisitor: function () {
		            var semanticDefinitionErrors = validateVisitor(this, ruleNames);
		            if (!(0, isEmpty_1.default)(semanticDefinitionErrors)) {
		                var errorMessages = (0, map_1.default)(semanticDefinitionErrors, function (currDefError) { return currDefError.msg; });
		                throw Error("Errors Detected in CST Visitor <".concat(this.constructor.name, ">:\n\t") +
		                    "".concat(errorMessages.join("\n\n").replace(/\n/g, "\n\t")));
		            }
		        }
		    };
		    derivedConstructor.prototype = semanticProto;
		    derivedConstructor.prototype.constructor = derivedConstructor;
		    derivedConstructor._RULE_NAMES = ruleNames;
		    return derivedConstructor;
		}
		exports.createBaseSemanticVisitorConstructor = createBaseSemanticVisitorConstructor;
		function createBaseVisitorConstructorWithDefaults(grammarName, ruleNames, baseConstructor) {
		    var derivedConstructor = function () { };
		    // can be overwritten according to:
		    // https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Function/
		    // name?redirectlocale=en-US&redirectslug=JavaScript%2FReference%2FGlobal_Objects%2FFunction%2Fname
		    (0, lang_extensions_1.defineNameProp)(derivedConstructor, grammarName + "BaseSemanticsWithDefaults");
		    var withDefaultsProto = Object.create(baseConstructor.prototype);
		    (0, forEach_1.default)(ruleNames, function (ruleName) {
		        withDefaultsProto[ruleName] = defaultVisit;
		    });
		    derivedConstructor.prototype = withDefaultsProto;
		    derivedConstructor.prototype.constructor = derivedConstructor;
		    return derivedConstructor;
		}
		exports.createBaseVisitorConstructorWithDefaults = createBaseVisitorConstructorWithDefaults;
		var CstVisitorDefinitionError;
		(function (CstVisitorDefinitionError) {
		    CstVisitorDefinitionError[CstVisitorDefinitionError["REDUNDANT_METHOD"] = 0] = "REDUNDANT_METHOD";
		    CstVisitorDefinitionError[CstVisitorDefinitionError["MISSING_METHOD"] = 1] = "MISSING_METHOD";
		})(CstVisitorDefinitionError = exports.CstVisitorDefinitionError || (exports.CstVisitorDefinitionError = {}));
		function validateVisitor(visitorInstance, ruleNames) {
		    var missingErrors = validateMissingCstMethods(visitorInstance, ruleNames);
		    return missingErrors;
		}
		exports.validateVisitor = validateVisitor;
		function validateMissingCstMethods(visitorInstance, ruleNames) {
		    var missingRuleNames = (0, filter_1.default)(ruleNames, function (currRuleName) {
		        return (0, isFunction_1.default)(visitorInstance[currRuleName]) === false;
		    });
		    var errors = (0, map_1.default)(missingRuleNames, function (currRuleName) {
		        return {
		            msg: "Missing visitor method: <".concat(currRuleName, "> on ").concat((visitorInstance.constructor.name), " CST Visitor."),
		            type: CstVisitorDefinitionError.MISSING_METHOD,
		            methodName: currRuleName
		        };
		    });
		    return (0, compact_1.default)(errors);
		}
		exports.validateMissingCstMethods = validateMissingCstMethods;
		
	} (cst_visitor));
	return cst_visitor;
}

var hasRequiredTree_builder;

function requireTree_builder () {
	if (hasRequiredTree_builder) return tree_builder;
	hasRequiredTree_builder = 1;
	var __importDefault = (tree_builder && tree_builder.__importDefault) || function (mod) {
	    return (mod && mod.__esModule) ? mod : { "default": mod };
	};
	Object.defineProperty(tree_builder, "__esModule", { value: true });
	tree_builder.TreeBuilder = void 0;
	var cst_1 = requireCst();
	var noop_1 = __importDefault(requireNoop());
	var has_1 = __importDefault(requireHas());
	var keys_1 = __importDefault(requireKeys$1());
	var isUndefined_1 = __importDefault(requireIsUndefined());
	var cst_visitor_1 = requireCst_visitor();
	var parser_1 = requireParser();
	/**
	 * This trait is responsible for the CST building logic.
	 */
	var TreeBuilder = /** @class */ (function () {
	    function TreeBuilder() {
	    }
	    TreeBuilder.prototype.initTreeBuilder = function (config) {
	        this.CST_STACK = [];
	        // outputCst is no longer exposed/defined in the pubic API
	        this.outputCst = config.outputCst;
	        this.nodeLocationTracking = (0, has_1.default)(config, "nodeLocationTracking")
	            ? config.nodeLocationTracking // assumes end user provides the correct config value/type
	            : parser_1.DEFAULT_PARSER_CONFIG.nodeLocationTracking;
	        if (!this.outputCst) {
	            this.cstInvocationStateUpdate = noop_1.default;
	            this.cstFinallyStateUpdate = noop_1.default;
	            this.cstPostTerminal = noop_1.default;
	            this.cstPostNonTerminal = noop_1.default;
	            this.cstPostRule = noop_1.default;
	        }
	        else {
	            if (/full/i.test(this.nodeLocationTracking)) {
	                if (this.recoveryEnabled) {
	                    this.setNodeLocationFromToken = cst_1.setNodeLocationFull;
	                    this.setNodeLocationFromNode = cst_1.setNodeLocationFull;
	                    this.cstPostRule = noop_1.default;
	                    this.setInitialNodeLocation = this.setInitialNodeLocationFullRecovery;
	                }
	                else {
	                    this.setNodeLocationFromToken = noop_1.default;
	                    this.setNodeLocationFromNode = noop_1.default;
	                    this.cstPostRule = this.cstPostRuleFull;
	                    this.setInitialNodeLocation = this.setInitialNodeLocationFullRegular;
	                }
	            }
	            else if (/onlyOffset/i.test(this.nodeLocationTracking)) {
	                if (this.recoveryEnabled) {
	                    this.setNodeLocationFromToken = cst_1.setNodeLocationOnlyOffset;
	                    this.setNodeLocationFromNode = cst_1.setNodeLocationOnlyOffset;
	                    this.cstPostRule = noop_1.default;
	                    this.setInitialNodeLocation =
	                        this.setInitialNodeLocationOnlyOffsetRecovery;
	                }
	                else {
	                    this.setNodeLocationFromToken = noop_1.default;
	                    this.setNodeLocationFromNode = noop_1.default;
	                    this.cstPostRule = this.cstPostRuleOnlyOffset;
	                    this.setInitialNodeLocation =
	                        this.setInitialNodeLocationOnlyOffsetRegular;
	                }
	            }
	            else if (/none/i.test(this.nodeLocationTracking)) {
	                this.setNodeLocationFromToken = noop_1.default;
	                this.setNodeLocationFromNode = noop_1.default;
	                this.cstPostRule = noop_1.default;
	                this.setInitialNodeLocation = noop_1.default;
	            }
	            else {
	                throw Error("Invalid <nodeLocationTracking> config option: \"".concat(config.nodeLocationTracking, "\""));
	            }
	        }
	    };
	    TreeBuilder.prototype.setInitialNodeLocationOnlyOffsetRecovery = function (cstNode) {
	        cstNode.location = {
	            startOffset: NaN,
	            endOffset: NaN
	        };
	    };
	    TreeBuilder.prototype.setInitialNodeLocationOnlyOffsetRegular = function (cstNode) {
	        cstNode.location = {
	            // without error recovery the starting Location of a new CstNode is guaranteed
	            // To be the next Token's startOffset (for valid inputs).
	            // For invalid inputs there won't be any CSTOutput so this potential
	            // inaccuracy does not matter
	            startOffset: this.LA(1).startOffset,
	            endOffset: NaN
	        };
	    };
	    TreeBuilder.prototype.setInitialNodeLocationFullRecovery = function (cstNode) {
	        cstNode.location = {
	            startOffset: NaN,
	            startLine: NaN,
	            startColumn: NaN,
	            endOffset: NaN,
	            endLine: NaN,
	            endColumn: NaN
	        };
	    };
	    /**
	       *  @see setInitialNodeLocationOnlyOffsetRegular for explanation why this work
	  
	       * @param cstNode
	       */
	    TreeBuilder.prototype.setInitialNodeLocationFullRegular = function (cstNode) {
	        var nextToken = this.LA(1);
	        cstNode.location = {
	            startOffset: nextToken.startOffset,
	            startLine: nextToken.startLine,
	            startColumn: nextToken.startColumn,
	            endOffset: NaN,
	            endLine: NaN,
	            endColumn: NaN
	        };
	    };
	    TreeBuilder.prototype.cstInvocationStateUpdate = function (fullRuleName) {
	        var cstNode = {
	            name: fullRuleName,
	            children: Object.create(null)
	        };
	        this.setInitialNodeLocation(cstNode);
	        this.CST_STACK.push(cstNode);
	    };
	    TreeBuilder.prototype.cstFinallyStateUpdate = function () {
	        this.CST_STACK.pop();
	    };
	    TreeBuilder.prototype.cstPostRuleFull = function (ruleCstNode) {
	        // casts to `required<CstNodeLocation>` are safe because `cstPostRuleFull` should only be invoked when full location is enabled
	        var prevToken = this.LA(0);
	        var loc = ruleCstNode.location;
	        // If this condition is true it means we consumed at least one Token
	        // In this CstNode.
	        if (loc.startOffset <= prevToken.startOffset === true) {
	            loc.endOffset = prevToken.endOffset;
	            loc.endLine = prevToken.endLine;
	            loc.endColumn = prevToken.endColumn;
	        }
	        // "empty" CstNode edge case
	        else {
	            loc.startOffset = NaN;
	            loc.startLine = NaN;
	            loc.startColumn = NaN;
	        }
	    };
	    TreeBuilder.prototype.cstPostRuleOnlyOffset = function (ruleCstNode) {
	        var prevToken = this.LA(0);
	        // `location' is not null because `cstPostRuleOnlyOffset` will only be invoked when location tracking is enabled.
	        var loc = ruleCstNode.location;
	        // If this condition is true it means we consumed at least one Token
	        // In this CstNode.
	        if (loc.startOffset <= prevToken.startOffset === true) {
	            loc.endOffset = prevToken.endOffset;
	        }
	        // "empty" CstNode edge case
	        else {
	            loc.startOffset = NaN;
	        }
	    };
	    TreeBuilder.prototype.cstPostTerminal = function (key, consumedToken) {
	        var rootCst = this.CST_STACK[this.CST_STACK.length - 1];
	        (0, cst_1.addTerminalToCst)(rootCst, consumedToken, key);
	        // This is only used when **both** error recovery and CST Output are enabled.
	        this.setNodeLocationFromToken(rootCst.location, consumedToken);
	    };
	    TreeBuilder.prototype.cstPostNonTerminal = function (ruleCstResult, ruleName) {
	        var preCstNode = this.CST_STACK[this.CST_STACK.length - 1];
	        (0, cst_1.addNoneTerminalToCst)(preCstNode, ruleName, ruleCstResult);
	        // This is only used when **both** error recovery and CST Output are enabled.
	        this.setNodeLocationFromNode(preCstNode.location, ruleCstResult.location);
	    };
	    TreeBuilder.prototype.getBaseCstVisitorConstructor = function () {
	        if ((0, isUndefined_1.default)(this.baseCstVisitorConstructor)) {
	            var newBaseCstVisitorConstructor = (0, cst_visitor_1.createBaseSemanticVisitorConstructor)(this.className, (0, keys_1.default)(this.gastProductionsCache));
	            this.baseCstVisitorConstructor = newBaseCstVisitorConstructor;
	            return newBaseCstVisitorConstructor;
	        }
	        return this.baseCstVisitorConstructor;
	    };
	    TreeBuilder.prototype.getBaseCstVisitorConstructorWithDefaults = function () {
	        if ((0, isUndefined_1.default)(this.baseCstVisitorWithDefaultsConstructor)) {
	            var newConstructor = (0, cst_visitor_1.createBaseVisitorConstructorWithDefaults)(this.className, (0, keys_1.default)(this.gastProductionsCache), this.getBaseCstVisitorConstructor());
	            this.baseCstVisitorWithDefaultsConstructor = newConstructor;
	            return newConstructor;
	        }
	        return this.baseCstVisitorWithDefaultsConstructor;
	    };
	    TreeBuilder.prototype.getLastExplicitRuleShortName = function () {
	        var ruleStack = this.RULE_STACK;
	        return ruleStack[ruleStack.length - 1];
	    };
	    TreeBuilder.prototype.getPreviousExplicitRuleShortName = function () {
	        var ruleStack = this.RULE_STACK;
	        return ruleStack[ruleStack.length - 2];
	    };
	    TreeBuilder.prototype.getLastExplicitRuleOccurrenceIndex = function () {
	        var occurrenceStack = this.RULE_OCCURRENCE_STACK;
	        return occurrenceStack[occurrenceStack.length - 1];
	    };
	    return TreeBuilder;
	}());
	tree_builder.TreeBuilder = TreeBuilder;
	
	return tree_builder;
}

var lexer_adapter = {};

var hasRequiredLexer_adapter;

function requireLexer_adapter () {
	if (hasRequiredLexer_adapter) return lexer_adapter;
	hasRequiredLexer_adapter = 1;
	Object.defineProperty(lexer_adapter, "__esModule", { value: true });
	lexer_adapter.LexerAdapter = void 0;
	var parser_1 = requireParser();
	/**
	 * Trait responsible abstracting over the interaction with Lexer output (Token vector).
	 *
	 * This could be generalized to support other kinds of lexers, e.g.
	 * - Just in Time Lexing / Lexer-Less parsing.
	 * - Streaming Lexer.
	 */
	var LexerAdapter = /** @class */ (function () {
	    function LexerAdapter() {
	    }
	    LexerAdapter.prototype.initLexerAdapter = function () {
	        this.tokVector = [];
	        this.tokVectorLength = 0;
	        this.currIdx = -1;
	    };
	    Object.defineProperty(LexerAdapter.prototype, "input", {
	        get: function () {
	            return this.tokVector;
	        },
	        set: function (newInput) {
	            // @ts-ignore - `this parameter` not supported in setters/getters
	            //   - https://www.typescriptlang.org/docs/handbook/functions.html#this-parameters
	            if (this.selfAnalysisDone !== true) {
	                throw Error("Missing <performSelfAnalysis> invocation at the end of the Parser's constructor.");
	            }
	            // @ts-ignore - `this parameter` not supported in setters/getters
	            //   - https://www.typescriptlang.org/docs/handbook/functions.html#this-parameters
	            this.reset();
	            this.tokVector = newInput;
	            this.tokVectorLength = newInput.length;
	        },
	        enumerable: false,
	        configurable: true
	    });
	    // skips a token and returns the next token
	    LexerAdapter.prototype.SKIP_TOKEN = function () {
	        if (this.currIdx <= this.tokVector.length - 2) {
	            this.consumeToken();
	            return this.LA(1);
	        }
	        else {
	            return parser_1.END_OF_FILE;
	        }
	    };
	    // Lexer (accessing Token vector) related methods which can be overridden to implement lazy lexers
	    // or lexers dependent on parser context.
	    LexerAdapter.prototype.LA = function (howMuch) {
	        var soughtIdx = this.currIdx + howMuch;
	        if (soughtIdx < 0 || this.tokVectorLength <= soughtIdx) {
	            return parser_1.END_OF_FILE;
	        }
	        else {
	            return this.tokVector[soughtIdx];
	        }
	    };
	    LexerAdapter.prototype.consumeToken = function () {
	        this.currIdx++;
	    };
	    LexerAdapter.prototype.exportLexerState = function () {
	        return this.currIdx;
	    };
	    LexerAdapter.prototype.importLexerState = function (newState) {
	        this.currIdx = newState;
	    };
	    LexerAdapter.prototype.resetLexerState = function () {
	        this.currIdx = -1;
	    };
	    LexerAdapter.prototype.moveToTerminatedState = function () {
	        this.currIdx = this.tokVector.length - 1;
	    };
	    LexerAdapter.prototype.getLexerPosition = function () {
	        return this.exportLexerState();
	    };
	    return LexerAdapter;
	}());
	lexer_adapter.LexerAdapter = LexerAdapter;
	
	return lexer_adapter;
}

var recognizer_api = {};

var hasRequiredRecognizer_api;

function requireRecognizer_api () {
	if (hasRequiredRecognizer_api) return recognizer_api;
	hasRequiredRecognizer_api = 1;
	var __importDefault = (recognizer_api && recognizer_api.__importDefault) || function (mod) {
	    return (mod && mod.__esModule) ? mod : { "default": mod };
	};
	Object.defineProperty(recognizer_api, "__esModule", { value: true });
	recognizer_api.RecognizerApi = void 0;
	var values_1 = __importDefault(requireValues());
	var includes_1 = __importDefault(requireIncludes());
	var exceptions_public_1 = requireExceptions_public();
	var parser_1 = requireParser();
	var errors_public_1 = requireErrors_public();
	var checks_1 = requireChecks();
	var gast_1 = requireApi$2();
	/**
	 * This trait is responsible for implementing the public API
	 * for defining Chevrotain parsers, i.e:
	 * - CONSUME
	 * - RULE
	 * - OPTION
	 * - ...
	 */
	var RecognizerApi = /** @class */ (function () {
	    function RecognizerApi() {
	    }
	    RecognizerApi.prototype.ACTION = function (impl) {
	        return impl.call(this);
	    };
	    RecognizerApi.prototype.consume = function (idx, tokType, options) {
	        return this.consumeInternal(tokType, idx, options);
	    };
	    RecognizerApi.prototype.subrule = function (idx, ruleToCall, options) {
	        return this.subruleInternal(ruleToCall, idx, options);
	    };
	    RecognizerApi.prototype.option = function (idx, actionORMethodDef) {
	        return this.optionInternal(actionORMethodDef, idx);
	    };
	    RecognizerApi.prototype.or = function (idx, altsOrOpts) {
	        return this.orInternal(altsOrOpts, idx);
	    };
	    RecognizerApi.prototype.many = function (idx, actionORMethodDef) {
	        return this.manyInternal(idx, actionORMethodDef);
	    };
	    RecognizerApi.prototype.atLeastOne = function (idx, actionORMethodDef) {
	        return this.atLeastOneInternal(idx, actionORMethodDef);
	    };
	    RecognizerApi.prototype.CONSUME = function (tokType, options) {
	        return this.consumeInternal(tokType, 0, options);
	    };
	    RecognizerApi.prototype.CONSUME1 = function (tokType, options) {
	        return this.consumeInternal(tokType, 1, options);
	    };
	    RecognizerApi.prototype.CONSUME2 = function (tokType, options) {
	        return this.consumeInternal(tokType, 2, options);
	    };
	    RecognizerApi.prototype.CONSUME3 = function (tokType, options) {
	        return this.consumeInternal(tokType, 3, options);
	    };
	    RecognizerApi.prototype.CONSUME4 = function (tokType, options) {
	        return this.consumeInternal(tokType, 4, options);
	    };
	    RecognizerApi.prototype.CONSUME5 = function (tokType, options) {
	        return this.consumeInternal(tokType, 5, options);
	    };
	    RecognizerApi.prototype.CONSUME6 = function (tokType, options) {
	        return this.consumeInternal(tokType, 6, options);
	    };
	    RecognizerApi.prototype.CONSUME7 = function (tokType, options) {
	        return this.consumeInternal(tokType, 7, options);
	    };
	    RecognizerApi.prototype.CONSUME8 = function (tokType, options) {
	        return this.consumeInternal(tokType, 8, options);
	    };
	    RecognizerApi.prototype.CONSUME9 = function (tokType, options) {
	        return this.consumeInternal(tokType, 9, options);
	    };
	    RecognizerApi.prototype.SUBRULE = function (ruleToCall, options) {
	        return this.subruleInternal(ruleToCall, 0, options);
	    };
	    RecognizerApi.prototype.SUBRULE1 = function (ruleToCall, options) {
	        return this.subruleInternal(ruleToCall, 1, options);
	    };
	    RecognizerApi.prototype.SUBRULE2 = function (ruleToCall, options) {
	        return this.subruleInternal(ruleToCall, 2, options);
	    };
	    RecognizerApi.prototype.SUBRULE3 = function (ruleToCall, options) {
	        return this.subruleInternal(ruleToCall, 3, options);
	    };
	    RecognizerApi.prototype.SUBRULE4 = function (ruleToCall, options) {
	        return this.subruleInternal(ruleToCall, 4, options);
	    };
	    RecognizerApi.prototype.SUBRULE5 = function (ruleToCall, options) {
	        return this.subruleInternal(ruleToCall, 5, options);
	    };
	    RecognizerApi.prototype.SUBRULE6 = function (ruleToCall, options) {
	        return this.subruleInternal(ruleToCall, 6, options);
	    };
	    RecognizerApi.prototype.SUBRULE7 = function (ruleToCall, options) {
	        return this.subruleInternal(ruleToCall, 7, options);
	    };
	    RecognizerApi.prototype.SUBRULE8 = function (ruleToCall, options) {
	        return this.subruleInternal(ruleToCall, 8, options);
	    };
	    RecognizerApi.prototype.SUBRULE9 = function (ruleToCall, options) {
	        return this.subruleInternal(ruleToCall, 9, options);
	    };
	    RecognizerApi.prototype.OPTION = function (actionORMethodDef) {
	        return this.optionInternal(actionORMethodDef, 0);
	    };
	    RecognizerApi.prototype.OPTION1 = function (actionORMethodDef) {
	        return this.optionInternal(actionORMethodDef, 1);
	    };
	    RecognizerApi.prototype.OPTION2 = function (actionORMethodDef) {
	        return this.optionInternal(actionORMethodDef, 2);
	    };
	    RecognizerApi.prototype.OPTION3 = function (actionORMethodDef) {
	        return this.optionInternal(actionORMethodDef, 3);
	    };
	    RecognizerApi.prototype.OPTION4 = function (actionORMethodDef) {
	        return this.optionInternal(actionORMethodDef, 4);
	    };
	    RecognizerApi.prototype.OPTION5 = function (actionORMethodDef) {
	        return this.optionInternal(actionORMethodDef, 5);
	    };
	    RecognizerApi.prototype.OPTION6 = function (actionORMethodDef) {
	        return this.optionInternal(actionORMethodDef, 6);
	    };
	    RecognizerApi.prototype.OPTION7 = function (actionORMethodDef) {
	        return this.optionInternal(actionORMethodDef, 7);
	    };
	    RecognizerApi.prototype.OPTION8 = function (actionORMethodDef) {
	        return this.optionInternal(actionORMethodDef, 8);
	    };
	    RecognizerApi.prototype.OPTION9 = function (actionORMethodDef) {
	        return this.optionInternal(actionORMethodDef, 9);
	    };
	    RecognizerApi.prototype.OR = function (altsOrOpts) {
	        return this.orInternal(altsOrOpts, 0);
	    };
	    RecognizerApi.prototype.OR1 = function (altsOrOpts) {
	        return this.orInternal(altsOrOpts, 1);
	    };
	    RecognizerApi.prototype.OR2 = function (altsOrOpts) {
	        return this.orInternal(altsOrOpts, 2);
	    };
	    RecognizerApi.prototype.OR3 = function (altsOrOpts) {
	        return this.orInternal(altsOrOpts, 3);
	    };
	    RecognizerApi.prototype.OR4 = function (altsOrOpts) {
	        return this.orInternal(altsOrOpts, 4);
	    };
	    RecognizerApi.prototype.OR5 = function (altsOrOpts) {
	        return this.orInternal(altsOrOpts, 5);
	    };
	    RecognizerApi.prototype.OR6 = function (altsOrOpts) {
	        return this.orInternal(altsOrOpts, 6);
	    };
	    RecognizerApi.prototype.OR7 = function (altsOrOpts) {
	        return this.orInternal(altsOrOpts, 7);
	    };
	    RecognizerApi.prototype.OR8 = function (altsOrOpts) {
	        return this.orInternal(altsOrOpts, 8);
	    };
	    RecognizerApi.prototype.OR9 = function (altsOrOpts) {
	        return this.orInternal(altsOrOpts, 9);
	    };
	    RecognizerApi.prototype.MANY = function (actionORMethodDef) {
	        this.manyInternal(0, actionORMethodDef);
	    };
	    RecognizerApi.prototype.MANY1 = function (actionORMethodDef) {
	        this.manyInternal(1, actionORMethodDef);
	    };
	    RecognizerApi.prototype.MANY2 = function (actionORMethodDef) {
	        this.manyInternal(2, actionORMethodDef);
	    };
	    RecognizerApi.prototype.MANY3 = function (actionORMethodDef) {
	        this.manyInternal(3, actionORMethodDef);
	    };
	    RecognizerApi.prototype.MANY4 = function (actionORMethodDef) {
	        this.manyInternal(4, actionORMethodDef);
	    };
	    RecognizerApi.prototype.MANY5 = function (actionORMethodDef) {
	        this.manyInternal(5, actionORMethodDef);
	    };
	    RecognizerApi.prototype.MANY6 = function (actionORMethodDef) {
	        this.manyInternal(6, actionORMethodDef);
	    };
	    RecognizerApi.prototype.MANY7 = function (actionORMethodDef) {
	        this.manyInternal(7, actionORMethodDef);
	    };
	    RecognizerApi.prototype.MANY8 = function (actionORMethodDef) {
	        this.manyInternal(8, actionORMethodDef);
	    };
	    RecognizerApi.prototype.MANY9 = function (actionORMethodDef) {
	        this.manyInternal(9, actionORMethodDef);
	    };
	    RecognizerApi.prototype.MANY_SEP = function (options) {
	        this.manySepFirstInternal(0, options);
	    };
	    RecognizerApi.prototype.MANY_SEP1 = function (options) {
	        this.manySepFirstInternal(1, options);
	    };
	    RecognizerApi.prototype.MANY_SEP2 = function (options) {
	        this.manySepFirstInternal(2, options);
	    };
	    RecognizerApi.prototype.MANY_SEP3 = function (options) {
	        this.manySepFirstInternal(3, options);
	    };
	    RecognizerApi.prototype.MANY_SEP4 = function (options) {
	        this.manySepFirstInternal(4, options);
	    };
	    RecognizerApi.prototype.MANY_SEP5 = function (options) {
	        this.manySepFirstInternal(5, options);
	    };
	    RecognizerApi.prototype.MANY_SEP6 = function (options) {
	        this.manySepFirstInternal(6, options);
	    };
	    RecognizerApi.prototype.MANY_SEP7 = function (options) {
	        this.manySepFirstInternal(7, options);
	    };
	    RecognizerApi.prototype.MANY_SEP8 = function (options) {
	        this.manySepFirstInternal(8, options);
	    };
	    RecognizerApi.prototype.MANY_SEP9 = function (options) {
	        this.manySepFirstInternal(9, options);
	    };
	    RecognizerApi.prototype.AT_LEAST_ONE = function (actionORMethodDef) {
	        this.atLeastOneInternal(0, actionORMethodDef);
	    };
	    RecognizerApi.prototype.AT_LEAST_ONE1 = function (actionORMethodDef) {
	        return this.atLeastOneInternal(1, actionORMethodDef);
	    };
	    RecognizerApi.prototype.AT_LEAST_ONE2 = function (actionORMethodDef) {
	        this.atLeastOneInternal(2, actionORMethodDef);
	    };
	    RecognizerApi.prototype.AT_LEAST_ONE3 = function (actionORMethodDef) {
	        this.atLeastOneInternal(3, actionORMethodDef);
	    };
	    RecognizerApi.prototype.AT_LEAST_ONE4 = function (actionORMethodDef) {
	        this.atLeastOneInternal(4, actionORMethodDef);
	    };
	    RecognizerApi.prototype.AT_LEAST_ONE5 = function (actionORMethodDef) {
	        this.atLeastOneInternal(5, actionORMethodDef);
	    };
	    RecognizerApi.prototype.AT_LEAST_ONE6 = function (actionORMethodDef) {
	        this.atLeastOneInternal(6, actionORMethodDef);
	    };
	    RecognizerApi.prototype.AT_LEAST_ONE7 = function (actionORMethodDef) {
	        this.atLeastOneInternal(7, actionORMethodDef);
	    };
	    RecognizerApi.prototype.AT_LEAST_ONE8 = function (actionORMethodDef) {
	        this.atLeastOneInternal(8, actionORMethodDef);
	    };
	    RecognizerApi.prototype.AT_LEAST_ONE9 = function (actionORMethodDef) {
	        this.atLeastOneInternal(9, actionORMethodDef);
	    };
	    RecognizerApi.prototype.AT_LEAST_ONE_SEP = function (options) {
	        this.atLeastOneSepFirstInternal(0, options);
	    };
	    RecognizerApi.prototype.AT_LEAST_ONE_SEP1 = function (options) {
	        this.atLeastOneSepFirstInternal(1, options);
	    };
	    RecognizerApi.prototype.AT_LEAST_ONE_SEP2 = function (options) {
	        this.atLeastOneSepFirstInternal(2, options);
	    };
	    RecognizerApi.prototype.AT_LEAST_ONE_SEP3 = function (options) {
	        this.atLeastOneSepFirstInternal(3, options);
	    };
	    RecognizerApi.prototype.AT_LEAST_ONE_SEP4 = function (options) {
	        this.atLeastOneSepFirstInternal(4, options);
	    };
	    RecognizerApi.prototype.AT_LEAST_ONE_SEP5 = function (options) {
	        this.atLeastOneSepFirstInternal(5, options);
	    };
	    RecognizerApi.prototype.AT_LEAST_ONE_SEP6 = function (options) {
	        this.atLeastOneSepFirstInternal(6, options);
	    };
	    RecognizerApi.prototype.AT_LEAST_ONE_SEP7 = function (options) {
	        this.atLeastOneSepFirstInternal(7, options);
	    };
	    RecognizerApi.prototype.AT_LEAST_ONE_SEP8 = function (options) {
	        this.atLeastOneSepFirstInternal(8, options);
	    };
	    RecognizerApi.prototype.AT_LEAST_ONE_SEP9 = function (options) {
	        this.atLeastOneSepFirstInternal(9, options);
	    };
	    RecognizerApi.prototype.RULE = function (name, implementation, config) {
	        if (config === void 0) { config = parser_1.DEFAULT_RULE_CONFIG; }
	        if ((0, includes_1.default)(this.definedRulesNames, name)) {
	            var errMsg = errors_public_1.defaultGrammarValidatorErrorProvider.buildDuplicateRuleNameError({
	                topLevelRule: name,
	                grammarName: this.className
	            });
	            var error = {
	                message: errMsg,
	                type: parser_1.ParserDefinitionErrorType.DUPLICATE_RULE_NAME,
	                ruleName: name
	            };
	            this.definitionErrors.push(error);
	        }
	        this.definedRulesNames.push(name);
	        var ruleImplementation = this.defineRule(name, implementation, config);
	        this[name] = ruleImplementation;
	        return ruleImplementation;
	    };
	    RecognizerApi.prototype.OVERRIDE_RULE = function (name, impl, config) {
	        if (config === void 0) { config = parser_1.DEFAULT_RULE_CONFIG; }
	        var ruleErrors = (0, checks_1.validateRuleIsOverridden)(name, this.definedRulesNames, this.className);
	        this.definitionErrors = this.definitionErrors.concat(ruleErrors);
	        var ruleImplementation = this.defineRule(name, impl, config);
	        this[name] = ruleImplementation;
	        return ruleImplementation;
	    };
	    RecognizerApi.prototype.BACKTRACK = function (grammarRule, args) {
	        return function () {
	            // save org state
	            this.isBackTrackingStack.push(1);
	            var orgState = this.saveRecogState();
	            try {
	                grammarRule.apply(this, args);
	                // if no exception was thrown we have succeed parsing the rule.
	                return true;
	            }
	            catch (e) {
	                if ((0, exceptions_public_1.isRecognitionException)(e)) {
	                    return false;
	                }
	                else {
	                    throw e;
	                }
	            }
	            finally {
	                this.reloadRecogState(orgState);
	                this.isBackTrackingStack.pop();
	            }
	        };
	    };
	    // GAST export APIs
	    RecognizerApi.prototype.getGAstProductions = function () {
	        return this.gastProductionsCache;
	    };
	    RecognizerApi.prototype.getSerializedGastProductions = function () {
	        return (0, gast_1.serializeGrammar)((0, values_1.default)(this.gastProductionsCache));
	    };
	    return RecognizerApi;
	}());
	recognizer_api.RecognizerApi = RecognizerApi;
	
	return recognizer_api;
}

var recognizer_engine = {};

var hasRequiredRecognizer_engine;

function requireRecognizer_engine () {
	if (hasRequiredRecognizer_engine) return recognizer_engine;
	hasRequiredRecognizer_engine = 1;
	var __importDefault = (recognizer_engine && recognizer_engine.__importDefault) || function (mod) {
	    return (mod && mod.__esModule) ? mod : { "default": mod };
	};
	Object.defineProperty(recognizer_engine, "__esModule", { value: true });
	recognizer_engine.RecognizerEngine = void 0;
	var isEmpty_1 = __importDefault(requireIsEmpty());
	var isArray_1 = __importDefault(requireIsArray());
	var flatten_1 = __importDefault(requireFlatten());
	var every_1 = __importDefault(requireEvery());
	var uniq_1 = __importDefault(requireUniq());
	var isObject_1 = __importDefault(requireIsObject());
	var has_1 = __importDefault(requireHas());
	var values_1 = __importDefault(requireValues());
	var reduce_1 = __importDefault(requireReduce());
	var clone_1 = __importDefault(requireClone());
	var keys_1 = requireKeys();
	var exceptions_public_1 = requireExceptions_public();
	var lookahead_1 = requireLookahead();
	var interpreter_1 = requireInterpreter();
	var parser_1 = requireParser();
	var recoverable_1 = requireRecoverable();
	var tokens_public_1 = requireTokens_public();
	var tokens_1 = requireTokens();
	/**
	 * This trait is responsible for the runtime parsing engine
	 * Used by the official API (recognizer_api.ts)
	 */
	var RecognizerEngine = /** @class */ (function () {
	    function RecognizerEngine() {
	    }
	    RecognizerEngine.prototype.initRecognizerEngine = function (tokenVocabulary, config) {
	        this.className = this.constructor.name;
	        // TODO: would using an ES6 Map or plain object be faster (CST building scenario)
	        this.shortRuleNameToFull = {};
	        this.fullRuleNameToShort = {};
	        this.ruleShortNameIdx = 256;
	        this.tokenMatcher = tokens_1.tokenStructuredMatcherNoCategories;
	        this.subruleIdx = 0;
	        this.definedRulesNames = [];
	        this.tokensMap = {};
	        this.isBackTrackingStack = [];
	        this.RULE_STACK = [];
	        this.RULE_OCCURRENCE_STACK = [];
	        this.gastProductionsCache = {};
	        if ((0, has_1.default)(config, "serializedGrammar")) {
	            throw Error("The Parser's configuration can no longer contain a <serializedGrammar> property.\n" +
	                "\tSee: https://chevrotain.io/docs/changes/BREAKING_CHANGES.html#_6-0-0\n" +
	                "\tFor Further details.");
	        }
	        if ((0, isArray_1.default)(tokenVocabulary)) {
	            // This only checks for Token vocabularies provided as arrays.
	            // That is good enough because the main objective is to detect users of pre-V4.0 APIs
	            // rather than all edge cases of empty Token vocabularies.
	            if ((0, isEmpty_1.default)(tokenVocabulary)) {
	                throw Error("A Token Vocabulary cannot be empty.\n" +
	                    "\tNote that the first argument for the parser constructor\n" +
	                    "\tis no longer a Token vector (since v4.0).");
	            }
	            if (typeof tokenVocabulary[0].startOffset === "number") {
	                throw Error("The Parser constructor no longer accepts a token vector as the first argument.\n" +
	                    "\tSee: https://chevrotain.io/docs/changes/BREAKING_CHANGES.html#_4-0-0\n" +
	                    "\tFor Further details.");
	            }
	        }
	        if ((0, isArray_1.default)(tokenVocabulary)) {
	            this.tokensMap = (0, reduce_1.default)(tokenVocabulary, function (acc, tokType) {
	                acc[tokType.name] = tokType;
	                return acc;
	            }, {});
	        }
	        else if ((0, has_1.default)(tokenVocabulary, "modes") &&
	            (0, every_1.default)((0, flatten_1.default)((0, values_1.default)(tokenVocabulary.modes)), tokens_1.isTokenType)) {
	            var allTokenTypes_1 = (0, flatten_1.default)((0, values_1.default)(tokenVocabulary.modes));
	            var uniqueTokens = (0, uniq_1.default)(allTokenTypes_1);
	            this.tokensMap = (0, reduce_1.default)(uniqueTokens, function (acc, tokType) {
	                acc[tokType.name] = tokType;
	                return acc;
	            }, {});
	        }
	        else if ((0, isObject_1.default)(tokenVocabulary)) {
	            this.tokensMap = (0, clone_1.default)(tokenVocabulary);
	        }
	        else {
	            throw new Error("<tokensDictionary> argument must be An Array of Token constructors," +
	                " A dictionary of Token constructors or an IMultiModeLexerDefinition");
	        }
	        // always add EOF to the tokenNames -> constructors map. it is useful to assure all the input has been
	        // parsed with a clear error message ("expecting EOF but found ...")
	        this.tokensMap["EOF"] = tokens_public_1.EOF;
	        var allTokenTypes = (0, has_1.default)(tokenVocabulary, "modes")
	            ? (0, flatten_1.default)((0, values_1.default)(tokenVocabulary.modes))
	            : (0, values_1.default)(tokenVocabulary);
	        var noTokenCategoriesUsed = (0, every_1.default)(allTokenTypes, function (tokenConstructor) {
	            return (0, isEmpty_1.default)(tokenConstructor.categoryMatches);
	        });
	        this.tokenMatcher = noTokenCategoriesUsed
	            ? tokens_1.tokenStructuredMatcherNoCategories
	            : tokens_1.tokenStructuredMatcher;
	        // Because ES2015+ syntax should be supported for creating Token classes
	        // We cannot assume that the Token classes were created using the "extendToken" utilities
	        // Therefore we must augment the Token classes both on Lexer initialization and on Parser initialization
	        (0, tokens_1.augmentTokenTypes)((0, values_1.default)(this.tokensMap));
	    };
	    RecognizerEngine.prototype.defineRule = function (ruleName, impl, config) {
	        if (this.selfAnalysisDone) {
	            throw Error("Grammar rule <".concat(ruleName, "> may not be defined after the 'performSelfAnalysis' method has been called'\n") +
	                "Make sure that all grammar rule definitions are done before 'performSelfAnalysis' is called.");
	        }
	        var resyncEnabled = (0, has_1.default)(config, "resyncEnabled")
	            ? config.resyncEnabled // assumes end user provides the correct config value/type
	            : parser_1.DEFAULT_RULE_CONFIG.resyncEnabled;
	        var recoveryValueFunc = (0, has_1.default)(config, "recoveryValueFunc")
	            ? config.recoveryValueFunc // assumes end user provides the correct config value/type
	            : parser_1.DEFAULT_RULE_CONFIG.recoveryValueFunc;
	        // performance optimization: Use small integers as keys for the longer human readable "full" rule names.
	        // this greatly improves Map access time (as much as 8% for some performance benchmarks).
	        var shortName = this.ruleShortNameIdx << (keys_1.BITS_FOR_METHOD_TYPE + keys_1.BITS_FOR_OCCURRENCE_IDX);
	        this.ruleShortNameIdx++;
	        this.shortRuleNameToFull[shortName] = ruleName;
	        this.fullRuleNameToShort[ruleName] = shortName;
	        var invokeRuleWithTry;
	        // Micro optimization, only check the condition **once** on rule definition
	        // instead of **every single** rule invocation.
	        if (this.outputCst === true) {
	            invokeRuleWithTry = function invokeRuleWithTry() {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                try {
	                    this.ruleInvocationStateUpdate(shortName, ruleName, this.subruleIdx);
	                    impl.apply(this, args);
	                    var cst = this.CST_STACK[this.CST_STACK.length - 1];
	                    this.cstPostRule(cst);
	                    return cst;
	                }
	                catch (e) {
	                    return this.invokeRuleCatch(e, resyncEnabled, recoveryValueFunc);
	                }
	                finally {
	                    this.ruleFinallyStateUpdate();
	                }
	            };
	        }
	        else {
	            invokeRuleWithTry = function invokeRuleWithTryCst() {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                try {
	                    this.ruleInvocationStateUpdate(shortName, ruleName, this.subruleIdx);
	                    return impl.apply(this, args);
	                }
	                catch (e) {
	                    return this.invokeRuleCatch(e, resyncEnabled, recoveryValueFunc);
	                }
	                finally {
	                    this.ruleFinallyStateUpdate();
	                }
	            };
	        }
	        var wrappedGrammarRule = Object.assign(invokeRuleWithTry, { ruleName: ruleName, originalGrammarAction: impl });
	        return wrappedGrammarRule;
	    };
	    RecognizerEngine.prototype.invokeRuleCatch = function (e, resyncEnabledConfig, recoveryValueFunc) {
	        var isFirstInvokedRule = this.RULE_STACK.length === 1;
	        // note the reSync is always enabled for the first rule invocation, because we must always be able to
	        // reSync with EOF and just output some INVALID ParseTree
	        // during backtracking reSync recovery is disabled, otherwise we can't be certain the backtracking
	        // path is really the most valid one
	        var reSyncEnabled = resyncEnabledConfig && !this.isBackTracking() && this.recoveryEnabled;
	        if ((0, exceptions_public_1.isRecognitionException)(e)) {
	            var recogError = e;
	            if (reSyncEnabled) {
	                var reSyncTokType = this.findReSyncTokenType();
	                if (this.isInCurrentRuleReSyncSet(reSyncTokType)) {
	                    recogError.resyncedTokens = this.reSyncTo(reSyncTokType);
	                    if (this.outputCst) {
	                        var partialCstResult = this.CST_STACK[this.CST_STACK.length - 1];
	                        partialCstResult.recoveredNode = true;
	                        return partialCstResult;
	                    }
	                    else {
	                        return recoveryValueFunc(e);
	                    }
	                }
	                else {
	                    if (this.outputCst) {
	                        var partialCstResult = this.CST_STACK[this.CST_STACK.length - 1];
	                        partialCstResult.recoveredNode = true;
	                        recogError.partialCstResult = partialCstResult;
	                    }
	                    // to be handled Further up the call stack
	                    throw recogError;
	                }
	            }
	            else if (isFirstInvokedRule) {
	                // otherwise a Redundant input error will be created as well and we cannot guarantee that this is indeed the case
	                this.moveToTerminatedState();
	                // the parser should never throw one of its own errors outside its flow.
	                // even if error recovery is disabled
	                return recoveryValueFunc(e);
	            }
	            else {
	                // to be recovered Further up the call stack
	                throw recogError;
	            }
	        }
	        else {
	            // some other Error type which we don't know how to handle (for example a built in JavaScript Error)
	            throw e;
	        }
	    };
	    // Implementation of parsing DSL
	    RecognizerEngine.prototype.optionInternal = function (actionORMethodDef, occurrence) {
	        var key = this.getKeyForAutomaticLookahead(keys_1.OPTION_IDX, occurrence);
	        return this.optionInternalLogic(actionORMethodDef, occurrence, key);
	    };
	    RecognizerEngine.prototype.optionInternalLogic = function (actionORMethodDef, occurrence, key) {
	        var _this = this;
	        var lookAheadFunc = this.getLaFuncFromCache(key);
	        var action;
	        if (typeof actionORMethodDef !== "function") {
	            action = actionORMethodDef.DEF;
	            var predicate_1 = actionORMethodDef.GATE;
	            // predicate present
	            if (predicate_1 !== undefined) {
	                var orgLookaheadFunction_1 = lookAheadFunc;
	                lookAheadFunc = function () {
	                    return predicate_1.call(_this) && orgLookaheadFunction_1.call(_this);
	                };
	            }
	        }
	        else {
	            action = actionORMethodDef;
	        }
	        if (lookAheadFunc.call(this) === true) {
	            return action.call(this);
	        }
	        return undefined;
	    };
	    RecognizerEngine.prototype.atLeastOneInternal = function (prodOccurrence, actionORMethodDef) {
	        var laKey = this.getKeyForAutomaticLookahead(keys_1.AT_LEAST_ONE_IDX, prodOccurrence);
	        return this.atLeastOneInternalLogic(prodOccurrence, actionORMethodDef, laKey);
	    };
	    RecognizerEngine.prototype.atLeastOneInternalLogic = function (prodOccurrence, actionORMethodDef, key) {
	        var _this = this;
	        var lookAheadFunc = this.getLaFuncFromCache(key);
	        var action;
	        if (typeof actionORMethodDef !== "function") {
	            action = actionORMethodDef.DEF;
	            var predicate_2 = actionORMethodDef.GATE;
	            // predicate present
	            if (predicate_2 !== undefined) {
	                var orgLookaheadFunction_2 = lookAheadFunc;
	                lookAheadFunc = function () {
	                    return predicate_2.call(_this) && orgLookaheadFunction_2.call(_this);
	                };
	            }
	        }
	        else {
	            action = actionORMethodDef;
	        }
	        if (lookAheadFunc.call(this) === true) {
	            var notStuck = this.doSingleRepetition(action);
	            while (lookAheadFunc.call(this) === true &&
	                notStuck === true) {
	                notStuck = this.doSingleRepetition(action);
	            }
	        }
	        else {
	            throw this.raiseEarlyExitException(prodOccurrence, lookahead_1.PROD_TYPE.REPETITION_MANDATORY, actionORMethodDef.ERR_MSG);
	        }
	        // note that while it may seem that this can cause an error because by using a recursive call to
	        // AT_LEAST_ONE we change the grammar to AT_LEAST_TWO, AT_LEAST_THREE ... , the possible recursive call
	        // from the tryInRepetitionRecovery(...) will only happen IFF there really are TWO/THREE/.... items.
	        // Performance optimization: "attemptInRepetitionRecovery" will be defined as NOOP unless recovery is enabled
	        this.attemptInRepetitionRecovery(this.atLeastOneInternal, [prodOccurrence, actionORMethodDef], lookAheadFunc, keys_1.AT_LEAST_ONE_IDX, prodOccurrence, interpreter_1.NextTerminalAfterAtLeastOneWalker);
	    };
	    RecognizerEngine.prototype.atLeastOneSepFirstInternal = function (prodOccurrence, options) {
	        var laKey = this.getKeyForAutomaticLookahead(keys_1.AT_LEAST_ONE_SEP_IDX, prodOccurrence);
	        this.atLeastOneSepFirstInternalLogic(prodOccurrence, options, laKey);
	    };
	    RecognizerEngine.prototype.atLeastOneSepFirstInternalLogic = function (prodOccurrence, options, key) {
	        var _this = this;
	        var action = options.DEF;
	        var separator = options.SEP;
	        var firstIterationLookaheadFunc = this.getLaFuncFromCache(key);
	        // 1st iteration
	        if (firstIterationLookaheadFunc.call(this) === true) {
	            action.call(this);
	            //  TODO: Optimization can move this function construction into "attemptInRepetitionRecovery"
	            //  because it is only needed in error recovery scenarios.
	            var separatorLookAheadFunc = function () {
	                return _this.tokenMatcher(_this.LA(1), separator);
	            };
	            // 2nd..nth iterations
	            while (this.tokenMatcher(this.LA(1), separator) === true) {
	                // note that this CONSUME will never enter recovery because
	                // the separatorLookAheadFunc checks that the separator really does exist.
	                this.CONSUME(separator);
	                action.call(this);
	            }
	            // Performance optimization: "attemptInRepetitionRecovery" will be defined as NOOP unless recovery is enabled
	            this.attemptInRepetitionRecovery(this.repetitionSepSecondInternal, [
	                prodOccurrence,
	                separator,
	                separatorLookAheadFunc,
	                action,
	                interpreter_1.NextTerminalAfterAtLeastOneSepWalker
	            ], separatorLookAheadFunc, keys_1.AT_LEAST_ONE_SEP_IDX, prodOccurrence, interpreter_1.NextTerminalAfterAtLeastOneSepWalker);
	        }
	        else {
	            throw this.raiseEarlyExitException(prodOccurrence, lookahead_1.PROD_TYPE.REPETITION_MANDATORY_WITH_SEPARATOR, options.ERR_MSG);
	        }
	    };
	    RecognizerEngine.prototype.manyInternal = function (prodOccurrence, actionORMethodDef) {
	        var laKey = this.getKeyForAutomaticLookahead(keys_1.MANY_IDX, prodOccurrence);
	        return this.manyInternalLogic(prodOccurrence, actionORMethodDef, laKey);
	    };
	    RecognizerEngine.prototype.manyInternalLogic = function (prodOccurrence, actionORMethodDef, key) {
	        var _this = this;
	        var lookaheadFunction = this.getLaFuncFromCache(key);
	        var action;
	        if (typeof actionORMethodDef !== "function") {
	            action = actionORMethodDef.DEF;
	            var predicate_3 = actionORMethodDef.GATE;
	            // predicate present
	            if (predicate_3 !== undefined) {
	                var orgLookaheadFunction_3 = lookaheadFunction;
	                lookaheadFunction = function () {
	                    return predicate_3.call(_this) && orgLookaheadFunction_3.call(_this);
	                };
	            }
	        }
	        else {
	            action = actionORMethodDef;
	        }
	        var notStuck = true;
	        while (lookaheadFunction.call(this) === true && notStuck === true) {
	            notStuck = this.doSingleRepetition(action);
	        }
	        // Performance optimization: "attemptInRepetitionRecovery" will be defined as NOOP unless recovery is enabled
	        this.attemptInRepetitionRecovery(this.manyInternal, [prodOccurrence, actionORMethodDef], lookaheadFunction, keys_1.MANY_IDX, prodOccurrence, interpreter_1.NextTerminalAfterManyWalker, 
	        // The notStuck parameter is only relevant when "attemptInRepetitionRecovery"
	        // is invoked from manyInternal, in the MANY_SEP case and AT_LEAST_ONE[_SEP]
	        // An infinite loop cannot occur as:
	        // - Either the lookahead is guaranteed to consume something (Single Token Separator)
	        // - AT_LEAST_ONE by definition is guaranteed to consume something (or error out).
	        notStuck);
	    };
	    RecognizerEngine.prototype.manySepFirstInternal = function (prodOccurrence, options) {
	        var laKey = this.getKeyForAutomaticLookahead(keys_1.MANY_SEP_IDX, prodOccurrence);
	        this.manySepFirstInternalLogic(prodOccurrence, options, laKey);
	    };
	    RecognizerEngine.prototype.manySepFirstInternalLogic = function (prodOccurrence, options, key) {
	        var _this = this;
	        var action = options.DEF;
	        var separator = options.SEP;
	        var firstIterationLaFunc = this.getLaFuncFromCache(key);
	        // 1st iteration
	        if (firstIterationLaFunc.call(this) === true) {
	            action.call(this);
	            var separatorLookAheadFunc = function () {
	                return _this.tokenMatcher(_this.LA(1), separator);
	            };
	            // 2nd..nth iterations
	            while (this.tokenMatcher(this.LA(1), separator) === true) {
	                // note that this CONSUME will never enter recovery because
	                // the separatorLookAheadFunc checks that the separator really does exist.
	                this.CONSUME(separator);
	                // No need for checking infinite loop here due to consuming the separator.
	                action.call(this);
	            }
	            // Performance optimization: "attemptInRepetitionRecovery" will be defined as NOOP unless recovery is enabled
	            this.attemptInRepetitionRecovery(this.repetitionSepSecondInternal, [
	                prodOccurrence,
	                separator,
	                separatorLookAheadFunc,
	                action,
	                interpreter_1.NextTerminalAfterManySepWalker
	            ], separatorLookAheadFunc, keys_1.MANY_SEP_IDX, prodOccurrence, interpreter_1.NextTerminalAfterManySepWalker);
	        }
	    };
	    RecognizerEngine.prototype.repetitionSepSecondInternal = function (prodOccurrence, separator, separatorLookAheadFunc, action, nextTerminalAfterWalker) {
	        while (separatorLookAheadFunc()) {
	            // note that this CONSUME will never enter recovery because
	            // the separatorLookAheadFunc checks that the separator really does exist.
	            this.CONSUME(separator);
	            action.call(this);
	        }
	        // we can only arrive to this function after an error
	        // has occurred (hence the name 'second') so the following
	        // IF will always be entered, its possible to remove it...
	        // however it is kept to avoid confusion and be consistent.
	        // Performance optimization: "attemptInRepetitionRecovery" will be defined as NOOP unless recovery is enabled
	        /* istanbul ignore else */
	        this.attemptInRepetitionRecovery(this.repetitionSepSecondInternal, [
	            prodOccurrence,
	            separator,
	            separatorLookAheadFunc,
	            action,
	            nextTerminalAfterWalker
	        ], separatorLookAheadFunc, keys_1.AT_LEAST_ONE_SEP_IDX, prodOccurrence, nextTerminalAfterWalker);
	    };
	    RecognizerEngine.prototype.doSingleRepetition = function (action) {
	        var beforeIteration = this.getLexerPosition();
	        action.call(this);
	        var afterIteration = this.getLexerPosition();
	        // This boolean will indicate if this repetition progressed
	        // or if we are "stuck" (potential infinite loop in the repetition).
	        return afterIteration > beforeIteration;
	    };
	    RecognizerEngine.prototype.orInternal = function (altsOrOpts, occurrence) {
	        var laKey = this.getKeyForAutomaticLookahead(keys_1.OR_IDX, occurrence);
	        var alts = (0, isArray_1.default)(altsOrOpts) ? altsOrOpts : altsOrOpts.DEF;
	        var laFunc = this.getLaFuncFromCache(laKey);
	        var altIdxToTake = laFunc.call(this, alts);
	        if (altIdxToTake !== undefined) {
	            var chosenAlternative = alts[altIdxToTake];
	            return chosenAlternative.ALT.call(this);
	        }
	        this.raiseNoAltException(occurrence, altsOrOpts.ERR_MSG);
	    };
	    RecognizerEngine.prototype.ruleFinallyStateUpdate = function () {
	        this.RULE_STACK.pop();
	        this.RULE_OCCURRENCE_STACK.pop();
	        // NOOP when cst is disabled
	        this.cstFinallyStateUpdate();
	        if (this.RULE_STACK.length === 0 && this.isAtEndOfInput() === false) {
	            var firstRedundantTok = this.LA(1);
	            var errMsg = this.errorMessageProvider.buildNotAllInputParsedMessage({
	                firstRedundant: firstRedundantTok,
	                ruleName: this.getCurrRuleFullName()
	            });
	            this.SAVE_ERROR(new exceptions_public_1.NotAllInputParsedException(errMsg, firstRedundantTok));
	        }
	    };
	    RecognizerEngine.prototype.subruleInternal = function (ruleToCall, idx, options) {
	        var ruleResult;
	        try {
	            var args = options !== undefined ? options.ARGS : undefined;
	            this.subruleIdx = idx;
	            ruleResult = ruleToCall.apply(this, args);
	            this.cstPostNonTerminal(ruleResult, options !== undefined && options.LABEL !== undefined
	                ? options.LABEL
	                : ruleToCall.ruleName);
	            return ruleResult;
	        }
	        catch (e) {
	            throw this.subruleInternalError(e, options, ruleToCall.ruleName);
	        }
	    };
	    RecognizerEngine.prototype.subruleInternalError = function (e, options, ruleName) {
	        if ((0, exceptions_public_1.isRecognitionException)(e) && e.partialCstResult !== undefined) {
	            this.cstPostNonTerminal(e.partialCstResult, options !== undefined && options.LABEL !== undefined
	                ? options.LABEL
	                : ruleName);
	            delete e.partialCstResult;
	        }
	        throw e;
	    };
	    RecognizerEngine.prototype.consumeInternal = function (tokType, idx, options) {
	        var consumedToken;
	        try {
	            var nextToken = this.LA(1);
	            if (this.tokenMatcher(nextToken, tokType) === true) {
	                this.consumeToken();
	                consumedToken = nextToken;
	            }
	            else {
	                this.consumeInternalError(tokType, nextToken, options);
	            }
	        }
	        catch (eFromConsumption) {
	            consumedToken = this.consumeInternalRecovery(tokType, idx, eFromConsumption);
	        }
	        this.cstPostTerminal(options !== undefined && options.LABEL !== undefined
	            ? options.LABEL
	            : tokType.name, consumedToken);
	        return consumedToken;
	    };
	    RecognizerEngine.prototype.consumeInternalError = function (tokType, nextToken, options) {
	        var msg;
	        var previousToken = this.LA(0);
	        if (options !== undefined && options.ERR_MSG) {
	            msg = options.ERR_MSG;
	        }
	        else {
	            msg = this.errorMessageProvider.buildMismatchTokenMessage({
	                expected: tokType,
	                actual: nextToken,
	                previous: previousToken,
	                ruleName: this.getCurrRuleFullName()
	            });
	        }
	        throw this.SAVE_ERROR(new exceptions_public_1.MismatchedTokenException(msg, nextToken, previousToken));
	    };
	    RecognizerEngine.prototype.consumeInternalRecovery = function (tokType, idx, eFromConsumption) {
	        // no recovery allowed during backtracking, otherwise backtracking may recover invalid syntax and accept it
	        // but the original syntax could have been parsed successfully without any backtracking + recovery
	        if (this.recoveryEnabled &&
	            // TODO: more robust checking of the exception type. Perhaps Typescript extending expressions?
	            eFromConsumption.name === "MismatchedTokenException" &&
	            !this.isBackTracking()) {
	            var follows = this.getFollowsForInRuleRecovery(tokType, idx);
	            try {
	                return this.tryInRuleRecovery(tokType, follows);
	            }
	            catch (eFromInRuleRecovery) {
	                if (eFromInRuleRecovery.name === recoverable_1.IN_RULE_RECOVERY_EXCEPTION) {
	                    // failed in RuleRecovery.
	                    // throw the original error in order to trigger reSync error recovery
	                    throw eFromConsumption;
	                }
	                else {
	                    throw eFromInRuleRecovery;
	                }
	            }
	        }
	        else {
	            throw eFromConsumption;
	        }
	    };
	    RecognizerEngine.prototype.saveRecogState = function () {
	        // errors is a getter which will clone the errors array
	        var savedErrors = this.errors;
	        var savedRuleStack = (0, clone_1.default)(this.RULE_STACK);
	        return {
	            errors: savedErrors,
	            lexerState: this.exportLexerState(),
	            RULE_STACK: savedRuleStack,
	            CST_STACK: this.CST_STACK
	        };
	    };
	    RecognizerEngine.prototype.reloadRecogState = function (newState) {
	        this.errors = newState.errors;
	        this.importLexerState(newState.lexerState);
	        this.RULE_STACK = newState.RULE_STACK;
	    };
	    RecognizerEngine.prototype.ruleInvocationStateUpdate = function (shortName, fullName, idxInCallingRule) {
	        this.RULE_OCCURRENCE_STACK.push(idxInCallingRule);
	        this.RULE_STACK.push(shortName);
	        // NOOP when cst is disabled
	        this.cstInvocationStateUpdate(fullName);
	    };
	    RecognizerEngine.prototype.isBackTracking = function () {
	        return this.isBackTrackingStack.length !== 0;
	    };
	    RecognizerEngine.prototype.getCurrRuleFullName = function () {
	        var shortName = this.getLastExplicitRuleShortName();
	        return this.shortRuleNameToFull[shortName];
	    };
	    RecognizerEngine.prototype.shortRuleNameToFullName = function (shortName) {
	        return this.shortRuleNameToFull[shortName];
	    };
	    RecognizerEngine.prototype.isAtEndOfInput = function () {
	        return this.tokenMatcher(this.LA(1), tokens_public_1.EOF);
	    };
	    RecognizerEngine.prototype.reset = function () {
	        this.resetLexerState();
	        this.subruleIdx = 0;
	        this.isBackTrackingStack = [];
	        this.errors = [];
	        this.RULE_STACK = [];
	        // TODO: extract a specific reset for TreeBuilder trait
	        this.CST_STACK = [];
	        this.RULE_OCCURRENCE_STACK = [];
	    };
	    return RecognizerEngine;
	}());
	recognizer_engine.RecognizerEngine = RecognizerEngine;
	
	return recognizer_engine;
}

var error_handler = {};

var hasRequiredError_handler;

function requireError_handler () {
	if (hasRequiredError_handler) return error_handler;
	hasRequiredError_handler = 1;
	var __importDefault = (error_handler && error_handler.__importDefault) || function (mod) {
	    return (mod && mod.__esModule) ? mod : { "default": mod };
	};
	Object.defineProperty(error_handler, "__esModule", { value: true });
	error_handler.ErrorHandler = void 0;
	var exceptions_public_1 = requireExceptions_public();
	var has_1 = __importDefault(requireHas());
	var clone_1 = __importDefault(requireClone());
	var lookahead_1 = requireLookahead();
	var parser_1 = requireParser();
	/**
	 * Trait responsible for runtime parsing errors.
	 */
	var ErrorHandler = /** @class */ (function () {
	    function ErrorHandler() {
	    }
	    ErrorHandler.prototype.initErrorHandler = function (config) {
	        this._errors = [];
	        this.errorMessageProvider = (0, has_1.default)(config, "errorMessageProvider")
	            ? config.errorMessageProvider // assumes end user provides the correct config value/type
	            : parser_1.DEFAULT_PARSER_CONFIG.errorMessageProvider;
	    };
	    ErrorHandler.prototype.SAVE_ERROR = function (error) {
	        if ((0, exceptions_public_1.isRecognitionException)(error)) {
	            error.context = {
	                ruleStack: this.getHumanReadableRuleStack(),
	                ruleOccurrenceStack: (0, clone_1.default)(this.RULE_OCCURRENCE_STACK)
	            };
	            this._errors.push(error);
	            return error;
	        }
	        else {
	            throw Error("Trying to save an Error which is not a RecognitionException");
	        }
	    };
	    Object.defineProperty(ErrorHandler.prototype, "errors", {
	        get: function () {
	            return (0, clone_1.default)(this._errors);
	        },
	        set: function (newErrors) {
	            this._errors = newErrors;
	        },
	        enumerable: false,
	        configurable: true
	    });
	    // TODO: consider caching the error message computed information
	    ErrorHandler.prototype.raiseEarlyExitException = function (occurrence, prodType, userDefinedErrMsg) {
	        var ruleName = this.getCurrRuleFullName();
	        var ruleGrammar = this.getGAstProductions()[ruleName];
	        var lookAheadPathsPerAlternative = (0, lookahead_1.getLookaheadPathsForOptionalProd)(occurrence, ruleGrammar, prodType, this.maxLookahead);
	        var insideProdPaths = lookAheadPathsPerAlternative[0];
	        var actualTokens = [];
	        for (var i = 1; i <= this.maxLookahead; i++) {
	            actualTokens.push(this.LA(i));
	        }
	        var msg = this.errorMessageProvider.buildEarlyExitMessage({
	            expectedIterationPaths: insideProdPaths,
	            actual: actualTokens,
	            previous: this.LA(0),
	            customUserDescription: userDefinedErrMsg,
	            ruleName: ruleName
	        });
	        throw this.SAVE_ERROR(new exceptions_public_1.EarlyExitException(msg, this.LA(1), this.LA(0)));
	    };
	    // TODO: consider caching the error message computed information
	    ErrorHandler.prototype.raiseNoAltException = function (occurrence, errMsgTypes) {
	        var ruleName = this.getCurrRuleFullName();
	        var ruleGrammar = this.getGAstProductions()[ruleName];
	        // TODO: getLookaheadPathsForOr can be slow for large enough maxLookahead and certain grammars, consider caching ?
	        var lookAheadPathsPerAlternative = (0, lookahead_1.getLookaheadPathsForOr)(occurrence, ruleGrammar, this.maxLookahead);
	        var actualTokens = [];
	        for (var i = 1; i <= this.maxLookahead; i++) {
	            actualTokens.push(this.LA(i));
	        }
	        var previousToken = this.LA(0);
	        var errMsg = this.errorMessageProvider.buildNoViableAltMessage({
	            expectedPathsPerAlt: lookAheadPathsPerAlternative,
	            actual: actualTokens,
	            previous: previousToken,
	            customUserDescription: errMsgTypes,
	            ruleName: this.getCurrRuleFullName()
	        });
	        throw this.SAVE_ERROR(new exceptions_public_1.NoViableAltException(errMsg, this.LA(1), previousToken));
	    };
	    return ErrorHandler;
	}());
	error_handler.ErrorHandler = ErrorHandler;
	
	return error_handler;
}

var context_assist = {};

var hasRequiredContext_assist;

function requireContext_assist () {
	if (hasRequiredContext_assist) return context_assist;
	hasRequiredContext_assist = 1;
	var __importDefault = (context_assist && context_assist.__importDefault) || function (mod) {
	    return (mod && mod.__esModule) ? mod : { "default": mod };
	};
	Object.defineProperty(context_assist, "__esModule", { value: true });
	context_assist.ContentAssist = void 0;
	var interpreter_1 = requireInterpreter();
	var first_1 = __importDefault(requireFirst());
	var isUndefined_1 = __importDefault(requireIsUndefined());
	var ContentAssist = /** @class */ (function () {
	    function ContentAssist() {
	    }
	    ContentAssist.prototype.initContentAssist = function () { };
	    ContentAssist.prototype.computeContentAssist = function (startRuleName, precedingInput) {
	        var startRuleGast = this.gastProductionsCache[startRuleName];
	        if ((0, isUndefined_1.default)(startRuleGast)) {
	            throw Error("Rule ->".concat(startRuleName, "<- does not exist in this grammar."));
	        }
	        return (0, interpreter_1.nextPossibleTokensAfter)([startRuleGast], precedingInput, this.tokenMatcher, this.maxLookahead);
	    };
	    // TODO: should this be a member method or a utility? it does not have any state or usage of 'this'...
	    // TODO: should this be more explicitly part of the public API?
	    ContentAssist.prototype.getNextPossibleTokenTypes = function (grammarPath) {
	        var topRuleName = (0, first_1.default)(grammarPath.ruleStack);
	        var gastProductions = this.getGAstProductions();
	        var topProduction = gastProductions[topRuleName];
	        var nextPossibleTokenTypes = new interpreter_1.NextAfterTokenWalker(topProduction, grammarPath).startWalking();
	        return nextPossibleTokenTypes;
	    };
	    return ContentAssist;
	}());
	context_assist.ContentAssist = ContentAssist;
	
	return context_assist;
}

var gast_recorder = {};

var hasRequiredGast_recorder;

function requireGast_recorder () {
	if (hasRequiredGast_recorder) return gast_recorder;
	hasRequiredGast_recorder = 1;
	var __importDefault = (gast_recorder && gast_recorder.__importDefault) || function (mod) {
	    return (mod && mod.__esModule) ? mod : { "default": mod };
	};
	Object.defineProperty(gast_recorder, "__esModule", { value: true });
	gast_recorder.GastRecorder = void 0;
	var last_1 = __importDefault(requireLast());
	var isArray_1 = __importDefault(requireIsArray());
	var some_1 = __importDefault(requireSome());
	var forEach_1 = __importDefault(requireForEach());
	var isFunction_1 = __importDefault(requireIsFunction());
	var has_1 = __importDefault(requireHas());
	var gast_1 = requireApi$2();
	var lexer_public_1 = requireLexer_public();
	var tokens_1 = requireTokens();
	var tokens_public_1 = requireTokens_public();
	var parser_1 = requireParser();
	var keys_1 = requireKeys();
	var RECORDING_NULL_OBJECT = {
	    description: "This Object indicates the Parser is during Recording Phase"
	};
	Object.freeze(RECORDING_NULL_OBJECT);
	var HANDLE_SEPARATOR = true;
	var MAX_METHOD_IDX = Math.pow(2, keys_1.BITS_FOR_OCCURRENCE_IDX) - 1;
	var RFT = (0, tokens_public_1.createToken)({ name: "RECORDING_PHASE_TOKEN", pattern: lexer_public_1.Lexer.NA });
	(0, tokens_1.augmentTokenTypes)([RFT]);
	var RECORDING_PHASE_TOKEN = (0, tokens_public_1.createTokenInstance)(RFT, "This IToken indicates the Parser is in Recording Phase\n\t" +
	    "" +
	    "See: https://chevrotain.io/docs/guide/internals.html#grammar-recording for details", 
	// Using "-1" instead of NaN (as in EOF) because an actual number is less likely to
	// cause errors if the output of LA or CONSUME would be (incorrectly) used during the recording phase.
	-1, -1, -1, -1, -1, -1);
	Object.freeze(RECORDING_PHASE_TOKEN);
	var RECORDING_PHASE_CSTNODE = {
	    name: "This CSTNode indicates the Parser is in Recording Phase\n\t" +
	        "See: https://chevrotain.io/docs/guide/internals.html#grammar-recording for details",
	    children: {}
	};
	/**
	 * This trait handles the creation of the GAST structure for Chevrotain Grammars
	 */
	var GastRecorder = /** @class */ (function () {
	    function GastRecorder() {
	    }
	    GastRecorder.prototype.initGastRecorder = function (config) {
	        this.recordingProdStack = [];
	        this.RECORDING_PHASE = false;
	    };
	    GastRecorder.prototype.enableRecording = function () {
	        var _this = this;
	        this.RECORDING_PHASE = true;
	        this.TRACE_INIT("Enable Recording", function () {
	            var _loop_1 = function (i) {
	                var idx = i > 0 ? i : "";
	                _this["CONSUME".concat(idx)] = function (arg1, arg2) {
	                    return this.consumeInternalRecord(arg1, i, arg2);
	                };
	                _this["SUBRULE".concat(idx)] = function (arg1, arg2) {
	                    return this.subruleInternalRecord(arg1, i, arg2);
	                };
	                _this["OPTION".concat(idx)] = function (arg1) {
	                    return this.optionInternalRecord(arg1, i);
	                };
	                _this["OR".concat(idx)] = function (arg1) {
	                    return this.orInternalRecord(arg1, i);
	                };
	                _this["MANY".concat(idx)] = function (arg1) {
	                    this.manyInternalRecord(i, arg1);
	                };
	                _this["MANY_SEP".concat(idx)] = function (arg1) {
	                    this.manySepFirstInternalRecord(i, arg1);
	                };
	                _this["AT_LEAST_ONE".concat(idx)] = function (arg1) {
	                    this.atLeastOneInternalRecord(i, arg1);
	                };
	                _this["AT_LEAST_ONE_SEP".concat(idx)] = function (arg1) {
	                    this.atLeastOneSepFirstInternalRecord(i, arg1);
	                };
	            };
	            /**
	             * Warning Dark Voodoo Magic upcoming!
	             * We are "replacing" the public parsing DSL methods API
	             * With **new** alternative implementations on the Parser **instance**
	             *
	             * So far this is the only way I've found to avoid performance regressions during parsing time.
	             * - Approx 30% performance regression was measured on Chrome 75 Canary when attempting to replace the "internal"
	             *   implementations directly instead.
	             */
	            for (var i = 0; i < 10; i++) {
	                _loop_1(i);
	            }
	            // DSL methods with the idx(suffix) as an argument
	            _this["consume"] = function (idx, arg1, arg2) {
	                return this.consumeInternalRecord(arg1, idx, arg2);
	            };
	            _this["subrule"] = function (idx, arg1, arg2) {
	                return this.subruleInternalRecord(arg1, idx, arg2);
	            };
	            _this["option"] = function (idx, arg1) {
	                return this.optionInternalRecord(arg1, idx);
	            };
	            _this["or"] = function (idx, arg1) {
	                return this.orInternalRecord(arg1, idx);
	            };
	            _this["many"] = function (idx, arg1) {
	                this.manyInternalRecord(idx, arg1);
	            };
	            _this["atLeastOne"] = function (idx, arg1) {
	                this.atLeastOneInternalRecord(idx, arg1);
	            };
	            _this.ACTION = _this.ACTION_RECORD;
	            _this.BACKTRACK = _this.BACKTRACK_RECORD;
	            _this.LA = _this.LA_RECORD;
	        });
	    };
	    GastRecorder.prototype.disableRecording = function () {
	        var _this = this;
	        this.RECORDING_PHASE = false;
	        // By deleting these **instance** properties, any future invocation
	        // will be deferred to the original methods on the **prototype** object
	        // This seems to get rid of any incorrect optimizations that V8 may
	        // do during the recording phase.
	        this.TRACE_INIT("Deleting Recording methods", function () {
	            var that = _this;
	            for (var i = 0; i < 10; i++) {
	                var idx = i > 0 ? i : "";
	                delete that["CONSUME".concat(idx)];
	                delete that["SUBRULE".concat(idx)];
	                delete that["OPTION".concat(idx)];
	                delete that["OR".concat(idx)];
	                delete that["MANY".concat(idx)];
	                delete that["MANY_SEP".concat(idx)];
	                delete that["AT_LEAST_ONE".concat(idx)];
	                delete that["AT_LEAST_ONE_SEP".concat(idx)];
	            }
	            delete that["consume"];
	            delete that["subrule"];
	            delete that["option"];
	            delete that["or"];
	            delete that["many"];
	            delete that["atLeastOne"];
	            delete that.ACTION;
	            delete that.BACKTRACK;
	            delete that.LA;
	        });
	    };
	    //   Parser methods are called inside an ACTION?
	    //   Maybe try/catch/finally on ACTIONS while disabling the recorders state changes?
	    // @ts-expect-error -- noop place holder
	    GastRecorder.prototype.ACTION_RECORD = function (impl) {
	        // NO-OP during recording
	    };
	    // Executing backtracking logic will break our recording logic assumptions
	    GastRecorder.prototype.BACKTRACK_RECORD = function (grammarRule, args) {
	        return function () { return true; };
	    };
	    // LA is part of the official API and may be used for custom lookahead logic
	    // by end users who may forget to wrap it in ACTION or inside a GATE
	    GastRecorder.prototype.LA_RECORD = function (howMuch) {
	        // We cannot use the RECORD_PHASE_TOKEN here because someone may depend
	        // On LA return EOF at the end of the input so an infinite loop may occur.
	        return parser_1.END_OF_FILE;
	    };
	    GastRecorder.prototype.topLevelRuleRecord = function (name, def) {
	        try {
	            var newTopLevelRule = new gast_1.Rule({ definition: [], name: name });
	            newTopLevelRule.name = name;
	            this.recordingProdStack.push(newTopLevelRule);
	            def.call(this);
	            this.recordingProdStack.pop();
	            return newTopLevelRule;
	        }
	        catch (originalError) {
	            if (originalError.KNOWN_RECORDER_ERROR !== true) {
	                try {
	                    originalError.message =
	                        originalError.message +
	                            '\n\t This error was thrown during the "grammar recording phase" For more info see:\n\t' +
	                            "https://chevrotain.io/docs/guide/internals.html#grammar-recording";
	                }
	                catch (mutabilityError) {
	                    // We may not be able to modify the original error object
	                    throw originalError;
	                }
	            }
	            throw originalError;
	        }
	    };
	    // Implementation of parsing DSL
	    GastRecorder.prototype.optionInternalRecord = function (actionORMethodDef, occurrence) {
	        return recordProd.call(this, gast_1.Option, actionORMethodDef, occurrence);
	    };
	    GastRecorder.prototype.atLeastOneInternalRecord = function (occurrence, actionORMethodDef) {
	        recordProd.call(this, gast_1.RepetitionMandatory, actionORMethodDef, occurrence);
	    };
	    GastRecorder.prototype.atLeastOneSepFirstInternalRecord = function (occurrence, options) {
	        recordProd.call(this, gast_1.RepetitionMandatoryWithSeparator, options, occurrence, HANDLE_SEPARATOR);
	    };
	    GastRecorder.prototype.manyInternalRecord = function (occurrence, actionORMethodDef) {
	        recordProd.call(this, gast_1.Repetition, actionORMethodDef, occurrence);
	    };
	    GastRecorder.prototype.manySepFirstInternalRecord = function (occurrence, options) {
	        recordProd.call(this, gast_1.RepetitionWithSeparator, options, occurrence, HANDLE_SEPARATOR);
	    };
	    GastRecorder.prototype.orInternalRecord = function (altsOrOpts, occurrence) {
	        return recordOrProd.call(this, altsOrOpts, occurrence);
	    };
	    GastRecorder.prototype.subruleInternalRecord = function (ruleToCall, occurrence, options) {
	        assertMethodIdxIsValid(occurrence);
	        if (!ruleToCall || (0, has_1.default)(ruleToCall, "ruleName") === false) {
	            var error = new Error("<SUBRULE".concat(getIdxSuffix(occurrence), "> argument is invalid") +
	                " expecting a Parser method reference but got: <".concat(JSON.stringify(ruleToCall), ">") +
	                "\n inside top level rule: <".concat(this.recordingProdStack[0].name, ">"));
	            error.KNOWN_RECORDER_ERROR = true;
	            throw error;
	        }
	        var prevProd = (0, last_1.default)(this.recordingProdStack);
	        var ruleName = ruleToCall.ruleName;
	        var newNoneTerminal = new gast_1.NonTerminal({
	            idx: occurrence,
	            nonTerminalName: ruleName,
	            label: options === null || options === void 0 ? void 0 : options.LABEL,
	            // The resolving of the `referencedRule` property will be done once all the Rule's GASTs have been created
	            referencedRule: undefined
	        });
	        prevProd.definition.push(newNoneTerminal);
	        return this.outputCst ? RECORDING_PHASE_CSTNODE : RECORDING_NULL_OBJECT;
	    };
	    GastRecorder.prototype.consumeInternalRecord = function (tokType, occurrence, options) {
	        assertMethodIdxIsValid(occurrence);
	        if (!(0, tokens_1.hasShortKeyProperty)(tokType)) {
	            var error = new Error("<CONSUME".concat(getIdxSuffix(occurrence), "> argument is invalid") +
	                " expecting a TokenType reference but got: <".concat(JSON.stringify(tokType), ">") +
	                "\n inside top level rule: <".concat(this.recordingProdStack[0].name, ">"));
	            error.KNOWN_RECORDER_ERROR = true;
	            throw error;
	        }
	        var prevProd = (0, last_1.default)(this.recordingProdStack);
	        var newNoneTerminal = new gast_1.Terminal({
	            idx: occurrence,
	            terminalType: tokType,
	            label: options === null || options === void 0 ? void 0 : options.LABEL
	        });
	        prevProd.definition.push(newNoneTerminal);
	        return RECORDING_PHASE_TOKEN;
	    };
	    return GastRecorder;
	}());
	gast_recorder.GastRecorder = GastRecorder;
	function recordProd(prodConstructor, mainProdArg, occurrence, handleSep) {
	    if (handleSep === void 0) { handleSep = false; }
	    assertMethodIdxIsValid(occurrence);
	    var prevProd = (0, last_1.default)(this.recordingProdStack);
	    var grammarAction = (0, isFunction_1.default)(mainProdArg) ? mainProdArg : mainProdArg.DEF;
	    var newProd = new prodConstructor({ definition: [], idx: occurrence });
	    if (handleSep) {
	        newProd.separator = mainProdArg.SEP;
	    }
	    if ((0, has_1.default)(mainProdArg, "MAX_LOOKAHEAD")) {
	        newProd.maxLookahead = mainProdArg.MAX_LOOKAHEAD;
	    }
	    this.recordingProdStack.push(newProd);
	    grammarAction.call(this);
	    prevProd.definition.push(newProd);
	    this.recordingProdStack.pop();
	    return RECORDING_NULL_OBJECT;
	}
	function recordOrProd(mainProdArg, occurrence) {
	    var _this = this;
	    assertMethodIdxIsValid(occurrence);
	    var prevProd = (0, last_1.default)(this.recordingProdStack);
	    // Only an array of alternatives
	    var hasOptions = (0, isArray_1.default)(mainProdArg) === false;
	    var alts = hasOptions === false ? mainProdArg : mainProdArg.DEF;
	    var newOrProd = new gast_1.Alternation({
	        definition: [],
	        idx: occurrence,
	        ignoreAmbiguities: hasOptions && mainProdArg.IGNORE_AMBIGUITIES === true
	    });
	    if ((0, has_1.default)(mainProdArg, "MAX_LOOKAHEAD")) {
	        newOrProd.maxLookahead = mainProdArg.MAX_LOOKAHEAD;
	    }
	    var hasPredicates = (0, some_1.default)(alts, function (currAlt) { return (0, isFunction_1.default)(currAlt.GATE); });
	    newOrProd.hasPredicates = hasPredicates;
	    prevProd.definition.push(newOrProd);
	    (0, forEach_1.default)(alts, function (currAlt) {
	        var currAltFlat = new gast_1.Alternative({ definition: [] });
	        newOrProd.definition.push(currAltFlat);
	        if ((0, has_1.default)(currAlt, "IGNORE_AMBIGUITIES")) {
	            currAltFlat.ignoreAmbiguities = currAlt.IGNORE_AMBIGUITIES; // assumes end user provides the correct config value/type
	        }
	        // **implicit** ignoreAmbiguities due to usage of gate
	        else if ((0, has_1.default)(currAlt, "GATE")) {
	            currAltFlat.ignoreAmbiguities = true;
	        }
	        _this.recordingProdStack.push(currAltFlat);
	        currAlt.ALT.call(_this);
	        _this.recordingProdStack.pop();
	    });
	    return RECORDING_NULL_OBJECT;
	}
	function getIdxSuffix(idx) {
	    return idx === 0 ? "" : "".concat(idx);
	}
	function assertMethodIdxIsValid(idx) {
	    if (idx < 0 || idx > MAX_METHOD_IDX) {
	        var error = new Error(
	        // The stack trace will contain all the needed details
	        "Invalid DSL Method idx value: <".concat(idx, ">\n\t") +
	            "Idx value must be a none negative value smaller than ".concat(MAX_METHOD_IDX + 1));
	        error.KNOWN_RECORDER_ERROR = true;
	        throw error;
	    }
	}
	
	return gast_recorder;
}

var perf_tracer = {};

var hasRequiredPerf_tracer;

function requirePerf_tracer () {
	if (hasRequiredPerf_tracer) return perf_tracer;
	hasRequiredPerf_tracer = 1;
	var __importDefault = (perf_tracer && perf_tracer.__importDefault) || function (mod) {
	    return (mod && mod.__esModule) ? mod : { "default": mod };
	};
	Object.defineProperty(perf_tracer, "__esModule", { value: true });
	perf_tracer.PerformanceTracer = void 0;
	var has_1 = __importDefault(requireHas());
	var utils_1 = requireApi$3();
	var parser_1 = requireParser();
	/**
	 * Trait responsible for runtime parsing errors.
	 */
	var PerformanceTracer = /** @class */ (function () {
	    function PerformanceTracer() {
	    }
	    PerformanceTracer.prototype.initPerformanceTracer = function (config) {
	        if ((0, has_1.default)(config, "traceInitPerf")) {
	            var userTraceInitPerf = config.traceInitPerf;
	            var traceIsNumber = typeof userTraceInitPerf === "number";
	            this.traceInitMaxIdent = traceIsNumber
	                ? userTraceInitPerf
	                : Infinity;
	            this.traceInitPerf = traceIsNumber
	                ? userTraceInitPerf > 0
	                : userTraceInitPerf; // assumes end user provides the correct config value/type
	        }
	        else {
	            this.traceInitMaxIdent = 0;
	            this.traceInitPerf = parser_1.DEFAULT_PARSER_CONFIG.traceInitPerf;
	        }
	        this.traceInitIndent = -1;
	    };
	    PerformanceTracer.prototype.TRACE_INIT = function (phaseDesc, phaseImpl) {
	        // No need to optimize this using NOOP pattern because
	        // It is not called in a hot spot...
	        if (this.traceInitPerf === true) {
	            this.traceInitIndent++;
	            var indent = new Array(this.traceInitIndent + 1).join("\t");
	            if (this.traceInitIndent < this.traceInitMaxIdent) {
	                console.log("".concat(indent, "--> <").concat(phaseDesc, ">"));
	            }
	            var _a = (0, utils_1.timer)(phaseImpl), time = _a.time, value = _a.value;
	            /* istanbul ignore next - Difficult to reproduce specific performance behavior (>10ms) in tests */
	            var traceMethod = time > 10 ? console.warn : console.log;
	            if (this.traceInitIndent < this.traceInitMaxIdent) {
	                traceMethod("".concat(indent, "<-- <").concat(phaseDesc, "> time: ").concat(time, "ms"));
	            }
	            this.traceInitIndent--;
	            return value;
	        }
	        else {
	            return phaseImpl();
	        }
	    };
	    return PerformanceTracer;
	}());
	perf_tracer.PerformanceTracer = PerformanceTracer;
	
	return perf_tracer;
}

var apply_mixins = {};

var hasRequiredApply_mixins;

function requireApply_mixins () {
	if (hasRequiredApply_mixins) return apply_mixins;
	hasRequiredApply_mixins = 1;
	Object.defineProperty(apply_mixins, "__esModule", { value: true });
	apply_mixins.applyMixins = void 0;
	function applyMixins(derivedCtor, baseCtors) {
	    baseCtors.forEach(function (baseCtor) {
	        var baseProto = baseCtor.prototype;
	        Object.getOwnPropertyNames(baseProto).forEach(function (propName) {
	            if (propName === "constructor") {
	                return;
	            }
	            var basePropDescriptor = Object.getOwnPropertyDescriptor(baseProto, propName);
	            // Handle Accessors
	            if (basePropDescriptor &&
	                (basePropDescriptor.get || basePropDescriptor.set)) {
	                Object.defineProperty(derivedCtor.prototype, propName, basePropDescriptor);
	            }
	            else {
	                derivedCtor.prototype[propName] = baseCtor.prototype[propName];
	            }
	        });
	    });
	}
	apply_mixins.applyMixins = applyMixins;
	
	return apply_mixins;
}

var hasRequiredParser;

function requireParser () {
	if (hasRequiredParser) return parser$1;
	hasRequiredParser = 1;
	(function (exports) {
		var __extends = (parser$1 && parser$1.__extends) || (function () {
		    var extendStatics = function (d, b) {
		        extendStatics = Object.setPrototypeOf ||
		            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
		            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
		        return extendStatics(d, b);
		    };
		    return function (d, b) {
		        if (typeof b !== "function" && b !== null)
		            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
		        extendStatics(d, b);
		        function __() { this.constructor = d; }
		        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
		    };
		})();
		var __importDefault = (parser$1 && parser$1.__importDefault) || function (mod) {
		    return (mod && mod.__esModule) ? mod : { "default": mod };
		};
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.EmbeddedActionsParser = exports.CstParser = exports.Parser = exports.EMPTY_ALT = exports.ParserDefinitionErrorType = exports.DEFAULT_RULE_CONFIG = exports.DEFAULT_PARSER_CONFIG = exports.END_OF_FILE = void 0;
		var isEmpty_1 = __importDefault(requireIsEmpty());
		var map_1 = __importDefault(requireMap());
		var forEach_1 = __importDefault(requireForEach());
		var values_1 = __importDefault(requireValues());
		var has_1 = __importDefault(requireHas());
		var clone_1 = __importDefault(requireClone());
		var utils_1 = requireApi$3();
		var follow_1 = requireFollow();
		var tokens_public_1 = requireTokens_public();
		var errors_public_1 = requireErrors_public();
		var gast_resolver_public_1 = requireGast_resolver_public();
		var recoverable_1 = requireRecoverable();
		var looksahead_1 = requireLooksahead();
		var tree_builder_1 = requireTree_builder();
		var lexer_adapter_1 = requireLexer_adapter();
		var recognizer_api_1 = requireRecognizer_api();
		var recognizer_engine_1 = requireRecognizer_engine();
		var error_handler_1 = requireError_handler();
		var context_assist_1 = requireContext_assist();
		var gast_recorder_1 = requireGast_recorder();
		var perf_tracer_1 = requirePerf_tracer();
		var apply_mixins_1 = requireApply_mixins();
		var checks_1 = requireChecks();
		exports.END_OF_FILE = (0, tokens_public_1.createTokenInstance)(tokens_public_1.EOF, "", NaN, NaN, NaN, NaN, NaN, NaN);
		Object.freeze(exports.END_OF_FILE);
		exports.DEFAULT_PARSER_CONFIG = Object.freeze({
		    recoveryEnabled: false,
		    maxLookahead: 3,
		    dynamicTokensEnabled: false,
		    outputCst: true,
		    errorMessageProvider: errors_public_1.defaultParserErrorProvider,
		    nodeLocationTracking: "none",
		    traceInitPerf: false,
		    skipValidations: false
		});
		exports.DEFAULT_RULE_CONFIG = Object.freeze({
		    recoveryValueFunc: function () { return undefined; },
		    resyncEnabled: true
		});
		(function (ParserDefinitionErrorType) {
		    ParserDefinitionErrorType[ParserDefinitionErrorType["INVALID_RULE_NAME"] = 0] = "INVALID_RULE_NAME";
		    ParserDefinitionErrorType[ParserDefinitionErrorType["DUPLICATE_RULE_NAME"] = 1] = "DUPLICATE_RULE_NAME";
		    ParserDefinitionErrorType[ParserDefinitionErrorType["INVALID_RULE_OVERRIDE"] = 2] = "INVALID_RULE_OVERRIDE";
		    ParserDefinitionErrorType[ParserDefinitionErrorType["DUPLICATE_PRODUCTIONS"] = 3] = "DUPLICATE_PRODUCTIONS";
		    ParserDefinitionErrorType[ParserDefinitionErrorType["UNRESOLVED_SUBRULE_REF"] = 4] = "UNRESOLVED_SUBRULE_REF";
		    ParserDefinitionErrorType[ParserDefinitionErrorType["LEFT_RECURSION"] = 5] = "LEFT_RECURSION";
		    ParserDefinitionErrorType[ParserDefinitionErrorType["NONE_LAST_EMPTY_ALT"] = 6] = "NONE_LAST_EMPTY_ALT";
		    ParserDefinitionErrorType[ParserDefinitionErrorType["AMBIGUOUS_ALTS"] = 7] = "AMBIGUOUS_ALTS";
		    ParserDefinitionErrorType[ParserDefinitionErrorType["CONFLICT_TOKENS_RULES_NAMESPACE"] = 8] = "CONFLICT_TOKENS_RULES_NAMESPACE";
		    ParserDefinitionErrorType[ParserDefinitionErrorType["INVALID_TOKEN_NAME"] = 9] = "INVALID_TOKEN_NAME";
		    ParserDefinitionErrorType[ParserDefinitionErrorType["NO_NON_EMPTY_LOOKAHEAD"] = 10] = "NO_NON_EMPTY_LOOKAHEAD";
		    ParserDefinitionErrorType[ParserDefinitionErrorType["AMBIGUOUS_PREFIX_ALTS"] = 11] = "AMBIGUOUS_PREFIX_ALTS";
		    ParserDefinitionErrorType[ParserDefinitionErrorType["TOO_MANY_ALTS"] = 12] = "TOO_MANY_ALTS";
		    ParserDefinitionErrorType[ParserDefinitionErrorType["CUSTOM_LOOKAHEAD_VALIDATION"] = 13] = "CUSTOM_LOOKAHEAD_VALIDATION";
		})(exports.ParserDefinitionErrorType || (exports.ParserDefinitionErrorType = {}));
		function EMPTY_ALT(value) {
		    if (value === void 0) { value = undefined; }
		    return function () {
		        return value;
		    };
		}
		exports.EMPTY_ALT = EMPTY_ALT;
		var Parser = /** @class */ (function () {
		    function Parser(tokenVocabulary, config) {
		        this.definitionErrors = [];
		        this.selfAnalysisDone = false;
		        var that = this;
		        that.initErrorHandler(config);
		        that.initLexerAdapter();
		        that.initLooksAhead(config);
		        that.initRecognizerEngine(tokenVocabulary, config);
		        that.initRecoverable(config);
		        that.initTreeBuilder(config);
		        that.initContentAssist();
		        that.initGastRecorder(config);
		        that.initPerformanceTracer(config);
		        if ((0, has_1.default)(config, "ignoredIssues")) {
		            throw new Error("The <ignoredIssues> IParserConfig property has been deprecated.\n\t" +
		                "Please use the <IGNORE_AMBIGUITIES> flag on the relevant DSL method instead.\n\t" +
		                "See: https://chevrotain.io/docs/guide/resolving_grammar_errors.html#IGNORING_AMBIGUITIES\n\t" +
		                "For further details.");
		        }
		        this.skipValidations = (0, has_1.default)(config, "skipValidations")
		            ? config.skipValidations // casting assumes the end user passing the correct type
		            : exports.DEFAULT_PARSER_CONFIG.skipValidations;
		    }
		    /**
		     *  @deprecated use the **instance** method with the same name instead
		     */
		    Parser.performSelfAnalysis = function (parserInstance) {
		        throw Error("The **static** `performSelfAnalysis` method has been deprecated." +
		            "\t\nUse the **instance** method with the same name instead.");
		    };
		    Parser.prototype.performSelfAnalysis = function () {
		        var _this = this;
		        this.TRACE_INIT("performSelfAnalysis", function () {
		            var defErrorsMsgs;
		            _this.selfAnalysisDone = true;
		            var className = _this.className;
		            _this.TRACE_INIT("toFastProps", function () {
		                // Without this voodoo magic the parser would be x3-x4 slower
		                // It seems it is better to invoke `toFastProperties` **before**
		                // Any manipulations of the `this` object done during the recording phase.
		                (0, utils_1.toFastProperties)(_this);
		            });
		            _this.TRACE_INIT("Grammar Recording", function () {
		                try {
		                    _this.enableRecording();
		                    // Building the GAST
		                    (0, forEach_1.default)(_this.definedRulesNames, function (currRuleName) {
		                        var wrappedRule = _this[currRuleName];
		                        var originalGrammarAction = wrappedRule["originalGrammarAction"];
		                        var recordedRuleGast;
		                        _this.TRACE_INIT("".concat(currRuleName, " Rule"), function () {
		                            recordedRuleGast = _this.topLevelRuleRecord(currRuleName, originalGrammarAction);
		                        });
		                        _this.gastProductionsCache[currRuleName] = recordedRuleGast;
		                    });
		                }
		                finally {
		                    _this.disableRecording();
		                }
		            });
		            var resolverErrors = [];
		            _this.TRACE_INIT("Grammar Resolving", function () {
		                resolverErrors = (0, gast_resolver_public_1.resolveGrammar)({
		                    rules: (0, values_1.default)(_this.gastProductionsCache)
		                });
		                _this.definitionErrors = _this.definitionErrors.concat(resolverErrors);
		            });
		            _this.TRACE_INIT("Grammar Validations", function () {
		                // only perform additional grammar validations IFF no resolving errors have occurred.
		                // as unresolved grammar may lead to unhandled runtime exceptions in the follow up validations.
		                if ((0, isEmpty_1.default)(resolverErrors) && _this.skipValidations === false) {
		                    var validationErrors = (0, gast_resolver_public_1.validateGrammar)({
		                        rules: (0, values_1.default)(_this.gastProductionsCache),
		                        tokenTypes: (0, values_1.default)(_this.tokensMap),
		                        errMsgProvider: errors_public_1.defaultGrammarValidatorErrorProvider,
		                        grammarName: className
		                    });
		                    var lookaheadValidationErrors = (0, checks_1.validateLookahead)({
		                        lookaheadStrategy: _this.lookaheadStrategy,
		                        rules: (0, values_1.default)(_this.gastProductionsCache),
		                        tokenTypes: (0, values_1.default)(_this.tokensMap),
		                        grammarName: className
		                    });
		                    _this.definitionErrors = _this.definitionErrors.concat(validationErrors, lookaheadValidationErrors);
		                }
		            });
		            // this analysis may fail if the grammar is not perfectly valid
		            if ((0, isEmpty_1.default)(_this.definitionErrors)) {
		                // The results of these computations are not needed unless error recovery is enabled.
		                if (_this.recoveryEnabled) {
		                    _this.TRACE_INIT("computeAllProdsFollows", function () {
		                        var allFollows = (0, follow_1.computeAllProdsFollows)((0, values_1.default)(_this.gastProductionsCache));
		                        _this.resyncFollows = allFollows;
		                    });
		                }
		                _this.TRACE_INIT("ComputeLookaheadFunctions", function () {
		                    var _a, _b;
		                    (_b = (_a = _this.lookaheadStrategy).initialize) === null || _b === void 0 ? void 0 : _b.call(_a, {
		                        rules: (0, values_1.default)(_this.gastProductionsCache)
		                    });
		                    _this.preComputeLookaheadFunctions((0, values_1.default)(_this.gastProductionsCache));
		                });
		            }
		            if (!Parser.DEFER_DEFINITION_ERRORS_HANDLING &&
		                !(0, isEmpty_1.default)(_this.definitionErrors)) {
		                defErrorsMsgs = (0, map_1.default)(_this.definitionErrors, function (defError) { return defError.message; });
		                throw new Error("Parser Definition Errors detected:\n ".concat(defErrorsMsgs.join("\n-------------------------------\n")));
		            }
		        });
		    };
		    // Set this flag to true if you don't want the Parser to throw error when problems in it's definition are detected.
		    // (normally during the parser's constructor).
		    // This is a design time flag, it will not affect the runtime error handling of the parser, just design time errors,
		    // for example: duplicate rule names, referencing an unresolved subrule, ect...
		    // This flag should not be enabled during normal usage, it is used in special situations, for example when
		    // needing to display the parser definition errors in some GUI(online playground).
		    Parser.DEFER_DEFINITION_ERRORS_HANDLING = false;
		    return Parser;
		}());
		exports.Parser = Parser;
		(0, apply_mixins_1.applyMixins)(Parser, [
		    recoverable_1.Recoverable,
		    looksahead_1.LooksAhead,
		    tree_builder_1.TreeBuilder,
		    lexer_adapter_1.LexerAdapter,
		    recognizer_engine_1.RecognizerEngine,
		    recognizer_api_1.RecognizerApi,
		    error_handler_1.ErrorHandler,
		    context_assist_1.ContentAssist,
		    gast_recorder_1.GastRecorder,
		    perf_tracer_1.PerformanceTracer
		]);
		var CstParser = /** @class */ (function (_super) {
		    __extends(CstParser, _super);
		    function CstParser(tokenVocabulary, config) {
		        if (config === void 0) { config = exports.DEFAULT_PARSER_CONFIG; }
		        var configClone = (0, clone_1.default)(config);
		        configClone.outputCst = true;
		        return _super.call(this, tokenVocabulary, configClone) || this;
		    }
		    return CstParser;
		}(Parser));
		exports.CstParser = CstParser;
		var EmbeddedActionsParser = /** @class */ (function (_super) {
		    __extends(EmbeddedActionsParser, _super);
		    function EmbeddedActionsParser(tokenVocabulary, config) {
		        if (config === void 0) { config = exports.DEFAULT_PARSER_CONFIG; }
		        var configClone = (0, clone_1.default)(config);
		        configClone.outputCst = false;
		        return _super.call(this, tokenVocabulary, configClone) || this;
		    }
		    return EmbeddedActionsParser;
		}(Parser));
		exports.EmbeddedActionsParser = EmbeddedActionsParser;
		
	} (parser$1));
	return parser$1;
}

var api = {};

var model = {};

var hasRequiredModel;

function requireModel () {
	if (hasRequiredModel) return model;
	hasRequiredModel = 1;
	var __extends = (model && model.__extends) || (function () {
	    var extendStatics = function (d, b) {
	        extendStatics = Object.setPrototypeOf ||
	            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
	            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
	        return extendStatics(d, b);
	    };
	    return function (d, b) {
	        if (typeof b !== "function" && b !== null)
	            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
	        extendStatics(d, b);
	        function __() { this.constructor = d; }
	        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
	    };
	})();
	var __importDefault = (model && model.__importDefault) || function (mod) {
	    return (mod && mod.__esModule) ? mod : { "default": mod };
	};
	Object.defineProperty(model, "__esModule", { value: true });
	model.buildModel = void 0;
	var gast_1 = requireApi$2();
	var map_1 = __importDefault(requireMap());
	var flatten_1 = __importDefault(requireFlatten());
	var values_1 = __importDefault(requireValues());
	var some_1 = __importDefault(requireSome());
	var groupBy_1 = __importDefault(requireGroupBy());
	var assign_1 = __importDefault(requireAssign());
	function buildModel(productions) {
	    var generator = new CstNodeDefinitionGenerator();
	    var allRules = (0, values_1.default)(productions);
	    return (0, map_1.default)(allRules, function (rule) { return generator.visitRule(rule); });
	}
	model.buildModel = buildModel;
	var CstNodeDefinitionGenerator = /** @class */ (function (_super) {
	    __extends(CstNodeDefinitionGenerator, _super);
	    function CstNodeDefinitionGenerator() {
	        return _super !== null && _super.apply(this, arguments) || this;
	    }
	    CstNodeDefinitionGenerator.prototype.visitRule = function (node) {
	        var rawElements = this.visitEach(node.definition);
	        var grouped = (0, groupBy_1.default)(rawElements, function (el) { return el.propertyName; });
	        var properties = (0, map_1.default)(grouped, function (group, propertyName) {
	            var allNullable = !(0, some_1.default)(group, function (el) { return !el.canBeNull; });
	            // In an alternation with a label a property name can have
	            // multiple types.
	            var propertyType = group[0].type;
	            if (group.length > 1) {
	                propertyType = (0, map_1.default)(group, function (g) { return g.type; });
	            }
	            return {
	                name: propertyName,
	                type: propertyType,
	                optional: allNullable
	            };
	        });
	        return {
	            name: node.name,
	            properties: properties
	        };
	    };
	    CstNodeDefinitionGenerator.prototype.visitAlternative = function (node) {
	        return this.visitEachAndOverrideWith(node.definition, { canBeNull: true });
	    };
	    CstNodeDefinitionGenerator.prototype.visitOption = function (node) {
	        return this.visitEachAndOverrideWith(node.definition, { canBeNull: true });
	    };
	    CstNodeDefinitionGenerator.prototype.visitRepetition = function (node) {
	        return this.visitEachAndOverrideWith(node.definition, { canBeNull: true });
	    };
	    CstNodeDefinitionGenerator.prototype.visitRepetitionMandatory = function (node) {
	        return this.visitEach(node.definition);
	    };
	    CstNodeDefinitionGenerator.prototype.visitRepetitionMandatoryWithSeparator = function (node) {
	        return this.visitEach(node.definition).concat({
	            propertyName: node.separator.name,
	            canBeNull: true,
	            type: getType(node.separator)
	        });
	    };
	    CstNodeDefinitionGenerator.prototype.visitRepetitionWithSeparator = function (node) {
	        return this.visitEachAndOverrideWith(node.definition, {
	            canBeNull: true
	        }).concat({
	            propertyName: node.separator.name,
	            canBeNull: true,
	            type: getType(node.separator)
	        });
	    };
	    CstNodeDefinitionGenerator.prototype.visitAlternation = function (node) {
	        return this.visitEachAndOverrideWith(node.definition, { canBeNull: true });
	    };
	    CstNodeDefinitionGenerator.prototype.visitTerminal = function (node) {
	        return [
	            {
	                propertyName: node.label || node.terminalType.name,
	                canBeNull: false,
	                type: getType(node)
	            }
	        ];
	    };
	    CstNodeDefinitionGenerator.prototype.visitNonTerminal = function (node) {
	        return [
	            {
	                propertyName: node.label || node.nonTerminalName,
	                canBeNull: false,
	                type: getType(node)
	            }
	        ];
	    };
	    CstNodeDefinitionGenerator.prototype.visitEachAndOverrideWith = function (definition, override) {
	        return (0, map_1.default)(this.visitEach(definition), function (definition) { return (0, assign_1.default)({}, definition, override); });
	    };
	    CstNodeDefinitionGenerator.prototype.visitEach = function (definition) {
	        var _this = this;
	        return (0, flatten_1.default)((0, map_1.default)(definition, function (definition) { return _this.visit(definition); }));
	    };
	    return CstNodeDefinitionGenerator;
	}(gast_1.GAstVisitor));
	function getType(production) {
	    if (production instanceof gast_1.NonTerminal) {
	        return {
	            kind: "rule",
	            name: production.referencedRule.name
	        };
	    }
	    return { kind: "token" };
	}
	
	return model;
}

var generate$1 = {};

var _castSlice;
var hasRequired_castSlice;

function require_castSlice () {
	if (hasRequired_castSlice) return _castSlice;
	hasRequired_castSlice = 1;
	var baseSlice = require_baseSlice();

	/**
	 * Casts `array` to a slice if it's needed.
	 *
	 * @private
	 * @param {Array} array The array to inspect.
	 * @param {number} start The start position.
	 * @param {number} [end=array.length] The end position.
	 * @returns {Array} Returns the cast slice.
	 */
	function castSlice(array, start, end) {
	  var length = array.length;
	  end = end === undefined ? length : end;
	  return (!start && end >= length) ? array : baseSlice(array, start, end);
	}

	_castSlice = castSlice;
	return _castSlice;
}

/** Used to compose unicode character classes. */

var _hasUnicode;
var hasRequired_hasUnicode;

function require_hasUnicode () {
	if (hasRequired_hasUnicode) return _hasUnicode;
	hasRequired_hasUnicode = 1;
	var rsAstralRange = '\\ud800-\\udfff',
	    rsComboMarksRange = '\\u0300-\\u036f',
	    reComboHalfMarksRange = '\\ufe20-\\ufe2f',
	    rsComboSymbolsRange = '\\u20d0-\\u20ff',
	    rsComboRange = rsComboMarksRange + reComboHalfMarksRange + rsComboSymbolsRange,
	    rsVarRange = '\\ufe0e\\ufe0f';

	/** Used to compose unicode capture groups. */
	var rsZWJ = '\\u200d';

	/** Used to detect strings with [zero-width joiners or code points from the astral planes](http://eev.ee/blog/2015/09/12/dark-corners-of-unicode/). */
	var reHasUnicode = RegExp('[' + rsZWJ + rsAstralRange  + rsComboRange + rsVarRange + ']');

	/**
	 * Checks if `string` contains Unicode symbols.
	 *
	 * @private
	 * @param {string} string The string to inspect.
	 * @returns {boolean} Returns `true` if a symbol is found, else `false`.
	 */
	function hasUnicode(string) {
	  return reHasUnicode.test(string);
	}

	_hasUnicode = hasUnicode;
	return _hasUnicode;
}

/**
 * Converts an ASCII `string` to an array.
 *
 * @private
 * @param {string} string The string to convert.
 * @returns {Array} Returns the converted array.
 */

var _asciiToArray;
var hasRequired_asciiToArray;

function require_asciiToArray () {
	if (hasRequired_asciiToArray) return _asciiToArray;
	hasRequired_asciiToArray = 1;
	function asciiToArray(string) {
	  return string.split('');
	}

	_asciiToArray = asciiToArray;
	return _asciiToArray;
}

/** Used to compose unicode character classes. */

var _unicodeToArray;
var hasRequired_unicodeToArray;

function require_unicodeToArray () {
	if (hasRequired_unicodeToArray) return _unicodeToArray;
	hasRequired_unicodeToArray = 1;
	var rsAstralRange = '\\ud800-\\udfff',
	    rsComboMarksRange = '\\u0300-\\u036f',
	    reComboHalfMarksRange = '\\ufe20-\\ufe2f',
	    rsComboSymbolsRange = '\\u20d0-\\u20ff',
	    rsComboRange = rsComboMarksRange + reComboHalfMarksRange + rsComboSymbolsRange,
	    rsVarRange = '\\ufe0e\\ufe0f';

	/** Used to compose unicode capture groups. */
	var rsAstral = '[' + rsAstralRange + ']',
	    rsCombo = '[' + rsComboRange + ']',
	    rsFitz = '\\ud83c[\\udffb-\\udfff]',
	    rsModifier = '(?:' + rsCombo + '|' + rsFitz + ')',
	    rsNonAstral = '[^' + rsAstralRange + ']',
	    rsRegional = '(?:\\ud83c[\\udde6-\\uddff]){2}',
	    rsSurrPair = '[\\ud800-\\udbff][\\udc00-\\udfff]',
	    rsZWJ = '\\u200d';

	/** Used to compose unicode regexes. */
	var reOptMod = rsModifier + '?',
	    rsOptVar = '[' + rsVarRange + ']?',
	    rsOptJoin = '(?:' + rsZWJ + '(?:' + [rsNonAstral, rsRegional, rsSurrPair].join('|') + ')' + rsOptVar + reOptMod + ')*',
	    rsSeq = rsOptVar + reOptMod + rsOptJoin,
	    rsSymbol = '(?:' + [rsNonAstral + rsCombo + '?', rsCombo, rsRegional, rsSurrPair, rsAstral].join('|') + ')';

	/** Used to match [string symbols](https://mathiasbynens.be/notes/javascript-unicode). */
	var reUnicode = RegExp(rsFitz + '(?=' + rsFitz + ')|' + rsSymbol + rsSeq, 'g');

	/**
	 * Converts a Unicode `string` to an array.
	 *
	 * @private
	 * @param {string} string The string to convert.
	 * @returns {Array} Returns the converted array.
	 */
	function unicodeToArray(string) {
	  return string.match(reUnicode) || [];
	}

	_unicodeToArray = unicodeToArray;
	return _unicodeToArray;
}

var _stringToArray;
var hasRequired_stringToArray;

function require_stringToArray () {
	if (hasRequired_stringToArray) return _stringToArray;
	hasRequired_stringToArray = 1;
	var asciiToArray = require_asciiToArray(),
	    hasUnicode = require_hasUnicode(),
	    unicodeToArray = require_unicodeToArray();

	/**
	 * Converts `string` to an array.
	 *
	 * @private
	 * @param {string} string The string to convert.
	 * @returns {Array} Returns the converted array.
	 */
	function stringToArray(string) {
	  return hasUnicode(string)
	    ? unicodeToArray(string)
	    : asciiToArray(string);
	}

	_stringToArray = stringToArray;
	return _stringToArray;
}

var _createCaseFirst;
var hasRequired_createCaseFirst;

function require_createCaseFirst () {
	if (hasRequired_createCaseFirst) return _createCaseFirst;
	hasRequired_createCaseFirst = 1;
	var castSlice = require_castSlice(),
	    hasUnicode = require_hasUnicode(),
	    stringToArray = require_stringToArray(),
	    toString = requireToString();

	/**
	 * Creates a function like `_.lowerFirst`.
	 *
	 * @private
	 * @param {string} methodName The name of the `String` case method to use.
	 * @returns {Function} Returns the new case function.
	 */
	function createCaseFirst(methodName) {
	  return function(string) {
	    string = toString(string);

	    var strSymbols = hasUnicode(string)
	      ? stringToArray(string)
	      : undefined;

	    var chr = strSymbols
	      ? strSymbols[0]
	      : string.charAt(0);

	    var trailing = strSymbols
	      ? castSlice(strSymbols, 1).join('')
	      : string.slice(1);

	    return chr[methodName]() + trailing;
	  };
	}

	_createCaseFirst = createCaseFirst;
	return _createCaseFirst;
}

var upperFirst_1;
var hasRequiredUpperFirst;

function requireUpperFirst () {
	if (hasRequiredUpperFirst) return upperFirst_1;
	hasRequiredUpperFirst = 1;
	var createCaseFirst = require_createCaseFirst();

	/**
	 * Converts the first character of `string` to upper case.
	 *
	 * @static
	 * @memberOf _
	 * @since 4.0.0
	 * @category String
	 * @param {string} [string=''] The string to convert.
	 * @returns {string} Returns the converted string.
	 * @example
	 *
	 * _.upperFirst('fred');
	 * // => 'Fred'
	 *
	 * _.upperFirst('FRED');
	 * // => 'FRED'
	 */
	var upperFirst = createCaseFirst('toUpperCase');

	upperFirst_1 = upperFirst;
	return upperFirst_1;
}

var hasRequiredGenerate;

function requireGenerate () {
	if (hasRequiredGenerate) return generate$1;
	hasRequiredGenerate = 1;
	var __importDefault = (generate$1 && generate$1.__importDefault) || function (mod) {
	    return (mod && mod.__esModule) ? mod : { "default": mod };
	};
	Object.defineProperty(generate$1, "__esModule", { value: true });
	generate$1.genDts = void 0;
	var flatten_1 = __importDefault(requireFlatten());
	var isArray_1 = __importDefault(requireIsArray());
	var map_1 = __importDefault(requireMap());
	var reduce_1 = __importDefault(requireReduce());
	var uniq_1 = __importDefault(requireUniq());
	var upperFirst_1 = __importDefault(requireUpperFirst());
	function genDts(model, options) {
	    var contentParts = [];
	    contentParts = contentParts.concat("import type { CstNode, ICstVisitor, IToken } from \"chevrotain\";");
	    contentParts = contentParts.concat((0, flatten_1.default)((0, map_1.default)(model, function (node) { return genCstNodeTypes(node); })));
	    if (options.includeVisitorInterface) {
	        contentParts = contentParts.concat(genVisitor(options.visitorInterfaceName, model));
	    }
	    return contentParts.join("\n\n") + "\n";
	}
	generate$1.genDts = genDts;
	function genCstNodeTypes(node) {
	    var nodeCstInterface = genNodeInterface(node);
	    var nodeChildrenInterface = genNodeChildrenType(node);
	    return [nodeCstInterface, nodeChildrenInterface];
	}
	function genNodeInterface(node) {
	    var nodeInterfaceName = getNodeInterfaceName(node.name);
	    var childrenTypeName = getNodeChildrenTypeName(node.name);
	    return "export interface ".concat(nodeInterfaceName, " extends CstNode {\n  name: \"").concat(node.name, "\";\n  children: ").concat(childrenTypeName, ";\n}");
	}
	function genNodeChildrenType(node) {
	    var typeName = getNodeChildrenTypeName(node.name);
	    return "export type ".concat(typeName, " = {\n  ").concat((0, map_1.default)(node.properties, function (property) { return genChildProperty(property); }).join("\n  "), "\n};");
	}
	function genChildProperty(prop) {
	    var typeName = buildTypeString(prop.type);
	    return "".concat(prop.name).concat(prop.optional ? "?" : "", ": ").concat(typeName, "[];");
	}
	function genVisitor(name, nodes) {
	    return "export interface ".concat(name, "<IN, OUT> extends ICstVisitor<IN, OUT> {\n  ").concat((0, map_1.default)(nodes, function (node) { return genVisitorFunction(node); }).join("\n  "), "\n}");
	}
	function genVisitorFunction(node) {
	    var childrenTypeName = getNodeChildrenTypeName(node.name);
	    return "".concat(node.name, "(children: ").concat(childrenTypeName, ", param?: IN): OUT;");
	}
	function buildTypeString(type) {
	    if ((0, isArray_1.default)(type)) {
	        var typeNames = (0, uniq_1.default)((0, map_1.default)(type, function (t) { return getTypeString(t); }));
	        var typeString = (0, reduce_1.default)(typeNames, function (sum, t) { return sum + " | " + t; });
	        return "(" + typeString + ")";
	    }
	    else {
	        return getTypeString(type);
	    }
	}
	function getTypeString(type) {
	    if (type.kind === "token") {
	        return "IToken";
	    }
	    return getNodeInterfaceName(type.name);
	}
	function getNodeInterfaceName(ruleName) {
	    return (0, upperFirst_1.default)(ruleName) + "CstNode";
	}
	function getNodeChildrenTypeName(ruleName) {
	    return (0, upperFirst_1.default)(ruleName) + "CstChildren";
	}
	
	return generate$1;
}

var hasRequiredApi$1;

function requireApi$1 () {
	if (hasRequiredApi$1) return api;
	hasRequiredApi$1 = 1;
	var __assign = (api && api.__assign) || function () {
	    __assign = Object.assign || function(t) {
	        for (var s, i = 1, n = arguments.length; i < n; i++) {
	            s = arguments[i];
	            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
	                t[p] = s[p];
	        }
	        return t;
	    };
	    return __assign.apply(this, arguments);
	};
	Object.defineProperty(api, "__esModule", { value: true });
	api.generateCstDts = void 0;
	var model_1 = requireModel();
	var generate_1 = requireGenerate();
	var defaultOptions = {
	    includeVisitorInterface: true,
	    visitorInterfaceName: "ICstNodeVisitor"
	};
	function generateCstDts(productions, options) {
	    var effectiveOptions = __assign(__assign({}, defaultOptions), options);
	    var model = (0, model_1.buildModel)(productions);
	    return (0, generate_1.genDts)(model, effectiveOptions);
	}
	api.generateCstDts = generateCstDts;
	
	return api;
}

var render_public = {};

var hasRequiredRender_public;

function requireRender_public () {
	if (hasRequiredRender_public) return render_public;
	hasRequiredRender_public = 1;
	Object.defineProperty(render_public, "__esModule", { value: true });
	render_public.createSyntaxDiagramsCode = void 0;
	var version_1 = requireVersion();
	function createSyntaxDiagramsCode(grammar, _a) {
	    var _b = _a === void 0 ? {} : _a, _c = _b.resourceBase, resourceBase = _c === void 0 ? "https://unpkg.com/chevrotain@".concat(version_1.VERSION, "/diagrams/") : _c, _d = _b.css, css = _d === void 0 ? "https://unpkg.com/chevrotain@".concat(version_1.VERSION, "/diagrams/diagrams.css") : _d;
	    var header = "\n<!-- This is a generated file -->\n<!DOCTYPE html>\n<meta charset=\"utf-8\">\n<style>\n  body {\n    background-color: hsl(30, 20%, 95%)\n  }\n</style>\n\n";
	    var cssHtml = "\n<link rel='stylesheet' href='".concat(css, "'>\n");
	    var scripts = "\n<script src='".concat(resourceBase, "vendor/railroad-diagrams.js'></script>\n<script src='").concat(resourceBase, "src/diagrams_builder.js'></script>\n<script src='").concat(resourceBase, "src/diagrams_behavior.js'></script>\n<script src='").concat(resourceBase, "src/main.js'></script>\n");
	    var diagramsDiv = "\n<div id=\"diagrams\" align=\"center\"></div>    \n";
	    var serializedGrammar = "\n<script>\n    window.serializedGrammar = ".concat(JSON.stringify(grammar, null, "  "), ";\n</script>\n");
	    var initLogic = "\n<script>\n    var diagramsDiv = document.getElementById(\"diagrams\");\n    main.drawDiagramsFromSerializedGrammar(serializedGrammar, diagramsDiv);\n</script>\n";
	    return (header + cssHtml + scripts + diagramsDiv + serializedGrammar + initLogic);
	}
	render_public.createSyntaxDiagramsCode = createSyntaxDiagramsCode;
	
	return render_public;
}

var hasRequiredApi;

function requireApi () {
	if (hasRequiredApi) return api$3;
	hasRequiredApi = 1;
	(function (exports) {
		/* istanbul ignore file - tricky to import some things from this module during testing */
		Object.defineProperty(exports, "__esModule", { value: true });
		exports.Parser = exports.createSyntaxDiagramsCode = exports.clearCache = exports.generateCstDts = exports.GAstVisitor = exports.serializeProduction = exports.serializeGrammar = exports.Terminal = exports.Rule = exports.RepetitionWithSeparator = exports.RepetitionMandatoryWithSeparator = exports.RepetitionMandatory = exports.Repetition = exports.Option = exports.NonTerminal = exports.Alternative = exports.Alternation = exports.defaultLexerErrorProvider = exports.NoViableAltException = exports.NotAllInputParsedException = exports.MismatchedTokenException = exports.isRecognitionException = exports.EarlyExitException = exports.defaultParserErrorProvider = exports.LLkLookaheadStrategy = exports.getLookaheadPaths = exports.tokenName = exports.tokenMatcher = exports.tokenLabel = exports.EOF = exports.createTokenInstance = exports.createToken = exports.LexerDefinitionErrorType = exports.Lexer = exports.EMPTY_ALT = exports.ParserDefinitionErrorType = exports.EmbeddedActionsParser = exports.CstParser = exports.VERSION = void 0;
		// semantic version
		var version_1 = requireVersion();
		Object.defineProperty(exports, "VERSION", { enumerable: true, get: function () { return version_1.VERSION; } });
		var parser_1 = requireParser();
		Object.defineProperty(exports, "CstParser", { enumerable: true, get: function () { return parser_1.CstParser; } });
		Object.defineProperty(exports, "EmbeddedActionsParser", { enumerable: true, get: function () { return parser_1.EmbeddedActionsParser; } });
		Object.defineProperty(exports, "ParserDefinitionErrorType", { enumerable: true, get: function () { return parser_1.ParserDefinitionErrorType; } });
		Object.defineProperty(exports, "EMPTY_ALT", { enumerable: true, get: function () { return parser_1.EMPTY_ALT; } });
		var lexer_public_1 = requireLexer_public();
		Object.defineProperty(exports, "Lexer", { enumerable: true, get: function () { return lexer_public_1.Lexer; } });
		Object.defineProperty(exports, "LexerDefinitionErrorType", { enumerable: true, get: function () { return lexer_public_1.LexerDefinitionErrorType; } });
		// Tokens utilities
		var tokens_public_1 = requireTokens_public();
		Object.defineProperty(exports, "createToken", { enumerable: true, get: function () { return tokens_public_1.createToken; } });
		Object.defineProperty(exports, "createTokenInstance", { enumerable: true, get: function () { return tokens_public_1.createTokenInstance; } });
		Object.defineProperty(exports, "EOF", { enumerable: true, get: function () { return tokens_public_1.EOF; } });
		Object.defineProperty(exports, "tokenLabel", { enumerable: true, get: function () { return tokens_public_1.tokenLabel; } });
		Object.defineProperty(exports, "tokenMatcher", { enumerable: true, get: function () { return tokens_public_1.tokenMatcher; } });
		Object.defineProperty(exports, "tokenName", { enumerable: true, get: function () { return tokens_public_1.tokenName; } });
		// Lookahead
		var lookahead_1 = requireLookahead();
		Object.defineProperty(exports, "getLookaheadPaths", { enumerable: true, get: function () { return lookahead_1.getLookaheadPaths; } });
		var llk_lookahead_1 = requireLlk_lookahead();
		Object.defineProperty(exports, "LLkLookaheadStrategy", { enumerable: true, get: function () { return llk_lookahead_1.LLkLookaheadStrategy; } });
		// Other Utilities
		var errors_public_1 = requireErrors_public();
		Object.defineProperty(exports, "defaultParserErrorProvider", { enumerable: true, get: function () { return errors_public_1.defaultParserErrorProvider; } });
		var exceptions_public_1 = requireExceptions_public();
		Object.defineProperty(exports, "EarlyExitException", { enumerable: true, get: function () { return exceptions_public_1.EarlyExitException; } });
		Object.defineProperty(exports, "isRecognitionException", { enumerable: true, get: function () { return exceptions_public_1.isRecognitionException; } });
		Object.defineProperty(exports, "MismatchedTokenException", { enumerable: true, get: function () { return exceptions_public_1.MismatchedTokenException; } });
		Object.defineProperty(exports, "NotAllInputParsedException", { enumerable: true, get: function () { return exceptions_public_1.NotAllInputParsedException; } });
		Object.defineProperty(exports, "NoViableAltException", { enumerable: true, get: function () { return exceptions_public_1.NoViableAltException; } });
		var lexer_errors_public_1 = requireLexer_errors_public();
		Object.defineProperty(exports, "defaultLexerErrorProvider", { enumerable: true, get: function () { return lexer_errors_public_1.defaultLexerErrorProvider; } });
		// grammar reflection API
		var gast_1 = requireApi$2();
		Object.defineProperty(exports, "Alternation", { enumerable: true, get: function () { return gast_1.Alternation; } });
		Object.defineProperty(exports, "Alternative", { enumerable: true, get: function () { return gast_1.Alternative; } });
		Object.defineProperty(exports, "NonTerminal", { enumerable: true, get: function () { return gast_1.NonTerminal; } });
		Object.defineProperty(exports, "Option", { enumerable: true, get: function () { return gast_1.Option; } });
		Object.defineProperty(exports, "Repetition", { enumerable: true, get: function () { return gast_1.Repetition; } });
		Object.defineProperty(exports, "RepetitionMandatory", { enumerable: true, get: function () { return gast_1.RepetitionMandatory; } });
		Object.defineProperty(exports, "RepetitionMandatoryWithSeparator", { enumerable: true, get: function () { return gast_1.RepetitionMandatoryWithSeparator; } });
		Object.defineProperty(exports, "RepetitionWithSeparator", { enumerable: true, get: function () { return gast_1.RepetitionWithSeparator; } });
		Object.defineProperty(exports, "Rule", { enumerable: true, get: function () { return gast_1.Rule; } });
		Object.defineProperty(exports, "Terminal", { enumerable: true, get: function () { return gast_1.Terminal; } });
		// GAST Utilities
		var gast_2 = requireApi$2();
		Object.defineProperty(exports, "serializeGrammar", { enumerable: true, get: function () { return gast_2.serializeGrammar; } });
		Object.defineProperty(exports, "serializeProduction", { enumerable: true, get: function () { return gast_2.serializeProduction; } });
		Object.defineProperty(exports, "GAstVisitor", { enumerable: true, get: function () { return gast_2.GAstVisitor; } });
		var cst_dts_gen_1 = requireApi$1();
		Object.defineProperty(exports, "generateCstDts", { enumerable: true, get: function () { return cst_dts_gen_1.generateCstDts; } });
		/* istanbul ignore next */
		function clearCache() {
		    console.warn("The clearCache function was 'soft' removed from the Chevrotain API." +
		        "\n\t It performs no action other than printing this message." +
		        "\n\t Please avoid using it as it will be completely removed in the future");
		}
		exports.clearCache = clearCache;
		var render_public_1 = requireRender_public();
		Object.defineProperty(exports, "createSyntaxDiagramsCode", { enumerable: true, get: function () { return render_public_1.createSyntaxDiagramsCode; } });
		var Parser = /** @class */ (function () {
		    function Parser() {
		        throw new Error("The Parser class has been deprecated, use CstParser or EmbeddedActionsParser instead.\t\n" +
		            "See: https://chevrotain.io/docs/changes/BREAKING_CHANGES.html#_7-0-0");
		    }
		    return Parser;
		}());
		exports.Parser = Parser;
		
	} (api$3));
	return api$3;
}

var apiExports = requireApi();
var mod = /*@__PURE__*/getDefaultExportFromCjs(apiExports);

mod.Alternation;
mod.Alternative;
mod.CstParser;
mod.EMPTY_ALT;
mod.EOF;
mod.EarlyExitException;
const EmbeddedActionsParser = mod.EmbeddedActionsParser;
mod.GAstVisitor;
mod.LLkLookaheadStrategy;
const Lexer = mod.Lexer;
mod.LexerDefinitionErrorType;
mod.MismatchedTokenException;
mod.NoViableAltException;
mod.NonTerminal;
mod.NotAllInputParsedException;
mod.Option;
mod.Parser;
mod.ParserDefinitionErrorType;
mod.Repetition;
mod.RepetitionMandatory;
mod.RepetitionMandatoryWithSeparator;
mod.RepetitionWithSeparator;
mod.Rule;
mod.Terminal;
mod.VERSION;
mod.clearCache;
mod.createSyntaxDiagramsCode;
const createToken = mod.createToken;
mod.createTokenInstance;
mod.defaultLexerErrorProvider;
mod.defaultParserErrorProvider;
mod.generateCstDts;
mod.getLookaheadPaths;
mod.isRecognitionException;
mod.serializeGrammar;
mod.serializeProduction;
mod.tokenLabel;
mod.tokenMatcher;
mod.tokenName;

var outExports = requireOut$1();

// https://chevrotain.io/docs/features/lexer_modes.html
// https://chevrotain.io/docs/features/custom_token_patterns.html => indentation
// special
const WhiteSpace$1 = createToken({ name: 'WhiteSpace', pattern: /\s+/, group: Lexer.SKIPPED });
const Identifier$1 = createToken({ name: 'Identifier', pattern: /"([^\\"]|\\\\|\\")*"|[a-zA-Z]\w*/ });
// values
const Float = createToken({ name: 'Float', pattern: /\d+\.\d+/ });
const Integer$1 = createToken({ name: 'Integer', pattern: /\d+/, longer_alt: Float });
const String$2 = createToken({ name: 'String', pattern: /'([^\\']|\\\\|\\')*'/ });
const Boolean$1 = createToken({ name: 'Boolean', pattern: /true|false/, longer_alt: Identifier$1 });
const valueTokens$1 = [Integer$1, Float, String$2, Boolean$1];
// keywords
const CreateTable = createToken({ name: 'CreateTable', pattern: /CREATE TABLE/, longer_alt: Identifier$1 });
const PrimaryKey$1 = createToken({ name: 'PrimaryKey', pattern: /PRIMARY KEY/, longer_alt: Identifier$1 });
const Constraint$1 = createToken({ name: 'Constraint', pattern: /CONSTRAINT/, longer_alt: Identifier$1 });
const Select$1 = createToken({ name: 'Select', pattern: /SELECT/, longer_alt: Identifier$1 });
const From$1 = createToken({ name: 'From', pattern: /FROM/, longer_alt: Identifier$1 });
const Where$1 = createToken({ name: 'Where', pattern: /WHERE/, longer_alt: Identifier$1 });
const And$1 = createToken({ name: 'And', pattern: /AND/ });
const Or$1 = createToken({ name: 'And', pattern: /OR/ });
const keywordTokens$1 = [CreateTable, PrimaryKey$1, Constraint$1, Select$1, From$1, Where$1, And$1, Or$1];
// chars
const Star = createToken({ name: 'Star', pattern: /\*/ });
const Dot$1 = createToken({ name: 'Dot', pattern: /\./ });
const Comma$1 = createToken({ name: 'Comma', pattern: /,/ });
const Semicolon$1 = createToken({ name: 'Semicolon', pattern: /;/ });
const Equal$1 = createToken({ name: 'Equal', pattern: /=/ });
const NotEqual = createToken({ name: 'NotEqual', pattern: /!=/ });
const GreaterThanOrEqual = createToken({ name: 'GreaterThanOrEqual', pattern: />=/ });
const GreaterThan$1 = createToken({ name: 'GreaterThan', pattern: />/ });
const LessThanOrEqual = createToken({ name: 'LessThanOrEqual', pattern: /<=/ });
const LessThan = createToken({ name: 'LessThan', pattern: /</ });
const LParen = createToken({ name: 'LParen', pattern: /\(/ });
const RParen = createToken({ name: 'RParen', pattern: /\)/ });
const LCurly = createToken({ name: 'LCurly', pattern: /\{/ });
const RCurly = createToken({ name: 'RCurly', pattern: /\}/ });
const LBraket = createToken({ name: 'LBraket', pattern: /\[/ });
const RBraket = createToken({ name: 'RBraket', pattern: /\]/ });
const charTokens$1 = [Star, Dot$1, Comma$1, Semicolon$1, Equal$1, NotEqual, GreaterThanOrEqual, GreaterThan$1, LessThanOrEqual, LessThan, LParen, RParen, LCurly, RCurly, LBraket, RBraket];
// token order is important as they are tried in order, so the Identifier must be last
const allTokens$1 = [WhiteSpace$1, ...charTokens$1, ...keywordTokens$1, ...valueTokens$1, Identifier$1];
class SqlParser extends EmbeddedActionsParser {
    constructor(tokens) {
        super(tokens);
        const $ = this;
        // common rules
        this.integerRule = $.RULE('integerRule', () => {
            const token = $.CONSUME(Integer$1);
            return { value: parseInt(token.image), parser: parserInfo(token) };
        });
        this.stringRule = $.RULE('stringRule', () => {
            const token = $.CONSUME(String$2);
            return { value: token.image.slice(1, -1).replaceAll(/\\'/g, "'"), parser: parserInfo(token) };
        });
        this.booleanRule = $.RULE('booleanRule', () => {
            const token = $.CONSUME(Boolean$1);
            return { value: token.image === 'true', parser: parserInfo(token) };
        });
        this.identifierRule = $.RULE('identifierRule', () => {
            const token = $.CONSUME(Identifier$1);
            if (token.image.startsWith('"')) {
                return { identifier: token.image.slice(1, -1).replaceAll(/\\"/g, '"'), parser: parserInfo(token) };
            }
            else {
                return { identifier: token.image, parser: parserInfo(token) };
            }
        });
        this.namespaceRule = $.RULE('namespaceRule', () => {
            const token = $.SUBRULE($.identifierRule);
            $.CONSUME(Dot$1);
            return token;
        });
        this.tableRefRule = $.RULE('tableRefRule', () => {
            const schema = $.OPTION(() => $.SUBRULE2($.namespaceRule));
            const table = $.SUBRULE($.identifierRule);
            return outExports.removeUndefined({ table, schema });
        });
        this.columnRefRule = $.RULE('columnRefRule', () => {
            let schema = $.OPTION(() => $.SUBRULE($.namespaceRule));
            let table = $.OPTION2(() => $.SUBRULE2($.namespaceRule));
            if (!table) {
                [table, schema] = [schema, table];
            }
            const column = $.SUBRULE3($.identifierRule);
            return outExports.removeUndefined({ column, table, schema });
        });
        this.conditionOpRule = $.RULE('conditionOpRule', () => {
            return $.OR([
                { ALT: () => ({ operator: '=', parser: parserInfo($.CONSUME(Equal$1)) }) },
                { ALT: () => ({ operator: '!=', parser: parserInfo($.CONSUME(NotEqual)) }) },
                { ALT: () => ({ operator: '<', parser: parserInfo($.CONSUME(LessThan)) }) },
                { ALT: () => ({ operator: '>', parser: parserInfo($.CONSUME(GreaterThan$1)) }) },
            ]);
        });
        this.conditionElemRule = $.RULE('conditionElemRule', () => {
            return $.OR([
                { ALT: () => $.SUBRULE($.integerRule) },
                { ALT: () => $.SUBRULE($.stringRule) },
                { ALT: () => $.SUBRULE($.booleanRule) },
                { ALT: () => $.SUBRULE($.columnRefRule) },
            ]);
        });
        this.conditionRule = $.RULE('conditionRule', () => {
            const left = $.SUBRULE($.conditionElemRule);
            const operation = $.SUBRULE($.conditionOpRule);
            const right = $.SUBRULE2($.conditionElemRule);
            return { left, operation, right };
        });
        // select rules
        this.selectResultColumnRule = $.RULE('selectResultColumnRule', () => {
            let schema = $.OPTION(() => $.SUBRULE($.namespaceRule));
            let table = $.OPTION2(() => $.SUBRULE2($.namespaceRule));
            if (!table) {
                [table, schema] = [schema, table];
            }
            const column = $.OR([
                { ALT: () => ({ wildcard: '*', parser: parserInfo($.CONSUME(Star)) }) },
                { ALT: () => $.SUBRULE3($.identifierRule) },
            ]);
            return outExports.removeUndefined({ column, table, schema });
        });
        this.selectResultRule = $.RULE('selectResultRule', () => {
            let columns = [];
            $.CONSUME(Select$1);
            $.AT_LEAST_ONE_SEP({
                SEP: Comma$1,
                DEF: () => {
                    columns.push($.SUBRULE($.selectResultColumnRule));
                }
            });
            return { columns };
        });
        this.selectFromRule = $.RULE('selectFromRule', () => {
            $.CONSUME(From$1);
            const table = $.SUBRULE($.tableRefRule);
            // TODO: alias, joins
            return table;
        });
        this.selectWhereRule = $.RULE('selectWhereRule', () => {
            $.CONSUME(Where$1);
            return $.SUBRULE($.conditionRule);
        });
        this.selectRule = $.RULE('selectRule', () => {
            const result = $.SUBRULE($.selectResultRule);
            const from = $.SUBRULE($.selectFromRule);
            const where = $.OPTION(() => $.SUBRULE($.selectWhereRule));
            $.CONSUME(Semicolon$1);
            return outExports.removeUndefined({ command: 'SELECT', result, from, where });
        });
        // general rules
        this.statementRule = $.RULE('statementRule', () => {
            return $.OR([
                { ALT: () => $.SUBRULE($.selectRule) },
            ]);
        });
        this.sqlScriptRule = $.RULE('sqlScriptRule', () => {
            const stmts = [];
            $.MANY_SEP({
                SEP: Semicolon$1,
                DEF: () => stmts.push($.SUBRULE($.statementRule))
            });
            return stmts;
        });
        this.performSelfAnalysis();
    }
}
const lexer$1 = new Lexer(allTokens$1);
const parser = new SqlParser(allTokens$1);
// exported only for tests, use the `parse` function instead
function parseRule$1(parse, input) {
    const lexingResult = lexer$1.tokenize(input);
    parser.input = lexingResult.tokens; // "input" is a setter which will reset the parser's state.
    const res = parse(parser);
    if (parser.errors.length > 0) {
        return outExports$1.ParserResult.failure(parser.errors.map(formatError));
    }
    return outExports$1.ParserResult.success(res);
}
function parse(input) {
    return parseRule$1(p => p.sqlScriptRule(), input);
}
function formatError(err) {
    return { message: err.message, kind: err.name, level: outExports$1.ParserErrorLevel.enum.error, ...parserInfo(err.token) };
}
function parserInfo(token) {
    return {
        // token: token.tokenType?.name || 'missing',
        offset: { start: token.startOffset, end: token.endOffset || 0 },
        position: {
            start: { line: token.startLine || 0, column: token.startColumn || 0 },
            end: { line: token.endLine || 0, column: token.endColumn || 0 }
        }
    };
}

function format(ast) {
    return ast.map(formatStatement);
}
function formatStatement(stm) {
    if (stm.command === 'SELECT') {
        return formatSelect(stm);
    }
    else {
        throw new Error(`Unsupported command: ${stm.command}`);
    }
}
function formatSelect(select) {
    return {
        command: 'SELECT',
        language: 'DML',
        operation: 'read',
        result: { columns: select.result.columns.map(formatSelectResultColumn) },
        from: { table: { entity: select.from.table.identifier } },
        joins: []
    };
}
function formatSelectResultColumn(col) {
    const alias = col.alias?.identifier;
    if ('identifier' in col.column) {
        return { name: col.column.identifier, alias, content: { kind: 'column', column: [col.column.identifier], table: col.table?.identifier, schema: col.schema?.identifier } };
    }
    else {
        return { name: col.column.wildcard, alias, content: { kind: 'wildcard', table: col.table?.identifier, schema: col.schema?.identifier } };
    }
}

function generate(sql) {
    return '';
}

function importDatabase(script) {
    return { extra: { source: `SQL parser <${packageJson.version}>` } };
}

const LineComment = createToken({ name: 'LineComment', pattern: /--.*/, group: 'comments' });
const BlockComment = createToken({ name: 'BlockComment', pattern: /\/\*[\s\S]*?\*\//, line_breaks: true, group: 'comments' });
const WhiteSpace = createToken({ name: 'WhiteSpace', pattern: /\s+/, group: Lexer.SKIPPED });
const Identifier = createToken({ name: 'Identifier', pattern: /\b[a-zA-Z_]\w*\b|"([^\\"]|\\\\|\\")+"/ });
const String$1 = createToken({ name: 'String', pattern: /E?'([^']|'')*'/i });
const StringDollar = createToken({ name: 'StringDollar', pattern: /\$(\w*)\$[\s\S]*?\$\1\$/i });
const Decimal = createToken({ name: 'Decimal', pattern: /\d+\.\d+/ });
const Integer = createToken({ name: 'Integer', pattern: /0|[1-9]\d*/, longer_alt: Decimal });
const valueTokens = [Integer, Decimal, String$1, StringDollar, Identifier, LineComment, BlockComment];
const Add = createToken({ name: 'Add', pattern: /\bADD\b/i, longer_alt: Identifier });
const After = createToken({ name: 'After', pattern: /\bAFTER\b/i, longer_alt: Identifier });
const All = createToken({ name: 'All', pattern: /\bALL\b/i, longer_alt: Identifier });
const Alter = createToken({ name: 'Alter', pattern: /\bALTER\b/i, longer_alt: Identifier });
const And = createToken({ name: 'And', pattern: /\bAND\b/i, longer_alt: Identifier });
const Array$1 = createToken({ name: 'Array', pattern: /\bARRAY\b/i, longer_alt: Identifier });
const As = createToken({ name: 'As', pattern: /\bAS\b/i, longer_alt: Identifier });
const Asc = createToken({ name: 'Asc', pattern: /\bASC\b/i, longer_alt: Identifier });
const Authorization = createToken({ name: 'Authorization', pattern: /\bAUTHORIZATION\b/i, longer_alt: Identifier });
const Before = createToken({ name: 'Before', pattern: /\bBEFORE\b/i, longer_alt: Identifier });
const Begin = createToken({ name: 'Begin', pattern: /\bBEGIN\b/i, longer_alt: Identifier });
const By = createToken({ name: 'By', pattern: /\bBY\b/i, longer_alt: Identifier });
const Cache = createToken({ name: 'Cache', pattern: /\bCACHE\b/i, longer_alt: Identifier });
const Called = createToken({ name: 'Called', pattern: /\bCALLED\b/i, longer_alt: Identifier });
const Cascade = createToken({ name: 'Cascade', pattern: /\bCASCADE\b/i, longer_alt: Identifier });
const Chain = createToken({ name: 'Chain', pattern: /\bCHAIN\b/i, longer_alt: Identifier });
const Check = createToken({ name: 'Check', pattern: /\bCHECK\b/i, longer_alt: Identifier });
const Collate = createToken({ name: 'Collate', pattern: /\bCOLLATE\b/i, longer_alt: Identifier });
const Column = createToken({ name: 'Column', pattern: /\bCOLUMN\b/i, longer_alt: Identifier });
const Comment = createToken({ name: 'Comment', pattern: /\bCOMMENT\b/i, longer_alt: Identifier });
const Commit = createToken({ name: 'Commit', pattern: /\bCOMMIT\b/i, longer_alt: Identifier });
const Concurrently = createToken({ name: 'Concurrently', pattern: /\bCONCURRENTLY\b/i, longer_alt: Identifier });
const Conflict = createToken({ name: 'Conflict', pattern: /\bCONFLICT\b/i, longer_alt: Identifier });
const Constraint = createToken({ name: 'Constraint', pattern: /\bCONSTRAINT\b/i, longer_alt: Identifier });
const Create = createToken({ name: 'Create', pattern: /\bCREATE\b/i, longer_alt: Identifier });
const Cross = createToken({ name: 'Cross', pattern: /\bCROSS\b/i, longer_alt: Identifier });
const CurrentRole = createToken({ name: 'CurrentRole', pattern: /\bCURRENT_ROLE\b/i, longer_alt: Identifier });
const CurrentUser = createToken({ name: 'CurrentUser', pattern: /\bCURRENT_USER\b/i, longer_alt: Identifier });
const Cycle = createToken({ name: 'Cycle', pattern: /\bCYCLE\b/i, longer_alt: Identifier });
const Data = createToken({ name: 'Data', pattern: /\bDATA\b/i, longer_alt: Identifier });
const Database = createToken({ name: 'Database', pattern: /\bDATABASE\b/i, longer_alt: Identifier });
const Default = createToken({ name: 'Default', pattern: /\bDEFAULT\b/i, longer_alt: Identifier });
const Deferrable = createToken({ name: 'Deferrable', pattern: /\bDEFERRABLE\b/i, longer_alt: Identifier });
const Deferred = createToken({ name: 'Deferred', pattern: /\bDEFERRED\b/i, longer_alt: Identifier });
const Delete = createToken({ name: 'Delete', pattern: /\bDELETE\b/i, longer_alt: Identifier });
const Desc = createToken({ name: 'Desc', pattern: /\bDESC\b/i, longer_alt: Identifier });
const Distinct = createToken({ name: 'Distinct', pattern: /\bDISTINCT\b/i, longer_alt: Identifier });
const Do = createToken({ name: 'Do', pattern: /\bDO\b/i, longer_alt: Identifier });
const Domain = createToken({ name: 'Domain', pattern: /\bDOMAIN\b/i, longer_alt: Identifier });
const Drop = createToken({ name: 'Drop', pattern: /\bDROP\b/i, longer_alt: Identifier });
const Each = createToken({ name: 'Each', pattern: /\bEACH\b/i, longer_alt: Identifier });
const Enum = createToken({ name: 'Enum', pattern: /\bENUM\b/i, longer_alt: Identifier });
const Except = createToken({ name: 'Except', pattern: /\bEXCEPT\b/i, longer_alt: Identifier });
const Execute = createToken({ name: 'Execute', pattern: /\bEXECUTE\b/i, longer_alt: Identifier });
const Exists = createToken({ name: 'Exists', pattern: /\bEXISTS\b/i, longer_alt: Identifier });
const Extension = createToken({ name: 'Extension', pattern: /\bEXTENSION\b/i, longer_alt: Identifier });
const False = createToken({ name: 'False', pattern: /\bFALSE\b/i, longer_alt: Identifier });
const Fetch = createToken({ name: 'Fetch', pattern: /\bFETCH\b/i, longer_alt: Identifier });
const Filter = createToken({ name: 'Filter', pattern: /\bFILTER\b/i, longer_alt: Identifier });
const First = createToken({ name: 'First', pattern: /\bFIRST\b/i, longer_alt: Identifier });
const For = createToken({ name: 'For', pattern: /\bFOR\b/i, longer_alt: Identifier });
const ForeignKey = createToken({ name: 'ForeignKey', pattern: /\bFOREIGN\s+KEY\b/i });
const From = createToken({ name: 'From', pattern: /\bFROM\b/i, longer_alt: Identifier });
const Full = createToken({ name: 'Full', pattern: /\bFULL\b/i, longer_alt: Identifier });
const Function$1 = createToken({ name: 'Function', pattern: /\bFUNCTION\b/i, longer_alt: Identifier });
const Global = createToken({ name: 'Global', pattern: /\bGLOBAL\b/i, longer_alt: Identifier });
const GroupBy = createToken({ name: 'GroupBy', pattern: /\bGROUP\s+BY\b/i });
const Having = createToken({ name: 'Having', pattern: /\bHAVING\b/i, longer_alt: Identifier });
const If = createToken({ name: 'If', pattern: /\bIF\b/i, longer_alt: Identifier });
const Immediate = createToken({ name: 'Immediate', pattern: /\bIMMEDIATE\b/i, longer_alt: Identifier });
const Immutable = createToken({ name: 'Immutable', pattern: /\bIMMUTABLE\b/i, longer_alt: Identifier });
const In = createToken({ name: 'In', pattern: /\bIN\b/i, longer_alt: Identifier });
const Include = createToken({ name: 'Include', pattern: /\bINCLUDE\b/i, longer_alt: Identifier });
const Increment = createToken({ name: 'Increment', pattern: /\bINCREMENT\b/i, longer_alt: Identifier });
const Index = createToken({ name: 'Index', pattern: /\bINDEX\b/i, longer_alt: Identifier });
const Initially = createToken({ name: 'Initially', pattern: /\bINITIALLY\b/i, longer_alt: Identifier });
const Inner = createToken({ name: 'Inner', pattern: /\bINNER\b/i, longer_alt: Identifier });
const InOut = createToken({ name: 'InOut', pattern: /\bINOUT\b/i, longer_alt: Identifier });
const Input = createToken({ name: 'Input', pattern: /\bINPUT\b/i, longer_alt: Identifier });
const Insert = createToken({ name: 'Insert', pattern: /\bINSERT\b/i, longer_alt: Identifier });
const InsteadOf = createToken({ name: 'InsteadOf', pattern: /\bINSTEAD\s+OF\b/i });
const Intersect = createToken({ name: 'Intersect', pattern: /\bINTERSECT\b/i, longer_alt: Identifier });
const Interval = createToken({ name: 'Interval', pattern: /\bINTERVAL\b/i, longer_alt: Identifier });
const Into = createToken({ name: 'Into', pattern: /\bINTO\b/i, longer_alt: Identifier });
const Is = createToken({ name: 'Is', pattern: /\bIS\b/i, longer_alt: Identifier });
const IsNull = createToken({ name: 'IsNull', pattern: /\bISNULL\b/i, longer_alt: Identifier });
const IsolationLevel = createToken({ name: 'IsolationLevel', pattern: /\bISOLATION\s+LEVEL\b/i });
const Join = createToken({ name: 'Join', pattern: /\bJOIN\b/i, longer_alt: Identifier });
const Language = createToken({ name: 'Language', pattern: /\bLANGUAGE\b/i, longer_alt: Identifier });
const Last = createToken({ name: 'Last', pattern: /\bLAST\b/i, longer_alt: Identifier });
const Left = createToken({ name: 'Left', pattern: /\bLEFT\b/i, longer_alt: Identifier });
const Like = createToken({ name: 'Like', pattern: /\bLIKE\b/i, longer_alt: Identifier });
const Limit = createToken({ name: 'Limit', pattern: /\bLIMIT\b/i, longer_alt: Identifier });
const Local = createToken({ name: 'Local', pattern: /\bLOCAL\b/i, longer_alt: Identifier });
const MaterializedView = createToken({ name: 'MaterializedView', pattern: /\bMATERIALIZED\s+VIEW\b/i });
const Maxvalue = createToken({ name: 'Maxvalue', pattern: /\bMAXVALUE\b/i, longer_alt: Identifier });
const Minvalue = createToken({ name: 'Minvalue', pattern: /\bMINVALUE\b/i, longer_alt: Identifier });
const Natural = createToken({ name: 'Natural', pattern: /\bNATURAL\b/i, longer_alt: Identifier });
const New = createToken({ name: 'New', pattern: /\bNEW\b/i, longer_alt: Identifier });
const Next = createToken({ name: 'Next', pattern: /\bNEXT\b/i, longer_alt: Identifier });
const No = createToken({ name: 'No', pattern: /\bNO\b/i, longer_alt: Identifier });
const NoAction = createToken({ name: 'NoAction', pattern: /\bNO\s+ACTION\b/i });
const None = createToken({ name: 'None', pattern: /\bNONE\b/i, longer_alt: Identifier });
const Not = createToken({ name: 'Not', pattern: /\bNOT\b/i, longer_alt: Identifier });
const Nothing = createToken({ name: 'Nothing', pattern: /\bNOTHING\b/i, longer_alt: Identifier });
const NotNull = createToken({ name: 'NotNull', pattern: /\bNOTNULL\b/i, longer_alt: Identifier });
const Null = createToken({ name: 'Null', pattern: /\bNULL\b/i, longer_alt: Identifier });
const Nulls = createToken({ name: 'Nulls', pattern: /\bNULLS\b/i, longer_alt: Identifier });
const Of = createToken({ name: 'Of', pattern: /\bOF\b/i, longer_alt: Identifier });
const Offset = createToken({ name: 'Offset', pattern: /\bOFFSET\b/i, longer_alt: Identifier });
const Old = createToken({ name: 'Old', pattern: /\bOLD\b/i, longer_alt: Identifier });
const On = createToken({ name: 'On', pattern: /\bON\b/i, longer_alt: Identifier });
const Only = createToken({ name: 'Only', pattern: /\bONLY\b/i, longer_alt: Identifier });
const Or = createToken({ name: 'Or', pattern: /\bOR\b/i, longer_alt: Identifier });
const OrderBy = createToken({ name: 'OrderBy', pattern: /\bORDER\s+BY\b/i });
const Out = createToken({ name: 'Out', pattern: /\bOUT\b/i, longer_alt: Identifier });
const Outer = createToken({ name: 'Outer', pattern: /\bOUTER\b/i, longer_alt: Identifier });
const Over = createToken({ name: 'Over', pattern: /\bOVER\b/i, longer_alt: Identifier });
const OwnedBy = createToken({ name: 'OwnedBy', pattern: /\bOWNED\s+BY\b/i });
const OwnerTo = createToken({ name: 'OwnerTo', pattern: /\bOWNER\s+TO\b/i });
const PartitionBy = createToken({ name: 'PartitionBy', pattern: /\bPARTITION\s+BY\b/i });
const PrimaryKey = createToken({ name: 'PrimaryKey', pattern: /\bPRIMARY\s+KEY\b/i });
const Procedure = createToken({ name: 'Procedure', pattern: /\bPROCEDURE\b/i, longer_alt: Identifier });
const ReadCommitted = createToken({ name: 'ReadCommitted', pattern: /\bREAD\s+COMMITTED\b/i });
const ReadOnly = createToken({ name: 'ReadOnly', pattern: /\bREAD\s+ONLY\b/i });
const ReadUncommitted = createToken({ name: 'ReadUncommitted', pattern: /\bREAD\s+UNCOMMITTED\b/i });
const ReadWrite = createToken({ name: 'ReadWrite', pattern: /\bREAD\s+WRITE\b/i });
const Recursive = createToken({ name: 'Recursive', pattern: /\bRECURSIVE\b/i, longer_alt: Identifier });
const References = createToken({ name: 'References', pattern: /\bREFERENCES\b/i, longer_alt: Identifier });
const Referencing = createToken({ name: 'Referencing', pattern: /\bREFERENCING\b/i, longer_alt: Identifier });
const RenameTo = createToken({ name: 'RenameTo', pattern: /\bRENAME\s+TO\b/i });
const RepeatableRead = createToken({ name: 'RepeatableRead', pattern: /\bREPEATABLE\s+READ\b/i });
const Replace = createToken({ name: 'Replace', pattern: /\bREPLACE\b/i, longer_alt: Identifier });
const Restrict = createToken({ name: 'Restrict', pattern: /\bRESTRICT\b/i, longer_alt: Identifier });
const Return = createToken({ name: 'Return', pattern: /\bRETURN\b/i, longer_alt: Identifier });
const Returning = createToken({ name: 'Returning', pattern: /\bRETURNING\b/i, longer_alt: Identifier });
const Returns = createToken({ name: 'Returns', pattern: /\bRETURNS\b/i, longer_alt: Identifier });
const Right = createToken({ name: 'Right', pattern: /\bRIGHT\b/i, longer_alt: Identifier });
const Row = createToken({ name: 'Row', pattern: /\bROW\b/i, longer_alt: Identifier });
const Rows = createToken({ name: 'Rows', pattern: /\bROWS\b/i, longer_alt: Identifier });
const Schema = createToken({ name: 'Schema', pattern: /\bSCHEMA\b/i, longer_alt: Identifier });
const Select = createToken({ name: 'Select', pattern: /\bSELECT\b/i, longer_alt: Identifier });
const Sequence = createToken({ name: 'Sequence', pattern: /\bSEQUENCE\b/i, longer_alt: Identifier });
const Serializable = createToken({ name: 'Serializable', pattern: /\bSERIALIZABLE\b/i, longer_alt: Identifier });
const Session = createToken({ name: 'Session', pattern: /\bSESSION\b/i, longer_alt: Identifier });
const SessionUser = createToken({ name: 'SessionUser', pattern: /\bSESSION_USER\b/i, longer_alt: Identifier });
const Set$1 = createToken({ name: 'Set', pattern: /\bSET\b/i, longer_alt: Identifier });
const SetOf = createToken({ name: 'SetOf', pattern: /\bSETOF\b/i, longer_alt: Identifier });
const Show = createToken({ name: 'Show', pattern: /\bSHOW\b/i, longer_alt: Identifier });
const Stable = createToken({ name: 'Stable', pattern: /\bSTABLE\b/i, longer_alt: Identifier });
const Start = createToken({ name: 'Start', pattern: /\bSTART\b/i, longer_alt: Identifier });
const Statement = createToken({ name: 'Statement', pattern: /\bSTATEMENT\b/i, longer_alt: Identifier });
const Strict = createToken({ name: 'Strict', pattern: /\bSTRICT\b/i, longer_alt: Identifier });
const Table = createToken({ name: 'Table', pattern: /\bTABLE\b/i, longer_alt: Identifier });
const Temp = createToken({ name: 'Temp', pattern: /\bTEMP\b/i, longer_alt: Identifier });
const Temporary = createToken({ name: 'Temporary', pattern: /\bTEMPORARY\b/i, longer_alt: Identifier });
const Ties = createToken({ name: 'Ties', pattern: /\bTIES\b/i, longer_alt: Identifier });
const To = createToken({ name: 'To', pattern: /\bTO\b/i, longer_alt: Identifier });
const Transaction = createToken({ name: 'Transaction', pattern: /\bTRANSACTION\b/i, longer_alt: Identifier });
const Trigger = createToken({ name: 'Trigger', pattern: /\bTRIGGER\b/i, longer_alt: Identifier });
const True = createToken({ name: 'True', pattern: /\bTRUE\b/i, longer_alt: Identifier });
const Truncate = createToken({ name: 'Truncate', pattern: /\bTRUNCATE\b/i, longer_alt: Identifier });
const Type = createToken({ name: 'Type', pattern: /\bTYPE\b/i, longer_alt: Identifier });
const Union = createToken({ name: 'Union', pattern: /\bUNION\b/i, longer_alt: Identifier });
const Unique = createToken({ name: 'Unique', pattern: /\bUNIQUE\b/i, longer_alt: Identifier });
const Unlogged = createToken({ name: 'Unlogged', pattern: /\bUNLOGGED\b/i, longer_alt: Identifier });
const Update = createToken({ name: 'Update', pattern: /\bUPDATE\b/i, longer_alt: Identifier });
const Using = createToken({ name: 'Using', pattern: /\bUSING\b/i, longer_alt: Identifier });
const Valid = createToken({ name: 'Valid', pattern: /\bVALID\b/i, longer_alt: Identifier });
const Values = createToken({ name: 'Values', pattern: /\bVALUES\b/i, longer_alt: Identifier });
const Variadic = createToken({ name: 'Variadic', pattern: /\bVARIADIC\b/i, longer_alt: Identifier });
const Version = createToken({ name: 'Version', pattern: /\bVERSION\b/i, longer_alt: Identifier });
const View = createToken({ name: 'View', pattern: /\bVIEW\b/i, longer_alt: Identifier });
const Volatile = createToken({ name: 'Volatile', pattern: /\bVOLATILE\b/i, longer_alt: Identifier });
const When = createToken({ name: 'When', pattern: /\bWHEN\b/i, longer_alt: Identifier });
const Where = createToken({ name: 'Where', pattern: /\bWHERE\b/i, longer_alt: Identifier });
const Window = createToken({ name: 'Window', pattern: /\bWINDOW\b/i, longer_alt: Identifier });
const With = createToken({ name: 'With', pattern: /\bWITH\b/i, longer_alt: Identifier });
const Work = createToken({ name: 'Work', pattern: /\bWORK\b/i, longer_alt: Identifier });
const keywordTokens = [
    Add, After, All, Alter, And, Array$1, As, Asc, Authorization, Before, Begin, By, Cache, Called, Cascade, Chain, Check,
    Collate, Column, Comment, Commit, Concurrently, Conflict, Constraint, Create, Cross, CurrentRole, CurrentUser, Cycle,
    Data, Database, Default, Deferrable, Deferred, Delete, Desc, Distinct, Do, Domain, Drop, Each, Enum, Except, Execute,
    Exists, Extension, False, Fetch, Filter, First, For, ForeignKey, From, Full, Function$1, Global, GroupBy, Having, If,
    Immediate, Immutable, In, Include, Increment, Index, Initially, Inner, InOut, Input, Insert, InsteadOf, Intersect,
    Interval, Into, Is, IsNull, IsolationLevel, Join, Language, Last, Left, Like, Limit, Local, MaterializedView, Maxvalue,
    Minvalue, Natural, New, Next, No, NoAction, None, Not, Nothing, NotNull, Null, Nulls, Of, Offset, Old, On, Only, Or,
    OrderBy, Out, Outer, Over, OwnedBy, OwnerTo, PartitionBy, PrimaryKey, Procedure, ReadCommitted, ReadOnly, ReadUncommitted,
    ReadWrite, Recursive, References, Referencing, RenameTo, RepeatableRead, Replace, Restrict, Return, Returning, Returns,
    Right, Row, Rows, Schema, Select, Sequence, Serializable, Session, SessionUser, Set$1, SetOf, Show, Stable, Start,
    Statement, Strict, Table, Temp, Temporary, Ties, To, Transaction, Trigger, True, Truncate, Type, Union, Unique,
    Unlogged, Update, Using, Valid, Values, Variadic, Version, View, Volatile, When, Where, Window, With, Work
];
const Amp = createToken({ name: 'Amp', pattern: /&/ });
const Asterisk = createToken({ name: 'Asterisk', pattern: /\*/ });
const BracketLeft = createToken({ name: 'BracketLeft', pattern: /\[/ });
const BracketRight = createToken({ name: 'BracketRight', pattern: /]/ });
const Caret = createToken({ name: 'Caret', pattern: /\^/ });
const Colon = createToken({ name: 'Colon', pattern: /:/ });
const Comma = createToken({ name: 'Comma', pattern: /,/ });
const CurlyLeft = createToken({ name: 'CurlyLeft', pattern: /\{/ });
const CurlyRight = createToken({ name: 'CurlyRight', pattern: /}/ });
const Dash = createToken({ name: 'Dash', pattern: /-/, longer_alt: LineComment });
const Dollar = createToken({ name: 'Dollar', pattern: /\$/, longer_alt: StringDollar });
const Dot = createToken({ name: 'Dot', pattern: /\./ });
const Equal = createToken({ name: 'Equal', pattern: /=/ });
const Exclamation = createToken({ name: 'Exclamation', pattern: /!/ });
const GreaterThan = createToken({ name: 'GreaterThan', pattern: />/ });
const Hash = createToken({ name: 'Hash', pattern: /#/ });
const LowerThan = createToken({ name: 'LowerThan', pattern: /</ });
const ParenLeft = createToken({ name: 'ParenLeft', pattern: /\(/ });
const ParenRight = createToken({ name: 'ParenRight', pattern: /\)/ });
const Percent = createToken({ name: 'Percent', pattern: /%/ });
const Pipe = createToken({ name: 'Pipe', pattern: /\|/ });
const Plus = createToken({ name: 'Plus', pattern: /\+/ });
const QuestionMark = createToken({ name: 'QuestionMark', pattern: /\?/ });
const Semicolon = createToken({ name: 'Semicolon', pattern: /;/ });
const Slash = createToken({ name: 'Slash', pattern: /\//, longer_alt: BlockComment });
const Tilde = createToken({ name: 'Tilde', pattern: /~/ });
const charTokens = [
    Amp, Asterisk, BracketLeft, BracketRight, Caret, Colon, Comma, CurlyLeft, CurlyRight, Dash, Dollar, Dot, Equal, Exclamation,
    GreaterThan, Hash, LowerThan, ParenLeft, ParenRight, Percent, Pipe, Plus, QuestionMark, Semicolon, Slash, Tilde
];
const allTokens = [WhiteSpace, ...keywordTokens, ...charTokens, ...valueTokens];
const defaultPos = -1; // used when error position is undefined
class PostgresParser extends EmbeddedActionsParser {
    constructor(tokens, recovery) {
        super(tokens, { recoveryEnabled: recovery });
        const $ = this;
        // statements
        this.statementsRule = $.RULE('statementsRule', () => {
            const statements = [];
            $.MANY(() => {
                const stmt = $.SUBRULE($.statementRule);
                stmt && statements.push(stmt); // `stmt` can be undefined on invalid input
            });
            return { statements };
        });
        this.statementRule = $.RULE('statementRule', () => $.OR([
            { ALT: () => {
                    const alter = $.CONSUME(Alter);
                    return $.OR2([
                        { ALT: () => $.SUBRULE($.alterFunctionStatementRule, { ARGS: [alter] }) },
                        { ALT: () => $.SUBRULE($.alterMaterializedViewStatementRule, { ARGS: [alter] }) },
                        { ALT: () => $.SUBRULE($.alterSchemaStatementRule, { ARGS: [alter] }) },
                        { ALT: () => $.SUBRULE($.alterSequenceStatementRule, { ARGS: [alter] }) },
                        { ALT: () => $.SUBRULE($.alterTableStatementRule, { ARGS: [alter] }) },
                        { ALT: () => $.SUBRULE($.alterTypeStatementRule, { ARGS: [alter] }) },
                        { ALT: () => $.SUBRULE($.alterViewStatementRule, { ARGS: [alter] }) },
                    ]);
                } },
            { ALT: () => $.SUBRULE($.beginStatementRule) },
            { ALT: () => $.SUBRULE($.commentOnStatementRule) },
            { ALT: () => $.SUBRULE($.commitStatementRule) },
            { ALT: () => {
                    const create = $.CONSUME(Create);
                    const replace = $.OPTION(() => ({ token: tokenInfo2($.CONSUME(Or), $.CONSUME(Replace)) }));
                    return $.OR3([
                        { ALT: () => $.SUBRULE($.createExtensionStatementRule, { ARGS: [create] }) },
                        { ALT: () => $.SUBRULE($.createFunctionStatementRule, { ARGS: [create, replace] }) },
                        { ALT: () => $.SUBRULE($.createIndexStatementRule, { ARGS: [create] }) },
                        { ALT: () => $.SUBRULE($.createMaterializedViewStatementRule, { ARGS: [create] }) },
                        { ALT: () => $.SUBRULE($.createSchemaStatementRule, { ARGS: [create] }) },
                        { ALT: () => $.SUBRULE($.createSequenceStatementRule, { ARGS: [create] }) },
                        { ALT: () => $.SUBRULE($.createTableStatementRule, { ARGS: [create] }) },
                        { ALT: () => $.SUBRULE($.createTriggerStatementRule, { ARGS: [create, replace] }) },
                        { ALT: () => $.SUBRULE($.createTypeStatementRule, { ARGS: [create] }) },
                        { ALT: () => $.SUBRULE($.createViewStatementRule, { ARGS: [create, replace] }) },
                    ]);
                } },
            { ALT: () => $.SUBRULE($.deleteStatementRule) },
            { ALT: () => $.SUBRULE($.dropStatementRule) },
            { ALT: () => $.SUBRULE($.insertIntoStatementRule) },
            { ALT: () => $.SUBRULE($.selectStatementRule) },
            { ALT: () => $.SUBRULE($.setStatementRule) },
            { ALT: () => $.SUBRULE($.showStatementRule) },
            { ALT: () => $.SUBRULE($.updateStatementRule) },
        ]));
        this.alterFunctionStatementRule = $.RULE('alterFunctionStatementRule', (alter) => {
            // https://www.postgresql.org/docs/current/sql-alterfunction.html
            const token = tokenInfo2(alter, $.CONSUME(Function$1));
            const object = $.SUBRULE($.objectNameRule);
            const args = $.OPTION(() => $.SUBRULE(functionArgumentsRule)) || [];
            const actions = [];
            $.MANY_SEP({ SEP: Comma, DEF: () => actions.push($.OR([
                    { ALT: () => $.SUBRULE(alterRenameRule) },
                    { ALT: () => $.SUBRULE(alterSetOwnerRule) },
                    { ALT: () => $.SUBRULE(alterSetSchemaRule) },
                ])) });
            const end = $.CONSUME(Semicolon);
            return outExports.removeUndefined({ kind: 'AlterFunction', meta: tokenInfo2(alter, end), token, ...object, args, actions });
        });
        this.alterMaterializedViewStatementRule = $.RULE('alterMaterializedViewStatementRule', (alter) => {
            // https://www.postgresql.org/docs/current/sql-altermaterializedview.html
            const token = tokenInfo2(alter, $.CONSUME(MaterializedView));
            const ifExists = $.SUBRULE(ifExistsRule);
            const object = $.SUBRULE($.objectNameRule);
            const actions = [];
            $.MANY_SEP({ SEP: Comma, DEF: () => actions.push($.OR([
                    { ALT: () => $.SUBRULE(alterRenameRule) },
                    { ALT: () => $.SUBRULE(alterSetOwnerRule) },
                    { ALT: () => $.SUBRULE(alterSetSchemaRule) },
                ])) });
            const end = $.CONSUME(Semicolon);
            return outExports.removeUndefined({ kind: 'AlterMaterializedView', meta: tokenInfo2(alter, end), token, ifExists, ...object, actions });
        });
        this.alterSchemaStatementRule = $.RULE('alterSchemaStatementRule', (alter) => {
            // https://www.postgresql.org/docs/current/sql-alterschema.html
            const token = tokenInfo2(alter, $.CONSUME(Schema));
            const schema = $.SUBRULE($.identifierRule);
            const actions = [];
            $.MANY_SEP({ SEP: Comma, DEF: () => actions.push($.OR([
                    { ALT: () => $.SUBRULE(alterRenameRule) },
                    { ALT: () => $.SUBRULE(alterSetOwnerRule) },
                ])) });
            const end = $.CONSUME(Semicolon);
            return outExports.removeUndefined({ kind: 'AlterSchema', meta: tokenInfo2(alter, end), token, schema, actions });
        });
        this.alterSequenceStatementRule = $.RULE('alterSequenceStatementRule', (alter) => {
            // https://www.postgresql.org/docs/current/sql-altersequence.html
            const token = tokenInfo2(alter, $.CONSUME(Sequence));
            const ifExists = $.SUBRULE(ifExistsRule);
            const object = $.SUBRULE($.objectNameRule);
            const as = $.OPTION3(() => $.SUBRULE(sequenceTypeRule));
            const start = $.OPTION4(() => $.SUBRULE(sequenceStartRule));
            const increment = $.OPTION5(() => $.SUBRULE(sequenceIncrementRule));
            const minValue = $.OPTION6(() => $.SUBRULE(sequenceMinValueRule));
            const maxValue = $.OPTION7(() => $.SUBRULE(sequenceMaxValueRule));
            const cache = $.OPTION8(() => $.SUBRULE(sequenceCacheRule));
            // TODO: CYCLE
            const ownedBy = $.OPTION9(() => $.SUBRULE(sequenceOwnedByRule));
            // TODO: RESTART
            const end = $.CONSUME(Semicolon);
            return outExports.removeUndefined({ kind: 'AlterSequence', meta: tokenInfo2(alter, end), token, ifExists, ...object, as, start, increment, minValue, maxValue, cache, ownedBy });
        });
        this.alterTableStatementRule = $.RULE('alterTableStatementRule', (alter) => {
            // https://www.postgresql.org/docs/current/sql-altertable.html
            const token = tokenInfo2(alter, $.CONSUME(Table));
            const ifExists = $.SUBRULE(ifExistsRule);
            const only = $.OPTION(() => ({ token: tokenInfo($.CONSUME(Only)) }));
            const object = $.SUBRULE($.objectNameRule);
            const actions = [];
            $.MANY_SEP({ SEP: Comma, DEF: () => actions.push($.OR([
                    { ALT: () => outExports.removeUndefined({ kind: 'AddColumn', token: tokenInfo2($.CONSUME(Add), $.OPTION2(() => $.CONSUME(Column))), ifNotExists: $.SUBRULE(ifNotExistsRule), column: $.SUBRULE($.tableColumnRule) }) },
                    { ALT: () => outExports.removeUndefined({ kind: 'DropColumn', token: tokenInfo2($.CONSUME(Drop), $.OPTION3(() => $.CONSUME2(Column))), ifExists: $.SUBRULE2(ifExistsRule), column: $.SUBRULE($.identifierRule) }) },
                    { ALT: () => outExports.removeUndefined({ kind: 'AlterColumn', token: tokenInfo2($.CONSUME2(Alter), $.OPTION4(() => $.CONSUME3(Column))), column: $.SUBRULE2($.identifierRule), action: $.OR2([
                                { ALT: () => outExports.removeUndefined({ kind: 'Default', action: $.SUBRULE(constraintActionRule), token: tokenInfo($.CONSUME(Default)), expression: $.OPTION5(() => $.SUBRULE($.expressionRule)) }) },
                                { ALT: () => ({ kind: 'NotNull', action: $.SUBRULE2(constraintActionRule), token: tokenInfo2($.CONSUME(Not), $.CONSUME(Null)) }) },
                            ]) }) },
                    { ALT: () => outExports.removeUndefined({ kind: 'AddConstraint', token: tokenInfo($.CONSUME2(Add)), constraint: $.SUBRULE($.tableConstraintRule), notValid: $.OPTION6(() => ({ token: tokenInfo2($.CONSUME2(Not), $.CONSUME(Valid)) })) }) },
                    { ALT: () => outExports.removeUndefined({ kind: 'DropConstraint', token: tokenInfo2($.CONSUME2(Drop), $.CONSUME(Constraint)), ifExists: $.SUBRULE3(ifExistsRule), constraint: $.SUBRULE3($.identifierRule) }) },
                    { ALT: () => $.SUBRULE(alterSetOwnerRule) },
                ])) });
            const end = $.CONSUME(Semicolon);
            return outExports.removeUndefined({ kind: 'AlterTable', meta: tokenInfo2(alter, end), token, ifExists, only, ...object, actions });
        });
        const constraintActionRule = $.RULE('constraintActionRule', () => $.OR([
            { ALT: () => ({ kind: 'Set', token: tokenInfo($.CONSUME(Set$1)) }) },
            { ALT: () => ({ kind: 'Drop', token: tokenInfo($.CONSUME(Drop)) }) },
        ]));
        this.alterTypeStatementRule = $.RULE('alterTypeStatementRule', (alter) => {
            // https://www.postgresql.org/docs/current/sql-altertype.html
            const token = tokenInfo2(alter, $.CONSUME(Type));
            const object = $.SUBRULE($.objectNameRule);
            const actions = [];
            $.MANY_SEP({ SEP: Comma, DEF: () => actions.push($.OR([
                    { ALT: () => $.SUBRULE(alterRenameRule) },
                    { ALT: () => $.SUBRULE(alterSetOwnerRule) },
                    { ALT: () => $.SUBRULE(alterSetSchemaRule) },
                ])) });
            const end = $.CONSUME(Semicolon);
            return outExports.removeUndefined({ kind: 'AlterType', meta: tokenInfo2(alter, end), token, ...object, actions });
        });
        this.alterViewStatementRule = $.RULE('alterViewStatementRule', (alter) => {
            // https://www.postgresql.org/docs/current/sql-alterview.html
            const token = tokenInfo2(alter, $.CONSUME(View));
            const ifExists = $.SUBRULE(ifExistsRule);
            const object = $.SUBRULE($.objectNameRule);
            const actions = [];
            $.MANY_SEP({ SEP: Comma, DEF: () => actions.push($.OR([
                    { ALT: () => $.SUBRULE(alterRenameRule) },
                    { ALT: () => $.SUBRULE(alterSetOwnerRule) },
                    { ALT: () => $.SUBRULE(alterSetSchemaRule) },
                ])) });
            const end = $.CONSUME(Semicolon);
            return outExports.removeUndefined({ kind: 'AlterView', meta: tokenInfo2(alter, end), token, ifExists, ...object, actions });
        });
        const alterRenameRule = $.RULE('alterRenameRule', () => {
            return { kind: 'Rename', token: tokenInfo($.CONSUME(RenameTo)), name: $.SUBRULE($.identifierRule) };
        });
        const alterSetOwnerRule = $.RULE('alterSetOwnerRule', () => {
            return { kind: 'SetOwner', token: tokenInfo($.CONSUME(OwnerTo)), owner: $.SUBRULE(ownerRule) };
        });
        const alterSetSchemaRule = $.RULE('alterSetSchemaRule', () => {
            return { kind: 'SetSchema', token: tokenInfo2($.CONSUME(Set$1), $.CONSUME(Schema)), schema: $.SUBRULE($.identifierRule) };
        });
        this.beginStatementRule = $.RULE('beginStatementRule', () => {
            // https://www.postgresql.org/docs/current/sql-begin.html
            const start = $.CONSUME(Begin);
            const token = tokenInfo(start);
            const object = $.OPTION(() => $.OR([
                { ALT: () => ({ kind: 'Work', token: tokenInfo($.CONSUME(Work)) }) },
                { ALT: () => ({ kind: 'Transaction', token: tokenInfo($.CONSUME(Transaction)) }) },
            ]));
            const modes = [];
            $.MANY_SEP({ SEP: Comma, DEF: () => modes.push($.OR2([
                    { ALT: () => ({ kind: 'IsolationLevel', token: tokenInfo($.CONSUME(IsolationLevel)), level: $.OR3([
                                { ALT: () => ({ kind: 'Serializable', token: tokenInfo($.CONSUME(Serializable)) }) },
                                { ALT: () => ({ kind: 'RepeatableRead', token: tokenInfo($.CONSUME(RepeatableRead)) }) },
                                { ALT: () => ({ kind: 'ReadCommitted', token: tokenInfo($.CONSUME(ReadCommitted)) }) },
                                { ALT: () => ({ kind: 'ReadUncommitted', token: tokenInfo($.CONSUME(ReadUncommitted)) }) },
                            ]) }) },
                    { ALT: () => ({ kind: 'ReadOnly', token: tokenInfo($.CONSUME(ReadOnly)) }) },
                    { ALT: () => ({ kind: 'ReadWrite', token: tokenInfo($.CONSUME(ReadWrite)) }) },
                    { ALT: () => ({ not: $.OPTION2(() => ({ token: tokenInfo($.CONSUME(Not)) })), kind: 'Deferrable', token: tokenInfo($.CONSUME(Deferrable)) }) }
                ])) });
            const end = $.CONSUME(Semicolon);
            return outExports.removeEmpty({ kind: 'Begin', meta: tokenInfo2(start, end), token, object, modes });
        });
        this.commentOnStatementRule = $.RULE('commentOnStatementRule', () => {
            // https://www.postgresql.org/docs/current/sql-comment.html
            const start = $.CONSUME(Comment);
            const token = tokenInfo2(start, $.CONSUME(On));
            const { object, schema, parent, entity } = $.OR([
                { ALT: () => ({ object: { kind: 'Column', token: tokenInfo($.CONSUME(Column)) }, ...$.SUBRULE(commentColumnRule) }) },
                { ALT: () => ({ object: { kind: 'Constraint', token: tokenInfo($.CONSUME(Constraint)) }, ...$.SUBRULE(commentConstraintRule) }) },
                { ALT: () => ({ object: { kind: 'Database', token: tokenInfo($.CONSUME(Database)) }, ...$.SUBRULE(commentObjectDefaultRule) }) },
                { ALT: () => ({ object: { kind: 'Extension', token: tokenInfo($.CONSUME(Extension)) }, ...$.SUBRULE2(commentObjectDefaultRule) }) },
                { ALT: () => ({ object: { kind: 'Index', token: tokenInfo($.CONSUME(Index)) }, ...$.SUBRULE3(commentObjectDefaultRule) }) },
                { ALT: () => ({ object: { kind: 'MaterializedView', token: tokenInfo($.CONSUME(MaterializedView)) }, ...$.SUBRULE4(commentObjectDefaultRule) }) },
                { ALT: () => ({ object: { kind: 'Schema', token: tokenInfo($.CONSUME(Schema)) }, ...$.SUBRULE5(commentObjectDefaultRule) }) },
                { ALT: () => ({ object: { kind: 'Table', token: tokenInfo($.CONSUME(Table)) }, ...$.SUBRULE6(commentObjectDefaultRule) }) },
                { ALT: () => ({ object: { kind: 'Type', token: tokenInfo($.CONSUME(Type)) }, ...$.SUBRULE7(commentObjectDefaultRule) }) },
                { ALT: () => ({ object: { kind: 'View', token: tokenInfo($.CONSUME(View)) }, ...$.SUBRULE8(commentObjectDefaultRule) }) },
            ]);
            $.CONSUME(Is);
            const comment = $.OR2([
                { ALT: () => $.SUBRULE($.stringRule) },
                { ALT: () => $.SUBRULE($.nullRule) },
            ]);
            const end = $.CONSUME(Semicolon);
            return outExports.removeUndefined({ kind: 'CommentOn', meta: tokenInfo2(start, end), token, object, schema, parent, entity, comment });
        });
        const commentObjectDefaultRule = $.RULE('commentObjectDefaultRule', () => {
            const object = $.SUBRULE($.objectNameRule);
            return outExports.removeUndefined({ schema: object.schema, entity: object.name });
        });
        const commentColumnRule = $.RULE('commentColumnRule', () => {
            const first = $.SUBRULE($.identifierRule);
            $.CONSUME(Dot);
            const second = $.SUBRULE2($.identifierRule);
            const third = $.OPTION2(() => {
                $.CONSUME2(Dot);
                return $.SUBRULE3($.identifierRule);
            });
            return third ? { schema: first, parent: second, entity: third } : { parent: first, entity: second };
        });
        const commentConstraintRule = $.RULE('commentConstraintRule', () => {
            const entity = $.SUBRULE($.identifierRule);
            $.CONSUME(On);
            $.OPTION(() => $.CONSUME(Domain));
            const object = $.SUBRULE($.objectNameRule);
            return { entity, schema: object.schema, parent: object.name };
        });
        this.commitStatementRule = $.RULE('commitStatementRule', () => {
            // https://www.postgresql.org/docs/current/sql-commit.html
            const start = $.CONSUME(Commit);
            const token = tokenInfo(start);
            const object = $.OPTION(() => $.OR([
                { ALT: () => ({ kind: 'Work', token: tokenInfo($.CONSUME(Work)) }) },
                { ALT: () => ({ kind: 'Transaction', token: tokenInfo($.CONSUME(Transaction)) }) },
            ]));
            const chain = $.OPTION2(() => {
                const and = $.CONSUME(And);
                const no = $.OPTION3(() => ({ token: tokenInfo($.CONSUME(No)) }));
                const chain = $.CONSUME(Chain);
                return { token: tokenInfo2(and, chain), no };
            });
            const end = $.CONSUME(Semicolon);
            return outExports.removeUndefined({ kind: 'Commit', meta: tokenInfo2(start, end), token, object, chain });
        });
        this.createExtensionStatementRule = $.RULE('createExtensionStatementRule', (create) => {
            // https://www.postgresql.org/docs/current/sql-createextension.html
            const token = tokenInfo2(create, $.CONSUME(Extension));
            const ifNotExists = $.SUBRULE(ifNotExistsRule);
            const name = $.SUBRULE($.identifierRule);
            const with_ = $.OPTION(() => ({ token: tokenInfo($.CONSUME(With)) }));
            const schema = $.OPTION2(() => ({ token: tokenInfo($.CONSUME(Schema)), name: $.SUBRULE2($.identifierRule) }));
            const version = $.OPTION3(() => ({ token: tokenInfo($.CONSUME(Version)), number: $.OR([{ ALT: () => $.SUBRULE($.stringRule) }, { ALT: () => $.SUBRULE3($.identifierRule) }]) }));
            const cascade = $.OPTION4(() => ({ token: tokenInfo($.CONSUME(Cascade)) }));
            const end = $.CONSUME(Semicolon);
            return outExports.removeUndefined({ kind: 'CreateExtension', meta: tokenInfo2(create, end), token, ifNotExists, name, with: with_, schema, version, cascade });
        });
        this.createFunctionStatementRule = $.RULE('createFunctionStatementRule', (create, replace) => {
            // https://www.postgresql.org/docs/current/sql-createfunction.html
            const token = tokenInfo2(create, $.CONSUME(Function$1));
            const object = $.SUBRULE($.objectNameRule);
            const args = $.SUBRULE(functionArgumentsRule);
            const statement = {};
            $.MANY({ DEF: () => $.OR([
                    { ALT: () => statement.returns = $.SUBRULE(functionReturnsRule) },
                    { ALT: () => statement.language = { token: tokenInfo($.CONSUME(Language)), name: $.SUBRULE3($.identifierRule) } },
                    { ALT: () => statement.behavior = $.OR2([
                            { ALT: () => ({ kind: 'Immutable', token: tokenInfo($.CONSUME(Immutable)) }) },
                            { ALT: () => ({ kind: 'Stable', token: tokenInfo($.CONSUME(Stable)) }) },
                            { ALT: () => ({ kind: 'Volatile', token: tokenInfo($.CONSUME(Volatile)) }) },
                        ]) },
                    { ALT: () => statement.definition = { token: tokenInfo($.CONSUME(As)), value: $.SUBRULE(this.stringRule) } },
                    { ALT: () => statement.nullBehavior = $.OR3([
                            { ALT: () => ({ kind: 'Called', token: tokenInfoN([$.CONSUME(Called), $.CONSUME(On), $.CONSUME(Null), $.CONSUME(Input)]) }) },
                            { ALT: () => ({ kind: 'ReturnsNull', token: tokenInfoN([$.CONSUME(Returns), $.CONSUME2(Null), $.CONSUME2(On), $.CONSUME3(Null), $.CONSUME2(Input)]) }) },
                            { ALT: () => ({ kind: 'Strict', token: tokenInfo($.CONSUME(Strict)) }) },
                        ]) },
                    { ALT: () => statement.return = { token: tokenInfo($.CONSUME(Return)), expression: $.SUBRULE(this.expressionRule) } },
                ]) });
            const end = $.CONSUME(Semicolon);
            return outExports.removeUndefined({ kind: 'CreateFunction', meta: tokenInfo2(create, end), token, replace, ...object, args, ...statement });
        });
        const functionArgumentsRule = $.RULE('functionArgumentsRule', () => {
            const args = [];
            $.CONSUME(ParenLeft);
            $.MANY_SEP({ SEP: Comma, DEF: () => {
                    const mode = $.OPTION(() => $.OR([
                        { ALT: () => ({ kind: 'In', token: tokenInfo($.CONSUME(In)) }) },
                        { ALT: () => ({ kind: 'Out', token: tokenInfo($.CONSUME(Out)) }) },
                        { ALT: () => ({ kind: 'InOut', token: tokenInfo($.CONSUME(InOut)) }) },
                        { ALT: () => ({ kind: 'Variadic', token: tokenInfo($.CONSUME(Variadic)) }) },
                    ]));
                    const name = $.OPTION2(() => $.SUBRULE($.identifierRule));
                    const type = $.SUBRULE($.columnTypeRule);
                    args.push(outExports.removeUndefined({ mode, name, type }));
                } });
            $.CONSUME(ParenRight);
            return args;
        });
        const functionReturnsRule = $.RULE('functionReturnsRule', () => {
            const ret = $.CONSUME(Returns);
            return $.OR([
                { ALT: () => outExports.removeUndefined({ kind: 'Type', token: tokenInfo(ret), setOf: $.OPTION(() => ({ token: tokenInfo($.CONSUME(SetOf)) })), type: $.SUBRULE($.columnTypeRule) }) },
                { ALT: () => {
                        const token = tokenInfo2(ret, $.CONSUME(Table));
                        $.CONSUME(ParenLeft);
                        const columns = [];
                        $.MANY_SEP({ SEP: Comma, DEF: () => columns.push({ name: $.SUBRULE($.identifierRule), type: $.SUBRULE2($.columnTypeRule) }) });
                        $.CONSUME(ParenRight);
                        return { kind: 'Table', token, columns };
                    } },
            ]);
        });
        this.createIndexStatementRule = $.RULE('createIndexStatementRule', (create) => {
            // https://www.postgresql.org/docs/current/sql-createindex.html
            const unique = $.OPTION(() => ({ token: tokenInfo($.CONSUME(Unique)) }));
            const token = tokenInfo2(create, $.CONSUME(Index));
            const concurrently = $.OPTION2(() => ({ token: tokenInfo($.CONSUME(Concurrently)) }));
            const name = $.OPTION3(() => ({ ifNotExists: $.SUBRULE(ifNotExistsRule), index: $.SUBRULE($.identifierRule) }));
            $.CONSUME(On);
            const only = $.OPTION4(() => ({ token: tokenInfo($.CONSUME(Only)) }));
            const object = $.SUBRULE($.objectNameRule);
            const using = $.OPTION5(() => ({ token: tokenInfo($.CONSUME(Using)), method: $.SUBRULE2($.identifierRule) }));
            $.CONSUME(ParenLeft);
            const columns = [];
            $.AT_LEAST_ONE_SEP({ SEP: Comma, DEF: () => columns.push($.SUBRULE(indexColumnRule)) });
            $.CONSUME(ParenRight);
            const include = $.OPTION6(() => {
                const token = tokenInfo($.CONSUME(Include));
                $.CONSUME2(ParenLeft);
                const columns = [];
                $.AT_LEAST_ONE_SEP2({ SEP: Comma, DEF: () => columns.push($.SUBRULE3($.identifierRule)) });
                $.CONSUME2(ParenRight);
                return { token, columns };
            });
            // TODO: NULLS [ NOT ] DISTINCT
            // TODO: WITH (parameters)
            // TODO: TABLESPACE name
            const where = $.OPTION7(() => $.SUBRULE($.whereClauseRule));
            const end = $.CONSUME(Semicolon);
            return outExports.removeUndefined({ kind: 'CreateIndex', meta: tokenInfo2(create, end), token, unique, concurrently, ...name, only, schema: object.schema, table: object.name, using, columns, include, where });
        });
        this.createMaterializedViewStatementRule = $.RULE('createMaterializedViewStatementRule', (create) => {
            // https://www.postgresql.org/docs/current/sql-creatematerializedview.html
            const token = tokenInfo2(create, $.CONSUME(MaterializedView));
            const ifNotExists = $.OPTION(() => $.SUBRULE(ifNotExistsRule));
            const object = $.SUBRULE($.objectNameRule);
            const columns = [];
            $.OPTION2(() => {
                $.CONSUME(ParenLeft);
                $.AT_LEAST_ONE_SEP({ SEP: Comma, DEF: () => columns.push($.SUBRULE($.identifierRule)) });
                $.CONSUME(ParenRight);
            });
            // TODO: USING
            // TODO: WITH
            // TODO: TABLESPACE
            $.CONSUME(As);
            const query = $.SUBRULE(selectStatementInnerRule);
            const withData = $.OPTION3(() => {
                const with_ = $.CONSUME(With);
                const no = $.OPTION4(() => ({ token: tokenInfo($.CONSUME(No)) }));
                const data = $.CONSUME(Data);
                return { token: tokenInfo2(with_, data), no };
            });
            const end = $.CONSUME(Semicolon);
            return outExports.removeEmpty({ kind: 'CreateMaterializedView', meta: tokenInfo2(create, end), token, ifNotExists, ...object, columns, query, withData });
        });
        this.createSchemaStatementRule = $.RULE('createSchemaStatementRule', (create) => {
            // https://www.postgresql.org/docs/current/sql-createschema.html
            const token = tokenInfo2(create, $.CONSUME(Schema));
            const ifNotExists = $.SUBRULE(ifNotExistsRule);
            const schema = $.OPTION2(() => $.SUBRULE($.identifierRule));
            const authorization = $.OPTION3(() => ({ token: tokenInfo($.CONSUME(Authorization)), owner: $.SUBRULE(ownerRule) }));
            const end = $.CONSUME(Semicolon);
            return outExports.removeEmpty({ kind: 'CreateSchema', meta: tokenInfo2(create, end), token, ifNotExists, schema, authorization });
        });
        const ownerRule = $.RULE('ownerRule', () => $.OR([
            { ALT: () => ({ kind: 'User', name: $.SUBRULE2($.identifierRule) }) },
            { ALT: () => ({ kind: 'CurrentRole', token: tokenInfo($.CONSUME(CurrentRole)) }) },
            { ALT: () => ({ kind: 'CurrentUser', token: tokenInfo($.CONSUME(CurrentUser)) }) },
            { ALT: () => ({ kind: 'SessionUser', token: tokenInfo($.CONSUME(SessionUser)) }) },
        ]));
        this.createSequenceStatementRule = $.RULE('createSequenceStatementRule', (create) => {
            // https://www.postgresql.org/docs/current/sql-createsequence.html
            const mode = $.OPTION(() => $.OR([
                { ALT: () => ({ kind: 'Unlogged', token: tokenInfo($.CONSUME(Unlogged)) }) },
                { ALT: () => ({ kind: 'Temporary', token: $.OR2([
                            { ALT: () => tokenInfo($.CONSUME(Temp)) },
                            { ALT: () => tokenInfo($.CONSUME(Temporary)) }
                        ]) }) }
            ]));
            const token = tokenInfo2(create, $.CONSUME(Sequence));
            const ifNotExists = $.OPTION2(() => $.SUBRULE(ifNotExistsRule));
            const object = $.SUBRULE($.objectNameRule);
            const statement = {};
            $.MANY({ DEF: () => $.OR3([
                    { ALT: () => statement.as = $.SUBRULE(sequenceTypeRule) },
                    { ALT: () => statement.start = $.SUBRULE(sequenceStartRule) },
                    { ALT: () => statement.increment = $.SUBRULE(sequenceIncrementRule) },
                    { ALT: () => statement.minValue = $.SUBRULE(sequenceMinValueRule) },
                    { ALT: () => statement.maxValue = $.SUBRULE(sequenceMaxValueRule) },
                    { ALT: () => statement.cache = $.SUBRULE(sequenceCacheRule) },
                    // TODO: CYCLE
                    { ALT: () => statement.ownedBy = $.SUBRULE(sequenceOwnedByRule) },
                ]) });
            const end = $.CONSUME(Semicolon);
            return outExports.removeUndefined({ kind: 'CreateSequence', meta: tokenInfo2(create, end), token, mode, ifNotExists, ...object, ...statement });
        });
        const sequenceTypeRule = $.RULE('sequenceTypeRule', () => ({ token: tokenInfo($.CONSUME(As)), type: $.SUBRULE($.identifierRule) }));
        const sequenceStartRule = $.RULE('sequenceStartRule', () => ({ token: tokenInfo2($.CONSUME(Start), $.OPTION(() => $.CONSUME(With))), value: $.SUBRULE($.integerRule) }));
        const sequenceIncrementRule = $.RULE('sequenceIncrementRule', () => ({ token: tokenInfo2($.CONSUME(Increment), $.OPTION(() => $.CONSUME(By))), value: $.SUBRULE($.integerRule) }));
        const sequenceMinValueRule = $.RULE('sequenceMinValueRule', () => $.OR([
            { ALT: () => ({ token: tokenInfo2($.CONSUME(No), $.CONSUME(Minvalue)) }) },
            { ALT: () => ({ token: tokenInfo($.CONSUME2(Minvalue)), value: $.SUBRULE($.integerRule) }) },
        ]));
        const sequenceMaxValueRule = $.RULE('sequenceMaxValueRule', () => $.OR([
            { ALT: () => ({ token: tokenInfo2($.CONSUME(No), $.CONSUME(Maxvalue)) }) },
            { ALT: () => ({ token: tokenInfo($.CONSUME2(Maxvalue)), value: $.SUBRULE($.integerRule) }) },
        ]));
        const sequenceCacheRule = $.RULE('sequenceCacheRule', () => ({ token: tokenInfo($.CONSUME(Cache)), value: $.SUBRULE5($.integerRule) }));
        const sequenceOwnedByRule = $.RULE('sequenceOwnedByRule', () => {
            return { token: tokenInfo($.CONSUME(OwnedBy)), owner: $.OR([
                    { ALT: () => ({ kind: 'None', token: $.CONSUME(None) }) },
                    { ALT: () => {
                            const first = $.SUBRULE($.identifierRule);
                            $.CONSUME(Dot);
                            const second = $.SUBRULE2($.identifierRule);
                            const third = $.OPTION(() => {
                                $.CONSUME2(Dot);
                                return $.SUBRULE3($.identifierRule);
                            });
                            return third ? { kind: 'Column', schema: first, table: second, column: third } : { kind: 'Column', table: first, column: second };
                        } },
                ]) };
        });
        this.createTableStatementRule = $.RULE('createTableStatementRule', (create) => {
            // https://www.postgresql.org/docs/current/sql-createtable.html
            const mode = $.OPTION(() => $.OR([
                { ALT: () => ({ kind: 'Unlogged', token: tokenInfo($.CONSUME(Unlogged)) }) },
                { ALT: () => {
                        const scope = $.OPTION2(() => $.OR2([
                            { ALT: () => ({ kind: 'Global', token: tokenInfo($.CONSUME(Global)) }) },
                            { ALT: () => ({ kind: 'Local', token: tokenInfo($.CONSUME(Local)) }) },
                        ]));
                        const temporary = $.OR3([{ ALT: () => tokenInfo($.CONSUME(Temp)) }, { ALT: () => tokenInfo($.CONSUME(Temporary)) }]);
                        return outExports.removeUndefined({ kind: 'Temporary', ...temporary, scope });
                    } }
            ]));
            const token = tokenInfo2(create, $.CONSUME(Table));
            const ifNotExists = $.OPTION3(() => $.SUBRULE(ifNotExistsRule));
            const object = $.SUBRULE($.objectNameRule);
            $.CONSUME(ParenLeft);
            const columns = [];
            const constraints = [];
            $.MANY_SEP({ SEP: Comma, DEF: () => $.OR4([
                    { ALT: () => columns.push($.SUBRULE($.tableColumnRule)) },
                    { ALT: () => constraints.push($.SUBRULE($.tableConstraintRule)) },
                ]) });
            $.CONSUME(ParenRight);
            const end = $.CONSUME(Semicolon);
            return outExports.removeEmpty({ kind: 'CreateTable', meta: tokenInfo2(create, end), token, mode, ifNotExists, ...object, columns: columns.filter(outExports.isNotUndefined), constraints: constraints.filter(outExports.isNotUndefined) });
        });
        this.createTriggerStatementRule = $.RULE('createTriggerStatementRule', (create, replace) => {
            // https://www.postgresql.org/docs/current/sql-createtrigger.html
            const constraint = $.OPTION(() => ({ token: tokenInfo($.CONSUME(Constraint)) }));
            const token = tokenInfo2(create, $.CONSUME(Trigger));
            const name = $.SUBRULE($.identifierRule);
            const timing = $.OR([
                { ALT: () => ({ kind: 'Before', token: tokenInfo($.CONSUME(Before)) }) },
                { ALT: () => ({ kind: 'After', token: tokenInfo($.CONSUME(After)) }) },
                { ALT: () => ({ kind: 'InsteadOf', token: tokenInfo($.CONSUME(InsteadOf)) }) },
            ]);
            const events = [];
            $.AT_LEAST_ONE_SEP({ SEP: Or, DEF: () => events.push($.SUBRULE(triggerEventRule)) });
            $.CONSUME2(On);
            const object = $.SUBRULE($.objectNameRule);
            const from = $.OPTION2(() => {
                const token = tokenInfo($.CONSUME(From));
                const object = $.SUBRULE2($.objectNameRule);
                return outExports.removeUndefined({ token, schema: object.schema, table: object.name });
            });
            const deferrable = $.OPTION3(() => $.SUBRULE(triggerDeferrableRule));
            const referencing = $.OPTION4(() => $.SUBRULE(triggerReferencingRule));
            const target = $.OPTION5(() => {
                const token = $.CONSUME(For);
                const each = $.OPTION6(() => $.CONSUME(Each));
                return $.OR2([
                    { ALT: () => ({ kind: 'Row', token: tokenInfo3(token, each, $.CONSUME(Row)) }) },
                    { ALT: () => ({ kind: 'Statement', token: tokenInfo3(token, each, $.CONSUME(Statement)) }) },
                ]);
            });
            const when = $.OPTION7(() => {
                const token = tokenInfo($.CONSUME(When));
                $.CONSUME(ParenLeft);
                const condition = $.SUBRULE($.expressionRule);
                $.CONSUME(ParenRight);
                return { token, condition };
            });
            const execToken = tokenInfo2($.CONSUME(Execute), $.OR3([{ ALT: () => $.CONSUME(Function$1) }, { ALT: () => $.CONSUME(Procedure) }]));
            const func = $.SUBRULE3($.objectNameRule);
            $.CONSUME2(ParenLeft);
            const execArguments = [];
            const execute = outExports.removeUndefined({ token: execToken, schema: func.schema, function: func.name, arguments: execArguments });
            $.MANY_SEP({ SEP: Comma, DEF: () => execArguments.push($.SUBRULE2($.expressionRule)) });
            $.CONSUME2(ParenRight);
            const end = $.CONSUME(Semicolon);
            return outExports.removeUndefined({ kind: 'CreateTrigger', meta: tokenInfo2(create, end), token, replace, constraint, name, timing, events, schema: object.schema, table: object.name, from, deferrable, referencing, target, when, execute });
        });
        const triggerEventRule = $.RULE('triggerEventRule', () => $.OR([
            { ALT: () => ({ kind: 'Insert', token: tokenInfo($.CONSUME(Insert)) }) },
            { ALT: () => outExports.removeUndefined({ kind: 'Update', token: tokenInfo($.CONSUME(Update)), columns: $.OPTION(() => {
                        $.CONSUME(Of);
                        const columns = [];
                        $.AT_LEAST_ONE_SEP2({ SEP: Comma, DEF: () => columns.push($.SUBRULE2($.identifierRule)) });
                        return columns;
                    }) }) },
            { ALT: () => ({ kind: 'Delete', token: tokenInfo($.CONSUME(Delete)) }) },
            { ALT: () => ({ kind: 'Truncate', token: tokenInfo($.CONSUME(Truncate)) }) },
        ]));
        const triggerDeferrableRule = $.RULE('triggerDeferrableRule', () => {
            const not = $.OPTION(() => $.CONSUME(Not));
            const token = tokenInfo2(not, $.CONSUME(Deferrable));
            const initially = $.OPTION2(() => {
                const init = $.CONSUME(Initially);
                return $.OR([
                    { ALT: () => ({ kind: 'Immediate', token: tokenInfo2(init, $.CONSUME(Immediate)) }) },
                    { ALT: () => ({ kind: 'Deferred', token: tokenInfo2(init, $.CONSUME(Deferred)) }) },
                ]);
            });
            return outExports.removeUndefined({ kind: not ? 'NotDeferrable' : 'Deferrable', token, initially });
        });
        const triggerReferencingRule = $.RULE('triggerReferencingRule', () => {
            const res = { token: tokenInfo($.CONSUME(Referencing)) };
            $.AT_LEAST_ONE({ DEF: () => {
                    const kind = $.OR([
                        { ALT: () => ({ old: true, token: $.CONSUME(Old) }) },
                        { ALT: () => ({ old: false, token: $.CONSUME(New) }) },
                    ]);
                    const token = tokenInfo3(kind.token, $.CONSUME(Table), $.OPTION(() => $.CONSUME(As)));
                    const name = $.SUBRULE($.identifierRule);
                    if (kind.old) {
                        res.old = { token, name };
                    }
                    else {
                        res.new = { token, name };
                    }
                } });
            return res;
        });
        this.createTypeStatementRule = $.RULE('createTypeStatementRule', (create) => {
            // https://www.postgresql.org/docs/current/sql-createtype.html
            const token = tokenInfo2(create, $.CONSUME(Type));
            const object = $.SUBRULE($.objectNameRule);
            const content = $.OPTION(() => $.OR([
                { ALT: () => ({ struct: { token: tokenInfo($.CONSUME(As)), attrs: $.SUBRULE(createTypeStructAttrs) } }) },
                { ALT: () => ({ enum: { token: tokenInfo2($.CONSUME2(As), $.CONSUME(Enum)), values: $.SUBRULE(createTypeEnumValues) } }) },
                // TODO: RANGE
                { ALT: () => ({ base: $.SUBRULE(createTypeBase) }) }
            ]));
            const end = $.CONSUME(Semicolon);
            return outExports.removeEmpty({ kind: 'CreateType', meta: tokenInfo2(create, end), token, ...object, ...content });
        });
        this.createViewStatementRule = $.RULE('createViewStatementRule', (create, replace) => {
            // https://www.postgresql.org/docs/current/sql-createview.html
            const temporary = $.OPTION2(() => ({ token: $.OR([{ ALT: () => tokenInfo($.CONSUME(Temp)) }, { ALT: () => tokenInfo($.CONSUME(Temporary)) }]) }));
            const recursive = $.OPTION3(() => ({ token: tokenInfo($.CONSUME(Recursive)) }));
            const token = tokenInfo2(create, $.CONSUME(View));
            const object = $.SUBRULE($.objectNameRule);
            const columns = [];
            $.OPTION4(() => {
                $.CONSUME(ParenLeft);
                $.AT_LEAST_ONE_SEP({ SEP: Comma, DEF: () => columns.push($.SUBRULE($.identifierRule)) });
                $.CONSUME(ParenRight);
            });
            $.CONSUME(As);
            const query = $.SUBRULE(selectStatementInnerRule);
            const end = $.CONSUME(Semicolon);
            return outExports.removeEmpty({ kind: 'CreateView', meta: tokenInfo2(create, end), token, replace, temporary, recursive, ...object, columns, query });
        });
        this.deleteStatementRule = $.RULE('deleteStatementRule', () => {
            const start = $.CONSUME(Delete);
            const token = tokenInfo2(start, $.CONSUME(From));
            const only = $.OPTION(() => ({ token: tokenInfo($.CONSUME(Only)) }));
            const object = $.SUBRULE($.objectNameRule);
            const descendants = $.OPTION2(() => ({ token: tokenInfo($.CONSUME(Asterisk)) }));
            const alias = $.OPTION3(() => $.SUBRULE($.aliasRule));
            const using = $.OPTION4(() => ({ token: tokenInfo($.CONSUME(Using)), ...$.SUBRULE(fromClauseItemRule) }));
            const where = $.OPTION5(() => $.SUBRULE($.whereClauseRule));
            const returning = $.OPTION6(() => $.SUBRULE(returningClauseRule));
            const end = $.CONSUME(Semicolon);
            return outExports.removeUndefined({ kind: 'Delete', meta: tokenInfo2(start, end), token, only, schema: object.schema, table: object.name, descendants, alias, using, where, returning });
        });
        this.dropStatementRule = $.RULE('dropStatementRule', () => {
            const start = $.CONSUME(Drop);
            const { token, object } = $.OR([
                { ALT: () => ({ token: tokenInfo2(start, $.CONSUME(Index)), object: 'Index' }) }, // https://www.postgresql.org/docs/current/sql-dropindex.html
                { ALT: () => ({ token: tokenInfo2(start, $.CONSUME(MaterializedView)), object: 'MaterializedView' }) }, // https://www.postgresql.org/docs/current/sql-dropmaterializedview.html
                { ALT: () => ({ token: tokenInfo2(start, $.CONSUME(Sequence)), object: 'Sequence' }) }, // https://www.postgresql.org/docs/current/sql-dropsequence.html
                { ALT: () => ({ token: tokenInfo2(start, $.CONSUME(Table)), object: 'Table' }) }, // https://www.postgresql.org/docs/current/sql-droptable.html
                { ALT: () => ({ token: tokenInfo2(start, $.CONSUME(Type)), object: 'Type' }) }, // https://www.postgresql.org/docs/current/sql-droptype.html
                { ALT: () => ({ token: tokenInfo2(start, $.CONSUME(View)), object: 'View' }) }, // https://www.postgresql.org/docs/current/sql-dropview.html
            ]);
            const concurrently = $.OPTION(() => ({ token: tokenInfo($.CONSUME(Concurrently)) }));
            const ifExists = $.SUBRULE(ifExistsRule);
            const objects = [];
            $.AT_LEAST_ONE_SEP({ SEP: Comma, DEF: () => objects.push($.SUBRULE($.objectNameRule)) });
            const mode = $.OPTION2(() => $.OR2([
                { ALT: () => ({ token: tokenInfo($.CONSUME(Cascade)), kind: 'Cascade' }) },
                { ALT: () => ({ token: tokenInfo($.CONSUME(Restrict)), kind: 'Restrict' }) },
            ]));
            const end = $.CONSUME(Semicolon);
            return outExports.removeUndefined({ kind: 'Drop', meta: tokenInfo2(start, end), token, object, entities: objects.filter(outExports.isNotUndefined), concurrently, ifExists, mode });
        });
        this.insertIntoStatementRule = $.RULE('insertIntoStatementRule', () => {
            // https://www.postgresql.org/docs/current/sql-insert.html
            const start = $.CONSUME(Insert);
            const into = $.CONSUME(Into);
            const object = $.SUBRULE($.objectNameRule);
            const columns = $.OPTION(() => {
                const columns = [];
                $.CONSUME(ParenLeft);
                $.AT_LEAST_ONE_SEP({ SEP: Comma, DEF: () => columns.push($.SUBRULE($.identifierRule)) });
                $.CONSUME(ParenRight);
                return columns;
            });
            $.CONSUME(Values);
            const values = [];
            $.AT_LEAST_ONE_SEP2({ SEP: Comma, DEF: () => {
                    const row = [];
                    $.CONSUME2(ParenLeft);
                    $.AT_LEAST_ONE_SEP3({ SEP: Comma, DEF: () => row.push($.OR([
                            { ALT: () => $.SUBRULE($.expressionRule) },
                            { ALT: () => ({ kind: 'Default', token: tokenInfo($.CONSUME(Default)) }) }
                        ])) });
                    $.CONSUME2(ParenRight);
                    values.push(row);
                } });
            const onConflict = $.OPTION2(() => $.SUBRULE(onConflictClauseRule));
            const returning = $.OPTION3(() => $.SUBRULE(returningClauseRule));
            const end = $.CONSUME(Semicolon);
            return outExports.removeUndefined({ kind: 'InsertInto', meta: tokenInfo2(start, end), token: tokenInfo2(start, into), schema: object.schema, table: object.name, columns, values, onConflict, returning });
        });
        const onConflictClauseRule = $.RULE('onConflictClauseRule', () => {
            const token = tokenInfo2($.CONSUME(On), $.CONSUME(Conflict));
            const target = $.OPTION(() => $.OR([
                { ALT: () => ({ kind: 'Constraint', token: tokenInfo2($.CONSUME2(On), $.CONSUME(Constraint)), name: $.SUBRULE($.identifierRule) }) },
                { ALT: () => {
                        $.CONSUME(ParenLeft);
                        const columns = [];
                        $.AT_LEAST_ONE_SEP({ SEP: Comma, DEF: () => columns.push($.SUBRULE2($.identifierRule)) });
                        $.CONSUME(ParenRight);
                        const where = $.OPTION2(() => $.SUBRULE($.whereClauseRule));
                        return { kind: 'Columns', columns, where };
                    } }
            ]));
            const do_ = $.CONSUME(Do);
            const action = $.OR2([
                { ALT: () => ({ kind: 'Nothing', token: tokenInfo2(do_, $.CONSUME(Nothing)) }) },
                { ALT: () => {
                        const token = tokenInfo2(do_, $.CONSUME(Update));
                        $.CONSUME(Set$1);
                        const columns = $.SUBRULE(updateColumnsRule);
                        const where = $.OPTION3(() => $.SUBRULE2($.whereClauseRule));
                        return ({ kind: 'Update', token, columns, where });
                    } },
            ]);
            return { token, target, action };
        });
        const returningClauseRule = $.RULE('returningClauseRule', () => {
            const token = tokenInfo($.CONSUME(Returning));
            const columns = [];
            $.AT_LEAST_ONE_SEP4({ SEP: Comma, DEF: () => columns.push($.SUBRULE(selectClauseColumnRule)) });
            return { token, columns };
        });
        this.selectStatementRule = $.RULE('selectStatementRule', () => {
            const select = $.SUBRULE(selectStatementInnerRule);
            const end = $.CONSUME(Semicolon);
            return outExports.removeUndefined({ kind: 'Select', meta: outExports$1.mergePositions([select?.token, tokenInfo(end)]), ...select });
        });
        const selectStatementInnerRule = $.RULE('selectStatementInnerRule', () => {
            // https://www.postgresql.org/docs/current/sql-select.html
            return $.OR([
                { ALT: () => {
                        const main = $.SUBRULE(selectStatementMainRule);
                        const result = $.SUBRULE(selectStatementResultRule);
                        return outExports.removeUndefined({ ...main, ...result });
                    } },
                { ALT: () => {
                        $.CONSUME(ParenLeft);
                        const main = $.SUBRULE2(selectStatementMainRule);
                        const result = $.SUBRULE2(selectStatementResultRule);
                        $.CONSUME(ParenRight);
                        const union = $.OPTION(() => $.SUBRULE(unionClauseRule));
                        return outExports.removeUndefined({ ...main, union, ...result });
                    } },
            ]);
        });
        const selectStatementMainRule = $.RULE('selectStatementMainRule', () => {
            const select = $.SUBRULE($.selectClauseRule);
            const from = $.OPTION(() => $.SUBRULE($.fromClauseRule));
            const where = $.OPTION2(() => $.SUBRULE($.whereClauseRule));
            const groupBy = $.OPTION3(() => $.SUBRULE(groupByClauseRule));
            const having = $.OPTION4(() => $.SUBRULE(havingClauseRule));
            const window = [];
            $.MANY(() => window.push($.SUBRULE(windowClauseRule)));
            return outExports.removeEmpty({ ...select, from, where, groupBy, having, window });
        });
        const selectStatementResultRule = $.RULE('selectStatementResultRule', () => {
            const union = $.OPTION(() => $.SUBRULE3(unionClauseRule));
            const orderBy = $.OPTION2(() => $.SUBRULE(orderByClauseRule));
            const limit = $.OPTION3(() => $.SUBRULE(limitClauseRule));
            const offset = $.OPTION4(() => $.SUBRULE(offsetClauseRule));
            const fetch = $.OPTION5(() => $.SUBRULE(fetchClauseRule));
            return outExports.removeUndefined({ union, orderBy, limit, offset, fetch });
        });
        this.setStatementRule = $.RULE('setStatementRule', () => {
            // https://www.postgresql.org/docs/current/sql-set.html
            const start = $.CONSUME(Set$1);
            const scope = $.OPTION(() => $.OR([
                { ALT: () => ({ kind: 'Session', token: tokenInfo($.CONSUME(Session)) }) },
                { ALT: () => ({ kind: 'Local', token: tokenInfo($.CONSUME(Local)) }) },
            ]));
            const parameter = $.SUBRULE($.identifierRule);
            const equal = $.OPTION2(() => $.OR2([
                { ALT: () => ({ kind: '=', token: tokenInfo($.CONSUME(Equal)) }) },
                { ALT: () => ({ kind: 'To', token: tokenInfo($.CONSUME(To)) }) },
            ]));
            const value = $.OR3([
                { ALT: () => ({ kind: 'Default', token: tokenInfo($.CONSUME(Default)) }) },
                { ALT: () => {
                        const values = [];
                        $.AT_LEAST_ONE_SEP({ SEP: Comma, DEF: () => values.push($.OR4([
                                { ALT: () => $.SUBRULE2($.literalRule) },
                                { ALT: () => $.SUBRULE3($.identifierRule) },
                                { ALT: () => toIdentifier($.CONSUME(On)) }, // special case, `on` being a valid identifier here
                            ])) });
                        return values.length === 1 ? values[0] : values;
                    } },
            ]);
            const end = $.CONSUME(Semicolon);
            return outExports.removeUndefined({ kind: 'Set', meta: tokenInfo2(start, end), token: tokenInfo(start), scope, parameter, equal, value });
        });
        this.showStatementRule = $.RULE('showStatementRule', () => {
            // https://www.postgresql.org/docs/current/sql-show.html
            const start = $.CONSUME(Show);
            const name = $.SUBRULE($.identifierRule);
            const end = $.CONSUME(Semicolon);
            return outExports.removeUndefined({ kind: 'Show', meta: tokenInfo2(start, end), token: tokenInfo(start), name });
        });
        this.updateStatementRule = $.RULE('updateStatementRule', () => {
            // https://www.postgresql.org/docs/current/sql-update.html
            const start = $.CONSUME(Update);
            const only = $.OPTION(() => ({ token: tokenInfo($.CONSUME(Only)) }));
            const object = $.SUBRULE($.objectNameRule);
            const descendants = $.OPTION2(() => ({ token: tokenInfo($.CONSUME(Asterisk)) }));
            const alias = $.OPTION3(() => $.SUBRULE($.aliasRule));
            $.CONSUME(Set$1);
            const columns = $.SUBRULE(updateColumnsRule);
            const where = $.OPTION4(() => $.SUBRULE($.whereClauseRule));
            const returning = $.OPTION5(() => $.SUBRULE(returningClauseRule));
            const end = $.CONSUME(Semicolon);
            return outExports.removeUndefined({ kind: 'Update', meta: tokenInfo2(start, end), token: tokenInfo(start), only, schema: object.schema, table: object.name, descendants, alias, columns, where, returning });
        });
        const updateColumnsRule = $.RULE('updateColumnsRule', () => {
            const columns = [];
            $.AT_LEAST_ONE_SEP({ SEP: Comma, DEF: () => {
                    const column = $.SUBRULE($.identifierRule);
                    $.CONSUME(Equal);
                    const value = $.OR([
                        { ALT: () => $.SUBRULE($.expressionRule) },
                        { ALT: () => ({ kind: 'Default', token: tokenInfo($.CONSUME(Default)) }) }
                    ]);
                    columns.push({ column, value });
                } });
            return columns;
        });
        // clauses
        this.selectClauseRule = $.RULE('selectClauseRule', () => {
            const token = tokenInfo($.CONSUME(Select));
            $.OPTION(() => $.CONSUME(All)); // default behavior, not specified most of the time but just in case...
            const distinct = $.OPTION2(() => outExports.removeUndefined({
                token: tokenInfo($.CONSUME(Distinct)),
                on: $.OPTION3(() => {
                    const token = tokenInfo($.CONSUME(On));
                    $.CONSUME(ParenLeft);
                    const columns = [];
                    $.AT_LEAST_ONE_SEP({ SEP: Comma, DEF: () => columns.push($.SUBRULE($.expressionRule)) });
                    $.CONSUME(ParenRight);
                    return ({ token, columns });
                })
            }));
            const columns = [];
            $.AT_LEAST_ONE_SEP2({ SEP: Comma, DEF: () => columns.push($.SUBRULE(selectClauseColumnRule)) });
            return { token, distinct, columns };
        });
        const selectClauseColumnRule = $.RULE('selectClauseColumnRule', () => {
            const expression = $.SUBRULE($.expressionRule);
            const filter = $.OPTION(() => {
                const token = tokenInfo($.CONSUME(Filter));
                $.CONSUME(ParenLeft);
                const where = $.SUBRULE($.whereClauseRule);
                $.CONSUME(ParenRight);
                return { token, where };
            });
            const over = $.OPTION2(() => {
                const token = tokenInfo($.CONSUME(Over));
                const content = $.OR([
                    { ALT: () => ({ name: $.SUBRULE($.identifierRule) }) },
                    { ALT: () => $.SUBRULE(windowClauseContentRule) }
                ]);
                return { token, ...content };
            });
            const alias = $.OPTION3(() => $.SUBRULE($.aliasRule));
            return outExports.removeUndefined({ ...expression, filter, over, alias });
        });
        this.fromClauseRule = $.RULE('fromClauseRule', () => {
            const token = tokenInfo($.CONSUME(From));
            const item = $.SUBRULE(fromClauseItemRule);
            const joins = [];
            $.MANY({ DEF: () => joins.push($.SUBRULE(fromClauseJoinRule)) });
            return outExports.removeEmpty({ token, ...item, joins });
        });
        const fromClauseItemRule = $.RULE('fromClauseItemRule', () => {
            const item = $.OR([
                { ALT: () => $.SUBRULE(fromClauseTableRule) },
                { ALT: () => $.SUBRULE(fromClauseQueryRule) },
            ]);
            const alias = $.OPTION(() => $.SUBRULE($.aliasRule));
            return outExports.removeUndefined({ ...item, alias });
        });
        const fromClauseTableRule = $.RULE('fromClauseTableRule', () => {
            const object = $.SUBRULE($.objectNameRule);
            return outExports.removeUndefined({ kind: 'Table', schema: object.schema, table: object.name });
        });
        const fromClauseQueryRule = $.RULE('fromClauseQueryRule', () => {
            $.CONSUME(ParenLeft);
            const select = $.SUBRULE(selectStatementInnerRule);
            $.CONSUME(ParenRight);
            return { kind: 'Select', select };
        });
        const fromClauseJoinRule = $.RULE('fromClauseJoinRule', () => {
            const natural = $.OPTION(() => ({ kind: 'Natural', token: tokenInfo($.CONSUME(Natural)) }));
            const { kind, token } = $.OR([
                { ALT: () => ({ kind: 'Inner', token: tokenInfo2($.OPTION2(() => $.CONSUME(Inner)), $.CONSUME(Join)) }) },
                { ALT: () => ({ kind: 'Left', token: tokenInfo3($.CONSUME(Left), $.OPTION3(() => $.CONSUME(Outer)), $.CONSUME2(Join)) }) },
                { ALT: () => ({ kind: 'Right', token: tokenInfo3($.CONSUME(Right), $.OPTION4(() => $.CONSUME2(Outer)), $.CONSUME3(Join)) }) },
                { ALT: () => ({ kind: 'Full', token: tokenInfo3($.CONSUME(Full), $.OPTION5(() => $.CONSUME3(Outer)), $.CONSUME4(Join)) }) },
                { ALT: () => ({ kind: 'Cross', token: tokenInfo2($.CONSUME(Cross), $.CONSUME5(Join)) }) },
            ]);
            const from = $.SUBRULE(fromClauseItemRule);
            const on = $.OPTION6(() => $.OR2([
                { ALT: () => ({ kind: 'On', token: tokenInfo($.CONSUME(On)), predicate: $.SUBRULE($.expressionRule) }) },
                { ALT: () => {
                        const token = tokenInfo($.CONSUME(Using));
                        const columns = [];
                        $.CONSUME(ParenLeft);
                        $.AT_LEAST_ONE_SEP({ SEP: Comma, DEF: () => columns.push($.SUBRULE($.identifierRule)) });
                        $.CONSUME(ParenRight);
                        return { kind: 'Using', token, columns: columns.filter(outExports.isNotUndefined) };
                    } },
            ])) || natural;
            const alias = $.OPTION7(() => $.SUBRULE($.aliasRule));
            return outExports.removeUndefined({ kind, token, from, on, alias });
        });
        this.whereClauseRule = $.RULE('whereClauseRule', () => {
            const token = tokenInfo($.CONSUME(Where));
            return { token, predicate: $.SUBRULE($.expressionRule) };
        });
        const groupByClauseRule = $.RULE('groupByClauseRule', () => {
            const token = tokenInfo($.CONSUME(GroupBy));
            const mode = $.OPTION(() => $.OR([
                { ALT: () => ({ kind: 'All', token: tokenInfo($.CONSUME(All)) }) },
                { ALT: () => ({ kind: 'Distinct', token: tokenInfo($.CONSUME(Distinct)) }) },
            ]));
            const expressions = [];
            $.AT_LEAST_ONE_SEP({ SEP: Comma, DEF: () => expressions.push($.SUBRULE($.expressionRule)) });
            return outExports.removeUndefined({ token, mode, expressions: expressions.filter(outExports.isNotUndefined) });
        });
        const havingClauseRule = $.RULE('havingClauseRule', () => {
            const token = tokenInfo($.CONSUME(Having));
            return { token, predicate: $.SUBRULE($.expressionRule) };
        });
        const windowClauseRule = $.RULE('windowClauseRule', () => {
            const token = tokenInfo($.CONSUME(Window));
            const name = $.SUBRULE($.identifierRule);
            $.CONSUME(As);
            const content = $.SUBRULE(windowClauseContentRule);
            return { token, name, ...content };
        });
        const windowClauseContentRule = $.RULE('windowClauseContentRule', () => {
            $.CONSUME(ParenLeft);
            const partitionBy = $.OPTION(() => {
                const token = tokenInfo($.CONSUME(PartitionBy));
                const columns = [];
                $.AT_LEAST_ONE_SEP({ SEP: Comma, DEF: () => columns.push($.SUBRULE($.expressionRule)) });
                return { token, columns };
            });
            const orderBy = $.OPTION2(() => $.SUBRULE(orderByClauseRule));
            $.CONSUME(ParenRight);
            return outExports.removeUndefined({ partitionBy, orderBy });
        });
        const unionClauseRule = $.RULE('unionClauseRule', () => {
            const { kind, token } = $.OR([
                { ALT: () => ({ kind: 'Union', token: tokenInfo($.CONSUME(Union)) }) },
                { ALT: () => ({ kind: 'Intersect', token: tokenInfo($.CONSUME(Intersect)) }) },
                { ALT: () => ({ kind: 'Except', token: tokenInfo($.CONSUME(Except)) }) },
            ]);
            const mode = $.OPTION(() => $.OR2([
                { ALT: () => ({ kind: 'All', token: tokenInfo($.CONSUME(All)) }) },
                { ALT: () => ({ kind: 'Distinct', token: tokenInfo($.CONSUME(Distinct)) }) },
            ]));
            const select = $.SUBRULE(selectStatementInnerRule);
            return outExports.removeUndefined({ kind, token, mode, select });
        });
        const orderByClauseRule = $.RULE('orderByClauseRule', () => {
            const token = tokenInfo($.CONSUME(OrderBy));
            const expressions = [];
            $.AT_LEAST_ONE_SEP({ SEP: Comma, DEF: () => {
                    const expr = $.SUBRULE($.expressionRule);
                    const order = $.OPTION2(() => $.SUBRULE(sortOrderRule));
                    const nulls = $.OPTION3(() => $.SUBRULE(sortNullsRule));
                    expr && expressions.push(outExports.removeUndefined({ ...expr, order, nulls }));
                } });
            return { token, expressions };
        });
        const limitClauseRule = $.RULE('limitClauseRule', () => {
            const token = tokenInfo($.CONSUME(Limit));
            const value = $.OR([
                { ALT: () => $.SUBRULE($.integerRule) },
                { ALT: () => $.SUBRULE($.parameterRule) },
                { ALT: () => ({ kind: 'All', token: tokenInfo($.CONSUME(All)) }) },
            ]);
            return { token, value };
        });
        const offsetClauseRule = $.RULE('offsetClauseRule', () => {
            const token = tokenInfo($.CONSUME(Offset));
            const value = $.OR([
                { ALT: () => $.SUBRULE($.integerRule) },
                { ALT: () => $.SUBRULE($.parameterRule) },
            ]);
            const rows = $.OPTION(() => $.OR2([
                { ALT: () => ({ kind: 'Row', token: tokenInfo($.CONSUME(Row)) }) },
                { ALT: () => ({ kind: 'Rows', token: tokenInfo($.CONSUME(Rows)) }) },
            ]));
            return outExports.removeUndefined({ token, value, rows });
        });
        const fetchClauseRule = $.RULE('fetchClauseRule', () => {
            const token = tokenInfo($.CONSUME(Fetch));
            const first = $.OR([
                { ALT: () => ({ kind: 'First', token: tokenInfo($.CONSUME(First)) }) },
                { ALT: () => ({ kind: 'Next', token: tokenInfo($.CONSUME(Next)) }) },
            ]);
            const value = $.OR2([
                { ALT: () => $.SUBRULE($.integerRule) },
                { ALT: () => $.SUBRULE($.parameterRule) },
            ]);
            const rows = $.OR3([
                { ALT: () => ({ kind: 'Row', token: tokenInfo($.CONSUME(Row)) }) },
                { ALT: () => ({ kind: 'Rows', token: tokenInfo($.CONSUME(Rows)) }) },
            ]);
            const mode = $.OR4([
                { ALT: () => ({ kind: 'Only', token: tokenInfo($.CONSUME(Only)) }) },
                { ALT: () => ({ kind: 'WithTies', token: tokenInfo2($.CONSUME(With), $.CONSUME(Ties)) }) },
            ]);
            return { token, first, value, rows, mode };
        });
        const indexColumnRule = $.RULE('indexColumnRule', () => {
            const expr = $.SUBRULE($.expressionRule);
            const collation = $.OPTION(() => ({ token: tokenInfo($.CONSUME(Collate)), name: $.SUBRULE3($.identifierRule) }));
            const order = $.OPTION2(() => $.SUBRULE(sortOrderRule));
            const nulls = $.OPTION3(() => $.SUBRULE(sortNullsRule));
            return outExports.removeUndefined({ ...expr, collation, order, nulls });
        });
        const sortOrderRule = $.RULE('sortOrderRule', () => {
            return $.OR([
                { ALT: () => ({ kind: 'Asc', token: tokenInfo($.CONSUME(Asc)) }) },
                { ALT: () => ({ kind: 'Desc', token: tokenInfo($.CONSUME(Desc)) }) },
            ]);
        });
        const sortNullsRule = $.RULE('sortNullsRule', () => {
            const nulls = $.CONSUME(Nulls);
            return $.OR([
                { ALT: () => ({ kind: 'First', token: tokenInfo2(nulls, $.CONSUME(First)) }) },
                { ALT: () => ({ kind: 'Last', token: tokenInfo2(nulls, $.CONSUME(Last)) }) },
            ]);
        });
        this.tableColumnRule = $.RULE('tableColumnRule', () => {
            const name = $.SUBRULE($.identifierRule);
            const type = $.SUBRULE($.columnTypeRule);
            const constraints = [];
            $.MANY(() => constraints.push($.SUBRULE(tableColumnConstraintRule)));
            return outExports.removeEmpty({ name, type, constraints: constraints.filter(outExports.isNotUndefined) });
        });
        const tableColumnConstraintRule = $.RULE('tableColumnConstraintRule', () => $.OR([
            { ALT: () => $.SUBRULE(tableColumnNullableRule) },
            { ALT: () => $.SUBRULE(tableColumnDefaultRule) },
            { ALT: () => $.SUBRULE(tableColumnPkRule) },
            { ALT: () => $.SUBRULE(tableColumnUniqueRule) },
            { ALT: () => $.SUBRULE(tableColumnCheckRule) },
            { ALT: () => $.SUBRULE(tableColumnFkRule) },
        ]));
        const tableColumnNullableRule = $.RULE('tableColumnNullableRule', () => {
            const constraint = $.OPTION(() => $.SUBRULE(constraintNameRule));
            const not = $.OPTION2(() => $.CONSUME(Not));
            const nullable = $.CONSUME(Null);
            return outExports.removeUndefined({ kind: 'Nullable', constraint, token: tokenInfo2(not, nullable), value: !not });
        });
        const tableColumnDefaultRule = $.RULE('tableColumnDefaultRule', () => {
            const constraint = $.OPTION(() => $.SUBRULE(constraintNameRule));
            const token = tokenInfo($.CONSUME(Default));
            const expression = $.SUBRULE(atomicExpressionRule);
            return outExports.removeUndefined({ kind: 'Default', constraint, token, expression });
        });
        const tableColumnPkRule = $.RULE('tableColumnPkRule', () => {
            const constraint = $.OPTION(() => $.SUBRULE(constraintNameRule));
            const token = tokenInfo($.CONSUME(PrimaryKey));
            return outExports.removeUndefined({ kind: 'PrimaryKey', constraint, token });
        });
        const tableColumnUniqueRule = $.RULE('tableColumnUniqueRule', () => {
            const constraint = $.OPTION(() => $.SUBRULE(constraintNameRule));
            const token = tokenInfo($.CONSUME(Unique));
            return outExports.removeUndefined({ kind: 'Unique', constraint, token });
        });
        const tableColumnCheckRule = $.RULE('tableColumnCheckRule', () => {
            const constraint = $.OPTION(() => $.SUBRULE(constraintNameRule));
            const token = tokenInfo($.CONSUME(Check));
            $.CONSUME(ParenLeft);
            const predicate = $.SUBRULE($.expressionRule);
            $.CONSUME(ParenRight);
            return outExports.removeUndefined({ kind: 'Check', constraint, token, predicate });
        });
        const tableColumnFkRule = $.RULE('tableColumnFkRule', () => {
            const constraint = $.OPTION(() => $.SUBRULE(constraintNameRule));
            const token = tokenInfo($.CONSUME(References));
            const object = $.SUBRULE($.objectNameRule);
            const column = $.OPTION2(() => {
                $.CONSUME(ParenLeft);
                const column = $.SUBRULE($.identifierRule);
                $.CONSUME(ParenRight);
                return column;
            });
            const onUpdate = $.OPTION3(() => ({ token: tokenInfo2($.CONSUME(On), $.CONSUME(Update)), ...$.SUBRULE(foreignKeyActionsRule) }));
            const onDelete = $.OPTION4(() => ({ token: tokenInfo2($.CONSUME2(On), $.CONSUME(Delete)), ...$.SUBRULE2(foreignKeyActionsRule) }));
            return outExports.removeUndefined({ kind: 'ForeignKey', constraint, token, schema: object.schema, table: object.name, column, onUpdate, onDelete });
        });
        this.tableConstraintRule = $.RULE('tableConstraintRule', () => $.OR([
            { ALT: () => $.SUBRULE(tablePkRule) },
            { ALT: () => $.SUBRULE(tableUniqueRule) },
            { ALT: () => $.SUBRULE(tableCheckRule) },
            { ALT: () => $.SUBRULE(tableFkRule) },
        ]));
        const tablePkRule = $.RULE('tablePkRule', () => {
            const constraint = $.OPTION(() => $.SUBRULE(constraintNameRule));
            const token = tokenInfo($.CONSUME(PrimaryKey));
            const columns = $.SUBRULE(columnNamesRule);
            return outExports.removeUndefined({ kind: 'PrimaryKey', constraint, token, columns });
        });
        const tableUniqueRule = $.RULE('tableUniqueRule', () => {
            const constraint = $.OPTION(() => $.SUBRULE(constraintNameRule));
            const token = tokenInfo($.CONSUME(Unique));
            const columns = $.SUBRULE(columnNamesRule);
            return outExports.removeUndefined({ kind: 'Unique', constraint, token, columns });
        });
        const tableCheckRule = tableColumnCheckRule; // exactly the same ^^
        const tableFkRule = $.RULE('tableFkRule', () => {
            const constraint = $.OPTION(() => $.SUBRULE(constraintNameRule));
            const token = tokenInfo($.CONSUME(ForeignKey));
            const columns = $.SUBRULE(columnNamesRule);
            const refToken = $.CONSUME(References);
            const refTable = $.SUBRULE($.objectNameRule);
            const refColumns = $.OPTION2(() => $.SUBRULE2(columnNamesRule));
            const onUpdate = $.OPTION3(() => ({ token: tokenInfo2($.CONSUME(On), $.CONSUME(Update)), ...$.SUBRULE(foreignKeyActionsRule) }));
            const onDelete = $.OPTION4(() => ({ token: tokenInfo2($.CONSUME2(On), $.CONSUME(Delete)), ...$.SUBRULE2(foreignKeyActionsRule) }));
            const ref = outExports.removeUndefined({ token: tokenInfo(refToken), schema: refTable.schema, table: refTable.name, columns: refColumns });
            return outExports.removeUndefined({ kind: 'ForeignKey', constraint, token, columns, ref, onUpdate, onDelete });
        });
        const createTypeStructAttrs = $.RULE('createTypeStructAttrs', () => {
            $.CONSUME(ParenLeft);
            const attrs = [];
            $.AT_LEAST_ONE_SEP({ SEP: Comma, DEF: () => attrs.push(outExports.removeUndefined({
                    name: $.SUBRULE($.identifierRule),
                    type: $.SUBRULE($.columnTypeRule),
                    collation: $.OPTION(() => ({ token: tokenInfo($.CONSUME(Collate)), name: $.SUBRULE2($.identifierRule) }))
                })) });
            $.CONSUME(ParenRight);
            return attrs.filter(outExports.isNotUndefined);
        });
        const createTypeEnumValues = $.RULE('createTypeEnumValues', () => {
            $.CONSUME(ParenLeft);
            const values = [];
            $.AT_LEAST_ONE_SEP({ SEP: Comma, DEF: () => values.push($.SUBRULE($.stringRule)) });
            $.CONSUME(ParenRight);
            return values;
        });
        const createTypeBase = $.RULE('createTypeBase', () => {
            $.CONSUME(ParenLeft);
            const params = [];
            $.AT_LEAST_ONE_SEP({ SEP: Comma, DEF: () => {
                    const name = $.SUBRULE($.identifierRule);
                    $.CONSUME(Equal);
                    const value = $.SUBRULE($.expressionRule);
                    params.push({ name, value });
                } });
            $.CONSUME(ParenRight);
            return params;
        });
        const constraintNameRule = $.RULE('constraintNameRule', () => ({ token: tokenInfo($.CONSUME(Constraint)), name: $.SUBRULE($.identifierRule) }));
        const foreignKeyActionsRule = $.RULE('foreignKeyActionsRule', () => {
            const action = $.OR([
                { ALT: () => ({ kind: 'NoAction', token: tokenInfo($.CONSUME(NoAction)) }) },
                { ALT: () => ({ kind: 'Restrict', token: tokenInfo($.CONSUME(Restrict)) }) },
                { ALT: () => ({ kind: 'Cascade', token: tokenInfo($.CONSUME(Cascade)) }) },
                { ALT: () => ({ kind: 'SetNull', token: tokenInfo2($.CONSUME(Set$1), $.CONSUME(Null)) }) },
                { ALT: () => ({ kind: 'SetDefault', token: tokenInfo2($.CONSUME2(Set$1), $.CONSUME(Default)) }) },
            ]);
            const columns = $.OPTION(() => $.SUBRULE(columnNamesRule));
            return outExports.removeEmpty({ action, columns });
        });
        const columnNamesRule = $.RULE('columnNamesRule', () => {
            $.CONSUME(ParenLeft);
            const columns = [];
            $.AT_LEAST_ONE_SEP({ SEP: Comma, DEF: () => columns.push($.SUBRULE($.identifierRule)) });
            $.CONSUME(ParenRight);
            return columns.filter(outExports.isNotUndefined);
        });
        const ifExistsRule = $.RULE('ifExistsRule', () => $.OPTION(() => ({ token: tokenInfo2($.CONSUME(If), $.CONSUME(Exists)) })));
        const ifNotExistsRule = $.RULE('ifNotExistsRule', () => $.OPTION(() => ({ token: tokenInfo3($.CONSUME(If), $.CONSUME(Not), $.CONSUME(Exists)) })));
        // basic parts
        this.aliasRule = $.RULE('aliasRule', () => {
            const token = $.OPTION(() => tokenInfo($.CONSUME(As)));
            const name = $.SUBRULE($.identifierRule);
            return outExports.removeUndefined({ token, name });
        });
        this.expressionRule = $.RULE('expressionRule', () => $.SUBRULE(orExpressionRule)); // Start with OR expressions (lowest precedence)
        const orExpressionRule = $.RULE('orExpressionRule', () => {
            let expr = $.SUBRULE(andExpressionRule); // AND has higher precedence than OR
            $.MANY(() => {
                const op = { kind: 'Or', token: tokenInfo($.CONSUME(Or)) };
                const right = $.SUBRULE2(andExpressionRule);
                expr = { kind: 'Operation', left: expr, op, right };
            });
            return expr;
        });
        const andExpressionRule = $.RULE('andExpressionRule', () => {
            let expr = $.SUBRULE(operatorExpressionRule); // operator has higher precedence than AND
            $.MANY(() => {
                const op = { kind: 'And', token: tokenInfo($.CONSUME(And)) };
                const right = $.SUBRULE2(operatorExpressionRule);
                expr = { kind: 'Operation', left: expr, op, right };
            });
            return expr;
        });
        const operatorExpressionRule = $.RULE('operatorExpressionRule', () => {
            let expr = $.SUBRULE(unaryExpressionRule); // unary has higher precedence than other operators
            $.MANY(() => {
                const op = $.SUBRULE(operatorRule);
                if (['In', 'NotIn'].includes(op?.kind)) {
                    const right = $.SUBRULE(listRule);
                    expr = { kind: 'Operation', left: expr, op, right };
                }
                else {
                    const right = $.SUBRULE2(unaryExpressionRule);
                    expr = { kind: 'Operation', left: expr, op, right };
                }
            });
            return expr;
        });
        const unaryExpressionRule = $.RULE('unaryExpressionRule', () => $.OR([
            { ALT: () => {
                    const opLeft = $.SUBRULE(operatorLeftRule);
                    const expr = $.SUBRULE(atomicExpressionRule); // atomic has the highest precedence
                    return { kind: 'OperationLeft', op: opLeft, right: expr };
                } },
            { ALT: () => {
                    const expr = $.SUBRULE2(atomicExpressionRule); // atomic has the highest precedence
                    const opRight = $.OPTION(() => $.SUBRULE(operatorRightRule));
                    return opRight ? { kind: 'OperationRight', left: expr, op: opRight } : expr;
                } },
        ]));
        const atomicExpressionRule = $.RULE('atomicExpressionRule', () => {
            const expr = $.OR([
                { ALT: () => $.SUBRULE(groupRule) },
                { ALT: () => $.SUBRULE($.literalRule) },
                { ALT: () => ({ kind: 'Wildcard', token: tokenInfo($.CONSUME(Asterisk)) }) },
                { ALT: () => {
                        const token = tokenInfo($.CONSUME(Array$1));
                        $.CONSUME(BracketLeft);
                        const items = [];
                        $.MANY_SEP({ SEP: Comma, DEF: () => items.push($.SUBRULE($.expressionRule)) });
                        $.CONSUME(BracketRight);
                        return { kind: 'Array', token, items };
                    } },
                { ALT: () => {
                        const first = $.SUBRULE($.identifierRule);
                        const nest = $.OPTION(() => $.OR2([
                            { ALT: () => outExports.removeUndefined({ kind: 'Function', function: first, ...($.SUBRULE(functionParamsRule) || {}) }) },
                            { ALT: () => {
                                    $.CONSUME(Dot);
                                    return $.OR3([
                                        { ALT: () => ({ kind: 'Wildcard', table: first, token: tokenInfo($.CONSUME2(Asterisk)) }) },
                                        { ALT: () => {
                                                const second = $.SUBRULE2($.identifierRule);
                                                const nest2 = $.OPTION2(() => $.OR4([
                                                    { ALT: () => outExports.removeUndefined({ kind: 'Function', schema: first, function: second, ...($.SUBRULE2(functionParamsRule) || {}) }) },
                                                    { ALT: () => {
                                                            $.CONSUME2(Dot);
                                                            return $.OR5([
                                                                { ALT: () => ({ kind: 'Wildcard', schema: first, table: second, token: tokenInfo($.CONSUME3(Asterisk)) }) },
                                                                { ALT: () => outExports.removeEmpty({ kind: 'Column', schema: first, table: second, column: $.SUBRULE3($.identifierRule), json: $.SUBRULE3(columnJsonRule) }) },
                                                            ]);
                                                        } }
                                                ]));
                                                return nest2 ? nest2 : outExports.removeEmpty({ kind: 'Column', table: first, column: second, json: $.SUBRULE2(columnJsonRule) });
                                            } }
                                    ]);
                                } }
                        ]));
                        return nest ? nest : outExports.removeEmpty({ kind: 'Column', column: first, json: $.SUBRULE(columnJsonRule) });
                    } }
            ]);
            const cast = $.OPTION3(() => ({ token: tokenInfo2($.CONSUME(Colon), $.CONSUME2(Colon)), type: $.SUBRULE($.columnTypeRule) }));
            return outExports.removeUndefined({ ...expr, cast });
        });
        const operatorRule = $.RULE('operatorRule', () => $.OR([
            // https://www.postgresql.org/docs/current/functions.html
            { ALT: () => ({ kind: '+', token: tokenInfo($.CONSUME(Plus)) }) },
            { ALT: () => ({ kind: '-', token: tokenInfo($.CONSUME(Dash)) }) },
            { ALT: () => ({ kind: '*', token: tokenInfo($.CONSUME(Asterisk)) }) },
            { ALT: () => ({ kind: '/', token: tokenInfo($.CONSUME(Slash)) }) },
            { ALT: () => ({ kind: '%', token: tokenInfo($.CONSUME(Percent)) }) },
            { ALT: () => ({ kind: '^', token: tokenInfo($.CONSUME(Caret)) }) },
            { ALT: () => ({ kind: '&', token: tokenInfo($.CONSUME(Amp)) }) },
            { ALT: () => ({ kind: '#', token: tokenInfo($.CONSUME(Hash)) }) },
            { ALT: () => ({ kind: '=', token: tokenInfo($.CONSUME(Equal)) }) },
            { ALT: () => ({ kind: 'In', token: tokenInfo($.CONSUME(In)) }) },
            { ALT: () => ({ kind: 'Like', token: tokenInfo($.CONSUME(Like)) }) },
            { ALT: () => {
                    const lt = $.CONSUME(LowerThan);
                    const res = $.OPTION(() => $.OR2([
                        { ALT: () => ({ kind: '<=', token: tokenInfo2(lt, $.CONSUME2(Equal)) }) },
                        { ALT: () => ({ kind: '<<', token: tokenInfo2(lt, $.CONSUME2(LowerThan)) }) },
                        { ALT: () => ({ kind: '<>', token: tokenInfo2(lt, $.CONSUME(GreaterThan)) }) },
                    ]));
                    return res ? res : { kind: '<', token: tokenInfo(lt) };
                } },
            { ALT: () => {
                    const gt = $.CONSUME2(GreaterThan);
                    const res = $.OPTION2(() => $.OR3([
                        { ALT: () => ({ kind: '>=', token: tokenInfo2(gt, $.CONSUME3(Equal)) }) },
                        { ALT: () => ({ kind: '>>', token: tokenInfo2(gt, $.CONSUME3(GreaterThan)) }) },
                    ]));
                    return res ? res : { kind: '>', token: tokenInfo(gt) };
                } },
            { ALT: () => {
                    const pipe = $.CONSUME(Pipe);
                    const res = $.OPTION3(() => ({ kind: '||', token: tokenInfo2(pipe, $.CONSUME2(Pipe)) }));
                    return res ? res : { kind: '|', token: tokenInfo(pipe) };
                } },
            { ALT: () => {
                    const tilde = $.CONSUME(Tilde);
                    const res = $.OPTION4(() => ({ kind: '~*', token: tokenInfo2(tilde, $.CONSUME2(Asterisk)) }));
                    return res ? res : { kind: '~', token: tokenInfo(tilde) };
                } },
            { ALT: () => {
                    const exclamation = $.CONSUME(Exclamation);
                    return $.OR4([
                        { ALT: () => ({ kind: '!=', token: tokenInfo2(exclamation, $.CONSUME4(Equal)) }) },
                        { ALT: () => {
                                const tilde = $.CONSUME2(Tilde);
                                const res = $.OPTION5(() => ({ kind: '!~*', token: tokenInfo3(exclamation, tilde, $.CONSUME3(Asterisk)) }));
                                return res ? res : { kind: '!~', token: tokenInfo2(exclamation, tilde) };
                            } }
                    ]);
                } },
            { ALT: () => {
                    const not = $.CONSUME2(Not);
                    return $.OPTION6(() => $.OR5([
                        { ALT: () => ({ kind: 'NotLike', token: tokenInfo2(not, $.CONSUME2(Like)) }) },
                        { ALT: () => ({ kind: 'NotIn', token: tokenInfo2(not, $.CONSUME2(In)) }) },
                    ]));
                } },
            { ALT: () => {
                    const is = $.CONSUME(Is);
                    const not = $.OPTION7(() => $.CONSUME3(Not));
                    const distinctFrom = $.OPTION8(() => { $.CONSUME(Distinct); return $.CONSUME(From); });
                    return distinctFrom ?
                        (not ? { kind: 'NotDistinctFrom', token: tokenInfo2(is, distinctFrom) } : { kind: 'DistinctFrom', token: tokenInfo2(is, distinctFrom) }) :
                        (not ? { kind: 'IsNot', token: tokenInfo2(is, not) } : { kind: 'Is', token: tokenInfo(is) });
                } },
        ]));
        const operatorLeftRule = $.RULE('operatorLeftRule', () => $.OR([
            { ALT: () => ({ kind: 'Not', token: tokenInfo($.CONSUME(Not)) }) },
            { ALT: () => ({ kind: 'Interval', token: tokenInfo($.CONSUME(Interval)) }) },
            { ALT: () => ({ kind: '~', token: tokenInfo($.CONSUME(Tilde)) }) },
        ]));
        const operatorRightRule = $.RULE('operatorRightRule', () => $.OR([
            { ALT: () => ({ kind: 'IsNull', token: tokenInfo($.CONSUME(IsNull)) }) },
            { ALT: () => ({ kind: 'NotNull', token: tokenInfo($.CONSUME(NotNull)) }) },
        ]));
        const groupRule = $.RULE('groupRule', () => {
            $.CONSUME(ParenLeft);
            const expression = $.SUBRULE($.expressionRule);
            $.CONSUME(ParenRight);
            return { kind: 'Group', expression };
        });
        const listRule = $.RULE('listRule', () => {
            $.CONSUME(ParenLeft);
            const items = [];
            $.AT_LEAST_ONE_SEP({ SEP: Comma, DEF: () => items.push($.SUBRULE($.literalRule)) });
            $.CONSUME(ParenRight);
            return { kind: 'List', items };
        });
        const columnJsonRule = $.RULE('columnJsonRule', () => {
            const res = [];
            $.MANY({ DEF: () => {
                    const nest = $.SUBRULE(jsonOpRule);
                    const field = $.OR([
                        { ALT: () => $.SUBRULE($.stringRule) },
                        { ALT: () => $.SUBRULE($.parameterRule) }
                    ]);
                    res.push({ ...nest, field });
                } });
            return res;
        });
        const jsonOpRule = $.RULE('jsonOpRule', () => {
            // https://www.postgresql.org/docs/current/functions-json.html
            const kind = $.OR([
                { ALT: () => ({ dash: true, token: $.CONSUME(Dash) }) },
                { ALT: () => ({ dash: false, token: $.CONSUME(Hash) }) },
            ]);
            const gt = $.CONSUME(GreaterThan);
            const gt2 = $.OPTION(() => $.CONSUME2(GreaterThan));
            return kind.dash ?
                (gt2 ? { kind: '->>', token: tokenInfo3(kind.token, gt, gt2) } : { kind: '->', token: tokenInfo2(kind.token, gt) }) :
                (gt2 ? { kind: '#>>', token: tokenInfo3(kind.token, gt, gt2) } : { kind: '#>', token: tokenInfo2(kind.token, gt) });
        });
        const functionParamsRule = $.RULE('functionParamsRule', () => {
            $.CONSUME(ParenLeft);
            const distinct = $.OPTION(() => ({ token: tokenInfo($.CONSUME(Distinct)) }));
            const parameters = [];
            $.MANY_SEP({ SEP: Comma, DEF: () => parameters.push($.SUBRULE($.expressionRule)) });
            $.CONSUME(ParenRight);
            return { distinct, parameters: parameters.filter(outExports.isNotUndefined) };
        });
        this.objectNameRule = $.RULE('objectNameRule', () => {
            const first = $.SUBRULE($.identifierRule);
            const second = $.OPTION(() => {
                $.CONSUME(Dot);
                return $.SUBRULE2($.identifierRule);
            });
            return second ? { schema: first, name: second } : { name: first };
        });
        this.columnTypeRule = $.RULE('columnTypeRule', () => {
            const schema = $.OPTION(() => {
                const s = $.SUBRULE($.identifierRule);
                $.CONSUME(Dot);
                return s;
            });
            const parts = [];
            $.AT_LEAST_ONE({ DEF: () => parts.push($.OR([
                    { ALT: () => {
                            const name = $.SUBRULE2($.identifierRule);
                            const params = $.OPTION2(() => {
                                $.CONSUME(ParenLeft);
                                const values = [];
                                $.AT_LEAST_ONE_SEP({ SEP: Comma, DEF: () => values.push($.SUBRULE($.integerRule)) });
                                const last = tokenInfo($.CONSUME(ParenRight));
                                return { values, last };
                            });
                            return { name, args: params?.values, last: params?.last };
                        } },
                    { ALT: () => ({ name: toIdentifier($.CONSUME(With)) }) }, // needed for `with time zone`
                ])) });
            const array = $.OPTION3(() => ({ token: tokenInfo2($.CONSUME(BracketLeft), $.CONSUME(BracketRight)) }));
            const name = {
                kind: 'Identifier',
                token: outExports$1.mergePositions(parts.flatMap(p => [p.name?.token, p.last]).concat([array?.token])),
                value: parts.filter(outExports.isNotUndefined).map(p => p.name?.value + (p.args ? `(${p.args.map(v => v.value).join(', ')})` : '')).join(' ') + (array ? '[]' : '')
            };
            return outExports.removeEmpty({ schema, name, args: parts.flatMap(p => p.args || []), array, token: outExports$1.mergePositions([schema?.token, name.token]) });
        });
        this.literalRule = $.RULE('literalRule', () => $.OR([
            { ALT: () => $.SUBRULE($.stringRule) },
            { ALT: () => $.SUBRULE($.decimalRule) },
            { ALT: () => $.SUBRULE($.integerRule) },
            { ALT: () => $.SUBRULE($.booleanRule) },
            { ALT: () => $.SUBRULE($.nullRule) },
            { ALT: () => $.SUBRULE($.parameterRule) },
        ]));
        // elements
        this.parameterRule = $.RULE('parameterRule', () => $.OR([
            { ALT: () => ({ kind: 'Parameter', value: '?', token: tokenInfo($.CONSUME(QuestionMark)) }) },
            { ALT: () => {
                    const d = $.CONSUME(Dollar);
                    const i = $.CONSUME(Integer);
                    return { kind: 'Parameter', value: `${d.image}${i.image}`, index: parseInt(i.image), token: tokenInfo2(d, i) };
                } }
        ]));
        this.identifierRule = $.RULE('identifierRule', () => $.OR([
            { ALT: () => {
                    const token = $.CONSUME(Identifier);
                    if (token.image.startsWith('"') && token.image.endsWith('"')) {
                        return { kind: 'Identifier', token: tokenInfo(token), value: token.image.slice(1, -1).replaceAll(/\\"/g, '"'), quoted: true };
                    }
                    else {
                        return { kind: 'Identifier', token: tokenInfo(token), value: token.image };
                    }
                } },
            // tokens allowed as identifiers:
            { ALT: () => toIdentifier($.CONSUME(Add)) },
            { ALT: () => toIdentifier($.CONSUME(Comment)) },
            { ALT: () => toIdentifier($.CONSUME(Commit)) },
            { ALT: () => toIdentifier($.CONSUME(Data)) },
            { ALT: () => toIdentifier($.CONSUME(Database)) },
            { ALT: () => toIdentifier($.CONSUME(Deferrable)) },
            { ALT: () => toIdentifier($.CONSUME(Domain)) },
            { ALT: () => toIdentifier($.CONSUME(Extension)) },
            { ALT: () => toIdentifier($.CONSUME(Increment)) },
            { ALT: () => toIdentifier($.CONSUME(Index)) },
            { ALT: () => toIdentifier($.CONSUME(Input)) },
            { ALT: () => toIdentifier($.CONSUME(New)) },
            { ALT: () => toIdentifier($.CONSUME(Nulls)) },
            { ALT: () => toIdentifier($.CONSUME(Old)) },
            { ALT: () => toIdentifier($.CONSUME(Rows)) },
            { ALT: () => toIdentifier($.CONSUME(Schema)) },
            { ALT: () => toIdentifier($.CONSUME(Session)) },
            { ALT: () => toIdentifier($.CONSUME(Start)) },
            { ALT: () => toIdentifier($.CONSUME(Temporary)) },
            { ALT: () => toIdentifier($.CONSUME(Trigger)) },
            { ALT: () => toIdentifier($.CONSUME(Type)) },
            { ALT: () => toIdentifier($.CONSUME(Version)) },
        ]));
        this.stringRule = $.RULE('stringRule', () => $.OR([
            { ALT: () => {
                    const token = $.CONSUME(String$1);
                    if (token.image.match(/^E/i)) {
                        // https://www.postgresql.org/docs/current/sql-syntax-lexical.html
                        return { kind: 'String', token: tokenInfo(token), value: token.image.slice(2, -1).replaceAll(/''/g, "'"), escaped: true };
                    }
                    else {
                        return { kind: 'String', token: tokenInfo(token), value: token.image.slice(1, -1).replaceAll(/''/g, "'") };
                    }
                } },
            { ALT: () => {
                    // https://www.postgresql.org/docs/current/sql-syntax-lexical.html#SQL-SYNTAX-DOLLAR-QUOTING
                    const token = $.CONSUME(StringDollar);
                    const [, dollar] = token.image.match(/^(\$[^$]*\$)/) || [];
                    const prefix = dollar?.length || 0;
                    return { kind: 'String', token: tokenInfo(token), value: token.image.slice(prefix, -prefix).trim(), dollar };
                } },
        ]));
        this.integerRule = $.RULE('integerRule', () => {
            const neg = $.OPTION(() => $.CONSUME(Dash));
            const token = $.CONSUME(Integer);
            return neg ? { kind: 'Integer', token: tokenInfo2(neg, token), value: parseInt(neg.image + token.image) } :
                { kind: 'Integer', token: tokenInfo(token), value: parseInt(token.image) };
        });
        this.decimalRule = $.RULE('decimalRule', () => {
            const neg = $.OPTION(() => $.CONSUME(Dash));
            const token = $.CONSUME(Decimal);
            return neg ? { kind: 'Decimal', token: tokenInfo2(neg, token), value: parseFloat(neg.image + token.image) } :
                { kind: 'Decimal', token: tokenInfo(token), value: parseFloat(token.image) };
        });
        this.booleanRule = $.RULE('booleanRule', () => $.OR([
            { ALT: () => ({ kind: 'Boolean', token: tokenInfo($.CONSUME(True)), value: true }) },
            { ALT: () => ({ kind: 'Boolean', token: tokenInfo($.CONSUME(False)), value: false }) },
        ]));
        this.nullRule = $.RULE('nullRule', () => ({ kind: 'Null', token: tokenInfo($.CONSUME(Null)) }));
        this.performSelfAnalysis();
    }
}
const lexer = new Lexer(allTokens);
const parserStrict = new PostgresParser(allTokens, false);
const parserWithRecovery = new PostgresParser(allTokens, true);
// exported only for tests, use the `parse` function instead
function parseRule(parse, input, strict = false) {
    const lexingResult = lexer.tokenize(input);
    const parser = strict ? parserStrict : parserWithRecovery;
    parser.input = lexingResult.tokens; // "input" is a setter which will reset the parser's state.
    const res = parse(parser);
    const errors = lexingResult.errors.map(formatLexerError).concat(parser.errors.map(formatParserError));
    const comments = lexingResult.groups.comments.map(buildComment);
    return new outExports$1.ParserResult(comments.length > 0 ? { ...res, comments } : res, errors);
}
function parsePostgresAst(input, opts = { strict: false }) {
    return parseRule(p => p.statementsRule(), input, opts.strict || false);
}
function buildComment(token) {
    if (token.tokenType.name === 'LineComment') {
        return { kind: 'line', token: tokenInfo(token), value: token.image.slice(2).trim() };
    }
    else if (token.image.startsWith('/*\n *') && token.image.endsWith('\n */')) {
        return { kind: 'doc', token: tokenInfo(token), value: token.image.slice(3, -4).split('\n').map(line => line.startsWith(' *') ? line.slice(2).trim() : line.trim()).join('\n') };
    }
    else {
        return { kind: 'block', token: tokenInfo(token), value: token.image.slice(2, -2).trim() };
    }
}
function formatLexerError(err) {
    return {
        message: err.message,
        kind: 'LexingError',
        level: outExports$1.ParserErrorLevel.enum.error,
        offset: { start: err.offset, end: err.offset + err.length },
        position: {
            start: { line: err.line || defaultPos, column: err.column || defaultPos },
            end: { line: err.line || defaultPos, column: (err.column || defaultPos) + err.length }
        }
    };
}
function formatParserError(err) {
    return { message: err.message, kind: err.name, level: outExports$1.ParserErrorLevel.enum.error, ...tokenInfo(err.token) };
}
function toIdentifier(token) {
    return { kind: 'Identifier', token: tokenInfo(token), value: token.image };
}
function tokenInfo(token, issues) {
    return outExports.removeEmpty({ ...tokenPosition(token), issues });
}
function tokenInfo2(start, end, issues) {
    return outExports.removeEmpty({ ...outExports$1.mergePositions([start, end].map(t => t ? tokenPosition(t) : undefined)), issues });
}
function tokenInfo3(start, middle, end, issues) {
    return outExports.removeEmpty({ ...outExports$1.mergePositions([start, middle, end].map(t => t ? tokenPosition(t) : undefined)), issues });
}
function tokenInfoN(tokens, issues) {
    return outExports.removeEmpty({ ...outExports$1.mergePositions(tokens.map(t => t ? tokenPosition(t) : undefined)), issues });
}
function tokenPosition(token) {
    return {
        offset: { start: pos(token.startOffset), end: pos(token.endOffset) },
        position: {
            start: { line: pos(token.startLine), column: pos(token.startColumn) },
            end: { line: pos(token.endLine), column: pos(token.endColumn) }
        }
    };
}
function pos(value) {
    return value !== undefined && !Number.isNaN(value) ? value : defaultPos;
}

const duplicated = (name, definedAtLine, position) => ({ message: `${name} already defined${definedAtLine !== undefined ? ` at line ${definedAtLine}` : ''}`, kind: 'Duplicated', level: outExports$1.ParserErrorLevel.enum.warning, offset: position.offset, position: position.position });

function buildPostgresDatabase(ast, start, parsed) {
    const db = { entities: [], relations: [], types: [] };
    const errors = [];
    ast.statements.forEach((stmt, i) => evolvePostgres(db, errors, i + 1, stmt));
    const done = Date.now();
    const extra = outExports.removeEmpty({
        source: `PostgreSQL parser <${packageJson.version}>`,
        createdAt: new Date().toISOString(),
        creationTimeMs: done - start,
        parsingTimeMs: parsed - start,
        formattingTimeMs: done - parsed,
        comments: ast.comments?.map(c => ({ line: c.token.position.start.line, comment: c.value })) || [],
    });
    return new outExports$1.ParserResult(outExports.removeEmpty({ ...db, extra: { ...db.extra, ...extra } }), errors);
}
function evolvePostgres(db, errors, index, stmt) {
    if (stmt.kind === 'AlterTable') {
        alterTable(index, stmt, db);
    }
    else if (stmt.kind === 'CommentOn') {
        commentOn(index, stmt, db);
    }
    else if (stmt.kind === 'CreateIndex') {
        if (!db.entities)
            db.entities = [];
        createIndex(index, stmt, db.entities);
    }
    else if (stmt.kind === 'CreateMaterializedView') {
        if (!db.entities)
            db.entities = [];
        const entity = createMaterializedView(index, stmt, db.entities);
        addEntity(db.entities, errors, entity, outExports$1.mergePositions([stmt.schema, stmt.name].map(v => v?.token)));
    }
    else if (stmt.kind === 'CreateTable') {
        if (!db.entities)
            db.entities = [];
        const res = createTable(index, stmt);
        addEntity(db.entities, errors, res.entity, outExports$1.mergePositions([stmt.schema, stmt.name].map(v => v?.token)));
        res.relations.forEach(r => db.relations?.push(r));
    }
    else if (stmt.kind === 'CreateType') {
        if (!db.types)
            db.types = [];
        const type = createType(index, stmt);
        addType(db.types, errors, type, outExports$1.mergePositions([stmt.schema, stmt.name].map(v => v?.token)));
    }
    else if (stmt.kind === 'CreateView') {
        if (!db.entities)
            db.entities = [];
        const entity = createView(index, stmt, db.entities);
        addEntity(db.entities, errors, entity, outExports$1.mergePositions([stmt.schema, stmt.name].map(v => v?.token)));
    }
    else if (stmt.kind === 'Drop') {
        if (stmt.object === 'Table' || stmt.object === 'View' || stmt.object === 'MaterializedView') {
            const index = db.entities?.findIndex(e => stmt.entities.some(ee => ee.name.value === e.name && (ee.schema?.value ? ee.schema.value === e.schema : true)));
            if (index !== undefined && index !== -1) {
                db.entities?.splice(index, 1);
                // TODO: remove all relations linked to this entity
            }
        }
        else if (stmt.object === 'Index') ;
        else if (stmt.object === 'Sequence') ;
        else if (stmt.object === 'Type') {
            const index = db.types?.findIndex(t => stmt.entities.some(e => e.name.value === t.name && (e.schema?.value ? e.schema.value === t.schema : true)));
            if (index !== undefined && index !== -1) {
                db.types?.splice(index, 1);
            }
        }
        else {
            outExports.isNever(stmt.object);
        }
    }
    else if (stmt.kind === 'Select') {
        addDQL(db, index, stmt);
    }
    else if (stmt.kind === 'InsertInto') {
        addDML(db, index, stmt);
    }
    else if (stmt.kind === 'Update') {
        addDML(db, index, stmt);
    }
    else if (stmt.kind === 'Delete') {
        addDML(db, index, stmt);
    }
    else if (stmt.kind === 'AlterFunction') ;
    else if (stmt.kind === 'AlterMaterializedView') ;
    else if (stmt.kind === 'AlterSchema') ;
    else if (stmt.kind === 'AlterSequence') ;
    else if (stmt.kind === 'AlterType') ;
    else if (stmt.kind === 'AlterView') ;
    else if (stmt.kind === 'Begin') ;
    else if (stmt.kind === 'Commit') ;
    else if (stmt.kind === 'CreateExtension') ;
    else if (stmt.kind === 'CreateFunction') ;
    else if (stmt.kind === 'CreateTrigger') ;
    else if (stmt.kind === 'CreateSchema') ;
    else if (stmt.kind === 'CreateSequence') ;
    else if (stmt.kind === 'Set') ;
    else if (stmt.kind === 'Show') ;
    else {
        outExports.isNever(stmt);
    }
}
function addEntity(entities, errors, entity, pos) {
    const ref = outExports$1.entityToRef(entity);
    const prevIndex = entities.findIndex(e => outExports$1.entityRefSame(outExports$1.entityToRef(e), ref));
    if (prevIndex !== -1) {
        const prev = entities[prevIndex];
        errors.push(duplicated(`Entity ${outExports$1.entityToId(entity)}`, prev.extra?.line ? prev.extra.line : undefined, pos));
        entities[prevIndex] = outExports$1.mergeEntity(prev, entity);
    }
    else {
        entities.push(entity);
    }
}
function addType(types, errors, type, pos) {
    const ref = outExports$1.typeToRef(type);
    const prevIndex = types.findIndex(t => outExports$1.typeRefSame(outExports$1.typeToRef(t), ref));
    if (prevIndex !== -1) {
        const prev = types[prevIndex];
        errors.push(duplicated(`Type ${outExports$1.typeToId(type)}`, prev.extra?.line ? prev.extra.line : undefined, pos));
        types[prevIndex] = outExports$1.mergeType(prev, type);
    }
    else {
        types.push(type);
    }
}
function addDQL(db, index, stmt) {
    if (!db.extra)
        db.extra = {};
    if (!db.extra.dql)
        db.extra.dql = [];
    db.extra.dql.push({ ...outExports.removeFieldsDeep(stmt, ['token']), statement: index });
}
function addDML(db, index, stmt) {
    if (!db.extra)
        db.extra = {};
    if (!db.extra.dql)
        db.extra.dml = [];
    db.extra.dml.push({ ...outExports.removeFieldsDeep(stmt, ['token']), statement: index });
}
function createTable(index, stmt) {
    const colPk = stmt.columns?.flatMap(col => col.constraints?.flatMap(c => c.kind === 'PrimaryKey' ? [outExports.removeUndefined({ attrs: [[col.name.value]], name: c.constraint?.name.value, extra: { line: c.token.position.start.line, statement: index } })] : []) || []) || [];
    const tablePk = stmt.constraints?.flatMap(c => c.kind === 'PrimaryKey' ? [outExports.removeUndefined({ attrs: c.columns.map(col => [col.value]), name: c.constraint?.name.value, extra: { line: c.token.position.start.line, statement: index } })] : []) || [];
    const pk = colPk.concat(tablePk);
    const colIndexes = stmt.columns?.flatMap(col => col.constraints?.flatMap(c => c.kind === 'Unique' ? [outExports.removeUndefined({ attrs: [[col.name.value]], unique: true, name: c.constraint?.name.value, extra: { line: c.token.position.start.line, statement: index } })] : []) || []) || [];
    const tableIndexes = stmt.constraints?.flatMap(c => c.kind === 'Unique' ? [outExports.removeUndefined({ attrs: c.columns.map(col => [col.value]), unique: true, name: c.constraint?.name.value, extra: { line: c.token.position.start.line, statement: index } })] : []) || [];
    const indexes = colIndexes.concat(tableIndexes);
    const colChecks = stmt.columns?.flatMap(col => col.constraints?.flatMap(c => c.kind === 'Check' ? [outExports.removeUndefined({ attrs: [[col.name.value]], predicate: expressionToString(c.predicate), name: c.constraint?.name.value, extra: { line: c.token.position.start.line, statement: index } })] : []) || []) || [];
    const tableChecks = stmt.constraints?.flatMap(c => c.kind === 'Check' ? [outExports.removeUndefined({ attrs: expressionAttrs(c.predicate), predicate: expressionToString(c.predicate), name: c.constraint?.name.value, extra: { line: c.token.position.start.line, statement: index } })] : []) || [];
    const checks = colChecks.concat(tableChecks);
    const colRels = stmt.columns?.flatMap(col => col.constraints?.flatMap(c => c.kind === 'ForeignKey' ? [outExports.removeUndefined({
            name: c.constraint?.name.value,
            src: outExports.removeUndefined({ schema: stmt.schema?.value, entity: stmt.name.value, attrs: [[col.name.value]] }),
            ref: outExports.removeUndefined({ schema: c.schema?.value, entity: c.table.value, attrs: [c.column ? [c.column.value] : []] }),
            extra: { line: c.token.position.start.line, statement: index },
        })] : []) || []) || [];
    const tableRels = stmt.constraints?.flatMap(c => c.kind === 'ForeignKey' ? [outExports.removeUndefined({
            name: c.constraint?.name.value,
            src: outExports.removeUndefined({ schema: stmt.schema?.value, entity: stmt.name.value, attrs: c.columns.map(col => [col.value]) }),
            ref: outExports.removeUndefined({ schema: c.ref.schema?.value, entity: c.ref.table.value, attrs: c.ref.columns?.map(col => [col.value]) || [] }),
            extra: { line: c.token.position.start.line, statement: index },
        })] : []) || [];
    const relations = colRels.concat(tableRels);
    return { entity: outExports.removeEmpty({
            schema: stmt.schema?.value,
            name: stmt.name.value,
            kind: undefined,
            def: undefined,
            attrs: (stmt.columns || []).map(c => buildTableAttr(index, c, !!pk.find(pk => pk.attrs.some(a => outExports$1.attributePathSame(a, [c.name.value]))))),
            pk: pk.length > 0 ? pk[0] : undefined,
            indexes,
            checks,
            // doc: z.string().optional(), // not defined in CREATE TABLE
            // stats: EntityStats.optional(), // no stats in SQL
            extra: { line: stmt.token.position.start.line, statement: index },
        }), relations };
}
function buildTableAttr(index, c, notNull) {
    return outExports.removeUndefined({
        name: c.name.value,
        type: c.type.name.value,
        null: (c.constraints?.find(c => c.kind === 'Nullable' ? !c.value : false) || notNull) ? undefined : true,
        // gen: z.boolean().optional(), // not handled for now
        default: (c.constraints || []).flatMap(c => c.kind === 'Default' ? [expressionToValue(c.expression)] : [])[0],
        // attrs: z.lazy(() => Attribute.array().optional()), // no nested attrs from SQL
        // doc: z.string().optional(), // not defined in CREATE TABLE
        // stats: AttributeStats.optional(), // no stats in SQL
        extra: { line: c.name.token.position.start.line, statement: index },
    });
}
function createView(index, stmt, entities) {
    return outExports.removeEmpty({
        schema: stmt.schema?.value,
        name: stmt.name.value,
        kind: 'view',
        def: selectInnerToString(stmt.query),
        attrs: buildViewAttrs(index, stmt.query, stmt.columns, entities),
        // pk: PrimaryKey.optional(), // not in VIEW
        // indexes: Index.array().optional(), // not in VIEW
        // checks: Check.array().optional(), // not in VIEW
        // doc: z.string().optional(), // not in VIEW
        // stats: EntityStats.optional(), // no stats in SQL
        extra: { line: stmt.token.position.start.line, statement: index },
    });
}
function createMaterializedView(index, stmt, entities) {
    return outExports.removeEmpty({
        schema: stmt.schema?.value,
        name: stmt.name.value,
        kind: 'materialized view',
        def: selectInnerToString(stmt.query),
        attrs: buildViewAttrs(index, stmt.query, stmt.columns, entities),
        // pk: PrimaryKey.optional(), // not in VIEW
        // indexes: Index.array().optional(), // not in VIEW
        // checks: Check.array().optional(), // not in VIEW
        // doc: z.string().optional(), // not in VIEW
        // stats: EntityStats.optional(), // no stats in SQL
        extra: { line: stmt.token.position.start.line, statement: index },
    });
}
function buildViewAttrs(index, query, columns, entities) {
    const attrs = selectEntities(query, entities).columns.map(c => outExports.removeUndefined({
        name: c.name,
        type: c.type || 'unknown',
        // null: z.boolean().optional(), // not in VIEW
        // gen: z.boolean().optional(), // not handled for now
        // default: AttributeValue.optional(), // now in VIEW
        // attrs: z.lazy(() => Attribute.array().optional()), // no nested attrs from SQL
        // doc: z.string().optional(), // not defined in CREATE TABLE
        // stats: AttributeStats.optional(), // no stats in SQL
        extra: { line: c.token.position.start.line, statement: index },
    }));
    return columns ? columns.map(c => attrs.find(a => a.name === c.value) || { name: c.value, type: 'unknown' }) : attrs;
}
function createIndex(index, stmt, entities) {
    const entity = entities.find(e => e.schema === stmt.schema?.value && e.name === stmt.table?.value);
    if (entity) {
        if (!entity.indexes)
            entity.indexes = [];
        entity.indexes.push(outExports.removeUndefined({
            name: stmt.name?.value,
            attrs: stmt.columns.map(c => {
                if (c.kind === 'Column') {
                    return [c.column.value]; // TODO: improve
                }
            }).filter(outExports.isNotUndefined),
            unique: stmt.unique ? true : undefined,
            partial: stmt.where ? expressionToString(stmt.where.predicate) : undefined,
            // TODO: definition: z.string().optional(),
            // doc: z.string().optional(),
            // stats: IndexStats.optional(),
            extra: { line: stmt.token.position.start.line, statement: index },
        }));
    }
}
function alterTable(index, stmt, db) {
    const entity = db.entities?.find(e => e.schema === stmt.schema?.value && e.name === stmt.name?.value);
    if (entity) {
        stmt.actions.forEach(action => {
            if (action.kind === 'AddColumn') {
                if (!entity.attrs)
                    entity.attrs = [];
                const exists = entity.attrs.find(a => a.name === action.column.name.value);
                if (!exists)
                    entity.attrs.push(buildTableAttr(index, action.column));
            }
            else if (action.kind === 'DropColumn') {
                const attrIndex = entity.attrs?.findIndex(a => a.name === action.column.value);
                if (attrIndex !== undefined && attrIndex !== -1)
                    entity.attrs?.splice(attrIndex, 1);
                // TODO: remove constraints depending on this column
            }
            else if (action.kind === 'AlterColumn') {
                const attr = entity.attrs?.find(a => a.name === action.column.value);
                if (attr) {
                    const aa = action.action;
                    if (aa.kind === 'Default') {
                        attr.default = aa.action.kind === 'Set' && aa.expression ? expressionToValue(aa.expression) : undefined;
                    }
                    else if (aa.kind === 'NotNull') {
                        attr.null = aa.action.kind === 'Set' ? undefined : true;
                    }
                    else {
                        outExports.isNever(aa);
                    }
                }
            }
            else if (action.kind === 'AddConstraint') {
                const constraint = action.constraint;
                if (constraint.kind === 'PrimaryKey') {
                    entity.pk = outExports.removeUndefined({ name: constraint.constraint?.name.value, attrs: constraint.columns.map(c => [c.value]), extra: { line: stmt.token.position.start.line, statement: index } });
                }
                else if (constraint.kind === 'Unique') {
                    if (!entity.indexes)
                        entity.indexes = [];
                    entity.indexes.push(outExports.removeUndefined({
                        name: constraint.constraint?.name.value,
                        attrs: constraint.columns.map(c => [c.value]),
                        unique: true,
                        extra: { line: stmt.token.position.start.line, statement: index },
                    }));
                }
                else if (constraint.kind === 'Check') {
                    if (!entity.checks)
                        entity.checks = [];
                    entity.checks.push(outExports.removeUndefined({
                        name: constraint.constraint?.name.value,
                        attrs: expressionAttrs(constraint.predicate),
                        predicate: expressionToString(constraint.predicate),
                        extra: { line: stmt.token.position.start.line, statement: index },
                    }));
                }
                else if (constraint.kind === 'ForeignKey') {
                    if (!db.relations)
                        db.relations = [];
                    db.relations.push(outExports.removeUndefined({
                        name: constraint.constraint?.name.value,
                        src: outExports.removeUndefined({ schema: stmt.schema?.value, entity: stmt.name?.value, attrs: constraint.columns.map(c => [c.value]) }),
                        ref: outExports.removeUndefined({ schema: constraint.ref.schema?.value, entity: constraint.ref.table.value, attrs: constraint.ref.columns?.map(c => [c.value]) || [] }),
                        extra: { line: stmt.token.position.start.line, statement: index },
                    }));
                }
                else {
                    outExports.isNever(constraint);
                }
            }
            else if (action.kind === 'DropConstraint') {
                if (entity.pk?.name === action.constraint.value)
                    entity.pk = undefined;
                const idxIndex = entity.indexes?.findIndex(a => a.name === action.constraint.value);
                if (idxIndex !== undefined && idxIndex !== -1)
                    entity.indexes?.splice(idxIndex, 1);
                const chkIndex = entity.checks?.findIndex(c => c.name === action.constraint.value);
                if (chkIndex !== undefined && chkIndex !== -1)
                    entity.checks?.splice(chkIndex, 1);
                const relIndex = db.relations?.findIndex(r => r.name === action.constraint.value && r.src.schema === stmt.schema?.value && r.src.entity === stmt.name?.value);
                if (relIndex !== undefined && relIndex !== -1)
                    db.relations?.splice(relIndex, 1);
                // TODO: also NOT NULL & DEFAULT constraints...
            }
            else if (action.kind === 'SetOwner') ;
            else {
                outExports.isNever(action);
            }
        });
    }
}
function createType(index, stmt) {
    return outExports.removeUndefined({
        schema: stmt.schema?.value,
        name: stmt.name.value,
        // alias: z.string().optional(), // does not exist in PostgreSQL
        values: stmt.enum?.values.map(v => v.value),
        attrs: stmt.struct?.attrs.map(a => ({
            name: a.name.value,
            type: a.type.name.value,
        })),
        definition: stmt.base ? '(' + stmt.base.map(p => `${p.name.value} = ${expressionToString(p.value)}`).join(', ') + ')' : undefined,
        // doc: z.string().optional(), // not defined in CREATE TYPE
        extra: { line: stmt.token.position.start.line, statement: index },
    });
}
function commentOn(index, stmt, db) {
    // TODO: store comment statement infos? where?
    const object = stmt.object.kind;
    if (object === 'Column') {
        const entity = db.entities?.find(e => e.schema === stmt.schema?.value && e.name === stmt.parent?.value);
        const attr = entity?.attrs?.find(a => a.name === stmt.entity.value);
        if (attr)
            attr.doc = stmt.comment.kind === 'String' ? stmt.comment.value : undefined;
    }
    else if (object === 'Constraint') {
        const entity = db.entities?.find(e => e.name === stmt.parent?.value && e.schema === stmt.schema?.value);
        if (entity) {
            if (entity.pk?.name === stmt.entity.value)
                entity.pk.doc = stmt.comment.kind === 'String' ? stmt.comment.value : undefined;
            const index = entity.indexes?.find(i => i.name === stmt.entity.value);
            if (index)
                index.doc = stmt.comment.kind === 'String' ? stmt.comment.value : undefined;
            const check = entity.checks?.find(c => c.name === stmt.entity.value);
            if (check)
                check.doc = stmt.comment.kind === 'String' ? stmt.comment.value : undefined;
        }
        const rel = db.relations?.find(r => r.name === stmt.entity.value && r.src.entity === stmt.parent?.value && r.src.schema === stmt.schema?.value);
        if (rel)
            rel.doc = stmt.comment.kind === 'String' ? stmt.comment.value : undefined;
    }
    else if (object === 'Database') {
        if (!db.extra)
            db.extra = {};
        db.extra.doc = stmt.comment.kind === 'String' ? stmt.comment.value : undefined;
    }
    else if (object === 'Extension') ;
    else if (object === 'Index') {
        const index = db.entities?.flatMap(e => e.schema === stmt.schema?.value && e.name === stmt.parent?.value ? e.indexes?.filter(i => i.name === stmt.entity.value) : [])?.[0];
        if (index)
            index.doc = stmt.comment.kind === 'String' ? stmt.comment.value : undefined;
    }
    else if (object === 'Schema') ;
    else if (object === 'Table' || object === 'View' || object === 'MaterializedView') {
        const entity = db.entities?.find(e => e.schema === stmt.schema?.value && e.name === stmt.entity.value);
        if (entity)
            entity.doc = stmt.comment.kind === 'String' ? stmt.comment.value : undefined;
    }
    else if (object === 'Type') {
        const type = db.types?.find(e => e.schema === stmt.schema?.value && e.name === stmt.entity.value);
        if (type)
            type.doc = stmt.comment.kind === 'String' ? stmt.comment.value : undefined;
    }
    else {
        outExports.isNever(object);
    }
}
function selectInnerToString(s) {
    const select = 'SELECT ' + s.columns.map(c => expressionToString(c) + (c.alias ? ' ' + aliasToString(c.alias) : '')).join(', ');
    const from = s.from ? fromToString(s.from) : '';
    const where = s.where ? ' WHERE ' + expressionToString(s.where.predicate) : '';
    return select + from + where;
}
function fromToString(f) {
    if (f.kind === 'Table')
        return ' FROM ' + (f.schema ? f.schema.value + '.' : '') + f.table.value + (f.alias ? ' ' + aliasToString(f.alias) : '');
    if (f.kind === 'Select')
        return ' FROM (' + selectInnerToString(f.select) + ')' + (f.alias ? ' ' + aliasToString(f.alias) : '');
    return outExports.isNever(f);
}
function aliasToString(a) {
    return (a.token ? 'AS ' : '') + a.name.value;
}
function expressionToString(e) {
    if (e.kind === 'String')
        return "'" + e.value + "'";
    if (e.kind === 'Integer')
        return e.value.toString();
    if (e.kind === 'Decimal')
        return e.value.toString();
    if (e.kind === 'Boolean')
        return e.value.toString();
    if (e.kind === 'Null')
        return 'null';
    if (e.kind === 'Parameter')
        return e.value;
    if (e.kind === 'Column')
        return columnToString(e);
    if (e.kind === 'Wildcard')
        return (e.schema ? e.schema.value + '.' : '') + (e.table ? e.table.value + '.' : '') + '*';
    if (e.kind === 'Function')
        return functionToString(e);
    if (e.kind === 'Group')
        return '(' + expressionToString(e.expression) + ')';
    if (e.kind === 'Operation')
        return expressionToString(e.left) + ' ' + operatorToString(e.op.kind) + ' ' + expressionToString(e.right);
    if (e.kind === 'OperationLeft')
        return operatorLeftToString(e.op.kind) + ' ' + expressionToString(e.right);
    if (e.kind === 'OperationRight')
        return expressionToString(e.left) + ' ' + operatorRightToString(e.op.kind);
    if (e.kind === 'Array')
        return '[' + e.items.map(expressionToString).join(', ') + ']';
    if (e.kind === 'List')
        return '(' + e.items.map(expressionToString).join(', ') + ')';
    return outExports.isNever(e);
}
function expressionToValue(e) {
    if (e.kind === 'String')
        return e.value;
    if (e.kind === 'Integer')
        return e.value;
    if (e.kind === 'Decimal')
        return e.value;
    if (e.kind === 'Boolean')
        return e.value;
    if (e.kind === 'Null')
        return null;
    if (e.kind === 'Parameter')
        return e.value;
    if (e.kind === 'Column')
        return columnToString(e);
    if (e.kind === 'Wildcard')
        return (e.schema ? e.schema.value + '.' : '') + (e.table ? e.table.value + '.' : '') + '*';
    if (e.kind === 'Function')
        return '`' + functionToString(e) + '`';
    if (e.kind === 'Group')
        return expressionToValue(e.expression);
    if (e.kind === 'Operation')
        return '`' + expressionToString(e) + '`';
    if (e.kind === 'OperationLeft')
        return '`' + expressionToString(e) + '`';
    if (e.kind === 'OperationRight')
        return '`' + expressionToString(e) + '`';
    if (e.kind === 'Array')
        return e.items.map(expressionToValue);
    if (e.kind === 'List')
        return e.items.map(expressionToValue);
    return outExports.isNever(e);
}
function columnToString(c) {
    const schema = c.schema ? c.schema.value + '.' : '';
    const table = c.table ? c.table.value + '.' : '';
    const json = c.json ? c.json.map(j => j.kind + j.field.value).join('') : '';
    return schema + table + c.column.value + json;
}
function functionToString(f) {
    const schema = f.schema ? f.schema.value + '.' : '';
    const distinct = f.distinct ? 'distinct ' : '';
    const params = f.parameters.map(expressionToString).join(', ');
    return schema + f.function.value + '(' + distinct + params + ')';
}
function operatorToString(o) {
    if (o === 'Or')
        return 'OR';
    if (o === 'And')
        return 'AND';
    if (o === 'Is')
        return 'IS';
    if (o === 'In')
        return 'IN';
    if (o === 'NotIn')
        return 'NOT IN';
    if (o === 'Like')
        return 'LIKE';
    if (o === 'NotLike')
        return 'NOT LIKE';
    return o;
}
function operatorLeftToString(o) {
    if (o === 'Not')
        return 'NOT';
    if (o === 'Interval')
        return 'INTERVAL';
    return o;
}
function operatorRightToString(o) {
    if (o === 'IsNull')
        return 'IS NULL';
    if (o === 'NotNull')
        return 'IS NOT NULL';
    return o;
}
function expressionAttrs(e) {
    if (e.kind === 'String')
        return [];
    if (e.kind === 'Integer')
        return [];
    if (e.kind === 'Decimal')
        return [];
    if (e.kind === 'Boolean')
        return [];
    if (e.kind === 'Null')
        return [];
    if (e.kind === 'Parameter')
        return [];
    if (e.kind === 'Column')
        return [[e.column.value]];
    if (e.kind === 'Wildcard')
        return [];
    if (e.kind === 'Function')
        return outExports.distinctBy(e.parameters.flatMap(expressionAttrs), p => p.join('.'));
    if (e.kind === 'Group')
        return expressionAttrs(e.expression);
    if (e.kind === 'Operation')
        return outExports.distinctBy(expressionAttrs(e.left).concat(expressionAttrs(e.right)), p => p.join('.'));
    if (e.kind === 'OperationLeft')
        return expressionAttrs(e.right);
    if (e.kind === 'OperationRight')
        return expressionAttrs(e.left);
    if (e.kind === 'Array')
        return [];
    if (e.kind === 'List')
        return [];
    return outExports.isNever(e);
}
// TODO: also extract entities in clauses such as WHERE, HAVING... (know all use involved tables & columns, wherever they are used)
function selectEntities(s, entities) {
    const sources = s.from ? selectTables(s.from, entities) : [];
    const columns = s.columns.flatMap((c, i) => selectColumn(c, i, sources));
    return { columns, sources };
}
function selectTables(f, entities) {
    const joins = f.joins?.map(j => fromTables(j.from, entities)) || [];
    return [fromTables(f, entities), ...joins];
}
function fromTables(i, entities) {
    if (i.kind === 'Table') {
        const entity = findEntity(entities, i.table.value, i.schema?.value);
        if (entity) {
            return { name: i.alias?.name.value || i.table.value, from: outExports.removeEmpty({ kind: 'Table', schema: entity.schema, table: entity.name, columns: entity.attrs?.map(a => ({ name: a.name, type: a.type })) || [] }) };
        }
        else {
            return { name: i.alias?.name.value || i.table.value, from: outExports.removeUndefined({ kind: 'Table', schema: i.schema?.value, table: i.table.value }) };
        }
    }
    else if (i.kind === 'Select') {
        return { name: i.alias?.name.value || '', from: { kind: 'Select', ...selectEntities(i.select, entities) } };
    }
    else {
        return outExports.isNever(i);
    }
}
function selectColumn(c, i, sources) {
    if (c.kind === 'Column') {
        const ref = outExports.removeUndefined({ schema: c.schema?.value, table: c.table?.value, name: c.alias ? c.alias.name.value : c.column.value, token: c.column.token });
        const source = findSource(sources, c.schema?.value, c.table?.value, c.column.value)?.from;
        if (source?.kind === 'Table') {
            const col = source.columns?.find(col => col.name === c.column.value);
            return [outExports.removeUndefined({ ...ref, type: col?.type, sources: [outExports.removeUndefined({ schema: source.schema, table: source.table, column: c.column.value, type: col?.type })] })];
        }
        else if (source?.kind === 'Select') {
            const col = source.columns.find(col => col.name === c.column.value);
            return [col ? col : { ...ref, sources: [] }]; // `col` should always exist in correct queries
        }
        else {
            return [{ ...ref, sources: [] }]; // `source` should always exist in correct queries
        }
    }
    else if (c.kind === 'Wildcard') {
        const ref = outExports.removeUndefined({ schema: c.schema?.value, table: c.table?.value, name: c.alias ? c.alias.name.value : '*', token: c.token });
        const source = findSource(sources, c.schema?.value, c.table?.value, undefined)?.from;
        if (source?.kind === 'Table') {
            const cols = source.columns || [];
            if (cols.length > 0) {
                return cols.map(a => ({ ...ref, name: a.name, type: a.type, sources: [outExports.removeUndefined({ schema: source.schema, table: source.table, column: a.name, type: a.type })] }));
            }
            else {
                return [{ ...ref, sources: [] }]; // Wildcard from a table with no columns :/ (happen when missing entities)
            }
        }
        else if (source?.kind === 'Select') {
            return source.columns.map(col => outExports.removeUndefined({ ...col, schema: c.schema?.value, table: c.table?.value }));
        }
        else {
            return [{ ...ref, sources: [] }]; // `source` should always exist in correct queries
        }
    }
    else if (c.kind === 'Function') {
        return [{ name: c.alias ? c.alias.name.value : c.function.value, sources: columnSources(c, sources), token: c.function.token }];
    }
    else {
        return [{ name: c.alias ? c.alias.name.value : `col_${i + 1}`, sources: columnSources(c, sources), token: expressionToken(c) }];
    }
}
function findEntity(entities, entity, schema) {
    const candidates = entities.filter(e => e.name === entity);
    if (schema !== undefined)
        return candidates.find(e => e.schema === schema);
    if (candidates.length === 1)
        return candidates[0];
    return candidates.find(e => e.schema === undefined) || candidates.find(e => e.schema === '');
}
function findSource(sources, schema, table, column) {
    // not sure what to do with `schema` :/
    if (table) {
        return sources.find(s => s.name === table);
    }
    if (column) {
        const candidates = sources.filter(s => {
            if (s.from.kind === 'Table') {
                return !!s.from.columns?.find(a => a.name === column);
            }
            else if (s.from.kind === 'Select') {
                return !!s.from.columns.find(c => c.name === column);
            }
            else {
                return outExports.isNever(s.from);
            }
        });
        return candidates.length === 1 ? candidates[0] : sources.length === 1 ? sources[0] : undefined;
    }
    return sources.length === 1 ? sources[0] : undefined;
}
function columnSources(c, sources) {
    if (c.kind === 'Column') {
        const source = c.table ? sources.find(s => s.name === c.table?.value) : sources.length === 1 ? sources[0] : undefined;
        if (source && source.from.kind === 'Table') {
            return [outExports.removeUndefined({ schema: source.from.schema, table: source.from.table, column: c.column.value })];
        }
    }
    if (c.kind === 'Wildcard')
        return [];
    if (c.kind === 'Function')
        return c.parameters.flatMap(p => columnSources(p, sources));
    if (c.kind === 'Group')
        return columnSources(c.expression, sources);
    if (c.kind === 'Operation')
        return columnSources(c.left, sources).concat(columnSources(c.right, sources));
    if (c.kind === 'OperationLeft')
        return columnSources(c.right, sources);
    if (c.kind === 'OperationRight')
        return columnSources(c.left, sources);
    return [];
}
function expressionToken(e) {
    if (e.kind === 'Parameter' || e.kind === 'String' || e.kind === 'Decimal' || e.kind === 'Integer' || e.kind === 'Boolean' || e.kind === 'Null') {
        return e.token;
    }
    else if (e.kind === 'Column') {
        return e.column.token;
    }
    else if (e.kind === 'Wildcard') {
        return e.token;
    }
    else if (e.kind === 'Function') {
        return e.function.token;
    }
    else if (e.kind === 'Group') {
        return expressionToken(e.expression);
    }
    else if (e.kind === 'Operation') {
        return outExports$1.mergePositions([expressionToken(e.left), expressionToken(e.right)]);
    }
    else if (e.kind === 'OperationLeft') {
        return outExports$1.mergePositions([e.op.token, expressionToken(e.right)]);
    }
    else if (e.kind === 'OperationRight') {
        return outExports$1.mergePositions([expressionToken(e.left), e.op.token]);
    }
    else if (e.kind === 'Array') {
        return outExports$1.mergePositions([e.token, ...e.items.map(expressionToken)]);
    }
    else if (e.kind === 'List') {
        return outExports$1.mergePositions(e.items.map(expressionToken));
    }
    else {
        return outExports.isNever(e);
    }
}

function generatePostgres(database) {
    const typesById = outExports.indexBy(database.types || [], outExports$1.typeToId);
    const drops = (database.entities || []).map(genEntityDrop).reverse().join('');
    const types = (database.types || []).map(genType).join('');
    let lineComments = database.extra?.comments || [];
    const entities = (database.entities || []).map(e => {
        const ref = outExports$1.entityToRef(e);
        const entityRelations = (database.relations || []).filter(r => outExports$1.entityRefSame(r.src, ref));
        const entityComments = lineComments.filter(c => c.line < (e.extra?.line || 0));
        lineComments = lineComments.slice(entityComments.length);
        const comments = entityComments.length > 0 ? entityComments.map(c => c.comment ? `-- ${c.comment}\n` : '--\n').join('') + '\n' : '';
        return comments + genEntity(e, entityRelations, typesById);
    }).join('\n');
    return [drops, types, entities].filter(v => !!v).join('\n');
}
function generatePostgresDiff(diff) {
    const createdTypes = diff.types?.created?.map(genType) || [];
    const updatedTypes = diff.types?.updated?.map(genTypeAlter) || [];
    const deletedTypes = diff.types?.deleted?.map(genTypeDrop) || [];
    const types = createdTypes.concat(updatedTypes, deletedTypes).join('');
    const createdEntities = diff.entities?.created?.map(e => genEntity(e, [], {})) || [];
    const updatedEntities = diff.entities?.updated?.map(e => genEntityAlter(e, [], {})) || [];
    const deletedEntities = diff.entities?.deleted?.map(genEntityDrop) || [];
    const entities = createdEntities.concat(updatedEntities, deletedEntities).join('\n');
    const createdRelations = diff.relations?.created?.map(r => genRelation(r)) || [];
    const updatedRelations = diff.relations?.updated?.flatMap(r => genRelationAlter(r)) || [];
    const deletedRelations = diff.relations?.deleted?.map(genRelationDrop) || [];
    const relations = createdRelations.concat(updatedRelations, deletedRelations).join('\n');
    return [types, entities, relations].filter(v => !!v).join('\n');
}
function genNamespace(n) {
    // database & catalog are not handled in PostgreSQL
    return n.schema ? genIdentifier(n.schema) + '.' : '';
}
function genEntityDrop(e) {
    return `DROP ${e.kind === 'view' ? 'VIEW' : 'TABLE'} IF EXISTS ${genEntityIdentifier(e)};\n`;
}
function genEntity(e, relations, typesById) {
    if (e.kind === 'view')
        return genView(e);
    return genTable(e, relations, typesById);
}
function genTable(e, relations, typesById) {
    const comment = e.extra?.comment ? `-- ${e.extra.comment}\n` : '';
    const attrs = (e.attrs || []).map(a => genAttribute(a, e.pk, e.indexes || [], e.checks || [], relations, typesById));
    const pk = e.pk && e.pk.attrs.length > 1 ? [{ value: `${genConstraintName(e.pk.name)}PRIMARY KEY (${e.pk.attrs.map(genAttributePath).join(', ')})` }] : [];
    const indexes = (e.indexes || []).filter(i => !i.unique).map(i => genIndexCommand(i, e)).join('');
    const uniques = (e.indexes || []).filter(i => i.unique && i.attrs.length > 1).map(genUniqueEntity); // TODO: also unique on a single column when there is several
    const checks = (e.checks || []).filter(c => c.attrs.length > 1).map(genCheckEntity); // TODO: also check on a single column when there is several
    const rels = relations.filter(r => r.src.attrs.length > 1).map(r => genRelationEntity(r));
    const comments = [genCommentTable(e), ...(e.attrs || []).map(a => genCommentAttribute(a, e))].filter(c => !!c).join('');
    const inner = attrs.concat(pk, uniques, checks, rels).map(({ value, comment }, i, arr) => '  ' + value + (arr[i + 1] ? `,` : '') + (comment ? ' -- ' + comment : '') + '\n').join('');
    return comment + `CREATE TABLE ${genEntityIdentifier(e)} (${inner ? '\n' + inner : ''});\n${indexes}${comments}`;
}
function genView(e) {
    const comments = [genCommentView(e), ...(e.attrs || []).map(a => genCommentAttribute(a, e))].filter(c => !!c).join('');
    if (e.def) {
        return `CREATE VIEW ${genEntityIdentifier(e)} AS\n${e.def};\n${comments}`;
    }
    else {
        return `-- CREATE VIEW ${genEntityIdentifier(e)} AS <missing definition>;\n${comments}`;
    }
}
function genEntityIdentifier(e) {
    return `${genNamespace(e)}${genIdentifier(e.name)}`;
}
function genAttributeIdentifier(a, e) {
    return `${genEntityIdentifier(e)}.${genIdentifier(a.name)}`;
}
function genAttribute(a, pk, indexes, checks, relations, typesById) {
    const [type, typeComment] = genAttributeType(a.type, typesById);
    const notNull = a.null || (pk?.attrs.find(aa => outExports$1.attributePathSame(aa, [a.name]))) ? '' : ' NOT NULL';
    const df = a.default ? ` DEFAULT ${genAttributeValue(a.default)}` : '';
    const primaryKey = pk && pk.attrs.length === 1 && outExports$1.attributePathSame(pk.attrs[0], [a.name]) ? ` ${genConstraintName(pk.name)}PRIMARY KEY` : '';
    const attrUniques = indexes.filter(u => u.unique && u.attrs.length === 1 && outExports$1.attributePathSame(u.attrs[0], [a.name]));
    const unique = attrUniques.length === 1 ? ' UNIQUE' : '';
    const attrChecks = checks.filter(c => c.attrs.length === 1 && outExports$1.attributePathSame(c.attrs[0], [a.name]));
    const [check, checkComment] = attrChecks.length === 1 ? genCheckInline(attrChecks[0]) : ['', ''];
    const attrRelations = relations.filter(r => r.src.attrs.length === 1 && outExports$1.attributePathSame(r.src.attrs[0], [a.name]));
    const [relation, relComment] = attrRelations.length === 1 ? genRelationInline(attrRelations[0]) : ['', ''];
    const relComment2 = attrRelations.length > 1 ? `references: ${outExports.joinLast(attrRelations.map(r => `${genEntityRef(r.ref)}.${showAttributePath(r.ref.attrs[0])}${r.polymorphic ? ` (${showAttributePath(r.polymorphic.attribute)}=${genAttributeValue(r.polymorphic.value)})` : ''}`), ', ', ' or ')}` : '';
    const comment = [typeComment, checkComment, relComment, relComment2, a.extra?.comment || ''].filter(c => !!c).join(', ');
    return { value: `${genIdentifier(a.name)} ${type}${notNull}${df}${primaryKey}${unique}${check}${relation}`, comment };
}
function genAttributeType(type, typesById) {
    // TODO: also look for type with entity namespace?
    const t = typesById[type];
    if (t && t.alias) {
        return [t.alias, `type alias '${type}'`];
    }
    else {
        return [type, ''];
    }
}
function genIndexCommand(i, e) {
    const name = i.name || `${e.name}_${i.attrs.map(a => a.join('')).join('_')}_${i.unique ? 'uniq' : 'idx'}`;
    return `CREATE ${i.unique ? 'UNIQUE ' : ''}INDEX ${name} ON ${genEntityIdentifier(e)}(${i.attrs.map(genAttributePathIndex).join(', ')});\n`;
}
function genUniqueEntity(u) {
    return { value: `${genConstraintName(u.name)}UNIQUE (${u.attrs.map(genAttributePath).join(', ')})` };
}
function genCheckInline(c) {
    return c.predicate ? [` ${genConstraintName(c.name)}CHECK (${c.predicate})`, ''] : ['', 'check constraint but no predicate'];
}
function genCheckEntity(c) {
    return { value: `${genConstraintName(c.name)}CHECK (${c.predicate})` };
}
function genEntityAlter(e, relations, typesById) {
    if (e.kind && e.kind.after !== e.kind.before) {
        // drop & create
        return `TODO ${e.kind.before || 'table'} -> ${e.kind.after || 'table'}`;
    }
    else if (e.kind?.after === 'view') {
        return genViewAlter(e);
    }
    else {
        return genTableAlter(e, relations, typesById);
    }
}
function genTableAlter(e, relations, typesById) {
    // see https://www.postgresql.org/docs/current/sql-altertable.html
    const res = [];
    if (e.rename) {
        if (e.rename.before === undefined && e.rename.after) {
            res.push(`ALTER TABLE <missing old name> RENAME TO ${genIdentifier(e.rename.after)};\n`);
        }
        else if (e.rename.before && e.rename.after === undefined) {
            res.push(`ALTER TABLE ${genIdentifier(e.rename.before)} RENAME TO <missing new name>;\n`);
        }
        else if (e.rename.before && e.rename.after) {
            res.push(`ALTER TABLE ${genIdentifier(e.rename.before)} RENAME TO ${genIdentifier(e.rename.after)};\n`);
        }
    }
    return [
        res,
        genEntityAlterAttributes(e, relations, typesById),
        e.pk ? genEntityAlertPrimaryKey(e, e.pk) : [],
        e.indexes ? genEntityAlterIndexes(e, e.indexes) : [],
        e.checks ? genEntityAlterChecks(e, e.checks) : [],
        e.doc ? [genComment('TABLE', genEntityIdentifier(e), e.doc.after)] : [],
    ].flat().join('');
}
function genEntityAlertPrimaryKey(e, pk) {
    if (pk.before === undefined && pk.after) {
        return [`ALTER TABLE ${genEntityIdentifier(e)} ADD ${genConstraintName(pk.after.name)}PRIMARY KEY (${pk.after.attrs.map(genAttributePathIndex).join(', ')});\n`];
    }
    else if (pk.before && pk.after === undefined) {
        if (pk.before.name) {
            return [`ALTER TABLE ${genEntityIdentifier(e)} DROP CONSTRAINT ${genIdentifier(pk.before.name)};\n`];
        }
        else {
            return [`-- ALTER TABLE ${genEntityIdentifier(e)} DROP CONSTRAINT -- missing primary key name\n`];
        }
    }
    else {
        if (pk.before && pk.before.name) {
            return [`-- ALTER TABLE ${genEntityIdentifier(e)} ALTER ${genConstraintName(pk.before.name)}-- missing props\n`];
        }
        else {
            return [`-- ALTER TABLE ${genEntityIdentifier(e)} ALTER CONSTRAINT -- missing primary key name and props\n`];
        }
    }
}
function genEntityAlterIndexes(e, indexes) {
    return [].concat((indexes.created || []).map(i => genIndexCreate(e, i)), (indexes.deleted || []).map(i => genIndexDrop(e, i)), (indexes.updated || []).flatMap(i => genIndexAlter(e, i)));
}
function genIndexCreate(e, i) {
    // see https://www.postgresql.org/docs/current/sql-createindex.html
    // see https://www.postgresql.org/docs/current/indexes-partial.html
    const def = i.definition ? i.definition : i.attrs.map(genAttributePathIndex).join(', ');
    const ref = i.name ? genIdentifier(i.name) : `<missing name for index on ${genEntityIdentifier(e)} (${i.attrs.map(genAttributePathIndex).join(', ')})>`;
    const doc = i.doc ? genCommentIndex(i.name, ref, i.doc) : '';
    return `CREATE ${i.unique ? 'UNIQUE ' : ''}INDEX ${i.name ? genIdentifier(i.name) + ' ' : ''}ON ${genEntityIdentifier(e)} (${def})${i.partial ? ` WHERE ${i.partial}` : ''};\n${doc}`;
}
function genIndexDrop(e, i) {
    // see https://www.postgresql.org/docs/current/sql-dropindex.html
    if (i.name) {
        return `DROP INDEX ${i.name};\n`;
    }
    else {
        return `-- DROP INDEX -- missing name for ${genEntityIdentifier(e)} (${i.attrs.map(genAttributePathIndex).join(', ')});\n`;
    }
}
function genIndexAlter(e, i) {
    // see https://www.postgresql.org/docs/current/sql-alterindex.html
    const res = [];
    const name = i.name || i.rename?.after || i.rename?.before;
    const ref = name ? genIdentifier(name) : `<missing name for index on ${genEntityIdentifier(e)} (${i.attrs.map(genAttributePathIndex).join(', ')})>`;
    if (i.rename) {
        if (i.rename.before === undefined && i.rename.after) {
            res.push(`-- ALTER INDEX <missing old name> RENAME TO ${genIdentifier(i.rename.after)};\n`);
        }
        else if (i.rename.before && i.rename.after === undefined) {
            res.push(`-- ALTER INDEX ${genIdentifier(i.rename.before)} RENAME TO <missing new name>;\n`);
        }
        else if (i.rename.before && i.rename.after) {
            res.push(`ALTER INDEX ${genIdentifier(i.rename.before)} RENAME TO ${genIdentifier(i.rename.after)};\n`);
        }
    }
    if (i.unique)
        res.push(`-- ALTER INDEX ${ref} -- can't ${i.unique.after ? 'set' : 'drop'} UNIQUE on PostgreSQL\n`);
    if (i.partial)
        res.push(`-- ALTER INDEX ${ref} -- can't update index partial clause to (${i.partial.after}) on PostgreSQL\n`);
    if (i.definition)
        res.push(`-- ALTER INDEX ${ref} -- can't update index definition to (${i.definition.after}) on PostgreSQL\n`);
    if (i.doc)
        res.push(genCommentIndex(name, ref, i.doc.after));
    return res;
}
function genCommentIndex(name, ref, doc) {
    return (name ? '' : '-- ') + genComment('INDEX', ref, doc);
}
function genEntityAlterChecks(e, checks) {
    return [].concat((checks.created || []).map(c => genCheckCreate(e, c)), (checks.deleted || []).map(c => genCheckDrop(e, c)), (checks.updated || []).flatMap(c => genCheckAlter(e, c)));
}
function genCheckCreate(e, c) {
    // see https://www.postgresql.org/docs/current/sql-altertable.html
    const doc = c.doc ? genCommentCheck(c.name, c.name ? genIdentifier(c.name) : '<missing check name>', e, c.doc) : '';
    return `ALTER TABLE ${genEntityIdentifier(e)} ADD ${genConstraintName(c.name)}CHECK (${c.predicate});\n${doc}`;
}
function genCheckDrop(e, c) {
    // see https://www.postgresql.org/docs/current/sql-altertable.html
    if (c.name) {
        return `ALTER TABLE ${genEntityIdentifier(e)} DROP CONSTRAINT ${genIdentifier(c.name)};\n`;
    }
    else {
        return `-- ALTER TABLE ${genEntityIdentifier(e)} DROP CONSTRAINT -- missing name for check (${c.predicate})\n`;
    }
}
function genCheckAlter(e, c) {
    const res = [];
    const name = c.name || c.rename?.after || c.rename?.before;
    const ref = name ? genIdentifier(name) : `<missing name for check>`;
    if (c.rename) {
        if (c.rename.before === undefined && c.rename.after) {
            res.push(`-- ALTER TABLE ${genEntityIdentifier(e)} RENAME CONSTRAINT <missing old name> TO ${genIdentifier(c.rename.after)};\n`);
        }
        else if (c.rename.before && c.rename.after === undefined) {
            res.push(`-- ALTER TABLE ${genEntityIdentifier(e)} RENAME ${genConstraintName(c.rename.before)}TO <missing new name>;\n`);
        }
        else if (c.rename.before && c.rename.after) {
            res.push(`ALTER TABLE ${genEntityIdentifier(e)} RENAME ${genConstraintName(c.rename.before)}TO ${genIdentifier(c.rename.after)};\n`);
        }
    }
    if (c.predicate)
        res.push(`-- ALTER TABLE ${genEntityIdentifier(e)} ALTER CONSTRAINT ${ref} -- can't update check predicate to (${c.predicate.after}) on PostgreSQL\n`);
    if (c.doc)
        res.push(genCommentCheck(name, ref, e, c.doc.after));
    return res;
}
function genCommentCheck(name, ref, e, value) {
    return (name ? '' : '-- ') + `COMMENT ON CONSTRAINT ${ref} ON ${genEntityIdentifier(e)} IS ${value ? genString(value) : 'NULL'};\n`;
}
function genViewAlter(e) {
    // TODO improve
    return [
        e.doc ? [genComment('VIEW', genEntityIdentifier(e), e.doc.after)] : [],
    ].flat().join('');
}
function genEntityAlterAttributes(e, relations, typesById) {
    if (e.attrs) {
        const renamed = e.attrs.deleted?.flatMap(d => {
            const replace = e.attrs?.created?.find(c => c.i === d.i);
            return replace && outExports.equalDeep(d, { ...replace, name: d.name }) ? [{ i: d.i, name: { before: d.name, after: replace.name } }] : [];
        }) || [];
        return [
            renamed.map(r => `ALTER TABLE ${genTypeIdentifier(e)} RENAME ${r.name.before} TO ${r.name.after};\n`),
            e.attrs.updated?.flatMap(a => genEntityAlterAttribute(a, e)),
            e.attrs.created?.flatMap(a => renamed.find(r => r.i === a.i) ? [] : [`ALTER TABLE ${genTypeIdentifier(e)} ADD ${genAttribute(a, e.pk?.after, [], [], relations, typesById).value};\n`]),
            e.attrs.deleted?.flatMap(a => renamed.find(r => r.i === a.i) ? [] : [`ALTER TABLE ${genTypeIdentifier(e)} DROP ${a.name};\n`]),
        ].flat();
    }
    return [];
}
function genEntityAlterAttribute(a, e, relations, typesById) {
    return [
        a.type?.after ? [`ALTER TABLE ${genTypeIdentifier(e)} ALTER ${a.name} TYPE ${a.type.after};\n`] : [],
        a.null ? [`ALTER TABLE ${genTypeIdentifier(e)} ALTER ${a.name} ${a.null.after ? 'DROP' : 'SET'} NOT NULL;\n`] : [],
        a.default ? [`ALTER TABLE ${genTypeIdentifier(e)} ALTER ${a.name} ${a.default.after === undefined ? 'DROP DEFAULT' : `SET DEFAULT ${genAttributeValue(a.default.after)}`};\n`] : [],
        a.doc ? [genComment('COLUMN', genAttributeIdentifier(a, e), a.doc.after)] : [],
    ].flat();
}
function genRelationInline(relation) {
    if (relation.ref.attrs.some(a => a.length > 1)) {
        return ['', `reference nested attribute ${genEntityRef(relation.ref)}(${relation.ref.attrs.map(showAttributePath).join(', ')})`];
    }
    else {
        return [` REFERENCES ${genEntityRef(relation.ref)}(${relation.ref.attrs.map(genAttributePath).join(', ')})`, ''];
    }
}
function genRelationEntity(relation) {
    const src = relation.src.attrs.map(genAttributePath).join(', ');
    const refTable = `${genNamespace(relation.ref)}${genIdentifier(relation.ref.entity)}`;
    const refAttrs = relation.ref.attrs.map(genAttributePath).join(', ');
    return { value: `${genConstraintName(relation.name)}FOREIGN KEY (${src}) REFERENCES ${refTable}(${refAttrs})` };
}
function genRelation(relation) {
    return `ALTER TABLE ${genEntityRef(relation.src)} ADD FOREIGN KEY (${relation.src.attrs.map(genAttributePath).join(', ')}) REFERENCES ${genEntityRef(relation.ref)}(${relation.ref.attrs.map(genAttributePath).join(', ')});\n`;
}
function genRelationAlter(relation) {
    const res = [];
    if (relation.rename) {
        if (relation.rename.before === undefined && relation.rename.after) {
            res.push(`-- ALTER TABLE ${genEntityRef(relation.src)} RENAME CONSTRAINT <missing old name> TO ${genIdentifier(relation.rename.after)};\n`);
        }
        else if (relation.rename.before && relation.rename.after === undefined) {
            res.push(`-- ALTER TABLE ${genEntityRef(relation.src)} RENAME CONSTRAINT ${genIdentifier(relation.rename.before)} TO <missing new name>;\n`);
        }
        else if (relation.rename.before && relation.rename.after) {
            res.push(`ALTER TABLE ${genEntityRef(relation.src)} RENAME CONSTRAINT ${genIdentifier(relation.rename.before)} TO ${genIdentifier(relation.rename.after)};\n`);
        }
    }
    return res;
}
function genRelationDrop(relation) {
    if (relation.name) {
        return `ALTER TABLE ${genEntityRef(relation.src)} DROP CONSTRAINT ${genIdentifier(relation.name)};\n`;
    }
    else {
        return `-- ALTER TABLE ${genEntityRef(relation.src)} DROP CONSTRAINT -- missing name for ${outExports$1.relationToId(relation)}\n`;
    }
}
function genConstraintName(name) {
    return name ? `CONSTRAINT ${genIdentifier(name)} ` : '';
}
function genTypeIdentifier(t) {
    return `${genNamespace(t)}${genIdentifier(t.name)}`;
}
function genType(t) {
    const content = genTypeContent(t);
    const comment = genCommentType(t);
    return `${t.alias ? '-- ' : ''}CREATE TYPE ${genTypeIdentifier(t)}${content};${t.alias ? ' -- type alias not supported on PostgreSQL' : ''}\n${comment}`;
}
function genTypeContent(t) {
    if (t.alias)
        return ` AS ${t.alias}`; // fake syntax, will be commented
    if (t.values)
        return ` AS ENUM (${t.values.map(v => "'" + v + "'").join(', ')})`;
    if (t.attrs)
        return ` AS (${t.attrs.map(a => genIdentifier(a.name) + ' ' + a.type).join(', ')})`;
    if (t.definition)
        return ` ${t.definition}`;
    return '';
}
function genTypeAlter(t) {
    return [
        genTypeRename(t),
        genTypeAlterAlias(t),
        genTypeAlterEnum(t),
        genTypeAlterStruct(t),
        genTypeAlterCustom(t),
        t.doc ? [genComment('TYPE', genTypeIdentifier(t), t.doc.after)] : [],
    ].flat().join('');
}
function genTypeRename(t) {
    if (t.rename) {
        if (t.rename.before === undefined && t.rename.after) {
            return [`-- ALTER TYPE <missing old name> RENAME TO ${genIdentifier(t.rename.after)};\n`];
        }
        else if (t.rename.before && t.rename.after === undefined) {
            return [`-- ALTER TYPE ${genIdentifier(t.rename.before)} RENAME TO <missing new name>;\n`];
        }
        else if (t.rename.before && t.rename.after) {
            return [`ALTER TYPE ${genIdentifier(t.rename.before)} RENAME TO ${genIdentifier(t.rename.after)};\n`];
        }
    }
    return [];
}
function genTypeAlterAlias(t) {
    if (t.alias?.after) {
        if (t.alias.before) { // was already an alias
            return [`-- ALTER TYPE ${genTypeIdentifier(t)} AS ${t.alias.after}; -- type alias not supported on PostgreSQL\n`];
        }
        else { // was something else
            return [genTypeDrop(t), genType({ ...outExports$1.namespace(t), name: t.name, alias: t.alias.after })];
        }
    }
    return [];
}
function genTypeAlterEnum(t) {
    if (t.values?.after) {
        if (t.values.before) { // already enum
            return t.values.after.flatMap(v => t.values?.before?.includes(v) ? [] : [`ALTER TYPE ${genTypeIdentifier(t)} ADD VALUE IF NOT EXISTS ${genString(v)};\n`])
                .concat(t.values.before.flatMap(v => t.values?.after?.includes(v) ? [] : [`-- ALTER TYPE ${genTypeIdentifier(t)} DROP VALUE ${genString(v)}; -- can't drop enum value in PostgreSQL\n`]));
        }
        else { // was something else
            return [genTypeDrop(t), genType({ ...outExports$1.namespace(t), name: t.name, values: t.values.after })];
        }
    }
    return [];
}
function genTypeAlterStruct(t) {
    if (t.attrs) {
        if (t.alias || t.values || t.values || t.definition) { // was not a struct
            return [genTypeDrop(t), genType({ ...outExports$1.namespace(t), name: t.name, attrs: t.attrs.created })];
        }
        else {
            const renamed = t.attrs.deleted?.flatMap(d => {
                const replace = t.attrs?.created?.find(c => c.i === d.i);
                return replace && outExports.equalDeep(d, { ...replace, name: d.name }) ? [{ i: d.i, name: { before: d.name, after: replace.name } }] : [];
            }) || [];
            return [
                renamed.map(r => `ALTER TYPE ${genTypeIdentifier(t)} RENAME ATTRIBUTE ${r.name.before} TO ${r.name.after};\n`),
                t.attrs.updated?.flatMap(a => a.type?.after ? [`ALTER TYPE ${genTypeIdentifier(t)} ALTER ATTRIBUTE ${a.name} TYPE ${a.type.after};\n`] : []),
                t.attrs.created?.flatMap(a => renamed.find(r => r.i === a.i) ? [] : [`ALTER TYPE ${genTypeIdentifier(t)} ADD ATTRIBUTE ${a.name} ${a.type};\n`]),
                t.attrs.deleted?.flatMap(a => renamed.find(r => r.i === a.i) ? [] : [`ALTER TYPE ${genTypeIdentifier(t)} DROP ATTRIBUTE IF EXISTS ${a.name};\n`]),
            ].flat();
        }
    }
    return [];
}
function genTypeAlterCustom(t) {
    if (t.definition?.after) { // can't update custom types, so drop & create
        return [genTypeDrop(t), genType({ ...outExports$1.namespace(t), name: t.name, definition: t.definition.after })];
    }
    return [];
}
function genTypeDrop(t) {
    return `DROP TYPE IF EXISTS ${genTypeIdentifier(t)};\n`;
}
function genCommentTable(e) {
    return e.doc ? genComment('TABLE', genEntityIdentifier(e), e.doc) : '';
}
function genCommentView(e) {
    return e.doc ? genComment('VIEW', genEntityIdentifier(e), e.doc) : '';
}
function genCommentAttribute(a, e) {
    return a.doc ? genComment('COLUMN', genAttributeIdentifier(a, e), a.doc) : '';
}
function genCommentType(t) {
    return t.doc ? genComment('TYPE', genTypeIdentifier(t), t.doc) : '';
}
function genComment(kind, identifier, value) {
    return `COMMENT ON ${kind} ${identifier} IS ${value ? genString(value) : 'NULL'};\n`;
}
function genEntityRef(ref) {
    return `${genNamespace(ref)}${genIdentifier(ref.entity)}`;
}
function genAttributePath(p) {
    const [head, ...tail] = p;
    return head + tail.map(a => `->'${a}'`).join('');
}
function genAttributePathIndex(p) {
    const [head, ...tail] = p;
    if (tail.length > 0) {
        return '(' + head + tail.map((a, i) => (tail[i + 1] ? '->' : '->>') + `'${a}'`).join('') + ')';
    }
    else {
        return head;
    }
}
function showAttributePath(p) {
    return p.join('.');
}
function genAttributeValue(v) {
    if (v === undefined)
        return '';
    if (v === null)
        return 'NULL';
    if (typeof v === 'string')
        return v.startsWith('`') ? v.slice(1, -1) : genString(v);
    if (typeof v === 'number')
        return v.toString();
    if (typeof v === 'boolean')
        return v ? 'TRUE' : 'FALSE';
    return `${v}`;
}
function genIdentifier(str) {
    if (str.match(/^[a-zA-Z_][a-zA-Z0-9_$]*$/))
        return str;
    return '"' + str + '"';
}
function genString(str) {
    if (str.indexOf('\n') !== -1) {
        return `E'${str.replaceAll(/'/g, "''").replaceAll(/\n/g, '\\n')}'`;
    }
    else {
        return `'${str.replaceAll(/'/g, "''")}'`;
    }
}

function parseSql(content, dialect, opts = {}) {
    const start = Date.now();
    if (dialect === outExports$1.DatabaseKind.enum.postgres)
        return parsePostgresAst(content, opts).flatMap(ast => buildPostgresDatabase(ast, start, Date.now()));
    return parseSqlScript(content).map(importDatabase);
}
function generateSql(database, dialect) {
    if (dialect === outExports$1.DatabaseKind.enum.postgres)
        return generatePostgres(database);
    return generateSqlScript();
}
function generateSqlDiff(diff, dialect) {
    if (dialect === outExports$1.DatabaseKind.enum.postgres)
        return generatePostgresDiff(diff);
    return `Can't generate SQL diff for ${dialect} dialect`;
}
function parseSqlScript(content) {
    return parseSqlAst(content).map(format);
}
function generateSqlScript(script) {
    return generate();
}
function parseSqlAst(content) {
    const { result, errors } = parse(content);
    if (result) {
        return outExports$1.ParserResult.success(result);
    }
    else {
        return outExports$1.ParserResult.failure(errors || []);
    }
}

const monaco = {};
const version = packageJson.version;
/*
  Parser:
  - https://github.com/tobymao/sqlglot: python parser, needs to be transpiled to JS (https://github.com/TranscryptOrg/Transcrypt)
  - https://github.com/taozhi8833998/node-sql-parser: parse and generate SQL, not exact but handle more syntaxes (inspired by https://github.com/florajs/sql-parser and https://github.com/alibaba/nquery)
  - https://github.com/nene/sql-parser-cst: parse and generate SQL, keep comments & formatting (forked from node-sql-parser below)
  see: https://npmtrends.com/flora-sql-parser-vs-js-sql-parser-vs-node-sql-parser-vs-node-sqlparser-vs-sql-parse-vs-sql-parser
  Formatter:
  - https://github.com/nene/prettier-plugin-sql-cst: made from sql-formatter below but not available as a lib :/
  - https://github.com/sql-formatter-org/sql-formatter
 */

exports.generateJsonDatabase = outExports$1.generateJsonDatabase;
exports.generateSql = generateSql;
exports.generateSqlDiff = generateSqlDiff;
exports.monaco = monaco;
exports.parseJsonDatabase = outExports$1.parseJsonDatabase;
exports.parseSql = parseSql;
exports.schemaJsonDatabase = outExports$1.DatabaseSchema;
exports.version = version;
//# sourceMappingURL=bundle.js.map
